/*************************************************************************************************************
    RASPA: a molecular-dynamics, monte-carlo and optimization code for nanoporous materials
    Copyright (C) 2006-2015 David Dubbeldam, Sofia Calero, Thijs Vlugt, Donald E. Ellis, and Randall Q. Snurr.

    D.Dubbeldam@uva.nl            http://molsim.science.uva.nl/
    scaldia@upo.es                http://www.upo.es/raspa/
    t.j.h.vlugt@tudelft.nl        http://homepage.tudelft.nl/v9k6y
    don-ellis@northwestern.edu    http://dvworld.northwestern.edu/
    snurr@northwestern.edu        http://zeolites.cqe.northwestern.edu/

    This file 'spacegroup.c' is part of RASPA-2.0

    RASPA is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RASPA is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *************************************************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "constants.h"
#include "molecule.h"
#include "framework.h"
#include "framework_energy.h"
#include "simulation.h"
#include "ewald.h"
#include "potentials.h"
#include "utils.h"
#include "output.h"
#include "cbmc.h"
#include "mc_moves.h"
#include "grids.h"
#include "spacegroup.h"


// b1     monoclinic unique axis b, cell choice 1, abc
// b2     monoclinic unique axis b, cell choice 2, abc
// b3     monoclinic unique axis b, cell choice 3, abc
// -b1    monoclinic unique axis b, cell choice 1, c-ba
// -b2    monoclinic unique axis b, cell choice 2, c-ba
// -b3    monoclinic unique axis b, cell choice 3, c-ba
// c1     monoclinic unique axis c, cell choice 1, abc
// c2     monoclinic unique axis c, cell choice 2, abc
// c3     monoclinic unique axis c, cell choice 3, abc
// -c1    monoclinic unique axis c, cell choice 1, ba-c
// -c2    monoclinic unique axis c, cell choice 2, ba-c
// -c3    monoclinic unique axis c, cell choice 3, ba-c
// a1     monoclinic unique axis a, cell choice 1, abc
// a2     monoclinic unique axis a, cell choice 2, abc
// a3     monoclinic unique axis a, cell choice 3, abc
// -a1    monoclinic unique axis a, cell choice 1, -acb
// -a2    monoclinic unique axis a, cell choice 2, -acb
// -a3    monoclinic unique axis a, cell choice 3, -acb
// abc    orthorhombic
// ba-c   orthorhombic
// cab    orthorhombic
// -cba   orthorhombic
// bca    orthorhombic
// a-cb   orthorhombic
// 1abc   orthorhombic origin choice 1
// 1ba-c  orthorhombic origin choice 1
// 1cab   orthorhombic origin choice 1
// 1-cba  orthorhombic origin choice 1
// 1bca   orthorhombic origin choice 1
// 1a-cb  orthorhombic origin choice 1
// 2abc   orthorhombic origin choice 2
// 2ba-c  orthorhombic origin choice 2
// 2cab   orthorhombic origin choice 2
// 2-cba  orthorhombic origin choice 2
// 2bca   orthorhombic origin choice 2
// 2a-cb  orthorhombic origin choice 2
// 1      tetragonal or cubic origin choice 1
// 2      tetragonal or cubic origin choice 2
// h      trigonal using hexagonal axes
// r      trigonal using rhombohedral axes

char *CoordinateSystemCode[40]={"b1","b2","b3","-b1","-b2","-b3","c1","c2","c3","-c1","-c2","-c3","a1","b2","a3","-a1","-a2","-a3",
                               "abc","ba-c","cab","-cba","bca","a-cb","1abc","1ba-c","1cab","1-cba","1bca","1a-cb","2abc","2ba-c","2cab","2-cba","2bca","2a-cb","1","2","h","r"};

enum {A_UNIQUE_CELL_1,A_UNIQUE_CELL_2,A_UNIQUE_CELL_3,
      B_UNIQUE_CELL_1,B_UNIQUE_CELL_2,B_UNIQUE_CELL_3,
      C_UNIQUE_CELL_1,C_UNIQUE_CELL_2,C_UNIQUE_CELL_3,
      mA_UNIQUE_CELL_1,mA_UNIQUE_CELL_2,mA_UNIQUE_CELL_3,
      mB_UNIQUE_CELL_1,mB_UNIQUE_CELL_2,mB_UNIQUE_CELL_3,
      mC_UNIQUE_CELL_1,mC_UNIQUE_CELL_2,mC_UNIQUE_CELL_3,
      ORIGIN_1_ABC,ORIGIN_1_CAB,ORIGIN_1_BCA,
      ORIGIN_1_BACm,ORIGIN_1_CmBA,ORIGIN_1_ACmB,
      ORIGIN_2_ABC,ORIGIN_2_CAB,ORIGIN_2_BCA,
      ORIGIN_2_BACm,ORIGIN_2_CmBA,ORIGIN_2_ACmB,
      HEXAGONAL,RHOMBOHEDRAL};

enum {PRIMITIVE_CENTERED,A_CENTERED,B_CENTERED,C_CENTERED,BODY_CENTERED,FACE_CENTERED,RHOMBOHEDRAL_CENTERED};


int SpaceGroupSize;
VECTOR SpaceGroupElement[200];

// Information on all the 530 space-groups (230 standard, 300 non-standard)
SPACE_GROUP_DATA SpaceGroupData[NUMBER_OF_SPACEGROUPS+1]=
{
{0,"",0,-1,1,"","","","",-1,"",-1,-1,{{0.0,0.0,0.0}},-1,-1,-1,-1},
{1,"P1",1,ORIGIN_1_ABC,1,"P 1","P 1","C1.1","P 1",TRICLINIC_SPACEGROUP,"1",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},1,FALSE,FALSE,FALSE,TRUE},

{2,"P-1",2,ORIGIN_1_ABC,1,"P -1","P -1","Ci.1","-P 1",TRICLINIC_SPACEGROUP,"-1",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},2,TRUE,FALSE,FALSE,TRUE},

{3,"P2",3,B_UNIQUE_CELL_1,1,"P 1 2 1","P 1 2 1","C2.1","P 2y",MONOCLINIC_SPACEGROUP,"2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},2,FALSE,TRUE,FALSE,TRUE},
{4,"P2",3,C_UNIQUE_CELL_1,2,"P 1 1 2","P 1 1 2","C2.1","P 2",MONOCLINIC_SPACEGROUP,"2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},2,FALSE,TRUE,FALSE,FALSE},
{5,"P2",3,A_UNIQUE_CELL_1,3,"P 2 1 1","P 2 1 1","C2.1","P 2x",MONOCLINIC_SPACEGROUP,"2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},2,FALSE,TRUE,FALSE,FALSE},

{6,"P21",4,B_UNIQUE_CELL_1,1,"P 1 21 1","P 1 21 1","C2.2","P 2yb",MONOCLINIC_SPACEGROUP,"2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},2,FALSE,TRUE,FALSE,TRUE},
{7,"P21",4,C_UNIQUE_CELL_1,2,"P 1 1 21","P 1 1 21","C2.2","P 2c",MONOCLINIC_SPACEGROUP,"2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},2,FALSE,TRUE,FALSE,FALSE},
{8,"P21",4,A_UNIQUE_CELL_1,3,"P 21 1 1","P 21 1 1","C2.2","P 2xa",MONOCLINIC_SPACEGROUP,"2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},2,FALSE,TRUE,FALSE,FALSE},

{9,"C2",5,B_UNIQUE_CELL_1,1,"C 1 2 1","C 1 2 1","C2.3","C 2y",MONOCLINIC_SPACEGROUP,"2",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},4,FALSE,TRUE,FALSE,TRUE},
{10,"A2",5,B_UNIQUE_CELL_2,2,"A 1 2 1","A 1 2 1","C2.3","A 2y",MONOCLINIC_SPACEGROUP,"2",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},4,FALSE,TRUE,FALSE,FALSE},
{11,"I2",5,B_UNIQUE_CELL_3,3,"I 1 2 1","I 1 2 1","C2.3","I 2y",MONOCLINIC_SPACEGROUP,"2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},4,FALSE,TRUE,FALSE,FALSE},
{12,"A2",5,C_UNIQUE_CELL_1,4,"A 1 1 2","A 1 1 2","C2.3","A 2",MONOCLINIC_SPACEGROUP,"2",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},4,FALSE,TRUE,FALSE,FALSE},
{13,"B2",5,C_UNIQUE_CELL_2,5,"B 1 1 2","B 1 1 2","C2.3","B 2",MONOCLINIC_SPACEGROUP,"2",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},4,FALSE,TRUE,FALSE,FALSE},
{14,"I2",5,C_UNIQUE_CELL_3,6,"I 1 1 2","I 1 1 2","C2.3","I 2",MONOCLINIC_SPACEGROUP,"2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},4,FALSE,TRUE,FALSE,FALSE},
{15,"B2",5,A_UNIQUE_CELL_1,7,"B 2 1 1","B 2 1 1","C2.3","B 2x",MONOCLINIC_SPACEGROUP,"2",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},4,FALSE,TRUE,FALSE,FALSE},
{16,"C2",5,A_UNIQUE_CELL_2,8,"C 2 1 1","C 2 1 1","C2.3","C 2x",MONOCLINIC_SPACEGROUP,"2",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},4,FALSE,TRUE,FALSE,FALSE},
{17,"I2",5,A_UNIQUE_CELL_3,9,"I 2 1 1","I 2 1 1","C2.3","I 2x",MONOCLINIC_SPACEGROUP,"2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},4,FALSE,TRUE,FALSE,FALSE},

{18,"BM",6,B_UNIQUE_CELL_1,1,"P 1 m 1","P 1 m 1","Cs.1","P -2y",MONOCLINIC_SPACEGROUP,"m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},2,FALSE,FALSE,FALSE,TRUE},
{19,"BM",6,C_UNIQUE_CELL_1,2,"P 1 1 m","P 1 1 m","Cs.1","P -2",MONOCLINIC_SPACEGROUP,"m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},2,FALSE,FALSE,FALSE,FALSE},
{20,"BM",6,A_UNIQUE_CELL_1,3,"P m 1 1","P m 1 1","Cs.1","P -2x",MONOCLINIC_SPACEGROUP,"m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},2,FALSE,FALSE,FALSE,FALSE},

{21,"PC",7,B_UNIQUE_CELL_1,1,"P 1 c 1","P 1 c 1","Cs.2","P -2yc",MONOCLINIC_SPACEGROUP,"m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},2,FALSE,FALSE,FALSE,TRUE},
{22,"PN",7,B_UNIQUE_CELL_2,2,"P 1 n 1","P 1 n 1","Cs.2","P -2yac",MONOCLINIC_SPACEGROUP,"m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},2,FALSE,FALSE,FALSE,FALSE},
{23,"PA",7,B_UNIQUE_CELL_3,3,"P 1 a 1","P 1 a 1","Cs.2","P -2ya",MONOCLINIC_SPACEGROUP,"m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},2,FALSE,FALSE,FALSE,FALSE},
{24,"PA",7,C_UNIQUE_CELL_1,4,"P 1 1 a","P 1 1 a","Cs.2","P -2a",MONOCLINIC_SPACEGROUP,"m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},2,FALSE,FALSE,FALSE,FALSE},
{25,"PN",7,C_UNIQUE_CELL_2,5,"P 1 1 n","P 1 1 n","Cs.2","P -2ab",MONOCLINIC_SPACEGROUP,"m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},2,FALSE,FALSE,FALSE,FALSE},
{26,"PB",7,C_UNIQUE_CELL_3,6,"P 1 1 b","P 1 1 b","Cs.2","P -2b",MONOCLINIC_SPACEGROUP,"m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},2,FALSE,FALSE,FALSE,FALSE},
{27,"PB",7,A_UNIQUE_CELL_1,7,"P b 1 1","P b 1 1","Cs.2","P -2xb",MONOCLINIC_SPACEGROUP,"m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},2,FALSE,FALSE,FALSE,FALSE},
{28,"PN",7,A_UNIQUE_CELL_2,8,"P n 1 1","P n 1 1","Cs.2","P -2xbc",MONOCLINIC_SPACEGROUP,"m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},2,FALSE,FALSE,FALSE,FALSE},
{29,"PC",7,A_UNIQUE_CELL_3,9,"P c 1 1","P c 1 1","Cs.2","P -2xc",MONOCLINIC_SPACEGROUP,"m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},2,FALSE,FALSE,FALSE,FALSE},

{30,"CM",8,B_UNIQUE_CELL_1,1,"C 1 m 1","C 1 m 1","Cs.3","C -2y",MONOCLINIC_SPACEGROUP,"m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},4,FALSE,FALSE,FALSE,TRUE},
{31,"AM",8,B_UNIQUE_CELL_2,2,"A 1 m 1","A 1 m 1","Cs.3","A -2y",MONOCLINIC_SPACEGROUP,"m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},4,FALSE,FALSE,FALSE,FALSE},
{32,"IM",8,B_UNIQUE_CELL_3,3,"I 1 m 1","I 1 m 1","Cs.3","I -2y",MONOCLINIC_SPACEGROUP,"m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},4,FALSE,FALSE,FALSE,FALSE},
{33,"AM",8,C_UNIQUE_CELL_1,4,"A 1 1 m","A 1 1 m","Cs.3","A -2",MONOCLINIC_SPACEGROUP,"m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},4,FALSE,FALSE,FALSE,FALSE},
{34,"BM",8,C_UNIQUE_CELL_2,5,"B 1 1 m","B 1 1 m","Cs.3","B -2",MONOCLINIC_SPACEGROUP,"m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},4,FALSE,FALSE,FALSE,FALSE},
{35,"IM",8,C_UNIQUE_CELL_3,6,"I 1 1 m","I 1 1 m","Cs.3","I -2",MONOCLINIC_SPACEGROUP,"m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},4,FALSE,FALSE,FALSE,FALSE},
{36,"BM",8,A_UNIQUE_CELL_1,7,"B m 1 1","B m 1 1","Cs.3","B -2x",MONOCLINIC_SPACEGROUP,"m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},4,FALSE,FALSE,FALSE,FALSE},
{37,"CM",8,A_UNIQUE_CELL_2,8,"C m 1 1","B m 1 1","Cs.3","C -2x",MONOCLINIC_SPACEGROUP,"m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{38,"IM",8,A_UNIQUE_CELL_3,9,"I m 1 1","I m 1 1","Cs.3","I -2x",MONOCLINIC_SPACEGROUP,"m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},4,FALSE,FALSE,FALSE,FALSE},

{39,"CC",9,B_UNIQUE_CELL_1,1,"C 1 c 1","C 1 c 1","Cs.4","C -2yc",MONOCLINIC_SPACEGROUP,"2/m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},4,FALSE,FALSE,FALSE,TRUE},
{40,"AN",9,B_UNIQUE_CELL_2,2,"A 1 n 1","A 1 n 1","Cs.4","A -2yab",MONOCLINIC_SPACEGROUP,"2/m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},4,FALSE,FALSE,FALSE,FALSE},
{41,"IA",9,B_UNIQUE_CELL_3,3,"I 1 a 1","I 1 a 1","Cs.4","I -2ya",MONOCLINIC_SPACEGROUP,"2/m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},4,FALSE,FALSE,FALSE,FALSE},
{42,"AA",9,mB_UNIQUE_CELL_1,10,"A 1 a 1","A 1 a 1","Cs.4","A -2ya",MONOCLINIC_SPACEGROUP,"2/m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},4,FALSE,FALSE,FALSE,FALSE},
{43,"CN",9,mB_UNIQUE_CELL_2,11,"C 1 n 1","C 1 n 1","Cs.4","C -2yac",MONOCLINIC_SPACEGROUP,"2/m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{44,"IC",9,mB_UNIQUE_CELL_3,12,"I 1 c 1","I 1 c 1","Cs.4","I -2yc",MONOCLINIC_SPACEGROUP,"2/m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},4,FALSE,FALSE,FALSE,FALSE},
{45,"AA",9,C_UNIQUE_CELL_1,4,"A 1 1 a","A 1 1 a","Cs.4","A -2a",MONOCLINIC_SPACEGROUP,"2/m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},4,FALSE,FALSE,FALSE,FALSE},
{46,"BN",9,C_UNIQUE_CELL_2,5,"B 1 1 n","B 1 1 n","Cs.4","B -2ab",MONOCLINIC_SPACEGROUP,"2/m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},4,FALSE,FALSE,FALSE,FALSE},
{47,"IB",9,C_UNIQUE_CELL_3,6,"I 1 1 b","I 1 1 b","Cs.4","I -2b",MONOCLINIC_SPACEGROUP,"2/m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},4,FALSE,FALSE,FALSE,FALSE},
{48,"BB",9,mC_UNIQUE_CELL_1,13,"B 1 1 b","B 1 1 b","Cs.4","B -2b",MONOCLINIC_SPACEGROUP,"2/m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},4,FALSE,FALSE,FALSE,FALSE},
{49,"AN",9,mC_UNIQUE_CELL_2,14,"A 1 1 n","A 1 1 n","Cs.4","A -2ab",MONOCLINIC_SPACEGROUP,"2/m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},4,FALSE,FALSE,FALSE,FALSE},
{50,"IA",9,mC_UNIQUE_CELL_3,15,"I 1 1 a","I 1 1 a","Cs.4","I -2a",MONOCLINIC_SPACEGROUP,"2/m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},4,FALSE,FALSE,FALSE,FALSE},
{51,"BB",9,A_UNIQUE_CELL_1,7,"B b 1 1","B b 1 1","Cs.4","B -2xb",MONOCLINIC_SPACEGROUP,"2/m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},4,FALSE,FALSE,FALSE,FALSE},
{52,"CN",9,A_UNIQUE_CELL_2,8,"C n 1 1","C n 1 1","Cs.4","C -2xac",MONOCLINIC_SPACEGROUP,"2/m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{53,"IC",9,A_UNIQUE_CELL_3,9,"I c 1 1","I c 1 1","Cs.4","I -2xc",MONOCLINIC_SPACEGROUP,"2/m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},4,FALSE,FALSE,FALSE,FALSE},
{54,"CC",9,mA_UNIQUE_CELL_1,16,"C c 1 1","C c 1 1","Cs.4","C -2xc",MONOCLINIC_SPACEGROUP,"2/m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{55,"BN",9,mA_UNIQUE_CELL_2,17,"B n 1 1","B n 1 1","Cs.4","B -2xab",MONOCLINIC_SPACEGROUP,"2/m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},4,FALSE,FALSE,FALSE,FALSE},
{56,"IB",9,mA_UNIQUE_CELL_3,18,"I b 1 1","I b 1 1","Cs.4","I -2xb",MONOCLINIC_SPACEGROUP,"2/m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},4,FALSE,FALSE,FALSE,FALSE},

{57,"P2/M",10,B_UNIQUE_CELL_1,1,"P 1 2/m 1","P 1 2/m 1","C2h.1","-P 2y",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,TRUE},
{58,"P2/M",10,C_UNIQUE_CELL_1,2,"P 1 1 2/m","P 1 1 2/m","C2h.1","-P 2",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},
{59,"P2/M",10,A_UNIQUE_CELL_1,3,"P 2/m 1 1","P 2/m 1 1","C2h.1","-P 2x",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},

{60,"P21/M",11,B_UNIQUE_CELL_1,1,"P 1 21/m 1","P 1 21/m 1","C2h.2","-P 2yb",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,TRUE},
{61,"P21/M",11,C_UNIQUE_CELL_1,2,"P 1 1 21/m","P 1 1 21/m","C2h.2","-P 2c",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},
{62,"P21/M",11,A_UNIQUE_CELL_1,3,"P 21/m 1 1","P 21/m 1 1","C2h.2","-P 2xa",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},

{63,"C2/M",12,B_UNIQUE_CELL_1,1,"C 1 2/m 1","C 1 2/m 1","C2h.3","-C 2y",MONOCLINIC_SPACEGROUP,"2/m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,TRUE,FALSE,FALSE,TRUE},
{64,"A2/M",12,B_UNIQUE_CELL_2,2,"A 1 2/m 1","A 1 2/m 1","C2h.3","-A 2y",MONOCLINIC_SPACEGROUP,"2/m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,TRUE,FALSE,FALSE,FALSE},
{65,"I2/M",12,B_UNIQUE_CELL_3,3,"I 1 2/m 1","I 1 2/m 1","C2h.3","-I 2y",MONOCLINIC_SPACEGROUP,"2/m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,TRUE,FALSE,FALSE,FALSE},
{66,"A2/M",12,C_UNIQUE_CELL_1,4,"A 1 1 2/m","A 1 1 2/m","C2h.3","-A 2",MONOCLINIC_SPACEGROUP,"2/m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,TRUE,FALSE,FALSE,FALSE},
{67,"B2/M",12,C_UNIQUE_CELL_2,5,"B 1 1 2/m","B 1 1 2/m","C2h.3","-B 2",MONOCLINIC_SPACEGROUP,"2/m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,TRUE,FALSE,FALSE,FALSE},
{68,"I2/M",12,C_UNIQUE_CELL_3,6,"I 1 1 2/m","I 1 1 2/m","C2h.3","-I 2",MONOCLINIC_SPACEGROUP,"2/m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,TRUE,FALSE,FALSE,FALSE},
{69,"B2/M",12,A_UNIQUE_CELL_1,7,"B 2/m 1 1","B 2/m 1 1","C2h.3","-B 2x",MONOCLINIC_SPACEGROUP,"2/m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,TRUE,FALSE,FALSE,FALSE},
{70,"C2/M",12,A_UNIQUE_CELL_2,8,"C 2/m 1 1","C 2/m 1 1","C2h.3","-C 2x",MONOCLINIC_SPACEGROUP,"2/m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{71,"I2/M",12,A_UNIQUE_CELL_3,9,"I 2/m 1 1","I 2/m 1 1","C2h.3","-I 2x",MONOCLINIC_SPACEGROUP,"2/m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,TRUE,FALSE,FALSE,FALSE},

{72,"P2/C",13,B_UNIQUE_CELL_1,1,"P 1 2/c 1","P 1 2/c 1","C2h.4","-P 2yc",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,TRUE},
{73,"P2/N",13,B_UNIQUE_CELL_2,2,"P 1 2/n 1","P 1 2/n 1","C2h.4","-P 2yac",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},
{74,"P2/A",13,B_UNIQUE_CELL_3,3,"P 1 2/a 1","P 1 2/a 1","C2h.4","-P 2ya",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},
{75,"P2/A",13,C_UNIQUE_CELL_1,4,"P 1 1 2/a","P 1 1 2/a","C2h.4","-P 2a",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},
{76,"P2/N",13,C_UNIQUE_CELL_2,5,"P 1 1 2/n","P 1 1 2/n","C2h.4","-P 2ab",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},
{77,"P2/B",13,C_UNIQUE_CELL_3,6,"P 1 1 2/b","P 1 1 2/b","C2h.4","-P 2b",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},
{78,"P2/B",13,A_UNIQUE_CELL_1,7,"P 2/b 1 1","P 2/b 1 1","C2h.4","-P 2xb",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},
{79,"P2/N",13,A_UNIQUE_CELL_2,8,"P 2/n 1 1","P 2/n 1 1","C2h.4","-P 2xbc",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},
{80,"P2/C",13,A_UNIQUE_CELL_3,9,"P 2/c 1 1","P 2/c 1 1","C2h.4","-P 2xc",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},

{81,"P21/C",14,B_UNIQUE_CELL_1,1,"P 1 21/c 1","P 1 21/c 1","C2h.5","-P 2ybc",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,TRUE},
{82,"P21/N",14,B_UNIQUE_CELL_2,2,"P 1 21/n 1","P 1 21/n 1","C2h.5","-P 2yn",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},
{83,"P21/A",14,B_UNIQUE_CELL_3,3,"P 1 21/a 1","P 1 21/a 1","C2h.5","-P 2yab",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},
{84,"P21/A",14,C_UNIQUE_CELL_1,4,"P 1 1 21/a","P 1 1 21/a","C2h.5","-P 2ac",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},
{85,"P21/N",14,C_UNIQUE_CELL_2,5,"P 1 1 21/n","P 1 1 21/n","C2h.5","-P 2n",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},
{86,"P21/B",14,C_UNIQUE_CELL_3,6,"P 1 1 21/b","P 1 1 21/b","C2h.5","-P 2bc",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},
{87,"P21/B",14,A_UNIQUE_CELL_1,7,"P 21/b 1 1","P 21/b 1 1","C2h.5","-P 2xab",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},
{88,"P21/N",14,A_UNIQUE_CELL_2,8,"P 21/n 1 1","P 21/n 1 1","C2h.5","-P 2xn",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},
{89,"P21/C",14,A_UNIQUE_CELL_3,9,"P 21/c 1 1","P 21/c 1 1","C2h.5","-P 2xac",MONOCLINIC_SPACEGROUP,"2/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,FALSE,FALSE,FALSE},

{90,"C2/C",15,B_UNIQUE_CELL_1,1,"C 1 2/c 1","C 1 2/c 1","C2h.6","-C 2yc",MONOCLINIC_SPACEGROUP,"2/m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,TRUE,FALSE,FALSE,TRUE},
{91,"A2/N",15,B_UNIQUE_CELL_2,2,"A 1 2/n 1","A 1 2/n 1","C2h.6","-A 2yab",MONOCLINIC_SPACEGROUP,"2/m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,TRUE,FALSE,FALSE,FALSE},
{92,"I2/A",15,B_UNIQUE_CELL_3,3,"I 1 2/a 1","I 1 2/a 1","C2h.6","-I 2ya",MONOCLINIC_SPACEGROUP,"2/m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,TRUE,FALSE,FALSE,FALSE},
{93,"A2/A",15,mB_UNIQUE_CELL_1,10,"A 1 2/a 1","A 1 2/a 1","C2h.6","-A 2ya",MONOCLINIC_SPACEGROUP,"2/m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,TRUE,FALSE,FALSE,FALSE},
{94,"C2/N",15,mB_UNIQUE_CELL_2,11,"C 1 2/n 1","C 1 2/n 1","C2h.6","-C 2yac",MONOCLINIC_SPACEGROUP,"2/m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{95,"I2/C",15,mB_UNIQUE_CELL_3,12,"I 1 2/c 1","I 1 2/c 1","C2h.6","-I 2yc",MONOCLINIC_SPACEGROUP,"2/m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,TRUE,FALSE,FALSE,FALSE},
{96,"A2/A",15,C_UNIQUE_CELL_1,4,"A 1 1 2/a","A 1 1 2/a","C2h.6","-A 2a",MONOCLINIC_SPACEGROUP,"2/m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,TRUE,FALSE,FALSE,FALSE},
{97,"B2/N",15,C_UNIQUE_CELL_2,5,"B 1 1 2/n","B 1 1 2/n","C2h.6","-B 2ab",MONOCLINIC_SPACEGROUP,"2/m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,TRUE,FALSE,FALSE,FALSE},
{98,"I2/B",15,C_UNIQUE_CELL_3,6,"I 1 1 2/b","I 1 1 2/b","C2h.6","-I 2b",MONOCLINIC_SPACEGROUP,"2/m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,TRUE,FALSE,FALSE,FALSE},
{99,"B2/B",15,mC_UNIQUE_CELL_1,13,"B 1 1 2/b","B 1 1 2/b","C2h.6","-B 2b",MONOCLINIC_SPACEGROUP,"2/m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,TRUE,FALSE,FALSE,FALSE},
{100,"A2/N",15,mC_UNIQUE_CELL_2,14,"A 1 1 2/n","A 1 1 2/n","C2h.6","-A 2ab",MONOCLINIC_SPACEGROUP,"2/m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,TRUE,FALSE,FALSE,FALSE},
{101,"I2/A",15,mC_UNIQUE_CELL_3,15,"I 1 1 2/a","I 1 1 2/a","C2h.6","-I 2a",MONOCLINIC_SPACEGROUP,"2/m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,TRUE,FALSE,FALSE,FALSE},
{102,"B2/B",15,A_UNIQUE_CELL_1,7,"B 2/b 1 1","B 2/b 1 1","C2h.6","-B 2xb",MONOCLINIC_SPACEGROUP,"2/m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,TRUE,FALSE,FALSE,FALSE},
{103,"C2/N",15,A_UNIQUE_CELL_2,8,"C 2/n 1 1","C 2/n 1 1","C2h.6","-C 2xac",MONOCLINIC_SPACEGROUP,"2/m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{104,"I2/C",15,A_UNIQUE_CELL_3,9,"I 2/c 1 1","I 2/c 1 1","C2h.6","-I 2xc",MONOCLINIC_SPACEGROUP,"2/m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,TRUE,FALSE,FALSE,FALSE},
{105,"C2/C",15,mA_UNIQUE_CELL_1,16,"C 2/c 1 1","C 2/c 1 1","C2h.6","-C 2xc",MONOCLINIC_SPACEGROUP,"2/m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{106,"B2/N",15,mA_UNIQUE_CELL_2,17,"B 2/n 1 1","B 2/n 1 1","C2h.6","-B 2xab",MONOCLINIC_SPACEGROUP,"2/m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,TRUE,FALSE,FALSE,FALSE},
{107,"I2/B",15,mA_UNIQUE_CELL_3,18,"I 2/b 1 1","I 2/b 1 1","C2h.6","-I 2xb",MONOCLINIC_SPACEGROUP,"2/m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,TRUE,FALSE,FALSE,FALSE},

{108,"P222",16,ORIGIN_1_ABC,1,"P 2 2 2","P 2 2 2","D2.1","P 2 2",ORTHORHOMBIC_SPACEGROUP,"2 2 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,TRUE,FALSE,TRUE},

{109,"P2221",17,ORIGIN_1_ABC,1,"P 2 2 21","P 2 2 21","D2.2","P 2c 2",ORTHORHOMBIC_SPACEGROUP,"2 2 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,TRUE,FALSE,TRUE},
{110,"P2122",17,ORIGIN_1_CAB,2,"P 21 2 2","P 21 2 2","D2.2","P 2a 2a",ORTHORHOMBIC_SPACEGROUP,"2 2 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,TRUE,FALSE,FALSE},
{111,"P2212",17,ORIGIN_1_BCA,3,"P 2 21 2","P 2 21 2","D2.2","P 2 2b",ORTHORHOMBIC_SPACEGROUP,"2 2 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,TRUE,FALSE,FALSE},

{112,"P21221",18,ORIGIN_1_ABC,1,"P 21 21 2","P 21 21 2","D2.3","P 2 2ab",ORTHORHOMBIC_SPACEGROUP,"2 2 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,TRUE,FALSE,TRUE},
{113,"P22121",18,ORIGIN_1_CAB,2,"P 2 21 21","P 2 21 21","D2.3","P 2bc 2",ORTHORHOMBIC_SPACEGROUP,"2 2 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,TRUE,FALSE,FALSE},
{114,"P21221",18,ORIGIN_1_BCA,3,"P 21 2 21","P 21 2 21","D2.3","P 2ac 2ac",ORTHORHOMBIC_SPACEGROUP,"2 2 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,TRUE,FALSE,FALSE},

{115,"P212121",19,ORIGIN_1_ABC,1,"P 21 21 21","P 21 21 21","D2.4","P 2ac 2ab",ORTHORHOMBIC_SPACEGROUP,"2 2 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,TRUE,TRUE,FALSE,TRUE},

{116,"C2212",20,ORIGIN_1_ABC,1,"C 2 2 21","C 2 2 21","D2.5","C 2c 2",ORTHORHOMBIC_SPACEGROUP,"2 2 2",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,TRUE,TRUE,FALSE,TRUE},
{117,"A2122",20,ORIGIN_1_CAB,2,"A 21 2 2","A 21 2 2","D2.5","A 2a 2a",ORTHORHOMBIC_SPACEGROUP,"2 2 2",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,TRUE,TRUE,FALSE,FALSE},
{118,"B2212",20,ORIGIN_1_BCA,3,"B 2 21 2","B 2 21 2","D2.5","B 2 2b",ORTHORHOMBIC_SPACEGROUP,"2 2 2",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,TRUE,TRUE,FALSE,FALSE},

{119,"C222",21,ORIGIN_1_ABC,1,"C 2 2 2","C 2 2 2","D2.6","C 2 2",ORTHORHOMBIC_SPACEGROUP,"2 2 2",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,FALSE,TRUE,FALSE,TRUE},
{120,"A222",21,ORIGIN_1_CAB,2,"A 2 2 2","A 2 2 2","D2.6","A 2 2",ORTHORHOMBIC_SPACEGROUP,"2 2 2",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,FALSE,TRUE,FALSE,FALSE},
{121,"B222",21,ORIGIN_1_BCA,3,"B 2 2 2","B 2 2 2","D2.6","B 2 2",ORTHORHOMBIC_SPACEGROUP,"2 2 2",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,FALSE,TRUE,FALSE,FALSE},

{122,"F222",22,ORIGIN_1_ABC,1,"F 2 2 2","F 2 2 2","D2.7","F 2 2",ORTHORHOMBIC_SPACEGROUP,"2 2 2",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},16,FALSE,TRUE,FALSE,TRUE},

{123,"I222",23,ORIGIN_1_ABC,1,"I 2 2 2","I 2 2 2","D2.8","I 2 2",ORTHORHOMBIC_SPACEGROUP,"2 2 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,FALSE,TRUE,FALSE,TRUE},

{124,"I212121",24,ORIGIN_1_ABC,1,"I 21 21 21","I 21 21 21","D2.9","I 2b 2c",ORTHORHOMBIC_SPACEGROUP,"2 2 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,FALSE,TRUE,FALSE,TRUE},

{125,"PMM2",25,ORIGIN_1_ABC,1,"P m m 2","P m m 2","C2v.1","P 2 -2",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,TRUE},
{126,"P2MM",25,ORIGIN_1_CAB,2,"P 2 m m","P 2 m m","C2v.1","P -2 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{127,"PM2M",25,ORIGIN_1_BCA,3,"P m 2 m","P m 2 m","C2v.1","P -2 -2",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},

{128,"PMC21",26,ORIGIN_1_ABC,1,"P m c 21","P m c 21","C2v.2","P 2c -2",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,TRUE},
{129,"PCM21",26,ORIGIN_1_BACm,4,"P c m 21","P c m 21","C2v.2","P 2c -2c",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{130,"P21MA",26,ORIGIN_1_CAB,2,"P 21 m a","P 21 m a","C2v.2","P -2a 2a",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{131,"P21AM",26,ORIGIN_1_CmBA,5,"P 21 a m","P 21 a m","C2v.2","P -2 2a",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{132,"PB21M",26,ORIGIN_1_BCA,3,"P b 21 m","P b 21 m","C2v.2","P -2 -2b",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{133,"PM21B",26,ORIGIN_1_ACmB,6,"P m 21 b","P m 21 b","C2v.2","P -2b -2",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},

{134,"P2AA",27,ORIGIN_1_ABC,1,"P c c 2","P c c 2","C2v.3","P 2 -2c",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,TRUE},
{135,"P2AA",27,ORIGIN_1_CAB,2,"P 2 a a","P 2 a a","C2v.3","P -2a 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{136,"PB2B",27,ORIGIN_1_BCA,3,"P b 2 b","P b 2 b","C2v.3","P -2b -2b",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},

{137,"PMA2",28,ORIGIN_1_ABC,1,"P m a 2","P m a 2","C2v.4","P 2 -2a",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,TRUE},
{138,"PBM2",28,ORIGIN_1_BACm,4,"P b m 2","P b m 2","C2v.4","P 2 -2b",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{139,"P2MB",28,ORIGIN_1_CAB,2,"P 2 m b","P 2 m b","C2v.4","P -2b 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{140,"P2CM",28,ORIGIN_1_CmBA,5,"P 2 c m","P 2 c m","C2v.4","P -2c 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{141,"PC2M",28,ORIGIN_1_BCA,3,"P c 2 m","P c 2 m","C2v.4","P -2c -2c",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{142,"PM2A",28,ORIGIN_1_ACmB,6,"P m 2 a","P m 2 a","C2v.4","P -2a -2a",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},

{143,"PCA21",29,ORIGIN_1_ABC,1,"P c a 21","P c a 21","C2v.5","P 2c -2ac",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,TRUE},
{144,"PBC21",29,ORIGIN_1_BACm,4,"P b c 21","P b c 21","C2v.5","P 2c -2b",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{145,"P21AB",29,ORIGIN_1_CAB,2,"P 21 a b","P 21 a b","C2v.5","P -2b 2a",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{146,"P21CA",29,ORIGIN_1_CmBA,5,"P 21 c a","P 21 c a","C2v.5","P -2ac 2a",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{147,"PC21B",29,ORIGIN_1_BCA,3,"P c 21 b","P c 21 b","C2v.5","P -2bc -2c",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{148,"PB21A",29,ORIGIN_1_ACmB,6,"P b 21 a","P b 21 a","C2v.5","P -2a -2ab",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},

{149,"PNC2",30,ORIGIN_1_ABC,1,"P n c 2","P n c 2","C2v.6","P 2 -2bc",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,TRUE},
{150,"PCN2",30,ORIGIN_1_BACm,4,"P c n 2","P c n 2","C2v.6","P 2 -2ac",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{151,"P2NA",30,ORIGIN_1_CAB,2,"P 2 n a","P 2 n a","C2v.6","P -2ac 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{152,"P2AN",30,ORIGIN_1_CmBA,5,"P 2 a n","P 2 a n","C2v.6","P -2ab 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{153,"PB2N",30,ORIGIN_1_BCA,3,"P b 2 n","P b 2 n","C2v.6","P -2ab -2ab",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{154,"PN2B",30,ORIGIN_1_ACmB,6,"P n 2 b","P n 2 b","C2v.6","P -2bc -2bc",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},

{155,"PMN21",31,ORIGIN_1_ABC,1,"P m n 21","P m n 21","C2v.7","P 2ac -2",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,TRUE},
{156,"PNM21",31,ORIGIN_1_BACm,4,"P n m 21","P n m 21","C2v.7","P 2bc -2bc",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{157,"P21MN",31,ORIGIN_1_CAB,2,"P 21 m n","P 21 m n","C2v.7","P -2ab 2ab",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{158,"P21NM",31,ORIGIN_1_CmBA,5,"P 21 n m","P 21 n m","C2v.7","P -2 2ac",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{159,"PN21M",31,ORIGIN_1_BCA,3,"P n 21 m","P n 21 m","C2v.7","P -2 -2bc",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{160,"PM21N",31,ORIGIN_1_ACmB,6,"P m 21 n","P m 21 n","C2v.7","P -2ab -2",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},

{161,"PBA2",32,ORIGIN_1_ABC,1,"P b a 2","P b a 2","C2v.8","P 2 -2ab",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,TRUE},
{162,"P2CB",32,ORIGIN_1_CAB,2,"P 2 c b","P 2 c b","C2v.8","P -2bc 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{163,"PC2A",32,ORIGIN_1_BCA,3,"P c 2 a","P c 2 a","C2v.8","P -2ac -2ac",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},

{164,"PNA21",33,ORIGIN_1_ABC,1,"P n a 21","P n a 21","C2v.9","P 2c -2n",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,TRUE},
{165,"PBN21",33,ORIGIN_1_BACm,4,"P b n 21","P b n 21","C2v.9","P 2c -2ab",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{166,"P21NB",33,ORIGIN_1_CAB,2,"P 21 n b","P 21 n b","C2v.9","P -2bc 2a",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{167,"P21CN",33,ORIGIN_1_CmBA,5,"P 21 c n","P 21 c n","C2v.9","P -2n 2a",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{168,"PC21N",33,ORIGIN_1_BCA,3,"P c 21 n","P c 21 n","C2v.9","P -2n -2ac",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{169,"PN21A",33,ORIGIN_1_ACmB,6,"P n 21 a","P n 21 a","C2v.9","P -2ac -2n",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},

{170,"PNN2",34,ORIGIN_1_ABC,1,"P n n 2","P n n 2","C2v.10","P 2 -2n",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,TRUE},
{171,"P2NN",34,ORIGIN_1_CAB,2,"P 2 n n","P 2 n n","C2v.10","P -2n 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},
{172,"PN2N",34,ORIGIN_1_BCA,3,"P n 2 n","P n 2 n","C2v.10","P -2n -2n",ORTHORHOMBIC_SPACEGROUP,"m m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,FALSE},

{173,"CMM2",35,ORIGIN_1_ABC,1,"C m m 2","C m m 2","C2v.11","C 2 -2",ORTHORHOMBIC_SPACEGROUP,"m m 2",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,FALSE,FALSE,FALSE,TRUE},
{174,"A2MM",35,ORIGIN_1_CAB,2,"A 2 m m","A 2 m m","C2v.11","A -2 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,FALSE,FALSE,FALSE,FALSE},
{175,"BM2M",35,ORIGIN_1_BCA,3,"B m 2 m","B m 2 m","C2v.11","B -2 -2",ORTHORHOMBIC_SPACEGROUP,"m m 2",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,FALSE,FALSE,FALSE,FALSE},

{176,"CMC21",36,ORIGIN_1_ABC,1,"C m c 21","C m c 21","C2v.12","C 2c -2",ORTHORHOMBIC_SPACEGROUP,"m m 2",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,FALSE,FALSE,FALSE,TRUE},
{177,"CCM21",36,ORIGIN_1_BACm,4,"C c m 21","C c m 21","C2v.12","C 2c -2c",ORTHORHOMBIC_SPACEGROUP,"m m 2",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,FALSE,FALSE,FALSE,FALSE},
{178,"A21MA",36,ORIGIN_1_CAB,2,"A 21 m a","A 21 m a","C2v.12","A -2a 2a",ORTHORHOMBIC_SPACEGROUP,"m m 2",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,FALSE,FALSE,FALSE,FALSE},
{179,"A21AM",36,ORIGIN_1_CmBA,5,"A 21 a m","A 21 a m","C2v.12","A -2 2a",ORTHORHOMBIC_SPACEGROUP,"m m 2",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,FALSE,FALSE,FALSE,FALSE},
{180,"BB21M",36,ORIGIN_1_BCA,3,"B b 21 m","B b 21 m","C2v.12","B -2 -2b",ORTHORHOMBIC_SPACEGROUP,"m m 2",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,FALSE,FALSE,FALSE,FALSE},
{181,"BM21B",36,ORIGIN_1_ACmB,6,"B m 21 b","B m 21 b","C2v.12","B -2b -2",ORTHORHOMBIC_SPACEGROUP,"m m 2",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,FALSE,FALSE,FALSE,FALSE},

{182,"CCC2",37,ORIGIN_1_ABC,1,"C c c 2","C c c 2","C2v.13","C 2 -2c",ORTHORHOMBIC_SPACEGROUP,"m m 2",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,FALSE,FALSE,FALSE,TRUE},
{183,"A2AA",37,ORIGIN_1_CAB,2,"A 2 a a","A 2 a a","C2v.13","A -2a 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,FALSE,FALSE,FALSE,FALSE},
{184,"BB2B",37,ORIGIN_1_BCA,3,"B b 2 b","B b 2 b","C2v.13","B -2b -2b",ORTHORHOMBIC_SPACEGROUP,"m m 2",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,FALSE,FALSE,FALSE,FALSE},

{185,"AMM2",38,ORIGIN_1_ABC,1,"A m m 2","A m m 2","C2v.14","A 2 -2",ORTHORHOMBIC_SPACEGROUP,"m m 2",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,FALSE,FALSE,FALSE,TRUE},
{186,"BMM2",38,ORIGIN_1_BACm,4,"B m m 2","B m m 2","C2v.14","B 2 -2",ORTHORHOMBIC_SPACEGROUP,"m m 2",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,FALSE,FALSE,FALSE,FALSE},
{187,"B2MM",38,ORIGIN_1_CAB,2,"B 2 m m","B 2 m m","C2v.14","B -2 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,FALSE,FALSE,FALSE,FALSE},
{188,"C2MM",38,ORIGIN_1_CmBA,5,"C 2 m m","C 2 m m","C2v.14","C -2 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,FALSE,FALSE,FALSE,FALSE},
{189,"CM2M",38,ORIGIN_1_BCA,3,"C m 2 m","C m 2 m","C2v.14","C -2 -2",ORTHORHOMBIC_SPACEGROUP,"m m 2",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,FALSE,FALSE,FALSE,FALSE},
{190,"AM2M",38,ORIGIN_1_ACmB,6,"A m 2 m","A m 2 m","C2v.14","A -2 -2",ORTHORHOMBIC_SPACEGROUP,"m m 2",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,FALSE,FALSE,FALSE,FALSE},

{191,"ABM2",39,ORIGIN_1_ABC,1,"A b m 2","A b m 2","C2v.15","A 2 -2b",ORTHORHOMBIC_SPACEGROUP,"m m 2",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,FALSE,FALSE,FALSE,TRUE},
{192,"BMA2",39,ORIGIN_1_BACm,4,"B m a 2","B m a 2","C2v.15","B 2 -2a",ORTHORHOMBIC_SPACEGROUP,"m m 2",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,FALSE,FALSE,FALSE,FALSE},
{193,"B2CM",39,ORIGIN_1_CAB,2,"B 2 c m","B 2 c m","C2v.15","B -2a 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,FALSE,FALSE,FALSE,FALSE},
{194,"C2MB",39,ORIGIN_1_CmBA,5,"C 2 m b","C 2 m b","C2v.15","C -2a 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,FALSE,FALSE,FALSE,FALSE},
{195,"CM2A",39,ORIGIN_1_BCA,3,"C m 2 a","C m 2 a","C2v.15","C -2a -2a",ORTHORHOMBIC_SPACEGROUP,"m m 2",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,FALSE,FALSE,FALSE,FALSE},
{196,"AC2M",39,ORIGIN_1_ACmB,6,"A c 2 m","A c 2 m","C2v.15","A -2b -2b",ORTHORHOMBIC_SPACEGROUP,"m m 2",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,FALSE,FALSE,FALSE,FALSE},

{197,"AMA2",40,ORIGIN_1_ABC,1,"A m a 2","A m a 2","C2v.16","A 2 -2a",ORTHORHOMBIC_SPACEGROUP,"m m 2",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,FALSE,FALSE,FALSE,TRUE},
{198,"BBM2",40,ORIGIN_1_BACm,4,"B b m 2","B b m 2","C2v.16","B 2 -2b",ORTHORHOMBIC_SPACEGROUP,"m m 2",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,FALSE,FALSE,FALSE,FALSE},
{199,"B2MB",40,ORIGIN_1_CAB,2,"B 2 m b","B 2 m b","C2v.16","B -2b 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,FALSE,FALSE,FALSE,FALSE},
{200,"C2CM",40,ORIGIN_1_CmBA,5,"C 2 c m","C 2 c m","C2v.16","C -2c 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,FALSE,FALSE,FALSE,FALSE},
{201,"CC2M",40,ORIGIN_1_BCA,3,"C c 2 m","C c 2 m","C2v.16","C -2c -2c",ORTHORHOMBIC_SPACEGROUP,"m m 2",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,FALSE,FALSE,FALSE,FALSE},
{202,"AM2A",40,ORIGIN_1_ACmB,6,"A m 2 a","A m 2 a","C2v.16","A -2a -2a",ORTHORHOMBIC_SPACEGROUP,"m m 2",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,FALSE,FALSE,FALSE,FALSE},

{203,"ABA2",41,ORIGIN_1_ABC,1,"A b a 2","A b a 2","C2v.17","A 2 -2ab",ORTHORHOMBIC_SPACEGROUP,"m m 2",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,FALSE,FALSE,FALSE,TRUE},
{204,"BBA2",41,ORIGIN_1_BACm,4,"B b a 2","B b a 2","C2v.17","B 2 -2ab",ORTHORHOMBIC_SPACEGROUP,"m m 2",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,FALSE,FALSE,FALSE,FALSE},
{205,"B2CB",41,ORIGIN_1_CAB,2,"B 2 c b","B 2 c b","C2v.17","B -2ab 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},8,FALSE,FALSE,FALSE,FALSE},
{206,"C2CB",41,ORIGIN_1_CmBA,5,"C 2 c b","C 2 c b","C2v.17","C -2ac 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,FALSE,FALSE,FALSE,FALSE},
{207,"CC2A",41,ORIGIN_1_BCA,3,"C c 2 a","C c 2 a","C2v.17","C -2ac -2ac",ORTHORHOMBIC_SPACEGROUP,"m m 2",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},8,FALSE,FALSE,FALSE,FALSE},
{208,"AC2A",41,ORIGIN_1_ACmB,6,"A c 2 a","A c 2 a","C2v.17","A -2ab -2ab",ORTHORHOMBIC_SPACEGROUP,"m m 2",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},8,FALSE,FALSE,FALSE,FALSE},

{209,"FMM2",42,ORIGIN_1_ABC,1,"F m m 2","F m m 2","C2v.18","F 2 -2",ORTHORHOMBIC_SPACEGROUP,"m m 2",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},16,FALSE,FALSE,FALSE,TRUE},
{210,"F2MM",42,ORIGIN_1_CAB,2,"F 2 m m","F 2 m m","C2v.18","F -2 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},16,FALSE,FALSE,FALSE,FALSE},
{211,"FM2M",42,ORIGIN_1_BCA,3,"F m 2 m","F m 2 m","C2v.18","F -2 -2",ORTHORHOMBIC_SPACEGROUP,"m m 2",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},16,FALSE,FALSE,FALSE,FALSE},

{212,"FDD2",43,ORIGIN_1_ABC,1,"F d d 2","F d d 2","C2v.19","F 2 -2d",ORTHORHOMBIC_SPACEGROUP,"m m 2",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},16,FALSE,FALSE,FALSE,TRUE},
{213,"F2DD",43,ORIGIN_1_CAB,2,"F 2 d d","F 2 d d","C2v.19","F -2d 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},16,FALSE,FALSE,FALSE,FALSE},
{214,"FD2D",43,ORIGIN_1_BCA,3,"F d 2 d","F d 2 d","C2v.19","F -2d -2d",ORTHORHOMBIC_SPACEGROUP,"m m 2",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},16,FALSE,FALSE,FALSE,FALSE},

{215,"IMM2",44,ORIGIN_1_ABC,1,"I m m 2","I m m 2","C2v.20","I 2 -2",ORTHORHOMBIC_SPACEGROUP,"m m 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8},
{216,"I2MM",44,ORIGIN_1_CAB,2,"I 2 m m","I 2 m m","C2v.20","I -2 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8},
{217,"IM2M",44,ORIGIN_1_BCA,3,"I m 2 m","I m 2 m","C2v.20","I -2 -2",ORTHORHOMBIC_SPACEGROUP,"m m 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8},

{218,"IBA2",45,ORIGIN_1_ABC,1,"I b a 2","I b a 2","C2v.21","I 2 -2c",ORTHORHOMBIC_SPACEGROUP,"m m 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,FALSE,FALSE,FALSE,TRUE},
{219,"I2CB",45,ORIGIN_1_CAB,2,"I 2 c b","I 2 c b","C2v.21","I -2a 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,FALSE,FALSE,FALSE,FALSE},
{220,"IC2A",45,ORIGIN_1_BCA,3,"I c 2 a","I c 2 a","C2v.21","I -2b -2b",ORTHORHOMBIC_SPACEGROUP,"m m 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,FALSE,FALSE,FALSE,FALSE},

{221,"IMA2",46,ORIGIN_1_ABC,1,"I m a 2","I m a 2","C2v.22","I 2 -2a",ORTHORHOMBIC_SPACEGROUP,"m m 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,FALSE,FALSE,FALSE,TRUE},
{222,"IBM2",46,ORIGIN_1_BACm,4,"I b m 2","I b m 2","C2v.22","I 2 -2b",ORTHORHOMBIC_SPACEGROUP,"m m 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,FALSE,FALSE,FALSE,FALSE},
{223,"I2MB",46,ORIGIN_1_CAB,2,"I 2 m b","I 2 m b","C2v.22","I -2b 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,FALSE,FALSE,FALSE,FALSE},
{224,"I2MB",46,ORIGIN_1_CmBA,5,"I 2 c m","I 2 c m","C2v.22","I -2c 2",ORTHORHOMBIC_SPACEGROUP,"m m 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,FALSE,FALSE,FALSE,FALSE},
{225,"IC2M",46,ORIGIN_1_BCA,3,"I c 2 m","I c 2 m","C2v.22","I -2c -2c",ORTHORHOMBIC_SPACEGROUP,"m m 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,FALSE,FALSE,FALSE,FALSE},
{226,"IM2A",46,ORIGIN_1_ACmB,6,"I m 2 a","I m 2 a","C2v.22","I -2a -2a",ORTHORHOMBIC_SPACEGROUP,"m m 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,FALSE,FALSE,FALSE,FALSE},

{227,"PMMM",47,ORIGIN_1_ABC,1,"P m m m","P 2/m 2/m 2/m","D2h.1","-P 2 2",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},

{228,"PNNN",48,ORIGIN_1_ABC,1,"P n n n:1","P 2/n 2/n 2/n:1","D2h.2","P 2 2 -1n",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},
{229,"PNNN",48,ORIGIN_2_ABC,2,"P n n n:2","P 2/n 2/n 2/n:2","D2h.2","-P 2ab 2bc",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},

{230,"PCCM",49,ORIGIN_1_ABC,1,"P c c m","P 2/c 2/c 2/m","D2h.3","-P 2 2c",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},
{231,"PMAA",49,ORIGIN_1_CAB,2,"P m a a","P 2/m 2/a 2/a","D2h.3","-P 2a 2",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{232,"PBMB",49,ORIGIN_1_BCA,3,"P b m b","P 2/b 2/m 2/b","D2h.3","-P 2b 2b",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},

{233,"PBAN",50,ORIGIN_1_ABC,1,"P b a n:1","P 2/b 2/a 2/n:1","D2h.4","P 2 2 -1ab",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},
{234,"PBAN",50,ORIGIN_2_ABC,4,"P b a n:2","P 2/b 2/a 2/n:2","D2h.4","-P 2ab 2b",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{235,"PNCB",50,ORIGIN_1_CAB,2,"P n c b:1","P 2/n 2/c 2/b:1","D2h.4","P 2 2 -1bc",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{236,"PNCB",50,ORIGIN_2_CAB,5,"P n c b:2","P 2/n 2/c 2/b:2","D2h.4","-P 2b 2bc",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{237,"PCNA",50,ORIGIN_1_BCA,3,"P c n a:1","P 2/c 2/n 2/a:1","D2h.4","P 2 2 -1ac",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{238,"PCNA",50,ORIGIN_2_BCA,6,"P c n a:2","P 2/c 2/n 2/a:2","D2h.4","-P 2a 2c",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},

{239,"PMMA",51,ORIGIN_1_ABC,1,"P m m a","P 21/m 2/m 2/a","D2h.5","-P 2a 2a",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},
{240,"PMMB",51,ORIGIN_1_BACm,4,"P m m b","P 2/m 21/m 2/b","D2h.5","-P 2b 2",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{241,"PBMM",51,ORIGIN_1_CAB,2,"P b m m","P 2/b 21/m 2/m","D2h.5","-P 2 2b",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{242,"PCMM",51,ORIGIN_1_CmBA,5,"P c m m","P 2/c 2/m 21/m","D2h.5","-P 2c 2c",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{243,"PMCM",51,ORIGIN_1_BCA,3,"P m c m","P 2/m 2/c 21/m","D2h.5","-P 2c 2",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{244,"PMAM",51,ORIGIN_1_ACmB,6,"P m a m","P 21/m 2/a 2/m","D2h.5","-P 2 2a",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},

{245,"PNNA",52,ORIGIN_1_ABC,1,"P n n a","P 2/n 21/n 2/a","D2h.6","-P 2a 2bc",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},
{246,"PNNB",52,ORIGIN_1_BACm,4,"P n n b","P 21/n 2/n 2/b","D2h.6","-P 2b 2n",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{247,"PBNN",52,ORIGIN_1_CAB,2,"P b n n","P 2/b 2/n 21/n","D2h.6","-P 2n 2b",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{248,"PCNN",52,ORIGIN_1_CmBA,5,"P c n n","P 2/c 21/n 2/n","D2h.6","-P 2ab 2c",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{249,"PNCN",52,ORIGIN_1_BCA,3,"P n c n","P 21/n 2/c 2/n","D2h.6","-P 2ab 2n",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{250,"PNAN",52,ORIGIN_1_ACmB,6,"P n a n","P 2/n 2/a 21/n","D2h.6","-P 2n 2bc",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},

{251,"PMNA",53,ORIGIN_1_ABC,1,"P m n a","P 2/m 2/n 21/a","D2h.7","-P 2ac 2",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},
{252,"PNMB",53,ORIGIN_1_BACm,4,"P n m b","P 2/n 2/m 21/b","D2h.7","-P 2bc 2bc",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{253,"PBMN",53,ORIGIN_1_CAB,2,"P b m n","P 21/b 2/m 2/n","D2h.7","-P 2ab 2ab",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{254,"PCNM",53,ORIGIN_1_CmBA,5,"P c n m","P 21/c 2/n 2/m","D2h.7","-P 2 2ac",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{255,"PNCM",53,ORIGIN_1_BCA,3,"P n c m","P 2/n 21/c 2/m","D2h.7","-P 2 2bc",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{256,"PMAN",53,ORIGIN_1_ACmB,6,"P m a n","P 2/m 21/a 2/n","D2h.7","-P 2ab 2",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},

{257,"PCCA",54,ORIGIN_1_ABC,1,"P c c a","P 21/c 2/c 2/a","D2h.8","-P 2a 2ac",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},
{258,"PCCB",54,ORIGIN_1_BACm,4,"P c c b","P 2/c 21/c 2/b","D2h.8","-P 2b 2c",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{259,"PBAA",54,ORIGIN_1_CAB,2,"P b a a","P 2/b 21/a 2/a","D2h.8","-P 2a 2b",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{260,"PCAA",54,ORIGIN_1_CmBA,5,"P c a a","P 2/c 2/a 21/a","D2h.8","-P 2ac 2c",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{261,"PBCB",54,ORIGIN_1_BCA,3,"P b c b","P 2/b 2/c 21/b","D2h.8","-P 2bc 2b",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{262,"PBAB",54,ORIGIN_1_ACmB,6,"P b a b","P 21/b 2/a 2/b","D2h.8","-P 2b 2ab",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},

{263,"PBAM",55,ORIGIN_1_ABC,1,"P b a m","P 21/b 21/a 2/m","D2h.9","-P 2 2ab",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},
{264,"PMCB",55,ORIGIN_1_CAB,2,"P m c b","P 2/m 21/c 21/b","D2h.9","-P 2bc 2",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{265,"PCMA",55,ORIGIN_1_BCA,3,"P c m a","P 21/c 2/m 21/a","D2h.9","-P 2ac 2ac",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},

{266,"PCCN",56,ORIGIN_1_ABC,1,"P c c n","P 21/c 21/c 2/n","D2h.10","-P 2ab 2ac",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},
{267,"PNAA",56,ORIGIN_1_CAB,2,"P n a a","P 2/n 21/a 21/a","D2h.10","-P 2ac 2bc",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{268,"PBNB",56,ORIGIN_1_BCA,3,"P b n b","P 21/b 2/n 21/b","D2h.10","-P 2bc 2ab",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},

{269,"PBCM",57,ORIGIN_1_ABC,1,"P b c m","P 2/b 21/c 21/m","D2h.11","-P 2c 2b",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},
{270,"PCAM",57,ORIGIN_1_BACm,4,"P c a m","P 21/c 2/a 21/m","D2h.11","-P 2c 2ac",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{271,"PMCA",57,ORIGIN_1_CAB,2,"P m c a","P 21/m 2/c 21/a","D2h.11","-P 2ac 2a",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{272,"PMAB",57,ORIGIN_1_CmBA,5,"P m a b","P 21/m 21/a 2/b","D2h.11","-P 2b 2a",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{273,"PBMA",57,ORIGIN_1_BCA,3,"P b m a","P 21/b 21/m 2/a","D2h.11","-P 2a 2ab",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{274,"PCMB",57,ORIGIN_1_ACmB,6,"P c m b","P 2/c 21/m 21/b","D2h.11","-P 2bc 2c",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},

{275,"PNNM",58,ORIGIN_1_ABC,1,"P n n m","P 21/n 21n 2/m","D2h.12","-P 2 2n",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},
{276,"PMNM",58,ORIGIN_1_CAB,2,"P m n n","P 2/m 21/n 21/n","D2h.12","-P 2n 2",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{277,"PNMN",58,ORIGIN_1_BCA,3,"P n m n","P 21/n 2/m 21/n","D2h.12","-P 2n 2n",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},

{278,"PMMN",59,ORIGIN_1_ABC,1,"P m m n:1","P 21/m 21/m 2/n:1","D2h.13","P 2 2ab -1ab",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},
{279,"PMMN",59,ORIGIN_2_ABC,2,"P m m n:2","P 21/m 21/m 2/n:2","D2h.13","-P 2ab 2a",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{280,"PNMM",59,ORIGIN_1_CAB,3,"P n m m:1","P 2/n 21/m 21/m:1","D2h.13","P 2bc 2 -1bc",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{281,"PNMM",59,ORIGIN_2_CAB,4,"P n m m:2","P 2/n 21/m 21/m:2","D2h.13","-P 2c 2bc",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{282,"PMNM",59,ORIGIN_1_BCA,5,"P m n m:1","P 21/m 2/n 21/m:1","D2h.13","P 2ac 2ac -1ac",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{283,"PMNM",59,ORIGIN_2_BCA,6,"P m n m:2","P 21/m 2/n 21/m:2","D2h.13","-P 2c 2a",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},

{284,"PBCN",60,ORIGIN_1_ABC,1,"P b c n","P 21/b 2/c 21/n","D2h.14","-P 2n 2ab",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},
{285,"PCAN",60,ORIGIN_1_BACm,4,"P c a n","P 2/c 21/a 21/n","D2h.14","-P 2n 2c",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{286,"PNCA",60,ORIGIN_1_CAB,2,"P n c a","P 21/n 21/a 2/b","D2h.14","-P 2a 2n",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{287,"PNAB",60,ORIGIN_1_CmBA,5,"P n a b","P 21/n 2/a 21/b","D2h.14","-P 2bc 2n",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{288,"PBNA",60,ORIGIN_1_BCA,3,"P b n a","P 2/b 21/n 21/a","D2h.14","-P 2ac 2b",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{289,"PCNB",60,ORIGIN_1_ACmB,6,"P c n b","P 21/c 21/n 2/b","D2h.14","-P 2b 2ac",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},

{290,"PBCA",61,ORIGIN_1_ABC,1,"P b c a","P 21/b 21/c 21/a","D2h.15","-P 2ac 2ab",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},
{291,"PCAB",61,ORIGIN_1_BACm,2,"P c a b","P 21/c 21/a 21/b","D2h.15","-P 2bc 2ac",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},

{292,"PNMA",62,ORIGIN_1_ABC,1,"P n m a","P 21/n 21/m 21/a","D2h.16","-P 2ac 2n",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},
{293,"PMNB",62,ORIGIN_1_BACm,4,"P m n b","P 21/m 21/n 21/b","D2h.16","-P 2bc 2a",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{294,"PBNM",62,ORIGIN_1_CAB,2,"P b n m","P 21/b 21/n 21/m","D2h.16","-P 2c 2ab",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{295,"PMCN",62,ORIGIN_1_CmBA,5,"P c m n","P 21/c 21/m 21/n","D2h.16","-P 2n 2ac",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{296,"PMCN",62,ORIGIN_1_BCA,3,"P m c n","P 21/m 21/c 21/n","D2h.16","-P 2n 2a",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},
{297,"PNAM",62,ORIGIN_1_ACmB,6,"P n a m","P 21/n 21/a 21/m","D2h.16","-P 2c 2n",ORTHORHOMBIC_SPACEGROUP,"m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},

{298,"CMCM",63,ORIGIN_1_ABC,1,"C m c m","C 2/m 2/c 21/m","D2h.17","-C 2c 2",ORTHORHOMBIC_SPACEGROUP,"m m m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},16,TRUE,FALSE,FALSE,TRUE},
{299,"CCMM",63,ORIGIN_1_BACm,4,"C c m m","C 2/c 2/m 21/m","D2h.17","-C 2c 2c",ORTHORHOMBIC_SPACEGROUP,"m m m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},16,TRUE,FALSE,FALSE,FALSE},
{300,"AMMA",63,ORIGIN_1_CAB,2,"A m m a","A 21/m 2/m 2/a","D2H.17","-A 2a 2a",ORTHORHOMBIC_SPACEGROUP,"m m m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{301,"AMAM",63,ORIGIN_1_CmBA,5,"A m a m","A 21/m 2/a 2/m","D2h.17","-A 2 2a",ORTHORHOMBIC_SPACEGROUP,"m m m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{302,"BBMM",63,ORIGIN_1_BCA,3,"B b m m","B 2/b 21/m 2/m","D2h.17","-B 2 2b",ORTHORHOMBIC_SPACEGROUP,"m m m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{303,"BMMB",63,ORIGIN_1_ACmB,6,"B m m b","B 2/m 21/m 2/b","D2h.17","-B 2b 2",ORTHORHOMBIC_SPACEGROUP,"m m m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},16,TRUE,FALSE,FALSE,FALSE},

{304,"CMCA",64,ORIGIN_1_ABC,1,"C m c a","C 2/m 2/c 21/a","D2h.18","-C 2ac 2",ORTHORHOMBIC_SPACEGROUP,"m m m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},16,TRUE,FALSE,FALSE,TRUE},
{305,"CCMB",64,ORIGIN_1_BACm,4,"C c m b","C 2/c 2/m 21/b","D2h.18","-C 2ac 2ac",ORTHORHOMBIC_SPACEGROUP,"m m m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},16,TRUE,FALSE,FALSE,FALSE},
{306,"ABMA",64,ORIGIN_1_CAB,2,"A b m a","A 21/b 2/m 2/a","D2h.18","-A 2ab 2ab",ORTHORHOMBIC_SPACEGROUP,"m m m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{307,"ACAM",64,ORIGIN_1_CmBA,5,"A c a m","A 21/c 2/a 2/m","D2h.18","-A 2 2ab",ORTHORHOMBIC_SPACEGROUP,"m m m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{308,"BBCM",64,ORIGIN_1_BCA,3,"B b c m","B 2/b 21/c 2/m","D2h.18","-B 2 2ab",ORTHORHOMBIC_SPACEGROUP,"m m m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{309,"BMAB",64,ORIGIN_1_ACmB,6,"B m a b","B 2/m 21/a 2/b","D2h.18","-B 2ab 2",ORTHORHOMBIC_SPACEGROUP,"m m m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},16,TRUE,FALSE,FALSE,FALSE},

{310,"CMMM",65,ORIGIN_1_ABC,1,"C m m m","C 2/m 2/m 2/m","D2h.19","-C 2 2",ORTHORHOMBIC_SPACEGROUP,"m m m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},16,TRUE,FALSE,FALSE,TRUE},
{311,"AMMM",65,ORIGIN_1_CAB,2,"A m m m","A 2/m 2/m 2/m","D2h.19","-A 2 2",ORTHORHOMBIC_SPACEGROUP,"m m m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{312,"BMMM",65,ORIGIN_1_BCA,3,"B m m m","B 2/m 2/m 2/m","D2h.19","-B 2 2",ORTHORHOMBIC_SPACEGROUP,"m m m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},16,TRUE,FALSE,FALSE,FALSE},

{313,"CCCM",66,ORIGIN_1_ABC,1,"C c c m","C 2/c 2/c 2/m","D2h.20","-C 2 2c",ORTHORHOMBIC_SPACEGROUP,"m m m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},16,TRUE,FALSE,FALSE,TRUE},
{314,"AMAA",66,ORIGIN_1_CAB,2,"A m a a","A 2/m 2/a 2/a","D2h.20","-A 2a 2",ORTHORHOMBIC_SPACEGROUP,"m m m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{315,"BBMB",66,ORIGIN_1_BCA,3,"B b m b","B 2/b 2/m 2/b","D2h.20","-B 2b 2b",ORTHORHOMBIC_SPACEGROUP,"m m m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},16,TRUE,FALSE,FALSE,FALSE},

{316,"CMMA",67,ORIGIN_1_ABC,1,"C m m a","C 2/m 2/m 2/a","D2h.21","-C 2a 2",ORTHORHOMBIC_SPACEGROUP,"m m m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},16,TRUE,FALSE,FALSE,TRUE},
{317,"CMMB",67,ORIGIN_1_BACm,4,"C m m b","C 2/m 2/m 2/b","D2h.21","-C 2a 2a",ORTHORHOMBIC_SPACEGROUP,"m m m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},16,TRUE,FALSE,FALSE,FALSE},
{318,"ABMM",67,ORIGIN_1_CAB,2,"A b m m","A 2/b 2/m 2/m","D2h.21","-A 2b 2b",ORTHORHOMBIC_SPACEGROUP,"m m m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{319,"ACMM",67,ORIGIN_1_CmBA,5,"A c m m","A 2/c 2/m 2/m","D2h.21","-A 2 2b",ORTHORHOMBIC_SPACEGROUP,"m m m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{320,"BMCM",67,ORIGIN_1_BCA,3,"B m c m","B 2/m 2/c 2/m","D2h.21","-B 2 2a",ORTHORHOMBIC_SPACEGROUP,"m m m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{321,"BMAM",67,ORIGIN_1_ACmB,6,"B m a m","B 2/m 2/a 2/m","D2h.21","-B 2a 2",ORTHORHOMBIC_SPACEGROUP,"m m m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},16,TRUE,FALSE,FALSE,FALSE},

{322,"CCCA",68,ORIGIN_1_ABC,1,"C c c a:1","C 2/c 2/c 2/a:1","D2h.22","C 2 2 -1ac",ORTHORHOMBIC_SPACEGROUP,"m m m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},16,TRUE,FALSE,FALSE,TRUE},
{323,"CCCA",68,ORIGIN_2_ABC,2,"C c c a:2","C 2/c 2/c 2/a:2","D2h.22","-C 2a 2ac",ORTHORHOMBIC_SPACEGROUP,"m m m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},16,TRUE,FALSE,FALSE,FALSE},
{324,"CCCB",68,ORIGIN_1_BACm,7,"C c c b:1","C 2/c 2/c 2/b:1","D2h.22","C 2 2 -1ac",ORTHORHOMBIC_SPACEGROUP,"m m m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},16,TRUE,FALSE,FALSE,FALSE},
{325,"CCCB",68,ORIGIN_2_BACm,8,"C c c b:2","C 2/c 2/c 2/b:2","D2h.22","-C 2a 2c",ORTHORHOMBIC_SPACEGROUP,"m m m",C_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.0}},16,TRUE,FALSE,FALSE,FALSE},
{326,"ABAA",68,ORIGIN_1_CAB,3,"A b a a:1","A 2/b 2/a 2/a:1","D2h.22","A 2 2 -1ab",ORTHORHOMBIC_SPACEGROUP,"m m m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{327,"ABAA",68,ORIGIN_2_CAB,4,"A b a a:2","A 2/b 2/a 2/a:2","D2h.22","-A 2a 2b",ORTHORHOMBIC_SPACEGROUP,"m m m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{328,"ACAA",68,ORIGIN_1_CmBA,9,"A c a a:1","A 2/c 2/a 2/a:1","D2h.22","A 2 2 -1ab",ORTHORHOMBIC_SPACEGROUP,"m m m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{329,"ACAA",68,ORIGIN_2_CmBA,10,"A c a a:2","A 2/c 2/a 2/a:2","D2h.22","-A 2ab 2b",ORTHORHOMBIC_SPACEGROUP,"m m m",A_CENTERED,2,{{0.0,0.0,0.0},{0.0,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{330,"BBCB",68,ORIGIN_1_BCA,5,"B b c b:1","B 2/b 2/c 2/b:1","D2h.22","B 2 2 -1ab",ORTHORHOMBIC_SPACEGROUP,"m m m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{331,"BBCB",68,ORIGIN_2_BCA,6,"B b c b:2","B 2/b 2/c 2/b:2","D2h.22","-B 2ab 2b",ORTHORHOMBIC_SPACEGROUP,"m m m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{332,"BBAB",68,ORIGIN_1_ACmB,11,"B b a b:1","B 2/b 2/a 2/b:1","D2h.22","B 2 2 -1ab",ORTHORHOMBIC_SPACEGROUP,"m m m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{333,"BBAB",68,ORIGIN_2_ACmB,12,"B b a b:2","B 2/b 2/a 2/b:2","D2h.22","-B 2b 2ab",ORTHORHOMBIC_SPACEGROUP,"m m m",B_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.0,0.5}},16,TRUE,FALSE,FALSE,FALSE},

{334,"FMMM",69,ORIGIN_1_ABC,1,"F m m m","F 2/m 2/m 2/m","D2h.23","-F 2 2",ORTHORHOMBIC_SPACEGROUP,"m m m",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},32,TRUE,FALSE,FALSE,TRUE},

{335,"FDDD",70,ORIGIN_1_ABC,1,"F d d d:1","F 2/d 2/d 2/d:1","D2h.24","F 2 2 -1d",ORTHORHOMBIC_SPACEGROUP,"m m m",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},32,TRUE,FALSE,FALSE,TRUE},
{336,"FDDD",70,ORIGIN_2_ABC,2,"F d d d:2","F 2/d 2/d 2/d:2","D2h.24","-F 2uv 2vw",ORTHORHOMBIC_SPACEGROUP,"m m m",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},32,TRUE,FALSE,FALSE,FALSE},

{337,"IMMM",71,ORIGIN_1_ABC,1,"I m m m","I 2/m 2/m 2/m","D2h.25","-I 2 2",ORTHORHOMBIC_SPACEGROUP,"m m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,TRUE,FALSE,FALSE,TRUE},

{338,"IBAM",72,ORIGIN_1_ABC,1,"I b a m","I 2/b 2/a 2/m","D2h.26","-I 2 2c",ORTHORHOMBIC_SPACEGROUP,"m m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,TRUE,FALSE,FALSE,TRUE},
{339,"IMCB",72,ORIGIN_1_CAB,2,"I m c b","I 2/m 2/c 2/b","D2h.26","-I 2a 2",ORTHORHOMBIC_SPACEGROUP,"m m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{340,"ICMA",72,ORIGIN_1_BCA,3,"I c m a","I 2/c 2/m 2/a","D2h.26","-I 2b 2b",ORTHORHOMBIC_SPACEGROUP,"m m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},

{341,"IBCA",73,ORIGIN_1_ABC,1,"I b c a","I 21/b 21/c 21/a","D2h.27","-I 2b 2c",ORTHORHOMBIC_SPACEGROUP,"m m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,TRUE,FALSE,FALSE,TRUE},
{342,"ICAB",73,ORIGIN_1_BACm,2,"I c a b","I 21/c 21/a 21/b","D2h.27","-I 2a 2b",ORTHORHOMBIC_SPACEGROUP,"m m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},

{343,"IMMA",74,ORIGIN_1_ABC,1,"I m m a","I 21/m 21/m 21/a","D2h.28","-I 2b 2",ORTHORHOMBIC_SPACEGROUP,"m m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,TRUE,FALSE,FALSE,TRUE},
{344,"IMMB",74,ORIGIN_1_BACm,4,"I m m b","I 21/m 21/m 21/b","D2h.28","-I 2a 2a",ORTHORHOMBIC_SPACEGROUP,"m m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{345,"IBMM",74,ORIGIN_1_CAB,2,"I b m m","I 21/b 21/m 21/m","D2h.28","-I 2c 2c",ORTHORHOMBIC_SPACEGROUP,"m m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{346,"ICMM",74,ORIGIN_1_CmBA,5,"I c m m","I 21/c 21/m 21/m","D2h.28","-I 2 2b",ORTHORHOMBIC_SPACEGROUP,"m m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{347,"IMCM",74,ORIGIN_1_BCA,3,"I m c m","I 21/m 21/c 21/m","D2h.28","-I 2 2a",ORTHORHOMBIC_SPACEGROUP,"m m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},
{348,"IMAM",74,ORIGIN_1_ACmB,6,"I m a m","I 21/m 21/a 21/m","D2h.28","-I 2c 2",ORTHORHOMBIC_SPACEGROUP,"m m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},

{349,"P4",75,ORIGIN_1_ABC,1,"P 4","P 4","C4.1","P 4",TETRAGONAL_SPACEGROUP,"4",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,TRUE,FALSE,TRUE},

{350,"P41",76,ORIGIN_1_ABC,1,"P 41","P 41","C4.2","P 4w",TETRAGONAL_SPACEGROUP,"4",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,TRUE,TRUE,TRUE},

{351,"P42",77,ORIGIN_1_ABC,1,"P 42","P 42","C4.3","P 4c",TETRAGONAL_SPACEGROUP,"4",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,TRUE,FALSE,TRUE},

{352,"P43",78,ORIGIN_1_ABC,1,"P 43","P 43","C4.4","P 4cw",TETRAGONAL_SPACEGROUP,"4",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,TRUE,TRUE,TRUE},

{353,"I4",79,ORIGIN_1_ABC,1,"I 4","I 4","C4.5","I 4",TETRAGONAL_SPACEGROUP,"4",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,FALSE,TRUE,FALSE,TRUE},

{354,"I41",80,ORIGIN_1_ABC,1,"I 41","I 41","C4.6","I 4bw",TETRAGONAL_SPACEGROUP,"4",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,FALSE,TRUE,FALSE},

{355,"P-4",81,ORIGIN_1_ABC,1,"P -4","P -4","S4.1","P -4",TETRAGONAL_SPACEGROUP,"-4",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},4,FALSE,FALSE,FALSE,TRUE},

{356,"I-4",82,ORIGIN_1_ABC,1,"I -4","I -4","S4.2","I -4",TETRAGONAL_SPACEGROUP,"-4",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},8,FALSE,FALSE,FALSE,TRUE},

{357,"P4/M",83,ORIGIN_1_ABC,1,"P 4/m","P 4/m","C4h.1","-P 4",TETRAGONAL_SPACEGROUP,"4/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},

{358,"P42/M",84,ORIGIN_1_ABC,1,"P 42/m","P 42/m","C4h.2","-P 4c",TETRAGONAL_SPACEGROUP,"4/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},

{359,"P4/N",85,ORIGIN_1_ABC,1,"P 4/n:1","P 4/n:1","C4h.3","P 4ab -1ab",TETRAGONAL_SPACEGROUP,"4/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},
{360,"P4/N",85,ORIGIN_2_ABC,2,"P 4/n:2","P 4/n:2","C4h.3","-P 4a",TETRAGONAL_SPACEGROUP,"4/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},

{361,"P42/N",86,ORIGIN_1_ABC,1,"P 42/n:1","P 42/n:1","C4h.4","P 4n -1n",TETRAGONAL_SPACEGROUP,"4/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,TRUE},
{362,"P42/N",86,ORIGIN_2_ABC,2,"P 42/n:2","P 42/n:2","C4h.4","-P 4bc",TETRAGONAL_SPACEGROUP,"4/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,TRUE,FALSE,FALSE,FALSE},

{363,"I4/M",87,ORIGIN_1_ABC,1,"I 4/m","I 4/m","C4h.5","-I 4",TETRAGONAL_SPACEGROUP,"4/m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,TRUE,FALSE,FALSE,TRUE},

{364,"I41/A",88,ORIGIN_1_ABC,1,"I 41/a:1","I 41/a:1","C4h.6","I 4bw -1bw",TETRAGONAL_SPACEGROUP,"4/m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,TRUE,FALSE,FALSE,TRUE},
{365,"I41/A",88,ORIGIN_2_ABC,2,"I 41/a:2","I 41/a:2","C4h.6","-I 4ad",TETRAGONAL_SPACEGROUP,"4/m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,TRUE,FALSE,FALSE,FALSE},

{366,"P422",89,ORIGIN_1_ABC,1,"P 4 2 2","P 4 2 2","D4.1","P 4 2",TETRAGONAL_SPACEGROUP,"4 2 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,TRUE,FALSE,TRUE},

{367,"P4212",90,ORIGIN_1_ABC,1,"P 4 21 2","P 4 21 2","D4.2","P 4ab 2ab",TETRAGONAL_SPACEGROUP,"4 2 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,TRUE,FALSE,TRUE},

{368,"P4122",91,ORIGIN_1_ABC,1,"P 41 2 2","P 41 2 2","D4.3","P 4w 2c",TETRAGONAL_SPACEGROUP,"4 2 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,TRUE,TRUE,TRUE},

{369,"P41212",92,ORIGIN_1_ABC,1,"P 41 21 2","P 41 21 2","D4.4","P 4abw 2nw",TETRAGONAL_SPACEGROUP,"4 2 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,TRUE,TRUE,TRUE},

{370,"P4222",93,ORIGIN_1_ABC,1,"P 42 2 2","P 42 2 2","D4.5","P 4c 2",TETRAGONAL_SPACEGROUP,"4 2 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,TRUE,FALSE,TRUE},

{371,"P42212",94,ORIGIN_1_ABC,1,"P 42 21 2","P 42 21 2","D4.6","P 4n 2n",TETRAGONAL_SPACEGROUP,"4 2 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,TRUE,FALSE,TRUE},

{372,"P4322",95,ORIGIN_1_ABC,1,"P 43 2 2","P 43 2 2","D4.7","P 4cw 2c",TETRAGONAL_SPACEGROUP,"4 2 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,TRUE,TRUE,TRUE},

{373,"P43212",96,ORIGIN_1_ABC,1,"P 43 21 2","P 43 21 2","D4.8","P 4nw 2abw",TETRAGONAL_SPACEGROUP,"4 2 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,TRUE,TRUE,TRUE},

{374,"I422",97,ORIGIN_1_ABC,1,"I 4 2 2","I 4 2 2","D4.9","I 4 2",TETRAGONAL_SPACEGROUP,"4 2 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,FALSE,TRUE,FALSE,TRUE},

{375,"I4122",98,ORIGIN_1_ABC,1,"I 41 2 2","I 41 2 2","D4.10","I 4bw 2bw",TETRAGONAL_SPACEGROUP,"4 2 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,FALSE,TRUE,FALSE,TRUE},

{376,"P4MM",99,ORIGIN_1_ABC,1,"P 4 m m","P 4 m m","C4v.1","P 4 -2",TETRAGONAL_SPACEGROUP,"4 m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,FALSE,FALSE,TRUE},

{377,"P4BM",100,ORIGIN_1_ABC,1,"P 4 b m","P 4 b n","C4v.2","P 4 -2ab",TETRAGONAL_SPACEGROUP,"4 m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,FALSE,FALSE,TRUE},

{378,"P42CM",101,ORIGIN_1_ABC,1,"P 42 c m","P 42 c m","C4v.3","P 4c -2c",TETRAGONAL_SPACEGROUP,"4 m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,FALSE,FALSE,TRUE},

{379,"P42NM",102,ORIGIN_1_ABC,1,"P 42 n m","P 42 n m","C4v.4","P 4n -2n",TETRAGONAL_SPACEGROUP,"4 m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,FALSE,FALSE,TRUE},

{380,"P4CC",103,ORIGIN_1_ABC,1,"P 4 c c","P 4 c c","C4v.5","P 4 -2c",TETRAGONAL_SPACEGROUP,"4 m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,FALSE,FALSE,TRUE},

{381,"P4NC",104,ORIGIN_1_ABC,1,"P 4 n c","P 4 n c","C4v.6","P 4 -2n",TETRAGONAL_SPACEGROUP,"4 m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,FALSE,FALSE,TRUE},

{382,"P42MC",105,ORIGIN_1_ABC,1,"P 42 m c","P 42 m c","C4v.7","P 4c -2",TETRAGONAL_SPACEGROUP,"4 m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,FALSE,FALSE,TRUE},

{383,"P42BC",106,ORIGIN_1_ABC,1,"P 42 b c","P 42 b c","C4v.8","P 4c -2ab",TETRAGONAL_SPACEGROUP,"4 m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,FALSE,FALSE,TRUE},

{384,"I4MM",107,ORIGIN_1_ABC,1,"I 4 m m","I 4 m m","C4v.9","I 4 -2",TETRAGONAL_SPACEGROUP,"4 m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,FALSE,FALSE,FALSE,TRUE},

{385,"I4CM",108,ORIGIN_1_ABC,1,"I 4 c m","I 4 c m","C4v.10","I 4 -2c",TETRAGONAL_SPACEGROUP,"4 m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,FALSE,FALSE,FALSE,TRUE},

{386,"I41MD",109,ORIGIN_1_ABC,1,"I 41 m d","I 41 m d","C4v.11","I 4bw -2",TETRAGONAL_SPACEGROUP,"4 m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,FALSE,FALSE,FALSE,TRUE},

{387,"I41CD",110,ORIGIN_1_ABC,1,"I 41 c d","I 41 c d","C4v.12","I 4bw -2c",TETRAGONAL_SPACEGROUP,"4 m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,FALSE,FALSE,FALSE,TRUE},

{388,"P-42M",111,ORIGIN_1_ABC,1,"P -4 2 m","P -4 2 m","D2d.1","P -4 2",TETRAGONAL_SPACEGROUP,"-4 2 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,FALSE,FALSE,TRUE},

{389,"P-42C",112,ORIGIN_1_ABC,1,"P -4 2 c","P -4 2 c","D2d.2","P -4 2c",TETRAGONAL_SPACEGROUP,"-4 2 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,FALSE,FALSE,TRUE},

{390,"P-421M",113,ORIGIN_1_ABC,1,"P -4 21 m","P -4 21 m","D2d.3","P -4 2ab",TETRAGONAL_SPACEGROUP,"-4 2 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,FALSE,FALSE,TRUE},

{391,"P-421C",114,ORIGIN_1_ABC,1,"P -4 21 c","P -4 21 c","D2d.4","P -4 2n",TETRAGONAL_SPACEGROUP,"-4 2 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,FALSE,FALSE,TRUE},

{392,"P-4M2",115,ORIGIN_1_ABC,1,"P -4 m 2","P -4 m 2","D2d.5","P -4 -2",TETRAGONAL_SPACEGROUP,"-4 2 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,FALSE,FALSE,TRUE},

{393,"P-4C2",116,ORIGIN_1_ABC,1,"P -4 c 2","P -4 c 2","D2d.6","P -4 -2c",TETRAGONAL_SPACEGROUP,"-4 2 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,FALSE,FALSE,TRUE},

{394,"P-4B2",117,ORIGIN_1_ABC,1,"P -4 b 2","P -4 b 2","D2d.7","P -4 -2ab",TETRAGONAL_SPACEGROUP,"-4 2 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,FALSE,FALSE,TRUE},

{395,"P-4N2",118,ORIGIN_1_ABC,1,"P -4 n 2","P -4 n 2","D2d.8","P -4 -2n",TETRAGONAL_SPACEGROUP,"-4 2 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},8,FALSE,FALSE,FALSE,TRUE},

{396,"I-4M2",119,ORIGIN_1_ABC,1,"I -4 m 2","I -4 m 2","D2d.9","I -4 -2",TETRAGONAL_SPACEGROUP,"-4 m 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,FALSE,FALSE,FALSE,TRUE},

{397,"I-4C2",120,ORIGIN_1_ABC,1,"I -4 c 2","I -4 c 2","D2d.10","I -4 -2c",TETRAGONAL_SPACEGROUP,"-4 m 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,FALSE,FALSE,FALSE,TRUE},

{398,"I-42M",121,ORIGIN_1_ABC,1,"I -4 2 m","I -4 2 m","D2d.11","I -4 2",TETRAGONAL_SPACEGROUP,"-4 m 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,FALSE,FALSE,FALSE,TRUE},

{399,"I-42D",122,ORIGIN_1_ABC,1,"I -4 2 d","I -4 2 d","D2d.12","I -4 2bw",TETRAGONAL_SPACEGROUP,"-4 m 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},16,FALSE,FALSE,FALSE,TRUE},

{400,"P4/MMM",123,ORIGIN_1_ABC,1,"P 4/m m m","P 4/m 2/m 2/m","D4h.1","-P 4 2",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,TRUE},

{401,"P4/MCC",124,ORIGIN_1_ABC,1,"P 4/m c c","P 4/m 2/c 2/c","D4h.2","-P 4 2c",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,TRUE},

{402,"P4/NBM",125,ORIGIN_1_ABC,1,"P 4/n b m:1","P 4/n 2/b 2/m:1","D4h.3","P 4 2 -1ab",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,TRUE},
{403,"P4/NBM",125,ORIGIN_2_ABC,2,"P 4/n b m:2","P 4/n 2/b 2/m:2","D4h.3","-P 4a 2b",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,FALSE},

{404,"P4/NNC",126,ORIGIN_1_ABC,1,"P 4/n n c:1","P 4/n 2/n 2/c:1","D4h.4","P 4 2 -1n",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,TRUE},
{405,"P4/NNC",126,ORIGIN_2_ABC,2,"P 4/n n c:2","P 4/n 2/n 2/c:2","D4h.4","-P 4a 2bc",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,FALSE},

{406,"P4/MBM",127,ORIGIN_1_ABC,1,"P 4/m b m","P 4/m 21/b 2/m","D4h.5","-P 4 2ab",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,TRUE},

{407,"P4/MNC",128,ORIGIN_1_ABC,1,"P 4/m n c","P 4/m 21/n 2/c","D4h.6","-P 4 2n",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,TRUE},

{408,"P4/NMM",129,ORIGIN_1_ABC,1,"P 4/n m m:1","P 4/n 21/m 2/m:1","D4h.7","P 4ab 2ab -1ab",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,TRUE},
{409,"P4/NMM",129,ORIGIN_2_ABC,2,"P 4/n m m:2","P 4/n 21/m 2/m:2","D4h.7","-P 4a 2a",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,FALSE},

{410,"P4/NCC",130,ORIGIN_1_ABC,1,"P 4/n c c:1","P 4/n 21/c 2/c:1","D4h.8","P 4ab 2n -1ab",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,TRUE},
{411,"P4/NCC",130,ORIGIN_2_ABC,2,"P 4/n c c:2","P 4/n 21/c 2/c:2","D4h.8","-P 4a 2ac",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,FALSE},

{412,"P42/MMC",131,ORIGIN_1_ABC,1,"P 42/m m c","P 42/m 2/m 2/c","D4h.9","-P 4c 2",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,TRUE},

{413,"P42/MCM",132,ORIGIN_1_ABC,1,"P 42/m c m","P 42/m 2/c 2/m","D4h.10","-P 4c 2c",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,TRUE},

{414,"P42/NBC",133,ORIGIN_1_ABC,1,"P 42/n b c:1","P 42/n 2/b 2/c:1","D4h.11","P 4n 2c -1n",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,TRUE},
{415,"P42/NBC",133,ORIGIN_2_ABC,2,"P 42/n b c:2","P 42/n 2/b 2/c:2","D4h.11","-P 4ac 2b",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,FALSE},

{416,"P42/NNM",134,ORIGIN_1_ABC,1,"P 42/n n m:1","P 42/n 2/n 2/m:1","D4h.12","P 4n 2 -1n",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,TRUE},
{417,"P42/NNM",134,ORIGIN_2_ABC,2,"P 42/n n m:2","P 42/n 2/n 2/m:2","D4h.12","-P 4ac 2bc",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,FALSE},

{418,"P42/MBC",135,ORIGIN_1_ABC,1,"P 42/m b c","P 42/m 21/b 2/c","D4h.13","-P 4c 2ab",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,TRUE},

{419,"P42/MNM",136,ORIGIN_1_ABC,1,"P 42/m n m","P 42/m 21/n 2/m","D4h.14","-P 4n 2n",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,TRUE},

{420,"P42/NMC",137,ORIGIN_1_ABC,1,"P 42/n m c:1","P 42/n 21/m 2/c:1","D4h.15","P 4n 2n -1n",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,TRUE},
{421,"P42/NMC",137,ORIGIN_2_ABC,2,"P 42/n m c:2","P 42/n 21/m 2/c:2","D4h.15","-P 4ac 2a",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,FALSE},

{422,"P42/NCM",138,ORIGIN_1_ABC,1,"P 42/n c m:1","P 42/n 21/c 2/m:1","D4h.16","P 4n 2ab -1n",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,TRUE},
{423,"P42/NCM",138,ORIGIN_2_ABC,2,"P 42/n c m:2","P 42/n 21/c 2/m:2","D4h.16","-P 4ac 2ac",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},16,TRUE,FALSE,FALSE,FALSE},

{424,"I4/MMM",139,ORIGIN_1_ABC,1,"I 4/m m m","I 4/m 2/m 2/m","D4h.17","-I 4 2",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},32,TRUE,FALSE,FALSE,TRUE},

{425,"I4/MCM",140,ORIGIN_1_ABC,2,"I 4/m c m","I 4/m 2/c 2/m","D4h.18","-I 4 2c",TETRAGONAL_SPACEGROUP,"4/m m m",PRIMITIVE_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},32,TRUE,FALSE,FALSE,TRUE},

{426,"I41/AMD",141,ORIGIN_1_ABC,1,"I 41/a m d:1","I 41/a 2/m 2/d:1","D4h.19","I 4bw 2bw -1bw",TETRAGONAL_SPACEGROUP,"4/m m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},32,TRUE,FALSE,FALSE,TRUE},
{427,"I41/AMD",141,ORIGIN_2_ABC,2,"I 41/a m d:2","I 41/a 2/m 2/d:2","D4h.19","-I 4bd 2",TETRAGONAL_SPACEGROUP,"4/m m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},32,TRUE,FALSE,FALSE,FALSE},

{428,"I41/ACD",142,ORIGIN_1_ABC,1,"I 41/a c d:1","I 41/a 2/c 2/d:1","D4h.20","I 4bw 2aw -1bw",TETRAGONAL_SPACEGROUP,"4/m m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},32,TRUE,FALSE,FALSE,TRUE},
{429,"I41/ACD",142,ORIGIN_2_ABC,2,"I 41/a c d:2","I 41/a 2/c 2/d:2","D4h.20","-I 4bd 2c",TETRAGONAL_SPACEGROUP,"4/m m m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},32,TRUE,FALSE,FALSE,FALSE},

{430,"P3",143,ORIGIN_1_ABC,1,"P 3","P 3","C3.1","P 3",TRIGONAL_SPACEGROUP,"3",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},3,FALSE,TRUE,FALSE,TRUE},

{431,"P31",144,ORIGIN_1_ABC,1,"P 31","P 31","C3.2","P 31",TRIGONAL_SPACEGROUP,"3",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},3,FALSE,TRUE,TRUE,TRUE},

{432,"P32",145,ORIGIN_1_ABC,1,"P 32","P 32","C3.3","P 32",TRIGONAL_SPACEGROUP,"3",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},3,FALSE,TRUE,FALSE,TRUE},

{433,"R3",146,HEXAGONAL,1,"R 3:H","R 3:H","C3.4","R 3",TRIGONAL_SPACEGROUP,"3",RHOMBOHEDRAL_CENTERED,3,{{0.0,0.0,0.0},{2.0/3.0,1.0/3.0,1.0/3.0},{1.0/3.0,2.0/3.0,2.0/3.0}},9,FALSE,TRUE,FALSE,TRUE},
{434,"R3",146,RHOMBOHEDRAL,2,"R 3:R","R 3:R","C3.4","P 3*",TRIGONAL_SPACEGROUP,"3",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},3,FALSE,TRUE,FALSE,TRUE},

{435,"P-3",147,ORIGIN_1_ABC,1,"P -3","P -3","C3i.1","-P 3",TRIGONAL_SPACEGROUP,"-3",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,TRUE,FALSE,FALSE,TRUE},

{436,"R-3",148,HEXAGONAL,1,"R -3:H","R -3:H","C3i.2","-R 3",TRIGONAL_SPACEGROUP,"-3",RHOMBOHEDRAL_CENTERED,3,{{0.0,0.0,0.0},{2.0/3.0,1.0/3.0,1.0/3.0},{1.0/3.0,2.0/3.0,2.0/3.0}},18,TRUE,FALSE,FALSE,TRUE},
{437,"R-3",148,RHOMBOHEDRAL,2,"R -3:R","R -3:R","C3i.1","-P 3*",TRIGONAL_SPACEGROUP,"-3",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,TRUE,FALSE,FALSE,FALSE},

{438,"P312",149,ORIGIN_1_ABC,1,"P 3 1 2","P 3 1 2","D3.1","P 3 2",TRIGONAL_SPACEGROUP,"3 1 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,TRUE,FALSE,TRUE},

{439,"P321",150,ORIGIN_1_ABC,1,"P 3 2 1","P 3 2 1","D3.2","P 3 2\"",TRIGONAL_SPACEGROUP,"3 1 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,TRUE,FALSE,TRUE},

{440,"P3112",151,ORIGIN_1_ABC,1,"P 31 1 2","P 31 1 2","D3.3","P 31 2 (0 0 4)",TRIGONAL_SPACEGROUP,"3 1 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,TRUE,TRUE,TRUE},

{441,"P3121",152,ORIGIN_1_ABC,1,"P 31 2 1","P 31 2 1","D3.4","P 31 2\"",TRIGONAL_SPACEGROUP,"3 1 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,TRUE,TRUE,TRUE},

{442,"P3212",153,ORIGIN_1_ABC,1,"P 32 1 2","P 32 1 2","D3.5","P 32 2 (0 0 2)",TRIGONAL_SPACEGROUP,"3 1 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,TRUE,TRUE,TRUE},

{443,"P3221",154,ORIGIN_1_ABC,1,"P 32 2 1","P 32 2 1","D3.6","P 32 2\"",TRIGONAL_SPACEGROUP,"3 1 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,TRUE,TRUE,TRUE},

{444,"R32",155,HEXAGONAL,1,"R 3 2:H","R 3 2:H","D3.7","R 3 2\"",TRIGONAL_SPACEGROUP,"3 2",RHOMBOHEDRAL_CENTERED,3,{{0.0,0.0,0.0},{2.0/3.0,1.0/3.0,1.0/3.0},{1.0/3.0,2.0/3.0,2.0/3.0}},18,FALSE,TRUE,FALSE,TRUE},
{445,"R32",155,RHOMBOHEDRAL,2,"R 3 2:R","R 3 2:R","D3.7","P 3* 2",TRIGONAL_SPACEGROUP,"3 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,TRUE,FALSE,FALSE},

{446,"P3M1",156,ORIGIN_1_ABC,1,"P 3 m 1","P 3 m 1","C3v.1","P 3 -2\"",TRIGONAL_SPACEGROUP,"3 m 1",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,FALSE,FALSE,TRUE},

{447,"P31M",157,ORIGIN_1_ABC,1,"P 3 1 m","P 3 1 m","C3v.2","P 3 -2",TRIGONAL_SPACEGROUP,"3 1 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,FALSE,FALSE,TRUE},

{448,"P3C1",158,ORIGIN_1_ABC,1,"P 3 c 1","P 3 c 1","C3v.3","P 3 -2\"c",TRIGONAL_SPACEGROUP,"3 m 1",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,FALSE,FALSE,TRUE},

{449,"P31C",159,ORIGIN_1_ABC,1,"P 3 1 c","P 3 1 c","C3v.4","P 3 -2c",TRIGONAL_SPACEGROUP,"3 1 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,FALSE,FALSE,TRUE},

{450,"R3M",160,HEXAGONAL,1,"R 3 m:H","R 3 m:H","C3v.5","R 3 -2\"",TRIGONAL_SPACEGROUP,"3 m",RHOMBOHEDRAL_CENTERED,3,{{0.0,0.0,0.0},{2.0/3.0,1.0/3.0,1.0/3.0},{1.0/3.0,2.0/3.0,2.0/3.0}},18,FALSE,FALSE,FALSE,TRUE},
{451,"R3M",160,RHOMBOHEDRAL,2,"R 3 m:R","R 3 m:R","C3v.5","P 3* -2",TRIGONAL_SPACEGROUP,"3 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,FALSE,FALSE,FALSE},

{452,"R3C",161,HEXAGONAL,1,"R 3 c:H","R 3 c:H","C3v.6","R 3 -2\"c",TRIGONAL_SPACEGROUP,"3 m",RHOMBOHEDRAL_CENTERED,3,{{0.0,0.0,0.0},{2.0/3.0,1.0/3.0,1.0/3.0},{1.0/3.0,2.0/3.0,2.0/3.0}},18,FALSE,FALSE,FALSE,TRUE},
{453,"R3C",161,RHOMBOHEDRAL,2,"R 3 c:R","R 3 c:R","C3v.6","P 3* -2n",TRIGONAL_SPACEGROUP,"3 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,FALSE,FALSE,FALSE},

{454,"P-31M",162,ORIGIN_1_ABC,1,"P -3 1 m","P -3 1 2/m","D3d.1","-P 3 2",TRIGONAL_SPACEGROUP,"-3 1 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,TRUE,FALSE,FALSE,TRUE},

{455,"P-312/C",163,ORIGIN_1_ABC,1,"P -3 1 c","P -3 1 2/c","D3d.2","-P 3 2c",TRIGONAL_SPACEGROUP,"-3 1 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,TRUE,FALSE,FALSE,TRUE},

{456,"P-3M1",164,ORIGIN_1_ABC,1,"P -3 m 1","P -3 2/m 1","D3d.3","-P 3 2\"",TRIGONAL_SPACEGROUP,"-3 1 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,TRUE,FALSE,FALSE,TRUE},

{457,"P-3C1",165,ORIGIN_1_ABC,1,"P -3 c 1","P -3 2/c 1","D3d.4","-P 3 2\"c",TRIGONAL_SPACEGROUP,"-3 1 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,TRUE,FALSE,FALSE,TRUE},

{458,"R-3M",166,HEXAGONAL,1,"R -3 m:H","R -3 2/m:H","D3d.5","-R 3 2\"",TRIGONAL_SPACEGROUP,"-3 m",RHOMBOHEDRAL_CENTERED,3,{{0.0,0.0,0.0},{2.0/3.0,1.0/3.0,1.0/3.0},{1.0/3.0,2.0/3.0,2.0/3.0}},36,TRUE,FALSE,FALSE,TRUE},
{459,"R-3M",166,RHOMBOHEDRAL,2,"R -3 m:R","R -3 2/m:R","D3d.5","-P 3* 2",TRIGONAL_SPACEGROUP,"-3 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,TRUE,FALSE,FALSE,FALSE},

{460,"R-3C",167,HEXAGONAL,1,"R -3 c:H","R -3 2/c:H","D3d.6","-R 3 2\"c",TRIGONAL_SPACEGROUP,"-3 m",RHOMBOHEDRAL_CENTERED,3,{{0.0,0.0,0.0},{2.0/3.0,1.0/3.0,1.0/3.0},{1.0/3.0,2.0/3.0,2.0/3.0}},36,TRUE,FALSE,FALSE,TRUE},
{461,"R-3C",167,RHOMBOHEDRAL,2,"R -3 c:R","R -3 2/c:R","D3d.6","-P 3* 2n",TRIGONAL_SPACEGROUP,"-3 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,TRUE,FALSE,FALSE,FALSE},

{462,"P6",168,ORIGIN_1_ABC,1,"P 6","P 6","C6.1","P 6",HEXAGONAL_SPACEGROUP,"6",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,TRUE,FALSE,TRUE},

{463,"P61",169,ORIGIN_1_ABC,1,"P 61","P 61","C6.2","P 61",HEXAGONAL_SPACEGROUP,"6",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,TRUE,TRUE,TRUE},

{464,"P65",170,ORIGIN_1_ABC,1,"P 65","P 65","C6.3","P 65",HEXAGONAL_SPACEGROUP,"6",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,TRUE,TRUE,TRUE},

{465,"P62",171,ORIGIN_1_ABC,1,"P 62","P 62","C6.4","P 62",HEXAGONAL_SPACEGROUP,"6",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,TRUE,TRUE,TRUE},

{466,"P64",172,ORIGIN_1_ABC,1,"P 64","P 64","C6.5","P 64",HEXAGONAL_SPACEGROUP,"6",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,TRUE,TRUE,TRUE},

{467,"P63",173,ORIGIN_1_ABC,1,"P 63","P 63","C6.6","P 6c",HEXAGONAL_SPACEGROUP,"6",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,TRUE,FALSE,TRUE},

{468,"P-6",174,ORIGIN_1_ABC,1,"P -6","P -6","C3h.1","P -6",HEXAGONAL_SPACEGROUP,"-6",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,FALSE,FALSE,FALSE,TRUE},

{469,"P6/M",175,ORIGIN_1_ABC,1,"P 6/m","P 6/m","C6h.1","-P 6",HEXAGONAL_SPACEGROUP,"6/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},6,TRUE,FALSE,FALSE,TRUE},

{470,"P63/M",176,ORIGIN_1_ABC,1,"P 63/m","P 63/m","C6h.2","-P 6c",HEXAGONAL_SPACEGROUP,"6/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,TRUE,FALSE,FALSE,TRUE},

{471,"P622",177,ORIGIN_1_ABC,1,"P 6 2 2","P 6 2 2","D6.1","P 6 2",HEXAGONAL_SPACEGROUP,"6 2 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,FALSE,TRUE,FALSE,TRUE},

{472,"P6122",178,ORIGIN_1_ABC,1,"P 61 2 2","P 61 2 2","D6.2","P 61 2 (0 0 5)",HEXAGONAL_SPACEGROUP,"6/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,FALSE,TRUE,TRUE,TRUE},

{473,"P6522",179,ORIGIN_1_ABC,1,"P 65 2 2","P 65 2 2","D6.3","P 65 2 (0 0 1)",HEXAGONAL_SPACEGROUP,"6/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,FALSE,TRUE,TRUE,TRUE},

{474,"P6222",180,ORIGIN_1_ABC,1,"P 62 2 2","P 62 2 2","D6.4","P 62 2 (0 0 4)",HEXAGONAL_SPACEGROUP,"6/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,FALSE,TRUE,TRUE,TRUE},

{475,"P6422",181,ORIGIN_1_ABC,1,"P 64 2 2","P 64 2 2","D6.5","P 64 2 (0 0 2)",HEXAGONAL_SPACEGROUP,"6/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,FALSE,TRUE,TRUE,TRUE},

{476,"P6322",182,ORIGIN_1_ABC,1,"P 63 2 2","P 63 2 2","D6.6","P 6c 2c",HEXAGONAL_SPACEGROUP,"6/m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,FALSE,TRUE,FALSE,TRUE},

{477,"P6MM",183,ORIGIN_1_ABC,1,"P 6 m m","P 6 m m","C6v.1","P 6 -2",HEXAGONAL_SPACEGROUP,"6 m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,FALSE,FALSE,FALSE,TRUE},

{478,"P6CC",184,ORIGIN_1_ABC,1,"P 6 c c","P 6 c c","C6v.2","P 6 -2c",HEXAGONAL_SPACEGROUP,"6 m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,FALSE,FALSE,FALSE,TRUE},

{479,"P63CM",185,ORIGIN_1_ABC,1,"P 63 c m","P 63 c m","C6v.3","P 6c -2",HEXAGONAL_SPACEGROUP,"6 m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,FALSE,FALSE,FALSE,TRUE},

{480,"P63MC",186,ORIGIN_1_ABC,1,"P 63 m c","P 63 m c","C6v.4","P 6c -2c",HEXAGONAL_SPACEGROUP,"6 m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,FALSE,FALSE,FALSE,TRUE},

{481,"P-6M2",187,ORIGIN_1_ABC,1,"P -6 m 2","P -6 m 2","D3h.1","P -6 2",HEXAGONAL_SPACEGROUP,"-6 m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,FALSE,FALSE,FALSE,TRUE},

{482,"P-6C2",188,ORIGIN_1_ABC,1,"P -6 c 2","P -6 c 2","D3h.2","P -6c 2",HEXAGONAL_SPACEGROUP,"-6 m 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,FALSE,FALSE,FALSE,TRUE},

{483,"P-62M",189,ORIGIN_1_ABC,1,"P -6 2 m","P -6 2 m","D3h.3","P -6 -2",HEXAGONAL_SPACEGROUP,"-6 2 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,FALSE,FALSE,FALSE,TRUE},

{484,"P-62C",190,ORIGIN_1_ABC,1,"P -6 2 c","P -6 2 c","D3h.4","P -6c -2c",HEXAGONAL_SPACEGROUP,"-6 2 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,FALSE,FALSE,FALSE,TRUE},

{485,"P6/MMM",191,ORIGIN_1_ABC,1,"P 6/m m m","P 6/m 2/m 2/m","D6h.1","-P 6 2",HEXAGONAL_SPACEGROUP,"6/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},24,TRUE,FALSE,FALSE,TRUE},

{486,"P6/MCC",192,ORIGIN_1_ABC,1,"P 6/m c c","P 6/m 2/c 2/c","D6h.2","-P 6 2c",HEXAGONAL_SPACEGROUP,"6/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},24,TRUE,FALSE,FALSE,TRUE},

{487,"P63/MCM",193,ORIGIN_1_ABC,1,"P 63/m c m","P 63/m 2/c 2/m","D6h.3","-P 6c 2",HEXAGONAL_SPACEGROUP,"6/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},24,TRUE,FALSE,FALSE,TRUE},

{488,"P63/MMC",194,ORIGIN_1_ABC,1,"P 63/m m c","P 63/m 2/m 2/c","D6h.4","-P 6c 2c",HEXAGONAL_SPACEGROUP,"6/m m m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},24,TRUE,FALSE,FALSE,TRUE},

{489,"P23",195,ORIGIN_1_ABC,1,"P 2 3","P 2 3","T.1","P 2 2 3",CUBIC_SPACEGROUP,"2 3",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,FALSE,TRUE,FALSE,TRUE},

{490,"F23",196,ORIGIN_1_ABC,1,"F 2 3","F 2 3","T.2","F 2 2 3",CUBIC_SPACEGROUP,"2 3",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},48,FALSE,TRUE,FALSE,TRUE},

{491,"I23",197,ORIGIN_1_ABC,1,"I 2 3","I 2 3","T.3","I 2 2 3",CUBIC_SPACEGROUP,"2 3",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},24,FALSE,TRUE,FALSE,TRUE},

{492,"P213",198,ORIGIN_1_ABC,1,"P 21 3","P 21 3","T.4","P 2ac 2ab 3",CUBIC_SPACEGROUP,"2 3",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},12,FALSE,TRUE,FALSE,TRUE},

{493,"I213",199,ORIGIN_1_ABC,1,"I 21 3","I 21 3","T.5","I 2b 2c 3",CUBIC_SPACEGROUP,"2 3",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},24,FALSE,TRUE,FALSE,TRUE},

{494,"PM-3",200,ORIGIN_1_ABC,1,"P m -3","P 2/m -3","Th.1","-P 2 2 3",CUBIC_SPACEGROUP,"m -3",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},24,TRUE,FALSE,FALSE,TRUE},

{495,"PN-3",201,ORIGIN_1_ABC,1,"P n -3:1","P 2/n -3:1","Th.2","P 2 2 3 -1n",CUBIC_SPACEGROUP,"m -3",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},24,TRUE,FALSE,FALSE,TRUE},
{496,"PN-3",201,ORIGIN_2_ABC,2,"P n -3:2","P 2/n -3:2","Th.2","-P 2ab 2bc 3",CUBIC_SPACEGROUP,"m -3",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},24,TRUE,FALSE,FALSE,FALSE},

{497,"FM-3",202,ORIGIN_1_ABC,1,"F m -3","F 2/m -3","Th.3","-F 2 2 3",CUBIC_SPACEGROUP,"m -3",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},96,TRUE,FALSE,FALSE,TRUE},

{498,"FD-3",203,ORIGIN_1_ABC,1,"F d -3:1","F 2/d -3:1","Th.4","F 2 2 3 -1d",CUBIC_SPACEGROUP,"m -3",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},96,TRUE,FALSE,FALSE,TRUE},
{499,"FD-3",203,ORIGIN_2_ABC,2,"F d -3:2","F 2/d -3:2","Th.4","-F 2uv 2vw 3",CUBIC_SPACEGROUP,"m -3",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},96,TRUE,FALSE,FALSE,FALSE},

{500,"IM-3",204,ORIGIN_1_ABC,1,"I m -3","I 2/m -3","Th.5","-I 2 2 3",CUBIC_SPACEGROUP,"m -3",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},48,TRUE,FALSE,FALSE,TRUE},

{501,"PA-3",205,ORIGIN_1_ABC,1,"P a -3","P 21/a -3","Th.6","-P 2ac 2ab 3",CUBIC_SPACEGROUP,"m -3",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},24,TRUE,FALSE,FALSE,TRUE},

{502,"IA-3",206,ORIGIN_1_ABC,1,"I a -3","I 21/a -3","Th.7","-I 2b 2c 3",CUBIC_SPACEGROUP,"m -3",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},48,TRUE,FALSE,FALSE,TRUE},

{503,"P432",207,ORIGIN_1_ABC,1,"P 4 3 2","P 4 3 2","O.1","P 4 2 3",CUBIC_SPACEGROUP,"4 3 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},24,FALSE,TRUE,FALSE,TRUE},

{504,"P4232",208,ORIGIN_1_ABC,1,"P 42 3 2","P 42 3 2","O.2","P 4n 2 3",CUBIC_SPACEGROUP,"4 3 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},24,FALSE,TRUE,FALSE,TRUE},

{505,"F432",209,ORIGIN_1_ABC,1,"F 4 3 2","F 4 3 2","O.3","F 4 2 3",CUBIC_SPACEGROUP,"4 3 2",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},96,FALSE,TRUE,FALSE,TRUE},

{506,"F4132",210,ORIGIN_1_ABC,1,"F 41 3 2","F 41 3 2","O.4","F 4d 2 3",CUBIC_SPACEGROUP,"4 3 2",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},96,FALSE,TRUE,FALSE,TRUE},

{507,"I32",211,ORIGIN_1_ABC,1,"I 4 3 2","I 4 3 2","O.5","I 4 2 3",CUBIC_SPACEGROUP,"4 3 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},48,FALSE,TRUE,FALSE,TRUE},

{508,"P4332",212,ORIGIN_1_ABC,1,"P 43 3 2","P 43 3 2","O.6","P 4acd 2ab 3",CUBIC_SPACEGROUP,"4 3 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},24,FALSE,TRUE,TRUE,TRUE},

{509,"P4132",213,ORIGIN_1_ABC,1,"P 41 3 2","P 41 3 2","O.7","P 4bd 2ab 3",CUBIC_SPACEGROUP,"4 3 2",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},24,FALSE,TRUE,TRUE,TRUE},

{510,"I4132",214,ORIGIN_1_ABC,1,"I 41 3 2","I 41 3 2","O.8","I 4bd 2c 3",CUBIC_SPACEGROUP,"4 3 2",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},48,FALSE,TRUE,FALSE,TRUE},

{511,"P-43m",215,ORIGIN_1_ABC,1,"P -4 3 m","P -4 3 m","Td.1","P -4 2 3",CUBIC_SPACEGROUP,"-4 3 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},24,FALSE,FALSE,FALSE,TRUE},

{512,"F-43m",216,ORIGIN_1_ABC,1,"F -4 3 m","F -4 3 m","Td.2","F -4 2 3",CUBIC_SPACEGROUP,"-4 3 m",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},96,FALSE,FALSE,FALSE,TRUE},

{513,"I-43M",217,ORIGIN_1_ABC,1,"I -4 3 m","I -4 3 m","Td.3","I -4 2 3",CUBIC_SPACEGROUP,"-4 3 m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},48,FALSE,FALSE,FALSE,TRUE},

{514,"P-43N",218,ORIGIN_1_ABC,1,"P -4 3 n","P -4 3 n","Td.4","P -4n 2 3",CUBIC_SPACEGROUP,"-4 3 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},24,FALSE,FALSE,FALSE,TRUE},

{515,"F-43C",219,ORIGIN_1_ABC,1,"F -4 3 c","F -4 3 c","Td.5","F -4a 2 3",CUBIC_SPACEGROUP,"-4 3 m",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},96,FALSE,FALSE,FALSE,TRUE},

{516,"I-43D",220,ORIGIN_1_ABC,1,"I -4 3 d","I -4 3 d","Td.6","I -4bd 2c 3",CUBIC_SPACEGROUP,"-4 3 m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},48,FALSE,FALSE,FALSE,TRUE},

{517,"PM-3M",221,ORIGIN_1_ABC,1,"P m -3 m","P 4/m -3 2/m","Oh.1","-P 4 2 3",CUBIC_SPACEGROUP,"m -3 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},48,TRUE,FALSE,FALSE,TRUE},

{518,"PN-3N",222,ORIGIN_1_ABC,1,"P n -3 n:1","P 4/n -3 2/n:1","Oh.2","P 4 2 3 -1n",CUBIC_SPACEGROUP,"m -3 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},48,TRUE,FALSE,FALSE,TRUE},
{519,"PN-3N",222,ORIGIN_2_ABC,2,"P n -3 n:2","P 4/n -3 2/n:2","Oh.2","-P 4a 2bc 3",CUBIC_SPACEGROUP,"m -3 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},48,TRUE,FALSE,FALSE,FALSE},

{520,"PM-3N",223,ORIGIN_1_ABC,1,"P m -3 n","P 42/m -3 2/n","Oh.3","-P 4n 2 3",CUBIC_SPACEGROUP,"m -3 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},48,TRUE,FALSE,FALSE,TRUE},

{521,"PN-3M",224,ORIGIN_1_ABC,1,"P n -3 m:1","P 42/n -3 2/m:1","Oh.4","P 4n 2 3 -1n",CUBIC_SPACEGROUP,"m -3 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},48,TRUE,FALSE,FALSE,TRUE},
{522,"PN-3M",224,ORIGIN_2_ABC,2,"P n -3 m:2","P 42/n -3 2/m:2","Oh.4","-P 4bc 2bc 3",CUBIC_SPACEGROUP,"m -3 m",PRIMITIVE_CENTERED,1,{{0.0,0.0,0.0}},48,TRUE,FALSE,FALSE,FALSE},

{523,"FM-3M",225,ORIGIN_1_ABC,1,"F m -3 m","F 4/m -3 2/m","Oh.5","-F 4 2 3",CUBIC_SPACEGROUP,"m -3 m",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},192,TRUE,FALSE,FALSE,TRUE},

{524,"FM-3C",226,ORIGIN_1_ABC,1,"F m -3 c","F 4/m -3 2/c","Oh.6","-F 4a 2 3",CUBIC_SPACEGROUP,"m -3 m",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},192,TRUE,FALSE,FALSE,TRUE},

{525,"FD-3M",227,ORIGIN_1_ABC,1,"F d -3 m:1","F 41/d -3 2/m:1","Oh.7","F 4d 2 3 -1d",CUBIC_SPACEGROUP,"m -3 m",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},192,TRUE,FALSE,FALSE,TRUE},
{526,"FD-3M",227,ORIGIN_2_ABC,2,"F d -3 m:2","F 41/d -3 2/m:2","Oh.7","-F 4vw 2vw 3",CUBIC_SPACEGROUP,"m -3 m",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},192,TRUE,FALSE,FALSE,FALSE},

{527,"FD-3C",228,ORIGIN_1_ABC,1,"F d -3 c:1","F 41/d -3 2/c","Oh.8","F 4d 2 3 -1ad",CUBIC_SPACEGROUP,"m -3 m",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},192,TRUE,FALSE,FALSE,TRUE},
{528,"FD-3C",228,ORIGIN_2_ABC,2,"F d -3 c:2","F 41/d -3 2/c","Oh.8","-F 4ud 2vw 3",CUBIC_SPACEGROUP,"m -3 m",FACE_CENTERED,4,{{0.0,0.0,0.0},{0.0,0.5,0.5},{0.5,0.0,0.5},{0.5,0.5,0.0}},192,TRUE,FALSE,FALSE,FALSE},

{529,"IM-3M",229,ORIGIN_1_ABC,1,"I m -3 m","I 4/m -3 2/m","Oh.9","-I 4 2 3",CUBIC_SPACEGROUP,"m -3 m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},96,TRUE,FALSE,FALSE,TRUE},

{530,"IA-3D",230,ORIGIN_1_ABC,1,"I a -3 d","I 41/a -3 2/d","Oh.10","-I 4bd 2c 3",CUBIC_SPACEGROUP,"m -3 m",BODY_CENTERED,2,{{0.0,0.0,0.0},{0.5,0.5,0.5}},96,TRUE,FALSE,FALSE,TRUE},
};

void PrintSpaceGroupChiralInversion(void)
{
  int i;

  for(i=1;i<=NUMBER_OF_SPACEGROUPS;i++)
  {
    if(SpaceGroupData[i].Centric)
      fprintf(stderr, "%d\n",i);
  }

}


void PrintSpaceGroupInformationLatex(void)
{
  int i;

  for(i=1;i<=NUMBER_OF_SPACEGROUPS;i++)
  {
    fprintf(stderr, "%d &",SpaceGroupData[i].Identifier);
    fprintf(stderr, "%d &",SpaceGroupData[i].Number);
/*
    switch(SpaceGroupData[i].CrystalSystem)
    {
      case MONOCLINIC_SPACEGROUP: fprintf(stderr, "monoclinic &"); break;
      case ORTHORHOMBIC_SPACEGROUP: fprintf(stderr, "orthorhombic &"); break;
      case TRICLINIC_SPACEGROUP: fprintf(stderr, "monoclinic &"); break;
      case TETRAGONAL_SPACEGROUP: fprintf(stderr, "tetragonal &"); break;
      case TRIGONAL_SPACEGROUP: fprintf(stderr, "trigonal &"); break;
      case HEXAGONAL_SPACEGROUP: fprintf(stderr, "hexagonal &"); break;
      case CUBIC_SPACEGROUP: fprintf(stderr, "cubic &"); break;
    }
*/

    fprintf(stderr, "%s &",SpaceGroupData[i].LongInternationalHermannMauguinSpaceGroupSymbol);
    //printf("%s &",SpaceGroupData[i].ShortInternationalHermannMauguinSpaceGroupSymbol);
    fprintf(stderr, "%s &",SpaceGroupData[i].HallSpaceGroupSymbol);
    switch(SpaceGroupData[i].SpaceGroupOption)
    {
      case A_UNIQUE_CELL_1: fprintf(stderr, "a, cell choice 1 &"); break;
      case A_UNIQUE_CELL_2: fprintf(stderr, "a, cell choice 2 &"); break;
      case A_UNIQUE_CELL_3: fprintf(stderr, "a, cell choice 3 &"); break;
      case B_UNIQUE_CELL_1: fprintf(stderr, "b, cell choice 1 &"); break;
      case B_UNIQUE_CELL_2: fprintf(stderr, "b, cell choice 2 &"); break;
      case B_UNIQUE_CELL_3: fprintf(stderr, "b, cell choice 3 &"); break;
      case C_UNIQUE_CELL_1: fprintf(stderr, "c, cell choice 1 &"); break;
      case C_UNIQUE_CELL_2: fprintf(stderr, "c, cell choice 2 &"); break;
      case C_UNIQUE_CELL_3: fprintf(stderr, "c, cell choice 3 &"); break;
      case mA_UNIQUE_CELL_1: fprintf(stderr, "-a, cell choice 1 &"); break;
      case mA_UNIQUE_CELL_2: fprintf(stderr, "-a, cell choice 2 &"); break;
      case mA_UNIQUE_CELL_3: fprintf(stderr, "-a, cell choice 3 &"); break;
      case mB_UNIQUE_CELL_1: fprintf(stderr, "-b, cell choice 1 &"); break;
      case mB_UNIQUE_CELL_2: fprintf(stderr, "-b, cell choice 2 &"); break;
      case mB_UNIQUE_CELL_3: fprintf(stderr, "-b, cell choice 3 &"); break;
      case mC_UNIQUE_CELL_1: fprintf(stderr, "-c, cell choice 1 &"); break;
      case mC_UNIQUE_CELL_2: fprintf(stderr, "-c, cell choice 2 &"); break;
      case mC_UNIQUE_CELL_3: fprintf(stderr, "-c, cell choice 3 &"); break;
      case ORIGIN_1_ABC: fprintf(stderr, "abc &"); break;
      case ORIGIN_1_CAB: fprintf(stderr, "cab &"); break;
      case ORIGIN_1_BCA: fprintf(stderr, "bca &"); break;
      case ORIGIN_1_BACm: fprintf(stderr, "ba-c &"); break;
      case ORIGIN_1_CmBA: fprintf(stderr, "-cba &"); break;
      case ORIGIN_1_ACmB: fprintf(stderr, "a-cb &"); break;
      case ORIGIN_2_CAB: fprintf(stderr, "cab, cell choice 2 &"); break;
      case ORIGIN_2_BCA: fprintf(stderr, "bca, cell choice 2 &"); break;
      case ORIGIN_2_BACm: fprintf(stderr, "ba-c, cell choice 2 &"); break;
      case ORIGIN_2_CmBA: fprintf(stderr, "-cba, cell choice 2 &"); break;
      case ORIGIN_2_ACmB: fprintf(stderr, "a-cb, cell choice 2 &"); break;
      case HEXAGONAL: fprintf(stderr, "hexagonal &"); break;
      case RHOMBOHEDRAL: fprintf(stderr, "Rhombohedral &"); break;
    }
    switch(SpaceGroupData[i].Centered)
    {
      case PRIMITIVE_CENTERED: fprintf(stderr, "primitive &"); break;
      case A_CENTERED: fprintf(stderr, "a &"); break;
      case B_CENTERED: fprintf(stderr, "b &"); break;
      case C_CENTERED: fprintf(stderr, "c &"); break;
      case BODY_CENTERED: fprintf(stderr, "body &"); break;
      case FACE_CENTERED: fprintf(stderr, "face &"); break;
      case RHOMBOHEDRAL_CENTERED: fprintf(stderr, "rhombohedral &"); break;
    }
    fprintf(stderr, "%d &",SpaceGroupData[i].NumberOfOperators);
    fprintf(stderr, "%s &",SpaceGroupData[i].Chiral?"yes":"no");
    fprintf(stderr, "%s &",SpaceGroupData[i].Centric?"yes":"no");
    fprintf(stderr, "%s \\\\ \n",SpaceGroupData[i].Enantiomorphic?"yes":"no");
  }
}


// Materials studio uses the elements of a space group to distinguish between different origins.
// e.g. space group 227 has two origins, both are labeled "F d -3 c". Materials Studio unfortunately ignores the Hall-symbol
// in cif-files. Therefore we list here the elements for space-groups that have different origins.
// The order of the elements is compatible with Materials Studio and equal to the International Tables for
// Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)

char StringSpaceGroupElements[NUMBER_OF_SPACEGROUPS+1][192][32]=
{
{{""}},

    // TRICLINIC GROUPS
    // ================

// [1] P 1 (P 1)
{{"x,y,z"}},

// [2] P -1 (P -1)
{{"x,y,z"},{"-x,-y,-z"}},

    // MONOCLINIC GROUPS
    // =================

// [3] P 1 2 1 unique b axis (P 2y)
{{"x,y,z"},{"-x,y,-z"}},

// [3] P 1 1 2 unique c axis (P 2)
{{"x,y,z"},{"-x,-y,z"}},

// [3] P 2 1 1 unique a axis (P 2x)
{{"x,y,z"},{"x,-y,-z"}},

// [4] P 1 21 1 unique b axis (P 2yb)
{{"x,y,z"},{"-x,y+1/2,-z"}},

// [4] P 1 1 21 unique c axis (P 2c)
{{"x,y,z"},{"-x,-y,z+1/2"}},

// [4] P 21 1 1 unique a axis (P 2xa)
{{"x,y,z"},{"x+1/2,-y,-z"}},

// [5] C 1 2 1 unique b axis: cell choice 1 (C 2y)
{{"x,y,z"},{"-x,y,-z"},{"x+1/2,y+1/2,z"},{"-x+1/2,y+1/2,-z"}},

// [5] A 1 2 1 unique b axis: cell choice 2 (A 2y)
{{"x,y,z"},{"-x,y,-z"},{"x,y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"}},

// [5] I 1 2 1 unique b axis: cell choice 3 (I 2y)
{{"x,y,z"},{"-x,y,-z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"}},

// [5] A 1 1 2 unique c axis: cell choice 1 (A 2)
{{"x,y,z"},{"-x,-y,z"},{"x,y+1/2,z+1/2"},{"-x,-y+1/2,z+1/2"}},

// [5] B 1 1 2 unique c axis: cell choice 2 (B 2)
{{"x,y,z"},{"-x,-y,z"},{"x+1/2,y,z+1/2"},{"-x+1/2,-y,z+1/2"}},

// [5] I 1 1 2 unique c axis: cell choice 3 (I 2)
{{"x,y,z"},{"-x,-y,z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"}},

// [5] B 2 1 1 unique a axis: cell choice 1 (B 2x)
{{"x,y,z"},{"x,-y,-z"},{"x+1/2,y,z+1/2"},{"x+1/2,-y,-z+1/2"}},

// [5] C 2 1 1 unique a axis: cell choice 2 (C 2x)
{{"x,y,z"},{"x,-y,-z"},{"x+1/2,y+1/2,z"},{"x+1/2,-y+1/2,-z"}},

// [5] I 2 1 1 unique a axis: cell choice 3 (I 2x)
{{"x,y,z"},{"x,-y,-z"},{"x+1/2,y+1/2,z+1/2"},{"x+1/2,-y+1/2,-z+1/2"}},

// [6] P 1 m 1 unique b axis (P -2y)
{{"x,y,z"},{"x,-y,z"}},

// [6] P 1 1 m unique c axis (P -2)
{{"x,y,z"},{"x,y,-z"}},

// [6] P m 1 1 unique a axis (P -2x)
{{"x,y,z"},{"-x,y,z"}},

// [7] P 1 c 1 unique b axis: cell choice 1 (P -2yc)
{{"x,y,z"},{"x,-y,z+1/2"}},

// [7] P 1 n 1 unique b axis: cell choice 2 (P -2yac)
{{"x,y,z"},{"x+1/2,-y,z+1/2"}},

// [7] P 1 a 1 unique b axis: cell choice 3 (P -2ya)
{{"x,y,z"},{"x+1/2,-y,z"}},

// [7] P 1 1 a unique c axis: cell choice 1 (P -2a)
{{"x,y,z"},{"x+1/2,y,-z"}},

// [7] P 1 1 n unique c axis: cell choice 2 (P -2ab)
{{"x,y,z"},{"x+1/2,y+1/2,-z"}},

// [7] P 1 1 b unique c axis: cell choice 3 (P -2b)
{{"x,y,z"},{"x,y+1/2,-z"}},

// [7] P b 1 1 unique a axis: cell choice 1 (P -2xb)
{{"x,y,z"},{"-x,y+1/2,z"}},

// [7] P n 1 1 unique a axis: cell choice 2 (P -2xbc)
{{"x,y,z"},{"-x,y+1/2,z+1/2"}},

// [7] P c 1 1 unique a axis: cell choice 3 (P -2xc)
{{"x,y,z"},{"-x,y,z+1/2"}},

// [8] C 1 m 1 unique b axis: cell choice 1 (C -2y)
{{"x,y,z"},{"x,-y,z"},{"x+1/2,y+1/2,z"},{"x+1/2,-y+1/2,z"}},

// [8] A 1 m 1 unique b axis: cell choice 2 (A -2y)
{{"x,y,z"},{"x,-y,z"},{"x,y+1/2,z+1/2"},{"x,-y+1/2,z+1/2"}},

// [8] I 1 m 1 unique b axis: cell choice 3 (I -2y)
{{"x,y,z"},{"x,-y,z"},{"x+1/2,y+1/2,z+1/2"},{"x+1/2,-y+1/2,z+1/2"}},

// [8] A 1 1 m unique c axis: cell choice 1 (A -2)
{{"x,y,z"},{"x,y,-z"},{"x,y+1/2,z+1/2"},{"x,y+1/2,-z+1/2"}},

// [8] B 1 1 m  unique c axis: cell choice 2 (B -2)
{{"x,y,z"},{"x,y,-z"},{"x+1/2,y,z+1/2"},{"x+1/2,y,-z+1/2"}},

// [8] I 1 1 m unique c axis: cell choice 3 (I -2)
{{"x,y,z"},{"x,y,-z"},{"x+1/2,y+1/2,z+1/2"},{"x+1/2,y+1/2,-z+1/2"}},

// [8] B m 1 1 unique a axis: cell choice 1 (B -2x)
{{"x,y,z"},{"-x,y,z"},{"x+1/2,y,z+1/2"},{"-x+1/2,y,z+1/2"}},

// [8] C m 1 1 unique a axis: cell choice 2 (C -2x)
{{"x,y,z"},{"-x,y,z"},{"x+1/2,y+1/2,z"},{"-x+1/2,y+1/2,z"}},

// [8] I m 1 1 unique a axis: cell choice 3 (I -2x)
{{"x,y,z"},{"-x,y,z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [9] C 1 c 1 unique b axis: cell choice 1 (C -2yc)
{{"x,y,z"},{"x,-y,z+1/2"},{"x+1/2,y+1/2,z"},{"x+1/2,-y+1/2,z+1/2"}},

// [9] A 1 n 1 unique b axis: cell choice 2 (A -2yab)
{{"x,y,z"},{"x+1/2,-y,z+1/2"},{"x,y+1/2,z+1/2"},{"x+1/2,-y+1/2,z"}},

// [9] I 1 a 1 unique b axis: cell choice 3 (I -2ya)
{{"x,y,z"},{"x+1/2,-y,z"},{"x+1/2,y+1/2,z+1/2"},{"x,-y+1/2,z+1/2"}},

// [9] A 1 a 1 unique -b axis: cell choice 1 (A -2ya)
{{"x,y,z"},{"x+1/2,-y,z"},{"x,y+1/2,z+1/2"},{"x+1/2,-y+1/2,z+1/2"}},

// [9] C 1 n 1 unique -b axis: cell choice 2 (C -2yac)
{{"x,y,z"},{"x+1/2,-y,z+1/2"},{"x+1/2,y+1/2,z"},{"x,-y+1/2,z+1/2"}},

// [9] I 1 c 1 unique -b axis: cell choice 3 (I -2yc)
{{"x,y,z"},{"x,-y,z+1/2"},{"x+1/2,y+1/2,z+1/2"},{"x+1/2,-y+1/2,z"}},

// [9] A 1 1 a unique c axis: cell choice 1 (A -2a)
{{"x,y,z"},{"x+1/2,y,-z"},{"x,y+1/2,z+1/2"},{"x+1/2,y+1/2,-z+1/2"}},

// [9] B 1 1 n unique c axis: cell choice 2 (B -2ab)
{{"x,y,z"},{"x+1/2,y+1/2,-z"},{"x+1/2,y,z+1/2"},{"x,y+1/2,-z+1/2"}},

// [9] I 1 1 b unique c axis: cell choice 3 (I -2b)
{{"x,y,z"},{"x,y+1/2,-z"},{"x+1/2,y+1/2,z+1/2"},{"x+1/2,y,-z+1/2"}},

// [9] B 1 1 b unique -c axis: cell choice 1 (B -2b)
{{"x,y,z"},{"x,y+1/2,-z"},{"x+1/2,y,z+1/2"},{"x+1/2,y+1/2,-z+1/2"}},

// [9] A 1 1 n unique -c axis: cell choice 2 (A -2ab)
{{"x,y,z"},{"x+1/2,y+1/2,-z"},{"x,y+1/2,z+1/2"},{"x+1/2,y,-z+1/2"}},

// [9] I 1 1 a unique -c axis: cell choice 3 (I -2a)
{{"x,y,z"},{"x+1/2,y,-z"},{"x+1/2,y+1/2,z+1/2"},{"x,y+1/2,-z+1/2"}},

// [9] B b 1 1 unique a axis: cell choice 1 (B -2xb)
{{"x,y,z"},{"-x,y+1/2,z"},{"x+1/2,y,z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [9] C n 1 1 unique a axis: cell choice 2 (C -2xac)
{{"x,y,z"},{"-x,y+1/2,z+1/2"},{"x+1/2,y+1/2,z"},{"-x+1/2,y,z+1/2"}},

// [9] I c 1 1 unique a axis: cell choice 3 (I -2xc)
{{"x,y,z"},{"-x,y,z+1/2"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,y+1/2,z"}},

// [9] C c 1 1 unique -a axis: cell choice 1 (C -2xc)
{{"x,y,z"},{"-x,y,z+1/2"},{"x+1/2,y+1/2,z"},{"-x+1/2,y+1/2,z+1/2"}},

// [9] B n 1 1 unique -a axis: cell choice 2 (B -2xab)
{{"x,y,z"},{"-x,y+1/2,z+1/2"},{"x+1/2,y,z+1/2"},{"-x+1/2,y+1/2,z"}},

// [9] I b 1 1 unique -a axis: cell choice 3 (I -2xb)
{{"x,y,z"},{"-x,y+1/2,z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,y,z+1/2"}},

// [10] P 1 2/m 1 unique b axis (-P 2y)
{{"x,y,z"},{"-x,y,-z"},{"-x,-y,-z"},{"x,-y,z"}},

// [10] P 1 1 2/m unique c axis (-P 2)
{{"x,y,z"},{"-x,-y,z"},{"-x,-y,-z"},{"x,y,-z"}},

// [10] P 2/m 1 1 unique a axis (-P 2x)
{{"x,y,z"},{"x,-y,-z"},{"-x,-y,-z"},{"-x,y,z"}},

// [11] P 1 21/m 1 unique b axis (-P 2yb)
{{"x,y,z"},{"-x,y+1/2,-z"},{"-x,-y,-z"},{"x,-y+1/2,z"}},

// [11] P 1 1 21/m unique c axis (-P 2c)
{{"x,y,z"},{"-x,-y,z+1/2"},{"-x,-y,-z"},{"x,y,-z+1/2"}},

// [11] P 21/m 1 1 unique a axis (-P 2xa)
{{"x,y,z"},{"x+1/2,-y,-z"},{"-x,-y,-z"},{"-x+1/2,y,z"}},

// [12] C 1 2/m 1 unique b axis: cell choice 1 (-C 2y)
{{"x,y,z"},{"-x,y,-z"},{"-x,-y,-z"},{"x,-y,z"},{"x+1/2,y+1/2,z"},{"-x+1/2,y+1/2,-z"},{"-x+1/2,-y+1/2,-z"},{"x+1/2,-y+1/2,z"}},

// [12] A 1 2/m 1 unique b axis: cell choice 2 (-A 2y)
{{"x,y,z"},{"-x,y,-z"},{"-x,-y,-z"},{"x,-y,z"},{"x,y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"-x,-y+1/2,-z+1/2"},{"x,-y+1/2,z+1/2"}},

// [12] I 1 2/m 1 unique b axis: cell choice 3 (-I 2y)
{{"x,y,z"},{"-x,y,-z"},{"-x,-y,-z"},{"x,-y,z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"}},

// [12] A 1 1 2/m unique c axis: cell choice 1 (-A 2)
{{"x,y,z"},{"-x,-y,z"},{"-x,-y,-z"},{"x,y,-z"},{"x,y+1/2,z+1/2"},{"-x,-y+1/2,z+1/2"},{"-x,-y+1/2,-z+1/2"},{"x,y+1/2,-z+1/2"}},

// [12] B 1 1 2/m unique c axis: cell choice 2 (-B 2)
{{"x,y,z"},{"-x,-y,z"},{"-x,-y,-z"},{"x,y,-z"},{"x+1/2,y,z+1/2"},{"-x+1/2,-y,z+1/2"},{"-x+1/2,-y,-z+1/2"},{"x+1/2,y,-z+1/2"}},

// [12] I 1 1 2/m unique c axis: cell choice 3 (-I 2)
{{"x,y,z"},{"-x,-y,z"},{"-x,-y,-z"},{"x,y,-z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"}},

// [12] B 2/m 1 1 unique a axis: cell choice 1 (-B 2x)
{{"x,y,z"},{"x,-y,-z"},{"-x,-y,-z"},{"-x,y,z"},{"x+1/2,y,z+1/2"},{"x+1/2,-y,-z+1/2"},{"-x+1/2,-y,-z+1/2"},{"-x+1/2,y,z+1/2"}},

// [12] C 2/m 1 1 unique a axis: cell choice 2 (-C 2x)
{{"x,y,z"},{"x,-y,-z"},{"-x,-y,-z"},{"-x,y,z"},{"x+1/2,y+1/2,z"},{"x+1/2,-y+1/2,-z"},{"-x+1/2,-y+1/2,-z"},{"-x+1/2,y+1/2,z"}},

// [12] I 2/m 1 1 unique a axis: cell choice 3 (-I 2x)
{{"x,y,z"},{"x,-y,-z"},{"-x,-y,-z"},{"-x,y,z"},{"x+1/2,y+1/2,z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [13] P 1 2/c 1 unique b axis: cell choice 1 (-P 2yc)
{{"x,y,z"},{"-x,y,-z+1/2"},{"-x,-y,-z"},{"x,-y,z+1/2"}},

// [13] P 1 2/n 1 unique b axis: cell choice 2 (-P 2yac)
{{"x,y,z"},{"-x+1/2,y,-z+1/2"},{"-x,-y,-z"},{"x+1/2,-y,z+1/2"}},

// [13] P 1 2/a 1 unique b axis: cell choice 3 (-P 2ya)
{{"x,y,z"},{"-x+1/2,y,-z"},{"-x,-y,-z"},{"x+1/2,-y,z"}},

// [13] P 1 1 2/a unique c axis: cell choice 1 (-P 2a)
{{"x,y,z"},{"-x+1/2,-y,z"},{"-x,-y,-z"},{"x+1/2,y,-z"}},

// [13] P 1 1 2/n unique c axis: cell choice 2 (-P 2ab)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"}},

// [13] P 1 1 2/b unique c axis: cell choice 3 (-P 2b)
{{"x,y,z"},{"-x,-y+1/2,z"},{"-x,-y,-z"},{"x,y+1/2,-z"}},

// [13] P 2/b 1 1 unique a axis: cell choice 1 (-P 2xb)
{{"x,y,z"},{"x,-y+1/2,-z"},{"-x,-y,-z"},{"-x,y+1/2,z"}},

// [13] P 2/n 1 1 unique a axis: cell choice 2 (-P 2xbc)
{{"x,y,z"},{"x,-y+1/2,-z+1/2"},{"-x,-y,-z"},{"-x,y+1/2,z+1/2"}},

// [13] P 2/c 1 1 unique a axis: cell choice 3 (-P 2xc)
{{"x,y,z"},{"x,-y,-z+1/2"},{"-x,-y,-z"},{"-x,y,z+1/2"}},

// [14] P 1 21/c 1 unique b axis: cell choice 1 (-P 2ybc)
{{"x,y,z"},{"-x,y+1/2,-z+1/2"},{"-x,-y,-z"},{"x,-y+1/2,z+1/2"}},

// [14] P 1 21/n 1 unique b axis: cell choice 2 (-P 2yn)
{{"x,y,z"},{"-x+1/2,y+1/2,-z+1/2"},{"-x,-y,-z"},{"x+1/2,-y+1/2,z+1/2"}},

// [14] P 1 21/a 1 unique b axis: cell choice 3 (-P 2yab)
{{"x,y,z"},{"-x+1/2,y+1/2,-z"},{"-x,-y,-z"},{"x+1/2,-y+1/2,z"}},

// [14] P 1 1 21/a unique c axis: cell choice 1 (-P 2ac)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"-x,-y,-z"},{"x+1/2,y,-z+1/2"}},

// [14] P 1 1 21/n unique c axis: cell choice 2 (-P 2n)
{{"x,y,z"},{"-x+1/2,-y+1/2,z+1/2"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z+1/2"}},

// [14] P 1 1 21/b unique c axis: cell choice 3 (-P 2bc)
{{"x,y,z"},{"-x,-y+1/2,z+1/2"},{"-x,-y,-z"},{"x,y+1/2,-z+1/2"}},

// [14] P 21/b 1 1 unique a axis: cell choice 1 (-P 2xab)
{{"x,y,z"},{"x+1/2,-y+1/2,-z"},{"-x,-y,-z"},{"-x+1/2,y+1/2,z"}},

// [14] P 21/n 1 1 unique a axis: cell choice 2 (-P 2xn)
{{"x,y,z"},{"x+1/2,-y+1/2,-z+1/2"},{"-x,-y,-z"},{"-x+1/2,y+1/2,z+1/2"}},

// [14] P 21/c 1 1 unique a axis: cell choice 3 (-P 2xac)
{{"x,y,z"},{"x+1/2,-y,-z+1/2"},{"-x,-y,-z"},{"-x+1/2,y,z+1/2"}},

// [15] C 1 2/c 1 unique b axis: cell choice 1 (-C 2yc)
{{"x,y,z"},{"-x,y,-z+1/2"},{"-x,-y,-z"},{"x,-y,z+1/2"},{"x+1/2,y+1/2,z"},{"-x+1/2,y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,-z"},{"x+1/2,-y+1/2,z+1/2"}},

// [15] A 1 2/n 1 unique b axis: cell choice 2 (-A 2yab)
{{"x,y,z"},{"-x+1/2,y,-z+1/2"},{"-x,-y,-z"},{"x+1/2,-y,z+1/2"},{"x,y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z"},{"-x,-y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z"}},

// [15] I 1 2/a 1 unique b axis: cell choice 3 (-I 2ya)
{{"x,y,z"},{"-x+1/2,y,-z"},{"-x,-y,-z"},{"x+1/2,-y,z"},{"x+1/2,y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x,-y+1/2,z+1/2"}},

// [15] A 1 2/a 1 unique -b axis: cell choice 1 (-A 2ya)
{{"x,y,z"},{"-x+1/2,y,-z"},{"-x,-y,-z"},{"x+1/2,-y,z"},{"x,y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"-x,-y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"}},

// [15] C 1 2/n 1 unique -b axis: cell choice 2 (-C 2yac)
{{"x,y,z"},{"-x+1/2,y,-z+1/2"},{"-x,-y,-z"},{"x+1/2,-y,z+1/2"},{"x+1/2,y+1/2,z"},{"-x,y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,-z"},{"x,-y+1/2,z+1/2"}},

// [15] I 1 2/c 1 unique -b axis: cell choice 3 (-I 2yc)
{{"x,y,z"},{"-x,y,-z+1/2"},{"-x,-y,-z"},{"x,-y,z+1/2"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z"}},

// [15] A 1 1 2/a unique c axis: cell choice 1 (-A 2a)
{{"x,y,z"},{"-x+1/2,-y,z"},{"-x,-y,-z"},{"x+1/2,y,-z"},{"x,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-x,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"}},

// [15] B 1 1 2/n unique c axis: cell choice 2 (-B 2ab)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"},{"x+1/2,y,z+1/2"},{"-x,-y+1/2,z+1/2"},{"-x+1/2,-y,-z+1/2"},{"x,y+1/2,-z+1/2"}},

// [15] I 1 1 2/b unique c axis: cell choice 3 (-I 2b)
{{"x,y,z"},{"-x,-y+1/2,z"},{"-x,-y,-z"},{"x,y+1/2,-z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y,z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y,-z+1/2"}},

// [15] B 1 1 2/b unique -c axis: cell choice 1 (-B 2b)
{{"x,y,z"},{"-x,-y+1/2,z"},{"-x,-y,-z"},{"x,y+1/2,-z"},{"x+1/2,y,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-x+1/2,-y,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"}},

// [15] A 1 1 2/n unique -c axis: cell choice 2 (-A 2ab)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"},{"x,y+1/2,z+1/2"},{"-x+1/2,-y,z+1/2"},{"-x,-y+1/2,-z+1/2"},{"x+1/2,y,-z+1/2"}},

// [15] I 1 1 2/a unique -c axis: cell choice 3 (-I 2a)
{{"x,y,z"},{"-x+1/2,-y,z"},{"-x,-y,-z"},{"x+1/2,y,-z"},{"x+1/2,y+1/2,z+1/2"},{"-x,-y+1/2,z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x,y+1/2,-z+1/2"}},

// [15] B 2/b 1 1 unique a axis: cell choice 1 (-B 2xb)
{{"x,y,z"},{"x,-y+1/2,-z"},{"-x,-y,-z"},{"-x,y+1/2,z"},{"x+1/2,y,z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,-y,-z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [15] C 2/n 1 1 unique a axis: cell choice 2 (-C 2xac)
{{"x,y,z"},{"x,-y+1/2,-z+1/2"},{"-x,-y,-z"},{"-x,y+1/2,z+1/2"},{"x+1/2,y+1/2,z"},{"x+1/2,-y,-z+1/2"},{"-x+1/2,-y+1/2,-z"},{"-x+1/2,y,z+1/2"}},

// [15] I 2/c 1 1 unique a axis: cell choice 3 (-I 2xc)
{{"x,y,z"},{"x,-y,-z+1/2"},{"-x,-y,-z"},{"-x,y,z+1/2"},{"x+1/2,y+1/2,z+1/2"},{"x+1/2,-y+1/2,-z"},{"-x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,y+1/2,z"}},

// [15] C 2/c 1 1 unique -a axis: cell choice 1 (-C 2xc)
{{"x,y,z"},{"x,-y,-z+1/2"},{"-x,-y,-z"},{"-x,y,z+1/2"},{"x+1/2,y+1/2,z"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,-z"},{"-x+1/2,y+1/2,z+1/2"}},

// [15] B 2/n 1 1 unique -a axis: cell choice 2 (-B 2xab)
{{"x,y,z"},{"x,-y+1/2,-z+1/2"},{"-x,-y,-z"},{"-x,y+1/2,z+1/2"},{"x+1/2,y,z+1/2"},{"x+1/2,-y+1/2,-z"},{"-x+1/2,-y,-z+1/2"},{"-x+1/2,y+1/2,z"}},

// [15] I 2/b 1 1 unique -a axis: cell choice 3 (-I 2xb)
{{"x,y,z"},{"x,-y+1/2,-z"},{"-x,-y,-z"},{"-x,y+1/2,z"},{"x+1/2,y+1/2,z+1/2"},{"x+1/2,-y,-z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,y,z+1/2"}},

// [16] P 2 2 2 (P 2 2)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"}},

// [17] P 2 2 21 Origin-1,abc (P 2c 2)
{{"x,y,z"},{"-x,-y,z+1/2"},{"-x,y,-z+1/2"},{"x,-y,-z"}},

// [17] P 21 2 2 Origin-1,cab (P 2a 2a)
{{"x,y,z"},{"x+1/2,-y,-z"},{"-x+1/2,-y,z"},{"-x,y,-z"}},

// [17] P 2 21 2 Origin-1,bca (P 2 2b)
{{"x,y,z"},{"-x,y+1/2,-z"},{"x,-y+1/2,-z"},{"-x,-y,z"}},

// [18] P 21 21 2 Origin-1,abc (P 2 2ab)
{{"x,y,z"},{"-x,-y,z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"}},

// [18] P 2 21 21 Origin-1,cab (P 2bc 2)
{{"x,y,z"},{"x,-y,-z"},{"-x,-y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"}},

// [18] P 21 2 21 Origin-1,bca (P 2ac 2ac)
{{"x,y,z"},{"-x,y,-z"},{"x+1/2,-y,-z+1/2"},{"-x+1/2,-y,z+1/2"}},

// [19] P 21 21 21 (P 2ac 2ab)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z"}},

// [20] C 2 2 21  Origin-1,abc (C 2c 2)
{{"x,y,z"},{"-x,-y,z+1/2"},{"-x,y,-z+1/2"},{"x,-y,-z"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z"}},

// [20] A 21 2 2  Origin-1,cba (A 2a 2a)
{{"x,y,z"},{"x+1/2,-y,-z"},{"-x+1/2,-y,z"},{"-x,y,-z"},{"x,y+1/2,z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"}},

// [20] B 2 21 2  Origin-1,bca (B 2 2b)
{{"x,y,z"},{"-x,y+1/2,-z"},{"x,-y+1/2,-z"},{"-x,-y,z"},{"x+1/2,y,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,-y,z+1/2"}},

// [21] C 2 2 2 Origin-1,abc (C 2 2)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"}},

// [21] A 2 2 2 Origin-1,cab (A 2 2)
{{"x,y,z"},{"x,-y,-z"},{"-x,-y,z"},{"-x,y,-z"},{"x,y+1/2,z+1/2"},{"x,-y+1/2,-z+1/2"},{"-x,-y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"}},

// [21] B 2 2 2 Origin-1,bca (B 2 2)
{{"x,y,z"},{"-x,y,-z"},{"-x,-y,z"},{"x,-y,-z"},{"x+1/2,y,z+1/2"},{"-x+1/2,y,-z+1/2"},{"-x+1/2,-y,z+1/2"},{"x+1/2,-y,-z+1/2"}},

// [22] F 2 2 2 (F 2 2)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"x,y+1/2,z+1/2"},{"-x,-y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},
{"x,-y+1/2,-z+1/2"},{"x+1/2,y,z+1/2"},{"-x+1/2,-y,z+1/2"},{"-x+1/2,y,-z+1/2"},{"x+1/2,-y,-z+1/2"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z"},
{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"}},

// [23] I 2 2 2 (I 2 2)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"}},

// [24] I 21 21 21 (I 2b 2c)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z"},{"x+1/2,y+1/2,z+1/2"},{"-x,-y+1/2,z"},{"-x+1/2,y,-z"},{"x,-y,-z+1/2"}},

// [25] P m m 2 (P 2 -2)
{{"x,y,z"},{"-x,-y,z"},{"x,-y,z"},{"-x,y,z"}},

// [25] P 2 m m (P -2 2)
{{"x,y,z"},{"x,-y,-z"},{"x,y,-z"},{"x,-y,z"}},

// [25] P m 2 m (P -2 -2)
{{"x,y,z"},{"-x,y,-z"},{"x,y,-z"},{"-x,y,z"}},

// [26] P m c 21 (P 2c -2)
{{"x,y,z"},{"-x,-y,z+1/2"},{"x,-y,z+1/2"},{"-x,y,z"}},

// [26] P c m 21 (P 2c -2c)
{{"x,y,z"},{"-x,-y,z+1/2"},{"-x,y,z+1/2"},{"x,-y,z"}},

// [26] P 21 m a (P -2a 2a)
{{"x,y,z"},{"x+1/2,-y,-z"},{"x+1/2,y,-z"},{"x,-y,z"}},

// [26] P 21 a m (P -2 2a)
{{"x,y,z"},{"x+1/2,-y,-z"},{"x+1/2,-y,z"},{"x,y,-z"}},

// [26] P b 21 m (P -2 -2b)
{{"x,y,z"},{"-x,y+1/2,-z"},{"-x,y+1/2,z"},{"x,y,-z"}},

// [26] P m 21 b (P -2b -2)
{{"x,y,z"},{"-x,y+1/2,-z"},{"x,y+1/2,-z"},{"-x,y,z"}},

// [27] P c c 2 (P 2 -2c)
{{"x,y,z"},{"-x,-y,z"},{"x,-y,z+1/2"},{"-x,y,z+1/2"}},

// [27] P 2 a a (P -2a 2)
{{"x,y,z"},{"x,-y,-z"},{"x+1/2,y,-z"},{"x+1/2,-y,z"}},

// [27] P b 2 b (P -2b -2b)
{{"x,y,z"},{"-x,y,-z"},{"x,y+1/2,-z"},{"-x,y+1/2,z"}},

// [28] P m a 2 (P 2 -2a)
{{"x,y,z"},{"-x,-y,z"},{"x+1/2,-y,z"},{"-x+1/2,y,z"}},

// [28] P b m 2 (P 2 -2b)
{{"x,y,z"},{"-x,-y,z"},{"-x,y+1/2,z"},{"x,-y+1/2,z"}},

// [28] P 2 m b (P -2b 2)
{{"x,y,z"},{"x,-y,-z"},{"x,y+1/2,-z"},{"x,-y+1/2,z"}},

// [28] P 2 c m (P -2c 2)
{{"x,y,z"},{"x,-y,-z"},{"x,-y,z+1/2"},{"x,y,-z+1/2"}},

// [28] P c 2 m (P -2c -2c)
{{"x,y,z"},{"-x,y,-z"},{"-x,y,z+1/2"},{"x,y,-z+1/2"}},

// [28] P m 2 a (P -2a -2a)
{{"x,y,z"},{"-x,y,-z"},{"x+1/2,y,-z"},{"-x+1/2,y,z"}},

// [29] P c a 21 (P 2c -2ac)
{{"x,y,z"},{"-x,-y,z+1/2"},{"x+1/2,-y,z"},{"-x+1/2,y,z+1/2"}},

// [29] P b c 21 (P 2c -2b)
{{"x,y,z"},{"-x,-y,z+1/2"},{"-x,y+1/2,z"},{"x,-y+1/2,z+1/2"}},

// [29] P 21 a b (P -2b 2a)
{{"x,y,z"},{"x+1/2,-y,-z"},{"x,y+1/2,-z"},{"x+1/2,-y+1/2,z"}},

// [29] P 21 c a (P -2ac 2a)
{{"x,y,z"},{"x+1/2,-y,-z"},{"x,-y,z+1/2"},{"x+1/2,y,-z+1/2"}},

// [29] P c 21 b (P -2bc -2c)
{{"x,y,z"},{"-x,y+1/2,-z"},{"-x,y,z+1/2"},{"x,y+1/2,-z+1/2"}},

// [29] P b 21 a (P -2a -2ab)
{{"x,y,z"},{"-x,y+1/2,-z"},{"x+1/2,y,-z"},{"-x+1/2,y+1/2,z"}},

// [30] P n c 2 (P 2 -2bc)
{{"x,y,z"},{"-x,-y,z"},{"x,-y+1/2,z+1/2"},{"-x,y+1/2,z+1/2"}},

// [30] P c n 2 (P 2 -2ac)
{{"x,y,z"},{"-x,-y,z"},{"-x+1/2,y,z+1/2"},{"x+1/2,-y,z+1/2"}},

// [30] P 2 n a (P -2ac 2)
{{"x,y,z"},{"x,-y,-z"},{"x+1/2,y,-z+1/2"},{"x+1/2,-y,z+1/2"}},

// [30] P 2 a n (P -2ab 2)
{{"x,y,z"},{"x,-y,-z"},{"x+1/2,-y+1/2,z"},{"x+1/2,y+1/2,-z"}},

// [30] P b 2 n (P -2ab -2ab)
{{"x,y,z"},{"-x,y,-z"},{"-x+1/2,y+1/2,z"},{"x+1/2,y+1/2,-z"}},

// [30] P n 2 b (P -2bc -2bc)
{{"x,y,z"},{"-x,y,-z"},{"x,y+1/2,-z+1/2"},{"-x,y+1/2,z+1/2"}},

// [31] P m n 21 (P 2ac -2)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"x+1/2,-y,z+1/2"},{"-x,y,z"}},

// [31] P n m 21 (P 2bc -2bc)
{{"x,y,z"},{"-x,-y+1/2,z+1/2"},{"-x,y+1/2,z+1/2"},{"x,-y,z"}},

// [31] P 21 m n (P -2ab 2ab)
{{"x,y,z"},{"x+1/2,-y+1/2,-z"},{"x+1/2,y+1/2,-z"},{"x,-y,z"}},

// [31] P 21 n m (P -2 2ac)
{{"x,y,z"},{"x+1/2,-y,-z+1/2"},{"x+1/2,-y,z+1/2"},{"x,y,-z"}},

// [31] P n 21 m (P -2 -2bc)
{{"x,y,z"},{"-x,y+1/2,-z+1/2"},{"-x,y+1/2,z+1/2"},{"x,y,-z"}},

// [31] P m 21 n (P -2ab -2)
{{"x,y,z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,y+1/2,-z"},{"-x,y,z"}},

// [32] P b a 2 (P 2 -2ab)
{{"x,y,z"},{"-x,-y,z"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"}},

// [32] P 2 c b (P -2bc 2)
{{"x,y,z"},{"x,-y,-z"},{"x,y+1/2,-z+1/2"},{"x,-y+1/2,z+1/2"}},

// [32] P c 2 a (P -2ac -2ac)
{{"x,y,z"},{"-x,y,-z"},{"x+1/2,y,-z+1/2"},{"-x+1/2,y,z+1/2"}},

// [33] P n a 21 (P 2c -2n)
{{"x,y,z"},{"-x,-y,z+1/2"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z+1/2"}},

// [33] P b n 21 (P 2c -2ab)
{{"x,y,z"},{"-x,-y,z+1/2"},{"-x+1/2,y+1/2,z"},{"x+1/2,-y+1/2,z+1/2"}},

// [33] P 21 n b (P -2bc 2a)
{{"x,y,z"},{"x+1/2,-y,-z"},{"x,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"}},

// [33] P 21 c n (P -2n 2a)
{{"x,y,z"},{"x+1/2,-y,-z"},{"x,-y+1/2,z+1/2"},{"x+1/2,y+1/2,-z+1/2"}},

// [33] P c 21 n (P -2n -2ac)
{{"x,y,z"},{"-x,y+1/2,-z"},{"-x+1/2,y,z+1/2"},{"x+1/2,y+1/2,-z+1/2"}},

// [33] P n 21 a (P -2ac -2n)
{{"x,y,z"},{"-x,y+1/2,-z"},{"x+1/2,y,-z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [34] P n n 2 (P 2 -2n)
{{"x,y,z"},{"-x,-y,z"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [34] P 2 n n (P -2n 2)
{{"x,y,z"},{"x,-y,-z"},{"x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"}},

// [34] P n 2 n (P -2n -2n)
{{"x,y,z"},{"-x,y,-z"},{"x+1/2,y+1/2,-z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [35] C m m 2 (C 2 -2)
{{"x,y,z"},{"-x,-y,z"},{"x,-y,z"},{"-x,y,z"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"}},

// [35] A 2 m m (A -2 2)
{{"x,y,z"},{"x,-y,-z"},{"x,y,-z"},{"x,-y,z"},{"x,y+1/2,z+1/2"},{"x,-y+1/2,-z+1/2"},{"x,y+1/2,-z+1/2"},{"x,-y+1/2,z+1/2"}},

// [35] B m 2 m (B -2 -2)
{{"x,y,z"},{"-x,y,-z"},{"x,y,-z"},{"-x,y,z"},{"x+1/2,y,z+1/2"},{"-x+1/2,y,-z+1/2"},{"x+1/2,y,-z+1/2"},{"-x+1/2,y,z+1/2"}},

// [36] C m c 21 (C 2c -2)
{{"x,y,z"},{"-x,-y,z+1/2"},{"x,-y,z+1/2"},{"-x,y,z"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z+1/2"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z"}},

// [36] C c m 21 (C 2c -2c)
{{"x,y,z"},{"-x,-y,z+1/2"},{"-x,y,z+1/2"},{"x,-y,z"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"},{"x+1/2,-y+1/2,z"}},

// [36] A 21 m a (A -2a 2a)
{{"x,y,z"},{"x+1/2,-y,-z"},{"x+1/2,y,-z"},{"x,-y,z"},{"x,y+1/2,z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"x,-y+1/2,z+1/2"}},

// [36] A 21 a m (A -2 2a)
{{"x,y,z"},{"x+1/2,-y,-z"},{"x+1/2,-y,z"},{"x,y,-z"},{"x,y+1/2,z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"},{"x,y+1/2,-z+1/2"}},

// [36] B b 21 m (B -2 -2b)
{{"x,y,z"},{"-x,y+1/2,-z"},{"-x,y+1/2,z"},{"x,y,-z"},{"x+1/2,y,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"-x+1/2,y+1/2,z+1/2"},{"x+1/2,y,-z+1/2"}},

// [36] B m 21 b (B -2b -2)
{{"x,y,z"},{"-x,y+1/2,-z"},{"x,y+1/2,-z"},{"-x,y,z"},{"x+1/2,y,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"-x+1/2,y,z+1/2"}},

// [37] C c c 2 (C 2 -2c)
{{"x,y,z"},{"-x,-y,z"},{"x,-y,z+1/2"},{"-x,y,z+1/2"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [37] A 2 a a (A -2a 2)
{{"x,y,z"},{"x,-y,-z"},{"x+1/2,y,-z"},{"x+1/2,-y,z"},{"x,y+1/2,z+1/2"},{"x,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"}},

// [37] B b 2 b (B -2b -2b)
{{"x,y,z"},{"-x,y,-z"},{"x,y+1/2,-z"},{"-x,y+1/2,z"},{"x+1/2,y,z+1/2"},{"-x+1/2,y,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"}},

// [38] A m m 2 (A 2 -2)
{{"x,y,z"},{"-x,-y,z"},{"x,-y,z"},{"-x,y,z"},{"x,y+1/2,z+1/2"},{"-x,-y+1/2,z+1/2"},{"x,-y+1/2,z+1/2"},{"-x,y+1/2,z+1/2"}},

// [38] B m m 2 (B 2 -2)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,z"},{"x,-y,z"},{"x+1/2,y,z+1/2"},{"-x+1/2,-y,z+1/2"},{"-x+1/2,y,z+1/2"},{"x+1/2,-y,z+1/2"}},

// [38] B 2 m m (B -2 2)
{{"x,y,z"},{"x,-y,-z"},{"x,y,-z"},{"x,-y,z"},{"x+1/2,y,z+1/2"},{"x+1/2,-y,-z+1/2"},{"x+1/2,y,-z+1/2"},{"x+1/2,-y,z+1/2"}},

// [38] C 2 m m (C -2 2)
{{"x,y,z"},{"x,-y,-z"},{"x,-y,z"},{"x,y,-z"},{"x+1/2,y+1/2,z"},{"x+1/2,-y+1/2,-z"},{"x+1/2,-y+1/2,z"},{"x+1/2,y+1/2,-z"}},

// [38] C m 2 m (C -2 -2)
{{"x,y,z"},{"-x,y,-z"},{"-x,y,z"},{"x,y,-z"},{"x+1/2,y+1/2,z"},{"-x+1/2,y+1/2,-z"},{"-x+1/2,y+1/2,z"},{"x+1/2,y+1/2,-z"}},

// [38] A m 2 m (A -2 -2)
{{"x,y,z"},{"-x,y,-z"},{"x,y,-z"},{"-x,y,z"},{"x,y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x,y+1/2,-z+1/2"},{"-x,y+1/2,z+1/2"}},

// [39] A b m 2 (A 2 -2b)
{{"x,y,z"},{"-x,-y,z"},{"x,-y+1/2,z"},{"-x,y+1/2,z"},{"x,y+1/2,z+1/2"},{"-x,-y+1/2,z+1/2"},{"x,-y,z+1/2"},{"-x,y,z+1/2"}},

// [39] B m a 2 (B 2 -2a)
{{"x,y,z"},{"-x,-y,z"},{"-x+1/2,y,z"},{"x+1/2,-y,z"},{"x+1/2,y,z+1/2"},{"-x+1/2,-y,z+1/2"},{"-x,y,z+1/2"},{"x,-y,z+1/2"}},

// [39] B 2 c m (B -2a 2)
{{"x,y,z"},{"x,-y,-z"},{"x,y,-z+1/2"},{"x,-y,z+1/2"},{"x+1/2,y,z+1/2"},{"x+1/2,-y,-z+1/2"},{"x+1/2,y,-z"},{"x+1/2,-y,z"}},

// [39] C 2 m b (C -2a 2)
{{"x,y,z"},{"x,-y,-z"},{"x,-y+1/2,z"},{"x,y+1/2,-z"},{"x+1/2,y+1/2,z"},{"x+1/2,-y+1/2,-z"},{"x+1/2,-y,z"},{"x+1/2,y,-z"}},

// [39] C m 2 a (C -2a -2a)
{{"x,y,z"},{"-x,y,-z"},{"-x+1/2,y,z"},{"x+1/2,y,-z"},{"x+1/2,y+1/2,z"},{"-x+1/2,y+1/2,-z"},{"-x,y+1/2,z"},{"x,y+1/2,-z"}},

// [39] A c 2 m (A -2b -2b)
{{"x,y,z"},{"-x,y,-z"},{"x,y,-z+1/2"},{"-x,y,z+1/2"},{"x,y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x,y+1/2,-z"},{"-x,y+1/2,z"}},

// [40] A m a 2 (A 2 -2a)
{{"x,y,z"},{"-x,-y,z"},{"x+1/2,-y,z"},{"-x+1/2,y,z"},{"x,y+1/2,z+1/2"},{"-x,-y+1/2,z+1/2"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [40] B b m 2 (B 2 -2b)
{{"x,y,z"},{"-x,-y,z"},{"-x,y+1/2,z"},{"x,-y+1/2,z"},{"x+1/2,y,z+1/2"},{"-x+1/2,-y,z+1/2"},{"-x+1/2,y+1/2,z+1/2"},{"x+1/2,-y+1/2,z+1/2"}},

// [40] B 2 m b (B -2b 2)
{{"x,y,z"},{"x,-y,-z"},{"x,y+1/2,-z"},{"x,-y+1/2,z"},{"x+1/2,y,z+1/2"},{"x+1/2,-y,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"}},

// [40] C 2 c m (C -2c 2)
{{"x,y,z"},{"x,-y,-z"},{"x,-y,z+1/2"},{"x,y,-z+1/2"},{"x+1/2,y+1/2,z"},{"x+1/2,-y+1/2,-z"},{"x+1/2,-y+1/2,z+1/2"},{"x+1/2,y+1/2,-z+1/2"}},

// [40] C c 2 m (C -2c -2c)
{{"x,y,z"},{"-x,y,-z"},{"-x,y,z+1/2"},{"x,y,-z+1/2"},{"x+1/2,y+1/2,z"},{"-x+1/2,y+1/2,-z"},{"-x+1/2,y+1/2,z+1/2"},{"x+1/2,y+1/2,-z+1/2"}},

// [40] A m 2 a (A -2a -2a)
{{"x,y,z"},{"-x,y,-z"},{"x+1/2,y,-z"},{"-x+1/2,y,z"},{"x,y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [41] A b a 2 (A 2 -2ab)
{{"x,y,z"},{"-x,-y,z"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"},{"x,y+1/2,z+1/2"},{"-x,-y+1/2,z+1/2"},{"x+1/2,-y,z+1/2"},{"-x+1/2,y,z+1/2"}},

// [41] B b a 2 (B 2 -2ab)
{{"x,y,z"},{"-x,-y,z"},{"-x+1/2,y+1/2,z"},{"x+1/2,-y+1/2,z"},{"x+1/2,y,z+1/2"},{"-x+1/2,-y,z+1/2"},{"-x,y+1/2,z+1/2"},{"x,-y+1/2,z+1/2"}},

// [41] B 2 c b (B -2ab 2)
{{"x,y,z"},{"x,-y,-z"},{"x,y+1/2,-z+1/2"},{"x,-y+1/2,z+1/2"},{"x+1/2,y,z+1/2"},{"x+1/2,-y,-z+1/2"},{"x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,z"}},

// [41] C 2 c b (C -2ac 2)
{{"x,y,z"},{"x,-y,-z"},{"x,-y+1/2,z+1/2"},{"x,y+1/2,-z+1/2"},{"x+1/2,y+1/2,z"},{"x+1/2,-y+1/2,-z"},{"x+1/2,-y,z+1/2"},{"x+1/2,y,-z+1/2"}},

// [41] C c 2 a (C -2ac -2ac)
{{"x,y,z"},{"-x,y,-z"},{"-x+1/2,y,z+1/2"},{"x+1/2,y,-z+1/2"},{"x+1/2,y+1/2,z"},{"-x+1/2,y+1/2,-z"},{"-x,y+1/2,z+1/2"},{"x,y+1/2,-z+1/2"}},

// [41] A c 2 a (A -2ab -2ab)
{{"x,y,z"},{"-x,y,-z"},{"x+1/2,y,-z+1/2"},{"-x+1/2,y,z+1/2"},{"x,y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z"},{"-x+1/2,y+1/2,z"}},

// [42] F m m 2 (F 2 -2)
{{"x,y,z"},{"-x,-y,z"},{"x,-y,z"},{"-x,y,z"},{"x,y+1/2,z+1/2"},{"-x,-y+1/2,z+1/2"},{"x,-y+1/2,z+1/2"},
{"-x,y+1/2,z+1/2"},{"x+1/2,y,z+1/2"},{"-x+1/2,-y,z+1/2"},{"x+1/2,-y,z+1/2"},{"-x+1/2,y,z+1/2"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z"},
{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"}},

// [42] F 2 m m (F -2 2)
{{"x,y,z"},{"x,-y,-z"},{"x,y,-z"},{"x,-y,z"},{"x,y+1/2,z+1/2"},{"x,-y+1/2,-z+1/2"},{"x,y+1/2,-z+1/2"},
{"x,-y+1/2,z+1/2"},{"x+1/2,y,z+1/2"},{"x+1/2,-y,-z+1/2"},{"x+1/2,y,-z+1/2"},{"x+1/2,-y,z+1/2"},{"x+1/2,y+1/2,z"},{"x+1/2,-y+1/2,-z"},
{"x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,z"}},

// [42] F m 2 m (F -2 -2)
{{"x,y,z"},{"-x,y,-z"},{"x,y,-z"},{"-x,y,z"},{"x,y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x,y+1/2,-z+1/2"},
{"-x,y+1/2,z+1/2"},{"x+1/2,y,z+1/2"},{"-x+1/2,y,-z+1/2"},{"x+1/2,y,-z+1/2"},{"-x+1/2,y,z+1/2"},{"x+1/2,y+1/2,z"},{"-x+1/2,y+1/2,-z"},
{"x+1/2,y+1/2,-z"},{"-x+1/2,y+1/2,z"}},

// [43] F d d 2 (F 2 -2d)
{{"x,y,z"},{"-x,-y,z"},{"x+1/4,-y+1/4,z+1/4"},{"-x+1/4,y+1/4,z+1/4"},{"x,y+1/2,z+1/2"},{"-x,-y+1/2,z+1/2"},{"x+1/4,-y+3/4,z+3/4"},
{"-x+1/4,y+3/4,z+3/4"},{"x+1/2,y,z+1/2"},{"-x+1/2,-y,z+1/2"},{"x+3/4,-y+1/4,z+3/4"},{"-x+3/4,y+1/4,z+3/4"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z"},
{"x+3/4,-y+3/4,z+1/4"},{"-x+3/4,y+3/4,z+1/4"}},

// [43] F 2 d d (F -2d 2)
{{"x,y,z"},{"x,-y,-z"},{"x+1/4,y+1/4,-z+1/4"},{"x+1/4,-y+1/4,z+1/4"},{"x,y+1/2,z+1/2"},{"x,-y+1/2,-z+1/2"},{"x+1/4,y+3/4,-z+3/4"},
{"x+1/4,-y+3/4,z+3/4"},{"x+1/2,y,z+1/2"},{"x+1/2,-y,-z+1/2"},{"x+3/4,y+1/4,-z+3/4"},{"x+3/4,-y+1/4,z+3/4"},{"x+1/2,y+1/2,z"},{"x+1/2,-y+1/2,-z"},
{"x+3/4,y+3/4,-z+1/4"},{"x+3/4,-y+3/4,z+1/4"}},

// [43] F d 2 d (F -2d -2d)
{{"x,y,z"},{"-x,y,-z"},{"-x+1/4,y+1/4,z+1/4"},{"x+1/4,y+1/4,-z+1/4"},{"x,y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"-x+1/4,y+3/4,z+3/4"},
{"x+1/4,y+3/4,-z+3/4"},{"x+1/2,y,z+1/2"},{"-x+1/2,y,-z+1/2"},{"-x+3/4,y+1/4,z+3/4"},{"x+3/4,y+1/4,-z+3/4"},{"x+1/2,y+1/2,z"},{"-x+1/2,y+1/2,-z"},
{"-x+3/4,y+3/4,z+1/4"},{"x+3/4,y+3/4,-z+1/4"}},

// [44] I m m 2 (I 2 -2)
{{"x,y,z"},{"-x,-y,z"},{"x,-y,z"},{"-x,y,z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [44] I 2 m m (I -2 2)
{{"x,y,z"},{"x,-y,-z"},{"x,y,-z"},{"x,-y,z"},{"x+1/2,y+1/2,z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"}},

// [44] I m 2 m (I -2 -2)
{{"x,y,z"},{"-x,y,-z"},{"x,y,-z"},{"-x,y,z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [45] I b a 2 (I 2 -2c)
{{"x,y,z"},{"-x,-y,z"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"x,-y,z+1/2"},{"-x,y,z+1/2"}},

// [45] I 2 c b (I -2a 2)
{{"x,y,z"},{"x,-y,-z"},{"x,y+1/2,-z+1/2"},{"x,-y+1/2,z+1/2"},{"x+1/2,y+1/2,z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y,-z"},{"x+1/2,-y,z"}},

// [45] I c 2 a (I -2b -2b)
{{"x,y,z"},{"-x,y,-z"},{"x+1/2,y,-z+1/2"},{"-x+1/2,y,z+1/2"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"x,y+1/2,-z"},{"-x,y+1/2,z"}},

// [46] I m a 2 (I 2 -2a)
{{"x,y,z"},{"-x,-y,z"},{"x+1/2,-y,z"},{"-x+1/2,y,z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"x,-y+1/2,z+1/2"},{"-x,y+1/2,z+1/2"}},

// [46] I b m 2 (I 2 -2b)
{{"x,y,z"},{"-x,-y,z"},{"-x,y+1/2,z"},{"x,-y+1/2,z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y,z+1/2"},{"x+1/2,-y,z+1/2"}},

// [46] I 2 m b (I -2b 2)
{{"x,y,z"},{"x,-y,-z"},{"x,y+1/2,-z"},{"x,-y+1/2,z"},{"x+1/2,y+1/2,z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y,-z+1/2"},{"x+1/2,-y,z+1/2"}},

// [46] I 2 c m (I -2c 2)
{{"x,y,z"},{"x,-y,-z"},{"x,-y,z+1/2"},{"x,y,-z+1/2"},{"x+1/2,y+1/2,z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z"},{"x+1/2,y+1/2,-z"}},

// [46] I c 2 m (I -2c -2c)
{{"x,y,z"},{"-x,y,-z"},{"-x,y,z+1/2"},{"x,y,-z+1/2"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"-x+1/2,y+1/2,z"},{"x+1/2,y+1/2,-z"}},

// [46] I m 2 a (I -2a -2a)
{{"x,y,z"},{"-x,y,-z"},{"x+1/2,y,-z"},{"-x+1/2,y,z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"x,y+1/2,-z+1/2"},{"-x,y+1/2,z+1/2"}},

// [47] P m m m (-P 2 2)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"-x,-y,-z"},{"x,y,-z"},{"x,-y,z"},{"-x,y,z"}},

    // ORTHORHOMBIC GROUPS
    // ===================

// [48] P n n n Origin choise 1 (P 2 2 -1n)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [48] P n n n Origin choise 2 (-P 2ab 2bc)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,y,-z+1/2"},{"x,-y+1/2,-z+1/2"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"},{"x+1/2,-y,z+1/2"},{"-x,y+1/2,z+1/2"}},

// [49] P c c m  (-P 2 2c)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z+1/2"},{"x,-y,-z+1/2"},{"-x,-y,-z"},{"x,y,-z"},{"x,-y,z+1/2"},{"-x,y,z+1/2"}},

// [49] P m a a (-P 2a 2)
{{"x,y,z"},{"x,-y,-z"},{"-x+1/2,-y,z"},{"-x+1/2,y,-z"},{"-x,-y,-z"},{"-x,y,z"},{"x+1/2,y,-z"},{"x+1/2,-y,z"}},

// [49] P b m b (-P 2b 2b)
{{"x,y,z"},{"-x,y,-z"},{"-x,-y+1/2,z"},{"x,-y+1/2,-z"},{"-x,-y,-z"},{"x,-y,z"},{"x,y+1/2,-z"},{"-x,y+1/2,z"}},

// [50] P b a n Origin choise 1 (P 2 2 -1ab)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"-x+1/2,-y+1/2,-z"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"}},

// [50] P b a n Origin choise 2 (-P 2ab 2b)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,y,-z"},{"x,-y+1/2,-z"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"},{"x+1/2,-y,z"},{"-x,y+1/2,z"}},

// [50] P n c b Origin choise 1 (P 2 2 -1bc)
{{"x,y,z"},{"x,-y,-z"},{"-x,-y,z"},{"-x,y,-z"},{"-x,-y+1/2,-z+1/2"},{"-x,y+1/2,z+1/2"},{"x,y+1/2,-z+1/2"},{"x,-y+1/2,z+1/2"}},

// [50] P n c b Origin choise 2 (-P 2b 2bc)
{{"x,y,z"},{"x,-y+1/2,-z+1/2"},{"-x,-y+1/2,z"},{"-x,y,-z+1/2"},{"-x,-y,-z"},{"-x,y+1/2,z+1/2"},{"x,y+1/2,-z"},{"x,-y,z+1/2"}},

// [50] P c n a Origin choise 1 (P 2 2 -1ac)
{{"x,y,z"},{"-x,y,-z"},{"-x,-y,z"},{"x,-y,-z"},{"-x+1/2,-y,-z+1/2"},{"x+1/2,-y,z+1/2"},{"x+1/2,y,-z+1/2"},{"-x+1/2,y,z+1/2"}},

// [50] P c n a Origin choise 2 (-P 2a 2c)
{{"x,y,z"},{"-x+1/2,y,-z+1/2"},{"-x+1/2,-y,z"},{"x,-y,-z+1/2"},{"-x,-y,-z"},{"x+1/2,-y,z+1/2"},{"x+1/2,y,-z"},{"-x,y,z+1/2"}},

// [51] P m m a (-P 2a 2a)
{{"x,y,z"},{"-x+1/2,-y,z"},{"-x,y,-z"},{"x+1/2,-y,-z"},{"-x,-y,-z"},{"x+1/2,y,-z"},{"x,-y,z"},{"-x+1/2,y,z"}},

// [51] P m m b (-P 2b 2)
{{"x,y,z"},{"-x,-y+1/2,z"},{"x,-y,-z"},{"-x,y+1/2,-z"},{"-x,-y,-z"},{"x,y+1/2,-z"},{"-x,y,z"},{"x,-y+1/2,z"}},

// [51] P b m m (-P 2 2b)
{{"x,y,z"},{"x,-y+1/2,-z"},{"-x,-y,z"},{"-x,y+1/2,-z"},{"-x,-y,-z"},{"-x,y+1/2,z"},{"x,y,-z"},{"x,-y+1/2,z"}},

// [51] P c m m (-P 2c 2c)
{{"x,y,z"},{"x,-y,-z+1/2"},{"-x,y,-z"},{"-x,-y,z+1/2"},{"-x,-y,-z"},{"-x,y,z+1/2"},{"x,-y,z"},{"x,y,-z+1/2"}},

// [51] P m c m (-P 2c 2)
{{"x,y,z"},{"-x,y,-z+1/2"},{"x,-y,-z"},{"-x,-y,z+1/2"},{"-x,-y,-z"},{"x,-y,z+1/2"},{"-x,y,z"},{"x,y,-z+1/2"}},

// [51] P m a m (-P 2 2a)
{{"x,y,z"},{"-x+1/2,y,-z"},{"-x,-y,z"},{"x+1/2,-y,-z"},{"-x,-y,-z"},{"x+1/2,-y,z"},{"x,y,-z"},{"-x+1/2,y,z"}},

// [52] P n n a (-P 2a 2bc)
{{"x,y,z"},{"-x+1/2,-y,z"},{"-x+1/2,y+1/2,-z+1/2"},{"x,-y+1/2,-z+1/2"},{"-x,-y,-z"},{"x+1/2,y,-z"},{"x+1/2,-y+1/2,z+1/2"},{"-x,y+1/2,z+1/2"}},

// [52] P n n b (-P 2b 2n)
{{"x,y,z"},{"-x,-y+1/2,z"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,y,-z+1/2"},{"-x,-y,-z"},{"x,y+1/2,-z"},{"-x+1/2,y+1/2,z+1/2"},{"x+1/2,-y,z+1/2"}},

// [52] P b n n (-P 2n 2b)
{{"x,y,z"},{"x,-y+1/2,-z"},{"-x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y,-z+1/2"},{"-x,-y,-z"},{"-x,y+1/2,z"},{"x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y,z+1/2"}},

// [52] P c n n (-P 2ab 2c)
{{"x,y,z"},{"x,-y,-z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,z"},{"-x,-y,-z"},{"-x,y,z+1/2"},{"x+1/2,-y+1/2,z+1/2"},{"x+1/2,y+1/2,-z"}},

// [52] P n c n (-P 2ab 2n)
{{"x,y,z"},{"-x,y,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,z"},{"-x,-y,-z"},{"x,-y,z+1/2"},{"-x+1/2,y+1/2,z+1/2"},{"x+1/2,y+1/2,-z"}},

// [52] P n a n (-P 2n 2bc)
{{"x,y,z"},{"-x+1/2,y,-z"},{"-x+1/2,-y+1/2,z+1/2"},{"x,-y+1/2,-z+1/2"},{"-x,-y,-z"},{"x+1/2,-y,z"},{"x+1/2,y+1/2,-z+1/2"},{"-x,y+1/2,z+1/2"}},

// [53] P m n a (-P 2ac 2)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"-x+1/2,y,-z+1/2"},{"x,-y,-z"},{"-x,-y,-z"},{"x+1/2,y,-z+1/2"},{"x+1/2,-y,z+1/2"},{"-x,y,z"}},

// [53] Pnmb (-P 2bc 2bc)
{{"x,y,z"},{"-x,-y+1/2,z+1/2"},{"x,-y+1/2,-z+1/2"},{"-x,y,-z"},{"-x,-y,-z"},{"x,y+1/2,-z+1/2"},{"-x,y+1/2,z+1/2"},{"x,-y,z"}},

// [53] P b m n (-P 2ab 2ab)
{{"x,y,z"},{"x+1/2,-y+1/2,-z"},{"-x+1/2,-y+1/2,z"},{"-x,y,-z"},{"-x,-y,-z"},{"-x+1/2,y+1/2,z"},{"x+1/2,y+1/2,-z"},{"x,-y,z"}},

// [53] P c n m (-P 2 2ac)
{{"x,y,z"},{"x+1/2,-y,-z+1/2"},{"-x+1/2,y,-z+1/2"},{"-x,-y,z"},{"-x,-y,-z"},{"-x+1/2,y,z+1/2"},{"x+1/2,-y,z+1/2"},{"x,y,-z"}},

// [53] P n c m (-P 2 2bc)
{{"x,y,z"},{"-x,y+1/2,-z+1/2"},{"x,-y+1/2,-z+1/2"},{"-x,-y,z"},{"-x,-y,-z"},{"x,-y+1/2,z+1/2"},{"-x,y+1/2,z+1/2"},{"x,y,-z"}},

// [53] P m a n (-P 2ab 2)
{{"x,y,z"},{"-x+1/2,y+1/2,-z"},{"-x+1/2,-y+1/2,z"},{"x,-y,-z"},{"-x,-y,-z"},{"x+1/2,-y+1/2,z"},{"x+1/2,y+1/2,-z"},{"-x,y,z"}},

// [54] P c c a (-P 2a 2ac)
{{"x,y,z"},{"-x+1/2,-y,z"},{"-x,y,-z+1/2"},{"x+1/2,-y,-z+1/2"},{"-x,-y,-z"},{"x+1/2,y,-z"},{"x,-y,z+1/2"},{"-x+1/2,y,z+1/2"}},

// [54] P c c b (-P 2b 2c)
{{"x,y,z"},{"-x,-y+1/2,z"},{"x,-y,-z+1/2"},{"-x,y+1/2,-z+1/2"},{"-x,-y,-z"},{"x,y+1/2,-z"},{"-x,y,z+1/2"},{"x,-y+1/2,z+1/2"}},

// [54] P b a a (-P 2a 2b)
{{"x,y,z"},{"x,-y+1/2,-z"},{"-x+1/2,-y,z"},{"-x+1/2,y+1/2,-z"},{"-x,-y,-z"},{"-x,y+1/2,z"},{"x+1/2,y,-z"},{"x+1/2,-y+1/2,z"}},

// [54] P c a a (-P 2ac 2c)
{{"x,y,z"},{"x,-y,-z+1/2"},{"-x+1/2,y,-z"},{"-x+1/2,-y,z+1/2"},{"-x,-y,-z"},{"-x,y,z+1/2"},{"x+1/2,-y,z"},{"x+1/2,y,-z+1/2"}},

// [54] P b c b (-P 2bc 2b)
{{"x,y,z"},{"-x,y,-z+1/2"},{"x,-y+1/2,-z"},{"-x,-y+1/2,z+1/2"},{"-x,-y,-z"},{"x,-y,z+1/2"},{"-x,y+1/2,z"},{"x,y+1/2,-z+1/2"}},

// [54] P b a b (-P 2b 2ab)
{{"x,y,z"},{"-x+1/2,y,-z"},{"-x,-y+1/2,z"},{"x+1/2,-y+1/2,-z"},{"-x,-y,-z"},{"x+1/2,-y,z"},{"x,y+1/2,-z"},{"-x+1/2,y+1/2,z"}},

// [55] P b a m (-P 2 2ab)
{{"x,y,z"},{"-x,-y,z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},{"-x,-y,-z"},{"x,y,-z"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"}},

// [55] P m c b (-P 2bc 2)
{{"x,y,z"},{"x,-y,-z"},{"-x,-y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"-x,-y,-z"},{"-x,y,z"},{"x,y+1/2,-z+1/2"},{"x,-y+1/2,z+1/2"}},

// [55] P c m a (-P 2ac 2ac)
{{"x,y,z"},{"-x,y,-z"},{"-x+1/2,-y,z+1/2"},{"x+1/2,-y,-z+1/2"},{"-x,-y,-z"},{"x,-y,z"},{"x+1/2,y,-z+1/2"},{"-x+1/2,y,z+1/2"}},

// [56] P c c n (-P 2ab 2ac)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-x,y+1/2,-z+1/2"},{"x+1/2,-y,-z+1/2"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"},{"x,-y+1/2,z+1/2"},{"-x+1/2,y,z+1/2"}},

// [56] P n a a (-P 2ac 2bc)
{{"x,y,z"},{"x,-y+1/2,-z+1/2"},{"-x+1/2,-y,z+1/2"},{"-x+1/2,y+1/2,-z"},{"-x,-y,-z"},{"-x,y+1/2,z+1/2"},{"x+1/2,y,-z+1/2"},{"x+1/2,-y+1/2,z"}},

// [56] P b n b (-P 2bc 2ab)
{{"x,y,z"},{"-x+1/2,y,-z+1/2"},{"-x,-y+1/2,z+1/2"},{"x+1/2,-y+1/2,-z"},{"-x,-y,-z"},{"x+1/2,-y,z+1/2"},{"x,y+1/2,-z+1/2"},{"-x+1/2,y+1/2,z"}},

// [57] P b c m (-P 2c 2b)
{{"x,y,z"},{"-x,-y,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x,-y+1/2,-z"},{"-x,-y,-z"},{"x,y,-z+1/2"},{"x,-y+1/2,z+1/2"},{"-x,y+1/2,z"}},

// [57] P c a m (-P 2c 2ac)
{{"x,y,z"},{"-x,-y,z+1/2"},{"x+1/2,-y,-z+1/2"},{"-x+1/2,y,-z"},{"-x,-y,-z"},{"x,y,-z+1/2"},{"-x+1/2,y,z+1/2"},{"x+1/2,-y,z"}},

// [57] P m c a (-P 2ac 2a)
{{"x,y,z"},{"x+1/2,-y,-z"},{"-x+1/2,-y,z+1/2"},{"-x,y,-z+1/2"},{"-x,-y,-z"},{"-x+1/2,y,z"},{"x+1/2,y,-z+1/2"},{"x,-y,z+1/2"}},

// [57] P m a b (-P 2b 2a)
{{"x,y,z"},{"x+1/2,-y,-z"},{"-x+1/2,y+1/2,-z"},{"-x,-y+1/2,z"},{"-x,-y,-z"},{"-x+1/2,y,z"},{"x+1/2,-y+1/2,z"},{"x,y+1/2,-z"}},

// [57] P b m a (-P 2a 2ab)
{{"x,y,z"},{"-x,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},{"-x+1/2,-y,z"},{"-x,-y,-z"},{"x,-y+1/2,z"},{"-x+1/2,y+1/2,z"},{"x+1/2,y,-z"}},

// [57] P c m b (-P 2bc 2c)
{{"x,y,z"},{"-x,y+1/2,-z"},{"-x,-y+1/2,z+1/2"},{"x,-y,-z+1/2"},{"-x,-y,-z"},{"x,-y+1/2,z"},{"x,y+1/2,-z+1/2"},{"-x,y,z+1/2"}},

// [58] P n n m (-P 2 2n)
{{"x,y,z"},{"-x,-y,z"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"-x,-y,-z"},{"x,y,-z"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [58] P m n n (-P 2n 2)
{{"x,y,z"},{"x,-y,-z"},{"-x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"-x,-y,-z"},{"-x,y,z"},{"x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"}},

// [58] P n m n (-P 2n 2n)
{{"x,y,z"},{"-x,y,-z"},{"-x+1/2,-y+1/2,z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"-x,-y,-z"},{"x,-y,z"},{"x+1/2,y+1/2,-z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [59] P m m n Origin choise 1 (P 2 2ab -1ab)
{{"x,y,z"},{"-x,-y,z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},{"-x+1/2,-y+1/2,-z"},{"x+1/2,y+1/2,-z"},{"x,-y,z"},{"-x,y,z"}},

// [59] P m m n Origin choise 2 (-P 2ab 2a)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-x,y+1/2,-z"},{"x+1/2,-y,-z"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"},{"x,-y+1/2,z"},{"-x+1/2,y,z"}},

// [59] P n m m Origin choise 1 (P 2bc 2 -1bc)
{{"x,y,z"},{"x,-y,-z"},{"-x,-y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"-x,-y+1/2,-z+1/2"},{"-x,y+1/2,z+1/2"},{"x,y,-z"},{"x,-y,z"}},

// [59] P n m m Origin choise 2 (-P 2c 2bc)
{{"x,y,z"},{"x,-y+1/2,-z+1/2"},{"-x,-y,z+1/2"},{"-x,y+1/2,-z"},{"-x,-y,-z"},{"-x,y+1/2,z+1/2"},{"x,y,-z+1/2"},{"x,-y+1/2,z"}},

// [59] P m n m Origin choise 1 (P 2ac 2ac -1ac)
{{"x,y,z"},{"-x,y,-z"},{"-x+1/2,-y,z+1/2"},{"x+1/2,-y,-z+1/2"},{"-x+1/2,-y,-z+1/2"},{"x+1/2,-y,z+1/2"},{"x,y,-z"},{"-x,y,z"}},

// [59] P m n m Origin choise 2 (-P 2c 2a)
{{"x,y,z"},{"-x+1/2,y,-z+1/2"},{"-x,-y,z+1/2"},{"x+1/2,-y,-z"},{"-x,-y,-z"},{"x+1/2,-y,z+1/2"},{"x,y,-z+1/2"},{"-x+1/2,y,z"}},

// [60] P b c n (-P 2n 2ab)
{{"x,y,z"},{"-x+1/2,-y+1/2,z+1/2"},{"-x,y,-z+1/2"},{"x+1/2,-y+1/2,-z"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z+1/2"},{"x,-y,z+1/2"},{"-x+1/2,y+1/2,z"}},

// [60] P c a n (-P 2n 2c)
{{"x,y,z"},{"-x+1/2,-y+1/2,z+1/2"},{"x,-y,-z+1/2"},{"-x+1/2,y+1/2,-z"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z+1/2"},{"-x,y,z+1/2"},{"x+1/2,-y+1/2,z"}},

// [60] P n c a (-P 2a 2n)
{{"x,y,z"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,-y,z"},{"-x,y+1/2,-z+1/2"},{"-x,-y,-z"},{"-x+1/2,y+1/2,z+1/2"},{"x+1/2,y,-z"},{"x,-y+1/2,z+1/2"}},

// [60] P n a b (-P 2bc 2n)
{{"x,y,z"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,y,-z"},{"-x,-y+1/2,z+1/2"},{"-x,-y,-z"},{"-x+1/2,y+1/2,z+1/2"},{"x+1/2,-y,z"},{"x,y+1/2,-z+1/2"}},

// [60] P b n a (-P 2ac 2b)
{{"x,y,z"},{"-x+1/2,y+1/2,-z+1/2"},{"x,-y+1/2,-z"},{"-x+1/2,-y,z+1/2"},{"-x,-y,-z"},{"x+1/2,-y+1/2,z+1/2"},{"-x,y+1/2,z"},{"x+1/2,y,-z+1/2"}},

// [60] P c n b (-P 2b 2ac)
{{"x,y,z"},{"-x+1/2,y+1/2,-z+1/2"},{"-x,-y+1/2,z"},{"x+1/2,-y,-z+1/2"},{"-x,-y,-z"},{"x+1/2,-y+1/2,z+1/2"},{"x,y+1/2,-z"},{"-x+1/2,y,z+1/2"}},

// [61] P b c a (-P 2ac 2ab)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z"},{"-x,-y,-z"},{"x+1/2,y,-z+1/2"},{"x,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z"}},

// [61] P c a b (-P 2bc 2ac)
{{"x,y,z"},{"-x,-y+1/2,z+1/2"},{"x+1/2,-y,-z+1/2"},{"-x+1/2,y+1/2,-z"},{"-x,-y,-z"},{"x,y+1/2,-z+1/2"},{"-x+1/2,y,z+1/2"},{"x+1/2,-y+1/2,z"}},

// [62] P n m a (-P 2ac 2n)      zeolites: MFI
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"-x,y+1/2,-z"},{"x+1/2,-y+1/2,-z+1/2"},{"-x,-y,-z"},{"x+1/2,y,-z+1/2"},{"x,-y+1/2,z"},{"-x+1/2,y+1/2,z+1/2"}},

// [62] P m n b (-P 2bc 2a)
{{"x,y,z"},{"-x,-y+1/2,z+1/2"},{"x+1/2,-y,-z"},{"-x+1/2,y+1/2,-z+1/2"},{"-x,-y,-z"},{"x,y+1/2,-z+1/2"},{"-x+1/2,y,z"},{"x+1/2,-y+1/2,z+1/2"}},

// [62] P b n m (-P 2c 2ab)
{{"x,y,z"},{"x+1/2,-y+1/2,-z"},{"-x,-y,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"-x,-y,-z"},{"-x+1/2,y+1/2,z"},{"x,y,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"}},

// [62] P c m n (-P 2n 2ac)
{{"x,y,z"},{"x+1/2,-y,-z+1/2"},{"-x,y+1/2,-z"},{"-x+1/2,-y+1/2,z+1/2"},{"-x,-y,-z"},{"-x+1/2,y,z+1/2"},{"x,-y+1/2,z"},{"x+1/2,y+1/2,-z+1/2"}},

// [62] P m c n (-P 2n 2a)
{{"x,y,z"},{"-x,y+1/2,-z+1/2"},{"x+1/2,-y,-z"},{"-x+1/2,-y+1/2,z+1/2"},{"-x,-y,-z"},{"x,-y+1/2,z+1/2"},{"-x+1/2,y,z"},{"x+1/2,y+1/2,-z+1/2"}},

// [62] P n a m (-P 2c 2n)
{{"x,y,z"},{"-x+1/2,y+1/2,-z"},{"-x,-y,z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"-x,-y,-z"},{"x+1/2,-y+1/2,z"},{"x,y,-z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [63] C m c m (-C 2c 2)
{{"x,y,z"},{"-x,-y,z+1/2"},{"-x,y,-z+1/2"},{"x,-y,-z"},{"-x,-y,-z"},{"x,y,-z+1/2"},{"x,-y,z+1/2"},
{"-x,y,z"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z"},{"-x+1/2,-y+1/2,-z"},{"x+1/2,y+1/2,-z+1/2"},
{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z"}},

// [63] C c m m (-C 2c 2c)
{{"x,y,z"},{"-x,-y,z+1/2"},{"x,-y,-z+1/2"},{"-x,y,-z"},{"-x,-y,-z"},{"x,y,-z+1/2"},{"-x,y,z+1/2"},
{"x,-y,z"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,y+1/2,-z"},{"-x+1/2,-y+1/2,-z"},{"x+1/2,y+1/2,-z+1/2"},
{"-x+1/2,y+1/2,z+1/2"},{"x+1/2,-y+1/2,z"}},

// [63] A m m a (-A 2a 2a)
{{"x,y,z"},{"x+1/2,-y,-z"},{"-x+1/2,-y,z"},{"-x,y,-z"},{"-x,-y,-z"},{"-x+1/2,y,z"},{"x+1/2,y,-z"},
{"x,-y,z"},{"x,y+1/2,z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"-x,-y+1/2,-z+1/2"},{"-x+1/2,y+1/2,z+1/2"},
{"x+1/2,y+1/2,-z+1/2"},{"x,-y+1/2,z+1/2"}},

// [63] A m a m (-A 2 2a)
{{"x,y,z"},{"x+1/2,-y,-z"},{"-x+1/2,y,-z"},{"-x,-y,z"},{"-x,-y,-z"},{"-x+1/2,y,z"},{"x+1/2,-y,z"},
{"x,y,-z"},{"x,y+1/2,z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"-x,-y+1/2,z+1/2"},{"-x,-y+1/2,-z+1/2"},{"-x+1/2,y+1/2,z+1/2"},
{"x+1/2,-y+1/2,z+1/2"},{"x,y+1/2,-z+1/2"}},

// [63] B b m m (-B 2 2b)
{{"x,y,z"},{"-x,y+1/2,-z"},{"x,-y+1/2,-z"},{"-x,-y,z"},{"-x,-y,-z"},{"x,-y+1/2,z"},{"-x,y+1/2,z"},
{"x,y,-z"},{"x+1/2,y,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,-y,z+1/2"},{"-x+1/2,-y,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"},
{"-x+1/2,y+1/2,z+1/2"},{"x+1/2,y,-z+1/2"}},

// [63] B m m b (-B 2b 2)
{{"x,y,z"},{"-x,y+1/2,-z"},{"-x,-y+1/2,z"},{"x,-y,-z"},{"-x,-y,-z"},{"x,-y+1/2,z"},{"x,y+1/2,-z"},
{"-x,y,z"},{"x+1/2,y,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"x+1/2,-y,-z+1/2"},{"-x+1/2,-y,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"},
{"x+1/2,y+1/2,-z+1/2"},{"-x+1/2,y,z+1/2"}},

// [64] C m c a (-C 2ac 2)
{{"x,y,z"},{"-x,-y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x,-y,-z"},{"-x,-y,-z"},{"x,y+1/2,-z+1/2"},{"x,-y+1/2,z+1/2"},
{"-x,y,z"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y,z+1/2"},{"-x+1/2,y,-z+1/2"},{"x+1/2,-y+1/2,-z"},{"-x+1/2,-y+1/2,-z"},{"x+1/2,y,-z+1/2"},
{"x+1/2,-y,z+1/2"},{"-x+1/2,y+1/2,z"}},

// [64] C c m b (-C 2ac 2ac)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"x+1/2,-y,-z+1/2"},{"-x,y,-z"},{"-x,-y,-z"},{"x+1/2,y,-z+1/2"},{"-x+1/2,y,z+1/2"},
{"x,-y,z"},{"x+1/2,y+1/2,z"},{"-x,-y+1/2,z+1/2"},{"x,-y+1/2,-z+1/2"},{"-x+1/2,y+1/2,-z"},{"-x+1/2,-y+1/2,-z"},{"x,y+1/2,-z+1/2"},
{"-x,y+1/2,z+1/2"},{"x+1/2,-y+1/2,z"}},

// [64] A b m a (-A 2ab 2ab)
{{"x,y,z"},{"x+1/2,-y,-z+1/2"},{"-x+1/2,-y,z+1/2"},{"-x,y,-z"},{"-x,-y,-z"},{"-x+1/2,y,z+1/2"},{"x+1/2,y,-z+1/2"},
{"x,-y,z"},{"x,y+1/2,z+1/2"},{"x+1/2,-y+1/2,-z"},{"-x+1/2,-y+1/2,z"},{"-x,y+1/2,-z+1/2"},{"-x,-y+1/2,-z+1/2"},{"-x+1/2,y+1/2,z"},
{"x+1/2,y+1/2,-z"},{"x,-y+1/2,z+1/2"}},

// [64] A c a m (-A 2 2ab)
{{"x,y,z"},{"x+1/2,-y+1/2,-z"},{"-x+1/2,y+1/2,-z"},{"-x,-y,z"},{"-x,-y,-z"},{"-x+1/2,y+1/2,z"},{"x+1/2,-y+1/2,z"},
{"x,y,-z"},{"x,y+1/2,z+1/2"},{"x+1/2,-y,-z+1/2"},{"-x+1/2,y,-z+1/2"},{"-x,-y+1/2,z+1/2"},{"-x,-y+1/2,-z+1/2"},{"-x+1/2,y,z+1/2"},
{"x+1/2,-y,z+1/2"},{"x,y+1/2,-z+1/2"}},

// [64] B b c m (-B 2 2ab)
{{"x,y,z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},{"-x,-y,z"},{"-x,-y,-z"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"},
{"x,y,-z"},{"x+1/2,y,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x,-y+1/2,-z+1/2"},{"-x+1/2,-y,z+1/2"},{"-x+1/2,-y,-z+1/2"},{"x,-y+1/2,z+1/2"},
{"-x,y+1/2,z+1/2"},{"x+1/2,y,-z+1/2"}},

// [64] B m a b (-B 2ab 2)
{{"x,y,z"},{"-x,y+1/2,-z+1/2"},{"-x,-y+1/2,z+1/2"},{"x,-y,-z"},{"-x,-y,-z"},{"x,-y+1/2,z+1/2"},{"x,y+1/2,-z+1/2"},
{"-x,y,z"},{"x+1/2,y,z+1/2"},{"-x+1/2,y+1/2,-z"},{"-x+1/2,-y+1/2,z"},{"x+1/2,-y,-z+1/2"},{"-x+1/2,-y,-z+1/2"},{"x+1/2,-y+1/2,z"},
{"x+1/2,y+1/2,-z"},{"-x+1/2,y,z+1/2"}},

// [65] C m m m (-C 2 2)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"-x,-y,-z"},{"x,y,-z"},{"x,-y,z"},
{"-x,y,z"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},{"-x+1/2,-y+1/2,-z"},{"x+1/2,y+1/2,-z"},
{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"}},

// [65] A m m m (-A 2 2)
{{"x,y,z"},{"x,-y,-z"},{"-x,-y,z"},{"-x,y,-z"},{"-x,-y,-z"},{"-x,y,z"},{"x,y,-z"},
{"x,-y,z"},{"x,y+1/2,z+1/2"},{"x,-y+1/2,-z+1/2"},{"-x,-y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"-x,-y+1/2,-z+1/2"},{"-x,y+1/2,z+1/2"},
{"x,y+1/2,-z+1/2"},{"x,-y+1/2,z+1/2"}},

// [65] B m m m (-B 2 2)
{{"x,y,z"},{"-x,y,-z"},{"-x,-y,z"},{"x,-y,-z"},{"-x,-y,-z"},{"x,-y,z"},{"x,y,-z"},
{"-x,y,z"},{"x+1/2,y,z+1/2"},{"-x+1/2,y,-z+1/2"},{"-x+1/2,-y,z+1/2"},{"x+1/2,-y,-z+1/2"},{"-x+1/2,-y,-z+1/2"},{"x+1/2,-y,z+1/2"},
{"x+1/2,y,-z+1/2"},{"-x+1/2,y,z+1/2"}},

// [66] C c c m (-C 2 2c)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z+1/2"},{"x,-y,-z+1/2"},{"-x,-y,-z"},{"x,y,-z"},{"x,-y,z+1/2"},
{"-x,y,z+1/2"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,-z"},{"x+1/2,y+1/2,-z"},
{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [66] A m a a (-A 2a 2)
{{"x,y,z"},{"x,-y,-z"},{"-x+1/2,-y,z"},{"-x+1/2,y,-z"},{"-x,-y,-z"},{"-x,y,z"},{"x+1/2,y,-z"},
{"x+1/2,-y,z"},{"x,y+1/2,z+1/2"},{"x,-y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"-x,-y+1/2,-z+1/2"},{"-x,y+1/2,z+1/2"},
{"x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"}},

// [66] B b m b (-B 2b 2b)
{{"x,y,z"},{"-x,y,-z"},{"-x,-y+1/2,z"},{"x,-y+1/2,-z"},{"-x,-y,-z"},{"x,-y,z"},{"x,y+1/2,-z"},
{"-x,y+1/2,z"},{"x+1/2,y,z+1/2"},{"-x+1/2,y,-z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,-y,-z+1/2"},{"x+1/2,-y,z+1/2"},
{"x+1/2,y+1/2,-z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [67] C m m a (-C 2a 2)
{{"x,y,z"},{"-x,-y+1/2,z"},{"-x,y+1/2,-z"},{"x,-y,-z"},{"-x,-y,-z"},{"x,y+1/2,-z"},{"x,-y+1/2,z"},
{"-x,y,z"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y,z"},{"-x+1/2,y,-z"},{"x+1/2,-y+1/2,-z"},{"-x+1/2,-y+1/2,-z"},{"x+1/2,y,-z"},
{"x+1/2,-y,z"},{"-x+1/2,y+1/2,z"}},

// [67] C m m b (-C 2a 2a)
{{"x,y,z"},{"-x+1/2,-y,z"},{"x+1/2,-y,-z"},{"-x,y,-z"},{"-x,-y,-z"},{"x+1/2,y,-z"},{"-x+1/2,y,z"},
{"x,-y,z"},{"x+1/2,y+1/2,z"},{"-x,-y+1/2,z"},{"x,-y+1/2,-z"},{"-x+1/2,y+1/2,-z"},{"-x+1/2,-y+1/2,-z"},{"x,y+1/2,-z"},
{"-x,y+1/2,z"},{"x+1/2,-y+1/2,z"}},

// [67] A b m m (-A 2b 2b)
{{"x,y,z"},{"x,-y,-z+1/2"},{"-x,-y,z+1/2"},{"-x,y,-z"},{"-x,-y,-z"},{"-x,y,z+1/2"},{"x,y,-z+1/2"},
{"x,-y,z"},{"x,y+1/2,z+1/2"},{"x,-y+1/2,-z"},{"-x,-y+1/2,z"},{"-x,y+1/2,-z+1/2"},{"-x,-y+1/2,-z+1/2"},{"-x,y+1/2,z"},
{"x,y+1/2,-z"},{"x,-y+1/2,z+1/2"}},

// [67] A c m m (-A 2 2b)
{{"x,y,z"},{"x,-y+1/2,-z"},{"-x,y+1/2,-z"},{"-x,-y,z"},{"-x,-y,-z"},{"-x,y+1/2,z"},{"x,-y+1/2,z"},
{"x,y,-z"},{"x,y+1/2,z+1/2"},{"x,-y,-z+1/2"},{"-x,y,-z+1/2"},{"-x,-y+1/2,z+1/2"},{"-x,-y+1/2,-z+1/2"},{"-x,y,z+1/2"},
{"x,-y,z+1/2"},{"x,y+1/2,-z+1/2"}},

// [67] B m c m (-B 2 2a)
{{"x,y,z"},{"-x+1/2,y,-z"},{"x+1/2,-y,-z"},{"-x,-y,z"},{"-x,-y,-z"},{"x+1/2,-y,z"},{"-x+1/2,y,z"},
{"x,y,-z"},{"x+1/2,y,z+1/2"},{"-x,y,-z+1/2"},{"x,-y,-z+1/2"},{"-x+1/2,-y,z+1/2"},{"-x+1/2,-y,-z+1/2"},{"x,-y,z+1/2"},
{"-x,y,z+1/2"},{"x+1/2,y,-z+1/2"}},

// [67] B m a m (-B 2a 2)
{{"x,y,z"},{"-x,y,-z+1/2"},{"-x,-y,z+1/2"},{"x,-y,-z"},{"-x,-y,-z"},{"x,-y,z+1/2"},{"x,y,-z+1/2"},
{"-x,y,z"},{"x+1/2,y,z+1/2"},{"-x+1/2,y,-z"},{"-x+1/2,-y,z"},{"x+1/2,-y,-z+1/2"},{"-x+1/2,-y,-z+1/2"},{"x+1/2,-y,z"},
{"x+1/2,y,-z"},{"-x+1/2,y,z+1/2"}},

// [68] C c c a Origin choise 1 (C 2 2 -1ac)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-x,y,-z"},{"x+1/2,-y+1/2,-z"},{"-x,-y+1/2,-z+1/2"},{"x+1/2,y,-z+1/2"},{"x,-y+1/2,z+1/2"},{"-x+1/2,y,z+1/2"},
{"x+1/2,y+1/2,z"},{"-x,-y,z"},{"-x+1/2,y+1/2,-z"},{"x,-y,-z"},{"-x+1/2,-y,-z+1/2"},{"x,y+1/2,-z+1/2"},{"x+1/2,-y,z+1/2"},{"-x,y+1/2,z+1/2"}},

// [68] C c c a Origin choise 2 (-C 2a 2ac)
{{"x,y,z"},{"-x+1/2,-y,z"},{"-x,y,-z+1/2"},{"x+1/2,-y,-z+1/2"},{"-x,-y,-z"},{"x+1/2,y,-z"},{"x,-y,z+1/2"},{"-x+1/2,y,z+1/2"},
{"x,y,z"},{"-x+1/2,-y,z"},{"-x,y,-z+1/2"},{"x+1/2,-y,-z+1/2"},{"-x,-y,-z"},{"x+1/2,y,-z"},{"x,-y,z+1/2"},{"-x+1/2,y,z+1/2"}},

// [68] C c c b Origin choise 1 (C 2 2 -1ac)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"x,-y,-z"},{"-x+1/2,y+1/2,-z"},{"-x+1/2,-y,-z+1/2"},{"x,y+1/2,-z+1/2"},{"-x+1/2,y,z+1/2"},
{"x,-y+1/2,z+1/2"},{"x+1/2,y+1/2,z"},{"-x,-y,z"},{"x+1/2,-y+1/2,-z"},{"-x,y,-z"},{"-x,-y+1/2,-z+1/2"},{"x+1/2,y,-z+1/2"},
{"-x,y+1/2,z+1/2"},{"x+1/2,-y,z+1/2"}},

// [68] C c c b Origin choise 2 (-C 2a 2c)
{{"x,y,z"},{"-x,-y+1/2,z"},{"x,-y,-z+1/2"},{"-x,y+1/2,-z+1/2"},{"-x,-y,-z"},{"x,y+1/2,-z"},{"-x,y,z+1/2"},
{"x,-y+1/2,z+1/2"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y,z"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,y,-z+1/2"},{"-x+1/2,-y+1/2,-z"},{"x+1/2,y,-z"},
{"-x+1/2,y+1/2,z+1/2"},{"x+1/2,-y,z+1/2"}},

// [68] A b a a Origin choise 1 (A 2 2 -1ab)
{{"x,y,z"},{"x,-y+1/2,-z+1/2"},{"-x,-y,z"},{"-x,y+1/2,-z+1/2"},{"-x+1/2,-y,-z+1/2"},{"-x+1/2,y+1/2,z"},{"x+1/2,y,-z+1/2"},
{"x+1/2,-y+1/2,z"},{"x,y+1/2,z+1/2"},{"x,-y,-z"},{"-x,-y+1/2,z+1/2"},{"-x,y,-z"},{"-x+1/2,-y+1/2,-z"},{"-x+1/2,y,z+1/2"},
{"x+1/2,y+1/2,-z"},{"x+1/2,-y,z+1/2"}},

// [68] A b a a Origin choise 2 (-A 2a 2b)
{{"x,y,z"},{"x,-y+1/2,-z"},{"-x+1/2,-y,z"},{"-x+1/2,y+1/2,-z"},{"-x,-y,-z"},{"-x,y+1/2,z"},{"x+1/2,y,-z"},
{"x+1/2,-y+1/2,z"},{"x,y+1/2,z+1/2"},{"x,-y,-z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y,-z+1/2"},{"-x,-y+1/2,-z+1/2"},{"-x,y,z+1/2"},
{"x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y,z+1/2"}},

// [68] A c a a Origin choise 1 (A 2 2 -1ab)
{{"x,y,z"},{"x,-y+1/2,-z+1/2"},{"-x,y,-z"},{"-x,-y+1/2,z+1/2"},{"-x+1/2,-y+1/2,-z"},{"-x+1/2,y,z+1/2"},{"x+1/2,-y+1/2,z"},
{"x+1/2,y,-z+1/2"},{"x,y+1/2,z+1/2"},{"x,-y,-z"},{"-x,y+1/2,-z+1/2"},{"-x,-y,z"},{"-x+1/2,-y,-z+1/2"},{"-x+1/2,y+1/2,z"},
{"x+1/2,-y,z+1/2"},{"x+1/2,y+1/2,-z"}},

// [68] A c a a Origin choise 2 (-A 2ab 2b)
{{"x,y,z"},{"x,-y,-z+1/2"},{"-x+1/2,y,-z"},{"-x+1/2,-y,z+1/2"},{"-x,-y,-z"},{"-x,y,z+1/2"},{"x+1/2,-y,z"},
{"x+1/2,y,-z+1/2"},{"x,y+1/2,z+1/2"},{"x,-y+1/2,-z"},{"-x+1/2,y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,z"},{"-x,-y+1/2,-z+1/2"},{"-x,y+1/2,z"},
{"x+1/2,-y+1/2,z+1/2"},{"x+1/2,y+1/2,-z"}},

// [68] B b c b Origin choise 1 (B 2 2 -1ab)
{{"x,y,z"},{"-x+1/2,y,-z+1/2"},{"x,-y,-z"},{"-x+1/2,-y,z+1/2"},{"-x+1/2,-y+1/2,-z"},{"x,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z"},
{"x,y+1/2,-z+1/2"},{"x+1/2,y,z+1/2"},{"-x,y,-z"},{"x+1/2,-y,-z+1/2"},{"-x,-y,z"},{"-x,-y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z"},
{"-x,y+1/2,z+1/2"},{"x+1/2,y+1/2,-z"}},

// [68] B b c b Origin choise 2 (-B 2ab 2b)
{{"x,y,z"},{"-x,y,-z+1/2"},{"x,-y+1/2,-z"},{"-x,-y+1/2,z+1/2"},{"-x,-y,-z"},{"x,-y,z+1/2"},{"-x,y+1/2,z"},
{"x,y+1/2,-z+1/2"},{"x+1/2,y,z+1/2"},{"-x+1/2,y,-z"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,-y,-z+1/2"},{"x+1/2,-y,z"},
{"-x+1/2,y+1/2,z+1/2"},{"x+1/2,y+1/2,-z"}},

// [68] B b a b Origin choise 1 (B 2 2 -1ab)
{{"x,y,z"},{"-x+1/2,y,-z+1/2"},{"-x,-y,z"},{"x+1/2,-y,-z+1/2"},{"-x,-y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z"},{"x,y+1/2,-z+1/2"},
{"-x+1/2,y+1/2,z"},{"x+1/2,y,z+1/2"},{"-x,y,-z"},{"-x+1/2,-y,z+1/2"},{"x,-y,-z"},{"-x+1/2,-y+1/2,-z"},{"x,-y+1/2,z+1/2"},
{"x+1/2,y+1/2,-z"},{"-x,y+1/2,z+1/2"}},

// [68] B b a b Origin choise 2 (-B 2b 2ab)
{{"x,y,z"},{"-x+1/2,y,-z"},{"-x,-y+1/2,z"},{"x+1/2,-y+1/2,-z"},{"-x,-y,-z"},{"x+1/2,-y,z"},{"x,y+1/2,-z"},
{"-x+1/2,y+1/2,z"},{"x+1/2,y,z+1/2"},{"-x,y,-z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"x,-y+1/2,-z+1/2"},{"-x+1/2,-y,-z+1/2"},{"x,-y,z+1/2"},
{"x+1/2,y+1/2,-z+1/2"},{"-x,y+1/2,z+1/2"}},

// [69] F m m m (-F 2 2)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"-x,-y,-z"},{"x,y,-z"},{"x,-y,z"},
{"-x,y,z"},{"x,y+1/2,z+1/2"},{"-x,-y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x,-y+1/2,-z+1/2"},{"-x,-y+1/2,-z+1/2"},{"x,y+1/2,-z+1/2"},
{"x,-y+1/2,z+1/2"},{"-x,y+1/2,z+1/2"},{"x+1/2,y,z+1/2"},{"-x+1/2,-y,z+1/2"},{"-x+1/2,y,-z+1/2"},{"x+1/2,-y,-z+1/2"},{"-x+1/2,-y,-z+1/2"},
{"x+1/2,y,-z+1/2"},{"x+1/2,-y,z+1/2"},{"-x+1/2,y,z+1/2"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},
{"-x+1/2,-y+1/2,-z"},{"x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"}},

// [70] F d d d Origin choice 1 (F 2 2 -1d)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"-x+1/4,-y+1/4,-z+1/4"},{"x+1/4,y+1/4,-z+1/4"},{"x+1/4,-y+1/4,z+1/4"},
{"-x+1/4,y+1/4,z+1/4"},{"x,y+1/2,z+1/2"},{"-x,-y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x,-y+1/2,-z+1/2"},{"-x+1/4,-y+3/4,-z+3/4"},{"x+1/4,y+3/4,-z+3/4"},
{"x+1/4,-y+3/4,z+3/4"},{"-x+1/4,y+3/4,z+3/4"},{"x+1/2,y,z+1/2"},{"-x+1/2,-y,z+1/2"},{"-x+1/2,y,-z+1/2"},{"x+1/2,-y,-z+1/2"},{"-x+3/4,-y+1/4,-z+3/4"},
{"x+3/4,y+1/4,-z+3/4"},{"x+3/4,-y+1/4,z+3/4"},{"-x+3/4,y+1/4,z+3/4"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},
{"-x+3/4,-y+3/4,-z+1/4"},{"x+3/4,y+3/4,-z+1/4"},{"x+3/4,-y+3/4,z+1/4"},{"-x+3/4,y+3/4,z+1/4"}},

// [70] F d d d Origin choice 2 (-F 2uv 2vw)
{{"x,y,z"},{"-x+3/4,-y+3/4,z"},{"-x+3/4,y,-z+3/4"},{"x,-y+3/4,-z+3/4"},{"-x,-y,-z"},{"x+1/4,y+1/4,-z"},{"x+1/4,-y,z+1/4"},
{"-x,y+1/4,z+1/4"},{"x,y+1/2,z+1/2"},{"-x+3/4,-y+1/4,z+1/2"},{"-x+3/4,y+1/2,-z+1/4"},{"x,-y+1/4,-z+1/4"},{"-x,-y+1/2,-z+1/2"},{"x+1/4,y+3/4,-z+1/2"},
{"x+1/4,-y+1/2,z+3/4"},{"-x,y+3/4,z+3/4"},{"x+1/2,y,z+1/2"},{"-x+1/4,-y+3/4,z+1/2"},{"-x+1/4,y,-z+1/4"},{"x+1/2,-y+3/4,-z+1/4"},{"-x+1/2,-y,-z+1/2"},
{"x+3/4,y+1/4,-z+1/2"},{"x+3/4,-y,z+3/4"},{"-x+1/2,y+1/4,z+3/4"},{"x+1/2,y+1/2,z"},{"-x+1/4,-y+1/4,z"},{"-x+1/4,y+1/2,-z+3/4"},{"x+1/2,-y+1/4,-z+3/4"},
{"-x+1/2,-y+1/2,-z"},{"x+3/4,y+3/4,-z"},{"x+3/4,-y+1/2,z+1/4"},{"-x+1/2,y+3/4,z+1/4"}},

// [71] I m m m (-I 2 2)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"-x,-y,-z"},{"x,y,-z"},{"x,-y,z"},
{"-x,y,z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},
{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [72] I b a m (-I 2 2c)
{{"x,y,z"},{"-x,-y,z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},{"-x,-y,-z"},{"x,y,-z"},{"x+1/2,-y+1/2,z"},
{"-x+1/2,y+1/2,z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-x,y,-z+1/2"},{"x,-y,-z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},
{"x,-y,z+1/2"},{"-x,y,z+1/2"}},

// [72] I m c b (-I 2a 2)
{{"x,y,z"},{"x,-y,-z"},{"-x,-y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"-x,-y,-z"},{"-x,y,z"},{"x,y+1/2,-z+1/2"},
{"x,-y+1/2,z+1/2"},{"x+1/2,y+1/2,z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,-y,z"},{"-x+1/2,y,-z"},{"-x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,y+1/2,z+1/2"},
{"x+1/2,y,-z"},{"x+1/2,-y,z"}},

// [72] I c m a (-I 2b 2b)
{{"x,y,z"},{"-x,y,-z"},{"-x+1/2,-y,z+1/2"},{"x+1/2,-y,-z+1/2"},{"-x,-y,-z"},{"x,-y,z"},{"x+1/2,y,-z+1/2"},
{"-x+1/2,y,z+1/2"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"-x,-y+1/2,z"},{"x,-y+1/2,-z"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"},
{"x,y+1/2,-z"},{"-x,y+1/2,z"}},

// [73] I b c a (-I 2b 2c)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z"},{"-x,-y,-z"},{"x+1/2,y,-z+1/2"},{"x,-y+1/2,z+1/2"},
{"-x+1/2,y+1/2,z"},{"x+1/2,y+1/2,z+1/2"},{"-x,-y+1/2,z"},{"-x+1/2,y,-z"},{"x,-y,-z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x,y+1/2,-z"},
{"x+1/2,-y,z"},{"-x,y,z+1/2"}},

// [73] I c a b (-I 2a 2b)
{{"x,y,z"},{"-x,-y+1/2,z+1/2"},{"x+1/2,-y,-z+1/2"},{"-x+1/2,y+1/2,-z"},{"-x,-y,-z"},{"x,y+1/2,-z+1/2"},{"-x+1/2,y,z+1/2"},
{"x+1/2,-y+1/2,z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y,z"},{"x,-y+1/2,-z"},{"-x,y,-z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y,-z"},
{"-x,y+1/2,z"},{"x,-y,z+1/2"}},

// [74] I m m a (-I 2b 2)
{{"x,y,z"},{"-x,-y+1/2,z"},{"-x,y+1/2,-z"},{"x,-y,-z"},{"-x,-y,-z"},{"x,y+1/2,-z"},{"x,-y+1/2,z"},
{"-x,y,z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y,z+1/2"},{"-x+1/2,y,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y,-z+1/2"},
{"x+1/2,-y,z+1/2"},{"-x+1/2,y+1/2,z+1/2"}},

// [74] I m m b (-I 2a 2a)
{{"x,y,z"},{"-x+1/2,-y,z"},{"x+1/2,-y,-z"},{"-x,y,-z"},{"-x,-y,-z"},{"x+1/2,y,-z"},{"-x+1/2,y,z"},
{"x,-y,z"},{"x+1/2,y+1/2,z+1/2"},{"-x,-y+1/2,z+1/2"},{"x,-y+1/2,-z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x,y+1/2,-z+1/2"},
{"-x,y+1/2,z+1/2"},{"x+1/2,-y+1/2,z+1/2"}},

// [74] I b m m (-I 2c 2c)
{{"x,y,z"},{"x,-y,-z+1/2"},{"-x,-y,z+1/2"},{"-x,y,-z"},{"-x,-y,-z"},{"-x,y,z+1/2"},{"x,y,-z+1/2"},
{"x,-y,z"},{"x+1/2,y+1/2,z+1/2"},{"x+1/2,-y+1/2,-z"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,y+1/2,z"},
{"x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,z+1/2"}},

// [74] I c m m (-I 2 2b)
{{"x,y,z"},{"x,-y+1/2,-z"},{"-x,y+1/2,-z"},{"-x,-y,z"},{"-x,-y,-z"},{"-x,y+1/2,z"},{"x,-y+1/2,z"},
{"x,y,-z"},{"x+1/2,y+1/2,z+1/2"},{"x+1/2,-y,-z+1/2"},{"-x+1/2,y,-z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,y,z+1/2"},
{"x+1/2,-y,z+1/2"},{"x+1/2,y+1/2,-z+1/2"}},

// [74] I m c m (-I 2 2a)
{{"x,y,z"},{"-x+1/2,y,-z"},{"x+1/2,-y,-z"},{"-x,-y,z"},{"-x,-y,-z"},{"x+1/2,-y,z"},{"-x+1/2,y,z"},
{"x,y,-z"},{"x+1/2,y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x,-y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x,-y+1/2,z+1/2"},
{"-x,y+1/2,z+1/2"},{"x+1/2,y+1/2,-z+1/2"}},

// [74] I m a m (-I 2c 2)
{{"x,y,z"},{"-x,y,-z+1/2"},{"-x,-y,z+1/2"},{"x,-y,-z"},{"-x,-y,-z"},{"x,-y,z+1/2"},{"x,y,-z+1/2"},
{"-x,y,z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z"},{"-x+1/2,-y+1/2,z"},{"x+1/2,-y+1/2,-z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z"},
{"x+1/2,y+1/2,-z"},{"-x+1/2,y+1/2,z+1/2"}},

// [75] P 4 (P 4)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"}},

// [76] P 41 (P 4w)
{{"x,y,z"},{"-x,-y,z+1/2"},{"-y,x,z+1/4"},{"y,-x,z+3/4"}},

// [77] P 42 (P 4c)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z+1/2"},{"y,-x,z+1/2"}},

// [78] P 43 (P 4cw)
{{"x,y,z"},{"-x,-y,z+1/2"},{"-y,x,z+3/4"},{"y,-x,z+1/4"}},

// [79] I 4 (I 4)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-y+1/2,x+1/2,z+1/2"},{"y+1/2,-x+1/2,z+1/2"}},

// [80] I 41 (I 4bw)
{{"x,y,z"},{"-x+1/2,-y+1/2,z+1/2"},{"-y,x+1/2,z+1/4"},{"y+1/2,-x,z+3/4"},{"x+1/2,y+1/2,z+1/2"},{"-x,-y,z"},{"-y+1/2,x,z+3/4"},{"y,-x+1/2,z+1/4"}},

// [81] P -4 (P -4)
{{"x,y,z"},{"-x,-y,z"},{"y,-x,-z"},{"-y,x,-z"}},

// [82] I -4 (I -4)
{{"x,y,z"},{"-x,-y,z"},{"y,-x,-z"},{"-y,x,-z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"y+1/2,-x+1/2,-z+1/2"},{"-y+1/2,x+1/2,-z+1/2"}},

// [83] P 4/m (-P 4)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"},{"-x,-y,-z"},{"x,y,-z"},{"y,-x,-z"},{"-y,x,-z"}},

// [84] P 42/m (-P 4c)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z+1/2"},{"y,-x,z+1/2"},{"-x,-y,-z"},{"x,y,-z"},{"y,-x,-z+1/2"},{"-y,x,-z+1/2"}},

    // TETRAGONAL GROUPS
    // =================

// [85] P 4/n Origin choice 1 (P 4ab -1ab)
{{"x,y,z"},{"-x,-y,z"},{"-y+1/2,x+1/2,z"},{"y+1/2,-x+1/2,z"},{"-x+1/2,-y+1/2,-z"},{"x+1/2,y+1/2,-z"},{"y,-x,-z"},{"-y,x,-z"}},

// [85] P 4/n Origin choice 2 (-P 4a)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-y+1/2,x,z"},{"y,-x+1/2,z"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"},{"y+1/2,-x,-z"},{"-y,x+1/2,-z"}},

// [86] P 42/n Origin choice 1 (P 4n -1n)
{{"x,y,z"},{"-x,-y,z"},{"-y+1/2,x+1/2,z+1/2"},{"y+1/2,-x+1/2,z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"y,-x,-z"},{"-y,x,-z"}},

// [86] P 42/n Origin choice 2 (-P 4bc)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-y,x+1/2,z+1/2"},{"y+1/2,-x,z+1/2"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"},{"y,-x+1/2,-z+1/2"},{"-y+1/2,x,-z+1/2"}},

// [87] I 4/m (-I 4)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"},{"-x,-y,-z"},{"x,y,-z"},{"y,-x,-z"},
{"-y,x,-z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-y+1/2,x+1/2,z+1/2"},{"y+1/2,-x+1/2,z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},
{"y+1/2,-x+1/2,-z+1/2"},{"-y+1/2,x+1/2,-z+1/2"}},

// [88] I 41/a Origin choice 1 (I 4bw -1bw)
{{"x,y,z"},{"-x+1/2,-y+1/2,z+1/2"},{"-y,x+1/2,z+1/4"},{"y+1/2,-x,z+3/4"},{"-x,-y+1/2,-z+1/4"},{"x+1/2,y,-z+3/4"},{"y,-x,-z"},
{"-y+1/2,x+1/2,-z+1/2"},{"x+1/2,y+1/2,z+1/2"},{"-x,-y,z"},{"-y+1/2,x,z+3/4"},{"y,-x+1/2,z+1/4"},{"-x+1/2,-y,-z+3/4"},{"x,y+1/2,-z+1/4"},
{"y+1/2,-x+1/2,-z+1/2"},{"-y,x,-z"}},

// [88] I 41/a Origin choice 2 (-I 4ad)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"-y+3/4,x+1/4,z+1/4"},{"y+3/4,-x+3/4,z+3/4"},{"-x,-y,-z"},{"x+1/2,y,-z+1/2"},{"y+1/4,-x+3/4,-z+3/4"},
{"-y+1/4,x+1/4,-z+1/4"},{"x+1/2,y+1/2,z+1/2"},{"-x,-y+1/2,z"},{"-y+1/4,x+3/4,z+3/4"},{"y+1/4,-x+1/4,z+1/4"},{"-x+1/2,-y+1/2,-z+1/2"},{"x,y+1/2,-z"},
{"y+3/4,-x+1/4,-z+1/4"},{"-y+3/4,x+3/4,-z+3/4"}},

// [89] P 4 2 2 (P 4 2)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"},{"-x,y,-z"},{"x,-y,-z"},{"y,x,-z"},{"-y,-x,-z"}},

// [90] P 4 21 2 (P 4ab 2ab)
{{"x,y,z"},{"-x,-y,z"},{"-y+1/2,x+1/2,z"},{"y+1/2,-x+1/2,z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},{"y,x,-z"},{"-y,-x,-z"}},

// [91] P 41 2 2 (P 4w 2c)
{{"x,y,z"},{"-x,-y,z+1/2"},{"-y,x,z+1/4"},{"y,-x,z+3/4"},{"-x,y,-z"},{"x,-y,-z+1/2"},{"y,x,-z+3/4"},{"-y,-x,-z+1/4"}},

// [92] P 41 21 2 (P 4abw 2nw)
{{"x,y,z"},{"-x,-y,z+1/2"},{"-y+1/2,x+1/2,z+1/4"},{"y+1/2,-x+1/2,z+3/4"},{"-x+1/2,y+1/2,-z+1/4"},{"x+1/2,-y+1/2,-z+3/4"},{"y,x,-z"},{"-y,-x,-z+1/2"}},

// [93] P 42 2 2 (P 4c 2)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z+1/2"},{"y,-x,z+1/2"},{"-x,y,-z"},{"x,-y,-z"},{"y,x,-z+1/2"},{"-y,-x,-z+1/2"}},

// [94] P 42 21 2 (P 4n 2n)
{{"x,y,z"},{"-x,-y,z"},{"-y+1/2,x+1/2,z+1/2"},{"y+1/2,-x+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"y,x,-z"},{"-y,-x,-z"}},

// [95] P 43 2 2 (P 4cw 2c)
{{"x,y,z"},{"-x,-y,z+1/2"},{"-y,x,z+3/4"},{"y,-x,z+1/4"},{"-x,y,-z"},{"x,-y,-z+1/2"},{"y,x,-z+1/4"},{"-y,-x,-z+3/4"}},

// [96] P 43 21 2 (P 4nw 2abw)
{{"x,y,z"},{"-x,-y,z+1/2"},{"-y+1/2,x+1/2,z+3/4"},{"y+1/2,-x+1/2,z+1/4"},{"-x+1/2,y+1/2,-z+3/4"},{"x+1/2,-y+1/2,-z+1/4"},{"y,x,-z"},{"-y,-x,-z+1/2"}},

// [97] I 4 2 2 (I 4 2)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"},{"-x,y,-z"},{"x,-y,-z"},{"y,x,-z"},
{"-y,-x,-z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-y+1/2,x+1/2,z+1/2"},{"y+1/2,-x+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},
{"y+1/2,x+1/2,-z+1/2"},{"-y+1/2,-x+1/2,-z+1/2"}},

// [98] I 41 2 2 (I 4bw 2bw)
{{"x,y,z"},{"-x+1/2,-y+1/2,z+1/2"},{"-y,x+1/2,z+1/4"},{"y+1/2,-x,z+3/4"},{"-x+1/2,y,-z+3/4"},{"x,-y+1/2,-z+1/4"},{"y+1/2,x+1/2,-z+1/2"},
{"-y,-x,-z"},{"x+1/2,y+1/2,z+1/2"},{"-x,-y,z"},{"-y+1/2,x,z+3/4"},{"y,-x+1/2,z+1/4"},{"-x,y+1/2,-z+1/4"},{"x+1/2,-y,-z+3/4"},
{"y,x,-z"},{"-y+1/2,-x+1/2,-z+1/2"}},

// [99] P 4 m m (P 4 -2)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"},{"x,-y,z"},{"-x,y,z"},{"-y,-x,z"},{"y,x,z"}},

// [100] P 4 b m (P 4 -2ab)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"},{"-y+1/2,-x+1/2,z"},{"y+1/2,x+1/2,z"}},

// [101] P 42 c m (P 4c -2c)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z+1/2"},{"y,-x,z+1/2"},{"x,-y,z+1/2"},{"-x,y,z+1/2"},{"-y,-x,z"},{"y,x,z"}},

// [102] P 42 n m (P 4n -2n)
{{"x,y,z"},{"-x,-y,z"},{"-y+1/2,x+1/2,z+1/2"},{"y+1/2,-x+1/2,z+1/2"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"},{"-y,-x,z"},{"y,x,z"}},

// [103] P 4 c c (P 4 -2c)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"},{"x,-y,z+1/2"},{"-x,y,z+1/2"},{"-y,-x,z+1/2"},{"y,x,z+1/2"}},

// [104] P 4 n c (P 4 -2n)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"},{"-y+1/2,-x+1/2,z+1/2"},{"y+1/2,x+1/2,z+1/2"}},

// [105] P 42 m c (P 4c -2)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z+1/2"},{"y,-x,z+1/2"},{"x,-y,z"},{"-x,y,z"},{"-y,-x,z+1/2"},{"y,x,z+1/2"}},

// [106] P 42 b c (P 4c -2ab)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z+1/2"},{"y,-x,z+1/2"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"},{"-y+1/2,-x+1/2,z+1/2"},{"y+1/2,x+1/2,z+1/2"}},

// [107] I 4 m m (I 4 -2)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"},{"x,-y,z"},{"-x,y,z"},{"-y,-x,z"},
{"y,x,z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-y+1/2,x+1/2,z+1/2"},{"y+1/2,-x+1/2,z+1/2"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"},
{"-y+1/2,-x+1/2,z+1/2"},{"y+1/2,x+1/2,z+1/2"}},

// [108] I 4 c m (I 4 -2c)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"},{"x,-y,z+1/2"},{"-x,y,z+1/2"},{"-y,-x,z+1/2"},
{"y,x,z+1/2"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-y+1/2,x+1/2,z+1/2"},{"y+1/2,-x+1/2,z+1/2"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"},
{"-y+1/2,-x+1/2,z"},{"y+1/2,x+1/2,z"}},

// [109] I 41 m d (I 4bw -2)
{{"x,y,z"},{"-x+1/2,-y+1/2,z+1/2"},{"-y,x+1/2,z+1/4"},{"y+1/2,-x,z+3/4"},{"x,-y,z"},{"-x+1/2,y+1/2,z+1/2"},{"-y,-x+1/2,z+1/4"},
{"y+1/2,x,z+3/4"},{"x+1/2,y+1/2,z+1/2"},{"-x,-y,z"},{"-y+1/2,x,z+3/4"},{"y,-x+1/2,z+1/4"},{"x+1/2,-y+1/2,z+1/2"},{"-x,y,z"},
{"-y+1/2,-x,z+3/4"},{"y,x+1/2,z+1/4"}},

// [110] I 41 c d (I 4bw -2c)
{{"x,y,z"},{"-x+1/2,-y+1/2,z+1/2"},{"-y,x+1/2,z+1/4"},{"y+1/2,-x,z+3/4"},{"x,-y,z+1/2"},{"-x+1/2,y+1/2,z"},{"-y,-x+1/2,z+3/4"},
{"y+1/2,x,z+1/4"},{"x+1/2,y+1/2,z+1/2"},{"-x,-y,z"},{"-y+1/2,x,z+3/4"},{"y,-x+1/2,z+1/4"},{"x+1/2,-y+1/2,z"},{"-x,y,z+1/2"},
{"-y+1/2,-x,z+1/4"},{"y,x+1/2,z+3/4"}},

// [111] P -4 2 m (P -4 2)
{{"x,y,z"},{"-x,-y,z"},{"y,-x,-z"},{"-y,x,-z"},{"-x,y,-z"},{"x,-y,-z"},{"-y,-x,z"},{"y,x,z"}},

// [112] P -4 2 c (P -4 2c)
{{"x,y,z"},{"-x,-y,z"},{"y,-x,-z"},{"-y,x,-z"},{"-x,y,-z+1/2"},{"x,-y,-z+1/2"},{"-y,-x,z+1/2"},{"y,x,z+1/2"}},

// [113] P -4 21 m (P -4 2ab)
{{"x,y,z"},{"-x,-y,z"},{"y,-x,-z"},{"-y,x,-z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},{"-y+1/2,-x+1/2,z"},{"y+1/2,x+1/2,z"}},

// [114] P -4 21 c (P -4 2n)
{{"x,y,z"},{"-x,-y,z"},{"y,-x,-z"},{"-y,x,-z"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"-y+1/2,-x+1/2,z+1/2"},{"y+1/2,x+1/2,z+1/2"}},

// [115] P -4 m 2 (P -4 -2)
{{"x,y,z"},{"-x,-y,z"},{"y,-x,-z"},{"-y,x,-z"},{"x,-y,z"},{"-x,y,z"},{"y,x,-z"},{"-y,-x,-z"}},

// [116] P -4 c 2 (P -4 -2c)
{{"x,y,z"},{"-x,-y,z"},{"y,-x,-z"},{"-y,x,-z"},{"x,-y,z+1/2"},{"-x,y,z+1/2"},{"y,x,-z+1/2"},{"-y,-x,-z+1/2"}},

// [117] P -4 b 2 (P -4 -2ab)
{{"x,y,z"},{"-x,-y,z"},{"y,-x,-z"},{"-y,x,-z"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"},{"y+1/2,x+1/2,-z"},{"-y+1/2,-x+1/2,-z"}},

// [118] P -4 n 2 (P -4 -2n)
{{"x,y,z"},{"-x,-y,z"},{"y,-x,-z"},{"-y,x,-z"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"},{"y+1/2,x+1/2,-z+1/2"},{"-y+1/2,-x+1/2,-z+1/2"}},

// [119] I -4 m 2 (I -4 -2)
{{"x,y,z"},{"-x,-y,z"},{"y,-x,-z"},{"-y,x,-z"},{"x,-y,z"},{"-x,y,z"},{"y,x,-z"},
{"-y,-x,-z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"y+1/2,-x+1/2,-z+1/2"},{"-y+1/2,x+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"},
{"y+1/2,x+1/2,-z+1/2"},{"-y+1/2,-x+1/2,-z+1/2"}},

// [120] I -4 c 2 (I -4 -2c)
{{"x,y,z"},{"-x,-y,z"},{"y,-x,-z"},{"-y,x,-z"},{"x,-y,z+1/2"},{"-x,y,z+1/2"},{"y,x,-z+1/2"},
{"-y,-x,-z+1/2"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"y+1/2,-x+1/2,-z+1/2"},{"-y+1/2,x+1/2,-z+1/2"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"},
{"y+1/2,x+1/2,-z"},{"-y+1/2,-x+1/2,-z"}},

// [121] I -4 2 m (I -4 2)
{{"x,y,z"},{"-x,-y,z"},{"y,-x,-z"},{"-y,x,-z"},{"-x,y,-z"},{"x,-y,-z"},{"-y,-x,z"},
{"y,x,z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"y+1/2,-x+1/2,-z+1/2"},{"-y+1/2,x+1/2,-z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},
{"-y+1/2,-x+1/2,z+1/2"},{"y+1/2,x+1/2,z+1/2"}},

// [122] I -4 2 d (I -4 2bw)
{{"x,y,z"},{"-x,-y,z"},{"y,-x,-z"},{"-y,x,-z"},{"-x+1/2,y,-z+3/4"},{"x+1/2,-y,-z+3/4"},{"-y+1/2,-x,z+3/4"},
{"y+1/2,x,z+3/4"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"y+1/2,-x+1/2,-z+1/2"},{"-y+1/2,x+1/2,-z+1/2"},{"-x,y+1/2,-z+1/4"},{"x,-y+1/2,-z+1/4"},
{"-y,-x+1/2,z+1/4"},{"y,x+1/2,z+1/4"}},

// [123] P 4/m m m (-P 4 2)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"},{"-x,y,-z"},{"x,-y,-z"},{"y,x,-z"},
{"-y,-x,-z"},{"-x,-y,-z"},{"x,y,-z"},{"y,-x,-z"},{"-y,x,-z"},{"x,-y,z"},{"-x,y,z"},
{"-y,-x,z"},{"y,x,z"}},

// [124] P 4/m c c (-P 4 2c)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"},{"-x,y,-z+1/2"},{"x,-y,-z+1/2"},{"y,x,-z+1/2"},
{"-y,-x,-z+1/2"},{"-x,-y,-z"},{"x,y,-z"},{"y,-x,-z"},{"-y,x,-z"},{"x,-y,z+1/2"},{"-x,y,z+1/2"},
{"-y,-x,z+1/2"},{"y,x,z+1/2"}},

// [125] P 4/n b m Origin choise 1 (P 4 2 -1ab)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"},{"-x,y,-z"},{"x,-y,-z"},{"y,x,-z"},
{"-y,-x,-z"},{"-x+1/2,-y+1/2,-z"},{"x+1/2,y+1/2,-z"},{"y+1/2,-x+1/2,-z"},{"-y+1/2,x+1/2,-z"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"},
{"-y+1/2,-x+1/2,z"},{"y+1/2,x+1/2,z"}},

// [125] P 4/n b m Origin choise 2 (-P 4a 2b)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-y+1/2,x,z"},{"y,-x+1/2,z"},{"-x+1/2,y,-z"},{"x,-y+1/2,-z"},{"y,x,-z"},
{"-y+1/2,-x+1/2,-z"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"},{"y+1/2,-x,-z"},{"-y,x+1/2,-z"},{"x+1/2,-y,z"},{"-x,y+1/2,z"},
{"-y,-x,z"},{"y+1/2,x+1/2,z"}},

// [126] P 4/n n c Origin choise 1 (P 4 2 -1n)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"},{"-x,y,-z"},{"x,-y,-z"},{"y,x,-z"},
{"-y,-x,-z"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"y+1/2,-x+1/2,-z+1/2"},{"-y+1/2,x+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"},
{"-y+1/2,-x+1/2,z+1/2"},{"y+1/2,x+1/2,z+1/2"}},

// [126] P 4/n n c Origin choise 2 (-P 4a 2bc)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-y+1/2,x,z"},{"y,-x+1/2,z"},{"-x+1/2,y,-z+1/2"},{"x,-y+1/2,-z+1/2"},{"y,x,-z+1/2"},
{"-y+1/2,-x+1/2,-z+1/2"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"},{"y+1/2,-x,-z"},{"-y,x+1/2,-z"},{"x+1/2,-y,z+1/2"},{"-x,y+1/2,z+1/2"},
{"-y,-x,z+1/2"},{"y+1/2,x+1/2,z+1/2"}},

// [127] P 4/m b m (-P 4 2ab)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},{"y+1/2,x+1/2,-z"},
{"-y+1/2,-x+1/2,-z"},{"-x,-y,-z"},{"x,y,-z"},{"y,-x,-z"},{"-y,x,-z"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"},
{"-y+1/2,-x+1/2,z"},{"y+1/2,x+1/2,z"}},

// [128] P 4/m n c (-P 4 2n)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"y+1/2,x+1/2,-z+1/2"},
{"-y+1/2,-x+1/2,-z+1/2"},{"-x,-y,-z"},{"x,y,-z"},{"y,-x,-z"},{"-y,x,-z"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"},
{"-y+1/2,-x+1/2,z+1/2"},{"y+1/2,x+1/2,z+1/2"}},

// [129] P 4/n m m Origin choise 1 (P 4ab 2ab -1ab)
{{"x,y,z"},{"-x,-y,z"},{"-y+1/2,x+1/2,z"},{"y+1/2,-x+1/2,z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},{"y,x,-z"},
{"-y,-x,-z"},{"-x+1/2,-y+1/2,-z"},{"x+1/2,y+1/2,-z"},{"y,-x,-z"},{"-y,x,-z"},{"x,-y,z"},{"-x,y,z"},
{"-y+1/2,-x+1/2,z"},{"y+1/2,x+1/2,z"}},

// [129] P 4/n m m Origin choise 2 (-P 4a 2a)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-y+1/2,x,z"},{"y,-x+1/2,z"},{"-x,y+1/2,-z"},{"x+1/2,-y,-z"},{"y+1/2,x+1/2,-z"},
{"-y,-x,-z"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"},{"y+1/2,-x,-z"},{"-y,x+1/2,-z"},{"x,-y+1/2,z"},{"-x+1/2,y,z"},
{"-y+1/2,-x+1/2,z"},{"y,x,z"}},

// [130] P 4/n c c Origin choise 1 (P 4ab 2n -1ab)
{{"x,y,z"},{"-x,-y,z"},{"-y+1/2,x+1/2,z"},{"y+1/2,-x+1/2,z"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"y,x,-z+1/2"},
{"-y,-x,-z+1/2"},{"-x+1/2,-y+1/2,-z"},{"x+1/2,y+1/2,-z"},{"y,-x,-z"},{"-y,x,-z"},{"x,-y,z+1/2"},{"-x,y,z+1/2"},
{"-y+1/2,-x+1/2,z+1/2"},{"y+1/2,x+1/2,z+1/2"}},

// [130] P 4/n c c Origin choise 2 (-P 4a 2ac)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-y+1/2,x,z"},{"y,-x+1/2,z"},{"-x,y+1/2,-z+1/2"},{"x+1/2,-y,-z+1/2"},{"y+1/2,x+1/2,-z+1/2"},
{"-y,-x,-z+1/2"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"},{"y+1/2,-x,-z"},{"-y,x+1/2,-z"},{"x,-y+1/2,z+1/2"},{"-x+1/2,y,z+1/2"},
{"-y+1/2,-x+1/2,z+1/2"},{"y,x,z+1/2"}},

// [131] P 42/m m c (-P 4c 2)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z+1/2"},{"y,-x,z+1/2"},{"-x,y,-z"},{"x,-y,-z"},{"y,x,-z+1/2"},
{"-y,-x,-z+1/2"},{"-x,-y,-z"},{"x,y,-z"},{"y,-x,-z+1/2"},{"-y,x,-z+1/2"},{"x,-y,z"},{"-x,y,z"},
{"-y,-x,z+1/2"},{"y,x,z+1/2"}},

// [132] P 42/m c m (-P 4c 2c)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z+1/2"},{"y,-x,z+1/2"},{"-x,y,-z+1/2"},{"x,-y,-z+1/2"},{"y,x,-z"},
{"-y,-x,-z"},{"-x,-y,-z"},{"x,y,-z"},{"y,-x,-z+1/2"},{"-y,x,-z+1/2"},{"x,-y,z+1/2"},{"-x,y,z+1/2"},
{"-y,-x,z"},{"y,x,z"}},

// [133] P 42/n b c Origin choise 1 (P 4n 2c -1n)
{{"x,y,z"},{"-x,-y,z"},{"-y+1/2,x+1/2,z+1/2"},{"y+1/2,-x+1/2,z+1/2"},{"-x,y,-z+1/2"},{"x,-y,-z+1/2"},{"y+1/2,x+1/2,-z"},
{"-y+1/2,-x+1/2,-z"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"y,-x,-z"},{"-y,x,-z"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"},
{"-y,-x,z+1/2"},{"y,x,z+1/2"}},

// [133] P 42/n b c Origin choise 2 (-P 4ac 2b)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-y+1/2,x,z+1/2"},{"y,-x+1/2,z+1/2"},{"-x+1/2,y,-z"},{"x,-y+1/2,-z"},{"y,x,-z+1/2"},
{"-y+1/2,-x+1/2,-z+1/2"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"},{"y+1/2,-x,-z+1/2"},{"-y,x+1/2,-z+1/2"},{"x+1/2,-y,z"},{"-x,y+1/2,z"},
{"-y,-x,z+1/2"},{"y+1/2,x+1/2,z+1/2"}},

// [134] P 42/n n m Origin choise 1 (P 4n 2 -1n)
{{"x,y,z"},{"-x,-y,z"},{"-y+1/2,x+1/2,z+1/2"},{"y+1/2,-x+1/2,z+1/2"},{"-x,y,-z"},{"x,-y,-z"},{"y+1/2,x+1/2,-z+1/2"},
{"-y+1/2,-x+1/2,-z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"y,-x,-z"},{"-y,x,-z"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"},
{"-y,-x,z"},{"y,x,z"}},

// [134] P 42/n n m Origin choise 2 (-P 4ac 2bc)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-y+1/2,x,z+1/2"},{"y,-x+1/2,z+1/2"},{"-x+1/2,y,-z+1/2"},{"x,-y+1/2,-z+1/2"},{"y,x,-z"},
{"-y+1/2,-x+1/2,-z"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"},{"y+1/2,-x,-z+1/2"},{"-y,x+1/2,-z+1/2"},{"x+1/2,-y,z+1/2"},{"-x,y+1/2,z+1/2"},
{"-y,-x,z"},{"y+1/2,x+1/2,z"}},

// [135] P 42/m b c (-P 4c 2ab)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z+1/2"},{"y,-x,z+1/2"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},{"y+1/2,x+1/2,-z+1/2"},
{"-y+1/2,-x+1/2,-z+1/2"},{"-x,-y,-z"},{"x,y,-z"},{"y,-x,-z+1/2"},{"-y,x,-z+1/2"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"},
{"-y+1/2,-x+1/2,z+1/2"},{"y+1/2,x+1/2,z+1/2"}},

// [136] P 42/m n m (-P 4n 2n)
{{"x,y,z"},{"-x,-y,z"},{"-y+1/2,x+1/2,z+1/2"},{"y+1/2,-x+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"y,x,-z"},
{"-y,-x,-z"},{"-x,-y,-z"},{"x,y,-z"},{"y+1/2,-x+1/2,-z+1/2"},{"-y+1/2,x+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"},
{"-y,-x,z"},{"y,x,z"}},

// [137] P 42/n c m Origin choise 1 (P 4n 2n -1n)
{{"x,y,z"},{"-x,-y,z"},{"-y+1/2,x+1/2,z+1/2"},{"y+1/2,-x+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"y,x,-z"},
{"-y,-x,-z"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"y,-x,-z"},{"-y,x,-z"},{"x,-y,z"},{"-x,y,z"},
{"-y+1/2,-x+1/2,z+1/2"},{"y+1/2,x+1/2,z+1/2"}},

// [137] P 42/n c m Origin choise 2 (-P 4ac 2a)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-y+1/2,x,z+1/2"},{"y,-x+1/2,z+1/2"},{"-x,y+1/2,-z"},{"x+1/2,-y,-z"},{"y+1/2,x+1/2,-z+1/2"},
{"-y,-x,-z+1/2"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"},{"y+1/2,-x,-z+1/2"},{"-y,x+1/2,-z+1/2"},{"x,-y+1/2,z"},{"-x+1/2,y,z"},
{"-y+1/2,-x+1/2,z+1/2"},{"y,x,z+1/2"}},

// [138] P 42/n c m Origin choise 1 (P 4n 2ab -1n)
{{"x,y,z"},{"-x,-y,z"},{"-y+1/2,x+1/2,z+1/2"},{"y+1/2,-x+1/2,z+1/2"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},{"y,x,-z+1/2"},
{"-y,-x,-z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"y,-x,-z"},{"-y,x,-z"},{"x,-y,z+1/2"},{"-x,y,z+1/2"},
{"-y+1/2,-x+1/2,z"},{"y+1/2,x+1/2,z"}},

// [138] P 42/n c m Origin choise 2 (-P 4ac 2ac)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-y+1/2,x,z+1/2"},{"y,-x+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x+1/2,-y,-z+1/2"},{"y+1/2,x+1/2,-z"},
{"-y,-x,-z"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"},{"y+1/2,-x,-z+1/2"},{"-y,x+1/2,-z+1/2"},{"x,-y+1/2,z+1/2"},{"-x+1/2,y,z+1/2"},
{"-y+1/2,-x+1/2,z"},{"y,x,z"}},

// [139] I 4/m m m (-I 4 2)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"},{"-x,y,-z"},{"x,-y,-z"},{"y,x,-z"},
{"-y,-x,-z"},{"-x,-y,-z"},{"x,y,-z"},{"y,-x,-z"},{"-y,x,-z"},{"x,-y,z"},{"-x,y,z"},
{"-y,-x,z"},{"y,x,z"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-y+1/2,x+1/2,z+1/2"},{"y+1/2,-x+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},
{"x+1/2,-y+1/2,-z+1/2"},{"y+1/2,x+1/2,-z+1/2"},{"-y+1/2,-x+1/2,-z+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"y+1/2,-x+1/2,-z+1/2"},{"-y+1/2,x+1/2,-z+1/2"},
{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"},{"-y+1/2,-x+1/2,z+1/2"},{"y+1/2,x+1/2,z+1/2"}},

// [140] I 4/m c m (-I 4 2c)
{{"x,y,z"},{"-x,-y,z"},{"-y,x,z"},{"y,-x,z"},{"-x,y,-z+1/2"},{"x,-y,-z+1/2"},{"y,x,-z+1/2"},
{"-y,-x,-z+1/2"},{"-x,-y,-z"},{"x,y,-z"},{"y,-x,-z"},{"-y,x,-z"},{"x,-y,z+1/2"},{"-x,y,z+1/2"},
{"-y,-x,z+1/2"},{"y,x,z+1/2"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-y+1/2,x+1/2,z+1/2"},{"y+1/2,-x+1/2,z+1/2"},{"-x+1/2,y+1/2,-z"},
{"x+1/2,-y+1/2,-z"},{"y+1/2,x+1/2,-z"},{"-y+1/2,-x+1/2,-z"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"y+1/2,-x+1/2,-z+1/2"},{"-y+1/2,x+1/2,-z+1/2"},
{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"},{"-y+1/2,-x+1/2,z"},{"y+1/2,x+1/2,z"}},

// [141] I 41/a m d Origin choise 1 (I 4bw 2bw -1bw)
{{"x,y,z"},{"-x+1/2,-y+1/2,z+1/2"},{"-y,x+1/2,z+1/4"},{"y+1/2,-x,z+3/4"},{"-x+1/2,y,-z+3/4"},{"x,-y+1/2,-z+1/4"},{"y+1/2,x+1/2,-z+1/2"},
{"-y,-x,-z"},{"-x,-y+1/2,-z+1/4"},{"x+1/2,y,-z+3/4"},{"y,-x,-z"},{"-y+1/2,x+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"},{"-x,y,z"},
{"-y+1/2,-x,z+3/4"},{"y,x+1/2,z+1/4"},{"x+1/2,y+1/2,z+1/2"},{"-x,-y,z"},{"-y+1/2,x,z+3/4"},{"y,-x+1/2,z+1/4"},{"-x,y+1/2,-z+1/4"},
{"x+1/2,-y,-z+3/4"},{"y,x,-z"},{"-y+1/2,-x+1/2,-z+1/2"},{"-x+1/2,-y,-z+3/4"},{"x,y+1/2,-z+1/4"},{"y+1/2,-x+1/2,-z+1/2"},{"-y,x,-z"},
{"x,-y,z"},{"-x+1/2,y+1/2,z+1/2"},{"-y,-x+1/2,z+1/4"},{"y+1/2,x,z+3/4"}},

// [141] I 41/a m d Origin choise 2 (-I 4bd 2)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"-y+1/4,x+3/4,z+1/4"},{"y+1/4,-x+1/4,z+3/4"},{"-x+1/2,y,-z+1/2"},{"x,-y,-z"},{"y+1/4,x+3/4,-z+1/4"},
{"-y+1/4,-x+1/4,-z+3/4"},{"-x,-y,-z"},{"x+1/2,y,-z+1/2"},{"y+3/4,-x+1/4,-z+3/4"},{"-y+3/4,x+3/4,-z+1/4"},{"x+1/2,-y,z+1/2"},{"-x,y,z"},
{"-y+3/4,-x+1/4,z+3/4"},{"y+3/4,x+3/4,z+1/4"},{"x+1/2,y+1/2,z+1/2"},{"-x,-y+1/2,z"},{"-y+3/4,x+1/4,z+3/4"},{"y+3/4,-x+3/4,z+1/4"},{"-x,y+1/2,-z"},
{"x+1/2,-y+1/2,-z+1/2"},{"y+3/4,x+1/4,-z+3/4"},{"-y+3/4,-x+3/4,-z+1/4"},{"-x+1/2,-y+1/2,-z+1/2"},{"x,y+1/2,-z"},{"y+1/4,-x+3/4,-z+1/4"},{"-y+1/4,x+1/4,-z+3/4"},
{"x,-y+1/2,z"},{"-x+1/2,y+1/2,z+1/2"},{"-y+1/4,-x+3/4,z+1/4"},{"y+1/4,x+1/4,z+3/4"}},

// [142] I 41/a c d Origin choise 1 (I 4bw 2aw -1bw)
{{"x,y,z"},{"-x+1/2,-y+1/2,z+1/2"},{"-y,x+1/2,z+1/4"},{"y+1/2,-x,z+3/4"},{"-x+1/2,y,-z+1/4"},{"x,-y+1/2,-z+3/4"},{"y+1/2,x+1/2,-z"},
{"-y,-x,-z+1/2"},{"-x,-y+1/2,-z+1/4"},{"x+1/2,y,-z+3/4"},{"y,-x,-z"},{"-y+1/2,x+1/2,-z+1/2"},{"x+1/2,-y+1/2,z"},{"-x,y,z+1/2"},
{"-y+1/2,-x,z+1/4"},{"y,x+1/2,z+3/4"},{"x+1/2,y+1/2,z+1/2"},{"-x,-y,z"},{"-y+1/2,x,z+3/4"},{"y,-x+1/2,z+1/4"},{"-x,y+1/2,-z+3/4"},
{"x+1/2,-y,-z+1/4"},{"y,x,-z+1/2"},{"-y+1/2,-x+1/2,-z"},{"-x+1/2,-y,-z+3/4"},{"x,y+1/2,-z+1/4"},{"y+1/2,-x+1/2,-z+1/2"},{"-y,x,-z"},
{"x,-y,z+1/2"},{"-x+1/2,y+1/2,z"},{"-y,-x+1/2,z+3/4"},{"y+1/2,x,z+1/4"}},

// [142] I 41/a c d Origin choise 2 (-I 4bd 2c)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"-y+1/4,x+3/4,z+1/4"},{"y+1/4,-x+1/4,z+3/4"},{"-x+1/2,y,-z"},{"x,-y,-z+1/2"},{"y+1/4,x+3/4,-z+3/4"},
{"-y+1/4,-x+1/4,-z+1/4"},{"-x,-y,-z"},{"x+1/2,y,-z+1/2"},{"y+3/4,-x+1/4,-z+3/4"},{"-y+3/4,x+3/4,-z+1/4"},{"x+1/2,-y,z"},{"-x,y,z+1/2"},
{"-y+3/4,-x+1/4,z+1/4"},{"y+3/4,x+3/4,z+3/4"},{"x+1/2,y+1/2,z+1/2"},{"-x,-y+1/2,z"},{"-y+3/4,x+1/4,z+3/4"},{"y+3/4,-x+3/4,z+1/4"},{"-x,y+1/2,-z+1/2"},
{"x+1/2,-y+1/2,-z"},{"y+3/4,x+1/4,-z+1/4"},{"-y+3/4,-x+3/4,-z+3/4"},{"-x+1/2,-y+1/2,-z+1/2"},{"x,y+1/2,-z"},{"y+1/4,-x+3/4,-z+1/4"},{"-y+1/4,x+1/4,-z+3/4"},
{"x,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z"},{"-y+1/4,-x+3/4,z+3/4"},{"y+1/4,x+1/4,z+1/4"}},

    // TRIGONAL GROUPS
    // ===============


// [143] P 3 (P 3)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"}},

// [144] P 31 (P 31)
{{"x,y,z"},{"-y,x-y,z+1/3"},{"-x+y,-x,z+2/3"}},

// [145] P 32 (P 32)
{{"x,y,z"},{"-y,x-y,z+2/3"},{"-x+y,-x,z+1/3"}},

// [146] R 3 hexagonal axes (R 3)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"x+2/3,y+1/3,z+1/3"},{"-y+2/3,x-y+1/3,z+1/3"},{"-x+y+2/3,-x+1/3,z+1/3"},{"x+1/3,y+2/3,z+2/3"},
{"-y+1/3,x-y+2/3,z+2/3"},{"-x+y+1/3,-x+2/3,z+2/3"}},

// [146] R 3 Rhombohedral axes (P 3*)
{{"x,y,z"},{"z,x,y"},{"y,z,x"}},

// [147] P -3 (P -3)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-x,-y,-z"},{"y,-x+y,-z"},{"x-y,x,-z"}},

// [148] R-3 hexagonal axes (-R 3)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-x,-y,-z"},{"y,-x+y,-z"},{"x-y,x,-z"},{"x+2/3,y+1/3,z+1/3"},
{"-y+2/3,x-y+1/3,z+1/3"},{"-x+y+2/3,-x+1/3,z+1/3"},{"-x+2/3,-y+1/3,-z+1/3"},{"y+2/3,-x+y+1/3,-z+1/3"},{"x-y+2/3,x+1/3,-z+1/3"},{"x+1/3,y+2/3,z+2/3"},{"-y+1/3,x-y+2/3,z+2/3"},
{"-x+y+1/3,-x+2/3,z+2/3"},{"-x+1/3,-y+2/3,-z+2/3"},{"y+1/3,-x+y+2/3,-z+2/3"},{"x-y+1/3,x+2/3,-z+2/3"}},

// [148] R -3 Rhombohedral axes (-P 3*)
{{"x,y,z"},{"z,x,y"},{"y,z,x"},{"-x,-y,-z"},{"-z,-x,-y"},{"-y,-z,-x"}},

// [149] P 3 1 2 (P 3 2)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-y,-x,-z"},{"-x+y,y,-z"},{"x,x-y,-z"}},

// [150] P 3 2 1 (P 3 2")
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"y,x,-z"},{"x-y,-y,-z"},{"-x,-x+y,-z"}},

// [151] P 31 1 2 (P 31 2 (0 0 4))
{{"x,y,z"},{"-y,x-y,z+1/3"},{"-x+y,-x,z+2/3"},{"-y,-x,-z+2/3"},{"-x+y,y,-z+1/3"},{"x,x-y,-z"}},

// [152] P 31 2 1 (P 31 2")
{{"x,y,z"},{"-y,x-y,z+1/3"},{"-x+y,-x,z+2/3"},{"y,x,-z"},{"x-y,-y,-z+2/3"},{"-x,-x+y,-z+1/3"}},

// [153] P 32 1 2 (P 32 2 (0 0 2))
{{"x,y,z"},{"-y,x-y,z+2/3"},{"-x+y,-x,z+1/3"},{"-y,-x,-z+1/3"},{"-x+y,y,-z+2/3"},{"x,x-y,-z"}},

// [154] P 32 2 1 (P 32 2")
{{"x,y,z"},{"-y,x-y,z+2/3"},{"-x+y,-x,z+1/3"},{"y,x,-z"},{"x-y,-y,-z+1/3"},{"-x,-x+y,-z+2/3"}},

// [155] R 3 2 Hexagonal axes (R 3 2")
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"y,x,-z"},{"x-y,-y,-z"},{"-x,-x+y,-z"},{"x+2/3,y+1/3,z+1/3"},
{"-y+2/3,x-y+1/3,z+1/3"},{"-x+y+2/3,-x+1/3,z+1/3"},{"y+2/3,x+1/3,-z+1/3"},{"x-y+2/3,-y+1/3,-z+1/3"},{"-x+2/3,-x+y+1/3,-z+1/3"},{"x+1/3,y+2/3,z+2/3"},{"-y+1/3,x-y+2/3,z+2/3"},
{"-x+y+1/3,-x+2/3,z+2/3"},{"y+1/3,x+2/3,-z+2/3"},{"x-y+1/3,-y+2/3,-z+2/3"},{"-x+1/3,-x+y+2/3,-z+2/3"}},

// [155] R 3 2 Rhombohedral axes (P 3* 2)
{{"x,y,z"},{"z,x,y"},{"y,z,x"},{"-y,-x,-z"},{"-x,-z,-y"},{"-z,-y,-x"}},

// [156] P 3 m 1 (P 3 -2")
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-y,-x,z"},{"-x+y,y,z"},{"x,x-y,z"}},

// [157] P 3 1 m (P 3 -2)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"y,x,z"},{"x-y,-y,z"},{"-x,-x+y,z"}},

// [158] P 3 c 1 (P 3 -2"c)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-y,-x,z+1/2"},{"-x+y,y,z+1/2"},{"x,x-y,z+1/2"}},

// [159] P 3 1 c (P 3 -2c)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"y,x,z+1/2"},{"x-y,-y,z+1/2"},{"-x,-x+y,z+1/2"}},

// [160] R 3 m Hexagonal axes (R 3 -2")
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-y,-x,z"},{"-x+y,y,z"},{"x,x-y,z"},{"x+2/3,y+1/3,z+1/3"},
{"-y+2/3,x-y+1/3,z+1/3"},{"-x+y+2/3,-x+1/3,z+1/3"},{"-y+2/3,-x+1/3,z+1/3"},{"-x+y+2/3,y+1/3,z+1/3"},{"x+2/3,x-y+1/3,z+1/3"},{"x+1/3,y+2/3,z+2/3"},{"-y+1/3,x-y+2/3,z+2/3"},
{"-x+y+1/3,-x+2/3,z+2/3"},{"-y+1/3,-x+2/3,z+2/3"},{"-x+y+1/3,y+2/3,z+2/3"},{"x+1/3,x-y+2/3,z+2/3"}},

// [160] R 3 m Rhombohedral axes (P 3* -2)
{{"x,y,z"},{"z,x,y"},{"y,z,x"},{"y,x,z"},{"x,z,y"},{"z,y,x"}},

// [161] R 3 c Hexagonal axes (R 3 -2"c)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-y,-x,z+1/2"},{"-x+y,y,z+1/2"},{"x,x-y,z+1/2"},{"x+2/3,y+1/3,z+1/3"},
{"-y+2/3,x-y+1/3,z+1/3"},{"-x+y+2/3,-x+1/3,z+1/3"},{"-y+2/3,-x+1/3,z+5/6"},{"-x+y+2/3,y+1/3,z+5/6"},{"x+2/3,x-y+1/3,z+5/6"},{"x+1/3,y+2/3,z+2/3"},{"-y+1/3,x-y+2/3,z+2/3"},
{"-x+y+1/3,-x+2/3,z+2/3"},{"-y+1/3,-x+2/3,z+1/6"},{"-x+y+1/3,y+2/3,z+1/6"},{"x+1/3,x-y+2/3,z+1/6"}},

// [161] R 3 c Rhombohedral axes (P 3* -2n)
{{"x,y,z"},{"z,x,y"},{"y,z,x"},{"y+1/2,x+1/2,z+1/2"},{"x+1/2,z+1/2,y+1/2"},{"z+1/2,y+1/2,x+1/2"}},

// [162] P -3 1 m (-P 3 2)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-y,-x,-z"},{"-x+y,y,-z"},{"x,x-y,-z"},{"-x,-y,-z"},
{"y,-x+y,-z"},{"x-y,x,-z"},{"y,x,z"},{"x-y,-y,z"},{"-x,-x+y,z"}},

// [163] P -3 1 c (-P 3 2c)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-y,-x,-z+1/2"},{"-x+y,y,-z+1/2"},{"x,x-y,-z+1/2"},{"-x,-y,-z"},
{"y,-x+y,-z"},{"x-y,x,-z"},{"y,x,z+1/2"},{"x-y,-y,z+1/2"},{"-x,-x+y,z+1/2"}},

// [164] P -3 m 1 (-P 3 2")
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"y,x,-z"},{"x-y,-y,-z"},{"-x,-x+y,-z"},{"-x,-y,-z"},
{"y,-x+y,-z"},{"x-y,x,-z"},{"-y,-x,z"},{"-x+y,y,z"},{"x,x-y,z"}},

// [165] P -3 c 1 (-P 3 2"c)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"y,x,-z+1/2"},{"x-y,-y,-z+1/2"},{"-x,-x+y,-z+1/2"},{"-x,-y,-z"},
{"y,-x+y,-z"},{"x-y,x,-z"},{"-y,-x,z+1/2"},{"-x+y,y,z+1/2"},{"x,x-y,z+1/2"}},

// [166] R -3 m Hexagonal axes (-R 3 2")
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"y,x,-z"},{"x-y,-y,-z"},{"-x,-x+y,-z"},{"-x,-y,-z"},
{"y,-x+y,-z"},{"x-y,x,-z"},{"-y,-x,z"},{"-x+y,y,z"},{"x,x-y,z"},{"x+2/3,y+1/3,z+1/3"},{"-y+2/3,x-y+1/3,z+1/3"},
{"-x+y+2/3,-x+1/3,z+1/3"},{"y+2/3,x+1/3,-z+1/3"},{"x-y+2/3,-y+1/3,-z+1/3"},{"-x+2/3,-x+y+1/3,-z+1/3"},{"-x+2/3,-y+1/3,-z+1/3"},{"y+2/3,-x+y+1/3,-z+1/3"},{"x-y+2/3,x+1/3,-z+1/3"},
{"-y+2/3,-x+1/3,z+1/3"},{"-x+y+2/3,y+1/3,z+1/3"},{"x+2/3,x-y+1/3,z+1/3"},{"x+1/3,y+2/3,z+2/3"},{"-y+1/3,x-y+2/3,z+2/3"},{"-x+y+1/3,-x+2/3,z+2/3"},{"y+1/3,x+2/3,-z+2/3"},
{"x-y+1/3,-y+2/3,-z+2/3"},{"-x+1/3,-x+y+2/3,-z+2/3"},{"-x+1/3,-y+2/3,-z+2/3"},{"y+1/3,-x+y+2/3,-z+2/3"},{"x-y+1/3,x+2/3,-z+2/3"},{"-y+1/3,-x+2/3,z+2/3"},{"-x+y+1/3,y+2/3,z+2/3"},
{"x+1/3,x-y+2/3,z+2/3"}},

// [166] R -3 m Rhombohedral axes (-P 3* 2)
{{"x,y,z"},{"z,x,y"},{"y,z,x"},{"-y,-x,-z"},{"-x,-z,-y"},{"-z,-y,-x"},{"-x,-y,-z"},
{"-z,-x,-y"},{"-y,-z,-x"},{"y,x,z"},{"x,z,y"},{"z,y,x"}},

// [167] R -3 c Hexagonal axes (-R 3 2"c)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"y,x,-z+1/2"},{"x-y,-y,-z+1/2"},{"-x,-x+y,-z+1/2"},{"-x,-y,-z"},
{"y,-x+y,-z"},{"x-y,x,-z"},{"-y,-x,z+1/2"},{"-x+y,y,z+1/2"},{"x,x-y,z+1/2"},{"x+2/3,y+1/3,z+1/3"},{"-y+2/3,x-y+1/3,z+1/3"},
{"-x+y+2/3,-x+1/3,z+1/3"},{"y+2/3,x+1/3,-z+5/6"},{"x-y+2/3,-y+1/3,-z+5/6"},{"-x+2/3,-x+y+1/3,-z+5/6"},{"-x+2/3,-y+1/3,-z+1/3"},{"y+2/3,-x+y+1/3,-z+1/3"},{"x-y+2/3,x+1/3,-z+1/3"},
{"-y+2/3,-x+1/3,z+5/6"},{"-x+y+2/3,y+1/3,z+5/6"},{"x+2/3,x-y+1/3,z+5/6"},{"x+1/3,y+2/3,z+2/3"},{"-y+1/3,x-y+2/3,z+2/3"},{"-x+y+1/3,-x+2/3,z+2/3"},{"y+1/3,x+2/3,-z+1/6"},
{"x-y+1/3,-y+2/3,-z+1/6"},{"-x+1/3,-x+y+2/3,-z+1/6"},{"-x+1/3,-y+2/3,-z+2/3"},{"y+1/3,-x+y+2/3,-z+2/3"},{"x-y+1/3,x+2/3,-z+2/3"},{"-y+1/3,-x+2/3,z+1/6"},{"-x+y+1/3,y+2/3,z+1/6"},
{"x+1/3,x-y+2/3,z+1/6"}},

// [167] R -3 c Rhombohedral axes (-P 3* 2n)
{{"x,y,z"},{"z,x,y"},{"y,z,x"},{"-y+1/2,-x+1/2,-z+1/2"},{"-x+1/2,-z+1/2,-y+1/2"},{"-z+1/2,-y+1/2,-x+1/2"},{"-x,-y,-z"},
{"-z,-x,-y"},{"-y,-z,-x"},{"y+1/2,x+1/2,z+1/2"},{"x+1/2,z+1/2,y+1/2"},{"z+1/2,y+1/2,x+1/2"}},

    // HEXAGONAL GROUPS
    // ================

// [168] P 6 (P 6)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-x,-y,z"},{"y,-x+y,z"},{"x-y,x,z"}},

// [169] P 61 (P 61)
{{"x,y,z"},{"-y,x-y,z+1/3"},{"-x+y,-x,z+2/3"},{"-x,-y,z+1/2"},{"y,-x+y,z+5/6"},{"x-y,x,z+1/6"}},

// [170] P 65 (P 65)
{{"x,y,z"},{"-y,x-y,z+2/3"},{"-x+y,-x,z+1/3"},{"-x,-y,z+1/2"},{"y,-x+y,z+1/6"},{"x-y,x,z+5/6"}},

// [171] P 62 (P 62)
{{"x,y,z"},{"-y,x-y,z+2/3"},{"-x+y,-x,z+1/3"},{"-x,-y,z"},{"y,-x+y,z+2/3"},{"x-y,x,z+1/3"}},

// [172] P 64 (P 64)
{{"x,y,z"},{"-y,x-y,z+1/3"},{"-x+y,-x,z+2/3"},{"-x,-y,z"},{"y,-x+y,z+1/3"},{"x-y,x,z+2/3"}},

// [173] P 63 (P 6c)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-x,-y,z+1/2"},{"y,-x+y,z+1/2"},{"x-y,x,z+1/2"}},

// [174] P -6 (P -6)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"x,y,-z"},{"-y,x-y,-z"},{"-x+y,-x,-z"}},

// [175] P6/m (-P 6)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-x,-y,z"},{"y,-x+y,z"},{"x-y,x,z"},{"-x,-y,-z"},
     {"y,-x+y,-z"},{"x-y,x,-z"},{"x,y,-z"},{"-y,x-y,-z"},{"-x+y,-x,-z"}},

// [176] P 63/m (-P 6c)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-x,-y,z+1/2"},{"y,-x+y,z+1/2"},{"x-y,x,z+1/2"},{"-x,-y,-z"},
     {"y,-x+y,-z"},{"x-y,x,-z"},{"x,y,-z+1/2"},{"-y,x-y,-z+1/2"},{"-x+y,-x,-z+1/2"}},

// [177] P 6 2 2 (P 6 2)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-x,-y,z"},{"y,-x+y,z"},{"x-y,x,z"},{"y,x,-z"},
{"x-y,-y,-z"},{"-x,-x+y,-z"},{"-y,-x,-z"},{"-x+y,y,-z"},{"x,x-y,-z"}},

// [178] P 61 2 2 (P 61 2 (0 0 5))
{{"x,y,z"},{"-y,x-y,z+1/3"},{"-x+y,-x,z+2/3"},{"-x,-y,z+1/2"},{"y,-x+y,z+5/6"},{"x-y,x,z+1/6"},{"y,x,-z+1/3"},
{"x-y,-y,-z"},{"-x,-x+y,-z+2/3"},{"-y,-x,-z+5/6"},{"-x+y,y,-z+1/2"},{"x,x-y,-z+1/6"}},

// [179] P 65 2 2 (P 65 2 (0 0 1))
{{"x,y,z"},{"-y,x-y,z+2/3"},{"-x+y,-x,z+1/3"},{"-x,-y,z+1/2"},{"y,-x+y,z+1/6"},{"x-y,x,z+5/6"},{"y,x,-z+2/3"},
{"x-y,-y,-z"},{"-x,-x+y,-z+1/3"},{"-y,-x,-z+1/6"},{"-x+y,y,-z+1/2"},{"x,x-y,-z+5/6"}},

// [180] P 62 2 2 (P 62 2 (0 0 4))
{{"x,y,z"},{"-y,x-y,z+2/3"},{"-x+y,-x,z+1/3"},{"-x,-y,z"},{"y,-x+y,z+2/3"},{"x-y,x,z+1/3"},{"y,x,-z+2/3"},
{"x-y,-y,-z"},{"-x,-x+y,-z+1/3"},{"-y,-x,-z+2/3"},{"-x+y,y,-z"},{"x,x-y,-z+1/3"}},

// [181] P 64 2 2 (P 64 2 (0 0 2))
{{"x,y,z"},{"-y,x-y,z+1/3"},{"-x+y,-x,z+2/3"},{"-x,-y,z"},{"y,-x+y,z+1/3"},{"x-y,x,z+2/3"},{"y,x,-z+1/3"},
{"x-y,-y,-z"},{"-x,-x+y,-z+2/3"},{"-y,-x,-z+1/3"},{"-x+y,y,-z"},{"x,x-y,-z+2/3"}},

// [182] P 63 2 2 (P 6c 2c)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-x,-y,z+1/2"},{"y,-x+y,z+1/2"},{"x-y,x,z+1/2"},{"y,x,-z"},
{"x-y,-y,-z"},{"-x,-x+y,-z"},{"-y,-x,-z+1/2"},{"-x+y,y,-z+1/2"},{"x,x-y,-z+1/2"}},

// [183] P 6 m m (P 6 -2)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-x,-y,z"},{"y,-x+y,z"},{"x-y,x,z"},{"-y,-x,z"},
{"-x+y,y,z"},{"x,x-y,z"},{"y,x,z"},{"x-y,-y,z"},{"-x,-x+y,z"}},

// [184] P 6 c c (P 6 -2c)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-x,-y,z"},{"y,-x+y,z"},{"x-y,x,z"},{"-y,-x,z+1/2"},
{"-x+y,y,z+1/2"},{"x,x-y,z+1/2"},{"y,x,z+1/2"},{"x-y,-y,z+1/2"},{"-x,-x+y,z+1/2"}},

// [185] P 63 c m (P 6c -2)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-x,-y,z+1/2"},{"y,-x+y,z+1/2"},{"x-y,x,z+1/2"},{"-y,-x,z+1/2"},
{"-x+y,y,z+1/2"},{"x,x-y,z+1/2"},{"y,x,z"},{"x-y,-y,z"},{"-x,-x+y,z"}},

// [186] P 63 m c (P 6c -2c)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-x,-y,z+1/2"},{"y,-x+y,z+1/2"},{"x-y,x,z+1/2"},{"-y,-x,z"},
{"-x+y,y,z"},{"x,x-y,z"},{"y,x,z+1/2"},{"x-y,-y,z+1/2"},{"-x,-x+y,z+1/2"}},

// [187] P -6 m 2 (P -6 2)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"x,y,-z"},{"-y,x-y,-z"},{"-x+y,-x,-z"},{"-y,-x,z"},
{"-x+y,y,z"},{"x,x-y,z"},{"-y,-x,-z"},{"-x+y,y,-z"},{"x,x-y,-z"}},

// [188] P -6 c 2 (P -6c 2)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"x,y,-z+1/2"},{"-y,x-y,-z+1/2"},{"-x+y,-x,-z+1/2"},{"-y,-x,z+1/2"},
{"-x+y,y,z+1/2"},{"x,x-y,z+1/2"},{"-y,-x,-z"},{"-x+y,y,-z"},{"x,x-y,-z"}},

// [189] P -6 2 m (P -6 -2)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"x,y,-z"},{"-y,x-y,-z"},{"-x+y,-x,-z"},{"y,x,-z"},
{"x-y,-y,-z"},{"-x,-x+y,-z"},{"y,x,z"},{"x-y,-y,z"},{"-x,-x+y,z"}},

// [190] P -6 2 c (P -6c -2c)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"x,y,-z+1/2"},{"-y,x-y,-z+1/2"},{"-x+y,-x,-z+1/2"},{"y,x,-z"},
{"x-y,-y,-z"},{"-x,-x+y,-z"},{"y,x,z+1/2"},{"x-y,-y,z+1/2"},{"-x,-x+y,z+1/2"}},

// [191] P 6/m m m (-P 6 2)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-x,-y,z"},{"y,-x+y,z"},{"x-y,x,z"},{"y,x,-z"},
{"x-y,-y,-z"},{"-x,-x+y,-z"},{"-y,-x,-z"},{"-x+y,y,-z"},{"x,x-y,-z"},{"-x,-y,-z"},{"y,-x+y,-z"},
{"x-y,x,-z"},{"x,y,-z"},{"-y,x-y,-z"},{"-x+y,-x,-z"},{"-y,-x,z"},{"-x+y,y,z"},{"x,x-y,z"},
{"y,x,z"},{"x-y,-y,z"},{"-x,-x+y,z"}},

// [192] P 6/m c c (-P 6 2c)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-x,-y,z"},{"y,-x+y,z"},{"x-y,x,z"},{"y,x,-z+1/2"},
{"x-y,-y,-z+1/2"},{"-x,-x+y,-z+1/2"},{"-y,-x,-z+1/2"},{"-x+y,y,-z+1/2"},{"x,x-y,-z+1/2"},{"-x,-y,-z"},{"y,-x+y,-z"},
{"x-y,x,-z"},{"x,y,-z"},{"-y,x-y,-z"},{"-x+y,-x,-z"},{"-y,-x,z+1/2"},{"-x+y,y,z+1/2"},{"x,x-y,z+1/2"},
{"y,x,z+1/2"},{"x-y,-y,z+1/2"},{"-x,-x+y,z+1/2"}},

// [193] P 63/m c m (-P 6c 2)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-x,-y,z+1/2"},{"y,-x+y,z+1/2"},{"x-y,x,z+1/2"},{"y,x,-z+1/2"},
{"x-y,-y,-z+1/2"},{"-x,-x+y,-z+1/2"},{"-y,-x,-z"},{"-x+y,y,-z"},{"x,x-y,-z"},{"-x,-y,-z"},{"y,-x+y,-z"},
{"x-y,x,-z"},{"x,y,-z+1/2"},{"-y,x-y,-z+1/2"},{"-x+y,-x,-z+1/2"},{"-y,-x,z+1/2"},{"-x+y,y,z+1/2"},{"x,x-y,z+1/2"},
{"y,x,z"},{"x-y,-y,z"},{"-x,-x+y,z"}},

// [194] P 63/m m c (-P 6c 2c)
{{"x,y,z"},{"-y,x-y,z"},{"-x+y,-x,z"},{"-x,-y,z+1/2"},{"y,-x+y,z+1/2"},{"x-y,x,z+1/2"},{"y,x,-z"},
{"x-y,-y,-z"},{"-x,-x+y,-z"},{"-y,-x,-z+1/2"},{"-x+y,y,-z+1/2"},{"x,x-y,-z+1/2"},{"-x,-y,-z"},{"y,-x+y,-z"},
{"x-y,x,-z"},{"x,y,-z+1/2"},{"-y,x-y,-z+1/2"},{"-x+y,-x,-z+1/2"},{"-y,-x,z"},{"-x+y,y,z"},{"x,x-y,z"},
{"y,x,z+1/2"},{"x-y,-y,z+1/2"},{"-x,-x+y,z+1/2"}},

    // CUBIC GROUPS
    // ============

// [195] P 2 3 (P 2 2 3)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"}},

// [196] F 2 3 (F 2 2 3)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"x,y+1/2,z+1/2"},{"-x,-y+1/2,z+1/2"},
{"-x,y+1/2,-z+1/2"},{"x,-y+1/2,-z+1/2"},{"z,x+1/2,y+1/2"},{"z,-x+1/2,-y+1/2"},{"-z,-x+1/2,y+1/2"},{"-z,x+1/2,-y+1/2"},{"y,z+1/2,x+1/2"},
{"-y,z+1/2,-x+1/2"},{"y,-z+1/2,-x+1/2"},{"-y,-z+1/2,x+1/2"},{"x+1/2,y,z+1/2"},{"-x+1/2,-y,z+1/2"},{"-x+1/2,y,-z+1/2"},{"x+1/2,-y,-z+1/2"},
{"z+1/2,x,y+1/2"},{"z+1/2,-x,-y+1/2"},{"-z+1/2,-x,y+1/2"},{"-z+1/2,x,-y+1/2"},{"y+1/2,z,x+1/2"},{"-y+1/2,z,-x+1/2"},{"y+1/2,-z,-x+1/2"},
{"-y+1/2,-z,x+1/2"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},{"z+1/2,x+1/2,y"},{"z+1/2,-x+1/2,-y"},
{"-z+1/2,-x+1/2,y"},{"-z+1/2,x+1/2,-y"},{"y+1/2,z+1/2,x"},{"-y+1/2,z+1/2,-x"},{"y+1/2,-z+1/2,-x"},{"-y+1/2,-z+1/2,x"}},

// [197] I 2 3 (I 2 2 3)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},
{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"z+1/2,x+1/2,y+1/2"},{"z+1/2,-x+1/2,-y+1/2"},{"-z+1/2,-x+1/2,y+1/2"},{"-z+1/2,x+1/2,-y+1/2"},{"y+1/2,z+1/2,x+1/2"},
{"-y+1/2,z+1/2,-x+1/2"},{"y+1/2,-z+1/2,-x+1/2"},{"-y+1/2,-z+1/2,x+1/2"}},

// [198] P 21 3 (P 2ac 2ab 3)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z"},{"z,x,y"},{"z+1/2,-x+1/2,-y"},{"-z+1/2,-x,y+1/2"},
{"-z,x+1/2,-y+1/2"},{"y,z,x"},{"-y,z+1/2,-x+1/2"},{"y+1/2,-z+1/2,-x"},{"-y+1/2,-z,x+1/2"}},

// [199] I 21 3 (I 2b 2c 3)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z"},{"z,x,y"},{"z+1/2,-x+1/2,-y"},{"-z+1/2,-x,y+1/2"},
{"-z,x+1/2,-y+1/2"},{"y,z,x"},{"-y,z+1/2,-x+1/2"},{"y+1/2,-z+1/2,-x"},{"-y+1/2,-z,x+1/2"},{"x+1/2,y+1/2,z+1/2"},{"-x,-y+1/2,z"},
{"-x+1/2,y,-z"},{"x,-y,-z+1/2"},{"z+1/2,x+1/2,y+1/2"},{"z,-x,-y+1/2"},{"-z,-x+1/2,y"},{"-z+1/2,x,-y"},{"y+1/2,z+1/2,x+1/2"},
{"-y+1/2,z,-x"},{"y,-z,-x+1/2"},{"-y,-z+1/2,x"}},

// [200] P m -3 (P 2 2 3 -1n)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"-x,-y,-z"},{"x,y,-z"},
{"x,-y,z"},{"-x,y,z"},{"-z,-x,-y"},{"-z,x,y"},{"z,x,-y"},{"z,-x,y"},{"-y,-z,-x"},
{"y,-z,x"},{"-y,z,x"},{"y,z,-x"}},

// [201] P n -3 Origin choise 1 (P 2 2 3 -1n)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},
{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"},{"-z+1/2,-x+1/2,-y+1/2"},{"-z+1/2,x+1/2,y+1/2"},{"z+1/2,x+1/2,-y+1/2"},{"z+1/2,-x+1/2,y+1/2"},{"-y+1/2,-z+1/2,-x+1/2"},
{"y+1/2,-z+1/2,x+1/2"},{"-y+1/2,z+1/2,x+1/2"},{"y+1/2,z+1/2,-x+1/2"}},

// [201] P n -3 Origin choise 2 (-P 2ab 2bc 3)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,y,-z+1/2"},{"x,-y+1/2,-z+1/2"},{"z,x,y"},{"z,-x+1/2,-y+1/2"},{"-z+1/2,-x+1/2,y"},
{"-z+1/2,x,-y+1/2"},{"y,z,x"},{"-y+1/2,z,-x+1/2"},{"y,-z+1/2,-x+1/2"},{"-y+1/2,-z+1/2,x"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"},
{"x+1/2,-y,z+1/2"},{"-x,y+1/2,z+1/2"},{"-z,-x,-y"},{"-z,x+1/2,y+1/2"},{"z+1/2,x+1/2,-y"},{"z+1/2,-x,y+1/2"},{"-y,-z,-x"},
{"y+1/2,-z,x+1/2"},{"-y,z+1/2,x+1/2"},{"y+1/2,z+1/2,-x"}},

// [202] F m -3 (-F 2 2 3)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"-x,-y,-z"},{"x,y,-z"},
{"x,-y,z"},{"-x,y,z"},{"-z,-x,-y"},{"-z,x,y"},{"z,x,-y"},{"z,-x,y"},{"-y,-z,-x"},
{"y,-z,x"},{"-y,z,x"},{"y,z,-x"},{"x,y+1/2,z+1/2"},{"-x,-y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x,-y+1/2,-z+1/2"},
{"z,x+1/2,y+1/2"},{"z,-x+1/2,-y+1/2"},{"-z,-x+1/2,y+1/2"},{"-z,x+1/2,-y+1/2"},{"y,z+1/2,x+1/2"},{"-y,z+1/2,-x+1/2"},{"y,-z+1/2,-x+1/2"},
{"-y,-z+1/2,x+1/2"},{"-x,-y+1/2,-z+1/2"},{"x,y+1/2,-z+1/2"},{"x,-y+1/2,z+1/2"},{"-x,y+1/2,z+1/2"},{"-z,-x+1/2,-y+1/2"},{"-z,x+1/2,y+1/2"},
{"z,x+1/2,-y+1/2"},{"z,-x+1/2,y+1/2"},{"-y,-z+1/2,-x+1/2"},{"y,-z+1/2,x+1/2"},{"-y,z+1/2,x+1/2"},{"y,z+1/2,-x+1/2"},{"x+1/2,y,z+1/2"},
{"-x+1/2,-y,z+1/2"},{"-x+1/2,y,-z+1/2"},{"x+1/2,-y,-z+1/2"},{"z+1/2,x,y+1/2"},{"z+1/2,-x,-y+1/2"},{"-z+1/2,-x,y+1/2"},{"-z+1/2,x,-y+1/2"},
{"y+1/2,z,x+1/2"},{"-y+1/2,z,-x+1/2"},{"y+1/2,-z,-x+1/2"},{"-y+1/2,-z,x+1/2"},{"-x+1/2,-y,-z+1/2"},{"x+1/2,y,-z+1/2"},{"x+1/2,-y,z+1/2"},
{"-x+1/2,y,z+1/2"},{"-z+1/2,-x,-y+1/2"},{"-z+1/2,x,y+1/2"},{"z+1/2,x,-y+1/2"},{"z+1/2,-x,y+1/2"},{"-y+1/2,-z,-x+1/2"},{"y+1/2,-z,x+1/2"},
{"-y+1/2,z,x+1/2"},{"y+1/2,z,-x+1/2"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},{"z+1/2,x+1/2,y"},
{"z+1/2,-x+1/2,-y"},{"-z+1/2,-x+1/2,y"},{"-z+1/2,x+1/2,-y"},{"y+1/2,z+1/2,x"},{"-y+1/2,z+1/2,-x"},{"y+1/2,-z+1/2,-x"},{"-y+1/2,-z+1/2,x"},
{"-x+1/2,-y+1/2,-z"},{"x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"},{"-z+1/2,-x+1/2,-y"},{"-z+1/2,x+1/2,y"},{"z+1/2,x+1/2,-y"},
{"z+1/2,-x+1/2,y"},{"-y+1/2,-z+1/2,-x"},{"y+1/2,-z+1/2,x"},{"-y+1/2,z+1/2,x"},{"y+1/2,z+1/2,-x"}},

// [203] F d -3 Origin choise 1 (F 2 2 3 -1d)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"-x+1/4,-y+1/4,-z+1/4"},{"x+1/4,y+1/4,-z+1/4"},
{"x+1/4,-y+1/4,z+1/4"},{"-x+1/4,y+1/4,z+1/4"},{"-z+1/4,-x+1/4,-y+1/4"},{"-z+1/4,x+1/4,y+1/4"},{"z+1/4,x+1/4,-y+1/4"},{"z+1/4,-x+1/4,y+1/4"},{"-y+1/4,-z+1/4,-x+1/4"},
{"y+1/4,-z+1/4,x+1/4"},{"-y+1/4,z+1/4,x+1/4"},{"y+1/4,z+1/4,-x+1/4"},{"x,y+1/2,z+1/2"},{"-x,-y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x,-y+1/2,-z+1/2"},
{"z,x+1/2,y+1/2"},{"z,-x+1/2,-y+1/2"},{"-z,-x+1/2,y+1/2"},{"-z,x+1/2,-y+1/2"},{"y,z+1/2,x+1/2"},{"-y,z+1/2,-x+1/2"},{"y,-z+1/2,-x+1/2"},
{"-y,-z+1/2,x+1/2"},{"-x+1/4,-y+3/4,-z+3/4"},{"x+1/4,y+3/4,-z+3/4"},{"x+1/4,-y+3/4,z+3/4"},{"-x+1/4,y+3/4,z+3/4"},{"-z+1/4,-x+3/4,-y+3/4"},{"-z+1/4,x+3/4,y+3/4"},
{"z+1/4,x+3/4,-y+3/4"},{"z+1/4,-x+3/4,y+3/4"},{"-y+1/4,-z+3/4,-x+3/4"},{"y+1/4,-z+3/4,x+3/4"},{"-y+1/4,z+3/4,x+3/4"},{"y+1/4,z+3/4,-x+3/4"},{"x+1/2,y,z+1/2"},
{"-x+1/2,-y,z+1/2"},{"-x+1/2,y,-z+1/2"},{"x+1/2,-y,-z+1/2"},{"z+1/2,x,y+1/2"},{"z+1/2,-x,-y+1/2"},{"-z+1/2,-x,y+1/2"},{"-z+1/2,x,-y+1/2"},
{"y+1/2,z,x+1/2"},{"-y+1/2,z,-x+1/2"},{"y+1/2,-z,-x+1/2"},{"-y+1/2,-z,x+1/2"},{"-x+3/4,-y+1/4,-z+3/4"},{"x+3/4,y+1/4,-z+3/4"},{"x+3/4,-y+1/4,z+3/4"},
{"-x+3/4,y+1/4,z+3/4"},{"-z+3/4,-x+1/4,-y+3/4"},{"-z+3/4,x+1/4,y+3/4"},{"z+3/4,x+1/4,-y+3/4"},{"z+3/4,-x+1/4,y+3/4"},{"-y+3/4,-z+1/4,-x+3/4"},{"y+3/4,-z+1/4,x+3/4"},
{"-y+3/4,z+1/4,x+3/4"},{"y+3/4,z+1/4,-x+3/4"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},{"z+1/2,x+1/2,y"},
{"z+1/2,-x+1/2,-y"},{"-z+1/2,-x+1/2,y"},{"-z+1/2,x+1/2,-y"},{"y+1/2,z+1/2,x"},{"-y+1/2,z+1/2,-x"},{"y+1/2,-z+1/2,-x"},{"-y+1/2,-z+1/2,x"},
{"-x+3/4,-y+3/4,-z+1/4"},{"x+3/4,y+3/4,-z+1/4"},{"x+3/4,-y+3/4,z+1/4"},{"-x+3/4,y+3/4,z+1/4"},{"-z+3/4,-x+3/4,-y+1/4"},{"-z+3/4,x+3/4,y+1/4"},{"z+3/4,x+3/4,-y+1/4"},
{"z+3/4,-x+3/4,y+1/4"},{"-y+3/4,-z+3/4,-x+1/4"},{"y+3/4,-z+3/4,x+1/4"},{"-y+3/4,z+3/4,x+1/4"},{"y+3/4,z+3/4,-x+1/4"}},

// [203] F d -3 Origin choise 2 (-F 2uv 2vw 3)
{{"x,y,z"},{"-x+1/4,-y+1/4,z"},{"-x+1/4,y,-z+1/4"},{"x,-y+1/4,-z+1/4"},{"z,x,y"},{"z,-x+1/4,-y+1/4"},{"-z+1/4,-x+1/4,y"},
{"-z+1/4,x,-y+1/4"},{"y,z,x"},{"-y+1/4,z,-x+1/4"},{"y,-z+1/4,-x+1/4"},{"-y+1/4,-z+1/4,x"},{"-x,-y,-z"},{"x+3/4,y+3/4,-z"},
{"x+3/4,-y,z+3/4"},{"-x,y+3/4,z+3/4"},{"-z,-x,-y"},{"-z,x+3/4,y+3/4"},{"z+3/4,x+3/4,-y"},{"z+3/4,-x,y+3/4"},{"-y,-z,-x"},
{"y+3/4,-z,x+3/4"},{"-y,z+3/4,x+3/4"},{"y+3/4,z+3/4,-x"},{"x,y+1/2,z+1/2"},{"-x+1/4,-y+3/4,z+1/2"},{"-x+1/4,y+1/2,-z+3/4"},{"x,-y+3/4,-z+3/4"},
{"z,x+1/2,y+1/2"},{"z,-x+3/4,-y+3/4"},{"-z+1/4,-x+3/4,y+1/2"},{"-z+1/4,x+1/2,-y+3/4"},{"y,z+1/2,x+1/2"},{"-y+1/4,z+1/2,-x+3/4"},{"y,-z+3/4,-x+3/4"},
{"-y+1/4,-z+3/4,x+1/2"},{"-x,-y+1/2,-z+1/2"},{"x+3/4,y+1/4,-z+1/2"},{"x+3/4,-y+1/2,z+1/4"},{"-x,y+1/4,z+1/4"},{"-z,-x+1/2,-y+1/2"},{"-z,x+1/4,y+1/4"},
{"z+3/4,x+1/4,-y+1/2"},{"z+3/4,-x+1/2,y+1/4"},{"-y,-z+1/2,-x+1/2"},{"y+3/4,-z+1/2,x+1/4"},{"-y,z+1/4,x+1/4"},{"y+3/4,z+1/4,-x+1/2"},{"x+1/2,y,z+1/2"},
{"-x+3/4,-y+1/4,z+1/2"},{"-x+3/4,y,-z+3/4"},{"x+1/2,-y+1/4,-z+3/4"},{"z+1/2,x,y+1/2"},{"z+1/2,-x+1/4,-y+3/4"},{"-z+3/4,-x+1/4,y+1/2"},{"-z+3/4,x,-y+3/4"},
{"y+1/2,z,x+1/2"},{"-y+3/4,z,-x+3/4"},{"y+1/2,-z+1/4,-x+3/4"},{"-y+3/4,-z+1/4,x+1/2"},{"-x+1/2,-y,-z+1/2"},{"x+1/4,y+3/4,-z+1/2"},{"x+1/4,-y,z+1/4"},
{"-x+1/2,y+3/4,z+1/4"},{"-z+1/2,-x,-y+1/2"},{"-z+1/2,x+3/4,y+1/4"},{"z+1/4,x+3/4,-y+1/2"},{"z+1/4,-x,y+1/4"},{"-y+1/2,-z,-x+1/2"},{"y+1/4,-z,x+1/4"},
{"-y+1/2,z+3/4,x+1/4"},{"y+1/4,z+3/4,-x+1/2"},{"x+1/2,y+1/2,z"},{"-x+3/4,-y+3/4,z"},{"-x+3/4,y+1/2,-z+1/4"},{"x+1/2,-y+3/4,-z+1/4"},{"z+1/2,x+1/2,y"},
{"z+1/2,-x+3/4,-y+1/4"},{"-z+3/4,-x+3/4,y"},{"-z+3/4,x+1/2,-y+1/4"},{"y+1/2,z+1/2,x"},{"-y+3/4,z+1/2,-x+1/4"},{"y+1/2,-z+3/4,-x+1/4"},{"-y+3/4,-z+3/4,x"},
{"-x+1/2,-y+1/2,-z"},{"x+1/4,y+1/4,-z"},{"x+1/4,-y+1/2,z+3/4"},{"-x+1/2,y+1/4,z+3/4"},{"-z+1/2,-x+1/2,-y"},{"-z+1/2,x+1/4,y+3/4"},{"z+1/4,x+1/4,-y"},
{"z+1/4,-x+1/2,y+3/4"},{"-y+1/2,-z+1/2,-x"},{"y+1/4,-z+1/2,x+3/4"},{"-y+1/2,z+1/4,x+3/4"},{"y+1/4,z+1/4,-x"}},

// [204] I m -3 (-I 2 2 3)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"-x,-y,-z"},{"x,y,-z"},
{"x,-y,z"},{"-x,y,z"},{"-z,-x,-y"},{"-z,x,y"},{"z,x,-y"},{"z,-x,y"},{"-y,-z,-x"},
{"y,-z,x"},{"-y,z,x"},{"y,z,-x"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},
{"z+1/2,x+1/2,y+1/2"},{"z+1/2,-x+1/2,-y+1/2"},{"-z+1/2,-x+1/2,y+1/2"},{"-z+1/2,x+1/2,-y+1/2"},{"y+1/2,z+1/2,x+1/2"},{"-y+1/2,z+1/2,-x+1/2"},{"y+1/2,-z+1/2,-x+1/2"},
{"-y+1/2,-z+1/2,x+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"},{"-z+1/2,-x+1/2,-y+1/2"},{"-z+1/2,x+1/2,y+1/2"},
{"z+1/2,x+1/2,-y+1/2"},{"z+1/2,-x+1/2,y+1/2"},{"-y+1/2,-z+1/2,-x+1/2"},{"y+1/2,-z+1/2,x+1/2"},{"-y+1/2,z+1/2,x+1/2"},{"y+1/2,z+1/2,-x+1/2"}},

// [205] P a -3 (-P 2ac 2ab 3)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z"},{"z,x,y"},{"z+1/2,-x+1/2,-y"},{"-z+1/2,-x,y+1/2"},
{"-z,x+1/2,-y+1/2"},{"y,z,x"},{"-y,z+1/2,-x+1/2"},{"y+1/2,-z+1/2,-x"},{"-y+1/2,-z,x+1/2"},{"-x,-y,-z"},{"x+1/2,y,-z+1/2"},
{"x,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z"},{"-z,-x,-y"},{"-z+1/2,x+1/2,y"},{"z+1/2,x,-y+1/2"},{"z,-x+1/2,y+1/2"},{"-y,-z,-x"},
{"y,-z+1/2,x+1/2"},{"-y+1/2,z+1/2,x"},{"y+1/2,z,-x+1/2"}},

// [206] I a -3 (-I 2b 2c 3)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z"},{"z,x,y"},{"z+1/2,-x+1/2,-y"},{"-z+1/2,-x,y+1/2"},
{"-z,x+1/2,-y+1/2"},{"y,z,x"},{"-y,z+1/2,-x+1/2"},{"y+1/2,-z+1/2,-x"},{"-y+1/2,-z,x+1/2"},{"-x,-y,-z"},{"x+1/2,y,-z+1/2"},
{"x,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z"},{"-z,-x,-y"},{"-z+1/2,x+1/2,y"},{"z+1/2,x,-y+1/2"},{"z,-x+1/2,y+1/2"},{"-y,-z,-x"},
{"y,-z+1/2,x+1/2"},{"-y+1/2,z+1/2,x"},{"y+1/2,z,-x+1/2"},{"x+1/2,y+1/2,z+1/2"},{"-x,-y+1/2,z"},{"-x+1/2,y,-z"},{"x,-y,-z+1/2"},
{"z+1/2,x+1/2,y+1/2"},{"z,-x,-y+1/2"},{"-z,-x+1/2,y"},{"-z+1/2,x,-y"},{"y+1/2,z+1/2,x+1/2"},{"-y+1/2,z,-x"},{"y,-z,-x+1/2"},
{"-y,-z+1/2,x"},{"-x+1/2,-y+1/2,-z+1/2"},{"x,y+1/2,-z"},{"x+1/2,-y,z"},{"-x,y,z+1/2"},{"-z+1/2,-x+1/2,-y+1/2"},{"-z,x,y+1/2"},
{"z,x+1/2,-y"},{"z+1/2,-x,y"},{"-y+1/2,-z+1/2,-x+1/2"},{"y+1/2,-z,x"},{"-y,z,x+1/2"},{"y,z+1/2,-x"}},

// [207] P 4 3 2 (P 4 2 3)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"y,x,-z"},{"-y,-x,-z"},
{"y,-x,z"},{"-y,x,z"},{"x,z,-y"},{"-x,z,y"},{"-x,-z,-y"},{"x,-z,y"},{"z,y,-x"},
{"z,-y,x"},{"-z,y,x"},{"-z,-y,-x"}},

// [208] P 42 3 2 (P 4n 2 3)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"y+1/2,x+1/2,-z+1/2"},{"-y+1/2,-x+1/2,-z+1/2"},
{"y+1/2,-x+1/2,z+1/2"},{"-y+1/2,x+1/2,z+1/2"},{"x+1/2,z+1/2,-y+1/2"},{"-x+1/2,z+1/2,y+1/2"},{"-x+1/2,-z+1/2,-y+1/2"},{"x+1/2,-z+1/2,y+1/2"},{"z+1/2,y+1/2,-x+1/2"},
{"z+1/2,-y+1/2,x+1/2"},{"-z+1/2,y+1/2,x+1/2"},{"-z+1/2,-y+1/2,-x+1/2"}},

// [209] F 4 3 2 (F 4 2 3)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"y,x,-z"},{"-y,-x,-z"},
{"y,-x,z"},{"-y,x,z"},{"x,z,-y"},{"-x,z,y"},{"-x,-z,-y"},{"x,-z,y"},{"z,y,-x"},
{"z,-y,x"},{"-z,y,x"},{"-z,-y,-x"},{"x,y+1/2,z+1/2"},{"-x,-y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x,-y+1/2,-z+1/2"},
{"z,x+1/2,y+1/2"},{"z,-x+1/2,-y+1/2"},{"-z,-x+1/2,y+1/2"},{"-z,x+1/2,-y+1/2"},{"y,z+1/2,x+1/2"},{"-y,z+1/2,-x+1/2"},{"y,-z+1/2,-x+1/2"},
{"-y,-z+1/2,x+1/2"},{"y,x+1/2,-z+1/2"},{"-y,-x+1/2,-z+1/2"},{"y,-x+1/2,z+1/2"},{"-y,x+1/2,z+1/2"},{"x,z+1/2,-y+1/2"},{"-x,z+1/2,y+1/2"},
{"-x,-z+1/2,-y+1/2"},{"x,-z+1/2,y+1/2"},{"z,y+1/2,-x+1/2"},{"z,-y+1/2,x+1/2"},{"-z,y+1/2,x+1/2"},{"-z,-y+1/2,-x+1/2"},{"x+1/2,y,z+1/2"},
{"-x+1/2,-y,z+1/2"},{"-x+1/2,y,-z+1/2"},{"x+1/2,-y,-z+1/2"},{"z+1/2,x,y+1/2"},{"z+1/2,-x,-y+1/2"},{"-z+1/2,-x,y+1/2"},{"-z+1/2,x,-y+1/2"},
{"y+1/2,z,x+1/2"},{"-y+1/2,z,-x+1/2"},{"y+1/2,-z,-x+1/2"},{"-y+1/2,-z,x+1/2"},{"y+1/2,x,-z+1/2"},{"-y+1/2,-x,-z+1/2"},{"y+1/2,-x,z+1/2"},
{"-y+1/2,x,z+1/2"},{"x+1/2,z,-y+1/2"},{"-x+1/2,z,y+1/2"},{"-x+1/2,-z,-y+1/2"},{"x+1/2,-z,y+1/2"},{"z+1/2,y,-x+1/2"},{"z+1/2,-y,x+1/2"},
{"-z+1/2,y,x+1/2"},{"-z+1/2,-y,-x+1/2"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},{"z+1/2,x+1/2,y"},
{"z+1/2,-x+1/2,-y"},{"-z+1/2,-x+1/2,y"},{"-z+1/2,x+1/2,-y"},{"y+1/2,z+1/2,x"},{"-y+1/2,z+1/2,-x"},{"y+1/2,-z+1/2,-x"},{"-y+1/2,-z+1/2,x"},
{"y+1/2,x+1/2,-z"},{"-y+1/2,-x+1/2,-z"},{"y+1/2,-x+1/2,z"},{"-y+1/2,x+1/2,z"},{"x+1/2,z+1/2,-y"},{"-x+1/2,z+1/2,y"},{"-x+1/2,-z+1/2,-y"},
{"x+1/2,-z+1/2,y"},{"z+1/2,y+1/2,-x"},{"z+1/2,-y+1/2,x"},{"-z+1/2,y+1/2,x"},{"-z+1/2,-y+1/2,-x"}},

// [210] F 41 3 2 (F 4d 2 3)
{{"x,y,z"},{"-x,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y,-z+1/2"},{"z,x,y"},{"z+1/2,-x,-y+1/2"},{"-z,-x+1/2,y+1/2"},
{"-z+1/2,x+1/2,-y"},{"y,z,x"},{"-y+1/2,z+1/2,-x"},{"y+1/2,-z,-x+1/2"},{"-y,-z+1/2,x+1/2"},{"y+3/4,x+1/4,-z+3/4"},{"-y+1/4,-x+1/4,-z+1/4"},
{"y+1/4,-x+3/4,z+3/4"},{"-y+3/4,x+3/4,z+1/4"},{"x+3/4,z+1/4,-y+3/4"},{"-x+3/4,z+3/4,y+1/4"},{"-x+1/4,-z+1/4,-y+1/4"},{"x+1/4,-z+3/4,y+3/4"},{"z+3/4,y+1/4,-x+3/4"},
{"z+1/4,-y+3/4,x+3/4"},{"-z+3/4,y+3/4,x+1/4"},{"-z+1/4,-y+1/4,-x+1/4"},{"x,y+1/2,z+1/2"},{"-x,-y,z"},{"-x+1/2,y,-z+1/2"},{"x+1/2,-y+1/2,-z"},
{"z,x+1/2,y+1/2"},{"z+1/2,-x+1/2,-y"},{"-z,-x,y"},{"-z+1/2,x,-y+1/2"},{"y,z+1/2,x+1/2"},{"-y+1/2,z,-x+1/2"},{"y+1/2,-z+1/2,-x"},
{"-y,-z,x"},{"y+3/4,x+3/4,-z+1/4"},{"-y+1/4,-x+3/4,-z+3/4"},{"y+1/4,-x+1/4,z+1/4"},{"-y+3/4,x+1/4,z+3/4"},{"x+3/4,z+3/4,-y+1/4"},{"-x+3/4,z+1/4,y+3/4"},
{"-x+1/4,-z+3/4,-y+3/4"},{"x+1/4,-z+1/4,y+1/4"},{"z+3/4,y+3/4,-x+1/4"},{"z+1/4,-y+1/4,x+1/4"},{"-z+3/4,y+1/4,x+3/4"},{"-z+1/4,-y+3/4,-x+3/4"},{"x+1/2,y,z+1/2"},
{"-x+1/2,-y+1/2,z"},{"-x,y+1/2,-z+1/2"},{"x,-y,-z"},{"z+1/2,x,y+1/2"},{"z,-x,-y"},{"-z+1/2,-x+1/2,y"},{"-z,x+1/2,-y+1/2"},
{"y+1/2,z,x+1/2"},{"-y,z+1/2,-x+1/2"},{"y,-z,-x"},{"-y+1/2,-z+1/2,x"},{"y+1/4,x+1/4,-z+1/4"},{"-y+3/4,-x+1/4,-z+3/4"},{"y+3/4,-x+3/4,z+1/4"},
{"-y+1/4,x+3/4,z+3/4"},{"x+1/4,z+1/4,-y+1/4"},{"-x+1/4,z+3/4,y+3/4"},{"-x+3/4,-z+1/4,-y+3/4"},{"x+3/4,-z+3/4,y+1/4"},{"z+1/4,y+1/4,-x+1/4"},{"z+3/4,-y+3/4,x+1/4"},
{"-z+1/4,y+3/4,x+3/4"},{"-z+3/4,-y+1/4,-x+3/4"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y,z+1/2"},{"-x,y,-z"},{"x,-y+1/2,-z+1/2"},{"z+1/2,x+1/2,y"},
{"z,-x+1/2,-y+1/2"},{"-z+1/2,-x,y+1/2"},{"-z,x,-y"},{"y+1/2,z+1/2,x"},{"-y,z,-x"},{"y,-z+1/2,-x+1/2"},{"-y+1/2,-z,x+1/2"},
{"y+1/4,x+3/4,-z+3/4"},{"-y+3/4,-x+3/4,-z+1/4"},{"y+3/4,-x+1/4,z+3/4"},{"-y+1/4,x+1/4,z+1/4"},{"x+1/4,z+3/4,-y+3/4"},{"-x+1/4,z+1/4,y+1/4"},{"-x+3/4,-z+3/4,-y+1/4"},
{"x+3/4,-z+1/4,y+3/4"},{"z+1/4,y+3/4,-x+3/4"},{"z+3/4,-y+1/4,x+3/4"},{"-z+1/4,y+1/4,x+1/4"},{"-z+3/4,-y+3/4,-x+1/4"}},

// [211] I 4 3 2 (I 4 2 3)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"y,x,-z"},{"-y,-x,-z"},
{"y,-x,z"},{"-y,x,z"},{"x,z,-y"},{"-x,z,y"},{"-x,-z,-y"},{"x,-z,y"},{"z,y,-x"},
{"z,-y,x"},{"-z,y,x"},{"-z,-y,-x"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},
{"z+1/2,x+1/2,y+1/2"},{"z+1/2,-x+1/2,-y+1/2"},{"-z+1/2,-x+1/2,y+1/2"},{"-z+1/2,x+1/2,-y+1/2"},{"y+1/2,z+1/2,x+1/2"},{"-y+1/2,z+1/2,-x+1/2"},{"y+1/2,-z+1/2,-x+1/2"},
{"-y+1/2,-z+1/2,x+1/2"},{"y+1/2,x+1/2,-z+1/2"},{"-y+1/2,-x+1/2,-z+1/2"},{"y+1/2,-x+1/2,z+1/2"},{"-y+1/2,x+1/2,z+1/2"},{"x+1/2,z+1/2,-y+1/2"},{"-x+1/2,z+1/2,y+1/2"},
{"-x+1/2,-z+1/2,-y+1/2"},{"x+1/2,-z+1/2,y+1/2"},{"z+1/2,y+1/2,-x+1/2"},{"z+1/2,-y+1/2,x+1/2"},{"-z+1/2,y+1/2,x+1/2"},{"-z+1/2,-y+1/2,-x+1/2"}},

// [212] P 43 3 2 (P 4acd 2ab 3)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z"},{"z,x,y"},{"z+1/2,-x+1/2,-y"},{"-z+1/2,-x,y+1/2"},
{"-z,x+1/2,-y+1/2"},{"y,z,x"},{"-y,z+1/2,-x+1/2"},{"y+1/2,-z+1/2,-x"},{"-y+1/2,-z,x+1/2"},{"y+1/4,x+3/4,-z+3/4"},{"-y+1/4,-x+1/4,-z+1/4"},
{"y+3/4,-x+3/4,z+1/4"},{"-y+3/4,x+1/4,z+3/4"},{"x+1/4,z+3/4,-y+3/4"},{"-x+3/4,z+1/4,y+3/4"},{"-x+1/4,-z+1/4,-y+1/4"},{"x+3/4,-z+3/4,y+1/4"},{"z+1/4,y+3/4,-x+3/4"},
{"z+3/4,-y+3/4,x+1/4"},{"-z+3/4,y+1/4,x+3/4"},{"-z+1/4,-y+1/4,-x+1/4"}},

// [213] P 41 3 2 (P 4bd 2ab 3)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z"},{"z,x,y"},{"z+1/2,-x+1/2,-y"},{"-z+1/2,-x,y+1/2"},
{"-z,x+1/2,-y+1/2"},{"y,z,x"},{"-y,z+1/2,-x+1/2"},{"y+1/2,-z+1/2,-x"},{"-y+1/2,-z,x+1/2"},{"y+3/4,x+1/4,-z+1/4"},{"-y+3/4,-x+3/4,-z+3/4"},
{"y+1/4,-x+1/4,z+3/4"},{"-y+1/4,x+3/4,z+1/4"},{"x+3/4,z+1/4,-y+1/4"},{"-x+1/4,z+3/4,y+1/4"},{"-x+3/4,-z+3/4,-y+3/4"},{"x+1/4,-z+1/4,y+3/4"},{"z+3/4,y+1/4,-x+1/4"},
{"z+1/4,-y+1/4,x+3/4"},{"-z+1/4,y+3/4,x+1/4"},{"-z+3/4,-y+3/4,-x+3/4"}},

// [214] I 41 3 2 (I 4bd 2c 3)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z"},{"z,x,y"},{"z+1/2,-x+1/2,-y"},{"-z+1/2,-x,y+1/2"},
{"-z,x+1/2,-y+1/2"},{"y,z,x"},{"-y,z+1/2,-x+1/2"},{"y+1/2,-z+1/2,-x"},{"-y+1/2,-z,x+1/2"},{"y+3/4,x+1/4,-z+1/4"},{"-y+3/4,-x+3/4,-z+3/4"},
{"y+1/4,-x+1/4,z+3/4"},{"-y+1/4,x+3/4,z+1/4"},{"x+3/4,z+1/4,-y+1/4"},{"-x+1/4,z+3/4,y+1/4"},{"-x+3/4,-z+3/4,-y+3/4"},{"x+1/4,-z+1/4,y+3/4"},{"z+3/4,y+1/4,-x+1/4"},
{"z+1/4,-y+1/4,x+3/4"},{"-z+1/4,y+3/4,x+1/4"},{"-z+3/4,-y+3/4,-x+3/4"},{"x+1/2,y+1/2,z+1/2"},{"-x,-y+1/2,z"},{"-x+1/2,y,-z"},{"x,-y,-z+1/2"},
{"z+1/2,x+1/2,y+1/2"},{"z,-x,-y+1/2"},{"-z,-x+1/2,y"},{"-z+1/2,x,-y"},{"y+1/2,z+1/2,x+1/2"},{"-y+1/2,z,-x"},{"y,-z,-x+1/2"},
{"-y,-z+1/2,x"},{"y+1/4,x+3/4,-z+3/4"},{"-y+1/4,-x+1/4,-z+1/4"},{"y+3/4,-x+3/4,z+1/4"},{"-y+3/4,x+1/4,z+3/4"},{"x+1/4,z+3/4,-y+3/4"},{"-x+3/4,z+1/4,y+3/4"},
{"-x+1/4,-z+1/4,-y+1/4"},{"x+3/4,-z+3/4,y+1/4"},{"z+1/4,y+3/4,-x+3/4"},{"z+3/4,-y+3/4,x+1/4"},{"-z+3/4,y+1/4,x+3/4"},{"-z+1/4,-y+1/4,-x+1/4"}},

// [215] P -4 3 m (P -4 2 3)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"y,x,z"},{"-y,-x,z"},
{"y,-x,-z"},{"-y,x,-z"},{"x,z,y"},{"-x,z,-y"},{"-x,-z,y"},{"x,-z,-y"},{"z,y,x"},
{"z,-y,-x"},{"-z,y,-x"},{"-z,-y,x"}},

// [216] F -4 3 m (F -4 2 3)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"y,x,z"},{"-y,-x,z"},
{"y,-x,-z"},{"-y,x,-z"},{"x,z,y"},{"-x,z,-y"},{"-x,-z,y"},{"x,-z,-y"},{"z,y,x"},
{"z,-y,-x"},{"-z,y,-x"},{"-z,-y,x"},{"x,y+1/2,z+1/2"},{"-x,-y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x,-y+1/2,-z+1/2"},
{"z,x+1/2,y+1/2"},{"z,-x+1/2,-y+1/2"},{"-z,-x+1/2,y+1/2"},{"-z,x+1/2,-y+1/2"},{"y,z+1/2,x+1/2"},{"-y,z+1/2,-x+1/2"},{"y,-z+1/2,-x+1/2"},
{"-y,-z+1/2,x+1/2"},{"y,x+1/2,z+1/2"},{"-y,-x+1/2,z+1/2"},{"y,-x+1/2,-z+1/2"},{"-y,x+1/2,-z+1/2"},{"x,z+1/2,y+1/2"},{"-x,z+1/2,-y+1/2"},
{"-x,-z+1/2,y+1/2"},{"x,-z+1/2,-y+1/2"},{"z,y+1/2,x+1/2"},{"z,-y+1/2,-x+1/2"},{"-z,y+1/2,-x+1/2"},{"-z,-y+1/2,x+1/2"},{"x+1/2,y,z+1/2"},
{"-x+1/2,-y,z+1/2"},{"-x+1/2,y,-z+1/2"},{"x+1/2,-y,-z+1/2"},{"z+1/2,x,y+1/2"},{"z+1/2,-x,-y+1/2"},{"-z+1/2,-x,y+1/2"},{"-z+1/2,x,-y+1/2"},
{"y+1/2,z,x+1/2"},{"-y+1/2,z,-x+1/2"},{"y+1/2,-z,-x+1/2"},{"-y+1/2,-z,x+1/2"},{"y+1/2,x,z+1/2"},{"-y+1/2,-x,z+1/2"},{"y+1/2,-x,-z+1/2"},
{"-y+1/2,x,-z+1/2"},{"x+1/2,z,y+1/2"},{"-x+1/2,z,-y+1/2"},{"-x+1/2,-z,y+1/2"},{"x+1/2,-z,-y+1/2"},{"z+1/2,y,x+1/2"},{"z+1/2,-y,-x+1/2"},
{"-z+1/2,y,-x+1/2"},{"-z+1/2,-y,x+1/2"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},{"z+1/2,x+1/2,y"},
{"z+1/2,-x+1/2,-y"},{"-z+1/2,-x+1/2,y"},{"-z+1/2,x+1/2,-y"},{"y+1/2,z+1/2,x"},{"-y+1/2,z+1/2,-x"},{"y+1/2,-z+1/2,-x"},{"-y+1/2,-z+1/2,x"},
{"y+1/2,x+1/2,z"},{"-y+1/2,-x+1/2,z"},{"y+1/2,-x+1/2,-z"},{"-y+1/2,x+1/2,-z"},{"x+1/2,z+1/2,y"},{"-x+1/2,z+1/2,-y"},{"-x+1/2,-z+1/2,y"},
{"x+1/2,-z+1/2,-y"},{"z+1/2,y+1/2,x"},{"z+1/2,-y+1/2,-x"},{"-z+1/2,y+1/2,-x"},{"-z+1/2,-y+1/2,x"}},

// [217] I -4 3 m (I -4 2 3)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"y,x,z"},{"-y,-x,z"},
{"y,-x,-z"},{"-y,x,-z"},{"x,z,y"},{"-x,z,-y"},{"-x,-z,y"},{"x,-z,-y"},{"z,y,x"},
{"z,-y,-x"},{"-z,y,-x"},{"-z,-y,x"},{"x+1/2,y+1/2,z+1/2"},{"-x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},
{"z+1/2,x+1/2,y+1/2"},{"z+1/2,-x+1/2,-y+1/2"},{"-z+1/2,-x+1/2,y+1/2"},{"-z+1/2,x+1/2,-y+1/2"},{"y+1/2,z+1/2,x+1/2"},{"-y+1/2,z+1/2,-x+1/2"},{"y+1/2,-z+1/2,-x+1/2"},
{"-y+1/2,-z+1/2,x+1/2"},{"y+1/2,x+1/2,z+1/2"},{"-y+1/2,-x+1/2,z+1/2"},{"y+1/2,-x+1/2,-z+1/2"},{"-y+1/2,x+1/2,-z+1/2"},{"x+1/2,z+1/2,y+1/2"},{"-x+1/2,z+1/2,-y+1/2"},
{"-x+1/2,-z+1/2,y+1/2"},{"x+1/2,-z+1/2,-y+1/2"},{"z+1/2,y+1/2,x+1/2"},{"z+1/2,-y+1/2,-x+1/2"},{"-z+1/2,y+1/2,-x+1/2"},{"-z+1/2,-y+1/2,x+1/2"}},

// [218] P -4 3 n (P -4n 2 3)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"y+1/2,x+1/2,z+1/2"},{"-y+1/2,-x+1/2,z+1/2"},
{"y+1/2,-x+1/2,-z+1/2"},{"-y+1/2,x+1/2,-z+1/2"},{"x+1/2,z+1/2,y+1/2"},{"-x+1/2,z+1/2,-y+1/2"},{"-x+1/2,-z+1/2,y+1/2"},{"x+1/2,-z+1/2,-y+1/2"},{"z+1/2,y+1/2,x+1/2"},
{"z+1/2,-y+1/2,-x+1/2"},{"-z+1/2,y+1/2,-x+1/2"},{"-z+1/2,-y+1/2,x+1/2"}},

// [219] F -4 3 c (F -4a 2 3)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"y+1/2,x+1/2,z+1/2"},{"-y+1/2,-x+1/2,z+1/2"},
{"y+1/2,-x+1/2,-z+1/2"},{"-y+1/2,x+1/2,-z+1/2"},{"x+1/2,z+1/2,y+1/2"},{"-x+1/2,z+1/2,-y+1/2"},{"-x+1/2,-z+1/2,y+1/2"},{"x+1/2,-z+1/2,-y+1/2"},{"z+1/2,y+1/2,x+1/2"},
{"z+1/2,-y+1/2,-x+1/2"},{"-z+1/2,y+1/2,-x+1/2"},{"-z+1/2,-y+1/2,x+1/2"},{"x,y+1/2,z+1/2"},{"-x,-y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x,-y+1/2,-z+1/2"},
{"z,x+1/2,y+1/2"},{"z,-x+1/2,-y+1/2"},{"-z,-x+1/2,y+1/2"},{"-z,x+1/2,-y+1/2"},{"y,z+1/2,x+1/2"},{"-y,z+1/2,-x+1/2"},{"y,-z+1/2,-x+1/2"},
{"-y,-z+1/2,x+1/2"},{"y+1/2,x,z"},{"-y+1/2,-x,z"},{"y+1/2,-x,-z"},{"-y+1/2,x,-z"},{"x+1/2,z,y"},{"-x+1/2,z,-y"},
{"-x+1/2,-z,y"},{"x+1/2,-z,-y"},{"z+1/2,y,x"},{"z+1/2,-y,-x"},{"-z+1/2,y,-x"},{"-z+1/2,-y,x"},{"x+1/2,y,z+1/2"},
{"-x+1/2,-y,z+1/2"},{"-x+1/2,y,-z+1/2"},{"x+1/2,-y,-z+1/2"},{"z+1/2,x,y+1/2"},{"z+1/2,-x,-y+1/2"},{"-z+1/2,-x,y+1/2"},{"-z+1/2,x,-y+1/2"},
{"y+1/2,z,x+1/2"},{"-y+1/2,z,-x+1/2"},{"y+1/2,-z,-x+1/2"},{"-y+1/2,-z,x+1/2"},{"y,x+1/2,z"},{"-y,-x+1/2,z"},{"y,-x+1/2,-z"},
{"-y,x+1/2,-z"},{"x,z+1/2,y"},{"-x,z+1/2,-y"},{"-x,-z+1/2,y"},{"x,-z+1/2,-y"},{"z,y+1/2,x"},{"z,-y+1/2,-x"},
{"-z,y+1/2,-x"},{"-z,-y+1/2,x"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,-z"},{"z+1/2,x+1/2,y"},
{"z+1/2,-x+1/2,-y"},{"-z+1/2,-x+1/2,y"},{"-z+1/2,x+1/2,-y"},{"y+1/2,z+1/2,x"},{"-y+1/2,z+1/2,-x"},{"y+1/2,-z+1/2,-x"},{"-y+1/2,-z+1/2,x"},
{"y,x,z+1/2"},{"-y,-x,z+1/2"},{"y,-x,-z+1/2"},{"-y,x,-z+1/2"},{"x,z,y+1/2"},{"-x,z,-y+1/2"},{"-x,-z,y+1/2"},
{"x,-z,-y+1/2"},{"z,y,x+1/2"},{"z,-y,-x+1/2"},{"-z,y,-x+1/2"},{"-z,-y,x+1/2"}},

// [220] I -4 3 d (I -4bd 2c 3)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z"},{"z,x,y"},{"z+1/2,-x+1/2,-y"},{"-z+1/2,-x,y+1/2"},
{"-z,x+1/2,-y+1/2"},{"y,z,x"},{"-y,z+1/2,-x+1/2"},{"y+1/2,-z+1/2,-x"},{"-y+1/2,-z,x+1/2"},{"y+1/4,x+1/4,z+1/4"},{"-y+1/4,-x+3/4,z+3/4"},
{"y+3/4,-x+1/4,-z+3/4"},{"-y+3/4,x+3/4,-z+1/4"},{"x+1/4,z+1/4,y+1/4"},{"-x+3/4,z+3/4,-y+1/4"},{"-x+1/4,-z+3/4,y+3/4"},{"x+3/4,-z+1/4,-y+3/4"},{"z+1/4,y+1/4,x+1/4"},
{"z+3/4,-y+1/4,-x+3/4"},{"-z+3/4,y+3/4,-x+1/4"},{"-z+1/4,-y+3/4,x+3/4"},{"x+1/2,y+1/2,z+1/2"},{"-x,-y+1/2,z"},{"-x+1/2,y,-z"},{"x,-y,-z+1/2"},
{"z+1/2,x+1/2,y+1/2"},{"z,-x,-y+1/2"},{"-z,-x+1/2,y"},{"-z+1/2,x,-y"},{"y+1/2,z+1/2,x+1/2"},{"-y+1/2,z,-x"},{"y,-z,-x+1/2"},
{"-y,-z+1/2,x"},{"y+3/4,x+3/4,z+3/4"},{"-y+3/4,-x+1/4,z+1/4"},{"y+1/4,-x+3/4,-z+1/4"},{"-y+1/4,x+1/4,-z+3/4"},{"x+3/4,z+3/4,y+3/4"},{"-x+1/4,z+1/4,-y+3/4"},
{"-x+3/4,-z+1/4,y+1/4"},{"x+1/4,-z+3/4,-y+1/4"},{"z+3/4,y+3/4,x+3/4"},{"z+1/4,-y+3/4,-x+1/4"},{"-z+1/4,y+1/4,-x+3/4"},{"-z+3/4,-y+1/4,x+1/4"}},

// [221] P m -3 m (-P 4 2 3)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"y,x,-z"},{"-y,-x,-z"},
{"y,-x,z"},{"-y,x,z"},{"x,z,-y"},{"-x,z,y"},{"-x,-z,-y"},{"x,-z,y"},{"z,y,-x"},
{"z,-y,x"},{"-z,y,x"},{"-z,-y,-x"},{"-x,-y,-z"},{"x,y,-z"},{"x,-y,z"},{"-x,y,z"},
{"-z,-x,-y"},{"-z,x,y"},{"z,x,-y"},{"z,-x,y"},{"-y,-z,-x"},{"y,-z,x"},{"-y,z,x"},
{"y,z,-x"},{"-y,-x,z"},{"y,x,z"},{"-y,x,-z"},{"y,-x,-z"},{"-x,-z,y"},{"x,-z,-y"},
{"x,z,y"},{"-x,z,-y"},{"-z,-y,x"},{"-z,y,-x"},{"z,-y,-x"},{"z,y,x"}},

// [222] P n -3 Origin choise 1 (P 4 2 3 -1n)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"y,x,-z"},{"-y,-x,-z"},
{"y,-x,z"},{"-y,x,z"},{"x,z,-y"},{"-x,z,y"},{"-x,-z,-y"},{"x,-z,y"},{"z,y,-x"},
{"z,-y,x"},{"-z,y,x"},{"-z,-y,-x"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"},
{"-z+1/2,-x+1/2,-y+1/2"},{"-z+1/2,x+1/2,y+1/2"},{"z+1/2,x+1/2,-y+1/2"},{"z+1/2,-x+1/2,y+1/2"},{"-y+1/2,-z+1/2,-x+1/2"},{"y+1/2,-z+1/2,x+1/2"},{"-y+1/2,z+1/2,x+1/2"},
{"y+1/2,z+1/2,-x+1/2"},{"-y+1/2,-x+1/2,z+1/2"},{"y+1/2,x+1/2,z+1/2"},{"-y+1/2,x+1/2,-z+1/2"},{"y+1/2,-x+1/2,-z+1/2"},{"-x+1/2,-z+1/2,y+1/2"},{"x+1/2,-z+1/2,-y+1/2"},
{"x+1/2,z+1/2,y+1/2"},{"-x+1/2,z+1/2,-y+1/2"},{"-z+1/2,-y+1/2,x+1/2"},{"-z+1/2,y+1/2,-x+1/2"},{"z+1/2,-y+1/2,-x+1/2"},{"z+1/2,y+1/2,x+1/2"}},

// [222] P n -3 Origin choise 2 (-P 4a 2bc 3)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,y,-z+1/2"},{"x,-y+1/2,-z+1/2"},{"z,x,y"},{"z,-x+1/2,-y+1/2"},{"-z+1/2,-x+1/2,y"},
{"-z+1/2,x,-y+1/2"},{"y,z,x"},{"-y+1/2,z,-x+1/2"},{"y,-z+1/2,-x+1/2"},{"-y+1/2,-z+1/2,x"},{"y,x,-z+1/2"},{"-y+1/2,-x+1/2,-z+1/2"},
{"y,-x+1/2,z"},{"-y+1/2,x,z"},{"x,z,-y+1/2"},{"-x+1/2,z,y"},{"-x+1/2,-z+1/2,-y+1/2"},{"x,-z+1/2,y"},{"z,y,-x+1/2"},
{"z,-y+1/2,x"},{"-z+1/2,y,x"},{"-z+1/2,-y+1/2,-x+1/2"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"},{"x+1/2,-y,z+1/2"},{"-x,y+1/2,z+1/2"},
{"-z,-x,-y"},{"-z,x+1/2,y+1/2"},{"z+1/2,x+1/2,-y"},{"z+1/2,-x,y+1/2"},{"-y,-z,-x"},{"y+1/2,-z,x+1/2"},{"-y,z+1/2,x+1/2"},
{"y+1/2,z+1/2,-x"},{"-y,-x,z+1/2"},{"y+1/2,x+1/2,z+1/2"},{"-y,x+1/2,-z"},{"y+1/2,-x,-z"},{"-x,-z,y+1/2"},{"x+1/2,-z,-y"},
{"x+1/2,z+1/2,y+1/2"},{"-x,z+1/2,-y"},{"-z,-y,x+1/2"},{"-z,y+1/2,-x"},{"z+1/2,-y,-x"},{"z+1/2,y+1/2,x+1/2"}},

// [223] P m -3 n (-P 4n 2 3)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"y+1/2,x+1/2,-z+1/2"},{"-y+1/2,-x+1/2,-z+1/2"},
{"y+1/2,-x+1/2,z+1/2"},{"-y+1/2,x+1/2,z+1/2"},{"x+1/2,z+1/2,-y+1/2"},{"-x+1/2,z+1/2,y+1/2"},{"-x+1/2,-z+1/2,-y+1/2"},{"x+1/2,-z+1/2,y+1/2"},{"z+1/2,y+1/2,-x+1/2"},
{"z+1/2,-y+1/2,x+1/2"},{"-z+1/2,y+1/2,x+1/2"},{"-z+1/2,-y+1/2,-x+1/2"},{"-x,-y,-z"},{"x,y,-z"},{"x,-y,z"},{"-x,y,z"},
{"-z,-x,-y"},{"-z,x,y"},{"z,x,-y"},{"z,-x,y"},{"-y,-z,-x"},{"y,-z,x"},{"-y,z,x"},
{"y,z,-x"},{"-y+1/2,-x+1/2,z+1/2"},{"y+1/2,x+1/2,z+1/2"},{"-y+1/2,x+1/2,-z+1/2"},{"y+1/2,-x+1/2,-z+1/2"},{"-x+1/2,-z+1/2,y+1/2"},{"x+1/2,-z+1/2,-y+1/2"},
{"x+1/2,z+1/2,y+1/2"},{"-x+1/2,z+1/2,-y+1/2"},{"-z+1/2,-y+1/2,x+1/2"},{"-z+1/2,y+1/2,-x+1/2"},{"z+1/2,-y+1/2,-x+1/2"},{"z+1/2,y+1/2,x+1/2"}},

// [224] P n -3 m Origin choise 1 (P 4n 2 3 -1n)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"y+1/2,x+1/2,-z+1/2"},{"-y+1/2,-x+1/2,-z+1/2"},
{"y+1/2,-x+1/2,z+1/2"},{"-y+1/2,x+1/2,z+1/2"},{"x+1/2,z+1/2,-y+1/2"},{"-x+1/2,z+1/2,y+1/2"},{"-x+1/2,-z+1/2,-y+1/2"},{"x+1/2,-z+1/2,y+1/2"},{"z+1/2,y+1/2,-x+1/2"},
{"z+1/2,-y+1/2,x+1/2"},{"-z+1/2,y+1/2,x+1/2"},{"-z+1/2,-y+1/2,-x+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"},
{"-z+1/2,-x+1/2,-y+1/2"},{"-z+1/2,x+1/2,y+1/2"},{"z+1/2,x+1/2,-y+1/2"},{"z+1/2,-x+1/2,y+1/2"},{"-y+1/2,-z+1/2,-x+1/2"},{"y+1/2,-z+1/2,x+1/2"},{"-y+1/2,z+1/2,x+1/2"},
{"y+1/2,z+1/2,-x+1/2"},{"-y,-x,z"},{"y,x,z"},{"-y,x,-z"},{"y,-x,-z"},{"-x,-z,y"},{"x,-z,-y"},
{"x,z,y"},{"-x,z,-y"},{"-z,-y,x"},{"-z,y,-x"},{"z,-y,-x"},{"z,y,x"}},

// [224] P n -3 m Origin choise 2 (-P 4bc 2bc 3)
{{"x,y,z"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,y,-z+1/2"},{"x,-y+1/2,-z+1/2"},{"z,x,y"},{"z,-x+1/2,-y+1/2"},{"-z+1/2,-x+1/2,y"},
{"-z+1/2,x,-y+1/2"},{"y,z,x"},{"-y+1/2,z,-x+1/2"},{"y,-z+1/2,-x+1/2"},{"-y+1/2,-z+1/2,x"},{"y+1/2,x+1/2,-z"},{"-y,-x,-z"},
{"y+1/2,-x,z+1/2"},{"-y,x+1/2,z+1/2"},{"x+1/2,z+1/2,-y"},{"-x,z+1/2,y+1/2"},{"-x,-z,-y"},{"x+1/2,-z,y+1/2"},{"z+1/2,y+1/2,-x"},
{"z+1/2,-y,x+1/2"},{"-z,y+1/2,x+1/2"},{"-z,-y,-x"},{"-x,-y,-z"},{"x+1/2,y+1/2,-z"},{"x+1/2,-y,z+1/2"},{"-x,y+1/2,z+1/2"},
{"-z,-x,-y"},{"-z,x+1/2,y+1/2"},{"z+1/2,x+1/2,-y"},{"z+1/2,-x,y+1/2"},{"-y,-z,-x"},{"y+1/2,-z,x+1/2"},{"-y,z+1/2,x+1/2"},
{"y+1/2,z+1/2,-x"},{"-y+1/2,-x+1/2,z"},{"y,x,z"},{"-y+1/2,x,-z+1/2"},{"y,-x+1/2,-z+1/2"},{"-x+1/2,-z+1/2,y"},{"x,-z+1/2,-y+1/2"},
{"x,z,y"},{"-x+1/2,z,-y+1/2"},{"-z+1/2,-y+1/2,x"},{"-z+1/2,y,-x+1/2"},{"z,-y+1/2,-x+1/2"},{"z,y,x"}},

// [225] F m -3 m (-F 4 2 3)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"y,x,-z"},{"-y,-x,-z"},
{"y,-x,z"},{"-y,x,z"},{"x,z,-y"},{"-x,z,y"},{"-x,-z,-y"},{"x,-z,y"},{"z,y,-x"},
{"z,-y,x"},{"-z,y,x"},{"-z,-y,-x"},{"-x,-y,-z"},{"x,y,-z"},{"x,-y,z"},{"-x,y,z"},
{"-z,-x,-y"},{"-z,x,y"},{"z,x,-y"},{"z,-x,y"},{"-y,-z,-x"},{"y,-z,x"},{"-y,z,x"},
{"y,z,-x"},{"-y,-x,z"},{"y,x,z"},{"-y,x,-z"},{"y,-x,-z"},{"-x,-z,y"},{"x,-z,-y"},
{"x,z,y"},{"-x,z,-y"},{"-z,-y,x"},{"-z,y,-x"},{"z,-y,-x"},{"z,y,x"},{"x,y+1/2,z+1/2"},
{"-x,-y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x,-y+1/2,-z+1/2"},{"z,x+1/2,y+1/2"},{"z,-x+1/2,-y+1/2"},{"-z,-x+1/2,y+1/2"},{"-z,x+1/2,-y+1/2"},
{"y,z+1/2,x+1/2"},{"-y,z+1/2,-x+1/2"},{"y,-z+1/2,-x+1/2"},{"-y,-z+1/2,x+1/2"},{"y,x+1/2,-z+1/2"},{"-y,-x+1/2,-z+1/2"},{"y,-x+1/2,z+1/2"},
{"-y,x+1/2,z+1/2"},{"x,z+1/2,-y+1/2"},{"-x,z+1/2,y+1/2"},{"-x,-z+1/2,-y+1/2"},{"x,-z+1/2,y+1/2"},{"z,y+1/2,-x+1/2"},{"z,-y+1/2,x+1/2"},
{"-z,y+1/2,x+1/2"},{"-z,-y+1/2,-x+1/2"},{"-x,-y+1/2,-z+1/2"},{"x,y+1/2,-z+1/2"},{"x,-y+1/2,z+1/2"},{"-x,y+1/2,z+1/2"},{"-z,-x+1/2,-y+1/2"},
{"-z,x+1/2,y+1/2"},{"z,x+1/2,-y+1/2"},{"z,-x+1/2,y+1/2"},{"-y,-z+1/2,-x+1/2"},{"y,-z+1/2,x+1/2"},{"-y,z+1/2,x+1/2"},{"y,z+1/2,-x+1/2"},
{"-y,-x+1/2,z+1/2"},{"y,x+1/2,z+1/2"},{"-y,x+1/2,-z+1/2"},{"y,-x+1/2,-z+1/2"},{"-x,-z+1/2,y+1/2"},{"x,-z+1/2,-y+1/2"},{"x,z+1/2,y+1/2"},
{"-x,z+1/2,-y+1/2"},{"-z,-y+1/2,x+1/2"},{"-z,y+1/2,-x+1/2"},{"z,-y+1/2,-x+1/2"},{"z,y+1/2,x+1/2"},{"x+1/2,y,z+1/2"},{"-x+1/2,-y,z+1/2"},
{"-x+1/2,y,-z+1/2"},{"x+1/2,-y,-z+1/2"},{"z+1/2,x,y+1/2"},{"z+1/2,-x,-y+1/2"},{"-z+1/2,-x,y+1/2"},{"-z+1/2,x,-y+1/2"},{"y+1/2,z,x+1/2"},
{"-y+1/2,z,-x+1/2"},{"y+1/2,-z,-x+1/2"},{"-y+1/2,-z,x+1/2"},{"y+1/2,x,-z+1/2"},{"-y+1/2,-x,-z+1/2"},{"y+1/2,-x,z+1/2"},{"-y+1/2,x,z+1/2"},
{"x+1/2,z,-y+1/2"},{"-x+1/2,z,y+1/2"},{"-x+1/2,-z,-y+1/2"},{"x+1/2,-z,y+1/2"},{"z+1/2,y,-x+1/2"},{"z+1/2,-y,x+1/2"},{"-z+1/2,y,x+1/2"},
{"-z+1/2,-y,-x+1/2"},{"-x+1/2,-y,-z+1/2"},{"x+1/2,y,-z+1/2"},{"x+1/2,-y,z+1/2"},{"-x+1/2,y,z+1/2"},{"-z+1/2,-x,-y+1/2"},{"-z+1/2,x,y+1/2"},
{"z+1/2,x,-y+1/2"},{"z+1/2,-x,y+1/2"},{"-y+1/2,-z,-x+1/2"},{"y+1/2,-z,x+1/2"},{"-y+1/2,z,x+1/2"},{"y+1/2,z,-x+1/2"},{"-y+1/2,-x,z+1/2"},
{"y+1/2,x,z+1/2"},{"-y+1/2,x,-z+1/2"},{"y+1/2,-x,-z+1/2"},{"-x+1/2,-z,y+1/2"},{"x+1/2,-z,-y+1/2"},{"x+1/2,z,y+1/2"},{"-x+1/2,z,-y+1/2"},
{"-z+1/2,-y,x+1/2"},{"-z+1/2,y,-x+1/2"},{"z+1/2,-y,-x+1/2"},{"z+1/2,y,x+1/2"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,-z"},
{"x+1/2,-y+1/2,-z"},{"z+1/2,x+1/2,y"},{"z+1/2,-x+1/2,-y"},{"-z+1/2,-x+1/2,y"},{"-z+1/2,x+1/2,-y"},{"y+1/2,z+1/2,x"},{"-y+1/2,z+1/2,-x"},
{"y+1/2,-z+1/2,-x"},{"-y+1/2,-z+1/2,x"},{"y+1/2,x+1/2,-z"},{"-y+1/2,-x+1/2,-z"},{"y+1/2,-x+1/2,z"},{"-y+1/2,x+1/2,z"},{"x+1/2,z+1/2,-y"},
{"-x+1/2,z+1/2,y"},{"-x+1/2,-z+1/2,-y"},{"x+1/2,-z+1/2,y"},{"z+1/2,y+1/2,-x"},{"z+1/2,-y+1/2,x"},{"-z+1/2,y+1/2,x"},{"-z+1/2,-y+1/2,-x"},
{"-x+1/2,-y+1/2,-z"},{"x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"},{"-z+1/2,-x+1/2,-y"},{"-z+1/2,x+1/2,y"},{"z+1/2,x+1/2,-y"},
{"z+1/2,-x+1/2,y"},{"-y+1/2,-z+1/2,-x"},{"y+1/2,-z+1/2,x"},{"-y+1/2,z+1/2,x"},{"y+1/2,z+1/2,-x"},{"-y+1/2,-x+1/2,z"},{"y+1/2,x+1/2,z"},
{"-y+1/2,x+1/2,-z"},{"y+1/2,-x+1/2,-z"},{"-x+1/2,-z+1/2,y"},{"x+1/2,-z+1/2,-y"},{"x+1/2,z+1/2,y"},{"-x+1/2,z+1/2,-y"},{"-z+1/2,-y+1/2,x"},
{"-z+1/2,y+1/2,-x"},{"z+1/2,-y+1/2,-x"},{"z+1/2,y+1/2,x"}},

// [226] F m -3 c (-F 4a 2 3)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"y+1/2,x+1/2,-z+1/2"},{"-y+1/2,-x+1/2,-z+1/2"},
{"y+1/2,-x+1/2,z+1/2"},{"-y+1/2,x+1/2,z+1/2"},{"x+1/2,z+1/2,-y+1/2"},{"-x+1/2,z+1/2,y+1/2"},{"-x+1/2,-z+1/2,-y+1/2"},{"x+1/2,-z+1/2,y+1/2"},{"z+1/2,y+1/2,-x+1/2"},
{"z+1/2,-y+1/2,x+1/2"},{"-z+1/2,y+1/2,x+1/2"},{"-z+1/2,-y+1/2,-x+1/2"},{"-x,-y,-z"},{"x,y,-z"},{"x,-y,z"},{"-x,y,z"},
{"-z,-x,-y"},{"-z,x,y"},{"z,x,-y"},{"z,-x,y"},{"-y,-z,-x"},{"y,-z,x"},{"-y,z,x"},
{"y,z,-x"},{"-y+1/2,-x+1/2,z+1/2"},{"y+1/2,x+1/2,z+1/2"},{"-y+1/2,x+1/2,-z+1/2"},{"y+1/2,-x+1/2,-z+1/2"},{"-x+1/2,-z+1/2,y+1/2"},{"x+1/2,-z+1/2,-y+1/2"},
{"x+1/2,z+1/2,y+1/2"},{"-x+1/2,z+1/2,-y+1/2"},{"-z+1/2,-y+1/2,x+1/2"},{"-z+1/2,y+1/2,-x+1/2"},{"z+1/2,-y+1/2,-x+1/2"},{"z+1/2,y+1/2,x+1/2"},{"x,y+1/2,z+1/2"},
{"-x,-y+1/2,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x,-y+1/2,-z+1/2"},{"z,x+1/2,y+1/2"},{"z,-x+1/2,-y+1/2"},{"-z,-x+1/2,y+1/2"},{"-z,x+1/2,-y+1/2"},
{"y,z+1/2,x+1/2"},{"-y,z+1/2,-x+1/2"},{"y,-z+1/2,-x+1/2"},{"-y,-z+1/2,x+1/2"},{"y+1/2,x,-z"},{"-y+1/2,-x,-z"},{"y+1/2,-x,z"},
{"-y+1/2,x,z"},{"x+1/2,z,-y"},{"-x+1/2,z,y"},{"-x+1/2,-z,-y"},{"x+1/2,-z,y"},{"z+1/2,y,-x"},{"z+1/2,-y,x"},
{"-z+1/2,y,x"},{"-z+1/2,-y,-x"},{"-x,-y+1/2,-z+1/2"},{"x,y+1/2,-z+1/2"},{"x,-y+1/2,z+1/2"},{"-x,y+1/2,z+1/2"},{"-z,-x+1/2,-y+1/2"},
{"-z,x+1/2,y+1/2"},{"z,x+1/2,-y+1/2"},{"z,-x+1/2,y+1/2"},{"-y,-z+1/2,-x+1/2"},{"y,-z+1/2,x+1/2"},{"-y,z+1/2,x+1/2"},{"y,z+1/2,-x+1/2"},
{"-y+1/2,-x,z"},{"y+1/2,x,z"},{"-y+1/2,x,-z"},{"y+1/2,-x,-z"},{"-x+1/2,-z,y"},{"x+1/2,-z,-y"},{"x+1/2,z,y"},
{"-x+1/2,z,-y"},{"-z+1/2,-y,x"},{"-z+1/2,y,-x"},{"z+1/2,-y,-x"},{"z+1/2,y,x"},{"x+1/2,y,z+1/2"},{"-x+1/2,-y,z+1/2"},
{"-x+1/2,y,-z+1/2"},{"x+1/2,-y,-z+1/2"},{"z+1/2,x,y+1/2"},{"z+1/2,-x,-y+1/2"},{"-z+1/2,-x,y+1/2"},{"-z+1/2,x,-y+1/2"},{"y+1/2,z,x+1/2"},
{"-y+1/2,z,-x+1/2"},{"y+1/2,-z,-x+1/2"},{"-y+1/2,-z,x+1/2"},{"y,x+1/2,-z"},{"-y,-x+1/2,-z"},{"y,-x+1/2,z"},{"-y,x+1/2,z"},
{"x,z+1/2,-y"},{"-x,z+1/2,y"},{"-x,-z+1/2,-y"},{"x,-z+1/2,y"},{"z,y+1/2,-x"},{"z,-y+1/2,x"},{"-z,y+1/2,x"},
{"-z,-y+1/2,-x"},{"-x+1/2,-y,-z+1/2"},{"x+1/2,y,-z+1/2"},{"x+1/2,-y,z+1/2"},{"-x+1/2,y,z+1/2"},{"-z+1/2,-x,-y+1/2"},{"-z+1/2,x,y+1/2"},
{"z+1/2,x,-y+1/2"},{"z+1/2,-x,y+1/2"},{"-y+1/2,-z,-x+1/2"},{"y+1/2,-z,x+1/2"},{"-y+1/2,z,x+1/2"},{"y+1/2,z,-x+1/2"},{"-y,-x+1/2,z"},
{"y,x+1/2,z"},{"-y,x+1/2,-z"},{"y,-x+1/2,-z"},{"-x,-z+1/2,y"},{"x,-z+1/2,-y"},{"x,z+1/2,y"},{"-x,z+1/2,-y"},
{"-z,-y+1/2,x"},{"-z,y+1/2,-x"},{"z,-y+1/2,-x"},{"z,y+1/2,x"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,-z"},
{"x+1/2,-y+1/2,-z"},{"z+1/2,x+1/2,y"},{"z+1/2,-x+1/2,-y"},{"-z+1/2,-x+1/2,y"},{"-z+1/2,x+1/2,-y"},{"y+1/2,z+1/2,x"},{"-y+1/2,z+1/2,-x"},
{"y+1/2,-z+1/2,-x"},{"-y+1/2,-z+1/2,x"},{"y,x,-z+1/2"},{"-y,-x,-z+1/2"},{"y,-x,z+1/2"},{"-y,x,z+1/2"},{"x,z,-y+1/2"},
{"-x,z,y+1/2"},{"-x,-z,-y+1/2"},{"x,-z,y+1/2"},{"z,y,-x+1/2"},{"z,-y,x+1/2"},{"-z,y,x+1/2"},{"-z,-y,-x+1/2"},
{"-x+1/2,-y+1/2,-z"},{"x+1/2,y+1/2,-z"},{"x+1/2,-y+1/2,z"},{"-x+1/2,y+1/2,z"},{"-z+1/2,-x+1/2,-y"},{"-z+1/2,x+1/2,y"},{"z+1/2,x+1/2,-y"},
{"z+1/2,-x+1/2,y"},{"-y+1/2,-z+1/2,-x"},{"y+1/2,-z+1/2,x"},{"-y+1/2,z+1/2,x"},{"y+1/2,z+1/2,-x"},{"-y,-x,z+1/2"},{"y,x,z+1/2"},
{"-y,x,-z+1/2"},{"y,-x,-z+1/2"},{"-x,-z,y+1/2"},{"x,-z,-y+1/2"},{"x,z,y+1/2"},{"-x,z,-y+1/2"},{"-z,-y,x+1/2"},
{"-z,y,-x+1/2"},{"z,-y,-x+1/2"},{"z,y,x+1/2"}},

// [227] F d -3 m Origin choice 1 (F 4d 2 3 -1d)
{{"x,y,z"},{"-x,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y,-z+1/2"},{"z,x,y"},{"z+1/2,-x,-y+1/2"},{"-z,-x+1/2,y+1/2"},
{"-z+1/2,x+1/2,-y"},{"y,z,x"},{"-y+1/2,z+1/2,-x"},{"y+1/2,-z,-x+1/2"},{"-y,-z+1/2,x+1/2"},{"y+3/4,x+1/4,-z+3/4"},{"-y+1/4,-x+1/4,-z+1/4"},
{"y+1/4,-x+3/4,z+3/4"},{"-y+3/4,x+3/4,z+1/4"},{"x+3/4,z+1/4,-y+3/4"},{"-x+3/4,z+3/4,y+1/4"},{"-x+1/4,-z+1/4,-y+1/4"},{"x+1/4,-z+3/4,y+3/4"},{"z+3/4,y+1/4,-x+3/4"},
{"z+1/4,-y+3/4,x+3/4"},{"-z+3/4,y+3/4,x+1/4"},{"-z+1/4,-y+1/4,-x+1/4"},{"-x+1/4,-y+1/4,-z+1/4"},{"x+1/4,y+3/4,-z+3/4"},{"x+3/4,-y+3/4,z+1/4"},{"-x+3/4,y+1/4,z+3/4"},
{"-z+1/4,-x+1/4,-y+1/4"},{"-z+3/4,x+1/4,y+3/4"},{"z+1/4,x+3/4,-y+3/4"},{"z+3/4,-x+3/4,y+1/4"},{"-y+1/4,-z+1/4,-x+1/4"},{"y+3/4,-z+3/4,x+1/4"},{"-y+3/4,z+1/4,x+3/4"},
{"y+1/4,z+3/4,-x+3/4"},{"-y+1/2,-x,z+1/2"},{"y,x,z"},{"-y,x+1/2,-z+1/2"},{"y+1/2,-x+1/2,-z"},{"-x+1/2,-z,y+1/2"},{"x+1/2,-z+1/2,-y"},
{"x,z,y"},{"-x,z+1/2,-y+1/2"},{"-z+1/2,-y,x+1/2"},{"-z,y+1/2,-x+1/2"},{"z+1/2,-y+1/2,-x"},{"z,y,x"},{"x,y+1/2,z+1/2"},
{"-x,-y,z"},{"-x+1/2,y,-z+1/2"},{"x+1/2,-y+1/2,-z"},{"z,x+1/2,y+1/2"},{"z+1/2,-x+1/2,-y"},{"-z,-x,y"},{"-z+1/2,x,-y+1/2"},
{"y,z+1/2,x+1/2"},{"-y+1/2,z,-x+1/2"},{"y+1/2,-z+1/2,-x"},{"-y,-z,x"},{"y+3/4,x+3/4,-z+1/4"},{"-y+1/4,-x+3/4,-z+3/4"},{"y+1/4,-x+1/4,z+1/4"},
{"-y+3/4,x+1/4,z+3/4"},{"x+3/4,z+3/4,-y+1/4"},{"-x+3/4,z+1/4,y+3/4"},{"-x+1/4,-z+3/4,-y+3/4"},{"x+1/4,-z+1/4,y+1/4"},{"z+3/4,y+3/4,-x+1/4"},{"z+1/4,-y+1/4,x+1/4"},
{"-z+3/4,y+1/4,x+3/4"},{"-z+1/4,-y+3/4,-x+3/4"},{"-x+1/4,-y+3/4,-z+3/4"},{"x+1/4,y+1/4,-z+1/4"},{"x+3/4,-y+1/4,z+3/4"},{"-x+3/4,y+3/4,z+1/4"},{"-z+1/4,-x+3/4,-y+3/4"},
{"-z+3/4,x+3/4,y+1/4"},{"z+1/4,x+1/4,-y+1/4"},{"z+3/4,-x+1/4,y+3/4"},{"-y+1/4,-z+3/4,-x+3/4"},{"y+3/4,-z+1/4,x+3/4"},{"-y+3/4,z+3/4,x+1/4"},{"y+1/4,z+1/4,-x+1/4"},
{"-y+1/2,-x+1/2,z"},{"y,x+1/2,z+1/2"},{"-y,x,-z"},{"y+1/2,-x,-z+1/2"},{"-x+1/2,-z+1/2,y"},{"x+1/2,-z,-y+1/2"},{"x,z+1/2,y+1/2"},
{"-x,z,-y"},{"-z+1/2,-y+1/2,x"},{"-z,y,-x"},{"z+1/2,-y,-x+1/2"},{"z,y+1/2,x+1/2"},{"x+1/2,y,z+1/2"},{"-x+1/2,-y+1/2,z"},
{"-x,y+1/2,-z+1/2"},{"x,-y,-z"},{"z+1/2,x,y+1/2"},{"z,-x,-y"},{"-z+1/2,-x+1/2,y"},{"-z,x+1/2,-y+1/2"},{"y+1/2,z,x+1/2"},
{"-y,z+1/2,-x+1/2"},{"y,-z,-x"},{"-y+1/2,-z+1/2,x"},{"y+1/4,x+1/4,-z+1/4"},{"-y+3/4,-x+1/4,-z+3/4"},{"y+3/4,-x+3/4,z+1/4"},{"-y+1/4,x+3/4,z+3/4"},
{"x+1/4,z+1/4,-y+1/4"},{"-x+1/4,z+3/4,y+3/4"},{"-x+3/4,-z+1/4,-y+3/4"},{"x+3/4,-z+3/4,y+1/4"},{"z+1/4,y+1/4,-x+1/4"},{"z+3/4,-y+3/4,x+1/4"},{"-z+1/4,y+3/4,x+3/4"},
{"-z+3/4,-y+1/4,-x+3/4"},{"-x+3/4,-y+1/4,-z+3/4"},{"x+3/4,y+3/4,-z+1/4"},{"x+1/4,-y+3/4,z+3/4"},{"-x+1/4,y+1/4,z+1/4"},{"-z+3/4,-x+1/4,-y+3/4"},{"-z+1/4,x+1/4,y+1/4"},
{"z+3/4,x+3/4,-y+1/4"},{"z+1/4,-x+3/4,y+3/4"},{"-y+3/4,-z+1/4,-x+3/4"},{"y+1/4,-z+3/4,x+3/4"},{"-y+1/4,z+1/4,x+1/4"},{"y+3/4,z+3/4,-x+1/4"},{"-y,-x,z"},
{"y+1/2,x,z+1/2"},{"-y+1/2,x+1/2,-z"},{"y,-x+1/2,-z+1/2"},{"-x,-z,y"},{"x,-z+1/2,-y+1/2"},{"x+1/2,z,y+1/2"},{"-x+1/2,z+1/2,-y"},
{"-z,-y,x"},{"-z+1/2,y+1/2,-x"},{"z,-y+1/2,-x+1/2"},{"z+1/2,y,x+1/2"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y,z+1/2"},{"-x,y,-z"},
{"x,-y+1/2,-z+1/2"},{"z+1/2,x+1/2,y"},{"z,-x+1/2,-y+1/2"},{"-z+1/2,-x,y+1/2"},{"-z,x,-y"},{"y+1/2,z+1/2,x"},{"-y,z,-x"},
{"y,-z+1/2,-x+1/2"},{"-y+1/2,-z,x+1/2"},{"y+1/4,x+3/4,-z+3/4"},{"-y+3/4,-x+3/4,-z+1/4"},{"y+3/4,-x+1/4,z+3/4"},{"-y+1/4,x+1/4,z+1/4"},{"x+1/4,z+3/4,-y+3/4"},
{"-x+1/4,z+1/4,y+1/4"},{"-x+3/4,-z+3/4,-y+1/4"},{"x+3/4,-z+1/4,y+3/4"},{"z+1/4,y+3/4,-x+3/4"},{"z+3/4,-y+1/4,x+3/4"},{"-z+1/4,y+1/4,x+1/4"},{"-z+3/4,-y+3/4,-x+1/4"},
{"-x+3/4,-y+3/4,-z+1/4"},{"x+3/4,y+1/4,-z+3/4"},{"x+1/4,-y+1/4,z+1/4"},{"-x+1/4,y+3/4,z+3/4"},{"-z+3/4,-x+3/4,-y+1/4"},{"-z+1/4,x+3/4,y+3/4"},{"z+3/4,x+1/4,-y+3/4"},
{"z+1/4,-x+1/4,y+1/4"},{"-y+3/4,-z+3/4,-x+1/4"},{"y+1/4,-z+1/4,x+1/4"},{"-y+1/4,z+3/4,x+3/4"},{"y+3/4,z+1/4,-x+3/4"},{"-y,-x+1/2,z+1/2"},{"y+1/2,x+1/2,z"},
{"-y+1/2,x,-z+1/2"},{"y,-x,-z"},{"-x,-z+1/2,y+1/2"},{"x,-z,-y"},{"x+1/2,z+1/2,y"},{"-x+1/2,z,-y+1/2"},{"-z,-y+1/2,x+1/2"},
{"-z+1/2,y,-x+1/2"},{"z,-y,-x"},{"z+1/2,y+1/2,x"}},

// [227] F d -3 m Origin choice 2 (-F 4vw 2vw 3)
{{"x,y,z"},{"-x+3/4,-y+1/4,z+1/2"},{"-x+1/4,y+1/2,-z+3/4"},{"x+1/2,-y+3/4,-z+1/4"},{"z,x,y"},{"z+1/2,-x+3/4,-y+1/4"},{"-z+3/4,-x+1/4,y+1/2"},
{"-z+1/4,x+1/2,-y+3/4"},{"y,z,x"},{"-y+1/4,z+1/2,-x+3/4"},{"y+1/2,-z+3/4,-x+1/4"},{"-y+3/4,-z+1/4,x+1/2"},{"y+3/4,x+1/4,-z+1/2"},{"-y,-x,-z"},
{"y+1/4,-x+1/2,z+3/4"},{"-y+1/2,x+3/4,z+1/4"},{"x+3/4,z+1/4,-y+1/2"},{"-x+1/2,z+3/4,y+1/4"},{"-x,-z,-y"},{"x+1/4,-z+1/2,y+3/4"},{"z+3/4,y+1/4,-x+1/2"},
{"z+1/4,-y+1/2,x+3/4"},{"-z+1/2,y+3/4,x+1/4"},{"-z,-y,-x"},{"-x,-y,-z"},{"x+1/4,y+3/4,-z+1/2"},{"x+3/4,-y+1/2,z+1/4"},{"-x+1/2,y+1/4,z+3/4"},
{"-z,-x,-y"},{"-z+1/2,x+1/4,y+3/4"},{"z+1/4,x+3/4,-y+1/2"},{"z+3/4,-x+1/2,y+1/4"},{"-y,-z,-x"},{"y+3/4,-z+1/2,x+1/4"},{"-y+1/2,z+1/4,x+3/4"},
{"y+1/4,z+3/4,-x+1/2"},{"-y+1/4,-x+3/4,z+1/2"},{"y,x,z"},{"-y+3/4,x+1/2,-z+1/4"},{"y+1/2,-x+1/4,-z+3/4"},{"-x+1/4,-z+3/4,y+1/2"},{"x+1/2,-z+1/4,-y+3/4"},
{"x,z,y"},{"-x+3/4,z+1/2,-y+1/4"},{"-z+1/4,-y+3/4,x+1/2"},{"-z+3/4,y+1/2,-x+1/4"},{"z+1/2,-y+1/4,-x+3/4"},{"z,y,x"},{"x,y+1/2,z+1/2"},
{"-x+3/4,-y+3/4,z"},{"-x+1/4,y,-z+1/4"},{"x+1/2,-y+1/4,-z+3/4"},{"z,x+1/2,y+1/2"},{"z+1/2,-x+1/4,-y+3/4"},{"-z+3/4,-x+3/4,y"},{"-z+1/4,x,-y+1/4"},
{"y,z+1/2,x+1/2"},{"-y+1/4,z,-x+1/4"},{"y+1/2,-z+1/4,-x+3/4"},{"-y+3/4,-z+3/4,x"},{"y+3/4,x+3/4,-z"},{"-y,-x+1/2,-z+1/2"},{"y+1/4,-x,z+1/4"},
{"-y+1/2,x+1/4,z+3/4"},{"x+3/4,z+3/4,-y"},{"-x+1/2,z+1/4,y+3/4"},{"-x,-z+1/2,-y+1/2"},{"x+1/4,-z,y+1/4"},{"z+3/4,y+3/4,-x"},{"z+1/4,-y,x+1/4"},
{"-z+1/2,y+1/4,x+3/4"},{"-z,-y+1/2,-x+1/2"},{"-x,-y+1/2,-z+1/2"},{"x+1/4,y+1/4,-z"},{"x+3/4,-y,z+3/4"},{"-x+1/2,y+3/4,z+1/4"},{"-z,-x+1/2,-y+1/2"},
{"-z+1/2,x+3/4,y+1/4"},{"z+1/4,x+1/4,-y"},{"z+3/4,-x,y+3/4"},{"-y,-z+1/2,-x+1/2"},{"y+3/4,-z,x+3/4"},{"-y+1/2,z+3/4,x+1/4"},{"y+1/4,z+1/4,-x"},
{"-y+1/4,-x+1/4,z"},{"y,x+1/2,z+1/2"},{"-y+3/4,x,-z+3/4"},{"y+1/2,-x+3/4,-z+1/4"},{"-x+1/4,-z+1/4,y"},{"x+1/2,-z+3/4,-y+1/4"},{"x,z+1/2,y+1/2"},
{"-x+3/4,z,-y+3/4"},{"-z+1/4,-y+1/4,x"},{"-z+3/4,y,-x+3/4"},{"z+1/2,-y+3/4,-x+1/4"},{"z,y+1/2,x+1/2"},{"x+1/2,y,z+1/2"},{"-x+1/4,-y+1/4,z"},
{"-x+3/4,y+1/2,-z+1/4"},{"x,-y+3/4,-z+3/4"},{"z+1/2,x,y+1/2"},{"z,-x+3/4,-y+3/4"},{"-z+1/4,-x+1/4,y"},{"-z+3/4,x+1/2,-y+1/4"},{"y+1/2,z,x+1/2"},
{"-y+3/4,z+1/2,-x+1/4"},{"y,-z+3/4,-x+3/4"},{"-y+1/4,-z+1/4,x"},{"y+1/4,x+1/4,-z"},{"-y+1/2,-x,-z+1/2"},{"y+3/4,-x+1/2,z+1/4"},{"-y,x+3/4,z+3/4"},
{"x+1/4,z+1/4,-y"},{"-x,z+3/4,y+3/4"},{"-x+1/2,-z,-y+1/2"},{"x+3/4,-z+1/2,y+1/4"},{"z+1/4,y+1/4,-x"},{"z+3/4,-y+1/2,x+1/4"},{"-z,y+3/4,x+3/4"},
{"-z+1/2,-y,-x+1/2"},{"-x+1/2,-y,-z+1/2"},{"x+3/4,y+3/4,-z"},{"x+1/4,-y+1/2,z+3/4"},{"-x,y+1/4,z+1/4"},{"-z+1/2,-x,-y+1/2"},{"-z,x+1/4,y+1/4"},
{"z+3/4,x+3/4,-y"},{"z+1/4,-x+1/2,y+3/4"},{"-y+1/2,-z,-x+1/2"},{"y+1/4,-z+1/2,x+3/4"},{"-y,z+1/4,x+1/4"},{"y+3/4,z+3/4,-x"},{"-y+3/4,-x+3/4,z"},
{"y+1/2,x,z+1/2"},{"-y+1/4,x+1/2,-z+3/4"},{"y,-x+1/4,-z+1/4"},{"-x+3/4,-z+3/4,y"},{"x,-z+1/4,-y+1/4"},{"x+1/2,z,y+1/2"},{"-x+1/4,z+1/2,-y+3/4"},
{"-z+3/4,-y+3/4,x"},{"-z+1/4,y+1/2,-x+3/4"},{"z,-y+1/4,-x+1/4"},{"z+1/2,y,x+1/2"},{"x+1/2,y+1/2,z"},{"-x+1/4,-y+3/4,z+1/2"},{"-x+3/4,y,-z+3/4"},
{"x,-y+1/4,-z+1/4"},{"z+1/2,x+1/2,y"},{"z,-x+1/4,-y+1/4"},{"-z+1/4,-x+3/4,y+1/2"},{"-z+3/4,x,-y+3/4"},{"y+1/2,z+1/2,x"},{"-y+3/4,z,-x+3/4"},
{"y,-z+1/4,-x+1/4"},{"-y+1/4,-z+3/4,x+1/2"},{"y+1/4,x+3/4,-z+1/2"},{"-y+1/2,-x+1/2,-z"},{"y+3/4,-x,z+3/4"},{"-y,x+1/4,z+1/4"},{"x+1/4,z+3/4,-y+1/2"},
{"-x,z+1/4,y+1/4"},{"-x+1/2,-z+1/2,-y"},{"x+3/4,-z,y+3/4"},{"z+1/4,y+3/4,-x+1/2"},{"z+3/4,-y,x+3/4"},{"-z,y+1/4,x+1/4"},{"-z+1/2,-y+1/2,-x"},
{"-x+1/2,-y+1/2,-z"},{"x+3/4,y+1/4,-z+1/2"},{"x+1/4,-y,z+1/4"},{"-x,y+3/4,z+3/4"},{"-z+1/2,-x+1/2,-y"},{"-z,x+3/4,y+3/4"},{"z+3/4,x+1/4,-y+1/2"},
{"z+1/4,-x,y+1/4"},{"-y+1/2,-z+1/2,-x"},{"y+1/4,-z,x+1/4"},{"-y,z+3/4,x+3/4"},{"y+3/4,z+1/4,-x+1/2"},{"-y+3/4,-x+1/4,z+1/2"},{"y+1/2,x+1/2,z"},
{"-y+1/4,x,-z+1/4"},{"y,-x+3/4,-z+3/4"},{"-x+3/4,-z+1/4,y+1/2"},{"x,-z+3/4,-y+3/4"},{"x+1/2,z+1/2,y"},{"-x+1/4,z,-y+1/4"},{"-z+3/4,-y+1/4,x+1/2"},
{"-z+1/4,y,-x+1/4"},{"z,-y+3/4,-x+3/4"},{"z+1/2,y+1/2,x"}},

// F d -3 c Origin choice 1 (F 4d 2 3 -1ad)
{{"x,y,z"},{"-x,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z"},{"x+1/2,-y,-z+1/2"},{"z,x,y"},{"z+1/2,-x,-y+1/2"},{"-z,-x+1/2,y+1/2"},
{"-z+1/2,x+1/2,-y"},{"y,z,x"},{"-y+1/2,z+1/2,-x"},{"y+1/2,-z,-x+1/2"},{"-y,-z+1/2,x+1/2"},{"y+3/4,x+1/4,-z+3/4"},{"-y+1/4,-x+1/4,-z+1/4"},
{"y+1/4,-x+3/4,z+3/4"},{"-y+3/4,x+3/4,z+1/4"},{"x+3/4,z+1/4,-y+3/4"},{"-x+3/4,z+3/4,y+1/4"},{"-x+1/4,-z+1/4,-y+1/4"},{"x+1/4,-z+3/4,y+3/4"},{"z+3/4,y+1/4,-x+3/4"},
{"z+1/4,-y+3/4,x+3/4"},{"-z+3/4,y+3/4,x+1/4"},{"-z+1/4,-y+1/4,-x+1/4"},{"-x+3/4,-y+3/4,-z+3/4"},{"x+3/4,y+1/4,-z+1/4"},{"x+1/4,-y+1/4,z+3/4"},{"-x+1/4,y+3/4,z+1/4"},
{"-z+3/4,-x+3/4,-y+3/4"},{"-z+1/4,x+3/4,y+1/4"},{"z+3/4,x+1/4,-y+1/4"},{"z+1/4,-x+1/4,y+3/4"},{"-y+3/4,-z+3/4,-x+3/4"},{"y+1/4,-z+1/4,x+3/4"},{"-y+1/4,z+3/4,x+1/4"},
{"y+3/4,z+1/4,-x+1/4"},{"-y,-x+1/2,z"},{"y+1/2,x+1/2,z+1/2"},{"-y+1/2,x,-z"},{"y,-x,-z+1/2"},{"-x,-z+1/2,y"},{"x,-z,-y+1/2"},
{"x+1/2,z+1/2,y+1/2"},{"-x+1/2,z,-y"},{"-z,-y+1/2,x"},{"-z+1/2,y,-x"},{"z,-y,-x+1/2"},{"z+1/2,y+1/2,x+1/2"},{"x,y+1/2,z+1/2"},
{"-x,-y,z"},{"-x+1/2,y,-z+1/2"},{"x+1/2,-y+1/2,-z"},{"z,x+1/2,y+1/2"},{"z+1/2,-x+1/2,-y"},{"-z,-x,y"},{"-z+1/2,x,-y+1/2"},
{"y,z+1/2,x+1/2"},{"-y+1/2,z,-x+1/2"},{"y+1/2,-z+1/2,-x"},{"-y,-z,x"},{"y+3/4,x+3/4,-z+1/4"},{"-y+1/4,-x+3/4,-z+3/4"},{"y+1/4,-x+1/4,z+1/4"},
{"-y+3/4,x+1/4,z+3/4"},{"x+3/4,z+3/4,-y+1/4"},{"-x+3/4,z+1/4,y+3/4"},{"-x+1/4,-z+3/4,-y+3/4"},{"x+1/4,-z+1/4,y+1/4"},{"z+3/4,y+3/4,-x+1/4"},{"z+1/4,-y+1/4,x+1/4"},
{"-z+3/4,y+1/4,x+3/4"},{"-z+1/4,-y+3/4,-x+3/4"},{"-x+3/4,-y+1/4,-z+1/4"},{"x+3/4,y+3/4,-z+3/4"},{"x+1/4,-y+3/4,z+1/4"},{"-x+1/4,y+1/4,z+3/4"},{"-z+3/4,-x+1/4,-y+1/4"},
{"-z+1/4,x+1/4,y+3/4"},{"z+3/4,x+3/4,-y+3/4"},{"z+1/4,-x+3/4,y+1/4"},{"-y+3/4,-z+1/4,-x+1/4"},{"y+1/4,-z+3/4,x+1/4"},{"-y+1/4,z+1/4,x+3/4"},{"y+3/4,z+3/4,-x+3/4"},
{"-y,-x,z+1/2"},{"y+1/2,x,z"},{"-y+1/2,x+1/2,-z+1/2"},{"y,-x+1/2,-z"},{"-x,-z,y+1/2"},{"x,-z+1/2,-y"},{"x+1/2,z,y"},
{"-x+1/2,z+1/2,-y+1/2"},{"-z,-y,x+1/2"},{"-z+1/2,y+1/2,-x+1/2"},{"z,-y+1/2,-x"},{"z+1/2,y,x"},{"x+1/2,y,z+1/2"},{"-x+1/2,-y+1/2,z"},
{"-x,y+1/2,-z+1/2"},{"x,-y,-z"},{"z+1/2,x,y+1/2"},{"z,-x,-y"},{"-z+1/2,-x+1/2,y"},{"-z,x+1/2,-y+1/2"},{"y+1/2,z,x+1/2"},
{"-y,z+1/2,-x+1/2"},{"y,-z,-x"},{"-y+1/2,-z+1/2,x"},{"y+1/4,x+1/4,-z+1/4"},{"-y+3/4,-x+1/4,-z+3/4"},{"y+3/4,-x+3/4,z+1/4"},{"-y+1/4,x+3/4,z+3/4"},
{"x+1/4,z+1/4,-y+1/4"},{"-x+1/4,z+3/4,y+3/4"},{"-x+3/4,-z+1/4,-y+3/4"},{"x+3/4,-z+3/4,y+1/4"},{"z+1/4,y+1/4,-x+1/4"},{"z+3/4,-y+3/4,x+1/4"},{"-z+1/4,y+3/4,x+3/4"},
{"-z+3/4,-y+1/4,-x+3/4"},{"-x+1/4,-y+3/4,-z+1/4"},{"x+1/4,y+1/4,-z+3/4"},{"x+3/4,-y+1/4,z+1/4"},{"-x+3/4,y+3/4,z+3/4"},{"-z+1/4,-x+3/4,-y+1/4"},{"-z+3/4,x+3/4,y+3/4"},
{"z+1/4,x+1/4,-y+3/4"},{"z+3/4,-x+1/4,y+1/4"},{"-y+1/4,-z+3/4,-x+1/4"},{"y+3/4,-z+1/4,x+1/4"},{"-y+3/4,z+3/4,x+3/4"},{"y+1/4,z+1/4,-x+3/4"},{"-y+1/2,-x+1/2,z+1/2"},
{"y,x+1/2,z"},{"-y,x,-z+1/2"},{"y+1/2,-x,-z"},{"-x+1/2,-z+1/2,y+1/2"},{"x+1/2,-z,-y"},{"x,z+1/2,y"},{"-x,z,-y+1/2"},
{"-z+1/2,-y+1/2,x+1/2"},{"-z,y,-x+1/2"},{"z+1/2,-y,-x"},{"z,y+1/2,x"},{"x+1/2,y+1/2,z"},{"-x+1/2,-y,z+1/2"},{"-x,y,-z"},
{"x,-y+1/2,-z+1/2"},{"z+1/2,x+1/2,y"},{"z,-x+1/2,-y+1/2"},{"-z+1/2,-x,y+1/2"},{"-z,x,-y"},{"y+1/2,z+1/2,x"},{"-y,z,-x"},
{"y,-z+1/2,-x+1/2"},{"-y+1/2,-z,x+1/2"},{"y+1/4,x+3/4,-z+3/4"},{"-y+3/4,-x+3/4,-z+1/4"},{"y+3/4,-x+1/4,z+3/4"},{"-y+1/4,x+1/4,z+1/4"},{"x+1/4,z+3/4,-y+3/4"},
{"-x+1/4,z+1/4,y+1/4"},{"-x+3/4,-z+3/4,-y+1/4"},{"x+3/4,-z+1/4,y+3/4"},{"z+1/4,y+3/4,-x+3/4"},{"z+3/4,-y+1/4,x+3/4"},{"-z+1/4,y+1/4,x+1/4"},{"-z+3/4,-y+3/4,-x+1/4"},
{"-x+1/4,-y+1/4,-z+3/4"},{"x+1/4,y+3/4,-z+1/4"},{"x+3/4,-y+3/4,z+3/4"},{"-x+3/4,y+1/4,z+1/4"},{"-z+1/4,-x+1/4,-y+3/4"},{"-z+3/4,x+1/4,y+1/4"},{"z+1/4,x+3/4,-y+1/4"},
{"z+3/4,-x+3/4,y+3/4"},{"-y+1/4,-z+1/4,-x+3/4"},{"y+3/4,-z+3/4,x+3/4"},{"-y+3/4,z+1/4,x+1/4"},{"y+1/4,z+3/4,-x+1/4"},{"-y+1/2,-x,z"},{"y,x,z+1/2"},
{"-y,x+1/2,-z"},{"y+1/2,-x+1/2,-z+1/2"},{"-x+1/2,-z,y"},{"x+1/2,-z+1/2,-y+1/2"},{"x,z,y+1/2"},{"-x,z+1/2,-y"},{"-z+1/2,-y,x"},
{"-z,y+1/2,-x"},{"z+1/2,-y+1/2,-x+1/2"},{"z,y,x+1/2"}},

// F d -3 c Origin choice 2 (-F 4ud 2vw 3)
{{"x,y,z"},{"-x+1/4,-y+3/4,z+1/2"},{"-x+3/4,y+1/2,-z+1/4"},{"x+1/2,-y+1/4,-z+3/4"},{"z,x,y"},{"z+1/2,-x+1/4,-y+3/4"},{"-z+1/4,-x+3/4,y+1/2"},
{"-z+3/4,x+1/2,-y+1/4"},{"y,z,x"},{"-y+3/4,z+1/2,-x+1/4"},{"y+1/2,-z+1/4,-x+3/4"},{"-y+1/4,-z+3/4,x+1/2"},{"y+3/4,x+1/4,-z"},{"-y+1/2,-x+1/2,-z+1/2"},
{"y+1/4,-x,z+3/4"},{"-y,x+3/4,z+1/4"},{"x+3/4,z+1/4,-y"},{"-x,z+3/4,y+1/4"},{"-x+1/2,-z+1/2,-y+1/2"},{"x+1/4,-z,y+3/4"},{"z+3/4,y+1/4,-x"},
{"z+1/4,-y,x+3/4"},{"-z,y+3/4,x+1/4"},{"-z+1/2,-y+1/2,-x+1/2"},{"-x,-y,-z"},{"x+3/4,y+1/4,-z+1/2"},{"x+1/4,-y+1/2,z+3/4"},{"-x+1/2,y+3/4,z+1/4"},
{"-z,-x,-y"},{"-z+1/2,x+3/4,y+1/4"},{"z+3/4,x+1/4,-y+1/2"},{"z+1/4,-x+1/2,y+3/4"},{"-y,-z,-x"},{"y+1/4,-z+1/2,x+3/4"},{"-y+1/2,z+3/4,x+1/4"},
{"y+3/4,z+1/4,-x+1/2"},{"-y+1/4,-x+3/4,z"},{"y+1/2,x+1/2,z+1/2"},{"-y+3/4,x,-z+1/4"},{"y,-x+1/4,-z+3/4"},{"-x+1/4,-z+3/4,y"},{"x,-z+1/4,-y+3/4"},
{"x+1/2,z+1/2,y+1/2"},{"-x+3/4,z,-y+1/4"},{"-z+1/4,-y+3/4,x"},{"-z+3/4,y,-x+1/4"},{"z,-y+1/4,-x+3/4"},{"z+1/2,y+1/2,x+1/2"},{"x,y+1/2,z+1/2"},
{"-x+1/4,-y+1/4,z"},{"-x+3/4,y,-z+3/4"},{"x+1/2,-y+3/4,-z+1/4"},{"z,x+1/2,y+1/2"},{"z+1/2,-x+3/4,-y+1/4"},{"-z+1/4,-x+1/4,y"},{"-z+3/4,x,-y+3/4"},
{"y,z+1/2,x+1/2"},{"-y+3/4,z,-x+3/4"},{"y+1/2,-z+3/4,-x+1/4"},{"-y+1/4,-z+1/4,x"},{"y+3/4,x+3/4,-z+1/2"},{"-y+1/2,-x,-z"},{"y+1/4,-x+1/2,z+1/4"},
{"-y,x+1/4,z+3/4"},{"x+3/4,z+3/4,-y+1/2"},{"-x,z+1/4,y+3/4"},{"-x+1/2,-z,-y"},{"x+1/4,-z+1/2,y+1/4"},{"z+3/4,y+3/4,-x+1/2"},{"z+1/4,-y+1/2,x+1/4"},
{"-z,y+1/4,x+3/4"},{"-z+1/2,-y,-x"},{"-x,-y+1/2,-z+1/2"},{"x+3/4,y+3/4,-z"},{"x+1/4,-y,z+1/4"},{"-x+1/2,y+1/4,z+3/4"},{"-z,-x+1/2,-y+1/2"},
{"-z+1/2,x+1/4,y+3/4"},{"z+3/4,x+3/4,-y"},{"z+1/4,-x,y+1/4"},{"-y,-z+1/2,-x+1/2"},{"y+1/4,-z,x+1/4"},{"-y+1/2,z+1/4,x+3/4"},{"y+3/4,z+3/4,-x"},
{"-y+1/4,-x+1/4,z+1/2"},{"y+1/2,x,z"},{"-y+3/4,x+1/2,-z+3/4"},{"y,-x+3/4,-z+1/4"},{"-x+1/4,-z+1/4,y+1/2"},{"x,-z+3/4,-y+1/4"},{"x+1/2,z,y"},
{"-x+3/4,z+1/2,-y+3/4"},{"-z+1/4,-y+1/4,x+1/2"},{"-z+3/4,y+1/2,-x+3/4"},{"z,-y+3/4,-x+1/4"},{"z+1/2,y,x"},{"x+1/2,y,z+1/2"},{"-x+3/4,-y+3/4,z"},
{"-x+1/4,y+1/2,-z+3/4"},{"x,-y+1/4,-z+1/4"},{"z+1/2,x,y+1/2"},{"z,-x+1/4,-y+1/4"},{"-z+3/4,-x+3/4,y"},{"-z+1/4,x+1/2,-y+3/4"},{"y+1/2,z,x+1/2"},
{"-y+1/4,z+1/2,-x+3/4"},{"y,-z+1/4,-x+1/4"},{"-y+3/4,-z+3/4,x"},{"y+1/4,x+1/4,-z+1/2"},{"-y,-x+1/2,-z"},{"y+3/4,-x,z+1/4"},{"-y+1/2,x+3/4,z+3/4"},
{"x+1/4,z+1/4,-y+1/2"},{"-x+1/2,z+3/4,y+3/4"},{"-x,-z+1/2,-y"},{"x+3/4,-z,y+1/4"},{"z+1/4,y+1/4,-x+1/2"},{"z+3/4,-y,x+1/4"},{"-z+1/2,y+3/4,x+3/4"},
{"-z,-y+1/2,-x"},{"-x+1/2,-y,-z+1/2"},{"x+1/4,y+1/4,-z"},{"x+3/4,-y+1/2,z+1/4"},{"-x,y+3/4,z+3/4"},{"-z+1/2,-x,-y+1/2"},{"-z,x+3/4,y+3/4"},
{"z+1/4,x+1/4,-y"},{"z+3/4,-x+1/2,y+1/4"},{"-y+1/2,-z,-x+1/2"},{"y+3/4,-z+1/2,x+1/4"},{"-y,z+3/4,x+3/4"},{"y+1/4,z+1/4,-x"},{"-y+3/4,-x+3/4,z+1/2"},
{"y,x+1/2,z"},{"-y+1/4,x,-z+3/4"},{"y+1/2,-x+1/4,-z+1/4"},{"-x+3/4,-z+3/4,y+1/2"},{"x+1/2,-z+1/4,-y+1/4"},{"x,z+1/2,y"},{"-x+1/4,z,-y+3/4"},
{"-z+3/4,-y+3/4,x+1/2"},{"-z+1/4,y,-x+3/4"},{"z+1/2,-y+1/4,-x+1/4"},{"z,y+1/2,x"},{"x+1/2,y+1/2,z"},{"-x+3/4,-y+1/4,z+1/2"},{"-x+1/4,y,-z+1/4"},
{"x,-y+3/4,-z+3/4"},{"z+1/2,x+1/2,y"},{"z,-x+3/4,-y+3/4"},{"-z+3/4,-x+1/4,y+1/2"},{"-z+1/4,x,-y+1/4"},{"y+1/2,z+1/2,x"},{"-y+1/4,z,-x+1/4"},
{"y,-z+3/4,-x+3/4"},{"-y+3/4,-z+1/4,x+1/2"},{"y+1/4,x+3/4,-z"},{"-y,-x,-z+1/2"},{"y+3/4,-x+1/2,z+3/4"},{"-y+1/2,x+1/4,z+1/4"},{"x+1/4,z+3/4,-y"},
{"-x+1/2,z+1/4,y+1/4"},{"-x,-z,-y+1/2"},{"x+3/4,-z+1/2,y+3/4"},{"z+1/4,y+3/4,-x"},{"z+3/4,-y+1/2,x+3/4"},{"-z+1/2,y+1/4,x+1/4"},{"-z,-y,-x+1/2"},
{"-x+1/2,-y+1/2,-z"},{"x+1/4,y+3/4,-z+1/2"},{"x+3/4,-y,z+3/4"},{"-x,y+1/4,z+1/4"},{"-z+1/2,-x+1/2,-y"},{"-z,x+1/4,y+1/4"},{"z+1/4,x+3/4,-y+1/2"},
{"z+3/4,-x,y+3/4"},{"-y+1/2,-z+1/2,-x"},{"y+3/4,-z,x+3/4"},{"-y,z+1/4,x+1/4"},{"y+1/4,z+3/4,-x+1/2"},{"-y+3/4,-x+1/4,z"},{"y,x,z+1/2"},
{"-y+1/4,x+1/2,-z+1/4"},{"y+1/2,-x+3/4,-z+3/4"},{"-x+3/4,-z+1/4,y"},{"x+1/2,-z+3/4,-y+3/4"},{"x,z,y+1/2"},{"-x+1/4,z+1/2,-y+1/4"},{"-z+3/4,-y+1/4,x"},
{"-z+1/4,y+1/2,-x+1/4"},{"z+1/2,-y+3/4,-x+3/4"},{"z,y,x+1/2"}},

// 229 I m -3 m (-I 4 2 3)
{{"x,y,z"},{"-x,-y,z"},{"-x,y,-z"},{"x,-y,-z"},{"z,x,y"},{"z,-x,-y"},{"-z,-x,y"},
{"-z,x,-y"},{"y,z,x"},{"-y,z,-x"},{"y,-z,-x"},{"-y,-z,x"},{"y,x,-z"},{"-y,-x,-z"},
{"y,-x,z"},{"-y,x,z"},{"x,z,-y"},{"-x,z,y"},{"-x,-z,-y"},{"x,-z,y"},{"z,y,-x"},
{"z,-y,x"},{"-z,y,x"},{"-z,-y,-x"},{"-x,-y,-z"},{"x,y,-z"},{"x,-y,z"},{"-x,y,z"},
{"-z,-x,-y"},{"-z,x,y"},{"z,x,-y"},{"z,-x,y"},{"-y,-z,-x"},{"y,-z,x"},{"-y,z,x"},
{"y,z,-x"},{"-y,-x,z"},{"y,x,z"},{"-y,x,-z"},{"y,-x,-z"},{"-x,-z,y"},{"x,-z,-y"},
{"x,z,y"},{"-x,z,-y"},{"-z,-y,x"},{"-z,y,-x"},{"z,-y,-x"},{"z,y,x"},{"x+1/2,y+1/2,z+1/2"},
{"-x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z+1/2"},{"z+1/2,x+1/2,y+1/2"},{"z+1/2,-x+1/2,-y+1/2"},{"-z+1/2,-x+1/2,y+1/2"},{"-z+1/2,x+1/2,-y+1/2"},
{"y+1/2,z+1/2,x+1/2"},{"-y+1/2,z+1/2,-x+1/2"},{"y+1/2,-z+1/2,-x+1/2"},{"-y+1/2,-z+1/2,x+1/2"},{"y+1/2,x+1/2,-z+1/2"},{"-y+1/2,-x+1/2,-z+1/2"},{"y+1/2,-x+1/2,z+1/2"},
{"-y+1/2,x+1/2,z+1/2"},{"x+1/2,z+1/2,-y+1/2"},{"-x+1/2,z+1/2,y+1/2"},{"-x+1/2,-z+1/2,-y+1/2"},{"x+1/2,-z+1/2,y+1/2"},{"z+1/2,y+1/2,-x+1/2"},{"z+1/2,-y+1/2,x+1/2"},
{"-z+1/2,y+1/2,x+1/2"},{"-z+1/2,-y+1/2,-x+1/2"},{"-x+1/2,-y+1/2,-z+1/2"},{"x+1/2,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z+1/2"},{"-z+1/2,-x+1/2,-y+1/2"},
{"-z+1/2,x+1/2,y+1/2"},{"z+1/2,x+1/2,-y+1/2"},{"z+1/2,-x+1/2,y+1/2"},{"-y+1/2,-z+1/2,-x+1/2"},{"y+1/2,-z+1/2,x+1/2"},{"-y+1/2,z+1/2,x+1/2"},{"y+1/2,z+1/2,-x+1/2"},
{"-y+1/2,-x+1/2,z+1/2"},{"y+1/2,x+1/2,z+1/2"},{"-y+1/2,x+1/2,-z+1/2"},{"y+1/2,-x+1/2,-z+1/2"},{"-x+1/2,-z+1/2,y+1/2"},{"x+1/2,-z+1/2,-y+1/2"},{"x+1/2,z+1/2,y+1/2"},
{"-x+1/2,z+1/2,-y+1/2"},{"-z+1/2,-y+1/2,x+1/2"},{"-z+1/2,y+1/2,-x+1/2"},{"z+1/2,-y+1/2,-x+1/2"},{"z+1/2,y+1/2,x+1/2"}},

// [230] I a -3 d (-I 4bd 2c 3)
{{"x,y,z"},{"-x+1/2,-y,z+1/2"},{"-x,y+1/2,-z+1/2"},{"x+1/2,-y+1/2,-z"},{"z,x,y"},{"z+1/2,-x+1/2,-y"},{"-z+1/2,-x,y+1/2"},
{"-z,x+1/2,-y+1/2"},{"y,z,x"},{"-y,z+1/2,-x+1/2"},{"y+1/2,-z+1/2,-x"},{"-y+1/2,-z,x+1/2"},{"y+3/4,x+1/4,-z+1/4"},{"-y+3/4,-x+3/4,-z+3/4"},
{"y+1/4,-x+1/4,z+3/4"},{"-y+1/4,x+3/4,z+1/4"},{"x+3/4,z+1/4,-y+1/4"},{"-x+1/4,z+3/4,y+1/4"},{"-x+3/4,-z+3/4,-y+3/4"},{"x+1/4,-z+1/4,y+3/4"},{"z+3/4,y+1/4,-x+1/4"},
{"z+1/4,-y+1/4,x+3/4"},{"-z+1/4,y+3/4,x+1/4"},{"-z+3/4,-y+3/4,-x+3/4"},{"-x,-y,-z"},{"x+1/2,y,-z+1/2"},{"x,-y+1/2,z+1/2"},{"-x+1/2,y+1/2,z"},
{"-z,-x,-y"},{"-z+1/2,x+1/2,y"},{"z+1/2,x,-y+1/2"},{"z,-x+1/2,y+1/2"},{"-y,-z,-x"},{"y,-z+1/2,x+1/2"},{"-y+1/2,z+1/2,x"},
{"y+1/2,z,-x+1/2"},{"-y+1/4,-x+3/4,z+3/4"},{"y+1/4,x+1/4,z+1/4"},{"-y+3/4,x+3/4,-z+1/4"},{"y+3/4,-x+1/4,-z+3/4"},{"-x+1/4,-z+3/4,y+3/4"},{"x+3/4,-z+1/4,-y+3/4"},
{"x+1/4,z+1/4,y+1/4"},{"-x+3/4,z+3/4,-y+1/4"},{"-z+1/4,-y+3/4,x+3/4"},{"-z+3/4,y+3/4,-x+1/4"},{"z+3/4,-y+1/4,-x+3/4"},{"z+1/4,y+1/4,x+1/4"},{"x+1/2,y+1/2,z+1/2"},
{"-x,-y+1/2,z"},{"-x+1/2,y,-z"},{"x,-y,-z+1/2"},{"z+1/2,x+1/2,y+1/2"},{"z,-x,-y+1/2"},{"-z,-x+1/2,y"},{"-z+1/2,x,-y"},
{"y+1/2,z+1/2,x+1/2"},{"-y+1/2,z,-x"},{"y,-z,-x+1/2"},{"-y,-z+1/2,x"},{"y+1/4,x+3/4,-z+3/4"},{"-y+1/4,-x+1/4,-z+1/4"},{"y+3/4,-x+3/4,z+1/4"},
{"-y+3/4,x+1/4,z+3/4"},{"x+1/4,z+3/4,-y+3/4"},{"-x+3/4,z+1/4,y+3/4"},{"-x+1/4,-z+1/4,-y+1/4"},{"x+3/4,-z+3/4,y+1/4"},{"z+1/4,y+3/4,-x+3/4"},{"z+3/4,-y+3/4,x+1/4"},
{"-z+3/4,y+1/4,x+3/4"},{"-z+1/4,-y+1/4,-x+1/4"},{"-x+1/2,-y+1/2,-z+1/2"},{"x,y+1/2,-z"},{"x+1/2,-y,z"},{"-x,y,z+1/2"},{"-z+1/2,-x+1/2,-y+1/2"},
{"-z,x,y+1/2"},{"z,x+1/2,-y"},{"z+1/2,-x,y"},{"-y+1/2,-z+1/2,-x+1/2"},{"y+1/2,-z,x"},{"-y,z,x+1/2"},{"y,z+1/2,-x"},
{"-y+3/4,-x+1/4,z+1/4"},{"y+3/4,x+3/4,z+3/4"},{"-y+1/4,x+1/4,-z+3/4"},{"y+1/4,-x+3/4,-z+1/4"},{"-x+3/4,-z+1/4,y+1/4"},{"x+1/4,-z+3/4,-y+1/4"},{"x+3/4,z+3/4,y+3/4"},
{"-x+1/4,z+1/4,-y+3/4"},{"-z+3/4,-y+1/4,x+1/4"},{"-z+1/4,y+1/4,-x+3/4"},{"z+1/4,-y+3/4,-x+1/4"},{"z+3/4,y+3/4,x+3/4"}}
};

/*********************************************************************************************************
 * Name       | ConvertStringSymmetryElement                                                             *
 * ----------------------------------------------------------------------------------------------------- *
 * Function   | converts a symmetry element string to a more standarized one                             *
 * Parameters | -                                                                                        *
 *********************************************************************************************************/

void ConvertStringSymmetryElement(char *string)
{
  char *a;
  int len;

  memmove(&string[1],&string[0],strlen(string)+1);
  string[0]=',';
  strcat(string,",");

  // change all '+x,' into 'x'
  while((a=strstr(string,",+x"))) strncpy(a,", x",3);

  // change all '+y,' into 'y'
  while((a=strstr(string,",+y"))) strncpy(a,", y",3);

  // change all '+z,' into 'z'
  while((a=strstr(string,",+z"))) strncpy(a,", z",3);

  // change all 'y-x,' into '-x+y'
  while((a=strstr(string,",y-x")))
  {
    memmove(a+1,a,strlen(a)+1);
    strncpy(a,",-x+y",5);
  }

  // change all '-1,' into '  ,'
  while((a=strstr(string,"-1,"))) strncpy(a,"  ,",3);

  // change all '+1,' into '  ,'
  while((a=strstr(string,"+1,"))) strncpy(a,"  ,",3);

  // change all '-1/2' into '+1/2'
  while((a=strstr(string,"-1/2,"))) strncpy(a,"+1/2,",5);

  // change all '-4/3' into '+2/3'
  while((a=strstr(string,"-4/3,"))) strncpy(a,"+2/3,",5);

  // change all '-2/3' into '+1/3'
  while((a=strstr(string,"-2/3,"))) strncpy(a,"+1/3,",5);

  // change all '-1/3' into '+2/3'
  while((a=strstr(string,"-1/3,"))) strncpy(a,"+2/3,",5);

  // change all '+4/3' into '+1/3'
  while((a=strstr(string,"+4/3,"))) strncpy(a,"+1/3,",5);

  // change all '-3/4' into '+1/4'
  while((a=strstr(string,"-3/4,"))) strncpy(a,"+1/4,",5);

  // change all '-1/4' into '+3/4'
  while((a=strstr(string,"-1/4,"))) strncpy(a,"+3/4,",5);

  // change all '+5/4' into '+1/4'
  while((a=strstr(string,"+5/4,"))) strncpy(a,"+1/4,",5);

  // change all '-5/6' into '+1/6'
  while((a=strstr(string,"-5/6,"))) strncpy(a,"+1/6,",5);

  // change all '-1/6' into '+5/6'
  while((a=strstr(string,"-1/6,"))) strncpy(a,"+5/6,",5);

  // change order of number and 'x','y','z'
  while((a=strstr(string,",1/2+x"))) strncpy(a,",x+1/2",6);
  while((a=strstr(string,",1/2+y"))) strncpy(a,",y+1/2",6);
  while((a=strstr(string,",1/2+z"))) strncpy(a,",z+1/2",6);
  while((a=strstr(string,",1/2-x"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-x+1/2",7);}
  while((a=strstr(string,",1/2-y"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-y+1/2",7);}
  while((a=strstr(string,",1/2-z"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-z+1/2",7);}

  while((a=strstr(string,",-1/2+x"))) strncpy(a,",x+1/2",6);
  while((a=strstr(string,",-1/2+y"))) strncpy(a,",y+1/2",6);
  while((a=strstr(string,",-1/2+z"))) strncpy(a,",z+1/2",6);
  while((a=strstr(string,",-1/2-x"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-x+1/2",7);}
  while((a=strstr(string,",-1/2-y"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-y+1/2",7);}
  while((a=strstr(string,",-1/2-z"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-z+1/2",7);}

  while((a=strstr(string,",1/3+x"))) strncpy(a,",x+1/3",6);
  while((a=strstr(string,",1/3+y"))) strncpy(a,",y+1/3",6);
  while((a=strstr(string,",1/3+z"))) strncpy(a,",z+1/3",6);
  while((a=strstr(string,",1/3-x"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-x+1/3",7);}
  while((a=strstr(string,",1/3-y"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-y+1/3",7);}
  while((a=strstr(string,",1/3-z"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-z+1/3",7);}

  while((a=strstr(string,",-1/3+x"))) strncpy(a,",x+2/3",6);
  while((a=strstr(string,",-1/3+y"))) strncpy(a,",y+2/3",6);
  while((a=strstr(string,",-1/3+z"))) strncpy(a,",z+2/3",6);
  while((a=strstr(string,",-1/3-x"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-x+2/3",7);}
  while((a=strstr(string,",-1/3-y"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-y+2/3",7);}
  while((a=strstr(string,",-1/3-z"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-z+2/3",7);}

  while((a=strstr(string,",2/3+x"))) strncpy(a,",x+2/3",6);
  while((a=strstr(string,",2/3+y"))) strncpy(a,",y+2/3",6);
  while((a=strstr(string,",2/3+z"))) strncpy(a,",z+2/3",6);
  while((a=strstr(string,",2/3-x"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-x+2/3",7);}
  while((a=strstr(string,",2/3-y"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-y+2/3",7);}
  while((a=strstr(string,",2/3-z"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-z+2/3",7);}

  while((a=strstr(string,",-2/3+x"))) strncpy(a,",x+1/3",6);
  while((a=strstr(string,",-2/3+y"))) strncpy(a,",y+1/3",6);
  while((a=strstr(string,",-2/3+z"))) strncpy(a,",z+1/3",6);
  while((a=strstr(string,",-2/3-x"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-x+1/3",7);}
  while((a=strstr(string,",-2/3-y"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-y+1/3",7);}
  while((a=strstr(string,",-2/3-z"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-z+1/3",7);}

  while((a=strstr(string,",1/4+x"))) strncpy(a,",x+1/4",6);
  while((a=strstr(string,",1/4+y"))) strncpy(a,",y+1/4",6);
  while((a=strstr(string,",1/4+z"))) strncpy(a,",z+1/4",6);
  while((a=strstr(string,",1/4-x"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-x+1/4",7);}
  while((a=strstr(string,",1/4-y"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-y+1/4",7);}
  while((a=strstr(string,",1/4-z"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-z+1/4",7);}

  while((a=strstr(string,",-1/4+x"))) strncpy(a,",x+3/4",6);
  while((a=strstr(string,",-1/4+y"))) strncpy(a,",y+3/4",6);
  while((a=strstr(string,",-1/4+z"))) strncpy(a,",z+3/4",6);
  while((a=strstr(string,",-1/4-x"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-x+3/4",7);}
  while((a=strstr(string,",-1/4-y"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-y+3/4",7);}
  while((a=strstr(string,",-1/4-z"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-z+3/4",7);}

  while((a=strstr(string,",3/4+x"))) strncpy(a,",x+3/4",6);
  while((a=strstr(string,",3/4+y"))) strncpy(a,",y+3/4",6);
  while((a=strstr(string,",3/4+z"))) strncpy(a,",z+3/4",6);
  while((a=strstr(string,",3/4-x"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-x+3/4",7);}
  while((a=strstr(string,",3/4-y"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-y+3/4",7);}
  while((a=strstr(string,",3/4-z"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-z+3/4",7);}

  while((a=strstr(string,",-3/4+x"))) strncpy(a,",x+1/4",6);
  while((a=strstr(string,",-3/4+y"))) strncpy(a,",y+1/4",6);
  while((a=strstr(string,",-3/4+z"))) strncpy(a,",z+1/4",6);
  while((a=strstr(string,",-3/4-x"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-x+1/4",7);}
  while((a=strstr(string,",-3/4-y"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-y+1/4",7);}
  while((a=strstr(string,",-3/4-z"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-z+1/4",7);}

  while((a=strstr(string,",1/6+x"))) strncpy(a,",x+1/6",6);
  while((a=strstr(string,",1/6+y"))) strncpy(a,",y+1/6",6);
  while((a=strstr(string,",1/6+z"))) strncpy(a,",z+1/6",6);
  while((a=strstr(string,",1/6-x"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-x+1/6",7);}
  while((a=strstr(string,",1/6-y"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-y+1/6",7);}
  while((a=strstr(string,",1/6-z"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-z+1/6",7);}

  while((a=strstr(string,",-1/6+x"))) strncpy(a,",x+5/6",6);
  while((a=strstr(string,",-1/6+y"))) strncpy(a,",y+5/6",6);
  while((a=strstr(string,",-1/6+z"))) strncpy(a,",z+5/6",6);
  while((a=strstr(string,",-1/6-x"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-x+5/6",7);}
  while((a=strstr(string,",-1/6-y"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-y+5/6",7);}
  while((a=strstr(string,",-1/6-z"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-z+5/6",7);}

  while((a=strstr(string,",5/6+x"))) strncpy(a,",x+5/6",6);
  while((a=strstr(string,",5/6+y"))) strncpy(a,",y+5/6",6);
  while((a=strstr(string,",5/6+z"))) strncpy(a,",z+5/6",6);
  while((a=strstr(string,",5/6-x"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-x+5/6",7);}
  while((a=strstr(string,",5/6-y"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-y+5/6",7);}
  while((a=strstr(string,",5/6-z"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-z+5/6",7);}

  while((a=strstr(string,",-5/6+x"))) strncpy(a,",x+1/6",6);
  while((a=strstr(string,",-5/6+y"))) strncpy(a,",y+1/6",6);
  while((a=strstr(string,",-5/6+z"))) strncpy(a,",z+1/6",6);
  while((a=strstr(string,",-5/6-x"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-x+1/6",7);}
  while((a=strstr(string,",-5/6-y"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-y+1/6",7);}
  while((a=strstr(string,",-5/6-z"))) {memmove(a+1,a,strlen(a)+1);strncpy(a,",-z+1/6",7);}

  len=strlen(string);
  string[0]=' ';
  string[len-1]='\0';

  RemoveWhiteSpacesFromString(string);
}


int GetSpacegroupFromSymmetryElements(int nr,char elements[192][32])
{
  int i,j,k;
  int found[192];
  int found_all;
  char string1[100],string2[100];

  for(i=1;i<=NUMBER_OF_SPACEGROUPS;i++)
  {
    if(strlen(StringSpaceGroupElements[i][0])==0) continue;

    // only compare to space groups of the same size
    if(nr==SpaceGroupData[i].NumberOfOperators)
    {
      // set state to no elements found
      for(j=0;j<nr;j++)
        found[j]=FALSE;

      // for each element, check whether it occurs in the space group
      // if so, set it to found and skip this element for further comparison (it must not occur twice)
      for(j=0;j<nr;j++)
        for(k=0;k<nr;k++)
        {
          strcpy(string1,elements[j]);
          strcpy(string2,StringSpaceGroupElements[i][k]);

          ConvertStringSymmetryElement(string1);
          ConvertStringSymmetryElement(string2);

          if((!found[k])&&(strcasecmp(string1,string2)==0)) found[k]=TRUE;
        }

      // check if all elements are found
      found_all=TRUE;
      for(j=0;j<nr;j++)
        found_all&=found[j];

      // if so, return space group identifier
      if(found_all) return i;
    }
  }
  return 0;
}

int GetSpaceGroupFromITSpaceGroupNumber(int sg)
{
  int i;

  for(i=1;i<=NUMBER_OF_SPACEGROUPS;i++)
    if(SpaceGroupData[i].Number==sg) return SpaceGroupData[i].Identifier;

  return 0;
}

int AdjustForNonStandardCSSROption(int sg,int Option)
{
  int i;

  if(Option==0) return sg;

  i=sg;
  // loop as long as we are in the same space-group IT number
  while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
  {
    if(SpaceGroupData[i].SpaceGroupOptionCSSR==Option) return i;
    i++;
  }
  return sg;
}

int AdjustForNonStandardCIFOption(int sg,char *string)
{
  int i;

  if(strcasecmp("b1",string)==0)         // monoclinic unique axis b, cell choice 1, abc
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==B_UNIQUE_CELL_1) return i;
      i++;
    }
  }
  else if(strcasecmp("b2",string)==0)    // monoclinic unique axis b, cell choice 2, abc
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==B_UNIQUE_CELL_2) return i;
      i++;
    }
  }
  else if(strcasecmp("b3",string)==0)    // monoclinic unique axis b, cell choice 3, abc
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==B_UNIQUE_CELL_3) return i;
      i++;
    }
  }
  else if(strcasecmp("-b1",string)==0)   // monoclinic unique axis b, cell choice 1, c-ba
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==mB_UNIQUE_CELL_1) return i;
      i++;
    }
  }
  else if(strcasecmp("-b2",string)==0)   // monoclinic unique axis b, cell choice 2, c-ba
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==mB_UNIQUE_CELL_2) return i;
      i++;
    }
  }
  else if(strcasecmp("-b3",string)==0)   // monoclinic unique axis b, cell choice 3, c-ba
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==mB_UNIQUE_CELL_3) return i;
      i++;
    }
  }
  else if(strcasecmp("c1",string)==0)    // monoclinic unique axis c, cell choice 1, abc
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==C_UNIQUE_CELL_1) return i;
      i++;
    }
  }
  else if(strcasecmp("c2",string)==0)    // monoclinic unique axis c, cell choice 2, abc
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==C_UNIQUE_CELL_2) return i;
      i++;
    }
  }
  else if(strcasecmp("c3",string)==0)    // monoclinic unique axis c, cell choice 3, abc
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==C_UNIQUE_CELL_3) return i;
      i++;
    }
  }
  else if(strcasecmp("-c1",string)==0)   // monoclinic unique axis c, cell choice 1, ba-c
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==mC_UNIQUE_CELL_1) return i;
      i++;
    }
  }
  else if(strcasecmp("-c2",string)==0)   // monoclinic unique axis c, cell choice 2, ba-c
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==mC_UNIQUE_CELL_2) return i;
      i++;
    }
  }
  else if(strcasecmp("-c3",string)==0)   // monoclinic unique axis c, cell choice 3, ba-c
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==mC_UNIQUE_CELL_3) return i;
      i++;
    }
  }
  else if(strcasecmp("a1",string)==0)    // monoclinic unique axis a, cell choice 1, abc
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==A_UNIQUE_CELL_1) return i;
      i++;
    }
  }
  else if(strcasecmp("a2",string)==0)    // monoclinic unique axis a, cell choice 2, abc
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==A_UNIQUE_CELL_2) return i;
      i++;
    }
  }
  else if(strcasecmp("a3",string)==0)    // monoclinic unique axis a, cell choice 3, abc
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==A_UNIQUE_CELL_3) return i;
      i++;
    }
  }
  else if(strcasecmp("-a1",string)==0)   // monoclinic unique axis a, cell choice 1, -acb
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==mA_UNIQUE_CELL_1) return i;
      i++;
    }
  }
  else if(strcasecmp("-a2",string)==0)   // monoclinic unique axis a, cell choice 2, -acb
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==mA_UNIQUE_CELL_2) return i;
      i++;
    }
  }
  else if(strcasecmp("-a3",string)==0)   // monoclinic unique axis a, cell choice 3, -acb
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==mA_UNIQUE_CELL_3) return i;
      i++;
    }
  }
  else if(strcasecmp("abc",string)==0)   // orthorhombic
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_1_ABC) return i;
      i++;
    }
  }
  else if(strcasecmp("ba-c",string)==0)  // orthorhombic
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_1_BACm) return i;
      i++;
    }
  }
  else if(strcasecmp("cab",string)==0)   // orthorhombic
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_1_CAB) return i;
      i++;
    }
  }
  else if(strcasecmp("-cba",string)==0)  // orthorhombic
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_1_CmBA) return i;
      i++;
    }
  }
  else if(strcasecmp("bca",string)==0)   // orthorhombic
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_1_BCA) return i;
      i++;
    }
  }
  else if(strcasecmp("a-cb",string)==0)  // orthorhombic
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_1_ACmB) return i;
      i++;
    }
  }
  else if(strcasecmp("1abc",string)==0)  // orthorhombic origin choice 1
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_1_ABC) return i;
      i++;
    }
  }
  else if(strcasecmp("1ba-c",string)==0) // orthorhombic origin choice 1
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_1_BACm) return i;
      i++;
    }
  }
  else if(strcasecmp("1cab",string)==0)  // orthorhombic origin choice 1
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_1_CAB) return i;
      i++;
    }
  }
  else if(strcasecmp("1-cba",string)==0) // orthorhombic origin choice 1
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_1_CmBA) return i;
      i++;
    }
  }
  else if(strcasecmp("1bca",string)==0)  // orthorhombic origin choice 1
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_1_BCA) return i;
      i++;
    }
  }
  else if(strcasecmp("1a-cb",string)==0) // orthorhombic origin choice 1
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_1_ACmB) return i;
      i++;
    }
  }
  else if(strcasecmp("2abc",string)==0)  // orthorhombic origin choice 2
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_2_ABC) return i;
      i++;
    }
  }
  else if(strcasecmp("2ba-c",string)==0) // orthorhombic origin choice 2
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_2_BACm) return i;
      i++;
    }
  }
  else if(strcasecmp("2cab",string)==0)  // orthorhombic origin choice 2
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_2_CAB) return i;
      i++;
    }
  }
  else if(strcasecmp("2-cba",string)==0) // orthorhombic origin choice 2
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_2_CmBA) return i;
      i++;
    }
  }
  else if(strcasecmp("2bca",string)==0)  // orthorhombic origin choice 2
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_2_BCA) return i;
      i++;
    }
  }
  else if(strcasecmp("2a-cb",string)==0) // orthorhombic origin choice 2
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_2_ACmB) return i;
      i++;
    }
  }
  else if(strcasecmp("1",string)==0)     // tetragonal or cubic origin choice 1
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_1_ABC) return i;
      i++;
    }
  }
  else if(strcasecmp("2",string)==0)     // tetragonal or cubic origin choice 2
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==ORIGIN_2_ABC) return i;
      i++;
    }
  }
  else if(strcasecmp("h",string)==0)     // trigonal using hexagonal axes
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==HEXAGONAL) return i;
      i++;
    }
  }
  else if(strcasecmp("r",string)==0)     // trigonal using rhombohedral axes
  {
    i=sg;
    while(SpaceGroupData[i].Number==SpaceGroupData[sg].Number)
    {
      if(SpaceGroupData[i].SpaceGroupOption==RHOMBOHEDRAL) return i;
      i++;
    }
  }

  return sg;
}


int GetSpaceGroupFromHermannMauguinString(char *string)
{
  int i;
  char HM_symbol[100];

  if(strlen(string)>0)
  {
    // try 'long-name' with information on the origin, e.g. "P n n n:1"
    for(i=1;i<=NUMBER_OF_SPACEGROUPS;i++)
      if(strncasecmp(string,SpaceGroupData[i].LongInternationalHermannMauguinSpaceGroupSymbol,MAX2(strlen(string),
         strlen(SpaceGroupData[i].LongInternationalHermannMauguinSpaceGroupSymbol)))==0) return SpaceGroupData[i].Identifier;

    // try 'short-name' with information on the origin, e.g. "P n n n:1"
    for(i=1;i<=NUMBER_OF_SPACEGROUPS;i++)
      if(strncasecmp(string,SpaceGroupData[i].ShortInternationalHermannMauguinSpaceGroupSymbol,MAX2(strlen(string),
         strlen(SpaceGroupData[i].ShortInternationalHermannMauguinSpaceGroupSymbol)))==0) return SpaceGroupData[i].Identifier;

    // try 'long-name' without information on the origin, e.g. "P n n n" instead of "P n n n:1"
    for(i=1;i<=NUMBER_OF_SPACEGROUPS;i++)
    {
      sscanf(SpaceGroupData[i].LongInternationalHermannMauguinSpaceGroupSymbol,"%[^:]%*[^\n]",HM_symbol);
      if(strncasecmp(string,HM_symbol,MAX2(strlen(string),
         strlen(HM_symbol)))==0) return SpaceGroupData[i].Identifier;
    }

    // try 'short-name' without information on the origin, e.g. "P n n n" instead of "P n n n:1"
    for(i=1;i<=NUMBER_OF_SPACEGROUPS;i++)
    {
      sscanf(SpaceGroupData[i].ShortInternationalHermannMauguinSpaceGroupSymbol,"%[^:]%*[^\n]",HM_symbol);
      if(strncasecmp(string,HM_symbol,MAX2(strlen(string),
         strlen(HM_symbol)))==0) return SpaceGroupData[i].Identifier;
    }

  }
  return 0;
}

int GetSpaceGroupFromHallString(char *hall)
{
  int i;

  if(strlen(hall)>0)
  {
    for(i=1;i<=NUMBER_OF_SPACEGROUPS;i++)
      if(strncasecmp(hall,SpaceGroupData[i].HallSpaceGroupSymbol,MAX2(strlen(hall),strlen(SpaceGroupData[i].HallSpaceGroupSymbol)))==0) return SpaceGroupData[i].Identifier;
  }
  return 0;
}

int GetSpaceGroupFromHallStringWithSpacesRemoved(char *hall)
{
  int i;
  char string1[64],string2[64];

  if(strlen(hall)>0)
  {
    for(i=1;i<=NUMBER_OF_SPACEGROUPS;i++)
    {
      strcpy(string1,hall);
      RemoveWhiteSpacesFromString(string1);
      strcpy(string2,SpaceGroupData[i].HallSpaceGroupSymbol);
      RemoveWhiteSpacesFromString(string2);
      if(strncasecmp(string1,string2,MAX2(strlen(string1),strlen(string2)))==0) return SpaceGroupData[i].Identifier;
    }
  }
  return 0;
}


int GetSpaceGroupFromHermannMauguinStringWithSpacesRemoved(char *hm)
{
  int i;
  char string1[64],string2[64];

  if(strlen(hm)>0)
  {
    for(i=1;i<=NUMBER_OF_SPACEGROUPS;i++)
    {
      strcpy(string1,hm);
      RemoveWhiteSpacesFromString(hm);
      strcpy(string2,SpaceGroupData[i].ShortInternationalHermannMauguinSpaceGroupSymbol);
      RemoveWhiteSpacesFromString(string2);
      if(strncasecmp(string1,string2,MAX2(strlen(string1),strlen(string2)))==0) return SpaceGroupData[i].Identifier;
    }
  }
  return 0;
}


void TestSpacegroup(void)
{
  int i,j,sp;
  REAL rr;
  VECTOR pos,pos1;

  pos.x=pos.y=pos.z=0.0;
  CurrentSystem=0;
  CurrentFramework=0;

  for(sp=1;sp<=NUMBER_OF_SPACEGROUPS;sp++)
  {
  SpaceGroupSymmetry(sp,pos);
  for(i=0;i<SpaceGroupSize;i++)
  {
    for(j=0;j<1000;j++)
    {
      pos.x=3.0*(RandomNumber()-0.5);
      pos.y=3.0*(RandomNumber()-0.5);
      pos.z=3.0*(RandomNumber()-0.5);
      //printf("%d %f %f %f\n",i,pos.x,pos.y,pos.z);

      SpaceGroupSymmetry(sp,pos);
      //printf("asym: %f %f %f\n",SpaceGroupElement[i].x,SpaceGroupElement[i].y,SpaceGroupElement[i].z);
      pos1=SpaceGroupElement[i];
      SpaceGroupInverseSymmetry(sp,pos1);
      //printf("Inverse => %f %f %f\n\n",SpaceGroupElement[i].x,SpaceGroupElement[i].y,SpaceGroupElement[i].z);

      rr=SQR(SpaceGroupElement[i].x-pos.x)+
         SQR(SpaceGroupElement[i].y-pos.y)+
         SQR(SpaceGroupElement[i].z-pos.z);
      if(rr>1e-8) fprintf(stderr, "ERROR in %d !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n",i);
    }
  }
  fprintf(stderr, "Space group %d: ok\n",sp);
  }
  exit(0);
}

VECTOR ConvertToAsymetricUnitCell(VECTOR pos)
{
  int i,j;
  VECTOR s,t;
  REAL tr[27][3]={{-1,-1,-1},{-1,-1,0},{-1,-1,1},{-1,0,-1},{-1,0,0},{-1,0,1},{-1,1,-1},{-1,1,0},{-1,1,1},{0,-1,-1},{0,-1,0},{0,-1,1},{0,0,-1},{0,0,0},{0,0,1},{0,1,-1},{0,1,0},{0,1,1},{1,-1,-1},{1,-1,0},{1,-1,1},{1,0,-1},{1,0,0},{1,0,1},{1,1,-1},{1,1,0},{1,1,1}};


  s.x=InverseUnitCellBox[CurrentSystem].ax*pos.x+InverseUnitCellBox[CurrentSystem].bx*pos.y+InverseUnitCellBox[CurrentSystem].cx*pos.z;
  s.y=InverseUnitCellBox[CurrentSystem].ay*pos.x+InverseUnitCellBox[CurrentSystem].by*pos.y+InverseUnitCellBox[CurrentSystem].cy*pos.z;
  s.z=InverseUnitCellBox[CurrentSystem].az*pos.x+InverseUnitCellBox[CurrentSystem].bz*pos.y+InverseUnitCellBox[CurrentSystem].cz*pos.z;

  s.x-=NINT(s.x);
  s.y-=NINT(s.y);
  s.z-=NINT(s.z);

  if(s.x<0.0) s.x+=1.0;
  if(s.y<0.0) s.y+=1.0;
  if(s.z<0.0) s.z+=1.0;
  SpaceGroupInverseSymmetry(Framework[CurrentSystem].SpaceGroupIdentifier[CurrentFramework],s);
  for(i=0;i<SpaceGroupSize;i++)
  {
    for(j=0;j<27;j++)
    {
      t.x=SpaceGroupElement[i].x-tr[j][0];
      t.y=SpaceGroupElement[i].y-tr[j][1];
      t.z=SpaceGroupElement[i].z-tr[j][2];
      if(AsymmetricUnit(Framework[CurrentSystem].SpaceGroupIdentifier[CurrentFramework],t,0.0))
        return t;
    }
  }

  return pos;
}

VECTOR ConvertIonsToAsymetricUnitCell(VECTOR pos)
{
  int i,j;
  VECTOR s,t;
  REAL tr[27][3]={{-1,-1,-1},{-1,-1,0},{-1,-1,1},{-1,0,-1},{-1,0,0},{-1,0,1},{-1,1,-1},{-1,1,0},{-1,1,1},{0,-1,-1},{0,-1,0},{0,-1,1},{0,0,-1},{0,0,0},{0,0,1},{0,1,-1},{0,1,0},{0,1,1},{1,-1,-1},{1,-1,0},{1,-1,1},{1,0,-1},{1,0,0},{1,0,1},{1,1,-1},{1,1,0},{1,1,1}};


  s.x=InverseUnitCellBox[CurrentSystem].ax*pos.x+InverseUnitCellBox[CurrentSystem].bx*pos.y+InverseUnitCellBox[CurrentSystem].cx*pos.z;
  s.y=InverseUnitCellBox[CurrentSystem].ay*pos.x+InverseUnitCellBox[CurrentSystem].by*pos.y+InverseUnitCellBox[CurrentSystem].cy*pos.z;
  s.z=InverseUnitCellBox[CurrentSystem].az*pos.x+InverseUnitCellBox[CurrentSystem].bz*pos.y+InverseUnitCellBox[CurrentSystem].cz*pos.z;

  s.x-=NINT(s.x);
  s.y-=NINT(s.y);
  s.z-=NINT(s.z);

  if(s.x<0.0) s.x+=1.0;
  if(s.y<0.0) s.y+=1.0;
  if(s.z<0.0) s.z+=1.0;
  SpaceGroupInverseSymmetry(Framework[CurrentSystem].SpaceGroupIdentifier[0],s);
  for(i=0;i<SpaceGroupSize;i++)
  {
    for(j=0;j<27;j++)
    {
      t.x=SpaceGroupElement[i].x-tr[j][0];
      t.y=SpaceGroupElement[i].y-tr[j][1];
      t.z=SpaceGroupElement[i].z-tr[j][2];
      if(AsymmetricUnit(Framework[CurrentSystem].SpaceGroupIdentifier[0],t,0.0))
        return t;
    }
  }

  return pos;
}

void SpaceGroupSymmetry(int spacegroup,VECTOR pos)
{
  REAL x,y,z;

  x=pos.x;
  y=pos.y;
  z=pos.z;

  switch(spacegroup)
  {
    // TRICLINIC GROUPS
    // ================

    case 1: // [1] P 1 (P 1)
      SpaceGroupSize=1;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      break;

    case 2: // [2] P -1 (-P 1)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;              // -1       -               -               0,0,0
      break;


    // MONOCLINIC GROUPS
    // =================

    case 3: // [3] P 1 2 1 unique b axis (P 2y)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      break;
    case 4: // [3] P 1 1 2 unique c axis (P 2)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      break;
    case 5: // [3] P 2 1 1 unique a axis (P 2x)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      break;

    case 6: // [4] P 1 21 1 unique b axis (P 2yb)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      break;
    case 7: // [4] P 1 1 21 unique c axis (P 2c)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      break;
    case 8: // [4] P 21 1 1 unique a axis (P 2xa)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      break;

    case 9:  // [5] C 1 2 1 unique b axis: cell choice 1 (C 2y)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // 1        -               -               -
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      break;
    case 10: // [5] A 1 2 1 unique b axis: cell choice 2 (A 2y)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      break;
    case 11: // [5] I 1 2 1 unique b axis: cell choice 3 (I 2y)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      break;
    case 12: // [5] A 1 1 2 unique c axis: cell choice 1 (A 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      break;
    case 13: // [5] B 1 1 2 unique c axis: cell choice 2 (B 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      break;
    case 14: // [5] I 1 1 2 unique c axis: cell choice 3 (I 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      break;
    case 15: // [5] B 2 1 1 unique a axis: cell choice 1 (B 2x)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      break;
    case 16: // [5] C 2 1 1 unique a axis: cell choice 2 (C 2x)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // 1        -               -               -
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      break;
    case 17: // [5] I 2 1 1 unique a axis: cell choice 3 (I 2x)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      break;

    case 18: // [6] P 1 m 1 unique b axis (P -2y)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      break;
    case 19: // [6] P 1 1 m unique c axis (P -2)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      break;
    case 20: // [6] P m 1 1 unique a axis (P -2x)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      break;

    case 21: // [7] P 1 c 1 unique b axis: cell choice 1 (P -2yc)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,0,0
      break;
    case 22: // [7] P 1 n 1 unique b axis: cell choice 2 (P -2yac)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,0,0
      break;
    case 23: // [7] P 1 a 1 unique b axis: cell choice 3 (P -2ya)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // -2       [0,1,0]         1/2,0,0         0,0,0
      break;
    case 24: // [7] P 1 1 a unique c axis: cell choice 1 (P -2a)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,0,0         0,0,0
      break;
    case 25: // [7] P 1 1 n unique c axis: cell choice 2 (P -2ab)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,0
      break;
    case 26: // [7] P 1 1 b unique c axis: cell choice 3 (P -2b)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                        Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,1/2,0         0,0,0
      break;
    case 27: // [7] P b 1 1 unique a axis: cell choice 1 (P -2xb)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=z;               // -2       [1,0,0]         0,1/2,0         0,0,0
      break;
    case 28: // [7] P n 1 1 unique a axis: cell choice 2 (P -2xbc)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       0,0,0
      break;
    case 29: // [7] P c 1 1 unique a axis: cell choice 3 (P -2xc)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      break;

    case 30: // [8] C 1 m 1 unique b axis: cell choice 1 (C -2y)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // 1        -               -               -
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;
    case 31: // [8] A 1 m 1 unique b axis: cell choice 2 (A -2y)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,1/4,0
      break;
    case 32: // [8] I 1 m 1 unique b axis: cell choice 3 (I -2y)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 33: // [8] A 1 1 m unique c axis: cell choice 1 (A -2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // -2       [0,0,1]         0,1/2,0         0,0,1/4
      break;
    case 34: // [8] B 1 1 m  unique c axis: cell choice 2 (B -2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // -2       [0,0,1]         1/2,0,0         0,0,1/4
      break;
    case 35: // [8] I 1 1 m unique c axis: cell choice 3 (I -2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      break;
    case 36: // [8] B m 1 1 unique a axis: cell choice 1 (B -2x)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         1/4,0,0
      break;
    case 37: // [8] C m 1 1 unique a axis: cell choice 2 (C -2x)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // 1        -               -               -
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      break;
    case 38: // [8] I m 1 1 unique a axis: cell choice 3 (I -2x)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      break;

    case 39: // [9] C 1 c 1 unique b axis: cell choice 1 (C -2yc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // 1        -               -               -
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 40: // [9] A 1 n 1 unique b axis: cell choice 2 (A -2yab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/2,0
      break;
    case 41: // [9] I 1 a 1 unique b axis: cell choice 3 (I -2ya)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // -2       [0,1,0]         1/2,0,0         0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         1,0,1/2         0,1/4,0
      break;
    case 42: // [9] A 1 a 1 unique -b axis: cell choice 1 (A -2ya)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // -2       [0,1,0]         1/2,0,0         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 43: // [9] C 1 n 1 unique -b axis: cell choice 2 (C -2yac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // 1        -               -               -
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         1,0,1/2         0,1/4,0
      SpaceGroupSize=4;
      break;
    case 44: // [9] I 1 c 1 unique -b axis: cell choice 3 (I -2yc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,1         0,1/4,0
      break;
    case 45: // [9] A 1 1 a unique c axis: cell choice 1 (A -2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,0,0         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      break;
    case 46: // [9] B 1 1 n unique c axis: cell choice 2 (B -2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // -2       [0,0,1]         1,1/2,0         0,0,1/4
      break;
    case 47: // [9] I 1 1 b unique c axis: cell choice 3 (I -2b)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // -2       [0,0,1]         1/2,1,0         0,0,1/4
      break;
    case 48: // [9] B 1 1 b unique -c axis: cell choice 1 (B -2b)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      break;
    case 49: // [9] A 1 1 n unique -c axis: cell choice 2 (A -2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // -2       [0,0,1]         1/2,1,0         0,0,1/4
      break;
    case 50: // [9] I 1 1 a unique -c axis: cell choice 3 (I -2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,0,0         0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // -2       [0,0,1]         1,1/2,0         0,0,1/4
      break;
    case 51: // [9] B b 1 1 unique a axis: cell choice 1 (B -2xb)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=z;               // -2       [1,0,0]         0,1/2,0         0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      break;
    case 52: // [9] C n 1 1 unique a axis: cell choice 2 (C -2xac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // 1        -               -               -
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/2,0,0
      break;
    case 53: // [9] I c 1 1 unique a axis: cell choice 3 (I -2xc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=z;               // -2       [1,0,0]         0,1/2,1         1/4,0,0
      break;
    case 54: // [9] C c 1 1 unique -a axis: cell choice 1 (C -2xc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // 1        -               -               -
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      break;
    case 55: // [9] B n 1 1 unique -a axis: cell choice 2 (B -2xab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/2,0,0
      break;
    case 56: // [9] I b 1 1 unique -a axis: cell choice 3 (I -2xb)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=z;               // -2       [1,0,0]         0,1/2,0         0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=z+0.5;           // -2       [1,0,0]         0,1,1/2         1/4,0,0
      break;

    case 57: // [10] P 1 2/m 1 unique b axis (-P 2y)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      break;
    case 58: // [10] P 1 1 2/m unique c axis (-P 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      break;
    case 59: // [10] P 2/m 1 1 unique a axis (-P 2x)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      break;

    case 60: // [11] P 1 21/m 1 unique axis b (-P 2yb)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y-0.5;         SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,-1/4,0
      break;
    case 61: // [11] P 1 1 21/m unique c axis (-P 2c)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z-0.5;          // -2       [0,0,1]         0,0,0           0,0,-1/4
      break;
    case 62: // [11] P 21/m 1 1 unique a axis (-P 2xa)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=-x-0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=z;               // -2       [1,0,0]         0,0,0           -1/4,0,0
      break;

    case 63: // [12] C 1 2/m 1 unique b axis: cell choice 1 (-C 2y)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z;              // -1       -               -               1/4,1/4,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;
    case 64: // [12] A 1 2/m 1 unique b axis: cell choice 2 (-A 2y)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               0,1/4,1/4
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,1/4,0
      break;
    case 65: // [12] I 1 2/m 1 unique b axis: cell choice 3 (-I 2y)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 66: // [12] A 1 1 2/m unique c axis: cell choice 1 (-A 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               0,1/4,1/4
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // -2       [0,0,1]         0,1/2,0         0,0,1/4
      break;
    case 67: // [12] B 1 1 2/m unique c axis: cell choice 2 (-B 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               1/4,0,1/4
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=-z+0.5;          // -2       [0,0,1]         1/2,0,0         0,0,1/4
      break;
    case 68: // [12] I 1 1 2/m unique c axis: cell choice 3 (-I 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      break;
    case 69: // [12] B 2/m 1 1 unique a axis: cell choice 1 (-B 2x)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               1/4,0,1/4
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         1/4,0,0
      break;
    case 70: // [12] C 2/m 1 1 unique a axis: cell choice 2 (-C 2x)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z;              // -1       -               -               1/4,1/4,0
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      break;
    case 71: // [12] I 2/m 1 1 unique a axis: cell choice 3 (-I 2x)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      break;

    case 72: // [13] P 1 2/c 1 unique b axis: cell choice 1 (-P 2yc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,0,0
      break;
    case 73: // [13] P 1 2/n 1 unique b axis: cell choice 2 (-P 2yac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x-0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,0,0
      break;
    case 74: // [13] P 1 2/a 1 unique b axis: cell choice 3 (-P 2ya)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x-0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      break;
    case 75: // [13] P 1 1 2/a unique c axis: cell choice 1 (-P 2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x-0.5;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      break;
    case 76: // [13] P 1 1 2/n unique c axis: cell choice 2 (-P 2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x-0.5;          SpaceGroupElement[3].y=y-0.5;          SpaceGroupElement[3].z=-z;              // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      break;
    case 77: // [13] P 1 1 2/b unique c axis: cell choice 3 (-P 2b)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y-0.5;          SpaceGroupElement[3].z=-z;              // -2       [0,0,1]         0,-1/2,0        0,0,0
      break;
    case 78: // [13] P 2/b 1 1 unique a axis: cell choice 1 (-P 2xb)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y-0.5;          SpaceGroupElement[3].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      break;
    case 79: // [13] P 2/n 1 1 unique a axis: cell choice 2 (-P 2xbc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y-0.5;          SpaceGroupElement[3].z=z-0.5;           // -2       [1,0,0]         0,-1/2,-1/2     0,0,0
      break;
    case 80: // [13] P 2/c 1 1 unique a axis: cell choice 3 (-P 2xc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        0,0,0
      break;

    case 81: // [14] P 1 21/c 1 unique b axis: cell choice 1 (-P 2ybc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y-0.5;         SpaceGroupElement[3].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,-1/4,0
      break;
    case 82: // [14] P 1 21/n 1 unique b axis: cell choice 2 (-P 2yn)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x-0.5;          SpaceGroupElement[3].y=-y-0.5;         SpaceGroupElement[3].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,-1/4,0
      break;
    case 83: // [14] P 1 21/a 1 unique b axis: cell choice 3 (-P 2yab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x-0.5;          SpaceGroupElement[3].y=-y-0.5;         SpaceGroupElement[3].z=z;               // -2       [0,1,0]         -1/2,0,0        0,-1/4,0
      break;
    case 84: // [14] P 1 1 21/a unique c axis: cell choice 1 (-P 2ac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x-0.5;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z-0.5;          // -2       [0,0,1]         -1/2,0,0        0,0,-1/4
      break;
    case 85: // [14] P 1 1 21/n unique c axis: cell choice 2 (-P 2n)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x-0.5;          SpaceGroupElement[3].y=y-0.5;          SpaceGroupElement[3].z=-z-0.5;          // -2       [0,0,1]         -1/2,-1/2,0     0,0,-1/4
      break;
    case 86: // [14] P 1 1 21/b unique c axis: cell choice 3 (-P 2bc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y-0.5;          SpaceGroupElement[3].z=-z-0.5;          // -2       [0,0,1]         0,-1/2,0        0,0,-1/4
      break;
    case 87: // [14] P 21/b 1 1 unique a axis: cell choice 1 (-P 2xab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=-x-0.5;         SpaceGroupElement[3].y=y-0.5;          SpaceGroupElement[3].z=z;               // -2       [1,0,0]         0,-1/2,0        -1/4,0,0
      break;
    case 88: // [14] P 21/n 1 1 unique a axis: cell choice 2 (-P 2xn)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=-x-0.5;         SpaceGroupElement[3].y=y-0.5;          SpaceGroupElement[3].z=z-0.5;           // -2       [1,0,0]         0,-1/2,-1/2     -1/4,0,0
      break;
    case 89: // [14] P 21/c 1 1 unique a axis: cell choice 3 (-P 2xac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=-x-0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        -1/4,0,0
      break;

    case 90: // [15] C 1 2/c 1 unique b axis: cell choice 1 (-C 2yc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z;              // -1       -               -               1/4,1/4,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         1/2,0,-1/2      0,1/4,0
      break;
    case 91: // [15] A 1 2/n 1 unique b axis: cell choice 2 (-A 2yab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x-0.5;          SpaceGroupElement[3].y=-y-0.5;         SpaceGroupElement[3].z=z;               // -2       [0,1,0]         -1/2,0,0        0,-1/4,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z+0.5;          // 2        [0,1,0]         0,1,0           1/4,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               0,1/4,1/4
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         -1/2,0,1/2      0,0,0
      break;
    case 92: // [15] I 1 2/a 1 unique b axis: cell choice 3 (-I 2ya)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x-0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/2,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,1/4,0
      break;
    case 93: // [15] A 1 2/a 1 unique -b axis: cell choice 1 (-A 2ya)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x-0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               0,1/4,1/4
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         -1/2,0,1/2      0,1/4,0
      break;
    case 94: // [15] C 1 2/n 1 unique -b axis: cell choice 2 (-C 2yac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x-0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/2,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z;              // -1       -               -               1/4,1/4,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,1/4,0
      break;
    case 95: // [15] I 1 2/c 1 unique -b axis: cell choice 3 (-I 2yc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,1/2
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;
    case 96: // [15] A 1 1 2/a unique c axis: cell choice 1 (-A 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x-0.5;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               0,1/4,1/4
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // -2       [0,0,1]         -1/2,1/2,0      0,0,1/4
      break;
    case 97: // [15] B 1 1 2/n unique c axis: cell choice 2 (-B 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x-0.5;          SpaceGroupElement[3].y=y-0.5;          SpaceGroupElement[3].z=-z;              // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/2,1/4,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               1/4,0,1/4
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=y-0.5;          SpaceGroupElement[7].z=-z+0.5;          // -2       [0,0,1]         0,-1/2,0        0,0,1/4
      break;
    case 98: // [15] I 1 1 2/b unique c axis: cell choice 3 (-I 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y-0.5;          SpaceGroupElement[3].z=-z;              // -2       [0,0,1]         0,-1/2,0        0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/2,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=-z+0.5;          // -2       [0,0,1]         1/2,0,0         0,0,1/4
      break;
    case 99: // [15] B 1 1 2/b unique -c axis: cell choice 1 (-B 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y-0.5;          SpaceGroupElement[3].z=-z;              // -2       [0,0,1]         0,-1/2,0        0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               1/4,0,1/4
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=y-0.5;          SpaceGroupElement[7].z=-z+0.5;          // -2       [0,0,1]         1/2,-1/2,0      0,0,1/4
      break;
    case 100: // [15] A 1 1 2/n unique -c axis: cell choice 2 (-A 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x-0.5;          SpaceGroupElement[3].y=y-0.5;          SpaceGroupElement[3].z=-z;              // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/2,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               0,1/4,1/4
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=-z+0.5;          // -2       [0,0,1]         -1/2,0,0        0,0,1/4
      break;
    case 101: // [15] I 1 1 2/a unique -c axis: cell choice 3 (-I 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=x-0.5;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/2,1/4,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // -2       [0,0,1]         0,1/2,0         0,0,1/4
      break;
    case 102: // [15] B 2/b 1 1 unique a axis: cell choice 1 (-B 2xb)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y-0.5;          SpaceGroupElement[3].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               1/4,0,1/4
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y-0.5;          SpaceGroupElement[7].z=z+0.5;           // -2       [1,0,0]         0,-1/2,1/2      1/4,0,0
      break;
    case 103: // [15] C 2/n 1 1 unique a axis: cell choice 2 (-C 2xac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=-x-0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        -1/4,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=-z+0.5;          // 2        [1,0,0]         1,0,0           0,1/4,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z;              // -1       -               -               1/4,1/4,0
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=z-0.5;           // -2       [1,0,0]         0,1/2,-1/2      0,0,0
      break;
    case 104: // [15] I 2/c 1 1 unique a axis: cell choice 3 (-I 2xc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,1/2
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      break;
    case 105: // [15] C 2/c 1 1 unique -a axis: cell choice 1 (-C 2xc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z;              // -1       -               -               1/4,1/4,0
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=z-0.5;           // -2       [1,0,0]         0,1/2,-1/2      1/4,0,0
      break;
    case 106: // [15] B 2/n 1 1 unique -a axis: cell choice 2 (-B 2xab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=-x-0.5;         SpaceGroupElement[3].y=y-0.5;          SpaceGroupElement[3].z=z;               // -2       [1,0,0]         0,-1/2,0        -1/4,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=-z+0.5;          // 2        [1,0,0]         1,0,0           0,1/4,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               1/4,0,1/4
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=y-0.5;          SpaceGroupElement[7].z=z+0.5;           // -2       [1,0,0]         0,-1/2,1/2      0,0,0
      break;
    case 107: // [15] I 2/b 1 1 unique -a axis: cell choice 3 (-I 2xb)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y-0.5;          SpaceGroupElement[3].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/2,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         1/4,0,0
      break;


    // ORTHORHOMBIC GROUPS
    // ===================

    case 108: // [16] P 2 2 2 (P 2 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      break;

    case 109: // [17] P 2 2 21 Origin-1,abc (P 2c 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      break;
    case 110: // [17] P 21 2 2 Origin-1,cab (P 2a 2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      break;
    case 111: // [17] P 2 21 2 Origin-1,bca (P 2 2b)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      break;

    case 112: // [18] P 21 21 2 Origin-1,abc (P 2 2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      break;
    case 113: // [18] P 2 21 21 Origin-1,cab (P 2bc 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      break;
    case 114: // [18] P 21 2 21 Origin-1,bca (P 2ac 2ac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      break;

    case 115: // [19] P 21 21 21 (P 2ac 2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      break;

    case 116: // [20] C 2 2 21  Origin-1,abc (C 2c 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      break;
    case 117: // [20] A 21 2 2  Origin-1,cba (A 2a 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      break;
    case 118: // [20] B 2 21 2  Origin-1,bca (B 2 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      break;

    case 119: // [21] C 2 2 2 Origin-1,abc (C 2 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      break;
    case 120: // [21] A 2 2 2 Origin-1,cab (A 2 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      break;
    case 121: // [21] B 2 2 2 Origin-1,bca (B 2 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      break;

    case 122: // [22] F 2 2 2 (F 2 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[12].x=x+0.5;         SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z;              // 1        -               -               -
      SpaceGroupElement[13].x=-x+0.5;        SpaceGroupElement[13].y=-y+0.5;        SpaceGroupElement[13].z=z;              // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[14].x=x+0.5;         SpaceGroupElement[14].y=-y+0.5;        SpaceGroupElement[14].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[15].x=-x+0.5;        SpaceGroupElement[15].y=y+0.5;         SpaceGroupElement[15].z=-z;             // 2        [0,1,0]         0,1/2,0         1/4,0,0
      break;

    case 123: // [23] I 2 2 2 (I 2 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      break;

    case 124: // [24] I 21 21 21 (I 2b 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/2,0
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,1/2
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=-z;              // 2        [0,1,0]         0,1,0           1/4,0,1/2
      break;

    case 125: // [25] P m m 2 (P 2 -2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      break;
    case 126: // [25] P 2 m m (P -2 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      break;
    case 127: // [25] P m 2 m (P -2 -2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      break;

    case 128: // [26] P m c 21 (P 2c -2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,0,0
      break;
    case 129: // [26] P c m 21 (P 2c -2c)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      break;
    case 130: // [26] P 21 m a (P -2a 2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,0,0         0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      break;
    case 131: // [26] P 21 a m (P -2 2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,0,0
      break;
    case 132: // [26] P b 21 m (P -2 -2b)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,1/2,0         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      break;
    case 133: // [26] P m 21 b (P -2b -2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      break;

    case 134: // [27] P c c 2 (P 2 -2c)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,0,0
      break;
    case 135: // [27] P 2 a a (P -2a 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,0,0         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,0,0
      break;
    case 136: // [27] P b 2 b (P -2b -2b)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,1/2,0         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      break;

    case 137: // [28] P m a 2 (P 2 -2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           1/4,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,0,0
      break;
    case 138: // [28] P b m 2 (P 2 -2b)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,1/2,0         0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,1/4,0
      break;
    case 139: // [28] P 2 m b (P -2b 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,1/4,0
      break;
    case 140: // [28] P 2 c m (P -2c 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         0,0,0           0,0,1/4
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,0,0
      break;
    case 141: // [28] P c 2 m (P -2c -2c)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         0,0,0           0,0,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      break;
    case 142: // [28] P m 2 a (P -2a -2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,0,0         0,0,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           1/4,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      break;

    case 143: // [29] P c a 21 (P 2c -2ac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,0,0
      break;
    case 144: // [29] P b c 21 (P 2c -2b)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,1/2,0         0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,1/4,0
      break;
    case 145: // [29] P 21 a b (P -2b 2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;
    case 146: // [29] P 21 c a (P -2ac 2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,0,0
      break;
    case 147: // [29] P c 21 b (P -2bc -2c)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      break;
    case 148: // [29] P b 21 a (P -2a -2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,0,0         0,0,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      break;

    case 149: // [30] P n c 2 (P 2 -2bc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,1/4,0
      break;
    case 150: // [30] P c n 2 (P 2 -2ac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,0,0
      break;
    case 151: // [30] P 2 n a (P -2ac 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,0,0
      break;
    case 152: // [30] P 2 a n (P -2ab 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;
    case 153: // [30] P b 2 n (P -2ab -2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      break;
    case 154: // [30] P n 2 b (P -2bc -2bc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      break;

    case 155: // [31] P m n 21 (P 2ac -2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,0,0
      break;
    case 156: // [31] P n m 21 (P 2bc -2bc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      break;
    case 157: // [31] P 21 m n (P -2ab 2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      break;
    case 158: // [31] P 21 n m (P -2 2ac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,0,0
      break;
    case 159: // [31] P n 21 m (P -2 -2bc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      break;
    case 160: // [31] P m 21 n (P -2ab -2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      break;

    case 161: // [32] P b a 2 (P 2 -2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;
    case 162: // [32] P 2 c b (P -2bc 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,1/4,0
      break;
    case 163: // [32] P c 2 a (P -2ac -2ac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      break;

    case 164: // [33] P n a 21 (P 2c -2n)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;
    case 165: // [33] P b n 21 (P 2c -2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 166: // [33] P 21 n b (P -2bc 2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 167: // [33] P 21 c n (P -2n 2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,1/4,0
      break;
    case 168: // [33] P c 21 n (P -2n -2ac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      break;
    case 169: // [33] P n 21 a (P -2ac -2n)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      break;

    case 170: // [34] P n n 2 (P 2 -2n)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 171: // [34] P 2 n n (P -2n 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 172: // [34] P n 2 n (P -2n -2n)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      break;

    case 173: // [35] C m m 2 (C 2 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;
    case 174: // [35] A 2 m m (A -2 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,1/4,0
      break;
    case 175: // [35] B m 2 m (B -2 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      break;

    case 176: // [36] C m c 21 (C 2c -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 177: // [36] C c m 21 (C 2c -2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;
    case 178: // [36] A 21 m a (A -2a 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,0,0         0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,1/4,0
      break;
    case 179: // [36] A 21 a m (A -2 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 180: // [36] B b 21 m (B -2 -2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,1/2,0         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      break;
    case 181: // [36] B m 21 b (B -2b -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      break;

    case 182: // [37] C c c 2 (C 2 -2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 183: // [37] A 2 a a (A -2a 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,0,0         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 184: // [37] B b 2 b (B -2b -2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,1/2,0         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      break;

    case 185: // [38] A m m 2 (A 2 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,1/4,0
      break;
    case 186: // [38] B m m 2 (B 2 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,0,0
      break;
    case 187: // [38] B 2 m m (B -2 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,0,0
      break;
    case 188: // [38] C 2 m m (C -2 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;
    case 189: // [38] C m 2 m (C -2 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      break;
    case 190: // [38] A m 2 m (A -2 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       0,0,0
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      break;

    case 191: // [39] A b m 2 (A 2 -2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,1/2,0         0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,1/4,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1,1/2         0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,1/2,0
      break;
    case 192: // [39] B m a 2 (B 2 -2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           1/4,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         1/2,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1,0,1/2         0,0,0
      break;
    case 193: // [39] B 2 c m (B -2a 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,0,0         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1,0,0           0,0,1/4
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1,0,1/2         0,0,0
      break;
    case 194: // [39] C 2 m b (C -2a 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,0,0         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         1,1/2,0         0,0,0
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         1,0,0           0,1/4,0
      break;
    case 195: // [39] C m 2 a (C -2a -2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,0,0         0,0,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           1/4,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         1,1/2,0         0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,1/2,0         1/2,0,0
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      break;
    case 196: // [39] A c 2 m (A -2b -2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         0,0,0           0,0,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,1/2,0         0,0,1/2
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,1/2,1         0,0,0
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      break;

    case 197: // [40] A m a 2 (A 2 -2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           1/4,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 198: // [40] B b m 2 (B 2 -2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,1/2,0         0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,1/4,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 199: // [40] B 2 m b (B -2b 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,1/4,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 200: // [40] C 2 c m (C -2c 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         0,0,0           0,0,1/4
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 201: // [40] C c 2 m (C -2c -2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         0,0,0           0,0,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      break;
    case 202: // [40] A m 2 a (A -2a -2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,0,0         0,0,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           1/4,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      break;

    case 203: // [41] A b a 2 (A 2 -2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1,1/2         1/4,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/2,0
      break;
    case 204: // [41] B b a 2 (B 2 -2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/2,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1,0,1/2         0,1/4,0
      break;
    case 205: // [41] B 2 c b (B -2ab 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1,1/2,0         0,0,1/4
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1,0,1/2         0,1/4,0
      break;
    case 206: // [41] C 2 c b (C -2ac 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1,1/2,0         0,0,1/4
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1,0,1/2         0,1/4,0
      break;
    case 207: // [41] C c 2 a (C -2ac -2ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // 1        -               -               -
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1,1/2,0         0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/2,0,0
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      break;
    case 208: // [41] A c 2 a (A -2ab -2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1/2,1,0         0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1,1/2         1/4,0,0
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      break;

    case 209: // [42] F m m 2 (F 2 -2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,1/4,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[10].x=-x+0.5;        SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=z+0.5;          // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[11].x=x+0.5;         SpaceGroupElement[11].y=-y;            SpaceGroupElement[11].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,0,0
      SpaceGroupElement[12].x=x+0.5;         SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z;              // 1        -               -               -
      SpaceGroupElement[13].x=-x+0.5;        SpaceGroupElement[13].y=-y+0.5;        SpaceGroupElement[13].z=z;              // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z;              // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z;              // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;
    case 210: // [42] F 2 m m (F -2 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,1/4,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=x+0.5;          SpaceGroupElement[9].y=y;              SpaceGroupElement[9].z=-z+0.5;          // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[11].x=x+0.5;         SpaceGroupElement[11].y=-y;            SpaceGroupElement[11].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,0,0
      SpaceGroupElement[12].x=x+0.5;         SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z;              // 1        -               -               -
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z;             // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[14].x=x+0.5;         SpaceGroupElement[14].y=-y+0.5;        SpaceGroupElement[14].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z;              // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;
    case 211: // [42] F m 2 m (F -2 -2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       0,0,0
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=x+0.5;          SpaceGroupElement[9].y=y;              SpaceGroupElement[9].z=-z+0.5;          // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[10].x=-x+0.5;        SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=z+0.5;          // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[12].x=x+0.5;         SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z;              // 1        -               -               -
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z;             // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z;              // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[15].x=-x+0.5;        SpaceGroupElement[15].y=y+0.5;         SpaceGroupElement[15].z=-z;             // 2        [0,1,0]         0,1/2,0         1/4,0,0
      break;

    case 212: // [43] F d d 2 (F 2 -2d)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+0.25;        SpaceGroupElement[2].y=y+0.25;         SpaceGroupElement[2].z=z+0.25;          // -2       [1,0,0]         0,1/4,1/4       1/8,0,0
      SpaceGroupElement[3].x=x+0.75;         SpaceGroupElement[3].y=-y+0.75;        SpaceGroupElement[3].z=z+0.25;          // -2       [0,1,0]         3/4,0,1/4       0,3/8,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[6].x=-x+0.25;        SpaceGroupElement[6].y=y+0.75;         SpaceGroupElement[6].z=z+0.75;          // -2       [1,0,0]         0,3/4,3/4       1/8,0,0
      SpaceGroupElement[7].x=x+0.75;         SpaceGroupElement[7].y=-y+0.25;        SpaceGroupElement[7].z=z+0.75;          // -2       [0,1,0]         3/4,0,3/4       0,5/8,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[10].x=-x+0.75;       SpaceGroupElement[10].y=y+0.25;        SpaceGroupElement[10].z=z+0.75;         // -2       [1,0,0]         0,1/4,3/4       3/8,0,0
      SpaceGroupElement[11].x=x+0.25;        SpaceGroupElement[11].y=-y+0.75;       SpaceGroupElement[11].z=z+0.75;         // -2       [0,1,0]         5/4,0,3/4       0,3/8,0
      SpaceGroupElement[12].x=x+0.5;         SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z;              // 1        -               -               -
      SpaceGroupElement[13].x=-x+0.5;        SpaceGroupElement[13].y=-y+0.5;        SpaceGroupElement[13].z=z;              // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[14].x=-x+0.75;       SpaceGroupElement[14].y=y+0.75;        SpaceGroupElement[14].z=z+0.25;         // -2       [1,0,0]         0,3/4,1/4       3/8,0,0
      SpaceGroupElement[15].x=x+0.25;        SpaceGroupElement[15].y=-y+0.25;       SpaceGroupElement[15].z=z+0.25;         // -2       [0,1,0]         5/4,0,1/4       0,5/8,0
      break;
    case 213: // [43] F 2 d d (F -2d 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.25;         SpaceGroupElement[1].y=y+0.25;         SpaceGroupElement[1].z=-z+0.25;         // -2       [0,0,1]         1/4,1/4,0       0,0,1/8
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x+0.25;         SpaceGroupElement[3].y=-y+0.25;        SpaceGroupElement[3].z=z+0.25;          // -2       [0,1,0]         1/4,0,1/4       0,1/8,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.25;         SpaceGroupElement[5].y=y+0.75;         SpaceGroupElement[5].z=-z+0.75;         // -2       [0,0,1]         1/4,3/4,0       0,0,3/8
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[7].x=x+0.25;         SpaceGroupElement[7].y=-y+0.75;        SpaceGroupElement[7].z=z+0.75;          // -2       [0,1,0]         1/4,0,3/4       0,3/8,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=x+0.75;         SpaceGroupElement[9].y=y+0.25;         SpaceGroupElement[9].z=-z+0.75;         // -2       [0,0,1]         3/4,1/4,0       0,0,3/8
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[11].x=x+0.75;        SpaceGroupElement[11].y=-y+0.25;       SpaceGroupElement[11].z=z+0.75;         // -2       [0,1,0]         3/4,0,3/4       0,1/8,0
      SpaceGroupElement[12].x=x+0.5;         SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z;              // 1        -               -               -
      SpaceGroupElement[13].x=x+0.75;        SpaceGroupElement[13].y=y+0.75;        SpaceGroupElement[13].z=-z+0.25;        // -2       [0,0,1]         3/4,3/4,0       0,0,1/8
      SpaceGroupElement[14].x=x+0.5;         SpaceGroupElement[14].y=-y+0.5;        SpaceGroupElement[14].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[15].x=x+0.75;        SpaceGroupElement[15].y=-y+0.75;       SpaceGroupElement[15].z=z+0.25;         // -2       [0,1,0]         3/4,0,1/4       0,3/8,0
      break;
    case 214: // [43] F d 2 d (F -2d -2d)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.25;         SpaceGroupElement[1].y=y+0.25;         SpaceGroupElement[1].z=-z+0.25;         // -2       [0,0,1]         1/4,1/4,0       0,0,1/8
      SpaceGroupElement[2].x=-x+0.25;        SpaceGroupElement[2].y=y+0.25;         SpaceGroupElement[2].z=z+0.25;          // -2       [1,0,0]         0,1/4,1/4       1/8,0,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.25;         SpaceGroupElement[5].y=y+0.75;         SpaceGroupElement[5].z=-z+0.75;         // -2       [0,0,1]         1/4,3/4,0       0,0,3/8
      SpaceGroupElement[6].x=-x+0.25;        SpaceGroupElement[6].y=y+0.75;         SpaceGroupElement[6].z=z+0.75;          // -2       [1,0,0]         0,3/4,3/4       1/8,0,0
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,1,0           1/4,0,1/4
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=x+0.75;         SpaceGroupElement[9].y=y+0.25;         SpaceGroupElement[9].z=-z+0.75;         // -2       [0,0,1]         3/4,1/4,0       0,0,3/8
      SpaceGroupElement[10].x=-x+0.75;       SpaceGroupElement[10].y=y+0.25;        SpaceGroupElement[10].z=z+0.75;         // -2       [1,0,0]         0,1/4,3/4       3/8,0,0
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/2,0,1/4
      SpaceGroupElement[12].x=x+0.5;         SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z;              // 1        -               -               -
      SpaceGroupElement[13].x=x+0.75;        SpaceGroupElement[13].y=y+0.75;        SpaceGroupElement[13].z=-z+0.25;        // -2       [0,0,1]         3/4,3/4,0       0,0,1/8
      SpaceGroupElement[14].x=-x+0.75;       SpaceGroupElement[14].y=y+0.75;        SpaceGroupElement[14].z=z+0.25;         // -2       [1,0,0]         0,3/4,1/4       3/8,0,0
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=y;             SpaceGroupElement[15].z=-z;             // 2        [0,1,0]         0,1,0           1/2,0,0
      break;

    case 215: // [44] I m m 2 (I 2 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 216: // [44] I 2 m m (I -2 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 217: // [44] I m 2 m (I -2 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      break;

    case 218: // [45] I b a 2 (I 2 -2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,1/2,1         1/4,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         1/2,0,1         0,1/4,0
      break;
    case 219: // [45] I 2 c b (I -2a 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,0,0         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1,1/2,0         0,0,1/4
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1,0,1/2         0,1/4,0
      break;
    case 220: // [45] I c 2 a (I -2b -2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,1/2,0         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1/2,1,0         0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1,1/2         1/4,0,0
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      break;

    case 221: // [46] I m a 2 (I 2 -2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           1/4,0,0
      SpaceGroupElement[3].x=x+0.5;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // -2       [0,1,0]         1/2,0,0         0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/2,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1,0,1/2         0,1/4,0
      break;
    case 222: // [46] I b m 2 (I 2 -2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y+0.5;          SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,1/2,0         0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,1/4,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-y+0.5;         SpaceGroupElement[5].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1,1/2         1/4,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/2,0
      break;
    case 223: // [46] I 2 m b (I -2b 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y+0.5;          SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y+0.5;         SpaceGroupElement[3].z=z;               // -2       [0,1,0]         0,0,0           0,1/4,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1/2,1,0         0,0,1/4
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/2,0
      break;
    case 224: // [46] I 2 c m (I -2c 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         0,0,0           0,0,1/4
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,1/2
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         1/2,0,1         0,1/4,0
      break;
    case 225: // [46] I c 2 m (I -2c -2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z+0.5;          // -2       [0,0,1]         0,0,0           0,0,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,1/2
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,1/2,1         1/4,0,0
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      break;
    case 226: // [46] I m 2 a (I -2a -2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x+0.5;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;              // -2       [0,0,1]         1/2,0,0         0,0,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;               // -2       [1,0,0]         0,0,0           1/4,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1,1/2,0         0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/2,0,0
      SpaceGroupElement[7].x=-x+0.5;         SpaceGroupElement[7].y=y+0.5;          SpaceGroupElement[7].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      break;

    case 227: // [47] P m m m (-P 2 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      break;

    case 228: // [48] P n n n Origin choice 1 (P 2 2 -1n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+0.5;         SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.5;          // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 229: // [48] P n n n Origin choice 2 (-P 2ab 2bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,-1/2,-1/2     0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,0,0
      break;

    case 230: // [49] P c c m  (-P 2 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,0,0
      break;
    case 231: // [49] P m a a (-P 2a 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      break;
    case 232: // [49] P b m b (-P 2b 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,-1/2,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      break;

    case 233: // [50] P b a n Origin choice 1 (P 2 2 -1ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+0.5;         SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z;              // -1       -               -               1/4,1/4,0
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;
    case 234: // [50] P b a n Origin choice 2 (-P 2ab 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      break;
    case 235: // [50] P n c b Origin choice 1 (P 2 2 -1bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.5;          // -1       -               -               0,1/4,1/4
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,1/4,0
      break;
    case 236: // [50] P n c b Origin choice 2 (-P 2b 2bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,-1/2,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,-1/2,-1/2     0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,0,0
      break;
    case 237: // [50] P c n a Origin choice 1 (P 2 2 -1ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+0.5;         SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z+0.5;          // -1       -               -               1/4,0,1/4
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,0,0
      break;
    case 238: // [50] P c n a Origin choice 2 (-P 2a 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,0,0
      break;

    case 239: // [51] P m m a (-P 2a 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      break;
    case 240: // [51] P m m b (-P 2b 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,-1/2,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,-1/4,0
      break;
    case 241: // [51] P b m m (-P 2 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,-1/4,0
      break;
    case 242: // [51] P c m m (-P 2c 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      break;
    case 243: // [51] P m c m (-P 2c 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,0,0
      break;
    case 244: // [51] P m a m (-P 2 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      break;

    case 245: // [52] P n n a (-P 2a 2bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,-1/2,-1/2     0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,-1/4,0
      break;
    case 246: // [52] P n n b (-P 2b 2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,-1/2,0        0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,-1/2,-1/2     -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,0,0
      break;
    case 247: // [52] P b n n (-P 2n 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         -1/2,-1/2,0     0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,0,0
      break;
    case 248: // [52] P c n n (-P 2ab 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,-1/4,0
      break;
    case 249: // [52] P n c n (-P 2ab 2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,-1/2,-1/2     -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,0,0
      break;
    case 250: // [52] P n a n (-P 2n 2bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         -1/2,-1/2,0     0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,-1/2,-1/2     0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      break;

    case 251: // [53] P m n a (-P 2ac 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         -1/2,0,0        0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,0,0
      break;
    case 252: // [53] Pnmb (-P 2bc 2bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,-1/2,0        0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,-1/2,-1/2     0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      break;
    case 253: // [53] P b m n (-P 2ab 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      break;
    case 254: // [53] P c n m (-P 2 2ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,0,0
      break;
    case 255: // [53] P n c m (-P 2 2bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,-1/2,-1/2     0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,-1/4,0
      break;
    case 256: // [53] P m a n (-P 2ab 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,-1/4,0
      break;

    case 257: // [54] P c c a (-P 2a 2ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,0,0
      break;
    case 258: // [54] P c c b (-P 2b 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,-1/2,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,-1/4,0
      break;
    case 259: // [54] P b a a (-P 2a 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,-1/4,0
      break;
    case 260: // [54] P c a a (-P 2ac 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         -1/2,0,0        0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      break;
    case 261: // [54] P b c b (-P 2bc 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,-1/2,0        0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,0,0
      break;
    case 262: // [54] P b a b (-P 2b 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,-1/2,0        0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      break;

    case 263: // [55] P b a m (-P 2 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,-1/4,0
      break;
    case 264: // [55] P m c b (-P 2bc 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,-1/2,0        0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,-1/4,0
      break;
    case 265: // [55] P c m a (-P 2ac 2ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         -1/2,0,0        0,0,-1/4
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      break;

    case 266: // [56] P c c n (-P 2ab 2ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,-1/4,0
      break;
    case 267: // [56] P n a a (-P 2ac 2bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         -1/2,0,0        0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,-1/2,-1/2     0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,-1/4,0
      break;
    case 268: // [56] P b n b (-P 2bc 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,-1/2,0        0,0,-1/4
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,0,0
      break;

    case 269: // [57] P b c m (-P 2c 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,-1/4,0
      break;
    case 270: // [57] P c a m (-P 2c 2ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      break;
    case 271: // [57] P m c a (-P 2ac 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         -1/2,0,0        0,0,-1/4
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,0,0
      break;
    case 272: // [57] P m a b (-P 2b 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,-1/2,0        0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,-1/4,0
      break;
    case 273: // [57] P b m a (-P 2a 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,-1/4,0
      break;
    case 274: // [57] P c m b (-P 2bc 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,-1/2,0        0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,-1/4,0
      break;

    case 275: // [58] P n n m (-P 2 2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,-1/2,-1/2     -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,-1/4,0
      break;
    case 276: // [58] P m n n (-P 2n 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         -1/2,-1/2,0     0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,-1/4,0
      break;
    case 277: // [58] P n m n (-P 2n 2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         -1/2,-1/2,0     0,0,-1/4
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,-1/2,-1/2     -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      break;

    case 278: // [59] P m m n Origin choice 1 (P 2 2ab -1ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[4].x=-x+0.5;         SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z;              // -1       -               -               1/4,1/4,0
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      break;
    case 279: // [59] P m m n Origin choice 2 (-P 2ab 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,-1/4,0
      break;
    case 280: // [59] P n m m Origin choice 1 (P 2bc 2 -1bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.5;          // -1       -               -               0,1/4,1/4
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      break;
    case 281: // [59] P n m m Origin choice 2 (-P 2c 2bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,-1/2,-1/2     0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,-1/4,0
      break;
    case 282: // [59] P m n m Origin choice 1 (P 2ac 2ac -1ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+0.5;         SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z+0.5;          // -1       -               -               1/4,0,1/4
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,0,0
      break;
    case 283: // [59] P m n m Origin choice 2 (-P 2c 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,0,0
      break;

    case 284: // [60] P b c n (-P 2n 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         -1/2,-1/2,0     0,0,-1/4
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,0,0
      break;
    case 285: // [60] P c a n (-P 2n 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         -1/2,-1/2,0     0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,-1/4,0
      break;
    case 286: // [60] P n c a (-P 2a 2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,-1/2,-1/2     -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,-1/4,0
      break;
    case 287: // [60] P n a b (-P 2bc 2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,-1/2,0        0,0,-1/4
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,-1/2,-1/2     -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      break;
    case 288: // [60] P b n a (-P 2ac 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         -1/2,0,0        0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,-1/4,0
      break;
    case 289: // [60] P c n b (-P 2b 2ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,-1/2,0        0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,-1/4,0
      break;

    case 290: // [61] P b c a (-P 2ac 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         -1/2,0,0        0,0,-1/4
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,-1/4,0
      break;
    case 291: // [61] P c a b (-P 2bc 2ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,-1/2,0        0,0,-1/4
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,-1/4,0
      break;

    case 292: // [62] P n m a (-P 2ac 2n)      zeolites: MFI
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         -1/2,0,0        0,0,-1/4
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,-1/2,-1/2     -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,-1/4,0
      break;
    case 293: // [62] P m n b (-P 2bc 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,-1/2,0        0,0,-1/4
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,-1/4,0
      break;
    case 294: // [62] P b n m (-P 2c 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,-1/4,0
      break;
    case 295: // [62] P c m n (-P 2n 2ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         -1/2,-1/2,0     0,0,-1/4
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,-1/4,0
      break;
    case 296: // [62] P m c n (-P 2n 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         -1/2,-1/2,0     0,0,-1/4
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,-1/4,0
      break;
    case 297: // [62] P n a m (-P 2c 2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,-1/2,-1/2     -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,-1/4,0
      break;

    case 298: // [63] C m c m (-C 2c 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z;               // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z;             // -1       -               -               1/4,1/4,0
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z-0.5;         // -2       [0,0,1]         1/2,1/2,0       0,0,-1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z;              // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z-0.5;          // -2       [0,1,0]         1/2,0,-1/2      0,1/4,0
      break;
    case 299: // [63] C c m m (-C 2c 2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z;               // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z;             // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z;             // -1       -               -               1/4,1/4,0
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z-0.5;         // -2       [0,0,1]         1/2,1/2,0       0,0,-1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z-0.5;          // -2       [1,0,0]         0,1/2,-1/2      1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z;              // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;
    case 300: // [63] A m m a (-A 2a 2a)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               0,1/4,1/4
      SpaceGroupElement[13].x=x-0.5;         SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         -1/2,1/2,0      0,0,1/4
      SpaceGroupElement[14].x=-x-0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       -1/4,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,1/4,0
      break;
    case 301: // [63] A m a m (-A 2 2a)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               0,1/4,1/4
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[14].x=-x-0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       -1/4,0,0
      SpaceGroupElement[15].x=x-0.5;         SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         -1/2,0,1/2      0,1/4,0
      break;
    case 302: // [63] B b m m (-B 2 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,-1/4,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,0,1/4
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y-0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,-1/2,1/2      1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y-0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,-1/4,0
      break;
    case 303: // [63] B m m b (-B 2b 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,-1/2,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,-1/4,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,0,1/4
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y-0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1/2,-1/2,0      0,0,1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y-0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,-1/4,0
      break;

    case 304: // [64] C m c a (-C 2ac 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         -1/2,0,0        0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z;               // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/2,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/2,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z;             // -1       -               -               1/4,1/4,0
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z-0.5;         // -2       [0,0,1]         0,1/2,0         0,0,-1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z;              // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z-0.5;          // -2       [0,1,0]         0,0,-1/2        0,1/4,0
      break;
    case 305: // [64] C c m b (-C 2ac 2ac)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         -1/2,0,0        0,0,-1/4
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z;               // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/2,1/4,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1,0,0           0,1/4,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z;             // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z;             // -1       -               -               1/4,1/4,0
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z-0.5;         // -2       [0,0,1]         0,1/2,0         0,0,-1/4
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z-0.5;          // -2       [1,0,0]         0,1/2,-1/2      0,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z;              // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;
    case 306: // [64] A b m a (-A 2ab 2ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/2,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/2,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               0,1/4,1/4
      SpaceGroupElement[13].x=x-0.5;         SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         -1/2,0,0        0,0,1/4
      SpaceGroupElement[14].x=-x-0.5;        SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,0,1/2         -1/4,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,1/4,0
      break;
    case 307: // [64] A c a m (-A 2 2ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,-1/4,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/2,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1,0           1/4,0,1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               0,1/4,1/4
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[14].x=-x-0.5;        SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,0,1/2         -1/4,0,0
      SpaceGroupElement[15].x=x-0.5;         SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         -1/2,0,1/2      0,0,0
      break;
    case 308: // [64] B b c m (-B 2 2ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,-1/4,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1,0,0           0,1/4,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/2,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,0,1/4
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y-0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,-1/2,1/2      0,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y-0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,-1/4,0
      break;
    case 309: // [64] B m a b (-B 2ab 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,-1/4,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/2,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/2,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,0,1/4
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y-0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         0,-1/2,0        0,0,1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y-0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,-1/4,0
      break;

    case 310: // [65] C m m m (-C 2 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z;               // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z;             // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z;             // -1       -               -               1/4,1/4,0
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z;             // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z;              // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z;              // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;
    case 311: // [65] A m m m (-A 2 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               0,1/4,1/4
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       0,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,1/4,0
      break;
    case 312: // [65] B m m m (-B 2 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,0,1/4
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,0,0
      break;

    case 313: // [66] C c c m (-C 2 2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z;               // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z;             // -1       -               -               1/4,1/4,0
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z;             // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z-0.5;          // -2       [1,0,0]         0,1/2,-1/2      1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z-0.5;          // -2       [0,1,0]         1/2,0,-1/2      0,1/4,0
      break;
    case 314: // [66] A m a a (-A 2a 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               0,1/4,1/4
      SpaceGroupElement[13].x=x-0.5;         SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         -1/2,1/2,0      0,0,1/4
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       0,0,0
      SpaceGroupElement[15].x=x-0.5;         SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         -1/2,0,1/2      0,1/4,0
      break;
    case 315: // [66] B b m b (-B 2b 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,-1/2,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,0,1/4
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y-0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1/2,-1/2,0      0,0,1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y-0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,-1/2,1/2      1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,0,0
      break;

    case 316: // [67] C m m a (-C 2a 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z;               // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z;               // 2        [0,0,1]         0,0,0           1/2,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z;             // 2        [0,1,0]         0,1/2,0         1/2,0,0
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z;             // -1       -               -               1/4,1/4,0
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z;             // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z;              // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z;              // -2       [0,1,0]         0,0,0           0,1/4,0
      break;
    case 317: // [67] C m m b (-C 2a 2a)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z;               // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z;               // 2        [0,0,1]         0,0,0           1/2,1/4,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z;             // 2        [1,0,0]         1,0,0           0,1/4,0
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z;             // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z;             // -1       -               -               1/4,1/4,0
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z;             // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z;              // -2       [1,0,0]         0,1/2,0         0,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z;              // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;
    case 318: // [67] A b m m (-A 2b 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,-1/2,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/2,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               0,1/4,1/4
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         0,0,0           0,0,1/4
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,1/4,0
      break;
    case 319: // [67] A c m m (-A 2 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,-1/4,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/2,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1,0           0,0,1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               0,1/4,1/4
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,0,0
      break;
    case 320: // [67] B m c m (-B 2 2a)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1,0,0           0,0,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,0,0           1/2,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,0,1/4
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,0,0
      break;
    case 321: // [67] B m a m (-B 2a 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,0,0           1/2,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,0,1/4
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         0,0,0           0,0,1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,0,0
      break;

    case 322: // [68] C c c a Origin choice 1 (C 2 2 -1ac)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+0.5;         SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z+0.5;          // -1       -               -               1/4,0,1/4
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z+0.5;          // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z;               // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z;             // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/2,1/4,1/4
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1,1/2,0         0,0,1/4
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/2,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         1,0,1/2         0,1/4,0
      break;
    case 323: // [68] C c c a Origin choice 2 (-C 2a 2ac)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z;               // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z;               // 2        [0,0,1]         0,0,0           1/2,1/4,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1,0,0           0,1/4,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z;             // -1       -               -               1/4,1/4,0
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z;             // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z-0.5;          // -2       [1,0,0]         0,1/2,-1/2      0,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z-0.5;          // -2       [0,1,0]         1/2,0,-1/2      0,1/4,0
      break;
    case 324: // [68] C c c b Origin choice 1 (C 2 2 -1ac)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z;               // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z;               // 2        [0,0,1]         0,0,0           1/2,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/2,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z;             // -1       -               -               1/4,1/4,0
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z;             // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z-0.5;          // -2       [1,0,0]         0,1/2,-1/2      1/4,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z-0.5;          // -2       [0,1,0]         0,0,-1/2        0,1/4,0
      break;
    case 325: // [68] C c c b Origin choice 2 (-C 2a 2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         -1/2,0,-1/2     0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z;               // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z;               // 2        [0,0,1]         0,0,0           1/2,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/2,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z;             // -1       -               -               1/4,1/4,0
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z;             // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z-0.5;          // -2       [1,0,0]         0,1/2,-1/2      1/4,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z-0.5;          // -2       [0,1,0]         0,0,-1/2        0,1/4,0
      break;
    case 326: // [68] A b a a Origin choice 1 (A 2 2 -1ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+0.5;         SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z;              // -1       -               -               1/4,1/4,0
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,1/2,1/4
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1/2,1,0         0,0,1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,1,1/2         1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,1/2,0
      break;
    case 327: // [68] A b a a Origin choice 2 (-A 2a 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,-1/4,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/2,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1,0           1/4,0,1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               0,1/4,1/4
      SpaceGroupElement[13].x=x-0.5;         SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         -1/2,1/2,0      0,0,1/4
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[15].x=x-0.5;         SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         -1/2,0,1/2      0,0,0
      break;
    case 328: // [68] A c a a Origin choice 1 (A 2 2 -1ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+0.5;         SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z;              // -1       -               -               1/4,1/4,0
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,1/2,1/4
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1/2,1,0         0,0,1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,1,1/2         1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,1/2,0
      break;
    case 329: // [68] A c a a Origin choice 2 (-A 2ab 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/2,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/2,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               0,1/4,1/4
      SpaceGroupElement[13].x=x-0.5;         SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         -1/2,0,0        0,0,1/4
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[15].x=x-0.5;         SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         -1/2,0,1/2      0,1/4,0
      break;
    case 330: // [68] B b c b Origin choice 1 (B 2 2 -1ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+0.5;         SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z;              // -1       -               -               1/4,1/4,0
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/2,1/4,1/4
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1,1/2,0         0,0,1/4
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/2,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         1,0,1/2         0,1/4,0
      break;
    case 331: // [68] B b c b Origin choice 2 (-B 2ab 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/2,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,0,0           1/2,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,0,1/4
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y-0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         0,-1/2,0        0,0,1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y-0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,-1/2,1/2      1/4,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,0,0
      break;
    case 332: // [68] B b a b Origin choice 1 (B 2 2 -1ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+0.5;         SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z;              // -1       -               -               1/4,1/4,0
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=y+0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[7].x=x+0.5;          SpaceGroupElement[7].y=-y+0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/2,1/4,1/4
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1,1/2,0         0,0,1/4
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/2,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         1,0,1/2         0,1/4,0
      break;
    case 333: // [68] B b a b Origin choice 2 (-B 2b 2ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,-1/2,0        0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1,0,0           0,1/4,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,0,0           1/2,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,0,1/4
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y-0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1/2,-1/2,0      0,0,1/4
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y-0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,-1/2,1/2      0,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,0,0
      break;

    case 334: // [69] F m m m (-F 2 2)
      SpaceGroupSize=32;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               0,1/4,1/4
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       0,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,1/4,0
      SpaceGroupElement[16].x=x+0.5;         SpaceGroupElement[16].y=y;             SpaceGroupElement[16].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[17].x=-x+0.5;        SpaceGroupElement[17].y=-y;            SpaceGroupElement[17].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[18].x=x+0.5;         SpaceGroupElement[18].y=-y;            SpaceGroupElement[18].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[19].x=-x+0.5;        SpaceGroupElement[19].y=y;             SpaceGroupElement[19].z=-z+0.5;         // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[20].x=-x+0.5;        SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-z+0.5;         // -1       -               -               1/4,0,1/4
      SpaceGroupElement[21].x=x+0.5;         SpaceGroupElement[21].y=y;             SpaceGroupElement[21].z=-z+0.5;         // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[22].x=-x+0.5;        SpaceGroupElement[22].y=y;             SpaceGroupElement[22].z=z+0.5;          // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[23].x=x+0.5;         SpaceGroupElement[23].y=-y;            SpaceGroupElement[23].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,0,0
      SpaceGroupElement[24].x=x+0.5;         SpaceGroupElement[24].y=y+0.5;         SpaceGroupElement[24].z=z;              // 1        -               -               -
      SpaceGroupElement[25].x=-x+0.5;        SpaceGroupElement[25].y=-y+0.5;        SpaceGroupElement[25].z=z;              // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[26].x=x+0.5;         SpaceGroupElement[26].y=-y+0.5;        SpaceGroupElement[26].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[27].x=-x+0.5;        SpaceGroupElement[27].y=y+0.5;         SpaceGroupElement[27].z=-z;             // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[28].x=-x+0.5;        SpaceGroupElement[28].y=-y+0.5;        SpaceGroupElement[28].z=-z;             // -1       -               -               1/4,1/4,0
      SpaceGroupElement[29].x=x+0.5;         SpaceGroupElement[29].y=y+0.5;         SpaceGroupElement[29].z=-z;             // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[30].x=-x+0.5;        SpaceGroupElement[30].y=y+0.5;         SpaceGroupElement[30].z=z;              // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[31].x=x+0.5;         SpaceGroupElement[31].y=-y+0.5;        SpaceGroupElement[31].z=z;              // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;

    case 335: // [70] F d d d:1 Origin choice 1 (F 2 2 -1d)
      SpaceGroupSize=32;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+0.25;        SpaceGroupElement[4].y=-y+0.25;        SpaceGroupElement[4].z=-z+0.25;         // -1       -               -               1/8,1/8,1/8
      SpaceGroupElement[5].x=x+0.25;         SpaceGroupElement[5].y=y+0.25;         SpaceGroupElement[5].z=-z+0.25;         // -2       [0,0,1]         1/4,1/4,0       0,0,1/8
      SpaceGroupElement[6].x=-x+0.25;        SpaceGroupElement[6].y=y+0.25;         SpaceGroupElement[6].z=z+0.25;          // -2       [1,0,0]         0,1/4,1/4       1/8,0,0
      SpaceGroupElement[7].x=x+0.25;         SpaceGroupElement[7].y=-y+0.25;        SpaceGroupElement[7].z=z+0.25;          // -2       [0,1,0]         1/4,0,1/4       0,1/8,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[12].x=-x+0.25;       SpaceGroupElement[12].y=-y+0.75;       SpaceGroupElement[12].z=-z+0.75;        // -1       -               -               1/8,3/8,3/8
      SpaceGroupElement[13].x=x+0.25;        SpaceGroupElement[13].y=y+0.75;        SpaceGroupElement[13].z=-z+0.75;        // -2       [0,0,1]         1/4,3/4,0       0,0,3/8
      SpaceGroupElement[14].x=-x+0.25;       SpaceGroupElement[14].y=y+0.75;        SpaceGroupElement[14].z=z+0.75;         // -2       [1,0,0]         0,3/4,3/4       1/8,0,0
      SpaceGroupElement[15].x=x+0.25;        SpaceGroupElement[15].y=-y+0.75;       SpaceGroupElement[15].z=z+0.75;         // -2       [0,1,0]         1/4,0,3/4       0,3/8,0
      SpaceGroupElement[16].x=x+0.5;         SpaceGroupElement[16].y=y;             SpaceGroupElement[16].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[17].x=-x+0.5;        SpaceGroupElement[17].y=-y;            SpaceGroupElement[17].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[18].x=x+0.5;         SpaceGroupElement[18].y=-y;            SpaceGroupElement[18].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[19].x=-x+0.5;        SpaceGroupElement[19].y=y;             SpaceGroupElement[19].z=-z+0.5;         // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[20].x=-x+0.75;       SpaceGroupElement[20].y=-y+0.25;       SpaceGroupElement[20].z=-z+0.75;        // -1       -               -               3/8,1/8,3/8
      SpaceGroupElement[21].x=x+0.75;        SpaceGroupElement[21].y=y+0.25;        SpaceGroupElement[21].z=-z+0.75;        // -2       [0,0,1]         3/4,1/4,0       0,0,3/8
      SpaceGroupElement[22].x=-x+0.75;       SpaceGroupElement[22].y=y+0.25;        SpaceGroupElement[22].z=z+0.75;         // -2       [1,0,0]         0,1/4,3/4       3/8,0,0
      SpaceGroupElement[23].x=x+0.75;        SpaceGroupElement[23].y=-y+0.25;       SpaceGroupElement[23].z=z+0.75;         // -2       [0,1,0]         3/4,0,3/4       0,1/8,0
      SpaceGroupElement[24].x=x+0.5;         SpaceGroupElement[24].y=y+0.5;         SpaceGroupElement[24].z=z;              // 1        -               -               -
      SpaceGroupElement[25].x=-x+0.5;        SpaceGroupElement[25].y=-y+0.5;        SpaceGroupElement[25].z=z;              // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[26].x=x+0.5;         SpaceGroupElement[26].y=-y+0.5;        SpaceGroupElement[26].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[27].x=-x+0.5;        SpaceGroupElement[27].y=y+0.5;         SpaceGroupElement[27].z=-z;             // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[28].x=-x+0.75;       SpaceGroupElement[28].y=-y+0.75;       SpaceGroupElement[28].z=-z+0.25;        // -1       -               -               3/8,3/8,1/8
      SpaceGroupElement[29].x=x+0.75;        SpaceGroupElement[29].y=y+0.75;        SpaceGroupElement[29].z=-z+0.25;        // -2       [0,0,1]         3/4,3/4,0       0,0,1/8
      SpaceGroupElement[30].x=-x+0.75;       SpaceGroupElement[30].y=y+0.75;        SpaceGroupElement[30].z=z+0.25;         // -2       [1,0,0]         0,3/4,1/4       3/8,0,0
      SpaceGroupElement[31].x=x+0.75;        SpaceGroupElement[31].y=-y+0.75;       SpaceGroupElement[31].z=z+0.25;         // -2       [0,1,0]         3/4,0,1/4       0,3/8,0
      break;
    case 336: // [70] F d d d:2 Origin choice 2 (-F 2uv 2vw)
      SpaceGroupSize=32;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.25;        SpaceGroupElement[1].y=-y+0.25;        SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/8,1/8,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.25;        SpaceGroupElement[2].z=-z+0.25;         // 2        [1,0,0]         0,0,0           0,1/8,1/8
      SpaceGroupElement[3].x=-x+0.25;        SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.25;         // 2        [0,1,0]         0,0,0           1/8,0,1/8
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.25;         SpaceGroupElement[5].y=y-0.25;         SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/4,-1/4,0     0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.25;         SpaceGroupElement[6].z=z-0.25;          // -2       [1,0,0]         0,-1/4,-1/4     0,0,0
      SpaceGroupElement[7].x=x-0.25;         SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.25;          // -2       [0,1,0]         -1/4,0,-1/4     0,0,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.25;        SpaceGroupElement[9].y=-y+0.75;        SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/8,3/8,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y+0.75;       SpaceGroupElement[10].z=-z+0.75;        // 2        [1,0,0]         0,0,0           0,3/8,3/8
      SpaceGroupElement[11].x=-x+0.25;       SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.75;        // 2        [0,1,0]         0,1/2,0         1/8,0,3/8
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               0,1/4,1/4
      SpaceGroupElement[13].x=x-0.25;        SpaceGroupElement[13].y=y+0.25;        SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         -1/4,1/4,0      0,0,1/4
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y+0.25;        SpaceGroupElement[14].z=z+0.25;         // -2       [1,0,0]         0,1/4,1/4       0,0,0
      SpaceGroupElement[15].x=x-0.25;        SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z+0.25;         // -2       [0,1,0]         -1/4,0,1/4      0,1/4,0
      SpaceGroupElement[16].x=x+0.5;         SpaceGroupElement[16].y=y;             SpaceGroupElement[16].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[17].x=-x+0.75;       SpaceGroupElement[17].y=-y+0.25;       SpaceGroupElement[17].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         3/8,1/8,0
      SpaceGroupElement[18].x=x+0.5;         SpaceGroupElement[18].y=-y+0.25;       SpaceGroupElement[18].z=-z+0.75;        // 2        [1,0,0]         1/2,0,0         0,1/8,3/8
      SpaceGroupElement[19].x=-x+0.75;       SpaceGroupElement[19].y=y;             SpaceGroupElement[19].z=-z+0.75;        // 2        [0,1,0]         0,0,0           3/8,0,3/8
      SpaceGroupElement[20].x=-x+0.5;        SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-z+0.5;         // -1       -               -               1/4,0,1/4
      SpaceGroupElement[21].x=x+0.25;        SpaceGroupElement[21].y=y-0.25;        SpaceGroupElement[21].z=-z+0.5;         // -2       [0,0,1]         1/4,-1/4,0      0,0,1/4
      SpaceGroupElement[22].x=-x+0.5;        SpaceGroupElement[22].y=y-0.25;        SpaceGroupElement[22].z=z+0.25;         // -2       [1,0,0]         0,-1/4,1/4      1/4,0,0
      SpaceGroupElement[23].x=x+0.25;        SpaceGroupElement[23].y=-y;            SpaceGroupElement[23].z=z+0.25;         // -2       [0,1,0]         1/4,0,1/4       0,0,0
      SpaceGroupElement[24].x=x+0.5;         SpaceGroupElement[24].y=y+0.5;         SpaceGroupElement[24].z=z;              // 1        -               -               -
      SpaceGroupElement[25].x=-x+0.75;       SpaceGroupElement[25].y=-y+0.75;       SpaceGroupElement[25].z=z;              // 2        [0,0,1]         0,0,0           3/8,3/8,0
      SpaceGroupElement[26].x=x+0.5;         SpaceGroupElement[26].y=-y+0.75;       SpaceGroupElement[26].z=-z+0.25;        // 2        [1,0,0]         1/2,0,0         0,3/8,1/8
      SpaceGroupElement[27].x=-x+0.75;       SpaceGroupElement[27].y=y+0.5;         SpaceGroupElement[27].z=-z+0.25;        // 2        [0,1,0]         0,1/2,0         3/8,0,1/8
      SpaceGroupElement[28].x=-x+0.5;        SpaceGroupElement[28].y=-y+0.5;        SpaceGroupElement[28].z=-z;             // -1       -               -               1/4,1/4,0
      SpaceGroupElement[29].x=x+0.25;        SpaceGroupElement[29].y=y+0.25;        SpaceGroupElement[29].z=-z;             // -2       [0,0,1]         1/4,1/4,0       0,0,0
      SpaceGroupElement[30].x=-x+0.5;        SpaceGroupElement[30].y=y+0.25;        SpaceGroupElement[30].z=z-0.25;         // -2       [1,0,0]         0,1/4,-1/4      1/4,0,0
      SpaceGroupElement[31].x=x+0.25;        SpaceGroupElement[31].y=-y+0.5;        SpaceGroupElement[31].z=z-0.25;         // -2       [0,1,0]         1/4,0,-1/4      0,1/4,0
      break;

    case 337: // [71] I m m m (-I 2 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;

    case 338: // [72] I b a m (-I 2 2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,1/2
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z;             // 2        [0,1,0]         0,1/2,0         1/4,0,1/2
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z;              // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z;              // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;
    case 339: // [72] I m c b (-I 2a 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/2,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/2,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,1/4,0
      break;
    case 340: // [72] I c m a (-I 2b 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,-1/2,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/2,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/2,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;

    case 341: // [73] I b c a (-I 2b 2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,-1/2,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,-1/4,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/2,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,1/2
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=-z;             // 2        [0,1,0]         0,1,0           1/4,0,1/2
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z;              // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z;              // -2       [0,1,0]         1/2,0,0         0,0,0
      break;
    case 342: // [73] I c a b (-I 2a 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,-1/4,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/2,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/2,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1,0           1/2,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,0,0
      break;

    case 343: // [74] I m m a (-I 2b 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y-0.5;          SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,-1/2,0        0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,-1/4,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/2,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1,0           1/4,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,0,0
      break;
    case 344: // [74] I m m b (-I 2a 2a)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x-0.5;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           -1/4,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/2,1/4,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1,0,0           0,1/4,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       0,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 345: // [74] I b m m (-I 2c 2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z-0.5;           // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z;               // 2        [0,0,1]         0,0,1           1/4,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,1/2
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z;             // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z;              // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      break;
    case 346: // [74] I c m m (-I 2 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y-0.5;         SpaceGroupElement[7].z=z;               // -2       [0,1,0]         0,0,0           0,-1/4,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/2,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1,0           1/4,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,0,0
      break;
    case 347: // [74] I m c m (-I 2 2a)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x-0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           -1/4,0,0
      SpaceGroupElement[7].x=x-0.5;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;               // -2       [0,1,0]         -1/2,0,0        0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1,0,0           0,1/4,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/2,0,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       0,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,1/4,0
      break;
    case 348: // [74] I m a m (-I 2c 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z-0.5;          // -2       [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z-0.5;           // -2       [0,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=-y+0.5;         SpaceGroupElement[9].z=z;               // 2        [0,0,1]         0,0,1           1/4,1/4,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=y+0.5;         SpaceGroupElement[11].z=-z;             // 2        [0,1,0]         0,1/2,0         1/4,0,1/2
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z;             // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z;              // -2       [0,1,0]         1/2,0,0         0,1/4,0
      break;

    // TETRAGONAL GROUPS
    // =================

    case 349: // [75] P 4 (P 4)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      break;

    case 350: // [76] P 41 (P 4w)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         0,0,0
      break;

    case 351: // [77] P 42 (P 4c)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         0,0,0
      break;

    case 352: // [78] P 43 (P 4cw)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.75;          // 4^1      [0,0,1]         0,0,3/4         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.25;          // 4^-1     [0,0,1]         0,0,1/4         0,0,0
      break;

    case 353: // [79] I 4 (I 4)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-y+0.5;         SpaceGroupElement[5].y=x+0.5;          SpaceGroupElement[5].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[7].x=y+0.5;          SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      break;

    case 354: // [80] I 41 (I 4bw)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         -1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         1/4,-1/4,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=-y+0.5;         SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=z+0.75;          // 4^1      [0,0,1]         0,0,3/4         -1/4,3/4,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=z;               // 2        [0,0,1]         0,0,1           1/2,1/2,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=z+0.25;          // 4^-1     [0,0,1]         0,0,5/4         3/4,-1/4,0
      break;

    case 355: // [81] P -4 (P -4)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      break;

    case 356: // [82] I -4 (I -4)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[5].x=y+0.5;          SpaceGroupElement[5].y=-x+0.5;         SpaceGroupElement[5].z=-z+0.5;          // -4^1     [0,0,1]         0,0,0           1/2,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[7].x=-y+0.5;         SpaceGroupElement[7].y=x+0.5;          SpaceGroupElement[7].z=-z+0.5;          // -4^-1    [0,0,1]         0,0,0           0,1/2,1/4
      break;

    case 357: // [83] P 4/m (-P 4)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      break;

    case 358: // [84] P 42/m (-P 4c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=-z-0.5;          // -4^1     [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-z-0.5;          // -4^-1    [0,0,1]         0,0,0           0,0,-1/4
      break;

    case 359: // [85] P 4/n Origin choice 1 (P 4ab -1ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,1/2,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           1/2,0,0
      SpaceGroupElement[4].x=-x+0.5;         SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z;              // -1       -               -               1/4,1/4,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z;              // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      break;
    case 360: // [85] P 4/n Origin choice 2 (-P 4a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=y-0.5;          SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=-z;              // -4^1     [0,0,1]         0,0,0           -1/4,1/4,0
      SpaceGroupElement[6].x=x-0.5;          SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=-z;              // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=x-0.5;          SpaceGroupElement[7].z=-z;              // -4^-1    [0,0,1]         0,0,0           1/4,-1/4,0
      break;

    case 361: // [86] P 42/n Origin choice 1 (P 4n -1n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[4].x=-x+0.5;         SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.5;          // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.5;          // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      break;
    case 362: // [86] P 42/n Origin choice 2 (-P 4bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         -1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         1/4,-1/4,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x-0.5;         SpaceGroupElement[5].z=-z-0.5;          // -4^1     [0,0,1]         0,0,0           -1/4,-1/4,-1/4
      SpaceGroupElement[6].x=x-0.5;          SpaceGroupElement[6].y=y-0.5;          SpaceGroupElement[6].z=-z;              // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[7].x=-y-0.5;         SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-z-0.5;          // -4^-1    [0,0,1]         0,0,0           -1/4,-1/4,-1/4
      break;

    case 363: // [87] I 4/m (-I 4)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-y+0.5;         SpaceGroupElement[9].y=x+0.5;          SpaceGroupElement[9].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[10].x=-x+0.5;        SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[11].x=y+0.5;         SpaceGroupElement[11].y=-x+0.5;        SpaceGroupElement[11].z=z+0.5;          // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[13].x=y+0.5;         SpaceGroupElement[13].y=-x+0.5;        SpaceGroupElement[13].z=-z+0.5;         // -4^1     [0,0,1]         0,0,0           1/2,0,1/4
      SpaceGroupElement[14].x=x+0.5;         SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=-z+0.5;         // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[15].x=-y+0.5;        SpaceGroupElement[15].y=x+0.5;         SpaceGroupElement[15].z=-z+0.5;         // -4^-1    [0,0,1]         0,0,0           0,1/2,1/4
      break;

    case 364: // [88] I 41/a Origin choice 1 (I 4bw -1bw)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         -1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         1/4,-1/4,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.25;         // -1       -               -               0,1/4,1/8
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=x-0.5;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z-0.25;         // -2       [0,0,1]         -1/2,0,0        0,0,-1/8
      SpaceGroupElement[7].x=-y-0.5;         SpaceGroupElement[7].y=x+0.5;          SpaceGroupElement[7].z=-z-0.5;          // -4^-1    [0,0,1]         0,0,0           -1/2,0,-1/4
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-y+0.5;         SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=z+0.75;          // 4^1      [0,0,1]         0,0,3/4         -1/4,3/4,0
      SpaceGroupElement[10].x=-x;            SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=z;              // 2        [0,0,1]         0,0,1           1/2,1/2,0
      SpaceGroupElement[11].x=y;             SpaceGroupElement[11].y=-x+0.5;        SpaceGroupElement[11].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         3/4,-1/4,0
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z+0.75;        // -1       -               -               1/4,1/2,3/8
      SpaceGroupElement[13].x=y+0.5;         SpaceGroupElement[13].y=-x+0.5;        SpaceGroupElement[13].z=-z+0.5;         // -4^1     [0,0,1]         0,0,0           1/2,0,1/4
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=-z+0.25;        // -2       [0,0,1]         0,1/2,0         0,0,1/8
      SpaceGroupElement[15].x=-y;            SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=-z;             // -4^-1    [0,0,1]         0,0,0           -1/2,1/2,0
      break;
    case 365: // [88] I 41/a Origin choice 2 (-I 4ad)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.75;        SpaceGroupElement[1].y=x+0.25;         SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         1/4,1/2,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[3].x=y+0.75;         SpaceGroupElement[3].y=-x+0.75;        SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         3/4,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[5].x=y-0.75;         SpaceGroupElement[5].y=-x-0.25;        SpaceGroupElement[5].z=-z-0.25;         // -4^1     [0,0,1]         0,0,0           -1/2,1/4,-1/8
      SpaceGroupElement[6].x=x-0.5;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z-0.5;          // -2       [0,0,1]         -1/2,0,0        0,0,-1/4
      SpaceGroupElement[7].x=-y-0.75;        SpaceGroupElement[7].y=x-0.75;         SpaceGroupElement[7].z=-z-0.75;         // -4^-1    [0,0,1]         0,0,0           0,-3/4,-3/8
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-y+0.25;        SpaceGroupElement[9].y=x+0.75;         SpaceGroupElement[9].z=z+0.75;          // 4^1      [0,0,1]         0,0,3/4         1/4,1,0
      SpaceGroupElement[10].x=-x;            SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=z;              // 2        [0,0,1]         0,0,1           1/2,1/4,0
      SpaceGroupElement[11].x=y+0.25;        SpaceGroupElement[11].y=-x+0.25;       SpaceGroupElement[11].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         5/4,0,0
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[13].x=y-0.25;        SpaceGroupElement[13].y=-x+0.25;       SpaceGroupElement[13].z=-z+0.25;        // -4^1     [0,0,1]         0,0,0           0,1/4,1/8
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=-z;             // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[15].x=-y-0.25;       SpaceGroupElement[15].y=x-0.25;        SpaceGroupElement[15].z=-z-0.25;        // -4^-1    [0,0,1]         0,0,0           0,-1/4,-1/8
      break;

    case 366: // [89] P 4 2 2 (P 4 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      break;

    case 367: // [90] P 4 21 2 (P 4ab 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,1/2,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           1/2,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      break;

    case 368: // [91] P 41 2 2 (P 4w 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z+0.75;         // 2        [1,1,0]         0,0,0           0,0,3/8
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z+0.25;         // 2        [-1,1,0]        0,0,0           0,0,1/8
      break;

    case 369: // [92] P 41 21 2 (P 4abw 2nw)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         0,1/2,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         1/2,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.75;         // 2        [1,0,0]         1/2,0,0         0,1/4,3/8
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.25;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/8
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           0,0,1/4
      break;

    case 370: // [93] P 42 2 2 (P 4c 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           0,0,1/4
      break;

    case 371: // [94] P 42 21 2 (P 4n 2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      break;

    case 372: // [95] P 43 2 2 (P 4cw 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.75;          // 4^1      [0,0,1]         0,0,3/4         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.25;          // 4^-1     [0,0,1]         0,0,1/4         0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z+0.25;         // 2        [1,1,0]         0,0,0           0,0,1/8
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z+0.75;         // 2        [-1,1,0]        0,0,0           0,0,3/8
      break;

    case 373: // [96] P 43 21 2 (P 4nw 2abw)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.75;          // 4^1      [0,0,1]         0,0,3/4         0,1/2,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z+0.25;          // 4^-1     [0,0,1]         0,0,1/4         1/2,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.25;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/8
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.75;         // 2        [0,1,0]         0,1/2,0         1/4,0,3/8
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           0,0,1/4
      break;

    case 374: // [97] I 4 2 2 (I 4 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-y+0.5;         SpaceGroupElement[9].y=x+0.5;          SpaceGroupElement[9].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[10].x=-x+0.5;        SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[11].x=y+0.5;         SpaceGroupElement[11].y=-x+0.5;        SpaceGroupElement[11].z=z+0.5;          // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[12].x=x+0.5;         SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[13].x=y+0.5;         SpaceGroupElement[13].y=x+0.5;         SpaceGroupElement[13].z=-z+0.5;         // 2        [1,1,0]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[15].x=-y+0.5;        SpaceGroupElement[15].y=-x+0.5;        SpaceGroupElement[15].z=-z+0.5;         // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      break;

    case 375: // [98] I 41 2 2 (I 4bw 2bw)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         -1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         1/4,-1/4,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.25;         // 2        [1,0,0]         0,0,0           0,1/4,1/8
      SpaceGroupElement[5].x=y+0.5;          SpaceGroupElement[5].y=x+0.5;          SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z+0.75;         // 2        [0,1,0]         0,0,0           1/4,0,3/8
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-y+0.5;         SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=z+0.75;          // 4^1      [0,0,1]         0,0,3/4         -1/4,3/4,0
      SpaceGroupElement[10].x=-x;            SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=z;              // 2        [0,0,1]         0,0,1           1/2,1/2,0
      SpaceGroupElement[11].x=y;             SpaceGroupElement[11].y=-x+0.5;        SpaceGroupElement[11].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         3/4,-1/4,0
      SpaceGroupElement[12].x=x+0.5;         SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z+0.75;        // 2        [1,0,0]         1/2,0,0         0,1/2,3/8
      SpaceGroupElement[13].x=y;             SpaceGroupElement[13].y=x;             SpaceGroupElement[13].z=-z;             // 2        [1,1,0]         1,1,0           0,0,1/2
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=-z+0.25;        // 2        [0,1,0]         0,1/2,0         1/2,0,5/8
      SpaceGroupElement[15].x=-y+0.5;        SpaceGroupElement[15].y=-x+0.5;        SpaceGroupElement[15].z=-z+0.5;         // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      break;

    case 376: // [99] P 4 m m (P 4 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z;               // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z;               // -2       [-1,1,0]        0,0,0           0,0,0
      break;

    case 377: // [100] P 4 b m (P 4 -2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+0.5;         SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[5].x=-y+0.5;         SpaceGroupElement[5].y=-x+0.5;         SpaceGroupElement[5].z=z;               // -2       [1,1,0]         0,0,0           1/2,0,0
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[7].x=y+0.5;          SpaceGroupElement[7].y=x+0.5;          SpaceGroupElement[7].z=z;               // -2       [-1,1,0]        1/2,1/2,0       0,0,0
      break;

    case 378: // [101] P 42 c m (P 4c -2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z;               // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z;               // -2       [-1,1,0]        0,0,0           0,0,0
      break;

    case 379: // [102] P 42 n m (P 4n -2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[4].x=-x+0.5;         SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z;               // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z;               // -2       [-1,1,0]        0,0,0           0,0,0
      break;

    case 380: // [103] P 4 c c (P 4 -2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z+0.5;           // -2       [1,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z+0.5;           // -2       [-1,1,0]        0,0,1/2         0,0,0
      break;

    case 381: // [104] P 4 n c (P 4 -2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+0.5;         SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[5].x=-y+0.5;         SpaceGroupElement[5].y=-x+0.5;         SpaceGroupElement[5].z=z+0.5;           // -2       [1,1,0]         0,0,1/2         1/2,0,0
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[7].x=y+0.5;          SpaceGroupElement[7].y=x+0.5;          SpaceGroupElement[7].z=z+0.5;           // -2       [-1,1,0]        1/2,1/2,1/2     0,0,0
      break;

    case 382: // [105] P 42 m c (P 4c -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z+0.5;           // -2       [1,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z+0.5;           // -2       [-1,1,0]        0,0,1/2         0,0,0
      break;

    case 383: // [106] P 42 b c (P 4c -2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=-x+0.5;         SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[5].x=-y+0.5;         SpaceGroupElement[5].y=-x+0.5;         SpaceGroupElement[5].z=z+0.5;           // -2       [1,1,0]         0,0,1/2         1/2,0,0
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[7].x=y+0.5;          SpaceGroupElement[7].y=x+0.5;          SpaceGroupElement[7].z=z+0.5;           // -2       [-1,1,0]        1/2,1/2,1/2     0,0,0
      break;

    case 384: // [107] I 4 m m (I 4 -2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z;               // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z;               // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-y+0.5;         SpaceGroupElement[9].y=x+0.5;          SpaceGroupElement[9].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[10].x=-x+0.5;        SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[11].x=y+0.5;         SpaceGroupElement[11].y=-x+0.5;        SpaceGroupElement[11].z=z+0.5;          // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[13].x=-y+0.5;        SpaceGroupElement[13].y=-x+0.5;        SpaceGroupElement[13].z=z+0.5;          // -2       [1,1,0]         0,0,1/2         1/2,0,0
      SpaceGroupElement[14].x=x+0.5;         SpaceGroupElement[14].y=-y+0.5;        SpaceGroupElement[14].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[15].x=y+0.5;         SpaceGroupElement[15].y=x+0.5;         SpaceGroupElement[15].z=z+0.5;          // -2       [-1,1,0]        1/2,1/2,1/2     0,0,0
      break;

    case 385: // [108] I 4 c m (I 4 -2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z+0.5;           // -2       [1,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z+0.5;           // -2       [-1,1,0]        0,0,1/2         0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-y+0.5;         SpaceGroupElement[9].y=x+0.5;          SpaceGroupElement[9].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[10].x=-x+0.5;        SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[11].x=y+0.5;         SpaceGroupElement[11].y=-x+0.5;        SpaceGroupElement[11].z=z+0.5;          // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z;              // -2       [1,0,0]         0,1/2,1         1/4,0,0
      SpaceGroupElement[13].x=-y+0.5;        SpaceGroupElement[13].y=-x+0.5;        SpaceGroupElement[13].z=z;              // -2       [1,1,0]         0,0,1           1/2,0,0
      SpaceGroupElement[14].x=x+0.5;         SpaceGroupElement[14].y=-y+0.5;        SpaceGroupElement[14].z=z;              // -2       [0,1,0]         1/2,0,1         0,1/4,0
      SpaceGroupElement[15].x=y+0.5;         SpaceGroupElement[15].y=x+0.5;         SpaceGroupElement[15].z=z;              // -2       [-1,1,0]        1/2,1/2,1       0,0,0
      break;

    case 386: // [109] I 41 m d (I 4bw -2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         -1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         1/4,-1/4,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x+0.5;         SpaceGroupElement[5].z=z+0.25;          // -2       [1,1,0]         -1/4,1/4,1/4    1/4,0,0
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[7].x=y+0.5;          SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z+0.75;          // -2       [-1,1,0]        1/4,1/4,3/4     1/4,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-y+0.5;         SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=z+0.75;          // 4^1      [0,0,1]         0,0,3/4         -1/4,3/4,0
      SpaceGroupElement[10].x=-x;            SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=z;              // 2        [0,0,1]         0,0,1           1/2,1/2,0
      SpaceGroupElement[11].x=y;             SpaceGroupElement[11].y=-x+0.5;        SpaceGroupElement[11].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         3/4,-1/4,0
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[13].x=-y+0.5;        SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z+0.75;         // -2       [1,1,0]         -1/4,1/4,3/4    3/4,0,0
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z;              // -2       [0,1,0]         1,0,1           0,1/2,0
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x+0.5;         SpaceGroupElement[15].z=z+0.25;         // -2       [-1,1,0]        3/4,3/4,5/4     1/4,0,0
      break;

    case 387: // [110] I 41 c d (I 4bw -2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         -1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         1/4,-1/4,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x+0.5;         SpaceGroupElement[5].z=z+0.75;          // -2       [1,1,0]         -1/4,1/4,3/4    1/4,0,0
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[7].x=y+0.5;          SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z+0.25;          // -2       [-1,1,0]        1/4,1/4,1/4     1/4,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=-y+0.5;         SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=z+0.75;          // 4^1      [0,0,1]         0,0,3/4         -1/4,3/4,0
      SpaceGroupElement[10].x=-x;            SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=z;              // 2        [0,0,1]         0,0,1           1/2,1/2,0
      SpaceGroupElement[11].x=y;             SpaceGroupElement[11].y=-x+0.5;        SpaceGroupElement[11].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         3/4,-1/4,0
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z;              // -2       [1,0,0]         0,1/2,1         1/4,0,0
      SpaceGroupElement[13].x=-y+0.5;        SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z+0.25;         // -2       [1,1,0]         -1/4,1/4,5/4    3/4,0,0
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z+0.5;          // -2       [0,1,0]         1,0,1/2         0,1/2,0
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x+0.5;         SpaceGroupElement[15].z=z+0.75;         // -2       [-1,1,0]        3/4,3/4,3/4     1/4,0,0
      break;

    case 388: // [111] P -4 2 m (P -4 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z;               // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z;               // -2       [-1,1,0]        0,0,0           0,0,0
      break;

    case 389: // [112] P -4 2 c (P -4 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z+0.5;           // -2       [1,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z+0.5;           // -2       [-1,1,0]        0,0,1/2         0,0,0
      break;

    case 390: // [113] P -4 21 m (P -4 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[5].x=-y+0.5;         SpaceGroupElement[5].y=-x+0.5;         SpaceGroupElement[5].z=z;               // -2       [1,1,0]         0,0,0           1/2,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[7].x=y+0.5;          SpaceGroupElement[7].y=x+0.5;          SpaceGroupElement[7].z=z;               // -2       [-1,1,0]        1/2,1/2,0       0,0,0
      break;

    case 391: // [114] P -4 21 c (P -4 2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[5].x=-y+0.5;         SpaceGroupElement[5].y=-x+0.5;         SpaceGroupElement[5].z=z+0.5;           // -2       [1,1,0]         0,0,1/2         1/2,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[7].x=y+0.5;          SpaceGroupElement[7].y=x+0.5;          SpaceGroupElement[7].z=z+0.5;           // -2       [-1,1,0]        1/2,1/2,1/2     0,0,0
      break;

    case 392: // [115] P -4 m 2 (P -4 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      break;

    case 393: // [116] P -4 c 2 (P -4 -2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           0,0,1/4
      break;

    case 394: // [117] P -4 b 2 (P -4 -2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+0.5;         SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z;               // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[5].x=y+0.5;          SpaceGroupElement[5].y=x+0.5;          SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         1/2,1/2,0       0,0,0
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=z;               // -2       [0,1,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[7].x=-y+0.5;         SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           1/2,0,0
      break;

    case 395: // [118] P -4 n 2 (P -4 -2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+0.5;         SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=z+0.5;           // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[5].x=y+0.5;          SpaceGroupElement[5].y=x+0.5;          SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=x+0.5;          SpaceGroupElement[6].y=-y+0.5;         SpaceGroupElement[6].z=z+0.5;           // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[7].x=-y+0.5;         SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      break;

    case 396: // [119] I -4 m 2 (I -4 -2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=y+0.5;          SpaceGroupElement[9].y=-x+0.5;         SpaceGroupElement[9].z=-z+0.5;          // -4^1     [0,0,1]         0,0,0           1/2,0,1/4
      SpaceGroupElement[10].x=-x+0.5;        SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[11].x=-y+0.5;        SpaceGroupElement[11].y=x+0.5;         SpaceGroupElement[11].z=-z+0.5;         // -4^-1    [0,0,1]         0,0,0           0,1/2,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[13].x=y+0.5;         SpaceGroupElement[13].y=x+0.5;         SpaceGroupElement[13].z=-z+0.5;         // 2        [1,1,0]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[14].x=x+0.5;         SpaceGroupElement[14].y=-y+0.5;        SpaceGroupElement[14].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[15].x=-y+0.5;        SpaceGroupElement[15].y=-x+0.5;        SpaceGroupElement[15].z=-z+0.5;         // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      break;

    case 397: // [120] I -4 c 2 (I -4 -2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           0,0,1/4
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=y+0.5;          SpaceGroupElement[9].y=-x+0.5;         SpaceGroupElement[9].z=-z+0.5;          // -4^1     [0,0,1]         0,0,0           1/2,0,1/4
      SpaceGroupElement[10].x=-x+0.5;        SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[11].x=-y+0.5;        SpaceGroupElement[11].y=x+0.5;         SpaceGroupElement[11].z=-z+0.5;         // -4^-1    [0,0,1]         0,0,0           0,1/2,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z;              // -2       [1,0,0]         0,1/2,1         1/4,0,0
      SpaceGroupElement[13].x=y+0.5;         SpaceGroupElement[13].y=x+0.5;         SpaceGroupElement[13].z=-z;             // 2        [1,1,0]         1/2,1/2,0       0,0,1/2
      SpaceGroupElement[14].x=x+0.5;         SpaceGroupElement[14].y=-y+0.5;        SpaceGroupElement[14].z=z;              // -2       [0,1,0]         1/2,0,1         0,1/4,0
      SpaceGroupElement[15].x=-y+0.5;        SpaceGroupElement[15].y=-x+0.5;        SpaceGroupElement[15].z=-z;             // 2        [-1,1,0]        0,0,0           1/2,0,1/2
      break;

    case 398: // [121] I -4 2 m (I -4 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z;               // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z;               // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=y+0.5;          SpaceGroupElement[9].y=-x+0.5;         SpaceGroupElement[9].z=-z+0.5;          // -4^1     [0,0,1]         0,0,0           1/2,0,1/4
      SpaceGroupElement[10].x=-x+0.5;        SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[11].x=-y+0.5;        SpaceGroupElement[11].y=x+0.5;         SpaceGroupElement[11].z=-z+0.5;         // -4^-1    [0,0,1]         0,0,0           0,1/2,1/4
      SpaceGroupElement[12].x=x+0.5;         SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[13].x=-y+0.5;        SpaceGroupElement[13].y=-x+0.5;        SpaceGroupElement[13].z=z+0.5;          // -2       [1,1,0]         0,0,1/2         1/2,0,0
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[15].x=y+0.5;         SpaceGroupElement[15].y=x+0.5;         SpaceGroupElement[15].z=z+0.5;          // -2       [-1,1,0]        1/2,1/2,1/2     0,0,0
      break;

    case 399: // [122] I -4 2 d (I -4 2bw)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.25;         // 2        [1,0,0]         0,0,0           0,1/4,1/8
      SpaceGroupElement[5].x=-y+0.5;         SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z+0.75;          // -2       [1,1,0]         1/4,-1/4,3/4    1/4,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.25;         // 2        [0,1,0]         0,1/2,0         0,0,1/8
      SpaceGroupElement[7].x=y+0.5;          SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z+0.75;          // -2       [-1,1,0]        1/4,1/4,3/4     1/4,0,0
      SpaceGroupElement[8].x=x+0.5;          SpaceGroupElement[8].y=y+0.5;          SpaceGroupElement[8].z=z+0.5;           // 1        -               -               -
      SpaceGroupElement[9].x=y+0.5;          SpaceGroupElement[9].y=-x+0.5;         SpaceGroupElement[9].z=-z+0.5;          // -4^1     [0,0,1]         0,0,0           1/2,0,1/4
      SpaceGroupElement[10].x=-x+0.5;        SpaceGroupElement[10].y=-y+0.5;        SpaceGroupElement[10].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[11].x=-y+0.5;        SpaceGroupElement[11].y=x+0.5;         SpaceGroupElement[11].z=-z+0.5;         // -4^-1    [0,0,1]         0,0,0           0,1/2,1/4
      SpaceGroupElement[12].x=x+0.5;         SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z+0.75;        // 2        [1,0,0]         1/2,0,0         0,1/2,3/8
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x+0.5;        SpaceGroupElement[13].z=z+0.25;         // -2       [1,1,0]         1/4,-1/4,5/4    3/4,0,0
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=-z+0.75;        // 2        [0,1,0]         0,1,0           1/4,0,3/8
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x+0.5;         SpaceGroupElement[15].z=z+0.25;         // -2       [-1,1,0]        3/4,3/4,5/4     1/4,0,0
      break;

    case 400: // [123] P 4/m m m (-P 4 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x;             SpaceGroupElement[11].z=-z;             // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z;              // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z;              // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z;              // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z;              // -2       [-1,1,0]        0,0,0           0,0,0
      break;

    case 401: // [124] P 4/m c c (-P 4 2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           0,0,1/4
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x;             SpaceGroupElement[11].z=-z;             // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z-0.5;          // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z-0.5;          // -2       [1,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z-0.5;          // -2       [0,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z-0.5;          // -2       [-1,1,0]        0,0,-1/2        0,0,0
      break;

    case 402: // [125] P 4/n b m Origin choice 1 (P 4 2 -1ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=-x+0.5;         SpaceGroupElement[8].y=-y+0.5;         SpaceGroupElement[8].z=-z;              // -1       -               -               1/4,1/4,0
      SpaceGroupElement[9].x=y+0.5;          SpaceGroupElement[9].y=-x+0.5;         SpaceGroupElement[9].z=-z;              // -4^1     [0,0,1]         0,0,0           1/2,0,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=y+0.5;         SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[11].x=-y+0.5;        SpaceGroupElement[11].y=x+0.5;         SpaceGroupElement[11].z=-z;             // -4^-1    [0,0,1]         0,0,0           0,1/2,0
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z;              // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[13].x=-y+0.5;        SpaceGroupElement[13].y=-x+0.5;        SpaceGroupElement[13].z=z;              // -2       [1,1,0]         0,0,0           1/2,0,0
      SpaceGroupElement[14].x=x+0.5;         SpaceGroupElement[14].y=-y+0.5;        SpaceGroupElement[14].z=z;              // -2       [0,1,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[15].x=y+0.5;         SpaceGroupElement[15].y=x+0.5;         SpaceGroupElement[15].z=z;              // -2       [-1,1,0]        1/2,1/2,0       0,0,0
      break;
    case 403: // [125] P 4/n b m Origin choice 2 (-P 4a 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[7].x=-y+0.5;         SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           1/2,0,0
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y-0.5;          SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z;              // -4^1     [0,0,1]         0,0,0           -1/4,1/4,0
      SpaceGroupElement[10].x=x-0.5;         SpaceGroupElement[10].y=y-0.5;         SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x-0.5;         SpaceGroupElement[11].z=-z;             // -4^-1    [0,0,1]         0,0,0           1/4,-1/4,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y-0.5;         SpaceGroupElement[12].z=z;              // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z;              // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[14].x=x-0.5;         SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z;              // -2       [0,1,0]         -1/2,0,0        0,0,0
      SpaceGroupElement[15].x=y-0.5;         SpaceGroupElement[15].y=x-0.5;         SpaceGroupElement[15].z=z;              // -2       [-1,1,0]        -1/2,-1/2,0     0,0,0
      break;

    case 404: // [126] P 4/n n c Origin choice 1 (P 4 2 -1n)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=-x+0.5;         SpaceGroupElement[8].y=-y+0.5;         SpaceGroupElement[8].z=-z+0.5;          // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[9].x=y+0.5;          SpaceGroupElement[9].y=-x+0.5;         SpaceGroupElement[9].z=-z+0.5;          // -4^1     [0,0,1]         0,0,0           1/2,0,1/4
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=y+0.5;         SpaceGroupElement[10].z=-z+0.5;         // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[11].x=-y+0.5;        SpaceGroupElement[11].y=x+0.5;         SpaceGroupElement[11].z=-z+0.5;         // -4^-1    [0,0,1]         0,0,0           0,1/2,1/4
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[13].x=-y+0.5;        SpaceGroupElement[13].y=-x+0.5;        SpaceGroupElement[13].z=z+0.5;          // -2       [1,1,0]         0,0,1/2         1/2,0,0
      SpaceGroupElement[14].x=x+0.5;         SpaceGroupElement[14].y=-y+0.5;        SpaceGroupElement[14].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[15].x=y+0.5;         SpaceGroupElement[15].y=x+0.5;         SpaceGroupElement[15].z=z+0.5;          // -2       [-1,1,0]        1/2,1/2,1/2     0,0,0
      break;
    case 405: // [126] P 4/n n c Origin choice 2 (-P 4a 2bc)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[7].x=-y+0.5;         SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y-0.5;          SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z;              // -4^1     [0,0,1]         0,0,0           -1/4,1/4,0
      SpaceGroupElement[10].x=x-0.5;         SpaceGroupElement[10].y=y-0.5;         SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x-0.5;         SpaceGroupElement[11].z=-z;             // -4^-1    [0,0,1]         0,0,0           1/4,-1/4,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y-0.5;         SpaceGroupElement[12].z=z-0.5;          // -2       [1,0,0]         0,-1/2,-1/2     0,0,0
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z-0.5;          // -2       [1,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[14].x=x-0.5;         SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z-0.5;          // -2       [0,1,0]         -1/2,0,-1/2     0,0,0
      SpaceGroupElement[15].x=y-0.5;         SpaceGroupElement[15].y=x-0.5;         SpaceGroupElement[15].z=z-0.5;          // -2       [-1,1,0]        -1/2,-1/2,-1/2  0,0,0
      break;

    case 406: // [127] P 4/m b m (-P 4 2ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[5].x=y+0.5;          SpaceGroupElement[5].y=x+0.5;          SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         1/2,1/2,0       0,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[7].x=-y+0.5;         SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           1/2,0,0
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x;             SpaceGroupElement[11].z=-z;             // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[12].x=-x-0.5;        SpaceGroupElement[12].y=y-0.5;         SpaceGroupElement[12].z=z;              // -2       [1,0,0]         0,-1/2,0        -1/4,0,0
      SpaceGroupElement[13].x=-y-0.5;        SpaceGroupElement[13].y=-x-0.5;        SpaceGroupElement[13].z=z;              // -2       [1,1,0]         0,0,0           -1/2,0,0
      SpaceGroupElement[14].x=x-0.5;         SpaceGroupElement[14].y=-y-0.5;        SpaceGroupElement[14].z=z;              // -2       [0,1,0]         -1/2,0,0        0,-1/4,0
      SpaceGroupElement[15].x=y-0.5;         SpaceGroupElement[15].y=x-0.5;         SpaceGroupElement[15].z=z;              // -2       [-1,1,0]        -1/2,-1/2,0     0,0,0
      break;

    case 407: // [128] P 4/m n c (-P 4 2n)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[5].x=y+0.5;          SpaceGroupElement[5].y=x+0.5;          SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[7].x=-y+0.5;         SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x;             SpaceGroupElement[11].z=-z;             // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[12].x=-x-0.5;        SpaceGroupElement[12].y=y-0.5;         SpaceGroupElement[12].z=z-0.5;          // -2       [1,0,0]         0,-1/2,-1/2     -1/4,0,0
      SpaceGroupElement[13].x=-y-0.5;        SpaceGroupElement[13].y=-x-0.5;        SpaceGroupElement[13].z=z-0.5;          // -2       [1,1,0]         0,0,-1/2        -1/2,0,0
      SpaceGroupElement[14].x=x-0.5;         SpaceGroupElement[14].y=-y-0.5;        SpaceGroupElement[14].z=z-0.5;          // -2       [0,1,0]         -1/2,0,-1/2     0,-1/4,0
      SpaceGroupElement[15].x=y-0.5;         SpaceGroupElement[15].y=x-0.5;         SpaceGroupElement[15].z=z-0.5;          // -2       [-1,1,0]        -1/2,-1/2,-1/2  0,0,0
      break;

    case 408: // [129] P 4/n m m Origin choice 1 (P 4ab 2ab -1ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,1/2,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           1/2,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=-x+0.5;         SpaceGroupElement[8].y=-y+0.5;         SpaceGroupElement[8].z=-z;              // -1       -               -               1/4,1/4,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=y+0.5;         SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x;             SpaceGroupElement[11].z=-z;             // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z;              // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[13].x=-y+0.5;        SpaceGroupElement[13].y=-x+0.5;        SpaceGroupElement[13].z=z;              // -2       [1,1,0]         0,0,0           1/2,0,0
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z;              // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[15].x=y+0.5;         SpaceGroupElement[15].y=x+0.5;         SpaceGroupElement[15].z=z;              // -2       [-1,1,0]        1/2,1/2,0       0,0,0
      break;
    case 409: // [129] P 4/n m m Origin choice 2 (-P 4a 2a)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[5].x=y+0.5;          SpaceGroupElement[5].y=x+0.5;          SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         1/2,1/2,0       0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y-0.5;          SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z;              // -4^1     [0,0,1]         0,0,0           -1/4,1/4,0
      SpaceGroupElement[10].x=x-0.5;         SpaceGroupElement[10].y=y-0.5;         SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x-0.5;         SpaceGroupElement[11].z=-z;             // -4^-1    [0,0,1]         0,0,0           1/4,-1/4,0
      SpaceGroupElement[12].x=-x-0.5;        SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z;              // -2       [1,0,0]         0,0,0           -1/4,0,0
      SpaceGroupElement[13].x=-y-0.5;        SpaceGroupElement[13].y=-x-0.5;        SpaceGroupElement[13].z=z;              // -2       [1,1,0]         0,0,0           -1/2,0,0
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y-0.5;        SpaceGroupElement[14].z=z;              // -2       [0,1,0]         0,0,0           0,-1/4,0
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z;              // -2       [-1,1,0]        0,0,0           0,0,0
      break;

    case 410: // [130] P 4/n c c Origin choice 1 (P 4ab 2n -1ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,1/2,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           1/2,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           0,0,1/4
      SpaceGroupElement[8].x=-x+0.5;         SpaceGroupElement[8].y=-y+0.5;         SpaceGroupElement[8].z=-z;              // -1       -               -               1/4,1/4,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=y+0.5;         SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x;             SpaceGroupElement[11].z=-z;             // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z-0.5;          // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[13].x=-y+0.5;        SpaceGroupElement[13].y=-x+0.5;        SpaceGroupElement[13].z=z-0.5;          // -2       [1,1,0]         0,0,-1/2        1/2,0,0
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z-0.5;          // -2       [0,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[15].x=y+0.5;         SpaceGroupElement[15].y=x+0.5;         SpaceGroupElement[15].z=z-0.5;          // -2       [-1,1,0]        1/2,1/2,-1/2    0,0,0
      break;
    case 411: // [130] P 4/n c c Origin choice 2 (-P 4a 2ac)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[5].x=y+0.5;          SpaceGroupElement[5].y=x+0.5;          SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           0,0,1/4
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y-0.5;          SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z;              // -4^1     [0,0,1]         0,0,0           -1/4,1/4,0
      SpaceGroupElement[10].x=x-0.5;         SpaceGroupElement[10].y=y-0.5;         SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x-0.5;         SpaceGroupElement[11].z=-z;             // -4^-1    [0,0,1]         0,0,0           1/4,-1/4,0
      SpaceGroupElement[12].x=-x-0.5;        SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z-0.5;          // -2       [1,0,0]         0,0,-1/2        -1/4,0,0
      SpaceGroupElement[13].x=-y-0.5;        SpaceGroupElement[13].y=-x-0.5;        SpaceGroupElement[13].z=z-0.5;          // -2       [1,1,0]         0,0,-1/2        -1/2,0,0
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y-0.5;        SpaceGroupElement[14].z=z-0.5;          // -2       [0,1,0]         0,0,-1/2        0,-1/4,0
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z-0.5;          // -2       [-1,1,0]        0,0,-1/2        0,0,0
      break;

    case 412: // [131] P 42/m m c (-P 4c 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           0,0,1/4
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z-0.5;          // -4^1     [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x;             SpaceGroupElement[11].z=-z-0.5;         // -4^-1    [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z;              // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z-0.5;          // -2       [1,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z;              // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z-0.5;          // -2       [-1,1,0]        0,0,-1/2        0,0,0
      break;

    case 413: // [132] P 42/m c m (-P 4c 2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z-0.5;          // -4^1     [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x;             SpaceGroupElement[11].z=-z-0.5;         // -4^-1    [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z-0.5;          // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z;              // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z-0.5;          // -2       [0,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z;              // -2       [-1,1,0]        0,0,0           0,0,0
      break;

    case 414: // [133] P 42/n b c Origin choice 1 (P 4n 2c -1n)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[5].x=y+0.5;          SpaceGroupElement[5].y=x+0.5;          SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         1/2,1/2,0       0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[7].x=-y+0.5;         SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           1/2,0,0
      SpaceGroupElement[8].x=-x+0.5;         SpaceGroupElement[8].y=-y+0.5;         SpaceGroupElement[8].z=-z+0.5;          // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=y+0.5;         SpaceGroupElement[10].z=-z+0.5;         // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x;             SpaceGroupElement[11].z=-z;             // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z;              // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z+0.5;          // -2       [1,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[14].x=x+0.5;         SpaceGroupElement[14].y=-y+0.5;        SpaceGroupElement[14].z=z;              // -2       [0,1,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z+0.5;          // -2       [-1,1,0]        0,0,1/2         0,0,0
      break;
    case 415: // [133] P 42/n b c Origin choice 2 (-P 4ac 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,1/4,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[7].x=-y+0.5;         SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y-0.5;          SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z-0.5;          // -4^1     [0,0,1]         0,0,0           -1/4,1/4,-1/4
      SpaceGroupElement[10].x=x-0.5;         SpaceGroupElement[10].y=y-0.5;         SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x-0.5;         SpaceGroupElement[11].z=-z-0.5;         // -4^-1    [0,0,1]         0,0,0           1/4,-1/4,-1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y-0.5;         SpaceGroupElement[12].z=z;              // -2       [1,0,0]         0,-1/2,0        0,0,0
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z-0.5;          // -2       [1,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[14].x=x-0.5;         SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z;              // -2       [0,1,0]         -1/2,0,0        0,0,0
      SpaceGroupElement[15].x=y-0.5;         SpaceGroupElement[15].y=x-0.5;         SpaceGroupElement[15].z=z-0.5;          // -2       [-1,1,0]        -1/2,-1/2,-1/2  0,0,0
      break;

    case 416: // [134] P 42/n n m Origin choice 1 (P 4n 2 -1n)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y+0.5;          SpaceGroupElement[5].y=x+0.5;          SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y+0.5;         SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      SpaceGroupElement[8].x=-x+0.5;         SpaceGroupElement[8].y=-y+0.5;         SpaceGroupElement[8].z=-z+0.5;          // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=y+0.5;         SpaceGroupElement[10].z=-z+0.5;         // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x;             SpaceGroupElement[11].z=-z;             // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z;              // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[14].x=x+0.5;         SpaceGroupElement[14].y=-y+0.5;        SpaceGroupElement[14].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z;              // -2       [-1,1,0]        0,0,0           0,0,0
      break;
    case 417: // [134] P 42/n n m Origin choice 2 (-P 4ac 2bc)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[7].x=-y+0.5;         SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           1/2,0,0
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y-0.5;          SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z-0.5;          // -4^1     [0,0,1]         0,0,0           -1/4,1/4,-1/4
      SpaceGroupElement[10].x=x-0.5;         SpaceGroupElement[10].y=y-0.5;         SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x-0.5;         SpaceGroupElement[11].z=-z-0.5;         // -4^-1    [0,0,1]         0,0,0           1/4,-1/4,-1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y-0.5;         SpaceGroupElement[12].z=z-0.5;          // -2       [1,0,0]         0,-1/2,-1/2     0,0,0
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z;              // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[14].x=x-0.5;         SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z-0.5;          // -2       [0,1,0]         -1/2,0,-1/2     0,0,0
      SpaceGroupElement[15].x=y-0.5;         SpaceGroupElement[15].y=x-0.5;         SpaceGroupElement[15].z=z;              // -2       [-1,1,0]        -1/2,-1/2,0     0,0,0
      break;

    case 418: // [135] P 42/m b c (-P 4c 2ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[5].x=y+0.5;          SpaceGroupElement[5].y=x+0.5;          SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[7].x=-y+0.5;         SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z-0.5;          // -4^1     [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x;             SpaceGroupElement[11].z=-z-0.5;         // -4^-1    [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[12].x=-x-0.5;        SpaceGroupElement[12].y=y-0.5;         SpaceGroupElement[12].z=z;              // -2       [1,0,0]         0,-1/2,0        -1/4,0,0
      SpaceGroupElement[13].x=-y-0.5;        SpaceGroupElement[13].y=-x-0.5;        SpaceGroupElement[13].z=z-0.5;          // -2       [1,1,0]         0,0,-1/2        -1/2,0,0
      SpaceGroupElement[14].x=x-0.5;         SpaceGroupElement[14].y=-y-0.5;        SpaceGroupElement[14].z=z;              // -2       [0,1,0]         -1/2,0,0        0,-1/4,0
      SpaceGroupElement[15].x=y-0.5;         SpaceGroupElement[15].y=x-0.5;         SpaceGroupElement[15].z=z-0.5;          // -2       [-1,1,0]        -1/2,-1/2,-1/2  0,0,0
      break;

    case 419: // [136] P 42/m n m (-P 4n 2n)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y-0.5;          SpaceGroupElement[9].y=-x-0.5;         SpaceGroupElement[9].z=-z-0.5;          // -4^1     [0,0,1]         0,0,0           -1/2,0,-1/4
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-y-0.5;        SpaceGroupElement[11].y=x-0.5;         SpaceGroupElement[11].z=-z-0.5;         // -4^-1    [0,0,1]         0,0,0           0,-1/2,-1/4
      SpaceGroupElement[12].x=-x-0.5;        SpaceGroupElement[12].y=y-0.5;         SpaceGroupElement[12].z=z-0.5;          // -2       [1,0,0]         0,-1/2,-1/2     -1/4,0,0
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z;              // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[14].x=x-0.5;         SpaceGroupElement[14].y=-y-0.5;        SpaceGroupElement[14].z=z-0.5;          // -2       [0,1,0]         -1/2,0,-1/2     0,-1/4,0
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z;              // -2       [-1,1,0]        0,0,0           0,0,0
      break;

    case 420: // [137] P 42/n m c Origin choice 1 (P 4n 2n -1n)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=-x+0.5;         SpaceGroupElement[8].y=-y+0.5;         SpaceGroupElement[8].z=-z+0.5;          // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=y+0.5;         SpaceGroupElement[10].z=-z+0.5;         // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x;             SpaceGroupElement[11].z=-z;             // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z;              // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[13].x=-y+0.5;        SpaceGroupElement[13].y=-x+0.5;        SpaceGroupElement[13].z=z+0.5;          // -2       [1,1,0]         0,0,1/2         1/2,0,0
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z;              // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[15].x=y+0.5;         SpaceGroupElement[15].y=x+0.5;         SpaceGroupElement[15].z=z+0.5;          // -2       [-1,1,0]        1/2,1/2,1/2     0,0,0
      break;
    case 421: // [137] P 42/n m c Origin choice 2 (-P 4ac 2a)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[5].x=y+0.5;          SpaceGroupElement[5].y=x+0.5;          SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,1/2,0         0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           0,0,1/4
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y-0.5;          SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z-0.5;          // -4^1     [0,0,1]         0,0,0           -1/4,1/4,-1/4
      SpaceGroupElement[10].x=x-0.5;         SpaceGroupElement[10].y=y-0.5;         SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x-0.5;         SpaceGroupElement[11].z=-z-0.5;         // -4^-1    [0,0,1]         0,0,0           1/4,-1/4,-1/4
      SpaceGroupElement[12].x=-x-0.5;        SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z;              // -2       [1,0,0]         0,0,0           -1/4,0,0
      SpaceGroupElement[13].x=-y-0.5;        SpaceGroupElement[13].y=-x-0.5;        SpaceGroupElement[13].z=z-0.5;          // -2       [1,1,0]         0,0,-1/2        -1/2,0,0
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y-0.5;        SpaceGroupElement[14].z=z;              // -2       [0,1,0]         0,0,0           0,-1/4,0
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z-0.5;          // -2       [-1,1,0]        0,0,-1/2        0,0,0
      break;

    case 422: // [138] P 42/n c m Origin choice 1 (P 4n 2ab -1n)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           0,0,1/4
      SpaceGroupElement[8].x=-x+0.5;         SpaceGroupElement[8].y=-y+0.5;         SpaceGroupElement[8].z=-z+0.5;          // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x+0.5;         SpaceGroupElement[10].y=y+0.5;         SpaceGroupElement[10].z=-z+0.5;         // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x;             SpaceGroupElement[11].z=-z;             // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z+0.5;          // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[13].x=-y+0.5;        SpaceGroupElement[13].y=-x+0.5;        SpaceGroupElement[13].z=z;              // -2       [1,1,0]         0,0,0           1/2,0,0
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[15].x=y+0.5;         SpaceGroupElement[15].y=x+0.5;         SpaceGroupElement[15].z=z;              // -2       [-1,1,0]        1/2,1/2,0       0,0,0
      break;
    case 423: // [138] P 42/n c m Origin choice 2 (-P 4ac 2ac)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[5].x=y+0.5;          SpaceGroupElement[5].y=x+0.5;          SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         1/2,1/2,0       0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y-0.5;          SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z-0.5;          // -4^1     [0,0,1]         0,0,0           -1/4,1/4,-1/4
      SpaceGroupElement[10].x=x-0.5;         SpaceGroupElement[10].y=y-0.5;         SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x-0.5;         SpaceGroupElement[11].z=-z-0.5;         // -4^-1    [0,0,1]         0,0,0           1/4,-1/4,-1/4
      SpaceGroupElement[12].x=-x-0.5;        SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z-0.5;          // -2       [1,0,0]         0,0,-1/2        -1/4,0,0
      SpaceGroupElement[13].x=-y-0.5;        SpaceGroupElement[13].y=-x-0.5;        SpaceGroupElement[13].z=z;              // -2       [1,1,0]         0,0,0           -1/2,0,0
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y-0.5;        SpaceGroupElement[14].z=z-0.5;          // -2       [0,1,0]         0,0,-1/2        0,-1/4,0
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z;              // -2       [-1,1,0]        0,0,0           0,0,0
      break;

    case 424: // [139] I 4/m m m (-I 4 2)
      SpaceGroupSize=32;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x;             SpaceGroupElement[11].z=-z;             // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z;              // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z;              // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z;              // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z;              // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[16].x=x+0.5;         SpaceGroupElement[16].y=y+0.5;         SpaceGroupElement[16].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[17].x=-y+0.5;        SpaceGroupElement[17].y=x+0.5;         SpaceGroupElement[17].z=z+0.5;          // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[18].x=-x+0.5;        SpaceGroupElement[18].y=-y+0.5;        SpaceGroupElement[18].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[19].x=y+0.5;         SpaceGroupElement[19].y=-x+0.5;        SpaceGroupElement[19].z=z+0.5;          // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[20].x=x+0.5;         SpaceGroupElement[20].y=-y+0.5;        SpaceGroupElement[20].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[21].x=y+0.5;         SpaceGroupElement[21].y=x+0.5;         SpaceGroupElement[21].z=-z+0.5;         // 2        [1,1,0]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[22].x=-x+0.5;        SpaceGroupElement[22].y=y+0.5;         SpaceGroupElement[22].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[23].x=-y+0.5;        SpaceGroupElement[23].y=-x+0.5;        SpaceGroupElement[23].z=-z+0.5;         // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      SpaceGroupElement[24].x=-x+0.5;        SpaceGroupElement[24].y=-y+0.5;        SpaceGroupElement[24].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[25].x=y+0.5;         SpaceGroupElement[25].y=-x+0.5;        SpaceGroupElement[25].z=-z+0.5;         // -4^1     [0,0,1]         0,0,0           1/2,0,1/4
      SpaceGroupElement[26].x=x+0.5;         SpaceGroupElement[26].y=y+0.5;         SpaceGroupElement[26].z=-z+0.5;         // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[27].x=-y+0.5;        SpaceGroupElement[27].y=x+0.5;         SpaceGroupElement[27].z=-z+0.5;         // -4^-1    [0,0,1]         0,0,0           0,1/2,1/4
      SpaceGroupElement[28].x=-x+0.5;        SpaceGroupElement[28].y=y+0.5;         SpaceGroupElement[28].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[29].x=-y+0.5;        SpaceGroupElement[29].y=-x+0.5;        SpaceGroupElement[29].z=z+0.5;          // -2       [1,1,0]         0,0,1/2         1/2,0,0
      SpaceGroupElement[30].x=x+0.5;         SpaceGroupElement[30].y=-y+0.5;        SpaceGroupElement[30].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[31].x=y+0.5;         SpaceGroupElement[31].y=x+0.5;         SpaceGroupElement[31].z=z+0.5;          // -2       [-1,1,0]        1/2,1/2,1/2     0,0,0
      break;

    case 425: // [140] I 4/m c m (-I 4 2c)
      SpaceGroupSize=32;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           0,0,1/4
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;             // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x;             SpaceGroupElement[11].z=-z;             // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z-0.5;          // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z-0.5;          // -2       [1,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z-0.5;          // -2       [0,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z-0.5;          // -2       [-1,1,0]        0,0,-1/2        0,0,0
      SpaceGroupElement[16].x=x+0.5;         SpaceGroupElement[16].y=y+0.5;         SpaceGroupElement[16].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[17].x=-y+0.5;        SpaceGroupElement[17].y=x+0.5;         SpaceGroupElement[17].z=z+0.5;          // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[18].x=-x+0.5;        SpaceGroupElement[18].y=-y+0.5;        SpaceGroupElement[18].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[19].x=y+0.5;         SpaceGroupElement[19].y=-x+0.5;        SpaceGroupElement[19].z=z+0.5;          // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[20].x=x+0.5;         SpaceGroupElement[20].y=-y+0.5;        SpaceGroupElement[20].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,1/2
      SpaceGroupElement[21].x=y+0.5;         SpaceGroupElement[21].y=x+0.5;         SpaceGroupElement[21].z=-z;             // 2        [1,1,0]         1/2,1/2,0       0,0,1/2
      SpaceGroupElement[22].x=-x+0.5;        SpaceGroupElement[22].y=y+0.5;         SpaceGroupElement[22].z=-z;             // 2        [0,1,0]         0,1/2,0         1/4,0,1/2
      SpaceGroupElement[23].x=-y+0.5;        SpaceGroupElement[23].y=-x+0.5;        SpaceGroupElement[23].z=-z;             // 2        [-1,1,0]        0,0,0           1/2,0,1/2
      SpaceGroupElement[24].x=-x+0.5;        SpaceGroupElement[24].y=-y+0.5;        SpaceGroupElement[24].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[25].x=y+0.5;         SpaceGroupElement[25].y=-x+0.5;        SpaceGroupElement[25].z=-z+0.5;         // -4^1     [0,0,1]         0,0,0           1/2,0,1/4
      SpaceGroupElement[26].x=x+0.5;         SpaceGroupElement[26].y=y+0.5;         SpaceGroupElement[26].z=-z+0.5;         // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[27].x=-y+0.5;        SpaceGroupElement[27].y=x+0.5;         SpaceGroupElement[27].z=-z+0.5;         // -4^-1    [0,0,1]         0,0,0           0,1/2,1/4
      SpaceGroupElement[28].x=-x+0.5;        SpaceGroupElement[28].y=y+0.5;         SpaceGroupElement[28].z=z;              // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[29].x=-y+0.5;        SpaceGroupElement[29].y=-x+0.5;        SpaceGroupElement[29].z=z;              // -2       [1,1,0]         0,0,0           1/2,0,0
      SpaceGroupElement[30].x=x+0.5;         SpaceGroupElement[30].y=-y+0.5;        SpaceGroupElement[30].z=z;              // -2       [0,1,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[31].x=y+0.5;         SpaceGroupElement[31].y=x+0.5;         SpaceGroupElement[31].z=z;              // -2       [-1,1,0]        1/2,1/2,0       0,0,0
      break;

    case 426: // [141] I 41/a m d Origin choice 1 (I 4bw 2bw -1bw)
      SpaceGroupSize=32;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         -1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         1/4,-1/4,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.25;         // 2        [1,0,0]         0,0,0           0,1/4,1/8
      SpaceGroupElement[5].x=y+0.5;          SpaceGroupElement[5].y=x+0.5;          SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z+0.75;         // 2        [0,1,0]         0,0,0           1/4,0,3/8
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y+0.5;         SpaceGroupElement[8].z=-z+0.25;         // -1       -               -               0,1/4,1/8
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x-0.5;         SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z-0.25;        // -2       [0,0,1]         -1/2,0,0        0,0,-1/8
      SpaceGroupElement[11].x=-y-0.5;        SpaceGroupElement[11].y=x+0.5;         SpaceGroupElement[11].z=-z-0.5;         // -4^-1    [0,0,1]         0,0,0           -1/2,0,-1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z;              // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[13].x=-y-0.5;        SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z-0.25;         // -2       [1,1,0]         -1/4,1/4,-1/4   -1/4,0,0
      SpaceGroupElement[14].x=x-0.5;         SpaceGroupElement[14].y=-y+0.5;        SpaceGroupElement[14].z=z-0.5;          // -2       [0,1,0]         -1/2,0,-1/2     0,1/4,0
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x+0.5;         SpaceGroupElement[15].z=z+0.25;         // -2       [-1,1,0]        1/4,1/4,1/4     -1/4,0,0
      SpaceGroupElement[16].x=x+0.5;         SpaceGroupElement[16].y=y+0.5;         SpaceGroupElement[16].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[17].x=-y+0.5;        SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=z+0.75;         // 4^1      [0,0,1]         0,0,3/4         -1/4,3/4,0
      SpaceGroupElement[18].x=-x;            SpaceGroupElement[18].y=-y;            SpaceGroupElement[18].z=z;              // 2        [0,0,1]         0,0,1           1/2,1/2,0
      SpaceGroupElement[19].x=y;             SpaceGroupElement[19].y=-x+0.5;        SpaceGroupElement[19].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         3/4,-1/4,0
      SpaceGroupElement[20].x=x+0.5;         SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-z+0.75;        // 2        [1,0,0]         1/2,0,0         0,1/2,3/8
      SpaceGroupElement[21].x=y;             SpaceGroupElement[21].y=x;             SpaceGroupElement[21].z=-z;             // 2        [1,1,0]         1,1,0           0,0,1/2
      SpaceGroupElement[22].x=-x;            SpaceGroupElement[22].y=y+0.5;         SpaceGroupElement[22].z=-z+0.25;        // 2        [0,1,0]         0,1/2,0         1/2,0,5/8
      SpaceGroupElement[23].x=-y+0.5;        SpaceGroupElement[23].y=-x+0.5;        SpaceGroupElement[23].z=-z+0.5;         // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      SpaceGroupElement[24].x=-x+0.5;        SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z+0.75;        // -1       -               -               1/4,1/2,3/8
      SpaceGroupElement[25].x=y+0.5;         SpaceGroupElement[25].y=-x+0.5;        SpaceGroupElement[25].z=-z+0.5;         // -4^1     [0,0,1]         0,0,0           1/2,0,1/4
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=y+0.5;         SpaceGroupElement[26].z=-z+0.25;        // -2       [0,0,1]         0,1/2,0         0,0,1/8
      SpaceGroupElement[27].x=-y;            SpaceGroupElement[27].y=x;             SpaceGroupElement[27].z=-z;             // -4^-1    [0,0,1]         0,0,0           -1/2,1/2,0
      SpaceGroupElement[28].x=-x+0.5;        SpaceGroupElement[28].y=y+0.5;         SpaceGroupElement[28].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[29].x=-y;            SpaceGroupElement[29].y=-x+0.5;        SpaceGroupElement[29].z=z+0.25;         // -2       [1,1,0]         -1/4,1/4,1/4    1/4,0,0
      SpaceGroupElement[30].x=x;             SpaceGroupElement[30].y=-y;            SpaceGroupElement[30].z=z;              // -2       [0,1,0]         0,0,0           0,1/2,0
      SpaceGroupElement[31].x=y+0.5;         SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=z+0.75;         // -2       [-1,1,0]        3/4,3/4,3/4     -1/4,0,0
      break;
    case 427: // [141] I 41/a m d Origin choice 2 (-I 4bd 2)
      SpaceGroupSize=32;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.25;        SpaceGroupElement[1].y=x+0.75;         SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         -1/4,1/2,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[3].x=y+0.25;         SpaceGroupElement[3].y=-x+0.25;        SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         1/4,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y+0.25;         SpaceGroupElement[5].y=x+0.75;         SpaceGroupElement[5].z=-z+0.25;         // 2        [1,1,0]         1/2,1/2,0       -1/4,0,1/8
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[7].x=-y+0.25;        SpaceGroupElement[7].y=-x+0.25;        SpaceGroupElement[7].z=-z+0.75;         // 2        [-1,1,0]        0,0,0           1/4,0,3/8
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y-0.25;         SpaceGroupElement[9].y=-x-0.75;        SpaceGroupElement[9].z=-z-0.25;         // -4^1     [0,0,1]         0,0,0           -1/2,-1/4,-1/8
      SpaceGroupElement[10].x=x-0.5;         SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z-0.5;         // -2       [0,0,1]         -1/2,0,0        0,0,-1/4
      SpaceGroupElement[11].x=-y-0.25;       SpaceGroupElement[11].y=x-0.25;        SpaceGroupElement[11].z=-z-0.75;        // -4^-1    [0,0,1]         0,0,0           0,-1/4,-3/8
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z;              // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[13].x=-y-0.25;       SpaceGroupElement[13].y=-x-0.75;       SpaceGroupElement[13].z=z-0.25;         // -2       [1,1,0]         1/4,-1/4,-1/4   -1/2,0,0
      SpaceGroupElement[14].x=x-0.5;         SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z-0.5;          // -2       [0,1,0]         -1/2,0,-1/2     0,0,0
      SpaceGroupElement[15].x=y-0.25;        SpaceGroupElement[15].y=x-0.25;        SpaceGroupElement[15].z=z-0.75;         // -2       [-1,1,0]        -1/4,-1/4,-3/4  0,0,0
      SpaceGroupElement[16].x=x+0.5;         SpaceGroupElement[16].y=y+0.5;         SpaceGroupElement[16].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[17].x=-y+0.75;       SpaceGroupElement[17].y=x+0.25;        SpaceGroupElement[17].z=z+0.75;         // 4^1      [0,0,1]         0,0,3/4         -1/4,1,0
      SpaceGroupElement[18].x=-x;            SpaceGroupElement[18].y=-y+0.5;        SpaceGroupElement[18].z=z;              // 2        [0,0,1]         0,0,1           1/2,1/4,0
      SpaceGroupElement[19].x=y+0.75;        SpaceGroupElement[19].y=-x+0.75;       SpaceGroupElement[19].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         3/4,0,0
      SpaceGroupElement[20].x=x+0.5;         SpaceGroupElement[20].y=-y+0.5;        SpaceGroupElement[20].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[21].x=y+0.75;        SpaceGroupElement[21].y=x+0.25;        SpaceGroupElement[21].z=-z+0.75;        // 2        [1,1,0]         1,1,0           -1/4,0,3/8
      SpaceGroupElement[22].x=-x;            SpaceGroupElement[22].y=y+0.5;         SpaceGroupElement[22].z=-z;             // 2        [0,1,0]         0,1/2,0         1/2,0,1/2
      SpaceGroupElement[23].x=-y+0.75;       SpaceGroupElement[23].y=-x+0.75;       SpaceGroupElement[23].z=-z+0.25;        // 2        [-1,1,0]        0,0,0           3/4,0,5/8
      SpaceGroupElement[24].x=-x+0.5;        SpaceGroupElement[24].y=-y+0.5;        SpaceGroupElement[24].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[25].x=y+0.25;        SpaceGroupElement[25].y=-x-0.25;       SpaceGroupElement[25].z=-z+0.25;        // -4^1     [0,0,1]         0,0,0           0,-1/4,1/8
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=y+0.5;         SpaceGroupElement[26].z=-z;             // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[27].x=-y+0.25;       SpaceGroupElement[27].y=x+0.25;        SpaceGroupElement[27].z=-z-0.25;        // -4^-1    [0,0,1]         0,0,0           0,1/4,-1/8
      SpaceGroupElement[28].x=-x+0.5;        SpaceGroupElement[28].y=y+0.5;         SpaceGroupElement[28].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[29].x=-y+0.25;       SpaceGroupElement[29].y=-x-0.25;       SpaceGroupElement[29].z=z+0.25;         // -2       [1,1,0]         1/4,-1/4,1/4    0,0,0
      SpaceGroupElement[30].x=x;             SpaceGroupElement[30].y=-y+0.5;        SpaceGroupElement[30].z=z;              // -2       [0,1,0]         0,0,0           0,1/4,0
      SpaceGroupElement[31].x=y+0.25;        SpaceGroupElement[31].y=x+0.25;        SpaceGroupElement[31].z=z-0.25;         // -2       [-1,1,0]        1/4,1/4,-1/4    0,0,0
      break;

    case 428: // [142] I 41/a c d Origin choice 1 (I 4bw 2aw -1bw)
      SpaceGroupSize=32;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         -1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         1/4,-1/4,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z+0.25;         // 2        [1,0,0]         1/2,0,0         0,0,1/8
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.75;         // 2        [0,1,0]         0,1/2,0         0,0,3/8
      SpaceGroupElement[7].x=-y+0.5;         SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           1/2,0,0
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y+0.5;         SpaceGroupElement[8].z=-z+0.25;         // -1       -               -               0,1/4,1/8
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x-0.5;         SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z-0.25;        // -2       [0,0,1]         -1/2,0,0        0,0,-1/8
      SpaceGroupElement[11].x=-y-0.5;        SpaceGroupElement[11].y=x+0.5;         SpaceGroupElement[11].z=-z-0.5;         // -4^-1    [0,0,1]         0,0,0           -1/2,0,-1/4
      SpaceGroupElement[12].x=-x-0.5;        SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z;              // -2       [1,0,0]         0,1/2,0         -1/4,0,0
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x+0.5;        SpaceGroupElement[13].z=z-0.25;         // -2       [1,1,0]         -1/4,1/4,-1/4   1/4,0,0
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z-0.5;          // -2       [0,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[15].x=y-0.5;         SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z+0.25;         // -2       [-1,1,0]        -1/4,-1/4,1/4   -1/4,0,0
      SpaceGroupElement[16].x=x+0.5;         SpaceGroupElement[16].y=y+0.5;         SpaceGroupElement[16].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[17].x=-y+0.5;        SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=z+0.75;         // 4^1      [0,0,1]         0,0,3/4         -1/4,3/4,0
      SpaceGroupElement[18].x=-x;            SpaceGroupElement[18].y=-y;            SpaceGroupElement[18].z=z;            // 2        [0,0,1]         0,0,1           1/2,1/2,0
      SpaceGroupElement[19].x=y;             SpaceGroupElement[19].y=-x+0.5;        SpaceGroupElement[19].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         3/4,-1/4,0
      SpaceGroupElement[20].x=x;             SpaceGroupElement[20].y=-y+0.5;        SpaceGroupElement[20].z=-z+0.75;        // 2        [1,0,0]         1,0,0           0,1/4,3/8
      SpaceGroupElement[21].x=y+0.5;         SpaceGroupElement[21].y=x+0.5;         SpaceGroupElement[21].z=-z;           // 2        [1,1,0]         1/2,1/2,0       0,0,1/2
      SpaceGroupElement[22].x=-x+0.5;        SpaceGroupElement[22].y=y;             SpaceGroupElement[22].z=-z+0.25;        // 2        [0,1,0]         0,1,0           1/4,0,5/8
      SpaceGroupElement[23].x=-y;            SpaceGroupElement[23].y=-x;            SpaceGroupElement[23].z=-z+0.5;         // 2        [-1,1,0]        0,0,0           1,0,1/4
      SpaceGroupElement[24].x=-x+0.5;        SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z+0.75;        // -1       -               -               1/4,1/2,3/8
      SpaceGroupElement[25].x=y+0.5;         SpaceGroupElement[25].y=-x+0.5;        SpaceGroupElement[25].z=-z+0.5;         // -4^1     [0,0,1]         0,0,0           1/2,0,1/4
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=y+0.5;         SpaceGroupElement[26].z=-z+0.25;        // -2       [0,0,1]         0,1/2,0         0,0,1/8
      SpaceGroupElement[27].x=-y;            SpaceGroupElement[27].y=x;             SpaceGroupElement[27].z=-z;             // -4^-1    [0,0,1]         0,0,0           -1/2,1/2,0
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=y;             SpaceGroupElement[28].z=z+0.5;          // -2       [1,0,0]         0,1,1/2         0,0,0
      SpaceGroupElement[29].x=-y+0.5;        SpaceGroupElement[29].y=-x;            SpaceGroupElement[29].z=z+0.25;         // -2       [1,1,0]         -1/4,1/4,1/4    3/4,0,0
      SpaceGroupElement[30].x=x+0.5;         SpaceGroupElement[30].y=-y+0.5;        SpaceGroupElement[30].z=z;              // -2       [0,1,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x+0.5;         SpaceGroupElement[31].z=z+0.75;         // -2       [-1,1,0]        1/4,1/4,3/4     -1/4,0,0
      break;
    case 429: // [142] I 41/a c d Origin choice 2 (-I 4bd 2c)
      SpaceGroupSize=32;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.25;        SpaceGroupElement[1].y=x+0.75;         SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         -1/4,1/2,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[3].x=y+0.25;         SpaceGroupElement[3].y=-x+0.25;        SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         1/4,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[5].x=y+0.25;         SpaceGroupElement[5].y=x+0.75;         SpaceGroupElement[5].z=-z+0.75;         // 2        [1,1,0]         1/2,1/2,0       -1/4,0,3/8
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[7].x=-y+0.25;        SpaceGroupElement[7].y=-x+0.25;        SpaceGroupElement[7].z=-z+0.25;         // 2        [-1,1,0]        0,0,0           1/4,0,1/8
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[9].x=y-0.25;         SpaceGroupElement[9].y=-x-0.75;        SpaceGroupElement[9].z=-z-0.25;         // -4^1     [0,0,1]         0,0,0           -1/2,-1/4,-1/8
      SpaceGroupElement[10].x=x-0.5;         SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z-0.5;         // -2       [0,0,1]         -1/2,0,0        0,0,-1/4
      SpaceGroupElement[11].x=-y-0.25;       SpaceGroupElement[11].y=x-0.25;        SpaceGroupElement[11].z=-z-0.75;        // -4^-1    [0,0,1]         0,0,0           0,-1/4,-3/8
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z-0.5;          // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[13].x=-y-0.25;       SpaceGroupElement[13].y=-x-0.75;       SpaceGroupElement[13].z=z-0.75;         // -2       [1,1,0]         1/4,-1/4,-3/4   -1/2,0,0
      SpaceGroupElement[14].x=x-0.5;         SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z;              // -2       [0,1,0]         -1/2,0,0        0,0,0
      SpaceGroupElement[15].x=y-0.25;        SpaceGroupElement[15].y=x-0.25;        SpaceGroupElement[15].z=z-0.25;         // -2       [-1,1,0]        -1/4,-1/4,-1/4  0,0,0
      SpaceGroupElement[16].x=x+0.5;         SpaceGroupElement[16].y=y+0.5;         SpaceGroupElement[16].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[17].x=-y+0.75;       SpaceGroupElement[17].y=x+0.25;        SpaceGroupElement[17].z=z+0.75;         // 4^1      [0,0,1]         0,0,3/4         -1/4,1,0
      SpaceGroupElement[18].x=-x;            SpaceGroupElement[18].y=-y+0.5;        SpaceGroupElement[18].z=z;            // 2        [0,0,1]         0,0,1           1/2,1/4,0
      SpaceGroupElement[19].x=y+0.75;        SpaceGroupElement[19].y=-x+0.75;       SpaceGroupElement[19].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         3/4,0,0
      SpaceGroupElement[20].x=x+0.5;         SpaceGroupElement[20].y=-y+0.5;        SpaceGroupElement[20].z=-z;           // 2        [1,0,0]         1/2,0,0         0,1/4,1/2
      SpaceGroupElement[21].x=y+0.75;        SpaceGroupElement[21].y=x+0.25;        SpaceGroupElement[21].z=-z+0.25;        // 2        [1,1,0]         1,1,0           -1/4,0,5/8
      SpaceGroupElement[22].x=-x;            SpaceGroupElement[22].y=y+0.5;         SpaceGroupElement[22].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/2,0,1/4
      SpaceGroupElement[23].x=-y+0.75;       SpaceGroupElement[23].y=-x+0.75;       SpaceGroupElement[23].z=-z+0.75;        // 2        [-1,1,0]        0,0,0           3/4,0,3/8
      SpaceGroupElement[24].x=-x+0.5;        SpaceGroupElement[24].y=-y+0.5;        SpaceGroupElement[24].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[25].x=y+0.25;        SpaceGroupElement[25].y=-x-0.25;       SpaceGroupElement[25].z=-z+0.25;        // -4^1     [0,0,1]         0,0,0           0,-1/4,1/8
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=y+0.5;         SpaceGroupElement[26].z=-z;             // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[27].x=-y+0.25;       SpaceGroupElement[27].y=x+0.25;        SpaceGroupElement[27].z=-z-0.25;        // -4^-1    [0,0,1]         0,0,0           0,1/4,-1/8
      SpaceGroupElement[28].x=-x+0.5;        SpaceGroupElement[28].y=y+0.5;         SpaceGroupElement[28].z=z;              // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[29].x=-y+0.25;       SpaceGroupElement[29].y=-x-0.25;       SpaceGroupElement[29].z=z-0.25;         // -2       [1,1,0]         1/4,-1/4,-1/4   0,0,0
      SpaceGroupElement[30].x=x;             SpaceGroupElement[30].y=-y+0.5;        SpaceGroupElement[30].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,1/4,0
      SpaceGroupElement[31].x=y+0.25;        SpaceGroupElement[31].y=x+0.25;        SpaceGroupElement[31].z=z+0.25;         // -2       [-1,1,0]        1/4,1/4,1/4     0,0,0
      break;


    // TRIGONAL GROUPS
    // ===============

    case 430: // [143] P 3 (P 3)
      SpaceGroupSize=3;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      break;

    case 431: // [144] P 31 (P 31)
      SpaceGroupSize=3;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z+1.0/3.0;       // 3^1      [0,0,1]         0,0,1/3         0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z+2.0/3.0;       // 3^-1     [0,0,1]         0,0,2/3         0,0,0
      break;

    case 432: // [145] P 32 (P 32)
      SpaceGroupSize=3;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z+2.0/3.0;       // 3^1      [0,0,1]         0,0,2/3         0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z+1.0/3.0;       // 3^-1     [0,0,1]         0,0,1/3         0,0,0
      break;

    case 433: // [146] R 3 hexagonal axes (R 3)
      SpaceGroupSize=9;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x+2.0/3.0;      SpaceGroupElement[3].y=y+1.0/3.0;      SpaceGroupElement[3].z=z+1.0/3.0;       // 1        -               -               -
      SpaceGroupElement[4].x=-y+2.0/3.0;     SpaceGroupElement[4].y=x-y+1.0/3.0;    SpaceGroupElement[4].z=z+1.0/3.0;       // 3^1      [0,0,1]         0,0,1/3         1/3,1/3,0
      SpaceGroupElement[5].x=-x+y+2.0/3.0;   SpaceGroupElement[5].y=-x+1.0/3.0;     SpaceGroupElement[5].z=z+1.0/3.0;       // 3^-1     [0,0,1]         0,0,1/3         1/3,0,0
      SpaceGroupElement[6].x=x+1.0/3.0;      SpaceGroupElement[6].y=y+2.0/3.0;      SpaceGroupElement[6].z=z+2.0/3.0;       // 1        -               -               -
      SpaceGroupElement[7].x=-y+1.0/3.0;     SpaceGroupElement[7].y=x-y+2.0/3.0;    SpaceGroupElement[7].z=z+2.0/3.0;       // 3^1      [0,0,1]         0,0,2/3         0,1/3,0
      SpaceGroupElement[8].x=-x+y+1.0/3.0;   SpaceGroupElement[8].y=-x+2.0/3.0;     SpaceGroupElement[8].z=z+2.0/3.0;       // 3^-1     [0,0,1]         0,0,2/3         1/3,1/3,0
      break;
    case 434: // [146] R 3 Rhombohedral axes (P 3*)
      SpaceGroupSize=3;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=z;              SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=y;              SpaceGroupElement[2].y=z;              SpaceGroupElement[2].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      break;

    case 435: // [147] P -3 (P -3)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=-z;              // -3^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // -3^-1    [0,0,1]         0,0,0           0,0,0
      break;

    case 436: // [148] R-3 hexagonal axes (-R 3)
      SpaceGroupSize=18;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[4].x=y;              SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=-z;              // -3^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // -3^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=x+2.0/3.0;      SpaceGroupElement[6].y=y+1.0/3.0;      SpaceGroupElement[6].z=z+1.0/3.0;       // 1        -               -               -
      SpaceGroupElement[7].x=-y+2.0/3.0;     SpaceGroupElement[7].y=x-y+1.0/3.0;    SpaceGroupElement[7].z=z+1.0/3.0;       // 3^1      [0,0,1]         0,0,1/3         1/3,1/3,0
      SpaceGroupElement[8].x=-x+y+2.0/3.0;   SpaceGroupElement[8].y=-x+1.0/3.0;     SpaceGroupElement[8].z=z+1.0/3.0;       // 3^-1     [0,0,1]         0,0,1/3         1/3,0,0
      SpaceGroupElement[9].x=-x+2.0/3.0;     SpaceGroupElement[9].y=-y+1.0/3.0;     SpaceGroupElement[9].z=-z+1.0/3.0;      // -1       -               -               1/3,1/6,1/6
      SpaceGroupElement[10].x=y+2.0/3.0;     SpaceGroupElement[10].y=-x+y+1.0/3.0;  SpaceGroupElement[10].z=-z+1.0/3.0;     // -3^1     [0,0,1]         0,0,0           1/3,-1/3,1/6
      SpaceGroupElement[11].x=x-y+2.0/3.0;   SpaceGroupElement[11].y=x+1.0/3.0;     SpaceGroupElement[11].z=-z+1.0/3.0;     // -3^-1    [0,0,1]         0,0,0           1/3,2/3,1/6
      SpaceGroupElement[12].x=x+1.0/3.0;     SpaceGroupElement[12].y=y+2.0/3.0;     SpaceGroupElement[12].z=z+2.0/3.0;      // 1        -               -               -
      SpaceGroupElement[13].x=-y+1.0/3.0;    SpaceGroupElement[13].y=x-y+2.0/3.0;   SpaceGroupElement[13].z=z+2.0/3.0;      // 3^1      [0,0,1]         0,0,2/3         0,1/3,0
      SpaceGroupElement[14].x=-x+y+1.0/3.0;  SpaceGroupElement[14].y=-x+2.0/3.0;    SpaceGroupElement[14].z=z+2.0/3.0;      // 3^-1     [0,0,1]         0,0,2/3         1/3,1/3,0
      SpaceGroupElement[15].x=-x+1.0/3.0;    SpaceGroupElement[15].y=-y+2.0/3.0;    SpaceGroupElement[15].z=-z+2.0/3.0;     // -1       -               -               1/6,1/3,1/3
      SpaceGroupElement[16].x=y+1.0/3.0;     SpaceGroupElement[16].y=-x+y+2.0/3.0;  SpaceGroupElement[16].z=-z+2.0/3.0;     // -3^1     [0,0,1]         0,0,0           2/3,1/3,1/3
      SpaceGroupElement[17].x=x-y+1.0/3.0;   SpaceGroupElement[17].y=x+2.0/3.0;     SpaceGroupElement[17].z=-z+2.0/3.0;     // -3^-1    [0,0,1]         0,0,0           -1/3,1/3,1/3
      break;
    case 437: // [148] R -3 Rhombohedral axes (-P 3*)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=z;              SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=y;              SpaceGroupElement[2].y=z;              SpaceGroupElement[2].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[4].x=-z;             SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=-y;              // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-z;             SpaceGroupElement[5].z=-x;              // -3^-1    [1,1,1]         0,0,0           0,0,0
      break;

    case 438: // [149] P 3 1 2 (P 3 2)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=-z;              // 2        [2,1,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // 2        [1,2,0]         0,0,0           0,0,0
      break;

    case 439: // [150] P 3 2 1 (P 3 2")
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      break;

    case 440: // [151] P 31 1 2 (P 31 2 (0 0 4))
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z+1.0/3.0;       // 3^1      [0,0,1]         0,0,1/3         0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z+2.0/3.0;       // 3^-1     [0,0,1]         0,0,2/3         0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z+2.0/3.0;      // 2        [-1,1,0]        0,0,0           0,0,1/3
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=-z;              // 2        [2,1,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z+1.0/3.0;      // 2        [1,2,0]         0,0,0           0,0,1/6
      break;

    case 441: // [152] P 31 2 1 (P 31 2")
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z+1.0/3.0;       // 3^1      [0,0,1]         0,0,1/3         0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z+2.0/3.0;       // 3^-1     [0,0,1]         0,0,2/3         0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=-z+1.0/3.0;      // 2        [0,1,0]         0,0,0           0,0,1/6
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-z+2.0/3.0;      // 2        [1,0,0]         0,0,0           0,0,1/3
      break;

    case 442: // [153] P 32 1 2 (P 32 2 (0 0 2))
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z+2.0/3.0;       // 3^1      [0,0,1]         0,0,2/3         0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z+1.0/3.0;       // 3^-1     [0,0,1]         0,0,1/3         0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z+1.0/3.0;      // 2        [-1,1,0]        0,0,0           0,0,1/6
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=-z;              // 2        [2,1,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z+2.0/3.0;      // 2        [1,2,0]         0,0,0           0,0,1/3
      break;

    case 443: // [154] P 32 2 1 (P 32 2")
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z+2.0/3.0;       // 3^1      [0,0,1]         0,0,2/3         0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z+1.0/3.0;       // 3^-1     [0,0,1]         0,0,1/3         0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=-z+2.0/3.0;      // 2        [0,1,0]         0,0,0           0,0,1/3
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-z+1.0/3.0;      // 2        [1,0,0]         0,0,0           0,0,1/6
      break;

    case 444: // [155] R 3 2 Hexagonal axes (R 3 2")
      SpaceGroupSize=18;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=x+2.0/3.0;      SpaceGroupElement[6].y=y+1.0/3.0;      SpaceGroupElement[6].z=z+1.0/3.0;       // 1        -               -               -
      SpaceGroupElement[7].x=-y+2.0/3.0;     SpaceGroupElement[7].y=x-y+1.0/3.0;    SpaceGroupElement[7].z=z+1.0/3.0;       // 3^1      [0,0,1]         0,0,1/3         1/3,1/3,0
      SpaceGroupElement[8].x=-x+y+2.0/3.0;   SpaceGroupElement[8].y=-x+1.0/3.0;     SpaceGroupElement[8].z=z+1.0/3.0;       // 3^-1     [0,0,1]         0,0,1/3         1/3,0,0
      SpaceGroupElement[9].x=y+2.0/3.0;      SpaceGroupElement[9].y=x+1.0/3.0;      SpaceGroupElement[9].z=-z+1.0/3.0;      // 2        [1,1,0]         1/2,1/2,0       1/6,0,1/6
      SpaceGroupElement[10].x=-x+2.0/3.0;    SpaceGroupElement[10].y=-x+y+1.0/3.0;  SpaceGroupElement[10].z=-z+1.0/3.0;     // 2        [0,1,0]         0,0,0           1/3,0,1/6
      SpaceGroupElement[11].x=x-y+2.0/3.0;   SpaceGroupElement[11].y=-y+1.0/3.0;    SpaceGroupElement[11].z=-z+1.0/3.0;     // 2        [1,0,0]         1/2,0,0         0,1/6,1/6
      SpaceGroupElement[12].x=x+1.0/3.0;     SpaceGroupElement[12].y=y+2.0/3.0;     SpaceGroupElement[12].z=z+2.0/3.0;      // 1        -               -               -
      SpaceGroupElement[13].x=-y+1.0/3.0;    SpaceGroupElement[13].y=x-y+2.0/3.0;   SpaceGroupElement[13].z=z+2.0/3.0;      // 3^1      [0,0,1]         0,0,2/3         0,1/3,0
      SpaceGroupElement[14].x=-x+y+1.0/3.0;  SpaceGroupElement[14].y=-x+2.0/3.0;    SpaceGroupElement[14].z=z+2.0/3.0;      // 3^-1     [0,0,1]         0,0,2/3         1/3,1/3,0
      SpaceGroupElement[15].x=y+1.0/3.0;     SpaceGroupElement[15].y=x+2.0/3.0;     SpaceGroupElement[15].z=-z+2.0/3.0;     // 2        [1,1,0]         1/2,1/2,0       -1/6,0,1/3
      SpaceGroupElement[16].x=-x+1.0/3.0;    SpaceGroupElement[16].y=-x+y+2.0/3.0;  SpaceGroupElement[16].z=-z+2.0/3.0;     // 2        [0,1,0]         0,1/2,0         1/6,0,1/3
      SpaceGroupElement[17].x=x-y+1.0/3.0;   SpaceGroupElement[17].y=-y+2.0/3.0;    SpaceGroupElement[17].z=-z+2.0/3.0;     // 2        [1,0,0]         0,0,0           0,1/3,1/3
      break;
    case 445: // [155] R 3 2 Rhombohedral axes (P 3* 2)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=z;              SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=y;              SpaceGroupElement[2].y=z;              SpaceGroupElement[2].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[4].x=-z;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-x;              // 2        [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=-z;             SpaceGroupElement[5].z=-y;              // 2        [0,-1,1]        0,0,0           0,0,0
      break;

    case 446: // [156] P 3 m 1 (P 3 -2")
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      break;

    case 447: // [157] P 3 1 m (P 3 -2)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;               // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=z;               // -2       [2,1,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=z;               // -2       [1,2,0]         0,0,0           0,0,0
      break;

    case 448: // [158] P 3 c 1 (P 3 -2"c)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.5;           // -2       [1,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      break;

    case 449: // [159] P 3 1 c (P 3 -2c)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z+0.5;           // -2       [-1,1,0]        0,0,1/2         0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=z+0.5;           // -2       [2,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=z+0.5;           // -2       [1,2,0]         0,0,1/2         0,0,0
      break;

    case 450: // [160] R 3 m Hexagonal axes (R 3 -2")
      SpaceGroupSize=18;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;               // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=x+2.0/3.0;      SpaceGroupElement[6].y=y+1.0/3.0;      SpaceGroupElement[6].z=z+1.0/3.0;       // 1        -               -               -
      SpaceGroupElement[7].x=-y+2.0/3.0;     SpaceGroupElement[7].y=x-y+1.0/3.0;    SpaceGroupElement[7].z=z+1.0/3.0;       // 3^1      [0,0,1]         0,0,1/3         1/3,1/3,0
      SpaceGroupElement[8].x=-x+y+2.0/3.0;   SpaceGroupElement[8].y=-x+1.0/3.0;     SpaceGroupElement[8].z=z+1.0/3.0;       // 3^-1     [0,0,1]         0,0,1/3         1/3,0,0
      SpaceGroupElement[9].x=-y+2.0/3.0;     SpaceGroupElement[9].y=-x+1.0/3.0;     SpaceGroupElement[9].z=z+1.0/3.0;       // -2       [1,1,0]         1/6,-1/6,1/3    1/2,0,0
      SpaceGroupElement[10].x=x+2.0/3.0;     SpaceGroupElement[10].y=x-y+1.0/3.0;   SpaceGroupElement[10].z=z+1.0/3.0;      // -2       [0,1,0]         2/3,1/3,1/3     0,0,0
      SpaceGroupElement[11].x=-x+y+2.0/3.0;  SpaceGroupElement[11].y=y+1.0/3.0;     SpaceGroupElement[11].z=z+1.0/3.0;      // -2       [1,0,0]         1/6,1/3,1/3     1/4,0,0
      SpaceGroupElement[12].x=x+1.0/3.0;     SpaceGroupElement[12].y=y+2.0/3.0;     SpaceGroupElement[12].z=z+2.0/3.0;      // 1        -               -               -
      SpaceGroupElement[13].x=-y+1.0/3.0;    SpaceGroupElement[13].y=x-y+2.0/3.0;   SpaceGroupElement[13].z=z+2.0/3.0;      // 3^1      [0,0,1]         0,0,2/3         0,1/3,0
      SpaceGroupElement[14].x=-x+y+1.0/3.0;  SpaceGroupElement[14].y=-x+2.0/3.0;    SpaceGroupElement[14].z=z+2.0/3.0;      // 3^-1     [0,0,1]         0,0,2/3         1/3,1/3,0
      SpaceGroupElement[15].x=-y+1.0/3.0;    SpaceGroupElement[15].y=-x+2.0/3.0;    SpaceGroupElement[15].z=z+2.0/3.0;      // -2       [1,1,0]         -1/6,1/6,2/3    1/2,0,0
      SpaceGroupElement[16].x=x+1.0/3.0;     SpaceGroupElement[16].y=x-y+2.0/3.0;   SpaceGroupElement[16].z=z+2.0/3.0;      // -2       [0,1,0]         1/3,1/6,2/3     -1/2,0,0
      SpaceGroupElement[17].x=-x+y+1.0/3.0;  SpaceGroupElement[17].y=y+2.0/3.0;     SpaceGroupElement[17].z=z+2.0/3.0;      // -2       [1,0,0]         1/3,2/3,2/3     0,0,0
      break;
    case 451: // [160] R 3 m Rhombohedral axes (P 3* -2)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=z;              SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=y;              SpaceGroupElement[2].y=z;              SpaceGroupElement[2].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;               // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[4].x=z;              SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=x;               // -2       [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=z;              SpaceGroupElement[5].z=y;               // -2       [0,-1,1]        0,0,0           0,0,0
      break;

    case 452: // [161] R 3 c Hexagonal axes (R 3 -2"c)
      SpaceGroupSize=18;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.5;           // -2       [1,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z+0.5;           // -2       [0,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[6].x=x+2.0/3.0;      SpaceGroupElement[6].y=y+1.0/3.0;      SpaceGroupElement[6].z=z+1.0/3.0;       // 1        -               -               -
      SpaceGroupElement[7].x=-y+2.0/3.0;     SpaceGroupElement[7].y=x-y+1.0/3.0;    SpaceGroupElement[7].z=z+1.0/3.0;       // 3^1      [0,0,1]         0,0,1/3         1/3,1/3,0
      SpaceGroupElement[8].x=-x+y+2.0/3.0;   SpaceGroupElement[8].y=-x+1.0/3.0;     SpaceGroupElement[8].z=z+1.0/3.0;       // 3^-1     [0,0,1]         0,0,1/3         1/3,0,0
      SpaceGroupElement[9].x=-y+2.0/3.0;     SpaceGroupElement[9].y=-x+1.0/3.0;     SpaceGroupElement[9].z=z+5.0/6.0;        // -2       [1,1,0]         1/6,-1/6,5/6    1/2,0,0
      SpaceGroupElement[10].x=x+2.0/3.0;     SpaceGroupElement[10].y=x-y+1.0/3.0;   SpaceGroupElement[10].z=z+5.0/6.0;       // -2       [0,1,0]         2/3,1/3,5/6     0,0,0
      SpaceGroupElement[11].x=-x+y+2.0/3.0;  SpaceGroupElement[11].y=y+1.0/3.0;     SpaceGroupElement[11].z=z+5.0/6.0;       // -2       [1,0,0]         1/6,1/3,5/6     1/4,0,0
      SpaceGroupElement[12].x=x+1.0/3.0;     SpaceGroupElement[12].y=y+2.0/3.0;     SpaceGroupElement[12].z=z+2.0/3.0;      // 1        -               -               -
      SpaceGroupElement[13].x=-y+1.0/3.0;    SpaceGroupElement[13].y=x-y+2.0/3.0;   SpaceGroupElement[13].z=z+2.0/3.0;      // 3^1      [0,0,1]         0,0,2/3         0,1/3,0
      SpaceGroupElement[14].x=-x+y+1.0/3.0;  SpaceGroupElement[14].y=-x+2.0/3.0;    SpaceGroupElement[14].z=z+2.0/3.0;      // 3^-1     [0,0,1]         0,0,2/3         1/3,1/3,0
      SpaceGroupElement[15].x=-y+1.0/3.0;    SpaceGroupElement[15].y=-x+2.0/3.0;    SpaceGroupElement[15].z=z+1.0/6.0;       // -2       [1,1,0]         -1/6,1/6,7/6    1/2,0,0
      SpaceGroupElement[16].x=x+1.0/3.0;     SpaceGroupElement[16].y=x-y+2.0/3.0;   SpaceGroupElement[16].z=z+1.0/6.0;       // -2       [0,1,0]         1/3,1/6,7/6     -1/2,0,0
      SpaceGroupElement[17].x=-x+y+1.0/3.0;  SpaceGroupElement[17].y=y+2.0/3.0;     SpaceGroupElement[17].z=z+1.0/6.0;       // -2       [1,0,0]         1/3,2/3,7/6     0,0,0
      break;
    case 453: // [161] R 3 c Rhombohedral axes (P 3* -2n)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=z;              SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=y;              SpaceGroupElement[2].y=z;              SpaceGroupElement[2].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=x+0.5;          SpaceGroupElement[3].z=z+0.5;           // -2       [-1,1,0]        1/2,1/2,1/2     0,0,0
      SpaceGroupElement[4].x=z+0.5;          SpaceGroupElement[4].y=y+0.5;          SpaceGroupElement[4].z=x+0.5;           // -2       [-1,0,1]        1/2,1/2,1/2     0,0,0
      SpaceGroupElement[5].x=x+0.5;          SpaceGroupElement[5].y=z+0.5;          SpaceGroupElement[5].z=y+0.5;           // -2       [0,-1,1]        1/2,1/2,1/2     0,0,0
      break;

    case 454: // [162] P -3 1 m (-P 3 2)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=-z;              // 2        [2,1,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;              // 2        [1,2,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-x+y;           SpaceGroupElement[7].z=-z;              // -3^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x-y;            SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=-z;              // -3^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=z;               // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[10].x=-x;            SpaceGroupElement[10].y=-x+y;          SpaceGroupElement[10].z=z;              // -2       [2,1,0]         0,0,0           0,0,0
      SpaceGroupElement[11].x=x-y;           SpaceGroupElement[11].y=-y;            SpaceGroupElement[11].z=z;              // -2       [1,2,0]         0,0,0           0,0,0
      break;

    case 455: // [163] P -3 1 c (-P 3 2c)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           0,0,1/4
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=-z+0.5;          // 2        [2,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z+0.5;          // 2        [1,2,0]         0,0,0           0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-x+y;           SpaceGroupElement[7].z=-z;              // -3^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x-y;            SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=-z;              // -3^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=z-0.5;           // -2       [-1,1,0]        0,0,-1/2        0,0,0
      SpaceGroupElement[10].x=-x;            SpaceGroupElement[10].y=-x+y;          SpaceGroupElement[10].z=z-0.5;          // -2       [2,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[11].x=x-y;           SpaceGroupElement[11].y=-y;            SpaceGroupElement[11].z=z-0.5;          // -2       [1,2,0]         0,0,-1/2        0,0,0
      break;

    case 456: // [164] P -3 m 1 (-P 3 2")
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-x+y;           SpaceGroupElement[7].z=-z;              // -3^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x-y;            SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=-z;              // -3^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=z;               // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=x-y;           SpaceGroupElement[10].z=z;              // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-x+y;          SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=z;              // -2       [1,0,0]         0,0,0           0,0,0
      break;

    case 457: // [165] P -3 c 1 (-P 3 2"c)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z+0.5;          // 2        [1,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-x+y;           SpaceGroupElement[7].z=-z;              // -3^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x-y;            SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=-z;              // -3^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=z-0.5;           // -2       [1,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=x-y;           SpaceGroupElement[10].z=z-0.5;          // -2       [0,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[11].x=-x+y;          SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=z-0.5;          // -2       [1,0,0]         0,0,-1/2        0,0,0
      break;

    case 458: // [166] R -3 m Hexagonal axes (-R 3 2")
      SpaceGroupSize=36;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-x+y;           SpaceGroupElement[7].z=-z;              // -3^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x-y;            SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=-z;              // -3^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=z;               // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=x-y;           SpaceGroupElement[10].z=z;              // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-x+y;          SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=z;              // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[12].x=x+2.0/3.0;     SpaceGroupElement[12].y=y+1.0/3.0;     SpaceGroupElement[12].z=z+1.0/3.0;      // 1        -               -               -
      SpaceGroupElement[13].x=-y+2.0/3.0;    SpaceGroupElement[13].y=x-y+1.0/3.0;   SpaceGroupElement[13].z=z+1.0/3.0;      // 3^1      [0,0,1]         0,0,1/3         1/3,1/3,0
      SpaceGroupElement[14].x=-x+y+2.0/3.0;  SpaceGroupElement[14].y=-x+1.0/3.0;    SpaceGroupElement[14].z=z+1.0/3.0;      // 3^-1     [0,0,1]         0,0,1/3         1/3,0,0
      SpaceGroupElement[15].x=y+2.0/3.0;     SpaceGroupElement[15].y=x+1.0/3.0;     SpaceGroupElement[15].z=-z+1.0/3.0;     // 2        [1,1,0]         1/2,1/2,0       1/6,0,1/6
      SpaceGroupElement[16].x=-x+2.0/3.0;    SpaceGroupElement[16].y=-x+y+1.0/3.0;  SpaceGroupElement[16].z=-z+1.0/3.0;     // 2        [0,1,0]         0,0,0           1/3,0,1/6
      SpaceGroupElement[17].x=x-y+2.0/3.0;   SpaceGroupElement[17].y=-y+1.0/3.0;    SpaceGroupElement[17].z=-z+1.0/3.0;     // 2        [1,0,0]         1/2,0,0         0,1/6,1/6
      SpaceGroupElement[18].x=-x+2.0/3.0;    SpaceGroupElement[18].y=-y+1.0/3.0;    SpaceGroupElement[18].z=-z+1.0/3.0;     // -1       -               -               1/3,1/6,1/6
      SpaceGroupElement[19].x=y+2.0/3.0;     SpaceGroupElement[19].y=-x+y+1.0/3.0;  SpaceGroupElement[19].z=-z+1.0/3.0;     // -3^1     [0,0,1]         0,0,0           1/3,-1/3,1/6
      SpaceGroupElement[20].x=x-y+2.0/3.0;   SpaceGroupElement[20].y=x+1.0/3.0;     SpaceGroupElement[20].z=-z+1.0/3.0;     // -3^-1    [0,0,1]         0,0,0           1/3,2/3,1/6
      SpaceGroupElement[21].x=-y+2.0/3.0;    SpaceGroupElement[21].y=-x+1.0/3.0;    SpaceGroupElement[21].z=z+1.0/3.0;      // -2       [1,1,0]         1/6,-1/6,1/3    1/2,0,0
      SpaceGroupElement[22].x=x+2.0/3.0;     SpaceGroupElement[22].y=x-y+1.0/3.0;   SpaceGroupElement[22].z=z+1.0/3.0;      // -2       [0,1,0]         2/3,1/3,1/3     0,0,0
      SpaceGroupElement[23].x=-x+y+2.0/3.0;  SpaceGroupElement[23].y=y+1.0/3.0;     SpaceGroupElement[23].z=z+1.0/3.0;      // -2       [1,0,0]         1/6,1/3,1/3     1/4,0,0
      SpaceGroupElement[24].x=x+1.0/3.0;     SpaceGroupElement[24].y=y+2.0/3.0;     SpaceGroupElement[24].z=z+2.0/3.0;      // 1        -               -               -
      SpaceGroupElement[25].x=-y+1.0/3.0;    SpaceGroupElement[25].y=x-y+2.0/3.0;   SpaceGroupElement[25].z=z+2.0/3.0;      // 3^1      [0,0,1]         0,0,2/3         0,1/3,0
      SpaceGroupElement[26].x=-x+y+1.0/3.0;  SpaceGroupElement[26].y=-x+2.0/3.0;    SpaceGroupElement[26].z=z+2.0/3.0;      // 3^-1     [0,0,1]         0,0,2/3         1/3,1/3,0
      SpaceGroupElement[27].x=y+1.0/3.0;     SpaceGroupElement[27].y=x+2.0/3.0;     SpaceGroupElement[27].z=-z+2.0/3.0;     // 2        [1,1,0]         1/2,1/2,0       -1/6,0,1/3
      SpaceGroupElement[28].x=-x+1.0/3.0;    SpaceGroupElement[28].y=-x+y+2.0/3.0;  SpaceGroupElement[28].z=-z+2.0/3.0;     // 2        [0,1,0]         0,1/2,0         1/6,0,1/3
      SpaceGroupElement[29].x=x-y+1.0/3.0;   SpaceGroupElement[29].y=-y+2.0/3.0;    SpaceGroupElement[29].z=-z+2.0/3.0;     // 2        [1,0,0]         0,0,0           0,1/3,1/3
      SpaceGroupElement[30].x=-x+1.0/3.0;    SpaceGroupElement[30].y=-y+2.0/3.0;    SpaceGroupElement[30].z=-z+2.0/3.0;     // -1       -               -               1/6,1/3,1/3
      SpaceGroupElement[31].x=y+1.0/3.0;     SpaceGroupElement[31].y=-x+y+2.0/3.0;  SpaceGroupElement[31].z=-z+2.0/3.0;     // -3^1     [0,0,1]         0,0,0           2/3,1/3,1/3
      SpaceGroupElement[32].x=x-y+1.0/3.0;   SpaceGroupElement[32].y=x+2.0/3.0;     SpaceGroupElement[32].z=-z+2.0/3.0;     // -3^-1    [0,0,1]         0,0,0           -1/3,1/3,1/3
      SpaceGroupElement[33].x=-y+1.0/3.0;    SpaceGroupElement[33].y=-x+2.0/3.0;    SpaceGroupElement[33].z=z+2.0/3.0;      // -2       [1,1,0]         -1/6,1/6,2/3    1/2,0,0
      SpaceGroupElement[34].x=x+1.0/3.0;     SpaceGroupElement[34].y=x-y+2.0/3.0;   SpaceGroupElement[34].z=z+2.0/3.0;      // -2       [0,1,0]         1/3,1/6,2/3     -1/2,0,0
      SpaceGroupElement[35].x=-x+y+1.0/3.0;  SpaceGroupElement[35].y=y+2.0/3.0;     SpaceGroupElement[35].z=z+2.0/3.0;      // -2       [1,0,0]         1/3,2/3,2/3     0,0,0
      break;
    case 459: // [166] R -3 m Rhombohedral axes (-P 3* 2)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=z;              SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=y;              SpaceGroupElement[2].y=z;              SpaceGroupElement[2].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[4].x=-z;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-x;              // 2        [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=-z;             SpaceGroupElement[5].z=-y;              // 2        [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[7].x=-z;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-y;              // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[8].x=-y;             SpaceGroupElement[8].y=-z;             SpaceGroupElement[8].z=-x;              // -3^-1    [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=z;               // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[10].x=z;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=x;              // -2       [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=z;             SpaceGroupElement[11].z=y;              // -2       [0,-1,1]        0,0,0           0,0,0
      break;

    case 460: // [167] R -3 c Hexagonal axes (-R 3 2"c)
      SpaceGroupSize=36;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z+0.5;          // 2        [1,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=-z+0.5;          // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-x+y;           SpaceGroupElement[7].z=-z;              // -3^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x-y;            SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=-z;              // -3^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=z-0.5;           // -2       [1,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=x-y;           SpaceGroupElement[10].z=z-0.5;          // -2       [0,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[11].x=-x+y;          SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=z-0.5;          // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[12].x=x+2.0/3.0;     SpaceGroupElement[12].y=y+1.0/3.0;     SpaceGroupElement[12].z=z+1.0/3.0;      // 1        -               -               -
      SpaceGroupElement[13].x=-y+2.0/3.0;    SpaceGroupElement[13].y=x-y+1.0/3.0;   SpaceGroupElement[13].z=z+1.0/3.0;      // 3^1      [0,0,1]         0,0,1/3         1/3,1/3,0
      SpaceGroupElement[14].x=-x+y+2.0/3.0;  SpaceGroupElement[14].y=-x+1.0/3.0;    SpaceGroupElement[14].z=z+1.0/3.0;      // 3^-1     [0,0,1]         0,0,1/3         1/3,0,0
      SpaceGroupElement[15].x=y+2.0/3.0;     SpaceGroupElement[15].y=x+1.0/3.0;     SpaceGroupElement[15].z=-z+5.0/6.0;      // 2        [1,1,0]         1/2,1/2,0       1/6,0,5/12
      SpaceGroupElement[16].x=-x+2.0/3.0;    SpaceGroupElement[16].y=-x+y+1.0/3.0;  SpaceGroupElement[16].z=-z+5.0/6.0;      // 2        [0,1,0]         0,0,0           1/3,0,5/12
      SpaceGroupElement[17].x=x-y+2.0/3.0;   SpaceGroupElement[17].y=-y+1.0/3.0;    SpaceGroupElement[17].z=-z+5.0/6.0;      // 2        [1,0,0]         1/2,0,0         0,1/6,5/12
      SpaceGroupElement[18].x=-x+2.0/3.0;    SpaceGroupElement[18].y=-y+1.0/3.0;    SpaceGroupElement[18].z=-z+1.0/3.0;     // -1       -               -               1/3,1/6,1/6
      SpaceGroupElement[19].x=y+2.0/3.0;     SpaceGroupElement[19].y=-x+y+1.0/3.0;  SpaceGroupElement[19].z=-z+1.0/3.0;     // -3^1     [0,0,1]         0,0,0           1/3,-1/3,1/6
      SpaceGroupElement[20].x=x-y+2.0/3.0;   SpaceGroupElement[20].y=x+1.0/3.0;     SpaceGroupElement[20].z=-z+1.0/3.0;     // -3^-1    [0,0,1]         0,0,0           1/3,2/3,1/6
      SpaceGroupElement[21].x=-y+2.0/3.0;    SpaceGroupElement[21].y=-x+1.0/3.0;    SpaceGroupElement[21].z=z-1.0/6.0;       // -2       [1,1,0]         1/6,-1/6,-1/6   1/2,0,0
      SpaceGroupElement[22].x=x+2.0/3.0;     SpaceGroupElement[22].y=x-y+1.0/3.0;   SpaceGroupElement[22].z=z-1.0/6.0;      // -2       [0,1,0]         2/3,1/3,-1/6    0,0,0
      SpaceGroupElement[23].x=-x+y+2.0/3.0;  SpaceGroupElement[23].y=y+1.0/3.0;     SpaceGroupElement[23].z=z-1.0/6.0;       // -2       [1,0,0]         1/6,1/3,-1/6    1/4,0,0
      SpaceGroupElement[24].x=x+1.0/3.0;     SpaceGroupElement[24].y=y+2.0/3.0;     SpaceGroupElement[24].z=z+2.0/3.0;      // 1        -               -               -
      SpaceGroupElement[25].x=-y+1.0/3.0;    SpaceGroupElement[25].y=x-y+2.0/3.0;   SpaceGroupElement[25].z=z+2.0/3.0;      // 3^1      [0,0,1]         0,0,2/3         0,1/3,0
      SpaceGroupElement[26].x=-x+y+1.0/3.0;  SpaceGroupElement[26].y=-x+2.0/3.0;    SpaceGroupElement[26].z=z+2.0/3.0;      // 3^-1     [0,0,1]         0,0,2/3         1/3,1/3,0
      SpaceGroupElement[27].x=y+1.0/3.0;     SpaceGroupElement[27].y=x+2.0/3.0;     SpaceGroupElement[27].z=-z+1.0/6.0;      // 2        [1,1,0]         1/2,1/2,0       -1/6,0,7/12
      SpaceGroupElement[28].x=-x+1.0/3.0;    SpaceGroupElement[28].y=-x+y+2.0/3.0;  SpaceGroupElement[28].z=-z+1.0/6.0;      // 2        [0,1,0]         0,1/2,0         1/6,0,7/12
      SpaceGroupElement[29].x=x-y+1.0/3.0;   SpaceGroupElement[29].y=-y+2.0/3.0;    SpaceGroupElement[29].z=-z+1.0/6.0;      // 2        [1,0,0]         0,0,0           0,1/3,7/12
      SpaceGroupElement[30].x=-x+1.0/3.0;    SpaceGroupElement[30].y=-y+2.0/3.0;    SpaceGroupElement[30].z=-z+2.0/3.0;     // -1       -               -               1/6,1/3,1/3
      SpaceGroupElement[31].x=y+1.0/3.0;     SpaceGroupElement[31].y=-x+y+2.0/3.0;  SpaceGroupElement[31].z=-z+2.0/3.0;     // -3^1     [0,0,1]         0,0,0           2/3,1/3,1/3
      SpaceGroupElement[32].x=x-y+1.0/3.0;   SpaceGroupElement[32].y=x+2.0/3.0;     SpaceGroupElement[32].z=-z+2.0/3.0;     // -3^-1    [0,0,1]         0,0,0           -1/3,1/3,1/3
      SpaceGroupElement[33].x=-y+1.0/3.0;    SpaceGroupElement[33].y=-x+2.0/3.0;    SpaceGroupElement[33].z=z+1.0/6.0;       // -2       [1,1,0]         -1/6,1/6,1/6    1/2,0,0
      SpaceGroupElement[34].x=x+1.0/3.0;     SpaceGroupElement[34].y=x-y+2.0/3.0;   SpaceGroupElement[34].z=z+1.0/6.0;       // -2       [0,1,0]         1/3,1/6,1/6     -1/2,0,0
      SpaceGroupElement[35].x=-x+y+1.0/3.0;  SpaceGroupElement[35].y=y+2.0/3.0;     SpaceGroupElement[35].z=z+1.0/6.0;      // -2       [1,0,0]         1/3,2/3,1/6     0,0,0
      break;
    case 461: // [167] R -3 c Rhombohedral axes (-P 3* 2n)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=z;              SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=y;              SpaceGroupElement[2].y=z;              SpaceGroupElement[2].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y+0.5;         SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      SpaceGroupElement[4].x=-z+0.5;         SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-x+0.5;          // 2        [-1,0,1]        0,0,0           1/2,1/4,0
      SpaceGroupElement[5].x=-x+0.5;         SpaceGroupElement[5].y=-z+0.5;         SpaceGroupElement[5].z=-y+0.5;          // 2        [0,-1,1]        0,0,0           1/4,1/2,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[7].x=-z;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-y;              // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[8].x=-y;             SpaceGroupElement[8].y=-z;             SpaceGroupElement[8].z=-x;              // -3^-1    [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y-0.5;          SpaceGroupElement[9].y=x-0.5;          SpaceGroupElement[9].z=z-0.5;           // -2       [-1,1,0]        -1/2,-1/2,-1/2  0,0,0
      SpaceGroupElement[10].x=z-0.5;         SpaceGroupElement[10].y=y-0.5;         SpaceGroupElement[10].z=x-0.5;          // -2       [-1,0,1]        -1/2,-1/2,-1/2  0,0,0
      SpaceGroupElement[11].x=x-0.5;         SpaceGroupElement[11].y=z-0.5;         SpaceGroupElement[11].z=y-0.5;          // -2       [0,-1,1]        -1/2,-1/2,-1/2  0,0,0
      break;


    // HEXAGONAL GROUPS
    // ================

    case 462: // [168] P 6 (P 6)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 6^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z;               // 6^-1     [0,0,1]         0,0,0           0,0,0
      break;

    case 463: // [169] P 61 (P 61)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+1.0/6.0;       // 6^1      [0,0,1]         0,0,1/6         0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z+1.0/3.0;       // 3^1      [0,0,1]         0,0,1/3         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z+2.0/3.0;       // 3^-1     [0,0,1]         0,0,2/3         0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z+5.0/6.0;       // 6^-1     [0,0,1]         0,0,5/6         0,0,0
      break;

    case 464: // [170] P 65 (P 65)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+5.0/6.0;       // 6^1      [0,0,1]         0,0,5/6         0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z+2.0/3.0;       // 3^1      [0,0,1]         0,0,2/3         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z+1.0/3.0;       // 3^-1     [0,0,1]         0,0,1/3         0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z+1.0/6.0;       // 6^-1     [0,0,1]         0,0,1/6         0,0,0
      break;

    case 465: // [171] P 62 (P 62)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+1.0/3.0;       // 6^1      [0,0,1]         0,0,1/3         0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z+2.0/3.0;       // 3^1      [0,0,1]         0,0,2/3         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z+1.0/3.0;       // 3^-1     [0,0,1]         0,0,1/3         0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z+2.0/3.0;       // 6^-1     [0,0,1]         0,0,2/3         0,0,0
      break;

    case 466: // [172] P 64 (P 64)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+2.0/3.0;       // 6^1      [0,0,1]         0,0,2/3         0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z+1.0/3.0;       // 3^1      [0,0,1]         0,0,1/3         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z+2.0/3.0;       // 3^-1     [0,0,1]         0,0,2/3         0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z+1.0/3.0;       // 6^-1     [0,0,1]         0,0,1/3         0,0,0
      break;

    case 467: // [173] P 63 (P 6c)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 6^1      [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z+0.5;           // 6^-1     [0,0,1]         0,0,1/2         0,0,0
      break;

    case 468: // [174] P -6 (P -6)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -6^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=x-y;            SpaceGroupElement[5].z=-z;              // -6^-1    [0,0,1]         0,0,0           0,0,0
      break;

    case 469: // [175] P6/m (-P 6)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 6^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z;               // 6^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[7].x=-x+y;           SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // -6^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=-z;              // -3^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=x;              SpaceGroupElement[9].y=y;              SpaceGroupElement[9].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x-y;           SpaceGroupElement[10].y=x;             SpaceGroupElement[10].z=-z;             // -3^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x-y;           SpaceGroupElement[11].z=-z;             // -6^-1    [0,0,1]         0,0,0           0,0,0
      break;

    case 470: // [176] P 63/m (-P 6c)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 6^1      [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z+0.5;           // 6^-1     [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;              // -1       -               -               0,0,0
      SpaceGroupElement[7].x=-x+y;           SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z-0.5;          // -6^1     [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=-z;              // -3^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=x;              SpaceGroupElement[9].y=y;              SpaceGroupElement[9].z=-z-0.5;          // -2       [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[10].x=x-y;           SpaceGroupElement[10].y=x;             SpaceGroupElement[10].z=-z;             // -3^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=x-y;           SpaceGroupElement[11].z=-z-0.5;         // -6^-1    [0,0,1]         0,0,0           0,0,-1/4
      break;

    case 471: // [177] P 6 2 2 (P 6 2)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 6^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z;               // 6^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=-z;              // 2        [2,1,0]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;             // 2        [1,2,0]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=-z;             // 2        [0,1,0]         0,0,0           0,0,0
      break;

    case 472: // [178] P 61 2 2 (P 61 2 (0 0 5))
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+1.0/6.0;       // 6^1      [0,0,1]         0,0,1/6         0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z+1.0/3.0;       // 3^1      [0,0,1]         0,0,1/3         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z+2.0/3.0;       // 3^-1     [0,0,1]         0,0,2/3         0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z+5.0/6.0;       // 6^-1     [0,0,1]         0,0,5/6         0,0,0
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-z+5.0/6.0;      // 2        [-1,1,0]        0,0,0           0,0,5/12
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=-z+1.0/6.0;      // 2        [2,1,0]         0,0,0           0,0,1/12
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z+1.0/3.0;      // 2        [1,1,0]         0,0,0           0,0,1/6
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z+0.5;         // 2        [1,2,0]         0,0,0           0,0,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=-z+2.0/3.0;     // 2        [0,1,0]         0,0,0           0,0,1/3
      break;

    case 473: // [179] P 65 2 2 (P 65 2 (0 0 1))
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+5.0/6.0;       // 6^1      [0,0,1]         0,0,5/6         0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z+2.0/3.0;       // 3^1      [0,0,1]         0,0,2/3         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z+1.0/3.0;       // 3^-1     [0,0,1]         0,0,1/3         0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z+1.0/6.0;       // 6^-1     [0,0,1]         0,0,1/6         0,0,0
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-z+1.0/6.0;      // 2        [-1,1,0]        0,0,0           0,0,1/12
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=-z+5.0/6.0;      // 2        [2,1,0]         0,0,0           0,0,5/12
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z+2.0/3.0;      // 2        [1,1,0]         0,0,0           0,0,1/3
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z+0.5;         // 2        [1,2,0]         0,0,0           0,0,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=-z+1.0/3.0;     // 2        [0,1,0]         0,0,0           0,0,1/6
      break;

    case 474: // [180] P 62 2 2 (P 62 2 (0 0 4))
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+1.0/3.0;       // 6^1      [0,0,1]         0,0,1/3         0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z+2.0/3.0;       // 3^1      [0,0,1]         0,0,2/3         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z+1.0/3.0;       // 3^-1     [0,0,1]         0,0,1/3         0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z+2.0/3.0;       // 6^-1     [0,0,1]         0,0,2/3         0,0,0
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-z+2.0/3.0;      // 2        [-1,1,0]        0,0,0           0,0,1/3
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=-z+1.0/3.0;      // 2        [2,1,0]         0,0,0           0,0,1/6
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z+2.0/3.0;      // 2        [1,1,0]         0,0,0           0,0,1/3
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;             // 2        [1,2,0]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=-z+1.0/3.0;     // 2        [0,1,0]         0,0,0           0,0,1/6
      break;

    case 475: // [181] P 64 2 2 (P 64 2 (0 0 2))
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+2.0/3.0;       // 6^1      [0,0,1]         0,0,2/3         0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z+1.0/3.0;       // 3^1      [0,0,1]         0,0,1/3         0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z+2.0/3.0;       // 3^-1     [0,0,1]         0,0,2/3         0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z+1.0/3.0;       // 6^-1     [0,0,1]         0,0,1/3         0,0,0
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-z+1.0/3.0;      // 2        [-1,1,0]        0,0,0           0,0,1/6
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=-z+2.0/3.0;      // 2        [2,1,0]         0,0,0           0,0,1/3
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z+1.0/3.0;      // 2        [1,1,0]         0,0,0           0,0,1/6
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;             // 2        [1,2,0]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=-z+2.0/3.0;     // 2        [0,1,0]         0,0,0           0,0,1/3
      break;

    case 476: // [182] P 63 2 2 (P 6c 2c)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 6^1      [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z+0.5;           // 6^-1     [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           0,0,1/4
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=-z+0.5;          // 2        [2,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z+0.5;         // 2        [1,2,0]         0,0,0           0,0,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=-z;             // 2        [0,1,0]         0,0,0           0,0,0
      break;

    case 477: // [183] P 6 m m (P 6 -2)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 6^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z;               // 6^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=y;              SpaceGroupElement[6].y=x;              SpaceGroupElement[6].z=z;               // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[7].x=-x+y;           SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=z;               // -2       [2,1,0]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=z;               // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x-y;           SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=z;              // -2       [1,2,0]         0,0,0           0,0,0
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=x-y;           SpaceGroupElement[11].z=z;              // -2       [0,1,0]         0,0,0           0,0,0
      break;

    case 478: // [184] P 6 c c (P 6 -2c)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 6^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z;               // 6^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=y;              SpaceGroupElement[6].y=x;              SpaceGroupElement[6].z=z+0.5;           // -2       [-1,1,0]        0,0,1/2         0,0,0
      SpaceGroupElement[7].x=-x+y;           SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=z+0.5;           // -2       [2,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=z+0.5;           // -2       [1,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[10].x=x-y;           SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=z+0.5;          // -2       [1,2,0]         0,0,1/2         0,0,0
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=x-y;           SpaceGroupElement[11].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,0,0
      break;

    case 479: // [185] P 63 c m (P 6c -2)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 6^1      [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z+0.5;           // 6^-1     [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[6].x=y;              SpaceGroupElement[6].y=x;              SpaceGroupElement[6].z=z;               // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[7].x=-x+y;           SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=z;               // -2       [2,1,0]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=z+0.5;           // -2       [1,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[10].x=x-y;           SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=z;              // -2       [1,2,0]         0,0,0           0,0,0
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=x-y;           SpaceGroupElement[11].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,0,0
      break;

    case 480: // [186] P 63 m c (P 6c -2c)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 6^1      [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z+0.5;           // 6^-1     [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[6].x=y;              SpaceGroupElement[6].y=x;              SpaceGroupElement[6].z=z+0.5;           // -2       [-1,1,0]        0,0,1/2         0,0,0
      SpaceGroupElement[7].x=-x+y;           SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=z+0.5;           // -2       [2,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=z;               // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x-y;           SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=z+0.5;          // -2       [1,2,0]         0,0,1/2         0,0,0
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=x-y;           SpaceGroupElement[11].z=z;              // -2       [0,1,0]         0,0,0           0,0,0
      break;

    case 481: // [187] P -6 m 2 (P -6 2)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -6^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=x-y;            SpaceGroupElement[5].z=-z;              // -6^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[7].x=-x+y;           SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=z;               // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=-z;              // 2        [2,1,0]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=z;               // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;             // 2        [1,2,0]         0,0,0           0,0,0
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=x-y;           SpaceGroupElement[11].z=z;              // -2       [0,1,0]         0,0,0           0,0,0
      break;

    case 482: // [188] P -6 c 2 (P -6c 2)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z+0.5;          // -6^1     [0,0,1]         0,0,0           0,0,1/4
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // -2       [0,0,1]         0,0,0           0,0,1/4
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=x-y;            SpaceGroupElement[5].z=-z+0.5;          // -6^-1    [0,0,1]         0,0,0           0,0,1/4
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[7].x=-x+y;           SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=z+0.5;           // -2       [1,0,0]         0,0,1/2         0,0,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=-z;              // 2        [2,1,0]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=z+0.5;           // -2       [1,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;             // 2        [1,2,0]         0,0,0           0,0,0
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=x-y;           SpaceGroupElement[11].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,0,0
      break;

    case 483: // [189] P -6 2 m (P -6 -2)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -6^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=x-y;            SpaceGroupElement[5].z=-z;              // -6^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=y;              SpaceGroupElement[6].y=x;              SpaceGroupElement[6].z=z;               // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=z;               // -2       [2,1,0]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x-y;           SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=z;              // -2       [1,2,0]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=-z;             // 2        [0,1,0]         0,0,0           0,0,0
      break;

    case 484: // [190] P -6 2 c (P -6c -2c)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z+0.5;          // -6^1     [0,0,1]         0,0,0           0,0,1/4
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // -2       [0,0,1]         0,0,0           0,0,1/4
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=x-y;            SpaceGroupElement[5].z=-z+0.5;          // -6^-1    [0,0,1]         0,0,0           0,0,1/4
      SpaceGroupElement[6].x=y;              SpaceGroupElement[6].y=x;              SpaceGroupElement[6].z=z+0.5;           // -2       [-1,1,0]        0,0,1/2         0,0,0
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=z+0.5;           // -2       [2,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[10].x=x-y;           SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=z+0.5;          // -2       [1,2,0]         0,0,1/2         0,0,0
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=-z;             // 2        [0,1,0]         0,0,0           0,0,0
      break;

    case 485: // [191] P 6/m m m (-P 6 2)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 6^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z;               // 6^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=-z;              // 2        [2,1,0]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;             // 2        [1,2,0]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=-z;             // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[13].x=-x+y;          SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=-z;             // -6^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-x+y;          SpaceGroupElement[14].z=-z;             // -3^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=y;             SpaceGroupElement[15].z=-z;             // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[16].x=x-y;           SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=-z;             // -3^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=-y;            SpaceGroupElement[17].y=x-y;           SpaceGroupElement[17].z=-z;             // -6^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[18].x=y;             SpaceGroupElement[18].y=x;             SpaceGroupElement[18].z=z;              // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[19].x=-x+y;          SpaceGroupElement[19].y=y;             SpaceGroupElement[19].z=z;              // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[20].x=-x;            SpaceGroupElement[20].y=-x+y;          SpaceGroupElement[20].z=z;              // -2       [2,1,0]         0,0,0           0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=z;              // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[22].x=x-y;           SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=z;              // -2       [1,2,0]         0,0,0           0,0,0
      SpaceGroupElement[23].x=x;             SpaceGroupElement[23].y=x-y;           SpaceGroupElement[23].z=z;              // -2       [0,1,0]         0,0,0           0,0,0
      break;

    case 486: // [192] P 6/m c c (-P 6 2c)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 6^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z;               // 6^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           0,0,1/4
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=-z+0.5;          // 2        [2,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z+0.5;          // 2        [1,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z+0.5;         // 2        [1,2,0]         0,0,0           0,0,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[13].x=-x+y;          SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=-z;             // -6^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-x+y;          SpaceGroupElement[14].z=-z;             // -3^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=y;             SpaceGroupElement[15].z=-z;             // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[16].x=x-y;           SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=-z;             // -3^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=-y;            SpaceGroupElement[17].y=x-y;           SpaceGroupElement[17].z=-z;             // -6^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[18].x=y;             SpaceGroupElement[18].y=x;             SpaceGroupElement[18].z=z-0.5;          // -2       [-1,1,0]        0,0,-1/2        0,0,0
      SpaceGroupElement[19].x=-x+y;          SpaceGroupElement[19].y=y;             SpaceGroupElement[19].z=z-0.5;          // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[20].x=-x;            SpaceGroupElement[20].y=-x+y;          SpaceGroupElement[20].z=z-0.5;          // -2       [2,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=z-0.5;          // -2       [1,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[22].x=x-y;           SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=z-0.5;          // -2       [1,2,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[23].x=x;             SpaceGroupElement[23].y=x-y;           SpaceGroupElement[23].z=z-0.5;          // -2       [0,1,0]         0,0,-1/2        0,0,0
      break;

    case 487: // [193] P 63/m c m (-P 6c 2)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 6^1      [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z+0.5;           // 6^-1     [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=-z;              // 2        [2,1,0]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z+0.5;          // 2        [1,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;             // 2        [1,2,0]         0,0,0           0,0,0
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=-z+0.5;         // 2        [0,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[13].x=-x+y;          SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=-z-0.5;         // -6^1     [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-x+y;          SpaceGroupElement[14].z=-z;             // -3^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=y;             SpaceGroupElement[15].z=-z-0.5;         // -2       [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[16].x=x-y;           SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=-z;             // -3^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=-y;            SpaceGroupElement[17].y=x-y;           SpaceGroupElement[17].z=-z-0.5;         // -6^-1    [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[18].x=y;             SpaceGroupElement[18].y=x;             SpaceGroupElement[18].z=z;              // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[19].x=-x+y;          SpaceGroupElement[19].y=y;             SpaceGroupElement[19].z=z-0.5;          // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[20].x=-x;            SpaceGroupElement[20].y=-x+y;          SpaceGroupElement[20].z=z;              // -2       [2,1,0]         0,0,0           0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=z-0.5;          // -2       [1,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[22].x=x-y;           SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=z;              // -2       [1,2,0]         0,0,0           0,0,0
      SpaceGroupElement[23].x=x;             SpaceGroupElement[23].y=x-y;           SpaceGroupElement[23].z=z-0.5;          // -2       [0,1,0]         0,0,-1/2        0,0,0
      break;

    case 488: // [194] P 63/m m c (-P 6c 2c)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z+0.5;           // 6^1      [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;               // 3^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;               // 3^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=z+0.5;           // 6^-1     [0,0,1]         0,0,1/2         0,0,0
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           0,0,1/4
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=-z+0.5;          // 2        [2,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z+0.5;         // 2        [1,2,0]         0,0,0           0,0,1/4
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=-z;             // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[13].x=-x+y;          SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=-z-0.5;         // -6^1     [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-x+y;          SpaceGroupElement[14].z=-z;             // -3^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=y;             SpaceGroupElement[15].z=-z-0.5;         // -2       [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[16].x=x-y;           SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=-z;             // -3^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=-y;            SpaceGroupElement[17].y=x-y;           SpaceGroupElement[17].z=-z-0.5;         // -6^-1    [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[18].x=y;             SpaceGroupElement[18].y=x;             SpaceGroupElement[18].z=z-0.5;          // -2       [-1,1,0]        0,0,-1/2        0,0,0
      SpaceGroupElement[19].x=-x+y;          SpaceGroupElement[19].y=y;             SpaceGroupElement[19].z=z;              // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[20].x=-x;            SpaceGroupElement[20].y=-x+y;          SpaceGroupElement[20].z=z-0.5;          // -2       [2,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=z;              // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[22].x=x-y;           SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=z-0.5;          // -2       [1,2,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[23].x=x;             SpaceGroupElement[23].y=x-y;           SpaceGroupElement[23].z=z;              // -2       [0,1,0]         0,0,0           0,0,0
      break;

    // CUBIC GROUPS
    // ============

    case 489: // [195] P 2 3 (P 2 2 3)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=z;              SpaceGroupElement[4].y=x;              SpaceGroupElement[4].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-z;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=y;               // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[6].x=z;              SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-y;              // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[7].x=-z;             SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-y;              // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-z;             SpaceGroupElement[9].z=-x;              // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      break;

    case 490: // [196] F 2 3 (F 2 2 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=z;              SpaceGroupElement[4].y=x;              SpaceGroupElement[4].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-z;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=y;               // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[6].x=z;              SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-y;              // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[7].x=-z;             SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-y;              // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-z;             SpaceGroupElement[9].z=-x;              // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[12].x=x;             SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[13].x=-x;            SpaceGroupElement[13].y=-y+0.5;        SpaceGroupElement[13].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y+0.5;        SpaceGroupElement[14].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=y+0.5;         SpaceGroupElement[15].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x+0.5;         SpaceGroupElement[16].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     -1/3,-1/6,0
      SpaceGroupElement[17].x=-z;            SpaceGroupElement[17].y=-x+0.5;        SpaceGroupElement[17].z=y+0.5;          // 3^-1     [-1,1,1]        -1/3,1/3,1/3    1/3,-1/6,0
      SpaceGroupElement[18].x=z;             SpaceGroupElement[18].y=-x+0.5;        SpaceGroupElement[18].z=-y+0.5;         // 3^-1     [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[19].x=-z;            SpaceGroupElement[19].y=x+0.5;         SpaceGroupElement[19].z=-y+0.5;         // 3^1      [-1,-1,1]       0,0,0           0,1/2,0
      SpaceGroupElement[20].x=y;             SpaceGroupElement[20].y=z+0.5;         SpaceGroupElement[20].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,1/6,0
      SpaceGroupElement[21].x=y;             SpaceGroupElement[21].y=-z+0.5;        SpaceGroupElement[21].z=-x+0.5;         // 3^-1     [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[22].x=-y;            SpaceGroupElement[22].y=z+0.5;         SpaceGroupElement[22].z=-x+0.5;         // 3^1      [-1,1,1]        -1/3,1/3,1/3    1/6,1/6,0
      SpaceGroupElement[23].x=-y;            SpaceGroupElement[23].y=-z+0.5;        SpaceGroupElement[23].z=x+0.5;          // 3^1      [1,-1,1]        0,0,0           -1/2,1/2,0
      SpaceGroupElement[24].x=x+0.5;         SpaceGroupElement[24].y=y;             SpaceGroupElement[24].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[25].x=-x+0.5;        SpaceGroupElement[25].y=-y;            SpaceGroupElement[25].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[26].x=x+0.5;         SpaceGroupElement[26].y=-y;            SpaceGroupElement[26].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[27].x=-x+0.5;        SpaceGroupElement[27].y=y;             SpaceGroupElement[27].z=-z+0.5;         // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[28].x=z+0.5;         SpaceGroupElement[28].y=x;             SpaceGroupElement[28].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,-1/6,0
      SpaceGroupElement[29].x=-z+0.5;        SpaceGroupElement[29].y=-x;            SpaceGroupElement[29].z=y+0.5;          // 3^-1     [-1,1,1]        0,0,0           1/2,-1/2,0
      SpaceGroupElement[30].x=z+0.5;         SpaceGroupElement[30].y=-x;            SpaceGroupElement[30].z=-y+0.5;         // 3^-1     [1,-1,1]        1/3,-1/3,1/3    1/6,1/6,0
      SpaceGroupElement[31].x=-z+0.5;        SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=-y+0.5;         // 3^1      [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[32].x=y+0.5;         SpaceGroupElement[32].y=z;             SpaceGroupElement[32].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,-1/3,0
      SpaceGroupElement[33].x=y+0.5;         SpaceGroupElement[33].y=-z;            SpaceGroupElement[33].z=-x+0.5;         // 3^-1     [-1,-1,1]       0,0,0           1/2,0,0
      SpaceGroupElement[34].x=-y+0.5;        SpaceGroupElement[34].y=z;             SpaceGroupElement[34].z=-x+0.5;         // 3^1      [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[35].x=-y+0.5;        SpaceGroupElement[35].y=-z;            SpaceGroupElement[35].z=x+0.5;          // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      SpaceGroupElement[36].x=x+0.5;         SpaceGroupElement[36].y=y+0.5;         SpaceGroupElement[36].z=z;              // 1        -               -               -
      SpaceGroupElement[37].x=-x+0.5;        SpaceGroupElement[37].y=-y+0.5;        SpaceGroupElement[37].z=z;              // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[38].x=x+0.5;         SpaceGroupElement[38].y=-y+0.5;        SpaceGroupElement[38].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[39].x=-x+0.5;        SpaceGroupElement[39].y=y+0.5;         SpaceGroupElement[39].z=-z;             // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[40].x=z+0.5;         SpaceGroupElement[40].y=x+0.5;         SpaceGroupElement[40].z=y;              // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,1/3,0
      SpaceGroupElement[41].x=-z+0.5;        SpaceGroupElement[41].y=-x+0.5;        SpaceGroupElement[41].z=y;              // 3^-1     [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[42].x=z+0.5;         SpaceGroupElement[42].y=-x+0.5;        SpaceGroupElement[42].z=-y;             // 3^-1     [1,-1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[43].x=-z+0.5;        SpaceGroupElement[43].y=x+0.5;         SpaceGroupElement[43].z=-y;             // 3^1      [-1,-1,1]       1/3,1/3,-1/3    1/6,1/3,0
      SpaceGroupElement[44].x=y+0.5;         SpaceGroupElement[44].y=z+0.5;         SpaceGroupElement[44].z=x;              // 3^-1     [1,1,1]         1/3,1/3,1/3     1/3,1/6,0
      SpaceGroupElement[45].x=y+0.5;         SpaceGroupElement[45].y=-z+0.5;        SpaceGroupElement[45].z=-x;             // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    1/3,1/6,0
      SpaceGroupElement[46].x=-y+0.5;        SpaceGroupElement[46].y=z+0.5;         SpaceGroupElement[46].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[47].x=-y+0.5;        SpaceGroupElement[47].y=-z+0.5;        SpaceGroupElement[47].z=x;              // 3^1      [1,-1,1]        0,0,0           0,1/2,0
      break;

    case 491: // [197] I 2 3 (I 2 2 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=z;              SpaceGroupElement[4].y=x;              SpaceGroupElement[4].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-z;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=y;               // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[6].x=z;              SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-y;              // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[7].x=-z;             SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-y;              // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-z;             SpaceGroupElement[9].z=-x;              // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[12].x=x+0.5;         SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[13].x=-x+0.5;        SpaceGroupElement[13].y=-y+0.5;        SpaceGroupElement[13].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[14].x=x+0.5;         SpaceGroupElement[14].y=-y+0.5;        SpaceGroupElement[14].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[15].x=-x+0.5;        SpaceGroupElement[15].y=y+0.5;         SpaceGroupElement[15].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[16].x=z+0.5;         SpaceGroupElement[16].y=x+0.5;         SpaceGroupElement[16].z=y+0.5;          // 3^1      [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[17].x=-z+0.5;        SpaceGroupElement[17].y=-x+0.5;        SpaceGroupElement[17].z=y+0.5;          // 3^-1     [-1,1,1]        -1/6,1/6,1/6    2/3,-1/3,0
      SpaceGroupElement[18].x=z+0.5;         SpaceGroupElement[18].y=-x+0.5;        SpaceGroupElement[18].z=-y+0.5;         // 3^-1     [1,-1,1]        1/6,-1/6,1/6    1/3,1/3,0
      SpaceGroupElement[19].x=-z+0.5;        SpaceGroupElement[19].y=x+0.5;         SpaceGroupElement[19].z=-y+0.5;         // 3^1      [-1,-1,1]       1/6,1/6,-1/6    1/3,2/3,0
      SpaceGroupElement[20].x=y+0.5;         SpaceGroupElement[20].y=z+0.5;         SpaceGroupElement[20].z=x+0.5;          // 3^-1     [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[21].x=y+0.5;         SpaceGroupElement[21].y=-z+0.5;        SpaceGroupElement[21].z=-x+0.5;         // 3^-1     [-1,-1,1]       1/6,1/6,-1/6    2/3,1/3,0
      SpaceGroupElement[22].x=-y+0.5;        SpaceGroupElement[22].y=z+0.5;         SpaceGroupElement[22].z=-x+0.5;         // 3^1      [-1,1,1]        -1/6,1/6,1/6    1/3,1/3,0
      SpaceGroupElement[23].x=-y+0.5;        SpaceGroupElement[23].y=-z+0.5;        SpaceGroupElement[23].z=x+0.5;          // 3^1      [1,-1,1]        1/6,-1/6,1/6    -1/3,2/3,0
      break;

    case 492: // [198] P 21 3 (P 2ac 2ab 3)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[4].x=z;              SpaceGroupElement[4].y=x;              SpaceGroupElement[4].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-z+0.5;         SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=y+0.5;           // 3^-1     [-1,1,1]        0,0,0           1/2,-1/2,0
      SpaceGroupElement[6].x=z+0.5;          SpaceGroupElement[6].y=-x+0.5;         SpaceGroupElement[6].z=-y;              // 3^-1     [1,-1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[7].x=-z;             SpaceGroupElement[7].y=x+0.5;          SpaceGroupElement[7].z=-y+0.5;          // 3^1      [-1,-1,1]       0,0,0           0,1/2,0
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y+0.5;          SpaceGroupElement[9].y=-z+0.5;         SpaceGroupElement[9].z=-x;              // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    1/3,1/6,0
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z+0.5;         SpaceGroupElement[10].z=-x+0.5;         // 3^1      [-1,1,1]        -1/3,1/3,1/3    1/6,1/6,0
      SpaceGroupElement[11].x=-y+0.5;        SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=x+0.5;          // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      break;

    case 493: // [199] I 21 3 (I 2b 2c 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[4].x=z;              SpaceGroupElement[4].y=x;              SpaceGroupElement[4].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-z;             SpaceGroupElement[5].y=-x+0.5;         SpaceGroupElement[5].z=y;               // 3^-1     [-1,1,1]        -1/6,1/6,1/6    1/6,1/6,0
      SpaceGroupElement[6].x=z;              SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-y+0.5;          // 3^-1     [1,-1,1]        1/6,-1/6,1/6    -1/6,1/3,0
      SpaceGroupElement[7].x=-z;             SpaceGroupElement[7].y=x+0.5;          SpaceGroupElement[7].z=-y+0.5;          // 3^1      [-1,-1,1]       0,0,0           0,1/2,0
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-z;             SpaceGroupElement[9].z=-x+0.5;          // 3^-1     [-1,-1,1]       -1/6,-1/6,1/6   1/3,1/6,0
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z+0.5;         SpaceGroupElement[10].z=-x+0.5;         // 3^1      [-1,1,1]        -1/3,1/3,1/3    1/6,1/6,0
      SpaceGroupElement[11].x=-y+0.5;        SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=x+0.5;          // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      SpaceGroupElement[12].x=x+0.5;         SpaceGroupElement[12].y=y+0.5;         SpaceGroupElement[12].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[13].x=-x+0.5;        SpaceGroupElement[13].y=-y;            SpaceGroupElement[13].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/2,0
      SpaceGroupElement[14].x=x+0.5;         SpaceGroupElement[14].y=-y+0.5;        SpaceGroupElement[14].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,1/2
      SpaceGroupElement[15].x=-x+0.5;        SpaceGroupElement[15].y=y;             SpaceGroupElement[15].z=-z;             // 2        [0,1,0]         0,1,0           1/4,0,1/2
      SpaceGroupElement[16].x=z+0.5;         SpaceGroupElement[16].y=x+0.5;         SpaceGroupElement[16].z=y+0.5;          // 3^1      [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[17].x=-z+0.5;        SpaceGroupElement[17].y=-x;            SpaceGroupElement[17].z=y+0.5;          // 3^-1     [-1,1,1]        -1/3,1/3,1/3    5/6,-1/6,0
      SpaceGroupElement[18].x=z+0.5;         SpaceGroupElement[18].y=-x+0.5;        SpaceGroupElement[18].z=-y;             // 3^-1     [1,-1,1]        1/3,-1/3,1/3    1/6,2/3,0
      SpaceGroupElement[19].x=-z+0.5;        SpaceGroupElement[19].y=x;             SpaceGroupElement[19].z=-y;             // 3^1      [-1,-1,1]       1/6,1/6,-1/6    1/3,7/6,0
      SpaceGroupElement[20].x=y+0.5;         SpaceGroupElement[20].y=z+0.5;         SpaceGroupElement[20].z=x+0.5;          // 3^-1     [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[21].x=y+0.5;         SpaceGroupElement[21].y=-z+0.5;        SpaceGroupElement[21].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           1,1/2,0
      SpaceGroupElement[22].x=-y+0.5;        SpaceGroupElement[22].y=z;             SpaceGroupElement[22].z=-x;             // 3^1      [-1,1,1]        -1/2,1/2,1/2    1/2,1/2,0
      SpaceGroupElement[23].x=-y;            SpaceGroupElement[23].y=-z+0.5;        SpaceGroupElement[23].z=x;              // 3^1      [1,-1,1]        1/2,-1/2,1/2    -1/2,1,0
      break;

    case 494: // [200] P m -3 (-P 2 2 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=z;              SpaceGroupElement[4].y=x;              SpaceGroupElement[4].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-z;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=y;               // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[6].x=z;              SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-y;              // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[7].x=-z;             SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-y;              // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-z;             SpaceGroupElement[9].z=-x;              // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z;             // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z;              // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z;              // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[16].x=-z;            SpaceGroupElement[16].y=-x;            SpaceGroupElement[16].z=-y;             // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=z;             SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=-y;             // -3^-1    [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[18].x=-z;            SpaceGroupElement[18].y=x;             SpaceGroupElement[18].z=y;              // -3^-1    [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[19].x=z;             SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=y;              // -3^1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[20].x=-y;            SpaceGroupElement[20].y=-z;            SpaceGroupElement[20].z=-x;             // -3^-1    [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=z;             SpaceGroupElement[21].z=x;              // -3^-1    [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[22].x=y;             SpaceGroupElement[22].y=-z;            SpaceGroupElement[22].z=x;              // -3^1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[23].x=y;             SpaceGroupElement[23].y=z;             SpaceGroupElement[23].z=-x;             // -3^1     [1,-1,1]        0,0,0           0,0,0
      break;

    case 495: // [201] P n -3 Origin choice 1 (P 2 2 3 -1n)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=z;              SpaceGroupElement[4].y=x;              SpaceGroupElement[4].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-z;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=y;               // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[6].x=z;              SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-y;              // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[7].x=-z;             SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-y;              // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-z;             SpaceGroupElement[9].z=-x;              // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[12].x=-x+0.5;        SpaceGroupElement[12].y=-y+0.5;        SpaceGroupElement[12].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=y+0.5;         SpaceGroupElement[13].z=-z+0.5;         // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[14].x=-x+0.5;        SpaceGroupElement[14].y=y+0.5;         SpaceGroupElement[14].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=-y+0.5;        SpaceGroupElement[15].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[16].x=-z+0.5;        SpaceGroupElement[16].y=-x+0.5;        SpaceGroupElement[16].z=-y+0.5;         // -3^1     [1,1,1]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[17].x=z+0.5;         SpaceGroupElement[17].y=x+0.5;         SpaceGroupElement[17].z=-y+0.5;         // -3^-1    [-1,1,1]        0,0,0           1/4,3/4,-1/4
      SpaceGroupElement[18].x=-z+0.5;        SpaceGroupElement[18].y=x+0.5;         SpaceGroupElement[18].z=y+0.5;          // -3^-1    [1,-1,1]        0,0,0           -1/4,1/4,3/4
      SpaceGroupElement[19].x=z+0.5;         SpaceGroupElement[19].y=-x+0.5;        SpaceGroupElement[19].z=y+0.5;          // -3^1     [-1,-1,1]       0,0,0           3/4,-1/4,1/4
      SpaceGroupElement[20].x=-y+0.5;        SpaceGroupElement[20].y=-z+0.5;        SpaceGroupElement[20].z=-x+0.5;         // -3^-1    [1,1,1]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[21].x=-y+0.5;        SpaceGroupElement[21].y=z+0.5;         SpaceGroupElement[21].z=x+0.5;          // -3^-1    [-1,-1,1]       0,0,0           -1/4,3/4,1/4
      SpaceGroupElement[22].x=y+0.5;         SpaceGroupElement[22].y=-z+0.5;        SpaceGroupElement[22].z=x+0.5;          // -3^1     [-1,1,1]        0,0,0           1/4,-1/4,3/4
      SpaceGroupElement[23].x=y+0.5;         SpaceGroupElement[23].y=z+0.5;         SpaceGroupElement[23].z=-x+0.5;         // -3^1     [1,-1,1]        0,0,0           3/4,1/4,-1/4
      break;
    case 496: // [201] P n -3 Origin choice 2 (-P 2ab 2bc 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[3].x=-x+0.5;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[4].x=z;              SpaceGroupElement[4].y=x;              SpaceGroupElement[4].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-z+0.5;         SpaceGroupElement[5].y=-x+0.5;         SpaceGroupElement[5].z=y;               // 3^-1     [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[6].x=z;              SpaceGroupElement[6].y=-x+0.5;         SpaceGroupElement[6].z=-y+0.5;          // 3^-1     [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[7].x=-z+0.5;         SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-y+0.5;          // 3^1      [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-z+0.5;         SpaceGroupElement[9].z=-x+0.5;          // 3^-1     [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[10].x=-y+0.5;        SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x+0.5;         // 3^1      [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[11].x=-y+0.5;        SpaceGroupElement[11].y=-z+0.5;        SpaceGroupElement[11].z=x;              // 3^1      [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[13].x=x-0.5;         SpaceGroupElement[13].y=y-0.5;         SpaceGroupElement[13].z=-z;             // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y-0.5;         SpaceGroupElement[14].z=z-0.5;          // -2       [1,0,0]         0,-1/2,-1/2     0,0,0
      SpaceGroupElement[15].x=x-0.5;         SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z-0.5;          // -2       [0,1,0]         -1/2,0,-1/2     0,0,0
      SpaceGroupElement[16].x=-z;            SpaceGroupElement[16].y=-x;            SpaceGroupElement[16].z=-y;             // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=z-0.5;         SpaceGroupElement[17].y=x-0.5;         SpaceGroupElement[17].z=-y;             // -3^-1    [-1,1,1]        0,0,0           0,-1/2,1/2
      SpaceGroupElement[18].x=-z;            SpaceGroupElement[18].y=x-0.5;         SpaceGroupElement[18].z=y-0.5;          // -3^-1    [1,-1,1]        0,0,0           1/2,0,-1/2
      SpaceGroupElement[19].x=z-0.5;         SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=y-0.5;          // -3^1     [-1,-1,1]       0,0,0           -1/2,1/2,0
      SpaceGroupElement[20].x=-y;            SpaceGroupElement[20].y=-z;            SpaceGroupElement[20].z=-x;             // -3^-1    [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=z-0.5;         SpaceGroupElement[21].z=x-0.5;          // -3^-1    [-1,-1,1]       0,0,0           1/2,-1/2,0
      SpaceGroupElement[22].x=y-0.5;         SpaceGroupElement[22].y=-z;            SpaceGroupElement[22].z=x-0.5;          // -3^1     [-1,1,1]        0,0,0           0,1/2,-1/2
      SpaceGroupElement[23].x=y-0.5;         SpaceGroupElement[23].y=z-0.5;         SpaceGroupElement[23].z=-x;             // -3^1     [1,-1,1]        0,0,0           -1/2,0,1/2
      break;

    case 497: // [202] F m -3 (-F 2 2 3)
      SpaceGroupSize=96;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=z;              SpaceGroupElement[4].y=x;              SpaceGroupElement[4].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-z;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=y;               // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[6].x=z;              SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-y;              // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[7].x=-z;             SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-y;              // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-z;             SpaceGroupElement[9].z=-x;              // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z;             // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z;              // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z;              // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[16].x=-z;            SpaceGroupElement[16].y=-x;            SpaceGroupElement[16].z=-y;             // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=z;             SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=-y;             // -3^-1    [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[18].x=-z;            SpaceGroupElement[18].y=x;             SpaceGroupElement[18].z=y;              // -3^-1    [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[19].x=z;             SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=y;              // -3^1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[20].x=-y;            SpaceGroupElement[20].y=-z;            SpaceGroupElement[20].z=-x;             // -3^-1    [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=z;             SpaceGroupElement[21].z=x;              // -3^-1    [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[22].x=y;             SpaceGroupElement[22].y=-z;            SpaceGroupElement[22].z=x;              // -3^1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[23].x=y;             SpaceGroupElement[23].y=z;             SpaceGroupElement[23].z=-x;             // -3^1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[24].x=x;             SpaceGroupElement[24].y=y+0.5;         SpaceGroupElement[24].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[25].x=-x;            SpaceGroupElement[25].y=-y+0.5;        SpaceGroupElement[25].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=-y+0.5;        SpaceGroupElement[26].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[27].x=-x;            SpaceGroupElement[27].y=y+0.5;         SpaceGroupElement[27].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[28].x=z;             SpaceGroupElement[28].y=x+0.5;         SpaceGroupElement[28].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     -1/3,-1/6,0
      SpaceGroupElement[29].x=-z;            SpaceGroupElement[29].y=-x+0.5;        SpaceGroupElement[29].z=y+0.5;          // 3^-1     [-1,1,1]        -1/3,1/3,1/3    1/3,-1/6,0
      SpaceGroupElement[30].x=z;             SpaceGroupElement[30].y=-x+0.5;        SpaceGroupElement[30].z=-y+0.5;         // 3^-1     [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[31].x=-z;            SpaceGroupElement[31].y=x+0.5;         SpaceGroupElement[31].z=-y+0.5;         // 3^1      [-1,-1,1]       0,0,0           0,1/2,0
      SpaceGroupElement[32].x=y;             SpaceGroupElement[32].y=z+0.5;         SpaceGroupElement[32].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,1/6,0
      SpaceGroupElement[33].x=y;             SpaceGroupElement[33].y=-z+0.5;        SpaceGroupElement[33].z=-x+0.5;         // 3^-1     [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[34].x=-y;            SpaceGroupElement[34].y=z+0.5;         SpaceGroupElement[34].z=-x+0.5;         // 3^1      [-1,1,1]        -1/3,1/3,1/3    1/6,1/6,0
      SpaceGroupElement[35].x=-y;            SpaceGroupElement[35].y=-z+0.5;        SpaceGroupElement[35].z=x+0.5;          // 3^1      [1,-1,1]        0,0,0           -1/2,1/2,0
      SpaceGroupElement[36].x=-x;            SpaceGroupElement[36].y=-y+0.5;        SpaceGroupElement[36].z=-z+0.5;         // -1       -               -               0,1/4,1/4
      SpaceGroupElement[37].x=x;             SpaceGroupElement[37].y=y+0.5;         SpaceGroupElement[37].z=-z+0.5;         // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[38].x=-x;            SpaceGroupElement[38].y=y+0.5;         SpaceGroupElement[38].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       0,0,0
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=-y+0.5;        SpaceGroupElement[39].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,1/4,0
      SpaceGroupElement[40].x=-z;            SpaceGroupElement[40].y=-x+0.5;        SpaceGroupElement[40].z=-y+0.5;         // -3^1     [1,1,1]         0,0,0           0,1/2,0
      SpaceGroupElement[41].x=z;             SpaceGroupElement[41].y=x+0.5;         SpaceGroupElement[41].z=-y+0.5;         // -3^-1    [-1,1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[42].x=-z;            SpaceGroupElement[42].y=x+0.5;         SpaceGroupElement[42].z=y+0.5;          // -3^-1    [1,-1,1]        0,0,0           -1/2,0,1/2
      SpaceGroupElement[43].x=z;             SpaceGroupElement[43].y=-x+0.5;        SpaceGroupElement[43].z=y+0.5;          // -3^1     [-1,-1,1]       0,0,0           1/2,0,1/2
      SpaceGroupElement[44].x=-y;            SpaceGroupElement[44].y=-z+0.5;        SpaceGroupElement[44].z=-x+0.5;         // -3^-1    [1,1,1]         0,0,0           0,0,1/2
      SpaceGroupElement[45].x=-y;            SpaceGroupElement[45].y=z+0.5;         SpaceGroupElement[45].z=x+0.5;          // -3^-1    [-1,-1,1]       0,0,0           -1/2,1/2,0
      SpaceGroupElement[46].x=y;             SpaceGroupElement[46].y=-z+0.5;        SpaceGroupElement[46].z=x+0.5;          // -3^1     [-1,1,1]        0,0,0           0,0,1/2
      SpaceGroupElement[47].x=y;             SpaceGroupElement[47].y=z+0.5;         SpaceGroupElement[47].z=-x+0.5;         // -3^1     [1,-1,1]        0,0,0           1/2,1/2,0
      SpaceGroupElement[48].x=x+0.5;         SpaceGroupElement[48].y=y;             SpaceGroupElement[48].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[49].x=-x+0.5;        SpaceGroupElement[49].y=-y;            SpaceGroupElement[49].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[50].x=x+0.5;         SpaceGroupElement[50].y=-y;            SpaceGroupElement[50].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[51].x=-x+0.5;        SpaceGroupElement[51].y=y;             SpaceGroupElement[51].z=-z+0.5;         // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[52].x=z+0.5;         SpaceGroupElement[52].y=x;             SpaceGroupElement[52].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,-1/6,0
      SpaceGroupElement[53].x=-z+0.5;        SpaceGroupElement[53].y=-x;            SpaceGroupElement[53].z=y+0.5;          // 3^-1     [-1,1,1]        0,0,0           1/2,-1/2,0
      SpaceGroupElement[54].x=z+0.5;         SpaceGroupElement[54].y=-x;            SpaceGroupElement[54].z=-y+0.5;         // 3^-1     [1,-1,1]        1/3,-1/3,1/3    1/6,1/6,0
      SpaceGroupElement[55].x=-z+0.5;        SpaceGroupElement[55].y=x;             SpaceGroupElement[55].z=-y+0.5;         // 3^1      [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[56].x=y+0.5;         SpaceGroupElement[56].y=z;             SpaceGroupElement[56].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,-1/3,0
      SpaceGroupElement[57].x=y+0.5;         SpaceGroupElement[57].y=-z;            SpaceGroupElement[57].z=-x+0.5;         // 3^-1     [-1,-1,1]       0,0,0           1/2,0,0
      SpaceGroupElement[58].x=-y+0.5;        SpaceGroupElement[58].y=z;             SpaceGroupElement[58].z=-x+0.5;         // 3^1      [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[59].x=-y+0.5;        SpaceGroupElement[59].y=-z;            SpaceGroupElement[59].z=x+0.5;          // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      SpaceGroupElement[60].x=-x+0.5;        SpaceGroupElement[60].y=-y;            SpaceGroupElement[60].z=-z+0.5;         // -1       -               -               1/4,0,1/4
      SpaceGroupElement[61].x=x+0.5;         SpaceGroupElement[61].y=y;             SpaceGroupElement[61].z=-z+0.5;         // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[62].x=-x+0.5;        SpaceGroupElement[62].y=y;             SpaceGroupElement[62].z=z+0.5;          // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[63].x=x+0.5;         SpaceGroupElement[63].y=-y;            SpaceGroupElement[63].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,0,0
      SpaceGroupElement[64].x=-z+0.5;        SpaceGroupElement[64].y=-x;            SpaceGroupElement[64].z=-y+0.5;         // -3^1     [1,1,1]         0,0,0           0,0,1/2
      SpaceGroupElement[65].x=z+0.5;         SpaceGroupElement[65].y=x;             SpaceGroupElement[65].z=-y+0.5;         // -3^-1    [-1,1,1]        0,0,0           1/2,1/2,0
      SpaceGroupElement[66].x=-z+0.5;        SpaceGroupElement[66].y=x;             SpaceGroupElement[66].z=y+0.5;          // -3^-1    [1,-1,1]        0,0,0           0,0,1/2
      SpaceGroupElement[67].x=z+0.5;         SpaceGroupElement[67].y=-x;            SpaceGroupElement[67].z=y+0.5;          // -3^1     [-1,-1,1]       0,0,0           1/2,-1/2,0
      SpaceGroupElement[68].x=-y+0.5;        SpaceGroupElement[68].y=-z;            SpaceGroupElement[68].z=-x+0.5;         // -3^-1    [1,1,1]         0,0,0           1/2,0,0
      SpaceGroupElement[69].x=-y+0.5;        SpaceGroupElement[69].y=z;             SpaceGroupElement[69].z=x+0.5;          // -3^-1    [-1,-1,1]       0,0,0           0,1/2,1/2
      SpaceGroupElement[70].x=y+0.5;         SpaceGroupElement[70].y=-z;            SpaceGroupElement[70].z=x+0.5;          // -3^1     [-1,1,1]        0,0,0           0,-1/2,1/2
      SpaceGroupElement[71].x=y+0.5;         SpaceGroupElement[71].y=z;             SpaceGroupElement[71].z=-x+0.5;         // -3^1     [1,-1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[72].x=x+0.5;         SpaceGroupElement[72].y=y+0.5;         SpaceGroupElement[72].z=z;              // 1        -               -               -
      SpaceGroupElement[73].x=-x+0.5;        SpaceGroupElement[73].y=-y+0.5;        SpaceGroupElement[73].z=z;              // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[74].x=x+0.5;         SpaceGroupElement[74].y=-y+0.5;        SpaceGroupElement[74].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[75].x=-x+0.5;        SpaceGroupElement[75].y=y+0.5;         SpaceGroupElement[75].z=-z;             // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[76].x=z+0.5;         SpaceGroupElement[76].y=x+0.5;         SpaceGroupElement[76].z=y;              // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,1/3,0
      SpaceGroupElement[77].x=-z+0.5;        SpaceGroupElement[77].y=-x+0.5;        SpaceGroupElement[77].z=y;              // 3^-1     [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[78].x=z+0.5;         SpaceGroupElement[78].y=-x+0.5;        SpaceGroupElement[78].z=-y;             // 3^-1     [1,-1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[79].x=-z+0.5;        SpaceGroupElement[79].y=x+0.5;         SpaceGroupElement[79].z=-y;             // 3^1      [-1,-1,1]       1/3,1/3,-1/3    1/6,1/3,0
      SpaceGroupElement[80].x=y+0.5;         SpaceGroupElement[80].y=z+0.5;         SpaceGroupElement[80].z=x;              // 3^-1     [1,1,1]         1/3,1/3,1/3     1/3,1/6,0
      SpaceGroupElement[81].x=y+0.5;         SpaceGroupElement[81].y=-z+0.5;        SpaceGroupElement[81].z=-x;             // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    1/3,1/6,0
      SpaceGroupElement[82].x=-y+0.5;        SpaceGroupElement[82].y=z+0.5;         SpaceGroupElement[82].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[83].x=-y+0.5;        SpaceGroupElement[83].y=-z+0.5;        SpaceGroupElement[83].z=x;              // 3^1      [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[84].x=-x+0.5;        SpaceGroupElement[84].y=-y+0.5;        SpaceGroupElement[84].z=-z;             // -1       -               -               1/4,1/4,0
      SpaceGroupElement[85].x=x+0.5;         SpaceGroupElement[85].y=y+0.5;         SpaceGroupElement[85].z=-z;             // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[86].x=-x+0.5;        SpaceGroupElement[86].y=y+0.5;         SpaceGroupElement[86].z=z;              // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[87].x=x+0.5;         SpaceGroupElement[87].y=-y+0.5;        SpaceGroupElement[87].z=z;              // -2       [0,1,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[88].x=-z+0.5;        SpaceGroupElement[88].y=-x+0.5;        SpaceGroupElement[88].z=-y;             // -3^1     [1,1,1]         0,0,0           1/2,0,0
      SpaceGroupElement[89].x=z+0.5;         SpaceGroupElement[89].y=x+0.5;         SpaceGroupElement[89].z=-y;             // -3^-1    [-1,1,1]        0,0,0           0,1/2,-1/2
      SpaceGroupElement[90].x=-z+0.5;        SpaceGroupElement[90].y=x+0.5;         SpaceGroupElement[90].z=y;              // -3^-1    [1,-1,1]        0,0,0           0,1/2,1/2
      SpaceGroupElement[91].x=z+0.5;         SpaceGroupElement[91].y=-x+0.5;        SpaceGroupElement[91].z=y;              // -3^1     [-1,-1,1]       0,0,0           1/2,0,0
      SpaceGroupElement[92].x=-y+0.5;        SpaceGroupElement[92].y=-z+0.5;        SpaceGroupElement[92].z=-x;             // -3^-1    [1,1,1]         0,0,0           0,1/2,0
      SpaceGroupElement[93].x=-y+0.5;        SpaceGroupElement[93].y=z+0.5;         SpaceGroupElement[93].z=x;              // -3^-1    [-1,-1,1]       0,0,0           0,1/2,0
      SpaceGroupElement[94].x=y+0.5;         SpaceGroupElement[94].y=-z+0.5;        SpaceGroupElement[94].z=x;              // -3^1     [-1,1,1]        0,0,0           1/2,0,1/2
      SpaceGroupElement[95].x=y+0.5;         SpaceGroupElement[95].y=z+0.5;         SpaceGroupElement[95].z=-x;             // -3^1     [1,-1,1]        0,0,0           1/2,0,-1/2
      break;

    case 498: // [203] F d -3 Origin choice 1 (F 2 2 3 -1d)
      SpaceGroupSize=96;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=z;              SpaceGroupElement[4].y=x;              SpaceGroupElement[4].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-z;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=y;               // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[6].x=z;              SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-y;              // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[7].x=-z;             SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-y;              // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-z;             SpaceGroupElement[9].z=-x;              // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[12].x=-x+0.25;       SpaceGroupElement[12].y=-y+0.25;       SpaceGroupElement[12].z=-z+0.25;        // -1       -               -               1/8,1/8,1/8
      SpaceGroupElement[13].x=x+0.25;        SpaceGroupElement[13].y=y+0.25;        SpaceGroupElement[13].z=-z+0.25;        // -2       [0,0,1]         1/4,1/4,0       0,0,1/8
      SpaceGroupElement[14].x=-x+0.25;       SpaceGroupElement[14].y=y+0.25;        SpaceGroupElement[14].z=z+0.25;         // -2       [1,0,0]         0,1/4,1/4       1/8,0,0
      SpaceGroupElement[15].x=x+0.25;        SpaceGroupElement[15].y=-y+0.25;       SpaceGroupElement[15].z=z+0.25;         // -2       [0,1,0]         1/4,0,1/4       0,1/8,0
      SpaceGroupElement[16].x=-z+0.25;       SpaceGroupElement[16].y=-x+0.25;       SpaceGroupElement[16].z=-y+0.25;        // -3^1     [1,1,1]         0,0,0           1/8,1/8,1/8
      SpaceGroupElement[17].x=z+0.25;        SpaceGroupElement[17].y=x+0.25;        SpaceGroupElement[17].z=-y+0.25;        // -3^-1    [-1,1,1]        0,0,0           1/8,3/8,-1/8
      SpaceGroupElement[18].x=-z+0.25;       SpaceGroupElement[18].y=x+0.25;        SpaceGroupElement[18].z=y+0.25;         // -3^-1    [1,-1,1]        0,0,0           -1/8,1/8,3/8
      SpaceGroupElement[19].x=z+0.25;        SpaceGroupElement[19].y=-x+0.25;       SpaceGroupElement[19].z=y+0.25;         // -3^1     [-1,-1,1]       0,0,0           3/8,-1/8,1/8
      SpaceGroupElement[20].x=-y+0.25;       SpaceGroupElement[20].y=-z+0.25;       SpaceGroupElement[20].z=-x+0.25;        // -3^-1    [1,1,1]         0,0,0           1/8,1/8,1/8
      SpaceGroupElement[21].x=-y+0.25;       SpaceGroupElement[21].y=z+0.25;        SpaceGroupElement[21].z=x+0.25;         // -3^-1    [-1,-1,1]       0,0,0           -1/8,3/8,1/8
      SpaceGroupElement[22].x=y+0.25;        SpaceGroupElement[22].y=-z+0.25;       SpaceGroupElement[22].z=x+0.25;         // -3^1     [-1,1,1]        0,0,0           1/8,-1/8,3/8
      SpaceGroupElement[23].x=y+0.25;        SpaceGroupElement[23].y=z+0.25;        SpaceGroupElement[23].z=-x+0.25;        // -3^1     [1,-1,1]        0,0,0           3/8,1/8,-1/8
      SpaceGroupElement[24].x=x;             SpaceGroupElement[24].y=y+0.5;         SpaceGroupElement[24].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[25].x=-x;            SpaceGroupElement[25].y=-y+0.5;        SpaceGroupElement[25].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=-y+0.5;        SpaceGroupElement[26].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[27].x=-x;            SpaceGroupElement[27].y=y+0.5;         SpaceGroupElement[27].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[28].x=z;             SpaceGroupElement[28].y=x+0.5;         SpaceGroupElement[28].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     -1/3,-1/6,0
      SpaceGroupElement[29].x=-z;            SpaceGroupElement[29].y=-x+0.5;        SpaceGroupElement[29].z=y+0.5;          // 3^-1     [-1,1,1]        -1/3,1/3,1/3    1/3,-1/6,0
      SpaceGroupElement[30].x=z;             SpaceGroupElement[30].y=-x+0.5;        SpaceGroupElement[30].z=-y+0.5;         // 3^-1     [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[31].x=-z;            SpaceGroupElement[31].y=x+0.5;         SpaceGroupElement[31].z=-y+0.5;         // 3^1      [-1,-1,1]       0,0,0           0,1/2,0
      SpaceGroupElement[32].x=y;             SpaceGroupElement[32].y=z+0.5;         SpaceGroupElement[32].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,1/6,0
      SpaceGroupElement[33].x=y;             SpaceGroupElement[33].y=-z+0.5;        SpaceGroupElement[33].z=-x+0.5;         // 3^-1     [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[34].x=-y;            SpaceGroupElement[34].y=z+0.5;         SpaceGroupElement[34].z=-x+0.5;         // 3^1      [-1,1,1]        -1/3,1/3,1/3    1/6,1/6,0
      SpaceGroupElement[35].x=-y;            SpaceGroupElement[35].y=-z+0.5;        SpaceGroupElement[35].z=x+0.5;          // 3^1      [1,-1,1]        0,0,0           -1/2,1/2,0
      SpaceGroupElement[36].x=-x+0.25;       SpaceGroupElement[36].y=-y+0.75;       SpaceGroupElement[36].z=-z+0.75;        // -1       -               -               1/8,3/8,3/8
      SpaceGroupElement[37].x=x+0.25;        SpaceGroupElement[37].y=y+0.75;        SpaceGroupElement[37].z=-z+0.75;        // -2       [0,0,1]         1/4,3/4,0       0,0,3/8
      SpaceGroupElement[38].x=-x+0.25;       SpaceGroupElement[38].y=y+0.75;        SpaceGroupElement[38].z=z+0.75;         // -2       [1,0,0]         0,3/4,3/4       1/8,0,0
      SpaceGroupElement[39].x=x+0.25;        SpaceGroupElement[39].y=-y+0.75;       SpaceGroupElement[39].z=z+0.75;         // -2       [0,1,0]         1/4,0,3/4       0,3/8,0
      SpaceGroupElement[40].x=-z+0.25;       SpaceGroupElement[40].y=-x+0.75;       SpaceGroupElement[40].z=-y+0.75;        // -3^1     [1,1,1]         0,0,0           1/8,5/8,1/8
      SpaceGroupElement[41].x=z+0.25;        SpaceGroupElement[41].y=x+0.75;        SpaceGroupElement[41].z=-y+0.75;        // -3^-1    [-1,1,1]        0,0,0           1/8,7/8,-1/8
      SpaceGroupElement[42].x=-z+0.25;       SpaceGroupElement[42].y=x+0.75;        SpaceGroupElement[42].z=y+0.75;         // -3^-1    [1,-1,1]        0,0,0           -5/8,1/8,7/8
      SpaceGroupElement[43].x=z+0.25;        SpaceGroupElement[43].y=-x+0.75;       SpaceGroupElement[43].z=y+0.75;         // -3^1     [-1,-1,1]       0,0,0           7/8,-1/8,5/8
      SpaceGroupElement[44].x=-y+0.25;       SpaceGroupElement[44].y=-z+0.75;       SpaceGroupElement[44].z=-x+0.75;        // -3^-1    [1,1,1]         0,0,0           1/8,1/8,5/8
      SpaceGroupElement[45].x=-y+0.25;       SpaceGroupElement[45].y=z+0.75;        SpaceGroupElement[45].z=x+0.75;         // -3^-1    [-1,-1,1]       0,0,0           -5/8,7/8,1/8
      SpaceGroupElement[46].x=y+0.25;        SpaceGroupElement[46].y=-z+0.75;       SpaceGroupElement[46].z=x+0.75;         // -3^1     [-1,1,1]        0,0,0           1/8,-1/8,7/8
      SpaceGroupElement[47].x=y+0.25;        SpaceGroupElement[47].y=z+0.75;        SpaceGroupElement[47].z=-x+0.75;        // -3^1     [1,-1,1]        0,0,0           7/8,5/8,-1/8
      SpaceGroupElement[48].x=x+0.5;         SpaceGroupElement[48].y=y;             SpaceGroupElement[48].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[49].x=-x+0.5;        SpaceGroupElement[49].y=-y;            SpaceGroupElement[49].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[50].x=x+0.5;         SpaceGroupElement[50].y=-y;            SpaceGroupElement[50].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[51].x=-x+0.5;        SpaceGroupElement[51].y=y;             SpaceGroupElement[51].z=-z+0.5;         // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[52].x=z+0.5;         SpaceGroupElement[52].y=x;             SpaceGroupElement[52].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,-1/6,0
      SpaceGroupElement[53].x=-z+0.5;        SpaceGroupElement[53].y=-x;            SpaceGroupElement[53].z=y+0.5;          // 3^-1     [-1,1,1]        0,0,0           1/2,-1/2,0
      SpaceGroupElement[54].x=z+0.5;         SpaceGroupElement[54].y=-x;            SpaceGroupElement[54].z=-y+0.5;         // 3^-1     [1,-1,1]        1/3,-1/3,1/3    1/6,1/6,0
      SpaceGroupElement[55].x=-z+0.5;        SpaceGroupElement[55].y=x;             SpaceGroupElement[55].z=-y+0.5;         // 3^1      [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[56].x=y+0.5;         SpaceGroupElement[56].y=z;             SpaceGroupElement[56].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,-1/3,0
      SpaceGroupElement[57].x=y+0.5;         SpaceGroupElement[57].y=-z;            SpaceGroupElement[57].z=-x+0.5;         // 3^-1     [-1,-1,1]       0,0,0           1/2,0,0
      SpaceGroupElement[58].x=-y+0.5;        SpaceGroupElement[58].y=z;             SpaceGroupElement[58].z=-x+0.5;         // 3^1      [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[59].x=-y+0.5;        SpaceGroupElement[59].y=-z;            SpaceGroupElement[59].z=x+0.5;          // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      SpaceGroupElement[60].x=-x+0.75;       SpaceGroupElement[60].y=-y+0.25;       SpaceGroupElement[60].z=-z+0.75;        // -1       -               -               3/8,1/8,3/8
      SpaceGroupElement[61].x=x+0.75;        SpaceGroupElement[61].y=y+0.25;        SpaceGroupElement[61].z=-z+0.75;        // -2       [0,0,1]         3/4,1/4,0       0,0,3/8
      SpaceGroupElement[62].x=-x+0.75;       SpaceGroupElement[62].y=y+0.25;        SpaceGroupElement[62].z=z+0.75;         // -2       [1,0,0]         0,1/4,3/4       3/8,0,0
      SpaceGroupElement[63].x=x+0.75;        SpaceGroupElement[63].y=-y+0.25;       SpaceGroupElement[63].z=z+0.75;         // -2       [0,1,0]         3/4,0,3/4       0,1/8,0
      SpaceGroupElement[64].x=-z+0.75;       SpaceGroupElement[64].y=-x+0.25;       SpaceGroupElement[64].z=-y+0.75;        // -3^1     [1,1,1]         0,0,0           1/8,1/8,5/8
      SpaceGroupElement[65].x=z+0.75;        SpaceGroupElement[65].y=x+0.25;        SpaceGroupElement[65].z=-y+0.75;        // -3^-1    [-1,1,1]        0,0,0           5/8,7/8,-1/8
      SpaceGroupElement[66].x=-z+0.75;       SpaceGroupElement[66].y=x+0.25;        SpaceGroupElement[66].z=y+0.75;         // -3^-1    [1,-1,1]        0,0,0           -1/8,1/8,7/8
      SpaceGroupElement[67].x=z+0.75;        SpaceGroupElement[67].y=-x+0.25;       SpaceGroupElement[67].z=y+0.75;         // -3^1     [-1,-1,1]       0,0,0           7/8,-5/8,1/8
      SpaceGroupElement[68].x=-y+0.75;       SpaceGroupElement[68].y=-z+0.25;       SpaceGroupElement[68].z=-x+0.75;        // -3^-1    [1,1,1]         0,0,0           5/8,1/8,1/8
      SpaceGroupElement[69].x=-y+0.75;       SpaceGroupElement[69].y=z+0.25;        SpaceGroupElement[69].z=x+0.75;         // -3^-1    [-1,-1,1]       0,0,0           -1/8,7/8,5/8
      SpaceGroupElement[70].x=y+0.75;        SpaceGroupElement[70].y=-z+0.25;       SpaceGroupElement[70].z=x+0.75;         // -3^1     [-1,1,1]        0,0,0           1/8,-5/8,7/8
      SpaceGroupElement[71].x=y+0.75;        SpaceGroupElement[71].y=z+0.25;        SpaceGroupElement[71].z=-x+0.75;        // -3^1     [1,-1,1]        0,0,0           7/8,1/8,-1/8
      SpaceGroupElement[72].x=x+0.5;         SpaceGroupElement[72].y=y+0.5;         SpaceGroupElement[72].z=z;              // 1        -               -               -
      SpaceGroupElement[73].x=-x+0.5;        SpaceGroupElement[73].y=-y+0.5;        SpaceGroupElement[73].z=z;              // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[74].x=x+0.5;         SpaceGroupElement[74].y=-y+0.5;        SpaceGroupElement[74].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[75].x=-x+0.5;        SpaceGroupElement[75].y=y+0.5;         SpaceGroupElement[75].z=-z;             // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[76].x=z+0.5;         SpaceGroupElement[76].y=x+0.5;         SpaceGroupElement[76].z=y;              // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,1/3,0
      SpaceGroupElement[77].x=-z+0.5;        SpaceGroupElement[77].y=-x+0.5;        SpaceGroupElement[77].z=y;              // 3^-1     [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[78].x=z+0.5;         SpaceGroupElement[78].y=-x+0.5;        SpaceGroupElement[78].z=-y;             // 3^-1     [1,-1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[79].x=-z+0.5;        SpaceGroupElement[79].y=x+0.5;         SpaceGroupElement[79].z=-y;             // 3^1      [-1,-1,1]       1/3,1/3,-1/3    1/6,1/3,0
      SpaceGroupElement[80].x=y+0.5;         SpaceGroupElement[80].y=z+0.5;         SpaceGroupElement[80].z=x;              // 3^-1     [1,1,1]         1/3,1/3,1/3     1/3,1/6,0
      SpaceGroupElement[81].x=y+0.5;         SpaceGroupElement[81].y=-z+0.5;        SpaceGroupElement[81].z=-x;             // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    1/3,1/6,0
      SpaceGroupElement[82].x=-y+0.5;        SpaceGroupElement[82].y=z+0.5;         SpaceGroupElement[82].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[83].x=-y+0.5;        SpaceGroupElement[83].y=-z+0.5;        SpaceGroupElement[83].z=x;              // 3^1      [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[84].x=-x+0.75;       SpaceGroupElement[84].y=-y+0.75;       SpaceGroupElement[84].z=-z+0.25;        // -1       -               -               3/8,3/8,1/8
      SpaceGroupElement[85].x=x+0.75;        SpaceGroupElement[85].y=y+0.75;        SpaceGroupElement[85].z=-z+0.25;        // -2       [0,0,1]         3/4,3/4,0       0,0,1/8
      SpaceGroupElement[86].x=-x+0.75;       SpaceGroupElement[86].y=y+0.75;        SpaceGroupElement[86].z=z+0.25;         // -2       [1,0,0]         0,3/4,1/4       3/8,0,0
      SpaceGroupElement[87].x=x+0.75;        SpaceGroupElement[87].y=-y+0.75;       SpaceGroupElement[87].z=z+0.25;         // -2       [0,1,0]         3/4,0,1/4       0,3/8,0
      SpaceGroupElement[88].x=-z+0.75;       SpaceGroupElement[88].y=-x+0.75;       SpaceGroupElement[88].z=-y+0.25;        // -3^1     [1,1,1]         0,0,0           5/8,1/8,1/8
      SpaceGroupElement[89].x=z+0.75;        SpaceGroupElement[89].y=x+0.75;        SpaceGroupElement[89].z=-y+0.25;        // -3^-1    [-1,1,1]        0,0,0           1/8,7/8,-5/8
      SpaceGroupElement[90].x=-z+0.75;       SpaceGroupElement[90].y=x+0.75;        SpaceGroupElement[90].z=y+0.25;         // -3^-1    [1,-1,1]        0,0,0           -1/8,5/8,7/8
      SpaceGroupElement[91].x=z+0.75;        SpaceGroupElement[91].y=-x+0.75;       SpaceGroupElement[91].z=y+0.25;         // -3^1     [-1,-1,1]       0,0,0           7/8,-1/8,1/8
      SpaceGroupElement[92].x=-y+0.75;       SpaceGroupElement[92].y=-z+0.75;       SpaceGroupElement[92].z=-x+0.25;        // -3^-1    [1,1,1]         0,0,0           1/8,5/8,1/8
      SpaceGroupElement[93].x=-y+0.75;       SpaceGroupElement[93].y=z+0.75;        SpaceGroupElement[93].z=x+0.25;         // -3^-1    [-1,-1,1]       0,0,0           -1/8,7/8,1/8
      SpaceGroupElement[94].x=y+0.75;        SpaceGroupElement[94].y=-z+0.75;       SpaceGroupElement[94].z=x+0.25;         // -3^1     [-1,1,1]        0,0,0           5/8,-1/8,7/8
      SpaceGroupElement[95].x=y+0.75;        SpaceGroupElement[95].y=z+0.75;        SpaceGroupElement[95].z=-x+0.25;        // -3^1     [1,-1,1]        0,0,0           7/8,1/8,-5/8
      break;
    case 499: // [203] F d -3 Origin choice 2 (-F 2uv 2vw 3)
      SpaceGroupSize=96;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.25;        SpaceGroupElement[1].y=-y+0.25;        SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           1/8,1/8,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y+0.25;        SpaceGroupElement[2].z=-z+0.25;         // 2        [1,0,0]         0,0,0           0,1/8,1/8
      SpaceGroupElement[3].x=-x+0.25;        SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z+0.25;         // 2        [0,1,0]         0,0,0           1/8,0,1/8
      SpaceGroupElement[4].x=z;              SpaceGroupElement[4].y=x;              SpaceGroupElement[4].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-z+0.25;        SpaceGroupElement[5].y=-x+0.25;        SpaceGroupElement[5].z=y;               // 3^-1     [-1,1,1]        0,0,0           1/4,0,0
      SpaceGroupElement[6].x=z;              SpaceGroupElement[6].y=-x+0.25;        SpaceGroupElement[6].z=-y+0.25;         // 3^-1     [1,-1,1]        0,0,0           0,1/4,0
      SpaceGroupElement[7].x=-z+0.25;        SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-y+0.25;         // 3^1      [-1,-1,1]       0,0,0           1/4,1/4,0
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-z+0.25;        SpaceGroupElement[9].z=-x+0.25;         // 3^-1     [-1,-1,1]       0,0,0           1/4,1/4,0
      SpaceGroupElement[10].x=-y+0.25;       SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x+0.25;        // 3^1      [-1,1,1]        0,0,0           1/4,0,0
      SpaceGroupElement[11].x=-y+0.25;       SpaceGroupElement[11].y=-z+0.25;       SpaceGroupElement[11].z=x;              // 3^1      [1,-1,1]        0,0,0           0,1/4,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[13].x=x-0.25;        SpaceGroupElement[13].y=y-0.25;        SpaceGroupElement[13].z=-z;             // -2       [0,0,1]         -1/4,-1/4,0     0,0,0
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y-0.25;        SpaceGroupElement[14].z=z-0.25;         // -2       [1,0,0]         0,-1/4,-1/4     0,0,0
      SpaceGroupElement[15].x=x-0.25;        SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z-0.25;         // -2       [0,1,0]         -1/4,0,-1/4     0,0,0
      SpaceGroupElement[16].x=-z;            SpaceGroupElement[16].y=-x;            SpaceGroupElement[16].z=-y;             // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=z-0.25;        SpaceGroupElement[17].y=x-0.25;        SpaceGroupElement[17].z=-y;             // -3^-1    [-1,1,1]        0,0,0           0,-1/4,1/4
      SpaceGroupElement[18].x=-z;            SpaceGroupElement[18].y=x-0.25;        SpaceGroupElement[18].z=y-0.25;         // -3^-1    [1,-1,1]        0,0,0           1/4,0,-1/4
      SpaceGroupElement[19].x=z-0.25;        SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=y-0.25;         // -3^1     [-1,-1,1]       0,0,0           -1/4,1/4,0
      SpaceGroupElement[20].x=-y;            SpaceGroupElement[20].y=-z;            SpaceGroupElement[20].z=-x;             // -3^-1    [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=z-0.25;        SpaceGroupElement[21].z=x-0.25;         // -3^-1    [-1,-1,1]       0,0,0           1/4,-1/4,0
      SpaceGroupElement[22].x=y-0.25;        SpaceGroupElement[22].y=-z;            SpaceGroupElement[22].z=x-0.25;         // -3^1     [-1,1,1]        0,0,0           0,1/4,-1/4
      SpaceGroupElement[23].x=y-0.25;        SpaceGroupElement[23].y=z-0.25;        SpaceGroupElement[23].z=-x;             // -3^1     [1,-1,1]        0,0,0           -1/4,0,1/4
      SpaceGroupElement[24].x=x;             SpaceGroupElement[24].y=y+0.5;         SpaceGroupElement[24].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[25].x=-x+0.25;       SpaceGroupElement[25].y=-y+0.75;       SpaceGroupElement[25].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/8,3/8,0
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=-y+0.75;       SpaceGroupElement[26].z=-z+0.75;        // 2        [1,0,0]         0,0,0           0,3/8,3/8
      SpaceGroupElement[27].x=-x+0.25;       SpaceGroupElement[27].y=y+0.5;         SpaceGroupElement[27].z=-z+0.75;        // 2        [0,1,0]         0,1/2,0         1/8,0,3/8
      SpaceGroupElement[28].x=z;             SpaceGroupElement[28].y=x+0.5;         SpaceGroupElement[28].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     -1/3,-1/6,0
      SpaceGroupElement[29].x=-z+0.25;       SpaceGroupElement[29].y=-x+0.75;       SpaceGroupElement[29].z=y+0.5;          // 3^-1     [-1,1,1]        -1/3,1/3,1/3    7/12,-1/6,0
      SpaceGroupElement[30].x=z;             SpaceGroupElement[30].y=-x+0.75;       SpaceGroupElement[30].z=-y+0.75;        // 3^-1     [1,-1,1]        0,0,0           0,3/4,0
      SpaceGroupElement[31].x=-z+0.25;       SpaceGroupElement[31].y=x+0.5;         SpaceGroupElement[31].z=-y+0.75;        // 3^1      [-1,-1,1]       0,0,0           1/4,3/4,0
      SpaceGroupElement[32].x=y;             SpaceGroupElement[32].y=z+0.5;         SpaceGroupElement[32].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,1/6,0
      SpaceGroupElement[33].x=y;             SpaceGroupElement[33].y=-z+0.75;       SpaceGroupElement[33].z=-x+0.75;        // 3^-1     [-1,-1,1]       0,0,0           3/4,3/4,0
      SpaceGroupElement[34].x=-y+0.25;       SpaceGroupElement[34].y=z+0.5;         SpaceGroupElement[34].z=-x+0.75;        // 3^1      [-1,1,1]        -1/3,1/3,1/3    5/12,1/6,0
      SpaceGroupElement[35].x=-y+0.25;       SpaceGroupElement[35].y=-z+0.75;       SpaceGroupElement[35].z=x+0.5;          // 3^1      [1,-1,1]        0,0,0           -1/2,3/4,0
      SpaceGroupElement[36].x=-x;            SpaceGroupElement[36].y=-y+0.5;        SpaceGroupElement[36].z=-z+0.5;         // -1       -               -               0,1/4,1/4
      SpaceGroupElement[37].x=x-0.25;        SpaceGroupElement[37].y=y+0.25;        SpaceGroupElement[37].z=-z+0.5;         // -2       [0,0,1]         -1/4,1/4,0      0,0,1/4
      SpaceGroupElement[38].x=-x;            SpaceGroupElement[38].y=y+0.25;        SpaceGroupElement[38].z=z+0.25;         // -2       [1,0,0]         0,1/4,1/4       0,0,0
      SpaceGroupElement[39].x=x-0.25;        SpaceGroupElement[39].y=-y+0.5;        SpaceGroupElement[39].z=z+0.25;         // -2       [0,1,0]         -1/4,0,1/4      0,1/4,0
      SpaceGroupElement[40].x=-z;            SpaceGroupElement[40].y=-x+0.5;        SpaceGroupElement[40].z=-y+0.5;         // -3^1     [1,1,1]         0,0,0           0,1/2,0
      SpaceGroupElement[41].x=z-0.25;        SpaceGroupElement[41].y=x+0.25;        SpaceGroupElement[41].z=-y+0.5;         // -3^-1    [-1,1,1]        0,0,0           0,1/4,1/4
      SpaceGroupElement[42].x=-z;            SpaceGroupElement[42].y=x+0.25;        SpaceGroupElement[42].z=y+0.25;         // -3^-1    [1,-1,1]        0,0,0           -1/4,0,1/4
      SpaceGroupElement[43].x=z-0.25;        SpaceGroupElement[43].y=-x+0.5;        SpaceGroupElement[43].z=y+0.25;         // -3^1     [-1,-1,1]       0,0,0           1/4,1/4,1/2
      SpaceGroupElement[44].x=-y;            SpaceGroupElement[44].y=-z+0.5;        SpaceGroupElement[44].z=-x+0.5;         // -3^-1    [1,1,1]         0,0,0           0,0,1/2
      SpaceGroupElement[45].x=-y;            SpaceGroupElement[45].y=z+0.25;        SpaceGroupElement[45].z=x+0.25;         // -3^-1    [-1,-1,1]       0,0,0           -1/4,1/4,0
      SpaceGroupElement[46].x=y-0.25;        SpaceGroupElement[46].y=-z+0.5;        SpaceGroupElement[46].z=x+0.25;         // -3^1     [-1,1,1]        0,0,0           0,1/4,1/4
      SpaceGroupElement[47].x=y-0.25;        SpaceGroupElement[47].y=z+0.25;        SpaceGroupElement[47].z=-x+0.5;         // -3^1     [1,-1,1]        0,0,0           1/4,1/2,1/4
      SpaceGroupElement[48].x=x+0.5;         SpaceGroupElement[48].y=y;             SpaceGroupElement[48].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[49].x=-x+0.75;       SpaceGroupElement[49].y=-y+0.25;       SpaceGroupElement[49].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         3/8,1/8,0
      SpaceGroupElement[50].x=x+0.5;         SpaceGroupElement[50].y=-y+0.25;       SpaceGroupElement[50].z=-z+0.75;        // 2        [1,0,0]         1/2,0,0         0,1/8,3/8
      SpaceGroupElement[51].x=-x+0.75;       SpaceGroupElement[51].y=y;             SpaceGroupElement[51].z=-z+0.75;        // 2        [0,1,0]         0,0,0           3/8,0,3/8
      SpaceGroupElement[52].x=z+0.5;         SpaceGroupElement[52].y=x;             SpaceGroupElement[52].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,-1/6,0
      SpaceGroupElement[53].x=-z+0.75;       SpaceGroupElement[53].y=-x+0.25;       SpaceGroupElement[53].z=y+0.5;          // 3^-1     [-1,1,1]        0,0,0           3/4,-1/2,0
      SpaceGroupElement[54].x=z+0.5;         SpaceGroupElement[54].y=-x+0.25;       SpaceGroupElement[54].z=-y+0.75;        // 3^-1     [1,-1,1]        1/3,-1/3,1/3    1/6,5/12,0
      SpaceGroupElement[55].x=-z+0.75;       SpaceGroupElement[55].y=x;             SpaceGroupElement[55].z=-y+0.75;        // 3^1      [-1,-1,1]       0,0,0           3/4,3/4,0
      SpaceGroupElement[56].x=y+0.5;         SpaceGroupElement[56].y=z;             SpaceGroupElement[56].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,-1/3,0
      SpaceGroupElement[57].x=y+0.5;         SpaceGroupElement[57].y=-z+0.25;       SpaceGroupElement[57].z=-x+0.75;        // 3^-1     [-1,-1,1]       0,0,0           3/4,1/4,0
      SpaceGroupElement[58].x=-y+0.75;       SpaceGroupElement[58].y=z;             SpaceGroupElement[58].z=-x+0.75;        // 3^1      [-1,1,1]        0,0,0           3/4,0,0
      SpaceGroupElement[59].x=-y+0.75;       SpaceGroupElement[59].y=-z+0.25;       SpaceGroupElement[59].z=x+0.5;          // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,7/12,0
      SpaceGroupElement[60].x=-x+0.5;        SpaceGroupElement[60].y=-y;            SpaceGroupElement[60].z=-z+0.5;         // -1       -               -               1/4,0,1/4
      SpaceGroupElement[61].x=x+0.25;        SpaceGroupElement[61].y=y-0.25;        SpaceGroupElement[61].z=-z+0.5;         // -2       [0,0,1]         1/4,-1/4,0      0,0,1/4
      SpaceGroupElement[62].x=-x+0.5;        SpaceGroupElement[62].y=y-0.25;        SpaceGroupElement[62].z=z+0.25;         // -2       [1,0,0]         0,-1/4,1/4      1/4,0,0
      SpaceGroupElement[63].x=x+0.25;        SpaceGroupElement[63].y=-y;            SpaceGroupElement[63].z=z+0.25;         // -2       [0,1,0]         1/4,0,1/4       0,0,0
      SpaceGroupElement[64].x=-z+0.5;        SpaceGroupElement[64].y=-x;            SpaceGroupElement[64].z=-y+0.5;         // -3^1     [1,1,1]         0,0,0           0,0,1/2
      SpaceGroupElement[65].x=z+0.25;        SpaceGroupElement[65].y=x-0.25;        SpaceGroupElement[65].z=-y+0.5;         // -3^-1    [-1,1,1]        0,0,0           1/2,1/4,1/4
      SpaceGroupElement[66].x=-z+0.5;        SpaceGroupElement[66].y=x-0.25;        SpaceGroupElement[66].z=y+0.25;         // -3^-1    [1,-1,1]        0,0,0           1/4,0,1/4
      SpaceGroupElement[67].x=z+0.25;        SpaceGroupElement[67].y=-x;            SpaceGroupElement[67].z=y+0.25;         // -3^1     [-1,-1,1]       0,0,0           1/4,-1/4,0
      SpaceGroupElement[68].x=-y+0.5;        SpaceGroupElement[68].y=-z;            SpaceGroupElement[68].z=-x+0.5;         // -3^-1    [1,1,1]         0,0,0           1/2,0,0
      SpaceGroupElement[69].x=-y+0.5;        SpaceGroupElement[69].y=z-0.25;        SpaceGroupElement[69].z=x+0.25;         // -3^-1    [-1,-1,1]       0,0,0           1/4,1/4,1/2
      SpaceGroupElement[70].x=y+0.25;        SpaceGroupElement[70].y=-z;            SpaceGroupElement[70].z=x+0.25;         // -3^1     [-1,1,1]        0,0,0           0,-1/4,1/4
      SpaceGroupElement[71].x=y+0.25;        SpaceGroupElement[71].y=z-0.25;        SpaceGroupElement[71].z=-x+0.5;         // -3^1     [1,-1,1]        0,0,0           1/4,0,1/4
      SpaceGroupElement[72].x=x+0.5;         SpaceGroupElement[72].y=y+0.5;         SpaceGroupElement[72].z=z;              // 1        -               -               -
      SpaceGroupElement[73].x=-x+0.75;       SpaceGroupElement[73].y=-y+0.75;       SpaceGroupElement[73].z=z;              // 2        [0,0,1]         0,0,0           3/8,3/8,0
      SpaceGroupElement[74].x=x+0.5;         SpaceGroupElement[74].y=-y+0.75;       SpaceGroupElement[74].z=-z+0.25;        // 2        [1,0,0]         1/2,0,0         0,3/8,1/8
      SpaceGroupElement[75].x=-x+0.75;       SpaceGroupElement[75].y=y+0.5;         SpaceGroupElement[75].z=-z+0.25;        // 2        [0,1,0]         0,1/2,0         3/8,0,1/8
      SpaceGroupElement[76].x=z+0.5;         SpaceGroupElement[76].y=x+0.5;         SpaceGroupElement[76].z=y;              // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,1/3,0
      SpaceGroupElement[77].x=-z+0.75;       SpaceGroupElement[77].y=-x+0.75;       SpaceGroupElement[77].z=y;              // 3^-1     [-1,1,1]        0,0,0           3/4,0,0
      SpaceGroupElement[78].x=z+0.5;         SpaceGroupElement[78].y=-x+0.75;       SpaceGroupElement[78].z=-y+0.25;        // 3^-1     [1,-1,1]        0,0,0           1/2,1/4,0
      SpaceGroupElement[79].x=-z+0.75;       SpaceGroupElement[79].y=x+0.5;         SpaceGroupElement[79].z=-y+0.25;        // 3^1      [-1,-1,1]       1/3,1/3,-1/3    5/12,7/12,0
      SpaceGroupElement[80].x=y+0.5;         SpaceGroupElement[80].y=z+0.5;         SpaceGroupElement[80].z=x;              // 3^-1     [1,1,1]         1/3,1/3,1/3     1/3,1/6,0
      SpaceGroupElement[81].x=y+0.5;         SpaceGroupElement[81].y=-z+0.75;       SpaceGroupElement[81].z=-x+0.25;        // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    7/12,5/12,0
      SpaceGroupElement[82].x=-y+0.75;       SpaceGroupElement[82].y=z+0.5;         SpaceGroupElement[82].z=-x+0.25;        // 3^1      [-1,1,1]        0,0,0           1/4,1/2,0
      SpaceGroupElement[83].x=-y+0.75;       SpaceGroupElement[83].y=-z+0.75;       SpaceGroupElement[83].z=x;              // 3^1      [1,-1,1]        0,0,0           0,3/4,0
      SpaceGroupElement[84].x=-x+0.5;        SpaceGroupElement[84].y=-y+0.5;        SpaceGroupElement[84].z=-z;             // -1       -               -               1/4,1/4,0
      SpaceGroupElement[85].x=x+0.25;        SpaceGroupElement[85].y=y+0.25;        SpaceGroupElement[85].z=-z;             // -2       [0,0,1]         1/4,1/4,0       0,0,0
      SpaceGroupElement[86].x=-x+0.5;        SpaceGroupElement[86].y=y+0.25;        SpaceGroupElement[86].z=z-0.25;         // -2       [1,0,0]         0,1/4,-1/4      1/4,0,0
      SpaceGroupElement[87].x=x+0.25;        SpaceGroupElement[87].y=-y+0.5;        SpaceGroupElement[87].z=z-0.25;         // -2       [0,1,0]         1/4,0,-1/4      0,1/4,0
      SpaceGroupElement[88].x=-z+0.5;        SpaceGroupElement[88].y=-x+0.5;        SpaceGroupElement[88].z=-y;             // -3^1     [1,1,1]         0,0,0           1/2,0,0
      SpaceGroupElement[89].x=z+0.25;        SpaceGroupElement[89].y=x+0.25;        SpaceGroupElement[89].z=-y;             // -3^-1    [-1,1,1]        0,0,0           0,1/4,-1/4
      SpaceGroupElement[90].x=-z+0.5;        SpaceGroupElement[90].y=x+0.25;        SpaceGroupElement[90].z=y-0.25;         // -3^-1    [1,-1,1]        0,0,0           1/4,1/2,1/4
      SpaceGroupElement[91].x=z+0.25;        SpaceGroupElement[91].y=-x+0.5;        SpaceGroupElement[91].z=y-0.25;         // -3^1     [-1,-1,1]       0,0,0           1/4,1/4,0
      SpaceGroupElement[92].x=-y+0.5;        SpaceGroupElement[92].y=-z+0.5;        SpaceGroupElement[92].z=-x;             // -3^-1    [1,1,1]         0,0,0           0,1/2,0
      SpaceGroupElement[93].x=-y+0.5;        SpaceGroupElement[93].y=z+0.25;        SpaceGroupElement[93].z=x-0.25;         // -3^-1    [-1,-1,1]       0,0,0           1/4,1/4,0
      SpaceGroupElement[94].x=y+0.25;        SpaceGroupElement[94].y=-z+0.5;        SpaceGroupElement[94].z=x-0.25;         // -3^1     [-1,1,1]        0,0,0           1/2,1/4,1/4
      SpaceGroupElement[95].x=y+0.25;        SpaceGroupElement[95].y=z+0.25;        SpaceGroupElement[95].z=-x;             // -3^1     [1,-1,1]        0,0,0           1/4,0,-1/4
      break;

    case 500: // [204] I m -3 (-I 2 2 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[4].x=z;              SpaceGroupElement[4].y=x;              SpaceGroupElement[4].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-z;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=y;               // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[6].x=z;              SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-y;              // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[7].x=-z;             SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-y;              // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-z;             SpaceGroupElement[9].z=-x;              // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=-y;            SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z;             // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z;              // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z;              // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[16].x=-z;            SpaceGroupElement[16].y=-x;            SpaceGroupElement[16].z=-y;             // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=z;             SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=-y;             // -3^-1    [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[18].x=-z;            SpaceGroupElement[18].y=x;             SpaceGroupElement[18].z=y;              // -3^-1    [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[19].x=z;             SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=y;              // -3^1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[20].x=-y;            SpaceGroupElement[20].y=-z;            SpaceGroupElement[20].z=-x;             // -3^-1    [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=z;             SpaceGroupElement[21].z=x;              // -3^-1    [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[22].x=y;             SpaceGroupElement[22].y=-z;            SpaceGroupElement[22].z=x;              // -3^1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[23].x=y;             SpaceGroupElement[23].y=z;             SpaceGroupElement[23].z=-x;             // -3^1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[24].x=x+0.5;         SpaceGroupElement[24].y=y+0.5;         SpaceGroupElement[24].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[25].x=-x+0.5;        SpaceGroupElement[25].y=-y+0.5;        SpaceGroupElement[25].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[26].x=x+0.5;         SpaceGroupElement[26].y=-y+0.5;        SpaceGroupElement[26].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[27].x=-x+0.5;        SpaceGroupElement[27].y=y+0.5;         SpaceGroupElement[27].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[28].x=z+0.5;         SpaceGroupElement[28].y=x+0.5;         SpaceGroupElement[28].z=y+0.5;          // 3^1      [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[29].x=-z+0.5;        SpaceGroupElement[29].y=-x+0.5;        SpaceGroupElement[29].z=y+0.5;          // 3^-1     [-1,1,1]        -1/6,1/6,1/6    2/3,-1/3,0
      SpaceGroupElement[30].x=z+0.5;         SpaceGroupElement[30].y=-x+0.5;        SpaceGroupElement[30].z=-y+0.5;         // 3^-1     [1,-1,1]        1/6,-1/6,1/6    1/3,1/3,0
      SpaceGroupElement[31].x=-z+0.5;        SpaceGroupElement[31].y=x+0.5;         SpaceGroupElement[31].z=-y+0.5;         // 3^1      [-1,-1,1]       1/6,1/6,-1/6    1/3,2/3,0
      SpaceGroupElement[32].x=y+0.5;         SpaceGroupElement[32].y=z+0.5;         SpaceGroupElement[32].z=x+0.5;          // 3^-1     [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[33].x=y+0.5;         SpaceGroupElement[33].y=-z+0.5;        SpaceGroupElement[33].z=-x+0.5;         // 3^-1     [-1,-1,1]       1/6,1/6,-1/6    2/3,1/3,0
      SpaceGroupElement[34].x=-y+0.5;        SpaceGroupElement[34].y=z+0.5;         SpaceGroupElement[34].z=-x+0.5;         // 3^1      [-1,1,1]        -1/6,1/6,1/6    1/3,1/3,0
      SpaceGroupElement[35].x=-y+0.5;        SpaceGroupElement[35].y=-z+0.5;        SpaceGroupElement[35].z=x+0.5;          // 3^1      [1,-1,1]        1/6,-1/6,1/6    -1/3,2/3,0
      SpaceGroupElement[36].x=-x+0.5;        SpaceGroupElement[36].y=-y+0.5;        SpaceGroupElement[36].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[37].x=x+0.5;         SpaceGroupElement[37].y=y+0.5;         SpaceGroupElement[37].z=-z+0.5;         // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[38].x=-x+0.5;        SpaceGroupElement[38].y=y+0.5;         SpaceGroupElement[38].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[39].x=x+0.5;         SpaceGroupElement[39].y=-y+0.5;        SpaceGroupElement[39].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[40].x=-z+0.5;        SpaceGroupElement[40].y=-x+0.5;        SpaceGroupElement[40].z=-y+0.5;         // -3^1     [1,1,1]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[41].x=z+0.5;         SpaceGroupElement[41].y=x+0.5;         SpaceGroupElement[41].z=-y+0.5;         // -3^-1    [-1,1,1]        0,0,0           1/4,3/4,-1/4
      SpaceGroupElement[42].x=-z+0.5;        SpaceGroupElement[42].y=x+0.5;         SpaceGroupElement[42].z=y+0.5;          // -3^-1    [1,-1,1]        0,0,0           -1/4,1/4,3/4
      SpaceGroupElement[43].x=z+0.5;         SpaceGroupElement[43].y=-x+0.5;        SpaceGroupElement[43].z=y+0.5;          // -3^1     [-1,-1,1]       0,0,0           3/4,-1/4,1/4
      SpaceGroupElement[44].x=-y+0.5;        SpaceGroupElement[44].y=-z+0.5;        SpaceGroupElement[44].z=-x+0.5;         // -3^-1    [1,1,1]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[45].x=-y+0.5;        SpaceGroupElement[45].y=z+0.5;         SpaceGroupElement[45].z=x+0.5;          // -3^-1    [-1,-1,1]       0,0,0           -1/4,3/4,1/4
      SpaceGroupElement[46].x=y+0.5;         SpaceGroupElement[46].y=-z+0.5;        SpaceGroupElement[46].z=x+0.5;          // -3^1     [-1,1,1]        0,0,0           1/4,-1/4,3/4
      SpaceGroupElement[47].x=y+0.5;         SpaceGroupElement[47].y=z+0.5;         SpaceGroupElement[47].z=-x+0.5;         // -3^1     [1,-1,1]        0,0,0           3/4,1/4,-1/4
      break;

    case 501: // [205] P a -3 (-P 2ac 2ab 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x+0.5;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[2].x=x+0.5;          SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[4].x=z;              SpaceGroupElement[4].y=x;              SpaceGroupElement[4].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-z+0.5;         SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=y+0.5;           // 3^-1     [-1,1,1]        0,0,0           1/2,-1/2,0
      SpaceGroupElement[6].x=z+0.5;          SpaceGroupElement[6].y=-x+0.5;         SpaceGroupElement[6].z=-y;              // 3^-1     [1,-1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[7].x=-z;             SpaceGroupElement[7].y=x+0.5;          SpaceGroupElement[7].z=-y+0.5;          // 3^1      [-1,-1,1]       0,0,0           0,1/2,0
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y+0.5;          SpaceGroupElement[9].y=-z+0.5;         SpaceGroupElement[9].z=-x;              // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    1/3,1/6,0
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z+0.5;         SpaceGroupElement[10].z=-x+0.5;         // 3^1      [-1,1,1]        -1/3,1/3,1/3    1/6,1/6,0
      SpaceGroupElement[11].x=-y+0.5;        SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=x+0.5;          // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[13].x=x-0.5;         SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z-0.5;         // -2       [0,0,1]         -1/2,0,0        0,0,-1/4
      SpaceGroupElement[14].x=-x-0.5;        SpaceGroupElement[14].y=y-0.5;         SpaceGroupElement[14].z=z;              // -2       [1,0,0]         0,-1/2,0        -1/4,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y-0.5;        SpaceGroupElement[15].z=z-0.5;          // -2       [0,1,0]         0,0,-1/2        0,-1/4,0
      SpaceGroupElement[16].x=-z;            SpaceGroupElement[16].y=-x;            SpaceGroupElement[16].z=-y;             // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=z-0.5;         SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=-y-0.5;         // -3^-1    [-1,1,1]        0,0,0           -1/2,-1/2,0
      SpaceGroupElement[18].x=-z-0.5;        SpaceGroupElement[18].y=x-0.5;         SpaceGroupElement[18].z=y;              // -3^-1    [1,-1,1]        0,0,0           0,-1/2,-1/2
      SpaceGroupElement[19].x=z;             SpaceGroupElement[19].y=-x-0.5;        SpaceGroupElement[19].z=y-0.5;          // -3^1     [-1,-1,1]       0,0,0           -1/2,0,-1/2
      SpaceGroupElement[20].x=-y;            SpaceGroupElement[20].y=-z;            SpaceGroupElement[20].z=-x;             // -3^-1    [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[21].x=-y-0.5;        SpaceGroupElement[21].y=z-0.5;         SpaceGroupElement[21].z=x;              // -3^-1    [-1,-1,1]       0,0,0           0,-1/2,0
      SpaceGroupElement[22].x=y;             SpaceGroupElement[22].y=-z-0.5;        SpaceGroupElement[22].z=x-0.5;          // -3^1     [-1,1,1]        0,0,0           0,0,-1/2
      SpaceGroupElement[23].x=y-0.5;         SpaceGroupElement[23].y=z;             SpaceGroupElement[23].z=-x-0.5;         // -3^1     [1,-1,1]        0,0,0           -1/2,0,0
      break;

    case 502: // [206] I a -3 (-I 2b 2c 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y+0.5;         SpaceGroupElement[1].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y+0.5;          SpaceGroupElement[3].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[4].x=z;              SpaceGroupElement[4].y=x;              SpaceGroupElement[4].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-z;             SpaceGroupElement[5].y=-x+0.5;         SpaceGroupElement[5].z=y;               // 3^-1     [-1,1,1]        -1/6,1/6,1/6    1/6,1/6,0
      SpaceGroupElement[6].x=z;              SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-y+0.5;          // 3^-1     [1,-1,1]        1/6,-1/6,1/6    -1/6,1/3,0
      SpaceGroupElement[7].x=-z;             SpaceGroupElement[7].y=x+0.5;          SpaceGroupElement[7].z=-y+0.5;          // 3^1      [-1,-1,1]       0,0,0           0,1/2,0
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;               // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=-z;             SpaceGroupElement[9].z=-x+0.5;          // 3^-1     [-1,-1,1]       -1/6,-1/6,1/6   1/3,1/6,0
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z+0.5;         SpaceGroupElement[10].z=-x+0.5;         // 3^1      [-1,1,1]        -1/3,1/3,1/3    1/6,1/6,0
      SpaceGroupElement[11].x=-y+0.5;        SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=x+0.5;          // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y-0.5;         SpaceGroupElement[13].z=-z;             // -2       [0,0,1]         0,-1/2,0        0,0,0
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z-0.5;          // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y-0.5;        SpaceGroupElement[15].z=z-0.5;          // -2       [0,1,0]         0,0,-1/2        0,-1/4,0
      SpaceGroupElement[16].x=-z;            SpaceGroupElement[16].y=-x;            SpaceGroupElement[16].z=-y;             // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=z;             SpaceGroupElement[17].y=x-0.5;         SpaceGroupElement[17].z=-y;             // -3^-1    [-1,1,1]        0,0,0           1/4,-1/4,1/4
      SpaceGroupElement[18].x=-z;            SpaceGroupElement[18].y=x;             SpaceGroupElement[18].z=y-0.5;          // -3^-1    [1,-1,1]        0,0,0           1/4,1/4,-1/4
      SpaceGroupElement[19].x=z;             SpaceGroupElement[19].y=-x-0.5;        SpaceGroupElement[19].z=y-0.5;          // -3^1     [-1,-1,1]       0,0,0           -1/2,0,-1/2
      SpaceGroupElement[20].x=-y;            SpaceGroupElement[20].y=-z;            SpaceGroupElement[20].z=-x;             // -3^-1    [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=z;             SpaceGroupElement[21].z=x-0.5;          // -3^-1    [-1,-1,1]       0,0,0           1/4,-1/4,-1/4
      SpaceGroupElement[22].x=y;             SpaceGroupElement[22].y=-z-0.5;        SpaceGroupElement[22].z=x-0.5;          // -3^1     [-1,1,1]        0,0,0           0,0,-1/2
      SpaceGroupElement[23].x=y-0.5;         SpaceGroupElement[23].y=z;             SpaceGroupElement[23].z=-x-0.5;         // -3^1     [1,-1,1]        0,0,0           -1/2,0,0
      SpaceGroupElement[24].x=x+0.5;         SpaceGroupElement[24].y=y+0.5;         SpaceGroupElement[24].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[25].x=-x+0.5;        SpaceGroupElement[25].y=-y;            SpaceGroupElement[25].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/2,0
      SpaceGroupElement[26].x=x+0.5;         SpaceGroupElement[26].y=-y+0.5;        SpaceGroupElement[26].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,1/2
      SpaceGroupElement[27].x=-x+0.5;        SpaceGroupElement[27].y=y;             SpaceGroupElement[27].z=-z;             // 2        [0,1,0]         0,1,0           1/4,0,1/2
      SpaceGroupElement[28].x=z+0.5;         SpaceGroupElement[28].y=x+0.5;         SpaceGroupElement[28].z=y+0.5;          // 3^1      [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[29].x=-z+0.5;        SpaceGroupElement[29].y=-x;            SpaceGroupElement[29].z=y+0.5;          // 3^-1     [-1,1,1]        -1/3,1/3,1/3    5/6,-1/6,0
      SpaceGroupElement[30].x=z+0.5;         SpaceGroupElement[30].y=-x+0.5;        SpaceGroupElement[30].z=-y;             // 3^-1     [1,-1,1]        1/3,-1/3,1/3    1/6,2/3,0
      SpaceGroupElement[31].x=-z+0.5;        SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=-y;             // 3^1      [-1,-1,1]       1/6,1/6,-1/6    1/3,7/6,0
      SpaceGroupElement[32].x=y+0.5;         SpaceGroupElement[32].y=z+0.5;         SpaceGroupElement[32].z=x+0.5;          // 3^-1     [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[33].x=y+0.5;         SpaceGroupElement[33].y=-z+0.5;        SpaceGroupElement[33].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           1,1/2,0
      SpaceGroupElement[34].x=-y+0.5;        SpaceGroupElement[34].y=z;             SpaceGroupElement[34].z=-x;             // 3^1      [-1,1,1]        -1/2,1/2,1/2    1/2,1/2,0
      SpaceGroupElement[35].x=-y;            SpaceGroupElement[35].y=-z+0.5;        SpaceGroupElement[35].z=x;              // 3^1      [1,-1,1]        1/2,-1/2,1/2    -1/2,1,0
      SpaceGroupElement[36].x=-x+0.5;        SpaceGroupElement[36].y=-y+0.5;        SpaceGroupElement[36].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[37].x=x+0.5;         SpaceGroupElement[37].y=y;             SpaceGroupElement[37].z=-z+0.5;         // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[38].x=-x+0.5;        SpaceGroupElement[38].y=y+0.5;         SpaceGroupElement[38].z=z;              // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[39].x=x+0.5;         SpaceGroupElement[39].y=-y;            SpaceGroupElement[39].z=z;              // -2       [0,1,0]         1/2,0,0         0,0,0
      SpaceGroupElement[40].x=-z+0.5;        SpaceGroupElement[40].y=-x+0.5;        SpaceGroupElement[40].z=-y+0.5;         // -3^1     [1,1,1]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[41].x=z+0.5;         SpaceGroupElement[41].y=x;             SpaceGroupElement[41].z=-y+0.5;         // -3^-1    [-1,1,1]        0,0,0           1/2,1/2,0
      SpaceGroupElement[42].x=-z+0.5;        SpaceGroupElement[42].y=x+0.5;         SpaceGroupElement[42].z=y;              // -3^-1    [1,-1,1]        0,0,0           0,1/2,1/2
      SpaceGroupElement[43].x=z+0.5;         SpaceGroupElement[43].y=-x;            SpaceGroupElement[43].z=y;              // -3^1     [-1,-1,1]       0,0,0           1/4,-1/4,-1/4
      SpaceGroupElement[44].x=-y+0.5;        SpaceGroupElement[44].y=-z+0.5;        SpaceGroupElement[44].z=-x+0.5;         // -3^-1    [1,1,1]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[45].x=-y+0.5;        SpaceGroupElement[45].y=z+0.5;         SpaceGroupElement[45].z=x;              // -3^-1    [-1,-1,1]       0,0,0           0,1/2,0
      SpaceGroupElement[46].x=y+0.5;         SpaceGroupElement[46].y=-z;            SpaceGroupElement[46].z=x;              // -3^1     [-1,1,1]        0,0,0           1/4,-1/4,1/4
      SpaceGroupElement[47].x=y;             SpaceGroupElement[47].y=z+0.5;         SpaceGroupElement[47].z=-x;             // -3^1     [1,-1,1]        0,0,0           1/4,1/4,-1/4
      break;

    case 503: // [207] P 4 3 2 (P 4 2 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=z;              SpaceGroupElement[9].z=y;               // 2        [0,1,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;              // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=y;              // 4^1      [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=z;             SpaceGroupElement[13].z=-y;             // 4^-1     [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-y;             // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=-y;             // 2        [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[18].x=z;             SpaceGroupElement[18].y=y;             SpaceGroupElement[18].z=-x;             // 4^1      [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-x;             // 2        [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=x;              // 2        [1,0,1]         0,0,0           0,0,0
      SpaceGroupElement[23].x=-z;            SpaceGroupElement[23].y=y;             SpaceGroupElement[23].z=x;              // 4^-1     [0,1,0]         0,0,0           0,0,0
      break;

    case 504: // [208] P 42 3 2 (P 4n 2 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y+0.5;          SpaceGroupElement[5].y=x+0.5;          SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y+0.5;         SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=z+0.5;          SpaceGroupElement[9].z=y+0.5;           // 2        [0,1,1]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;              // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=x+0.5;         SpaceGroupElement[11].y=-z+0.5;        SpaceGroupElement[11].z=y+0.5;          // 4^1      [1,0,0]         1/2,0,0         0,0,1/2
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=z+0.5;         SpaceGroupElement[13].z=-y+0.5;         // 4^-1     [1,0,0]         1/2,0,0         0,1/2,0
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-y;             // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[15].x=-x+0.5;        SpaceGroupElement[15].y=-z+0.5;        SpaceGroupElement[15].z=-y+0.5;         // 2        [0,-1,1]        0,0,0           1/4,1/2,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[18].x=z+0.5;         SpaceGroupElement[18].y=y+0.5;         SpaceGroupElement[18].z=-x+0.5;         // 4^1      [0,1,0]         0,1/2,0         1/2,0,0
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[20].x=-z+0.5;        SpaceGroupElement[20].y=-y+0.5;        SpaceGroupElement[20].z=-x+0.5;         // 2        [-1,0,1]        0,0,0           1/2,1/4,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[22].x=z+0.5;         SpaceGroupElement[22].y=-y+0.5;        SpaceGroupElement[22].z=x+0.5;          // 2        [1,0,1]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[23].x=-z+0.5;        SpaceGroupElement[23].y=y+0.5;         SpaceGroupElement[23].z=x+0.5;          // 4^-1     [0,1,0]         0,1/2,0         0,0,1/2
      break;

    case 505: // [209] F 4 3 2 (F 4 2 3)
      SpaceGroupSize=96;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=z;              SpaceGroupElement[9].z=y;               // 2        [0,1,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;              // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=y;              // 4^1      [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=z;             SpaceGroupElement[13].z=-y;             // 4^-1     [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-y;             // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=-y;             // 2        [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[18].x=z;             SpaceGroupElement[18].y=y;             SpaceGroupElement[18].z=-x;             // 4^1      [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-x;             // 2        [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=x;              // 2        [1,0,1]         0,0,0           0,0,0
      SpaceGroupElement[23].x=-z;            SpaceGroupElement[23].y=y;             SpaceGroupElement[23].z=x;              // 4^-1     [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[24].x=x;             SpaceGroupElement[24].y=y+0.5;         SpaceGroupElement[24].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[25].x=-y;            SpaceGroupElement[25].y=x+0.5;         SpaceGroupElement[25].z=z+0.5;          // 4^1      [0,0,1]         0,0,1/2         -1/4,1/4,0
      SpaceGroupElement[26].x=-x;            SpaceGroupElement[26].y=-y+0.5;        SpaceGroupElement[26].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[27].x=y;             SpaceGroupElement[27].y=-x+0.5;        SpaceGroupElement[27].z=z+0.5;          // 4^-1     [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[28].x=x;             SpaceGroupElement[28].y=-y+0.5;        SpaceGroupElement[28].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[29].x=y;             SpaceGroupElement[29].y=x+0.5;         SpaceGroupElement[29].z=-z+0.5;         // 2        [1,1,0]         1/4,1/4,0       -1/4,0,1/4
      SpaceGroupElement[30].x=-x;            SpaceGroupElement[30].y=y+0.5;         SpaceGroupElement[30].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[31].x=-y;            SpaceGroupElement[31].y=-x+0.5;        SpaceGroupElement[31].z=-z+0.5;         // 2        [-1,1,0]        -1/4,1/4,0      1/4,0,1/4
      SpaceGroupElement[32].x=z;             SpaceGroupElement[32].y=x+0.5;         SpaceGroupElement[32].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     -1/3,-1/6,0
      SpaceGroupElement[33].x=-x;            SpaceGroupElement[33].y=z+0.5;         SpaceGroupElement[33].z=y+0.5;          // 2        [0,1,1]         0,1/2,1/2       0,0,0
      SpaceGroupElement[34].x=-z;            SpaceGroupElement[34].y=-x+0.5;        SpaceGroupElement[34].z=y+0.5;          // 3^-1     [-1,1,1]        -1/3,1/3,1/3    1/3,-1/6,0
      SpaceGroupElement[35].x=x;             SpaceGroupElement[35].y=-z+0.5;        SpaceGroupElement[35].z=y+0.5;          // 4^1      [1,0,0]         0,0,0           0,0,1/2
      SpaceGroupElement[36].x=z;             SpaceGroupElement[36].y=-x+0.5;        SpaceGroupElement[36].z=-y+0.5;         // 3^-1     [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[37].x=x;             SpaceGroupElement[37].y=z+0.5;         SpaceGroupElement[37].z=-y+0.5;         // 4^-1     [1,0,0]         0,0,0           0,1/2,0
      SpaceGroupElement[38].x=-z;            SpaceGroupElement[38].y=x+0.5;         SpaceGroupElement[38].z=-y+0.5;         // 3^1      [-1,-1,1]       0,0,0           0,1/2,0
      SpaceGroupElement[39].x=-x;            SpaceGroupElement[39].y=-z+0.5;        SpaceGroupElement[39].z=-y+0.5;         // 2        [0,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[40].x=y;             SpaceGroupElement[40].y=z+0.5;         SpaceGroupElement[40].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,1/6,0
      SpaceGroupElement[41].x=y;             SpaceGroupElement[41].y=-z+0.5;        SpaceGroupElement[41].z=-x+0.5;         // 3^-1     [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[42].x=z;             SpaceGroupElement[42].y=y+0.5;         SpaceGroupElement[42].z=-x+0.5;         // 4^1      [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[43].x=-y;            SpaceGroupElement[43].y=z+0.5;         SpaceGroupElement[43].z=-x+0.5;         // 3^1      [-1,1,1]        -1/3,1/3,1/3    1/6,1/6,0
      SpaceGroupElement[44].x=-z;            SpaceGroupElement[44].y=-y+0.5;        SpaceGroupElement[44].z=-x+0.5;         // 2        [-1,0,1]        -1/4,0,1/4      1/4,1/4,0
      SpaceGroupElement[45].x=-y;            SpaceGroupElement[45].y=-z+0.5;        SpaceGroupElement[45].z=x+0.5;          // 3^1      [1,-1,1]        0,0,0           -1/2,1/2,0
      SpaceGroupElement[46].x=z;             SpaceGroupElement[46].y=-y+0.5;        SpaceGroupElement[46].z=x+0.5;          // 2        [1,0,1]         1/4,0,1/4       -1/4,1/4,0
      SpaceGroupElement[47].x=-z;            SpaceGroupElement[47].y=y+0.5;         SpaceGroupElement[47].z=x+0.5;          // 4^-1     [0,1,0]         0,1/2,0         -1/4,0,1/4
      SpaceGroupElement[48].x=x+0.5;         SpaceGroupElement[48].y=y;             SpaceGroupElement[48].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[49].x=-y+0.5;        SpaceGroupElement[49].y=x;             SpaceGroupElement[49].z=z+0.5;          // 4^1      [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[50].x=-x+0.5;        SpaceGroupElement[50].y=-y;            SpaceGroupElement[50].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[51].x=y+0.5;         SpaceGroupElement[51].y=-x;            SpaceGroupElement[51].z=z+0.5;          // 4^-1     [0,0,1]         0,0,1/2         1/4,-1/4,0
      SpaceGroupElement[52].x=x+0.5;         SpaceGroupElement[52].y=-y;            SpaceGroupElement[52].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[53].x=y+0.5;         SpaceGroupElement[53].y=x;             SpaceGroupElement[53].z=-z+0.5;         // 2        [1,1,0]         1/4,1/4,0       1/4,0,1/4
      SpaceGroupElement[54].x=-x+0.5;        SpaceGroupElement[54].y=y;             SpaceGroupElement[54].z=-z+0.5;         // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[55].x=-y+0.5;        SpaceGroupElement[55].y=-x;            SpaceGroupElement[55].z=-z+0.5;         // 2        [-1,1,0]        1/4,-1/4,0      1/4,0,1/4
      SpaceGroupElement[56].x=z+0.5;         SpaceGroupElement[56].y=x;             SpaceGroupElement[56].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,-1/6,0
      SpaceGroupElement[57].x=-x+0.5;        SpaceGroupElement[57].y=z;             SpaceGroupElement[57].z=y+0.5;          // 2        [0,1,1]         0,1/4,1/4       1/4,-1/4,0
      SpaceGroupElement[58].x=-z+0.5;        SpaceGroupElement[58].y=-x;            SpaceGroupElement[58].z=y+0.5;          // 3^-1     [-1,1,1]        0,0,0           1/2,-1/2,0
      SpaceGroupElement[59].x=x+0.5;         SpaceGroupElement[59].y=-z;            SpaceGroupElement[59].z=y+0.5;          // 4^1      [1,0,0]         1/2,0,0         0,-1/4,1/4
      SpaceGroupElement[60].x=z+0.5;         SpaceGroupElement[60].y=-x;            SpaceGroupElement[60].z=-y+0.5;         // 3^-1     [1,-1,1]        1/3,-1/3,1/3    1/6,1/6,0
      SpaceGroupElement[61].x=x+0.5;         SpaceGroupElement[61].y=z;             SpaceGroupElement[61].z=-y+0.5;         // 4^-1     [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[62].x=-z+0.5;        SpaceGroupElement[62].y=x;             SpaceGroupElement[62].z=-y+0.5;         // 3^1      [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[63].x=-x+0.5;        SpaceGroupElement[63].y=-z;            SpaceGroupElement[63].z=-y+0.5;         // 2        [0,-1,1]        0,-1/4,1/4      1/4,1/4,0
      SpaceGroupElement[64].x=y+0.5;         SpaceGroupElement[64].y=z;             SpaceGroupElement[64].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,-1/3,0
      SpaceGroupElement[65].x=y+0.5;         SpaceGroupElement[65].y=-z;            SpaceGroupElement[65].z=-x+0.5;         // 3^-1     [-1,-1,1]       0,0,0           1/2,0,0
      SpaceGroupElement[66].x=z+0.5;         SpaceGroupElement[66].y=y;             SpaceGroupElement[66].z=-x+0.5;         // 4^1      [0,1,0]         0,0,0           1/2,0,0
      SpaceGroupElement[67].x=-y+0.5;        SpaceGroupElement[67].y=z;             SpaceGroupElement[67].z=-x+0.5;         // 3^1      [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[68].x=-z+0.5;        SpaceGroupElement[68].y=-y;            SpaceGroupElement[68].z=-x+0.5;         // 2        [-1,0,1]        0,0,0           1/2,0,0
      SpaceGroupElement[69].x=-y+0.5;        SpaceGroupElement[69].y=-z;            SpaceGroupElement[69].z=x+0.5;          // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      SpaceGroupElement[70].x=z+0.5;         SpaceGroupElement[70].y=-y;            SpaceGroupElement[70].z=x+0.5;          // 2        [1,0,1]         1/2,0,1/2       0,0,0
      SpaceGroupElement[71].x=-z+0.5;        SpaceGroupElement[71].y=y;             SpaceGroupElement[71].z=x+0.5;          // 4^-1     [0,1,0]         0,0,0           0,0,1/2
      SpaceGroupElement[72].x=x+0.5;         SpaceGroupElement[72].y=y+0.5;         SpaceGroupElement[72].z=z;              // 1        -               -               -
      SpaceGroupElement[73].x=-y+0.5;        SpaceGroupElement[73].y=x+0.5;         SpaceGroupElement[73].z=z;              // 4^1      [0,0,1]         0,0,0           0,1/2,0
      SpaceGroupElement[74].x=-x+0.5;        SpaceGroupElement[74].y=-y+0.5;        SpaceGroupElement[74].z=z;              // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[75].x=y+0.5;         SpaceGroupElement[75].y=-x+0.5;        SpaceGroupElement[75].z=z;              // 4^-1     [0,0,1]         0,0,0           1/2,0,0
      SpaceGroupElement[76].x=x+0.5;         SpaceGroupElement[76].y=-y+0.5;        SpaceGroupElement[76].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[77].x=y+0.5;         SpaceGroupElement[77].y=x+0.5;         SpaceGroupElement[77].z=-z;             // 2        [1,1,0]         1/2,1/2,0       0,0,0
      SpaceGroupElement[78].x=-x+0.5;        SpaceGroupElement[78].y=y+0.5;         SpaceGroupElement[78].z=-z;             // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[79].x=-y+0.5;        SpaceGroupElement[79].y=-x+0.5;        SpaceGroupElement[79].z=-z;             // 2        [-1,1,0]        0,0,0           1/2,0,0
      SpaceGroupElement[80].x=z+0.5;         SpaceGroupElement[80].y=x+0.5;         SpaceGroupElement[80].z=y;              // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,1/3,0
      SpaceGroupElement[81].x=-x+0.5;        SpaceGroupElement[81].y=z+0.5;         SpaceGroupElement[81].z=y;              // 2        [0,1,1]         0,1/4,1/4       1/4,1/4,0
      SpaceGroupElement[82].x=-z+0.5;        SpaceGroupElement[82].y=-x+0.5;        SpaceGroupElement[82].z=y;              // 3^-1     [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[83].x=x+0.5;         SpaceGroupElement[83].y=-z+0.5;        SpaceGroupElement[83].z=y;              // 4^1      [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[84].x=z+0.5;         SpaceGroupElement[84].y=-x+0.5;        SpaceGroupElement[84].z=-y;             // 3^-1     [1,-1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[85].x=x+0.5;         SpaceGroupElement[85].y=z+0.5;         SpaceGroupElement[85].z=-y;             // 4^-1     [1,0,0]         1/2,0,0         0,1/4,-1/4
      SpaceGroupElement[86].x=-z+0.5;        SpaceGroupElement[86].y=x+0.5;         SpaceGroupElement[86].z=-y;             // 3^1      [-1,-1,1]       1/3,1/3,-1/3    1/6,1/3,0
      SpaceGroupElement[87].x=-x+0.5;        SpaceGroupElement[87].y=-z+0.5;        SpaceGroupElement[87].z=-y;             // 2        [0,-1,1]        0,1/4,-1/4      1/4,1/4,0
      SpaceGroupElement[88].x=y+0.5;         SpaceGroupElement[88].y=z+0.5;         SpaceGroupElement[88].z=x;              // 3^-1     [1,1,1]         1/3,1/3,1/3     1/3,1/6,0
      SpaceGroupElement[89].x=y+0.5;         SpaceGroupElement[89].y=-z+0.5;        SpaceGroupElement[89].z=-x;             // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    1/3,1/6,0
      SpaceGroupElement[90].x=z+0.5;         SpaceGroupElement[90].y=y+0.5;         SpaceGroupElement[90].z=-x;             // 4^1      [0,1,0]         0,1/2,0         1/4,0,-1/4
      SpaceGroupElement[91].x=-y+0.5;        SpaceGroupElement[91].y=z+0.5;         SpaceGroupElement[91].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[92].x=-z+0.5;        SpaceGroupElement[92].y=-y+0.5;        SpaceGroupElement[92].z=-x;             // 2        [-1,0,1]        1/4,0,-1/4      1/4,1/4,0
      SpaceGroupElement[93].x=-y+0.5;        SpaceGroupElement[93].y=-z+0.5;        SpaceGroupElement[93].z=x;              // 3^1      [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[94].x=z+0.5;         SpaceGroupElement[94].y=-y+0.5;        SpaceGroupElement[94].z=x;              // 2        [1,0,1]         1/4,0,1/4       1/4,1/4,0
      SpaceGroupElement[95].x=-z+0.5;        SpaceGroupElement[95].y=y+0.5;         SpaceGroupElement[95].z=x;              // 4^-1     [0,1,0]         0,1/2,0         1/4,0,1/4
      break;

    case 506: // [210] F 41 3 2 (F 4d 2 3)
      SpaceGroupSize=96;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.25;        SpaceGroupElement[1].y=x+0.25;         SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         0,1/4,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[3].x=y+0.75;         SpaceGroupElement[3].y=-x+0.25;        SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         1/2,-1/4,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y+0.25;         SpaceGroupElement[5].y=x+0.25;         SpaceGroupElement[5].z=-z+0.25;         // 2        [1,1,0]         1/4,1/4,0       0,0,1/8
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[7].x=-y+0.75;        SpaceGroupElement[7].y=-x+0.25;        SpaceGroupElement[7].z=-z+0.75;         // 2        [-1,1,0]        1/4,-1/4,0      1/2,0,3/8
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x+0.25;        SpaceGroupElement[9].y=z+0.25;         SpaceGroupElement[9].z=y+0.25;          // 2        [0,1,1]         0,1/4,1/4       1/8,0,0
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x+0.5;        SpaceGroupElement[10].z=y+0.5;          // 3^-1     [-1,1,1]        -1/3,1/3,1/3    1/3,-1/6,0
      SpaceGroupElement[11].x=x+0.75;        SpaceGroupElement[11].y=-z+0.25;       SpaceGroupElement[11].z=y+0.75;         // 4^1      [1,0,0]         3/4,0,0         0,-1/4,1/2
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[13].x=x+0.25;        SpaceGroupElement[13].y=z+0.25;        SpaceGroupElement[13].z=-y+0.25;        // 4^-1     [1,0,0]         1/4,0,0         0,1/4,0
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x+0.5;         SpaceGroupElement[14].z=-y+0.5;         // 3^1      [-1,-1,1]       0,0,0           0,1/2,0
      SpaceGroupElement[15].x=-x+0.75;       SpaceGroupElement[15].y=-z+0.25;       SpaceGroupElement[15].z=-y+0.75;        // 2        [0,-1,1]        0,-1/4,1/4      3/8,1/2,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y+0.5;         SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=-x+0.5;         // 3^-1     [-1,-1,1]       0,0,0           1/2,0,0
      SpaceGroupElement[18].x=z+0.25;        SpaceGroupElement[18].y=y+0.75;        SpaceGroupElement[18].z=-x+0.75;        // 4^1      [0,1,0]         0,3/4,0         1/2,0,1/4
      SpaceGroupElement[19].x=-y+0.5;        SpaceGroupElement[19].y=z+0.5;         SpaceGroupElement[19].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[20].x=-z+0.25;       SpaceGroupElement[20].y=-y+0.25;       SpaceGroupElement[20].z=-x+0.25;        // 2        [-1,0,1]        0,0,0           1/4,1/8,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[22].x=z+0.25;        SpaceGroupElement[22].y=-y+0.75;       SpaceGroupElement[22].z=x+0.75;         // 2        [1,0,1]         1/2,0,1/2       -1/4,3/8,0
      SpaceGroupElement[23].x=-z+0.75;       SpaceGroupElement[23].y=y+0.75;        SpaceGroupElement[23].z=x+0.25;         // 4^-1     [0,1,0]         0,3/4,0         1/4,0,1/2
      SpaceGroupElement[24].x=x;             SpaceGroupElement[24].y=y+0.5;         SpaceGroupElement[24].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[25].x=-y+0.25;       SpaceGroupElement[25].y=x+0.75;        SpaceGroupElement[25].z=z+0.75;         // 4^1      [0,0,1]         0,0,3/4         -1/4,1/2,0
      SpaceGroupElement[26].x=-x;            SpaceGroupElement[26].y=-y;            SpaceGroupElement[26].z=z;              // 2        [0,0,1]         0,0,1           0,1/2,0
      SpaceGroupElement[27].x=y+0.75;        SpaceGroupElement[27].y=-x+0.75;       SpaceGroupElement[27].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         3/4,0,0
      SpaceGroupElement[28].x=x;             SpaceGroupElement[28].y=-y+0.5;        SpaceGroupElement[28].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[29].x=y+0.25;        SpaceGroupElement[29].y=x+0.75;        SpaceGroupElement[29].z=-z+0.75;        // 2        [1,1,0]         1/2,1/2,0       -1/4,0,3/8
      SpaceGroupElement[30].x=-x;            SpaceGroupElement[30].y=y;             SpaceGroupElement[30].z=-z;             // 2        [0,1,0]         0,1,0           0,0,1/2
      SpaceGroupElement[31].x=-y+0.75;       SpaceGroupElement[31].y=-x+0.75;       SpaceGroupElement[31].z=-z+0.25;        // 2        [-1,1,0]        0,0,0           3/4,0,5/8
      SpaceGroupElement[32].x=z;             SpaceGroupElement[32].y=x+0.5;         SpaceGroupElement[32].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     -1/3,-1/6,0
      SpaceGroupElement[33].x=-x+0.25;       SpaceGroupElement[33].y=z+0.75;        SpaceGroupElement[33].z=y+0.75;         // 2        [0,1,1]         0,3/4,3/4       1/8,0,0
      SpaceGroupElement[34].x=-z;            SpaceGroupElement[34].y=-x;            SpaceGroupElement[34].z=y;              // 3^-1     [-1,1,1]        -2/3,2/3,2/3    2/3,-1/3,0
      SpaceGroupElement[35].x=x+0.75;        SpaceGroupElement[35].y=-z+0.75;       SpaceGroupElement[35].z=y+0.25;         // 4^1      [1,0,0]         3/4,0,0         0,-1/4,1
      SpaceGroupElement[36].x=z;             SpaceGroupElement[36].y=-x+0.5;        SpaceGroupElement[36].z=-y+0.5;         // 3^-1     [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[37].x=x+0.25;        SpaceGroupElement[37].y=z+0.75;        SpaceGroupElement[37].z=-y+0.75;        // 4^-1     [1,0,0]         1/4,0,0         0,3/4,0
      SpaceGroupElement[38].x=-z;            SpaceGroupElement[38].y=x;             SpaceGroupElement[38].z=-y;             // 3^1      [-1,-1,1]       0,0,0           0,1,0
      SpaceGroupElement[39].x=-x+0.75;       SpaceGroupElement[39].y=-z+0.75;       SpaceGroupElement[39].z=-y+0.25;        // 2        [0,-1,1]        0,-1/4,1/4      3/8,1,0
      SpaceGroupElement[40].x=y;             SpaceGroupElement[40].y=z+0.5;         SpaceGroupElement[40].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,1/6,0
      SpaceGroupElement[41].x=y+0.5;         SpaceGroupElement[41].y=-z+0.5;        SpaceGroupElement[41].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           1,1/2,0
      SpaceGroupElement[42].x=z+0.25;        SpaceGroupElement[42].y=y+0.25;        SpaceGroupElement[42].z=-x+0.25;        // 4^1      [0,1,0]         0,5/4,0         3/4,0,1/2
      SpaceGroupElement[43].x=-y+0.5;        SpaceGroupElement[43].y=z;             SpaceGroupElement[43].z=-x+0.5;         // 3^1      [-1,1,1]        -1/3,1/3,1/3    1/6,2/3,0
      SpaceGroupElement[44].x=-z+0.25;       SpaceGroupElement[44].y=-y+0.75;       SpaceGroupElement[44].z=-x+0.75;        // 2        [-1,0,1]        -1/4,0,1/4      1/2,3/8,0
      SpaceGroupElement[45].x=-y;            SpaceGroupElement[45].y=-z+0.5;        SpaceGroupElement[45].z=x+0.5;          // 3^1      [1,-1,1]        0,0,0           -1/2,1/2,0
      SpaceGroupElement[46].x=z+0.25;        SpaceGroupElement[46].y=-y+0.25;       SpaceGroupElement[46].z=x+0.25;         // 2        [1,0,1]         3/4,0,3/4       -1/2,5/8,0
      SpaceGroupElement[47].x=-z+0.75;       SpaceGroupElement[47].y=y+0.25;        SpaceGroupElement[47].z=x+0.75;         // 4^-1     [0,1,0]         0,5/4,0         0,0,3/4
      SpaceGroupElement[48].x=x+0.5;         SpaceGroupElement[48].y=y;             SpaceGroupElement[48].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[49].x=-y+0.75;       SpaceGroupElement[49].y=x+0.25;        SpaceGroupElement[49].z=z+0.75;         // 4^1      [0,0,1]         0,0,3/4         1/4,1/2,0
      SpaceGroupElement[50].x=-x+0.5;        SpaceGroupElement[50].y=-y+0.5;        SpaceGroupElement[50].z=z;              // 2        [0,0,1]         0,0,1           1/4,1/4,0
      SpaceGroupElement[51].x=y+0.25;        SpaceGroupElement[51].y=-x+0.25;       SpaceGroupElement[51].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         3/4,-1/2,0
      SpaceGroupElement[52].x=x+0.5;         SpaceGroupElement[52].y=-y;            SpaceGroupElement[52].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[53].x=y+0.75;        SpaceGroupElement[53].y=x+0.25;        SpaceGroupElement[53].z=-z+0.75;        // 2        [1,1,0]         1/2,1/2,0       1/4,0,3/8
      SpaceGroupElement[54].x=-x+0.5;        SpaceGroupElement[54].y=y+0.5;         SpaceGroupElement[54].z=-z;             // 2        [0,1,0]         0,1/2,0         1/4,0,1/2
      SpaceGroupElement[55].x=-y+0.25;       SpaceGroupElement[55].y=-x+0.25;       SpaceGroupElement[55].z=-z+0.25;        // 2        [-1,1,0]        1/2,-1/2,0      3/4,0,5/8
      SpaceGroupElement[56].x=z+0.5;         SpaceGroupElement[56].y=x;             SpaceGroupElement[56].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,-1/6,0
      SpaceGroupElement[57].x=-x+0.75;       SpaceGroupElement[57].y=z+0.25;        SpaceGroupElement[57].z=y+0.75;         // 2        [0,1,1]         0,1/2,1/2       3/8,-1/4,0
      SpaceGroupElement[58].x=-z+0.5;        SpaceGroupElement[58].y=-x+0.5;        SpaceGroupElement[58].z=y;              // 3^-1     [-1,1,1]        -1/3,1/3,1/3    5/6,-2/3,0
      SpaceGroupElement[59].x=x+0.25;        SpaceGroupElement[59].y=-z+0.25;       SpaceGroupElement[59].z=y+0.25;         // 4^1      [1,0,0]         5/4,0,0         0,-1/2,3/4
      SpaceGroupElement[60].x=z+0.5;         SpaceGroupElement[60].y=-x;            SpaceGroupElement[60].z=-y+0.5;         // 3^-1     [1,-1,1]        1/3,-1/3,1/3    1/6,1/6,0
      SpaceGroupElement[61].x=x+0.75;        SpaceGroupElement[61].y=z+0.25;        SpaceGroupElement[61].z=-y+0.75;        // 4^-1     [1,0,0]         3/4,0,0         0,1/2,1/4
      SpaceGroupElement[62].x=-z+0.5;        SpaceGroupElement[62].y=x+0.5;         SpaceGroupElement[62].z=-y;             // 3^1      [-1,-1,1]       0,0,0           1/2,1,0
      SpaceGroupElement[63].x=-x+0.25;       SpaceGroupElement[63].y=-z+0.25;       SpaceGroupElement[63].z=-y+0.25;        // 2        [0,-1,1]        0,-1/2,1/2      5/8,3/4,0
      SpaceGroupElement[64].x=y+0.5;         SpaceGroupElement[64].y=z;             SpaceGroupElement[64].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,-1/3,0
      SpaceGroupElement[65].x=y;             SpaceGroupElement[65].y=-z;            SpaceGroupElement[65].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           1,0,0
      SpaceGroupElement[66].x=z+0.75;        SpaceGroupElement[66].y=y+0.75;        SpaceGroupElement[66].z=-x+0.25;        // 4^1      [0,1,0]         0,3/4,0         1,0,1/4
      SpaceGroupElement[67].x=-y;            SpaceGroupElement[67].y=z+0.5;         SpaceGroupElement[67].z=-x+0.5;         // 3^1      [-1,1,1]        0,0,0           1/2,1/2,0
      SpaceGroupElement[68].x=-z+0.75;       SpaceGroupElement[68].y=-y+0.25;       SpaceGroupElement[68].z=-x+0.75;        // 2        [-1,0,1]        0,0,0           3/4,1/8,0
      SpaceGroupElement[69].x=-y+0.5;        SpaceGroupElement[69].y=-z;            SpaceGroupElement[69].z=x+0.5;          // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      SpaceGroupElement[70].x=z+0.75;        SpaceGroupElement[70].y=-y+0.75;       SpaceGroupElement[70].z=x+0.25;         // 2        [1,0,1]         1,0,1           -1/4,3/8,0
      SpaceGroupElement[71].x=-z+0.25;       SpaceGroupElement[71].y=y+0.75;        SpaceGroupElement[71].z=x+0.75;         // 4^-1     [0,1,0]         0,3/4,0         1/4,0,1
      SpaceGroupElement[72].x=x+0.5;         SpaceGroupElement[72].y=y+0.5;         SpaceGroupElement[72].z=z;              // 1        -               -               -
      SpaceGroupElement[73].x=-y+0.75;       SpaceGroupElement[73].y=x+0.75;        SpaceGroupElement[73].z=z+0.25;         // 4^1      [0,0,1]         0,0,1/4         0,3/4,0
      SpaceGroupElement[74].x=-x+0.5;        SpaceGroupElement[74].y=-y;            SpaceGroupElement[74].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/2,0
      SpaceGroupElement[75].x=y+0.25;        SpaceGroupElement[75].y=-x+0.75;       SpaceGroupElement[75].z=z+0.75;         // 4^-1     [0,0,1]         0,0,3/4         1,-1/4,0
      SpaceGroupElement[76].x=x+0.5;         SpaceGroupElement[76].y=-y+0.5;        SpaceGroupElement[76].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[77].x=y+0.75;        SpaceGroupElement[77].y=x+0.75;        SpaceGroupElement[77].z=-z+0.25;        // 2        [1,1,0]         3/4,3/4,0       0,0,1/8
      SpaceGroupElement[78].x=-x+0.5;        SpaceGroupElement[78].y=y;             SpaceGroupElement[78].z=-z+0.5;         // 2        [0,1,0]         0,1,0           1/4,0,1/4
      SpaceGroupElement[79].x=-y+0.25;       SpaceGroupElement[79].y=-x+0.75;       SpaceGroupElement[79].z=-z+0.75;        // 2        [-1,1,0]        1/4,-1/4,0      1,0,3/8
      SpaceGroupElement[80].x=z+0.5;         SpaceGroupElement[80].y=x+0.5;         SpaceGroupElement[80].z=y;              // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,1/3,0
      SpaceGroupElement[81].x=-x+0.75;       SpaceGroupElement[81].y=z+0.75;        SpaceGroupElement[81].z=y+0.25;         // 2        [0,1,1]         0,1/2,1/2       3/8,1/4,0
      SpaceGroupElement[82].x=-z+0.5;        SpaceGroupElement[82].y=-x;            SpaceGroupElement[82].z=y+0.5;          // 3^-1     [-1,1,1]        -1/3,1/3,1/3    5/6,-1/6,0
      SpaceGroupElement[83].x=x+0.25;        SpaceGroupElement[83].y=-z+0.75;       SpaceGroupElement[83].z=y+0.75;         // 4^1      [1,0,0]         5/4,0,0         0,0,3/4
      SpaceGroupElement[84].x=z+0.5;         SpaceGroupElement[84].y=-x+0.5;        SpaceGroupElement[84].z=-y;             // 3^-1     [1,-1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[85].x=x+0.75;        SpaceGroupElement[85].y=z+0.75;        SpaceGroupElement[85].z=-y+0.25;        // 4^-1     [1,0,0]         3/4,0,0         0,1/2,-1/4
      SpaceGroupElement[86].x=-z+0.5;        SpaceGroupElement[86].y=x;             SpaceGroupElement[86].z=-y+0.5;         // 3^1      [-1,-1,1]       1/3,1/3,-1/3    1/6,5/6,0
      SpaceGroupElement[87].x=-x+0.25;       SpaceGroupElement[87].y=-z+0.75;       SpaceGroupElement[87].z=-y+0.75;        // 2        [0,-1,1]        0,0,0           5/8,3/4,0
      SpaceGroupElement[88].x=y+0.5;         SpaceGroupElement[88].y=z+0.5;         SpaceGroupElement[88].z=x;              // 3^-1     [1,1,1]         1/3,1/3,1/3     1/3,1/6,0
      SpaceGroupElement[89].x=y;             SpaceGroupElement[89].y=-z+0.5;        SpaceGroupElement[89].z=-x+0.5;         // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    5/6,1/6,0
      SpaceGroupElement[90].x=z+0.75;        SpaceGroupElement[90].y=y+0.25;        SpaceGroupElement[90].z=-x+0.75;        // 4^1      [0,1,0]         0,5/4,0         3/4,0,0
      SpaceGroupElement[91].x=-y;            SpaceGroupElement[91].y=z;             SpaceGroupElement[91].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,1,0
      SpaceGroupElement[92].x=-z+0.75;       SpaceGroupElement[92].y=-y+0.75;       SpaceGroupElement[92].z=-x+0.25;        // 2        [-1,0,1]        1/4,0,-1/4      1/2,3/8,0
      SpaceGroupElement[93].x=-y+0.5;        SpaceGroupElement[93].y=-z+0.5;        SpaceGroupElement[93].z=x;              // 3^1      [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[94].x=z+0.75;        SpaceGroupElement[94].y=-y+0.25;       SpaceGroupElement[94].z=x+0.75;         // 2        [1,0,1]         3/4,0,3/4       0,5/8,0
      SpaceGroupElement[95].x=-z+0.25;       SpaceGroupElement[95].y=y+0.25;        SpaceGroupElement[95].z=x+0.25;         // 4^-1     [0,1,0]         0,5/4,0         1/2,0,3/4
      break;

    case 507: // [211] I 4 3 2 (I 4 2 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=z;              SpaceGroupElement[9].z=y;               // 2        [0,1,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;              // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=y;              // 4^1      [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=z;             SpaceGroupElement[13].z=-y;             // 4^-1     [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-y;             // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=-y;             // 2        [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[18].x=z;             SpaceGroupElement[18].y=y;             SpaceGroupElement[18].z=-x;             // 4^1      [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-x;             // 2        [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=x;              // 2        [1,0,1]         0,0,0           0,0,0
      SpaceGroupElement[23].x=-z;            SpaceGroupElement[23].y=y;             SpaceGroupElement[23].z=x;              // 4^-1     [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[24].x=x+0.5;         SpaceGroupElement[24].y=y+0.5;         SpaceGroupElement[24].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[25].x=-y+0.5;        SpaceGroupElement[25].y=x+0.5;         SpaceGroupElement[25].z=z+0.5;          // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[26].x=-x+0.5;        SpaceGroupElement[26].y=-y+0.5;        SpaceGroupElement[26].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[27].x=y+0.5;         SpaceGroupElement[27].y=-x+0.5;        SpaceGroupElement[27].z=z+0.5;          // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[28].x=x+0.5;         SpaceGroupElement[28].y=-y+0.5;        SpaceGroupElement[28].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[29].x=y+0.5;         SpaceGroupElement[29].y=x+0.5;         SpaceGroupElement[29].z=-z+0.5;         // 2        [1,1,0]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[30].x=-x+0.5;        SpaceGroupElement[30].y=y+0.5;         SpaceGroupElement[30].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[31].x=-y+0.5;        SpaceGroupElement[31].y=-x+0.5;        SpaceGroupElement[31].z=-z+0.5;         // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      SpaceGroupElement[32].x=z+0.5;         SpaceGroupElement[32].y=x+0.5;         SpaceGroupElement[32].z=y+0.5;          // 3^1      [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[33].x=-x+0.5;        SpaceGroupElement[33].y=z+0.5;         SpaceGroupElement[33].z=y+0.5;          // 2        [0,1,1]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[34].x=-z+0.5;        SpaceGroupElement[34].y=-x+0.5;        SpaceGroupElement[34].z=y+0.5;          // 3^-1     [-1,1,1]        -1/6,1/6,1/6    2/3,-1/3,0
      SpaceGroupElement[35].x=x+0.5;         SpaceGroupElement[35].y=-z+0.5;        SpaceGroupElement[35].z=y+0.5;          // 4^1      [1,0,0]         1/2,0,0         0,0,1/2
      SpaceGroupElement[36].x=z+0.5;         SpaceGroupElement[36].y=-x+0.5;        SpaceGroupElement[36].z=-y+0.5;         // 3^-1     [1,-1,1]        1/6,-1/6,1/6    1/3,1/3,0
      SpaceGroupElement[37].x=x+0.5;         SpaceGroupElement[37].y=z+0.5;         SpaceGroupElement[37].z=-y+0.5;         // 4^-1     [1,0,0]         1/2,0,0         0,1/2,0
      SpaceGroupElement[38].x=-z+0.5;        SpaceGroupElement[38].y=x+0.5;         SpaceGroupElement[38].z=-y+0.5;         // 3^1      [-1,-1,1]       1/6,1/6,-1/6    1/3,2/3,0
      SpaceGroupElement[39].x=-x+0.5;        SpaceGroupElement[39].y=-z+0.5;        SpaceGroupElement[39].z=-y+0.5;         // 2        [0,-1,1]        0,0,0           1/4,1/2,0
      SpaceGroupElement[40].x=y+0.5;         SpaceGroupElement[40].y=z+0.5;         SpaceGroupElement[40].z=x+0.5;          // 3^-1     [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[41].x=y+0.5;         SpaceGroupElement[41].y=-z+0.5;        SpaceGroupElement[41].z=-x+0.5;         // 3^-1     [-1,-1,1]       1/6,1/6,-1/6    2/3,1/3,0
      SpaceGroupElement[42].x=z+0.5;         SpaceGroupElement[42].y=y+0.5;         SpaceGroupElement[42].z=-x+0.5;         // 4^1      [0,1,0]         0,1/2,0         1/2,0,0
      SpaceGroupElement[43].x=-y+0.5;        SpaceGroupElement[43].y=z+0.5;         SpaceGroupElement[43].z=-x+0.5;         // 3^1      [-1,1,1]        -1/6,1/6,1/6    1/3,1/3,0
      SpaceGroupElement[44].x=-z+0.5;        SpaceGroupElement[44].y=-y+0.5;        SpaceGroupElement[44].z=-x+0.5;         // 2        [-1,0,1]        0,0,0           1/2,1/4,0
      SpaceGroupElement[45].x=-y+0.5;        SpaceGroupElement[45].y=-z+0.5;        SpaceGroupElement[45].z=x+0.5;          // 3^1      [1,-1,1]        1/6,-1/6,1/6    -1/3,2/3,0
      SpaceGroupElement[46].x=z+0.5;         SpaceGroupElement[46].y=-y+0.5;        SpaceGroupElement[46].z=x+0.5;          // 2        [1,0,1]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[47].x=-z+0.5;        SpaceGroupElement[47].y=y+0.5;         SpaceGroupElement[47].z=x+0.5;          // 4^-1     [0,1,0]         0,1/2,0         0,0,1/2
      break;

    case 508: // [212] P 43 3 2 (P 4acd 2ab 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.75;        SpaceGroupElement[1].y=x+0.25;         SpaceGroupElement[1].z=z+0.75;          // 4^1      [0,0,1]         0,0,3/4         1/4,1/2,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[3].x=y+0.75;         SpaceGroupElement[3].y=-x+0.75;        SpaceGroupElement[3].z=z+0.25;          // 4^-1     [0,0,1]         0,0,1/4         3/4,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[5].x=y+0.25;         SpaceGroupElement[5].y=x+0.75;         SpaceGroupElement[5].z=-z+0.75;         // 2        [1,1,0]         1/2,1/2,0       -1/4,0,3/8
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[7].x=-y+0.25;        SpaceGroupElement[7].y=-x+0.25;        SpaceGroupElement[7].z=-z+0.25;         // 2        [-1,1,0]        0,0,0           1/4,0,1/8
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x+0.75;        SpaceGroupElement[9].y=z+0.25;         SpaceGroupElement[9].z=y+0.75;          // 2        [0,1,1]         0,1/2,1/2       3/8,-1/4,0
      SpaceGroupElement[10].x=-z+0.5;        SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y+0.5;          // 3^-1     [-1,1,1]        0,0,0           1/2,-1/2,0
      SpaceGroupElement[11].x=x+0.75;        SpaceGroupElement[11].y=-z+0.75;       SpaceGroupElement[11].z=y+0.25;         // 4^1      [1,0,0]         3/4,0,0         0,1/4,1/2
      SpaceGroupElement[12].x=z+0.5;         SpaceGroupElement[12].y=-x+0.5;        SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[13].x=x+0.25;        SpaceGroupElement[13].y=z+0.75;        SpaceGroupElement[13].z=-y+0.75;        // 4^-1     [1,0,0]         1/4,0,0         0,3/4,0
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x+0.5;         SpaceGroupElement[14].z=-y+0.5;         // 3^1      [-1,-1,1]       0,0,0           0,1/2,0
      SpaceGroupElement[15].x=-x+0.25;       SpaceGroupElement[15].y=-z+0.25;       SpaceGroupElement[15].z=-y+0.25;        // 2        [0,-1,1]        0,0,0           1/8,1/4,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y+0.5;         SpaceGroupElement[17].y=-z+0.5;        SpaceGroupElement[17].z=-x;             // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    1/3,1/6,0
      SpaceGroupElement[18].x=z+0.25;        SpaceGroupElement[18].y=y+0.75;        SpaceGroupElement[18].z=-x+0.75;        // 4^1      [0,1,0]         0,3/4,0         1/2,0,1/4
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z+0.5;         SpaceGroupElement[19].z=-x+0.5;         // 3^1      [-1,1,1]        -1/3,1/3,1/3    1/6,1/6,0
      SpaceGroupElement[20].x=-z+0.25;       SpaceGroupElement[20].y=-y+0.25;       SpaceGroupElement[20].z=-x+0.25;        // 2        [-1,0,1]        0,0,0           1/4,1/8,0
      SpaceGroupElement[21].x=-y+0.5;        SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x+0.5;          // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      SpaceGroupElement[22].x=z+0.75;        SpaceGroupElement[22].y=-y+0.75;       SpaceGroupElement[22].z=x+0.25;         // 2        [1,0,1]         1/2,0,1/2       1/4,3/8,0
      SpaceGroupElement[23].x=-z+0.75;       SpaceGroupElement[23].y=y+0.25;        SpaceGroupElement[23].z=x+0.75;         // 4^-1     [0,1,0]         0,1/4,0         0,0,3/4
      break;

    case 509: // [213] P 41 3 2 (P 4bd 2ab 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.25;        SpaceGroupElement[1].y=x+0.75;         SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         -1/4,1/2,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[3].x=y+0.25;         SpaceGroupElement[3].y=-x+0.25;        SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         1/4,0,0
      SpaceGroupElement[4].x=x+0.5;          SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[5].x=y+0.75;         SpaceGroupElement[5].y=x+0.25;         SpaceGroupElement[5].z=-z+0.25;         // 2        [1,1,0]         1/2,1/2,0       1/4,0,1/8
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[7].x=-y+0.75;        SpaceGroupElement[7].y=-x+0.75;        SpaceGroupElement[7].z=-z+0.75;         // 2        [-1,1,0]        0,0,0           3/4,0,3/8
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x+0.25;        SpaceGroupElement[9].y=z+0.75;         SpaceGroupElement[9].z=y+0.25;          // 2        [0,1,1]         0,1/2,1/2       1/8,1/4,0
      SpaceGroupElement[10].x=-z+0.5;        SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y+0.5;          // 3^-1     [-1,1,1]        0,0,0           1/2,-1/2,0
      SpaceGroupElement[11].x=x+0.25;        SpaceGroupElement[11].y=-z+0.25;       SpaceGroupElement[11].z=y+0.75;         // 4^1      [1,0,0]         1/4,0,0         0,-1/4,1/2
      SpaceGroupElement[12].x=z+0.5;         SpaceGroupElement[12].y=-x+0.5;        SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[13].x=x+0.75;        SpaceGroupElement[13].y=z+0.25;        SpaceGroupElement[13].z=-y+0.25;        // 4^-1     [1,0,0]         3/4,0,0         0,1/4,0
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x+0.5;         SpaceGroupElement[14].z=-y+0.5;         // 3^1      [-1,-1,1]       0,0,0           0,1/2,0
      SpaceGroupElement[15].x=-x+0.75;       SpaceGroupElement[15].y=-z+0.75;       SpaceGroupElement[15].z=-y+0.75;        // 2        [0,-1,1]        0,0,0           3/8,3/4,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y+0.5;         SpaceGroupElement[17].y=-z+0.5;        SpaceGroupElement[17].z=-x;             // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    1/3,1/6,0
      SpaceGroupElement[18].x=z+0.75;        SpaceGroupElement[18].y=y+0.25;        SpaceGroupElement[18].z=-x+0.25;        // 4^1      [0,1,0]         0,1/4,0         1/2,0,-1/4
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z+0.5;         SpaceGroupElement[19].z=-x+0.5;         // 3^1      [-1,1,1]        -1/3,1/3,1/3    1/6,1/6,0
      SpaceGroupElement[20].x=-z+0.75;       SpaceGroupElement[20].y=-y+0.75;       SpaceGroupElement[20].z=-x+0.75;        // 2        [-1,0,1]        0,0,0           3/4,3/8,0
      SpaceGroupElement[21].x=-y+0.5;        SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x+0.5;          // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      SpaceGroupElement[22].x=z+0.25;        SpaceGroupElement[22].y=-y+0.25;       SpaceGroupElement[22].z=x+0.75;         // 2        [1,0,1]         1/2,0,1/2       -1/4,1/8,0
      SpaceGroupElement[23].x=-z+0.25;       SpaceGroupElement[23].y=y+0.75;        SpaceGroupElement[23].z=x+0.25;         // 4^-1     [0,1,0]         0,3/4,0         0,0,1/4
      break;

    case 510: // [214] I 41 3 2 (I 4bd 2c 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.25;        SpaceGroupElement[1].y=x+0.75;         SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         -1/4,1/2,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[3].x=y+0.25;         SpaceGroupElement[3].y=-x+0.25;        SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         1/4,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[5].x=y+0.25;         SpaceGroupElement[5].y=x+0.75;         SpaceGroupElement[5].z=-z+0.75;         // 2        [1,1,0]         1/2,1/2,0       -1/4,0,3/8
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[7].x=-y+0.25;        SpaceGroupElement[7].y=-x+0.25;        SpaceGroupElement[7].z=-z+0.25;         // 2        [-1,1,0]        0,0,0           1/4,0,1/8
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x+0.25;        SpaceGroupElement[9].y=z+0.75;         SpaceGroupElement[9].z=y+0.25;          // 2        [0,1,1]         0,1/2,1/2       1/8,1/4,0
      SpaceGroupElement[10].x=-z+0.5;        SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y+0.5;          // 3^-1     [-1,1,1]        0,0,0           1/2,-1/2,0
      SpaceGroupElement[11].x=x+0.25;        SpaceGroupElement[11].y=-z+0.25;       SpaceGroupElement[11].z=y+0.75;         // 4^1      [1,0,0]         1/4,0,0         0,-1/4,1/2
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y+0.5;         // 3^-1     [1,-1,1]        1/6,-1/6,1/6    -1/6,1/3,0
      SpaceGroupElement[13].x=x+0.25;        SpaceGroupElement[13].y=z+0.75;        SpaceGroupElement[13].z=-y+0.75;        // 4^-1     [1,0,0]         1/4,0,0         0,3/4,0
      SpaceGroupElement[14].x=-z+0.5;        SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-y;             // 3^1      [-1,-1,1]       1/6,1/6,-1/6    1/3,1/6,0
      SpaceGroupElement[15].x=-x+0.25;       SpaceGroupElement[15].y=-z+0.25;       SpaceGroupElement[15].z=-y+0.25;        // 2        [0,-1,1]        0,0,0           1/8,1/4,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y+0.5;         SpaceGroupElement[17].y=-z+0.5;        SpaceGroupElement[17].z=-x;             // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    1/3,1/6,0
      SpaceGroupElement[18].x=z+0.75;        SpaceGroupElement[18].y=y+0.25;        SpaceGroupElement[18].z=-x+0.25;        // 4^1      [0,1,0]         0,1/4,0         1/2,0,-1/4
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z+0.5;         SpaceGroupElement[19].z=-x+0.5;         // 3^1      [-1,1,1]        -1/3,1/3,1/3    1/6,1/6,0
      SpaceGroupElement[20].x=-z+0.25;       SpaceGroupElement[20].y=-y+0.25;       SpaceGroupElement[20].z=-x+0.25;        // 2        [-1,0,1]        0,0,0           1/4,1/8,0
      SpaceGroupElement[21].x=-y+0.5;        SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x+0.5;          // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      SpaceGroupElement[22].x=z+0.75;        SpaceGroupElement[22].y=-y+0.75;       SpaceGroupElement[22].z=x+0.25;         // 2        [1,0,1]         1/2,0,1/2       1/4,3/8,0
      SpaceGroupElement[23].x=-z+0.75;       SpaceGroupElement[23].y=y+0.25;        SpaceGroupElement[23].z=x+0.75;         // 4^-1     [0,1,0]         0,1/4,0         0,0,3/4
      SpaceGroupElement[24].x=x+0.5;         SpaceGroupElement[24].y=y+0.5;         SpaceGroupElement[24].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[25].x=-y+0.75;       SpaceGroupElement[25].y=x+0.25;        SpaceGroupElement[25].z=z+0.75;         // 4^1      [0,0,1]         0,0,3/4         -1/4,1,0
      SpaceGroupElement[26].x=-x;            SpaceGroupElement[26].y=-y+0.5;        SpaceGroupElement[26].z=z;               // 2        [0,0,1]         0,0,1           1/2,1/4,0
      SpaceGroupElement[27].x=y+0.75;        SpaceGroupElement[27].y=-x+0.75;       SpaceGroupElement[27].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         3/4,0,0
      SpaceGroupElement[28].x=x+0.5;         SpaceGroupElement[28].y=-y+0.5;        SpaceGroupElement[28].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,1/2
      SpaceGroupElement[29].x=y+0.75;        SpaceGroupElement[29].y=x+0.25;        SpaceGroupElement[29].z=-z+0.25;        // 2        [1,1,0]         1,1,0           -1/4,0,5/8
      SpaceGroupElement[30].x=-x;            SpaceGroupElement[30].y=y+0.5;         SpaceGroupElement[30].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/2,0,1/4
      SpaceGroupElement[31].x=-y+0.75;       SpaceGroupElement[31].y=-x+0.75;       SpaceGroupElement[31].z=-z+0.75;        // 2        [-1,1,0]        0,0,0           3/4,0,3/8
      SpaceGroupElement[32].x=z+0.5;         SpaceGroupElement[32].y=x+0.5;         SpaceGroupElement[32].z=y+0.5;          // 3^1      [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[33].x=-x+0.75;       SpaceGroupElement[33].y=z+0.25;        SpaceGroupElement[33].z=y+0.75;         // 2        [0,1,1]         0,1,1           3/8,1/4,0
      SpaceGroupElement[34].x=-z;            SpaceGroupElement[34].y=-x+0.5;        SpaceGroupElement[34].z=y;              // 3^-1     [-1,1,1]        -1/6,1/6,1/6    7/6,-5/6,0
      SpaceGroupElement[35].x=x+0.75;        SpaceGroupElement[35].y=-z+0.75;       SpaceGroupElement[35].z=y+0.25;         // 4^1      [1,0,0]         3/4,0,0         0,-1/4,1
      SpaceGroupElement[36].x=z+0.5;         SpaceGroupElement[36].y=-x+0.5;        SpaceGroupElement[36].z=-y;             // 3^-1     [1,-1,1]        1/3,-1/3,1/3    1/6,2/3,0
      SpaceGroupElement[37].x=x+0.75;        SpaceGroupElement[37].y=z+0.25;        SpaceGroupElement[37].z=-y+0.25;        // 4^-1     [1,0,0]         3/4,0,0         0,5/4,0
      SpaceGroupElement[38].x=-z;            SpaceGroupElement[38].y=x+0.5;         SpaceGroupElement[38].z=-y+0.5;         // 3^1      [-1,-1,1]       1/3,1/3,-1/3    2/3,5/6,0
      SpaceGroupElement[39].x=-x+0.75;       SpaceGroupElement[39].y=-z+0.75;       SpaceGroupElement[39].z=-y+0.75;        // 2        [0,-1,1]        0,0,0           3/8,3/4,0
      SpaceGroupElement[40].x=y+0.5;         SpaceGroupElement[40].y=z+0.5;         SpaceGroupElement[40].z=x+0.5;          // 3^-1     [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[41].x=y;             SpaceGroupElement[41].y=-z;            SpaceGroupElement[41].z=-x+0.5;         // 3^-1     [-1,-1,1]       1/2,1/2,-1/2    1,1/2,0
      SpaceGroupElement[42].x=z+0.25;        SpaceGroupElement[42].y=y+0.75;        SpaceGroupElement[42].z=-x+0.75;        // 4^1      [0,1,0]         0,3/4,0         1,0,-1/4
      SpaceGroupElement[43].x=-y+0.5;        SpaceGroupElement[43].y=z;             SpaceGroupElement[43].z=-x;             // 3^1      [-1,1,1]        -1/2,1/2,1/2    1/2,1/2,0
      SpaceGroupElement[44].x=-z+0.75;       SpaceGroupElement[44].y=-y+0.75;       SpaceGroupElement[44].z=-x+0.75;        // 2        [-1,0,1]        0,0,0           3/4,3/8,0
      SpaceGroupElement[45].x=-y;            SpaceGroupElement[45].y=-z+0.5;        SpaceGroupElement[45].z=x;              // 3^1      [1,-1,1]        1/2,-1/2,1/2    -1/2,1,0
      SpaceGroupElement[46].x=z+0.25;        SpaceGroupElement[46].y=-y+0.25;       SpaceGroupElement[46].z=x+0.75;         // 2        [1,0,1]         1,0,1           1/4,5/8,0
      SpaceGroupElement[47].x=-z+0.25;       SpaceGroupElement[47].y=y+0.75;        SpaceGroupElement[47].z=x+0.25;         // 4^-1     [0,1,0]         0,3/4,0         0,0,5/4
      break;

    case 511: // [215] P -4 3 m (P -4 2 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z;               // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z;               // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=x;              SpaceGroupElement[9].y=-z;             SpaceGroupElement[9].z=-y;              // -2       [0,1,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;              // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=z;             SpaceGroupElement[11].z=-y;             // -4^1     [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[13].x=-x;            SpaceGroupElement[13].y=-z;            SpaceGroupElement[13].z=y;              // -4^-1    [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-y;             // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=z;             SpaceGroupElement[15].z=y;              // -2       [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[18].x=-z;            SpaceGroupElement[18].y=-y;            SpaceGroupElement[18].z=x;              // -4^1     [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[20].x=z;             SpaceGroupElement[20].y=y;             SpaceGroupElement[20].z=x;              // -2       [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[22].x=-z;            SpaceGroupElement[22].y=y;             SpaceGroupElement[22].z=-x;             // -2       [1,0,1]         0,0,0           0,0,0
      SpaceGroupElement[23].x=z;             SpaceGroupElement[23].y=-y;            SpaceGroupElement[23].z=-x;             // -4^-1    [0,1,0]         0,0,0           0,0,0
      break;

    case 512: // [216] F -4 3 m (F -4 2 3)
      SpaceGroupSize=96;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z;               // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z;               // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=x;              SpaceGroupElement[9].y=-z;             SpaceGroupElement[9].z=-y;              // -2       [0,1,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;              // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=z;             SpaceGroupElement[11].z=-y;             // -4^1     [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[13].x=-x;            SpaceGroupElement[13].y=-z;            SpaceGroupElement[13].z=y;              // -4^-1    [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-y;             // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=z;             SpaceGroupElement[15].z=y;              // -2       [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[18].x=-z;            SpaceGroupElement[18].y=-y;            SpaceGroupElement[18].z=x;              // -4^1     [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[20].x=z;             SpaceGroupElement[20].y=y;             SpaceGroupElement[20].z=x;              // -2       [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[22].x=-z;            SpaceGroupElement[22].y=y;             SpaceGroupElement[22].z=-x;             // -2       [1,0,1]         0,0,0           0,0,0
      SpaceGroupElement[23].x=z;             SpaceGroupElement[23].y=-y;            SpaceGroupElement[23].z=-x;             // -4^-1    [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[24].x=x;             SpaceGroupElement[24].y=y+0.5;         SpaceGroupElement[24].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[25].x=y;             SpaceGroupElement[25].y=-x+0.5;        SpaceGroupElement[25].z=-z+0.5;         // -4^1     [0,0,1]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[26].x=-x;            SpaceGroupElement[26].y=-y+0.5;        SpaceGroupElement[26].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[27].x=-y;            SpaceGroupElement[27].y=x+0.5;         SpaceGroupElement[27].z=-z+0.5;         // -4^-1    [0,0,1]         0,0,0           -1/4,1/4,1/4
      SpaceGroupElement[28].x=x;             SpaceGroupElement[28].y=-y+0.5;        SpaceGroupElement[28].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[29].x=-y;            SpaceGroupElement[29].y=-x+0.5;        SpaceGroupElement[29].z=z+0.5;          // -2       [1,1,0]         -1/4,1/4,1/2    1/4,0,0
      SpaceGroupElement[30].x=-x;            SpaceGroupElement[30].y=y+0.5;         SpaceGroupElement[30].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x+0.5;         SpaceGroupElement[31].z=z+0.5;          // -2       [-1,1,0]        1/4,1/4,1/2     -1/4,0,0
      SpaceGroupElement[32].x=z;             SpaceGroupElement[32].y=x+0.5;         SpaceGroupElement[32].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     -1/3,-1/6,0
      SpaceGroupElement[33].x=x;             SpaceGroupElement[33].y=-z+0.5;        SpaceGroupElement[33].z=-y+0.5;         // -2       [0,1,1]         0,0,0           0,1/2,0
      SpaceGroupElement[34].x=-z;            SpaceGroupElement[34].y=-x+0.5;        SpaceGroupElement[34].z=y+0.5;          // 3^-1     [-1,1,1]        -1/3,1/3,1/3    1/3,-1/6,0
      SpaceGroupElement[35].x=-x;            SpaceGroupElement[35].y=z+0.5;         SpaceGroupElement[35].z=-y+0.5;         // -4^1     [1,0,0]         0,0,0           0,1/2,0
      SpaceGroupElement[36].x=z;             SpaceGroupElement[36].y=-x+0.5;        SpaceGroupElement[36].z=-y+0.5;         // 3^-1     [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[37].x=-x;            SpaceGroupElement[37].y=-z+0.5;        SpaceGroupElement[37].z=y+0.5;          // -4^-1    [1,0,0]         0,0,0           0,0,1/2
      SpaceGroupElement[38].x=-z;            SpaceGroupElement[38].y=x+0.5;         SpaceGroupElement[38].z=-y+0.5;         // 3^1      [-1,-1,1]       0,0,0           0,1/2,0
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=z+0.5;         SpaceGroupElement[39].z=y+0.5;          // -2       [0,-1,1]        0,1/2,1/2       0,0,0
      SpaceGroupElement[40].x=y;             SpaceGroupElement[40].y=z+0.5;         SpaceGroupElement[40].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,1/6,0
      SpaceGroupElement[41].x=y;             SpaceGroupElement[41].y=-z+0.5;        SpaceGroupElement[41].z=-x+0.5;         // 3^-1     [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[42].x=-z;            SpaceGroupElement[42].y=-y+0.5;        SpaceGroupElement[42].z=x+0.5;          // -4^1     [0,1,0]         0,0,0           -1/4,1/4,1/4
      SpaceGroupElement[43].x=-y;            SpaceGroupElement[43].y=z+0.5;         SpaceGroupElement[43].z=-x+0.5;         // 3^1      [-1,1,1]        -1/3,1/3,1/3    1/6,1/6,0
      SpaceGroupElement[44].x=z;             SpaceGroupElement[44].y=y+0.5;         SpaceGroupElement[44].z=x+0.5;          // -2       [-1,0,1]        1/4,1/2,1/4     -1/4,0,0
      SpaceGroupElement[45].x=-y;            SpaceGroupElement[45].y=-z+0.5;        SpaceGroupElement[45].z=x+0.5;          // 3^1      [1,-1,1]        0,0,0           -1/2,1/2,0
      SpaceGroupElement[46].x=-z;            SpaceGroupElement[46].y=y+0.5;         SpaceGroupElement[46].z=-x+0.5;         // -2       [1,0,1]         -1/4,1/2,1/4    1/4,0,0
      SpaceGroupElement[47].x=z;             SpaceGroupElement[47].y=-y+0.5;        SpaceGroupElement[47].z=-x+0.5;         // -4^-1    [0,1,0]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[48].x=x+0.5;         SpaceGroupElement[48].y=y;             SpaceGroupElement[48].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[49].x=y+0.5;         SpaceGroupElement[49].y=-x;            SpaceGroupElement[49].z=-z+0.5;         // -4^1     [0,0,1]         0,0,0           1/4,-1/4,1/4
      SpaceGroupElement[50].x=-x+0.5;        SpaceGroupElement[50].y=-y;            SpaceGroupElement[50].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[51].x=-y+0.5;        SpaceGroupElement[51].y=x;             SpaceGroupElement[51].z=-z+0.5;         // -4^-1    [0,0,1]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[52].x=x+0.5;         SpaceGroupElement[52].y=-y;            SpaceGroupElement[52].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[53].x=-y+0.5;        SpaceGroupElement[53].y=-x;            SpaceGroupElement[53].z=z+0.5;          // -2       [1,1,0]         1/4,-1/4,1/2    1/4,0,0
      SpaceGroupElement[54].x=-x+0.5;        SpaceGroupElement[54].y=y;             SpaceGroupElement[54].z=-z+0.5;         // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[55].x=y+0.5;         SpaceGroupElement[55].y=x;             SpaceGroupElement[55].z=z+0.5;          // -2       [-1,1,0]        1/4,1/4,1/2     1/4,0,0
      SpaceGroupElement[56].x=z+0.5;         SpaceGroupElement[56].y=x;             SpaceGroupElement[56].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,-1/6,0
      SpaceGroupElement[57].x=x+0.5;         SpaceGroupElement[57].y=-z;            SpaceGroupElement[57].z=-y+0.5;         // -2       [0,1,1]         1/2,-1/4,1/4    0,1/4,0
      SpaceGroupElement[58].x=-z+0.5;        SpaceGroupElement[58].y=-x;            SpaceGroupElement[58].z=y+0.5;          // 3^-1     [-1,1,1]        0,0,0           1/2,-1/2,0
      SpaceGroupElement[59].x=-x+0.5;        SpaceGroupElement[59].y=z;             SpaceGroupElement[59].z=-y+0.5;         // -4^1     [1,0,0]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[60].x=z+0.5;         SpaceGroupElement[60].y=-x;            SpaceGroupElement[60].z=-y+0.5;         // 3^-1     [1,-1,1]        1/3,-1/3,1/3    1/6,1/6,0
      SpaceGroupElement[61].x=-x+0.5;        SpaceGroupElement[61].y=-z;            SpaceGroupElement[61].z=y+0.5;          // -4^-1    [1,0,0]         0,0,0           1/4,-1/4,1/4
      SpaceGroupElement[62].x=-z+0.5;        SpaceGroupElement[62].y=x;             SpaceGroupElement[62].z=-y+0.5;         // 3^1      [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[63].x=x+0.5;         SpaceGroupElement[63].y=z;             SpaceGroupElement[63].z=y+0.5;          // -2       [0,-1,1]        1/2,1/4,1/4     0,-1/4,0
      SpaceGroupElement[64].x=y+0.5;         SpaceGroupElement[64].y=z;             SpaceGroupElement[64].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,-1/3,0
      SpaceGroupElement[65].x=y+0.5;         SpaceGroupElement[65].y=-z;            SpaceGroupElement[65].z=-x+0.5;         // 3^-1     [-1,-1,1]       0,0,0           1/2,0,0
      SpaceGroupElement[66].x=-z+0.5;        SpaceGroupElement[66].y=-y;            SpaceGroupElement[66].z=x+0.5;          // -4^1     [0,1,0]         0,0,0           0,0,1/2
      SpaceGroupElement[67].x=-y+0.5;        SpaceGroupElement[67].y=z;             SpaceGroupElement[67].z=-x+0.5;         // 3^1      [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[68].x=z+0.5;         SpaceGroupElement[68].y=y;             SpaceGroupElement[68].z=x+0.5;          // -2       [-1,0,1]        1/2,0,1/2       0,0,0
      SpaceGroupElement[69].x=-y+0.5;        SpaceGroupElement[69].y=-z;            SpaceGroupElement[69].z=x+0.5;          // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      SpaceGroupElement[70].x=-z+0.5;        SpaceGroupElement[70].y=y;             SpaceGroupElement[70].z=-x+0.5;         // -2       [1,0,1]         0,0,0           1/2,0,0
      SpaceGroupElement[71].x=z+0.5;         SpaceGroupElement[71].y=-y;            SpaceGroupElement[71].z=-x+0.5;         // -4^-1    [0,1,0]         0,0,0           1/2,0,0
      SpaceGroupElement[72].x=x+0.5;         SpaceGroupElement[72].y=y+0.5;         SpaceGroupElement[72].z=z;              // 1        -               -               -
      SpaceGroupElement[73].x=y+0.5;         SpaceGroupElement[73].y=-x+0.5;        SpaceGroupElement[73].z=-z;             // -4^1     [0,0,1]         0,0,0           1/2,0,0
      SpaceGroupElement[74].x=-x+0.5;        SpaceGroupElement[74].y=-y+0.5;        SpaceGroupElement[74].z=z;              // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[75].x=-y+0.5;        SpaceGroupElement[75].y=x+0.5;         SpaceGroupElement[75].z=-z;             // -4^-1    [0,0,1]         0,0,0           0,1/2,0
      SpaceGroupElement[76].x=x+0.5;         SpaceGroupElement[76].y=-y+0.5;        SpaceGroupElement[76].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[77].x=-y+0.5;        SpaceGroupElement[77].y=-x+0.5;        SpaceGroupElement[77].z=z;              // -2       [1,1,0]         0,0,0           1/2,0,0
      SpaceGroupElement[78].x=-x+0.5;        SpaceGroupElement[78].y=y+0.5;         SpaceGroupElement[78].z=-z;             // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[79].x=y+0.5;         SpaceGroupElement[79].y=x+0.5;         SpaceGroupElement[79].z=z;              // -2       [-1,1,0]        1/2,1/2,0       0,0,0
      SpaceGroupElement[80].x=z+0.5;         SpaceGroupElement[80].y=x+0.5;         SpaceGroupElement[80].z=y;              // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,1/3,0
      SpaceGroupElement[81].x=x+0.5;         SpaceGroupElement[81].y=-z+0.5;        SpaceGroupElement[81].z=-y;             // -2       [0,1,1]         1/2,1/4,-1/4    0,1/4,0
      SpaceGroupElement[82].x=-z+0.5;        SpaceGroupElement[82].y=-x+0.5;        SpaceGroupElement[82].z=y;              // 3^-1     [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[83].x=-x+0.5;        SpaceGroupElement[83].y=z+0.5;         SpaceGroupElement[83].z=-y;             // -4^1     [1,0,0]         0,0,0           1/4,1/4,-1/4
      SpaceGroupElement[84].x=z+0.5;         SpaceGroupElement[84].y=-x+0.5;        SpaceGroupElement[84].z=-y;             // 3^-1     [1,-1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[85].x=-x+0.5;        SpaceGroupElement[85].y=-z+0.5;        SpaceGroupElement[85].z=y;              // -4^-1    [1,0,0]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[86].x=-z+0.5;        SpaceGroupElement[86].y=x+0.5;         SpaceGroupElement[86].z=-y;             // 3^1      [-1,-1,1]       1/3,1/3,-1/3    1/6,1/3,0
      SpaceGroupElement[87].x=x+0.5;         SpaceGroupElement[87].y=z+0.5;         SpaceGroupElement[87].z=y;              // -2       [0,-1,1]        1/2,1/4,1/4     0,1/4,0
      SpaceGroupElement[88].x=y+0.5;         SpaceGroupElement[88].y=z+0.5;         SpaceGroupElement[88].z=x;              // 3^-1     [1,1,1]         1/3,1/3,1/3     1/3,1/6,0
      SpaceGroupElement[89].x=y+0.5;         SpaceGroupElement[89].y=-z+0.5;        SpaceGroupElement[89].z=-x;             // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    1/3,1/6,0
      SpaceGroupElement[90].x=-z+0.5;        SpaceGroupElement[90].y=-y+0.5;        SpaceGroupElement[90].z=x;              // -4^1     [0,1,0]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[91].x=-y+0.5;        SpaceGroupElement[91].y=z+0.5;         SpaceGroupElement[91].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[92].x=z+0.5;         SpaceGroupElement[92].y=y+0.5;         SpaceGroupElement[92].z=x;              // -2       [-1,0,1]        1/4,1/2,1/4     1/4,0,0
      SpaceGroupElement[93].x=-y+0.5;        SpaceGroupElement[93].y=-z+0.5;        SpaceGroupElement[93].z=x;              // 3^1      [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[94].x=-z+0.5;        SpaceGroupElement[94].y=y+0.5;         SpaceGroupElement[94].z=-x;             // -2       [1,0,1]         1/4,1/2,-1/4    1/4,0,0
      SpaceGroupElement[95].x=z+0.5;         SpaceGroupElement[95].y=-y+0.5;        SpaceGroupElement[95].z=-x;             // -4^-1    [0,1,0]         0,0,0           1/4,1/4,-1/4
      break;

    case 513: // [217] I -4 3 m (I -4 2 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;              // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z;               // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z;               // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=x;              SpaceGroupElement[9].y=-z;             SpaceGroupElement[9].z=-y;              // -2       [0,1,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;              // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=z;             SpaceGroupElement[11].z=-y;             // -4^1     [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[13].x=-x;            SpaceGroupElement[13].y=-z;            SpaceGroupElement[13].z=y;              // -4^-1    [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-y;             // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=z;             SpaceGroupElement[15].z=y;              // -2       [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[18].x=-z;            SpaceGroupElement[18].y=-y;            SpaceGroupElement[18].z=x;              // -4^1     [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[20].x=z;             SpaceGroupElement[20].y=y;             SpaceGroupElement[20].z=x;              // -2       [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[22].x=-z;            SpaceGroupElement[22].y=y;             SpaceGroupElement[22].z=-x;             // -2       [1,0,1]         0,0,0           0,0,0
      SpaceGroupElement[23].x=z;             SpaceGroupElement[23].y=-y;            SpaceGroupElement[23].z=-x;             // -4^-1    [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[24].x=x+0.5;         SpaceGroupElement[24].y=y+0.5;         SpaceGroupElement[24].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[25].x=y+0.5;         SpaceGroupElement[25].y=-x+0.5;        SpaceGroupElement[25].z=-z+0.5;         // -4^1     [0,0,1]         0,0,0           1/2,0,1/4
      SpaceGroupElement[26].x=-x+0.5;        SpaceGroupElement[26].y=-y+0.5;        SpaceGroupElement[26].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[27].x=-y+0.5;        SpaceGroupElement[27].y=x+0.5;         SpaceGroupElement[27].z=-z+0.5;         // -4^-1    [0,0,1]         0,0,0           0,1/2,1/4
      SpaceGroupElement[28].x=x+0.5;         SpaceGroupElement[28].y=-y+0.5;        SpaceGroupElement[28].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[29].x=-y+0.5;        SpaceGroupElement[29].y=-x+0.5;        SpaceGroupElement[29].z=z+0.5;          // -2       [1,1,0]         0,0,1/2         1/2,0,0
      SpaceGroupElement[30].x=-x+0.5;        SpaceGroupElement[30].y=y+0.5;         SpaceGroupElement[30].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[31].x=y+0.5;         SpaceGroupElement[31].y=x+0.5;         SpaceGroupElement[31].z=z+0.5;          // -2       [-1,1,0]        1/2,1/2,1/2     0,0,0
      SpaceGroupElement[32].x=z+0.5;         SpaceGroupElement[32].y=x+0.5;         SpaceGroupElement[32].z=y+0.5;          // 3^1      [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[33].x=x+0.5;         SpaceGroupElement[33].y=-z+0.5;        SpaceGroupElement[33].z=-y+0.5;         // -2       [0,1,1]         1/2,0,0         0,1/2,0
      SpaceGroupElement[34].x=-z+0.5;        SpaceGroupElement[34].y=-x+0.5;        SpaceGroupElement[34].z=y+0.5;          // 3^-1     [-1,1,1]        -1/6,1/6,1/6    2/3,-1/3,0
      SpaceGroupElement[35].x=-x+0.5;        SpaceGroupElement[35].y=z+0.5;         SpaceGroupElement[35].z=-y+0.5;         // -4^1     [1,0,0]         0,0,0           1/4,1/2,0
      SpaceGroupElement[36].x=z+0.5;         SpaceGroupElement[36].y=-x+0.5;        SpaceGroupElement[36].z=-y+0.5;         // 3^-1     [1,-1,1]        1/6,-1/6,1/6    1/3,1/3,0
      SpaceGroupElement[37].x=-x+0.5;        SpaceGroupElement[37].y=-z+0.5;        SpaceGroupElement[37].z=y+0.5;          // -4^-1    [1,0,0]         0,0,0           1/4,0,1/2
      SpaceGroupElement[38].x=-z+0.5;        SpaceGroupElement[38].y=x+0.5;         SpaceGroupElement[38].z=-y+0.5;         // 3^1      [-1,-1,1]       1/6,1/6,-1/6    1/3,2/3,0
      SpaceGroupElement[39].x=x+0.5;         SpaceGroupElement[39].y=z+0.5;         SpaceGroupElement[39].z=y+0.5;          // -2       [0,-1,1]        1/2,1/2,1/2     0,0,0
      SpaceGroupElement[40].x=y+0.5;         SpaceGroupElement[40].y=z+0.5;         SpaceGroupElement[40].z=x+0.5;          // 3^-1     [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[41].x=y+0.5;         SpaceGroupElement[41].y=-z+0.5;        SpaceGroupElement[41].z=-x+0.5;         // 3^-1     [-1,-1,1]       1/6,1/6,-1/6    2/3,1/3,0
      SpaceGroupElement[42].x=-z+0.5;        SpaceGroupElement[42].y=-y+0.5;        SpaceGroupElement[42].z=x+0.5;          // -4^1     [0,1,0]         0,0,0           0,1/4,1/2
      SpaceGroupElement[43].x=-y+0.5;        SpaceGroupElement[43].y=z+0.5;         SpaceGroupElement[43].z=-x+0.5;         // 3^1      [-1,1,1]        -1/6,1/6,1/6    1/3,1/3,0
      SpaceGroupElement[44].x=z+0.5;         SpaceGroupElement[44].y=y+0.5;         SpaceGroupElement[44].z=x+0.5;          // -2       [-1,0,1]        1/2,1/2,1/2     0,0,0
      SpaceGroupElement[45].x=-y+0.5;        SpaceGroupElement[45].y=-z+0.5;        SpaceGroupElement[45].z=x+0.5;          // 3^1      [1,-1,1]        1/6,-1/6,1/6    -1/3,2/3,0
      SpaceGroupElement[46].x=-z+0.5;        SpaceGroupElement[46].y=y+0.5;         SpaceGroupElement[46].z=-x+0.5;         // -2       [1,0,1]         0,1/2,0         1/2,0,0
      SpaceGroupElement[47].x=z+0.5;         SpaceGroupElement[47].y=-y+0.5;        SpaceGroupElement[47].z=-x+0.5;         // -4^-1    [0,1,0]         0,0,0           1/2,1/4,0
      break;

    case 514: // [218] P -4 3 n (P -4n 2 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y+0.5;          SpaceGroupElement[1].y=-x+0.5;         SpaceGroupElement[1].z=-z+0.5;          // -4^1     [0,0,1]         0,0,0           1/2,0,1/4
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=-y+0.5;         SpaceGroupElement[3].y=x+0.5;          SpaceGroupElement[3].z=-z+0.5;          // -4^-1    [0,0,1]         0,0,0           0,1/2,1/4
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-y+0.5;         SpaceGroupElement[5].y=-x+0.5;         SpaceGroupElement[5].z=z+0.5;           // -2       [1,1,0]         0,0,1/2         1/2,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=y+0.5;          SpaceGroupElement[7].y=x+0.5;          SpaceGroupElement[7].z=z+0.5;           // -2       [-1,1,0]        1/2,1/2,1/2     0,0,0
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=x+0.5;          SpaceGroupElement[9].y=-z+0.5;         SpaceGroupElement[9].z=-y+0.5;          // -2       [0,1,1]         1/2,0,0         0,1/2,0
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;              // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=-x+0.5;        SpaceGroupElement[11].y=z+0.5;         SpaceGroupElement[11].z=-y+0.5;         // -4^1     [1,0,0]         0,0,0           1/4,1/2,0
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[13].x=-x+0.5;        SpaceGroupElement[13].y=-z+0.5;        SpaceGroupElement[13].z=y+0.5;          // -4^-1    [1,0,0]         0,0,0           1/4,0,1/2
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-y;             // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[15].x=x+0.5;         SpaceGroupElement[15].y=z+0.5;         SpaceGroupElement[15].z=y+0.5;          // -2       [0,-1,1]        1/2,1/2,1/2     0,0,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[18].x=-z+0.5;        SpaceGroupElement[18].y=-y+0.5;        SpaceGroupElement[18].z=x+0.5;          // -4^1     [0,1,0]         0,0,0           0,1/4,1/2
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[20].x=z+0.5;         SpaceGroupElement[20].y=y+0.5;         SpaceGroupElement[20].z=x+0.5;          // -2       [-1,0,1]        1/2,1/2,1/2     0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[22].x=-z+0.5;        SpaceGroupElement[22].y=y+0.5;         SpaceGroupElement[22].z=-x+0.5;         // -2       [1,0,1]         0,1/2,0         1/2,0,0
      SpaceGroupElement[23].x=z+0.5;         SpaceGroupElement[23].y=-y+0.5;        SpaceGroupElement[23].z=-x+0.5;         // -4^-1    [0,1,0]         0,0,0           1/2,1/4,0
      break;

    case 515: // [219] F -4 3 c (F -4a 2 3)
      SpaceGroupSize=96;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y+0.5;          SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-z;              // -4^1     [0,0,1]         0,0,0           1/4,-1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x+0.5;          SpaceGroupElement[3].z=-z;              // -4^-1    [0,0,1]         0,0,0           -1/4,1/4,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=-y+0.5;         SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z;               // -2       [1,1,0]         1/4,-1/4,0      1/4,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x+0.5;          SpaceGroupElement[7].z=z;               // -2       [-1,1,0]        1/4,1/4,0       -1/4,0,0
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=x+0.5;          SpaceGroupElement[9].y=-z;             SpaceGroupElement[9].z=-y;              // -2       [0,1,1]         1/2,0,0         0,0,0
      SpaceGroupElement[10].x=-z+0.5;        SpaceGroupElement[10].y=-x+0.5;        SpaceGroupElement[10].z=y;              // 3^-1     [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=z+0.5;         SpaceGroupElement[11].z=-y;             // -4^1     [1,0,0]         0,0,0           0,1/4,-1/4
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[13].x=-x+0.5;        SpaceGroupElement[13].y=-z;            SpaceGroupElement[13].z=y;              // -4^-1    [1,0,0]         0,0,0           1/4,0,0
      SpaceGroupElement[14].x=-z+0.5;        SpaceGroupElement[14].y=x+0.5;         SpaceGroupElement[14].z=-y;             // 3^1      [-1,-1,1]       1/3,1/3,-1/3    1/6,1/3,0
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=z+0.5;         SpaceGroupElement[15].z=y;              // -2       [0,-1,1]        0,1/4,1/4       0,1/4,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z+0.5;        SpaceGroupElement[17].z=-x+0.5;         // 3^-1     [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[18].x=-z;            SpaceGroupElement[18].y=-y;            SpaceGroupElement[18].z=x+0.5;          // -4^1     [0,1,0]         0,0,0           -1/4,0,1/4
      SpaceGroupElement[19].x=-y+0.5;        SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=-x+0.5;         // 3^1      [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[20].x=z+0.5;         SpaceGroupElement[20].y=y;             SpaceGroupElement[20].z=x;              // -2       [-1,0,1]        1/4,0,1/4       1/4,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[22].x=-z;            SpaceGroupElement[22].y=y;             SpaceGroupElement[22].z=-x+0.5;         // -2       [1,0,1]         -1/4,0,1/4      1/4,0,0
      SpaceGroupElement[23].x=z+0.5;         SpaceGroupElement[23].y=-y+0.5;        SpaceGroupElement[23].z=-x+0.5;         // -4^-1    [0,1,0]         0,0,0           1/2,1/4,0
      SpaceGroupElement[24].x=x;             SpaceGroupElement[24].y=y+0.5;         SpaceGroupElement[24].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[25].x=y+0.5;         SpaceGroupElement[25].y=-x+0.5;        SpaceGroupElement[25].z=-z+0.5;         // -4^1     [0,0,1]         0,0,0           1/2,0,1/4
      SpaceGroupElement[26].x=-x+0.5;        SpaceGroupElement[26].y=-y;            SpaceGroupElement[26].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/2,0
      SpaceGroupElement[27].x=-y;            SpaceGroupElement[27].y=x;             SpaceGroupElement[27].z=-z+0.5;         // -4^-1    [0,0,1]         0,0,0           -1/2,1/2,1/4
      SpaceGroupElement[28].x=x;             SpaceGroupElement[28].y=-y+0.5;        SpaceGroupElement[28].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[29].x=-y+0.5;        SpaceGroupElement[29].y=-x+0.5;        SpaceGroupElement[29].z=z+0.5;          // -2       [1,1,0]         0,0,1/2         1/2,0,0
      SpaceGroupElement[30].x=-x+0.5;        SpaceGroupElement[30].y=y;             SpaceGroupElement[30].z=-z+0.5;         // 2        [0,1,0]         0,1,0           1/4,0,1/4
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=z+0.5;          // -2       [-1,1,0]        1/2,1/2,1/2     -1/2,0,0
      SpaceGroupElement[32].x=z;             SpaceGroupElement[32].y=x+0.5;         SpaceGroupElement[32].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     -1/3,-1/6,0
      SpaceGroupElement[33].x=x+0.5;         SpaceGroupElement[33].y=-z+0.5;        SpaceGroupElement[33].z=-y+0.5;         // -2       [0,1,1]         1/2,0,0         0,1/2,0
      SpaceGroupElement[34].x=-z+0.5;        SpaceGroupElement[34].y=-x;            SpaceGroupElement[34].z=y+0.5;          // 3^-1     [-1,1,1]        -1/3,1/3,1/3    5/6,-1/6,0
      SpaceGroupElement[35].x=-x;            SpaceGroupElement[35].y=z;             SpaceGroupElement[35].z=-y+0.5;         // -4^1     [1,0,0]         0,0,0           0,3/4,-1/4
      SpaceGroupElement[36].x=z;             SpaceGroupElement[36].y=-x+0.5;        SpaceGroupElement[36].z=-y+0.5;         // 3^-1     [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[37].x=-x+0.5;        SpaceGroupElement[37].y=-z+0.5;        SpaceGroupElement[37].z=y+0.5;          // -4^-1    [1,0,0]         0,0,0           1/4,0,1/2
      SpaceGroupElement[38].x=-z+0.5;        SpaceGroupElement[38].y=x;             SpaceGroupElement[38].z=-y+0.5;         // 3^1      [-1,-1,1]       1/3,1/3,-1/3    1/6,5/6,0
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=z;             SpaceGroupElement[39].z=y+0.5;          // -2       [0,-1,1]        0,3/4,3/4       0,1/4,0
      SpaceGroupElement[40].x=y;             SpaceGroupElement[40].y=z+0.5;         SpaceGroupElement[40].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,1/6,0
      SpaceGroupElement[41].x=y;             SpaceGroupElement[41].y=-z;            SpaceGroupElement[41].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           1,1,0
      SpaceGroupElement[42].x=-z;            SpaceGroupElement[42].y=-y+0.5;        SpaceGroupElement[42].z=x;              // -4^1     [0,1,0]         0,0,0           -1/2,1/4,1/2
      SpaceGroupElement[43].x=-y+0.5;        SpaceGroupElement[43].y=z+0.5;         SpaceGroupElement[43].z=-x;             // 3^1      [-1,1,1]        -1/3,1/3,1/3    2/3,1/6,0
      SpaceGroupElement[44].x=z+0.5;         SpaceGroupElement[44].y=y+0.5;         SpaceGroupElement[44].z=x+0.5;          // -2       [-1,0,1]        1/2,1/2,1/2     0,0,0
      SpaceGroupElement[45].x=-y;            SpaceGroupElement[45].y=-z+0.5;        SpaceGroupElement[45].z=x+0.5;          // 3^1      [1,-1,1]        0,0,0           -1/2,1/2,0
      SpaceGroupElement[46].x=-z;            SpaceGroupElement[46].y=y+0.5;         SpaceGroupElement[46].z=-x;             // -2       [1,0,1]         -1/2,1/2,1/2    1/2,0,0
      SpaceGroupElement[47].x=z+0.5;         SpaceGroupElement[47].y=-y;            SpaceGroupElement[47].z=-x;             // -4^-1    [0,1,0]         0,0,0           3/4,1/2,1/4
      SpaceGroupElement[48].x=x+0.5;         SpaceGroupElement[48].y=y;             SpaceGroupElement[48].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[49].x=y;             SpaceGroupElement[49].y=-x;            SpaceGroupElement[49].z=-z+0.5;         // -4^1     [0,0,1]         0,0,0           1/2,-1/2,1/4
      SpaceGroupElement[50].x=-x;            SpaceGroupElement[50].y=-y+0.5;        SpaceGroupElement[50].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/2,1/4,0
      SpaceGroupElement[51].x=-y+0.5;        SpaceGroupElement[51].y=x+0.5;         SpaceGroupElement[51].z=-z+0.5;         // -4^-1    [0,0,1]         0,0,0           0,1/2,1/4
      SpaceGroupElement[52].x=x+0.5;         SpaceGroupElement[52].y=-y;            SpaceGroupElement[52].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[53].x=-y;            SpaceGroupElement[53].y=-x;            SpaceGroupElement[53].z=z+0.5;          // -2       [1,1,0]         1/2,-1/2,1/2    1/2,0,0
      SpaceGroupElement[54].x=-x;            SpaceGroupElement[54].y=y+0.5;         SpaceGroupElement[54].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/2,0,1/4
      SpaceGroupElement[55].x=y+0.5;         SpaceGroupElement[55].y=x+0.5;         SpaceGroupElement[55].z=z+0.5;          // -2       [-1,1,0]        1/2,1/2,1/2     0,0,0
      SpaceGroupElement[56].x=z+0.5;         SpaceGroupElement[56].y=x;             SpaceGroupElement[56].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,-1/6,0
      SpaceGroupElement[57].x=x;             SpaceGroupElement[57].y=-z;            SpaceGroupElement[57].z=-y+0.5;         // -2       [0,1,1]         1,-1/4,1/4      0,1/4,0
      SpaceGroupElement[58].x=-z;            SpaceGroupElement[58].y=-x+0.5;        SpaceGroupElement[58].z=y+0.5;          // 3^-1     [-1,1,1]        0,0,0           1,-1/2,0
      SpaceGroupElement[59].x=-x+0.5;        SpaceGroupElement[59].y=z+0.5;         SpaceGroupElement[59].z=-y+0.5;         // -4^1     [1,0,0]         0,0,0           1/4,1/2,0
      SpaceGroupElement[60].x=z+0.5;         SpaceGroupElement[60].y=-x;            SpaceGroupElement[60].z=-y+0.5;         // 3^-1     [1,-1,1]        1/3,-1/3,1/3    1/6,1/6,0
      SpaceGroupElement[61].x=-x;            SpaceGroupElement[61].y=-z;            SpaceGroupElement[61].z=y+0.5;          // -4^-1    [1,0,0]         0,0,0           1/2,-1/4,1/4
      SpaceGroupElement[62].x=-z;            SpaceGroupElement[62].y=x+0.5;         SpaceGroupElement[62].z=-y+0.5;         // 3^1      [-1,-1,1]       1/3,1/3,-1/3    2/3,5/6,0
      SpaceGroupElement[63].x=x+0.5;         SpaceGroupElement[63].y=z+0.5;         SpaceGroupElement[63].z=y+0.5;          // -2       [0,-1,1]        1/2,1/2,1/2     0,0,0
      SpaceGroupElement[64].x=y+0.5;         SpaceGroupElement[64].y=z;             SpaceGroupElement[64].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,-1/3,0
      SpaceGroupElement[65].x=y+0.5;         SpaceGroupElement[65].y=-z+0.5;        SpaceGroupElement[65].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           1,1/2,0
      SpaceGroupElement[66].x=-z+0.5;        SpaceGroupElement[66].y=-y;            SpaceGroupElement[66].z=x;              // -4^1     [0,1,0]         0,0,0           -1/4,0,3/4
      SpaceGroupElement[67].x=-y;            SpaceGroupElement[67].y=z;             SpaceGroupElement[67].z=-x;             // 3^1      [-1,1,1]        0,0,0           1,0,0
      SpaceGroupElement[68].x=z;             SpaceGroupElement[68].y=y;             SpaceGroupElement[68].z=x+0.5;          // -2       [-1,0,1]        3/4,0,3/4       1/4,0,0
      SpaceGroupElement[69].x=-y+0.5;        SpaceGroupElement[69].y=-z;            SpaceGroupElement[69].z=x+0.5;          // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      SpaceGroupElement[70].x=-z+0.5;        SpaceGroupElement[70].y=y;             SpaceGroupElement[70].z=-x;             // -2       [1,0,1]         -1/4,0,1/4      3/4,0,0
      SpaceGroupElement[71].x=z;             SpaceGroupElement[71].y=-y+0.5;        SpaceGroupElement[71].z=-x;             // -4^-1    [0,1,0]         0,0,0           1,1/4,0
      SpaceGroupElement[72].x=x+0.5;         SpaceGroupElement[72].y=y+0.5;         SpaceGroupElement[72].z=z;              // 1        -               -               -
      SpaceGroupElement[73].x=y;             SpaceGroupElement[73].y=-x+0.5;        SpaceGroupElement[73].z=-z;             // -4^1     [0,0,1]         0,0,0           3/4,-1/4,0
      SpaceGroupElement[74].x=-x;            SpaceGroupElement[74].y=-y;            SpaceGroupElement[74].z=z;              // 2        [0,0,1]         0,0,0           1/2,1/2,0
      SpaceGroupElement[75].x=-y+0.5;        SpaceGroupElement[75].y=x;             SpaceGroupElement[75].z=-z;             // -4^-1    [0,0,1]         0,0,0           -1/4,3/4,0
      SpaceGroupElement[76].x=x+0.5;         SpaceGroupElement[76].y=-y+0.5;        SpaceGroupElement[76].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[77].x=-y;            SpaceGroupElement[77].y=-x+0.5;        SpaceGroupElement[77].z=z;              // -2       [1,1,0]         1/4,-1/4,0      3/4,0,0
      SpaceGroupElement[78].x=-x;            SpaceGroupElement[78].y=y;             SpaceGroupElement[78].z=-z;             // 2        [0,1,0]         0,1,0           1/2,0,0
      SpaceGroupElement[79].x=y+0.5;         SpaceGroupElement[79].y=x;             SpaceGroupElement[79].z=z;              // -2       [-1,1,0]        3/4,3/4,0       -1/4,0,0
      SpaceGroupElement[80].x=z+0.5;         SpaceGroupElement[80].y=x+0.5;         SpaceGroupElement[80].z=y;              // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,1/3,0
      SpaceGroupElement[81].x=x;             SpaceGroupElement[81].y=-z+0.5;        SpaceGroupElement[81].z=-y;             // -2       [0,1,1]         1,1/4,-1/4      0,1/4,0
      SpaceGroupElement[82].x=-z;            SpaceGroupElement[82].y=-x;            SpaceGroupElement[82].z=y;              // 3^-1     [-1,1,1]        0,0,0           1,0,0
      SpaceGroupElement[83].x=-x+0.5;        SpaceGroupElement[83].y=z;             SpaceGroupElement[83].z=-y;             // -4^1     [1,0,0]         0,0,0           1/4,1/2,-1/2
      SpaceGroupElement[84].x=z+0.5;         SpaceGroupElement[84].y=-x+0.5;        SpaceGroupElement[84].z=-y;             // 3^-1     [1,-1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[85].x=-x;            SpaceGroupElement[85].y=-z+0.5;        SpaceGroupElement[85].z=y;              // -4^-1    [1,0,0]         0,0,0           1/2,1/4,1/4
      SpaceGroupElement[86].x=-z;            SpaceGroupElement[86].y=x;             SpaceGroupElement[86].z=-y;             // 3^1      [-1,-1,1]       2/3,2/3,-2/3    1/3,2/3,0
      SpaceGroupElement[87].x=x+0.5;         SpaceGroupElement[87].y=z;             SpaceGroupElement[87].z=y;              // -2       [0,-1,1]        1/2,1/2,1/2     0,1/2,0
      SpaceGroupElement[88].x=y+0.5;         SpaceGroupElement[88].y=z+0.5;         SpaceGroupElement[88].z=x;              // 3^-1     [1,1,1]         1/3,1/3,1/3     1/3,1/6,0
      SpaceGroupElement[89].x=y+0.5;         SpaceGroupElement[89].y=-z;            SpaceGroupElement[89].z=-x+0.5;         // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    5/6,2/3,0
      SpaceGroupElement[90].x=-z+0.5;        SpaceGroupElement[90].y=-y+0.5;        SpaceGroupElement[90].z=x+0.5;          // -4^1     [0,1,0]         0,0,0           0,1/4,1/2
      SpaceGroupElement[91].x=-y;            SpaceGroupElement[91].y=z+0.5;         SpaceGroupElement[91].z=-x+0.5;         // 3^1      [-1,1,1]        0,0,0           1/2,1/2,0
      SpaceGroupElement[92].x=z;             SpaceGroupElement[92].y=y+0.5;         SpaceGroupElement[92].z=x;              // -2       [-1,0,1]        1/2,1/2,1/2     1/2,0,0
      SpaceGroupElement[93].x=-y+0.5;        SpaceGroupElement[93].y=-z+0.5;        SpaceGroupElement[93].z=x;              // 3^1      [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[94].x=-z+0.5;        SpaceGroupElement[94].y=y+0.5;         SpaceGroupElement[94].z=-x+0.5;         // -2       [1,0,1]         0,1/2,0         1/2,0,0
      SpaceGroupElement[95].x=z;             SpaceGroupElement[95].y=-y;            SpaceGroupElement[95].z=-x+0.5;         // -4^-1    [0,1,0]         0,0,0           3/4,1/2,-1/4
      break;

    case 516: // [220] I -4 3 d (I -4bd 2c 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=y+0.25;         SpaceGroupElement[1].y=-x+0.75;        SpaceGroupElement[1].z=-z+0.25;         // -4^1     [0,0,1]         0,0,0           1/2,1/4,1/8
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[3].x=-y+0.75;        SpaceGroupElement[3].y=x+0.75;         SpaceGroupElement[3].z=-z+0.25;         // -4^-1    [0,0,1]         0,0,0           0,3/4,1/8
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[5].x=-y+0.25;        SpaceGroupElement[5].y=-x+0.75;        SpaceGroupElement[5].z=z+0.75;          // -2       [1,1,0]         -1/4,1/4,3/4    1/2,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[7].x=y+0.75;         SpaceGroupElement[7].y=x+0.75;         SpaceGroupElement[7].z=z+0.75;          // -2       [-1,1,0]        3/4,3/4,3/4     0,0,0
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=x+0.25;         SpaceGroupElement[9].y=-z+0.75;        SpaceGroupElement[9].z=-y+0.25;         // -2       [0,1,1]         1/4,1/4,-1/4    0,1/2,0
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x+0.5;        SpaceGroupElement[10].z=y;              // 3^-1     [-1,1,1]        -1/6,1/6,1/6    1/6,1/6,0
      SpaceGroupElement[11].x=-x+0.75;       SpaceGroupElement[11].y=z+0.75;        SpaceGroupElement[11].z=-y+0.25;        // -4^1     [1,0,0]         0,0,0           3/8,1/2,-1/4
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y+0.5;         // 3^-1     [1,-1,1]        1/6,-1/6,1/6    -1/6,1/3,0
      SpaceGroupElement[13].x=-x+0.25;       SpaceGroupElement[13].y=-z+0.75;       SpaceGroupElement[13].z=y+0.75;         // -4^-1    [1,0,0]         0,0,0           1/8,0,3/4
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x+0.5;         SpaceGroupElement[14].z=-y+0.5;         // 3^1      [-1,-1,1]       0,0,0           0,1/2,0
      SpaceGroupElement[15].x=x+0.75;        SpaceGroupElement[15].y=z+0.75;        SpaceGroupElement[15].z=y+0.75;         // -2       [0,-1,1]        3/4,3/4,3/4     0,0,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=-x+0.5;         // 3^-1     [-1,-1,1]       -1/6,-1/6,1/6   1/3,1/6,0
      SpaceGroupElement[18].x=-z+0.25;       SpaceGroupElement[18].y=-y+0.75;       SpaceGroupElement[18].z=x+0.75;         // -4^1     [0,1,0]         0,0,0           -1/4,3/8,1/2
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z+0.5;         SpaceGroupElement[19].z=-x+0.5;         // 3^1      [-1,1,1]        -1/3,1/3,1/3    1/6,1/6,0
      SpaceGroupElement[20].x=z+0.25;        SpaceGroupElement[20].y=y+0.25;        SpaceGroupElement[20].z=x+0.25;         // -2       [-1,0,1]        1/4,1/4,1/4     0,0,0
      SpaceGroupElement[21].x=-y+0.5;        SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x+0.5;          // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      SpaceGroupElement[22].x=-z+0.25;       SpaceGroupElement[22].y=y+0.25;        SpaceGroupElement[22].z=-x+0.75;        // -2       [1,0,1]         -1/4,1/4,1/4    1/2,0,0
      SpaceGroupElement[23].x=z+0.75;        SpaceGroupElement[23].y=-y+0.25;       SpaceGroupElement[23].z=-x+0.75;        // -4^-1    [0,1,0]         0,0,0           3/4,1/8,0
      SpaceGroupElement[24].x=x+0.5;         SpaceGroupElement[24].y=y+0.5;         SpaceGroupElement[24].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[25].x=y+0.75;        SpaceGroupElement[25].y=-x+0.25;       SpaceGroupElement[25].z=-z+0.75;        // -4^1     [0,0,1]         0,0,0           1,1/4,3/8
      SpaceGroupElement[26].x=-x+0.5;        SpaceGroupElement[26].y=-y;            SpaceGroupElement[26].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/2,0
      SpaceGroupElement[27].x=-y+0.25;       SpaceGroupElement[27].y=x+0.25;        SpaceGroupElement[27].z=-z+0.75;        // -4^-1    [0,0,1]         0,0,0           0,5/4,3/8
      SpaceGroupElement[28].x=x+0.5;         SpaceGroupElement[28].y=-y+0.5;        SpaceGroupElement[28].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,1/2
      SpaceGroupElement[29].x=-y+0.75;       SpaceGroupElement[29].y=-x+0.25;       SpaceGroupElement[29].z=z+0.25;         // -2       [1,1,0]         -1/4,1/4,5/4    1,0,0
      SpaceGroupElement[30].x=-x+0.5;        SpaceGroupElement[30].y=y;             SpaceGroupElement[30].z=-z;             // 2        [0,1,0]         0,1,0           1/4,0,1/2
      SpaceGroupElement[31].x=y+0.25;        SpaceGroupElement[31].y=x+0.25;        SpaceGroupElement[31].z=z+0.25;         // -2       [-1,1,0]        5/4,5/4,5/4     0,0,0
      SpaceGroupElement[32].x=z+0.5;         SpaceGroupElement[32].y=x+0.5;         SpaceGroupElement[32].z=y+0.5;          // 3^1      [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[33].x=x+0.75;        SpaceGroupElement[33].y=-z+0.25;       SpaceGroupElement[33].z=-y+0.75;        // -2       [0,1,1]         3/4,1/4,-1/4    0,1,0
      SpaceGroupElement[34].x=-z+0.5;        SpaceGroupElement[34].y=-x;            SpaceGroupElement[34].z=y+0.5;          // 3^-1     [-1,1,1]        -1/3,1/3,1/3    5/6,-1/6,0
      SpaceGroupElement[35].x=-x+0.25;       SpaceGroupElement[35].y=z+0.25;        SpaceGroupElement[35].z=-y+0.75;        // -4^1     [1,0,0]         0,0,0           5/8,1,-1/4
      SpaceGroupElement[36].x=z+0.5;         SpaceGroupElement[36].y=-x+0.5;        SpaceGroupElement[36].z=-y;             // 3^-1     [1,-1,1]        1/3,-1/3,1/3    1/6,2/3,0
      SpaceGroupElement[37].x=-x+0.75;       SpaceGroupElement[37].y=-z+0.25;       SpaceGroupElement[37].z=y+0.25;         // -4^-1    [1,0,0]         0,0,0           3/8,0,5/4
      SpaceGroupElement[38].x=-z+0.5;        SpaceGroupElement[38].y=x;             SpaceGroupElement[38].z=-y;             // 3^1      [-1,-1,1]       1/6,1/6,-1/6    1/3,7/6,0
      SpaceGroupElement[39].x=x+0.25;        SpaceGroupElement[39].y=z+0.25;        SpaceGroupElement[39].z=y+0.25;         // -2       [0,-1,1]        5/4,5/4,5/4     0,0,0
      SpaceGroupElement[40].x=y+0.5;         SpaceGroupElement[40].y=z+0.5;         SpaceGroupElement[40].z=x+0.5;          // 3^-1     [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[41].x=y+0.5;         SpaceGroupElement[41].y=-z+0.5;        SpaceGroupElement[41].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           1,1/2,0
      SpaceGroupElement[42].x=-z+0.75;       SpaceGroupElement[42].y=-y+0.25;       SpaceGroupElement[42].z=x+0.25;         // -4^1     [0,1,0]         0,0,0           -1/4,5/8,1
      SpaceGroupElement[43].x=-y+0.5;        SpaceGroupElement[43].y=z;             SpaceGroupElement[43].z=-x;             // 3^1      [-1,1,1]        -1/2,1/2,1/2    1/2,1/2,0
      SpaceGroupElement[44].x=z+0.75;        SpaceGroupElement[44].y=y+0.75;        SpaceGroupElement[44].z=x+0.75;         // -2       [-1,0,1]        3/4,3/4,3/4     0,0,0
      SpaceGroupElement[45].x=-y;            SpaceGroupElement[45].y=-z+0.5;        SpaceGroupElement[45].z=x;              // 3^1      [1,-1,1]        1/2,-1/2,1/2    -1/2,1,0
      SpaceGroupElement[46].x=-z+0.75;       SpaceGroupElement[46].y=y+0.75;        SpaceGroupElement[46].z=-x+0.25;        // -2       [1,0,1]         -1/4,3/4,1/4    1,0,0
      SpaceGroupElement[47].x=z+0.25;        SpaceGroupElement[47].y=-y+0.75;       SpaceGroupElement[47].z=-x+0.25;        // -4^-1    [0,1,0]         0,0,0           5/4,3/8,0
      break;

    case 517: // [221] P m -3 m (-P 4 2 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=z;              SpaceGroupElement[9].z=y;               // 2        [0,1,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;              // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=y;              // 4^1      [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=z;             SpaceGroupElement[13].z=-y;             // 4^-1     [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-y;             // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=-y;             // 2        [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[18].x=z;             SpaceGroupElement[18].y=y;             SpaceGroupElement[18].z=-x;             // 4^1      [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-x;             // 2        [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=x;              // 2        [1,0,1]         0,0,0           0,0,0
      SpaceGroupElement[23].x=-z;            SpaceGroupElement[23].y=y;             SpaceGroupElement[23].z=x;              // 4^-1     [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[25].x=y;             SpaceGroupElement[25].y=-x;            SpaceGroupElement[25].z=-z;             // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=y;             SpaceGroupElement[26].z=-z;             // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[27].x=-y;            SpaceGroupElement[27].y=x;             SpaceGroupElement[27].z=-z;             // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=y;             SpaceGroupElement[28].z=z;              // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[29].x=-y;            SpaceGroupElement[29].y=-x;            SpaceGroupElement[29].z=z;              // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[30].x=x;             SpaceGroupElement[30].y=-y;            SpaceGroupElement[30].z=z;              // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=z;              // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[32].x=-z;            SpaceGroupElement[32].y=-x;            SpaceGroupElement[32].z=-y;             // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[33].x=x;             SpaceGroupElement[33].y=-z;            SpaceGroupElement[33].z=-y;             // -2       [0,1,1]         0,0,0           0,0,0
      SpaceGroupElement[34].x=z;             SpaceGroupElement[34].y=x;             SpaceGroupElement[34].z=-y;             // -3^-1    [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[35].x=-x;            SpaceGroupElement[35].y=z;             SpaceGroupElement[35].z=-y;             // -4^1     [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[36].x=-z;            SpaceGroupElement[36].y=x;             SpaceGroupElement[36].z=y;              // -3^-1    [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[37].x=-x;            SpaceGroupElement[37].y=-z;            SpaceGroupElement[37].z=y;              // -4^-1    [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[38].x=z;             SpaceGroupElement[38].y=-x;            SpaceGroupElement[38].z=y;              // -3^1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=z;             SpaceGroupElement[39].z=y;              // -2       [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[40].x=-y;            SpaceGroupElement[40].y=-z;            SpaceGroupElement[40].z=-x;             // -3^-1    [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[41].x=-y;            SpaceGroupElement[41].y=z;             SpaceGroupElement[41].z=x;              // -3^-1    [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[42].x=-z;            SpaceGroupElement[42].y=-y;            SpaceGroupElement[42].z=x;              // -4^1     [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[43].x=y;             SpaceGroupElement[43].y=-z;            SpaceGroupElement[43].z=x;              // -3^1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[44].x=z;             SpaceGroupElement[44].y=y;             SpaceGroupElement[44].z=x;              // -2       [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[45].x=y;             SpaceGroupElement[45].y=z;             SpaceGroupElement[45].z=-x;             // -3^1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[46].x=-z;            SpaceGroupElement[46].y=y;             SpaceGroupElement[46].z=-x;             // -2       [1,0,1]         0,0,0           0,0,0
      SpaceGroupElement[47].x=z;             SpaceGroupElement[47].y=-y;            SpaceGroupElement[47].z=-x;             // -4^-1    [0,1,0]         0,0,0           0,0,0
      break;

    case 518: // [222] P n -3 n Origin choice 1 (P 4 2 3 -1n)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=z;              SpaceGroupElement[9].z=y;               // 2        [0,1,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;              // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=y;              // 4^1      [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=z;             SpaceGroupElement[13].z=-y;             // 4^-1     [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-y;             // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=-y;             // 2        [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[18].x=z;             SpaceGroupElement[18].y=y;             SpaceGroupElement[18].z=-x;             // 4^1      [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-x;             // 2        [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=x;              // 2        [1,0,1]         0,0,0           0,0,0
      SpaceGroupElement[23].x=-z;            SpaceGroupElement[23].y=y;             SpaceGroupElement[23].z=x;              // 4^-1     [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[24].x=-x+0.5;        SpaceGroupElement[24].y=-y+0.5;        SpaceGroupElement[24].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[25].x=y+0.5;         SpaceGroupElement[25].y=-x+0.5;        SpaceGroupElement[25].z=-z+0.5;         // -4^1     [0,0,1]         0,0,0           1/2,0,1/4
      SpaceGroupElement[26].x=x+0.5;         SpaceGroupElement[26].y=y+0.5;         SpaceGroupElement[26].z=-z+0.5;         // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[27].x=-y+0.5;        SpaceGroupElement[27].y=x+0.5;         SpaceGroupElement[27].z=-z+0.5;         // -4^-1    [0,0,1]         0,0,0           0,1/2,1/4
      SpaceGroupElement[28].x=-x+0.5;        SpaceGroupElement[28].y=y+0.5;         SpaceGroupElement[28].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[29].x=-y+0.5;        SpaceGroupElement[29].y=-x+0.5;        SpaceGroupElement[29].z=z+0.5;          // -2       [1,1,0]         0,0,1/2         1/2,0,0
      SpaceGroupElement[30].x=x+0.5;         SpaceGroupElement[30].y=-y+0.5;        SpaceGroupElement[30].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[31].x=y+0.5;         SpaceGroupElement[31].y=x+0.5;         SpaceGroupElement[31].z=z+0.5;          // -2       [-1,1,0]        1/2,1/2,1/2     0,0,0
      SpaceGroupElement[32].x=-z+0.5;        SpaceGroupElement[32].y=-x+0.5;        SpaceGroupElement[32].z=-y+0.5;         // -3^1     [1,1,1]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[33].x=x+0.5;         SpaceGroupElement[33].y=-z+0.5;        SpaceGroupElement[33].z=-y+0.5;         // -2       [0,1,1]         1/2,0,0         0,1/2,0
      SpaceGroupElement[34].x=z+0.5;         SpaceGroupElement[34].y=x+0.5;         SpaceGroupElement[34].z=-y+0.5;         // -3^-1    [-1,1,1]        0,0,0           1/4,3/4,-1/4
      SpaceGroupElement[35].x=-x+0.5;        SpaceGroupElement[35].y=z+0.5;         SpaceGroupElement[35].z=-y+0.5;         // -4^1     [1,0,0]         0,0,0           1/4,1/2,0
      SpaceGroupElement[36].x=-z+0.5;        SpaceGroupElement[36].y=x+0.5;         SpaceGroupElement[36].z=y+0.5;          // -3^-1    [1,-1,1]        0,0,0           -1/4,1/4,3/4
      SpaceGroupElement[37].x=-x+0.5;        SpaceGroupElement[37].y=-z+0.5;        SpaceGroupElement[37].z=y+0.5;          // -4^-1    [1,0,0]         0,0,0           1/4,0,1/2
      SpaceGroupElement[38].x=z+0.5;         SpaceGroupElement[38].y=-x+0.5;        SpaceGroupElement[38].z=y+0.5;          // -3^1     [-1,-1,1]       0,0,0           3/4,-1/4,1/4
      SpaceGroupElement[39].x=x+0.5;         SpaceGroupElement[39].y=z+0.5;         SpaceGroupElement[39].z=y+0.5;          // -2       [0,-1,1]        1/2,1/2,1/2     0,0,0
      SpaceGroupElement[40].x=-y+0.5;        SpaceGroupElement[40].y=-z+0.5;        SpaceGroupElement[40].z=-x+0.5;         // -3^-1    [1,1,1]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[41].x=-y+0.5;        SpaceGroupElement[41].y=z+0.5;         SpaceGroupElement[41].z=x+0.5;          // -3^-1    [-1,-1,1]       0,0,0           -1/4,3/4,1/4
      SpaceGroupElement[42].x=-z+0.5;        SpaceGroupElement[42].y=-y+0.5;        SpaceGroupElement[42].z=x+0.5;          // -4^1     [0,1,0]         0,0,0           0,1/4,1/2
      SpaceGroupElement[43].x=y+0.5;         SpaceGroupElement[43].y=-z+0.5;        SpaceGroupElement[43].z=x+0.5;          // -3^1     [-1,1,1]        0,0,0           1/4,-1/4,3/4
      SpaceGroupElement[44].x=z+0.5;         SpaceGroupElement[44].y=y+0.5;         SpaceGroupElement[44].z=x+0.5;          // -2       [-1,0,1]        1/2,1/2,1/2     0,0,0
      SpaceGroupElement[45].x=y+0.5;         SpaceGroupElement[45].y=z+0.5;         SpaceGroupElement[45].z=-x+0.5;         // -3^1     [1,-1,1]        0,0,0           3/4,1/4,-1/4
      SpaceGroupElement[46].x=-z+0.5;        SpaceGroupElement[46].y=y+0.5;         SpaceGroupElement[46].z=-x+0.5;         // -2       [1,0,1]         0,1/2,0         1/2,0,0
      SpaceGroupElement[47].x=z+0.5;         SpaceGroupElement[47].y=-y+0.5;        SpaceGroupElement[47].z=-x+0.5;         // -4^-1    [0,1,0]         0,0,0           1/2,1/4,0
      break;
    case 519: // [222] P n -3 n Origin choice 2 (-P 4a 2bc 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         0,0,0           0,0,1/4
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[7].x=-y+0.5;         SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=z;              SpaceGroupElement[9].z=y;               // 2        [0,1,1]         0,0,0           1/4,0,0
      SpaceGroupElement[10].x=-z+0.5;        SpaceGroupElement[10].y=-x+0.5;        SpaceGroupElement[10].z=y;              // 3^-1     [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=-z+0.5;        SpaceGroupElement[11].z=y;              // 4^1      [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x+0.5;        SpaceGroupElement[12].z=-y+0.5;         // 3^-1     [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=z;             SpaceGroupElement[13].z=-y+0.5;         // 4^-1     [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[14].x=-z+0.5;        SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-y+0.5;         // 3^1      [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[15].x=-x+0.5;        SpaceGroupElement[15].y=-z+0.5;        SpaceGroupElement[15].z=-y+0.5;         // 2        [0,-1,1]        0,0,0           1/4,1/2,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z+0.5;        SpaceGroupElement[17].z=-x+0.5;         // 3^-1     [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[18].x=z;             SpaceGroupElement[18].y=y;             SpaceGroupElement[18].z=-x+0.5;         // 4^1      [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[19].x=-y+0.5;        SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=-x+0.5;         // 3^1      [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[20].x=-z+0.5;        SpaceGroupElement[20].y=-y+0.5;        SpaceGroupElement[20].z=-x+0.5;         // 2        [-1,0,1]        0,0,0           1/2,1/4,0
      SpaceGroupElement[21].x=-y+0.5;        SpaceGroupElement[21].y=-z+0.5;        SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=-y+0.5;        SpaceGroupElement[22].z=x;              // 2        [1,0,1]         0,0,0           0,1/4,0
      SpaceGroupElement[23].x=-z+0.5;        SpaceGroupElement[23].y=y;             SpaceGroupElement[23].z=x;              // 4^-1     [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[25].x=y-0.5;         SpaceGroupElement[25].y=-x;            SpaceGroupElement[25].z=-z;             // -4^1     [0,0,1]         0,0,0           -1/4,1/4,0
      SpaceGroupElement[26].x=x-0.5;         SpaceGroupElement[26].y=y-0.5;         SpaceGroupElement[26].z=-z;             // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[27].x=-y;            SpaceGroupElement[27].y=x-0.5;         SpaceGroupElement[27].z=-z;             // -4^-1    [0,0,1]         0,0,0           1/4,-1/4,0
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=y-0.5;         SpaceGroupElement[28].z=z-0.5;          // -2       [1,0,0]         0,-1/2,-1/2     0,0,0
      SpaceGroupElement[29].x=-y;            SpaceGroupElement[29].y=-x;            SpaceGroupElement[29].z=z-0.5;          // -2       [1,1,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[30].x=x-0.5;         SpaceGroupElement[30].y=-y;            SpaceGroupElement[30].z=z-0.5;          // -2       [0,1,0]         -1/2,0,-1/2     0,0,0
      SpaceGroupElement[31].x=y-0.5;         SpaceGroupElement[31].y=x-0.5;         SpaceGroupElement[31].z=z-0.5;          // -2       [-1,1,0]        -1/2,-1/2,-1/2  0,0,0
      SpaceGroupElement[32].x=-z;            SpaceGroupElement[32].y=-x;            SpaceGroupElement[32].z=-y;             // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[33].x=x-0.5;         SpaceGroupElement[33].y=-z;            SpaceGroupElement[33].z=-y;             // -2       [0,1,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[34].x=z-0.5;         SpaceGroupElement[34].y=x-0.5;         SpaceGroupElement[34].z=-y;             // -3^-1    [-1,1,1]        0,0,0           0,-1/2,1/2
      SpaceGroupElement[35].x=-x;            SpaceGroupElement[35].y=z-0.5;         SpaceGroupElement[35].z=-y;             // -4^1     [1,0,0]         0,0,0           0,-1/4,1/4
      SpaceGroupElement[36].x=-z;            SpaceGroupElement[36].y=x-0.5;         SpaceGroupElement[36].z=y-0.5;          // -3^-1    [1,-1,1]        0,0,0           1/2,0,-1/2
      SpaceGroupElement[37].x=-x;            SpaceGroupElement[37].y=-z;            SpaceGroupElement[37].z=y-0.5;          // -4^-1    [1,0,0]         0,0,0           0,1/4,-1/4
      SpaceGroupElement[38].x=z-0.5;         SpaceGroupElement[38].y=-x;            SpaceGroupElement[38].z=y-0.5;          // -3^1     [-1,-1,1]       0,0,0           -1/2,1/2,0
      SpaceGroupElement[39].x=x-0.5;         SpaceGroupElement[39].y=z-0.5;         SpaceGroupElement[39].z=y-0.5;          // -2       [0,-1,1]        -1/2,-1/2,-1/2  0,0,0
      SpaceGroupElement[40].x=-y;            SpaceGroupElement[40].y=-z;            SpaceGroupElement[40].z=-x;             // -3^-1    [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[41].x=-y;            SpaceGroupElement[41].y=z-0.5;         SpaceGroupElement[41].z=x-0.5;          // -3^-1    [-1,-1,1]       0,0,0           1/2,-1/2,0
      SpaceGroupElement[42].x=-z;            SpaceGroupElement[42].y=-y;            SpaceGroupElement[42].z=x-0.5;          // -4^1     [0,1,0]         0,0,0           1/4,0,-1/4
      SpaceGroupElement[43].x=y-0.5;         SpaceGroupElement[43].y=-z;            SpaceGroupElement[43].z=x-0.5;          // -3^1     [-1,1,1]        0,0,0           0,1/2,-1/2
      SpaceGroupElement[44].x=z-0.5;         SpaceGroupElement[44].y=y-0.5;         SpaceGroupElement[44].z=x-0.5;          // -2       [-1,0,1]        -1/2,-1/2,-1/2  0,0,0
      SpaceGroupElement[45].x=y-0.5;         SpaceGroupElement[45].y=z-0.5;         SpaceGroupElement[45].z=-x;             // -3^1     [1,-1,1]        0,0,0           -1/2,0,1/2
      SpaceGroupElement[46].x=-z;            SpaceGroupElement[46].y=y-0.5;         SpaceGroupElement[46].z=-x;             // -2       [1,0,1]         0,-1/2,0        0,0,0
      SpaceGroupElement[47].x=z-0.5;         SpaceGroupElement[47].y=-y;            SpaceGroupElement[47].z=-x;             // -4^-1    [0,1,0]         0,0,0           -1/4,0,1/4
      break;

    case 520: // [223] P m -3 n (-P 4n 2 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y+0.5;          SpaceGroupElement[5].y=x+0.5;          SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y+0.5;         SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=z+0.5;          SpaceGroupElement[9].z=y+0.5;           // 2        [0,1,1]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;              // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=x+0.5;         SpaceGroupElement[11].y=-z+0.5;        SpaceGroupElement[11].z=y+0.5;          // 4^1      [1,0,0]         1/2,0,0         0,0,1/2
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=z+0.5;         SpaceGroupElement[13].z=-y+0.5;         // 4^-1     [1,0,0]         1/2,0,0         0,1/2,0
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-y;             // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[15].x=-x+0.5;        SpaceGroupElement[15].y=-z+0.5;        SpaceGroupElement[15].z=-y+0.5;         // 2        [0,-1,1]        0,0,0           1/4,1/2,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[18].x=z+0.5;         SpaceGroupElement[18].y=y+0.5;         SpaceGroupElement[18].z=-x+0.5;         // 4^1      [0,1,0]         0,1/2,0         1/2,0,0
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[20].x=-z+0.5;        SpaceGroupElement[20].y=-y+0.5;        SpaceGroupElement[20].z=-x+0.5;         // 2        [-1,0,1]        0,0,0           1/2,1/4,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[22].x=z+0.5;         SpaceGroupElement[22].y=-y+0.5;        SpaceGroupElement[22].z=x+0.5;          // 2        [1,0,1]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[23].x=-z+0.5;        SpaceGroupElement[23].y=y+0.5;         SpaceGroupElement[23].z=x+0.5;          // 4^-1     [0,1,0]         0,1/2,0         0,0,1/2
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[25].x=y-0.5;         SpaceGroupElement[25].y=-x-0.5;        SpaceGroupElement[25].z=-z-0.5;         // -4^1     [0,0,1]         0,0,0           -1/2,0,-1/4
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=y;             SpaceGroupElement[26].z=-z;             // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[27].x=-y-0.5;        SpaceGroupElement[27].y=x-0.5;         SpaceGroupElement[27].z=-z-0.5;         // -4^-1    [0,0,1]         0,0,0           0,-1/2,-1/4
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=y;             SpaceGroupElement[28].z=z;              // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[29].x=-y-0.5;        SpaceGroupElement[29].y=-x-0.5;        SpaceGroupElement[29].z=z-0.5;          // -2       [1,1,0]         0,0,-1/2        -1/2,0,0
      SpaceGroupElement[30].x=x;             SpaceGroupElement[30].y=-y;            SpaceGroupElement[30].z=z;              // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[31].x=y-0.5;         SpaceGroupElement[31].y=x-0.5;         SpaceGroupElement[31].z=z-0.5;          // -2       [-1,1,0]        -1/2,-1/2,-1/2  0,0,0
      SpaceGroupElement[32].x=-z;            SpaceGroupElement[32].y=-x;            SpaceGroupElement[32].z=-y;             // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[33].x=x-0.5;         SpaceGroupElement[33].y=-z-0.5;        SpaceGroupElement[33].z=-y-0.5;         // -2       [0,1,1]         -1/2,0,0        0,-1/2,0
      SpaceGroupElement[34].x=z;             SpaceGroupElement[34].y=x;             SpaceGroupElement[34].z=-y;             // -3^-1    [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[35].x=-x-0.5;        SpaceGroupElement[35].y=z-0.5;         SpaceGroupElement[35].z=-y-0.5;         // -4^1     [1,0,0]         0,0,0           -1/4,-1/2,0
      SpaceGroupElement[36].x=-z;            SpaceGroupElement[36].y=x;             SpaceGroupElement[36].z=y;              // -3^-1    [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[37].x=-x-0.5;        SpaceGroupElement[37].y=-z-0.5;        SpaceGroupElement[37].z=y-0.5;          // -4^-1    [1,0,0]         0,0,0           -1/4,0,-1/2
      SpaceGroupElement[38].x=z;             SpaceGroupElement[38].y=-x;            SpaceGroupElement[38].z=y;              // -3^1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[39].x=x-0.5;         SpaceGroupElement[39].y=z-0.5;         SpaceGroupElement[39].z=y-0.5;          // -2       [0,-1,1]        -1/2,-1/2,-1/2  0,0,0
      SpaceGroupElement[40].x=-y;            SpaceGroupElement[40].y=-z;            SpaceGroupElement[40].z=-x;             // -3^-1    [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[41].x=-y;            SpaceGroupElement[41].y=z;             SpaceGroupElement[41].z=x;              // -3^-1    [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[42].x=-z-0.5;        SpaceGroupElement[42].y=-y-0.5;        SpaceGroupElement[42].z=x-0.5;          // -4^1     [0,1,0]         0,0,0           0,-1/4,-1/2
      SpaceGroupElement[43].x=y;             SpaceGroupElement[43].y=-z;            SpaceGroupElement[43].z=x;              // -3^1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[44].x=z-0.5;         SpaceGroupElement[44].y=y-0.5;         SpaceGroupElement[44].z=x-0.5;          // -2       [-1,0,1]        -1/2,-1/2,-1/2  0,0,0
      SpaceGroupElement[45].x=y;             SpaceGroupElement[45].y=z;             SpaceGroupElement[45].z=-x;             // -3^1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[46].x=-z-0.5;        SpaceGroupElement[46].y=y-0.5;         SpaceGroupElement[46].z=-x-0.5;         // -2       [1,0,1]         0,-1/2,0        -1/2,0,0
      SpaceGroupElement[47].x=z-0.5;         SpaceGroupElement[47].y=-y-0.5;        SpaceGroupElement[47].z=-x-0.5;         // -4^-1    [0,1,0]         0,0,0           -1/2,-1/4,0
      break;

    case 521: // [224] P n -3 m Origin choice 1 (P 4n 2 3 -1n)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y+0.5;          SpaceGroupElement[5].y=x+0.5;          SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y+0.5;         SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=-z+0.5;          // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=z+0.5;          SpaceGroupElement[9].z=y+0.5;           // 2        [0,1,1]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;              // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=x+0.5;         SpaceGroupElement[11].y=-z+0.5;        SpaceGroupElement[11].z=y+0.5;          // 4^1      [1,0,0]         1/2,0,0         0,0,1/2
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=z+0.5;         SpaceGroupElement[13].z=-y+0.5;         // 4^-1     [1,0,0]         1/2,0,0         0,1/2,0
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-y;             // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[15].x=-x+0.5;        SpaceGroupElement[15].y=-z+0.5;        SpaceGroupElement[15].z=-y+0.5;         // 2        [0,-1,1]        0,0,0           1/4,1/2,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[18].x=z+0.5;         SpaceGroupElement[18].y=y+0.5;         SpaceGroupElement[18].z=-x+0.5;         // 4^1      [0,1,0]         0,1/2,0         1/2,0,0
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[20].x=-z+0.5;        SpaceGroupElement[20].y=-y+0.5;        SpaceGroupElement[20].z=-x+0.5;         // 2        [-1,0,1]        0,0,0           1/2,1/4,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[22].x=z+0.5;         SpaceGroupElement[22].y=-y+0.5;        SpaceGroupElement[22].z=x+0.5;          // 2        [1,0,1]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[23].x=-z+0.5;        SpaceGroupElement[23].y=y+0.5;         SpaceGroupElement[23].z=x+0.5;          // 4^-1     [0,1,0]         0,1/2,0         0,0,1/2
      SpaceGroupElement[24].x=-x+0.5;        SpaceGroupElement[24].y=-y+0.5;        SpaceGroupElement[24].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[25].x=y;             SpaceGroupElement[25].y=-x;            SpaceGroupElement[25].z=-z;             // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[26].x=x+0.5;         SpaceGroupElement[26].y=y+0.5;         SpaceGroupElement[26].z=-z+0.5;         // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[27].x=-y;            SpaceGroupElement[27].y=x;             SpaceGroupElement[27].z=-z;             // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[28].x=-x+0.5;        SpaceGroupElement[28].y=y+0.5;         SpaceGroupElement[28].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[29].x=-y;            SpaceGroupElement[29].y=-x;            SpaceGroupElement[29].z=z;              // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[30].x=x+0.5;         SpaceGroupElement[30].y=-y+0.5;        SpaceGroupElement[30].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=z;              // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[32].x=-z+0.5;        SpaceGroupElement[32].y=-x+0.5;        SpaceGroupElement[32].z=-y+0.5;         // -3^1     [1,1,1]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[33].x=x;             SpaceGroupElement[33].y=-z;            SpaceGroupElement[33].z=-y;             // -2       [0,1,1]         0,0,0           0,0,0
      SpaceGroupElement[34].x=z+0.5;         SpaceGroupElement[34].y=x+0.5;         SpaceGroupElement[34].z=-y+0.5;         // -3^-1    [-1,1,1]        0,0,0           1/4,3/4,-1/4
      SpaceGroupElement[35].x=-x;            SpaceGroupElement[35].y=z;             SpaceGroupElement[35].z=-y;             // -4^1     [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[36].x=-z+0.5;        SpaceGroupElement[36].y=x+0.5;         SpaceGroupElement[36].z=y+0.5;          // -3^-1    [1,-1,1]        0,0,0           -1/4,1/4,3/4
      SpaceGroupElement[37].x=-x;            SpaceGroupElement[37].y=-z;            SpaceGroupElement[37].z=y;              // -4^-1    [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[38].x=z+0.5;         SpaceGroupElement[38].y=-x+0.5;        SpaceGroupElement[38].z=y+0.5;          // -3^1     [-1,-1,1]       0,0,0           3/4,-1/4,1/4
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=z;             SpaceGroupElement[39].z=y;              // -2       [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[40].x=-y+0.5;        SpaceGroupElement[40].y=-z+0.5;        SpaceGroupElement[40].z=-x+0.5;         // -3^-1    [1,1,1]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[41].x=-y+0.5;        SpaceGroupElement[41].y=z+0.5;         SpaceGroupElement[41].z=x+0.5;          // -3^-1    [-1,-1,1]       0,0,0           -1/4,3/4,1/4
      SpaceGroupElement[42].x=-z;            SpaceGroupElement[42].y=-y;            SpaceGroupElement[42].z=x;              // -4^1     [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[43].x=y+0.5;         SpaceGroupElement[43].y=-z+0.5;        SpaceGroupElement[43].z=x+0.5;          // -3^1     [-1,1,1]        0,0,0           1/4,-1/4,3/4
      SpaceGroupElement[44].x=z;             SpaceGroupElement[44].y=y;             SpaceGroupElement[44].z=x;              // -2       [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[45].x=y+0.5;         SpaceGroupElement[45].y=z+0.5;         SpaceGroupElement[45].z=-x+0.5;         // -3^1     [1,-1,1]        0,0,0           3/4,1/4,-1/4
      SpaceGroupElement[46].x=-z;            SpaceGroupElement[46].y=y;             SpaceGroupElement[46].z=-x;             // -2       [1,0,1]         0,0,0           0,0,0
      SpaceGroupElement[47].x=z;             SpaceGroupElement[47].y=-y;            SpaceGroupElement[47].z=-x;             // -4^-1    [0,1,0]         0,0,0           0,0,0
      break;
    case 522: // [224] P n -3 m Origin choice 2 (-P 4bc 2bc 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x+0.5;          SpaceGroupElement[1].z=z+0.5;           // 4^1      [0,0,1]         0,0,1/2         -1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[3].x=y+0.5;          SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.5;           // 4^-1     [0,0,1]         0,0,1/2         1/4,-1/4,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y+0.5;         SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[5].x=y+0.5;          SpaceGroupElement[5].y=x+0.5;          SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         1/2,1/2,0       0,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=z+0.5;          SpaceGroupElement[9].z=y+0.5;           // 2        [0,1,1]         0,1/2,1/2       0,0,0
      SpaceGroupElement[10].x=-z+0.5;        SpaceGroupElement[10].y=-x+0.5;        SpaceGroupElement[10].z=y;              // 3^-1     [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[11].x=x+0.5;         SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=y+0.5;          // 4^1      [1,0,0]         1/2,0,0         0,-1/4,1/4
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x+0.5;        SpaceGroupElement[12].z=-y+0.5;         // 3^-1     [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=z+0.5;         SpaceGroupElement[13].z=-y;             // 4^-1     [1,0,0]         1/2,0,0         0,1/4,-1/4
      SpaceGroupElement[14].x=-z+0.5;        SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-y+0.5;         // 3^1      [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=-y;             // 2        [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z+0.5;        SpaceGroupElement[17].z=-x+0.5;         // 3^-1     [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[18].x=z+0.5;         SpaceGroupElement[18].y=y+0.5;         SpaceGroupElement[18].z=-x;             // 4^1      [0,1,0]         0,1/2,0         1/4,0,-1/4
      SpaceGroupElement[19].x=-y+0.5;        SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=-x+0.5;         // 3^1      [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-x;             // 2        [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[21].x=-y+0.5;        SpaceGroupElement[21].y=-z+0.5;        SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[22].x=z+0.5;         SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=x+0.5;          // 2        [1,0,1]         1/2,0,1/2       0,0,0
      SpaceGroupElement[23].x=-z;            SpaceGroupElement[23].y=y+0.5;         SpaceGroupElement[23].z=x+0.5;          // 4^-1     [0,1,0]         0,1/2,0         -1/4,0,1/4
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[25].x=y;             SpaceGroupElement[25].y=-x-0.5;        SpaceGroupElement[25].z=-z-0.5;         // -4^1     [0,0,1]         0,0,0           -1/4,-1/4,-1/4
      SpaceGroupElement[26].x=x-0.5;         SpaceGroupElement[26].y=y-0.5;         SpaceGroupElement[26].z=-z;             // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[27].x=-y-0.5;        SpaceGroupElement[27].y=x;             SpaceGroupElement[27].z=-z-0.5;         // -4^-1    [0,0,1]         0,0,0           -1/4,-1/4,-1/4
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=y-0.5;         SpaceGroupElement[28].z=z-0.5;          // -2       [1,0,0]         0,-1/2,-1/2     0,0,0
      SpaceGroupElement[29].x=-y-0.5;        SpaceGroupElement[29].y=-x-0.5;        SpaceGroupElement[29].z=z;              // -2       [1,1,0]         0,0,0           -1/2,0,0
      SpaceGroupElement[30].x=x-0.5;         SpaceGroupElement[30].y=-y;            SpaceGroupElement[30].z=z-0.5;          // -2       [0,1,0]         -1/2,0,-1/2     0,0,0
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=z;              // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[32].x=-z;            SpaceGroupElement[32].y=-x;            SpaceGroupElement[32].z=-y;             // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[33].x=x;             SpaceGroupElement[33].y=-z-0.5;        SpaceGroupElement[33].z=-y-0.5;         // -2       [0,1,1]         0,0,0           0,-1/2,0
      SpaceGroupElement[34].x=z-0.5;         SpaceGroupElement[34].y=x-0.5;         SpaceGroupElement[34].z=-y;             // -3^-1    [-1,1,1]        0,0,0           0,-1/2,1/2
      SpaceGroupElement[35].x=-x-0.5;        SpaceGroupElement[35].y=z;             SpaceGroupElement[35].z=-y-0.5;         // -4^1     [1,0,0]         0,0,0           -1/4,-1/4,-1/4
      SpaceGroupElement[36].x=-z;            SpaceGroupElement[36].y=x-0.5;         SpaceGroupElement[36].z=y-0.5;          // -3^-1    [1,-1,1]        0,0,0           1/2,0,-1/2
      SpaceGroupElement[37].x=-x-0.5;        SpaceGroupElement[37].y=-z-0.5;        SpaceGroupElement[37].z=y;              // -4^-1    [1,0,0]         0,0,0           -1/4,-1/4,-1/4
      SpaceGroupElement[38].x=z-0.5;         SpaceGroupElement[38].y=-x;            SpaceGroupElement[38].z=y-0.5;          // -3^1     [-1,-1,1]       0,0,0           -1/2,1/2,0
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=z;             SpaceGroupElement[39].z=y;              // -2       [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[40].x=-y;            SpaceGroupElement[40].y=-z;            SpaceGroupElement[40].z=-x;             // -3^-1    [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[41].x=-y;            SpaceGroupElement[41].y=z-0.5;         SpaceGroupElement[41].z=x-0.5;          // -3^-1    [-1,-1,1]       0,0,0           1/2,-1/2,0
      SpaceGroupElement[42].x=-z-0.5;        SpaceGroupElement[42].y=-y-0.5;        SpaceGroupElement[42].z=x;              // -4^1     [0,1,0]         0,0,0           -1/4,-1/4,-1/4
      SpaceGroupElement[43].x=y-0.5;         SpaceGroupElement[43].y=-z;            SpaceGroupElement[43].z=x-0.5;          // -3^1     [-1,1,1]        0,0,0           0,1/2,-1/2
      SpaceGroupElement[44].x=z;             SpaceGroupElement[44].y=y;             SpaceGroupElement[44].z=x;              // -2       [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[45].x=y-0.5;         SpaceGroupElement[45].y=z-0.5;         SpaceGroupElement[45].z=-x;             // -3^1     [1,-1,1]        0,0,0           -1/2,0,1/2
      SpaceGroupElement[46].x=-z-0.5;        SpaceGroupElement[46].y=y;             SpaceGroupElement[46].z=-x-0.5;         // -2       [1,0,1]         0,0,0           -1/2,0,0
      SpaceGroupElement[47].x=z;             SpaceGroupElement[47].y=-y-0.5;        SpaceGroupElement[47].z=-x-0.5;         // -4^-1    [0,1,0]         0,0,0           -1/4,-1/4,-1/4
      break;

    case 523: // [225] F m -3 m (-F 4 2 3)
      SpaceGroupSize=192;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=z;              SpaceGroupElement[9].z=y;               // 2        [0,1,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;              // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=y;              // 4^1      [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=z;             SpaceGroupElement[13].z=-y;             // 4^-1     [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-y;             // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=-y;             // 2        [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[18].x=z;             SpaceGroupElement[18].y=y;             SpaceGroupElement[18].z=-x;             // 4^1      [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-x;             // 2        [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=x;              // 2        [1,0,1]         0,0,0           0,0,0
      SpaceGroupElement[23].x=-z;            SpaceGroupElement[23].y=y;             SpaceGroupElement[23].z=x;              // 4^-1     [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[25].x=y;             SpaceGroupElement[25].y=-x;            SpaceGroupElement[25].z=-z;             // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=y;             SpaceGroupElement[26].z=-z;             // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[27].x=-y;            SpaceGroupElement[27].y=x;             SpaceGroupElement[27].z=-z;             // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=y;             SpaceGroupElement[28].z=z;              // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[29].x=-y;            SpaceGroupElement[29].y=-x;            SpaceGroupElement[29].z=z;              // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[30].x=x;             SpaceGroupElement[30].y=-y;            SpaceGroupElement[30].z=z;              // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=z;              // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[32].x=-z;            SpaceGroupElement[32].y=-x;            SpaceGroupElement[32].z=-y;             // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[33].x=x;             SpaceGroupElement[33].y=-z;            SpaceGroupElement[33].z=-y;             // -2       [0,1,1]         0,0,0           0,0,0
      SpaceGroupElement[34].x=z;             SpaceGroupElement[34].y=x;             SpaceGroupElement[34].z=-y;             // -3^-1    [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[35].x=-x;            SpaceGroupElement[35].y=z;             SpaceGroupElement[35].z=-y;             // -4^1     [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[36].x=-z;            SpaceGroupElement[36].y=x;             SpaceGroupElement[36].z=y;              // -3^-1    [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[37].x=-x;            SpaceGroupElement[37].y=-z;            SpaceGroupElement[37].z=y;              // -4^-1    [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[38].x=z;             SpaceGroupElement[38].y=-x;            SpaceGroupElement[38].z=y;              // -3^1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=z;             SpaceGroupElement[39].z=y;              // -2       [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[40].x=-y;            SpaceGroupElement[40].y=-z;            SpaceGroupElement[40].z=-x;             // -3^-1    [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[41].x=-y;            SpaceGroupElement[41].y=z;             SpaceGroupElement[41].z=x;              // -3^-1    [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[42].x=-z;            SpaceGroupElement[42].y=-y;            SpaceGroupElement[42].z=x;              // -4^1     [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[43].x=y;             SpaceGroupElement[43].y=-z;            SpaceGroupElement[43].z=x;              // -3^1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[44].x=z;             SpaceGroupElement[44].y=y;             SpaceGroupElement[44].z=x;              // -2       [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[45].x=y;             SpaceGroupElement[45].y=z;             SpaceGroupElement[45].z=-x;             // -3^1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[46].x=-z;            SpaceGroupElement[46].y=y;             SpaceGroupElement[46].z=-x;             // -2       [1,0,1]         0,0,0           0,0,0
      SpaceGroupElement[47].x=z;             SpaceGroupElement[47].y=-y;            SpaceGroupElement[47].z=-x;             // -4^-1    [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[48].x=x;             SpaceGroupElement[48].y=y+0.5;         SpaceGroupElement[48].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[49].x=-y;            SpaceGroupElement[49].y=x+0.5;         SpaceGroupElement[49].z=z+0.5;          // 4^1      [0,0,1]         0,0,1/2         -1/4,1/4,0
      SpaceGroupElement[50].x=-x;            SpaceGroupElement[50].y=-y+0.5;        SpaceGroupElement[50].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[51].x=y;             SpaceGroupElement[51].y=-x+0.5;        SpaceGroupElement[51].z=z+0.5;          // 4^-1     [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[52].x=x;             SpaceGroupElement[52].y=-y+0.5;        SpaceGroupElement[52].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[53].x=y;             SpaceGroupElement[53].y=x+0.5;         SpaceGroupElement[53].z=-z+0.5;         // 2        [1,1,0]         1/4,1/4,0       -1/4,0,1/4
      SpaceGroupElement[54].x=-x;            SpaceGroupElement[54].y=y+0.5;         SpaceGroupElement[54].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[55].x=-y;            SpaceGroupElement[55].y=-x+0.5;        SpaceGroupElement[55].z=-z+0.5;         // 2        [-1,1,0]        -1/4,1/4,0      1/4,0,1/4
      SpaceGroupElement[56].x=z;             SpaceGroupElement[56].y=x+0.5;         SpaceGroupElement[56].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     -1/3,-1/6,0
      SpaceGroupElement[57].x=-x;            SpaceGroupElement[57].y=z+0.5;         SpaceGroupElement[57].z=y+0.5;          // 2        [0,1,1]         0,1/2,1/2       0,0,0
      SpaceGroupElement[58].x=-z;            SpaceGroupElement[58].y=-x+0.5;        SpaceGroupElement[58].z=y+0.5;          // 3^-1     [-1,1,1]        -1/3,1/3,1/3    1/3,-1/6,0
      SpaceGroupElement[59].x=x;             SpaceGroupElement[59].y=-z+0.5;        SpaceGroupElement[59].z=y+0.5;          // 4^1      [1,0,0]         0,0,0           0,0,1/2
      SpaceGroupElement[60].x=z;             SpaceGroupElement[60].y=-x+0.5;        SpaceGroupElement[60].z=-y+0.5;         // 3^-1     [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[61].x=x;             SpaceGroupElement[61].y=z+0.5;         SpaceGroupElement[61].z=-y+0.5;         // 4^-1     [1,0,0]         0,0,0           0,1/2,0
      SpaceGroupElement[62].x=-z;            SpaceGroupElement[62].y=x+0.5;         SpaceGroupElement[62].z=-y+0.5;         // 3^1      [-1,-1,1]       0,0,0           0,1/2,0
      SpaceGroupElement[63].x=-x;            SpaceGroupElement[63].y=-z+0.5;        SpaceGroupElement[63].z=-y+0.5;         // 2        [0,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[64].x=y;             SpaceGroupElement[64].y=z+0.5;         SpaceGroupElement[64].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,1/6,0
      SpaceGroupElement[65].x=y;             SpaceGroupElement[65].y=-z+0.5;        SpaceGroupElement[65].z=-x+0.5;         // 3^-1     [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[66].x=z;             SpaceGroupElement[66].y=y+0.5;         SpaceGroupElement[66].z=-x+0.5;         // 4^1      [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[67].x=-y;            SpaceGroupElement[67].y=z+0.5;         SpaceGroupElement[67].z=-x+0.5;         // 3^1      [-1,1,1]        -1/3,1/3,1/3    1/6,1/6,0
      SpaceGroupElement[68].x=-z;            SpaceGroupElement[68].y=-y+0.5;        SpaceGroupElement[68].z=-x+0.5;         // 2        [-1,0,1]        -1/4,0,1/4      1/4,1/4,0
      SpaceGroupElement[69].x=-y;            SpaceGroupElement[69].y=-z+0.5;        SpaceGroupElement[69].z=x+0.5;          // 3^1      [1,-1,1]        0,0,0           -1/2,1/2,0
      SpaceGroupElement[70].x=z;             SpaceGroupElement[70].y=-y+0.5;        SpaceGroupElement[70].z=x+0.5;          // 2        [1,0,1]         1/4,0,1/4       -1/4,1/4,0
      SpaceGroupElement[71].x=-z;            SpaceGroupElement[71].y=y+0.5;         SpaceGroupElement[71].z=x+0.5;          // 4^-1     [0,1,0]         0,1/2,0         -1/4,0,1/4
      SpaceGroupElement[72].x=-x;            SpaceGroupElement[72].y=-y+0.5;        SpaceGroupElement[72].z=-z+0.5;         // -1       -               -               0,1/4,1/4
      SpaceGroupElement[73].x=y;             SpaceGroupElement[73].y=-x+0.5;        SpaceGroupElement[73].z=-z+0.5;         // -4^1     [0,0,1]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[74].x=x;             SpaceGroupElement[74].y=y+0.5;         SpaceGroupElement[74].z=-z+0.5;         // -2       [0,0,1]         0,1/2,0         0,0,1/4
      SpaceGroupElement[75].x=-y;            SpaceGroupElement[75].y=x+0.5;         SpaceGroupElement[75].z=-z+0.5;         // -4^-1    [0,0,1]         0,0,0           -1/4,1/4,1/4
      SpaceGroupElement[76].x=-x;            SpaceGroupElement[76].y=y+0.5;         SpaceGroupElement[76].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       0,0,0
      SpaceGroupElement[77].x=-y;            SpaceGroupElement[77].y=-x+0.5;        SpaceGroupElement[77].z=z+0.5;          // -2       [1,1,0]         -1/4,1/4,1/2    1/4,0,0
      SpaceGroupElement[78].x=x;             SpaceGroupElement[78].y=-y+0.5;        SpaceGroupElement[78].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,1/4,0
      SpaceGroupElement[79].x=y;             SpaceGroupElement[79].y=x+0.5;         SpaceGroupElement[79].z=z+0.5;          // -2       [-1,1,0]        1/4,1/4,1/2     -1/4,0,0
      SpaceGroupElement[80].x=-z;            SpaceGroupElement[80].y=-x+0.5;        SpaceGroupElement[80].z=-y+0.5;         // -3^1     [1,1,1]         0,0,0           0,1/2,0
      SpaceGroupElement[81].x=x;             SpaceGroupElement[81].y=-z+0.5;        SpaceGroupElement[81].z=-y+0.5;         // -2       [0,1,1]         0,0,0           0,1/2,0
      SpaceGroupElement[82].x=z;             SpaceGroupElement[82].y=x+0.5;         SpaceGroupElement[82].z=-y+0.5;         // -3^-1    [-1,1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[83].x=-x;            SpaceGroupElement[83].y=z+0.5;         SpaceGroupElement[83].z=-y+0.5;         // -4^1     [1,0,0]         0,0,0           0,1/2,0
      SpaceGroupElement[84].x=-z;            SpaceGroupElement[84].y=x+0.5;         SpaceGroupElement[84].z=y+0.5;          // -3^-1    [1,-1,1]        0,0,0           -1/2,0,1/2
      SpaceGroupElement[85].x=-x;            SpaceGroupElement[85].y=-z+0.5;        SpaceGroupElement[85].z=y+0.5;          // -4^-1    [1,0,0]         0,0,0           0,0,1/2
      SpaceGroupElement[86].x=z;             SpaceGroupElement[86].y=-x+0.5;        SpaceGroupElement[86].z=y+0.5;          // -3^1     [-1,-1,1]       0,0,0           1/2,0,1/2
      SpaceGroupElement[87].x=x;             SpaceGroupElement[87].y=z+0.5;         SpaceGroupElement[87].z=y+0.5;          // -2       [0,-1,1]        0,1/2,1/2       0,0,0
      SpaceGroupElement[88].x=-y;            SpaceGroupElement[88].y=-z+0.5;        SpaceGroupElement[88].z=-x+0.5;         // -3^-1    [1,1,1]         0,0,0           0,0,1/2
      SpaceGroupElement[89].x=-y;            SpaceGroupElement[89].y=z+0.5;         SpaceGroupElement[89].z=x+0.5;          // -3^-1    [-1,-1,1]       0,0,0           -1/2,1/2,0
      SpaceGroupElement[90].x=-z;            SpaceGroupElement[90].y=-y+0.5;        SpaceGroupElement[90].z=x+0.5;          // -4^1     [0,1,0]         0,0,0           -1/4,1/4,1/4
      SpaceGroupElement[91].x=y;             SpaceGroupElement[91].y=-z+0.5;        SpaceGroupElement[91].z=x+0.5;          // -3^1     [-1,1,1]        0,0,0           0,0,1/2
      SpaceGroupElement[92].x=z;             SpaceGroupElement[92].y=y+0.5;         SpaceGroupElement[92].z=x+0.5;          // -2       [-1,0,1]        1/4,1/2,1/4     -1/4,0,0
      SpaceGroupElement[93].x=y;             SpaceGroupElement[93].y=z+0.5;         SpaceGroupElement[93].z=-x+0.5;         // -3^1     [1,-1,1]        0,0,0           1/2,1/2,0
      SpaceGroupElement[94].x=-z;            SpaceGroupElement[94].y=y+0.5;         SpaceGroupElement[94].z=-x+0.5;         // -2       [1,0,1]         -1/4,1/2,1/4    1/4,0,0
      SpaceGroupElement[95].x=z;             SpaceGroupElement[95].y=-y+0.5;        SpaceGroupElement[95].z=-x+0.5;         // -4^-1    [0,1,0]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[96].x=x+0.5;         SpaceGroupElement[96].y=y;             SpaceGroupElement[96].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[97].x=-y+0.5;        SpaceGroupElement[97].y=x;             SpaceGroupElement[97].z=z+0.5;          // 4^1      [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[98].x=-x+0.5;        SpaceGroupElement[98].y=-y;            SpaceGroupElement[98].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[99].x=y+0.5;         SpaceGroupElement[99].y=-x;            SpaceGroupElement[99].z=z+0.5;          // 4^-1     [0,0,1]         0,0,1/2         1/4,-1/4,0
      SpaceGroupElement[100].x=x+0.5;        SpaceGroupElement[100].y=-y;           SpaceGroupElement[100].z=-z+0.5;        // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[101].x=y+0.5;        SpaceGroupElement[101].y=x;            SpaceGroupElement[101].z=-z+0.5;        // 2        [1,1,0]         1/4,1/4,0       1/4,0,1/4
      SpaceGroupElement[102].x=-x+0.5;       SpaceGroupElement[102].y=y;            SpaceGroupElement[102].z=-z+0.5;        // 2        [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[103].x=-y+0.5;       SpaceGroupElement[103].y=-x;           SpaceGroupElement[103].z=-z+0.5;        // 2        [-1,1,0]        1/4,-1/4,0      1/4,0,1/4
      SpaceGroupElement[104].x=z+0.5;        SpaceGroupElement[104].y=x;            SpaceGroupElement[104].z=y+0.5;         // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,-1/6,0
      SpaceGroupElement[105].x=-x+0.5;       SpaceGroupElement[105].y=z;            SpaceGroupElement[105].z=y+0.5;         // 2        [0,1,1]         0,1/4,1/4       1/4,-1/4,0
      SpaceGroupElement[106].x=-z+0.5;       SpaceGroupElement[106].y=-x;           SpaceGroupElement[106].z=y+0.5;         // 3^-1     [-1,1,1]        0,0,0           1/2,-1/2,0
      SpaceGroupElement[107].x=x+0.5;        SpaceGroupElement[107].y=-z;           SpaceGroupElement[107].z=y+0.5;         // 4^1      [1,0,0]         1/2,0,0         0,-1/4,1/4
      SpaceGroupElement[108].x=z+0.5;        SpaceGroupElement[108].y=-x;           SpaceGroupElement[108].z=-y+0.5;        // 3^-1     [1,-1,1]        1/3,-1/3,1/3    1/6,1/6,0
      SpaceGroupElement[109].x=x+0.5;        SpaceGroupElement[109].y=z;            SpaceGroupElement[109].z=-y+0.5;        // 4^-1     [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[110].x=-z+0.5;       SpaceGroupElement[110].y=x;            SpaceGroupElement[110].z=-y+0.5;        // 3^1      [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[111].x=-x+0.5;       SpaceGroupElement[111].y=-z;           SpaceGroupElement[111].z=-y+0.5;        // 2        [0,-1,1]        0,-1/4,1/4      1/4,1/4,0
      SpaceGroupElement[112].x=y+0.5;        SpaceGroupElement[112].y=z;            SpaceGroupElement[112].z=x+0.5;         // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,-1/3,0
      SpaceGroupElement[113].x=y+0.5;        SpaceGroupElement[113].y=-z;           SpaceGroupElement[113].z=-x+0.5;        // 3^-1     [-1,-1,1]       0,0,0           1/2,0,0
      SpaceGroupElement[114].x=z+0.5;        SpaceGroupElement[114].y=y;            SpaceGroupElement[114].z=-x+0.5;        // 4^1      [0,1,0]         0,0,0           1/2,0,0
      SpaceGroupElement[115].x=-y+0.5;       SpaceGroupElement[115].y=z;            SpaceGroupElement[115].z=-x+0.5;        // 3^1      [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[116].x=-z+0.5;       SpaceGroupElement[116].y=-y;           SpaceGroupElement[116].z=-x+0.5;        // 2        [-1,0,1]        0,0,0           1/2,0,0
      SpaceGroupElement[117].x=-y+0.5;       SpaceGroupElement[117].y=-z;           SpaceGroupElement[117].z=x+0.5;         // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      SpaceGroupElement[118].x=z+0.5;        SpaceGroupElement[118].y=-y;           SpaceGroupElement[118].z=x+0.5;         // 2        [1,0,1]         1/2,0,1/2       0,0,0
      SpaceGroupElement[119].x=-z+0.5;       SpaceGroupElement[119].y=y;            SpaceGroupElement[119].z=x+0.5;         // 4^-1     [0,1,0]         0,0,0           0,0,1/2
      SpaceGroupElement[120].x=-x+0.5;       SpaceGroupElement[120].y=-y;           SpaceGroupElement[120].z=-z+0.5;        // -1       -               -               1/4,0,1/4
      SpaceGroupElement[121].x=y+0.5;        SpaceGroupElement[121].y=-x;           SpaceGroupElement[121].z=-z+0.5;        // -4^1     [0,0,1]         0,0,0           1/4,-1/4,1/4
      SpaceGroupElement[122].x=x+0.5;        SpaceGroupElement[122].y=y;            SpaceGroupElement[122].z=-z+0.5;        // -2       [0,0,1]         1/2,0,0         0,0,1/4
      SpaceGroupElement[123].x=-y+0.5;       SpaceGroupElement[123].y=x;            SpaceGroupElement[123].z=-z+0.5;        // -4^-1    [0,0,1]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[124].x=-x+0.5;       SpaceGroupElement[124].y=y;            SpaceGroupElement[124].z=z+0.5;         // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[125].x=-y+0.5;       SpaceGroupElement[125].y=-x;           SpaceGroupElement[125].z=z+0.5;         // -2       [1,1,0]         1/4,-1/4,1/2    1/4,0,0
      SpaceGroupElement[126].x=x+0.5;        SpaceGroupElement[126].y=-y;           SpaceGroupElement[126].z=z+0.5;         // -2       [0,1,0]         1/2,0,1/2       0,0,0
      SpaceGroupElement[127].x=y+0.5;        SpaceGroupElement[127].y=x;            SpaceGroupElement[127].z=z+0.5;         // -2       [-1,1,0]        1/4,1/4,1/2     1/4,0,0
      SpaceGroupElement[128].x=-z+0.5;       SpaceGroupElement[128].y=-x;           SpaceGroupElement[128].z=-y+0.5;        // -3^1     [1,1,1]         0,0,0           0,0,1/2
      SpaceGroupElement[129].x=x+0.5;        SpaceGroupElement[129].y=-z;           SpaceGroupElement[129].z=-y+0.5;        // -2       [0,1,1]         1/2,-1/4,1/4    0,1/4,0
      SpaceGroupElement[130].x=z+0.5;        SpaceGroupElement[130].y=x;            SpaceGroupElement[130].z=-y+0.5;        // -3^-1    [-1,1,1]        0,0,0           1/2,1/2,0
      SpaceGroupElement[131].x=-x+0.5;       SpaceGroupElement[131].y=z;            SpaceGroupElement[131].z=-y+0.5;        // -4^1     [1,0,0]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[132].x=-z+0.5;       SpaceGroupElement[132].y=x;            SpaceGroupElement[132].z=y+0.5;         // -3^-1    [1,-1,1]        0,0,0           0,0,1/2
      SpaceGroupElement[133].x=-x+0.5;       SpaceGroupElement[133].y=-z;           SpaceGroupElement[133].z=y+0.5;         // -4^-1    [1,0,0]         0,0,0           1/4,-1/4,1/4
      SpaceGroupElement[134].x=z+0.5;        SpaceGroupElement[134].y=-x;           SpaceGroupElement[134].z=y+0.5;         // -3^1     [-1,-1,1]       0,0,0           1/2,-1/2,0
      SpaceGroupElement[135].x=x+0.5;        SpaceGroupElement[135].y=z;            SpaceGroupElement[135].z=y+0.5;         // -2       [0,-1,1]        1/2,1/4,1/4     0,-1/4,0
      SpaceGroupElement[136].x=-y+0.5;       SpaceGroupElement[136].y=-z;           SpaceGroupElement[136].z=-x+0.5;        // -3^-1    [1,1,1]         0,0,0           1/2,0,0
      SpaceGroupElement[137].x=-y+0.5;       SpaceGroupElement[137].y=z;            SpaceGroupElement[137].z=x+0.5;         // -3^-1    [-1,-1,1]       0,0,0           0,1/2,1/2
      SpaceGroupElement[138].x=-z+0.5;       SpaceGroupElement[138].y=-y;           SpaceGroupElement[138].z=x+0.5;         // -4^1     [0,1,0]         0,0,0           0,0,1/2
      SpaceGroupElement[139].x=y+0.5;        SpaceGroupElement[139].y=-z;           SpaceGroupElement[139].z=x+0.5;         // -3^1     [-1,1,1]        0,0,0           0,-1/2,1/2
      SpaceGroupElement[140].x=z+0.5;        SpaceGroupElement[140].y=y;            SpaceGroupElement[140].z=x+0.5;         // -2       [-1,0,1]        1/2,0,1/2       0,0,0
      SpaceGroupElement[141].x=y+0.5;        SpaceGroupElement[141].y=z;            SpaceGroupElement[141].z=-x+0.5;        // -3^1     [1,-1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[142].x=-z+0.5;       SpaceGroupElement[142].y=y;            SpaceGroupElement[142].z=-x+0.5;        // -2       [1,0,1]         0,0,0           1/2,0,0
      SpaceGroupElement[143].x=z+0.5;        SpaceGroupElement[143].y=-y;           SpaceGroupElement[143].z=-x+0.5;        // -4^-1    [0,1,0]         0,0,0           1/2,0,0
      SpaceGroupElement[144].x=x+0.5;        SpaceGroupElement[144].y=y+0.5;        SpaceGroupElement[144].z=z;             // 1        -               -               -
      SpaceGroupElement[145].x=-y+0.5;       SpaceGroupElement[145].y=x+0.5;        SpaceGroupElement[145].z=z;             // 4^1      [0,0,1]         0,0,0           0,1/2,0
      SpaceGroupElement[146].x=-x+0.5;       SpaceGroupElement[146].y=-y+0.5;       SpaceGroupElement[146].z=z;             // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[147].x=y+0.5;        SpaceGroupElement[147].y=-x+0.5;       SpaceGroupElement[147].z=z;             // 4^-1     [0,0,1]         0,0,0           1/2,0,0
      SpaceGroupElement[148].x=x+0.5;        SpaceGroupElement[148].y=-y+0.5;       SpaceGroupElement[148].z=-z;            // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[149].x=y+0.5;        SpaceGroupElement[149].y=x+0.5;        SpaceGroupElement[149].z=-z;            // 2        [1,1,0]         1/2,1/2,0       0,0,0
      SpaceGroupElement[150].x=-x+0.5;       SpaceGroupElement[150].y=y+0.5;        SpaceGroupElement[150].z=-z;            // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[151].x=-y+0.5;       SpaceGroupElement[151].y=-x+0.5;       SpaceGroupElement[151].z=-z;            // 2        [-1,1,0]        0,0,0           1/2,0,0
      SpaceGroupElement[152].x=z+0.5;        SpaceGroupElement[152].y=x+0.5;        SpaceGroupElement[152].z=y;             // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,1/3,0
      SpaceGroupElement[153].x=-x+0.5;       SpaceGroupElement[153].y=z+0.5;        SpaceGroupElement[153].z=y;             // 2        [0,1,1]         0,1/4,1/4       1/4,1/4,0
      SpaceGroupElement[154].x=-z+0.5;       SpaceGroupElement[154].y=-x+0.5;       SpaceGroupElement[154].z=y;             // 3^-1     [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[155].x=x+0.5;        SpaceGroupElement[155].y=-z+0.5;       SpaceGroupElement[155].z=y;             // 4^1      [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[156].x=z+0.5;        SpaceGroupElement[156].y=-x+0.5;       SpaceGroupElement[156].z=-y;            // 3^-1     [1,-1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[157].x=x+0.5;        SpaceGroupElement[157].y=z+0.5;        SpaceGroupElement[157].z=-y;            // 4^-1     [1,0,0]         1/2,0,0         0,1/4,-1/4
      SpaceGroupElement[158].x=-z+0.5;       SpaceGroupElement[158].y=x+0.5;        SpaceGroupElement[158].z=-y;            // 3^1      [-1,-1,1]       1/3,1/3,-1/3    1/6,1/3,0
      SpaceGroupElement[159].x=-x+0.5;       SpaceGroupElement[159].y=-z+0.5;       SpaceGroupElement[159].z=-y;            // 2        [0,-1,1]        0,1/4,-1/4      1/4,1/4,0
      SpaceGroupElement[160].x=y+0.5;        SpaceGroupElement[160].y=z+0.5;        SpaceGroupElement[160].z=x;             // 3^-1     [1,1,1]         1/3,1/3,1/3     1/3,1/6,0
      SpaceGroupElement[161].x=y+0.5;        SpaceGroupElement[161].y=-z+0.5;       SpaceGroupElement[161].z=-x;            // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    1/3,1/6,0
      SpaceGroupElement[162].x=z+0.5;        SpaceGroupElement[162].y=y+0.5;        SpaceGroupElement[162].z=-x;            // 4^1      [0,1,0]         0,1/2,0         1/4,0,-1/4
      SpaceGroupElement[163].x=-y+0.5;       SpaceGroupElement[163].y=z+0.5;        SpaceGroupElement[163].z=-x;            // 3^1      [-1,1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[164].x=-z+0.5;       SpaceGroupElement[164].y=-y+0.5;       SpaceGroupElement[164].z=-x;            // 2        [-1,0,1]        1/4,0,-1/4      1/4,1/4,0
      SpaceGroupElement[165].x=-y+0.5;       SpaceGroupElement[165].y=-z+0.5;       SpaceGroupElement[165].z=x;             // 3^1      [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[166].x=z+0.5;        SpaceGroupElement[166].y=-y+0.5;       SpaceGroupElement[166].z=x;             // 2        [1,0,1]         1/4,0,1/4       1/4,1/4,0
      SpaceGroupElement[167].x=-z+0.5;       SpaceGroupElement[167].y=y+0.5;        SpaceGroupElement[167].z=x;             // 4^-1     [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[168].x=-x+0.5;       SpaceGroupElement[168].y=-y+0.5;       SpaceGroupElement[168].z=-z;            // -1       -               -               1/4,1/4,0
      SpaceGroupElement[169].x=y+0.5;        SpaceGroupElement[169].y=-x+0.5;       SpaceGroupElement[169].z=-z;            // -4^1     [0,0,1]         0,0,0           1/2,0,0
      SpaceGroupElement[170].x=x+0.5;        SpaceGroupElement[170].y=y+0.5;        SpaceGroupElement[170].z=-z;            // -2       [0,0,1]         1/2,1/2,0       0,0,0
      SpaceGroupElement[171].x=-y+0.5;       SpaceGroupElement[171].y=x+0.5;        SpaceGroupElement[171].z=-z;            // -4^-1    [0,0,1]         0,0,0           0,1/2,0
      SpaceGroupElement[172].x=-x+0.5;       SpaceGroupElement[172].y=y+0.5;        SpaceGroupElement[172].z=z;             // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[173].x=-y+0.5;       SpaceGroupElement[173].y=-x+0.5;       SpaceGroupElement[173].z=z;             // -2       [1,1,0]         0,0,0           1/2,0,0
      SpaceGroupElement[174].x=x+0.5;        SpaceGroupElement[174].y=-y+0.5;       SpaceGroupElement[174].z=z;             // -2       [0,1,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[175].x=y+0.5;        SpaceGroupElement[175].y=x+0.5;        SpaceGroupElement[175].z=z;             // -2       [-1,1,0]        1/2,1/2,0       0,0,0
      SpaceGroupElement[176].x=-z+0.5;       SpaceGroupElement[176].y=-x+0.5;       SpaceGroupElement[176].z=-y;            // -3^1     [1,1,1]         0,0,0           1/2,0,0
      SpaceGroupElement[177].x=x+0.5;        SpaceGroupElement[177].y=-z+0.5;       SpaceGroupElement[177].z=-y;            // -2       [0,1,1]         1/2,1/4,-1/4    0,1/4,0
      SpaceGroupElement[178].x=z+0.5;        SpaceGroupElement[178].y=x+0.5;        SpaceGroupElement[178].z=-y;            // -3^-1    [-1,1,1]        0,0,0           0,1/2,-1/2
      SpaceGroupElement[179].x=-x+0.5;       SpaceGroupElement[179].y=z+0.5;        SpaceGroupElement[179].z=-y;            // -4^1     [1,0,0]         0,0,0           1/4,1/4,-1/4
      SpaceGroupElement[180].x=-z+0.5;       SpaceGroupElement[180].y=x+0.5;        SpaceGroupElement[180].z=y;             // -3^-1    [1,-1,1]        0,0,0           0,1/2,1/2
      SpaceGroupElement[181].x=-x+0.5;       SpaceGroupElement[181].y=-z+0.5;       SpaceGroupElement[181].z=y;             // -4^-1    [1,0,0]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[182].x=z+0.5;        SpaceGroupElement[182].y=-x+0.5;       SpaceGroupElement[182].z=y;             // -3^1     [-1,-1,1]       0,0,0           1/2,0,0
      SpaceGroupElement[183].x=x+0.5;        SpaceGroupElement[183].y=z+0.5;        SpaceGroupElement[183].z=y;             // -2       [0,-1,1]        1/2,1/4,1/4     0,1/4,0
      SpaceGroupElement[184].x=-y+0.5;       SpaceGroupElement[184].y=-z+0.5;       SpaceGroupElement[184].z=-x;            // -3^-1    [1,1,1]         0,0,0           0,1/2,0
      SpaceGroupElement[185].x=-y+0.5;       SpaceGroupElement[185].y=z+0.5;        SpaceGroupElement[185].z=x;             // -3^-1    [-1,-1,1]       0,0,0           0,1/2,0
      SpaceGroupElement[186].x=-z+0.5;       SpaceGroupElement[186].y=-y+0.5;       SpaceGroupElement[186].z=x;             // -4^1     [0,1,0]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[187].x=y+0.5;        SpaceGroupElement[187].y=-z+0.5;       SpaceGroupElement[187].z=x;             // -3^1     [-1,1,1]        0,0,0           1/2,0,1/2
      SpaceGroupElement[188].x=z+0.5;        SpaceGroupElement[188].y=y+0.5;        SpaceGroupElement[188].z=x;             // -2       [-1,0,1]        1/4,1/2,1/4     1/4,0,0
      SpaceGroupElement[189].x=y+0.5;        SpaceGroupElement[189].y=z+0.5;        SpaceGroupElement[189].z=-x;            // -3^1     [1,-1,1]        0,0,0           1/2,0,-1/2
      SpaceGroupElement[190].x=-z+0.5;       SpaceGroupElement[190].y=y+0.5;        SpaceGroupElement[190].z=-x;            // -2       [1,0,1]         1/4,1/2,-1/4    1/4,0,0
      SpaceGroupElement[191].x=z+0.5;        SpaceGroupElement[191].y=-y+0.5;       SpaceGroupElement[191].z=-x;            // -4^-1    [0,1,0]         0,0,0           1/4,1/4,-1/4
      break;

    case 524: // [226] F m -3 c (-F 4a 2 3)
      SpaceGroupSize=192;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y+0.5;          SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         1/4,1/4,0       1/4,0,0
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        -1/4,1/4,0      1/4,0,0
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=z;              SpaceGroupElement[9].z=y;               // 2        [0,1,1]         0,0,0           1/4,0,0
      SpaceGroupElement[10].x=-z+0.5;        SpaceGroupElement[10].y=-x+0.5;        SpaceGroupElement[10].z=y;              // 3^-1     [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=-z+0.5;        SpaceGroupElement[11].z=y;              // 4^1      [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[13].x=x+0.5;         SpaceGroupElement[13].y=z;             SpaceGroupElement[13].z=-y;             // 4^-1     [1,0,0]         1/2,0,0         0,0,0
      SpaceGroupElement[14].x=-z+0.5;        SpaceGroupElement[14].y=x+0.5;         SpaceGroupElement[14].z=-y;             // 3^1      [-1,-1,1]       1/3,1/3,-1/3    1/6,1/3,0
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z+0.5;        SpaceGroupElement[15].z=-y;             // 2        [0,-1,1]        0,1/4,-1/4      0,1/4,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z+0.5;        SpaceGroupElement[17].z=-x+0.5;         // 3^-1     [-1,-1,1]       0,0,0           1/2,1/2,0
      SpaceGroupElement[18].x=z;             SpaceGroupElement[18].y=y;             SpaceGroupElement[18].z=-x+0.5;         // 4^1      [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[19].x=-y+0.5;        SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=-x+0.5;         // 3^1      [-1,1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[20].x=-z+0.5;        SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-x;             // 2        [-1,0,1]        1/4,0,-1/4      1/4,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=x+0.5;          // 2        [1,0,1]         1/4,0,1/4       -1/4,0,0
      SpaceGroupElement[23].x=-z+0.5;        SpaceGroupElement[23].y=y+0.5;         SpaceGroupElement[23].z=x+0.5;          // 4^-1     [0,1,0]         0,1/2,0         0,0,1/2
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[25].x=y-0.5;         SpaceGroupElement[25].y=-x;            SpaceGroupElement[25].z=-z;             // -4^1     [0,0,1]         0,0,0           -1/4,1/4,0
      SpaceGroupElement[26].x=x-0.5;         SpaceGroupElement[26].y=y-0.5;         SpaceGroupElement[26].z=-z;             // -2       [0,0,1]         -1/2,-1/2,0     0,0,0
      SpaceGroupElement[27].x=-y;            SpaceGroupElement[27].y=x-0.5;         SpaceGroupElement[27].z=-z;             // -4^-1    [0,0,1]         0,0,0           1/4,-1/4,0
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=y;             SpaceGroupElement[28].z=z;              // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[29].x=-y-0.5;        SpaceGroupElement[29].y=-x;            SpaceGroupElement[29].z=z;              // -2       [1,1,0]         -1/4,1/4,0      -1/4,0,0
      SpaceGroupElement[30].x=x-0.5;         SpaceGroupElement[30].y=-y-0.5;        SpaceGroupElement[30].z=z;              // -2       [0,1,0]         -1/2,0,0        0,-1/4,0
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x-0.5;         SpaceGroupElement[31].z=z;              // -2       [-1,1,0]        -1/4,-1/4,0     1/4,0,0
      SpaceGroupElement[32].x=-z;            SpaceGroupElement[32].y=-x;            SpaceGroupElement[32].z=-y;             // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[33].x=x-0.5;         SpaceGroupElement[33].y=-z;            SpaceGroupElement[33].z=-y;             // -2       [0,1,1]         -1/2,0,0        0,0,0
      SpaceGroupElement[34].x=z-0.5;         SpaceGroupElement[34].y=x-0.5;         SpaceGroupElement[34].z=-y;             // -3^-1    [-1,1,1]        0,0,0           0,-1/2,1/2
      SpaceGroupElement[35].x=-x;            SpaceGroupElement[35].y=z-0.5;         SpaceGroupElement[35].z=-y;             // -4^1     [1,0,0]         0,0,0           0,-1/4,1/4
      SpaceGroupElement[36].x=-z;            SpaceGroupElement[36].y=x;             SpaceGroupElement[36].z=y;              // -3^-1    [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[37].x=-x-0.5;        SpaceGroupElement[37].y=-z;            SpaceGroupElement[37].z=y;              // -4^-1    [1,0,0]         0,0,0           -1/4,0,0
      SpaceGroupElement[38].x=z-0.5;         SpaceGroupElement[38].y=-x-0.5;        SpaceGroupElement[38].z=y;              // -3^1     [-1,-1,1]       0,0,0           -1/2,0,0
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=z-0.5;         SpaceGroupElement[39].z=y;              // -2       [0,-1,1]        0,-1/4,-1/4     0,-1/4,0
      SpaceGroupElement[40].x=-y;            SpaceGroupElement[40].y=-z;            SpaceGroupElement[40].z=-x;             // -3^-1    [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[41].x=-y;            SpaceGroupElement[41].y=z-0.5;         SpaceGroupElement[41].z=x-0.5;          // -3^-1    [-1,-1,1]       0,0,0           1/2,-1/2,0
      SpaceGroupElement[42].x=-z;            SpaceGroupElement[42].y=-y;            SpaceGroupElement[42].z=x-0.5;          // -4^1     [0,1,0]         0,0,0           1/4,0,-1/4
      SpaceGroupElement[43].x=y-0.5;         SpaceGroupElement[43].y=-z;            SpaceGroupElement[43].z=x-0.5;          // -3^1     [-1,1,1]        0,0,0           0,1/2,-1/2
      SpaceGroupElement[44].x=z-0.5;         SpaceGroupElement[44].y=y;             SpaceGroupElement[44].z=x;              // -2       [-1,0,1]        -1/4,0,-1/4     -1/4,0,0
      SpaceGroupElement[45].x=y;             SpaceGroupElement[45].y=z;             SpaceGroupElement[45].z=-x;             // -3^1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[46].x=-z;            SpaceGroupElement[46].y=y;             SpaceGroupElement[46].z=-x-0.5;         // -2       [1,0,1]         1/4,0,-1/4      -1/4,0,0
      SpaceGroupElement[47].x=z-0.5;         SpaceGroupElement[47].y=-y-0.5;        SpaceGroupElement[47].z=-x-0.5;         // -4^-1    [0,1,0]         0,0,0           -1/2,-1/4,0
      SpaceGroupElement[48].x=x;             SpaceGroupElement[48].y=y+0.5;         SpaceGroupElement[48].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[49].x=-y+0.5;        SpaceGroupElement[49].y=x+0.5;         SpaceGroupElement[49].z=z+0.5;          // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[50].x=-x+0.5;        SpaceGroupElement[50].y=-y;            SpaceGroupElement[50].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/2,0
      SpaceGroupElement[51].x=y;             SpaceGroupElement[51].y=-x;            SpaceGroupElement[51].z=z+0.5;          // 4^-1     [0,0,1]         0,0,1/2         1/2,1/2,0
      SpaceGroupElement[52].x=x;             SpaceGroupElement[52].y=-y+0.5;        SpaceGroupElement[52].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[53].x=y+0.5;         SpaceGroupElement[53].y=x+0.5;         SpaceGroupElement[53].z=-z+0.5;         // 2        [1,1,0]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[54].x=-x+0.5;        SpaceGroupElement[54].y=y;             SpaceGroupElement[54].z=-z+0.5;         // 2        [0,1,0]         0,1,0           1/4,0,1/4
      SpaceGroupElement[55].x=-y;            SpaceGroupElement[55].y=-x;            SpaceGroupElement[55].z=-z+0.5;         // 2        [-1,1,0]        -1/2,1/2,0      1/2,0,1/4
      SpaceGroupElement[56].x=z;             SpaceGroupElement[56].y=x+0.5;         SpaceGroupElement[56].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     -1/3,-1/6,0
      SpaceGroupElement[57].x=-x+0.5;        SpaceGroupElement[57].y=z+0.5;         SpaceGroupElement[57].z=y+0.5;          // 2        [0,1,1]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[58].x=-z+0.5;        SpaceGroupElement[58].y=-x;            SpaceGroupElement[58].z=y+0.5;          // 3^-1     [-1,1,1]        -1/3,1/3,1/3    5/6,-1/6,0
      SpaceGroupElement[59].x=x;             SpaceGroupElement[59].y=-z;            SpaceGroupElement[59].z=y+0.5;          // 4^1      [1,0,0]         0,0,0           0,1/4,3/4
      SpaceGroupElement[60].x=z;             SpaceGroupElement[60].y=-x+0.5;        SpaceGroupElement[60].z=-y+0.5;         // 3^-1     [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[61].x=x+0.5;         SpaceGroupElement[61].y=z+0.5;         SpaceGroupElement[61].z=-y+0.5;         // 4^-1     [1,0,0]         1/2,0,0         0,1/2,0
      SpaceGroupElement[62].x=-z+0.5;        SpaceGroupElement[62].y=x;             SpaceGroupElement[62].z=-y+0.5;         // 3^1      [-1,-1,1]       1/3,1/3,-1/3    1/6,5/6,0
      SpaceGroupElement[63].x=-x;            SpaceGroupElement[63].y=-z;            SpaceGroupElement[63].z=-y+0.5;         // 2        [0,-1,1]        0,1/4,-1/4      0,3/4,0
      SpaceGroupElement[64].x=y;             SpaceGroupElement[64].y=z+0.5;         SpaceGroupElement[64].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,1/6,0
      SpaceGroupElement[65].x=y;             SpaceGroupElement[65].y=-z;            SpaceGroupElement[65].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           1,1,0
      SpaceGroupElement[66].x=z;             SpaceGroupElement[66].y=y+0.5;         SpaceGroupElement[66].z=-x;             // 4^1      [0,1,0]         0,1/2,0         1/2,0,1/2
      SpaceGroupElement[67].x=-y+0.5;        SpaceGroupElement[67].y=z+0.5;         SpaceGroupElement[67].z=-x;             // 3^1      [-1,1,1]        -1/3,1/3,1/3    2/3,1/6,0
      SpaceGroupElement[68].x=-z+0.5;        SpaceGroupElement[68].y=-y+0.5;        SpaceGroupElement[68].z=-x+0.5;         // 2        [-1,0,1]        0,0,0           1/2,1/4,0
      SpaceGroupElement[69].x=-y;            SpaceGroupElement[69].y=-z+0.5;        SpaceGroupElement[69].z=x+0.5;          // 3^1      [1,-1,1]        0,0,0           -1/2,1/2,0
      SpaceGroupElement[70].x=z;             SpaceGroupElement[70].y=-y+0.5;        SpaceGroupElement[70].z=x;              // 2        [1,0,1]         1/2,0,1/2       -1/2,1/4,0
      SpaceGroupElement[71].x=-z+0.5;        SpaceGroupElement[71].y=y;             SpaceGroupElement[71].z=x;              // 4^-1     [0,1,0]         0,1,0           -1/4,0,3/4
      SpaceGroupElement[72].x=-x;            SpaceGroupElement[72].y=-y+0.5;        SpaceGroupElement[72].z=-z+0.5;         // -1       -               -               0,1/4,1/4
      SpaceGroupElement[73].x=y-0.5;         SpaceGroupElement[73].y=-x+0.5;        SpaceGroupElement[73].z=-z+0.5;         // -4^1     [0,0,1]         0,0,0           0,1/2,1/4
      SpaceGroupElement[74].x=x-0.5;         SpaceGroupElement[74].y=y;             SpaceGroupElement[74].z=-z+0.5;         // -2       [0,0,1]         -1/2,0,0        0,0,1/4
      SpaceGroupElement[75].x=-y;            SpaceGroupElement[75].y=x;             SpaceGroupElement[75].z=-z+0.5;         // -4^-1    [0,0,1]         0,0,0           0,0,1/4
      SpaceGroupElement[76].x=-x;            SpaceGroupElement[76].y=y+0.5;         SpaceGroupElement[76].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       0,0,0
      SpaceGroupElement[77].x=-y-0.5;        SpaceGroupElement[77].y=-x+0.5;        SpaceGroupElement[77].z=z+0.5;          // -2       [1,1,0]         -1/2,1/2,1/2    0,0,0
      SpaceGroupElement[78].x=x-0.5;         SpaceGroupElement[78].y=-y;            SpaceGroupElement[78].z=z+0.5;          // -2       [0,1,0]         -1/2,0,1/2      0,0,0
      SpaceGroupElement[79].x=y;             SpaceGroupElement[79].y=x;             SpaceGroupElement[79].z=z+0.5;          // -2       [-1,1,0]        0,0,1/2         0,0,0
      SpaceGroupElement[80].x=-z;            SpaceGroupElement[80].y=-x+0.5;        SpaceGroupElement[80].z=-y+0.5;         // -3^1     [1,1,1]         0,0,0           0,1/2,0
      SpaceGroupElement[81].x=x-0.5;         SpaceGroupElement[81].y=-z+0.5;        SpaceGroupElement[81].z=-y+0.5;         // -2       [0,1,1]         -1/2,0,0        0,1/2,0
      SpaceGroupElement[82].x=z-0.5;         SpaceGroupElement[82].y=x;             SpaceGroupElement[82].z=-y+0.5;         // -3^-1    [-1,1,1]        0,0,0           0,0,1/2
      SpaceGroupElement[83].x=-x;            SpaceGroupElement[83].y=z;             SpaceGroupElement[83].z=-y+0.5;         // -4^1     [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[84].x=-z;            SpaceGroupElement[84].y=x+0.5;         SpaceGroupElement[84].z=y+0.5;          // -3^-1    [1,-1,1]        0,0,0           -1/2,0,1/2
      SpaceGroupElement[85].x=-x-0.5;        SpaceGroupElement[85].y=-z+0.5;        SpaceGroupElement[85].z=y+0.5;          // -4^-1    [1,0,0]         0,0,0           -1/4,0,1/2
      SpaceGroupElement[86].x=z-0.5;         SpaceGroupElement[86].y=-x;            SpaceGroupElement[86].z=y+0.5;          // -3^1     [-1,-1,1]       0,0,0           0,0,1/2
      SpaceGroupElement[87].x=x;             SpaceGroupElement[87].y=z;             SpaceGroupElement[87].z=y+0.5;          // -2       [0,-1,1]        0,1/4,1/4       0,-1/4,0
      SpaceGroupElement[88].x=-y;            SpaceGroupElement[88].y=-z+0.5;        SpaceGroupElement[88].z=-x+0.5;         // -3^-1    [1,1,1]         0,0,0           0,0,1/2
      SpaceGroupElement[89].x=-y;            SpaceGroupElement[89].y=z;             SpaceGroupElement[89].z=x;              // -3^-1    [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[90].x=-z;            SpaceGroupElement[90].y=-y+0.5;        SpaceGroupElement[90].z=x;              // -4^1     [0,1,0]         0,0,0           0,1/4,0
      SpaceGroupElement[91].x=y-0.5;         SpaceGroupElement[91].y=-z+0.5;        SpaceGroupElement[91].z=x;              // -3^1     [-1,1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[92].x=z-0.5;         SpaceGroupElement[92].y=y+0.5;         SpaceGroupElement[92].z=x+0.5;          // -2       [-1,0,1]        0,1/2,0         -1/2,0,0
      SpaceGroupElement[93].x=y;             SpaceGroupElement[93].y=z+0.5;         SpaceGroupElement[93].z=-x+0.5;         // -3^1     [1,-1,1]        0,0,0           1/2,1/2,0
      SpaceGroupElement[94].x=-z;            SpaceGroupElement[94].y=y+0.5;         SpaceGroupElement[94].z=-x;             // -2       [1,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[95].x=z-0.5;         SpaceGroupElement[95].y=-y;            SpaceGroupElement[95].z=-x;             // -4^-1    [0,1,0]         0,0,0           -1/4,0,1/4
      SpaceGroupElement[96].x=x+0.5;         SpaceGroupElement[96].y=y;             SpaceGroupElement[96].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[97].x=-y;            SpaceGroupElement[97].y=x;             SpaceGroupElement[97].z=z+0.5;          // 4^1      [0,0,1]         0,0,1/2         1/2,1/2,0
      SpaceGroupElement[98].x=-x;            SpaceGroupElement[98].y=-y+0.5;        SpaceGroupElement[98].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/2,1/4,0
      SpaceGroupElement[99].x=y+0.5;         SpaceGroupElement[99].y=-x+0.5;        SpaceGroupElement[99].z=z+0.5;          // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[100].x=x+0.5;        SpaceGroupElement[100].y=-y;           SpaceGroupElement[100].z=-z+0.5;        // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[101].x=y;            SpaceGroupElement[101].y=x;            SpaceGroupElement[101].z=-z+0.5;        // 2        [1,1,0]         1/2,1/2,0       1/2,0,1/4
      SpaceGroupElement[102].x=-x;           SpaceGroupElement[102].y=y+0.5;        SpaceGroupElement[102].z=-z+0.5;        // 2        [0,1,0]         0,1/2,0         1/2,0,1/4
      SpaceGroupElement[103].x=-y+0.5;       SpaceGroupElement[103].y=-x+0.5;       SpaceGroupElement[103].z=-z+0.5;        // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      SpaceGroupElement[104].x=z+0.5;        SpaceGroupElement[104].y=x;            SpaceGroupElement[104].z=y+0.5;         // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,-1/6,0
      SpaceGroupElement[105].x=-x;           SpaceGroupElement[105].y=z;            SpaceGroupElement[105].z=y+0.5;         // 2        [0,1,1]         0,1/4,1/4       1/2,-1/4,0
      SpaceGroupElement[106].x=-z;           SpaceGroupElement[106].y=-x+0.5;       SpaceGroupElement[106].z=y+0.5;         // 3^-1     [-1,1,1]        0,0,0           1,-1/2,0
      SpaceGroupElement[107].x=x+0.5;        SpaceGroupElement[107].y=-z+0.5;       SpaceGroupElement[107].z=y+0.5;         // 4^1      [1,0,0]         1/2,0,0         0,0,1/2
      SpaceGroupElement[108].x=z+0.5;        SpaceGroupElement[108].y=-x;           SpaceGroupElement[108].z=-y+0.5;        // 3^-1     [1,-1,1]        1/3,-1/3,1/3    1/6,1/6,0
      SpaceGroupElement[109].x=x;            SpaceGroupElement[109].y=z;            SpaceGroupElement[109].z=-y+0.5;        // 4^-1     [1,0,0]         1,0,0           0,1/4,1/4
      SpaceGroupElement[110].x=-z;           SpaceGroupElement[110].y=x+0.5;        SpaceGroupElement[110].z=-y+0.5;        // 3^1      [-1,-1,1]       1/3,1/3,-1/3    2/3,5/6,0
      SpaceGroupElement[111].x=-x+0.5;       SpaceGroupElement[111].y=-z+0.5;       SpaceGroupElement[111].z=-y+0.5;        // 2        [0,-1,1]        0,0,0           1/4,1/2,0
      SpaceGroupElement[112].x=y+0.5;        SpaceGroupElement[112].y=z;            SpaceGroupElement[112].z=x+0.5;         // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,-1/3,0
      SpaceGroupElement[113].x=y+0.5;        SpaceGroupElement[113].y=-z+0.5;       SpaceGroupElement[113].z=-x;            // 3^-1     [-1,-1,1]       0,0,0           1,1/2,0
      SpaceGroupElement[114].x=z+0.5;        SpaceGroupElement[114].y=y;            SpaceGroupElement[114].z=-x;            // 4^1      [0,1,0]         0,0,0           3/4,0,1/4
      SpaceGroupElement[115].x=-y;           SpaceGroupElement[115].y=z;            SpaceGroupElement[115].z=-x;            // 3^1      [-1,1,1]        0,0,0           1,0,0
      SpaceGroupElement[116].x=-z;           SpaceGroupElement[116].y=-y;           SpaceGroupElement[116].z=-x+0.5;        // 2        [-1,0,1]        1/4,0,-1/4      3/4,0,0
      SpaceGroupElement[117].x=-y+0.5;       SpaceGroupElement[117].y=-z;           SpaceGroupElement[117].z=x+0.5;         // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      SpaceGroupElement[118].x=z+0.5;        SpaceGroupElement[118].y=-y;           SpaceGroupElement[118].z=x;             // 2        [1,0,1]         3/4,0,3/4       -1/4,0,0
      SpaceGroupElement[119].x=-z;           SpaceGroupElement[119].y=y+0.5;        SpaceGroupElement[119].z=x;             // 4^-1     [0,1,0]         0,1/2,0         0,0,1
      SpaceGroupElement[120].x=-x+0.5;       SpaceGroupElement[120].y=-y;           SpaceGroupElement[120].z=-z+0.5;        // -1       -               -               1/4,0,1/4
      SpaceGroupElement[121].x=y;            SpaceGroupElement[121].y=-x;           SpaceGroupElement[121].z=-z+0.5;        // -4^1     [0,0,1]         0,0,0           0,0,1/4
      SpaceGroupElement[122].x=x;            SpaceGroupElement[122].y=y-0.5;        SpaceGroupElement[122].z=-z+0.5;        // -2       [0,0,1]         0,-1/2,0        0,0,1/4
      SpaceGroupElement[123].x=-y+0.5;       SpaceGroupElement[123].y=x-0.5;        SpaceGroupElement[123].z=-z+0.5;        // -4^-1    [0,0,1]         0,0,0           1/2,0,1/4
      SpaceGroupElement[124].x=-x+0.5;       SpaceGroupElement[124].y=y;            SpaceGroupElement[124].z=z+0.5;         // -2       [1,0,0]         0,0,1/2         1/4,0,0
      SpaceGroupElement[125].x=-y;           SpaceGroupElement[125].y=-x;           SpaceGroupElement[125].z=z+0.5;         // -2       [1,1,0]         0,0,1/2         0,0,0
      SpaceGroupElement[126].x=x;            SpaceGroupElement[126].y=-y-0.5;       SpaceGroupElement[126].z=z+0.5;         // -2       [0,1,0]         0,0,1/2         0,-1/4,0
      SpaceGroupElement[127].x=y+0.5;        SpaceGroupElement[127].y=x-0.5;        SpaceGroupElement[127].z=z+0.5;         // -2       [-1,1,0]        0,0,1/2         1/2,0,0
      SpaceGroupElement[128].x=-z+0.5;       SpaceGroupElement[128].y=-x;           SpaceGroupElement[128].z=-y+0.5;        // -3^1     [1,1,1]         0,0,0           0,0,1/2
      SpaceGroupElement[129].x=x;            SpaceGroupElement[129].y=-z;           SpaceGroupElement[129].z=-y+0.5;        // -2       [0,1,1]         0,-1/4,1/4      0,1/4,0
      SpaceGroupElement[130].x=z;            SpaceGroupElement[130].y=x-0.5;        SpaceGroupElement[130].z=-y+0.5;        // -3^-1    [-1,1,1]        0,0,0           1/2,0,1/2
      SpaceGroupElement[131].x=-x+0.5;       SpaceGroupElement[131].y=z-0.5;        SpaceGroupElement[131].z=-y+0.5;        // -4^1     [1,0,0]         0,0,0           1/4,0,1/2
      SpaceGroupElement[132].x=-z+0.5;       SpaceGroupElement[132].y=x;            SpaceGroupElement[132].z=y+0.5;         // -3^-1    [1,-1,1]        0,0,0           0,0,1/2
      SpaceGroupElement[133].x=-x;           SpaceGroupElement[133].y=-z;           SpaceGroupElement[133].z=y+0.5;         // -4^-1    [1,0,0]         0,0,0           0,-1/4,1/4
      SpaceGroupElement[134].x=z;            SpaceGroupElement[134].y=-x-0.5;       SpaceGroupElement[134].z=y+0.5;         // -3^1     [-1,-1,1]       0,0,0           0,-1/2,0
      SpaceGroupElement[135].x=x+0.5;        SpaceGroupElement[135].y=z-0.5;        SpaceGroupElement[135].z=y+0.5;         // -2       [0,-1,1]        1/2,0,0         0,-1/2,0
      SpaceGroupElement[136].x=-y+0.5;       SpaceGroupElement[136].y=-z;           SpaceGroupElement[136].z=-x+0.5;        // -3^-1    [1,1,1]         0,0,0           1/2,0,0
      SpaceGroupElement[137].x=-y+0.5;       SpaceGroupElement[137].y=z-0.5;        SpaceGroupElement[137].z=x;             // -3^-1    [-1,-1,1]       0,0,0           1/2,0,1/2
      SpaceGroupElement[138].x=-z+0.5;       SpaceGroupElement[138].y=-y;           SpaceGroupElement[138].z=x;             // -4^1     [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[139].x=y;            SpaceGroupElement[139].y=-z;           SpaceGroupElement[139].z=x;             // -3^1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[140].x=z;            SpaceGroupElement[140].y=y;            SpaceGroupElement[140].z=x+0.5;         // -2       [-1,0,1]        1/4,0,1/4       -1/4,0,0
      SpaceGroupElement[141].x=y+0.5;        SpaceGroupElement[141].y=z;            SpaceGroupElement[141].z=-x+0.5;        // -3^1     [1,-1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[142].x=-z+0.5;       SpaceGroupElement[142].y=y;            SpaceGroupElement[142].z=-x;            // -2       [1,0,1]         1/4,0,-1/4      1/4,0,0
      SpaceGroupElement[143].x=z;            SpaceGroupElement[143].y=-y-0.5;       SpaceGroupElement[143].z=-x;            // -4^-1    [0,1,0]         0,0,0           0,-1/4,0
      SpaceGroupElement[144].x=x+0.5;        SpaceGroupElement[144].y=y+0.5;        SpaceGroupElement[144].z=z;             // 1        -               -               -
      SpaceGroupElement[145].x=-y;           SpaceGroupElement[145].y=x+0.5;        SpaceGroupElement[145].z=z;             // 4^1      [0,0,1]         0,0,0           1/4,3/4,0
      SpaceGroupElement[146].x=-x;           SpaceGroupElement[146].y=-y;           SpaceGroupElement[146].z=z;             // 2        [0,0,1]         0,0,0           1/2,1/2,0
      SpaceGroupElement[147].x=y+0.5;        SpaceGroupElement[147].y=-x;           SpaceGroupElement[147].z=z;             // 4^-1     [0,0,1]         0,0,0           3/4,1/4,0
      SpaceGroupElement[148].x=x+0.5;        SpaceGroupElement[148].y=-y+0.5;       SpaceGroupElement[148].z=-z;            // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[149].x=y;            SpaceGroupElement[149].y=x+0.5;        SpaceGroupElement[149].z=-z;            // 2        [1,1,0]         3/4,3/4,0       1/4,0,0
      SpaceGroupElement[150].x=-x;           SpaceGroupElement[150].y=y;            SpaceGroupElement[150].z=-z;            // 2        [0,1,0]         0,1,0           1/2,0,0
      SpaceGroupElement[151].x=-y+0.5;       SpaceGroupElement[151].y=-x;           SpaceGroupElement[151].z=-z;            // 2        [-1,1,0]        -1/4,1/4,0      3/4,0,0
      SpaceGroupElement[152].x=z+0.5;        SpaceGroupElement[152].y=x+0.5;        SpaceGroupElement[152].z=y;             // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,1/3,0
      SpaceGroupElement[153].x=-x;           SpaceGroupElement[153].y=z+0.5;        SpaceGroupElement[153].z=y;             // 2        [0,1,1]         0,1/4,1/4       1/2,1/4,0
      SpaceGroupElement[154].x=-z;           SpaceGroupElement[154].y=-x;           SpaceGroupElement[154].z=y;             // 3^-1     [-1,1,1]        0,0,0           1,0,0
      SpaceGroupElement[155].x=x+0.5;        SpaceGroupElement[155].y=-z;           SpaceGroupElement[155].z=y;             // 4^1      [1,0,0]         1/2,0,0         0,1/2,1/2
      SpaceGroupElement[156].x=z+0.5;        SpaceGroupElement[156].y=-x+0.5;       SpaceGroupElement[156].z=-y;            // 3^-1     [1,-1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[157].x=x;            SpaceGroupElement[157].y=z+0.5;        SpaceGroupElement[157].z=-y;            // 4^-1     [1,0,0]         1,0,0           0,1/4,-1/4
      SpaceGroupElement[158].x=-z;           SpaceGroupElement[158].y=x;            SpaceGroupElement[158].z=-y;            // 3^1      [-1,-1,1]       2/3,2/3,-2/3    1/3,2/3,0
      SpaceGroupElement[159].x=-x+0.5;       SpaceGroupElement[159].y=-z;           SpaceGroupElement[159].z=-y;            // 2        [0,-1,1]        0,1/2,-1/2      1/4,1/2,0
      SpaceGroupElement[160].x=y+0.5;        SpaceGroupElement[160].y=z+0.5;        SpaceGroupElement[160].z=x;             // 3^-1     [1,1,1]         1/3,1/3,1/3     1/3,1/6,0
      SpaceGroupElement[161].x=y+0.5;        SpaceGroupElement[161].y=-z;           SpaceGroupElement[161].z=-x+0.5;        // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    5/6,2/3,0
      SpaceGroupElement[162].x=z+0.5;        SpaceGroupElement[162].y=y+0.5;        SpaceGroupElement[162].z=-x+0.5;        // 4^1      [0,1,0]         0,1/2,0         1/2,0,0
      SpaceGroupElement[163].x=-y;           SpaceGroupElement[163].y=z+0.5;        SpaceGroupElement[163].z=-x+0.5;        // 3^1      [-1,1,1]        0,0,0           1/2,1/2,0
      SpaceGroupElement[164].x=-z;           SpaceGroupElement[164].y=-y+0.5;       SpaceGroupElement[164].z=-x;            // 2        [-1,0,1]        1/2,0,-1/2      1/2,1/4,0
      SpaceGroupElement[165].x=-y+0.5;       SpaceGroupElement[165].y=-z+0.5;       SpaceGroupElement[165].z=x;             // 3^1      [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[166].x=z+0.5;        SpaceGroupElement[166].y=-y+0.5;       SpaceGroupElement[166].z=x+0.5;         // 2        [1,0,1]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[167].x=-z;           SpaceGroupElement[167].y=y;            SpaceGroupElement[167].z=x+0.5;         // 4^-1     [0,1,0]         0,1,0           1/4,0,3/4
      SpaceGroupElement[168].x=-x+0.5;       SpaceGroupElement[168].y=-y+0.5;       SpaceGroupElement[168].z=-z;            // -1       -               -               1/4,1/4,0
      SpaceGroupElement[169].x=y;            SpaceGroupElement[169].y=-x+0.5;       SpaceGroupElement[169].z=-z;            // -4^1     [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[170].x=x;            SpaceGroupElement[170].y=y;            SpaceGroupElement[170].z=-z;            // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[171].x=-y+0.5;       SpaceGroupElement[171].y=x;            SpaceGroupElement[171].z=-z;            // -4^-1    [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[172].x=-x+0.5;       SpaceGroupElement[172].y=y+0.5;        SpaceGroupElement[172].z=z;             // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[173].x=-y;           SpaceGroupElement[173].y=-x+0.5;       SpaceGroupElement[173].z=z;             // -2       [1,1,0]         -1/4,1/4,0      1/4,0,0
      SpaceGroupElement[174].x=x;            SpaceGroupElement[174].y=-y;           SpaceGroupElement[174].z=z;             // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[175].x=y+0.5;        SpaceGroupElement[175].y=x;            SpaceGroupElement[175].z=z;             // -2       [-1,1,0]        1/4,1/4,0       1/4,0,0
      SpaceGroupElement[176].x=-z+0.5;       SpaceGroupElement[176].y=-x+0.5;       SpaceGroupElement[176].z=-y;            // -3^1     [1,1,1]         0,0,0           1/2,0,0
      SpaceGroupElement[177].x=x;            SpaceGroupElement[177].y=-z+0.5;       SpaceGroupElement[177].z=-y;            // -2       [0,1,1]         0,1/4,-1/4      0,1/4,0
      SpaceGroupElement[178].x=z;            SpaceGroupElement[178].y=x;            SpaceGroupElement[178].z=-y;            // -3^-1    [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[179].x=-x+0.5;       SpaceGroupElement[179].y=z;            SpaceGroupElement[179].z=-y;            // -4^1     [1,0,0]         0,0,0           1/4,0,0
      SpaceGroupElement[180].x=-z+0.5;       SpaceGroupElement[180].y=x+0.5;        SpaceGroupElement[180].z=y;             // -3^-1    [1,-1,1]        0,0,0           0,1/2,1/2
      SpaceGroupElement[181].x=-x;           SpaceGroupElement[181].y=-z+0.5;       SpaceGroupElement[181].z=y;             // -4^-1    [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[182].x=z;            SpaceGroupElement[182].y=-x;           SpaceGroupElement[182].z=y;             // -3^1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[183].x=x+0.5;        SpaceGroupElement[183].y=z;            SpaceGroupElement[183].z=y;             // -2       [0,-1,1]        1/2,0,0         0,0,0
      SpaceGroupElement[184].x=-y+0.5;       SpaceGroupElement[184].y=-z+0.5;       SpaceGroupElement[184].z=-x;            // -3^-1    [1,1,1]         0,0,0           0,1/2,0
      SpaceGroupElement[185].x=-y+0.5;       SpaceGroupElement[185].y=z;            SpaceGroupElement[185].z=x-0.5;         // -3^-1    [-1,-1,1]       0,0,0           1/2,0,0
      SpaceGroupElement[186].x=-z+0.5;       SpaceGroupElement[186].y=-y+0.5;       SpaceGroupElement[186].z=x-0.5;         // -4^1     [0,1,0]         0,0,0           1/2,1/4,0
      SpaceGroupElement[187].x=y;            SpaceGroupElement[187].y=-z+0.5;       SpaceGroupElement[187].z=x-0.5;         // -3^1     [-1,1,1]        0,0,0           1/2,1/2,0
      SpaceGroupElement[188].x=z;            SpaceGroupElement[188].y=y+0.5;        SpaceGroupElement[188].z=x;             // -2       [-1,0,1]        0,1/2,0         0,0,0
      SpaceGroupElement[189].x=y+0.5;        SpaceGroupElement[189].y=z+0.5;        SpaceGroupElement[189].z=-x;            // -3^1     [1,-1,1]        0,0,0           1/2,0,-1/2
      SpaceGroupElement[190].x=-z+0.5;       SpaceGroupElement[190].y=y+0.5;        SpaceGroupElement[190].z=-x-0.5;        // -2       [1,0,1]         1/2,1/2,-1/2    0,0,0
      SpaceGroupElement[191].x=z;            SpaceGroupElement[191].y=-y;           SpaceGroupElement[191].z=-x-0.5;        // -4^-1    [0,1,0]         0,0,0           -1/4,0,-1/4
      break;

    case 525: // [227] F d -3 m Origin choice 1 (F 4d 2 3 -1d)
      SpaceGroupSize=192;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.25;        SpaceGroupElement[1].y=x+0.25;         SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         0,1/4,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[3].x=y+0.75;         SpaceGroupElement[3].y=-x+0.25;        SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         1/2,-1/4,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y+0.25;         SpaceGroupElement[5].y=x+0.25;         SpaceGroupElement[5].z=-z+0.25;         // 2        [1,1,0]         1/4,1/4,0       0,0,1/8
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[7].x=-y+0.75;        SpaceGroupElement[7].y=-x+0.25;        SpaceGroupElement[7].z=-z+0.75;         // 2        [-1,1,0]        1/4,-1/4,0      1/2,0,3/8
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x+0.25;        SpaceGroupElement[9].y=z+0.25;         SpaceGroupElement[9].z=y+0.25;          // 2        [0,1,1]         0,1/4,1/4       1/8,0,0
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x+0.5;        SpaceGroupElement[10].z=y+0.5;          // 3^-1     [-1,1,1]        -1/3,1/3,1/3    1/3,-1/6,0
      SpaceGroupElement[11].x=x+0.75;        SpaceGroupElement[11].y=-z+0.25;       SpaceGroupElement[11].z=y+0.75;         // 4^1      [1,0,0]         3/4,0,0         0,-1/4,1/2
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[13].x=x+0.25;        SpaceGroupElement[13].y=z+0.25;        SpaceGroupElement[13].z=-y+0.25;        // 4^-1     [1,0,0]         1/4,0,0         0,1/4,0
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x+0.5;         SpaceGroupElement[14].z=-y+0.5;         // 3^1      [-1,-1,1]       0,0,0           0,1/2,0
      SpaceGroupElement[15].x=-x+0.75;       SpaceGroupElement[15].y=-z+0.25;       SpaceGroupElement[15].z=-y+0.75;        // 2        [0,-1,1]        0,-1/4,1/4      3/8,1/2,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y+0.5;         SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=-x+0.5;         // 3^-1     [-1,-1,1]       0,0,0           1/2,0,0
      SpaceGroupElement[18].x=z+0.25;        SpaceGroupElement[18].y=y+0.75;        SpaceGroupElement[18].z=-x+0.75;        // 4^1      [0,1,0]         0,3/4,0         1/2,0,1/4
      SpaceGroupElement[19].x=-y+0.5;        SpaceGroupElement[19].y=z+0.5;         SpaceGroupElement[19].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[20].x=-z+0.25;       SpaceGroupElement[20].y=-y+0.25;       SpaceGroupElement[20].z=-x+0.25;        // 2        [-1,0,1]        0,0,0           1/4,1/8,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[22].x=z+0.25;        SpaceGroupElement[22].y=-y+0.75;       SpaceGroupElement[22].z=x+0.75;         // 2        [1,0,1]         1/2,0,1/2       -1/4,3/8,0
      SpaceGroupElement[23].x=-z+0.75;       SpaceGroupElement[23].y=y+0.75;        SpaceGroupElement[23].z=x+0.25;         // 4^-1     [0,1,0]         0,3/4,0         1/4,0,1/2
      SpaceGroupElement[24].x=-x+0.25;       SpaceGroupElement[24].y=-y+0.25;       SpaceGroupElement[24].z=-z+0.25;        // -1       -               -               1/8,1/8,1/8
      SpaceGroupElement[25].x=y;             SpaceGroupElement[25].y=-x;            SpaceGroupElement[25].z=-z;             // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[26].x=x+0.25;        SpaceGroupElement[26].y=y-0.25;        SpaceGroupElement[26].z=-z-0.25;        // -2       [0,0,1]         1/4,-1/4,0      0,0,-1/8
      SpaceGroupElement[27].x=-y-0.5;        SpaceGroupElement[27].y=x;             SpaceGroupElement[27].z=-z-0.5;         // -4^-1    [0,0,1]         0,0,0           -1/4,-1/4,-1/4
      SpaceGroupElement[28].x=-x+0.25;       SpaceGroupElement[28].y=y+0.25;        SpaceGroupElement[28].z=z+0.25;         // -2       [1,0,0]         0,1/4,1/4       1/8,0,0
      SpaceGroupElement[29].x=-y;            SpaceGroupElement[29].y=-x;            SpaceGroupElement[29].z=z;              // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[30].x=x+0.25;        SpaceGroupElement[30].y=-y-0.25;       SpaceGroupElement[30].z=z-0.25;         // -2       [0,1,0]         1/4,0,-1/4      0,-1/8,0
      SpaceGroupElement[31].x=y-0.5;         SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=z-0.5;          // -2       [-1,1,0]        -1/4,-1/4,-1/2  -1/4,0,0
      SpaceGroupElement[32].x=-z+0.25;       SpaceGroupElement[32].y=-x+0.25;       SpaceGroupElement[32].z=-y+0.25;        // -3^1     [1,1,1]         0,0,0           1/8,1/8,1/8
      SpaceGroupElement[33].x=x;             SpaceGroupElement[33].y=-z;            SpaceGroupElement[33].z=-y;             // -2       [0,1,1]         0,0,0           0,0,0
      SpaceGroupElement[34].x=z+0.25;        SpaceGroupElement[34].y=x-0.25;        SpaceGroupElement[34].z=-y-0.25;        // -3^-1    [-1,1,1]        0,0,0           1/8,-1/8,-1/8
      SpaceGroupElement[35].x=-x-0.5;        SpaceGroupElement[35].y=z;             SpaceGroupElement[35].z=-y-0.5;         // -4^1     [1,0,0]         0,0,0           -1/4,-1/4,-1/4
      SpaceGroupElement[36].x=-z+0.25;       SpaceGroupElement[36].y=x+0.25;        SpaceGroupElement[36].z=y+0.25;         // -3^-1    [1,-1,1]        0,0,0           -1/8,1/8,3/8
      SpaceGroupElement[37].x=-x;            SpaceGroupElement[37].y=-z;            SpaceGroupElement[37].z=y;              // -4^-1    [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[38].x=z+0.25;        SpaceGroupElement[38].y=-x-0.25;       SpaceGroupElement[38].z=y-0.25;         // -3^1     [-1,-1,1]       0,0,0           -1/8,-1/8,-3/8
      SpaceGroupElement[39].x=x-0.5;         SpaceGroupElement[39].y=z;             SpaceGroupElement[39].z=y-0.5;          // -2       [0,-1,1]        -1/2,-1/4,-1/4  0,1/4,0
      SpaceGroupElement[40].x=-y+0.25;       SpaceGroupElement[40].y=-z+0.25;       SpaceGroupElement[40].z=-x+0.25;        // -3^-1    [1,1,1]         0,0,0           1/8,1/8,1/8
      SpaceGroupElement[41].x=-y-0.25;       SpaceGroupElement[41].y=z+0.25;        SpaceGroupElement[41].z=x-0.25;         // -3^-1    [-1,-1,1]       0,0,0           -1/8,-1/8,-3/8
      SpaceGroupElement[42].x=-z;            SpaceGroupElement[42].y=-y-0.5;        SpaceGroupElement[42].z=x-0.5;          // -4^1     [0,1,0]         0,0,0           1/4,-1/4,-1/4
      SpaceGroupElement[43].x=y-0.25;        SpaceGroupElement[43].y=-z-0.25;       SpaceGroupElement[43].z=x+0.25;         // -3^1     [-1,1,1]        0,0,0           -3/8,-1/8,-1/8
      SpaceGroupElement[44].x=z;             SpaceGroupElement[44].y=y;             SpaceGroupElement[44].z=x;              // -2       [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[45].x=y+0.25;        SpaceGroupElement[45].y=z+0.25;        SpaceGroupElement[45].z=-x+0.25;        // -3^1     [1,-1,1]        0,0,0           3/8,1/8,-1/8
      SpaceGroupElement[46].x=-z;            SpaceGroupElement[46].y=y-0.5;         SpaceGroupElement[46].z=-x-0.5;         // -2       [1,0,1]         1/4,-1/2,-1/4   -1/4,0,0
      SpaceGroupElement[47].x=z-0.5;         SpaceGroupElement[47].y=-y-0.5;        SpaceGroupElement[47].z=-x;             // -4^-1    [0,1,0]         0,0,0           -1/4,-1/4,1/4
      SpaceGroupElement[48].x=x;             SpaceGroupElement[48].y=y+0.5;         SpaceGroupElement[48].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[49].x=-y+0.25;       SpaceGroupElement[49].y=x+0.75;        SpaceGroupElement[49].z=z+0.75;         // 4^1      [0,0,1]         0,0,3/4         -1/4,1/2,0
      SpaceGroupElement[50].x=-x;            SpaceGroupElement[50].y=-y;            SpaceGroupElement[50].z=z;              // 2        [0,0,1]         0,0,1           0,1/2,0
      SpaceGroupElement[51].x=y+0.75;        SpaceGroupElement[51].y=-x+0.75;       SpaceGroupElement[51].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         3/4,0,0
      SpaceGroupElement[52].x=x;             SpaceGroupElement[52].y=-y+0.5;        SpaceGroupElement[52].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[53].x=y+0.25;        SpaceGroupElement[53].y=x+0.75;        SpaceGroupElement[53].z=-z+0.75;        // 2        [1,1,0]         1/2,1/2,0       -1/4,0,3/8
      SpaceGroupElement[54].x=-x;            SpaceGroupElement[54].y=y;             SpaceGroupElement[54].z=-z;             // 2        [0,1,0]         0,1,0           0,0,1/2
      SpaceGroupElement[55].x=-y+0.75;       SpaceGroupElement[55].y=-x+0.75;       SpaceGroupElement[55].z=-z+0.25;        // 2        [-1,1,0]        0,0,0           3/4,0,5/8
      SpaceGroupElement[56].x=z;             SpaceGroupElement[56].y=x+0.5;         SpaceGroupElement[56].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     -1/3,-1/6,0
      SpaceGroupElement[57].x=-x+0.25;       SpaceGroupElement[57].y=z+0.75;        SpaceGroupElement[57].z=y+0.75;         // 2        [0,1,1]         0,3/4,3/4       1/8,0,0
      SpaceGroupElement[58].x=-z;            SpaceGroupElement[58].y=-x;            SpaceGroupElement[58].z=y;              // 3^-1     [-1,1,1]        -2/3,2/3,2/3    2/3,-1/3,0
      SpaceGroupElement[59].x=x+0.75;        SpaceGroupElement[59].y=-z+0.75;       SpaceGroupElement[59].z=y+0.25;         // 4^1      [1,0,0]         3/4,0,0         0,-1/4,1
      SpaceGroupElement[60].x=z;             SpaceGroupElement[60].y=-x+0.5;        SpaceGroupElement[60].z=-y+0.5;         // 3^-1     [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[61].x=x+0.25;        SpaceGroupElement[61].y=z+0.75;        SpaceGroupElement[61].z=-y+0.75;        // 4^-1     [1,0,0]         1/4,0,0         0,3/4,0
      SpaceGroupElement[62].x=-z;            SpaceGroupElement[62].y=x;             SpaceGroupElement[62].z=-y;             // 3^1      [-1,-1,1]       0,0,0           0,1,0
      SpaceGroupElement[63].x=-x+0.75;       SpaceGroupElement[63].y=-z+0.75;       SpaceGroupElement[63].z=-y+0.25;        // 2        [0,-1,1]        0,-1/4,1/4      3/8,1,0
      SpaceGroupElement[64].x=y;             SpaceGroupElement[64].y=z+0.5;         SpaceGroupElement[64].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,1/6,0
      SpaceGroupElement[65].x=y+0.5;         SpaceGroupElement[65].y=-z+0.5;        SpaceGroupElement[65].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           1,1/2,0
      SpaceGroupElement[66].x=z+0.25;        SpaceGroupElement[66].y=y+0.25;        SpaceGroupElement[66].z=-x+0.25;        // 4^1      [0,1,0]         0,5/4,0         3/4,0,1/2
      SpaceGroupElement[67].x=-y+0.5;        SpaceGroupElement[67].y=z;             SpaceGroupElement[67].z=-x+0.5;         // 3^1      [-1,1,1]        -1/3,1/3,1/3    1/6,2/3,0
      SpaceGroupElement[68].x=-z+0.25;       SpaceGroupElement[68].y=-y+0.75;       SpaceGroupElement[68].z=-x+0.75;        // 2        [-1,0,1]        -1/4,0,1/4      1/2,3/8,0
      SpaceGroupElement[69].x=-y;            SpaceGroupElement[69].y=-z+0.5;        SpaceGroupElement[69].z=x+0.5;          // 3^1      [1,-1,1]        0,0,0           -1/2,1/2,0
      SpaceGroupElement[70].x=z+0.25;        SpaceGroupElement[70].y=-y+0.25;       SpaceGroupElement[70].z=x+0.25;         // 2        [1,0,1]         3/4,0,3/4       -1/2,5/8,0
      SpaceGroupElement[71].x=-z+0.75;       SpaceGroupElement[71].y=y+0.25;        SpaceGroupElement[71].z=x+0.75;         // 4^-1     [0,1,0]         0,5/4,0         0,0,3/4
      SpaceGroupElement[72].x=-x+0.25;       SpaceGroupElement[72].y=-y+0.75;       SpaceGroupElement[72].z=-z+0.75;        // -1       -               -               1/8,3/8,3/8
      SpaceGroupElement[73].x=y;             SpaceGroupElement[73].y=-x+0.5;        SpaceGroupElement[73].z=-z+0.5;         // -4^1     [0,0,1]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[74].x=x+0.25;        SpaceGroupElement[74].y=y+0.25;        SpaceGroupElement[74].z=-z+0.25;        // -2       [0,0,1]         1/4,1/4,0       0,0,1/8
      SpaceGroupElement[75].x=-y-0.5;        SpaceGroupElement[75].y=x+0.5;         SpaceGroupElement[75].z=-z;             // -4^-1    [0,0,1]         0,0,0           -1/2,0,0
      SpaceGroupElement[76].x=-x+0.25;       SpaceGroupElement[76].y=y+0.75;        SpaceGroupElement[76].z=z+0.75;         // -2       [1,0,0]         0,3/4,3/4       1/8,0,0
      SpaceGroupElement[77].x=-y;            SpaceGroupElement[77].y=-x+0.5;        SpaceGroupElement[77].z=z+0.5;          // -2       [1,1,0]         -1/4,1/4,1/2    1/4,0,0
      SpaceGroupElement[78].x=x+0.25;        SpaceGroupElement[78].y=-y+0.25;       SpaceGroupElement[78].z=z+0.25;         // -2       [0,1,0]         1/4,0,1/4       0,1/8,0
      SpaceGroupElement[79].x=y-0.5;         SpaceGroupElement[79].y=x+0.5;         SpaceGroupElement[79].z=z;              // -2       [-1,1,0]        0,0,0           -1/2,0,0
      SpaceGroupElement[80].x=-z+0.25;       SpaceGroupElement[80].y=-x+0.75;       SpaceGroupElement[80].z=-y+0.75;        // -3^1     [1,1,1]         0,0,0           1/8,5/8,1/8
      SpaceGroupElement[81].x=x;             SpaceGroupElement[81].y=-z+0.5;        SpaceGroupElement[81].z=-y+0.5;         // -2       [0,1,1]         0,0,0           0,1/2,0
      SpaceGroupElement[82].x=z+0.25;        SpaceGroupElement[82].y=x+0.25;        SpaceGroupElement[82].z=-y+0.25;        // -3^-1    [-1,1,1]        0,0,0           1/8,3/8,-1/8
      SpaceGroupElement[83].x=-x-0.5;        SpaceGroupElement[83].y=z+0.5;         SpaceGroupElement[83].z=-y;             // -4^1     [1,0,0]         0,0,0           -1/4,1/4,-1/4
      SpaceGroupElement[84].x=-z+0.25;       SpaceGroupElement[84].y=x+0.75;        SpaceGroupElement[84].z=y+0.75;         // -3^-1    [1,-1,1]        0,0,0           -5/8,1/8,7/8
      SpaceGroupElement[85].x=-x;            SpaceGroupElement[85].y=-z+0.5;        SpaceGroupElement[85].z=y+0.5;          // -4^-1    [1,0,0]         0,0,0           0,0,1/2
      SpaceGroupElement[86].x=z+0.25;        SpaceGroupElement[86].y=-x+0.25;       SpaceGroupElement[86].z=y+0.25;         // -3^1     [-1,-1,1]       0,0,0           3/8,-1/8,1/8
      SpaceGroupElement[87].x=x-0.5;         SpaceGroupElement[87].y=z+0.5;         SpaceGroupElement[87].z=y;              // -2       [0,-1,1]        -1/2,1/4,1/4    0,1/4,0
      SpaceGroupElement[88].x=-y+0.25;       SpaceGroupElement[88].y=-z+0.75;       SpaceGroupElement[88].z=-x+0.75;        // -3^-1    [1,1,1]         0,0,0           1/8,1/8,5/8
      SpaceGroupElement[89].x=-y-0.25;       SpaceGroupElement[89].y=z+0.75;        SpaceGroupElement[89].z=x+0.25;         // -3^-1    [-1,-1,1]       0,0,0           -5/8,3/8,-3/8
      SpaceGroupElement[90].x=-z;            SpaceGroupElement[90].y=-y;            SpaceGroupElement[90].z=x;              // -4^1     [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[91].x=y-0.25;        SpaceGroupElement[91].y=-z+0.25;       SpaceGroupElement[91].z=x+0.75;         // -3^1     [-1,1,1]        0,0,0           -3/8,-1/8,3/8
      SpaceGroupElement[92].x=z;             SpaceGroupElement[92].y=y+0.5;         SpaceGroupElement[92].z=x+0.5;          // -2       [-1,0,1]        1/4,1/2,1/4     -1/4,0,0
      SpaceGroupElement[93].x=y+0.25;        SpaceGroupElement[93].y=z+0.75;        SpaceGroupElement[93].z=-x+0.75;        // -3^1     [1,-1,1]        0,0,0           7/8,5/8,-1/8
      SpaceGroupElement[94].x=-z;            SpaceGroupElement[94].y=y;             SpaceGroupElement[94].z=-x;             // -2       [1,0,1]         0,0,0           0,0,0
      SpaceGroupElement[95].x=z-0.5;         SpaceGroupElement[95].y=-y;            SpaceGroupElement[95].z=-x+0.5;         // -4^-1    [0,1,0]         0,0,0           0,0,1/2
      SpaceGroupElement[96].x=x+0.5;         SpaceGroupElement[96].y=y;             SpaceGroupElement[96].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[97].x=-y+0.75;       SpaceGroupElement[97].y=x+0.25;        SpaceGroupElement[97].z=z+0.75;         // 4^1      [0,0,1]         0,0,3/4         1/4,1/2,0
      SpaceGroupElement[98].x=-x+0.5;        SpaceGroupElement[98].y=-y+0.5;        SpaceGroupElement[98].z=z;              // 2        [0,0,1]         0,0,1           1/4,1/4,0
      SpaceGroupElement[99].x=y+0.25;        SpaceGroupElement[99].y=-x+0.25;       SpaceGroupElement[99].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         3/4,-1/2,0
      SpaceGroupElement[100].x=x+0.5;        SpaceGroupElement[100].y=-y;           SpaceGroupElement[100].z=-z+0.5;        // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[101].x=y+0.75;       SpaceGroupElement[101].y=x+0.25;       SpaceGroupElement[101].z=-z+0.75;       // 2        [1,1,0]         1/2,1/2,0       1/4,0,3/8
      SpaceGroupElement[102].x=-x+0.5;       SpaceGroupElement[102].y=y+0.5;        SpaceGroupElement[102].z=-z;            // 2        [0,1,0]         0,1/2,0         1/4,0,1/2
      SpaceGroupElement[103].x=-y+0.25;      SpaceGroupElement[103].y=-x+0.25;      SpaceGroupElement[103].z=-z+0.25;       // 2        [-1,1,0]        1/2,-1/2,0      3/4,0,5/8
      SpaceGroupElement[104].x=z+0.5;        SpaceGroupElement[104].y=x;            SpaceGroupElement[104].z=y+0.5;         // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,-1/6,0
      SpaceGroupElement[105].x=-x+0.75;      SpaceGroupElement[105].y=z+0.25;       SpaceGroupElement[105].z=y+0.75;        // 2        [0,1,1]         0,1/2,1/2       3/8,-1/4,0
      SpaceGroupElement[106].x=-z+0.5;       SpaceGroupElement[106].y=-x+0.5;       SpaceGroupElement[106].z=y;             // 3^-1     [-1,1,1]        -1/3,1/3,1/3    5/6,-2/3,0
      SpaceGroupElement[107].x=x+0.25;       SpaceGroupElement[107].y=-z+0.25;      SpaceGroupElement[107].z=y+0.25;        // 4^1      [1,0,0]         5/4,0,0         0,-1/2,3/4
      SpaceGroupElement[108].x=z+0.5;        SpaceGroupElement[108].y=-x;           SpaceGroupElement[108].z=-y+0.5;        // 3^-1     [1,-1,1]        1/3,-1/3,1/3    1/6,1/6,0
      SpaceGroupElement[109].x=x+0.75;       SpaceGroupElement[109].y=z+0.25;       SpaceGroupElement[109].z=-y+0.75;       // 4^-1     [1,0,0]         3/4,0,0         0,1/2,1/4
      SpaceGroupElement[110].x=-z+0.5;       SpaceGroupElement[110].y=x+0.5;        SpaceGroupElement[110].z=-y;            // 3^1      [-1,-1,1]       0,0,0           1/2,1,0
      SpaceGroupElement[111].x=-x+0.25;      SpaceGroupElement[111].y=-z+0.25;      SpaceGroupElement[111].z=-y+0.25;       // 2        [0,-1,1]        0,-1/2,1/2      5/8,3/4,0
      SpaceGroupElement[112].x=y+0.5;        SpaceGroupElement[112].y=z;            SpaceGroupElement[112].z=x+0.5;         // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,-1/3,0
      SpaceGroupElement[113].x=y;            SpaceGroupElement[113].y=-z;           SpaceGroupElement[113].z=-x;            // 3^-1     [-1,-1,1]       0,0,0           1,0,0
      SpaceGroupElement[114].x=z+0.75;       SpaceGroupElement[114].y=y+0.75;       SpaceGroupElement[114].z=-x+0.25;       // 4^1      [0,1,0]         0,3/4,0         1,0,1/4
      SpaceGroupElement[115].x=-y;           SpaceGroupElement[115].y=z+0.5;        SpaceGroupElement[115].z=-x+0.5;        // 3^1      [-1,1,1]        0,0,0           1/2,1/2,0
      SpaceGroupElement[116].x=-z+0.75;      SpaceGroupElement[116].y=-y+0.25;      SpaceGroupElement[116].z=-x+0.75;       // 2        [-1,0,1]        0,0,0           3/4,1/8,0
      SpaceGroupElement[117].x=-y+0.5;       SpaceGroupElement[117].y=-z;           SpaceGroupElement[117].z=x+0.5;         // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      SpaceGroupElement[118].x=z+0.75;       SpaceGroupElement[118].y=-y+0.75;      SpaceGroupElement[118].z=x+0.25;        // 2        [1,0,1]         1,0,1           -1/4,3/8,0
      SpaceGroupElement[119].x=-z+0.25;      SpaceGroupElement[119].y=y+0.75;       SpaceGroupElement[119].z=x+0.75;        // 4^-1     [0,1,0]         0,3/4,0         1/4,0,1
      SpaceGroupElement[120].x=-x+0.75;      SpaceGroupElement[120].y=-y+0.25;      SpaceGroupElement[120].z=-z+0.75;       // -1       -               -               3/8,1/8,3/8
      SpaceGroupElement[121].x=y+0.5;        SpaceGroupElement[121].y=-x;           SpaceGroupElement[121].z=-z+0.5;        // -4^1     [0,0,1]         0,0,0           1/4,-1/4,1/4
      SpaceGroupElement[122].x=x+0.75;       SpaceGroupElement[122].y=y-0.25;       SpaceGroupElement[122].z=-z+0.25;       // -2       [0,0,1]         3/4,-1/4,0      0,0,1/8
      SpaceGroupElement[123].x=-y;           SpaceGroupElement[123].y=x;            SpaceGroupElement[123].z=-z;            // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[124].x=-x+0.75;      SpaceGroupElement[124].y=y+0.25;       SpaceGroupElement[124].z=z+0.75;        // -2       [1,0,0]         0,1/4,3/4       3/8,0,0
      SpaceGroupElement[125].x=-y+0.5;       SpaceGroupElement[125].y=-x;           SpaceGroupElement[125].z=z+0.5;         // -2       [1,1,0]         1/4,-1/4,1/2    1/4,0,0
      SpaceGroupElement[126].x=x+0.75;       SpaceGroupElement[126].y=-y-0.25;      SpaceGroupElement[126].z=z+0.25;        // -2       [0,1,0]         3/4,0,1/4       0,-1/8,0
      SpaceGroupElement[127].x=y;            SpaceGroupElement[127].y=x;            SpaceGroupElement[127].z=z;             // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[128].x=-z+0.75;      SpaceGroupElement[128].y=-x+0.25;      SpaceGroupElement[128].z=-y+0.75;       // -3^1     [1,1,1]         0,0,0           1/8,1/8,5/8
      SpaceGroupElement[129].x=x+0.5;        SpaceGroupElement[129].y=-z;           SpaceGroupElement[129].z=-y+0.5;        // -2       [0,1,1]         1/2,-1/4,1/4    0,1/4,0
      SpaceGroupElement[130].x=z+0.75;       SpaceGroupElement[130].y=x-0.25;       SpaceGroupElement[130].z=-y+0.25;       // -3^-1    [-1,1,1]        0,0,0           5/8,3/8,-1/8
      SpaceGroupElement[131].x=-x;           SpaceGroupElement[131].y=z;            SpaceGroupElement[131].z=-y;            // -4^1     [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[132].x=-z+0.75;      SpaceGroupElement[132].y=x+0.25;       SpaceGroupElement[132].z=y+0.75;        // -3^-1    [1,-1,1]        0,0,0           -1/8,1/8,7/8
      SpaceGroupElement[133].x=-x+0.5;       SpaceGroupElement[133].y=-z;           SpaceGroupElement[133].z=y+0.5;         // -4^-1    [1,0,0]         0,0,0           1/4,-1/4,1/4
      SpaceGroupElement[134].x=z+0.75;       SpaceGroupElement[134].y=-x-0.25;      SpaceGroupElement[134].z=y+0.25;        // -3^1     [-1,-1,1]       0,0,0           3/8,-5/8,-3/8
      SpaceGroupElement[135].x=x;            SpaceGroupElement[135].y=z;            SpaceGroupElement[135].z=y;             // -2       [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[136].x=-y+0.75;      SpaceGroupElement[136].y=-z+0.25;      SpaceGroupElement[136].z=-x+0.75;       // -3^-1    [1,1,1]         0,0,0           5/8,1/8,1/8
      SpaceGroupElement[137].x=-y+0.25;      SpaceGroupElement[137].y=z+0.25;       SpaceGroupElement[137].z=x+0.25;        // -3^-1    [-1,-1,1]       0,0,0           -1/8,3/8,1/8
      SpaceGroupElement[138].x=-z+0.5;       SpaceGroupElement[138].y=-y-0.5;       SpaceGroupElement[138].z=x;             // -4^1     [0,1,0]         0,0,0           1/4,-1/4,1/4
      SpaceGroupElement[139].x=y+0.25;       SpaceGroupElement[139].y=-z-0.25;      SpaceGroupElement[139].z=x+0.75;        // -3^1     [-1,1,1]        0,0,0           -3/8,-5/8,3/8
      SpaceGroupElement[140].x=z+0.5;        SpaceGroupElement[140].y=y;            SpaceGroupElement[140].z=x+0.5;         // -2       [-1,0,1]        1/2,0,1/2       0,0,0
      SpaceGroupElement[141].x=y+0.75;       SpaceGroupElement[141].y=z+0.25;       SpaceGroupElement[141].z=-x+0.75;       // -3^1     [1,-1,1]        0,0,0           7/8,1/8,-1/8
      SpaceGroupElement[142].x=-z+0.5;       SpaceGroupElement[142].y=y-0.5;        SpaceGroupElement[142].z=-x;            // -2       [1,0,1]         1/4,-1/2,-1/4   1/4,0,0
      SpaceGroupElement[143].x=z;            SpaceGroupElement[143].y=-y-0.5;       SpaceGroupElement[143].z=-x+0.5;        // -4^-1    [0,1,0]         0,0,0           1/4,-1/4,1/4
      SpaceGroupElement[144].x=x+0.5;        SpaceGroupElement[144].y=y+0.5;        SpaceGroupElement[144].z=z;             // 1        -               -               -
      SpaceGroupElement[145].x=-y+0.75;      SpaceGroupElement[145].y=x+0.75;       SpaceGroupElement[145].z=z+0.25;        // 4^1      [0,0,1]         0,0,1/4         0,3/4,0
      SpaceGroupElement[146].x=-x+0.5;       SpaceGroupElement[146].y=-y;           SpaceGroupElement[146].z=z+0.5;         // 2        [0,0,1]         0,0,1/2         1/4,1/2,0
      SpaceGroupElement[147].x=y+0.25;       SpaceGroupElement[147].y=-x+0.75;      SpaceGroupElement[147].z=z+0.75;        // 4^-1     [0,0,1]         0,0,3/4         1,-1/4,0
      SpaceGroupElement[148].x=x+0.5;        SpaceGroupElement[148].y=-y+0.5;       SpaceGroupElement[148].z=-z;            // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[149].x=y+0.75;       SpaceGroupElement[149].y=x+0.75;       SpaceGroupElement[149].z=-z+0.25;       // 2        [1,1,0]         3/4,3/4,0       0,0,1/8
      SpaceGroupElement[150].x=-x+0.5;       SpaceGroupElement[150].y=y;            SpaceGroupElement[150].z=-z+0.5;        // 2        [0,1,0]         0,1,0           1/4,0,1/4
      SpaceGroupElement[151].x=-y+0.25;      SpaceGroupElement[151].y=-x+0.75;      SpaceGroupElement[151].z=-z+0.75;       // 2        [-1,1,0]        1/4,-1/4,0      1,0,3/8
      SpaceGroupElement[152].x=z+0.5;        SpaceGroupElement[152].y=x+0.5;        SpaceGroupElement[152].z=y;             // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,1/3,0
      SpaceGroupElement[153].x=-x+0.75;      SpaceGroupElement[153].y=z+0.75;       SpaceGroupElement[153].z=y+0.25;        // 2        [0,1,1]         0,1/2,1/2       3/8,1/4,0
      SpaceGroupElement[154].x=-z+0.5;       SpaceGroupElement[154].y=-x;           SpaceGroupElement[154].z=y+0.5;         // 3^-1     [-1,1,1]        -1/3,1/3,1/3    5/6,-1/6,0
      SpaceGroupElement[155].x=x+0.25;       SpaceGroupElement[155].y=-z+0.75;      SpaceGroupElement[155].z=y+0.75;        // 4^1      [1,0,0]         5/4,0,0         0,0,3/4
      SpaceGroupElement[156].x=z+0.5;        SpaceGroupElement[156].y=-x+0.5;       SpaceGroupElement[156].z=-y;            // 3^-1     [1,-1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[157].x=x+0.75;       SpaceGroupElement[157].y=z+0.75;       SpaceGroupElement[157].z=-y+0.25;       // 4^-1     [1,0,0]         3/4,0,0         0,1/2,-1/4
      SpaceGroupElement[158].x=-z+0.5;       SpaceGroupElement[158].y=x;            SpaceGroupElement[158].z=-y+0.5;        // 3^1      [-1,-1,1]       1/3,1/3,-1/3    1/6,5/6,0
      SpaceGroupElement[159].x=-x+0.25;      SpaceGroupElement[159].y=-z+0.75;      SpaceGroupElement[159].z=-y+0.75;       // 2        [0,-1,1]        0,0,0           5/8,3/4,0
      SpaceGroupElement[160].x=y+0.5;        SpaceGroupElement[160].y=z+0.5;        SpaceGroupElement[160].z=x;             // 3^-1     [1,1,1]         1/3,1/3,1/3     1/3,1/6,0
      SpaceGroupElement[161].x=y;            SpaceGroupElement[161].y=-z+0.5;       SpaceGroupElement[161].z=-x+0.5;        // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    5/6,1/6,0
      SpaceGroupElement[162].x=z+0.75;       SpaceGroupElement[162].y=y+0.25;       SpaceGroupElement[162].z=-x+0.75;       // 4^1      [0,1,0]         0,5/4,0         3/4,0,0
      SpaceGroupElement[163].x=-y;           SpaceGroupElement[163].y=z;            SpaceGroupElement[163].z=-x;            // 3^1      [-1,1,1]        0,0,0           0,1,0
      SpaceGroupElement[164].x=-z+0.75;      SpaceGroupElement[164].y=-y+0.75;      SpaceGroupElement[164].z=-x+0.25;       // 2        [-1,0,1]        1/4,0,-1/4      1/2,3/8,0
      SpaceGroupElement[165].x=-y+0.5;       SpaceGroupElement[165].y=-z+0.5;       SpaceGroupElement[165].z=x;             // 3^1      [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[166].x=z+0.75;       SpaceGroupElement[166].y=-y+0.25;      SpaceGroupElement[166].z=x+0.75;        // 2        [1,0,1]         3/4,0,3/4       0,5/8,0
      SpaceGroupElement[167].x=-z+0.25;      SpaceGroupElement[167].y=y+0.25;       SpaceGroupElement[167].z=x+0.25;        // 4^-1     [0,1,0]         0,5/4,0         1/2,0,3/4
      SpaceGroupElement[168].x=-x+0.75;      SpaceGroupElement[168].y=-y+0.75;      SpaceGroupElement[168].z=-z+0.25;       // -1       -               -               3/8,3/8,1/8
      SpaceGroupElement[169].x=y+0.5;        SpaceGroupElement[169].y=-x+0.5;       SpaceGroupElement[169].z=-z;            // -4^1     [0,0,1]         0,0,0           1/2,0,0
      SpaceGroupElement[170].x=x+0.75;       SpaceGroupElement[170].y=y+0.25;       SpaceGroupElement[170].z=-z-0.25;       // -2       [0,0,1]         3/4,1/4,0       0,0,-1/8
      SpaceGroupElement[171].x=-y;           SpaceGroupElement[171].y=x+0.5;        SpaceGroupElement[171].z=-z-0.5;        // -4^-1    [0,0,1]         0,0,0           -1/4,1/4,-1/4
      SpaceGroupElement[172].x=-x+0.75;      SpaceGroupElement[172].y=y+0.75;       SpaceGroupElement[172].z=z+0.25;        // -2       [1,0,0]         0,3/4,1/4       3/8,0,0
      SpaceGroupElement[173].x=-y+0.5;       SpaceGroupElement[173].y=-x+0.5;       SpaceGroupElement[173].z=z;             // -2       [1,1,0]         0,0,0           1/2,0,0
      SpaceGroupElement[174].x=x+0.75;       SpaceGroupElement[174].y=-y+0.25;      SpaceGroupElement[174].z=z-0.25;        // -2       [0,1,0]         3/4,0,-1/4      0,1/8,0
      SpaceGroupElement[175].x=y;            SpaceGroupElement[175].y=x+0.5;        SpaceGroupElement[175].z=z-0.5;         // -2       [-1,1,0]        1/4,1/4,-1/2    -1/4,0,0
      SpaceGroupElement[176].x=-z+0.75;      SpaceGroupElement[176].y=-x+0.75;      SpaceGroupElement[176].z=-y+0.25;       // -3^1     [1,1,1]         0,0,0           5/8,1/8,1/8
      SpaceGroupElement[177].x=x+0.5;        SpaceGroupElement[177].y=-z+0.5;       SpaceGroupElement[177].z=-y;            // -2       [0,1,1]         1/2,1/4,-1/4    0,1/4,0
      SpaceGroupElement[178].x=z+0.75;       SpaceGroupElement[178].y=x+0.25;       SpaceGroupElement[178].z=-y-0.25;       // -3^-1    [-1,1,1]        0,0,0           1/8,3/8,-5/8
      SpaceGroupElement[179].x=-x;           SpaceGroupElement[179].y=z+0.5;        SpaceGroupElement[179].z=-y-0.5;        // -4^1     [1,0,0]         0,0,0           0,0,-1/2
      SpaceGroupElement[180].x=-z+0.75;      SpaceGroupElement[180].y=x+0.75;       SpaceGroupElement[180].z=y+0.25;        // -3^-1    [1,-1,1]        0,0,0           -1/8,5/8,7/8
      SpaceGroupElement[181].x=-x+0.5;       SpaceGroupElement[181].y=-z+0.5;       SpaceGroupElement[181].z=y;             // -4^-1    [1,0,0]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[182].x=z+0.75;       SpaceGroupElement[182].y=-x+0.25;      SpaceGroupElement[182].z=y-0.25;        // -3^1     [-1,-1,1]       0,0,0           3/8,-1/8,-3/8
      SpaceGroupElement[183].x=x;            SpaceGroupElement[183].y=z+0.5;        SpaceGroupElement[183].z=y-0.5;         // -2       [0,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[184].x=-y+0.75;      SpaceGroupElement[184].y=-z+0.75;      SpaceGroupElement[184].z=-x+0.25;       // -3^-1    [1,1,1]         0,0,0           1/8,5/8,1/8
      SpaceGroupElement[185].x=-y+0.25;      SpaceGroupElement[185].y=z+0.75;       SpaceGroupElement[185].z=x-0.25;        // -3^-1    [-1,-1,1]       0,0,0           -1/8,3/8,-3/8
      SpaceGroupElement[186].x=-z+0.5;       SpaceGroupElement[186].y=-y;           SpaceGroupElement[186].z=x-0.5;         // -4^1     [0,1,0]         0,0,0           1/2,0,0
      SpaceGroupElement[187].x=y+0.25;       SpaceGroupElement[187].y=-z+0.25;      SpaceGroupElement[187].z=x+0.25;        // -3^1     [-1,1,1]        0,0,0           1/8,-1/8,3/8
      SpaceGroupElement[188].x=z+0.5;        SpaceGroupElement[188].y=y+0.5;        SpaceGroupElement[188].z=x;             // -2       [-1,0,1]        1/4,1/2,1/4     1/4,0,0
      SpaceGroupElement[189].x=y+0.75;       SpaceGroupElement[189].y=z+0.75;       SpaceGroupElement[189].z=-x+0.25;       // -3^1     [1,-1,1]        0,0,0           7/8,1/8,-5/8
      SpaceGroupElement[190].x=-z+0.5;       SpaceGroupElement[190].y=y;            SpaceGroupElement[190].z=-x-0.5;        // -2       [1,0,1]         1/2,0,-1/2      0,0,0
      SpaceGroupElement[191].x=z;            SpaceGroupElement[191].y=-y;           SpaceGroupElement[191].z=-x;            // -4^-1    [0,1,0]         0,0,0           0,0,0
      break;
    case 526: // [227] F d -3 m Origin choice 2 (-F 4vw 2vw 3)      e.g. MIL-100,101
      SpaceGroupSize=192;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x+0.25;         SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         -1/8,1/8,0
      SpaceGroupElement[2].x=-x+0.75;        SpaceGroupElement[2].y=-y+0.25;        SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         3/8,1/8,0
      SpaceGroupElement[3].x=y+0.75;         SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         3/8,-3/8,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y+0.25;        SpaceGroupElement[4].z=-z+0.25;         // 2        [1,0,0]         0,0,0           0,1/8,1/8
      SpaceGroupElement[5].x=y+0.75;         SpaceGroupElement[5].y=x+0.25;         SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         1/2,1/2,0       1/4,0,1/4
      SpaceGroupElement[6].x=-x+0.75;        SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z+0.75;         // 2        [0,1,0]         0,0,0           3/8,0,3/8
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=z+0.25;         SpaceGroupElement[9].z=y+0.25;          // 2        [0,1,1]         0,1/4,1/4       0,0,0
      SpaceGroupElement[10].x=-z+0.75;       SpaceGroupElement[10].y=-x+0.25;       SpaceGroupElement[10].z=y+0.5;          // 3^-1     [-1,1,1]        0,0,0           3/4,-1/2,0
      SpaceGroupElement[11].x=x+0.75;        SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=y+0.75;         // 4^1      [1,0,0]         3/4,0,0         0,-3/8,3/8
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x+0.25;       SpaceGroupElement[12].z=-y+0.25;        // 3^-1     [1,-1,1]        0,0,0           0,1/4,0
      SpaceGroupElement[13].x=x+0.75;        SpaceGroupElement[13].y=z+0.25;        SpaceGroupElement[13].z=-y+0.5;         // 4^-1     [1,0,0]         3/4,0,0         0,3/8,1/8
      SpaceGroupElement[14].x=-z+0.75;       SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-y+0.75;        // 3^1      [-1,-1,1]       0,0,0           3/4,3/4,0
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=-y;             // 2        [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y+0.5;         SpaceGroupElement[17].y=-z+0.75;       SpaceGroupElement[17].z=-x+0.25;        // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    7/12,5/12,0
      SpaceGroupElement[18].x=z+0.25;        SpaceGroupElement[18].y=y+0.75;        SpaceGroupElement[18].z=-x+0.5;         // 4^1      [0,1,0]         0,3/4,0         3/8,0,1/8
      SpaceGroupElement[19].x=-y+0.25;       SpaceGroupElement[19].y=z+0.5;         SpaceGroupElement[19].z=-x+0.75;        // 3^1      [-1,1,1]        -1/3,1/3,1/3    5/12,1/6,0
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-y+0.5;        SpaceGroupElement[20].z=-x+0.5;         // 2        [-1,0,1]        -1/4,0,1/4      1/4,1/4,0
      SpaceGroupElement[21].x=-y+0.25;       SpaceGroupElement[21].y=-z+0.25;       SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,1/4,0
      SpaceGroupElement[22].x=z+0.25;        SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=x+0.25;         // 2        [1,0,1]         1/4,0,1/4       0,0,0
      SpaceGroupElement[23].x=-z+0.5;        SpaceGroupElement[23].y=y+0.25;        SpaceGroupElement[23].z=x+0.75;         // 4^-1     [0,1,0]         0,1/4,0         -1/8,0,5/8
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[25].x=y;             SpaceGroupElement[25].y=-x-0.25;       SpaceGroupElement[25].z=-z-0.25;        // -4^1     [0,0,1]         0,0,0           -1/8,-1/8,-1/8
      SpaceGroupElement[26].x=x-0.75;        SpaceGroupElement[26].y=y-0.25;        SpaceGroupElement[26].z=-z-0.5;         // -2       [0,0,1]         -3/4,-1/4,0     0,0,-1/4
      SpaceGroupElement[27].x=-y-0.75;       SpaceGroupElement[27].y=x;             SpaceGroupElement[27].z=-z-0.75;        // -4^-1    [0,0,1]         0,0,0           -3/8,-3/8,-3/8
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=y-0.25;        SpaceGroupElement[28].z=z-0.25;         // -2       [1,0,0]         0,-1/4,-1/4     0,0,0
      SpaceGroupElement[29].x=-y-0.75;       SpaceGroupElement[29].y=-x-0.25;       SpaceGroupElement[29].z=z-0.5;          // -2       [1,1,0]         -1/4,1/4,-1/2   -1/2,0,0
      SpaceGroupElement[30].x=x-0.75;        SpaceGroupElement[30].y=-y;            SpaceGroupElement[30].z=z-0.75;         // -2       [0,1,0]         -3/4,0,-3/4     0,0,0
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=z;              // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[32].x=-z;            SpaceGroupElement[32].y=-x;            SpaceGroupElement[32].z=-y;             // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[33].x=x;             SpaceGroupElement[33].y=-z-0.25;       SpaceGroupElement[33].z=-y-0.25;        // -2       [0,1,1]         0,0,0           0,-1/4,0
      SpaceGroupElement[34].x=z-0.75;        SpaceGroupElement[34].y=x-0.25;        SpaceGroupElement[34].z=-y-0.5;         // -3^-1    [-1,1,1]        0,0,0           -1/2,-3/4,1/4
      SpaceGroupElement[35].x=-x-0.75;       SpaceGroupElement[35].y=z;             SpaceGroupElement[35].z=-y-0.75;        // -4^1     [1,0,0]         0,0,0           -3/8,-3/8,-3/8
      SpaceGroupElement[36].x=-z;            SpaceGroupElement[36].y=x-0.25;        SpaceGroupElement[36].z=y-0.25;         // -3^-1    [1,-1,1]        0,0,0           1/4,0,-1/4
      SpaceGroupElement[37].x=-x-0.75;       SpaceGroupElement[37].y=-z-0.25;       SpaceGroupElement[37].z=y-0.5;          // -4^-1    [1,0,0]         0,0,0           -3/8,1/8,-3/8
      SpaceGroupElement[38].x=z-0.75;        SpaceGroupElement[38].y=-x;            SpaceGroupElement[38].z=y-0.75;         // -3^1     [-1,-1,1]       0,0,0           -3/4,3/4,0
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=z;             SpaceGroupElement[39].z=y;              // -2       [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[40].x=-y;            SpaceGroupElement[40].y=-z;            SpaceGroupElement[40].z=-x;             // -3^-1    [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[41].x=-y-0.5;        SpaceGroupElement[41].y=z-0.75;        SpaceGroupElement[41].z=x-0.25;         // -3^-1    [-1,-1,1]       0,0,0           1/4,-3/4,0
      SpaceGroupElement[42].x=-z-0.25;       SpaceGroupElement[42].y=-y-0.75;       SpaceGroupElement[42].z=x-0.5;          // -4^1     [0,1,0]         0,0,0           1/8,-3/8,-3/8
      SpaceGroupElement[43].x=y-0.25;        SpaceGroupElement[43].y=-z-0.5;        SpaceGroupElement[43].z=x-0.75;         // -3^1     [-1,1,1]        0,0,0           0,1/4,-3/4
      SpaceGroupElement[44].x=z;             SpaceGroupElement[44].y=y-0.5;         SpaceGroupElement[44].z=x-0.5;          // -2       [-1,0,1]        -1/4,-1/2,-1/4  1/4,0,0
      SpaceGroupElement[45].x=y-0.25;        SpaceGroupElement[45].y=z-0.25;        SpaceGroupElement[45].z=-x;             // -3^1     [1,-1,1]        0,0,0           -1/4,0,1/4
      SpaceGroupElement[46].x=-z-0.25;       SpaceGroupElement[46].y=y;             SpaceGroupElement[46].z=-x-0.25;        // -2       [1,0,1]         0,0,0           -1/4,0,0
      SpaceGroupElement[47].x=z-0.5;         SpaceGroupElement[47].y=-y-0.25;       SpaceGroupElement[47].z=-x-0.75;        // -4^-1    [0,1,0]         0,0,0           -5/8,-1/8,-1/8
      SpaceGroupElement[48].x=x;             SpaceGroupElement[48].y=y+0.5;         SpaceGroupElement[48].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[49].x=-y;            SpaceGroupElement[49].y=x+0.75;        SpaceGroupElement[49].z=z+0.75;         // 4^1      [0,0,1]         0,0,3/4         -3/8,3/8,0
      SpaceGroupElement[50].x=-x+0.75;       SpaceGroupElement[50].y=-y+0.75;       SpaceGroupElement[50].z=z;              // 2        [0,0,1]         0,0,1           3/8,3/8,0
      SpaceGroupElement[51].x=y+0.75;        SpaceGroupElement[51].y=-x+0.5;        SpaceGroupElement[51].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         5/8,-1/8,0
      SpaceGroupElement[52].x=x;             SpaceGroupElement[52].y=-y+0.75;       SpaceGroupElement[52].z=-z+0.75;        // 2        [1,0,0]         0,0,0           0,3/8,3/8
      SpaceGroupElement[53].x=y+0.75;        SpaceGroupElement[53].y=x+0.75;        SpaceGroupElement[53].z=-z;             // 2        [1,1,0]         3/4,3/4,0       0,0,1/2
      SpaceGroupElement[54].x=-x+0.75;       SpaceGroupElement[54].y=y+0.5;         SpaceGroupElement[54].z=-z+0.25;        // 2        [0,1,0]         0,1/2,0         3/8,0,5/8
      SpaceGroupElement[55].x=-y;            SpaceGroupElement[55].y=-x+0.5;        SpaceGroupElement[55].z=-z+0.5;         // 2        [-1,1,0]        -1/4,1/4,0      1/4,0,1/4
      SpaceGroupElement[56].x=z;             SpaceGroupElement[56].y=x+0.5;         SpaceGroupElement[56].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     -1/3,-1/6,0
      SpaceGroupElement[57].x=-x;            SpaceGroupElement[57].y=z+0.75;        SpaceGroupElement[57].z=y+0.75;         // 2        [0,1,1]         0,3/4,3/4       0,0,0
      SpaceGroupElement[58].x=-z+0.75;       SpaceGroupElement[58].y=-x+0.75;       SpaceGroupElement[58].z=y;              // 3^-1     [-1,1,1]        -1/3,1/3,1/3    13/12,-2/3,0
      SpaceGroupElement[59].x=x+0.75;        SpaceGroupElement[59].y=-z+0.5;        SpaceGroupElement[59].z=y+0.25;         // 4^1      [1,0,0]         3/4,0,0         0,-3/8,7/8
      SpaceGroupElement[60].x=z;             SpaceGroupElement[60].y=-x+0.75;       SpaceGroupElement[60].z=-y+0.75;        // 3^-1     [1,-1,1]        0,0,0           0,3/4,0
      SpaceGroupElement[61].x=x+0.75;        SpaceGroupElement[61].y=z+0.75;        SpaceGroupElement[61].z=-y;             // 4^-1     [1,0,0]         3/4,0,0         0,7/8,1/8
      SpaceGroupElement[62].x=-z+0.75;       SpaceGroupElement[62].y=x+0.5;         SpaceGroupElement[62].z=-y+0.25;        // 3^1      [-1,-1,1]       0,0,0           3/4,5/4,0
      SpaceGroupElement[63].x=-x;            SpaceGroupElement[63].y=-z+0.5;        SpaceGroupElement[63].z=-y+0.5;         // 2        [0,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[64].x=y;             SpaceGroupElement[64].y=z+0.5;         SpaceGroupElement[64].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,1/6,0
      SpaceGroupElement[65].x=y+0.5;         SpaceGroupElement[65].y=-z+0.25;       SpaceGroupElement[65].z=-x+0.75;        // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    13/12,11/12,0
      SpaceGroupElement[66].x=z+0.25;        SpaceGroupElement[66].y=y+0.25;        SpaceGroupElement[66].z=-x;             // 4^1      [0,1,0]         0,5/4,0         5/8,0,3/8
      SpaceGroupElement[67].x=-y+0.25;       SpaceGroupElement[67].y=z;             SpaceGroupElement[67].z=-x+0.25;        // 3^1      [-1,1,1]        -2/3,2/3,2/3    7/12,1/3,0
      SpaceGroupElement[68].x=-z;            SpaceGroupElement[68].y=-y;            SpaceGroupElement[68].z=-x;             // 2        [-1,0,1]        -1/2,0,1/2      1/2,1/2,0
      SpaceGroupElement[69].x=-y+0.25;       SpaceGroupElement[69].y=-z+0.75;       SpaceGroupElement[69].z=x+0.5;          // 3^1      [1,-1,1]        0,0,0           -1/2,3/4,0
      SpaceGroupElement[70].x=z+0.25;        SpaceGroupElement[70].y=-y+0.5;        SpaceGroupElement[70].z=x+0.75;         // 2        [1,0,1]         1/2,0,1/2       -1/4,1/4,0
      SpaceGroupElement[71].x=-z+0.5;        SpaceGroupElement[71].y=y+0.75;        SpaceGroupElement[71].z=x+0.25;         // 4^-1     [0,1,0]         0,3/4,0         -3/8,0,7/8
      SpaceGroupElement[72].x=-x;            SpaceGroupElement[72].y=-y+0.5;        SpaceGroupElement[72].z=-z+0.5;         // -1       -               -               0,1/4,1/4
      SpaceGroupElement[73].x=y;             SpaceGroupElement[73].y=-x+0.25;       SpaceGroupElement[73].z=-z+0.25;        // -4^1     [0,0,1]         0,0,0           1/8,1/8,1/8
      SpaceGroupElement[74].x=x-0.75;        SpaceGroupElement[74].y=y+0.25;        SpaceGroupElement[74].z=-z;             // -2       [0,0,1]         -3/4,1/4,0      0,0,0
      SpaceGroupElement[75].x=-y-0.75;       SpaceGroupElement[75].y=x+0.5;         SpaceGroupElement[75].z=-z-0.25;        // -4^-1    [0,0,1]         0,0,0           -5/8,-1/8,-1/8
      SpaceGroupElement[76].x=-x;            SpaceGroupElement[76].y=y+0.25;        SpaceGroupElement[76].z=z+0.25;         // -2       [1,0,0]         0,1/4,1/4       0,0,0
      SpaceGroupElement[77].x=-y-0.75;       SpaceGroupElement[77].y=-x+0.25;       SpaceGroupElement[77].z=z;              // -2       [1,1,0]         -1/2,1/2,0      -1/4,0,0
      SpaceGroupElement[78].x=x-0.75;        SpaceGroupElement[78].y=-y+0.5;        SpaceGroupElement[78].z=z-0.25;         // -2       [0,1,0]         -3/4,0,-1/4     0,1/4,0
      SpaceGroupElement[79].x=y;             SpaceGroupElement[79].y=x+0.5;         SpaceGroupElement[79].z=z+0.5;          // -2       [-1,1,0]        1/4,1/4,1/2     -1/4,0,0
      SpaceGroupElement[80].x=-z;            SpaceGroupElement[80].y=-x+0.5;        SpaceGroupElement[80].z=-y+0.5;         // -3^1     [1,1,1]         0,0,0           0,1/2,0
      SpaceGroupElement[81].x=x;             SpaceGroupElement[81].y=-z+0.25;       SpaceGroupElement[81].z=-y+0.25;        // -2       [0,1,1]         0,0,0           0,1/4,0
      SpaceGroupElement[82].x=z-0.75;        SpaceGroupElement[82].y=x+0.25;        SpaceGroupElement[82].z=-y;             // -3^-1    [-1,1,1]        0,0,0           -1/2,-1/4,1/4
      SpaceGroupElement[83].x=-x-0.75;       SpaceGroupElement[83].y=z+0.5;         SpaceGroupElement[83].z=-y-0.25;        // -4^1     [1,0,0]         0,0,0           -3/8,1/8,-3/8
      SpaceGroupElement[84].x=-z;            SpaceGroupElement[84].y=x+0.25;        SpaceGroupElement[84].z=y+0.25;         // -3^-1    [1,-1,1]        0,0,0           -1/4,0,1/4
      SpaceGroupElement[85].x=-x-0.75;       SpaceGroupElement[85].y=-z+0.25;       SpaceGroupElement[85].z=y;              // -4^-1    [1,0,0]         0,0,0           -3/8,1/8,1/8
      SpaceGroupElement[86].x=z-0.75;        SpaceGroupElement[86].y=-x+0.5;        SpaceGroupElement[86].z=y-0.25;         // -3^1     [-1,-1,1]       0,0,0           -1/4,3/4,1/2
      SpaceGroupElement[87].x=x;             SpaceGroupElement[87].y=z+0.5;         SpaceGroupElement[87].z=y+0.5;          // -2       [0,-1,1]        0,1/2,1/2       0,0,0
      SpaceGroupElement[88].x=-y;            SpaceGroupElement[88].y=-z+0.5;        SpaceGroupElement[88].z=-x+0.5;         // -3^-1    [1,1,1]         0,0,0           0,0,1/2
      SpaceGroupElement[89].x=-y-0.5;        SpaceGroupElement[89].y=z-0.25;        SpaceGroupElement[89].z=x+0.25;         // -3^-1    [-1,-1,1]       0,0,0           -1/4,-1/4,0
      SpaceGroupElement[90].x=-z-0.25;       SpaceGroupElement[90].y=-y-0.25;       SpaceGroupElement[90].z=x;              // -4^1     [0,1,0]         0,0,0           -1/8,-1/8,-1/8
      SpaceGroupElement[91].x=y-0.25;        SpaceGroupElement[91].y=-z;            SpaceGroupElement[91].z=x-0.25;         // -3^1     [-1,1,1]        0,0,0           0,1/4,-1/4
      SpaceGroupElement[92].x=z;             SpaceGroupElement[92].y=y;             SpaceGroupElement[92].z=x;              // -2       [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[93].x=y-0.25;        SpaceGroupElement[93].y=z+0.25;        SpaceGroupElement[93].z=-x+0.5;         // -3^1     [1,-1,1]        0,0,0           1/4,1/2,1/4
      SpaceGroupElement[94].x=-z-0.25;       SpaceGroupElement[94].y=y+0.5;         SpaceGroupElement[94].z=-x+0.25;        // -2       [1,0,1]         -1/4,1/2,1/4    0,0,0
      SpaceGroupElement[95].x=z-0.5;         SpaceGroupElement[95].y=-y+0.25;       SpaceGroupElement[95].z=-x-0.25;        // -4^-1    [0,1,0]         0,0,0           -3/8,1/8,1/8
      SpaceGroupElement[96].x=x+0.5;         SpaceGroupElement[96].y=y;             SpaceGroupElement[96].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[97].x=-y+0.5;        SpaceGroupElement[97].y=x+0.25;        SpaceGroupElement[97].z=z+0.75;         // 4^1      [0,0,1]         0,0,3/4         1/8,3/8,0
      SpaceGroupElement[98].x=-x+0.25;       SpaceGroupElement[98].y=-y+0.25;       SpaceGroupElement[98].z=z;              // 2        [0,0,1]         0,0,1           5/8,1/8,0
      SpaceGroupElement[99].x=y+0.25;        SpaceGroupElement[99].y=-x;            SpaceGroupElement[99].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         5/8,-5/8,0
      SpaceGroupElement[100].x=x+0.5;        SpaceGroupElement[100].y=-y+0.25;      SpaceGroupElement[100].z=-z+0.75;       // 2        [1,0,0]         1/2,0,0         0,1/8,3/8
      SpaceGroupElement[101].x=y+0.25;       SpaceGroupElement[101].y=x+0.25;       SpaceGroupElement[101].z=-z;            // 2        [1,1,0]         3/4,3/4,0       1/2,0,1/2
      SpaceGroupElement[102].x=-x+0.25;      SpaceGroupElement[102].y=y;            SpaceGroupElement[102].z=-z+0.25;       // 2        [0,1,0]         0,0,0           5/8,0,5/8
      SpaceGroupElement[103].x=-y+0.5;       SpaceGroupElement[103].y=-x;           SpaceGroupElement[103].z=-z+0.5;        // 2        [-1,1,0]        1/4,-1/4,0      1/4,0,1/4
      SpaceGroupElement[104].x=z+0.5;        SpaceGroupElement[104].y=x;            SpaceGroupElement[104].z=y+0.5;         // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,-1/6,0
      SpaceGroupElement[105].x=-x+0.5;       SpaceGroupElement[105].y=z+0.25;       SpaceGroupElement[105].z=y+0.75;        // 2        [0,1,1]         0,1/2,1/2       1/4,-1/4,0
      SpaceGroupElement[106].x=-z+0.25;      SpaceGroupElement[106].y=-x+0.25;      SpaceGroupElement[106].z=y;             // 3^-1     [-1,1,1]        0,0,0           5/4,-1,0
      SpaceGroupElement[107].x=x+0.25;       SpaceGroupElement[107].y=-z;           SpaceGroupElement[107].z=y+0.25;        // 4^1      [1,0,0]         5/4,0,0         0,-5/8,5/8
      SpaceGroupElement[108].x=z+0.5;        SpaceGroupElement[108].y=-x+0.25;      SpaceGroupElement[108].z=-y+0.75;       // 3^-1     [1,-1,1]        1/3,-1/3,1/3    1/6,5/12,0
      SpaceGroupElement[109].x=x+0.25;       SpaceGroupElement[109].y=z+0.25;       SpaceGroupElement[109].z=-y;            // 4^-1     [1,0,0]         5/4,0,0         0,5/8,3/8
      SpaceGroupElement[110].x=-z+0.25;      SpaceGroupElement[110].y=x;            SpaceGroupElement[110].z=-y+0.25;       // 3^1      [-1,-1,1]       0,0,0           5/4,5/4,0
      SpaceGroupElement[111].x=-x+0.5;       SpaceGroupElement[111].y=-z;           SpaceGroupElement[111].z=-y+0.5;        // 2        [0,-1,1]        0,-1/4,1/4      1/4,1/4,0
      SpaceGroupElement[112].x=y+0.5;        SpaceGroupElement[112].y=z;            SpaceGroupElement[112].z=x+0.5;         // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,-1/3,0
      SpaceGroupElement[113].x=y;            SpaceGroupElement[113].y=-z+0.75;      SpaceGroupElement[113].z=-x+0.75;       // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    13/12,5/12,0
      SpaceGroupElement[114].x=z+0.75;       SpaceGroupElement[114].y=y+0.75;       SpaceGroupElement[114].z=-x;            // 4^1      [0,1,0]         0,3/4,0         7/8,0,1/8
      SpaceGroupElement[115].x=-y+0.75;      SpaceGroupElement[115].y=z+0.5;        SpaceGroupElement[115].z=-x+0.25;       // 3^1      [-1,1,1]        -1/3,1/3,1/3    11/12,1/6,0
      SpaceGroupElement[116].x=-z+0.5;       SpaceGroupElement[116].y=-y+0.5;       SpaceGroupElement[116].z=-x;            // 2        [-1,0,1]        -1/4,0,1/4      3/4,1/4,0
      SpaceGroupElement[117].x=-y+0.75;      SpaceGroupElement[117].y=-z+0.25;      SpaceGroupElement[117].z=x+0.5;         // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,7/12,0
      SpaceGroupElement[118].x=z+0.75;       SpaceGroupElement[118].y=-y;           SpaceGroupElement[118].z=x+0.75;        // 2        [1,0,1]         3/4,0,3/4       0,0,0
      SpaceGroupElement[119].x=-z;           SpaceGroupElement[119].y=y+0.25;       SpaceGroupElement[119].z=x+0.25;        // 4^-1     [0,1,0]         0,1/4,0         -1/8,0,9/8
      SpaceGroupElement[120].x=-x+0.5;       SpaceGroupElement[120].y=-y;           SpaceGroupElement[120].z=-z+0.5;        // -1       -               -               1/4,0,1/4
      SpaceGroupElement[121].x=y+0.5;        SpaceGroupElement[121].y=-x-0.25;      SpaceGroupElement[121].z=-z+0.25;       // -4^1     [0,0,1]         0,0,0           1/8,-3/8,1/8
      SpaceGroupElement[122].x=x-0.25;       SpaceGroupElement[122].y=y-0.25;       SpaceGroupElement[122].z=-z;            // -2       [0,0,1]         -1/4,-1/4,0     0,0,0
      SpaceGroupElement[123].x=-y-0.25;      SpaceGroupElement[123].y=x;            SpaceGroupElement[123].z=-z-0.25;       // -4^-1    [0,0,1]         0,0,0           -1/8,-1/8,-1/8
      SpaceGroupElement[124].x=-x+0.5;       SpaceGroupElement[124].y=y-0.25;       SpaceGroupElement[124].z=z+0.25;        // -2       [1,0,0]         0,-1/4,1/4      1/4,0,0
      SpaceGroupElement[125].x=-y-0.25;      SpaceGroupElement[125].y=-x-0.25;      SpaceGroupElement[125].z=z;             // -2       [1,1,0]         0,0,0           -1/4,0,0
      SpaceGroupElement[126].x=x-0.25;       SpaceGroupElement[126].y=-y;           SpaceGroupElement[126].z=z-0.25;        // -2       [0,1,0]         -1/4,0,-1/4     0,0,0
      SpaceGroupElement[127].x=y+0.5;        SpaceGroupElement[127].y=x;            SpaceGroupElement[127].z=z+0.5;         // -2       [-1,1,0]        1/4,1/4,1/2     1/4,0,0
      SpaceGroupElement[128].x=-z+0.5;       SpaceGroupElement[128].y=-x;           SpaceGroupElement[128].z=-y+0.5;        // -3^1     [1,1,1]         0,0,0           0,0,1/2
      SpaceGroupElement[129].x=x+0.5;        SpaceGroupElement[129].y=-z-0.25;      SpaceGroupElement[129].z=-y+0.25;       // -2       [0,1,1]         1/2,-1/4,1/4    0,0,0
      SpaceGroupElement[130].x=z-0.25;       SpaceGroupElement[130].y=x-0.25;       SpaceGroupElement[130].z=-y;            // -3^-1    [-1,1,1]        0,0,0           0,-1/4,1/4
      SpaceGroupElement[131].x=-x-0.25;      SpaceGroupElement[131].y=z;            SpaceGroupElement[131].z=-y-0.25;       // -4^1     [1,0,0]         0,0,0           -1/8,-1/8,-1/8
      SpaceGroupElement[132].x=-z+0.5;       SpaceGroupElement[132].y=x-0.25;       SpaceGroupElement[132].z=y+0.25;        // -3^-1    [1,-1,1]        0,0,0           1/4,0,1/4
      SpaceGroupElement[133].x=-x-0.25;      SpaceGroupElement[133].y=-z-0.25;      SpaceGroupElement[133].z=y;             // -4^-1    [1,0,0]         0,0,0           -1/8,-1/8,-1/8
      SpaceGroupElement[134].x=z-0.25;       SpaceGroupElement[134].y=-x;           SpaceGroupElement[134].z=y-0.25;        // -3^1     [-1,-1,1]       0,0,0           -1/4,1/4,0
      SpaceGroupElement[135].x=x+0.5;        SpaceGroupElement[135].y=z;            SpaceGroupElement[135].z=y+0.5;         // -2       [0,-1,1]        1/2,1/4,1/4     0,-1/4,0
      SpaceGroupElement[136].x=-y+0.5;       SpaceGroupElement[136].y=-z;           SpaceGroupElement[136].z=-x+0.5;        // -3^-1    [1,1,1]         0,0,0           1/2,0,0
      SpaceGroupElement[137].x=-y;           SpaceGroupElement[137].y=z-0.75;       SpaceGroupElement[137].z=x+0.25;        // -3^-1    [-1,-1,1]       0,0,0           1/4,-1/4,1/2
      SpaceGroupElement[138].x=-z+0.25;      SpaceGroupElement[138].y=-y-0.75;      SpaceGroupElement[138].z=x;             // -4^1     [0,1,0]         0,0,0           1/8,-3/8,1/8
      SpaceGroupElement[139].x=y+0.25;       SpaceGroupElement[139].y=-z-0.5;       SpaceGroupElement[139].z=x-0.25;        // -3^1     [-1,1,1]        0,0,0           0,-1/4,-1/4
      SpaceGroupElement[140].x=z+0.5;        SpaceGroupElement[140].y=y-0.5;        SpaceGroupElement[140].z=x;             // -2       [-1,0,1]        1/4,-1/2,1/4    1/4,0,0
      SpaceGroupElement[141].x=y+0.25;       SpaceGroupElement[141].y=z-0.25;       SpaceGroupElement[141].z=-x+0.5;        // -3^1     [1,-1,1]        0,0,0           1/4,0,1/4
      SpaceGroupElement[142].x=-z+0.25;      SpaceGroupElement[142].y=y;            SpaceGroupElement[142].z=-x+0.25;       // -2       [1,0,1]         0,0,0           1/4,0,0
      SpaceGroupElement[143].x=z;            SpaceGroupElement[143].y=-y-0.25;      SpaceGroupElement[143].z=-x-0.25;       // -4^-1    [0,1,0]         0,0,0           -1/8,-1/8,-1/8
      SpaceGroupElement[144].x=x+0.5;        SpaceGroupElement[144].y=y+0.5;        SpaceGroupElement[144].z=z;             // 1        -               -               -
      SpaceGroupElement[145].x=-y+0.5;       SpaceGroupElement[145].y=x+0.75;       SpaceGroupElement[145].z=z+0.25;        // 4^1      [0,0,1]         0,0,1/4         -1/8,5/8,0
      SpaceGroupElement[146].x=-x+0.25;      SpaceGroupElement[146].y=-y+0.75;      SpaceGroupElement[146].z=z+0.5;         // 2        [0,0,1]         0,0,1/2         5/8,3/8,0
      SpaceGroupElement[147].x=y+0.25;       SpaceGroupElement[147].y=-x+0.5;       SpaceGroupElement[147].z=z+0.75;        // 4^-1     [0,0,1]         0,0,3/4         7/8,-3/8,0
      SpaceGroupElement[148].x=x+0.5;        SpaceGroupElement[148].y=-y+0.75;      SpaceGroupElement[148].z=-z+0.25;       // 2        [1,0,0]         1/2,0,0         0,3/8,1/8
      SpaceGroupElement[149].x=y+0.25;       SpaceGroupElement[149].y=x+0.75;       SpaceGroupElement[149].z=-z+0.5;        // 2        [1,1,0]         1,1,0           1/4,0,1/4
      SpaceGroupElement[150].x=-x+0.25;      SpaceGroupElement[150].y=y+0.5;        SpaceGroupElement[150].z=-z+0.75;       // 2        [0,1,0]         0,1/2,0         5/8,0,3/8
      SpaceGroupElement[151].x=-y+0.5;       SpaceGroupElement[151].y=-x+0.5;       SpaceGroupElement[151].z=-z;            // 2        [-1,1,0]        0,0,0           1/2,0,0
      SpaceGroupElement[152].x=z+0.5;        SpaceGroupElement[152].y=x+0.5;        SpaceGroupElement[152].z=y;             // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,1/3,0
      SpaceGroupElement[153].x=-x+0.5;       SpaceGroupElement[153].y=z+0.75;       SpaceGroupElement[153].z=y+0.25;        // 2        [0,1,1]         0,1/2,1/2       1/4,1/4,0
      SpaceGroupElement[154].x=-z+0.25;      SpaceGroupElement[154].y=-x+0.75;      SpaceGroupElement[154].z=y+0.5;         // 3^-1     [-1,1,1]        0,0,0           5/4,-1/2,0
      SpaceGroupElement[155].x=x+0.25;       SpaceGroupElement[155].y=-z+0.5;       SpaceGroupElement[155].z=y+0.75;        // 4^1      [1,0,0]         5/4,0,0         0,-1/8,5/8
      SpaceGroupElement[156].x=z+0.5;        SpaceGroupElement[156].y=-x+0.75;      SpaceGroupElement[156].z=-y+0.25;       // 3^-1     [1,-1,1]        0,0,0           1/2,1/4,0
      SpaceGroupElement[157].x=x+0.25;       SpaceGroupElement[157].y=z+0.75;       SpaceGroupElement[157].z=-y+0.5;        // 4^-1     [1,0,0]         5/4,0,0         0,5/8,-1/8
      SpaceGroupElement[158].x=-z+0.25;      SpaceGroupElement[158].y=x+0.5;        SpaceGroupElement[158].z=-y+0.75;       // 3^1      [-1,-1,1]       1/3,1/3,-1/3    11/12,13/12,0
      SpaceGroupElement[159].x=-x+0.5;       SpaceGroupElement[159].y=-z+0.5;       SpaceGroupElement[159].z=-y;            // 2        [0,-1,1]        0,1/4,-1/4      1/4,1/4,0
      SpaceGroupElement[160].x=y+0.5;        SpaceGroupElement[160].y=z+0.5;        SpaceGroupElement[160].z=x;             // 3^-1     [1,1,1]         1/3,1/3,1/3     1/3,1/6,0
      SpaceGroupElement[161].x=y;            SpaceGroupElement[161].y=-z+0.25;      SpaceGroupElement[161].z=-x+0.25;       // 3^-1     [-1,-1,1]       2/3,2/3,-2/3    11/12,7/12,0
      SpaceGroupElement[162].x=z+0.75;       SpaceGroupElement[162].y=y+0.25;       SpaceGroupElement[162].z=-x+0.5;        // 4^1      [0,1,0]         0,5/4,0         5/8,0,-1/8
      SpaceGroupElement[163].x=-y+0.75;      SpaceGroupElement[163].y=z;            SpaceGroupElement[163].z=-x+0.75;       // 3^1      [-1,1,1]        -1/3,1/3,1/3    5/12,2/3,0
      SpaceGroupElement[164].x=-z+0.5;       SpaceGroupElement[164].y=-y;           SpaceGroupElement[164].z=-x+0.5;        // 2        [-1,0,1]        0,0,0           1/2,1/2,0
      SpaceGroupElement[165].x=-y+0.75;      SpaceGroupElement[165].y=-z+0.75;      SpaceGroupElement[165].z=x;             // 3^1      [1,-1,1]        0,0,0           0,3/4,0
      SpaceGroupElement[166].x=z+0.75;       SpaceGroupElement[166].y=-y+0.5;       SpaceGroupElement[166].z=x+0.25;        // 2        [1,0,1]         1/2,0,1/2       1/4,1/4,0
      SpaceGroupElement[167].x=-z;           SpaceGroupElement[167].y=y+0.75;       SpaceGroupElement[167].z=x+0.75;        // 4^-1     [0,1,0]         0,3/4,0         1/8,0,7/8
      SpaceGroupElement[168].x=-x+0.5;       SpaceGroupElement[168].y=-y+0.5;       SpaceGroupElement[168].z=-z;            // -1       -               -               1/4,1/4,0
      SpaceGroupElement[169].x=y+0.5;        SpaceGroupElement[169].y=-x+0.25;      SpaceGroupElement[169].z=-z-0.25;       // -4^1     [0,0,1]         0,0,0           3/8,-1/8,-1/8
      SpaceGroupElement[170].x=x-0.25;       SpaceGroupElement[170].y=y+0.25;       SpaceGroupElement[170].z=-z-0.5;        // -2       [0,0,1]         -1/4,1/4,0      0,0,-1/4
      SpaceGroupElement[171].x=-y-0.25;      SpaceGroupElement[171].y=x+0.5;        SpaceGroupElement[171].z=-z-0.75;       // -4^-1    [0,0,1]         0,0,0           -3/8,1/8,-3/8
      SpaceGroupElement[172].x=-x+0.5;       SpaceGroupElement[172].y=y+0.25;       SpaceGroupElement[172].z=z-0.25;        // -2       [1,0,0]         0,1/4,-1/4      1/4,0,0
      SpaceGroupElement[173].x=-y-0.25;      SpaceGroupElement[173].y=-x+0.25;      SpaceGroupElement[173].z=z-0.5;         // -2       [1,1,0]         -1/4,1/4,-1/2   0,0,0
      SpaceGroupElement[174].x=x-0.25;       SpaceGroupElement[174].y=-y+0.5;       SpaceGroupElement[174].z=z-0.75;        // -2       [0,1,0]         -1/4,0,-3/4     0,1/4,0
      SpaceGroupElement[175].x=y+0.5;        SpaceGroupElement[175].y=x+0.5;        SpaceGroupElement[175].z=z;             // -2       [-1,1,0]        1/2,1/2,0       0,0,0
      SpaceGroupElement[176].x=-z+0.5;       SpaceGroupElement[176].y=-x+0.5;       SpaceGroupElement[176].z=-y;            // -3^1     [1,1,1]         0,0,0           1/2,0,0
      SpaceGroupElement[177].x=x+0.5;        SpaceGroupElement[177].y=-z+0.25;      SpaceGroupElement[177].z=-y-0.25;       // -2       [0,1,1]         1/2,1/4,-1/4    0,0,0
      SpaceGroupElement[178].x=z-0.25;       SpaceGroupElement[178].y=x+0.25;       SpaceGroupElement[178].z=-y-0.5;        // -3^-1    [-1,1,1]        0,0,0           -1/2,-1/4,-1/4
      SpaceGroupElement[179].x=-x-0.25;      SpaceGroupElement[179].y=z+0.5;        SpaceGroupElement[179].z=-y-0.75;       // -4^1     [1,0,0]         0,0,0           -1/8,-1/8,-5/8
      SpaceGroupElement[180].x=-z+0.5;       SpaceGroupElement[180].y=x+0.25;       SpaceGroupElement[180].z=y-0.25;        // -3^-1    [1,-1,1]        0,0,0           1/4,1/2,1/4
      SpaceGroupElement[181].x=-x-0.25;      SpaceGroupElement[181].y=-z+0.25;      SpaceGroupElement[181].z=y-0.5;         // -4^-1    [1,0,0]         0,0,0           -1/8,3/8,-1/8
      SpaceGroupElement[182].x=z-0.25;       SpaceGroupElement[182].y=-x+0.5;       SpaceGroupElement[182].z=y-0.75;        // -3^1     [-1,-1,1]       0,0,0           -1/4,3/4,0
      SpaceGroupElement[183].x=x+0.5;        SpaceGroupElement[183].y=z+0.5;        SpaceGroupElement[183].z=y;             // -2       [0,-1,1]        1/2,1/4,1/4     0,1/4,0
      SpaceGroupElement[184].x=-y+0.5;       SpaceGroupElement[184].y=-z+0.5;       SpaceGroupElement[184].z=-x;            // -3^-1    [1,1,1]         0,0,0           0,1/2,0
      SpaceGroupElement[185].x=-y;           SpaceGroupElement[185].y=z-0.25;       SpaceGroupElement[185].z=x-0.25;        // -3^-1    [-1,-1,1]       0,0,0           1/4,-1/4,0
      SpaceGroupElement[186].x=-z+0.25;      SpaceGroupElement[186].y=-y-0.25;      SpaceGroupElement[186].z=x-0.5;         // -4^1     [0,1,0]         0,0,0           3/8,-1/8,-1/8
      SpaceGroupElement[187].x=y+0.25;       SpaceGroupElement[187].y=-z;           SpaceGroupElement[187].z=x-0.75;        // -3^1     [-1,1,1]        0,0,0           1/2,1/4,-1/4
      SpaceGroupElement[188].x=z+0.5;        SpaceGroupElement[188].y=y;            SpaceGroupElement[188].z=x-0.5;         // -2       [-1,0,1]        0,0,0           1/2,0,0
      SpaceGroupElement[189].x=y+0.25;       SpaceGroupElement[189].y=z+0.25;       SpaceGroupElement[189].z=-x;            // -3^1     [1,-1,1]        0,0,0           1/4,0,-1/4
      SpaceGroupElement[190].x=-z+0.25;      SpaceGroupElement[190].y=y+0.5;        SpaceGroupElement[190].z=-x-0.25;       // -2       [1,0,1]         1/4,1/2,-1/4    0,0,0
      SpaceGroupElement[191].x=z;            SpaceGroupElement[191].y=-y+0.25;      SpaceGroupElement[191].z=-x-0.75;       // -4^-1    [0,1,0]         0,0,0           -3/8,1/8,-3/8
      break;

    case 527: // [228] F d -3 c Origin choice 1 (F 4d 2 3 -1ad)
      SpaceGroupSize=192;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.25;        SpaceGroupElement[1].y=x+0.25;         SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         0,1/4,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y+0.5;         SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         0,1/4,0
      SpaceGroupElement[3].x=y+0.75;         SpaceGroupElement[3].y=-x+0.25;        SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         1/2,-1/4,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y+0.25;         SpaceGroupElement[5].y=x+0.25;         SpaceGroupElement[5].z=-z+0.25;         // 2        [1,1,0]         1/4,1/4,0       0,0,1/8
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.5;          // 2        [0,1,0]         0,1/2,0         0,0,1/4
      SpaceGroupElement[7].x=-y+0.75;        SpaceGroupElement[7].y=-x+0.25;        SpaceGroupElement[7].z=-z+0.75;         // 2        [-1,1,0]        1/4,-1/4,0      1/2,0,3/8
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x+0.25;        SpaceGroupElement[9].y=z+0.25;         SpaceGroupElement[9].z=y+0.25;          // 2        [0,1,1]         0,1/4,1/4       1/8,0,0
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x+0.5;        SpaceGroupElement[10].z=y+0.5;          // 3^-1     [-1,1,1]        -1/3,1/3,1/3    1/3,-1/6,0
      SpaceGroupElement[11].x=x+0.75;        SpaceGroupElement[11].y=-z+0.25;       SpaceGroupElement[11].z=y+0.75;         // 4^1      [1,0,0]         3/4,0,0         0,-1/4,1/2
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[13].x=x+0.25;        SpaceGroupElement[13].y=z+0.25;        SpaceGroupElement[13].z=-y+0.25;        // 4^-1     [1,0,0]         1/4,0,0         0,1/4,0
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x+0.5;         SpaceGroupElement[14].z=-y+0.5;         // 3^1      [-1,-1,1]       0,0,0           0,1/2,0
      SpaceGroupElement[15].x=-x+0.75;       SpaceGroupElement[15].y=-z+0.25;       SpaceGroupElement[15].z=-y+0.75;        // 2        [0,-1,1]        0,-1/4,1/4      3/8,1/2,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y+0.5;         SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=-x+0.5;         // 3^-1     [-1,-1,1]       0,0,0           1/2,0,0
      SpaceGroupElement[18].x=z+0.25;        SpaceGroupElement[18].y=y+0.75;        SpaceGroupElement[18].z=-x+0.75;        // 4^1      [0,1,0]         0,3/4,0         1/2,0,1/4
      SpaceGroupElement[19].x=-y+0.5;        SpaceGroupElement[19].y=z+0.5;         SpaceGroupElement[19].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[20].x=-z+0.25;       SpaceGroupElement[20].y=-y+0.25;       SpaceGroupElement[20].z=-x+0.25;        // 2        [-1,0,1]        0,0,0           1/4,1/8,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[22].x=z+0.25;        SpaceGroupElement[22].y=-y+0.75;       SpaceGroupElement[22].z=x+0.75;         // 2        [1,0,1]         1/2,0,1/2       -1/4,3/8,0
      SpaceGroupElement[23].x=-z+0.75;       SpaceGroupElement[23].y=y+0.75;        SpaceGroupElement[23].z=x+0.25;         // 4^-1     [0,1,0]         0,3/4,0         1/4,0,1/2
      SpaceGroupElement[24].x=-x+0.75;       SpaceGroupElement[24].y=-y+0.25;       SpaceGroupElement[24].z=-z+0.25;        // -1       -               -               3/8,1/8,1/8
      SpaceGroupElement[25].x=y+0.5;         SpaceGroupElement[25].y=-x;            SpaceGroupElement[25].z=-z;             // -4^1     [0,0,1]         0,0,0           1/4,-1/4,0
      SpaceGroupElement[26].x=x+0.75;        SpaceGroupElement[26].y=y-0.25;        SpaceGroupElement[26].z=-z-0.25;        // -2       [0,0,1]         3/4,-1/4,0      0,0,-1/8
      SpaceGroupElement[27].x=-y;            SpaceGroupElement[27].y=x;             SpaceGroupElement[27].z=-z-0.5;         // -4^-1    [0,0,1]         0,0,0           0,0,-1/4
      SpaceGroupElement[28].x=-x+0.75;       SpaceGroupElement[28].y=y+0.25;        SpaceGroupElement[28].z=z+0.25;         // -2       [1,0,0]         0,1/4,1/4       3/8,0,0
      SpaceGroupElement[29].x=-y+0.5;        SpaceGroupElement[29].y=-x;            SpaceGroupElement[29].z=z;              // -2       [1,1,0]         1/4,-1/4,0      1/4,0,0
      SpaceGroupElement[30].x=x+0.75;        SpaceGroupElement[30].y=-y-0.25;       SpaceGroupElement[30].z=z-0.25;         // -2       [0,1,0]         3/4,0,-1/4      0,-1/8,0
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=z-0.5;          // -2       [-1,1,0]        0,0,-1/2        0,0,0
      SpaceGroupElement[32].x=-z+0.75;       SpaceGroupElement[32].y=-x+0.25;       SpaceGroupElement[32].z=-y+0.25;        // -3^1     [1,1,1]         0,0,0           3/8,-1/8,3/8
      SpaceGroupElement[33].x=x+0.5;         SpaceGroupElement[33].y=-z;            SpaceGroupElement[33].z=-y;             // -2       [0,1,1]         1/2,0,0         0,0,0
      SpaceGroupElement[34].x=z+0.75;        SpaceGroupElement[34].y=x-0.25;        SpaceGroupElement[34].z=-y-0.25;        // -3^-1    [-1,1,1]        0,0,0           3/8,1/8,-3/8
      SpaceGroupElement[35].x=-x;            SpaceGroupElement[35].y=z;             SpaceGroupElement[35].z=-y-0.5;         // -4^1     [1,0,0]         0,0,0           0,-1/4,-1/4
      SpaceGroupElement[36].x=-z+0.75;       SpaceGroupElement[36].y=x+0.25;        SpaceGroupElement[36].z=y+0.25;         // -3^-1    [1,-1,1]        0,0,0           1/8,3/8,5/8
      SpaceGroupElement[37].x=-x+0.5;        SpaceGroupElement[37].y=-z;            SpaceGroupElement[37].z=y;              // -4^-1    [1,0,0]         0,0,0           1/4,0,0
      SpaceGroupElement[38].x=z+0.75;        SpaceGroupElement[38].y=-x-0.25;       SpaceGroupElement[38].z=y-0.25;         // -3^1     [-1,-1,1]       0,0,0           1/8,-3/8,-5/8
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=z;             SpaceGroupElement[39].z=y-0.5;          // -2       [0,-1,1]        0,-1/4,-1/4     0,1/4,0
      SpaceGroupElement[40].x=-y+0.75;       SpaceGroupElement[40].y=-z+0.25;       SpaceGroupElement[40].z=-x+0.25;        // -3^-1    [1,1,1]         0,0,0           3/8,3/8,-1/8
      SpaceGroupElement[41].x=-y+0.25;       SpaceGroupElement[41].y=z+0.25;        SpaceGroupElement[41].z=x-0.25;         // -3^-1    [-1,-1,1]       0,0,0           1/8,1/8,-1/8
      SpaceGroupElement[42].x=-z+0.5;        SpaceGroupElement[42].y=-y-0.5;        SpaceGroupElement[42].z=x-0.5;          // -4^1     [0,1,0]         0,0,0           1/2,-1/4,0
      SpaceGroupElement[43].x=y+0.25;        SpaceGroupElement[43].y=-z-0.25;       SpaceGroupElement[43].z=x+0.25;         // -3^1     [-1,1,1]        0,0,0           -1/8,-3/8,1/8
      SpaceGroupElement[44].x=z+0.5;         SpaceGroupElement[44].y=y;             SpaceGroupElement[44].z=x;              // -2       [-1,0,1]        1/4,0,1/4       1/4,0,0
      SpaceGroupElement[45].x=y+0.75;        SpaceGroupElement[45].y=z+0.25;        SpaceGroupElement[45].z=-x+0.25;        // -3^1     [1,-1,1]        0,0,0           5/8,-1/8,-3/8
      SpaceGroupElement[46].x=-z+0.5;        SpaceGroupElement[46].y=y-0.5;         SpaceGroupElement[46].z=-x-0.5;         // -2       [1,0,1]         1/2,-1/2,-1/2   0,0,0
      SpaceGroupElement[47].x=z;             SpaceGroupElement[47].y=-y-0.5;        SpaceGroupElement[47].z=-x;             // -4^-1    [0,1,0]         0,0,0           0,-1/4,0
      SpaceGroupElement[48].x=x;             SpaceGroupElement[48].y=y+0.5;         SpaceGroupElement[48].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[49].x=-y+0.25;       SpaceGroupElement[49].y=x+0.75;        SpaceGroupElement[49].z=z+0.75;         // 4^1      [0,0,1]         0,0,3/4         -1/4,1/2,0
      SpaceGroupElement[50].x=-x;            SpaceGroupElement[50].y=-y;            SpaceGroupElement[50].z=z;              // 2        [0,0,1]         0,0,1           0,1/2,0
      SpaceGroupElement[51].x=y+0.75;        SpaceGroupElement[51].y=-x+0.75;       SpaceGroupElement[51].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         3/4,0,0
      SpaceGroupElement[52].x=x;             SpaceGroupElement[52].y=-y+0.5;        SpaceGroupElement[52].z=-z+0.5;         // 2        [1,0,0]         0,0,0           0,1/4,1/4
      SpaceGroupElement[53].x=y+0.25;        SpaceGroupElement[53].y=x+0.75;        SpaceGroupElement[53].z=-z+0.75;        // 2        [1,1,0]         1/2,1/2,0       -1/4,0,3/8
      SpaceGroupElement[54].x=-x;            SpaceGroupElement[54].y=y;             SpaceGroupElement[54].z=-z;             // 2        [0,1,0]         0,1,0           0,0,1/2
      SpaceGroupElement[55].x=-y+0.75;       SpaceGroupElement[55].y=-x+0.75;       SpaceGroupElement[55].z=-z+0.25;        // 2        [-1,1,0]        0,0,0           3/4,0,5/8
      SpaceGroupElement[56].x=z;             SpaceGroupElement[56].y=x+0.5;         SpaceGroupElement[56].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     -1/3,-1/6,0
      SpaceGroupElement[57].x=-x+0.25;       SpaceGroupElement[57].y=z+0.75;        SpaceGroupElement[57].z=y+0.75;         // 2        [0,1,1]         0,3/4,3/4       1/8,0,0
      SpaceGroupElement[58].x=-z;            SpaceGroupElement[58].y=-x;            SpaceGroupElement[58].z=y;              // 3^-1     [-1,1,1]        -2/3,2/3,2/3    2/3,-1/3,0
      SpaceGroupElement[59].x=x+0.75;        SpaceGroupElement[59].y=-z+0.75;       SpaceGroupElement[59].z=y+0.25;         // 4^1      [1,0,0]         3/4,0,0         0,-1/4,1
      SpaceGroupElement[60].x=z;             SpaceGroupElement[60].y=-x+0.5;        SpaceGroupElement[60].z=-y+0.5;         // 3^-1     [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[61].x=x+0.25;        SpaceGroupElement[61].y=z+0.75;        SpaceGroupElement[61].z=-y+0.75;        // 4^-1     [1,0,0]         1/4,0,0         0,3/4,0
      SpaceGroupElement[62].x=-z;            SpaceGroupElement[62].y=x;             SpaceGroupElement[62].z=-y;             // 3^1      [-1,-1,1]       0,0,0           0,1,0
      SpaceGroupElement[63].x=-x+0.75;       SpaceGroupElement[63].y=-z+0.75;       SpaceGroupElement[63].z=-y+0.25;        // 2        [0,-1,1]        0,-1/4,1/4      3/8,1,0
      SpaceGroupElement[64].x=y;             SpaceGroupElement[64].y=z+0.5;         SpaceGroupElement[64].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,1/6,0
      SpaceGroupElement[65].x=y+0.5;         SpaceGroupElement[65].y=-z+0.5;        SpaceGroupElement[65].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           1,1/2,0
      SpaceGroupElement[66].x=z+0.25;        SpaceGroupElement[66].y=y+0.25;        SpaceGroupElement[66].z=-x+0.25;        // 4^1      [0,1,0]         0,5/4,0         3/4,0,1/2
      SpaceGroupElement[67].x=-y+0.5;        SpaceGroupElement[67].y=z;             SpaceGroupElement[67].z=-x+0.5;         // 3^1      [-1,1,1]        -1/3,1/3,1/3    1/6,2/3,0
      SpaceGroupElement[68].x=-z+0.25;       SpaceGroupElement[68].y=-y+0.75;       SpaceGroupElement[68].z=-x+0.75;        // 2        [-1,0,1]        -1/4,0,1/4      1/2,3/8,0
      SpaceGroupElement[69].x=-y;            SpaceGroupElement[69].y=-z+0.5;        SpaceGroupElement[69].z=x+0.5;          // 3^1      [1,-1,1]        0,0,0           -1/2,1/2,0
      SpaceGroupElement[70].x=z+0.25;        SpaceGroupElement[70].y=-y+0.25;       SpaceGroupElement[70].z=x+0.25;         // 2        [1,0,1]         3/4,0,3/4       -1/2,5/8,0
      SpaceGroupElement[71].x=-z+0.75;       SpaceGroupElement[71].y=y+0.25;        SpaceGroupElement[71].z=x+0.75;         // 4^-1     [0,1,0]         0,5/4,0         0,0,3/4
      SpaceGroupElement[72].x=-x+0.75;       SpaceGroupElement[72].y=-y+0.75;       SpaceGroupElement[72].z=-z+0.75;        // -1       -               -               3/8,3/8,3/8
      SpaceGroupElement[73].x=y+0.5;         SpaceGroupElement[73].y=-x+0.5;        SpaceGroupElement[73].z=-z+0.5;         // -4^1     [0,0,1]         0,0,0           1/2,0,1/4
      SpaceGroupElement[74].x=x+0.75;        SpaceGroupElement[74].y=y+0.25;        SpaceGroupElement[74].z=-z+0.25;        // -2       [0,0,1]         3/4,1/4,0       0,0,1/8
      SpaceGroupElement[75].x=-y;            SpaceGroupElement[75].y=x+0.5;         SpaceGroupElement[75].z=-z;             // -4^-1    [0,0,1]         0,0,0           -1/4,1/4,0
      SpaceGroupElement[76].x=-x+0.75;       SpaceGroupElement[76].y=y+0.75;        SpaceGroupElement[76].z=z+0.75;         // -2       [1,0,0]         0,3/4,3/4       3/8,0,0
      SpaceGroupElement[77].x=-y+0.5;        SpaceGroupElement[77].y=-x+0.5;        SpaceGroupElement[77].z=z+0.5;          // -2       [1,1,0]         0,0,1/2         1/2,0,0
      SpaceGroupElement[78].x=x+0.75;        SpaceGroupElement[78].y=-y+0.25;       SpaceGroupElement[78].z=z+0.25;         // -2       [0,1,0]         3/4,0,1/4       0,1/8,0
      SpaceGroupElement[79].x=y;             SpaceGroupElement[79].y=x+0.5;         SpaceGroupElement[79].z=z;              // -2       [-1,1,0]        1/4,1/4,0       -1/4,0,0
      SpaceGroupElement[80].x=-z+0.75;       SpaceGroupElement[80].y=-x+0.75;       SpaceGroupElement[80].z=-y+0.75;        // -3^1     [1,1,1]         0,0,0           3/8,3/8,3/8
      SpaceGroupElement[81].x=x+0.5;         SpaceGroupElement[81].y=-z+0.5;        SpaceGroupElement[81].z=-y+0.5;         // -2       [0,1,1]         1/2,0,0         0,1/2,0
      SpaceGroupElement[82].x=z+0.75;        SpaceGroupElement[82].y=x+0.25;        SpaceGroupElement[82].z=-y+0.25;        // -3^-1    [-1,1,1]        0,0,0           3/8,5/8,-3/8
      SpaceGroupElement[83].x=-x;            SpaceGroupElement[83].y=z+0.5;         SpaceGroupElement[83].z=-y;             // -4^1     [1,0,0]         0,0,0           0,1/4,-1/4
      SpaceGroupElement[84].x=-z+0.75;       SpaceGroupElement[84].y=x+0.75;        SpaceGroupElement[84].z=y+0.75;         // -3^-1    [1,-1,1]        0,0,0           -3/8,3/8,9/8
      SpaceGroupElement[85].x=-x+0.5;        SpaceGroupElement[85].y=-z+0.5;        SpaceGroupElement[85].z=y+0.5;          // -4^-1    [1,0,0]         0,0,0           1/4,0,1/2
      SpaceGroupElement[86].x=z+0.75;        SpaceGroupElement[86].y=-x+0.25;       SpaceGroupElement[86].z=y+0.25;         // -3^1     [-1,-1,1]       0,0,0           5/8,-3/8,-1/8
      SpaceGroupElement[87].x=x;             SpaceGroupElement[87].y=z+0.5;         SpaceGroupElement[87].z=y;              // -2       [0,-1,1]        0,1/4,1/4       0,1/4,0
      SpaceGroupElement[88].x=-y+0.75;       SpaceGroupElement[88].y=-z+0.75;       SpaceGroupElement[88].z=-x+0.75;        // -3^-1    [1,1,1]         0,0,0           3/8,3/8,3/8
      SpaceGroupElement[89].x=-y+0.25;       SpaceGroupElement[89].y=z+0.75;        SpaceGroupElement[89].z=x+0.25;         // -3^-1    [-1,-1,1]       0,0,0           -3/8,5/8,-1/8
      SpaceGroupElement[90].x=-z+0.5;        SpaceGroupElement[90].y=-y;            SpaceGroupElement[90].z=x;              // -4^1     [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[91].x=y+0.25;        SpaceGroupElement[91].y=-z+0.25;       SpaceGroupElement[91].z=x+0.75;         // -3^1     [-1,1,1]        0,0,0           -1/8,-3/8,5/8
      SpaceGroupElement[92].x=z+0.5;         SpaceGroupElement[92].y=y+0.5;         SpaceGroupElement[92].z=x+0.5;          // -2       [-1,0,1]        1/2,1/2,1/2     0,0,0
      SpaceGroupElement[93].x=y+0.75;        SpaceGroupElement[93].y=z+0.75;        SpaceGroupElement[93].z=-x+0.75;        // -3^1     [1,-1,1]        0,0,0           9/8,3/8,-3/8
      SpaceGroupElement[94].x=-z+0.5;        SpaceGroupElement[94].y=y;             SpaceGroupElement[94].z=-x;             // -2       [1,0,1]         1/4,0,-1/4      1/4,0,0
      SpaceGroupElement[95].x=z;             SpaceGroupElement[95].y=-y;            SpaceGroupElement[95].z=-x+0.5;         // -4^-1    [0,1,0]         0,0,0           1/4,0,1/4
      SpaceGroupElement[96].x=x+0.5;         SpaceGroupElement[96].y=y;             SpaceGroupElement[96].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[97].x=-y+0.75;       SpaceGroupElement[97].y=x+0.25;        SpaceGroupElement[97].z=z+0.75;         // 4^1      [0,0,1]         0,0,3/4         1/4,1/2,0
      SpaceGroupElement[98].x=-x+0.5;        SpaceGroupElement[98].y=-y+0.5;        SpaceGroupElement[98].z=z;              // 2        [0,0,1]         0,0,1           1/4,1/4,0
      SpaceGroupElement[99].x=y+0.25;        SpaceGroupElement[99].y=-x+0.25;       SpaceGroupElement[99].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         3/4,-1/2,0
      SpaceGroupElement[100].x=x+0.5;        SpaceGroupElement[100].y=-y;           SpaceGroupElement[100].z=-z+0.5;        // 2        [1,0,0]         1/2,0,0         0,0,1/4
      SpaceGroupElement[101].x=y+0.75;       SpaceGroupElement[101].y=x+0.25;       SpaceGroupElement[101].z=-z+0.75;       // 2        [1,1,0]         1/2,1/2,0       1/4,0,3/8
      SpaceGroupElement[102].x=-x+0.5;       SpaceGroupElement[102].y=y+0.5;        SpaceGroupElement[102].z=-z;            // 2        [0,1,0]         0,1/2,0         1/4,0,1/2
      SpaceGroupElement[103].x=-y+0.25;      SpaceGroupElement[103].y=-x+0.25;      SpaceGroupElement[103].z=-z+0.25;       // 2        [-1,1,0]        1/2,-1/2,0      3/4,0,5/8
      SpaceGroupElement[104].x=z+0.5;        SpaceGroupElement[104].y=x;            SpaceGroupElement[104].z=y+0.5;         // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,-1/6,0
      SpaceGroupElement[105].x=-x+0.75;      SpaceGroupElement[105].y=z+0.25;       SpaceGroupElement[105].z=y+0.75;        // 2        [0,1,1]         0,1/2,1/2       3/8,-1/4,0
      SpaceGroupElement[106].x=-z+0.5;       SpaceGroupElement[106].y=-x+0.5;       SpaceGroupElement[106].z=y;             // 3^-1     [-1,1,1]        -1/3,1/3,1/3    5/6,-2/3,0
      SpaceGroupElement[107].x=x+0.25;       SpaceGroupElement[107].y=-z+0.25;      SpaceGroupElement[107].z=y+0.25;        // 4^1      [1,0,0]         5/4,0,0         0,-1/2,3/4
      SpaceGroupElement[108].x=z+0.5;        SpaceGroupElement[108].y=-x;           SpaceGroupElement[108].z=-y+0.5;        // 3^-1     [1,-1,1]        1/3,-1/3,1/3    1/6,1/6,0
      SpaceGroupElement[109].x=x+0.75;       SpaceGroupElement[109].y=z+0.25;       SpaceGroupElement[109].z=-y+0.75;       // 4^-1     [1,0,0]         3/4,0,0         0,1/2,1/4
      SpaceGroupElement[110].x=-z+0.5;       SpaceGroupElement[110].y=x+0.5;        SpaceGroupElement[110].z=-y;            // 3^1      [-1,-1,1]       0,0,0           1/2,1,0
      SpaceGroupElement[111].x=-x+0.25;      SpaceGroupElement[111].y=-z+0.25;      SpaceGroupElement[111].z=-y+0.25;       // 2        [0,-1,1]        0,-1/2,1/2      5/8,3/4,0
      SpaceGroupElement[112].x=y+0.5;        SpaceGroupElement[112].y=z;            SpaceGroupElement[112].z=x+0.5;         // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,-1/3,0
      SpaceGroupElement[113].x=y;            SpaceGroupElement[113].y=-z;           SpaceGroupElement[113].z=-x;            // 3^-1     [-1,-1,1]       0,0,0           1,0,0
      SpaceGroupElement[114].x=z+0.75;       SpaceGroupElement[114].y=y+0.75;       SpaceGroupElement[114].z=-x+0.25;       // 4^1      [0,1,0]         0,3/4,0         1,0,1/4
      SpaceGroupElement[115].x=-y;           SpaceGroupElement[115].y=z+0.5;        SpaceGroupElement[115].z=-x+0.5;        // 3^1      [-1,1,1]        0,0,0           1/2,1/2,0
      SpaceGroupElement[116].x=-z+0.75;      SpaceGroupElement[116].y=-y+0.25;      SpaceGroupElement[116].z=-x+0.75;       // 2        [-1,0,1]        0,0,0           3/4,1/8,0
      SpaceGroupElement[117].x=-y+0.5;       SpaceGroupElement[117].y=-z;           SpaceGroupElement[117].z=x+0.5;         // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      SpaceGroupElement[118].x=z+0.75;       SpaceGroupElement[118].y=-y+0.75;      SpaceGroupElement[118].z=x+0.25;        // 2        [1,0,1]         1,0,1           -1/4,3/8,0
      SpaceGroupElement[119].x=-z+0.25;      SpaceGroupElement[119].y=y+0.75;       SpaceGroupElement[119].z=x+0.75;        // 4^-1     [0,1,0]         0,3/4,0         1/4,0,1
      SpaceGroupElement[120].x=-x+0.25;      SpaceGroupElement[120].y=-y+0.25;      SpaceGroupElement[120].z=-z+0.75;       // -1       -               -               5/8,1/8,3/8
      SpaceGroupElement[121].x=y;            SpaceGroupElement[121].y=-x;           SpaceGroupElement[121].z=-z+0.5;        // -4^1     [0,0,1]         0,0,0           1/2,-1/2,1/4
      SpaceGroupElement[122].x=x+0.25;       SpaceGroupElement[122].y=y-0.25;       SpaceGroupElement[122].z=-z+0.25;       // -2       [0,0,1]         5/4,-1/4,0      0,0,1/8
      SpaceGroupElement[123].x=-y+0.5;       SpaceGroupElement[123].y=x;            SpaceGroupElement[123].z=-z;            // -4^-1    [0,0,1]         0,0,0           1/4,1/4,0
      SpaceGroupElement[124].x=-x+0.25;      SpaceGroupElement[124].y=y+0.25;       SpaceGroupElement[124].z=z+0.75;        // -2       [1,0,0]         0,1/4,3/4       5/8,0,0
      SpaceGroupElement[125].x=-y;           SpaceGroupElement[125].y=-x;           SpaceGroupElement[125].z=z+0.5;         // -2       [1,1,0]         1/2,-1/2,1/2    1/2,0,0
      SpaceGroupElement[126].x=x+0.25;       SpaceGroupElement[126].y=-y-0.25;      SpaceGroupElement[126].z=z+0.25;        // -2       [0,1,0]         5/4,0,1/4       0,-1/8,0
      SpaceGroupElement[127].x=y+0.5;        SpaceGroupElement[127].y=x;            SpaceGroupElement[127].z=z;             // -2       [-1,1,0]        1/4,1/4,0       1/4,0,0
      SpaceGroupElement[128].x=-z+0.25;      SpaceGroupElement[128].y=-x+0.25;      SpaceGroupElement[128].z=-y+0.75;       // -3^1     [1,1,1]         0,0,0           3/8,-1/8,7/8
      SpaceGroupElement[129].x=x;            SpaceGroupElement[129].y=-z;           SpaceGroupElement[129].z=-y+0.5;        // -2       [0,1,1]         1,-1/4,1/4      0,1/4,0
      SpaceGroupElement[130].x=z+0.25;       SpaceGroupElement[130].y=x-0.25;       SpaceGroupElement[130].z=-y+0.25;       // -3^-1    [-1,1,1]        0,0,0           7/8,5/8,-3/8
      SpaceGroupElement[131].x=-x+0.5;       SpaceGroupElement[131].y=z;            SpaceGroupElement[131].z=-y;            // -4^1     [1,0,0]         0,0,0           1/4,0,0
      SpaceGroupElement[132].x=-z+0.25;      SpaceGroupElement[132].y=x+0.25;       SpaceGroupElement[132].z=y+0.75;        // -3^-1    [1,-1,1]        0,0,0           1/8,3/8,9/8
      SpaceGroupElement[133].x=-x;           SpaceGroupElement[133].y=-z;           SpaceGroupElement[133].z=y+0.5;         // -4^-1    [1,0,0]         0,0,0           1/2,-1/4,1/4
      SpaceGroupElement[134].x=z+0.25;       SpaceGroupElement[134].y=-x-0.25;      SpaceGroupElement[134].z=y+0.25;        // -3^1     [-1,-1,1]       0,0,0           5/8,-7/8,-5/8
      SpaceGroupElement[135].x=x+0.5;        SpaceGroupElement[135].y=z;            SpaceGroupElement[135].z=y;             // -2       [0,-1,1]        1/2,0,0         0,0,0
      SpaceGroupElement[136].x=-y+0.25;      SpaceGroupElement[136].y=-z+0.25;      SpaceGroupElement[136].z=-x+0.75;       // -3^-1    [1,1,1]         0,0,0           7/8,3/8,-1/8
      SpaceGroupElement[137].x=-y+0.75;      SpaceGroupElement[137].y=z+0.25;       SpaceGroupElement[137].z=x+0.25;        // -3^-1    [-1,-1,1]       0,0,0           1/8,5/8,3/8
      SpaceGroupElement[138].x=-z;           SpaceGroupElement[138].y=-y-0.5;       SpaceGroupElement[138].z=x;             // -4^1     [0,1,0]         0,0,0           1/2,-1/4,1/2
      SpaceGroupElement[139].x=y+0.75;       SpaceGroupElement[139].y=-z-0.25;      SpaceGroupElement[139].z=x+0.75;        // -3^1     [-1,1,1]        0,0,0           -1/8,-7/8,5/8
      SpaceGroupElement[140].x=z;            SpaceGroupElement[140].y=y;            SpaceGroupElement[140].z=x+0.5;         // -2       [-1,0,1]        3/4,0,3/4       1/4,0,0
      SpaceGroupElement[141].x=y+0.25;       SpaceGroupElement[141].y=z+0.25;       SpaceGroupElement[141].z=-x+0.75;       // -3^1     [1,-1,1]        0,0,0           9/8,-1/8,-3/8
      SpaceGroupElement[142].x=-z;           SpaceGroupElement[142].y=y-0.5;        SpaceGroupElement[142].z=-x;            // -2       [1,0,1]         1/2,-1/2,-1/2   1/2,0,0
      SpaceGroupElement[143].x=z+0.5;        SpaceGroupElement[143].y=-y-0.5;       SpaceGroupElement[143].z=-x+0.5;        // -4^-1    [0,1,0]         0,0,0           1/2,-1/4,0
      SpaceGroupElement[144].x=x+0.5;        SpaceGroupElement[144].y=y+0.5;        SpaceGroupElement[144].z=z;             // 1        -               -               -
      SpaceGroupElement[145].x=-y+0.75;      SpaceGroupElement[145].y=x+0.75;       SpaceGroupElement[145].z=z+0.25;        // 4^1      [0,0,1]         0,0,1/4         0,3/4,0
      SpaceGroupElement[146].x=-x+0.5;       SpaceGroupElement[146].y=-y;           SpaceGroupElement[146].z=z+0.5;         // 2        [0,0,1]         0,0,1/2         1/4,1/2,0
      SpaceGroupElement[147].x=y+0.25;       SpaceGroupElement[147].y=-x+0.75;      SpaceGroupElement[147].z=z+0.75;        // 4^-1     [0,0,1]         0,0,3/4         1,-1/4,0
      SpaceGroupElement[148].x=x+0.5;        SpaceGroupElement[148].y=-y+0.5;       SpaceGroupElement[148].z=-z;            // 2        [1,0,0]         1/2,0,0         0,1/4,0
      SpaceGroupElement[149].x=y+0.75;       SpaceGroupElement[149].y=x+0.75;       SpaceGroupElement[149].z=-z+0.25;       // 2        [1,1,0]         3/4,3/4,0       0,0,1/8
      SpaceGroupElement[150].x=-x+0.5;       SpaceGroupElement[150].y=y;            SpaceGroupElement[150].z=-z+0.5;        // 2        [0,1,0]         0,1,0           1/4,0,1/4
      SpaceGroupElement[151].x=-y+0.25;      SpaceGroupElement[151].y=-x+0.75;      SpaceGroupElement[151].z=-z+0.75;       // 2        [-1,1,0]        1/4,-1/4,0      1,0,3/8
      SpaceGroupElement[152].x=z+0.5;        SpaceGroupElement[152].y=x+0.5;        SpaceGroupElement[152].z=y;             // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,1/3,0
      SpaceGroupElement[153].x=-x+0.75;      SpaceGroupElement[153].y=z+0.75;       SpaceGroupElement[153].z=y+0.25;        // 2        [0,1,1]         0,1/2,1/2       3/8,1/4,0
      SpaceGroupElement[154].x=-z+0.5;       SpaceGroupElement[154].y=-x;           SpaceGroupElement[154].z=y+0.5;         // 3^-1     [-1,1,1]        -1/3,1/3,1/3    5/6,-1/6,0
      SpaceGroupElement[155].x=x+0.25;       SpaceGroupElement[155].y=-z+0.75;      SpaceGroupElement[155].z=y+0.75;        // 4^1      [1,0,0]         5/4,0,0         0,0,3/4
      SpaceGroupElement[156].x=z+0.5;        SpaceGroupElement[156].y=-x+0.5;       SpaceGroupElement[156].z=-y;            // 3^-1     [1,-1,1]        0,0,0           1/2,0,0
      SpaceGroupElement[157].x=x+0.75;       SpaceGroupElement[157].y=z+0.75;       SpaceGroupElement[157].z=-y+0.25;       // 4^-1     [1,0,0]         3/4,0,0         0,1/2,-1/4
      SpaceGroupElement[158].x=-z+0.5;       SpaceGroupElement[158].y=x;            SpaceGroupElement[158].z=-y+0.5;        // 3^1      [-1,-1,1]       1/3,1/3,-1/3    1/6,5/6,0
      SpaceGroupElement[159].x=-x+0.25;      SpaceGroupElement[159].y=-z+0.75;      SpaceGroupElement[159].z=-y+0.75;       // 2        [0,-1,1]        0,0,0           5/8,3/4,0
      SpaceGroupElement[160].x=y+0.5;        SpaceGroupElement[160].y=z+0.5;        SpaceGroupElement[160].z=x;             // 3^-1     [1,1,1]         1/3,1/3,1/3     1/3,1/6,0
      SpaceGroupElement[161].x=y;            SpaceGroupElement[161].y=-z+0.5;       SpaceGroupElement[161].z=-x+0.5;        // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    5/6,1/6,0
      SpaceGroupElement[162].x=z+0.75;       SpaceGroupElement[162].y=y+0.25;       SpaceGroupElement[162].z=-x+0.75;       // 4^1      [0,1,0]         0,5/4,0         3/4,0,0
      SpaceGroupElement[163].x=-y;           SpaceGroupElement[163].y=z;            SpaceGroupElement[163].z=-x;            // 3^1      [-1,1,1]        0,0,0           0,1,0
      SpaceGroupElement[164].x=-z+0.75;      SpaceGroupElement[164].y=-y+0.75;      SpaceGroupElement[164].z=-x+0.25;       // 2        [-1,0,1]        1/4,0,-1/4      1/2,3/8,0
      SpaceGroupElement[165].x=-y+0.5;       SpaceGroupElement[165].y=-z+0.5;       SpaceGroupElement[165].z=x;             // 3^1      [1,-1,1]        0,0,0           0,1/2,0
      SpaceGroupElement[166].x=z+0.75;       SpaceGroupElement[166].y=-y+0.25;      SpaceGroupElement[166].z=x+0.75;        // 2        [1,0,1]         3/4,0,3/4       0,5/8,0
      SpaceGroupElement[167].x=-z+0.25;      SpaceGroupElement[167].y=y+0.25;       SpaceGroupElement[167].z=x+0.25;        // 4^-1     [0,1,0]         0,5/4,0         1/2,0,3/4
      SpaceGroupElement[168].x=-x+0.25;      SpaceGroupElement[168].y=-y+0.75;      SpaceGroupElement[168].z=-z+0.25;       // -1       -               -               5/8,3/8,1/8
      SpaceGroupElement[169].x=y;            SpaceGroupElement[169].y=-x+0.5;       SpaceGroupElement[169].z=-z;            // -4^1     [0,0,1]         0,0,0           3/4,-1/4,0
      SpaceGroupElement[170].x=x+0.25;       SpaceGroupElement[170].y=y+0.25;       SpaceGroupElement[170].z=-z-0.25;       // -2       [0,0,1]         5/4,1/4,0       0,0,-1/8
      SpaceGroupElement[171].x=-y+0.5;       SpaceGroupElement[171].y=x+0.5;        SpaceGroupElement[171].z=-z-0.5;        // -4^-1    [0,0,1]         0,0,0           0,1/2,-1/4
      SpaceGroupElement[172].x=-x+0.25;      SpaceGroupElement[172].y=y+0.75;       SpaceGroupElement[172].z=z+0.25;        // -2       [1,0,0]         0,3/4,1/4       5/8,0,0
      SpaceGroupElement[173].x=-y;           SpaceGroupElement[173].y=-x+0.5;       SpaceGroupElement[173].z=z;             // -2       [1,1,0]         1/4,-1/4,0      3/4,0,0
      SpaceGroupElement[174].x=x+0.25;       SpaceGroupElement[174].y=-y+0.25;      SpaceGroupElement[174].z=z-0.25;        // -2       [0,1,0]         5/4,0,-1/4      0,1/8,0
      SpaceGroupElement[175].x=y+0.5;        SpaceGroupElement[175].y=x+0.5;        SpaceGroupElement[175].z=z-0.5;         // -2       [-1,1,0]        1/2,1/2,-1/2    0,0,0
      SpaceGroupElement[176].x=-z+0.25;      SpaceGroupElement[176].y=-x+0.75;      SpaceGroupElement[176].z=-y+0.25;       // -3^1     [1,1,1]         0,0,0           7/8,-1/8,3/8
      SpaceGroupElement[177].x=x;            SpaceGroupElement[177].y=-z+0.5;       SpaceGroupElement[177].z=-y;            // -2       [0,1,1]         1,1/4,-1/4      0,1/4,0
      SpaceGroupElement[178].x=z+0.25;       SpaceGroupElement[178].y=x+0.25;       SpaceGroupElement[178].z=-y-0.25;       // -3^-1    [-1,1,1]        0,0,0           3/8,5/8,-7/8
      SpaceGroupElement[179].x=-x+0.5;       SpaceGroupElement[179].y=z+0.5;        SpaceGroupElement[179].z=-y-0.5;        // -4^1     [1,0,0]         0,0,0           1/4,0,-1/2
      SpaceGroupElement[180].x=-z+0.25;      SpaceGroupElement[180].y=x+0.75;       SpaceGroupElement[180].z=y+0.25;        // -3^-1    [1,-1,1]        0,0,0           1/8,7/8,9/8
      SpaceGroupElement[181].x=-x;           SpaceGroupElement[181].y=-z+0.5;       SpaceGroupElement[181].z=y;             // -4^-1    [1,0,0]         0,0,0           1/2,1/4,1/4
      SpaceGroupElement[182].x=z+0.25;       SpaceGroupElement[182].y=-x+0.25;      SpaceGroupElement[182].z=y-0.25;        // -3^1     [-1,-1,1]       0,0,0           5/8,-3/8,-5/8
      SpaceGroupElement[183].x=x+0.5;        SpaceGroupElement[183].y=z+0.5;        SpaceGroupElement[183].z=y-0.5;         // -2       [0,-1,1]        1/2,0,0         0,1/2,0
      SpaceGroupElement[184].x=-y+0.25;      SpaceGroupElement[184].y=-z+0.75;      SpaceGroupElement[184].z=-x+0.25;       // -3^-1    [1,1,1]         0,0,0           3/8,7/8,-1/8
      SpaceGroupElement[185].x=-y+0.75;      SpaceGroupElement[185].y=z+0.75;       SpaceGroupElement[185].z=x-0.25;        // -3^-1    [-1,-1,1]       0,0,0           1/8,5/8,-1/8
      SpaceGroupElement[186].x=-z;           SpaceGroupElement[186].y=-y;           SpaceGroupElement[186].z=x-0.5;         // -4^1     [0,1,0]         0,0,0           3/4,0,1/4
      SpaceGroupElement[187].x=y+0.75;       SpaceGroupElement[187].y=-z+0.25;      SpaceGroupElement[187].z=x+0.25;        // -3^1     [-1,1,1]        0,0,0           3/8,-3/8,5/8
      SpaceGroupElement[188].x=z;            SpaceGroupElement[188].y=y+0.5;        SpaceGroupElement[188].z=x;             // -2       [-1,0,1]        1/2,1/2,1/2     1/2,0,0
      SpaceGroupElement[189].x=y+0.25;       SpaceGroupElement[189].y=z+0.75;       SpaceGroupElement[189].z=-x+0.25;       // -3^1     [1,-1,1]        0,0,0           9/8,-1/8,-7/8
      SpaceGroupElement[190].x=-z;           SpaceGroupElement[190].y=y;            SpaceGroupElement[190].z=-x-0.5;        // -2       [1,0,1]         3/4,0,-3/4      1/4,0,0
      SpaceGroupElement[191].x=z+0.5;        SpaceGroupElement[191].y=-y;           SpaceGroupElement[191].z=-x;            // -4^-1    [0,1,0]         0,0,0           1/4,0,-1/4
      break;
    case 528: // [228] F d -3 c Origin choice 2 (-F 4ud 2vw 3)
      SpaceGroupSize=192;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.5;         SpaceGroupElement[1].y=x+0.25;         SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         1/8,3/8,0
      SpaceGroupElement[2].x=-x+0.25;        SpaceGroupElement[2].y=-y+0.75;        SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/8,3/8,0
      SpaceGroupElement[3].x=y+0.75;         SpaceGroupElement[3].y=-x+0.5;         SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         5/8,-1/8,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y+0.25;        SpaceGroupElement[4].z=-z+0.25;         // 2        [1,0,0]         0,0,0           0,1/8,1/8
      SpaceGroupElement[5].x=y+0.25;         SpaceGroupElement[5].y=x+0.25;         SpaceGroupElement[5].z=-z+0.5;          // 2        [1,1,0]         1/4,1/4,0       0,0,1/4
      SpaceGroupElement[6].x=-x+0.25;        SpaceGroupElement[6].y=y+0.5;          SpaceGroupElement[6].z=-z+0.75;         // 2        [0,1,0]         0,1/2,0         1/8,0,3/8
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x+0.5;         SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        -1/4,1/4,0      1/4,0,0
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x+0.5;         SpaceGroupElement[9].y=z+0.25;         SpaceGroupElement[9].z=y+0.25;          // 2        [0,1,1]         0,1/4,1/4       1/4,0,0
      SpaceGroupElement[10].x=-z+0.25;       SpaceGroupElement[10].y=-x+0.75;       SpaceGroupElement[10].z=y+0.5;          // 3^-1     [-1,1,1]        -1/3,1/3,1/3    7/12,-1/6,0
      SpaceGroupElement[11].x=x+0.75;        SpaceGroupElement[11].y=-z+0.5;        SpaceGroupElement[11].z=y+0.75;         // 4^1      [1,0,0]         3/4,0,0         0,-1/8,5/8
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x+0.25;       SpaceGroupElement[12].z=-y+0.25;        // 3^-1     [1,-1,1]        0,0,0           0,1/4,0
      SpaceGroupElement[13].x=x+0.25;        SpaceGroupElement[13].y=z+0.25;        SpaceGroupElement[13].z=-y+0.5;         // 4^-1     [1,0,0]         1/4,0,0         0,3/8,1/8
      SpaceGroupElement[14].x=-z+0.25;       SpaceGroupElement[14].y=x+0.5;         SpaceGroupElement[14].z=-y+0.75;        // 3^1      [-1,-1,1]       0,0,0           1/4,3/4,0
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z+0.5;        SpaceGroupElement[15].z=-y;             // 2        [0,-1,1]        0,1/4,-1/4      0,1/4,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y+0.5;         SpaceGroupElement[17].y=-z+0.25;       SpaceGroupElement[17].z=-x+0.75;        // 3^-1     [-1,-1,1]       0,0,0           3/4,1/4,0
      SpaceGroupElement[18].x=z+0.25;        SpaceGroupElement[18].y=y+0.75;        SpaceGroupElement[18].z=-x;             // 4^1      [0,1,0]         0,3/4,0         1/8,0,-1/8
      SpaceGroupElement[19].x=-y+0.75;       SpaceGroupElement[19].y=z+0.5;         SpaceGroupElement[19].z=-x+0.25;        // 3^1      [-1,1,1]        0,0,0           1/4,1/2,0
      SpaceGroupElement[20].x=-z+0.5;        SpaceGroupElement[20].y=-y+0.5;        SpaceGroupElement[20].z=-x+0.5;         // 2        [-1,0,1]        0,0,0           1/2,1/4,0
      SpaceGroupElement[21].x=-y+0.25;       SpaceGroupElement[21].y=-z+0.25;       SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,1/4,0
      SpaceGroupElement[22].x=z+0.25;        SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=x+0.75;         // 2        [1,0,1]         1/2,0,1/2       -1/4,0,0
      SpaceGroupElement[23].x=-z;            SpaceGroupElement[23].y=y+0.75;        SpaceGroupElement[23].z=x+0.25;         // 4^-1     [0,1,0]         0,3/4,0         -1/8,0,1/8
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[25].x=y-0.5;         SpaceGroupElement[25].y=-x-0.25;       SpaceGroupElement[25].z=-z-0.25;        // -4^1     [0,0,1]         0,0,0           -3/8,1/8,-1/8
      SpaceGroupElement[26].x=x-0.25;        SpaceGroupElement[26].y=y-0.75;        SpaceGroupElement[26].z=-z-0.5;         // -2       [0,0,1]         -1/4,-3/4,0     0,0,-1/4
      SpaceGroupElement[27].x=-y-0.75;       SpaceGroupElement[27].y=x-0.5;         SpaceGroupElement[27].z=-z-0.75;        // -4^-1    [0,0,1]         0,0,0           -1/8,-5/8,-3/8
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=y-0.25;        SpaceGroupElement[28].z=z-0.25;         // -2       [1,0,0]         0,-1/4,-1/4     0,0,0
      SpaceGroupElement[29].x=-y-0.25;       SpaceGroupElement[29].y=-x-0.25;       SpaceGroupElement[29].z=z-0.5;          // -2       [1,1,0]         0,0,-1/2        -1/4,0,0
      SpaceGroupElement[30].x=x-0.25;        SpaceGroupElement[30].y=-y-0.5;        SpaceGroupElement[30].z=z-0.75;         // -2       [0,1,0]         -1/4,0,-3/4     0,-1/4,0
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x-0.5;         SpaceGroupElement[31].z=z;              // -2       [-1,1,0]        -1/4,-1/4,0     1/4,0,0
      SpaceGroupElement[32].x=-z;            SpaceGroupElement[32].y=-x;            SpaceGroupElement[32].z=-y;             // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[33].x=x-0.5;         SpaceGroupElement[33].y=-z-0.25;       SpaceGroupElement[33].z=-y-0.25;        // -2       [0,1,1]         -1/2,0,0        0,-1/4,0
      SpaceGroupElement[34].x=z-0.25;        SpaceGroupElement[34].y=x-0.75;        SpaceGroupElement[34].z=-y-0.5;         // -3^-1    [-1,1,1]        0,0,0           0,-3/4,1/4
      SpaceGroupElement[35].x=-x-0.75;       SpaceGroupElement[35].y=z-0.5;         SpaceGroupElement[35].z=-y-0.75;        // -4^1     [1,0,0]         0,0,0           -3/8,-5/8,-1/8
      SpaceGroupElement[36].x=-z;            SpaceGroupElement[36].y=x-0.25;        SpaceGroupElement[36].z=y-0.25;         // -3^-1    [1,-1,1]        0,0,0           1/4,0,-1/4
      SpaceGroupElement[37].x=-x-0.25;       SpaceGroupElement[37].y=-z-0.25;       SpaceGroupElement[37].z=y-0.5;          // -4^-1    [1,0,0]         0,0,0           -1/8,1/8,-3/8
      SpaceGroupElement[38].x=z-0.25;        SpaceGroupElement[38].y=-x-0.5;        SpaceGroupElement[38].z=y-0.75;         // -3^1     [-1,-1,1]       0,0,0           -3/4,1/4,-1/2
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=z-0.5;         SpaceGroupElement[39].z=y;              // -2       [0,-1,1]        0,-1/4,-1/4     0,-1/4,0
      SpaceGroupElement[40].x=-y;            SpaceGroupElement[40].y=-z;            SpaceGroupElement[40].z=-x;             // -3^-1    [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[41].x=-y-0.5;        SpaceGroupElement[41].y=z-0.25;        SpaceGroupElement[41].z=x-0.75;         // -3^-1    [-1,-1,1]       0,0,0           1/4,-3/4,-1/2
      SpaceGroupElement[42].x=-z-0.25;       SpaceGroupElement[42].y=-y-0.75;       SpaceGroupElement[42].z=x;              // -4^1     [0,1,0]         0,0,0           -1/8,-3/8,-1/8
      SpaceGroupElement[43].x=y-0.75;        SpaceGroupElement[43].y=-z-0.5;        SpaceGroupElement[43].z=x-0.25;         // -3^1     [-1,1,1]        0,0,0           -1/2,1/4,-3/4
      SpaceGroupElement[44].x=z-0.5;         SpaceGroupElement[44].y=y-0.5;         SpaceGroupElement[44].z=x-0.5;          // -2       [-1,0,1]        -1/2,-1/2,-1/2  0,0,0
      SpaceGroupElement[45].x=y-0.25;        SpaceGroupElement[45].y=z-0.25;        SpaceGroupElement[45].z=-x;             // -3^1     [1,-1,1]        0,0,0           -1/4,0,1/4
      SpaceGroupElement[46].x=-z-0.25;       SpaceGroupElement[46].y=y;             SpaceGroupElement[46].z=-x-0.75;        // -2       [1,0,1]         1/4,0,-1/4      -1/2,0,0
      SpaceGroupElement[47].x=z;             SpaceGroupElement[47].y=-y-0.75;       SpaceGroupElement[47].z=-x-0.25;        // -4^-1    [0,1,0]         0,0,0           -1/8,-3/8,-1/8
      SpaceGroupElement[48].x=x;             SpaceGroupElement[48].y=y+0.5;         SpaceGroupElement[48].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[49].x=-y+0.5;        SpaceGroupElement[49].y=x+0.75;        SpaceGroupElement[49].z=z+0.75;         // 4^1      [0,0,1]         0,0,3/4         -1/8,5/8,0
      SpaceGroupElement[50].x=-x+0.25;       SpaceGroupElement[50].y=-y+0.25;       SpaceGroupElement[50].z=z;              // 2        [0,0,1]         0,0,1           1/8,5/8,0
      SpaceGroupElement[51].x=y+0.75;        SpaceGroupElement[51].y=-x;            SpaceGroupElement[51].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         7/8,1/8,0
      SpaceGroupElement[52].x=x;             SpaceGroupElement[52].y=-y+0.75;       SpaceGroupElement[52].z=-z+0.75;        // 2        [1,0,0]         0,0,0           0,3/8,3/8
      SpaceGroupElement[53].x=y+0.25;        SpaceGroupElement[53].y=x+0.75;        SpaceGroupElement[53].z=-z;             // 2        [1,1,0]         1/2,1/2,0       -1/4,0,1/2
      SpaceGroupElement[54].x=-x+0.25;       SpaceGroupElement[54].y=y;             SpaceGroupElement[54].z=-z+0.25;        // 2        [0,1,0]         0,1,0           1/8,0,5/8
      SpaceGroupElement[55].x=-y;            SpaceGroupElement[55].y=-x;            SpaceGroupElement[55].z=-z+0.5;         // 2        [-1,1,0]        -1/2,1/2,0      1/2,0,1/4
      SpaceGroupElement[56].x=z;             SpaceGroupElement[56].y=x+0.5;         SpaceGroupElement[56].z=y+0.5;          // 3^1      [1,1,1]         1/3,1/3,1/3     -1/3,-1/6,0
      SpaceGroupElement[57].x=-x+0.5;        SpaceGroupElement[57].y=z+0.75;        SpaceGroupElement[57].z=y+0.75;         // 2        [0,1,1]         0,3/4,3/4       1/4,0,0
      SpaceGroupElement[58].x=-z+0.25;       SpaceGroupElement[58].y=-x+0.25;       SpaceGroupElement[58].z=y;              // 3^-1     [-1,1,1]        -2/3,2/3,2/3    11/12,-1/3,0
      SpaceGroupElement[59].x=x+0.75;        SpaceGroupElement[59].y=-z;            SpaceGroupElement[59].z=y+0.25;         // 4^1      [1,0,0]         3/4,0,0         0,-1/8,9/8
      SpaceGroupElement[60].x=z;             SpaceGroupElement[60].y=-x+0.75;       SpaceGroupElement[60].z=-y+0.75;        // 3^-1     [1,-1,1]        0,0,0           0,3/4,0
      SpaceGroupElement[61].x=x+0.25;        SpaceGroupElement[61].y=z+0.75;        SpaceGroupElement[61].z=-y;             // 4^-1     [1,0,0]         1/4,0,0         0,7/8,1/8
      SpaceGroupElement[62].x=-z+0.25;       SpaceGroupElement[62].y=x;             SpaceGroupElement[62].z=-y+0.25;        // 3^1      [-1,-1,1]       0,0,0           1/4,5/4,0
      SpaceGroupElement[63].x=-x;            SpaceGroupElement[63].y=-z;            SpaceGroupElement[63].z=-y+0.5;         // 2        [0,-1,1]        0,1/4,-1/4      0,3/4,0
      SpaceGroupElement[64].x=y;             SpaceGroupElement[64].y=z+0.5;         SpaceGroupElement[64].z=x+0.5;          // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,1/6,0
      SpaceGroupElement[65].x=y+0.5;         SpaceGroupElement[65].y=-z+0.75;       SpaceGroupElement[65].z=-x+0.25;        // 3^-1     [-1,-1,1]       0,0,0           5/4,3/4,0
      SpaceGroupElement[66].x=z+0.25;        SpaceGroupElement[66].y=y+0.25;        SpaceGroupElement[66].z=-x+0.5;         // 4^1      [0,1,0]         0,5/4,0         3/8,0,1/8
      SpaceGroupElement[67].x=-y+0.75;       SpaceGroupElement[67].y=z;             SpaceGroupElement[67].z=-x+0.75;        // 3^1      [-1,1,1]        -1/3,1/3,1/3    5/12,2/3,0
      SpaceGroupElement[68].x=-z+0.5;        SpaceGroupElement[68].y=-y;            SpaceGroupElement[68].z=-x;             // 2        [-1,0,1]        -1/4,0,1/4      3/4,1/2,0
      SpaceGroupElement[69].x=-y+0.25;       SpaceGroupElement[69].y=-z+0.75;       SpaceGroupElement[69].z=x+0.5;          // 3^1      [1,-1,1]        0,0,0           -1/2,3/4,0
      SpaceGroupElement[70].x=z+0.25;        SpaceGroupElement[70].y=-y+0.5;        SpaceGroupElement[70].z=x+0.25;         // 2        [1,0,1]         3/4,0,3/4       -1/2,1/4,0
      SpaceGroupElement[71].x=-z;            SpaceGroupElement[71].y=y+0.25;        SpaceGroupElement[71].z=x+0.75;         // 4^-1     [0,1,0]         0,5/4,0         -3/8,0,3/8
      SpaceGroupElement[72].x=-x;            SpaceGroupElement[72].y=-y+0.5;        SpaceGroupElement[72].z=-z+0.5;         // -1       -               -               0,1/4,1/4
      SpaceGroupElement[73].x=y-0.5;         SpaceGroupElement[73].y=-x+0.25;       SpaceGroupElement[73].z=-z+0.25;        // -4^1     [0,0,1]         0,0,0           -1/8,3/8,1/8
      SpaceGroupElement[74].x=x-0.25;        SpaceGroupElement[74].y=y-0.25;        SpaceGroupElement[74].z=-z;             // -2       [0,0,1]         -1/4,-1/4,0     0,0,0
      SpaceGroupElement[75].x=-y-0.75;       SpaceGroupElement[75].y=x;             SpaceGroupElement[75].z=-z-0.25;        // -4^-1    [0,0,1]         0,0,0           -3/8,-3/8,-1/8
      SpaceGroupElement[76].x=-x;            SpaceGroupElement[76].y=y+0.25;        SpaceGroupElement[76].z=z+0.25;         // -2       [1,0,0]         0,1/4,1/4       0,0,0
      SpaceGroupElement[77].x=-y-0.25;       SpaceGroupElement[77].y=-x+0.25;       SpaceGroupElement[77].z=z;              // -2       [1,1,0]         -1/4,1/4,0      0,0,0
      SpaceGroupElement[78].x=x-0.25;        SpaceGroupElement[78].y=-y;            SpaceGroupElement[78].z=z-0.25;         // -2       [0,1,0]         -1/4,0,-1/4     0,0,0
      SpaceGroupElement[79].x=y;             SpaceGroupElement[79].y=x;             SpaceGroupElement[79].z=z+0.5;          // -2       [-1,1,0]        0,0,1/2         0,0,0
      SpaceGroupElement[80].x=-z;            SpaceGroupElement[80].y=-x+0.5;        SpaceGroupElement[80].z=-y+0.5;         // -3^1     [1,1,1]         0,0,0           0,1/2,0
      SpaceGroupElement[81].x=x-0.5;         SpaceGroupElement[81].y=-z+0.25;       SpaceGroupElement[81].z=-y+0.25;        // -2       [0,1,1]         -1/2,0,0        0,1/4,0
      SpaceGroupElement[82].x=z-0.25;        SpaceGroupElement[82].y=x-0.25;        SpaceGroupElement[82].z=-y;             // -3^-1    [-1,1,1]        0,0,0           0,-1/4,1/4
      SpaceGroupElement[83].x=-x-0.75;       SpaceGroupElement[83].y=z;             SpaceGroupElement[83].z=-y-0.25;        // -4^1     [1,0,0]         0,0,0           -3/8,-1/8,-1/8
      SpaceGroupElement[84].x=-z;            SpaceGroupElement[84].y=x+0.25;        SpaceGroupElement[84].z=y+0.25;         // -3^-1    [1,-1,1]        0,0,0           -1/4,0,1/4
      SpaceGroupElement[85].x=-x-0.25;       SpaceGroupElement[85].y=-z+0.25;       SpaceGroupElement[85].z=y;              // -4^-1    [1,0,0]         0,0,0           -1/8,1/8,1/8
      SpaceGroupElement[86].x=z-0.25;        SpaceGroupElement[86].y=-x;            SpaceGroupElement[86].z=y-0.25;         // -3^1     [-1,-1,1]       0,0,0           -1/4,1/4,0
      SpaceGroupElement[87].x=x;             SpaceGroupElement[87].y=z;             SpaceGroupElement[87].z=y+0.5;          // -2       [0,-1,1]        0,1/4,1/4       0,-1/4,0
      SpaceGroupElement[88].x=-y;            SpaceGroupElement[88].y=-z+0.5;        SpaceGroupElement[88].z=-x+0.5;         // -3^-1    [1,1,1]         0,0,0           0,0,1/2
      SpaceGroupElement[89].x=-y-0.5;        SpaceGroupElement[89].y=z+0.25;        SpaceGroupElement[89].z=x-0.25;         // -3^-1    [-1,-1,1]       0,0,0           -1/4,-1/4,-1/2
      SpaceGroupElement[90].x=-z-0.25;       SpaceGroupElement[90].y=-y-0.25;       SpaceGroupElement[90].z=x+0.5;          // -4^1     [0,1,0]         0,0,0           -3/8,-1/8,1/8
      SpaceGroupElement[91].x=y-0.75;        SpaceGroupElement[91].y=-z;            SpaceGroupElement[91].z=x+0.25;         // -3^1     [-1,1,1]        0,0,0           -1/2,1/4,-1/4
      SpaceGroupElement[92].x=z-0.5;         SpaceGroupElement[92].y=y;             SpaceGroupElement[92].z=x;              // -2       [-1,0,1]        -1/4,0,-1/4     -1/4,0,0
      SpaceGroupElement[93].x=y-0.25;        SpaceGroupElement[93].y=z+0.25;        SpaceGroupElement[93].z=-x+0.5;         // -3^1     [1,-1,1]        0,0,0           1/4,1/2,1/4
      SpaceGroupElement[94].x=-z-0.25;       SpaceGroupElement[94].y=y+0.5;         SpaceGroupElement[94].z=-x-0.25;        // -2       [1,0,1]         0,1/2,0         -1/4,0,0
      SpaceGroupElement[95].x=z;             SpaceGroupElement[95].y=-y-0.25;       SpaceGroupElement[95].z=-x+0.25;        // -4^-1    [0,1,0]         0,0,0           1/8,-1/8,1/8
      SpaceGroupElement[96].x=x+0.5;         SpaceGroupElement[96].y=y;             SpaceGroupElement[96].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[97].x=-y;            SpaceGroupElement[97].y=x+0.25;        SpaceGroupElement[97].z=z+0.75;         // 4^1      [0,0,1]         0,0,3/4         3/8,5/8,0
      SpaceGroupElement[98].x=-x+0.75;       SpaceGroupElement[98].y=-y+0.75;       SpaceGroupElement[98].z=z;              // 2        [0,0,1]         0,0,1           3/8,3/8,0
      SpaceGroupElement[99].x=y+0.25;        SpaceGroupElement[99].y=-x+0.5;        SpaceGroupElement[99].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         7/8,-3/8,0
      SpaceGroupElement[100].x=x+0.5;        SpaceGroupElement[100].y=-y+0.25;      SpaceGroupElement[100].z=-z+0.75;       // 2        [1,0,0]         1/2,0,0         0,1/8,3/8
      SpaceGroupElement[101].x=y+0.75;       SpaceGroupElement[101].y=x+0.25;       SpaceGroupElement[101].z=-z;            // 2        [1,1,0]         1/2,1/2,0       1/4,0,1/2
      SpaceGroupElement[102].x=-x+0.75;      SpaceGroupElement[102].y=y+0.5;        SpaceGroupElement[102].z=-z+0.25;       // 2        [0,1,0]         0,1/2,0         3/8,0,5/8
      SpaceGroupElement[103].x=-y+0.5;       SpaceGroupElement[103].y=-x+0.5;       SpaceGroupElement[103].z=-z+0.5;        // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      SpaceGroupElement[104].x=z+0.5;        SpaceGroupElement[104].y=x;            SpaceGroupElement[104].z=y+0.5;         // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,-1/6,0
      SpaceGroupElement[105].x=-x;           SpaceGroupElement[105].y=z+0.25;       SpaceGroupElement[105].z=y+0.75;        // 2        [0,1,1]         0,1/2,1/2       1/2,-1/4,0
      SpaceGroupElement[106].x=-z+0.75;      SpaceGroupElement[106].y=-x+0.75;      SpaceGroupElement[106].z=y;             // 3^-1     [-1,1,1]        -1/3,1/3,1/3    13/12,-2/3,0
      SpaceGroupElement[107].x=x+0.25;       SpaceGroupElement[107].y=-z+0.5;       SpaceGroupElement[107].z=y+0.25;        // 4^1      [1,0,0]         5/4,0,0         0,-3/8,7/8
      SpaceGroupElement[108].x=z+0.5;        SpaceGroupElement[108].y=-x+0.25;      SpaceGroupElement[108].z=-y+0.75;       // 3^-1     [1,-1,1]        1/3,-1/3,1/3    1/6,5/12,0
      SpaceGroupElement[109].x=x+0.75;       SpaceGroupElement[109].y=z+0.25;       SpaceGroupElement[109].z=-y;            // 4^-1     [1,0,0]         3/4,0,0         0,5/8,3/8
      SpaceGroupElement[110].x=-z+0.75;      SpaceGroupElement[110].y=x+0.5;        SpaceGroupElement[110].z=-y+0.25;       // 3^1      [-1,-1,1]       0,0,0           3/4,5/4,0
      SpaceGroupElement[111].x=-x+0.5;       SpaceGroupElement[111].y=-z+0.5;       SpaceGroupElement[111].z=-y+0.5;        // 2        [0,-1,1]        0,0,0           1/4,1/2,0
      SpaceGroupElement[112].x=y+0.5;        SpaceGroupElement[112].y=z;            SpaceGroupElement[112].z=x+0.5;         // 3^-1     [1,1,1]         1/3,1/3,1/3     -1/6,-1/3,0
      SpaceGroupElement[113].x=y;            SpaceGroupElement[113].y=-z+0.25;      SpaceGroupElement[113].z=-x+0.25;       // 3^-1     [-1,-1,1]       0,0,0           5/4,1/4,0
      SpaceGroupElement[114].x=z+0.75;       SpaceGroupElement[114].y=y+0.75;       SpaceGroupElement[114].z=-x+0.5;        // 4^1      [0,1,0]         0,3/4,0         5/8,0,-1/8
      SpaceGroupElement[115].x=-y+0.25;      SpaceGroupElement[115].y=z+0.5;        SpaceGroupElement[115].z=-x+0.75;       // 3^1      [-1,1,1]        0,0,0           3/4,1/2,0
      SpaceGroupElement[116].x=-z;           SpaceGroupElement[116].y=-y+0.5;       SpaceGroupElement[116].z=-x;            // 2        [-1,0,1]        0,0,0           1,1/4,0
      SpaceGroupElement[117].x=-y+0.75;      SpaceGroupElement[117].y=-z+0.25;      SpaceGroupElement[117].z=x+0.5;         // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,7/12,0
      SpaceGroupElement[118].x=z+0.75;       SpaceGroupElement[118].y=-y;           SpaceGroupElement[118].z=x+0.25;        // 2        [1,0,1]         1,0,1           -1/4,0,0
      SpaceGroupElement[119].x=-z+0.5;       SpaceGroupElement[119].y=y+0.75;       SpaceGroupElement[119].z=x+0.75;        // 4^-1     [0,1,0]         0,3/4,0         -1/8,0,5/8
      SpaceGroupElement[120].x=-x+0.5;       SpaceGroupElement[120].y=-y;           SpaceGroupElement[120].z=-z+0.5;        // -1       -               -               1/4,0,1/4
      SpaceGroupElement[121].x=y;            SpaceGroupElement[121].y=-x-0.25;      SpaceGroupElement[121].z=-z+0.25;       // -4^1     [0,0,1]         0,0,0           -1/8,-1/8,1/8
      SpaceGroupElement[122].x=x+0.25;       SpaceGroupElement[122].y=y-0.75;       SpaceGroupElement[122].z=-z;            // -2       [0,0,1]         1/4,-3/4,0      0,0,0
      SpaceGroupElement[123].x=-y-0.25;      SpaceGroupElement[123].y=x-0.5;        SpaceGroupElement[123].z=-z-0.25;       // -4^-1    [0,0,1]         0,0,0           1/8,-3/8,-1/8
      SpaceGroupElement[124].x=-x+0.5;       SpaceGroupElement[124].y=y-0.25;       SpaceGroupElement[124].z=z+0.25;        // -2       [1,0,0]         0,-1/4,1/4      1/4,0,0
      SpaceGroupElement[125].x=-y+0.25;      SpaceGroupElement[125].y=-x-0.25;      SpaceGroupElement[125].z=z;             // -2       [1,1,0]         1/4,-1/4,0      0,0,0
      SpaceGroupElement[126].x=x+0.25;       SpaceGroupElement[126].y=-y-0.5;       SpaceGroupElement[126].z=z-0.25;        // -2       [0,1,0]         1/4,0,-1/4      0,-1/4,0
      SpaceGroupElement[127].x=y+0.5;        SpaceGroupElement[127].y=x-0.5;        SpaceGroupElement[127].z=z+0.5;         // -2       [-1,1,0]        0,0,1/2         1/2,0,0
      SpaceGroupElement[128].x=-z+0.5;       SpaceGroupElement[128].y=-x;           SpaceGroupElement[128].z=-y+0.5;        // -3^1     [1,1,1]         0,0,0           0,0,1/2
      SpaceGroupElement[129].x=x;            SpaceGroupElement[129].y=-z-0.25;      SpaceGroupElement[129].z=-y+0.25;       // -2       [0,1,1]         0,-1/4,1/4      0,0,0
      SpaceGroupElement[130].x=z+0.25;       SpaceGroupElement[130].y=x-0.75;       SpaceGroupElement[130].z=-y;            // -3^-1    [-1,1,1]        0,0,0           1/2,-1/4,1/4
      SpaceGroupElement[131].x=-x-0.25;      SpaceGroupElement[131].y=z-0.5;        SpaceGroupElement[131].z=-y-0.25;       // -4^1     [1,0,0]         0,0,0           -1/8,-3/8,1/8
      SpaceGroupElement[132].x=-z+0.5;       SpaceGroupElement[132].y=x-0.25;       SpaceGroupElement[132].z=y+0.25;        // -3^-1    [1,-1,1]        0,0,0           1/4,0,1/4
      SpaceGroupElement[133].x=-x+0.25;      SpaceGroupElement[133].y=-z-0.25;      SpaceGroupElement[133].z=y;             // -4^-1    [1,0,0]         0,0,0           1/8,-1/8,-1/8
      SpaceGroupElement[134].x=z+0.25;       SpaceGroupElement[134].y=-x-0.5;       SpaceGroupElement[134].z=y-0.25;        // -3^1     [-1,-1,1]       0,0,0           -1/4,-1/4,-1/2
      SpaceGroupElement[135].x=x+0.5;        SpaceGroupElement[135].y=z-0.5;        SpaceGroupElement[135].z=y+0.5;         // -2       [0,-1,1]        1/2,0,0         0,-1/2,0
      SpaceGroupElement[136].x=-y+0.5;       SpaceGroupElement[136].y=-z;           SpaceGroupElement[136].z=-x+0.5;        // -3^-1    [1,1,1]         0,0,0           1/2,0,0
      SpaceGroupElement[137].x=-y;           SpaceGroupElement[137].y=z-0.25;       SpaceGroupElement[137].z=x-0.25;        // -3^-1    [-1,-1,1]       0,0,0           1/4,-1/4,0
      SpaceGroupElement[138].x=-z+0.25;      SpaceGroupElement[138].y=-y-0.75;      SpaceGroupElement[138].z=x+0.5;         // -4^1     [0,1,0]         0,0,0           -1/8,-3/8,3/8
      SpaceGroupElement[139].x=y-0.25;       SpaceGroupElement[139].y=-z-0.5;       SpaceGroupElement[139].z=x+0.25;        // -3^1     [-1,1,1]        0,0,0           -1/2,-1/4,-1/4
      SpaceGroupElement[140].x=z;            SpaceGroupElement[140].y=y-0.5;        SpaceGroupElement[140].z=x;             // -2       [-1,0,1]        0,-1/2,0        0,0,0
      SpaceGroupElement[141].x=y+0.25;       SpaceGroupElement[141].y=z-0.25;       SpaceGroupElement[141].z=-x+0.5;        // -3^1     [1,-1,1]        0,0,0           1/4,0,1/4
      SpaceGroupElement[142].x=-z+0.25;      SpaceGroupElement[142].y=y;            SpaceGroupElement[142].z=-x-0.25;       // -2       [1,0,1]         1/4,0,-1/4      0,0,0
      SpaceGroupElement[143].x=z+0.5;        SpaceGroupElement[143].y=-y-0.75;      SpaceGroupElement[143].z=-x+0.25;       // -4^-1    [0,1,0]         0,0,0           3/8,-3/8,-1/8
      SpaceGroupElement[144].x=x+0.5;        SpaceGroupElement[144].y=y+0.5;        SpaceGroupElement[144].z=z;             // 1        -               -               -
      SpaceGroupElement[145].x=-y;           SpaceGroupElement[145].y=x+0.75;       SpaceGroupElement[145].z=z+0.25;        // 4^1      [0,0,1]         0,0,1/4         1/8,7/8,0
      SpaceGroupElement[146].x=-x+0.75;      SpaceGroupElement[146].y=-y+0.25;      SpaceGroupElement[146].z=z+0.5;         // 2        [0,0,1]         0,0,1/2         3/8,5/8,0
      SpaceGroupElement[147].x=y+0.25;       SpaceGroupElement[147].y=-x;           SpaceGroupElement[147].z=z+0.75;        // 4^-1     [0,0,1]         0,0,3/4         9/8,-1/8,0
      SpaceGroupElement[148].x=x+0.5;        SpaceGroupElement[148].y=-y+0.75;      SpaceGroupElement[148].z=-z+0.25;       // 2        [1,0,0]         1/2,0,0         0,3/8,1/8
      SpaceGroupElement[149].x=y+0.75;       SpaceGroupElement[149].y=x+0.75;       SpaceGroupElement[149].z=-z+0.5;        // 2        [1,1,0]         3/4,3/4,0       0,0,1/4
      SpaceGroupElement[150].x=-x+0.75;      SpaceGroupElement[150].y=y;            SpaceGroupElement[150].z=-z+0.75;       // 2        [0,1,0]         0,1,0           3/8,0,3/8
      SpaceGroupElement[151].x=-y+0.5;       SpaceGroupElement[151].y=-x;           SpaceGroupElement[151].z=-z;            // 2        [-1,1,0]        -1/4,1/4,0      3/4,0,0
      SpaceGroupElement[152].x=z+0.5;        SpaceGroupElement[152].y=x+0.5;        SpaceGroupElement[152].z=y;             // 3^1      [1,1,1]         1/3,1/3,1/3     1/6,1/3,0
      SpaceGroupElement[153].x=-x;           SpaceGroupElement[153].y=z+0.75;       SpaceGroupElement[153].z=y+0.25;        // 2        [0,1,1]         0,1/2,1/2       1/2,1/4,0
      SpaceGroupElement[154].x=-z+0.75;      SpaceGroupElement[154].y=-x+0.25;      SpaceGroupElement[154].z=y+0.5;         // 3^-1     [-1,1,1]        -1/3,1/3,1/3    13/12,-1/6,0
      SpaceGroupElement[155].x=x+0.25;       SpaceGroupElement[155].y=-z;           SpaceGroupElement[155].z=y+0.75;        // 4^1      [1,0,0]         5/4,0,0         0,1/8,7/8
      SpaceGroupElement[156].x=z+0.5;        SpaceGroupElement[156].y=-x+0.75;      SpaceGroupElement[156].z=-y+0.25;       // 3^-1     [1,-1,1]        0,0,0           1/2,1/4,0
      SpaceGroupElement[157].x=x+0.75;       SpaceGroupElement[157].y=z+0.75;       SpaceGroupElement[157].z=-y+0.5;        // 4^-1     [1,0,0]         3/4,0,0         0,5/8,-1/8
      SpaceGroupElement[158].x=-z+0.75;      SpaceGroupElement[158].y=x;            SpaceGroupElement[158].z=-y+0.75;       // 3^1      [-1,-1,1]       1/3,1/3,-1/3    5/12,13/12,0
      SpaceGroupElement[159].x=-x+0.5;       SpaceGroupElement[159].y=-z;           SpaceGroupElement[159].z=-y;            // 2        [0,-1,1]        0,1/2,-1/2      1/4,1/2,0
      SpaceGroupElement[160].x=y+0.5;        SpaceGroupElement[160].y=z+0.5;        SpaceGroupElement[160].z=x;             // 3^-1     [1,1,1]         1/3,1/3,1/3     1/3,1/6,0
      SpaceGroupElement[161].x=y;            SpaceGroupElement[161].y=-z+0.75;      SpaceGroupElement[161].z=-x+0.75;       // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    13/12,5/12,0
      SpaceGroupElement[162].x=z+0.75;       SpaceGroupElement[162].y=y+0.25;       SpaceGroupElement[162].z=-x;            // 4^1      [0,1,0]         0,5/4,0         3/8,0,-3/8
      SpaceGroupElement[163].x=-y+0.25;      SpaceGroupElement[163].y=z;            SpaceGroupElement[163].z=-x+0.25;       // 3^1      [-1,1,1]        0,0,0           1/4,1,0
      SpaceGroupElement[164].x=-z;           SpaceGroupElement[164].y=-y;           SpaceGroupElement[164].z=-x+0.5;        // 2        [-1,0,1]        1/4,0,-1/4      3/4,1/2,0
      SpaceGroupElement[165].x=-y+0.75;      SpaceGroupElement[165].y=-z+0.75;      SpaceGroupElement[165].z=x;             // 3^1      [1,-1,1]        0,0,0           0,3/4,0
      SpaceGroupElement[166].x=z+0.75;       SpaceGroupElement[166].y=-y+0.5;       SpaceGroupElement[166].z=x+0.75;        // 2        [1,0,1]         3/4,0,3/4       0,1/4,0
      SpaceGroupElement[167].x=-z+0.5;       SpaceGroupElement[167].y=y+0.25;       SpaceGroupElement[167].z=x+0.25;        // 4^-1     [0,1,0]         0,5/4,0         1/8,0,3/8
      SpaceGroupElement[168].x=-x+0.5;       SpaceGroupElement[168].y=-y+0.5;       SpaceGroupElement[168].z=-z;            // -1       -               -               1/4,1/4,0
      SpaceGroupElement[169].x=y;            SpaceGroupElement[169].y=-x+0.25;      SpaceGroupElement[169].z=-z-0.25;       // -4^1     [0,0,1]         0,0,0           1/8,1/8,-1/8
      SpaceGroupElement[170].x=x+0.25;       SpaceGroupElement[170].y=y-0.25;       SpaceGroupElement[170].z=-z-0.5;        // -2       [0,0,1]         1/4,-1/4,0      0,0,-1/4
      SpaceGroupElement[171].x=-y-0.25;      SpaceGroupElement[171].y=x;            SpaceGroupElement[171].z=-z-0.75;       // -4^-1    [0,0,1]         0,0,0           -1/8,-1/8,-3/8
      SpaceGroupElement[172].x=-x+0.5;       SpaceGroupElement[172].y=y+0.25;       SpaceGroupElement[172].z=z-0.25;        // -2       [1,0,0]         0,1/4,-1/4      1/4,0,0
      SpaceGroupElement[173].x=-y+0.25;      SpaceGroupElement[173].y=-x+0.25;      SpaceGroupElement[173].z=z-0.5;         // -2       [1,1,0]         0,0,-1/2        1/4,0,0
      SpaceGroupElement[174].x=x+0.25;       SpaceGroupElement[174].y=-y;           SpaceGroupElement[174].z=z-0.75;        // -2       [0,1,0]         1/4,0,-3/4      0,0,0
      SpaceGroupElement[175].x=y+0.5;        SpaceGroupElement[175].y=x;            SpaceGroupElement[175].z=z;             // -2       [-1,1,0]        1/4,1/4,0       1/4,0,0
      SpaceGroupElement[176].x=-z+0.5;       SpaceGroupElement[176].y=-x+0.5;       SpaceGroupElement[176].z=-y;            // -3^1     [1,1,1]         0,0,0           1/2,0,0
      SpaceGroupElement[177].x=x;            SpaceGroupElement[177].y=-z+0.25;      SpaceGroupElement[177].z=-y-0.25;       // -2       [0,1,1]         0,1/4,-1/4      0,0,0
      SpaceGroupElement[178].x=z+0.25;       SpaceGroupElement[178].y=x-0.25;       SpaceGroupElement[178].z=-y-0.5;        // -3^-1    [-1,1,1]        0,0,0           0,-1/4,-1/4
      SpaceGroupElement[179].x=-x-0.25;      SpaceGroupElement[179].y=z;            SpaceGroupElement[179].z=-y-0.75;       // -4^1     [1,0,0]         0,0,0           -1/8,-3/8,-3/8
      SpaceGroupElement[180].x=-z+0.5;       SpaceGroupElement[180].y=x+0.25;       SpaceGroupElement[180].z=y-0.25;        // -3^-1    [1,-1,1]        0,0,0           1/4,1/2,1/4
      SpaceGroupElement[181].x=-x+0.25;      SpaceGroupElement[181].y=-z+0.25;      SpaceGroupElement[181].z=y-0.5;         // -4^-1    [1,0,0]         0,0,0           1/8,3/8,-1/8
      SpaceGroupElement[182].x=z+0.25;       SpaceGroupElement[182].y=-x;           SpaceGroupElement[182].z=y-0.75;        // -3^1     [-1,-1,1]       0,0,0           -1/4,1/4,-1/2
      SpaceGroupElement[183].x=x+0.5;        SpaceGroupElement[183].y=z;            SpaceGroupElement[183].z=y;             // -2       [0,-1,1]        1/2,0,0         0,0,0
      SpaceGroupElement[184].x=-y+0.5;       SpaceGroupElement[184].y=-z+0.5;       SpaceGroupElement[184].z=-x;            // -3^-1    [1,1,1]         0,0,0           0,1/2,0
      SpaceGroupElement[185].x=-y;           SpaceGroupElement[185].y=z+0.25;       SpaceGroupElement[185].z=x-0.75;        // -3^-1    [-1,-1,1]       0,0,0           1/4,-1/4,-1/2
      SpaceGroupElement[186].x=-z+0.25;      SpaceGroupElement[186].y=-y-0.25;      SpaceGroupElement[186].z=x;             // -4^1     [0,1,0]         0,0,0           1/8,-1/8,1/8
      SpaceGroupElement[187].x=y-0.25;       SpaceGroupElement[187].y=-z;           SpaceGroupElement[187].z=x-0.25;        // -3^1     [-1,1,1]        0,0,0           0,1/4,-1/4
      SpaceGroupElement[188].x=z;            SpaceGroupElement[188].y=y;            SpaceGroupElement[188].z=x-0.5;         // -2       [-1,0,1]        -1/4,0,-1/4     1/4,0,0
      SpaceGroupElement[189].x=y+0.25;       SpaceGroupElement[189].y=z+0.25;       SpaceGroupElement[189].z=-x;            // -3^1     [1,-1,1]        0,0,0           1/4,0,-1/4
      SpaceGroupElement[190].x=-z+0.25;      SpaceGroupElement[190].y=y+0.5;        SpaceGroupElement[190].z=-x-0.75;       // -2       [1,0,1]         1/2,1/2,-1/2    -1/4,0,0
      SpaceGroupElement[191].x=z+0.5;        SpaceGroupElement[191].y=-y-0.25;      SpaceGroupElement[191].z=-x-0.25;       // -4^-1    [0,1,0]         0,0,0           1/8,-1/8,-3/8
       break;

    case 529: // [229] I m -3 m (-I 4 2 3)
      SpaceGroupSize=96;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=z;               // 4^1      [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;               // 2        [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;               // 4^-1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;              // 2        [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;              // 2        [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;              // 2        [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=z;              SpaceGroupElement[9].z=y;               // 2        [0,1,1]         0,0,0           0,0,0
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;              // 3^-1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=y;              // 4^1      [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y;             // 3^-1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=z;             SpaceGroupElement[13].z=-y;             // 4^-1     [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[14].x=-z;            SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-y;             // 3^1      [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=-y;             // 2        [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=-x;             // 3^-1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[18].x=z;             SpaceGroupElement[18].y=y;             SpaceGroupElement[18].z=-x;             // 4^1      [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=-x;             // 3^1      [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-x;             // 2        [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x;              // 3^1      [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=x;              // 2        [1,0,1]         0,0,0           0,0,0
      SpaceGroupElement[23].x=-z;            SpaceGroupElement[23].y=y;             SpaceGroupElement[23].z=x;              // 4^-1     [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[25].x=y;             SpaceGroupElement[25].y=-x;            SpaceGroupElement[25].z=-z;             // -4^1     [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=y;             SpaceGroupElement[26].z=-z;             // -2       [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[27].x=-y;            SpaceGroupElement[27].y=x;             SpaceGroupElement[27].z=-z;             // -4^-1    [0,0,1]         0,0,0           0,0,0
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=y;             SpaceGroupElement[28].z=z;              // -2       [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[29].x=-y;            SpaceGroupElement[29].y=-x;            SpaceGroupElement[29].z=z;              // -2       [1,1,0]         0,0,0           0,0,0
      SpaceGroupElement[30].x=x;             SpaceGroupElement[30].y=-y;            SpaceGroupElement[30].z=z;              // -2       [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=z;              // -2       [-1,1,0]        0,0,0           0,0,0
      SpaceGroupElement[32].x=-z;            SpaceGroupElement[32].y=-x;            SpaceGroupElement[32].z=-y;             // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[33].x=x;             SpaceGroupElement[33].y=-z;            SpaceGroupElement[33].z=-y;             // -2       [0,1,1]         0,0,0           0,0,0
      SpaceGroupElement[34].x=z;             SpaceGroupElement[34].y=x;             SpaceGroupElement[34].z=-y;             // -3^-1    [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[35].x=-x;            SpaceGroupElement[35].y=z;             SpaceGroupElement[35].z=-y;             // -4^1     [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[36].x=-z;            SpaceGroupElement[36].y=x;             SpaceGroupElement[36].z=y;              // -3^-1    [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[37].x=-x;            SpaceGroupElement[37].y=-z;            SpaceGroupElement[37].z=y;              // -4^-1    [1,0,0]         0,0,0           0,0,0
      SpaceGroupElement[38].x=z;             SpaceGroupElement[38].y=-x;            SpaceGroupElement[38].z=y;              // -3^1     [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=z;             SpaceGroupElement[39].z=y;              // -2       [0,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[40].x=-y;            SpaceGroupElement[40].y=-z;            SpaceGroupElement[40].z=-x;             // -3^-1    [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[41].x=-y;            SpaceGroupElement[41].y=z;             SpaceGroupElement[41].z=x;              // -3^-1    [-1,-1,1]       0,0,0           0,0,0
      SpaceGroupElement[42].x=-z;            SpaceGroupElement[42].y=-y;            SpaceGroupElement[42].z=x;              // -4^1     [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[43].x=y;             SpaceGroupElement[43].y=-z;            SpaceGroupElement[43].z=x;              // -3^1     [-1,1,1]        0,0,0           0,0,0
      SpaceGroupElement[44].x=z;             SpaceGroupElement[44].y=y;             SpaceGroupElement[44].z=x;              // -2       [-1,0,1]        0,0,0           0,0,0
      SpaceGroupElement[45].x=y;             SpaceGroupElement[45].y=z;             SpaceGroupElement[45].z=-x;             // -3^1     [1,-1,1]        0,0,0           0,0,0
      SpaceGroupElement[46].x=-z;            SpaceGroupElement[46].y=y;             SpaceGroupElement[46].z=-x;             // -2       [1,0,1]         0,0,0           0,0,0
      SpaceGroupElement[47].x=z;             SpaceGroupElement[47].y=-y;            SpaceGroupElement[47].z=-x;             // -4^-1    [0,1,0]         0,0,0           0,0,0
      SpaceGroupElement[48].x=x+0.5;         SpaceGroupElement[48].y=y+0.5;         SpaceGroupElement[48].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[49].x=-y+0.5;        SpaceGroupElement[49].y=x+0.5;         SpaceGroupElement[49].z=z+0.5;          // 4^1      [0,0,1]         0,0,1/2         0,1/2,0
      SpaceGroupElement[50].x=-x+0.5;        SpaceGroupElement[50].y=-y+0.5;        SpaceGroupElement[50].z=z+0.5;          // 2        [0,0,1]         0,0,1/2         1/4,1/4,0
      SpaceGroupElement[51].x=y+0.5;         SpaceGroupElement[51].y=-x+0.5;        SpaceGroupElement[51].z=z+0.5;          // 4^-1     [0,0,1]         0,0,1/2         1/2,0,0
      SpaceGroupElement[52].x=x+0.5;         SpaceGroupElement[52].y=-y+0.5;        SpaceGroupElement[52].z=-z+0.5;         // 2        [1,0,0]         1/2,0,0         0,1/4,1/4
      SpaceGroupElement[53].x=y+0.5;         SpaceGroupElement[53].y=x+0.5;         SpaceGroupElement[53].z=-z+0.5;         // 2        [1,1,0]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[54].x=-x+0.5;        SpaceGroupElement[54].y=y+0.5;         SpaceGroupElement[54].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/4,0,1/4
      SpaceGroupElement[55].x=-y+0.5;        SpaceGroupElement[55].y=-x+0.5;        SpaceGroupElement[55].z=-z+0.5;         // 2        [-1,1,0]        0,0,0           1/2,0,1/4
      SpaceGroupElement[56].x=z+0.5;         SpaceGroupElement[56].y=x+0.5;         SpaceGroupElement[56].z=y+0.5;          // 3^1      [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[57].x=-x+0.5;        SpaceGroupElement[57].y=z+0.5;         SpaceGroupElement[57].z=y+0.5;          // 2        [0,1,1]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[58].x=-z+0.5;        SpaceGroupElement[58].y=-x+0.5;        SpaceGroupElement[58].z=y+0.5;          // 3^-1     [-1,1,1]        -1/6,1/6,1/6    2/3,-1/3,0
      SpaceGroupElement[59].x=x+0.5;         SpaceGroupElement[59].y=-z+0.5;        SpaceGroupElement[59].z=y+0.5;          // 4^1      [1,0,0]         1/2,0,0         0,0,1/2
      SpaceGroupElement[60].x=z+0.5;         SpaceGroupElement[60].y=-x+0.5;        SpaceGroupElement[60].z=-y+0.5;         // 3^-1     [1,-1,1]        1/6,-1/6,1/6    1/3,1/3,0
      SpaceGroupElement[61].x=x+0.5;         SpaceGroupElement[61].y=z+0.5;         SpaceGroupElement[61].z=-y+0.5;         // 4^-1     [1,0,0]         1/2,0,0         0,1/2,0
      SpaceGroupElement[62].x=-z+0.5;        SpaceGroupElement[62].y=x+0.5;         SpaceGroupElement[62].z=-y+0.5;         // 3^1      [-1,-1,1]       1/6,1/6,-1/6    1/3,2/3,0
      SpaceGroupElement[63].x=-x+0.5;        SpaceGroupElement[63].y=-z+0.5;        SpaceGroupElement[63].z=-y+0.5;         // 2        [0,-1,1]        0,0,0           1/4,1/2,0
      SpaceGroupElement[64].x=y+0.5;         SpaceGroupElement[64].y=z+0.5;         SpaceGroupElement[64].z=x+0.5;          // 3^-1     [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[65].x=y+0.5;         SpaceGroupElement[65].y=-z+0.5;        SpaceGroupElement[65].z=-x+0.5;         // 3^-1     [-1,-1,1]       1/6,1/6,-1/6    2/3,1/3,0
      SpaceGroupElement[66].x=z+0.5;         SpaceGroupElement[66].y=y+0.5;         SpaceGroupElement[66].z=-x+0.5;         // 4^1      [0,1,0]         0,1/2,0         1/2,0,0
      SpaceGroupElement[67].x=-y+0.5;        SpaceGroupElement[67].y=z+0.5;         SpaceGroupElement[67].z=-x+0.5;         // 3^1      [-1,1,1]        -1/6,1/6,1/6    1/3,1/3,0
      SpaceGroupElement[68].x=-z+0.5;        SpaceGroupElement[68].y=-y+0.5;        SpaceGroupElement[68].z=-x+0.5;         // 2        [-1,0,1]        0,0,0           1/2,1/4,0
      SpaceGroupElement[69].x=-y+0.5;        SpaceGroupElement[69].y=-z+0.5;        SpaceGroupElement[69].z=x+0.5;          // 3^1      [1,-1,1]        1/6,-1/6,1/6    -1/3,2/3,0
      SpaceGroupElement[70].x=z+0.5;         SpaceGroupElement[70].y=-y+0.5;        SpaceGroupElement[70].z=x+0.5;          // 2        [1,0,1]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[71].x=-z+0.5;        SpaceGroupElement[71].y=y+0.5;         SpaceGroupElement[71].z=x+0.5;          // 4^-1     [0,1,0]         0,1/2,0         0,0,1/2
      SpaceGroupElement[72].x=-x+0.5;        SpaceGroupElement[72].y=-y+0.5;        SpaceGroupElement[72].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[73].x=y+0.5;         SpaceGroupElement[73].y=-x+0.5;        SpaceGroupElement[73].z=-z+0.5;         // -4^1     [0,0,1]         0,0,0           1/2,0,1/4
      SpaceGroupElement[74].x=x+0.5;         SpaceGroupElement[74].y=y+0.5;         SpaceGroupElement[74].z=-z+0.5;         // -2       [0,0,1]         1/2,1/2,0       0,0,1/4
      SpaceGroupElement[75].x=-y+0.5;        SpaceGroupElement[75].y=x+0.5;         SpaceGroupElement[75].z=-z+0.5;         // -4^-1    [0,0,1]         0,0,0           0,1/2,1/4
      SpaceGroupElement[76].x=-x+0.5;        SpaceGroupElement[76].y=y+0.5;         SpaceGroupElement[76].z=z+0.5;          // -2       [1,0,0]         0,1/2,1/2       1/4,0,0
      SpaceGroupElement[77].x=-y+0.5;        SpaceGroupElement[77].y=-x+0.5;        SpaceGroupElement[77].z=z+0.5;          // -2       [1,1,0]         0,0,1/2         1/2,0,0
      SpaceGroupElement[78].x=x+0.5;         SpaceGroupElement[78].y=-y+0.5;        SpaceGroupElement[78].z=z+0.5;          // -2       [0,1,0]         1/2,0,1/2       0,1/4,0
      SpaceGroupElement[79].x=y+0.5;         SpaceGroupElement[79].y=x+0.5;         SpaceGroupElement[79].z=z+0.5;          // -2       [-1,1,0]        1/2,1/2,1/2     0,0,0
      SpaceGroupElement[80].x=-z+0.5;        SpaceGroupElement[80].y=-x+0.5;        SpaceGroupElement[80].z=-y+0.5;         // -3^1     [1,1,1]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[81].x=x+0.5;         SpaceGroupElement[81].y=-z+0.5;        SpaceGroupElement[81].z=-y+0.5;         // -2       [0,1,1]         1/2,0,0         0,1/2,0
      SpaceGroupElement[82].x=z+0.5;         SpaceGroupElement[82].y=x+0.5;         SpaceGroupElement[82].z=-y+0.5;         // -3^-1    [-1,1,1]        0,0,0           1/4,3/4,-1/4
      SpaceGroupElement[83].x=-x+0.5;        SpaceGroupElement[83].y=z+0.5;         SpaceGroupElement[83].z=-y+0.5;         // -4^1     [1,0,0]         0,0,0           1/4,1/2,0
      SpaceGroupElement[84].x=-z+0.5;        SpaceGroupElement[84].y=x+0.5;         SpaceGroupElement[84].z=y+0.5;          // -3^-1    [1,-1,1]        0,0,0           -1/4,1/4,3/4
      SpaceGroupElement[85].x=-x+0.5;        SpaceGroupElement[85].y=-z+0.5;        SpaceGroupElement[85].z=y+0.5;          // -4^-1    [1,0,0]         0,0,0           1/4,0,1/2
      SpaceGroupElement[86].x=z+0.5;         SpaceGroupElement[86].y=-x+0.5;        SpaceGroupElement[86].z=y+0.5;          // -3^1     [-1,-1,1]       0,0,0           3/4,-1/4,1/4
      SpaceGroupElement[87].x=x+0.5;         SpaceGroupElement[87].y=z+0.5;         SpaceGroupElement[87].z=y+0.5;          // -2       [0,-1,1]        1/2,1/2,1/2     0,0,0
      SpaceGroupElement[88].x=-y+0.5;        SpaceGroupElement[88].y=-z+0.5;        SpaceGroupElement[88].z=-x+0.5;         // -3^-1    [1,1,1]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[89].x=-y+0.5;        SpaceGroupElement[89].y=z+0.5;         SpaceGroupElement[89].z=x+0.5;          // -3^-1    [-1,-1,1]       0,0,0           -1/4,3/4,1/4
      SpaceGroupElement[90].x=-z+0.5;        SpaceGroupElement[90].y=-y+0.5;        SpaceGroupElement[90].z=x+0.5;          // -4^1     [0,1,0]         0,0,0           0,1/4,1/2
      SpaceGroupElement[91].x=y+0.5;         SpaceGroupElement[91].y=-z+0.5;        SpaceGroupElement[91].z=x+0.5;          // -3^1     [-1,1,1]        0,0,0           1/4,-1/4,3/4
      SpaceGroupElement[92].x=z+0.5;         SpaceGroupElement[92].y=y+0.5;         SpaceGroupElement[92].z=x+0.5;          // -2       [-1,0,1]        1/2,1/2,1/2     0,0,0
      SpaceGroupElement[93].x=y+0.5;         SpaceGroupElement[93].y=z+0.5;         SpaceGroupElement[93].z=-x+0.5;         // -3^1     [1,-1,1]        0,0,0           3/4,1/4,-1/4
      SpaceGroupElement[94].x=-z+0.5;        SpaceGroupElement[94].y=y+0.5;         SpaceGroupElement[94].z=-x+0.5;         // -2       [1,0,1]         0,1/2,0         1/2,0,0
      SpaceGroupElement[95].x=z+0.5;         SpaceGroupElement[95].y=-y+0.5;        SpaceGroupElement[95].z=-x+0.5;         // -4^-1    [0,1,0]         0,0,0           1/2,1/4,0
      break;

    case 530: // [230] I a -3 d (-I 4bd 2c 3)
      SpaceGroupSize=96;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation                       Rotation Axis direction  Screw/glide     Origin shift
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;               // 1        -               -               -
      SpaceGroupElement[1].x=-y+0.25;        SpaceGroupElement[1].y=x+0.75;         SpaceGroupElement[1].z=z+0.25;          // 4^1      [0,0,1]         0,0,1/4         -1/4,1/2,0
      SpaceGroupElement[2].x=-x+0.5;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z+0.5;           // 2        [0,0,1]         0,0,1/2         1/4,0,0
      SpaceGroupElement[3].x=y+0.25;         SpaceGroupElement[3].y=-x+0.25;        SpaceGroupElement[3].z=z+0.75;          // 4^-1     [0,0,1]         0,0,3/4         1/4,0,0
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z+0.5;          // 2        [1,0,0]         0,0,0           0,0,1/4
      SpaceGroupElement[5].x=y+0.25;         SpaceGroupElement[5].y=x+0.75;         SpaceGroupElement[5].z=-z+0.75;         // 2        [1,1,0]         1/2,1/2,0       -1/4,0,3/8
      SpaceGroupElement[6].x=-x+0.5;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;              // 2        [0,1,0]         0,0,0           1/4,0,0
      SpaceGroupElement[7].x=-y+0.25;        SpaceGroupElement[7].y=-x+0.25;        SpaceGroupElement[7].z=-z+0.25;         // 2        [-1,1,0]        0,0,0           1/4,0,1/8
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;               // 3^1      [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[9].x=-x+0.25;        SpaceGroupElement[9].y=z+0.75;         SpaceGroupElement[9].z=y+0.25;          // 2        [0,1,1]         0,1/2,1/2       1/8,1/4,0
      SpaceGroupElement[10].x=-z+0.5;        SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y+0.5;          // 3^-1     [-1,1,1]        0,0,0           1/2,-1/2,0
      SpaceGroupElement[11].x=x+0.25;        SpaceGroupElement[11].y=-z+0.25;       SpaceGroupElement[11].z=y+0.75;         // 4^1      [1,0,0]         1/4,0,0         0,-1/4,1/2
      SpaceGroupElement[12].x=z;             SpaceGroupElement[12].y=-x;            SpaceGroupElement[12].z=-y+0.5;         // 3^-1     [1,-1,1]        1/6,-1/6,1/6    -1/6,1/3,0
      SpaceGroupElement[13].x=x+0.25;        SpaceGroupElement[13].y=z+0.75;        SpaceGroupElement[13].z=-y+0.75;        // 4^-1     [1,0,0]         1/4,0,0         0,3/4,0
      SpaceGroupElement[14].x=-z+0.5;        SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-y;             // 3^1      [-1,-1,1]       1/6,1/6,-1/6    1/3,1/6,0
      SpaceGroupElement[15].x=-x+0.25;       SpaceGroupElement[15].y=-z+0.25;       SpaceGroupElement[15].z=-y+0.25;        // 2        [0,-1,1]        0,0,0           1/8,1/4,0
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=z;             SpaceGroupElement[16].z=x;              // 3^-1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[17].x=y+0.5;         SpaceGroupElement[17].y=-z+0.5;        SpaceGroupElement[17].z=-x;             // 3^-1     [-1,-1,1]       1/3,1/3,-1/3    1/3,1/6,0
      SpaceGroupElement[18].x=z+0.75;        SpaceGroupElement[18].y=y+0.25;        SpaceGroupElement[18].z=-x+0.25;        // 4^1      [0,1,0]         0,1/4,0         1/2,0,-1/4
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z+0.5;         SpaceGroupElement[19].z=-x+0.5;         // 3^1      [-1,1,1]        -1/3,1/3,1/3    1/6,1/6,0
      SpaceGroupElement[20].x=-z+0.25;       SpaceGroupElement[20].y=-y+0.25;       SpaceGroupElement[20].z=-x+0.25;        // 2        [-1,0,1]        0,0,0           1/4,1/8,0
      SpaceGroupElement[21].x=-y+0.5;        SpaceGroupElement[21].y=-z;            SpaceGroupElement[21].z=x+0.5;          // 3^1      [1,-1,1]        1/3,-1/3,1/3    -1/6,1/3,0
      SpaceGroupElement[22].x=z+0.75;        SpaceGroupElement[22].y=-y+0.75;       SpaceGroupElement[22].z=x+0.25;         // 2        [1,0,1]         1/2,0,1/2       1/4,3/8,0
      SpaceGroupElement[23].x=-z+0.75;       SpaceGroupElement[23].y=y+0.25;        SpaceGroupElement[23].z=x+0.75;         // 4^-1     [0,1,0]         0,1/4,0         0,0,3/4
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;             // -1       -               -               0,0,0
      SpaceGroupElement[25].x=y-0.25;        SpaceGroupElement[25].y=-x-0.75;       SpaceGroupElement[25].z=-z-0.25;        // -4^1     [0,0,1]         0,0,0           -1/2,-1/4,-1/8
      SpaceGroupElement[26].x=x-0.5;         SpaceGroupElement[26].y=y;             SpaceGroupElement[26].z=-z-0.5;         // -2       [0,0,1]         -1/2,0,0        0,0,-1/4
      SpaceGroupElement[27].x=-y-0.25;       SpaceGroupElement[27].y=x-0.25;        SpaceGroupElement[27].z=-z-0.75;        // -4^-1    [0,0,1]         0,0,0           0,-1/4,-3/8
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=y;             SpaceGroupElement[28].z=z-0.5;          // -2       [1,0,0]         0,0,-1/2        0,0,0
      SpaceGroupElement[29].x=-y-0.25;       SpaceGroupElement[29].y=-x-0.75;       SpaceGroupElement[29].z=z-0.75;         // -2       [1,1,0]         1/4,-1/4,-3/4   -1/2,0,0
      SpaceGroupElement[30].x=x-0.5;         SpaceGroupElement[30].y=-y;            SpaceGroupElement[30].z=z;              // -2       [0,1,0]         -1/2,0,0        0,0,0
      SpaceGroupElement[31].x=y-0.25;        SpaceGroupElement[31].y=x-0.25;        SpaceGroupElement[31].z=z-0.25;         // -2       [-1,1,0]        -1/4,-1/4,-1/4  0,0,0
      SpaceGroupElement[32].x=-z;            SpaceGroupElement[32].y=-x;            SpaceGroupElement[32].z=-y;             // -3^1     [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[33].x=x-0.25;        SpaceGroupElement[33].y=-z-0.75;       SpaceGroupElement[33].z=-y-0.25;        // -2       [0,1,1]         -1/4,-1/4,1/4   0,-1/2,0
      SpaceGroupElement[34].x=z-0.5;         SpaceGroupElement[34].y=x;             SpaceGroupElement[34].z=-y-0.5;         // -3^-1    [-1,1,1]        0,0,0           -1/2,-1/2,0
      SpaceGroupElement[35].x=-x-0.25;       SpaceGroupElement[35].y=z-0.25;        SpaceGroupElement[35].z=-y-0.75;        // -4^1     [1,0,0]         0,0,0           -1/8,-1/2,-1/4
      SpaceGroupElement[36].x=-z;            SpaceGroupElement[36].y=x;             SpaceGroupElement[36].z=y-0.5;          // -3^-1    [1,-1,1]        0,0,0           1/4,1/4,-1/4
      SpaceGroupElement[37].x=-x-0.25;       SpaceGroupElement[37].y=-z-0.75;       SpaceGroupElement[37].z=y-0.75;         // -4^-1    [1,0,0]         0,0,0           -1/8,0,-3/4
      SpaceGroupElement[38].x=z-0.5;         SpaceGroupElement[38].y=-x;            SpaceGroupElement[38].z=y;              // -3^1     [-1,-1,1]       0,0,0           -1/4,1/4,1/4
      SpaceGroupElement[39].x=x-0.25;        SpaceGroupElement[39].y=z-0.25;        SpaceGroupElement[39].z=y-0.25;         // -2       [0,-1,1]        -1/4,-1/4,-1/4  0,0,0
      SpaceGroupElement[40].x=-y;            SpaceGroupElement[40].y=-z;            SpaceGroupElement[40].z=-x;             // -3^-1    [1,1,1]         0,0,0           0,0,0
      SpaceGroupElement[41].x=-y-0.5;        SpaceGroupElement[41].y=z-0.5;         SpaceGroupElement[41].z=x;              // -3^-1    [-1,-1,1]       0,0,0           0,-1/2,0
      SpaceGroupElement[42].x=-z-0.75;       SpaceGroupElement[42].y=-y-0.25;       SpaceGroupElement[42].z=x-0.25;         // -4^1     [0,1,0]         0,0,0           -1/4,-1/8,-1/2
      SpaceGroupElement[43].x=y;             SpaceGroupElement[43].y=-z-0.5;        SpaceGroupElement[43].z=x-0.5;          // -3^1     [-1,1,1]        0,0,0           0,0,-1/2
      SpaceGroupElement[44].x=z-0.25;        SpaceGroupElement[44].y=y-0.25;        SpaceGroupElement[44].z=x-0.25;         // -2       [-1,0,1]        -1/4,-1/4,-1/4  0,0,0
      SpaceGroupElement[45].x=y-0.5;         SpaceGroupElement[45].y=z;             SpaceGroupElement[45].z=-x-0.5;         // -3^1     [1,-1,1]        0,0,0           -1/2,0,0
      SpaceGroupElement[46].x=-z-0.75;       SpaceGroupElement[46].y=y-0.75;        SpaceGroupElement[46].z=-x-0.25;        // -2       [1,0,1]         -1/4,-3/4,1/4   -1/2,0,0
      SpaceGroupElement[47].x=z-0.75;        SpaceGroupElement[47].y=-y-0.25;       SpaceGroupElement[47].z=-x-0.75;        // -4^-1    [0,1,0]         0,0,0           -3/4,-1/8,0
      SpaceGroupElement[48].x=x+0.5;         SpaceGroupElement[48].y=y+0.5;         SpaceGroupElement[48].z=z+0.5;          // 1        -               -               -
      SpaceGroupElement[49].x=-y+0.75;       SpaceGroupElement[49].y=x+0.25;        SpaceGroupElement[49].z=z+0.75;         // 4^1      [0,0,1]         0,0,3/4         -1/4,1,0
      SpaceGroupElement[50].x=-x;            SpaceGroupElement[50].y=-y+0.5;        SpaceGroupElement[50].z=z;              // 2        [0,0,1]         0,0,1           1/2,1/4,0
      SpaceGroupElement[51].x=y+0.75;        SpaceGroupElement[51].y=-x+0.75;       SpaceGroupElement[51].z=z+0.25;         // 4^-1     [0,0,1]         0,0,5/4         3/4,0,0
      SpaceGroupElement[52].x=x+0.5;         SpaceGroupElement[52].y=-y+0.5;        SpaceGroupElement[52].z=-z;             // 2        [1,0,0]         1/2,0,0         0,1/4,1/2
      SpaceGroupElement[53].x=y+0.75;        SpaceGroupElement[53].y=x+0.25;        SpaceGroupElement[53].z=-z+0.25;        // 2        [1,1,0]         1,1,0           -1/4,0,5/8
      SpaceGroupElement[54].x=-x;            SpaceGroupElement[54].y=y+0.5;         SpaceGroupElement[54].z=-z+0.5;         // 2        [0,1,0]         0,1/2,0         1/2,0,1/4
      SpaceGroupElement[55].x=-y+0.75;       SpaceGroupElement[55].y=-x+0.75;       SpaceGroupElement[55].z=-z+0.75;        // 2        [-1,1,0]        0,0,0           3/4,0,3/8
      SpaceGroupElement[56].x=z+0.5;         SpaceGroupElement[56].y=x+0.5;         SpaceGroupElement[56].z=y+0.5;          // 3^1      [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[57].x=-x+0.75;       SpaceGroupElement[57].y=z+0.25;        SpaceGroupElement[57].z=y+0.75;         // 2        [0,1,1]         0,1,1           3/8,1/4,0
      SpaceGroupElement[58].x=-z;            SpaceGroupElement[58].y=-x+0.5;        SpaceGroupElement[58].z=y;              // 3^-1     [-1,1,1]        -1/6,1/6,1/6    7/6,-5/6,0
      SpaceGroupElement[59].x=x+0.75;        SpaceGroupElement[59].y=-z+0.75;       SpaceGroupElement[59].z=y+0.25;         // 4^1      [1,0,0]         3/4,0,0         0,-1/4,1
      SpaceGroupElement[60].x=z+0.5;         SpaceGroupElement[60].y=-x+0.5;        SpaceGroupElement[60].z=-y;             // 3^-1     [1,-1,1]        1/3,-1/3,1/3    1/6,2/3,0
      SpaceGroupElement[61].x=x+0.75;        SpaceGroupElement[61].y=z+0.25;        SpaceGroupElement[61].z=-y+0.25;        // 4^-1     [1,0,0]         3/4,0,0         0,5/4,0
      SpaceGroupElement[62].x=-z;            SpaceGroupElement[62].y=x+0.5;         SpaceGroupElement[62].z=-y+0.5;         // 3^1      [-1,-1,1]       1/3,1/3,-1/3    2/3,5/6,0
      SpaceGroupElement[63].x=-x+0.75;       SpaceGroupElement[63].y=-z+0.75;       SpaceGroupElement[63].z=-y+0.75;        // 2        [0,-1,1]        0,0,0           3/8,3/4,0
      SpaceGroupElement[64].x=y+0.5;         SpaceGroupElement[64].y=z+0.5;         SpaceGroupElement[64].z=x+0.5;          // 3^-1     [1,1,1]         1/2,1/2,1/2     0,0,0
      SpaceGroupElement[65].x=y;             SpaceGroupElement[65].y=-z;            SpaceGroupElement[65].z=-x+0.5;         // 3^-1     [-1,-1,1]       1/2,1/2,-1/2    1,1/2,0
      SpaceGroupElement[66].x=z+0.25;        SpaceGroupElement[66].y=y+0.75;        SpaceGroupElement[66].z=-x+0.75;        // 4^1      [0,1,0]         0,3/4,0         1,0,-1/4
      SpaceGroupElement[67].x=-y+0.5;        SpaceGroupElement[67].y=z;             SpaceGroupElement[67].z=-x;             // 3^1      [-1,1,1]        -1/2,1/2,1/2    1/2,1/2,0
      SpaceGroupElement[68].x=-z+0.75;       SpaceGroupElement[68].y=-y+0.75;       SpaceGroupElement[68].z=-x+0.75;        // 2        [-1,0,1]        0,0,0           3/4,3/8,0
      SpaceGroupElement[69].x=-y;            SpaceGroupElement[69].y=-z+0.5;        SpaceGroupElement[69].z=x;              // 3^1      [1,-1,1]        1/2,-1/2,1/2    -1/2,1,0
      SpaceGroupElement[70].x=z+0.25;        SpaceGroupElement[70].y=-y+0.25;       SpaceGroupElement[70].z=x+0.75;         // 2        [1,0,1]         1,0,1           1/4,5/8,0
      SpaceGroupElement[71].x=-z+0.25;       SpaceGroupElement[71].y=y+0.75;        SpaceGroupElement[71].z=x+0.25;         // 4^-1     [0,1,0]         0,3/4,0         0,0,5/4
      SpaceGroupElement[72].x=-x+0.5;        SpaceGroupElement[72].y=-y+0.5;        SpaceGroupElement[72].z=-z+0.5;         // -1       -               -               1/4,1/4,1/4
      SpaceGroupElement[73].x=y+0.25;        SpaceGroupElement[73].y=-x-0.25;       SpaceGroupElement[73].z=-z+0.25;        // -4^1     [0,0,1]         0,0,0           0,-1/4,1/8
      SpaceGroupElement[74].x=x;             SpaceGroupElement[74].y=y+0.5;         SpaceGroupElement[74].z=-z;             // -2       [0,0,1]         0,1/2,0         0,0,0
      SpaceGroupElement[75].x=-y+0.25;       SpaceGroupElement[75].y=x+0.25;        SpaceGroupElement[75].z=-z-0.25;        // -4^-1    [0,0,1]         0,0,0           0,1/4,-1/8
      SpaceGroupElement[76].x=-x+0.5;        SpaceGroupElement[76].y=y+0.5;         SpaceGroupElement[76].z=z;              // -2       [1,0,0]         0,1/2,0         1/4,0,0
      SpaceGroupElement[77].x=-y+0.25;       SpaceGroupElement[77].y=-x-0.25;       SpaceGroupElement[77].z=z-0.25;         // -2       [1,1,0]         1/4,-1/4,-1/4   0,0,0
      SpaceGroupElement[78].x=x;             SpaceGroupElement[78].y=-y+0.5;        SpaceGroupElement[78].z=z+0.5;          // -2       [0,1,0]         0,0,1/2         0,1/4,0
      SpaceGroupElement[79].x=y+0.25;        SpaceGroupElement[79].y=x+0.25;        SpaceGroupElement[79].z=z+0.25;         // -2       [-1,1,0]        1/4,1/4,1/4     0,0,0
      SpaceGroupElement[80].x=-z+0.5;        SpaceGroupElement[80].y=-x+0.5;        SpaceGroupElement[80].z=-y+0.5;         // -3^1     [1,1,1]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[81].x=x+0.25;        SpaceGroupElement[81].y=-z-0.25;       SpaceGroupElement[81].z=-y+0.25;        // -2       [0,1,1]         1/4,-1/4,1/4    0,0,0
      SpaceGroupElement[82].x=z;             SpaceGroupElement[82].y=x+0.5;         SpaceGroupElement[82].z=-y;             // -3^-1    [-1,1,1]        0,0,0           -1/4,1/4,-1/4
      SpaceGroupElement[83].x=-x+0.25;       SpaceGroupElement[83].y=z+0.25;        SpaceGroupElement[83].z=-y-0.25;        // -4^1     [1,0,0]         0,0,0           1/8,0,-1/4
      SpaceGroupElement[84].x=-z+0.5;        SpaceGroupElement[84].y=x+0.5;         SpaceGroupElement[84].z=y;              // -3^-1    [1,-1,1]        0,0,0           0,1/2,1/2
      SpaceGroupElement[85].x=-x+0.25;       SpaceGroupElement[85].y=-z-0.25;       SpaceGroupElement[85].z=y-0.25;         // -4^-1    [1,0,0]         0,0,0           1/8,0,-1/4
      SpaceGroupElement[86].x=z;             SpaceGroupElement[86].y=-x+0.5;        SpaceGroupElement[86].z=y+0.5;          // -3^1     [-1,-1,1]       0,0,0           1/2,0,1/2
      SpaceGroupElement[87].x=x+0.25;        SpaceGroupElement[87].y=z+0.25;        SpaceGroupElement[87].z=y+0.25;         // -2       [0,-1,1]        1/4,1/4,1/4     0,0,0
      SpaceGroupElement[88].x=-y+0.5;        SpaceGroupElement[88].y=-z+0.5;        SpaceGroupElement[88].z=-x+0.5;         // -3^-1    [1,1,1]         0,0,0           1/4,1/4,1/4
      SpaceGroupElement[89].x=-y;            SpaceGroupElement[89].y=z;             SpaceGroupElement[89].z=x+0.5;          // -3^-1    [-1,-1,1]       0,0,0           -1/4,1/4,1/4
      SpaceGroupElement[90].x=-z-0.25;       SpaceGroupElement[90].y=-y+0.25;       SpaceGroupElement[90].z=x+0.25;         // -4^1     [0,1,0]         0,0,0           -1/4,1/8,0
      SpaceGroupElement[91].x=y+0.5;         SpaceGroupElement[91].y=-z;            SpaceGroupElement[91].z=x;              // -3^1     [-1,1,1]        0,0,0           1/4,-1/4,1/4
      SpaceGroupElement[92].x=z+0.25;        SpaceGroupElement[92].y=y+0.25;        SpaceGroupElement[92].z=x+0.25;         // -2       [-1,0,1]        1/4,1/4,1/4     0,0,0
      SpaceGroupElement[93].x=y;             SpaceGroupElement[93].y=z+0.5;         SpaceGroupElement[93].z=-x;             // -3^1     [1,-1,1]        0,0,0           1/4,1/4,-1/4
      SpaceGroupElement[94].x=-z-0.25;       SpaceGroupElement[94].y=y-0.25;        SpaceGroupElement[94].z=-x+0.25;        // -2       [1,0,1]         -1/4,-1/4,1/4   0,0,0
      SpaceGroupElement[95].x=z-0.25;        SpaceGroupElement[95].y=-y+0.25;       SpaceGroupElement[95].z=-x-0.25;        // -4^-1    [0,1,0]         0,0,0           -1/4,1/8,0
      break;
    default:
      SpaceGroupSize=0;
      break;
  }
}

void PrintSpaceGroupElements(int SpaceGroup,FILE *Stream)
{

  switch(SpaceGroup)
  {
    // TRICLINIC GROUPS
    // ================

    case 1:   // [1] P 1 (P 1)
      fprintf(Stream,"\t'x,y,z'\n");
      break;

    case 2:   // [2] P -1 (-P 1)
      break;


    // MONOCLINIC GROUPS
    // =================

    case 3: // [3] P 1 2 1 unique b axis (P 2y)
      break;
    case 4: // [3] P 1 1 2 unique c axis (P 2)
      break;
    case 5: // [3] P 2 1 1 unique a axis (P 2x)
      break;

    case 6: // [4] P 1 21 1 unique b axis (P 2yb)
      break;
    case 7: // [4] P 1 1 21 unique c axis (P 2c)
      break;
    case 8: // [4] P 21 1 1 unique a axis (P 2xa)
      break;

    case 9:  // [5] C 1 2 1 unique b axis: cell choice 1 (C 2y)
      break;
    case 10: // [5] A 1 2 1 unique b axis: cell choice 2 (A 2y)
      break;
    case 11: // [5] I 1 2 1 unique b axis: cell choice 3 (I 2y)
      break;
    case 12: // [5] A 1 1 2 unique c axis: cell choice 1 (A 2)
      break;
    case 13: // [5] B 1 1 2 unique c axis: cell choice 2 (B 2)
      break;
    case 14: // [5] I 1 1 2 unique c axis: cell choice 3 (I 2)
      break;
    case 15: // [5] B 2 1 1 unique a axis: cell choice 1 (B 2x)
      break;
    case 16: // [5] C 2 1 1 unique a axis: cell choice 2 (C 2x)
      break;
    case 17: // [5] I 2 1 1 unique a axis: cell choice 3 (I 2x)
      break;

    case 18: // [6] P 1 m 1 unique b axis (P -2y)
      break;
    case 19: // [6] P 1 1 m unique c axis (P -2)
      break;
    case 20: // [6] P m 1 1 unique a axis (P -2x)
      break;

    case 21: // [7] P 1 c 1 unique b axis: cell choice 1 (P -2yc)
      break;
    case 22: // [7] P 1 n 1 unique b axis: cell choice 2 (P -2yac)
      break;
    case 23: // [7] P 1 a 1 unique b axis: cell choice 3 (P -2ya)
      break;
    case 24: // [7] P 1 1 a unique c axis: cell choice 1 (P -2a)
      break;
    case 25: // [7] P 1 1 n unique c axis: cell choice 2 (P -2ab)
      break;
    case 26: // [7] P 1 1 b unique c axis: cell choice 3 (P -2b)
      break;
    case 27: // [7] P b 1 1 unique a axis: cell choice 1 (P -2xb)
      break;
    case 28: // [7] P n 1 1 unique a axis: cell choice 2 (P -2xbc)
      break;
    case 29: // [7] P c 1 1 unique a axis: cell choice 3 (P -2xc)
      break;

    case 30: // [8] C 1 m 1 unique b axis: cell choice 1 (C -2y)
      break;
    case 31: // [8] A 1 m 1 unique b axis: cell choice 2 (A -2y)
      break;
    case 32: // [8] I 1 m 1 unique b axis: cell choice 3 (I -2y)
      break;
    case 33: // [8] A 1 1 m unique c axis: cell choice 1 (A -2)
      break;
    case 34: // [8] B 1 1 m  unique c axis: cell choice 2 (B -2)
      break;
    case 35: // [8] I 1 1 m unique c axis: cell choice 3 (I -2)
      break;
    case 36: // [8] B m 1 1 unique a axis: cell choice 1 (B -2x)
      break;
    case 37: // [8] C m 1 1 unique a axis: cell choice 2 (C -2x)
      break;
    case 38: // [8] I m 1 1 unique a axis: cell choice 3 (I -2x)
      break;

    case 39: // [9] C 1 c 1 unique b axis: cell choice 1 (C -2yc)
      break;
    case 40: // [9] A 1 n 1 unique b axis: cell choice 2 (A -2yab)
      break;
    case 41: // [9] I 1 a 1 unique b axis: cell choice 3 (I -2ya)
      break;
    case 42: // [9] A 1 a 1 unique -b axis: cell choice 1 (A -2ya)
      break;
    case 43: // [9] C 1 n 1 unique -b axis: cell choice 2 (C -2yac)
      break;
    case 44: // [9] I 1 c 1 unique -b axis: cell choice 3 (I -2yc)
      break;
    case 45: // [9] A 1 1 a unique c axis: cell choice 1 (A -2a)
      break;
    case 46: // [9] B 1 1 n unique c axis: cell choice 2 (B -2ab)
      break;
    case 47: // [9] I 1 1 b unique c axis: cell choice 3 (I -2b)
      break;
    case 48: // [9] B 1 1 b unique -c axis: cell choice 1 (B -2b)
      break;
    case 49: // [9] A 1 1 n unique -c axis: cell choice 2 (A -2ab)
      break;
    case 50: // [9] I 1 1 a unique -c axis: cell choice 3 (I -2a)
      break;
    case 51: // [9] B b 1 1 unique a axis: cell choice 1 (B -2xb)
      break;
    case 52: // [9] C n 1 1 unique a axis: cell choice 2 (C -2xac)
      break;
    case 53: // [9] I c 1 1 unique a axis: cell choice 3 (I -2xc)
      break;
    case 54: // [9] C c 1 1 unique -a axis: cell choice 1 (C -2xc)
      break;
    case 55: // [9] B n 1 1 unique -a axis: cell choice 2 (B -2xab)
      break;
    case 56: // [9] I b 1 1 unique -a axis: cell choice 3 (I -2xb)
      break;

    case 57: // [10] P 1 2/m 1 unique b axis (-P 2y)
      break;
    case 58: // [10] P 1 1 2/m unique c axis (-P 2)
      break;
    case 59: // [10] P 2/m 1 1 unique a axis (-P 2x)
      break;

    case 60: // [11] P 1 21/m 1 unique axis b (-P 2yb)
      break;
    case 61: // [11] P 1 1 21/m unique c axis (-P 2c)
      break;
    case 62: // [11] P 21/m 1 1 unique a axis (-P 2xa)
      break;

    case 63: // [12] C 1 2/m 1 unique b axis: cell choice 1 (-C 2y)
      break;
    case 64: // [12] A 1 2/m 1 unique b axis: cell choice 2 (-A 2y)
      break;
    case 65: // [12] I 1 2/m 1 unique b axis: cell choice 3 (-I 2y)
      break;
    case 66: // [12] A 1 1 2/m unique c axis: cell choice 1 (-A 2)
      break;
    case 67: // [12] B 1 1 2/m unique c axis: cell choice 2 (-B 2)
      break;
    case 68: // [12] I 1 1 2/m unique c axis: cell choice 3 (-I 2)
      break;
    case 69: // [12] B 2/m 1 1 unique a axis: cell choice 1 (-B 2x)
      break;
    case 70: // [12] C 2/m 1 1 unique a axis: cell choice 2 (-C 2x)
      break;
    case 71: // [12] I 2/m 1 1 unique a axis: cell choice 3 (-I 2x)
      break;

    case 72: // [13] P 1 2/c 1 unique b axis: cell choice 1 (-P 2yc)
      break;
    case 73: // [13] P 1 2/n 1 unique b axis: cell choice 2 (-P 2yac)
      break;
    case 74: // [13] P 1 2/a 1 unique b axis: cell choice 3 (-P 2ya)
      break;
    case 75: // [13] P 1 1 2/a unique c axis: cell choice 1 (-P 2a)
      break;
    case 76: // [13] P 1 1 2/n unique c axis: cell choice 2 (-P 2ab)
      break;
    case 77: // [13] P 1 1 2/b unique c axis: cell choice 3 (-P 2b)
      break;
    case 78: // [13] P 2/b 1 1 unique a axis: cell choice 1 (-P 2xb)
      break;
    case 79: // [13] P 2/n 1 1 unique a axis: cell choice 2 (-P 2xbc)
      break;
    case 80: // [13] P 2/c 1 1 unique a axis: cell choice 3 (-P 2xc)
      break;

    case 81: // [14] P 1 21/c 1 unique b axis: cell choice 1 (-P 2ybc)
      break;
    case 82: // [14] P 1 21/n 1 unique b axis: cell choice 2 (-P 2yn)
      break;
    case 83: // [14] P 1 21/a 1 unique b axis: cell choice 3 (-P 2yab)
      break;
    case 84: // [14] P 1 1 21/a unique c axis: cell choice 1 (-P 2ac)
      break;
    case 85: // [14] P 1 1 21/n unique c axis: cell choice 2 (-P 2n)
      break;
    case 86: // [14] P 1 1 21/b unique c axis: cell choice 3 (-P 2bc)
      break;
    case 87: // [14] P 21/b 1 1 unique a axis: cell choice 1 (-P 2xab)
      break;
    case 88: // [14] P 21/n 1 1 unique a axis: cell choice 2 (-P 2xn)
      break;
    case 89: // [14] P 21/c 1 1 unique a axis: cell choice 3 (-P 2xac)
      break;

    case 90: // [15] C 1 2/c 1 unique b axis: cell choice 1 (-C 2yc)
      break;
    case 91: // [15] A 1 2/n 1 unique b axis: cell choice 2 (-A 2yab)
      break;
    case 92: // [15] I 1 2/a 1 unique b axis: cell choice 3 (-I 2ya)
      break;
    case 93: // [15] A 1 2/a 1 unique -b axis: cell choice 1 (-A 2ya)
      break;
    case 94: // [15] C 1 2/n 1 unique -b axis: cell choice 2 (-C 2yac)
      break;
    case 95: // [15] I 1 2/c 1 unique -b axis: cell choice 3 (-I 2yc)
      break;
    case 96: // [15] A 1 1 2/a unique c axis: cell choice 1 (-A 2a)
      break;
    case 97: // [15] B 1 1 2/n unique c axis: cell choice 2 (-B 2ab)
      break;
    case 98: // [15] I 1 1 2/b unique c axis: cell choice 3 (-I 2b)
      break;
    case 99: // [15] B 1 1 2/b unique -c axis: cell choice 1 (-B 2b)
      break;
    case 100: // [15] A 1 1 2/n unique -c axis: cell choice 2 (-A 2ab)
      break;
    case 101: // [15] I 1 1 2/a unique -c axis: cell choice 3 (-I 2a)
      break;
    case 102: // [15] B 2/b 1 1 unique a axis: cell choice 1 (-B 2xb)
      break;
    case 103: // [15] C 2/n 1 1 unique a axis: cell choice 2 (-C 2xac)
      break;
    case 104: // [15] I 2/c 1 1 unique a axis: cell choice 3 (-I 2xc)
      break;
    case 105: // [15] C 2/c 1 1 unique -a axis: cell choice 1 (-C 2xc)
      break;
    case 106: // [15] B 2/n 1 1 unique -a axis: cell choice 2 (-B 2xab)
      break;
    case 107: // [15] I 2/b 1 1 unique -a axis: cell choice 3 (-I 2xb)
      break;


    // ORTHORHOMBIC GROUPS
    // ===================

    case 108: // [16] P 2 2 2 (P 2 2)
      break;

    case 109: // [17] P 2 2 21 Origin-1,abc (P 2c 2)
      break;
    case 110: // [17] P 21 2 2 Origin-1,cab (P 2a 2a)
      break;
    case 111: // [17] P 2 21 2 Origin-1,bca (P 2 2b)
      break;

    case 112: // [18] P 21 21 2 Origin-1,abc (P 2 2ab)
      break;
    case 113: // [18] P 2 21 21 Origin-1,cab (P 2bc 2)
      break;
    case 114: // [18] P 21 2 21 Origin-1,bca (P 2ac 2ac)
      break;

    case 115: // [19] P 21 21 21 (P 2ac 2ab)
      break;

    case 116: // [20] C 2 2 21  Origin-1,abc (C 2c 2)
      break;
    case 117: // [20] A 21 2 2  Origin-1,cba (A 2a 2a)
      break;
    case 118: // [20] B 2 21 2  Origin-1,bca (B 2 2b)
      break;

    case 119: // [21] C 2 2 2 Origin-1,abc (C 2 2)
      break;
    case 120: // [21] A 2 2 2 Origin-1,cab (A 2 2)
      break;
    case 121: // [21] B 2 2 2 Origin-1,bca (B 2 2)
      break;

    case 122: // [22] F 2 2 2 (F 2 2)
      break;

    case 123: // [23] I 2 2 2 (I 2 2)
      break;

    case 124: // [24] I 21 21 21 (I 2b 2c)
      break;

    case 125: // [25] P m m 2 (P 2 -2)
      break;
    case 126: // [25] P 2 m m (P -2 2)
      break;
    case 127: // [25] P m 2 m (P -2 -2)
      break;

    case 128: // [26] P m c 21 (P 2c -2)
      break;
    case 129: // [26] P c m 21 (P 2c -2c)
      break;
    case 130: // [26] P 21 m a (P -2a 2a)
      break;
    case 131: // [26] P 21 a m (P -2 2a)
      break;
    case 132: // [26] P b 21 m (P -2 -2b)
      break;
    case 133: // [26] P m 21 b (P -2b -2)
      break;

    case 134: // [27] P c c 2 (P 2 -2c)
      break;
    case 135: // [27] P 2 a a (P -2a 2)
      break;
    case 136: // [27] P b 2 b (P -2b -2b)
      break;

    case 137: // [28] P m a 2 (P 2 -2a)
      break;
    case 138: // [28] P b m 2 (P 2 -2b)
      break;
    case 139: // [28] P 2 m b (P -2b 2)
      break;
    case 140: // [28] P 2 c m (P -2c 2)
      break;
    case 141: // [28] P c 2 m (P -2c -2c)
      break;
    case 142: // [28] P m 2 a (P -2a -2a)
      break;

    case 143: // [29] P c a 21 (P 2c -2ac)
      break;
    case 144: // [29] P b c 21 (P 2c -2b)
      break;
    case 145: // [29] P 21 a b (P -2b 2a)
      break;
    case 146: // [29] P 21 c a (P -2ac 2a)
      break;
    case 147: // [29] P c 21 b (P -2bc -2c)
      break;
    case 148: // [29] P b 21 a (P -2a -2ab)
      break;

    case 149: // [30] P n c 2 (P 2 -2bc)
      break;
    case 150: // [30] P c n 2 (P 2 -2ac)
      break;
    case 151: // [30] P 2 n a (P -2ac 2)
      break;
    case 152: // [30] P 2 a n (P -2ab 2)
      break;
    case 153: // [30] P b 2 n (P -2ab -2ab)
      break;
    case 154: // [30] P n 2 b (P -2bc -2bc)
      break;

    case 155: // [31] P m n 21 (P 2ac -2)
      break;
    case 156: // [31] P n m 21 (P 2bc -2bc)
      break;
    case 157: // [31] P 21 m n (P -2ab 2ab)
      break;
    case 158: // [31] P 21 n m (P -2 2ac)
      break;
    case 159: // [31] P n 21 m (P -2 -2bc)
      break;
    case 160: // [31] P m 21 n (P -2ab -2)
      break;

    case 161: // [32] P b a 2 (P 2 -2ab)
      break;
    case 162: // [32] P 2 c b (P -2bc 2)
      break;
    case 163: // [32] P c 2 a (P -2ac -2ac)
      break;

    case 164: // [33] P n a 21 (P 2c -2n)
      break;
    case 165: // [33] P b n 21 (P 2c -2ab)
      break;
    case 166: // [33] P 21 n b (P -2bc 2a)
      break;
    case 167: // [33] P 21 c n (P -2n 2a)
      break;
    case 168: // [33] P c 21 n (P -2n -2ac)
      break;
    case 169: // [33] P n 21 a (P -2ac -2n)
      break;

    case 170: // [34] P n n 2 (P 2 -2n)
      break;
    case 171: // [34] P 2 n n (P -2n 2)
      break;
    case 172: // [34] P n 2 n (P -2n -2n)
      break;

    case 173: // [35] C m m 2 (C 2 -2)
      break;
    case 174: // [35] A 2 m m (A -2 2)
      break;
    case 175: // [35] B m 2 m (B -2 -2)
      break;

    case 176: // [36] C m c 21 (C 2c -2)
      break;
    case 177: // [36] C c m 21 (C 2c -2c)
      break;
    case 178: // [36] A 21 m a (A -2a 2a)
      break;
    case 179: // [36] A 21 a m (A -2 2a)
      break;
    case 180: // [36] B b 21 m (B -2 -2b)
      break;
    case 181: // [36] B m 21 b (B -2b -2)
      break;

    case 182: // [37] C c c 2 (C 2 -2c)
      break;
    case 183: // [37] A 2 a a (A -2a 2)
      break;
    case 184: // [37] B b 2 b (B -2b -2b)
      break;

    case 185: // [38] A m m 2 (A 2 -2)
      break;
    case 186: // [38] B m m 2 (B 2 -2)
      break;
    case 187: // [38] B 2 m m (B -2 2)
      break;
    case 188: // [38] C 2 m m (C -2 2)
      break;
    case 189: // [38] C m 2 m (C -2 -2)
      break;
    case 190: // [38] A m 2 m (A -2 -2)
      break;

    case 191: // [39] A b m 2 (A 2 -2b)
      break;
    case 192: // [39] B m a 2 (B 2 -2a)
      break;
    case 193: // [39] B 2 c m (B -2a 2)
      break;
    case 194: // [39] C 2 m b (C -2a 2)
      break;
    case 195: // [39] C m 2 a (C -2a -2a)
      break;
    case 196: // [39] A c 2 m (A -2b -2b)
      break;

    case 197: // [40] A m a 2 (A 2 -2a)
      break;
    case 198: // [40] B b m 2 (B 2 -2b)
      break;
    case 199: // [40] B 2 m b (B -2b 2)
      break;
    case 200: // [40] C 2 c m (C -2c 2)
      break;
    case 201: // [40] C c 2 m (C -2c -2c)
      break;
    case 202: // [40] A m 2 a (A -2a -2a)
      break;

    case 203: // [41] A b a 2 (A 2 -2ab)
      break;
    case 204: // [41] B b a 2 (B 2 -2ab)
      break;
    case 205: // [41] B 2 c b (B -2ab 2)
      break;
    case 206: // [41] C 2 c b (C -2ac 2)
      break;
    case 207: // [41] C c 2 a (C -2ac -2ac)
      break;
    case 208: // [41] A c 2 a (A -2ab -2ab)
      break;

    case 209: // [42] F m m 2 (F 2 -2)
      break;
    case 210: // [42] F 2 m m (F -2 2)
      break;
    case 211: // [42] F m 2 m (F -2 -2)
      break;

    case 212: // [43] F d d 2 (F 2 -2d)
      break;
    case 213: // [43] F 2 d d (F -2d 2)
      break;
    case 214: // [43] F d 2 d (F -2d -2d)
      break;

    case 215: // [44] I m m 2 (I 2 -2)
      break;
    case 216: // [44] I 2 m m (I -2 2)
      break;
    case 217: // [44] I m 2 m (I -2 -2)
      break;

    case 218: // [45] I b a 2 (I 2 -2c)
      break;
    case 219: // [45] I 2 c b (I -2a 2)
      break;
    case 220: // [45] I c 2 a (I -2b -2b)
      break;

    case 221: // [46] I m a 2 (I 2 -2a)
      break;
    case 222: // [46] I b m 2 (I 2 -2b)
      break;
    case 223: // [46] I 2 m b (I -2b 2)
      break;
    case 224: // [46] I 2 c m (I -2c 2)
      break;
    case 225: // [46] I c 2 m (I -2c -2c)
      break;
    case 226: // [46] I m 2 a (I -2a -2a)
      break;

    case 227: // [47] P m m m (-P 2 2)
      break;

    case 228: // [48] P n n n Origin choice 1 (P 2 2 -1n)
      break;
    case 229: // [48] P n n n Origin choice 2 (-P 2ab 2bc)
      break;

    case 230: // [49] P c c m  (-P 2 2c)
      break;
    case 231: // [49] P m a a (-P 2a 2)
      break;
    case 232: // [49] P b m b (-P 2b 2b)
      break;

    case 233: // [50] P b a n Origin choice 1 (P 2 2 -1ab)
      break;
    case 234: // [50] P b a n Origin choice 2 (-P 2ab 2b)
      break;
    case 235: // [50] P n c b Origin choice 1 (P 2 2 -1bc)
      break;
    case 236: // [50] P n c b Origin choice 2 (-P 2b 2bc)
      break;
    case 237: // [50] P c n a Origin choice 1 (P 2 2 -1ac)
      break;
    case 238: // [50] P c n a Origin choice 2 (-P 2a 2c)
      break;

    case 239: // [51] P m m a (-P 2a 2a)
      break;
    case 240: // [51] P m m b (-P 2b 2)
      break;
    case 241: // [51] P b m m (-P 2 2b)
      break;
    case 242: // [51] P c m m (-P 2c 2c)
      break;
    case 243: // [51] P m c m (-P 2c 2)
      break;
    case 244: // [51] P m a m (-P 2 2a)
      break;

    case 245: // [52] P n n a (-P 2a 2bc)
      break;
    case 246: // [52] P n n b (-P 2b 2n)
      break;
    case 247: // [52] P b n n (-P 2n 2b)
      break;
    case 248: // [52] P c n n (-P 2ab 2c)
      break;
    case 249: // [52] P n c n (-P 2ab 2n)
      break;
    case 250: // [52] P n a n (-P 2n 2bc)
      break;

    case 251: // [53] P m n a (-P 2ac 2)
      break;
    case 252: // [53] Pnmb (-P 2bc 2bc)
      break;
    case 253: // [53] P b m n (-P 2ab 2ab)
      break;
    case 254: // [53] P c n m (-P 2 2ac)
      break;
    case 255: // [53] P n c m (-P 2 2bc)
      break;
    case 256: // [53] P m a n (-P 2ab 2)
      break;

    case 257: // [54] P c c a (-P 2a 2ac)
      break;
    case 258: // [54] P c c b (-P 2b 2c)
      break;
    case 259: // [54] P b a a (-P 2a 2b)
      break;
    case 260: // [54] P c a a (-P 2ac 2c)
      break;
    case 261: // [54] P b c b (-P 2bc 2b)
      break;
    case 262: // [54] P b a b (-P 2b 2ab)
      break;

    case 263: // [55] P b a m (-P 2 2ab)
      break;
    case 264: // [55] P m c b (-P 2bc 2)
      break;
    case 265: // [55] P c m a (-P 2ac 2ac)
      break;

    case 266: // [56] P c c n (-P 2ab 2ac)
      break;
    case 267: // [56] P n a a (-P 2ac 2bc)
      break;
    case 268: // [56] P b n b (-P 2bc 2ab)
      break;

    case 269: // [57] P b c m (-P 2c 2b)
      break;
    case 270: // [57] P c a m (-P 2c 2ac)
      break;
    case 271: // [57] P m c a (-P 2ac 2a)
      break;
    case 272: // [57] P m a b (-P 2b 2a)
      break;
    case 273: // [57] P b m a (-P 2a 2ab)
      break;
    case 274: // [57] P c m b (-P 2bc 2c)
      break;

    case 275: // [58] P n n m (-P 2 2n)
      break;
    case 276: // [58] P m n n (-P 2n 2)
      break;
    case 277: // [58] P n m n (-P 2n 2n)
      break;

    case 278: // [59] P m m n Origin choice 1 (P 2 2ab -1ab)
      break;
    case 279: // [59] P m m n Origin choice 2 (-P 2ab 2a)
      break;
    case 280: // [59] P n m m Origin choice 1 (P 2bc 2 -1bc)
      break;
    case 281: // [59] P n m m Origin choice 2 (-P 2c 2bc)
      break;
    case 282: // [59] P m n m Origin choice 1 (P 2ac 2ac -1ac)
      break;
    case 283: // [59] P m n m Origin choice 2 (-P 2c 2a)
      break;

    case 284: // [60] P b c n (-P 2n 2ab)
      break;
    case 285: // [60] P c a n (-P 2n 2c)
      break;
    case 286: // [60] P n c a (-P 2a 2n)
      break;
    case 287: // [60] P n a b (-P 2bc 2n)
      break;
    case 288: // [60] P b n a (-P 2ac 2b)
      break;
    case 289: // [60] P c n b (-P 2b 2ac)
      break;

    case 290: // [61] P b c a (-P 2ac 2ab)
      break;
    case 291: // [61] P c a b (-P 2bc 2ac)
      break;

    case 292: // [62] P n m a (-P 2ac 2n)      zeolites: MFI
      fprintf(Stream,"\t'x,y,z'\n\t'-x+1/2,-y,z+1/2'\n\t'x+1/2,-y+1/2,-z+1/2'\n\t'-x,y+1/2,-z'\n\t'-x,-y,-z'\n\t'x-1/2,y,-z-1/2'\n\t'-x-1/2,y-1/2,z-1/2'\n\t'x,-y-1/2,z'\n");
      break;
    case 293: // [62] P m n b (-P 2bc 2a)
      fprintf(Stream,"\t'x,y,z'\n\t'-x,-y+1/2,z+1/2'\n\t'x+1/2,-y,-z'\n\t'-x+1/2,y+1/2,-z+1/2'\n\t'-x,-y,-z'\n\t'x,y-1/2,-z-1/2'\n\t'-x-1/2,y,z'\n\t'x-1/2,-y-1/2,z-1/2'\n");
      break;
    case 294: // [62] P b n m (-P 2c 2ab)
      fprintf(Stream,"\t'x,y,z'\n\t'-x,-y,z+1/2'\n\t'x+1/2,-y+1/2,-z'\n\t'-x+1/2,y+1/2,-z+1/2'\n\t'-x,-y,-z'\n\t'x,y,-z-1/2'\n\t'-x-1/2,y-1/2,z'\n\t'x-1/2,-y-1/2,z-1/2'\n");
      break;
    case 295: // [62] P c m n (-P 2n 2ac)
      fprintf(Stream,"\t'x,y,z'\n\t'-x+1/2,-y+1/2,z+1/2'\n\t'x+1/2,-y,-z+1/2'\n\t'-x,y+1/2,-z'\n\t'-x,-y,-z'\n\t'x-1/2,y-1/2,-z-1/2'\n\t'-x-1/2,y,z-1/2'\n\t'x,-y-1/2,z'\n");
      break;
    case 296: // [62] P m c n (-P 2n 2a)
      fprintf(Stream,"\t'x,y,z'\n\t'-x+1/2,-y+1/2,z+1/2'\n\t'x+1/2,-y,-z'\n\t'-x,y+1/2,-z+1/2'\n\t'-x,-y,-z'\n\t'x-1/2,y-1/2,-z-1/2'\n\t'-x-1/2,y,z'\n\t'x,-y-1/2,z-1/2'\n");
      break;
    case 297: // [62] P n a m (-P 2c 2n)
      fprintf(Stream,"\t'x,y,z'\n\t'-x,-y,z+1/2'\n\t'x+1/2,-y+1/2,-z+1/2'\n\t'-x+1/2,y+1/2,-z'\n\t'-x,-y,-z'\n\t'x,y,-z-1/2'\n\t'-x-1/2,y-1/2,z-1/2'\n\t'x-1/2,-y-1/2,z'\n");
      break;

    case 298: // [63] C m c m (-C 2c 2)
      fprintf(Stream,"\t'x,y,z'\n\t'-x,-y,z+1/2'\n\t'x,-y,-z'\n\t'-x,y,-z+1/2'\n\t'-x,-y,-z'\n\t'x,y,-z-1/2'\n\t'-x,y,z'\n"
        "\t'x,-y,z-1/2'\n\t'x+1/2,y+1/2,z'\n\t'-x+1/2,-y+1/2,z+1/2'\n\t'x+1/2,-y+1/2,-z'\n\t'-x+1/2,y+1/2,-z+1/2'\n\t'-x+1/2,-y+1/2,-z'\n\t'x+1/2,y+1/2,-z-1/2'\n"
        "\t'-x+1/2,y+1/2,z'\n\t'x+1/2,-y+1/2,z-1/2'\n");
      break;
    case 299: // [63] C c m m (-C 2c 2c)
      fprintf(Stream,"\t'x,y,z'\n\t'-x,-y,z+1/2'\n\t'x,-y,-z+1/2'\n\t'-x,y,-z'\n\t'-x,-y,-z'\n\t'x,y,-z-1/2'\n\t'-x,y,z-1/2'\n"
        "\t'x,-y,z'\n\t'x+1/2,y+1/2,z'\n\t'-x+1/2,-y+1/2,z+1/2'\n\t'x+1/2,-y+1/2,-z+1/2'\n\t'-x+1/2,y+1/2,-z'\n\t'-x+1/2,-y+1/2,-z'\n\t'x+1/2,y+1/2,-z-1/2'\n"
        "\t'-x+1/2,y+1/2,z-1/2'\n\t'x+1/2,-y+1/2,z'\n");
      break;
    case 300: // [63] A m m a (-A 2a 2a)
      fprintf(Stream,"\t'x,y,z'\n\t'-x+1/2,-y,z'\n\t'x+1/2,-y,-z'\n\t'-x,y,-z'\n\t'-x,-y,-z'\n\t'x-1/2,y,-z'\n\t'-x-1/2,y,z'\n"
        "\t'x,-y,z'\n\t'x,y+1/2,z+1/2'\n\t'-x+1/2,-y+1/2,z+1/2'\n\t'x+1/2,-y+1/2,-z+1/2'\n\t'-x,y+1/2,-z+1/2'\n\t'-x,-y+1/2,-z+1/2'\n\t'x-1/2,y+1/2,-z+1/2'\n"
        "\t'-x-1/2,y+1/2,z+1/2'\n\t'x,-y+1/2,z+1/2'\n");
      break;
    case 301: // [63] A m a m (-A 2 2a)
      fprintf(Stream,"\t'x,y,z'\n\t'-x,-y,z'\n\t'x+1/2,-y,-z'\n\t'-x+1/2,y,-z'\n\t'-x,-y,-z'\n\t'x,y,-z'\n\t'-x-1/2,y,z'\n"
        "\t'x-1/2,-y,z'\n\t'x,y+1/2,z+1/2'\n\t'-x,-y+1/2,z+1/2'\n\t'x+1/2,-y+1/2,-z+1/2'\n\t'-x+1/2,y+1/2,-z+1/2'\n\t'-x,-y+1/2,-z+1/2'\n\t'x,y+1/2,-z+1/2'\n"
        "\t'-x-1/2,y+1/2,z+1/2'\n\t'x-1/2,-y+1/2,z+1/2'\n");
      break;
    case 302: // [63] B b m m (-B 2 2b)
      fprintf(Stream,"\t'x,y,z'\n\t'-x,-y,z'\n\t'x,-y+1/2,-z'\n\t'-x,y+1/2,-z'\n\t'-x,-y,-z'\n\t'x,y,-z'\n\t'-x,y-1/2,z'\n"
        "\t'x,-y-1/2,z'\n\t'x+1/2,y,z+1/2'\n\t'-x+1/2,-y,z+1/2'\n\t'x+1/2,-y+1/2,-z+1/2'\n\t'-x+1/2,y+1/2,-z+1/2'\n\t'-x+1/2,-y,-z+1/2'\n\t'x+1/2,y,-z+1/2'\n"
        "\t'-x+1/2,y-1/2,z+1/2'\n\t'x+1/2,-y-1/2,z+1/2'\n");
      break;
    case 303: // [63] B m m b (-B 2b 2)
      fprintf(Stream,"\t'x,y,z'\n\t'-x,-y+1/2,z'\n\t'x,-y,-z'\n\t'-x,y+1/2,-z'\n\t'-x,-y,-z'\n\t'x,y-1/2,-z'\n\t'-x,y,z'\n"
        "\t'x,-y-1/2,z'\n\t'x+1/2,y,z+1/2'\n\t'-x+1/2,-y+1/2,z+1/2'\n\t'x+1/2,-y,-z+1/2'\n\t'-x+1/2,y+1/2,-z+1/2'\n\t'-x+1/2,-y,-z+1/2'\n\t'x+1/2,y-1/2,-z+1/2'\n"
        "\t'-x+1/2,y,z+1/2'\n\t'x+1/2,-y-1/2,z+1/2'\n");
      break;

    case 304: // [64] C m c a (-C 2ac 2)
      break;
    case 305: // [64] C c m b (-C 2ac 2ac)
      break;
    case 306: // [64] A b m a (-A 2ab 2ab)
      break;
    case 307: // [64] A c a m (-A 2 2ab)
      break;
    case 308: // [64] B b c m (-B 2 2ab)
      break;
    case 309: // [64] B m a b (-B 2ab 2)
      break;

    case 310: // [65] C m m m (-C 2 2)
      break;
    case 311: // [65] A m m m (-A 2 2)
      break;
    case 312: // [65] B m m m (-B 2 2)
      break;

    case 313: // [66] C c c m (-C 2 2c)
      break;
    case 314: // [66] A m a a (-A 2a 2)
      break;
    case 315: // [66] B b m b (-B 2b 2b)
      break;

    case 316: // [67] C m m a (-C 2a 2)
      break;
    case 317: // [67] C m m b (-C 2a 2a)
      break;
    case 318: // [67] A b m m (-A 2b 2b)
      break;
    case 319: // [67] A c m m (-A 2 2b)
      break;
    case 320: // [67] B m c m (-B 2 2a)
      break;
    case 321: // [67] B m a m (-B 2a 2)
      break;

    case 322: // [68] C c c a Origin choice 1 (C 2 2 -1ac)
      break;
    case 323: // [68] C c c a Origin choice 2 (-C 2a 2ac)
      break;
    case 324: // [68] C c c b Origin choice 1 (C 2 2 -1ac)
      break;
    case 325: // [68] C c c b Origin choice 2 (-C 2a 2c)
      break;
    case 326: // [68] A b a a Origin choice 1 (A 2 2 -1ab)
      break;
    case 327: // [68] A b a a Origin choice 2 (-A 2a 2b)
      break;
    case 328: // [68] A c a a Origin choice 1 (A 2 2 -1ab)
      break;
    case 329: // [68] A c a a Origin choice 2 (-A 2ab 2b)
      break;
    case 330: // [68] B b c b Origin choice 1 (B 2 2 -1ab)
      break;
    case 331: // [68] B b c b Origin choice 2 (-B 2ab 2b)
      break;
    case 332: // [68] B b a b Origin choice 1 (B 2 2 -1ab)
      break;
    case 333: // [68] B b a b Origin choice 2 (-B 2b 2ab)
      break;

    case 334: // [69] F m m m (-F 2 2)
      break;

    case 335: // [70] F d d d:1 Origin choice 1 (F 2 2 -1d)
      break;
    case 336: // [70] F d d d:2 Origin choice 2 (-F 2uv 2vw)
      break;

    case 337: // [71] I m m m (-I 2 2)
      break;

    case 338: // [72] I b a m (-I 2 2c)
      break;
    case 339: // [72] I m c b (-I 2a 2)
      break;
    case 340: // [72] I c m a (-I 2b 2b)
      break;

    case 341: // [73] I b c a (-I 2b 2c)
      break;
    case 342: // [73] I c a b (-I 2a 2b)
      break;

    case 343: // [74] I m m a (-I 2b 2)
      break;
    case 344: // [74] I m m b (-I 2a 2a)
      break;
    case 345: // [74] I b m m (-I 2c 2c)
      break;
    case 346: // [74] I c m m (-I 2 2b)
      break;
    case 347: // [74] I m c m (-I 2 2a)
      break;
    case 348: // [74] I m a m (-I 2c 2)
      break;

    // TETRAGONAL GROUPS
    // =================

    case 349: // [75] P 4 (P 4)
      break;

    case 350: // [76] P 41 (P 4w)
      break;

    case 351: // [77] P 42 (P 4c)
      break;

    case 352: // [78] P 43 (P 4cw)
      break;

    case 353: // [79] I 4 (I 4)
      break;

    case 354: // [80] I 41 (I 4bw)
      break;

    case 355: // [81] P -4 (P -4)
      break;

    case 356: // [82] I -4 (I -4)
      break;

    case 357: // [83] P 4/m (-P 4)
      break;

    case 358: // [84] P 42/m (-P 4c)
      break;

    case 359: // [85] P 4/n Origin choice 1 (P 4ab -1ab)
      break;
    case 360: // [85] P 4/n Origin choice 2 (-P 4a)
      break;

    case 361: // [86] P 42/n Origin choice 1 (P 4n -1n)
      break;
    case 362: // [86] P 42/n Origin choice 2 (-P 4bc)
      break;

    case 363: // [87] I 4/m (-I 4)
      break;

    case 364: // [88] I 41/a Origin choice 1 (I 4bw -1bw)
      break;
    case 365: // [88] I 41/a Origin choice 2 (-I 4ad)
      break;

    case 366: // [89] P 4 2 2 (P 4 2)
      break;

    case 367: // [90] P 4 21 2 (P 4ab 2ab)
      break;

    case 368: // [91] P 41 2 2 (P 4w 2c)
      break;

    case 369: // [92] P 41 21 2 (P 4abw 2nw)
      break;

    case 370: // [93] P 42 2 2 (P 4c 2)
      break;

    case 371: // [94] P 42 21 2 (P 4n 2n)
      break;

    case 372: // [95] P 43 2 2 (P 4cw 2c)
      break;

    case 373: // [96] P 43 21 2 (P 4nw 2abw)
      break;

    case 374: // [97] I 4 2 2 (I 4 2)
      break;

    case 375: // [98] I 41 2 2 (I 4bw 2bw)
      break;

    case 376: // [99] P 4 m m (P 4 -2)
      break;

    case 377: // [100] P 4 b m (P 4 -2ab)
      break;

    case 378: // [101] P 42 c m (P 4c -2c)
      break;

    case 379: // [102] P 42 n m (P 4n -2n)
      break;

    case 380: // [103] P 4 c c (P 4 -2c)
      break;

    case 381: // [104] P 4 n c (P 4 -2n)
      break;

    case 382: // [105] P 42 m c (P 4c -2)
      break;

    case 383: // [106] P 42 b c (P 4c -2ab)
      break;

    case 384: // [107] I 4 m m (I 4 -2)
      break;

    case 385: // [108] I 4 c m (I 4 -2c)
      break;

    case 386: // [109] I 41 m d (I 4bw -2)
      break;

    case 387: // [110] I 41 c d (I 4bw -2c)
      break;

    case 388: // [111] P -4 2 m (P -4 2)
      break;

    case 389: // [112] P -4 2 c (P -4 2c)
      break;

    case 390: // [113] P -4 21 m (P -4 2ab)
      break;

    case 391: // [114] P -4 21 c (P -4 2n)
      break;

    case 392: // [115] P -4 m 2 (P -4 -2)
      break;

    case 393: // [116] P -4 c 2 (P -4 -2c)
      break;

    case 394: // [117] P -4 b 2 (P -4 -2ab)
      break;

    case 395: // [118] P -4 n 2 (P -4 -2n)
      break;

    case 396: // [119] I -4 m 2 (I -4 -2)
      break;

    case 397: // [120] I -4 c 2 (I -4 -2c)
      break;

    case 398: // [121] I -4 2 m (I -4 2)
      break;

    case 399: // [122] I -4 2 d (I -4 2bw)
      break;

    case 400: // [123] P 4/m m m (-P 4 2)
      break;

    case 401: // [124] P 4/m c c (-P 4 2c)
      break;

    case 402: // [125] P 4/n b m Origin choice 1 (P 4 2 -1ab)
      break;
    case 403: // [125] P 4/n b m Origin choice 2 (-P 4a 2b)
      break;

    case 404: // [126] P 4/n n c Origin choice 1 (P 4 2 -1n)
      break;
    case 405: // [126] P 4/n n c Origin choice 2 (-P 4a 2bc)
      break;

    case 406: // [127] P 4/m b m (-P 4 2ab)
      break;

    case 407: // [128] P 4/m n c (-P 4 2n)
      break;

    case 408: // [129] P 4/n m m Origin choice 1 (P 4ab 2ab -1ab)
      break;
    case 409: // [129] P 4/n m m Origin choice 2 (-P 4a 2a)
      break;

    case 410: // [130] P 4/n c c Origin choice 1 (P 4ab 2n -1ab)
      break;
    case 411: // [130] P 4/n c c Origin choice 2 (-P 4a 2ac)
      break;

    case 412: // [131] P 42/m m c (-P 4c 2)
      break;

    case 413: // [132] P 42/m c m (-P 4c 2c)
      break;

    case 414: // [133] P 42/n b c Origin choice 1 (P 4n 2c -1n)
      break;
    case 415: // [133] P 42/n b c Origin choice 2 (-P 4ac 2b)
      break;

    case 416: // [134] P 42/n n n Origin choice 1 (P 4n 2 -1n)
      break;
    case 417: // [134] P 42/n n n Origin choice 2 (-P 4ac 2bc)
      break;

    case 418: // [135] P 42/m b c (-P 4c 2ab)
      break;

    case 419: // [136] P 42/m n m (-P 4n 2n)
      break;

    case 420: // [137] P 42/n m c Origin choice 1 (P 4n 2n -1n)
      break;
    case 421: // [137] P 42/n m c Origin choice 2 (-P 4ac 2a)
      break;

    case 422: // [138] P 42/n c m Origin choice 1 (P 4n 2ab -1n)
      break;
    case 423: // [138] P 42/n c m Origin choice 2 (-P 4ac 2ac)
      break;

    case 424: // [139] I 4/m m m (-I 4 2)
      break;

    case 425: // [140] I 4/m c m (-I 4 2c)
      break;

    case 426: // [141] I 41/a m d Origin choice 1 (I 4bw 2bw -1bw)
      break;
    case 427: // [141] I 41/a m d Origin choice 2 (-I 4bd 2)
      break;

    case 428: // [142] I 41/a c d Origin choice 1 (I 4bw 2aw -1bw)
      break;
    case 429: // [142] I 41/a c d Origin choice 2 (-I 4bd 2c)
      break;


    // TRIGONAL GROUPS
    // ===============

    case 430: // [143] P 3 (P 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z'\n\t'-x+y,-x,z'\n");
      break;

    case 431: // [144] P 31 (P 31)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z+1/3'\n\t'-x+y,-x,z+2/3'\n");
      break;

    case 432: // [145] P 32 (P 32)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z+2/3'\n\t'-x+y,-x,z+1/3'\n");
      break;

    case 433: // [146] R 3 hexagonal axes (R 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z'\n\t'-x+y,-x,z'\n\t'x+2/3,y+1/3,z+1/3'\n\t'-y+2/3,x-y+1/3,z+1/3'\n\t'-x+y+2/3,-x+1/3,z+1/3'\n\t'x+1/3,y+2/3,z+2/3'\n"
        "\t'-y+1/3,x-y+2/3,z+2/3'\n\t'-x+y+1/3,-x+2/3,z+2/3'\n");
      break;
    case 434: // [146] R 3 Rhombohedral axes (P 3*)
      fprintf(Stream,"\t'x,y,z'\n\t'z,x,y'\n\t'y,z,x'\n");
      break;

    case 435: // [147] P -3 (P -3)
      fprintf(Stream,"\t'x,y,z'\n\t'y,-x+y,-z'\n\t'-x+y,-x,z'\n\t'-x,-y,-z'\n\t'-y,x-y,z'\n\t'x-y,x,-z'\n");
      break;

    case 436: // [148] R-3 hexagonal axes (-R 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z'\n\t'-x+y,-x,z'\n\t'-x,-y,-z'\n\t'y,-x+y,-z'\n\t'x-y,x,-z'\n\t'x+2/3,y+1/3,z+1/3'\n"
        "\t'-y+2/3,x-y+1/3,z+1/3'\n\t'-x+y+2/3,-x+1/3,z+1/3'\n\t'-x+2/3,-y+1/3,-z+1/3'\n\t'y+2/3,-x+y+1/3,-z+1/3'\n\t'x-y+2/3,x+1/3,-z+1/3'\n\t'x+1/3,y+2/3,z+2/3'\n\t'-y+1/3,x-y+2/3,z+2/3'\n"
        "\t'-x+y+1/3,-x+2/3,z+2/3'\n\t'-x+1/3,-y+2/3,-z+2/3'\n\t'y+1/3,-x+y+2/3,-z+2/3'\n\t'x-y+1/3,x+2/3,-z+2/3'\n");
      break;
    case 437: // [148] R -3 Rhombohedral axes (-P 3*)
      fprintf(Stream,"\t'x,y,z'\n\t'z,x,y'\n\t'y,z,x'\n\t'-x,-y,-z'\n\t'-z,-x,-y'\n\t'-y,-z,-x'\n");
      break;

    case 438: // [149] P 3 1 2 (P 3 2)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z'\n\t'-x+y,-x,z'\n\t'-y,-x,-z'\n\t'x,x-y,-z'\n\t'-x+y,y,-z'\n");
      break;

    case 439: // [150] P 3 2 1 (P 3 2")
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z'\n\t'-x+y,-x,z'\n\t'y,x,-z'\n\t'-x,-x+y,-z'\n\t'x-y,-y,-z'\n");
      break;

    case 440: // [151] P 31 1 2 (P 31 2 (0 0 4))
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z+1/3'\n\t'-x+y,-x,z+2/3'\n\t'-y,-x,-z+2/3'\n\t'x,x-y,-z'\n\t'-x+y,y,-z+1/3'\n");
      break;

    case 441: // [152] P 31 2 1 (P 31 2")
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z+1/3'\n\t'-x+y,-x,z+2/3'\n\t'y,x,-z'\n\t'-x,-x+y,-z+1/3'\n\t'x-y,-y,-z+2/3'\n");
      break;

    case 442: // [153] P 32 1 2 (P 32 2 (0 0 2))
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z+2/3'\n\t'-x+y,-x,z+1/3'\n\t'-y,-x,-z+1/3'\n\t'x,x-y,-z'\n\t'-x+y,y,-z+2/3'\n");
      break;

    case 443: // [154] P 32 2 1 (P 32 2")
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z+2/3'\n\t'-x+y,-x,z+1/3'\n\t'y,x,-z'\n\t'-x,-x+y,-z+2/3'\n\t'x-y,-y,-z+1/3'\n");
      break;

    case 444: // [155] R 3 2 Hexagonal axes (R 3 2")
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z'\n\t'-x+y,-x,z'\n\t'y,x,-z'\n\t'-x,-x+y,-z'\n\t'x-y,-y,-z'\n\t'x+2/3,y+1/3,z+1/3'\n"
        "\t'-y+2/3,x-y+1/3,z+1/3'\n\t'-x+y+2/3,-x+1/3,z+1/3'\n\t'y+2/3,x+1/3,-z+1/3'\n\t'-x+2/3,-x+y+1/3,-z+1/3'\n\t'x-y+2/3,-y+1/3,-z+1/3'\n\t'x+1/3,y+2/3,z+2/3'\n\t'-y+1/3,x-y+2/3,z+2/3'\n"
        "\t'-x+y+1/3,-x+2/3,z+2/3'\n\t'y+1/3,x+2/3,-z+2/3'\n\t'-x+1/3,-x+y+2/3,-z+2/3'\n\t'x-y+1/3,-y+2/3,-z+2/3'\n");
      break;
    case 445: // [155] R 3 2 Rhombohedral axes (P 3* 2)
      fprintf(Stream,"\t'x,y,z'\n\t'z,x,y'\n\t'y,z,x'\n\t'-y,-x,-z'\n\t'-z,-y,-x'\n\t'-x,-z,-y'\n");
      break;

    case 446: // [156] P 3 m 1 (P 3 -2")
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z'\n\t'-x+y,-x,z'\n\t'-y,-x,z'\n\t'x,x-y,z'\n\t'-x+y,y,z'\n");
      break;

    case 447: // [157] P 3 1 m (P 3 -2)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z'\n\t'-x+y,-x,z'\n\t'y,x,z'\n\t'-x,-x+y,z'\n\t'x-y,-y,z'\n");
      break;

    case 448: // [158] P 3 c 1 (P 3 -2"c)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z'\n\t'-x+y,-x,z'\n\t'-y,-x,z+1/2'\n\t'x,x-y,z+1/2'\n\t'-x+y,y,z+1/2'\n");
      break;

    case 449: // [159] P 3 1 c (P 3 -2c)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z'\n\t'-x+y,-x,z'\n\t'y,x,z+1/2'\n\t'-x,-x+y,z+1/2'\n\t'x-y,-y,z+1/2'\n");
      break;

    case 450: // [160] R 3 m Hexagonal axes (R 3 -2")
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z'\n\t'-x+y,-x,z'\n\t'-y,-x,z'\n\t'x,x-y,z'\n\t'-x+y,y,z'\n\t'x+2/3,y+1/3,z+1/3'\n"
        "\t'-y+2/3,x-y+1/3,z+1/3'\n\t'-x+y+2/3,-x+1/3,z+1/3'\n\t'-y+2/3,-x+1/3,z+1/3'\n\t'x+2/3,x-y+1/3,z+1/3'\n\t'-x+y+2/3,y+1/3,z+1/3'\n\t'x+1/3,y+2/3,z+2/3'\n\t'-y+1/3,x-y+2/3,z+2/3'\n"
        "\t'-x+y+1/3,-x+2/3,z+2/3'\n\t'-y+1/3,-x+2/3,z+2/3'\n\t'x+1/3,x-y+2/3,z+2/3'\n\t'-x+y+1/3,y+2/3,z+2/3'\n");
      break;
    case 451: // [160] R 3 m Rhombohedral axes (P 3* -2)
      fprintf(Stream,"\t'x,y,z'\n\t'z,x,y'\n\t'y,z,x'\n\t'y,x,z'\n\t'z,y,x'\n\t'x,z,y'\n");
      break;

    case 452: // [161] R 3 c Hexagonal axes (R 3 -2"c)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z'\n\t'-x+y,-x,z'\n\t'-y,-x,z+1/2'\n\t'x,x-y,z+1/2'\n\t'-x+y,y,z+1/2'\n\t'x+2/3,y+1/3,z+1/3'\n"
        "\t'-y+2/3,x-y+1/3,z+1/3'\n\t'-x+y+2/3,-x+1/3,z+1/3'\n\t'-y+2/3,-x+1/3,z+5/6'\n\t'x+2/3,x-y+1/3,z+5/6'\n\t'-x+y+2/3,y+1/3,z+5/6'\n\t'x+1/3,y+2/3,z+2/3'\n\t'-y+1/3,x-y+2/3,z+2/3'\n"
        "\t'-x+y+1/3,-x+2/3,z+2/3'\n\t'-y+1/3,-x+2/3,z+7/6'\n\t'x+1/3,x-y+2/3,z+7/6'\n\t'-x+y+1/3,y+2/3,z+7/6'\n");
      break;
    case 453: // [161] R 3 c Rhombohedral axes (P 3* -2n)
      fprintf(Stream,"\t'x,y,z'\n\t'z,x,y'\n\t'y,z,x'\n\t'y+1/2,x+1/2,z+1/2'\n\t'z+1/2,y+1/2,x+1/2'\n\t'x+1/2,z+1/2,y+1/2'\n");
      break;

    case 454: // [162] P -3 1 m (-P 3 2)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z'\n\t'-x+y,-x,z'\n\t'-y,-x,-z'\n\t'x,x-y,-z'\n\t'-x+y,y,-z'\n\t'-x,-y,-z'\n"
        "\t'y,-x+y,-z'\n\t'x-y,x,-z'\n\t'y,x,z'\n\t'-x,-x+y,z'\n\t'x-y,-y,z'\n");
      break;

    case 455: // [163] P -3 1 c (-P 3 2c)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z'\n\t'-x+y,-x,z'\n\t'-y,-x,-z+1/2'\n\t'x,x-y,-z+1/2'\n\t'-x+y,y,-z+1/2'\n\t'-x,-y,-z'\n"
        "\t'y,-x+y,-z'\n\t'x-y,x,-z'\n\t'y,x,z-1/2'\n\t'-x,-x+y,z-1/2'\n\t'x-y,-y,z-1/2'\n");
      break;

    case 456: // [164] P -3 m 1 (-P 3 2")
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z'\n\t'-x+y,-x,z'\n\t'y,x,-z'\n\t'-x,-x+y,-z'\n\t'x-y,-y,-z'\n\t'-x,-y,-z'\n"
        "\t'y,-x+y,-z'\n\t'x-y,x,-z'\n\t'-y,-x,z'\n\t'x,x-y,z'\n\t'-x+y,y,z'\n");
      break;

    case 457: // [165] P -3 c 1 (-P 3 2"c)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z'\n\t'-x+y,-x,z'\n\t'y,x,-z+1/2'\n\t'-x,-x+y,-z+1/2'\n\t'x-y,-y,-z+1/2'\n\t'-x,-y,-z'\n"
        "\t'y,-x+y,-z'\n\t'x-y,x,-z'\n\t'-y,-x,z-1/2'\n\t'x,x-y,z-1/2'\n\t'-x+y,y,z-1/2'\n");
      break;

    case 458: // [166] R -3 m Hexagonal axes (-R 3 2")
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z'\n\t'-x+y,-x,z'\n\t'y,x,-z'\n\t'-x,-x+y,-z'\n\t'x-y,-y,-z'\n\t'-x,-y,-z'\n"
        "\t'y,-x+y,-z'\n\t'x-y,x,-z'\n\t'-y,-x,z'\n\t'x,x-y,z'\n\t'-x+y,y,z'\n\t'x+2/3,y+1/3,z+1/3'\n\t'-y+2/3,x-y+1/3,z+1/3'\n"
        "\t'-x+y+2/3,-x+1/3,z+1/3'\n\t'y+2/3,x+1/3,-z+1/3'\n\t'-x+2/3,-x+y+1/3,-z+1/3'\n\t'x-y+2/3,-y+1/3,-z+1/3'\n\t'-x+2/3,-y+1/3,-z+1/3'\n\t'y+2/3,-x+y+1/3,-z+1/3'\n\t'x-y+2/3,x+1/3,-z+1/3'\n"
        "\t'-y+2/3,-x+1/3,z+1/3'\n\t'x+2/3,x-y+1/3,z+1/3'\n\t'-x+y+2/3,y+1/3,z+1/3'\n\t'x+1/3,y+2/3,z+2/3'\n\t'-y+1/3,x-y+2/3,z+2/3'\n\t'-x+y+1/3,-x+2/3,z+2/3'\n\t'y+1/3,x+2/3,-z+2/3'\n"
        "\t'-x+1/3,-x+y+2/3,-z+2/3'\n\t'x-y+1/3,-y+2/3,-z+2/3'\n\t'-x+1/3,-y+2/3,-z+2/3'\n\t'y+1/3,-x+y+2/3,-z+2/3'\n\t'x-y+1/3,x+2/3,-z+2/3'\n\t'-y+1/3,-x+2/3,z+2/3'\n\t'x+1/3,x-y+2/3,z+2/3'\n"
        "\t'-x+y+1/3,y+2/3,z+2/3'\n");
      break;
    case 459: // [166] R -3 m Rhombohedral axes (-P 3* 2)
      fprintf(Stream,"\t'x,y,z'\n\t'z,x,y'\n\t'y,z,x'\n\t'-y,-x,-z'\n\t'-z,-y,-x'\n\t'-x,-z,-y'\n\t'-x,-y,-z'\n"
        "\t'-z,-x,-y'\n\t'-y,-z,-x'\n\t'y,x,z'\n\t'z,y,x'\n\t'x,z,y'\n");
      break;

    case 460: // [167] R -3 c Hexagonal axes (-R 3 2"c)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x-y,z'\n\t'-x+y,-x,z'\n\t'y,x,-z+1/2'\n\t'-x,-x+y,-z+1/2'\n\t'x-y,-y,-z+1/2'\n\t'-x,-y,-z'\n"
        "\t'y,-x+y,-z'\n\t'x-y,x,-z'\n\t'-y,-x,z-1/2'\n\t'x,x-y,z-1/2'\n\t'-x+y,y,z-1/2'\n\t'x+2/3,y+1/3,z+1/3'\n\t'-y+2/3,x-y+1/3,z+1/3'\n"
        "\t'-x+y+2/3,-x+1/3,z+1/3'\n\t'y+2/3,x+1/3,-z+5/6'\n\t'-x+2/3,-x+y+1/3,-z+5/6'\n\t'x-y+2/3,-y+1/3,-z+5/6'\n\t'-x+2/3,-y+1/3,-z+1/3'\n\t'y+2/3,-x+y+1/3,-z+1/3'\n\t'x-y+2/3,x+1/3,-z+1/3'\n"
        "\t'-y+2/3,-x+1/3,z-1/6'\n\t'x+2/3,x-y+1/3,z-1/6'\n\t'-x+y+2/3,y+1/3,z-1/6'\n\t'x+1/3,y+2/3,z+2/3'\n\t'-y+1/3,x-y+2/3,z+2/3'\n\t'-x+y+1/3,-x+2/3,z+2/3'\n\t'y+1/3,x+2/3,-z+7/6'\n"
        "\t'-x+1/3,-x+y+2/3,-z+7/6'\n\t'x-y+1/3,-y+2/3,-z+7/6'\n\t'-x+1/3,-y+2/3,-z+2/3'\n\t'y+1/3,-x+y+2/3,-z+2/3'\n\t'x-y+1/3,x+2/3,-z+2/3'\n\t'-y+1/3,-x+2/3,z+1/6'\n\t'x+1/3,x-y+2/3,z+1/6'\n"
        "\t'-x+y+1/3,y+2/3,z+1/6'\n");
      break;
    case 461: // [167] R -3 c Rhombohedral axes (-P 3* 2n)
      fprintf(Stream,"\t'x,y,z'\n\t'z,x,y'\n\t'y,z,x'\n\t'-y+1/2,-x+1/2,-z+1/2'\n\t'-z+1/2,-y+1/2,-x+1/2'\n\t'-x+1/2,-z+1/2,-y+1/2'\n\t'-x,-y,-z'\n"
        "\t'-z,-x,-y'\n\t'-y,-z,-x'\n\t'y-1/2,x-1/2,z-1/2'\n\t'z-1/2,y-1/2,x-1/2'\n\t'x-1/2,z-1/2,y-1/2'\n");
      break;


    // HEXAGONAL GROUPS
    // ================

    case 462: // [168] P 6 (P 6)
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z'\n\t'-y,x-y,z'\n\t'-x,-y,z'\n\t'-x+y,-x,z'\n\t'y,-x+y,z'\n");
      break;

    case 463: // [169] P 61 (P 61)
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z+1/6'\n\t'-y,x-y,z+1/3'\n\t'-x,-y,z+1/2'\n\t'-x+y,-x,z+2/3'\n\t'y,-x+y,z+5/6'\n");
      break;

    case 464: // [170] P 65 (P 65)
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z+5/6'\n\t'-y,x-y,z+2/3'\n\t'-x,-y,z+1/2'\n\t'-x+y,-x,z+1/3'\n\t'y,-x+y,z+1/6'\n");
      break;

    case 465: // [171] P 62 (P 62)
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z+1/3'\n\t'-y,x-y,z+2/3'\n\t'-x,-y,z'\n\t'-x+y,-x,z+1/3'\n\t'y,-x+y,z+2/3'\n");
      break;

    case 466: // [172] P 64 (P 64)
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z+2/3'\n\t'-y,x-y,z+1/3'\n\t'-x,-y,z'\n\t'-x+y,-x,z+2/3'\n\t'y,-x+y,z+1/3'\n");
      break;

    case 467: // [173] P 63 (P 6c)
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z+1/2'\n\t'-y,x-y,z'\n\t'-x,-y,z+1/2'\n\t'-x+y,-x,z'\n\t'y,-x+y,z+1/2'\n");
      break;

    case 468: // [174] P -6 (P -6)
      fprintf(Stream,"\t'x,y,z'\n\t'-x+y,-x,-z'\n\t'-y,x-y,z'\n\t'x,y,-z'\n\t'-x+y,-x,z'\n\t'-y,x-y,-z'\n");
      break;

    case 469: // [175] P6/m (-P 6)
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z'\n\t'-y,x-y,z'\n\t'-x,-y,z'\n\t'-x+y,-x,z'\n\t'y,-x+y,z'\n\t'-x,-y,-z'\n"
        "\t'-x+y,-x,-z'\n\t'y,-x+y,-z'\n\t'x,y,-z'\n\t'x-y,x,-z'\n\t'-y,x-y,-z'\n");
      break;

    case 470: // [176] P 63/m (-P 6c)
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z+1/2'\n\t'-y,x-y,z'\n\t'-x,-y,z+1/2'\n\t'-x+y,-x,z'\n\t'y,-x+y,z+1/2'\n\t'-x,-y,-z'\n"
        "\t'-x+y,-x,-z-1/2'\n\t'y,-x+y,-z'\n\t'x,y,-z-1/2'\n\t'x-y,x,-z'\n\t'-y,x-y,-z-1/2'\n");
      break;

    case 471: // [177] P 6 2 2 (P 6 2)
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z'\n\t'-y,x-y,z'\n\t'-x,-y,z'\n\t'-x+y,-x,z'\n\t'y,-x+y,z'\n\t'-y,-x,-z'\n"
        "\t'x-y,-y,-z'\n\t'x,x-y,-z'\n\t'y,x,-z'\n\t'-x+y,y,-z'\n\t'-x,-x+y,-z'\n");
      break;

    case 472: // [178] P 61 2 2 (P 61 2 (0 0 5))
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z+1/6'\n\t'-y,x-y,z+1/3'\n\t'-x,-y,z+1/2'\n\t'-x+y,-x,z+2/3'\n\t'y,-x+y,z+5/6'\n\t'-y,-x,-z+5/6'\n"
        "\t'x-y,-y,-z'\n\t'x,x-y,-z+1/6'\n\t'y,x,-z+1/3'\n\t'-x+y,y,-z+1/2'\n\t'-x,-x+y,-z+2/3'\n");
      break;

    case 473: // [179] P 65 2 2 (P 65 2 (0 0 1))
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z+5/6'\n\t'-y,x-y,z+2/3'\n\t'-x,-y,z+1/2'\n\t'-x+y,-x,z+1/3'\n\t'y,-x+y,z+1/6'\n\t'-y,-x,-z+1/6'\n"
        "\t'x-y,-y,-z'\n\t'x,x-y,-z+5/6'\n\t'y,x,-z+2/3'\n\t'-x+y,y,-z+1/2'\n\t'-x,-x+y,-z+1/3'\n");
      break;

    case 474: // [180] P 62 2 2 (P 62 2 (0 0 4))
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z+1/3'\n\t'-y,x-y,z+2/3'\n\t'-x,-y,z'\n\t'-x+y,-x,z+1/3'\n\t'y,-x+y,z+2/3'\n\t'-y,-x,-z+2/3'\n"
        "\t'x-y,-y,-z'\n\t'x,x-y,-z+1/3'\n\t'y,x,-z+2/3'\n\t'-x+y,y,-z'\n\t'-x,-x+y,-z+1/3'\n");
      break;

    case 475: // [181] P 64 2 2 (P 64 2 (0 0 2))
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z+2/3'\n\t'-y,x-y,z+1/3'\n\t'-x,-y,z'\n\t'-x+y,-x,z+2/3'\n\t'y,-x+y,z+1/3'\n\t'-y,-x,-z+1/3'\n"
        "\t'x-y,-y,-z'\n\t'x,x-y,-z+2/3'\n\t'y,x,-z+1/3'\n\t'-x+y,y,-z'\n\t'-x,-x+y,-z+2/3'\n");
      break;

    case 476: // [182] P 63 2 2 (P 6c 2c)
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z+1/2'\n\t'-y,x-y,z'\n\t'-x,-y,z+1/2'\n\t'-x+y,-x,z'\n\t'y,-x+y,z+1/2'\n\t'-y,-x,-z+1/2'\n"
        "\t'x-y,-y,-z'\n\t'x,x-y,-z+1/2'\n\t'y,x,-z'\n\t'-x+y,y,-z+1/2'\n\t'-x,-x+y,-z'\n");
      break;

    case 477: // [183] P 6 m m (P 6 -2)
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z'\n\t'-y,x-y,z'\n\t'-x,-y,z'\n\t'-x+y,-x,z'\n\t'y,-x+y,z'\n\t'y,x,z'\n"
        "\t'-x+y,y,z'\n\t'-x,-x+y,z'\n\t'-y,-x,z'\n\t'x-y,-y,z'\n\t'x,x-y,z'\n");
      break;

    case 478: // [184] P 6 c c (P 6 -2c)
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z'\n\t'-y,x-y,z'\n\t'-x,-y,z'\n\t'-x+y,-x,z'\n\t'y,-x+y,z'\n\t'y,x,z+1/2'\n"
        "\t'-x+y,y,z+1/2'\n\t'-x,-x+y,z+1/2'\n\t'-y,-x,z+1/2'\n\t'x-y,-y,z+1/2'\n\t'x,x-y,z+1/2'\n");
      break;

    case 479: // [185] P 63 c m (P 6c -2)
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z+1/2'\n\t'-y,x-y,z'\n\t'-x,-y,z+1/2'\n\t'-x+y,-x,z'\n\t'y,-x+y,z+1/2'\n\t'y,x,z'\n"
        "\t'-x+y,y,z+1/2'\n\t'-x,-x+y,z'\n\t'-y,-x,z+1/2'\n\t'x-y,-y,z'\n\t'x,x-y,z+1/2'\n");
      break;

    case 480: // [186] P 63 m c (P 6c -2c)
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z+1/2'\n\t'-y,x-y,z'\n\t'-x,-y,z+1/2'\n\t'-x+y,-x,z'\n\t'y,-x+y,z+1/2'\n\t'y,x,z+1/2'\n"
        "\t'-x+y,y,z'\n\t'-x,-x+y,z+1/2'\n\t'-y,-x,z'\n\t'x-y,-y,z+1/2'\n\t'x,x-y,z'\n");
      break;

    case 481: // [187] P -6 m 2 (P -6 2)
      fprintf(Stream,"\t'x,y,z'\n\t'-x+y,-x,-z'\n\t'-y,x-y,z'\n\t'x,y,-z'\n\t'-x+y,-x,z'\n\t'-y,x-y,-z'\n\t'-y,-x,-z'\n"
        "\t'-x+y,y,z'\n\t'x,x-y,-z'\n\t'-y,-x,z'\n\t'-x+y,y,-z'\n\t'x,x-y,z'\n");
      break;

    case 482: // [188] P -6 c 2 (P -6c 2)
      fprintf(Stream,"\t'x,y,z'\n\t'-x+y,-x,-z+1/2'\n\t'-y,x-y,z'\n\t'x,y,-z+1/2'\n\t'-x+y,-x,z'\n\t'-y,x-y,-z+1/2'\n\t'-y,-x,-z'\n"
        "\t'-x+y,y,z+1/2'\n\t'x,x-y,-z'\n\t'-y,-x,z+1/2'\n\t'-x+y,y,-z'\n\t'x,x-y,z+1/2'\n");
      break;

    case 483: // [189] P -6 2 m (P -6 -2)
      fprintf(Stream,"\t'x,y,z'\n\t'-x+y,-x,-z'\n\t'-y,x-y,z'\n\t'x,y,-z'\n\t'-x+y,-x,z'\n\t'-y,x-y,-z'\n\t'y,x,z'\n"
        "\t'x-y,-y,-z'\n\t'-x,-x+y,z'\n\t'y,x,-z'\n\t'x-y,-y,z'\n\t'-x,-x+y,-z'\n");
      break;

    case 484: // [190] P -6 2 c (P -6c -2c)
      fprintf(Stream,"\t'x,y,z'\n\t'-x+y,-x,-z+1/2'\n\t'-y,x-y,z'\n\t'x,y,-z+1/2'\n\t'-x+y,-x,z'\n\t'-y,x-y,-z+1/2'\n\t'y,x,z+1/2'\n"
        "\t'x-y,-y,-z'\n\t'-x,-x+y,z+1/2'\n\t'y,x,-z'\n\t'x-y,-y,z+1/2'\n\t'-x,-x+y,-z'\n");
      break;

    case 485: // [191] P 6/m m m (-P 6 2)
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z'\n\t'-y,x-y,z'\n\t'-x,-y,z'\n\t'-x+y,-x,z'\n\t'y,-x+y,z'\n\t'-y,-x,-z'\n"
        "\t'x-y,-y,-z'\n\t'x,x-y,-z'\n\t'y,x,-z'\n\t'-x+y,y,-z'\n\t'-x,-x+y,-z'\n\t'-x,-y,-z'\n\t'-x+y,-x,-z'\n"
        "\t'y,-x+y,-z'\n\t'x,y,-z'\n\t'x-y,x,-z'\n\t'-y,x-y,-z'\n\t'y,x,z'\n\t'-x+y,y,z'\n\t'-x,-x+y,z'\n"
        "\t'-y,-x,z'\n\t'x-y,-y,z'\n\t'x,x-y,z'\n");
      break;

    case 486: // [192] P 6/m c c (-P 6 2c)
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z'\n\t'-y,x-y,z'\n\t'-x,-y,z'\n\t'-x+y,-x,z'\n\t'y,-x+y,z'\n\t'-y,-x,-z+1/2'\n"
        "\t'x-y,-y,-z+1/2'\n\t'x,x-y,-z+1/2'\n\t'y,x,-z+1/2'\n\t'-x+y,y,-z+1/2'\n\t'-x,-x+y,-z+1/2'\n\t'-x,-y,-z'\n\t'-x+y,-x,-z'\n"
        "\t'y,-x+y,-z'\n\t'x,y,-z'\n\t'x-y,x,-z'\n\t'-y,x-y,-z'\n\t'y,x,z-1/2'\n\t'-x+y,y,z-1/2'\n\t'-x,-x+y,z-1/2'\n"
        "\t'-y,-x,z-1/2'\n\t'x-y,-y,z-1/2'\n\t'x,x-y,z-1/2'\n");
      break;

    case 487: // [193] P 63/m c m (-P 6c 2)
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z+1/2'\n\t'-y,x-y,z'\n\t'-x,-y,z+1/2'\n\t'-x+y,-x,z'\n\t'y,-x+y,z+1/2'\n\t'-y,-x,-z'\n"
        "\t'x-y,-y,-z+1/2'\n\t'x,x-y,-z'\n\t'y,x,-z+1/2'\n\t'-x+y,y,-z'\n\t'-x,-x+y,-z+1/2'\n\t'-x,-y,-z'\n\t'-x+y,-x,-z-1/2'\n"
        "\t'y,-x+y,-z'\n\t'x,y,-z-1/2'\n\t'x-y,x,-z'\n\t'-y,x-y,-z-1/2'\n\t'y,x,z'\n\t'-x+y,y,z-1/2'\n\t'-x,-x+y,z'\n"
        "\t'-y,-x,z-1/2'\n\t'x-y,-y,z'\n\t'x,x-y,z-1/2'\n");
      break;

    case 488: // [194] P 63/m m c (-P 6c 2c)
      fprintf(Stream,"\t'x,y,z'\n\t'x-y,x,z+1/2'\n\t'-y,x-y,z'\n\t'-x,-y,z+1/2'\n\t'-x+y,-x,z'\n\t'y,-x+y,z+1/2'\n\t'-y,-x,-z+1/2'\n"
        "\t'x-y,-y,-z'\n\t'x,x-y,-z+1/2'\n\t'y,x,-z'\n\t'-x+y,y,-z+1/2'\n\t'-x,-x+y,-z'\n\t'-x,-y,-z'\n\t'-x+y,-x,-z-1/2'\n"
        "\t'y,-x+y,-z'\n\t'x,y,-z-1/2'\n\t'x-y,x,-z'\n\t'-y,x-y,-z-1/2'\n\t'y,x,z-1/2'\n\t'-x+y,y,z'\n\t'-x,-x+y,z-1/2'\n"
        "\t'-y,-x,z'\n\t'x-y,-y,z-1/2'\n\t'x,x-y,z'\n");
      break;

    // CUBIC GROUPS
    // ============

    case 489: // [195] P 2 3 (P 2 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-x,-y,z'\n\t'x,-y,-z'\n\t'-x,y,-z'\n\t'z,x,y'\n\t'-z,-x,y'\n\t'z,-x,-y'\n"
        "\t'-z,x,-y'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'-y,z,-x'\n\t'-y,-z,x'\n");
      break;

    case 490: // [196] F 2 3 (F 2 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-x,-y,z'\n\t'x,-y,-z'\n\t'-x,y,-z'\n\t'z,x,y'\n\t'-z,-x,y'\n\t'z,-x,-y'\n"
        "\t'-z,x,-y'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'-y,z,-x'\n\t'-y,-z,x'\n\t'x,y+1/2,z+1/2'\n\t'-x,-y+1/2,z+1/2'\n"
        "\t'x,-y+1/2,-z+1/2'\n\t'-x,y+1/2,-z+1/2'\n\t'z,x+1/2,y+1/2'\n\t'-z,-x+1/2,y+1/2'\n\t'z,-x+1/2,-y+1/2'\n\t'-z,x+1/2,-y+1/2'\n\t'y,z+1/2,x+1/2'\n"
        "\t'y,-z+1/2,-x+1/2'\n\t'-y,z+1/2,-x+1/2'\n\t'-y,-z+1/2,x+1/2'\n\t'x+1/2,y,z+1/2'\n\t'-x+1/2,-y,z+1/2'\n\t'x+1/2,-y,-z+1/2'\n\t'-x+1/2,y,-z+1/2'\n"
        "\t'z+1/2,x,y+1/2'\n\t'-z+1/2,-x,y+1/2'\n\t'z+1/2,-x,-y+1/2'\n\t'-z+1/2,x,-y+1/2'\n\t'y+1/2,z,x+1/2'\n\t'y+1/2,-z,-x+1/2'\n\t'-y+1/2,z,-x+1/2'\n"
        "\t'-y+1/2,-z,x+1/2'\n\t'x+1/2,y+1/2,z'\n\t'-x+1/2,-y+1/2,z'\n\t'x+1/2,-y+1/2,-z'\n\t'-x+1/2,y+1/2,-z'\n\t'z+1/2,x+1/2,y'\n\t'-z+1/2,-x+1/2,y'\n"
        "\t'z+1/2,-x+1/2,-y'\n\t'-z+1/2,x+1/2,-y'\n\t'y+1/2,z+1/2,x'\n\t'y+1/2,-z+1/2,-x'\n\t'-y+1/2,z+1/2,-x'\n\t'-y+1/2,-z+1/2,x'\n");
      break;

    case 491: // [197] I 2 3 (I 2 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-x,-y,z'\n\t'x,-y,-z'\n\t'-x,y,-z'\n\t'z,x,y'\n\t'-z,-x,y'\n\t'z,-x,-y'\n"
        "\t'-z,x,-y'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'-y,z,-x'\n\t'-y,-z,x'\n\t'x+1/2,y+1/2,z+1/2'\n\t'-x+1/2,-y+1/2,z+1/2'\n"
        "\t'x+1/2,-y+1/2,-z+1/2'\n\t'-x+1/2,y+1/2,-z+1/2'\n\t'z+1/2,x+1/2,y+1/2'\n\t'-z+1/2,-x+1/2,y+1/2'\n\t'z+1/2,-x+1/2,-y+1/2'\n\t'-z+1/2,x+1/2,-y+1/2'\n\t'y+1/2,z+1/2,x+1/2'\n"
        "\t'y+1/2,-z+1/2,-x+1/2'\n\t'-y+1/2,z+1/2,-x+1/2'\n\t'-y+1/2,-z+1/2,x+1/2'\n");
      break;

    case 492: // [198] P 21 3 (P 2ac 2ab 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-x+1/2,-y,z+1/2'\n\t'x+1/2,-y+1/2,-z'\n\t'-x,y+1/2,-z+1/2'\n\t'z,x,y'\n\t'-z+1/2,-x,y+1/2'\n\t'z+1/2,-x+1/2,-y'\n"
        "\t'-z,x+1/2,-y+1/2'\n\t'y,z,x'\n\t'y+1/2,-z+1/2,-x'\n\t'-y,z+1/2,-x+1/2'\n\t'-y+1/2,-z,x+1/2'\n");
      break;

    case 493: // [199] I 21 3 (I 2b 2c 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-x,-y+1/2,z'\n\t'x,-y,-z+1/2'\n\t'-x,y+1/2,-z+1/2'\n\t'z,x,y'\n\t'-z,-x+1/2,y'\n\t'z,-x,-y+1/2'\n"
        "\t'-z,x+1/2,-y+1/2'\n\t'y,z,x'\n\t'y,-z,-x+1/2'\n\t'-y,z+1/2,-x+1/2'\n\t'-y+1/2,-z,x+1/2'\n\t'x+1/2,y+1/2,z+1/2'\n\t'-x+1/2,-y+1,z+1/2'\n"
        "\t'x+1/2,-y+1/2,-z+1'\n\t'-x+1/2,y+1,-z+1'\n\t'z+1/2,x+1/2,y+1/2'\n\t'-z+1/2,-x+1,y+1/2'\n\t'z+1/2,-x+1/2,-y+1'\n\t'-z+1/2,x+1,-y+1'\n\t'y+1/2,z+1/2,x+1/2'\n"
        "\t'y+1/2,-z+1/2,-x+1'\n\t'-y+1/2,z+1,-x+1'\n\t'-y+1,-z+1/2,x+1'\n");
      break;

    case 494: // [200] P m -3 (-P 2 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-x,-y,z'\n\t'x,-y,-z'\n\t'-x,y,-z'\n\t'z,x,y'\n\t'-z,-x,y'\n\t'z,-x,-y'\n"
      "\t'-z,x,-y'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'-y,z,-x'\n\t'-y,-z,x'\n\t'-x,-y,-z'\n\t'x,y,-z'\n"
      "\t'-x,y,z'\n\t'x,-y,z'\n\t'-z,-x,-y'\n\t'z,x,-y'\n\t'-z,x,y'\n\t'z,-x,y'\n\t'-y,-z,-x'\n"
      "\t'-y,z,x'\n\t'y,-z,x'\n\t'y,z,-x'\n");
      break;

    case 495: // [201] P n -3 Origin choice 1 (P 2 2 3 -1n)
      fprintf(Stream,"\t'x,y,z'\n\t'-x,-y,z'\n\t'x,-y,-z'\n\t'-x,y,-z'\n\t'z,x,y'\n\t'-z,-x,y'\n\t'z,-x,-y'\n"
        "\t'-z,x,-y'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'-y,z,-x'\n\t'-y,-z,x'\n\t'-x+1/2,-y+1/2,-z+1/2'\n\t'x+1/2,y+1/2,-z+1/2'\n"
        "\t'-x+1/2,y+1/2,z+1/2'\n\t'x+1/2,-y+1/2,z+1/2'\n\t'-z+1/2,-x+1/2,-y+1/2'\n\t'z+1/2,x+1/2,-y+1/2'\n\t'-z+1/2,x+1/2,y+1/2'\n\t'z+1/2,-x+1/2,y+1/2'\n\t'-y+1/2,-z+1/2,-x+1/2'\n"
        "\t'-y+1/2,z+1/2,x+1/2'\n\t'y+1/2,-z+1/2,x+1/2'\n\t'y+1/2,z+1/2,-x+1/2'\n");
      break;
    case 496: // [201] P n -3 Origin choice 2 (-P 2ab 2bc 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-x+1/2,-y+1/2,z'\n\t'x,-y+1/2,-z+1/2'\n\t'-x+1/2,y,-z+1/2'\n\t'z,x,y'\n\t'-z+1/2,-x+1/2,y'\n\t'z,-x+1/2,-y+1/2'\n"
        "\t'-z+1/2,x,-y+1/2'\n\t'y,z,x'\n\t'y,-z+1/2,-x+1/2'\n\t'-y+1/2,z,-x+1/2'\n\t'-y+1/2,-z+1/2,x'\n\t'-x,-y,-z'\n\t'x-1/2,y-1/2,-z'\n"
        "\t'-x,y-1/2,z-1/2'\n\t'x-1/2,-y,z-1/2'\n\t'-z,-x,-y'\n\t'z-1/2,x-1/2,-y'\n\t'-z,x-1/2,y-1/2'\n\t'z-1/2,-x,y-1/2'\n\t'-y,-z,-x'\n"
        "\t'-y,z-1/2,x-1/2'\n\t'y-1/2,-z,x-1/2'\n\t'y-1/2,z-1/2,-x'\n");
      break;

    case 497: // [202] F m -3 (-F 2 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-x,-y,z'\n\t'x,-y,-z'\n\t'-x,y,-z'\n\t'z,x,y'\n\t'-z,-x,y'\n\t'z,-x,-y'\n"
        "\t'-z,x,-y'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'-y,z,-x'\n\t'-y,-z,x'\n\t'-x,-y,-z'\n\t'x,y,-z'\n"
        "\t'-x,y,z'\n\t'x,-y,z'\n\t'-z,-x,-y'\n\t'z,x,-y'\n\t'-z,x,y'\n\t'z,-x,y'\n\t'-y,-z,-x'\n"
        "\t'-y,z,x'\n\t'y,-z,x'\n\t'y,z,-x'\n\t'x,y+1/2,z+1/2'\n\t'-x,-y+1/2,z+1/2'\n\t'x,-y+1/2,-z+1/2'\n\t'-x,y+1/2,-z+1/2'\n"
        "\t'z,x+1/2,y+1/2'\n\t'-z,-x+1/2,y+1/2'\n\t'z,-x+1/2,-y+1/2'\n\t'-z,x+1/2,-y+1/2'\n\t'y,z+1/2,x+1/2'\n\t'y,-z+1/2,-x+1/2'\n\t'-y,z+1/2,-x+1/2'\n"
        "\t'-y,-z+1/2,x+1/2'\n\t'-x,-y+1/2,-z+1/2'\n\t'x,y+1/2,-z+1/2'\n\t'-x,y+1/2,z+1/2'\n\t'x,-y+1/2,z+1/2'\n\t'-z,-x+1/2,-y+1/2'\n\t'z,x+1/2,-y+1/2'\n"
        "\t'-z,x+1/2,y+1/2'\n\t'z,-x+1/2,y+1/2'\n\t'-y,-z+1/2,-x+1/2'\n\t'-y,z+1/2,x+1/2'\n\t'y,-z+1/2,x+1/2'\n\t'y,z+1/2,-x+1/2'\n\t'x+1/2,y,z+1/2'\n"
        "\t'-x+1/2,-y,z+1/2'\n\t'x+1/2,-y,-z+1/2'\n\t'-x+1/2,y,-z+1/2'\n\t'z+1/2,x,y+1/2'\n\t'-z+1/2,-x,y+1/2'\n\t'z+1/2,-x,-y+1/2'\n\t'-z+1/2,x,-y+1/2'\n"
        "\t'y+1/2,z,x+1/2'\n\t'y+1/2,-z,-x+1/2'\n\t'-y+1/2,z,-x+1/2'\n\t'-y+1/2,-z,x+1/2'\n\t'-x+1/2,-y,-z+1/2'\n\t'x+1/2,y,-z+1/2'\n\t'-x+1/2,y,z+1/2'\n"
        "\t'x+1/2,-y,z+1/2'\n\t'-z+1/2,-x,-y+1/2'\n\t'z+1/2,x,-y+1/2'\n\t'-z+1/2,x,y+1/2'\n\t'z+1/2,-x,y+1/2'\n\t'-y+1/2,-z,-x+1/2'\n\t'-y+1/2,z,x+1/2'\n"
        "\t'y+1/2,-z,x+1/2'\n\t'y+1/2,z,-x+1/2'\n\t'x+1/2,y+1/2,z'\n\t'-x+1/2,-y+1/2,z'\n\t'x+1/2,-y+1/2,-z'\n\t'-x+1/2,y+1/2,-z'\n\t'z+1/2,x+1/2,y'\n"
        "\t'-z+1/2,-x+1/2,y'\n\t'z+1/2,-x+1/2,-y'\n\t'-z+1/2,x+1/2,-y'\n\t'y+1/2,z+1/2,x'\n\t'y+1/2,-z+1/2,-x'\n\t'-y+1/2,z+1/2,-x'\n\t'-y+1/2,-z+1/2,x'\n"
        "\t'-x+1/2,-y+1/2,-z'\n\t'x+1/2,y+1/2,-z'\n\t'-x+1/2,y+1/2,z'\n\t'x+1/2,-y+1/2,z'\n\t'-z+1/2,-x+1/2,-y'\n\t'z+1/2,x+1/2,-y'\n\t'-z+1/2,x+1/2,y'\n"
        "\t'z+1/2,-x+1/2,y'\n\t'-y+1/2,-z+1/2,-x'\n\t'-y+1/2,z+1/2,x'\n\t'y+1/2,-z+1/2,x'\n\t'y+1/2,z+1/2,-x'\n");
      break;

    case 498: // [203] F d -3 Origin choice 1 (F 2 2 3 -1d)
      fprintf(Stream,"\t'x,y,z'\n\t'-x,-y,z'\n\t'x,-y,-z'\n\t'-x,y,-z'\n\t'z,x,y'\n\t'-z,-x,y'\n\t'z,-x,-y'\n"
        "\t'-z,x,-y'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'-y,z,-x'\n\t'-y,-z,x'\n\t'-x+1/4,-y+1/4,-z+1/4'\n\t'x+1/4,y+1/4,-z+1/4'\n"
        "\t'-x+1/4,y+1/4,z+1/4'\n\t'x+1/4,-y+1/4,z+1/4'\n\t'-z+1/4,-x+1/4,-y+1/4'\n\t'z+1/4,x+1/4,-y+1/4'\n\t'-z+1/4,x+1/4,y+1/4'\n\t'z+1/4,-x+1/4,y+1/4'\n\t'-y+1/4,-z+1/4,-x+1/4'\n"
        "\t'-y+1/4,z+1/4,x+1/4'\n\t'y+1/4,-z+1/4,x+1/4'\n\t'y+1/4,z+1/4,-x+1/4'\n\t'x,y+1/2,z+1/2'\n\t'-x,-y+1/2,z+1/2'\n\t'x,-y+1/2,-z+1/2'\n\t'-x,y+1/2,-z+1/2'\n"
        "\t'z,x+1/2,y+1/2'\n\t'-z,-x+1/2,y+1/2'\n\t'z,-x+1/2,-y+1/2'\n\t'-z,x+1/2,-y+1/2'\n\t'y,z+1/2,x+1/2'\n\t'y,-z+1/2,-x+1/2'\n\t'-y,z+1/2,-x+1/2'\n"
        "\t'-y,-z+1/2,x+1/2'\n\t'-x+1/4,-y+3/4,-z+3/4'\n\t'x+1/4,y+3/4,-z+3/4'\n\t'-x+1/4,y+3/4,z+3/4'\n\t'x+1/4,-y+3/4,z+3/4'\n\t'-z+1/4,-x+3/4,-y+3/4'\n\t'z+1/4,x+3/4,-y+3/4'\n"
        "\t'-z+1/4,x+3/4,y+3/4'\n\t'z+1/4,-x+3/4,y+3/4'\n\t'-y+1/4,-z+3/4,-x+3/4'\n\t'-y+1/4,z+3/4,x+3/4'\n\t'y+1/4,-z+3/4,x+3/4'\n\t'y+1/4,z+3/4,-x+3/4'\n\t'x+1/2,y,z+1/2'\n"
        "\t'-x+1/2,-y,z+1/2'\n\t'x+1/2,-y,-z+1/2'\n\t'-x+1/2,y,-z+1/2'\n\t'z+1/2,x,y+1/2'\n\t'-z+1/2,-x,y+1/2'\n\t'z+1/2,-x,-y+1/2'\n\t'-z+1/2,x,-y+1/2'\n"
        "\t'y+1/2,z,x+1/2'\n\t'y+1/2,-z,-x+1/2'\n\t'-y+1/2,z,-x+1/2'\n\t'-y+1/2,-z,x+1/2'\n\t'-x+3/4,-y+1/4,-z+3/4'\n\t'x+3/4,y+1/4,-z+3/4'\n\t'-x+3/4,y+1/4,z+3/4'\n"
        "\t'x+3/4,-y+1/4,z+3/4'\n\t'-z+3/4,-x+1/4,-y+3/4'\n\t'z+3/4,x+1/4,-y+3/4'\n\t'-z+3/4,x+1/4,y+3/4'\n\t'z+3/4,-x+1/4,y+3/4'\n\t'-y+3/4,-z+1/4,-x+3/4'\n\t'-y+3/4,z+1/4,x+3/4'\n"
        "\t'y+3/4,-z+1/4,x+3/4'\n\t'y+3/4,z+1/4,-x+3/4'\n\t'x+1/2,y+1/2,z'\n\t'-x+1/2,-y+1/2,z'\n\t'x+1/2,-y+1/2,-z'\n\t'-x+1/2,y+1/2,-z'\n\t'z+1/2,x+1/2,y'\n"
        "\t'-z+1/2,-x+1/2,y'\n\t'z+1/2,-x+1/2,-y'\n\t'-z+1/2,x+1/2,-y'\n\t'y+1/2,z+1/2,x'\n\t'y+1/2,-z+1/2,-x'\n\t'-y+1/2,z+1/2,-x'\n\t'-y+1/2,-z+1/2,x'\n"
        "\t'-x+3/4,-y+3/4,-z+1/4'\n\t'x+3/4,y+3/4,-z+1/4'\n\t'-x+3/4,y+3/4,z+1/4'\n\t'x+3/4,-y+3/4,z+1/4'\n\t'-z+3/4,-x+3/4,-y+1/4'\n\t'z+3/4,x+3/4,-y+1/4'\n\t'-z+3/4,x+3/4,y+1/4'\n"
        "\t'z+3/4,-x+3/4,y+1/4'\n\t'-y+3/4,-z+3/4,-x+1/4'\n\t'-y+3/4,z+3/4,x+1/4'\n\t'y+3/4,-z+3/4,x+1/4'\n\t'y+3/4,z+3/4,-x+1/4'\n");
      break;
    case 499: // [203] F d -3 Origin choice 2 (-F 2uv 2vw 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-x+1/4,-y+1/4,z'\n\t'x,-y+1/4,-z+1/4'\n\t'-x+1/4,y,-z+1/4'\n\t'z,x,y'\n\t'-z+1/4,-x+1/4,y'\n\t'z,-x+1/4,-y+1/4'\n"
        "\t'-z+1/4,x,-y+1/4'\n\t'y,z,x'\n\t'y,-z+1/4,-x+1/4'\n\t'-y+1/4,z,-x+1/4'\n\t'-y+1/4,-z+1/4,x'\n\t'-x,-y,-z'\n\t'x-1/4,y-1/4,-z'\n"
        "\t'-x,y-1/4,z-1/4'\n\t'x-1/4,-y,z-1/4'\n\t'-z,-x,-y'\n\t'z-1/4,x-1/4,-y'\n\t'-z,x-1/4,y-1/4'\n\t'z-1/4,-x,y-1/4'\n\t'-y,-z,-x'\n"
        "\t'-y,z-1/4,x-1/4'\n\t'y-1/4,-z,x-1/4'\n\t'y-1/4,z-1/4,-x'\n\t'x,y+1/2,z+1/2'\n\t'-x+1/4,-y+3/4,z+1/2'\n\t'x,-y+3/4,-z+3/4'\n\t'-x+1/4,y+1/2,-z+3/4'\n"
        "\t'z,x+1/2,y+1/2'\n\t'-z+1/4,-x+3/4,y+1/2'\n\t'z,-x+3/4,-y+3/4'\n\t'-z+1/4,x+1/2,-y+3/4'\n\t'y,z+1/2,x+1/2'\n\t'y,-z+3/4,-x+3/4'\n\t'-y+1/4,z+1/2,-x+3/4'\n"
        "\t'-y+1/4,-z+3/4,x+1/2'\n\t'-x,-y+1/2,-z+1/2'\n\t'x-1/4,y+1/4,-z+1/2'\n\t'-x,y+1/4,z+1/4'\n\t'x-1/4,-y+1/2,z+1/4'\n\t'-z,-x+1/2,-y+1/2'\n\t'z-1/4,x+1/4,-y+1/2'\n"
        "\t'-z,x+1/4,y+1/4'\n\t'z-1/4,-x+1/2,y+1/4'\n\t'-y,-z+1/2,-x+1/2'\n\t'-y,z+1/4,x+1/4'\n\t'y-1/4,-z+1/2,x+1/4'\n\t'y-1/4,z+1/4,-x+1/2'\n\t'x+1/2,y,z+1/2'\n"
        "\t'-x+3/4,-y+1/4,z+1/2'\n\t'x+1/2,-y+1/4,-z+3/4'\n\t'-x+3/4,y,-z+3/4'\n\t'z+1/2,x,y+1/2'\n\t'-z+3/4,-x+1/4,y+1/2'\n\t'z+1/2,-x+1/4,-y+3/4'\n\t'-z+3/4,x,-y+3/4'\n"
        "\t'y+1/2,z,x+1/2'\n\t'y+1/2,-z+1/4,-x+3/4'\n\t'-y+3/4,z,-x+3/4'\n\t'-y+3/4,-z+1/4,x+1/2'\n\t'-x+1/2,-y,-z+1/2'\n\t'x+1/4,y-1/4,-z+1/2'\n\t'-x+1/2,y-1/4,z+1/4'\n"
        "\t'x+1/4,-y,z+1/4'\n\t'-z+1/2,-x,-y+1/2'\n\t'z+1/4,x-1/4,-y+1/2'\n\t'-z+1/2,x-1/4,y+1/4'\n\t'z+1/4,-x,y+1/4'\n\t'-y+1/2,-z,-x+1/2'\n\t'-y+1/2,z-1/4,x+1/4'\n"
        "\t'y+1/4,-z,x+1/4'\n\t'y+1/4,z-1/4,-x+1/2'\n\t'x+1/2,y+1/2,z'\n\t'-x+3/4,-y+3/4,z'\n\t'x+1/2,-y+3/4,-z+1/4'\n\t'-x+3/4,y+1/2,-z+1/4'\n\t'z+1/2,x+1/2,y'\n"
        "\t'-z+3/4,-x+3/4,y'\n\t'z+1/2,-x+3/4,-y+1/4'\n\t'-z+3/4,x+1/2,-y+1/4'\n\t'y+1/2,z+1/2,x'\n\t'y+1/2,-z+3/4,-x+1/4'\n\t'-y+3/4,z+1/2,-x+1/4'\n\t'-y+3/4,-z+3/4,x'\n"
        "\t'-x+1/2,-y+1/2,-z'\n\t'x+1/4,y+1/4,-z'\n\t'-x+1/2,y+1/4,z-1/4'\n\t'x+1/4,-y+1/2,z-1/4'\n\t'-z+1/2,-x+1/2,-y'\n\t'z+1/4,x+1/4,-y'\n\t'-z+1/2,x+1/4,y-1/4'\n"
        "\t'z+1/4,-x+1/2,y-1/4'\n\t'-y+1/2,-z+1/2,-x'\n\t'-y+1/2,z+1/4,x-1/4'\n\t'y+1/4,-z+1/2,x-1/4'\n\t'y+1/4,z+1/4,-x'\n");
      break;

    case 500: // [204] I m -3 (-I 2 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-x,-y,z'\n\t'x,-y,-z'\n\t'-x,y,-z'\n\t'z,x,y'\n\t'-z,-x,y'\n\t'z,-x,-y'\n"
        "\t'-z,x,-y'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'-y,z,-x'\n\t'-y,-z,x'\n\t'-x,-y,-z'\n\t'x,y,-z'\n"
        "\t'-x,y,z'\n\t'x,-y,z'\n\t'-z,-x,-y'\n\t'z,x,-y'\n\t'-z,x,y'\n\t'z,-x,y'\n\t'-y,-z,-x'\n"
        "\t'-y,z,x'\n\t'y,-z,x'\n\t'y,z,-x'\n\t'x+1/2,y+1/2,z+1/2'\n\t'-x+1/2,-y+1/2,z+1/2'\n\t'x+1/2,-y+1/2,-z+1/2'\n\t'-x+1/2,y+1/2,-z+1/2'\n"
        "\t'z+1/2,x+1/2,y+1/2'\n\t'-z+1/2,-x+1/2,y+1/2'\n\t'z+1/2,-x+1/2,-y+1/2'\n\t'-z+1/2,x+1/2,-y+1/2'\n\t'y+1/2,z+1/2,x+1/2'\n\t'y+1/2,-z+1/2,-x+1/2'\n\t'-y+1/2,z+1/2,-x+1/2'\n"
        "\t'-y+1/2,-z+1/2,x+1/2'\n\t'-x+1/2,-y+1/2,-z+1/2'\n\t'x+1/2,y+1/2,-z+1/2'\n\t'-x+1/2,y+1/2,z+1/2'\n\t'x+1/2,-y+1/2,z+1/2'\n\t'-z+1/2,-x+1/2,-y+1/2'\n\t'z+1/2,x+1/2,-y+1/2'\n"
        "\t'-z+1/2,x+1/2,y+1/2'\n\t'z+1/2,-x+1/2,y+1/2'\n\t'-y+1/2,-z+1/2,-x+1/2'\n\t'-y+1/2,z+1/2,x+1/2'\n\t'y+1/2,-z+1/2,x+1/2'\n\t'y+1/2,z+1/2,-x+1/2'\n");
      break;

    case 501: // [205] P a -3 (-P 2ac 2ab 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-x+1/2,-y,z+1/2'\n\t'x+1/2,-y+1/2,-z'\n\t'-x,y+1/2,-z+1/2'\n\t'z,x,y'\n\t'-z+1/2,-x,y+1/2'\n\t'z+1/2,-x+1/2,-y'\n"
        "\t'-z,x+1/2,-y+1/2'\n\t'y,z,x'\n\t'y+1/2,-z+1/2,-x'\n\t'-y,z+1/2,-x+1/2'\n\t'-y+1/2,-z,x+1/2'\n\t'-x,-y,-z'\n\t'x-1/2,y,-z-1/2'\n"
        "\t'-x-1/2,y-1/2,z'\n\t'x,-y-1/2,z-1/2'\n\t'-z,-x,-y'\n\t'z-1/2,x,-y-1/2'\n\t'-z-1/2,x-1/2,y'\n\t'z,-x-1/2,y-1/2'\n\t'-y,-z,-x'\n"
        "\t'-y-1/2,z-1/2,x'\n\t'y,-z-1/2,x-1/2'\n\t'y-1/2,z,-x-1/2'\n");
      break;

    case 502: // [206] I a -3 (-I 2b 2c 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-x,-y+1/2,z'\n\t'x,-y,-z+1/2'\n\t'-x,y+1/2,-z+1/2'\n\t'z,x,y'\n\t'-z,-x+1/2,y'\n\t'z,-x,-y+1/2'\n"
        "\t'-z,x+1/2,-y+1/2'\n\t'y,z,x'\n\t'y,-z,-x+1/2'\n\t'-y,z+1/2,-x+1/2'\n\t'-y+1/2,-z,x+1/2'\n\t'-x,-y,-z'\n\t'x,y-1/2,-z'\n"
        "\t'-x,y,z-1/2'\n\t'x,-y-1/2,z-1/2'\n\t'-z,-x,-y'\n\t'z,x-1/2,-y'\n\t'-z,x,y-1/2'\n\t'z,-x-1/2,y-1/2'\n\t'-y,-z,-x'\n"
        "\t'-y,z,x-1/2'\n\t'y,-z-1/2,x-1/2'\n\t'y-1/2,z,-x-1/2'\n\t'x+1/2,y+1/2,z+1/2'\n\t'-x+1/2,-y+1,z+1/2'\n\t'x+1/2,-y+1/2,-z+1'\n\t'-x+1/2,y+1,-z+1'\n"
        "\t'z+1/2,x+1/2,y+1/2'\n\t'-z+1/2,-x+1,y+1/2'\n\t'z+1/2,-x+1/2,-y+1'\n\t'-z+1/2,x+1,-y+1'\n\t'y+1/2,z+1/2,x+1/2'\n\t'y+1/2,-z+1/2,-x+1'\n\t'-y+1/2,z+1,-x+1'\n"
        "\t'-y+1,-z+1/2,x+1'\n\t'-x+1/2,-y+1/2,-z+1/2'\n\t'x+1/2,y,-z+1/2'\n\t'-x+1/2,y+1/2,z'\n\t'x+1/2,-y,z'\n\t'-z+1/2,-x+1/2,-y+1/2'\n\t'z+1/2,x,-y+1/2'\n"
        "\t'-z+1/2,x+1/2,y'\n\t'z+1/2,-x,y'\n\t'-y+1/2,-z+1/2,-x+1/2'\n\t'-y+1/2,z+1/2,x'\n\t'y+1/2,-z,x'\n\t'y,z+1/2,-x'\n");
      break;

    case 503: // [207] P 4 3 2 (P 4 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x,z'\n\t'-x,-y,z'\n\t'y,-x,z'\n\t'x,-y,-z'\n\t'y,x,-z'\n\t'-x,y,-z'\n"
        "\t'-y,-x,-z'\n\t'z,x,y'\n\t'-x,z,y'\n\t'-z,-x,y'\n\t'x,-z,y'\n\t'z,-x,-y'\n\t'x,z,-y'\n"
        "\t'-z,x,-y'\n\t'-x,-z,-y'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'z,y,-x'\n\t'-y,z,-x'\n\t'-z,-y,-x'\n"
        "\t'-y,-z,x'\n\t'z,-y,x'\n\t'-z,y,x'\n");
      break;

    case 504: // [208] P 42 3 2 (P 4n 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-y+1/2,x+1/2,z+1/2'\n\t'-x,-y,z'\n\t'y+1/2,-x+1/2,z+1/2'\n\t'x,-y,-z'\n\t'y+1/2,x+1/2,-z+1/2'\n\t'-x,y,-z'\n"
        "\t'-y+1/2,-x+1/2,-z+1/2'\n\t'z,x,y'\n\t'-x+1/2,z+1/2,y+1/2'\n\t'-z,-x,y'\n\t'x+1/2,-z+1/2,y+1/2'\n\t'z,-x,-y'\n\t'x+1/2,z+1/2,-y+1/2'\n"
        "\t'-z,x,-y'\n\t'-x+1/2,-z+1/2,-y+1/2'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'z+1/2,y+1/2,-x+1/2'\n\t'-y,z,-x'\n\t'-z+1/2,-y+1/2,-x+1/2'\n"
        "\t'-y,-z,x'\n\t'z+1/2,-y+1/2,x+1/2'\n\t'-z+1/2,y+1/2,x+1/2'\n");
      break;

    case 505: // [209] F 4 3 2 (F 4 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x,z'\n\t'-x,-y,z'\n\t'y,-x,z'\n\t'x,-y,-z'\n\t'y,x,-z'\n\t'-x,y,-z'\n"
        "\t'-y,-x,-z'\n\t'z,x,y'\n\t'-x,z,y'\n\t'-z,-x,y'\n\t'x,-z,y'\n\t'z,-x,-y'\n\t'x,z,-y'\n"
        "\t'-z,x,-y'\n\t'-x,-z,-y'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'z,y,-x'\n\t'-y,z,-x'\n\t'-z,-y,-x'\n"
        "\t'-y,-z,x'\n\t'z,-y,x'\n\t'-z,y,x'\n\t'x,y+1/2,z+1/2'\n\t'-y,x+1/2,z+1/2'\n\t'-x,-y+1/2,z+1/2'\n\t'y,-x+1/2,z+1/2'\n"
        "\t'x,-y+1/2,-z+1/2'\n\t'y,x+1/2,-z+1/2'\n\t'-x,y+1/2,-z+1/2'\n\t'-y,-x+1/2,-z+1/2'\n\t'z,x+1/2,y+1/2'\n\t'-x,z+1/2,y+1/2'\n\t'-z,-x+1/2,y+1/2'\n"
        "\t'x,-z+1/2,y+1/2'\n\t'z,-x+1/2,-y+1/2'\n\t'x,z+1/2,-y+1/2'\n\t'-z,x+1/2,-y+1/2'\n\t'-x,-z+1/2,-y+1/2'\n\t'y,z+1/2,x+1/2'\n\t'y,-z+1/2,-x+1/2'\n"
        "\t'z,y+1/2,-x+1/2'\n\t'-y,z+1/2,-x+1/2'\n\t'-z,-y+1/2,-x+1/2'\n\t'-y,-z+1/2,x+1/2'\n\t'z,-y+1/2,x+1/2'\n\t'-z,y+1/2,x+1/2'\n\t'x+1/2,y,z+1/2'\n"
        "\t'-y+1/2,x,z+1/2'\n\t'-x+1/2,-y,z+1/2'\n\t'y+1/2,-x,z+1/2'\n\t'x+1/2,-y,-z+1/2'\n\t'y+1/2,x,-z+1/2'\n\t'-x+1/2,y,-z+1/2'\n\t'-y+1/2,-x,-z+1/2'\n"
        "\t'z+1/2,x,y+1/2'\n\t'-x+1/2,z,y+1/2'\n\t'-z+1/2,-x,y+1/2'\n\t'x+1/2,-z,y+1/2'\n\t'z+1/2,-x,-y+1/2'\n\t'x+1/2,z,-y+1/2'\n\t'-z+1/2,x,-y+1/2'\n"
        "\t'-x+1/2,-z,-y+1/2'\n\t'y+1/2,z,x+1/2'\n\t'y+1/2,-z,-x+1/2'\n\t'z+1/2,y,-x+1/2'\n\t'-y+1/2,z,-x+1/2'\n\t'-z+1/2,-y,-x+1/2'\n\t'-y+1/2,-z,x+1/2'\n"
        "\t'z+1/2,-y,x+1/2'\n\t'-z+1/2,y,x+1/2'\n\t'x+1/2,y+1/2,z'\n\t'-y+1/2,x+1/2,z'\n\t'-x+1/2,-y+1/2,z'\n\t'y+1/2,-x+1/2,z'\n\t'x+1/2,-y+1/2,-z'\n"
        "\t'y+1/2,x+1/2,-z'\n\t'-x+1/2,y+1/2,-z'\n\t'-y+1/2,-x+1/2,-z'\n\t'z+1/2,x+1/2,y'\n\t'-x+1/2,z+1/2,y'\n\t'-z+1/2,-x+1/2,y'\n\t'x+1/2,-z+1/2,y'\n"
        "\t'z+1/2,-x+1/2,-y'\n\t'x+1/2,z+1/2,-y'\n\t'-z+1/2,x+1/2,-y'\n\t'-x+1/2,-z+1/2,-y'\n\t'y+1/2,z+1/2,x'\n\t'y+1/2,-z+1/2,-x'\n\t'z+1/2,y+1/2,-x'\n"
        "\t'-y+1/2,z+1/2,-x'\n\t'-z+1/2,-y+1/2,-x'\n\t'-y+1/2,-z+1/2,x'\n\t'z+1/2,-y+1/2,x'\n\t'-z+1/2,y+1/2,x'\n");
      break;

    case 506: // [210] F 41 3 2 (F 4d 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-y+1/4,x+1/4,z+1/4'\n\t'-x,-y+1/2,z+1/2'\n\t'y+3/4,-x+1/4,z+3/4'\n\t'x,-y,-z'\n\t'y+1/4,x+1/4,-z+1/4'\n\t'-x,y+1/2,-z+1/2'\n"
        "\t'-y+3/4,-x+1/4,-z+3/4'\n\t'z,x,y'\n\t'-x+1/4,z+1/4,y+1/4'\n\t'-z,-x+1/2,y+1/2'\n\t'x+3/4,-z+1/4,y+3/4'\n\t'z,-x,-y'\n\t'x+1/4,z+1/4,-y+1/4'\n"
        "\t'-z,x+1/2,-y+1/2'\n\t'-x+3/4,-z+1/4,-y+3/4'\n\t'y,z,x'\n\t'y+1/2,-z,-x+1/2'\n\t'z+1/4,y+3/4,-x+3/4'\n\t'-y+1/2,z+1/2,-x'\n\t'-z+1/4,-y+1/4,-x+1/4'\n"
        "\t'-y,-z,x'\n\t'z+1/4,-y+3/4,x+3/4'\n\t'-z+3/4,y+3/4,x+1/4'\n\t'x,y+1/2,z+1/2'\n\t'-y+1/4,x+3/4,z+3/4'\n\t'-x,-y+1,z+1'\n\t'y+3/4,-x+3/4,z+1/4'\n"
        "\t'x,-y+1/2,-z+1/2'\n\t'y+1/4,x+3/4,-z+3/4'\n\t'-x,y+1,-z+1'\n\t'-y+3/4,-x+3/4,-z+1/4'\n\t'z,x+1/2,y+1/2'\n\t'-x+1/4,z+3/4,y+3/4'\n\t'-z,-x+1,y+1'\n"
        "\t'x+3/4,-z+3/4,y+1/4'\n\t'z,-x+1/2,-y+1/2'\n\t'x+1/4,z+3/4,-y+3/4'\n\t'-z,x+1,-y+1'\n\t'-x+3/4,-z+3/4,-y+1/4'\n\t'y,z+1/2,x+1/2'\n\t'y+1/2,-z+1/2,-x+1'\n"
        "\t'z+1/4,y+1/4,-x+1/4'\n\t'-y+1/2,z+1,-x+1/2'\n\t'-z+1/4,-y+3/4,-x+3/4'\n\t'-y,-z+1/2,x+1/2'\n\t'z+1/4,-y+1/4,x+1/4'\n\t'-z+3/4,y+1/4,x+3/4'\n\t'x+1/2,y,z+1/2'\n"
        "\t'-y+3/4,x+1/4,z+3/4'\n\t'-x+1/2,-y+1/2,z+1'\n\t'y+1/4,-x+1/4,z+1/4'\n\t'x+1/2,-y,-z+1/2'\n\t'y+3/4,x+1/4,-z+3/4'\n\t'-x+1/2,y+1/2,-z+1'\n\t'-y+1/4,-x+1/4,-z+1/4'\n"
        "\t'z+1/2,x,y+1/2'\n\t'-x+3/4,z+1/4,y+3/4'\n\t'-z+1/2,-x+1/2,y+1'\n\t'x+1/4,-z+1/4,y+1/4'\n\t'z+1/2,-x,-y+1/2'\n\t'x+3/4,z+1/4,-y+3/4'\n\t'-z+1/2,x+1/2,-y+1'\n"
        "\t'-x+1/4,-z+1/4,-y+1/4'\n\t'y+1/2,z,x+1/2'\n\t'y+1,-z,-x+1'\n\t'z+3/4,y+3/4,-x+1/4'\n\t'-y+1,z+1/2,-x+1/2'\n\t'-z+3/4,-y+1/4,-x+3/4'\n\t'-y+1/2,-z,x+1/2'\n"
        "\t'z+3/4,-y+3/4,x+1/4'\n\t'-z+1/4,y+3/4,x+3/4'\n\t'x+1/2,y+1/2,z'\n\t'-y+3/4,x+3/4,z+1/4'\n\t'-x+1/2,-y+1,z+1/2'\n\t'y+1/4,-x+3/4,z+3/4'\n\t'x+1/2,-y+1/2,-z'\n"
        "\t'y+3/4,x+3/4,-z+1/4'\n\t'-x+1/2,y+1,-z+1/2'\n\t'-y+1/4,-x+3/4,-z+3/4'\n\t'z+1/2,x+1/2,y'\n\t'-x+3/4,z+3/4,y+1/4'\n\t'-z+1/2,-x+1,y+1/2'\n\t'x+1/4,-z+3/4,y+3/4'\n"
        "\t'z+1/2,-x+1/2,-y'\n\t'x+3/4,z+3/4,-y+1/4'\n\t'-z+1/2,x+1,-y+1/2'\n\t'-x+1/4,-z+3/4,-y+3/4'\n\t'y+1/2,z+1/2,x'\n\t'y+1,-z+1/2,-x+1/2'\n\t'z+3/4,y+1/4,-x+3/4'\n"
        "\t'-y+1,z+1,-x'\n\t'-z+3/4,-y+3/4,-x+1/4'\n\t'-y+1/2,-z+1/2,x'\n\t'z+3/4,-y+1/4,x+3/4'\n\t'-z+1/4,y+1/4,x+1/4'\n");
      break;

    case 507: // [211] I 4 3 2 (I 4 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x,z'\n\t'-x,-y,z'\n\t'y,-x,z'\n\t'x,-y,-z'\n\t'y,x,-z'\n\t'-x,y,-z'\n"
        "\t'-y,-x,-z'\n\t'z,x,y'\n\t'-x,z,y'\n\t'-z,-x,y'\n\t'x,-z,y'\n\t'z,-x,-y'\n\t'x,z,-y'\n"
        "\t'-z,x,-y'\n\t'-x,-z,-y'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'z,y,-x'\n\t'-y,z,-x'\n\t'-z,-y,-x'\n"
        "\t'-y,-z,x'\n\t'z,-y,x'\n\t'-z,y,x'\n\t'x+1/2,y+1/2,z+1/2'\n\t'-y+1/2,x+1/2,z+1/2'\n\t'-x+1/2,-y+1/2,z+1/2'\n\t'y+1/2,-x+1/2,z+1/2'\n"
        "\t'x+1/2,-y+1/2,-z+1/2'\n\t'y+1/2,x+1/2,-z+1/2'\n\t'-x+1/2,y+1/2,-z+1/2'\n\t'-y+1/2,-x+1/2,-z+1/2'\n\t'z+1/2,x+1/2,y+1/2'\n\t'-x+1/2,z+1/2,y+1/2'\n\t'-z+1/2,-x+1/2,y+1/2'\n"
        "\t'x+1/2,-z+1/2,y+1/2'\n\t'z+1/2,-x+1/2,-y+1/2'\n\t'x+1/2,z+1/2,-y+1/2'\n\t'-z+1/2,x+1/2,-y+1/2'\n\t'-x+1/2,-z+1/2,-y+1/2'\n\t'y+1/2,z+1/2,x+1/2'\n\t'y+1/2,-z+1/2,-x+1/2'\n"
        "\t'z+1/2,y+1/2,-x+1/2'\n\t'-y+1/2,z+1/2,-x+1/2'\n\t'-z+1/2,-y+1/2,-x+1/2'\n\t'-y+1/2,-z+1/2,x+1/2'\n\t'z+1/2,-y+1/2,x+1/2'\n\t'-z+1/2,y+1/2,x+1/2'\n");
      break;

    case 508: // [212] P 43 3 2 (P 4acd 2ab 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-y+3/4,x+1/4,z+3/4'\n\t'-x+1/2,-y,z+1/2'\n\t'y+3/4,-x+3/4,z+1/4'\n\t'x+1/2,-y+1/2,-z'\n\t'y+1/4,x+3/4,-z+3/4'\n\t'-x,y+1/2,-z+1/2'\n"
        "\t'-y+1/4,-x+1/4,-z+1/4'\n\t'z,x,y'\n\t'-x+3/4,z+1/4,y+3/4'\n\t'-z+1/2,-x,y+1/2'\n\t'x+3/4,-z+3/4,y+1/4'\n\t'z+1/2,-x+1/2,-y'\n\t'x+1/4,z+3/4,-y+3/4'\n"
        "\t'-z,x+1/2,-y+1/2'\n\t'-x+1/4,-z+1/4,-y+1/4'\n\t'y,z,x'\n\t'y+1/2,-z+1/2,-x'\n\t'z+1/4,y+3/4,-x+3/4'\n\t'-y,z+1/2,-x+1/2'\n\t'-z+1/4,-y+1/4,-x+1/4'\n"
        "\t'-y+1/2,-z,x+1/2'\n\t'z+3/4,-y+3/4,x+1/4'\n\t'-z+3/4,y+1/4,x+3/4'\n");
      break;

    case 509: // [213] P 41 3 2 (P 4bd 2ab 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-y+1/4,x+3/4,z+1/4'\n\t'-x+1/2,-y,z+1/2'\n\t'y+1/4,-x+1/4,z+3/4'\n\t'x+1/2,-y+1/2,-z'\n\t'y+3/4,x+1/4,-z+1/4'\n\t'-x,y+1/2,-z+1/2'\n"
        "\t'-y+3/4,-x+3/4,-z+3/4'\n\t'z,x,y'\n\t'-x+1/4,z+3/4,y+1/4'\n\t'-z+1/2,-x,y+1/2'\n\t'x+1/4,-z+1/4,y+3/4'\n\t'z+1/2,-x+1/2,-y'\n\t'x+3/4,z+1/4,-y+1/4'\n"
        "\t'-z,x+1/2,-y+1/2'\n\t'-x+3/4,-z+3/4,-y+3/4'\n\t'y,z,x'\n\t'y+1/2,-z+1/2,-x'\n\t'z+3/4,y+1/4,-x+1/4'\n\t'-y,z+1/2,-x+1/2'\n\t'-z+3/4,-y+3/4,-x+3/4'\n"
        "\t'-y+1/2,-z,x+1/2'\n\t'z+1/4,-y+1/4,x+3/4'\n\t'-z+1/4,y+3/4,x+1/4'\n");
      break;

    case 510: // [214] I 41 3 2 (I 4bd 2c 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-y+1/4,x+3/4,z+1/4'\n\t'-x+1/2,-y,z+1/2'\n\t'y+1/4,-x+1/4,z+3/4'\n\t'x,-y,-z+1/2'\n\t'y+1/4,x+3/4,-z+3/4'\n\t'-x+1/2,y,-z'\n"
        "\t'-y+1/4,-x+1/4,-z+1/4'\n\t'z,x,y'\n\t'-x+1/4,z+3/4,y+1/4'\n\t'-z+1/2,-x,y+1/2'\n\t'x+1/4,-z+1/4,y+3/4'\n\t'z,-x,-y+1/2'\n\t'x+1/4,z+3/4,-y+3/4'\n"
        "\t'-z+1/2,x,-y'\n\t'-x+1/4,-z+1/4,-y+1/4'\n\t'y,z,x'\n\t'y+1/2,-z+1/2,-x'\n\t'z+3/4,y+1/4,-x+1/4'\n\t'-y,z+1/2,-x+1/2'\n\t'-z+1/4,-y+1/4,-x+1/4'\n"
        "\t'-y+1/2,-z,x+1/2'\n\t'z+3/4,-y+3/4,x+1/4'\n\t'-z+3/4,y+1/4,x+3/4'\n\t'x+1/2,y+1/2,z+1/2'\n\t'-y+3/4,x+1/4,z+3/4'\n\t'-x+1,-y+1/2,z+1'\n\t'y+3/4,-x+3/4,z+1/4'\n"
        "\t'x+1/2,-y+1/2,-z+1'\n\t'y+3/4,x+1/4,-z+1/4'\n\t'-x+1,y+1/2,-z+1/2'\n\t'-y+3/4,-x+3/4,-z+3/4'\n\t'z+1/2,x+1/2,y+1/2'\n\t'-x+3/4,z+1/4,y+3/4'\n\t'-z+1,-x+1/2,y+1'\n"
        "\t'x+3/4,-z+3/4,y+1/4'\n\t'z+1/2,-x+1/2,-y+1'\n\t'x+3/4,z+1/4,-y+1/4'\n\t'-z+1,x+1/2,-y+1/2'\n\t'-x+3/4,-z+3/4,-y+3/4'\n\t'y+1/2,z+1/2,x+1/2'\n\t'y+1,-z+1,-x+1/2'\n"
        "\t'z+1/4,y+3/4,-x+3/4'\n\t'-y+1/2,z+1,-x+1'\n\t'-z+3/4,-y+3/4,-x+3/4'\n\t'-y+1,-z+1/2,x+1'\n\t'z+1/4,-y+1/4,x+3/4'\n\t'-z+1/4,y+3/4,x+1/4'\n");
      break;

    case 511: // [215] P -4 3 m (P -4 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'y,-x,-z'\n\t'-x,-y,z'\n\t'-y,x,-z'\n\t'x,-y,-z'\n\t'-y,-x,z'\n\t'-x,y,-z'\n"
        "\t'y,x,z'\n\t'z,x,y'\n\t'x,-z,-y'\n\t'-z,-x,y'\n\t'-x,z,-y'\n\t'z,-x,-y'\n\t'-x,-z,y'\n"
        "\t'-z,x,-y'\n\t'x,z,y'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'-z,-y,x'\n\t'-y,z,-x'\n\t'z,y,x'\n"
        "\t'-y,-z,x'\n\t'-z,y,-x'\n\t'z,-y,-x'\n");
      break;

    case 512: // [216] F -4 3 m (F -4 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'y,-x,-z'\n\t'-x,-y,z'\n\t'-y,x,-z'\n\t'x,-y,-z'\n\t'-y,-x,z'\n\t'-x,y,-z'\n"
        "\t'y,x,z'\n\t'z,x,y'\n\t'x,-z,-y'\n\t'-z,-x,y'\n\t'-x,z,-y'\n\t'z,-x,-y'\n\t'-x,-z,y'\n"
        "\t'-z,x,-y'\n\t'x,z,y'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'-z,-y,x'\n\t'-y,z,-x'\n\t'z,y,x'\n"
        "\t'-y,-z,x'\n\t'-z,y,-x'\n\t'z,-y,-x'\n\t'x,y+1/2,z+1/2'\n\t'y,-x+1/2,-z+1/2'\n\t'-x,-y+1/2,z+1/2'\n\t'-y,x+1/2,-z+1/2'\n"
        "\t'x,-y+1/2,-z+1/2'\n\t'-y,-x+1/2,z+1/2'\n\t'-x,y+1/2,-z+1/2'\n\t'y,x+1/2,z+1/2'\n\t'z,x+1/2,y+1/2'\n\t'x,-z+1/2,-y+1/2'\n\t'-z,-x+1/2,y+1/2'\n"
        "\t'-x,z+1/2,-y+1/2'\n\t'z,-x+1/2,-y+1/2'\n\t'-x,-z+1/2,y+1/2'\n\t'-z,x+1/2,-y+1/2'\n\t'x,z+1/2,y+1/2'\n\t'y,z+1/2,x+1/2'\n\t'y,-z+1/2,-x+1/2'\n"
        "\t'-z,-y+1/2,x+1/2'\n\t'-y,z+1/2,-x+1/2'\n\t'z,y+1/2,x+1/2'\n\t'-y,-z+1/2,x+1/2'\n\t'-z,y+1/2,-x+1/2'\n\t'z,-y+1/2,-x+1/2'\n\t'x+1/2,y,z+1/2'\n"
        "\t'y+1/2,-x,-z+1/2'\n\t'-x+1/2,-y,z+1/2'\n\t'-y+1/2,x,-z+1/2'\n\t'x+1/2,-y,-z+1/2'\n\t'-y+1/2,-x,z+1/2'\n\t'-x+1/2,y,-z+1/2'\n\t'y+1/2,x,z+1/2'\n"
        "\t'z+1/2,x,y+1/2'\n\t'x+1/2,-z,-y+1/2'\n\t'-z+1/2,-x,y+1/2'\n\t'-x+1/2,z,-y+1/2'\n\t'z+1/2,-x,-y+1/2'\n\t'-x+1/2,-z,y+1/2'\n\t'-z+1/2,x,-y+1/2'\n"
        "\t'x+1/2,z,y+1/2'\n\t'y+1/2,z,x+1/2'\n\t'y+1/2,-z,-x+1/2'\n\t'-z+1/2,-y,x+1/2'\n\t'-y+1/2,z,-x+1/2'\n\t'z+1/2,y,x+1/2'\n\t'-y+1/2,-z,x+1/2'\n"
        "\t'-z+1/2,y,-x+1/2'\n\t'z+1/2,-y,-x+1/2'\n\t'x+1/2,y+1/2,z'\n\t'y+1/2,-x+1/2,-z'\n\t'-x+1/2,-y+1/2,z'\n\t'-y+1/2,x+1/2,-z'\n\t'x+1/2,-y+1/2,-z'\n"
        "\t'-y+1/2,-x+1/2,z'\n\t'-x+1/2,y+1/2,-z'\n\t'y+1/2,x+1/2,z'\n\t'z+1/2,x+1/2,y'\n\t'x+1/2,-z+1/2,-y'\n\t'-z+1/2,-x+1/2,y'\n\t'-x+1/2,z+1/2,-y'\n"
        "\t'z+1/2,-x+1/2,-y'\n\t'-x+1/2,-z+1/2,y'\n\t'-z+1/2,x+1/2,-y'\n\t'x+1/2,z+1/2,y'\n\t'y+1/2,z+1/2,x'\n\t'y+1/2,-z+1/2,-x'\n\t'-z+1/2,-y+1/2,x'\n"
        "\t'-y+1/2,z+1/2,-x'\n\t'z+1/2,y+1/2,x'\n\t'-y+1/2,-z+1/2,x'\n\t'-z+1/2,y+1/2,-x'\n\t'z+1/2,-y+1/2,-x'\n");
      break;

    case 513: // [217] I -4 3 m (I -4 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'y,-x,-z'\n\t'-x,-y,z'\n\t'-y,x,-z'\n\t'x,-y,-z'\n\t'-y,-x,z'\n\t'-x,y,-z'\n"
        "\t'y,x,z'\n\t'z,x,y'\n\t'x,-z,-y'\n\t'-z,-x,y'\n\t'-x,z,-y'\n\t'z,-x,-y'\n\t'-x,-z,y'\n"
        "\t'-z,x,-y'\n\t'x,z,y'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'-z,-y,x'\n\t'-y,z,-x'\n\t'z,y,x'\n"
        "\t'-y,-z,x'\n\t'-z,y,-x'\n\t'z,-y,-x'\n\t'x+1/2,y+1/2,z+1/2'\n\t'y+1/2,-x+1/2,-z+1/2'\n\t'-x+1/2,-y+1/2,z+1/2'\n\t'-y+1/2,x+1/2,-z+1/2'\n"
        "\t'x+1/2,-y+1/2,-z+1/2'\n\t'-y+1/2,-x+1/2,z+1/2'\n\t'-x+1/2,y+1/2,-z+1/2'\n\t'y+1/2,x+1/2,z+1/2'\n\t'z+1/2,x+1/2,y+1/2'\n\t'x+1/2,-z+1/2,-y+1/2'\n\t'-z+1/2,-x+1/2,y+1/2'\n"
        "\t'-x+1/2,z+1/2,-y+1/2'\n\t'z+1/2,-x+1/2,-y+1/2'\n\t'-x+1/2,-z+1/2,y+1/2'\n\t'-z+1/2,x+1/2,-y+1/2'\n\t'x+1/2,z+1/2,y+1/2'\n\t'y+1/2,z+1/2,x+1/2'\n\t'y+1/2,-z+1/2,-x+1/2'\n"
        "\t'-z+1/2,-y+1/2,x+1/2'\n\t'-y+1/2,z+1/2,-x+1/2'\n\t'z+1/2,y+1/2,x+1/2'\n\t'-y+1/2,-z+1/2,x+1/2'\n\t'-z+1/2,y+1/2,-x+1/2'\n\t'z+1/2,-y+1/2,-x+1/2'\n");
      break;

    case 514: // [218] P -4 3 n (P -4n 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'y+1/2,-x+1/2,-z+1/2'\n\t'-x,-y,z'\n\t'-y+1/2,x+1/2,-z+1/2'\n\t'x,-y,-z'\n\t'-y+1/2,-x+1/2,z+1/2'\n\t'-x,y,-z'\n"
        "\t'y+1/2,x+1/2,z+1/2'\n\t'z,x,y'\n\t'x+1/2,-z+1/2,-y+1/2'\n\t'-z,-x,y'\n\t'-x+1/2,z+1/2,-y+1/2'\n\t'z,-x,-y'\n\t'-x+1/2,-z+1/2,y+1/2'\n"
        "\t'-z,x,-y'\n\t'x+1/2,z+1/2,y+1/2'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'-z+1/2,-y+1/2,x+1/2'\n\t'-y,z,-x'\n\t'z+1/2,y+1/2,x+1/2'\n"
        "\t'-y,-z,x'\n\t'-z+1/2,y+1/2,-x+1/2'\n\t'z+1/2,-y+1/2,-x+1/2'\n");
      break;

    case 515: // [219] F -4 3 c (F -4a 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'y+1/2,-x,-z'\n\t'-x+1/2,-y+1/2,z'\n\t'-y,x+1/2,-z'\n\t'x,-y,-z'\n\t'-y+1/2,-x,z'\n\t'-x+1/2,y+1/2,-z'\n"
        "\t'y,x+1/2,z'\n\t'z,x,y'\n\t'x+1/2,-z,-y'\n\t'-z+1/2,-x+1/2,y'\n\t'-x,z+1/2,-y'\n\t'z,-x,-y'\n\t'-x+1/2,-z,y'\n"
        "\t'-z+1/2,x+1/2,-y'\n\t'x,z+1/2,y'\n\t'y,z,x'\n\t'y,-z+1/2,-x+1/2'\n\t'-z,-y,x+1/2'\n\t'-y+1/2,z,-x+1/2'\n\t'z+1/2,y,x'\n"
        "\t'-y,-z,x'\n\t'-z,y,-x+1/2'\n\t'z+1/2,-y+1/2,-x+1/2'\n\t'x,y+1/2,z+1/2'\n\t'y+1/2,-x+1/2,-z+1/2'\n\t'-x+1/2,-y+1,z+1/2'\n\t'-y,x+1,-z+1/2'\n"
        "\t'x,-y+1/2,-z+1/2'\n\t'-y+1/2,-x+1/2,z+1/2'\n\t'-x+1/2,y+1,-z+1/2'\n\t'y,x+1,z+1/2'\n\t'z,x+1/2,y+1/2'\n\t'x+1/2,-z+1/2,-y+1/2'\n\t'-z+1/2,-x+1,y+1/2'\n"
        "\t'-x,z+1,-y+1/2'\n\t'z,-x+1/2,-y+1/2'\n\t'-x+1/2,-z+1/2,y+1/2'\n\t'-z+1/2,x+1,-y+1/2'\n\t'x,z+1,y+1/2'\n\t'y,z+1/2,x+1/2'\n\t'y,-z+1,-x+1'\n"
        "\t'-z,-y+1/2,x+1'\n\t'-y+1/2,z+1/2,-x+1'\n\t'z+1/2,y+1/2,x+1/2'\n\t'-y,-z+1/2,x+1/2'\n\t'-z,y+1/2,-x+1'\n\t'z+1/2,-y+1,-x+1'\n\t'x+1/2,y,z+1/2'\n"
        "\t'y+1,-x,-z+1/2'\n\t'-x+1,-y+1/2,z+1/2'\n\t'-y+1/2,x+1/2,-z+1/2'\n\t'x+1/2,-y,-z+1/2'\n\t'-y+1,-x,z+1/2'\n\t'-x+1,y+1/2,-z+1/2'\n\t'y+1/2,x+1/2,z+1/2'\n"
        "\t'z+1/2,x,y+1/2'\n\t'x+1,-z,-y+1/2'\n\t'-z+1,-x+1/2,y+1/2'\n\t'-x+1/2,z+1/2,-y+1/2'\n\t'z+1/2,-x,-y+1/2'\n\t'-x+1,-z,y+1/2'\n\t'-z+1,x+1/2,-y+1/2'\n"
        "\t'x+1/2,z+1/2,y+1/2'\n\t'y+1/2,z,x+1/2'\n\t'y+1/2,-z+1/2,-x+1'\n\t'-z+1/2,-y,x+1'\n\t'-y+1,z,-x+1'\n\t'z+1,y,x+1/2'\n\t'-y+1/2,-z,x+1/2'\n"
        "\t'-z+1/2,y,-x+1'\n\t'z+1,-y+1/2,-x+1'\n\t'x+1/2,y+1/2,z'\n\t'y+1,-x+1/2,-z'\n\t'-x+1,-y+1,z'\n\t'-y+1/2,x+1,-z'\n\t'x+1/2,-y+1/2,-z'\n"
        "\t'-y+1,-x+1/2,z'\n\t'-x+1,y+1,-z'\n\t'y+1/2,x+1,z'\n\t'z+1/2,x+1/2,y'\n\t'x+1,-z+1/2,-y'\n\t'-z+1,-x+1,y'\n\t'-x+1/2,z+1,-y'\n"
        "\t'z+1/2,-x+1/2,-y'\n\t'-x+1,-z+1/2,y'\n\t'-z+1,x+1,-y'\n\t'x+1/2,z+1,y'\n\t'y+1/2,z+1/2,x'\n\t'y+1/2,-z+1,-x+1/2'\n\t'-z+1/2,-y+1/2,x+1/2'\n"
        "\t'-y+1,z+1/2,-x+1/2'\n\t'z+1,y+1/2,x'\n\t'-y+1/2,-z+1/2,x'\n\t'-z+1/2,y+1/2,-x+1/2'\n\t'z+1,-y+1,-x+1/2'\n");
      break;

    case 516: // [220] I -4 3 d (I -4bd 2c 3)
      fprintf(Stream,"\t'x,y,z'\n\t'y+1/4,-x+3/4,-z+1/4'\n\t'-x,-y+1/2,z'\n\t'-y+3/4,x+3/4,-z+1/4'\n\t'x,-y,-z+1/2'\n\t'-y+1/4,-x+3/4,z+3/4'\n\t'-x,y+1/2,-z+1/2'\n"
        "\t'y+3/4,x+3/4,z+3/4'\n\t'z,x,y'\n\t'x+1/4,-z+3/4,-y+1/4'\n\t'-z,-x+1/2,y'\n\t'-x+3/4,z+3/4,-y+1/4'\n\t'z,-x,-y+1/2'\n\t'-x+1/4,-z+3/4,y+3/4'\n"
        "\t'-z,x+1/2,-y+1/2'\n\t'x+3/4,z+3/4,y+3/4'\n\t'y,z,x'\n\t'y,-z,-x+1/2'\n\t'-z+1/4,-y+3/4,x+3/4'\n\t'-y,z+1/2,-x+1/2'\n\t'z+1/4,y+1/4,x+1/4'\n"
        "\t'-y+1/2,-z,x+1/2'\n\t'-z+1/4,y+1/4,-x+3/4'\n\t'z+3/4,-y+1/4,-x+3/4'\n\t'x+1/2,y+1/2,z+1/2'\n\t'y+3/4,-x+1/4,-z+3/4'\n\t'-x+1/2,-y+1,z+1/2'\n\t'-y+1/4,x+1/4,-z+3/4'\n"
        "\t'x+1/2,-y+1/2,-z+1'\n\t'-y+3/4,-x+1/4,z+1/4'\n\t'-x+1/2,y+1,-z+1'\n\t'y+1/4,x+1/4,z+1/4'\n\t'z+1/2,x+1/2,y+1/2'\n\t'x+3/4,-z+1/4,-y+3/4'\n\t'-z+1/2,-x+1,y+1/2'\n"
        "\t'-x+1/4,z+1/4,-y+3/4'\n\t'z+1/2,-x+1/2,-y+1'\n\t'-x+3/4,-z+1/4,y+1/4'\n\t'-z+1/2,x+1,-y+1'\n\t'x+1/4,z+1/4,y+1/4'\n\t'y+1/2,z+1/2,x+1/2'\n\t'y+1/2,-z+1/2,-x+1'\n"
        "\t'-z+3/4,-y+1/4,x+1/4'\n\t'-y+1/2,z+1,-x+1'\n\t'z+3/4,y+3/4,x+3/4'\n\t'-y+1,-z+1/2,x+1'\n\t'-z+3/4,y+3/4,-x+1/4'\n\t'z+1/4,-y+3/4,-x+1/4'\n");
      break;

    case 517: // [221] P m -3 m (-P 4 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x,z'\n\t'-x,-y,z'\n\t'y,-x,z'\n\t'x,-y,-z'\n\t'y,x,-z'\n\t'-x,y,-z'\n"
        "\t'-y,-x,-z'\n\t'z,x,y'\n\t'-x,z,y'\n\t'-z,-x,y'\n\t'x,-z,y'\n\t'z,-x,-y'\n\t'x,z,-y'\n"
        "\t'-z,x,-y'\n\t'-x,-z,-y'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'z,y,-x'\n\t'-y,z,-x'\n\t'-z,-y,-x'\n"
        "\t'-y,-z,x'\n\t'z,-y,x'\n\t'-z,y,x'\n\t'-x,-y,-z'\n\t'y,-x,-z'\n\t'x,y,-z'\n\t'-y,x,-z'\n"
        "\t'-x,y,z'\n\t'-y,-x,z'\n\t'x,-y,z'\n\t'y,x,z'\n\t'-z,-x,-y'\n\t'x,-z,-y'\n\t'z,x,-y'\n"
        "\t'-x,z,-y'\n\t'-z,x,y'\n\t'-x,-z,y'\n\t'z,-x,y'\n\t'x,z,y'\n\t'-y,-z,-x'\n\t'-y,z,x'\n"
        "\t'-z,-y,x'\n\t'y,-z,x'\n\t'z,y,x'\n\t'y,z,-x'\n\t'-z,y,-x'\n\t'z,-y,-x'\n");
      break;

    case 518: // [222] P n -3 n Origin choice 1 (P 4 2 3 -1n)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x,z'\n\t'-x,-y,z'\n\t'y,-x,z'\n\t'x,-y,-z'\n\t'y,x,-z'\n\t'-x,y,-z'\n"
        "\t'-y,-x,-z'\n\t'z,x,y'\n\t'-x,z,y'\n\t'-z,-x,y'\n\t'x,-z,y'\n\t'z,-x,-y'\n\t'x,z,-y'\n"
        "\t'-z,x,-y'\n\t'-x,-z,-y'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'z,y,-x'\n\t'-y,z,-x'\n\t'-z,-y,-x'\n"
        "\t'-y,-z,x'\n\t'z,-y,x'\n\t'-z,y,x'\n\t'-x+1/2,-y+1/2,-z+1/2'\n\t'y+1/2,-x+1/2,-z+1/2'\n\t'x+1/2,y+1/2,-z+1/2'\n\t'-y+1/2,x+1/2,-z+1/2'\n"
        "\t'-x+1/2,y+1/2,z+1/2'\n\t'-y+1/2,-x+1/2,z+1/2'\n\t'x+1/2,-y+1/2,z+1/2'\n\t'y+1/2,x+1/2,z+1/2'\n\t'-z+1/2,-x+1/2,-y+1/2'\n\t'x+1/2,-z+1/2,-y+1/2'\n\t'z+1/2,x+1/2,-y+1/2'\n"
        "\t'-x+1/2,z+1/2,-y+1/2'\n\t'-z+1/2,x+1/2,y+1/2'\n\t'-x+1/2,-z+1/2,y+1/2'\n\t'z+1/2,-x+1/2,y+1/2'\n\t'x+1/2,z+1/2,y+1/2'\n\t'-y+1/2,-z+1/2,-x+1/2'\n\t'-y+1/2,z+1/2,x+1/2'\n"
        "\t'-z+1/2,-y+1/2,x+1/2'\n\t'y+1/2,-z+1/2,x+1/2'\n\t'z+1/2,y+1/2,x+1/2'\n\t'y+1/2,z+1/2,-x+1/2'\n\t'-z+1/2,y+1/2,-x+1/2'\n\t'z+1/2,-y+1/2,-x+1/2'\n");
      break;
    case 519: // [222] P n -3 n Origin choice 2 (-P 4a 2bc 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-y+1/2,x,z'\n\t'-x+1/2,-y+1/2,z'\n\t'y,-x+1/2,z'\n\t'x,-y+1/2,-z+1/2'\n\t'y,x,-z+1/2'\n\t'-x+1/2,y,-z+1/2'\n"
        "\t'-y+1/2,-x+1/2,-z+1/2'\n\t'z,x,y'\n\t'-x+1/2,z,y'\n\t'-z+1/2,-x+1/2,y'\n\t'x,-z+1/2,y'\n\t'z,-x+1/2,-y+1/2'\n\t'x,z,-y+1/2'\n"
        "\t'-z+1/2,x,-y+1/2'\n\t'-x+1/2,-z+1/2,-y+1/2'\n\t'y,z,x'\n\t'y,-z+1/2,-x+1/2'\n\t'z,y,-x+1/2'\n\t'-y+1/2,z,-x+1/2'\n\t'-z+1/2,-y+1/2,-x+1/2'\n"
        "\t'-y+1/2,-z+1/2,x'\n\t'z,-y+1/2,x'\n\t'-z+1/2,y,x'\n\t'-x,-y,-z'\n\t'y-1/2,-x,-z'\n\t'x-1/2,y-1/2,-z'\n\t'-y,x-1/2,-z'\n"
        "\t'-x,y-1/2,z-1/2'\n\t'-y,-x,z-1/2'\n\t'x-1/2,-y,z-1/2'\n\t'y-1/2,x-1/2,z-1/2'\n\t'-z,-x,-y'\n\t'x-1/2,-z,-y'\n\t'z-1/2,x-1/2,-y'\n"
        "\t'-x,z-1/2,-y'\n\t'-z,x-1/2,y-1/2'\n\t'-x,-z,y-1/2'\n\t'z-1/2,-x,y-1/2'\n\t'x-1/2,z-1/2,y-1/2'\n\t'-y,-z,-x'\n\t'-y,z-1/2,x-1/2'\n"
        "\t'-z,-y,x-1/2'\n\t'y-1/2,-z,x-1/2'\n\t'z-1/2,y-1/2,x-1/2'\n\t'y-1/2,z-1/2,-x'\n\t'-z,y-1/2,-x'\n\t'z-1/2,-y,-x'\n");
      break;

    case 520: // [223] P m -3 n (-P 4n 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-y+1/2,x+1/2,z+1/2'\n\t'-x,-y,z'\n\t'y+1/2,-x+1/2,z+1/2'\n\t'x,-y,-z'\n\t'y+1/2,x+1/2,-z+1/2'\n\t'-x,y,-z'\n"
        "\t'-y+1/2,-x+1/2,-z+1/2'\n\t'z,x,y'\n\t'-x+1/2,z+1/2,y+1/2'\n\t'-z,-x,y'\n\t'x+1/2,-z+1/2,y+1/2'\n\t'z,-x,-y'\n\t'x+1/2,z+1/2,-y+1/2'\n"
        "\t'-z,x,-y'\n\t'-x+1/2,-z+1/2,-y+1/2'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'z+1/2,y+1/2,-x+1/2'\n\t'-y,z,-x'\n\t'-z+1/2,-y+1/2,-x+1/2'\n"
        "\t'-y,-z,x'\n\t'z+1/2,-y+1/2,x+1/2'\n\t'-z+1/2,y+1/2,x+1/2'\n\t'-x,-y,-z'\n\t'y-1/2,-x-1/2,-z-1/2'\n\t'x,y,-z'\n\t'-y-1/2,x-1/2,-z-1/2'\n"
        "\t'-x,y,z'\n\t'-y-1/2,-x-1/2,z-1/2'\n\t'x,-y,z'\n\t'y-1/2,x-1/2,z-1/2'\n\t'-z,-x,-y'\n\t'x-1/2,-z-1/2,-y-1/2'\n\t'z,x,-y'\n"
        "\t'-x-1/2,z-1/2,-y-1/2'\n\t'-z,x,y'\n\t'-x-1/2,-z-1/2,y-1/2'\n\t'z,-x,y'\n\t'x-1/2,z-1/2,y-1/2'\n\t'-y,-z,-x'\n\t'-y,z,x'\n"
        "\t'-z-1/2,-y-1/2,x-1/2'\n\t'y,-z,x'\n\t'z-1/2,y-1/2,x-1/2'\n\t'y,z,-x'\n\t'-z-1/2,y-1/2,-x-1/2'\n\t'z-1/2,-y-1/2,-x-1/2'\n");
      break;

    case 521: // [224] P n -3 m Origin choice 1 (P 4n 2 3 -1n)
      fprintf(Stream,"\t'x,y,z'\n\t'-y+1/2,x+1/2,z+1/2'\n\t'-x,-y,z'\n\t'y+1/2,-x+1/2,z+1/2'\n\t'x,-y,-z'\n\t'y+1/2,x+1/2,-z+1/2'\n\t'-x,y,-z'\n"
        "\t'-y+1/2,-x+1/2,-z+1/2'\n\t'z,x,y'\n\t'-x+1/2,z+1/2,y+1/2'\n\t'-z,-x,y'\n\t'x+1/2,-z+1/2,y+1/2'\n\t'z,-x,-y'\n\t'x+1/2,z+1/2,-y+1/2'\n"
        "\t'-z,x,-y'\n\t'-x+1/2,-z+1/2,-y+1/2'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'z+1/2,y+1/2,-x+1/2'\n\t'-y,z,-x'\n\t'-z+1/2,-y+1/2,-x+1/2'\n"
        "\t'-y,-z,x'\n\t'z+1/2,-y+1/2,x+1/2'\n\t'-z+1/2,y+1/2,x+1/2'\n\t'-x+1/2,-y+1/2,-z+1/2'\n\t'y,-x,-z'\n\t'x+1/2,y+1/2,-z+1/2'\n\t'-y,x,-z'\n"
        "\t'-x+1/2,y+1/2,z+1/2'\n\t'-y,-x,z'\n\t'x+1/2,-y+1/2,z+1/2'\n\t'y,x,z'\n\t'-z+1/2,-x+1/2,-y+1/2'\n\t'x,-z,-y'\n\t'z+1/2,x+1/2,-y+1/2'\n"
        "\t'-x,z,-y'\n\t'-z+1/2,x+1/2,y+1/2'\n\t'-x,-z,y'\n\t'z+1/2,-x+1/2,y+1/2'\n\t'x,z,y'\n\t'-y+1/2,-z+1/2,-x+1/2'\n\t'-y+1/2,z+1/2,x+1/2'\n"
        "\t'-z,-y,x'\n\t'y+1/2,-z+1/2,x+1/2'\n\t'z,y,x'\n\t'y+1/2,z+1/2,-x+1/2'\n\t'-z,y,-x'\n\t'z,-y,-x'\n");
      break;
    case 522: // [224] P n -3 m Origin choice 2 (-P 4bc 2bc 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x+1/2,z+1/2'\n\t'-x+1/2,-y+1/2,z'\n\t'y+1/2,-x,z+1/2'\n\t'x,-y+1/2,-z+1/2'\n\t'y+1/2,x+1/2,-z'\n\t'-x+1/2,y,-z+1/2'\n"
        "\t'-y,-x,-z'\n\t'z,x,y'\n\t'-x,z+1/2,y+1/2'\n\t'-z+1/2,-x+1/2,y'\n\t'x+1/2,-z,y+1/2'\n\t'z,-x+1/2,-y+1/2'\n\t'x+1/2,z+1/2,-y'\n"
        "\t'-z+1/2,x,-y+1/2'\n\t'-x,-z,-y'\n\t'y,z,x'\n\t'y,-z+1/2,-x+1/2'\n\t'z+1/2,y+1/2,-x'\n\t'-y+1/2,z,-x+1/2'\n\t'-z,-y,-x'\n"
        "\t'-y+1/2,-z+1/2,x'\n\t'z+1/2,-y,x+1/2'\n\t'-z,y+1/2,x+1/2'\n\t'-x,-y,-z'\n\t'y,-x-1/2,-z-1/2'\n\t'x-1/2,y-1/2,-z'\n\t'-y-1/2,x,-z-1/2'\n"
        "\t'-x,y-1/2,z-1/2'\n\t'-y-1/2,-x-1/2,z'\n\t'x-1/2,-y,z-1/2'\n\t'y,x,z'\n\t'-z,-x,-y'\n\t'x,-z-1/2,-y-1/2'\n\t'z-1/2,x-1/2,-y'\n"
        "\t'-x-1/2,z,-y-1/2'\n\t'-z,x-1/2,y-1/2'\n\t'-x-1/2,-z-1/2,y'\n\t'z-1/2,-x,y-1/2'\n\t'x,z,y'\n\t'-y,-z,-x'\n\t'-y,z-1/2,x-1/2'\n"
        "\t'-z-1/2,-y-1/2,x'\n\t'y-1/2,-z,x-1/2'\n\t'z,y,x'\n\t'y-1/2,z-1/2,-x'\n\t'-z-1/2,y,-x-1/2'\n\t'z,-y-1/2,-x-1/2'\n");
      break;

    case 523: // [225] F m -3 m (-F 4 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x,z'\n\t'-x,-y,z'\n\t'y,-x,z'\n\t'x,-y,-z'\n\t'y,x,-z'\n\t'-x,y,-z'\n"
        "\t'-y,-x,-z'\n\t'z,x,y'\n\t'-x,z,y'\n\t'-z,-x,y'\n\t'x,-z,y'\n\t'z,-x,-y'\n\t'x,z,-y'\n"
        "\t'-z,x,-y'\n\t'-x,-z,-y'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'z,y,-x'\n\t'-y,z,-x'\n\t'-z,-y,-x'\n"
        "\t'-y,-z,x'\n\t'z,-y,x'\n\t'-z,y,x'\n\t'-x,-y,-z'\n\t'y,-x,-z'\n\t'x,y,-z'\n\t'-y,x,-z'\n"
        "\t'-x,y,z'\n\t'-y,-x,z'\n\t'x,-y,z'\n\t'y,x,z'\n\t'-z,-x,-y'\n\t'x,-z,-y'\n\t'z,x,-y'\n"
        "\t'-x,z,-y'\n\t'-z,x,y'\n\t'-x,-z,y'\n\t'z,-x,y'\n\t'x,z,y'\n\t'-y,-z,-x'\n\t'-y,z,x'\n"
        "\t'-z,-y,x'\n\t'y,-z,x'\n\t'z,y,x'\n\t'y,z,-x'\n\t'-z,y,-x'\n\t'z,-y,-x'\n\t'x,y+1/2,z+1/2'\n"
        "\t'-y,x+1/2,z+1/2'\n\t'-x,-y+1/2,z+1/2'\n\t'y,-x+1/2,z+1/2'\n\t'x,-y+1/2,-z+1/2'\n\t'y,x+1/2,-z+1/2'\n\t'-x,y+1/2,-z+1/2'\n\t'-y,-x+1/2,-z+1/2'\n"
        "\t'z,x+1/2,y+1/2'\n\t'-x,z+1/2,y+1/2'\n\t'-z,-x+1/2,y+1/2'\n\t'x,-z+1/2,y+1/2'\n\t'z,-x+1/2,-y+1/2'\n\t'x,z+1/2,-y+1/2'\n\t'-z,x+1/2,-y+1/2'\n"
        "\t'-x,-z+1/2,-y+1/2'\n\t'y,z+1/2,x+1/2'\n\t'y,-z+1/2,-x+1/2'\n\t'z,y+1/2,-x+1/2'\n\t'-y,z+1/2,-x+1/2'\n\t'-z,-y+1/2,-x+1/2'\n\t'-y,-z+1/2,x+1/2'\n"
        "\t'z,-y+1/2,x+1/2'\n\t'-z,y+1/2,x+1/2'\n\t'-x,-y+1/2,-z+1/2'\n\t'y,-x+1/2,-z+1/2'\n\t'x,y+1/2,-z+1/2'\n\t'-y,x+1/2,-z+1/2'\n\t'-x,y+1/2,z+1/2'\n"
        "\t'-y,-x+1/2,z+1/2'\n\t'x,-y+1/2,z+1/2'\n\t'y,x+1/2,z+1/2'\n\t'-z,-x+1/2,-y+1/2'\n\t'x,-z+1/2,-y+1/2'\n\t'z,x+1/2,-y+1/2'\n\t'-x,z+1/2,-y+1/2'\n"
        "\t'-z,x+1/2,y+1/2'\n\t'-x,-z+1/2,y+1/2'\n\t'z,-x+1/2,y+1/2'\n\t'x,z+1/2,y+1/2'\n\t'-y,-z+1/2,-x+1/2'\n\t'-y,z+1/2,x+1/2'\n\t'-z,-y+1/2,x+1/2'\n"
        "\t'y,-z+1/2,x+1/2'\n\t'z,y+1/2,x+1/2'\n\t'y,z+1/2,-x+1/2'\n\t'-z,y+1/2,-x+1/2'\n\t'z,-y+1/2,-x+1/2'\n\t'x+1/2,y,z+1/2'\n\t'-y+1/2,x,z+1/2'\n"
        "\t'-x+1/2,-y,z+1/2'\n\t'y+1/2,-x,z+1/2'\n\t'x+1/2,-y,-z+1/2'\n\t'y+1/2,x,-z+1/2'\n\t'-x+1/2,y,-z+1/2'\n\t'-y+1/2,-x,-z+1/2'\n\t'z+1/2,x,y+1/2'\n"
        "\t'-x+1/2,z,y+1/2'\n\t'-z+1/2,-x,y+1/2'\n\t'x+1/2,-z,y+1/2'\n\t'z+1/2,-x,-y+1/2'\n\t'x+1/2,z,-y+1/2'\n\t'-z+1/2,x,-y+1/2'\n\t'-x+1/2,-z,-y+1/2'\n"
        "\t'y+1/2,z,x+1/2'\n\t'y+1/2,-z,-x+1/2'\n\t'z+1/2,y,-x+1/2'\n\t'-y+1/2,z,-x+1/2'\n\t'-z+1/2,-y,-x+1/2'\n\t'-y+1/2,-z,x+1/2'\n\t'z+1/2,-y,x+1/2'\n"
        "\t'-z+1/2,y,x+1/2'\n\t'-x+1/2,-y,-z+1/2'\n\t'y+1/2,-x,-z+1/2'\n\t'x+1/2,y,-z+1/2'\n\t'-y+1/2,x,-z+1/2'\n\t'-x+1/2,y,z+1/2'\n\t'-y+1/2,-x,z+1/2'\n"
        "\t'x+1/2,-y,z+1/2'\n\t'y+1/2,x,z+1/2'\n\t'-z+1/2,-x,-y+1/2'\n\t'x+1/2,-z,-y+1/2'\n\t'z+1/2,x,-y+1/2'\n\t'-x+1/2,z,-y+1/2'\n\t'-z+1/2,x,y+1/2'\n"
        "\t'-x+1/2,-z,y+1/2'\n\t'z+1/2,-x,y+1/2'\n\t'x+1/2,z,y+1/2'\n\t'-y+1/2,-z,-x+1/2'\n\t'-y+1/2,z,x+1/2'\n\t'-z+1/2,-y,x+1/2'\n\t'y+1/2,-z,x+1/2'\n"
        "\t'z+1/2,y,x+1/2'\n\t'y+1/2,z,-x+1/2'\n\t'-z+1/2,y,-x+1/2'\n\t'z+1/2,-y,-x+1/2'\n\t'x+1/2,y+1/2,z'\n\t'-y+1/2,x+1/2,z'\n\t'-x+1/2,-y+1/2,z'\n"
        "\t'y+1/2,-x+1/2,z'\n\t'x+1/2,-y+1/2,-z'\n\t'y+1/2,x+1/2,-z'\n\t'-x+1/2,y+1/2,-z'\n\t'-y+1/2,-x+1/2,-z'\n\t'z+1/2,x+1/2,y'\n\t'-x+1/2,z+1/2,y'\n"
        "\t'-z+1/2,-x+1/2,y'\n\t'x+1/2,-z+1/2,y'\n\t'z+1/2,-x+1/2,-y'\n\t'x+1/2,z+1/2,-y'\n\t'-z+1/2,x+1/2,-y'\n\t'-x+1/2,-z+1/2,-y'\n\t'y+1/2,z+1/2,x'\n"
        "\t'y+1/2,-z+1/2,-x'\n\t'z+1/2,y+1/2,-x'\n\t'-y+1/2,z+1/2,-x'\n\t'-z+1/2,-y+1/2,-x'\n\t'-y+1/2,-z+1/2,x'\n\t'z+1/2,-y+1/2,x'\n\t'-z+1/2,y+1/2,x'\n"
        "\t'-x+1/2,-y+1/2,-z'\n\t'y+1/2,-x+1/2,-z'\n\t'x+1/2,y+1/2,-z'\n\t'-y+1/2,x+1/2,-z'\n\t'-x+1/2,y+1/2,z'\n\t'-y+1/2,-x+1/2,z'\n\t'x+1/2,-y+1/2,z'\n"
        "\t'y+1/2,x+1/2,z'\n\t'-z+1/2,-x+1/2,-y'\n\t'x+1/2,-z+1/2,-y'\n\t'z+1/2,x+1/2,-y'\n\t'-x+1/2,z+1/2,-y'\n\t'-z+1/2,x+1/2,y'\n\t'-x+1/2,-z+1/2,y'\n"
        "\t'z+1/2,-x+1/2,y'\n\t'x+1/2,z+1/2,y'\n\t'-y+1/2,-z+1/2,-x'\n\t'-y+1/2,z+1/2,x'\n\t'-z+1/2,-y+1/2,x'\n\t'y+1/2,-z+1/2,x'\n\t'z+1/2,y+1/2,x'\n"
        "\t'y+1/2,z+1/2,-x'\n\t'-z+1/2,y+1/2,-x'\n\t'z+1/2,-y+1/2,-x'\n");
      break;

    case 524: // [226] F m -3 c (-F 4a 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-y+1/2,x,z'\n\t'-x+1/2,-y+1/2,z'\n\t'y,-x+1/2,z'\n\t'x,-y,-z'\n\t'y+1/2,x,-z'\n\t'-x+1/2,y+1/2,-z'\n"
        "\t'-y,-x+1/2,-z'\n\t'z,x,y'\n\t'-x+1/2,z,y'\n\t'-z+1/2,-x+1/2,y'\n\t'x,-z+1/2,y'\n\t'z,-x,-y'\n\t'x+1/2,z,-y'\n"
        "\t'-z+1/2,x+1/2,-y'\n\t'-x,-z+1/2,-y'\n\t'y,z,x'\n\t'y,-z+1/2,-x+1/2'\n\t'z,y,-x+1/2'\n\t'-y+1/2,z,-x+1/2'\n\t'-z+1/2,-y,-x'\n"
        "\t'-y,-z,x'\n\t'z,-y,x+1/2'\n\t'-z+1/2,y+1/2,x+1/2'\n\t'-x,-y,-z'\n\t'y-1/2,-x,-z'\n\t'x-1/2,y-1/2,-z'\n\t'-y,x-1/2,-z'\n"
        "\t'-x,y,z'\n\t'-y-1/2,-x,z'\n\t'x-1/2,-y-1/2,z'\n\t'y,x-1/2,z'\n\t'-z,-x,-y'\n\t'x-1/2,-z,-y'\n\t'z-1/2,x-1/2,-y'\n"
        "\t'-x,z-1/2,-y'\n\t'-z,x,y'\n\t'-x-1/2,-z,y'\n\t'z-1/2,-x-1/2,y'\n\t'x,z-1/2,y'\n\t'-y,-z,-x'\n\t'-y,z-1/2,x-1/2'\n"
        "\t'-z,-y,x-1/2'\n\t'y-1/2,-z,x-1/2'\n\t'z-1/2,y,x'\n\t'y,z,-x'\n\t'-z,y,-x-1/2'\n\t'z-1/2,-y-1/2,-x-1/2'\n\t'x,y+1/2,z+1/2'\n"
        "\t'-y+1/2,x+1/2,z+1/2'\n\t'-x+1/2,-y+1,z+1/2'\n\t'y,-x+1,z+1/2'\n\t'x,-y+1/2,-z+1/2'\n\t'y+1/2,x+1/2,-z+1/2'\n\t'-x+1/2,y+1,-z+1/2'\n\t'-y,-x+1,-z+1/2'\n"
        "\t'z,x+1/2,y+1/2'\n\t'-x+1/2,z+1/2,y+1/2'\n\t'-z+1/2,-x+1,y+1/2'\n\t'x,-z+1,y+1/2'\n\t'z,-x+1/2,-y+1/2'\n\t'x+1/2,z+1/2,-y+1/2'\n\t'-z+1/2,x+1,-y+1/2'\n"
        "\t'-x,-z+1,-y+1/2'\n\t'y,z+1/2,x+1/2'\n\t'y,-z+1,-x+1'\n\t'z,y+1/2,-x+1'\n\t'-y+1/2,z+1/2,-x+1'\n\t'-z+1/2,-y+1/2,-x+1/2'\n\t'-y,-z+1/2,x+1/2'\n"
        "\t'z,-y+1/2,x+1'\n\t'-z+1/2,y+1,x+1'\n\t'-x,-y+1/2,-z+1/2'\n\t'y-1/2,-x+1/2,-z+1/2'\n\t'x-1/2,y,-z+1/2'\n\t'-y,x,-z+1/2'\n\t'-x,y+1/2,z+1/2'\n"
        "\t'-y-1/2,-x+1/2,z+1/2'\n\t'x-1/2,-y,z+1/2'\n\t'y,x,z+1/2'\n\t'-z,-x+1/2,-y+1/2'\n\t'x-1/2,-z+1/2,-y+1/2'\n\t'z-1/2,x,-y+1/2'\n\t'-x,z,-y+1/2'\n"
        "\t'-z,x+1/2,y+1/2'\n\t'-x-1/2,-z+1/2,y+1/2'\n\t'z-1/2,-x,y+1/2'\n\t'x,z,y+1/2'\n\t'-y,-z+1/2,-x+1/2'\n\t'-y,z,x'\n\t'-z,-y+1/2,x'\n"
        "\t'y-1/2,-z+1/2,x'\n\t'z-1/2,y+1/2,x+1/2'\n\t'y,z+1/2,-x+1/2'\n\t'-z,y+1/2,-x'\n\t'z-1/2,-y,-x'\n\t'x+1/2,y,z+1/2'\n\t'-y+1,x,z+1/2'\n"
        "\t'-x+1,-y+1/2,z+1/2'\n\t'y+1/2,-x+1/2,z+1/2'\n\t'x+1/2,-y,-z+1/2'\n\t'y+1,x,-z+1/2'\n\t'-x+1,y+1/2,-z+1/2'\n\t'-y+1/2,-x+1/2,-z+1/2'\n\t'z+1/2,x,y+1/2'\n"
        "\t'-x+1,z,y+1/2'\n\t'-z+1,-x+1/2,y+1/2'\n\t'x+1/2,-z+1/2,y+1/2'\n\t'z+1/2,-x,-y+1/2'\n\t'x+1,z,-y+1/2'\n\t'-z+1,x+1/2,-y+1/2'\n\t'-x+1/2,-z+1/2,-y+1/2'\n"
        "\t'y+1/2,z,x+1/2'\n\t'y+1/2,-z+1/2,-x+1'\n\t'z+1/2,y,-x+1'\n\t'-y+1,z,-x+1'\n\t'-z+1,-y,-x+1/2'\n\t'-y+1/2,-z,x+1/2'\n\t'z+1/2,-y,x+1'\n"
        "\t'-z+1,y+1/2,x+1'\n\t'-x+1/2,-y,-z+1/2'\n\t'y,-x,-z+1/2'\n\t'x,y-1/2,-z+1/2'\n\t'-y+1/2,x-1/2,-z+1/2'\n\t'-x+1/2,y,z+1/2'\n\t'-y,-x,z+1/2'\n"
        "\t'x,-y-1/2,z+1/2'\n\t'y+1/2,x-1/2,z+1/2'\n\t'-z+1/2,-x,-y+1/2'\n\t'x,-z,-y+1/2'\n\t'z,x-1/2,-y+1/2'\n\t'-x+1/2,z-1/2,-y+1/2'\n\t'-z+1/2,x,y+1/2'\n"
        "\t'-x,-z,y+1/2'\n\t'z,-x-1/2,y+1/2'\n\t'x+1/2,z-1/2,y+1/2'\n\t'-y+1/2,-z,-x+1/2'\n\t'-y+1/2,z-1/2,x'\n\t'-z+1/2,-y,x'\n\t'y,-z,x'\n"
        "\t'z,y,x+1/2'\n\t'y+1/2,z,-x+1/2'\n\t'-z+1/2,y,-x'\n\t'z,-y-1/2,-x'\n\t'x+1/2,y+1/2,z'\n\t'-y+1,x+1/2,z'\n\t'-x+1,-y+1,z'\n"
        "\t'y+1/2,-x+1,z'\n\t'x+1/2,-y+1/2,-z'\n\t'y+1,x+1/2,-z'\n\t'-x+1,y+1,-z'\n\t'-y+1/2,-x+1,-z'\n\t'z+1/2,x+1/2,y'\n\t'-x+1,z+1/2,y'\n"
        "\t'-z+1,-x+1,y'\n\t'x+1/2,-z+1,y'\n\t'z+1/2,-x+1/2,-y'\n\t'x+1,z+1/2,-y'\n\t'-z+1,x+1,-y'\n\t'-x+1/2,-z+1,-y'\n\t'y+1/2,z+1/2,x'\n"
        "\t'y+1/2,-z+1,-x+1/2'\n\t'z+1/2,y+1/2,-x+1/2'\n\t'-y+1,z+1/2,-x+1/2'\n\t'-z+1,-y+1/2,-x'\n\t'-y+1/2,-z+1/2,x'\n\t'z+1/2,-y+1/2,x+1/2'\n\t'-z+1,y+1,x+1/2'\n"
        "\t'-x+1/2,-y+1/2,-z'\n\t'y,-x+1/2,-z'\n\t'x,y,-z'\n\t'-y+1/2,x,-z'\n\t'-x+1/2,y+1/2,z'\n\t'-y,-x+1/2,z'\n\t'x,-y,z'\n"
        "\t'y+1/2,x,z'\n\t'-z+1/2,-x+1/2,-y'\n\t'x,-z+1/2,-y'\n\t'z,x,-y'\n\t'-x+1/2,z,-y'\n\t'-z+1/2,x+1/2,y'\n\t'-x,-z+1/2,y'\n"
        "\t'z,-x,y'\n\t'x+1/2,z,y'\n\t'-y+1/2,-z+1/2,-x'\n\t'-y+1/2,z,x-1/2'\n\t'-z+1/2,-y+1/2,x-1/2'\n\t'y,-z+1/2,x-1/2'\n\t'z,y+1/2,x'\n"
        "\t'y+1/2,z+1/2,-x'\n\t'-z+1/2,y+1/2,-x-1/2'\n\t'z,-y,-x-1/2'\n");
      break;

    case 525: // [227] F d -3 m Origin choice 1 (F 4d 2 3 -1d)
      fprintf(Stream,"\t'x,y,z'\n\t'-y+1/4,x+1/4,z+1/4'\n\t'-x,-y+1/2,z+1/2'\n\t'y+3/4,-x+1/4,z+3/4'\n\t'x,-y,-z'\n\t'y+1/4,x+1/4,-z+1/4'\n\t'-x,y+1/2,-z+1/2'\n"
        "\t'-y+3/4,-x+1/4,-z+3/4'\n\t'z,x,y'\n\t'-x+1/4,z+1/4,y+1/4'\n\t'-z,-x+1/2,y+1/2'\n\t'x+3/4,-z+1/4,y+3/4'\n\t'z,-x,-y'\n\t'x+1/4,z+1/4,-y+1/4'\n"
        "\t'-z,x+1/2,-y+1/2'\n\t'-x+3/4,-z+1/4,-y+3/4'\n\t'y,z,x'\n\t'y+1/2,-z,-x+1/2'\n\t'z+1/4,y+3/4,-x+3/4'\n\t'-y+1/2,z+1/2,-x'\n\t'-z+1/4,-y+1/4,-x+1/4'\n"
        "\t'-y,-z,x'\n\t'z+1/4,-y+3/4,x+3/4'\n\t'-z+3/4,y+3/4,x+1/4'\n\t'-x+1/4,-y+1/4,-z+1/4'\n\t'y,-x,-z'\n\t'x+1/4,y-1/4,-z-1/4'\n\t'-y-1/2,x,-z-1/2'\n"
        "\t'-x+1/4,y+1/4,z+1/4'\n\t'-y,-x,z'\n\t'x+1/4,-y-1/4,z-1/4'\n\t'y-1/2,x,z-1/2'\n\t'-z+1/4,-x+1/4,-y+1/4'\n\t'x,-z,-y'\n\t'z+1/4,x-1/4,-y-1/4'\n"
        "\t'-x-1/2,z,-y-1/2'\n\t'-z+1/4,x+1/4,y+1/4'\n\t'-x,-z,y'\n\t'z+1/4,-x-1/4,y-1/4'\n\t'x-1/2,z,y-1/2'\n\t'-y+1/4,-z+1/4,-x+1/4'\n\t'-y-1/4,z+1/4,x-1/4'\n"
        "\t'-z,-y-1/2,x-1/2'\n\t'y-1/4,-z-1/4,x+1/4'\n\t'z,y,x'\n\t'y+1/4,z+1/4,-x+1/4'\n\t'-z,y-1/2,-x-1/2'\n\t'z-1/2,-y-1/2,-x'\n\t'x,y+1/2,z+1/2'\n"
        "\t'-y+1/4,x+3/4,z+3/4'\n\t'-x,-y+1,z+1'\n\t'y+3/4,-x+3/4,z+0.25'\n\t'x,-y+1/2,-z+1/2'\n\t'y+1/4,x+3/4,-z+3/4'\n\t'-x,y+1,-z+1'\n\t'-y+3/4,-x+3/4,-z+0.25'\n"
        "\t'z,x+1/2,y+1/2'\n\t'-x+1/4,z+3/4,y+3/4'\n\t'-z,-x+1,y+1'\n\t'x+3/4,-z+3/4,y+0.25'\n\t'z,-x+1/2,-y+1/2'\n\t'x+1/4,z+3/4,-y+3/4'\n\t'-z,x+1,-y+1'\n"
        "\t'-x+3/4,-z+3/4,-y+0.25'\n\t'y,z+1/2,x+1/2'\n\t'y+1/2,-z+1/2,-x+1'\n\t'z+1/4,y+0.25,-x+0.25'\n\t'-y+1/2,z+1,-x+1/2'\n\t'-z+1/4,-y+3/4,-x+3/4'\n\t'-y,-z+1/2,x+1/2'\n"
        "\t'z+1/4,-y+0.25,x+0.25'\n\t'-z+3/4,y+0.25,x+3/4'\n\t'-x+1/4,-y+3/4,-z+3/4'\n\t'y,-x+1/2,-z+1/2'\n\t'x+1/4,y+1/4,-z+1/4'\n\t'-y-1/2,x+1/2,-z'\n\t'-x+1/4,y+3/4,z+3/4'\n"
        "\t'-y,-x+1/2,z+1/2'\n\t'x+1/4,-y+1/4,z+1/4'\n\t'y-1/2,x+1/2,z'\n\t'-z+1/4,-x+3/4,-y+3/4'\n\t'x,-z+1/2,-y+1/2'\n\t'z+1/4,x+1/4,-y+1/4'\n\t'-x-1/2,z+1/2,-y'\n"
        "\t'-z+1/4,x+3/4,y+3/4'\n\t'-x,-z+1/2,y+1/2'\n\t'z+1/4,-x+1/4,y+1/4'\n\t'x-1/2,z+1/2,y'\n\t'-y+1/4,-z+3/4,-x+3/4'\n\t'-y-1/4,z+3/4,x+1/4'\n\t'-z,-y,x'\n"
        "\t'y-1/4,-z+1/4,x+3/4'\n\t'z,y+1/2,x+1/2'\n\t'y+1/4,z+3/4,-x+3/4'\n\t'-z,y,-x'\n\t'z-1/2,-y,-x+1/2'\n\t'x+1/2,y,z+1/2'\n\t'-y+3/4,x+1/4,z+3/4'\n"
        "\t'-x+1/2,-y+1/2,z+1'\n\t'y+0.25,-x+1/4,z+0.25'\n\t'x+1/2,-y,-z+1/2'\n\t'y+3/4,x+1/4,-z+3/4'\n\t'-x+1/2,y+1/2,-z+1'\n\t'-y+0.25,-x+1/4,-z+0.25'\n\t'z+1/2,x,y+1/2'\n"
        "\t'-x+3/4,z+1/4,y+3/4'\n\t'-z+1/2,-x+1/2,y+1'\n\t'x+0.25,-z+1/4,y+0.25'\n\t'z+1/2,-x,-y+1/2'\n\t'x+3/4,z+1/4,-y+3/4'\n\t'-z+1/2,x+1/2,-y+1'\n\t'-x+0.25,-z+1/4,-y+0.25'\n"
        "\t'y+1/2,z,x+1/2'\n\t'y+1,-z,-x+1'\n\t'z+3/4,y+3/4,-x+0.25'\n\t'-y+1,z+1/2,-x+1/2'\n\t'-z+3/4,-y+1/4,-x+3/4'\n\t'-y+1/2,-z,x+1/2'\n\t'z+3/4,-y+3/4,x+0.25'\n"
        "\t'-z+0.25,y+3/4,x+3/4'\n\t'-x+3/4,-y+1/4,-z+3/4'\n\t'y+1/2,-x,-z+1/2'\n\t'x+3/4,y-1/4,-z+1/4'\n\t'-y,x,-z'\n\t'-x+3/4,y+1/4,z+3/4'\n\t'-y+1/2,-x,z+1/2'\n"
        "\t'x+3/4,-y-1/4,z+1/4'\n\t'y,x,z'\n\t'-z+3/4,-x+1/4,-y+3/4'\n\t'x+1/2,-z,-y+1/2'\n\t'z+3/4,x-1/4,-y+1/4'\n\t'-x,z,-y'\n\t'-z+3/4,x+1/4,y+3/4'\n"
        "\t'-x+1/2,-z,y+1/2'\n\t'z+3/4,-x-1/4,y+1/4'\n\t'x,z,y'\n\t'-y+3/4,-z+1/4,-x+3/4'\n\t'-y+1/4,z+1/4,x+1/4'\n\t'-z+1/2,-y-1/2,x'\n\t'y+1/4,-z-1/4,x+3/4'\n"
        "\t'z+1/2,y,x+1/2'\n\t'y+3/4,z+1/4,-x+3/4'\n\t'-z+1/2,y-1/2,-x'\n\t'z,-y-1/2,-x+1/2'\n\t'x+1/2,y+1/2,z'\n\t'-y+3/4,x+3/4,z+1/4'\n\t'-x+1/2,-y+1,z+1/2'\n"
        "\t'y+0.25,-x+3/4,z+3/4'\n\t'x+1/2,-y+1/2,-z'\n\t'y+3/4,x+3/4,-z+1/4'\n\t'-x+1/2,y+1,-z+1/2'\n\t'-y+0.25,-x+3/4,-z+3/4'\n\t'z+1/2,x+1/2,y'\n\t'-x+3/4,z+3/4,y+1/4'\n"
        "\t'-z+1/2,-x+1,y+1/2'\n\t'x+0.25,-z+3/4,y+3/4'\n\t'z+1/2,-x+1/2,-y'\n\t'x+3/4,z+3/4,-y+1/4'\n\t'-z+1/2,x+1,-y+1/2'\n\t'-x+0.25,-z+3/4,-y+3/4'\n\t'y+1/2,z+1/2,x'\n"
        "\t'y+1,-z+1/2,-x+1/2'\n\t'z+3/4,y+0.25,-x+3/4'\n\t'-y+1,z+1,-x'\n\t'-z+3/4,-y+3/4,-x+1/4'\n\t'-y+1/2,-z+1/2,x'\n\t'z+3/4,-y+0.25,x+3/4'\n\t'-z+0.25,y+0.25,x+1/4'\n"
        "\t'-x+3/4,-y+3/4,-z+1/4'\n\t'y+1/2,-x+1/2,-z'\n\t'x+3/4,y+1/4,-z-1/4'\n\t'-y,x+1/2,-z-1/2'\n\t'-x+3/4,y+3/4,z+1/4'\n\t'-y+1/2,-x+1/2,z'\n\t'x+3/4,-y+1/4,z-1/4'\n"
        "\t'y,x+1/2,z-1/2'\n\t'-z+3/4,-x+3/4,-y+1/4'\n\t'x+1/2,-z+1/2,-y'\n\t'z+3/4,x+1/4,-y-1/4'\n\t'-x,z+1/2,-y-1/2'\n\t'-z+3/4,x+3/4,y+1/4'\n\t'-x+1/2,-z+1/2,y'\n"
        "\t'z+3/4,-x+1/4,y-1/4'\n\t'x,z+1/2,y-1/2'\n\t'-y+3/4,-z+3/4,-x+1/4'\n\t'-y+1/4,z+3/4,x-1/4'\n\t'-z+1/2,-y,x-1/2'\n\t'y+1/4,-z+1/4,x+1/4'\n\t'z+1/2,y+1/2,x'\n"
        "\t'y+3/4,z+3/4,-x+1/4'\n\t'-z+1/2,y,-x-1/2'\n\t'z,-y,-x'\n");
      break;
    case 526: // [227] F d -3 m Origin choice 2 (-F 4vw 2vw 3)      e.g. MIL-100,101
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x+1/4,z+1/4'\n\t'-x+3/4,-y+1/4,z+1/2'\n\t'y+3/4,-x,z+3/4'\n\t'x,-y+1/4,-z+1/4'\n\t'y+3/4,x+1/4,-z+1/2'\n\t'-x+3/4,y,-z+3/4'\n"
        "\t'-y,-x,-z'\n\t'z,x,y'\n\t'-x,z+1/4,y+1/4'\n\t'-z+3/4,-x+1/4,y+1/2'\n\t'x+3/4,-z,y+3/4'\n\t'z,-x+1/4,-y+1/4'\n\t'x+3/4,z+1/4,-y+1/2'\n"
        "\t'-z+3/4,x,-y+3/4'\n\t'-x,-z,-y'\n\t'y,z,x'\n\t'y+1/2,-z+3/4,-x+1/4'\n\t'z+1/4,y+3/4,-x+1/2'\n\t'-y+1/4,z+1/2,-x+3/4'\n\t'-z,-y+1/2,-x+1/2'\n"
        "\t'-y+1/4,-z+1/4,x'\n\t'z+1/4,-y,x+1/4'\n\t'-z+1/2,y+1/4,x+3/4'\n\t'-x,-y,-z'\n\t'y,-x-1/4,-z-1/4'\n\t'x-3/4,y-1/4,-z-1/2'\n\t'-y-3/4,x,-z-3/4'\n"
        "\t'-x,y-1/4,z-1/4'\n\t'-y-3/4,-x-1/4,z-1/2'\n\t'x-3/4,-y,z-3/4'\n\t'y,x,z'\n\t'-z,-x,-y'\n\t'x,-z-1/4,-y-1/4'\n\t'z-3/4,x-1/4,-y-1/2'\n"
        "\t'-x-3/4,z,-y-3/4'\n\t'-z,x-1/4,y-1/4'\n\t'-x-3/4,-z-1/4,y-1/2'\n\t'z-3/4,-x,y-3/4'\n\t'x,z,y'\n\t'-y,-z,-x'\n\t'-y-1/2,z-3/4,x-1/4'\n"
        "\t'-z-1/4,-y-3/4,x-1/2'\n\t'y-1/4,-z-1/2,x-3/4'\n\t'z,y-1/2,x-1/2'\n\t'y-1/4,z-1/4,-x'\n\t'-z-1/4,y,-x-1/4'\n\t'z-1/2,-y-1/4,-x-3/4'\n\t'x,y+1/2,z+1/2'\n"
        "\t'-y,x+3/4,z+3/4'\n\t'-x+3/4,-y+3/4,z+1'\n\t'y+3/4,-x+1/2,z+0.25'\n\t'x,-y+3/4,-z+3/4'\n\t'y+3/4,x+3/4,-z+1'\n\t'-x+3/4,y+1/2,-z+0.25'\n\t'-y,-x+1/2,-z+1/2'\n"
        "\t'z,x+1/2,y+1/2'\n\t'-x,z+3/4,y+3/4'\n\t'-z+3/4,-x+3/4,y+1'\n\t'x+3/4,-z+1/2,y+0.25'\n\t'z,-x+3/4,-y+3/4'\n\t'x+3/4,z+3/4,-y+1'\n\t'-z+3/4,x+1/2,-y+0.25'\n"
        "\t'-x,-z+1/2,-y+1/2'\n\t'y,z+1/2,x+1/2'\n\t'y+1/2,-z+0.25,-x+3/4'\n\t'z+1/4,y+0.25,-x+1'\n\t'-y+1/4,z+1,-x+0.25'\n\t'-z,-y+1,-x+1'\n\t'-y+1/4,-z+3/4,x+1/2'\n"
        "\t'z+1/4,-y+1/2,x+3/4'\n\t'-z+1/2,y+3/4,x+0.25'\n\t'-x,-y+1/2,-z+1/2'\n\t'y,-x+1/4,-z+1/4'\n\t'x-3/4,y+1/4,-z'\n\t'-y-3/4,x+1/2,-z-1/4'\n\t'-x,y+1/4,z+1/4'\n"
        "\t'-y-3/4,-x+1/4,z'\n\t'x-3/4,-y+1/2,z-1/4'\n\t'y,x+1/2,z+1/2'\n\t'-z,-x+1/2,-y+1/2'\n\t'x,-z+1/4,-y+1/4'\n\t'z-3/4,x+1/4,-y'\n\t'-x-3/4,z+1/2,-y-1/4'\n"
        "\t'-z,x+1/4,y+1/4'\n\t'-x-3/4,-z+1/4,y'\n\t'z-3/4,-x+1/2,y-1/4'\n\t'x,z+1/2,y+1/2'\n\t'-y,-z+1/2,-x+1/2'\n\t'-y-1/2,z-1/4,x+1/4'\n\t'-z-1/4,-y-1/4,x'\n"
        "\t'y-1/4,-z,x-1/4'\n\t'z,y,x'\n\t'y-1/4,z+1/4,-x+1/2'\n\t'-z-1/4,y+1/2,-x+1/4'\n\t'z-1/2,-y+1/4,-x-1/4'\n\t'x+1/2,y,z+1/2'\n\t'-y+1/2,x+1/4,z+3/4'\n"
        "\t'-x+0.25,-y+1/4,z+1'\n\t'y+0.25,-x,z+0.25'\n\t'x+1/2,-y+1/4,-z+3/4'\n\t'y+0.25,x+1/4,-z+1'\n\t'-x+0.25,y,-z+0.25'\n\t'-y+1/2,-x,-z+1/2'\n\t'z+1/2,x,y+1/2'\n"
        "\t'-x+1/2,z+1/4,y+3/4'\n\t'-z+0.25,-x+1/4,y+1'\n\t'x+0.25,-z,y+0.25'\n\t'z+1/2,-x+1/4,-y+3/4'\n\t'x+0.25,z+1/4,-y+1'\n\t'-z+0.25,x,-y+0.25'\n\t'-x+1/2,-z,-y+1/2'\n"
        "\t'y+1/2,z,x+1/2'\n\t'y+1,-z+3/4,-x+3/4'\n\t'z+3/4,y+3/4,-x+1'\n\t'-y+3/4,z+1/2,-x+0.25'\n\t'-z+1/2,-y+1/2,-x+1'\n\t'-y+3/4,-z+1/4,x+1/2'\n\t'z+3/4,-y,x+3/4'\n"
        "\t'-z+1,y+1/4,x+0.25'\n\t'-x+1/2,-y,-z+1/2'\n\t'y+1/2,-x-1/4,-z+1/4'\n\t'x-1/4,y-1/4,-z'\n\t'-y-1/4,x,-z-1/4'\n\t'-x+1/2,y-1/4,z+1/4'\n\t'-y-1/4,-x-1/4,z'\n"
        "\t'x-1/4,-y,z-1/4'\n\t'y+1/2,x,z+1/2'\n\t'-z+1/2,-x,-y+1/2'\n\t'x+1/2,-z-1/4,-y+1/4'\n\t'z-1/4,x-1/4,-y'\n\t'-x-1/4,z,-y-1/4'\n\t'-z+1/2,x-1/4,y+1/4'\n"
        "\t'-x-1/4,-z-1/4,y'\n\t'z-1/4,-x,y-1/4'\n\t'x+1/2,z,y+1/2'\n\t'-y+1/2,-z,-x+1/2'\n\t'-y,z-3/4,x+1/4'\n\t'-z+1/4,-y-3/4,x'\n\t'y+1/4,-z-1/2,x-1/4'\n"
        "\t'z+1/2,y-1/2,x'\n\t'y+1/4,z-1/4,-x+1/2'\n\t'-z+1/4,y,-x+1/4'\n\t'z,-y-1/4,-x-1/4'\n\t'x+1/2,y+1/2,z'\n\t'-y+1/2,x+3/4,z+1/4'\n\t'-x+0.25,-y+3/4,z+1/2'\n"
        "\t'y+0.25,-x+1/2,z+3/4'\n\t'x+1/2,-y+3/4,-z+1/4'\n\t'y+0.25,x+3/4,-z+1/2'\n\t'-x+0.25,y+1/2,-z+3/4'\n\t'-y+1/2,-x+1/2,-z'\n\t'z+1/2,x+1/2,y'\n\t'-x+1/2,z+3/4,y+1/4'\n"
        "\t'-z+0.25,-x+3/4,y+1/2'\n\t'x+0.25,-z+1/2,y+3/4'\n\t'z+1/2,-x+3/4,-y+1/4'\n\t'x+0.25,z+3/4,-y+1/2'\n\t'-z+0.25,x+1/2,-y+3/4'\n\t'-x+1/2,-z+1/2,-y'\n\t'y+1/2,z+1/2,x'\n"
        "\t'y+1,-z+0.25,-x+1/4'\n\t'z+3/4,y+0.25,-x+1/2'\n\t'-y+3/4,z+1,-x+3/4'\n\t'-z+1/2,-y+1,-x+1/2'\n\t'-y+3/4,-z+3/4,x'\n\t'z+3/4,-y+1/2,x+1/4'\n\t'-z+1,y+3/4,x+3/4'\n"
        "\t'-x+1/2,-y+1/2,-z'\n\t'y+1/2,-x+1/4,-z-1/4'\n\t'x-1/4,y+1/4,-z-1/2'\n\t'-y-1/4,x+1/2,-z-3/4'\n\t'-x+1/2,y+1/4,z-1/4'\n\t'-y-1/4,-x+1/4,z-1/2'\n\t'x-1/4,-y+1/2,z-3/4'\n"
        "\t'y+1/2,x+1/2,z'\n\t'-z+1/2,-x+1/2,-y'\n\t'x+1/2,-z+1/4,-y-1/4'\n\t'z-1/4,x+1/4,-y-1/2'\n\t'-x-1/4,z+1/2,-y-3/4'\n\t'-z+1/2,x+1/4,y-1/4'\n\t'-x-1/4,-z+1/4,y-1/2'\n"
        "\t'z-1/4,-x+1/2,y-3/4'\n\t'x+1/2,z+1/2,y'\n\t'-y+1/2,-z+1/2,-x'\n\t'-y,z-1/4,x-1/4'\n\t'-z+1/4,-y-1/4,x-1/2'\n\t'y+1/4,-z,x-3/4'\n\t'z+1/2,y,x-1/2'\n"
        "\t'y+1/4,z+1/4,-x'\n\t'-z+1/4,y+1/2,-x-1/4'\n\t'z,-y+1/4,-x-3/4'\n");
      break;

    case 527: // [228] F d -3 c Origin choice 1 (F 4d 2 3 -1ad)
      fprintf(Stream,"\t'x,y,z'\n\t'-y+1/4,x+1/4,z+1/4'\n\t'-x,-y+1/2,z+1/2'\n\t'y+3/4,-x+1/4,z+3/4'\n\t'x,-y,-z'\n\t'y+1/4,x+1/4,-z+1/4'\n\t'-x,y+1/2,-z+1/2'\n"
        "\t'-y+3/4,-x+1/4,-z+3/4'\n\t'z,x,y'\n\t'-x+1/4,z+1/4,y+1/4'\n\t'-z,-x+1/2,y+1/2'\n\t'x+3/4,-z+1/4,y+3/4'\n\t'z,-x,-y'\n\t'x+1/4,z+1/4,-y+1/4'\n"
        "\t'-z,x+1/2,-y+1/2'\n\t'-x+3/4,-z+1/4,-y+3/4'\n\t'y,z,x'\n\t'y+1/2,-z,-x+1/2'\n\t'z+1/4,y+3/4,-x+3/4'\n\t'-y+1/2,z+1/2,-x'\n\t'-z+1/4,-y+1/4,-x+1/4'\n"
        "\t'-y,-z,x'\n\t'z+1/4,-y+3/4,x+3/4'\n\t'-z+3/4,y+3/4,x+1/4'\n\t'-x+3/4,-y+1/4,-z+1/4'\n\t'y+1/2,-x,-z'\n\t'x+3/4,y-1/4,-z-1/4'\n\t'-y,x,-z-1/2'\n"
        "\t'-x+3/4,y+1/4,z+1/4'\n\t'-y+1/2,-x,z'\n\t'x+3/4,-y-1/4,z-1/4'\n\t'y,x,z-1/2'\n\t'-z+3/4,-x+1/4,-y+1/4'\n\t'x+1/2,-z,-y'\n\t'z+3/4,x-1/4,-y-1/4'\n"
        "\t'-x,z,-y-1/2'\n\t'-z+3/4,x+1/4,y+1/4'\n\t'-x+1/2,-z,y'\n\t'z+3/4,-x-1/4,y-1/4'\n\t'x,z,y-1/2'\n\t'-y+3/4,-z+1/4,-x+1/4'\n\t'-y+1/4,z+1/4,x-1/4'\n"
        "\t'-z+1/2,-y-1/2,x-1/2'\n\t'y+1/4,-z-1/4,x+1/4'\n\t'z+1/2,y,x'\n\t'y+3/4,z+1/4,-x+1/4'\n\t'-z+1/2,y-1/2,-x-1/2'\n\t'z,-y-1/2,-x'\n\t'x,y+1/2,z+1/2'\n"
        "\t'-y+1/4,x+3/4,z+3/4'\n\t'-x,-y+1,z+1'\n\t'y+3/4,-x+3/4,z+0.25'\n\t'x,-y+1/2,-z+1/2'\n\t'y+1/4,x+3/4,-z+3/4'\n\t'-x,y+1,-z+1'\n\t'-y+3/4,-x+3/4,-z+0.25'\n"
        "\t'z,x+1/2,y+1/2'\n\t'-x+1/4,z+3/4,y+3/4'\n\t'-z,-x+1,y+1'\n\t'x+3/4,-z+3/4,y+0.25'\n\t'z,-x+1/2,-y+1/2'\n\t'x+1/4,z+3/4,-y+3/4'\n\t'-z,x+1,-y+1'\n"
        "\t'-x+3/4,-z+3/4,-y+0.25'\n\t'y,z+1/2,x+1/2'\n\t'y+1/2,-z+1/2,-x+1'\n\t'z+1/4,y+0.25,-x+0.25'\n\t'-y+1/2,z+1,-x+1/2'\n\t'-z+1/4,-y+3/4,-x+3/4'\n\t'-y,-z+1/2,x+1/2'\n"
        "\t'z+1/4,-y+0.25,x+0.25'\n\t'-z+3/4,y+0.25,x+3/4'\n\t'-x+3/4,-y+3/4,-z+3/4'\n\t'y+1/2,-x+1/2,-z+1/2'\n\t'x+3/4,y+1/4,-z+1/4'\n\t'-y,x+1/2,-z'\n\t'-x+3/4,y+3/4,z+3/4'\n"
        "\t'-y+1/2,-x+1/2,z+1/2'\n\t'x+3/4,-y+1/4,z+1/4'\n\t'y,x+1/2,z'\n\t'-z+3/4,-x+3/4,-y+3/4'\n\t'x+1/2,-z+1/2,-y+1/2'\n\t'z+3/4,x+1/4,-y+1/4'\n\t'-x,z+1/2,-y'\n"
        "\t'-z+3/4,x+3/4,y+3/4'\n\t'-x+1/2,-z+1/2,y+1/2'\n\t'z+3/4,-x+1/4,y+1/4'\n\t'x,z+1/2,y'\n\t'-y+3/4,-z+3/4,-x+3/4'\n\t'-y+1/4,z+3/4,x+1/4'\n\t'-z+1/2,-y,x'\n"
        "\t'y+1/4,-z+1/4,x+3/4'\n\t'z+1/2,y+1/2,x+1/2'\n\t'y+3/4,z+3/4,-x+3/4'\n\t'-z+1/2,y,-x'\n\t'z,-y,-x+1/2'\n\t'x+1/2,y,z+1/2'\n\t'-y+3/4,x+1/4,z+3/4'\n"
        "\t'-x+1/2,-y+1/2,z+1'\n\t'y+0.25,-x+1/4,z+0.25'\n\t'x+1/2,-y,-z+1/2'\n\t'y+3/4,x+1/4,-z+3/4'\n\t'-x+1/2,y+1/2,-z+1'\n\t'-y+0.25,-x+1/4,-z+0.25'\n\t'z+1/2,x,y+1/2'\n"
        "\t'-x+3/4,z+1/4,y+3/4'\n\t'-z+1/2,-x+1/2,y+1'\n\t'x+0.25,-z+1/4,y+0.25'\n\t'z+1/2,-x,-y+1/2'\n\t'x+3/4,z+1/4,-y+3/4'\n\t'-z+1/2,x+1/2,-y+1'\n\t'-x+0.25,-z+1/4,-y+0.25'\n"
        "\t'y+1/2,z,x+1/2'\n\t'y+1,-z,-x+1'\n\t'z+3/4,y+3/4,-x+0.25'\n\t'-y+1,z+1/2,-x+1/2'\n\t'-z+3/4,-y+1/4,-x+3/4'\n\t'-y+1/2,-z,x+1/2'\n\t'z+3/4,-y+3/4,x+0.25'\n"
        "\t'-z+0.25,y+3/4,x+3/4'\n\t'-x+0.25,-y+1/4,-z+3/4'\n\t'y+1,-x,-z+1/2'\n\t'x+0.25,y-1/4,-z+1/4'\n\t'-y+1/2,x,-z'\n\t'-x+0.25,y+1/4,z+3/4'\n\t'-y+1,-x,z+1/2'\n"
        "\t'x+0.25,-y-1/4,z+1/4'\n\t'y+1/2,x,z'\n\t'-z+0.25,-x+1/4,-y+3/4'\n\t'x+1,-z,-y+1/2'\n\t'z+0.25,x-1/4,-y+1/4'\n\t'-x+1/2,z,-y'\n\t'-z+0.25,x+1/4,y+3/4'\n"
        "\t'-x+1,-z,y+1/2'\n\t'z+0.25,-x-1/4,y+1/4'\n\t'x+1/2,z,y'\n\t'-y+0.25,-z+1/4,-x+3/4'\n\t'-y+3/4,z+1/4,x+1/4'\n\t'-z+1,-y-1/2,x'\n\t'y+3/4,-z-1/4,x+3/4'\n"
        "\t'z+1,y,x+1/2'\n\t'y+0.25,z+1/4,-x+3/4'\n\t'-z+1,y-1/2,-x'\n\t'z+1/2,-y-1/2,-x+1/2'\n\t'x+1/2,y+1/2,z'\n\t'-y+3/4,x+3/4,z+1/4'\n\t'-x+1/2,-y+1,z+1/2'\n"
        "\t'y+0.25,-x+3/4,z+3/4'\n\t'x+1/2,-y+1/2,-z'\n\t'y+3/4,x+3/4,-z+1/4'\n\t'-x+1/2,y+1,-z+1/2'\n\t'-y+0.25,-x+3/4,-z+3/4'\n\t'z+1/2,x+1/2,y'\n\t'-x+3/4,z+3/4,y+1/4'\n"
        "\t'-z+1/2,-x+1,y+1/2'\n\t'x+0.25,-z+3/4,y+3/4'\n\t'z+1/2,-x+1/2,-y'\n\t'x+3/4,z+3/4,-y+1/4'\n\t'-z+1/2,x+1,-y+1/2'\n\t'-x+0.25,-z+3/4,-y+3/4'\n\t'y+1/2,z+1/2,x'\n"
        "\t'y+1,-z+1/2,-x+1/2'\n\t'z+3/4,y+0.25,-x+3/4'\n\t'-y+1,z+1,-x'\n\t'-z+3/4,-y+3/4,-x+1/4'\n\t'-y+1/2,-z+1/2,x'\n\t'z+3/4,-y+0.25,x+3/4'\n\t'-z+0.25,y+0.25,x+1/4'\n"
        "\t'-x+0.25,-y+3/4,-z+1/4'\n\t'y+1,-x+1/2,-z'\n\t'x+0.25,y+1/4,-z-1/4'\n\t'-y+1/2,x+1/2,-z-1/2'\n\t'-x+0.25,y+3/4,z+1/4'\n\t'-y+1,-x+1/2,z'\n\t'x+0.25,-y+1/4,z-1/4'\n"
        "\t'y+1/2,x+1/2,z-1/2'\n\t'-z+0.25,-x+3/4,-y+1/4'\n\t'x+1,-z+1/2,-y'\n\t'z+0.25,x+1/4,-y-1/4'\n\t'-x+1/2,z+1/2,-y-1/2'\n\t'-z+0.25,x+3/4,y+1/4'\n\t'-x+1,-z+1/2,y'\n"
        "\t'z+0.25,-x+1/4,y-1/4'\n\t'x+1/2,z+1/2,y-1/2'\n\t'-y+0.25,-z+3/4,-x+1/4'\n\t'-y+3/4,z+3/4,x-1/4'\n\t'-z+1,-y,x-1/2'\n\t'y+3/4,-z+1/4,x+1/4'\n\t'z+1,y+1/2,x'\n"
        "\t'y+0.25,z+3/4,-x+1/4'\n\t'-z+1,y,-x-1/2'\n\t'z+1/2,-y,-x'\n");
      break;
    case 528: // [228] F d -3 c Origin choice 2 (-F 4ud 2vw 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-y+1/2,x+1/4,z+1/4'\n\t'-x+1/4,-y+3/4,z+1/2'\n\t'y+3/4,-x+1/2,z+3/4'\n\t'x,-y+1/4,-z+1/4'\n\t'y+1/4,x+1/4,-z+1/2'\n\t'-x+1/4,y+1/2,-z+3/4'\n"
        "\t'-y,-x+1/2,-z'\n\t'z,x,y'\n\t'-x+1/2,z+1/4,y+1/4'\n\t'-z+1/4,-x+3/4,y+1/2'\n\t'x+3/4,-z+1/2,y+3/4'\n\t'z,-x+1/4,-y+1/4'\n\t'x+1/4,z+1/4,-y+1/2'\n"
        "\t'-z+1/4,x+1/2,-y+3/4'\n\t'-x,-z+1/2,-y'\n\t'y,z,x'\n\t'y+1/2,-z+1/4,-x+3/4'\n\t'z+1/4,y+3/4,-x'\n\t'-y+3/4,z+1/2,-x+1/4'\n\t'-z+1/2,-y+1/2,-x+1/2'\n"
        "\t'-y+1/4,-z+1/4,x'\n\t'z+1/4,-y,x+3/4'\n\t'-z,y+3/4,x+1/4'\n\t'-x,-y,-z'\n\t'y-1/2,-x-1/4,-z-1/4'\n\t'x-1/4,y-3/4,-z-1/2'\n\t'-y-3/4,x-1/2,-z-3/4'\n"
        "\t'-x,y-1/4,z-1/4'\n\t'-y-1/4,-x-1/4,z-1/2'\n\t'x-1/4,-y-1/2,z-3/4'\n\t'y,x-1/2,z'\n\t'-z,-x,-y'\n\t'x-1/2,-z-1/4,-y-1/4'\n\t'z-1/4,x-3/4,-y-1/2'\n"
        "\t'-x-3/4,z-1/2,-y-3/4'\n\t'-z,x-1/4,y-1/4'\n\t'-x-1/4,-z-1/4,y-1/2'\n\t'z-1/4,-x-1/2,y-3/4'\n\t'x,z-1/2,y'\n\t'-y,-z,-x'\n\t'-y-1/2,z-1/4,x-3/4'\n"
        "\t'-z-1/4,-y-3/4,x'\n\t'y-3/4,-z-1/2,x-1/4'\n\t'z-1/2,y-1/2,x-1/2'\n\t'y-1/4,z-1/4,-x'\n\t'-z-1/4,y,-x-3/4'\n\t'z,-y-3/4,-x-1/4'\n\t'x,y+1/2,z+1/2'\n"
        "\t'-y+1/2,x+3/4,z+3/4'\n\t'-x+1/4,-y+0.25,z+1'\n\t'y+3/4,-x+1,z+0.25'\n\t'x,-y+3/4,-z+3/4'\n\t'y+1/4,x+3/4,-z+1'\n\t'-x+1/4,y+1,-z+0.25'\n\t'-y,-x+1,-z+1/2'\n"
        "\t'z,x+1/2,y+1/2'\n\t'-x+1/2,z+3/4,y+3/4'\n\t'-z+1/4,-x+0.25,y+1'\n\t'x+3/4,-z+1,y+0.25'\n\t'z,-x+3/4,-y+3/4'\n\t'x+1/4,z+3/4,-y+1'\n\t'-z+1/4,x+1,-y+0.25'\n"
        "\t'-x,-z+1,-y+1/2'\n\t'y,z+1/2,x+1/2'\n\t'y+1/2,-z+3/4,-x+0.25'\n\t'z+1/4,y+0.25,-x+1/2'\n\t'-y+3/4,z+1,-x+3/4'\n\t'-z+1/2,-y+1,-x+1'\n\t'-y+1/4,-z+3/4,x+1/2'\n"
        "\t'z+1/4,-y+1/2,x+0.25'\n\t'-z,y+0.25,x+3/4'\n\t'-x,-y+1/2,-z+1/2'\n\t'y-1/2,-x+1/4,-z+1/4'\n\t'x-1/4,y-1/4,-z'\n\t'-y-3/4,x,-z-1/4'\n\t'-x,y+1/4,z+1/4'\n"
        "\t'-y-1/4,-x+1/4,z'\n\t'x-1/4,-y,z-1/4'\n\t'y,x,z+1/2'\n\t'-z,-x+1/2,-y+1/2'\n\t'x-1/2,-z+1/4,-y+1/4'\n\t'z-1/4,x-1/4,-y'\n\t'-x-3/4,z,-y-1/4'\n"
        "\t'-z,x+1/4,y+1/4'\n\t'-x-1/4,-z+1/4,y'\n\t'z-1/4,-x,y-1/4'\n\t'x,z,y+1/2'\n\t'-y,-z+1/2,-x+1/2'\n\t'-y-1/2,z+1/4,x-1/4'\n\t'-z-1/4,-y-1/4,x+1/2'\n"
        "\t'y-3/4,-z,x+1/4'\n\t'z-1/2,y,x'\n\t'y-1/4,z+1/4,-x+1/2'\n\t'-z-1/4,y+1/2,-x-1/4'\n\t'z,-y-1/4,-x+1/4'\n\t'x+1/2,y,z+1/2'\n\t'-y+1,x+1/4,z+3/4'\n"
        "\t'-x+3/4,-y+3/4,z+1'\n\t'y+0.25,-x+1/2,z+0.25'\n\t'x+1/2,-y+1/4,-z+3/4'\n\t'y+3/4,x+1/4,-z+1'\n\t'-x+3/4,y+1/2,-z+0.25'\n\t'-y+1/2,-x+1/2,-z+1/2'\n\t'z+1/2,x,y+1/2'\n"
        "\t'-x+1,z+1/4,y+3/4'\n\t'-z+3/4,-x+3/4,y+1'\n\t'x+0.25,-z+1/2,y+0.25'\n\t'z+1/2,-x+1/4,-y+3/4'\n\t'x+3/4,z+1/4,-y+1'\n\t'-z+3/4,x+1/2,-y+0.25'\n\t'-x+1/2,-z+1/2,-y+1/2'\n"
        "\t'y+1/2,z,x+1/2'\n\t'y+1,-z+1/4,-x+0.25'\n\t'z+3/4,y+3/4,-x+1/2'\n\t'-y+0.25,z+1/2,-x+3/4'\n\t'-z+1,-y+1/2,-x+1'\n\t'-y+3/4,-z+1/4,x+1/2'\n\t'z+3/4,-y,x+0.25'\n"
        "\t'-z+1/2,y+3/4,x+3/4'\n\t'-x+1/2,-y,-z+1/2'\n\t'y,-x-1/4,-z+1/4'\n\t'x+1/4,y-3/4,-z'\n\t'-y-1/4,x-1/2,-z-1/4'\n\t'-x+1/2,y-1/4,z+1/4'\n\t'-y+1/4,-x-1/4,z'\n"
        "\t'x+1/4,-y-1/2,z-1/4'\n\t'y+1/2,x-1/2,z+1/2'\n\t'-z+1/2,-x,-y+1/2'\n\t'x,-z-1/4,-y+1/4'\n\t'z+1/4,x-3/4,-y'\n\t'-x-1/4,z-1/2,-y-1/4'\n\t'-z+1/2,x-1/4,y+1/4'\n"
        "\t'-x+1/4,-z-1/4,y'\n\t'z+1/4,-x-1/2,y-1/4'\n\t'x+1/2,z-1/2,y+1/2'\n\t'-y+1/2,-z,-x+1/2'\n\t'-y,z-1/4,x-1/4'\n\t'-z+1/4,-y-3/4,x+1/2'\n\t'y-1/4,-z-1/2,x+1/4'\n"
        "\t'z,y-1/2,x'\n\t'y+1/4,z-1/4,-x+1/2'\n\t'-z+1/4,y,-x-1/4'\n\t'z+1/2,-y-3/4,-x+1/4'\n\t'x+1/2,y+1/2,z'\n\t'-y+1,x+3/4,z+1/4'\n\t'-x+3/4,-y+0.25,z+1/2'\n"
        "\t'y+0.25,-x+1,z+3/4'\n\t'x+1/2,-y+3/4,-z+1/4'\n\t'y+3/4,x+3/4,-z+1/2'\n\t'-x+3/4,y+1,-z+3/4'\n\t'-y+1/2,-x+1,-z'\n\t'z+1/2,x+1/2,y'\n\t'-x+1,z+3/4,y+1/4'\n"
        "\t'-z+3/4,-x+0.25,y+1/2'\n\t'x+0.25,-z+1,y+3/4'\n\t'z+1/2,-x+3/4,-y+1/4'\n\t'x+3/4,z+3/4,-y+1/2'\n\t'-z+3/4,x+1,-y+3/4'\n\t'-x+1/2,-z+1,-y'\n\t'y+1/2,z+1/2,x'\n"
        "\t'y+1,-z+3/4,-x+3/4'\n\t'z+3/4,y+0.25,-x'\n\t'-y+0.25,z+1,-x+1/4'\n\t'-z+1,-y+1,-x+1/2'\n\t'-y+3/4,-z+3/4,x'\n\t'z+3/4,-y+1/2,x+3/4'\n\t'-z+1/2,y+0.25,x+1/4'\n"
        "\t'-x+1/2,-y+1/2,-z'\n\t'y,-x+1/4,-z-1/4'\n\t'x+1/4,y-1/4,-z-1/2'\n\t'-y-1/4,x,-z-3/4'\n\t'-x+1/2,y+1/4,z-1/4'\n\t'-y+1/4,-x+1/4,z-1/2'\n\t'x+1/4,-y,z-3/4'\n"
        "\t'y+1/2,x,z'\n\t'-z+1/2,-x+1/2,-y'\n\t'x,-z+1/4,-y-1/4'\n\t'z+1/4,x-1/4,-y-1/2'\n\t'-x-1/4,z,-y-3/4'\n\t'-z+1/2,x+1/4,y-1/4'\n\t'-x+1/4,-z+1/4,y-1/2'\n"
        "\t'z+1/4,-x,y-3/4'\n\t'x+1/2,z,y'\n\t'-y+1/2,-z+1/2,-x'\n\t'-y,z+1/4,x-3/4'\n\t'-z+1/4,-y-1/4,x'\n\t'y-1/4,-z,x-1/4'\n\t'z,y,x-1/2'\n"
        "\t'y+1/4,z+1/4,-x'\n\t'-z+1/4,y+1/2,-x-3/4'\n\t'z+1/2,-y-1/4,-x-1/4'\n");
      break;

    case 529: // [229] I m -3 m (-I 4 2 3)
      fprintf(Stream,"\t'x,y,z'\n\t'-y,x,z'\n\t'-x,-y,z'\n\t'y,-x,z'\n\t'x,-y,-z'\n\t'y,x,-z'\n\t'-x,y,-z'\n"
        "\t'-y,-x,-z'\n\t'z,x,y'\n\t'-x,z,y'\n\t'-z,-x,y'\n\t'x,-z,y'\n\t'z,-x,-y'\n\t'x,z,-y'\n"
        "\t'-z,x,-y'\n\t'-x,-z,-y'\n\t'y,z,x'\n\t'y,-z,-x'\n\t'z,y,-x'\n\t'-y,z,-x'\n\t'-z,-y,-x'\n"
        "\t'-y,-z,x'\n\t'z,-y,x'\n\t'-z,y,x'\n\t'-x,-y,-z'\n\t'y,-x,-z'\n\t'x,y,-z'\n\t'-y,x,-z'\n"
        "\t'-x,y,z'\n\t'-y,-x,z'\n\t'x,-y,z'\n\t'y,x,z'\n\t'-z,-x,-y'\n\t'x,-z,-y'\n\t'z,x,-y'\n"
        "\t'-x,z,-y'\n\t'-z,x,y'\n\t'-x,-z,y'\n\t'z,-x,y'\n\t'x,z,y'\n\t'-y,-z,-x'\n\t'-y,z,x'\n"
        "\t'-z,-y,x'\n\t'y,-z,x'\n\t'z,y,x'\n\t'y,z,-x'\n\t'-z,y,-x'\n\t'z,-y,-x'\n\t'x+1/2,y+1/2,z+1/2'\n"
        "\t'-y+1/2,x+1/2,z+1/2'\n\t'-x+1/2,-y+1/2,z+1/2'\n\t'y+1/2,-x+1/2,z+1/2'\n\t'x+1/2,-y+1/2,-z+1/2'\n\t'y+1/2,x+1/2,-z+1/2'\n\t'-x+1/2,y+1/2,-z+1/2'\n\t'-y+1/2,-x+1/2,-z+1/2'\n"
        "\t'z+1/2,x+1/2,y+1/2'\n\t'-x+1/2,z+1/2,y+1/2'\n\t'-z+1/2,-x+1/2,y+1/2'\n\t'x+1/2,-z+1/2,y+1/2'\n\t'z+1/2,-x+1/2,-y+1/2'\n\t'x+1/2,z+1/2,-y+1/2'\n\t'-z+1/2,x+1/2,-y+1/2'\n"
        "\t'-x+1/2,-z+1/2,-y+1/2'\n\t'y+1/2,z+1/2,x+1/2'\n\t'y+1/2,-z+1/2,-x+1/2'\n\t'z+1/2,y+1/2,-x+1/2'\n\t'-y+1/2,z+1/2,-x+1/2'\n\t'-z+1/2,-y+1/2,-x+1/2'\n\t'-y+1/2,-z+1/2,x+1/2'\n"
        "\t'z+1/2,-y+1/2,x+1/2'\n\t'-z+1/2,y+1/2,x+1/2'\n\t'-x+1/2,-y+1/2,-z+1/2'\n\t'y+1/2,-x+1/2,-z+1/2'\n\t'x+1/2,y+1/2,-z+1/2'\n\t'-y+1/2,x+1/2,-z+1/2'\n\t'-x+1/2,y+1/2,z+1/2'\n"
        "\t'-y+1/2,-x+1/2,z+1/2'\n\t'x+1/2,-y+1/2,z+1/2'\n\t'y+1/2,x+1/2,z+1/2'\n\t'-z+1/2,-x+1/2,-y+1/2'\n\t'x+1/2,-z+1/2,-y+1/2'\n\t'z+1/2,x+1/2,-y+1/2'\n\t'-x+1/2,z+1/2,-y+1/2'\n"
        "\t'-z+1/2,x+1/2,y+1/2'\n\t'-x+1/2,-z+1/2,y+1/2'\n\t'z+1/2,-x+1/2,y+1/2'\n\t'x+1/2,z+1/2,y+1/2'\n\t'-y+1/2,-z+1/2,-x+1/2'\n\t'-y+1/2,z+1/2,x+1/2'\n\t'-z+1/2,-y+1/2,x+1/2'\n"
        "\t'y+1/2,-z+1/2,x+1/2'\n\t'z+1/2,y+1/2,x+1/2'\n\t'y+1/2,z+1/2,-x+1/2'\n\t'-z+1/2,y+1/2,-x+1/2'\n\t'z+1/2,-y+1/2,-x+1/2'\n");
      break;

    case 530: // [230] I a -3 d (-I 4bd 2c 3)
      fprintf(Stream,"x,y,z\n,-y+0.25,x+0.75,z+0.25\n,-x+0.5,-y,z+0.5\n,y+0.25,-x+0.25,z+0.75\n,x,-y,-z+0.5\n,y+0.25,x+0.75,-z+0.75\n,-x+0.5,y,-z\n,-y+0.25,-x+0.25,-z+0.25\n,"
        "z,x,y\n,-x+0.25,z+0.75,y+0.25\n,-z+0.5,-x,y+0.5\n,x+0.25,-z+0.25,y+0.75\n,z,-x,-y+0.5\n,x+0.25,z+0.75,-y+0.75\n,-z+0.5,x,-y\n,-x+0.25,-z+0.25,-y+0.25\n,"
        "y,z,x\n,y+0.5,-z+0.5,-x\n,z+0.75,y+0.25,-x+0.25\n,-y,z+0.5,-x+0.5\n,-z+0.25,-y+0.25,-x+0.25\n,-y+0.5,-z,x+0.5\n,z+0.75,-y+0.75,x+0.25\n,-z+0.75,y+0.25,x+0.75\n,"
        "-x,-y,-z\n,y-0.25,-x-0.75,-z-0.25\n,x-0.5,y,-z-0.5\n,-y-0.25,x-0.25,-z-0.75\n,-x,y,z-0.5\n,-y-0.25,-x-0.75,z-0.75\n,x-0.5,-y,z\n,y-0.25,x-0.25,z-0.25\n,"
        "-z,-x,-y\n,x-0.25,-z-0.75,-y-0.25\n,z-0.5,x,-y-0.5\n,-x-0.25,z-0.25,-y-0.75\n,-z,x,y-0.5\n,-x-0.25,-z-0.75,y-0.75\n,z-0.5,-x,y\n,x-0.25,z-0.25,y-0.25\n,"
        "-y,-z,-x\n,-y-0.5,z-0.5,x\n,-z-0.75,-y-0.25,x-0.25\n,y,-z-0.5,x-0.5\n,z-0.25,y-0.25,x-0.25\n,y-0.5,z,-x-0.5\n,-z-0.75,y-0.75,-x-0.25\n,z-0.75,-y-0.25,-x-0.75\n,"
        "x+0.5,y+0.5,z+0.5\n,-y+0.75,x+0.25,z+0.75\n,-x+1,-y+0.5,z+1\n,y+0.75,-x+0.75,z+0.25\n,x+0.5,-y+0.5,-z+1\n,y+0.75,x+0.25,-z+0.25\n,-x+1,y+0.5,-z+0.5\n,-y+0.75,-x+0.75,-z+0.75\n,"
        "z+0.5,x+0.5,y+0.5\n,-x+0.75,z+0.25,y+0.75\n,-z+1,-x+0.5,y+1\n,x+0.75,-z+0.75,y+0.25\n,z+0.5,-x+0.5,-y+1\n,x+0.75,z+0.25,-y+0.25\n,-z+1,x+0.5,-y+0.5\n,-x+0.75,-z+0.75,-y+0.75\n,"
        "y+0.5,z+0.5,x+0.5\n,y+1,-z+1,-x+0.5\n,z+0.25,y+0.75,-x+0.75\n,-y+0.5,z+1,-x+1\n,-z+0.75,-y+0.75,-x+0.75\n,-y+1,-z+0.5,x+1\n,z+0.25,-y+0.25,x+0.75\n,-z+0.25,y+0.75,x+0.25\n,"
        "-x+0.5,-y+0.5,-z+0.5\n,y+0.25,-x-0.25,-z+0.25\n,x,y+0.5,-z\n,-y+0.25,x+0.25,-z-0.25\n,-x+0.5,y+0.5,z\n,-y+0.25,-x-0.25,z-0.25\n,x,-y+0.5,z+0.5\n,y+0.25,x+0.25,z+0.25\n,"
        "-z+0.5,-x+0.5,-y+0.5\n,x+0.25,-z-0.25,-y+0.25\n,z,x+0.5,-y\n,-x+0.25,z+0.25,-y-0.25\n,-z+0.5,x+0.5,y\n,-x+0.25,-z-0.25,y-0.25\n,z,-x+0.5,y+0.5\n,x+0.25,z+0.25,y+0.25\n,"
        "-y+0.5,-z+0.5,-x+0.5\n,-y,z,x+0.5\n,-z-0.25,-y+0.25,x+0.25\n,y+0.5,-z,x\n,z+0.25,y+0.25,x+0.25\n,y,z+0.5,-x\n,-z-0.25,y-0.25,-x+0.25\n,z-0.25,-y+0.25,-x-0.25\n");
      break;
    default:
      fprintf(stderr, "ERROR in 'PrintSymmetryElements'\n");
      exit(0);
      SpaceGroupSize=0;
      break;
  }
}

void SpaceGroupInverseSymmetry(int spacegroup,VECTOR pos)
{
  REAL x,y,z;

  x=pos.x;
  y=pos.y;
  z=pos.z;

  switch(spacegroup)
  {
    // TRICLINIC GROUPS
    // ================

    case 1:   // [1] P 1 (P 1)
      SpaceGroupSize=1;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      break;

    case 2:   // [2] P -1 (-P 1)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;
      break;


    // MONOCLINIC GROUPS
    // =================

    case 3: // [3] P 1 2 1 unique b axis (P 2y)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      break;
    case 4: // [3] P 1 1 2 unique c axis (P 2)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      break;
    case 5: // [3] P 2 1 1 unique a axis (P 2x)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;
      break;

    case 6: // [4] P 1 21 1 unique b axis (P 2yb)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      break;
    case 7: // [4] P 1 1 21 unique c axis (P 2c)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      break;
    case 8: // [4] P 21 1 1 unique a axis (P 2xa)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;
      break;

    case 9:  // [5] C 1 2 1 unique b axis: cell choice 1 (C 2y)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      break;
    case 10: // [5] A 1 2 1 unique b axis: cell choice 2 (A 2y)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      break;
    case 11: // [5] I 1 2 1 unique b axis: cell choice 3 (I 2y)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      break;
    case 12: // [5] A 1 1 2 unique c axis: cell choice 1 (A 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=-0.5+z;
      break;
    case 13: // [5] B 1 1 2 unique c axis: cell choice 2 (B 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      break;
    case 14: // [5] I 1 1 2 unique c axis: cell choice 3 (I 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=-0.5+z;
      break;
    case 15: // [5] B 2 1 1 unique a axis: cell choice 1 (B 2x)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=0.5-z;
      break;
    case 16: // [5] C 2 1 1 unique a axis: cell choice 2 (C 2x)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=-z;
      break;
    case 17: // [5] I 2 1 1 unique a axis: cell choice 3 (I 2x)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=0.5-z;
      break;

    case 18: // [6] P 1 m 1 unique b axis (P -2y)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      break;
    case 19: // [6] P 1 1 m unique c axis (P -2)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      break;
    case 20: // [6] P m 1 1 unique a axis (P -2x)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=z;
      break;

    case 21: // [7] P 1 c 1 unique b axis: cell choice 1 (P -2yc)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      break;
    case 22: // [7] P 1 n 1 unique b axis: cell choice 2 (P -2yac)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      break;
    case 23: // [7] P 1 a 1 unique b axis: cell choice 3 (P -2ya)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      break;
    case 24: // [7] P 1 1 a unique c axis: cell choice 1 (P -2a)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      break;
    case 25: // [7] P 1 1 n unique c axis: cell choice 2 (P -2ab)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      break;
    case 26: // [7] P 1 1 b unique c axis: cell choice 3 (P -2b)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      break;
    case 27: // [7] P b 1 1 unique a axis: cell choice 1 (P -2xb)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=z;
      break;
    case 28: // [7] P n 1 1 unique a axis: cell choice 2 (P -2xbc)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-0.5+z;
      break;
    case 29: // [7] P c 1 1 unique a axis: cell choice 3 (P -2xc)
      SpaceGroupSize=2;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-0.5+z;
      break;

    case 30: // [8] C 1 m 1 unique b axis: cell choice 1 (C -2y)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=z;
      break;
    case 31: // [8] A 1 m 1 unique b axis: cell choice 2 (A -2y)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=-0.5+z;
      break;
    case 32: // [8] I 1 m 1 unique b axis: cell choice 3 (I -2y)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=-0.5+z;
      break;
    case 33: // [8] A 1 1 m unique c axis: cell choice 1 (A -2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      break;
    case 34: // [8] B 1 1 m  unique c axis: cell choice 2 (B -2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      break;
    case 35: // [8] I 1 1 m unique c axis: cell choice 3 (I -2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      break;
    case 36: // [8] B m 1 1 unique a axis: cell choice 1 (B -2x)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-0.5+z;
      break;
    case 37: // [8] C m 1 1 unique a axis: cell choice 2 (C -2x)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=z;
      break;
    case 38: // [8] I m 1 1 unique a axis: cell choice 3 (I -2x)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-0.5+z;
      break;

    case 39: // [9] C 1 c 1 unique b axis: cell choice 1 (C -2yc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=-0.5+z;
      break;
    case 40: // [9] A 1 n 1 unique b axis: cell choice 2 (A -2yab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      break;
    case 41: // [9] I 1 a 1 unique b axis: cell choice 3 (I -2ya)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=-0.5+z;
      break;
    case 42: // [9] A 1 a 1 unique -b axis: cell choice 1 (A -2ya)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=-0.5+z;
      break;
    case 43: // [9] C 1 n 1 unique -b axis: cell choice 2 (C -2yac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=-0.5+z;
      break;
    case 44: // [9] I 1 c 1 unique -b axis: cell choice 3 (I -2yc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=z;
      break;
    case 45: // [9] A 1 1 a unique c axis: cell choice 1 (A -2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      break;
    case 46: // [9] B 1 1 n unique c axis: cell choice 2 (B -2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      break;
    case 47: // [9] I 1 1 b unique c axis: cell choice 3 (I -2b)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      break;
    case 48: // [9] B 1 1 b unique -c axis: cell choice 1 (B -2b)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      break;
    case 49: // [9] A 1 1 n unique -c axis: cell choice 2 (A -2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      break;
    case 50: // [9] I 1 1 a unique -c axis: cell choice 3 (I -2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      break;
    case 51: // [9] B b 1 1 unique a axis: cell choice 1 (B -2xb)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-0.5+z;
      break;
    case 52: // [9] C n 1 1 unique a axis: cell choice 2 (C -2xac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-0.5+z;
      break;
    case 53: // [9] I c 1 1 unique a axis: cell choice 3 (I -2xc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=z;
      break;
    case 54: // [9] C c 1 1 unique -a axis: cell choice 1 (C -2xc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-0.5+z;
      break;
    case 55: // [9] B n 1 1 unique -a axis: cell choice 2 (B -2xab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-x;            SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-0.5+z;
      break;
    case 56: // [9] I b 1 1 unique -a axis: cell choice 3 (I -2xb)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-0.5+z;
      break;

    case 57: // [10] P 1 2/m 1 unique b axis (-P 2y)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      break;
    case 58: // [10] P 1 1 2/m unique c axis (-P 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      break;
    case 59: // [10] P 2/m 1 1 unique a axis (-P 2x)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=z;
      break;

    case 60: // [11] P 1 21/m 1 unique axis b (-P 2yb)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-0.5-y;         SpaceGroupElement[3].z=z;
      break;
    case 61: // [11] P 1 1 21/m unique c axis (-P 2c)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-0.5-z;
      break;
    case 62: // [11] P 21/m 1 1 unique a axis (-P 2xa)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5-x;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=z;
      break;

    case 63: // [12] C 1 2/m 1 unique b axis: cell choice 1 (-C 2y)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=z;
      break;
    case 64: // [12] A 1 2/m 1 unique b axis: cell choice 2 (-A 2y)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 65: // [12] I 1 2/m 1 unique b axis: cell choice 3 (-I 2y)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 66: // [12] A 1 1 2/m unique c axis: cell choice 1 (-A 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      break;
    case 67: // [12] B 1 1 2/m unique c axis: cell choice 2 (-B 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=0.5-z;
      break;
    case 68: // [12] I 1 1 2/m unique c axis: cell choice 3 (-I 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      break;
    case 69: // [12] B 2/m 1 1 unique a axis: cell choice 1 (-B 2x)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=-0.5+z;
      break;
    case 70: // [12] C 2/m 1 1 unique a axis: cell choice 2 (-C 2x)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=z;
      break;
    case 71: // [12] I 2/m 1 1 unique a axis: cell choice 3 (-I 2x)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=-0.5+z;
      break;

    case 72: // [13] P 1 2/c 1 unique b axis: cell choice 1 (-P 2yc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=0.5+z;
      break;
    case 73: // [13] P 1 2/n 1 unique b axis: cell choice 2 (-P 2yac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5+x;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=0.5+z;
      break;
    case 74: // [13] P 1 2/a 1 unique b axis: cell choice 3 (-P 2ya)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5+x;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      break;
    case 75: // [13] P 1 1 2/a unique c axis: cell choice 1 (-P 2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5+x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      break;
    case 76: // [13] P 1 1 2/n unique c axis: cell choice 2 (-P 2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5+x;          SpaceGroupElement[3].y=0.5+y;          SpaceGroupElement[3].z=-z;
      break;
    case 77: // [13] P 1 1 2/b unique c axis: cell choice 3 (-P 2b)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=0.5+y;          SpaceGroupElement[3].z=-z;
      break;
    case 78: // [13] P 2/b 1 1 unique a axis: cell choice 1 (-P 2xb)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=0.5+y;          SpaceGroupElement[3].z=z;
      break;
    case 79: // [13] P 2/n 1 1 unique a axis: cell choice 2 (-P 2xbc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=0.5+y;          SpaceGroupElement[3].z=0.5+z;
      break;
    case 80: // [13] P 2/c 1 1 unique a axis: cell choice 3 (-P 2xc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5+z;
      break;

    case 81: // [14] P 1 21/c 1 unique b axis: cell choice 1 (-P 2ybc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-0.5-y;         SpaceGroupElement[3].z=0.5+z;
      break;
    case 82: // [14] P 1 21/n 1 unique b axis: cell choice 2 (-P 2yn)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5+x;          SpaceGroupElement[3].y=-0.5-y;         SpaceGroupElement[3].z=0.5+z;
      break;
    case 83: // [14] P 1 21/a 1 unique b axis: cell choice 3 (-P 2yab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5+x;          SpaceGroupElement[3].y=-0.5-y;         SpaceGroupElement[3].z=z;
      break;
    case 84: // [14] P 1 1 21/a unique c axis: cell choice 1 (-P 2ac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5+x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-0.5-z;
      break;
    case 85: // [14] P 1 1 21/n unique c axis: cell choice 2 (-P 2n)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5+x;          SpaceGroupElement[3].y=0.5+y;          SpaceGroupElement[3].z=-0.5-z;
      break;
    case 86: // [14] P 1 1 21/b unique c axis: cell choice 3 (-P 2bc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=0.5+y;          SpaceGroupElement[3].z=-0.5-z;
      break;
    case 87: // [14] P 21/b 1 1 unique a axis: cell choice 1 (-P 2xab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5-x;         SpaceGroupElement[3].y=0.5+y;          SpaceGroupElement[3].z=z;
      break;
    case 88: // [14] P 21/n 1 1 unique a axis: cell choice 2 (-P 2xn)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5-x;         SpaceGroupElement[3].y=0.5+y;          SpaceGroupElement[3].z=0.5+z;
      break;
    case 89: // [14] P 21/c 1 1 unique a axis: cell choice 3 (-P 2xac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5-x;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5+z;
      break;

    case 90: // [15] C 1 2/c 1 unique b axis: cell choice 1 (-C 2yc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=0.5+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=0.5+z;
      break;
    case 91: // [15] A 1 2/n 1 unique b axis: cell choice 2 (-A 2yab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5+x;          SpaceGroupElement[3].y=-0.5-y;         SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-0.5+z;
      break;
    case 92: // [15] I 1 2/a 1 unique b axis: cell choice 3 (-I 2ya)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5+x;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 93: // [15] A 1 2/a 1 unique -b axis: cell choice 1 (-A 2ya)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5+x;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 94: // [15] C 1 2/n 1 unique -b axis: cell choice 2 (-C 2yac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5+x;          SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=0.5+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=0.5+z;
      break;
    case 95: // [15] I 1 2/c 1 unique -b axis: cell choice 3 (-I 2yc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=0.5+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=z;
      break;
    case 96: // [15] A 1 1 2/a unique c axis: cell choice 1 (-A 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5+x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      break;
    case 97: // [15] B 1 1 2/n unique c axis: cell choice 2 (-B 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5+x;          SpaceGroupElement[3].y=0.5+y;          SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=0.5+y;          SpaceGroupElement[7].z=0.5-z;
      break;
    case 98: // [15] I 1 1 2/b unique c axis: cell choice 3 (-I 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=0.5+y;          SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=0.5-z;
      break;
    case 99: // [15] B 1 1 2/b unique -c axis: cell choice 1 (-B 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=0.5+y;          SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5+y;          SpaceGroupElement[7].z=0.5-z;
      break;
    case 100: // [15] A 1 1 2/n unique -c axis: cell choice 2 (-A 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5+x;          SpaceGroupElement[3].y=0.5+y;          SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=0.5-z;
      break;
    case 101: // [15] I 1 1 2/a unique -c axis: cell choice 3 (-I 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5+x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      break;
    case 102: // [15] B 2/b 1 1 unique a axis: cell choice 1 (-B 2xb)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=0.5+y;          SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=0.5+y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 103: // [15] C 2/n 1 1 unique a axis: cell choice 2 (-C 2xac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5-x;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5+z;
      break;
    case 104: // [15] I 2/c 1 1 unique a axis: cell choice 3 (-I 2xc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=z;
      break;
    case 105: // [15] C 2/c 1 1 unique -a axis: cell choice 1 (-C 2xc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5+z;
      break;
    case 106: // [15] B 2/n 1 1 unique -a axis: cell choice 2 (-B 2xab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5-x;         SpaceGroupElement[3].y=0.5+y;          SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=0.5+y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 107: // [15] I 2/b 1 1 unique -a axis: cell choice 3 (-I 2xb)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=0.5+y;          SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=-0.5+z;
      break;


    // ORTHORHOMBIC GROUPS
    // ===================

    case 108: // [16] P 2 2 2 (P 2 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      break;

    case 109: // [17] P 2 2 21 Origin-1,abc (P 2c 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      break;
    case 110: // [17] P 21 2 2 Origin-1,cab (P 2a 2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      break;
    case 111: // [17] P 2 21 2 Origin-1,bca (P 2 2b)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      break;

    case 112: // [18] P 21 21 2 Origin-1,abc (P 2 2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      break;
    case 113: // [18] P 2 21 21 Origin-1,cab (P 2bc 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      break;
    case 114: // [18] P 21 2 21 Origin-1,bca (P 2ac 2ac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      break;

    case 115: // [19] P 21 21 21 (P 2ac 2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      break;

    case 116: // [20] C 2 2 21  Origin-1,abc (C 2c 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      break;
    case 117: // [20] A 21 2 2  Origin-1,cba (A 2a 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      break;
    case 118: // [20] B 2 21 2  Origin-1,bca (B 2 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      break;

    case 119: // [21] C 2 2 2 Origin-1,abc (C 2 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=-z;
      break;
    case 120: // [21] A 2 2 2 Origin-1,cab (A 2 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      break;
    case 121: // [21] B 2 2 2 Origin-1,bca (B 2 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=0.5-z;
      break;

    case 122: // [22] F 2 2 2 (F 2 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-0.5+x;        SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=0.5-x;         SpaceGroupElement[13].y=0.5-y;         SpaceGroupElement[13].z=z;
      SpaceGroupElement[14].x=-0.5+x;        SpaceGroupElement[14].y=0.5-y;         SpaceGroupElement[14].z=-z;
      SpaceGroupElement[15].x=0.5-x;         SpaceGroupElement[15].y=-0.5+y;        SpaceGroupElement[15].z=-z;
      break;

    case 123: // [23] I 2 2 2 (I 2 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      break;

    case 124: // [24] I 21 21 21 (I 2b 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=-z;
      break;

    case 125: // [25] P m m 2 (P 2 -2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      break;
    case 126: // [25] P 2 m m (P -2 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      break;
    case 127: // [25] P m 2 m (P -2 -2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      break;

    case 128: // [26] P m c 21 (P 2c -2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      break;
    case 129: // [26] P c m 21 (P 2c -2c)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      break;
    case 130: // [26] P 21 m a (P -2a 2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      break;
    case 131: // [26] P 21 a m (P -2 2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      break;
    case 132: // [26] P b 21 m (P -2 -2b)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      break;
    case 133: // [26] P m 21 b (P -2b -2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      break;

    case 134: // [27] P c c 2 (P 2 -2c)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      break;
    case 135: // [27] P 2 a a (P -2a 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      break;
    case 136: // [27] P b 2 b (P -2b -2b)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      break;

    case 137: // [28] P m a 2 (P 2 -2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      break;
    case 138: // [28] P b m 2 (P 2 -2b)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=z;
      break;
    case 139: // [28] P 2 m b (P -2b 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=z;
      break;
    case 140: // [28] P 2 c m (P -2c 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      break;
    case 141: // [28] P c 2 m (P -2c -2c)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      break;
    case 142: // [28] P m 2 a (P -2a -2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      break;

    case 143: // [29] P c a 21 (P 2c -2ac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      break;
    case 144: // [29] P b c 21 (P 2c -2b)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=-0.5+z;
      break;
    case 145: // [29] P 21 a b (P -2b 2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=z;
      break;
    case 146: // [29] P 21 c a (P -2ac 2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      break;
    case 147: // [29] P c 21 b (P -2bc -2c)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      break;
    case 148: // [29] P b 21 a (P -2a -2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      break;

    case 149: // [30] P n c 2 (P 2 -2bc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=-0.5+z;
      break;
    case 150: // [30] P c n 2 (P 2 -2ac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      break;
    case 151: // [30] P 2 n a (P -2ac 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      break;
    case 152: // [30] P 2 a n (P -2ab 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=z;
      break;
    case 153: // [30] P b 2 n (P -2ab -2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      break;
    case 154: // [30] P n 2 b (P -2bc -2bc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      break;

    case 155: // [31] P m n 21 (P 2ac -2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      break;
    case 156: // [31] P n m 21 (P 2bc -2bc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      break;
    case 157: // [31] P 21 m n (P -2ab 2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      break;
    case 158: // [31] P 21 n m (P -2 2ac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      break;
    case 159: // [31] P n 21 m (P -2 -2bc)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      break;
    case 160: // [31] P m 21 n (P -2ab -2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      break;

    case 161: // [32] P b a 2 (P 2 -2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=z;
      break;
    case 162: // [32] P 2 c b (P -2bc 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=-0.5+z;
      break;
    case 163: // [32] P c 2 a (P -2ac -2ac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      break;

    case 164: // [33] P n a 21 (P 2c -2n)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=z;
      break;
    case 165: // [33] P b n 21 (P 2c -2ab)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=-0.5+z;
      break;
    case 166: // [33] P 21 n b (P -2bc 2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=-0.5+z;
      break;
    case 167: // [33] P 21 c n (P -2n 2a)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=-0.5+z;
      break;
    case 168: // [33] P c 21 n (P -2n -2ac)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      break;
    case 169: // [33] P n 21 a (P -2ac -2n)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      break;

    case 170: // [34] P n n 2 (P 2 -2n)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=-0.5+z;
      break;
    case 171: // [34] P 2 n n (P -2n 2)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=-0.5+z;
      break;
    case 172: // [34] P n 2 n (P -2n -2n)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      break;

    case 173: // [35] C m m 2 (C 2 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=z;
      break;
    case 174: // [35] A 2 m m (A -2 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 175: // [35] B m 2 m (B -2 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=0.5-z;
      break;

    case 176: // [36] C m c 21 (C 2c -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 177: // [36] C c m 21 (C 2c -2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=z;
      break;
    case 178: // [36] A 21 m a (A -2a 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 179: // [36] A 21 a m (A -2 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 180: // [36] B b 21 m (B -2 -2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      break;
    case 181: // [36] B m 21 b (B -2b -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      break;

    case 182: // [37] C c c 2 (C 2 -2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 183: // [37] A 2 a a (A -2a 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 184: // [37] B b 2 b (B -2b -2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=0.5-z;
      break;

    case 185: // [38] A m m 2 (A 2 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 186: // [38] B m m 2 (B 2 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-0.5+z;
      break;
    case 187: // [38] B 2 m m (B -2 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-0.5+z;
      break;
    case 188: // [38] C 2 m m (C -2 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=z;
      break;
    case 189: // [38] C m 2 m (C -2 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=-z;
      break;
    case 190: // [38] A m 2 m (A -2 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      break;

    case 191: // [39] A b m 2 (A 2 -2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-0.5+z;
      break;
    case 192: // [39] B m a 2 (B 2 -2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-0.5+z;
      break;
    case 193: // [39] B 2 c m (B -2a 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-0.5+z;
      break;
    case 194: // [39] C 2 m b (C -2a 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=z;
      break;
    case 195: // [39] C m 2 a (C -2a -2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=-z;
      break;
    case 196: // [39] A c 2 m (A -2b -2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      break;

    case 197: // [40] A m a 2 (A 2 -2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 198: // [40] B b m 2 (B 2 -2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 199: // [40] B 2 m b (B -2b 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 200: // [40] C 2 c m (C -2c 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 201: // [40] C c 2 m (C -2c -2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=-z;
      break;
    case 202: // [40] A m 2 a (A -2a -2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      break;

    case 203: // [41] A b a 2 (A 2 -2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-0.5+z;
      break;
    case 204: // [41] B b a 2 (B 2 -2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 205: // [41] B 2 c b (B -2ab 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 206: // [41] C 2 c b (C -2ac 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 207: // [41] C c 2 a (C -2ac -2ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=-z;
      break;
    case 208: // [41] A c 2 a (A -2ab -2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      break;

    case 209: // [42] F m m 2 (F 2 -2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=0.5-x;         SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-0.5+z;
      SpaceGroupElement[11].x=-0.5+x;        SpaceGroupElement[11].y=-y;            SpaceGroupElement[11].z=-0.5+z;
      SpaceGroupElement[12].x=-0.5+x;        SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=0.5-x;         SpaceGroupElement[13].y=0.5-y;         SpaceGroupElement[13].z=z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=z;
      break;
    case 210: // [42] F 2 m m (F -2 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-0.5+x;         SpaceGroupElement[9].y=y;              SpaceGroupElement[9].z=0.5-z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-0.5+x;        SpaceGroupElement[11].y=-y;            SpaceGroupElement[11].z=-0.5+z;
      SpaceGroupElement[12].x=-0.5+x;        SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=-0.5+x;        SpaceGroupElement[14].y=0.5-y;         SpaceGroupElement[14].z=-z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=z;
      break;
    case 211: // [42] F m 2 m (F -2 -2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-x;             SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-0.5+x;         SpaceGroupElement[9].y=y;              SpaceGroupElement[9].z=0.5-z;
      SpaceGroupElement[10].x=0.5-x;         SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-0.5+z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-0.5+x;        SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=0.5-x;         SpaceGroupElement[15].y=-0.5+y;        SpaceGroupElement[15].z=-z;
      break;

    case 212: // [43] F d d 2 (F 2 -2d)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=0.25-x;         SpaceGroupElement[2].y=-0.25+y;        SpaceGroupElement[2].z=-0.25+z;
      SpaceGroupElement[3].x=-0.75+x;        SpaceGroupElement[3].y=0.75-y;         SpaceGroupElement[3].z=-0.25+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=0.25-x;         SpaceGroupElement[6].y=-0.75+y;        SpaceGroupElement[6].z=-0.75+z;
      SpaceGroupElement[7].x=-0.75+x;        SpaceGroupElement[7].y=0.25-y;         SpaceGroupElement[7].z=-0.75+z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=0.75-x;        SpaceGroupElement[10].y=-0.25+y;       SpaceGroupElement[10].z=-0.75+z;
      SpaceGroupElement[11].x=-0.25+x;       SpaceGroupElement[11].y=0.75-y;        SpaceGroupElement[11].z=-0.75+z;
      SpaceGroupElement[12].x=-0.5+x;        SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=0.5-x;         SpaceGroupElement[13].y=0.5-y;         SpaceGroupElement[13].z=z;
      SpaceGroupElement[14].x=0.75-x;        SpaceGroupElement[14].y=-0.75+y;       SpaceGroupElement[14].z=-0.25+z;
      SpaceGroupElement[15].x=-0.25+x;       SpaceGroupElement[15].y=0.25-y;        SpaceGroupElement[15].z=-0.25+z;
      break;
    case 213: // [43] F 2 d d (F -2d 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.25+x;        SpaceGroupElement[1].y=-0.25+y;        SpaceGroupElement[1].z=0.25-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.25+x;        SpaceGroupElement[3].y=0.25-y;         SpaceGroupElement[3].z=-0.25+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.25+x;        SpaceGroupElement[5].y=-0.75+y;        SpaceGroupElement[5].z=0.75-z;
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-0.25+x;        SpaceGroupElement[7].y=0.75-y;         SpaceGroupElement[7].z=-0.75+z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-0.75+x;        SpaceGroupElement[9].y=-0.25+y;        SpaceGroupElement[9].z=0.75-z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-0.75+x;       SpaceGroupElement[11].y=0.25-y;        SpaceGroupElement[11].z=-0.75+z;
      SpaceGroupElement[12].x=-0.5+x;        SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=-0.75+x;       SpaceGroupElement[13].y=-0.75+y;       SpaceGroupElement[13].z=0.25-z;
      SpaceGroupElement[14].x=-0.5+x;        SpaceGroupElement[14].y=0.5-y;         SpaceGroupElement[14].z=-z;
      SpaceGroupElement[15].x=-0.75+x;       SpaceGroupElement[15].y=0.75-y;        SpaceGroupElement[15].z=-0.25+z;
      break;
    case 214: // [43] F d 2 d (F -2d -2d)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.25+x;        SpaceGroupElement[1].y=-0.25+y;        SpaceGroupElement[1].z=0.25-z;
      SpaceGroupElement[2].x=0.25-x;         SpaceGroupElement[2].y=-0.25+y;        SpaceGroupElement[2].z=-0.25+z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.25+x;        SpaceGroupElement[5].y=-0.75+y;        SpaceGroupElement[5].z=0.75-z;
      SpaceGroupElement[6].x=0.25-x;         SpaceGroupElement[6].y=-0.75+y;        SpaceGroupElement[6].z=-0.75+z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-0.75+x;        SpaceGroupElement[9].y=-0.25+y;        SpaceGroupElement[9].z=0.75-z;
      SpaceGroupElement[10].x=0.75-x;        SpaceGroupElement[10].y=-0.25+y;       SpaceGroupElement[10].z=-0.75+z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-0.5+x;        SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=-0.75+x;       SpaceGroupElement[13].y=-0.75+y;       SpaceGroupElement[13].z=0.25-z;
      SpaceGroupElement[14].x=0.75-x;        SpaceGroupElement[14].y=-0.75+y;       SpaceGroupElement[14].z=-0.25+z;
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=y;             SpaceGroupElement[15].z=-z;
      break;

    case 215: // [44] I m m 2 (I 2 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 216: // [44] I 2 m m (I -2 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 217: // [44] I m 2 m (I -2 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      break;

    case 218: // [45] I b a 2 (I 2 -2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=z;
      break;
    case 219: // [45] I 2 c b (I -2a 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 220: // [45] I c 2 a (I -2b -2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      break;

    case 221: // [46] I m a 2 (I 2 -2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-0.5+x;         SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 222: // [46] I b m 2 (I 2 -2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-0.5+y;         SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=0.5-y;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-0.5+z;
      break;
    case 223: // [46] I 2 m b (I -2b 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=-0.5+y;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=0.5-y;          SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-0.5+z;
      break;
    case 224: // [46] I 2 c m (I -2c 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=z;
      break;
    case 225: // [46] I c 2 m (I -2c -2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x;              SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      break;
    case 226: // [46] I m 2 a (I -2a -2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+x;         SpaceGroupElement[1].y=y;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=y;              SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=0.5-x;          SpaceGroupElement[7].y=-0.5+y;         SpaceGroupElement[7].z=0.5-z;
      break;

    case 227: // [47] P m m m (-P 2 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      break;

    case 228: // [48] P n n n Origin choice 1 (P 2 2 -1n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=0.5-x;          SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 229: // [48] P n n n Origin choice 2 (-P 2ab 2bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      break;

    case 230: // [49] P c c m  (-P 2 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      break;
    case 231: // [49] P m a a (-P 2a 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      break;
    case 232: // [49] P b m b (-P 2b 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      break;

    case 233: // [50] P b a n Origin choice 1 (P 2 2 -1ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=0.5-x;          SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=z;
      break;
    case 234: // [50] P b a n Origin choice 2 (-P 2ab 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      break;
    case 235: // [50] P n c b Origin choice 1 (P 2 2 -1bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=-0.5+z;
      break;
    case 236: // [50] P n c b Origin choice 2 (-P 2b 2bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      break;
    case 237: // [50] P c n a Origin choice 1 (P 2 2 -1ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=0.5-x;          SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-0.5+z;
      break;
    case 238: // [50] P c n a Origin choice 2 (-P 2a 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      break;

    case 239: // [51] P m m a (-P 2a 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      break;
    case 240: // [51] P m m b (-P 2b 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      break;
    case 241: // [51] P b m m (-P 2 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      break;
    case 242: // [51] P c m m (-P 2c 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      break;
    case 243: // [51] P m c m (-P 2c 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      break;
    case 244: // [51] P m a m (-P 2 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      break;

    case 245: // [52] P n n a (-P 2a 2bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=0.5+z;
      break;
    case 246: // [52] P n n b (-P 2b 2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      break;
    case 247: // [52] P b n n (-P 2n 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      break;
    case 248: // [52] P c n n (-P 2ab 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=0.5+z;
      break;
    case 249: // [52] P n c n (-P 2ab 2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      break;
    case 250: // [52] P n a n (-P 2n 2bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      break;

    case 251: // [53] P m n a (-P 2ac 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      break;
    case 252: // [53] Pnmb (-P 2bc 2bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      break;
    case 253: // [53] P b m n (-P 2ab 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      break;
    case 254: // [53] P c n m (-P 2 2ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      break;
    case 255: // [53] P n c m (-P 2 2bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=0.5+z;
      break;
    case 256: // [53] P m a n (-P 2ab 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      break;

    case 257: // [54] P c c a (-P 2a 2ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      break;
    case 258: // [54] P c c b (-P 2b 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=0.5+z;
      break;
    case 259: // [54] P b a a (-P 2a 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      break;
    case 260: // [54] P c a a (-P 2ac 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      break;
    case 261: // [54] P b c b (-P 2bc 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      break;
    case 262: // [54] P b a b (-P 2b 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      break;

    case 263: // [55] P b a m (-P 2 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      break;
    case 264: // [55] P m c b (-P 2bc 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=0.5+z;
      break;
    case 265: // [55] P c m a (-P 2ac 2ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      break;

    case 266: // [56] P c c n (-P 2ab 2ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=0.5+z;
      break;
    case 267: // [56] P n a a (-P 2ac 2bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      break;
    case 268: // [56] P b n b (-P 2bc 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      break;

    case 269: // [57] P b c m (-P 2c 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=0.5+z;
      break;
    case 270: // [57] P c a m (-P 2c 2ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      break;
    case 271: // [57] P m c a (-P 2ac 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      break;
    case 272: // [57] P m a b (-P 2b 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      break;
    case 273: // [57] P b m a (-P 2a 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      break;
    case 274: // [57] P c m b (-P 2bc 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      break;

    case 275: // [58] P n n m (-P 2 2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=0.5+z;
      break;
    case 276: // [58] P m n n (-P 2n 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=0.5+z;
      break;
    case 277: // [58] P n m n (-P 2n 2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      break;

    case 278: // [59] P m m n Origin choice 1 (P 2 2ab -1ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=0.5-x;          SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      break;
    case 279: // [59] P m m n Origin choice 2 (-P 2ab 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      break;
    case 280: // [59] P n m m Origin choice 1 (P 2bc 2 -1bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      break;
    case 281: // [59] P n m m Origin choice 2 (-P 2c 2bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      break;
    case 282: // [59] P m n m Origin choice 1 (P 2ac 2ac -1ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=0.5-x;          SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-0.5+z;
      break;
    case 283: // [59] P m n m Origin choice 2 (-P 2c 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      break;

    case 284: // [60] P b c n (-P 2n 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      break;
    case 285: // [60] P c a n (-P 2n 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      break;
    case 286: // [60] P n c a (-P 2a 2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=0.5+z;
      break;
    case 287: // [60] P n a b (-P 2bc 2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      break;
    case 288: // [60] P b n a (-P 2ac 2b)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=0.5+z;
      break;
    case 289: // [60] P c n b (-P 2b 2ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=0.5+z;
      break;

    case 290: // [61] P b c a (-P 2ac 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=0.5+z;
      break;
    case 291: // [61] P c a b (-P 2bc 2ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      break;

    case 292: // [62] P n m a (-P 2ac 2n)      zeolites: MFI
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      break;
    case 293: // [62] P m n b (-P 2bc 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=0.5+z;
      break;
    case 294: // [62] P b n m (-P 2c 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=0.5+z;
      break;
    case 295: // [62] P c m n (-P 2n 2ac)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      break;
    case 296: // [62] P m c n (-P 2n 2a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=0.5+z;
      break;
    case 297: // [62] P n a m (-P 2c 2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      break;

    case 298: // [63] C m c m (-C 2c 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=-0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=0.5+z;
      break;
    case 299: // [63] C c m m (-C 2c 2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=-0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=z;
      break;
    case 300: // [63] A m m a (-A 2a 2a)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=0.5+x;         SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-0.5-x;        SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=-0.5+z;
      break;
    case 301: // [63] A m a m (-A 2 2a)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-0.5-x;        SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=0.5+x;         SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=-0.5+z;
      break;
    case 302: // [63] B b m m (-B 2 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=0.5+y;         SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=-0.5-y;        SpaceGroupElement[15].z=-0.5+z;
      break;
    case 303: // [63] B m m b (-B 2b 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=0.5+y;         SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=-0.5-y;        SpaceGroupElement[15].z=-0.5+z;
      break;

    case 304: // [64] C m c a (-C 2ac 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=-0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=0.5+z;
      break;
    case 305: // [64] C c m b (-C 2ac 2ac)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=-0.5-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=z;
      break;
    case 306: // [64] A b m a (-A 2ab 2ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=0.5+x;         SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-0.5-x;        SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=-0.5+z;
      break;
    case 307: // [64] A c a m (-A 2 2ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-0.5-x;        SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=0.5+x;         SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=-0.5+z;
      break;
    case 308: // [64] B b c m (-B 2 2ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=0.5+y;         SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-0.5-y;        SpaceGroupElement[15].z=-0.5+z;
      break;
    case 309: // [64] B m a b (-B 2ab 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=0.5+y;         SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-0.5-y;        SpaceGroupElement[15].z=-0.5+z;
      break;

    case 310: // [65] C m m m (-C 2 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=z;
      break;
    case 311: // [65] A m m m (-A 2 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=-0.5+z;
      break;
    case 312: // [65] B m m m (-B 2 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=-0.5+z;
      break;

    case 313: // [66] C c c m (-C 2 2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=0.5+z;
      break;
    case 314: // [66] A m a a (-A 2a 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=0.5+x;         SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=0.5+x;         SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=-0.5+z;
      break;
    case 315: // [66] B b m b (-B 2b 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=0.5+y;         SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=0.5+y;         SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=-0.5+z;
      break;

    case 316: // [67] C m m a (-C 2a 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=z;
      break;
    case 317: // [67] C m m b (-C 2a 2a)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=z;
      break;
    case 318: // [67] A b m m (-A 2b 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=-0.5+z;
      break;
    case 319: // [67] A c m m (-A 2 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=-0.5+z;
      break;
    case 320: // [67] B m c m (-B 2 2a)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=-0.5+z;
      break;
    case 321: // [67] B m a m (-B 2a 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=-0.5+z;
      break;

    case 322: // [68] C c c a Origin choice 1 (C 2 2 -1ac)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=0.5-x;          SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-0.5+z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=-0.5+z;
      break;
    case 323: // [68] C c c a Origin choice 2 (-C 2a 2ac)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=0.5+z;
      break;
    case 324: // [68] C c c b Origin choice 1 (C 2 2 -1ac)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=0.5+z;
      break;
    case 325: // [68] C c c b Origin choice 2 (-C 2a 2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=0.5+z;
      break;
    case 326: // [68] A b a a Origin choice 1 (A 2 2 -1ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=0.5-x;          SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=-0.5+z;
      break;
    case 327: // [68] A b a a Origin choice 2 (-A 2a 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=0.5+x;         SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=0.5+x;         SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=-0.5+z;
      break;
    case 328: // [68] A c a a Origin choice 1 (A 2 2 -1ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=0.5-x;          SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=-0.5+z;
      break;
    case 329: // [68] A c a a Origin choice 2 (-A 2ab 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=0.5+x;         SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=0.5+x;         SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=-0.5+z;
      break;
    case 330: // [68] B b c b Origin choice 1 (B 2 2 -1ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=0.5-x;          SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=-0.5+z;
      break;
    case 331: // [68] B b c b Origin choice 2 (-B 2ab 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=0.5+y;         SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=0.5+y;         SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=-0.5+z;
      break;
    case 332: // [68] B b a b Origin choice 1 (B 2 2 -1ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=0.5-x;          SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+y;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=-0.5+x;         SpaceGroupElement[7].y=0.5-y;          SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=-0.5+z;
      break;
    case 333: // [68] B b a b Origin choice 2 (-B 2b 2ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=y;              SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=0.5+y;         SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=0.5+y;         SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=-0.5+z;
      break;

    case 334: // [69] F m m m (-F 2 2)
      SpaceGroupSize=32;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=-0.5+z;
      SpaceGroupElement[16].x=-0.5+x;        SpaceGroupElement[16].y=y;             SpaceGroupElement[16].z=-0.5+z;
      SpaceGroupElement[17].x=0.5-x;         SpaceGroupElement[17].y=-y;            SpaceGroupElement[17].z=-0.5+z;
      SpaceGroupElement[18].x=-0.5+x;        SpaceGroupElement[18].y=-y;            SpaceGroupElement[18].z=0.5-z;
      SpaceGroupElement[19].x=0.5-x;         SpaceGroupElement[19].y=y;             SpaceGroupElement[19].z=0.5-z;
      SpaceGroupElement[20].x=0.5-x;         SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=0.5-z;
      SpaceGroupElement[21].x=-0.5+x;        SpaceGroupElement[21].y=y;             SpaceGroupElement[21].z=0.5-z;
      SpaceGroupElement[22].x=0.5-x;         SpaceGroupElement[22].y=y;             SpaceGroupElement[22].z=-0.5+z;
      SpaceGroupElement[23].x=-0.5+x;        SpaceGroupElement[23].y=-y;            SpaceGroupElement[23].z=-0.5+z;
      SpaceGroupElement[24].x=-0.5+x;        SpaceGroupElement[24].y=-0.5+y;        SpaceGroupElement[24].z=z;
      SpaceGroupElement[25].x=0.5-x;         SpaceGroupElement[25].y=0.5-y;         SpaceGroupElement[25].z=z;
      SpaceGroupElement[26].x=-0.5+x;        SpaceGroupElement[26].y=0.5-y;         SpaceGroupElement[26].z=-z;
      SpaceGroupElement[27].x=0.5-x;         SpaceGroupElement[27].y=-0.5+y;        SpaceGroupElement[27].z=-z;
      SpaceGroupElement[28].x=0.5-x;         SpaceGroupElement[28].y=0.5-y;         SpaceGroupElement[28].z=-z;
      SpaceGroupElement[29].x=-0.5+x;        SpaceGroupElement[29].y=-0.5+y;        SpaceGroupElement[29].z=-z;
      SpaceGroupElement[30].x=0.5-x;         SpaceGroupElement[30].y=-0.5+y;        SpaceGroupElement[30].z=z;
      SpaceGroupElement[31].x=-0.5+x;        SpaceGroupElement[31].y=0.5-y;         SpaceGroupElement[31].z=z;
      break;

    case 335: // [70] F d d d:1 Origin choice 1 (F 2 2 -1d)
      SpaceGroupSize=32;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=0.25-x;         SpaceGroupElement[4].y=0.25-y;         SpaceGroupElement[4].z=0.25-z;
      SpaceGroupElement[5].x=-0.25+x;        SpaceGroupElement[5].y=-0.25+y;        SpaceGroupElement[5].z=0.25-z;
      SpaceGroupElement[6].x=0.25-x;         SpaceGroupElement[6].y=-0.25+y;        SpaceGroupElement[6].z=-0.25+z;
      SpaceGroupElement[7].x=-0.25+x;        SpaceGroupElement[7].y=0.25-y;         SpaceGroupElement[7].z=-0.25+z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.25-x;        SpaceGroupElement[12].y=0.75-y;        SpaceGroupElement[12].z=0.75-z;
      SpaceGroupElement[13].x=-0.25+x;       SpaceGroupElement[13].y=-0.75+y;       SpaceGroupElement[13].z=0.75-z;
      SpaceGroupElement[14].x=0.25-x;        SpaceGroupElement[14].y=-0.75+y;       SpaceGroupElement[14].z=-0.75+z;
      SpaceGroupElement[15].x=-0.25+x;       SpaceGroupElement[15].y=0.75-y;        SpaceGroupElement[15].z=-0.75+z;
      SpaceGroupElement[16].x=-0.5+x;        SpaceGroupElement[16].y=y;             SpaceGroupElement[16].z=-0.5+z;
      SpaceGroupElement[17].x=0.5-x;         SpaceGroupElement[17].y=-y;            SpaceGroupElement[17].z=-0.5+z;
      SpaceGroupElement[18].x=-0.5+x;        SpaceGroupElement[18].y=-y;            SpaceGroupElement[18].z=0.5-z;
      SpaceGroupElement[19].x=0.5-x;         SpaceGroupElement[19].y=y;             SpaceGroupElement[19].z=0.5-z;
      SpaceGroupElement[20].x=0.75-x;        SpaceGroupElement[20].y=0.25-y;        SpaceGroupElement[20].z=0.75-z;
      SpaceGroupElement[21].x=-0.75+x;       SpaceGroupElement[21].y=-0.25+y;       SpaceGroupElement[21].z=0.75-z;
      SpaceGroupElement[22].x=0.75-x;        SpaceGroupElement[22].y=-0.25+y;       SpaceGroupElement[22].z=-0.75+z;
      SpaceGroupElement[23].x=-0.75+x;       SpaceGroupElement[23].y=0.25-y;        SpaceGroupElement[23].z=-0.75+z;
      SpaceGroupElement[24].x=-0.5+x;        SpaceGroupElement[24].y=-0.5+y;        SpaceGroupElement[24].z=z;
      SpaceGroupElement[25].x=0.5-x;         SpaceGroupElement[25].y=0.5-y;         SpaceGroupElement[25].z=z;
      SpaceGroupElement[26].x=-0.5+x;        SpaceGroupElement[26].y=0.5-y;         SpaceGroupElement[26].z=-z;
      SpaceGroupElement[27].x=0.5-x;         SpaceGroupElement[27].y=-0.5+y;        SpaceGroupElement[27].z=-z;
      SpaceGroupElement[28].x=0.75-x;        SpaceGroupElement[28].y=0.75-y;        SpaceGroupElement[28].z=0.25-z;
      SpaceGroupElement[29].x=-0.75+x;       SpaceGroupElement[29].y=-0.75+y;       SpaceGroupElement[29].z=0.25-z;
      SpaceGroupElement[30].x=0.75-x;        SpaceGroupElement[30].y=-0.75+y;       SpaceGroupElement[30].z=-0.25+z;
      SpaceGroupElement[31].x=-0.75+x;       SpaceGroupElement[31].y=0.75-y;        SpaceGroupElement[31].z=-0.25+z;
      break;
    case 336: // [70] F d d d:2 Origin choice 2 (-F 2uv 2vw)
      SpaceGroupSize=32;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.25-x;         SpaceGroupElement[1].y=0.25-y;         SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.25-y;         SpaceGroupElement[2].z=0.25-z;
      SpaceGroupElement[3].x=0.25-x;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.25-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.25+x;         SpaceGroupElement[5].y=0.25+y;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.25+y;         SpaceGroupElement[6].z=0.25+z;
      SpaceGroupElement[7].x=0.25+x;         SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.25+z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.25-x;         SpaceGroupElement[9].y=0.75-y;         SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=0.75-y;        SpaceGroupElement[10].z=0.75-z;
      SpaceGroupElement[11].x=0.25-x;        SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.75-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=0.25+x;        SpaceGroupElement[13].y=-0.25+y;       SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=-0.25+y;       SpaceGroupElement[14].z=-0.25+z;
      SpaceGroupElement[15].x=0.25+x;        SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=-0.25+z;
      SpaceGroupElement[16].x=-0.5+x;        SpaceGroupElement[16].y=y;             SpaceGroupElement[16].z=-0.5+z;
      SpaceGroupElement[17].x=0.75-x;        SpaceGroupElement[17].y=0.25-y;        SpaceGroupElement[17].z=-0.5+z;
      SpaceGroupElement[18].x=-0.5+x;        SpaceGroupElement[18].y=0.25-y;        SpaceGroupElement[18].z=0.75-z;
      SpaceGroupElement[19].x=0.75-x;        SpaceGroupElement[19].y=y;             SpaceGroupElement[19].z=0.75-z;
      SpaceGroupElement[20].x=0.5-x;         SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=0.5-z;
      SpaceGroupElement[21].x=-0.25+x;       SpaceGroupElement[21].y=0.25+y;        SpaceGroupElement[21].z=0.5-z;
      SpaceGroupElement[22].x=0.5-x;         SpaceGroupElement[22].y=0.25+y;        SpaceGroupElement[22].z=-0.25+z;
      SpaceGroupElement[23].x=-0.25+x;       SpaceGroupElement[23].y=-y;            SpaceGroupElement[23].z=-0.25+z;
      SpaceGroupElement[24].x=-0.5+x;        SpaceGroupElement[24].y=-0.5+y;        SpaceGroupElement[24].z=z;
      SpaceGroupElement[25].x=0.75-x;        SpaceGroupElement[25].y=0.75-y;        SpaceGroupElement[25].z=z;
      SpaceGroupElement[26].x=-0.5+x;        SpaceGroupElement[26].y=0.75-y;        SpaceGroupElement[26].z=0.25-z;
      SpaceGroupElement[27].x=0.75-x;        SpaceGroupElement[27].y=-0.5+y;        SpaceGroupElement[27].z=0.25-z;
      SpaceGroupElement[28].x=0.5-x;         SpaceGroupElement[28].y=0.5-y;         SpaceGroupElement[28].z=-z;
      SpaceGroupElement[29].x=-0.25+x;       SpaceGroupElement[29].y=-0.25+y;       SpaceGroupElement[29].z=-z;
      SpaceGroupElement[30].x=0.5-x;         SpaceGroupElement[30].y=-0.25+y;       SpaceGroupElement[30].z=0.25+z;
      SpaceGroupElement[31].x=-0.25+x;       SpaceGroupElement[31].y=0.5-y;         SpaceGroupElement[31].z=0.25+z;
      break;

    case 337: // [71] I m m m (-I 2 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=-0.5+z;
      break;

    case 338: // [72] I b a m (-I 2 2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=z;
      break;
    case 339: // [72] I m c b (-I 2a 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=-0.5+z;
      break;
    case 340: // [72] I c m a (-I 2b 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=-0.5+z;
      break;

    case 341: // [73] I b c a (-I 2b 2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=0.5+z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z;
      break;
    case 342: // [73] I c a b (-I 2a 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=-0.5+z;
      break;

    case 343: // [74] I m m a (-I 2b 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=0.5+y;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=-y;             SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=-0.5+z;
      break;
    case 344: // [74] I m m b (-I 2a 2a)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5+x;          SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=-0.5+z;
      break;
    case 345: // [74] I b m m (-I 2c 2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5+z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=-0.5+z;
      break;
    case 346: // [74] I c m m (-I 2 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-0.5-y;         SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=-0.5+z;
      break;
    case 347: // [74] I m c m (-I 2 2a)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5+x;          SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=-0.5+z;
      break;
    case 348: // [74] I m a m (-I 2c 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x;              SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5+z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=0.5-y;          SpaceGroupElement[9].z=z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=-0.5+y;        SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=z;
      break;

    // TETRAGONAL GROUPS
    // =================

    case 349: // [75] P 4 (P 4)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      break;

    case 350: // [76] P 41 (P 4w)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-0.75+z;
      break;

    case 351: // [77] P 42 (P 4c)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-0.5+z;
      break;

    case 352: // [78] P 43 (P 4cw)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.75+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-0.25+z;
      break;

    case 353: // [79] I 4 (I 4)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+y;         SpaceGroupElement[5].y=0.5-x;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=-0.5+x;         SpaceGroupElement[7].z=-0.5+z;
      break;

    case 354: // [80] I 41 (I 4bw)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=0.5-x;          SpaceGroupElement[5].z=-0.75+z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-0.25+z;
      break;

    case 355: // [81] P -4 (P -4)
      SpaceGroupSize=4;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      break;

    case 356: // [82] I -4 (I -4)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-y;          SpaceGroupElement[5].y=-0.5+x;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-0.5+y;         SpaceGroupElement[7].y=0.5-x;          SpaceGroupElement[7].z=0.5-z;
      break;

    case 357: // [83] P 4/m (-P 4)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      break;

    case 358: // [84] P 42/m (-P 4c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-0.5-z;
      break;

    case 359: // [85] P 4/n Origin choice 1 (P 4ab -1ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=0.5-x;          SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      break;
    case 360: // [85] P 4/n Origin choice 2 (-P 4a)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=0.5+x;          SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5+x;          SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=0.5+y;          SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      break;

    case 361: // [86] P 42/n Origin choice 1 (P 4n -1n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=0.5-x;          SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      break;
    case 362: // [86] P 42/n Origin choice 2 (-P 4bc)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.5-y;         SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-0.5-z;
      SpaceGroupElement[6].x=0.5+x;          SpaceGroupElement[6].y=0.5+y;          SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-0.5-x;         SpaceGroupElement[7].z=-0.5-z;
      break;

    case 363: // [87] I 4/m (-I 4)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-0.5+y;         SpaceGroupElement[9].y=0.5-x;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=0.5-x;         SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=-0.5+z;
      SpaceGroupElement[11].x=0.5-y;         SpaceGroupElement[11].y=-0.5+x;        SpaceGroupElement[11].z=-0.5+z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=0.5-y;         SpaceGroupElement[13].y=-0.5+x;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-0.5+x;        SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=0.5-z;
      SpaceGroupElement[15].x=-0.5+y;        SpaceGroupElement[15].y=0.5-x;         SpaceGroupElement[15].z=0.5-z;
      break;

    case 364: // [88] I 41/a Origin choice 1 (I 4bw -1bw)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.25-z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5+x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-0.25-z;
      SpaceGroupElement[7].x=-0.5+y;         SpaceGroupElement[7].y=-0.5-x;         SpaceGroupElement[7].z=-0.5-z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=0.5-x;          SpaceGroupElement[9].z=-0.75+z;
      SpaceGroupElement[10].x=-x;            SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=z;
      SpaceGroupElement[11].x=0.5-y;         SpaceGroupElement[11].y=x;             SpaceGroupElement[11].z=-0.25+z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=0.75-z;
      SpaceGroupElement[13].x=0.5-y;         SpaceGroupElement[13].y=-0.5+x;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=0.25-z;
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=-x;            SpaceGroupElement[15].z=-z;
      break;
    case 365: // [88] I 41/a Origin choice 2 (-I 4ad)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.25+y;        SpaceGroupElement[1].y=0.75-x;         SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.75-y;         SpaceGroupElement[3].y=-0.75+x;        SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.25-y;        SpaceGroupElement[5].y=0.75+x;         SpaceGroupElement[5].z=-0.25-z;
      SpaceGroupElement[6].x=0.5+x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-0.5-z;
      SpaceGroupElement[7].x=0.75+y;         SpaceGroupElement[7].y=-0.75-x;        SpaceGroupElement[7].z=-0.75-z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-0.75+y;        SpaceGroupElement[9].y=0.25-x;         SpaceGroupElement[9].z=-0.75+z;
      SpaceGroupElement[10].x=-x;            SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=z;
      SpaceGroupElement[11].x=0.25-y;        SpaceGroupElement[11].y=-0.25+x;       SpaceGroupElement[11].z=-0.25+z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=0.25-y;        SpaceGroupElement[13].y=0.25+x;        SpaceGroupElement[13].z=0.25-z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=-z;
      SpaceGroupElement[15].x=0.25+y;        SpaceGroupElement[15].y=-0.25-x;       SpaceGroupElement[15].z=-0.25-z;
      break;

    case 366: // [89] P 4 2 2 (P 4 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      break;

    case 367: // [90] P 4 21 2 (P 4ab 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      break;

    case 368: // [91] P 41 2 2 (P 4w 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=0.75-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=0.25-z;
      break;

    case 369: // [92] P 41 21 2 (P 4abw 2nw)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.75-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.25-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=0.5-z;
      break;

    case 370: // [93] P 42 2 2 (P 4c 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=0.5-z;
      break;

    case 371: // [94] P 42 21 2 (P 4n 2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      break;

    case 372: // [95] P 43 2 2 (P 4cw 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.75+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-0.25+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=0.25-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=0.75-z;
      break;

    case 373: // [96] P 43 21 2 (P 4nw 2abw)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=-0.75+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.25+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.25-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.75-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=0.5-z;
      break;

    case 374: // [97] I 4 2 2 (I 4 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-0.5+y;         SpaceGroupElement[9].y=0.5-x;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=0.5-x;         SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=-0.5+z;
      SpaceGroupElement[11].x=0.5-y;         SpaceGroupElement[11].y=-0.5+x;        SpaceGroupElement[11].z=-0.5+z;
      SpaceGroupElement[12].x=-0.5+x;        SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+y;        SpaceGroupElement[13].y=-0.5+x;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=0.5-z;
      SpaceGroupElement[15].x=0.5-y;         SpaceGroupElement[15].y=0.5-x;         SpaceGroupElement[15].z=0.5-z;
      break;

    case 375: // [98] I 41 2 2 (I 4bw 2bw)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.25-z;
      SpaceGroupElement[5].x=-0.5+y;         SpaceGroupElement[5].y=-0.5+x;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.75-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=0.5-x;          SpaceGroupElement[9].z=-0.75+z;
      SpaceGroupElement[10].x=-x;            SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=z;
      SpaceGroupElement[11].x=0.5-y;         SpaceGroupElement[11].y=x;             SpaceGroupElement[11].z=-0.25+z;
      SpaceGroupElement[12].x=-0.5+x;        SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=0.75-z;
      SpaceGroupElement[13].x=y;             SpaceGroupElement[13].y=x;             SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=0.25-z;
      SpaceGroupElement[15].x=0.5-y;         SpaceGroupElement[15].y=0.5-x;         SpaceGroupElement[15].z=0.5-z;
      break;

    case 376: // [99] P 4 m m (P 4 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z;
      break;

    case 377: // [100] P 4 b m (P 4 -2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=0.5-x;          SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=0.5-y;          SpaceGroupElement[5].y=0.5-x;          SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=-0.5+y;         SpaceGroupElement[7].y=-0.5+x;         SpaceGroupElement[7].z=z;
      break;

    case 378: // [101] P 42 c m (P 4c -2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z;
      break;

    case 379: // [102] P 42 n m (P 4n -2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=0.5-x;          SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z;
      break;

    case 380: // [103] P 4 c c (P 4 -2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-0.5+z;
      break;

    case 381: // [104] P 4 n c (P 4 -2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=0.5-x;          SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-y;          SpaceGroupElement[5].y=0.5-x;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-0.5+y;         SpaceGroupElement[7].y=-0.5+x;         SpaceGroupElement[7].z=-0.5+z;
      break;

    case 382: // [105] P 42 m c (P 4c -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-0.5+z;
      break;

    case 383: // [106] P 42 b c (P 4c -2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=0.5-x;          SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=0.5-y;          SpaceGroupElement[5].y=0.5-x;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=-0.5+y;         SpaceGroupElement[7].y=-0.5+x;         SpaceGroupElement[7].z=-0.5+z;
      break;

    case 384: // [107] I 4 m m (I 4 -2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-0.5+y;         SpaceGroupElement[9].y=0.5-x;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=0.5-x;         SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=-0.5+z;
      SpaceGroupElement[11].x=0.5-y;         SpaceGroupElement[11].y=-0.5+x;        SpaceGroupElement[11].z=-0.5+z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=-0.5+z;
      SpaceGroupElement[13].x=0.5-y;         SpaceGroupElement[13].y=0.5-x;         SpaceGroupElement[13].z=-0.5+z;
      SpaceGroupElement[14].x=-0.5+x;        SpaceGroupElement[14].y=0.5-y;         SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=-0.5+y;        SpaceGroupElement[15].y=-0.5+x;        SpaceGroupElement[15].z=-0.5+z;
      break;

    case 385: // [108] I 4 c m (I 4 -2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-0.5+z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-0.5+y;         SpaceGroupElement[9].y=0.5-x;          SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=0.5-x;         SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=-0.5+z;
      SpaceGroupElement[11].x=0.5-y;         SpaceGroupElement[11].y=-0.5+x;        SpaceGroupElement[11].z=-0.5+z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=0.5-y;         SpaceGroupElement[13].y=0.5-x;         SpaceGroupElement[13].z=z;
      SpaceGroupElement[14].x=-0.5+x;        SpaceGroupElement[14].y=0.5-y;         SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=-0.5+y;        SpaceGroupElement[15].y=-0.5+x;        SpaceGroupElement[15].z=z;
      break;

    case 386: // [109] I 41 m d (I 4bw -2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=0.5-y;          SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=-0.25+z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-0.5+x;         SpaceGroupElement[7].z=-0.75+z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=0.5-x;          SpaceGroupElement[9].z=-0.75+z;
      SpaceGroupElement[10].x=-x;            SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=z;
      SpaceGroupElement[11].x=0.5-y;         SpaceGroupElement[11].y=x;             SpaceGroupElement[11].z=-0.25+z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=-0.5+z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=0.5-x;         SpaceGroupElement[13].z=-0.75+z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=-0.5+y;        SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=-0.25+z;
      break;

    case 387: // [110] I 41 c d (I 4bw -2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=0.5-y;          SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=-0.75+z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-0.5+x;         SpaceGroupElement[7].z=-0.25+z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=0.5-x;          SpaceGroupElement[9].z=-0.75+z;
      SpaceGroupElement[10].x=-x;            SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=z;
      SpaceGroupElement[11].x=0.5-y;         SpaceGroupElement[11].y=x;             SpaceGroupElement[11].z=-0.25+z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=0.5-x;         SpaceGroupElement[13].z=-0.25+z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=-0.5+y;        SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=-0.75+z;
      break;

    case 388: // [111] P -4 2 m (P -4 2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z;
      break;

    case 389: // [112] P -4 2 c (P -4 2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-0.5+z;
      break;

    case 390: // [113] P -4 21 m (P -4 2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5-y;          SpaceGroupElement[5].y=0.5-x;          SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-0.5+y;         SpaceGroupElement[7].y=-0.5+x;         SpaceGroupElement[7].z=z;
      break;

    case 391: // [114] P -4 21 c (P -4 2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=0.5-y;          SpaceGroupElement[5].y=0.5-x;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-0.5+y;         SpaceGroupElement[7].y=-0.5+x;         SpaceGroupElement[7].z=-0.5+z;
      break;

    case 392: // [115] P -4 m 2 (P -4 -2)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      break;

    case 393: // [116] P -4 c 2 (P -4 -2c)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=0.5-z;
      break;

    case 394: // [117] P -4 b 2 (P -4 -2ab)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=0.5-x;          SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=-0.5+y;         SpaceGroupElement[5].y=-0.5+x;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=0.5-x;          SpaceGroupElement[7].z=-z;
      break;

    case 395: // [118] P -4 n 2 (P -4 -2n)
      SpaceGroupSize=8;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=0.5-x;          SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-0.5+y;         SpaceGroupElement[5].y=-0.5+x;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-0.5+x;         SpaceGroupElement[6].y=0.5-y;          SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=0.5-x;          SpaceGroupElement[7].z=0.5-z;
      break;

    case 396: // [119] I -4 m 2 (I -4 -2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-y;          SpaceGroupElement[9].y=-0.5+x;         SpaceGroupElement[9].z=0.5-z;
      SpaceGroupElement[10].x=0.5-x;         SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=-0.5+z;
      SpaceGroupElement[11].x=-0.5+y;        SpaceGroupElement[11].y=0.5-x;         SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=-0.5+z;
      SpaceGroupElement[13].x=-0.5+y;        SpaceGroupElement[13].y=-0.5+x;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=-0.5+x;        SpaceGroupElement[14].y=0.5-y;         SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=0.5-y;         SpaceGroupElement[15].y=0.5-x;         SpaceGroupElement[15].z=0.5-z;
      break;

    case 397: // [120] I -4 c 2 (I -4 -2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=x;              SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-y;          SpaceGroupElement[9].y=-0.5+x;         SpaceGroupElement[9].z=0.5-z;
      SpaceGroupElement[10].x=0.5-x;         SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=-0.5+z;
      SpaceGroupElement[11].x=-0.5+y;        SpaceGroupElement[11].y=0.5-x;         SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=-0.5+y;        SpaceGroupElement[13].y=-0.5+x;        SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=-0.5+x;        SpaceGroupElement[14].y=0.5-y;         SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=0.5-y;         SpaceGroupElement[15].y=0.5-x;         SpaceGroupElement[15].z=-z;
      break;

    case 398: // [121] I -4 2 m (I -4 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-y;          SpaceGroupElement[9].y=-0.5+x;         SpaceGroupElement[9].z=0.5-z;
      SpaceGroupElement[10].x=0.5-x;         SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=-0.5+z;
      SpaceGroupElement[11].x=-0.5+y;        SpaceGroupElement[11].y=0.5-x;         SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-0.5+x;        SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=0.5-y;         SpaceGroupElement[13].y=0.5-x;         SpaceGroupElement[13].z=-0.5+z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=0.5-z;
      SpaceGroupElement[15].x=-0.5+y;        SpaceGroupElement[15].y=-0.5+x;        SpaceGroupElement[15].z=-0.5+z;
      break;

    case 399: // [122] I -4 2 d (I -4 2bw)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.25-z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=0.5-x;          SpaceGroupElement[5].z=-0.75+z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.25-z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-0.5+x;         SpaceGroupElement[7].z=-0.75+z;
      SpaceGroupElement[8].x=-0.5+x;         SpaceGroupElement[8].y=-0.5+y;         SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=0.5-y;          SpaceGroupElement[9].y=-0.5+x;         SpaceGroupElement[9].z=0.5-z;
      SpaceGroupElement[10].x=0.5-x;         SpaceGroupElement[10].y=0.5-y;         SpaceGroupElement[10].z=-0.5+z;
      SpaceGroupElement[11].x=-0.5+y;        SpaceGroupElement[11].y=0.5-x;         SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-0.5+x;        SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=0.75-z;
      SpaceGroupElement[13].x=0.5-y;         SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=-0.25+z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=0.75-z;
      SpaceGroupElement[15].x=-0.5+y;        SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=-0.25+z;
      break;

    case 400: // [123] P 4/m m m (-P 4 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=y;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z;
      break;

    case 401: // [124] P 4/m c c (-P 4 2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=y;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=0.5+z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=0.5+z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=0.5+z;
      break;

    case 402: // [125] P 4/n b m Origin choice 1 (P 4 2 -1ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=0.5-x;          SpaceGroupElement[8].y=0.5-y;          SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=0.5-y;          SpaceGroupElement[9].y=-0.5+x;         SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-0.5+y;        SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=-0.5+y;        SpaceGroupElement[11].y=0.5-x;         SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=0.5-y;         SpaceGroupElement[13].y=0.5-x;         SpaceGroupElement[13].z=z;
      SpaceGroupElement[14].x=-0.5+x;        SpaceGroupElement[14].y=0.5-y;         SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=-0.5+y;        SpaceGroupElement[15].y=-0.5+x;        SpaceGroupElement[15].z=z;
      break;
    case 403: // [125] P 4/n b m Origin choice 2 (-P 4a 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=0.5-x;          SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=0.5+x;          SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=0.5+x;         SpaceGroupElement[10].y=0.5+y;         SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=0.5+y;         SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=0.5+y;         SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z;
      SpaceGroupElement[14].x=0.5+x;         SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=0.5+y;         SpaceGroupElement[15].y=0.5+x;         SpaceGroupElement[15].z=z;
      break;

    case 404: // [126] P 4/n n c Origin choice 1 (P 4 2 -1n)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=0.5-x;          SpaceGroupElement[8].y=0.5-y;          SpaceGroupElement[8].z=0.5-z;
      SpaceGroupElement[9].x=0.5-y;          SpaceGroupElement[9].y=-0.5+x;         SpaceGroupElement[9].z=0.5-z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-0.5+y;        SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-0.5+y;        SpaceGroupElement[11].y=0.5-x;         SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=-0.5+z;
      SpaceGroupElement[13].x=0.5-y;         SpaceGroupElement[13].y=0.5-x;         SpaceGroupElement[13].z=-0.5+z;
      SpaceGroupElement[14].x=-0.5+x;        SpaceGroupElement[14].y=0.5-y;         SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=-0.5+y;        SpaceGroupElement[15].y=-0.5+x;        SpaceGroupElement[15].z=-0.5+z;
      break;
    case 405: // [126] P 4/n n c Origin choice 2 (-P 4a 2bc)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=0.5-x;          SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=0.5+x;          SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=0.5+x;         SpaceGroupElement[10].y=0.5+y;         SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=0.5+y;         SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=0.5+y;         SpaceGroupElement[12].z=0.5+z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=0.5+z;
      SpaceGroupElement[14].x=0.5+x;         SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=0.5+y;         SpaceGroupElement[15].y=0.5+x;         SpaceGroupElement[15].z=0.5+z;
      break;

    case 406: // [127] P 4/m b m (-P 4 2ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.5+y;         SpaceGroupElement[5].y=-0.5+x;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=0.5-x;          SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=y;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=-0.5-x;        SpaceGroupElement[12].y=0.5+y;         SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=-0.5-y;        SpaceGroupElement[13].y=-0.5-x;        SpaceGroupElement[13].z=z;
      SpaceGroupElement[14].x=0.5+x;         SpaceGroupElement[14].y=-0.5-y;        SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=0.5+y;         SpaceGroupElement[15].y=0.5+x;         SpaceGroupElement[15].z=z;
      break;

    case 407: // [128] P 4/m n c (-P 4 2n)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=-0.5+y;         SpaceGroupElement[5].y=-0.5+x;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=0.5-x;          SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=y;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=-0.5-x;        SpaceGroupElement[12].y=0.5+y;         SpaceGroupElement[12].z=0.5+z;
      SpaceGroupElement[13].x=-0.5-y;        SpaceGroupElement[13].y=-0.5-x;        SpaceGroupElement[13].z=0.5+z;
      SpaceGroupElement[14].x=0.5+x;         SpaceGroupElement[14].y=-0.5-y;        SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=0.5+y;         SpaceGroupElement[15].y=0.5+x;         SpaceGroupElement[15].z=0.5+z;
      break;

    case 408: // [129] P 4/n m m Origin choice 1 (P 4ab 2ab -1ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=0.5-x;          SpaceGroupElement[8].y=0.5-y;          SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-0.5+y;        SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=y;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=0.5-y;         SpaceGroupElement[13].y=0.5-x;         SpaceGroupElement[13].z=z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=-0.5+y;        SpaceGroupElement[15].y=-0.5+x;        SpaceGroupElement[15].z=z;
      break;
    case 409: // [129] P 4/n m m Origin choice 2 (-P 4a 2a)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.5+y;         SpaceGroupElement[5].y=-0.5+x;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=0.5+x;          SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=0.5+x;         SpaceGroupElement[10].y=0.5+y;         SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=0.5+y;         SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=-0.5-x;        SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=-0.5-y;        SpaceGroupElement[13].y=-0.5-x;        SpaceGroupElement[13].z=z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-0.5-y;        SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z;
      break;

    case 410: // [130] P 4/n c c Origin choice 1 (P 4ab 2n -1ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=0.5-x;          SpaceGroupElement[8].y=0.5-y;          SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-0.5+y;        SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=y;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=0.5+z;
      SpaceGroupElement[13].x=0.5-y;         SpaceGroupElement[13].y=0.5-x;         SpaceGroupElement[13].z=0.5+z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=-0.5+y;        SpaceGroupElement[15].y=-0.5+x;        SpaceGroupElement[15].z=0.5+z;
      break;
    case 411: // [130] P 4/n c c Origin choice 2 (-P 4a 2ac)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=-0.5+y;         SpaceGroupElement[5].y=-0.5+x;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=0.5+x;          SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=0.5+x;         SpaceGroupElement[10].y=0.5+y;         SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=0.5+y;         SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=-0.5-x;        SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=0.5+z;
      SpaceGroupElement[13].x=-0.5-y;        SpaceGroupElement[13].y=-0.5-x;        SpaceGroupElement[13].z=0.5+z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-0.5-y;        SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=0.5+z;
      break;

    case 412: // [131] P 42/m m c (-P 4c 2)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-0.5-z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=y;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=0.5+z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=0.5+z;
      break;

    case 413: // [132] P 42/m c m (-P 4c 2c)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-0.5-z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=y;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=0.5+z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z;
      break;

    case 414: // [133] P 42/n b c Origin choice 1 (P 4n 2c -1n)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=-0.5+y;         SpaceGroupElement[5].y=-0.5+x;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=0.5-x;          SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=0.5-x;          SpaceGroupElement[8].y=0.5-y;          SpaceGroupElement[8].z=0.5-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-0.5+y;        SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=y;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=-0.5+z;
      SpaceGroupElement[14].x=-0.5+x;        SpaceGroupElement[14].y=0.5-y;         SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=-0.5+z;
      break;
    case 415: // [133] P 42/n b c Origin choice 2 (-P 4ac 2b)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=0.5-x;          SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=0.5+x;          SpaceGroupElement[9].z=-0.5-z;
      SpaceGroupElement[10].x=0.5+x;         SpaceGroupElement[10].y=0.5+y;         SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=0.5+y;         SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=0.5+y;         SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=0.5+z;
      SpaceGroupElement[14].x=0.5+x;         SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=0.5+y;         SpaceGroupElement[15].y=0.5+x;         SpaceGroupElement[15].z=0.5+z;
      break;

    case 416: // [134] P 42/n n m Origin choice 1 (P 4n 2 -1n)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.5+y;         SpaceGroupElement[5].y=-0.5+x;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=0.5-x;          SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=0.5-x;          SpaceGroupElement[8].y=0.5-y;          SpaceGroupElement[8].z=0.5-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-0.5+y;        SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=y;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=-0.5+z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z;
      SpaceGroupElement[14].x=-0.5+x;        SpaceGroupElement[14].y=0.5-y;         SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z;
      break;
    case 417: // [134] P 42/n n m Origin choice 2 (-P 4ac 2bc)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=0.5-x;          SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=0.5+x;          SpaceGroupElement[9].z=-0.5-z;
      SpaceGroupElement[10].x=0.5+x;         SpaceGroupElement[10].y=0.5+y;         SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=0.5+y;         SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=0.5+y;         SpaceGroupElement[12].z=0.5+z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z;
      SpaceGroupElement[14].x=0.5+x;         SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=0.5+y;         SpaceGroupElement[15].y=0.5+x;         SpaceGroupElement[15].z=z;
      break;

    case 418: // [135] P 42/m b c (-P 4c 2ab)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.5+y;         SpaceGroupElement[5].y=-0.5+x;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=0.5-x;          SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-0.5-z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=y;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-0.5-z;
      SpaceGroupElement[12].x=-0.5-x;        SpaceGroupElement[12].y=0.5+y;         SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=-0.5-y;        SpaceGroupElement[13].y=-0.5-x;        SpaceGroupElement[13].z=0.5+z;
      SpaceGroupElement[14].x=0.5+x;         SpaceGroupElement[14].y=-0.5-y;        SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=0.5+y;         SpaceGroupElement[15].y=0.5+x;         SpaceGroupElement[15].z=0.5+z;
      break;

    case 419: // [136] P 42/m n m (-P 4n 2n)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-0.5-y;         SpaceGroupElement[9].y=0.5+x;          SpaceGroupElement[9].z=-0.5-z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=0.5+y;         SpaceGroupElement[11].y=-0.5-x;        SpaceGroupElement[11].z=-0.5-z;
      SpaceGroupElement[12].x=-0.5-x;        SpaceGroupElement[12].y=0.5+y;         SpaceGroupElement[12].z=0.5+z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z;
      SpaceGroupElement[14].x=0.5+x;         SpaceGroupElement[14].y=-0.5-y;        SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z;
      break;

    case 420: // [137] P 42/n m c Origin choice 1 (P 4n 2n -1n)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=0.5-x;          SpaceGroupElement[8].y=0.5-y;          SpaceGroupElement[8].z=0.5-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-0.5+y;        SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=y;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=0.5-y;         SpaceGroupElement[13].y=0.5-x;         SpaceGroupElement[13].z=-0.5+z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=-0.5+y;        SpaceGroupElement[15].y=-0.5+x;        SpaceGroupElement[15].z=-0.5+z;
      break;
    case 421: // [137] P 42/n m c Origin choice 2 (-P 4ac 2a)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.5+y;         SpaceGroupElement[5].y=-0.5+x;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=0.5+x;          SpaceGroupElement[9].z=-0.5-z;
      SpaceGroupElement[10].x=0.5+x;         SpaceGroupElement[10].y=0.5+y;         SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=0.5+y;         SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-0.5-z;
      SpaceGroupElement[12].x=-0.5-x;        SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=-0.5-y;        SpaceGroupElement[13].y=-0.5-x;        SpaceGroupElement[13].z=0.5+z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-0.5-y;        SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=0.5+z;
      break;

    case 422: // [138] P 42/n c m Origin choice 1 (P 4n 2ab -1n)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=0.5-x;          SpaceGroupElement[8].y=0.5-y;          SpaceGroupElement[8].z=0.5-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=-0.5+x;        SpaceGroupElement[10].y=-0.5+y;        SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=y;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=-0.5+z;
      SpaceGroupElement[13].x=0.5-y;         SpaceGroupElement[13].y=0.5-x;         SpaceGroupElement[13].z=z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=-0.5+y;        SpaceGroupElement[15].y=-0.5+x;        SpaceGroupElement[15].z=z;
      break;
    case 423: // [138] P 42/n c m Origin choice 2 (-P 4ac 2ac)
      SpaceGroupSize=16;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=-0.5+y;         SpaceGroupElement[5].y=-0.5+x;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=0.5+x;          SpaceGroupElement[9].z=-0.5-z;
      SpaceGroupElement[10].x=0.5+x;         SpaceGroupElement[10].y=0.5+y;         SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=0.5+y;         SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-0.5-z;
      SpaceGroupElement[12].x=-0.5-x;        SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=0.5+z;
      SpaceGroupElement[13].x=-0.5-y;        SpaceGroupElement[13].y=-0.5-x;        SpaceGroupElement[13].z=z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-0.5-y;        SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z;
      break;

    case 424: // [139] I 4/m m m (-I 4 2)
      SpaceGroupSize=32;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=y;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=z;
      SpaceGroupElement[16].x=-0.5+x;        SpaceGroupElement[16].y=-0.5+y;        SpaceGroupElement[16].z=-0.5+z;
      SpaceGroupElement[17].x=-0.5+y;        SpaceGroupElement[17].y=0.5-x;         SpaceGroupElement[17].z=-0.5+z;
      SpaceGroupElement[18].x=0.5-x;         SpaceGroupElement[18].y=0.5-y;         SpaceGroupElement[18].z=-0.5+z;
      SpaceGroupElement[19].x=0.5-y;         SpaceGroupElement[19].y=-0.5+x;        SpaceGroupElement[19].z=-0.5+z;
      SpaceGroupElement[20].x=-0.5+x;        SpaceGroupElement[20].y=0.5-y;         SpaceGroupElement[20].z=0.5-z;
      SpaceGroupElement[21].x=-0.5+y;        SpaceGroupElement[21].y=-0.5+x;        SpaceGroupElement[21].z=0.5-z;
      SpaceGroupElement[22].x=0.5-x;         SpaceGroupElement[22].y=-0.5+y;        SpaceGroupElement[22].z=0.5-z;
      SpaceGroupElement[23].x=0.5-y;         SpaceGroupElement[23].y=0.5-x;         SpaceGroupElement[23].z=0.5-z;
      SpaceGroupElement[24].x=0.5-x;         SpaceGroupElement[24].y=0.5-y;         SpaceGroupElement[24].z=0.5-z;
      SpaceGroupElement[25].x=0.5-y;         SpaceGroupElement[25].y=-0.5+x;        SpaceGroupElement[25].z=0.5-z;
      SpaceGroupElement[26].x=-0.5+x;        SpaceGroupElement[26].y=-0.5+y;        SpaceGroupElement[26].z=0.5-z;
      SpaceGroupElement[27].x=-0.5+y;        SpaceGroupElement[27].y=0.5-x;         SpaceGroupElement[27].z=0.5-z;
      SpaceGroupElement[28].x=0.5-x;         SpaceGroupElement[28].y=-0.5+y;        SpaceGroupElement[28].z=-0.5+z;
      SpaceGroupElement[29].x=0.5-y;         SpaceGroupElement[29].y=0.5-x;         SpaceGroupElement[29].z=-0.5+z;
      SpaceGroupElement[30].x=-0.5+x;        SpaceGroupElement[30].y=0.5-y;         SpaceGroupElement[30].z=-0.5+z;
      SpaceGroupElement[31].x=-0.5+y;        SpaceGroupElement[31].y=-0.5+x;        SpaceGroupElement[31].z=-0.5+z;
      break;

    case 425: // [140] I 4/m c m (-I 4 2c)
      SpaceGroupSize=32;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=y;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=0.5+z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=0.5+z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=0.5+z;
      SpaceGroupElement[16].x=-0.5+x;        SpaceGroupElement[16].y=-0.5+y;        SpaceGroupElement[16].z=-0.5+z;
      SpaceGroupElement[17].x=-0.5+y;        SpaceGroupElement[17].y=0.5-x;         SpaceGroupElement[17].z=-0.5+z;
      SpaceGroupElement[18].x=0.5-x;         SpaceGroupElement[18].y=0.5-y;         SpaceGroupElement[18].z=-0.5+z;
      SpaceGroupElement[19].x=0.5-y;         SpaceGroupElement[19].y=-0.5+x;        SpaceGroupElement[19].z=-0.5+z;
      SpaceGroupElement[20].x=-0.5+x;        SpaceGroupElement[20].y=0.5-y;         SpaceGroupElement[20].z=-z;
      SpaceGroupElement[21].x=-0.5+y;        SpaceGroupElement[21].y=-0.5+x;        SpaceGroupElement[21].z=-z;
      SpaceGroupElement[22].x=0.5-x;         SpaceGroupElement[22].y=-0.5+y;        SpaceGroupElement[22].z=-z;
      SpaceGroupElement[23].x=0.5-y;         SpaceGroupElement[23].y=0.5-x;         SpaceGroupElement[23].z=-z;
      SpaceGroupElement[24].x=0.5-x;         SpaceGroupElement[24].y=0.5-y;         SpaceGroupElement[24].z=0.5-z;
      SpaceGroupElement[25].x=0.5-y;         SpaceGroupElement[25].y=-0.5+x;        SpaceGroupElement[25].z=0.5-z;
      SpaceGroupElement[26].x=-0.5+x;        SpaceGroupElement[26].y=-0.5+y;        SpaceGroupElement[26].z=0.5-z;
      SpaceGroupElement[27].x=-0.5+y;        SpaceGroupElement[27].y=0.5-x;         SpaceGroupElement[27].z=0.5-z;
      SpaceGroupElement[28].x=0.5-x;         SpaceGroupElement[28].y=-0.5+y;        SpaceGroupElement[28].z=z;
      SpaceGroupElement[29].x=0.5-y;         SpaceGroupElement[29].y=0.5-x;         SpaceGroupElement[29].z=z;
      SpaceGroupElement[30].x=-0.5+x;        SpaceGroupElement[30].y=0.5-y;         SpaceGroupElement[30].z=z;
      SpaceGroupElement[31].x=-0.5+y;        SpaceGroupElement[31].y=-0.5+x;        SpaceGroupElement[31].z=z;
      break;

    case 426: // [141] I 41/a m d Origin choice 1 (I 4bw 2bw -1bw)
      SpaceGroupSize=32;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.25-z;
      SpaceGroupElement[5].x=-0.5+y;         SpaceGroupElement[5].y=-0.5+x;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.75-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=0.5-y;          SpaceGroupElement[8].z=0.25-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=0.5+x;         SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-0.25-z;
      SpaceGroupElement[11].x=-0.5+y;        SpaceGroupElement[11].y=-0.5-x;        SpaceGroupElement[11].z=-0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=-0.5-x;        SpaceGroupElement[13].z=0.25+z;
      SpaceGroupElement[14].x=0.5+x;         SpaceGroupElement[14].y=0.5-y;         SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=-0.5+y;        SpaceGroupElement[15].y=x;             SpaceGroupElement[15].z=-0.25+z;
      SpaceGroupElement[16].x=-0.5+x;        SpaceGroupElement[16].y=-0.5+y;        SpaceGroupElement[16].z=-0.5+z;
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=0.5-x;         SpaceGroupElement[17].z=-0.75+z;
      SpaceGroupElement[18].x=-x;            SpaceGroupElement[18].y=-y;            SpaceGroupElement[18].z=z;
      SpaceGroupElement[19].x=0.5-y;         SpaceGroupElement[19].y=x;             SpaceGroupElement[19].z=-0.25+z;
      SpaceGroupElement[20].x=-0.5+x;        SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=0.75-z;
      SpaceGroupElement[21].x=y;             SpaceGroupElement[21].y=x;             SpaceGroupElement[21].z=-z;
      SpaceGroupElement[22].x=-x;            SpaceGroupElement[22].y=-0.5+y;        SpaceGroupElement[22].z=0.25-z;
      SpaceGroupElement[23].x=0.5-y;         SpaceGroupElement[23].y=0.5-x;         SpaceGroupElement[23].z=0.5-z;
      SpaceGroupElement[24].x=0.5-x;         SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=0.75-z;
      SpaceGroupElement[25].x=0.5-y;         SpaceGroupElement[25].y=-0.5+x;        SpaceGroupElement[25].z=0.5-z;
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=-0.5+y;        SpaceGroupElement[26].z=0.25-z;
      SpaceGroupElement[27].x=y;             SpaceGroupElement[27].y=-x;            SpaceGroupElement[27].z=-z;
      SpaceGroupElement[28].x=0.5-x;         SpaceGroupElement[28].y=-0.5+y;        SpaceGroupElement[28].z=-0.5+z;
      SpaceGroupElement[29].x=0.5-y;         SpaceGroupElement[29].y=-x;            SpaceGroupElement[29].z=-0.25+z;
      SpaceGroupElement[30].x=x;             SpaceGroupElement[30].y=-y;            SpaceGroupElement[30].z=z;
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=-0.5+x;        SpaceGroupElement[31].z=-0.75+z;
      break;
    case 427: // [141] I 41/a m d Origin choice 2 (-I 4bd 2)
      SpaceGroupSize=32;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.75+y;        SpaceGroupElement[1].y=0.25-x;         SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.25-y;         SpaceGroupElement[3].y=-0.25+x;        SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.75+y;        SpaceGroupElement[5].y=-0.25+x;        SpaceGroupElement[5].z=0.25-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.25-y;         SpaceGroupElement[7].y=0.25-x;         SpaceGroupElement[7].z=0.75-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-0.75-y;        SpaceGroupElement[9].y=0.25+x;         SpaceGroupElement[9].z=-0.25-z;
      SpaceGroupElement[10].x=0.5+x;         SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-0.5-z;
      SpaceGroupElement[11].x=0.25+y;        SpaceGroupElement[11].y=-0.25-x;       SpaceGroupElement[11].z=-0.75-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=-0.75-y;       SpaceGroupElement[13].y=-0.25-x;       SpaceGroupElement[13].z=0.25+z;
      SpaceGroupElement[14].x=0.5+x;         SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=0.25+y;        SpaceGroupElement[15].y=0.25+x;        SpaceGroupElement[15].z=0.75+z;
      SpaceGroupElement[16].x=-0.5+x;        SpaceGroupElement[16].y=-0.5+y;        SpaceGroupElement[16].z=-0.5+z;
      SpaceGroupElement[17].x=-0.25+y;       SpaceGroupElement[17].y=0.75-x;        SpaceGroupElement[17].z=-0.75+z;
      SpaceGroupElement[18].x=-x;            SpaceGroupElement[18].y=0.5-y;         SpaceGroupElement[18].z=z;
      SpaceGroupElement[19].x=0.75-y;        SpaceGroupElement[19].y=-0.75+x;       SpaceGroupElement[19].z=-0.25+z;
      SpaceGroupElement[20].x=-0.5+x;        SpaceGroupElement[20].y=0.5-y;         SpaceGroupElement[20].z=0.5-z;
      SpaceGroupElement[21].x=-0.25+y;       SpaceGroupElement[21].y=-0.75+x;       SpaceGroupElement[21].z=0.75-z;
      SpaceGroupElement[22].x=-x;            SpaceGroupElement[22].y=-0.5+y;        SpaceGroupElement[22].z=-z;
      SpaceGroupElement[23].x=0.75-y;        SpaceGroupElement[23].y=0.75-x;        SpaceGroupElement[23].z=0.25-z;
      SpaceGroupElement[24].x=0.5-x;         SpaceGroupElement[24].y=0.5-y;         SpaceGroupElement[24].z=0.5-z;
      SpaceGroupElement[25].x=-0.25-y;       SpaceGroupElement[25].y=-0.25+x;       SpaceGroupElement[25].z=0.25-z;
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=-0.5+y;        SpaceGroupElement[26].z=-z;
      SpaceGroupElement[27].x=-0.25+y;       SpaceGroupElement[27].y=0.25-x;        SpaceGroupElement[27].z=-0.25-z;
      SpaceGroupElement[28].x=0.5-x;         SpaceGroupElement[28].y=-0.5+y;        SpaceGroupElement[28].z=-0.5+z;
      SpaceGroupElement[29].x=-0.25-y;       SpaceGroupElement[29].y=0.25-x;        SpaceGroupElement[29].z=-0.25+z;
      SpaceGroupElement[30].x=x;             SpaceGroupElement[30].y=0.5-y;         SpaceGroupElement[30].z=z;
      SpaceGroupElement[31].x=-0.25+y;       SpaceGroupElement[31].y=-0.25+x;       SpaceGroupElement[31].z=0.25+z;
      break;

    case 428: // [142] I 41/a c d Origin choice 1 (I 4bw 2aw -1bw)
      SpaceGroupSize=32;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=0.25-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.75-z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=0.5-x;          SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=0.5-y;          SpaceGroupElement[8].z=0.25-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=0.5+x;         SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-0.25-z;
      SpaceGroupElement[11].x=-0.5+y;        SpaceGroupElement[11].y=-0.5-x;        SpaceGroupElement[11].z=-0.5-z;
      SpaceGroupElement[12].x=-0.5-x;        SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=z;
      SpaceGroupElement[13].x=0.5-y;         SpaceGroupElement[13].y=-x;            SpaceGroupElement[13].z=0.25+z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=y;             SpaceGroupElement[15].y=0.5+x;         SpaceGroupElement[15].z=-0.25+z;
      SpaceGroupElement[16].x=-0.5+x;        SpaceGroupElement[16].y=-0.5+y;        SpaceGroupElement[16].z=-0.5+z;
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=0.5-x;         SpaceGroupElement[17].z=-0.75+z;
      SpaceGroupElement[18].x=-x;            SpaceGroupElement[18].y=-y;            SpaceGroupElement[18].z=z;
      SpaceGroupElement[19].x=0.5-y;         SpaceGroupElement[19].y=x;             SpaceGroupElement[19].z=-0.25+z;
      SpaceGroupElement[20].x=x;             SpaceGroupElement[20].y=0.5-y;         SpaceGroupElement[20].z=0.75-z;
      SpaceGroupElement[21].x=-0.5+y;        SpaceGroupElement[21].y=-0.5+x;        SpaceGroupElement[21].z=-z;
      SpaceGroupElement[22].x=0.5-x;         SpaceGroupElement[22].y=y;             SpaceGroupElement[22].z=0.25-z;
      SpaceGroupElement[23].x=-y;            SpaceGroupElement[23].y=-x;            SpaceGroupElement[23].z=0.5-z;
      SpaceGroupElement[24].x=0.5-x;         SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=0.75-z;
      SpaceGroupElement[25].x=0.5-y;         SpaceGroupElement[25].y=-0.5+x;        SpaceGroupElement[25].z=0.5-z;
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=-0.5+y;        SpaceGroupElement[26].z=0.25-z;
      SpaceGroupElement[27].x=y;             SpaceGroupElement[27].y=-x;            SpaceGroupElement[27].z=-z;
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=y;             SpaceGroupElement[28].z=-0.5+z;
      SpaceGroupElement[29].x=-y;            SpaceGroupElement[29].y=0.5-x;         SpaceGroupElement[29].z=-0.25+z;
      SpaceGroupElement[30].x=-0.5+x;        SpaceGroupElement[30].y=0.5-y;         SpaceGroupElement[30].z=z;
      SpaceGroupElement[31].x=-0.5+y;        SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=-0.75+z;
      break;
    case 429: // [142] I 41/a c d Origin choice 2 (-I 4bd 2c)
      SpaceGroupSize=32;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.75+y;        SpaceGroupElement[1].y=0.25-x;         SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.25-y;         SpaceGroupElement[3].y=-0.25+x;        SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=-0.75+y;        SpaceGroupElement[5].y=-0.25+x;        SpaceGroupElement[5].z=0.75-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=0.25-y;         SpaceGroupElement[7].y=0.25-x;         SpaceGroupElement[7].z=0.25-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-y;             SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-0.75-y;        SpaceGroupElement[9].y=0.25+x;         SpaceGroupElement[9].z=-0.25-z;
      SpaceGroupElement[10].x=0.5+x;         SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-0.5-z;
      SpaceGroupElement[11].x=0.25+y;        SpaceGroupElement[11].y=-0.25-x;       SpaceGroupElement[11].z=-0.75-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=y;             SpaceGroupElement[12].z=0.5+z;
      SpaceGroupElement[13].x=-0.75-y;       SpaceGroupElement[13].y=-0.25-x;       SpaceGroupElement[13].z=0.75+z;
      SpaceGroupElement[14].x=0.5+x;         SpaceGroupElement[14].y=-y;            SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=0.25+y;        SpaceGroupElement[15].y=0.25+x;        SpaceGroupElement[15].z=0.25+z;
      SpaceGroupElement[16].x=-0.5+x;        SpaceGroupElement[16].y=-0.5+y;        SpaceGroupElement[16].z=-0.5+z;
      SpaceGroupElement[17].x=-0.25+y;       SpaceGroupElement[17].y=0.75-x;        SpaceGroupElement[17].z=-0.75+z;
      SpaceGroupElement[18].x=-x;            SpaceGroupElement[18].y=0.5-y;         SpaceGroupElement[18].z=z;
      SpaceGroupElement[19].x=0.75-y;        SpaceGroupElement[19].y=-0.75+x;       SpaceGroupElement[19].z=-0.25+z;
      SpaceGroupElement[20].x=-0.5+x;        SpaceGroupElement[20].y=0.5-y;         SpaceGroupElement[20].z=-z;
      SpaceGroupElement[21].x=-0.25+y;       SpaceGroupElement[21].y=-0.75+x;       SpaceGroupElement[21].z=0.25-z;
      SpaceGroupElement[22].x=-x;            SpaceGroupElement[22].y=-0.5+y;        SpaceGroupElement[22].z=0.5-z;
      SpaceGroupElement[23].x=0.75-y;        SpaceGroupElement[23].y=0.75-x;        SpaceGroupElement[23].z=0.75-z;
      SpaceGroupElement[24].x=0.5-x;         SpaceGroupElement[24].y=0.5-y;         SpaceGroupElement[24].z=0.5-z;
      SpaceGroupElement[25].x=-0.25-y;       SpaceGroupElement[25].y=-0.25+x;       SpaceGroupElement[25].z=0.25-z;
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=-0.5+y;        SpaceGroupElement[26].z=-z;
      SpaceGroupElement[27].x=-0.25+y;       SpaceGroupElement[27].y=0.25-x;        SpaceGroupElement[27].z=-0.25-z;
      SpaceGroupElement[28].x=0.5-x;         SpaceGroupElement[28].y=-0.5+y;        SpaceGroupElement[28].z=z;
      SpaceGroupElement[29].x=-0.25-y;       SpaceGroupElement[29].y=0.25-x;        SpaceGroupElement[29].z=0.25+z;
      SpaceGroupElement[30].x=x;             SpaceGroupElement[30].y=0.5-y;         SpaceGroupElement[30].z=-0.5+z;
      SpaceGroupElement[31].x=-0.25+y;       SpaceGroupElement[31].y=-0.25+x;       SpaceGroupElement[31].z=-0.25+z;
      break;


    // TRIGONAL GROUPS
    // ===============

    case 430: // [143] P 3 (P 3)
      SpaceGroupSize=3;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;
      break;

    case 431: // [144] P 31 (P 31)
      SpaceGroupSize=3;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-1.0/3.0+z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=-2.0/3.0+z;
      break;

    case 432: // [145] P 32 (P 32)
      SpaceGroupSize=3;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-2.0/3.0+z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=-1.0/3.0+z;
      break;

    case 433: // [146] R 3 hexagonal axes (R 3)
      SpaceGroupSize=9;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-2.0/3.0+x;     SpaceGroupElement[3].y=-1.0/3.0+y;     SpaceGroupElement[3].z=-1.0/3.0+z;
      SpaceGroupElement[4].x=1.0/3.0-x+y;    SpaceGroupElement[4].y=2.0/3.0-x;      SpaceGroupElement[4].z=-1.0/3.0+z;
      SpaceGroupElement[5].x=1.0/3.0-y;      SpaceGroupElement[5].y=-1.0/3.0+x-y;   SpaceGroupElement[5].z=-1.0/3.0+z;
      SpaceGroupElement[6].x=-1.0/3.0+x;     SpaceGroupElement[6].y=-2.0/3.0+y;     SpaceGroupElement[6].z=-2.0/3.0+z;
      SpaceGroupElement[7].x=-1.0/3.0-x+y;   SpaceGroupElement[7].y=1.0/3.0-x;      SpaceGroupElement[7].z=-2.0/3.0+z;
      SpaceGroupElement[8].x=2.0/3.0-y;      SpaceGroupElement[8].y=1.0/3.0+x-y;    SpaceGroupElement[8].z=-2.0/3.0+z;
      break;
    case 434: // [146] R 3 Rhombohedral axes (P 3*)
      SpaceGroupSize=3;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=z;              SpaceGroupElement[1].z=x;
      SpaceGroupElement[2].x=z;              SpaceGroupElement[2].y=x;              SpaceGroupElement[2].z=y;
      break;

    case 435: // [147] P -3 (P -3)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=x-y;            SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x+y;           SpaceGroupElement[4].y=-x;             SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=-z;
      break;

    case 436: // [148] R-3 hexagonal axes (-R 3)
      SpaceGroupSize=18;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x-y;            SpaceGroupElement[4].y=x;              SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-x+y;           SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-2.0/3.0+x;     SpaceGroupElement[6].y=-1.0/3.0+y;     SpaceGroupElement[6].z=-1.0/3.0+z;
      SpaceGroupElement[7].x=1.0/3.0-x+y;    SpaceGroupElement[7].y=2.0/3.0-x;      SpaceGroupElement[7].z=-1.0/3.0+z;
      SpaceGroupElement[8].x=1.0/3.0-y;      SpaceGroupElement[8].y=-1.0/3.0+x-y;   SpaceGroupElement[8].z=-1.0/3.0+z;
      SpaceGroupElement[9].x=2.0/3.0-x;      SpaceGroupElement[9].y=1.0/3.0-y;      SpaceGroupElement[9].z=1.0/3.0-z;
      SpaceGroupElement[10].x=-1.0/3.0+x-y;  SpaceGroupElement[10].y=-2.0/3.0+x;    SpaceGroupElement[10].z=1.0/3.0-z;
      SpaceGroupElement[11].x=-1.0/3.0+y;    SpaceGroupElement[11].y=1.0/3.0-x+y;   SpaceGroupElement[11].z=1.0/3.0-z;
      SpaceGroupElement[12].x=-1.0/3.0+x;    SpaceGroupElement[12].y=-2.0/3.0+y;    SpaceGroupElement[12].z=-2.0/3.0+z;
      SpaceGroupElement[13].x=-1.0/3.0-x+y;  SpaceGroupElement[13].y=1.0/3.0-x;     SpaceGroupElement[13].z=-2.0/3.0+z;
      SpaceGroupElement[14].x=2.0/3.0-y;     SpaceGroupElement[14].y=1.0/3.0+x-y;   SpaceGroupElement[14].z=-2.0/3.0+z;
      SpaceGroupElement[15].x=1.0/3.0-x;     SpaceGroupElement[15].y=2.0/3.0-y;     SpaceGroupElement[15].z=2.0/3.0-z;
      SpaceGroupElement[16].x=1.0/3.0+x-y;   SpaceGroupElement[16].y=-1.0/3.0+x;    SpaceGroupElement[16].z=2.0/3.0-z;
      SpaceGroupElement[17].x=-2.0/3.0+y;    SpaceGroupElement[17].y=-1.0/3.0-x+y;  SpaceGroupElement[17].z=2.0/3.0-z;
      break;
    case 437: // [148] R -3 Rhombohedral axes (-P 3*)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=z;              SpaceGroupElement[1].z=x;
      SpaceGroupElement[2].x=z;              SpaceGroupElement[2].y=x;              SpaceGroupElement[2].z=y;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=-z;             SpaceGroupElement[4].z=-x;
      SpaceGroupElement[5].x=-z;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=-y;
      break;

    case 438: // [149] P 3 1 2 (P 3 2)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      break;

    case 439: // [150] P 3 2 1 (P 3 2")
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-z;
      break;

    case 440: // [151] P 31 1 2 (P 31 2 (0 0 4))
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-1.0/3.0+z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=-2.0/3.0+z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=2.0/3.0-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=1.0/3.0-z;
      break;

    case 441: // [152] P 31 2 1 (P 31 2")
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-1.0/3.0+z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=-2.0/3.0+z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=1.0/3.0-z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=2.0/3.0-z;
      break;

    case 442: // [153] P 32 1 2 (P 32 2 (0 0 2))
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-2.0/3.0+z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=-1.0/3.0+z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=1.0/3.0-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=2.0/3.0-z;
      break;

    case 443: // [154] P 32 2 1 (P 32 2")
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-2.0/3.0+z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=-1.0/3.0+z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=2.0/3.0-z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=1.0/3.0-z;
      break;

    case 444: // [155] R 3 2 Hexagonal axes (R 3 2")
      SpaceGroupSize=18;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-2.0/3.0+x;     SpaceGroupElement[6].y=-1.0/3.0+y;     SpaceGroupElement[6].z=-1.0/3.0+z;
      SpaceGroupElement[7].x=1.0/3.0-x+y;    SpaceGroupElement[7].y=2.0/3.0-x;      SpaceGroupElement[7].z=-1.0/3.0+z;
      SpaceGroupElement[8].x=1.0/3.0-y;      SpaceGroupElement[8].y=-1.0/3.0+x-y;   SpaceGroupElement[8].z=-1.0/3.0+z;
      SpaceGroupElement[9].x=-1.0/3.0+y;     SpaceGroupElement[9].y=-2.0/3.0+x;     SpaceGroupElement[9].z=1.0/3.0-z;
      SpaceGroupElement[10].x=2.0/3.0-x;     SpaceGroupElement[10].y=1.0/3.0-x+y;   SpaceGroupElement[10].z=1.0/3.0-z;
      SpaceGroupElement[11].x=-1.0/3.0+x-y;  SpaceGroupElement[11].y=1.0/3.0-y;     SpaceGroupElement[11].z=1.0/3.0-z;
      SpaceGroupElement[12].x=-1.0/3.0+x;    SpaceGroupElement[12].y=-2.0/3.0+y;    SpaceGroupElement[12].z=-2.0/3.0+z;
      SpaceGroupElement[13].x=-1.0/3.0-x+y;  SpaceGroupElement[13].y=1.0/3.0-x;     SpaceGroupElement[13].z=-2.0/3.0+z;
      SpaceGroupElement[14].x=2.0/3.0-y;     SpaceGroupElement[14].y=1.0/3.0+x-y;   SpaceGroupElement[14].z=-2.0/3.0+z;
      SpaceGroupElement[15].x=-2.0/3.0+y;    SpaceGroupElement[15].y=-1.0/3.0+x;    SpaceGroupElement[15].z=2.0/3.0-z;
      SpaceGroupElement[16].x=1.0/3.0-x;     SpaceGroupElement[16].y=-1.0/3.0-x+y;  SpaceGroupElement[16].z=2.0/3.0-z;
      SpaceGroupElement[17].x=1.0/3.0+x-y;   SpaceGroupElement[17].y=2.0/3.0-y;     SpaceGroupElement[17].z=2.0/3.0-z;
      break;
    case 445: // [155] R 3 2 Rhombohedral axes (P 3* 2)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=z;              SpaceGroupElement[1].z=x;
      SpaceGroupElement[2].x=z;              SpaceGroupElement[2].y=x;              SpaceGroupElement[2].z=y;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-z;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-x;
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=-z;             SpaceGroupElement[5].z=-y;
      break;

    case 446: // [156] P 3 m 1 (P 3 -2")
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=z;
      break;

    case 447: // [157] P 3 1 m (P 3 -2)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=z;
      break;

    case 448: // [158] P 3 c 1 (P 3 -2"c)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5+z;
      break;

    case 449: // [159] P 3 1 c (P 3 -2c)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-0.5+z;
      break;

    case 450: // [160] R 3 m Hexagonal axes (R 3 -2")
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=z;
      break;
    case 451: // [160] R 3 m Rhombohedral axes (P 3* -2)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=z;              SpaceGroupElement[1].z=x;
      SpaceGroupElement[2].x=z;              SpaceGroupElement[2].y=x;              SpaceGroupElement[2].z=y;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=z;              SpaceGroupElement[4].y=y;              SpaceGroupElement[4].z=x;
      SpaceGroupElement[5].x=x;              SpaceGroupElement[5].y=z;              SpaceGroupElement[5].z=y;
      break;

    case 452: // [161] R 3 c Hexagonal axes (R 3 -2"c)
      SpaceGroupSize=18;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=-0.5+z;
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-2.0/3.0+x;     SpaceGroupElement[6].y=-1.0/3.0+y;     SpaceGroupElement[6].z=-1.0/3.0+z;
      SpaceGroupElement[7].x=1.0/3.0-x+y;    SpaceGroupElement[7].y=2.0/3.0-x;      SpaceGroupElement[7].z=-1.0/3.0+z;
      SpaceGroupElement[8].x=1.0/3.0-y;      SpaceGroupElement[8].y=-1.0/3.0+x-y;   SpaceGroupElement[8].z=-1.0/3.0+z;
      SpaceGroupElement[9].x=1.0/3.0-y;      SpaceGroupElement[9].y=2.0/3.0-x;      SpaceGroupElement[9].z=-5.0/6.0+z;
      SpaceGroupElement[10].x=-2.0/3.0+x;    SpaceGroupElement[10].y=-1.0/3.0+x-y;  SpaceGroupElement[10].z=-5.0/6.0+z;
      SpaceGroupElement[11].x=1.0/3.0-x+y;   SpaceGroupElement[11].y=-1.0/3.0+y;    SpaceGroupElement[11].z=-5.0/6.0+z;
      SpaceGroupElement[12].x=-1.0/3.0+x;    SpaceGroupElement[12].y=-2.0/3.0+y;    SpaceGroupElement[12].z=-2.0/3.0+z;
      SpaceGroupElement[13].x=-1.0/3.0-x+y;  SpaceGroupElement[13].y=1.0/3.0-x;     SpaceGroupElement[13].z=-2.0/3.0+z;
      SpaceGroupElement[14].x=2.0/3.0-y;     SpaceGroupElement[14].y=1.0/3.0+x-y;   SpaceGroupElement[14].z=-2.0/3.0+z;
      SpaceGroupElement[15].x=2.0/3.0-y;     SpaceGroupElement[15].y=1.0/3.0-x;     SpaceGroupElement[15].z=-1.0/6.0+z;
      SpaceGroupElement[16].x=-1.0/3.0+x;    SpaceGroupElement[16].y=1.0/3.0+x-y;   SpaceGroupElement[16].z=-1.0/6.0+z;
      SpaceGroupElement[17].x=-1.0/3.0-x+y;  SpaceGroupElement[17].y=-2.0/3.0+y;    SpaceGroupElement[17].z=-1.0/6.0+z;
      break;
    case 453: // [161] R 3 c Rhombohedral axes (P 3* -2n)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=z;              SpaceGroupElement[1].z=x;
      SpaceGroupElement[2].x=z;              SpaceGroupElement[2].y=x;              SpaceGroupElement[2].z=y;
      SpaceGroupElement[3].x=-0.5+y;         SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-0.5+z;         SpaceGroupElement[4].y=-0.5+y;         SpaceGroupElement[4].z=-0.5+x;
      SpaceGroupElement[5].x=-0.5+x;         SpaceGroupElement[5].y=-0.5+z;         SpaceGroupElement[5].z=-0.5+y;
      break;

    case 454: // [162] P -3 1 m (-P 3 2)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=z;
      SpaceGroupElement[10].x=-x;            SpaceGroupElement[10].y=-x+y;          SpaceGroupElement[10].z=z;
      SpaceGroupElement[11].x=x-y;           SpaceGroupElement[11].y=-y;            SpaceGroupElement[11].z=z;
      break;

    case 455: // [163] P -3 1 c (-P 3 2c)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=y;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=0.5+z;
      SpaceGroupElement[10].x=-x;            SpaceGroupElement[10].y=-x+y;          SpaceGroupElement[10].z=0.5+z;
      SpaceGroupElement[11].x=x-y;           SpaceGroupElement[11].y=-y;            SpaceGroupElement[11].z=0.5+z;
      break;

    case 456: // [164] P -3 m 1 (-P 3 2")
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=x-y;           SpaceGroupElement[10].z=z;
      SpaceGroupElement[11].x=-x+y;          SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=z;
      break;

    case 457: // [165] P -3 c 1 (-P 3 2"c)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=0.5+z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=x-y;           SpaceGroupElement[10].z=0.5+z;
      SpaceGroupElement[11].x=-x+y;          SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=0.5+z;
      break;

    case 458: // [166] R -3 m Hexagonal axes (-R 3 2")
      SpaceGroupSize=36;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=x-y;           SpaceGroupElement[10].z=z;
      SpaceGroupElement[11].x=-x+y;          SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=z;
      SpaceGroupElement[12].x=-2.0/3.0+x;    SpaceGroupElement[12].y=-1.0/3.0+y;    SpaceGroupElement[12].z=-1.0/3.0+z;
      SpaceGroupElement[13].x=1.0/3.0-x+y;   SpaceGroupElement[13].y=2.0/3.0-x;     SpaceGroupElement[13].z=-1.0/3.0+z;
      SpaceGroupElement[14].x=1.0/3.0-y;     SpaceGroupElement[14].y=-1.0/3.0+x-y;  SpaceGroupElement[14].z=-1.0/3.0+z;
      SpaceGroupElement[15].x=-1.0/3.0+y;    SpaceGroupElement[15].y=-2.0/3.0+x;    SpaceGroupElement[15].z=1.0/3.0-z;
      SpaceGroupElement[16].x=2.0/3.0-x;     SpaceGroupElement[16].y=1.0/3.0-x+y;   SpaceGroupElement[16].z=1.0/3.0-z;
      SpaceGroupElement[17].x=-1.0/3.0+x-y;  SpaceGroupElement[17].y=1.0/3.0-y;     SpaceGroupElement[17].z=1.0/3.0-z;
      SpaceGroupElement[18].x=2.0/3.0-x;     SpaceGroupElement[18].y=1.0/3.0-y;     SpaceGroupElement[18].z=1.0/3.0-z;
      SpaceGroupElement[19].x=-1.0/3.0+x-y;  SpaceGroupElement[19].y=-2.0/3.0+x;    SpaceGroupElement[19].z=1.0/3.0-z;
      SpaceGroupElement[20].x=-1.0/3.0+y;    SpaceGroupElement[20].y=1.0/3.0-x+y;   SpaceGroupElement[20].z=1.0/3.0-z;
      SpaceGroupElement[21].x=1.0/3.0-y;     SpaceGroupElement[21].y=2.0/3.0-x;     SpaceGroupElement[21].z=-1.0/3.0+z;
      SpaceGroupElement[22].x=-2.0/3.0+x;    SpaceGroupElement[22].y=-1.0/3.0+x-y;  SpaceGroupElement[22].z=-1.0/3.0+z;
      SpaceGroupElement[23].x=1.0/3.0-x+y;   SpaceGroupElement[23].y=-1.0/3.0+y;    SpaceGroupElement[23].z=-1.0/3.0+z;
      SpaceGroupElement[24].x=-1.0/3.0+x;    SpaceGroupElement[24].y=-2.0/3.0+y;    SpaceGroupElement[24].z=-2.0/3.0+z;
      SpaceGroupElement[25].x=-1.0/3.0-x+y;  SpaceGroupElement[25].y=1.0/3.0-x;     SpaceGroupElement[25].z=-2.0/3.0+z;
      SpaceGroupElement[26].x=2.0/3.0-y;     SpaceGroupElement[26].y=1.0/3.0+x-y;   SpaceGroupElement[26].z=-2.0/3.0+z;
      SpaceGroupElement[27].x=-2.0/3.0+y;    SpaceGroupElement[27].y=-1.0/3.0+x;    SpaceGroupElement[27].z=2.0/3.0-z;
      SpaceGroupElement[28].x=1.0/3.0-x;     SpaceGroupElement[28].y=-1.0/3.0-x+y;  SpaceGroupElement[28].z=2.0/3.0-z;
      SpaceGroupElement[29].x=1.0/3.0+x-y;   SpaceGroupElement[29].y=2.0/3.0-y;     SpaceGroupElement[29].z=2.0/3.0-z;
      SpaceGroupElement[30].x=1.0/3.0-x;     SpaceGroupElement[30].y=2.0/3.0-y;     SpaceGroupElement[30].z=2.0/3.0-z;
      SpaceGroupElement[31].x=1.0/3.0+x-y;   SpaceGroupElement[31].y=-1.0/3.0+x;    SpaceGroupElement[31].z=2.0/3.0-z;
      SpaceGroupElement[32].x=-2.0/3.0+y;    SpaceGroupElement[32].y=-1.0/3.0-x+y;  SpaceGroupElement[32].z=2.0/3.0-z;
      SpaceGroupElement[33].x=2.0/3.0-y;     SpaceGroupElement[33].y=1.0/3.0-x;     SpaceGroupElement[33].z=-2.0/3.0+z;
      SpaceGroupElement[34].x=-1.0/3.0+x;    SpaceGroupElement[34].y=1.0/3.0+x-y;   SpaceGroupElement[34].z=-2.0/3.0+z;
      SpaceGroupElement[35].x=-1.0/3.0-x+y;  SpaceGroupElement[35].y=-2.0/3.0+y;    SpaceGroupElement[35].z=-2.0/3.0+z;
      break;
    case 459: // [166] R -3 m Rhombohedral axes (-P 3* 2)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=z;              SpaceGroupElement[1].z=x;
      SpaceGroupElement[2].x=z;              SpaceGroupElement[2].y=x;              SpaceGroupElement[2].z=y;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-z;             SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-x;
      SpaceGroupElement[5].x=-x;             SpaceGroupElement[5].y=-z;             SpaceGroupElement[5].z=-y;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-z;             SpaceGroupElement[7].z=-x;
      SpaceGroupElement[8].x=-z;             SpaceGroupElement[8].y=-x;             SpaceGroupElement[8].z=-y;
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=z;
      SpaceGroupElement[10].x=z;             SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=x;
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=z;             SpaceGroupElement[11].z=y;
      break;

    case 460: // [167] R -3 c Hexagonal axes (-R 3 2"c)
      SpaceGroupSize=36;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x+y;           SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-y;             SpaceGroupElement[2].y=x-y;            SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-x;             SpaceGroupElement[4].y=-x+y;           SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=-y;             SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=0.5+z;
      SpaceGroupElement[10].x=x;             SpaceGroupElement[10].y=x-y;           SpaceGroupElement[10].z=0.5+z;
      SpaceGroupElement[11].x=-x+y;          SpaceGroupElement[11].y=y;             SpaceGroupElement[11].z=0.5+z;
      SpaceGroupElement[12].x=-2.0/3.0+x;    SpaceGroupElement[12].y=-1.0/3.0+y;    SpaceGroupElement[12].z=-1.0/3.0+z;
      SpaceGroupElement[13].x=1.0/3.0-x+y;   SpaceGroupElement[13].y=2.0/3.0-x;     SpaceGroupElement[13].z=-1.0/3.0+z;
      SpaceGroupElement[14].x=1.0/3.0-y;     SpaceGroupElement[14].y=-1.0/3.0+x-y;  SpaceGroupElement[14].z=-1.0/3.0+z;
      SpaceGroupElement[15].x=-1.0/3.0+y;    SpaceGroupElement[15].y=-2.0/3.0+x;    SpaceGroupElement[15].z=5.0/6.0-z;
      SpaceGroupElement[16].x=2.0/3.0-x;     SpaceGroupElement[16].y=1.0/3.0-x+y;   SpaceGroupElement[16].z=5.0/6.0-z;
      SpaceGroupElement[17].x=-1.0/3.0+x-y;  SpaceGroupElement[17].y=1.0/3.0-y;     SpaceGroupElement[17].z=5.0/6.0-z;
      SpaceGroupElement[18].x=2.0/3.0-x;     SpaceGroupElement[18].y=1.0/3.0-y;     SpaceGroupElement[18].z=1.0/3.0-z;
      SpaceGroupElement[19].x=-1.0/3.0+x-y;  SpaceGroupElement[19].y=-2.0/3.0+x;    SpaceGroupElement[19].z=1.0/3.0-z;
      SpaceGroupElement[20].x=-1.0/3.0+y;    SpaceGroupElement[20].y=1.0/3.0-x+y;   SpaceGroupElement[20].z=1.0/3.0-z;
      SpaceGroupElement[21].x=1.0/3.0-y;     SpaceGroupElement[21].y=2.0/3.0-x;     SpaceGroupElement[21].z=1.0/6.0+z;
      SpaceGroupElement[22].x=-2.0/3.0+x;    SpaceGroupElement[22].y=-1.0/3.0+x-y;  SpaceGroupElement[22].z=1.0/6.0+z;
      SpaceGroupElement[23].x=1.0/3.0-x+y;   SpaceGroupElement[23].y=-1.0/3.0+y;    SpaceGroupElement[23].z=1.0/6.0+z;
      SpaceGroupElement[24].x=-1.0/3.0+x;    SpaceGroupElement[24].y=-2.0/3.0+y;    SpaceGroupElement[24].z=-2.0/3.0+z;
      SpaceGroupElement[25].x=-1.0/3.0-x+y;  SpaceGroupElement[25].y=1.0/3.0-x;     SpaceGroupElement[25].z=-2.0/3.0+z;
      SpaceGroupElement[26].x=2.0/3.0-y;     SpaceGroupElement[26].y=1.0/3.0+x-y;   SpaceGroupElement[26].z=-2.0/3.0+z;
      SpaceGroupElement[27].x=-2.0/3.0+y;    SpaceGroupElement[27].y=-1.0/3.0+x;    SpaceGroupElement[27].z=1.0/6.0-z;
      SpaceGroupElement[28].x=1.0/3.0-x;     SpaceGroupElement[28].y=-1.0/3.0-x+y;  SpaceGroupElement[28].z=1.0/6.0-z;
      SpaceGroupElement[29].x=1.0/3.0+x-y;   SpaceGroupElement[29].y=2.0/3.0-y;     SpaceGroupElement[29].z=1.0/6.0-z;
      SpaceGroupElement[30].x=1.0/3.0-x;     SpaceGroupElement[30].y=2.0/3.0-y;     SpaceGroupElement[30].z=2.0/3.0-z;
      SpaceGroupElement[31].x=1.0/3.0+x-y;   SpaceGroupElement[31].y=-1.0/3.0+x;    SpaceGroupElement[31].z=2.0/3.0-z;
      SpaceGroupElement[32].x=-2.0/3.0+y;    SpaceGroupElement[32].y=-1.0/3.0-x+y;  SpaceGroupElement[32].z=2.0/3.0-z;
      SpaceGroupElement[33].x=2.0/3.0-y;     SpaceGroupElement[33].y=1.0/3.0-x;     SpaceGroupElement[33].z=-1.0/6.0+z;
      SpaceGroupElement[34].x=-1.0/3.0+x;    SpaceGroupElement[34].y=1.0/3.0+x-y;   SpaceGroupElement[34].z=-1.0/6.0+z;
      SpaceGroupElement[35].x=-1.0/3.0-x+y;  SpaceGroupElement[35].y=-2.0/3.0+y;    SpaceGroupElement[35].z=-1.0/6.0+z;
      break;
    case 461: // [167] R -3 c Rhombohedral axes (-P 3* 2n)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=z;              SpaceGroupElement[1].z=x;
      SpaceGroupElement[2].x=z;              SpaceGroupElement[2].y=x;              SpaceGroupElement[2].z=y;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=0.5-x;          SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=0.5-z;          SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.5-x;
      SpaceGroupElement[5].x=0.5-x;          SpaceGroupElement[5].y=0.5-z;          SpaceGroupElement[5].z=0.5-y;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-z;             SpaceGroupElement[7].z=-x;
      SpaceGroupElement[8].x=-z;             SpaceGroupElement[8].y=-x;             SpaceGroupElement[8].z=-y;
      SpaceGroupElement[9].x=0.5+y;          SpaceGroupElement[9].y=0.5+x;          SpaceGroupElement[9].z=0.5+z;
      SpaceGroupElement[10].x=0.5+z;         SpaceGroupElement[10].y=0.5+y;         SpaceGroupElement[10].z=0.5+x;
      SpaceGroupElement[11].x=0.5+x;         SpaceGroupElement[11].y=0.5+z;         SpaceGroupElement[11].z=0.5+y;
      break;


    // HEXAGONAL GROUPS
    // ================

    case 462: // [168] P 6 (P 6)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=z;
      break;

    case 463: // [169] P 61 (P 61)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=-1.0/6.0+z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=-1.0/3.0+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=-2.0/3.0+z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-5.0/6.0+z;
      break;

    case 464: // [170] P 65 (P 65)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=-5.0/6.0+z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=-2.0/3.0+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=-1.0/3.0+z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-1.0/6.0+z;
      break;

    case 465: // [171] P 62 (P 62)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=-1.0/3.0+z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=-2.0/3.0+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=-1.0/3.0+z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-2.0/3.0+z;
      break;

    case 466: // [172] P 64 (P 64)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=-2.0/3.0+z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=-1.0/3.0+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=-2.0/3.0+z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-1.0/3.0+z;
      break;

    case 467: // [173] P 63 (P 6c)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-0.5+z;
      break;

    case 468: // [174] P -6 (P -6)
      SpaceGroupSize=6;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=-z;
      break;

    case 469: // [175] P6/m (-P 6)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=x-y;            SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=x-y;            SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=x;              SpaceGroupElement[9].y=y;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=y;             SpaceGroupElement[10].y=-x+y;          SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=-x+y;          SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-z;
      break;

    case 470: // [176] P 63/m (-P 6c)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-y;             SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=x-y;            SpaceGroupElement[7].z=-0.5-z;
      SpaceGroupElement[8].x=x-y;            SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=x;              SpaceGroupElement[9].y=y;              SpaceGroupElement[9].z=-0.5-z;
      SpaceGroupElement[10].x=y;             SpaceGroupElement[10].y=-x+y;          SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=-x+y;          SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-0.5-z;
      break;

    case 471: // [177] P 6 2 2 (P 6 2)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=-z;
      break;

    case 472: // [178] P 61 2 2 (P 61 2 (0 0 5))
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=-1.0/6.0+z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=-1.0/3.0+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=-2.0/3.0+z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-5.0/6.0+z;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=5.0/6.0-z;
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=1.0/6.0-z;
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=1.0/3.0-z;
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=2.0/3.0-z;
      break;

    case 473: // [179] P 65 2 2 (P 65 2 (0 0 1))
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=-5.0/6.0+z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=-2.0/3.0+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=-1.0/3.0+z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-1.0/6.0+z;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=1.0/6.0-z;
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=5.0/6.0-z;
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=2.0/3.0-z;
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=1.0/3.0-z;
      break;

    case 474: // [180] P 62 2 2 (P 62 2 (0 0 4))
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=-1.0/3.0+z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=-2.0/3.0+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=-1.0/3.0+z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-2.0/3.0+z;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=2.0/3.0-z;
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=1.0/3.0-z;
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=2.0/3.0-z;
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=1.0/3.0-z;
      break;

    case 475: // [181] P 64 2 2 (P 64 2 (0 0 2))
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=-2.0/3.0+z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=-1.0/3.0+z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=-2.0/3.0+z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-1.0/3.0+z;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=1.0/3.0-z;
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=2.0/3.0-z;
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=1.0/3.0-z;
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=2.0/3.0-z;
      break;

    case 476: // [182] P 63 2 2 (P 6c 2c)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=0.5-z;
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=-z;
      break;

    case 477: // [183] P 6 m m (P 6 -2)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=y;              SpaceGroupElement[6].y=x;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=-x+y;           SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=z;
      SpaceGroupElement[10].x=x-y;           SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=z;
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=x-y;           SpaceGroupElement[11].z=z;
      break;

    case 478: // [184] P 6 c c (P 6 -2c)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=y;              SpaceGroupElement[6].y=x;              SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-x+y;           SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=-0.5+z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=x-y;           SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-0.5+z;
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=x-y;           SpaceGroupElement[11].z=-0.5+z;
      break;

    case 479: // [185] P 63 c m (P 6c -2)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=y;              SpaceGroupElement[6].y=x;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=-x+y;           SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=-0.5+z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=x-y;           SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=z;
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=x-y;           SpaceGroupElement[11].z=-0.5+z;
      break;

    case 480: // [186] P 63 m c (P 6c -2c)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=y;              SpaceGroupElement[6].y=x;              SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=-x+y;           SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=z;
      SpaceGroupElement[10].x=x-y;           SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-0.5+z;
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=x-y;           SpaceGroupElement[11].z=z;
      break;

    case 481: // [187] P -6 m 2 (P -6 2)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-x+y;           SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=z;
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=x-y;           SpaceGroupElement[11].z=z;
      break;

    case 482: // [188] P -6 c 2 (P -6c 2)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-x+y;           SpaceGroupElement[7].y=y;              SpaceGroupElement[7].z=-0.5+z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=-y;             SpaceGroupElement[9].y=-x;             SpaceGroupElement[9].z=-0.5+z;
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=x-y;           SpaceGroupElement[11].z=-0.5+z;
      break;

    case 483: // [189] P -6 2 m (P -6 -2)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=y;              SpaceGroupElement[6].y=x;              SpaceGroupElement[6].z=z;
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=z;
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=x-y;           SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=-z;
      break;

    case 484: // [190] P -6 2 c (P -6c -2c)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x-y;            SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=x;              SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=-x+y;           SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=y;              SpaceGroupElement[6].y=x;              SpaceGroupElement[6].z=-0.5+z;
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=-x;             SpaceGroupElement[8].y=-x+y;           SpaceGroupElement[8].z=-0.5+z;
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=x-y;           SpaceGroupElement[10].y=-y;            SpaceGroupElement[10].z=-0.5+z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=-z;
      break;

    case 485: // [191] P 6/m m m (-P 6 2)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=x-y;           SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=x-y;           SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=y;             SpaceGroupElement[15].z=-z;
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=-x+y;          SpaceGroupElement[16].z=-z;
      SpaceGroupElement[17].x=-x+y;          SpaceGroupElement[17].y=-x;            SpaceGroupElement[17].z=-z;
      SpaceGroupElement[18].x=y;             SpaceGroupElement[18].y=x;             SpaceGroupElement[18].z=z;
      SpaceGroupElement[19].x=-x+y;          SpaceGroupElement[19].y=y;             SpaceGroupElement[19].z=z;
      SpaceGroupElement[20].x=-x;            SpaceGroupElement[20].y=-x+y;          SpaceGroupElement[20].z=z;
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=z;
      SpaceGroupElement[22].x=x-y;           SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=z;
      SpaceGroupElement[23].x=x;             SpaceGroupElement[23].y=x-y;           SpaceGroupElement[23].z=z;
      break;

    case 486: // [192] P 6/m c c (-P 6 2c)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=0.5-z;
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=0.5-z;
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=x-y;           SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=x-y;           SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=y;             SpaceGroupElement[15].z=-z;
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=-x+y;          SpaceGroupElement[16].z=-z;
      SpaceGroupElement[17].x=-x+y;          SpaceGroupElement[17].y=-x;            SpaceGroupElement[17].z=-z;
      SpaceGroupElement[18].x=y;             SpaceGroupElement[18].y=x;             SpaceGroupElement[18].z=0.5+z;
      SpaceGroupElement[19].x=-x+y;          SpaceGroupElement[19].y=y;             SpaceGroupElement[19].z=0.5+z;
      SpaceGroupElement[20].x=-x;            SpaceGroupElement[20].y=-x+y;          SpaceGroupElement[20].z=0.5+z;
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=0.5+z;
      SpaceGroupElement[22].x=x-y;           SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=0.5+z;
      SpaceGroupElement[23].x=x;             SpaceGroupElement[23].y=x-y;           SpaceGroupElement[23].z=0.5+z;
      break;

    case 487: // [193] P 63/m c m (-P 6c 2)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=-z;
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=0.5-z;
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=0.5-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=x-y;           SpaceGroupElement[13].z=-0.5-z;
      SpaceGroupElement[14].x=x-y;           SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=y;             SpaceGroupElement[15].z=-0.5-z;
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=-x+y;          SpaceGroupElement[16].z=-z;
      SpaceGroupElement[17].x=-x+y;          SpaceGroupElement[17].y=-x;            SpaceGroupElement[17].z=-0.5-z;
      SpaceGroupElement[18].x=y;             SpaceGroupElement[18].y=x;             SpaceGroupElement[18].z=z;
      SpaceGroupElement[19].x=-x+y;          SpaceGroupElement[19].y=y;             SpaceGroupElement[19].z=0.5+z;
      SpaceGroupElement[20].x=-x;            SpaceGroupElement[20].y=-x+y;          SpaceGroupElement[20].z=z;
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=0.5+z;
      SpaceGroupElement[22].x=x-y;           SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=z;
      SpaceGroupElement[23].x=x;             SpaceGroupElement[23].y=x-y;           SpaceGroupElement[23].z=0.5+z;
      break;

    case 488: // [194] P 63/m m c (-P 6c 2c)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x+y;           SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x+y;           SpaceGroupElement[2].y=-x;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-y;             SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=-y;             SpaceGroupElement[4].y=x-y;            SpaceGroupElement[4].z=z;
      SpaceGroupElement[5].x=x-y;            SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-x;             SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=x-y;            SpaceGroupElement[7].y=-y;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=x;              SpaceGroupElement[8].y=x-y;            SpaceGroupElement[8].z=0.5-z;
      SpaceGroupElement[9].x=y;              SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-z;
      SpaceGroupElement[10].x=-x+y;          SpaceGroupElement[10].y=y;             SpaceGroupElement[10].z=0.5-z;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-x+y;          SpaceGroupElement[11].z=-z;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=-y;            SpaceGroupElement[13].y=x-y;           SpaceGroupElement[13].z=-0.5-z;
      SpaceGroupElement[14].x=x-y;           SpaceGroupElement[14].y=x;             SpaceGroupElement[14].z=-z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=y;             SpaceGroupElement[15].z=-0.5-z;
      SpaceGroupElement[16].x=y;             SpaceGroupElement[16].y=-x+y;          SpaceGroupElement[16].z=-z;
      SpaceGroupElement[17].x=-x+y;          SpaceGroupElement[17].y=-x;            SpaceGroupElement[17].z=-0.5-z;
      SpaceGroupElement[18].x=y;             SpaceGroupElement[18].y=x;             SpaceGroupElement[18].z=0.5+z;
      SpaceGroupElement[19].x=-x+y;          SpaceGroupElement[19].y=y;             SpaceGroupElement[19].z=z;
      SpaceGroupElement[20].x=-x;            SpaceGroupElement[20].y=-x+y;          SpaceGroupElement[20].z=0.5+z;
      SpaceGroupElement[21].x=-y;            SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=z;
      SpaceGroupElement[22].x=x-y;           SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=0.5+z;
      SpaceGroupElement[23].x=x;             SpaceGroupElement[23].y=x-y;           SpaceGroupElement[23].z=z;
      break;

    // CUBIC GROUPS
    // ============

    case 489: // [195] P 2 3 (P 2 2 3)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=y;              SpaceGroupElement[4].y=z;              SpaceGroupElement[4].z=x;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=z;              SpaceGroupElement[5].z=-x;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-z;             SpaceGroupElement[6].z=x;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-z;             SpaceGroupElement[7].z=-x;
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;
      SpaceGroupElement[9].x=-z;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-y;
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;
      SpaceGroupElement[11].x=z;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-y;
      break;

    case 490: // [196] F 2 3 (F 2 2 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=y;              SpaceGroupElement[4].y=z;              SpaceGroupElement[4].z=x;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=z;              SpaceGroupElement[5].z=-x;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-z;             SpaceGroupElement[6].z=x;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-z;             SpaceGroupElement[7].z=-x;
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;
      SpaceGroupElement[9].x=-z;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-y;
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;
      SpaceGroupElement[11].x=z;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-y;
      SpaceGroupElement[12].x=x;             SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=-0.5+z;
      SpaceGroupElement[13].x=-x;            SpaceGroupElement[13].y=0.5-y;         SpaceGroupElement[13].z=-0.5+z;
      SpaceGroupElement[14].x=x;             SpaceGroupElement[14].y=0.5-y;         SpaceGroupElement[14].z=0.5-z;
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-0.5+y;        SpaceGroupElement[15].z=0.5-z;
      SpaceGroupElement[16].x=-0.5+y;        SpaceGroupElement[16].y=-0.5+z;        SpaceGroupElement[16].z=x;
      SpaceGroupElement[17].x=0.5-y;         SpaceGroupElement[17].y=-0.5+z;        SpaceGroupElement[17].z=-x;
      SpaceGroupElement[18].x=0.5-y;         SpaceGroupElement[18].y=0.5-z;         SpaceGroupElement[18].z=x;
      SpaceGroupElement[19].x=-0.5+y;        SpaceGroupElement[19].y=0.5-z;         SpaceGroupElement[19].z=-x;
      SpaceGroupElement[20].x=-0.5+z;        SpaceGroupElement[20].y=x;             SpaceGroupElement[20].z=-0.5+y;
      SpaceGroupElement[21].x=0.5-z;         SpaceGroupElement[21].y=x;             SpaceGroupElement[21].z=0.5-y;
      SpaceGroupElement[22].x=0.5-z;         SpaceGroupElement[22].y=-x;            SpaceGroupElement[22].z=-0.5+y;
      SpaceGroupElement[23].x=-0.5+z;        SpaceGroupElement[23].y=-x;            SpaceGroupElement[23].z=0.5-y;
      SpaceGroupElement[24].x=-0.5+x;        SpaceGroupElement[24].y=y;             SpaceGroupElement[24].z=-0.5+z;
      SpaceGroupElement[25].x=0.5-x;         SpaceGroupElement[25].y=-y;            SpaceGroupElement[25].z=-0.5+z;
      SpaceGroupElement[26].x=-0.5+x;        SpaceGroupElement[26].y=-y;            SpaceGroupElement[26].z=0.5-z;
      SpaceGroupElement[27].x=0.5-x;         SpaceGroupElement[27].y=y;             SpaceGroupElement[27].z=0.5-z;
      SpaceGroupElement[28].x=y;             SpaceGroupElement[28].y=-0.5+z;        SpaceGroupElement[28].z=-0.5+x;
      SpaceGroupElement[29].x=-y;            SpaceGroupElement[29].y=-0.5+z;        SpaceGroupElement[29].z=0.5-x;
      SpaceGroupElement[30].x=-y;            SpaceGroupElement[30].y=0.5-z;         SpaceGroupElement[30].z=-0.5+x;
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=0.5-z;         SpaceGroupElement[31].z=0.5-x;
      SpaceGroupElement[32].x=-0.5+z;        SpaceGroupElement[32].y=-0.5+x;        SpaceGroupElement[32].z=y;
      SpaceGroupElement[33].x=0.5-z;         SpaceGroupElement[33].y=-0.5+x;        SpaceGroupElement[33].z=-y;
      SpaceGroupElement[34].x=0.5-z;         SpaceGroupElement[34].y=0.5-x;         SpaceGroupElement[34].z=y;
      SpaceGroupElement[35].x=-0.5+z;        SpaceGroupElement[35].y=0.5-x;         SpaceGroupElement[35].z=-y;
      SpaceGroupElement[36].x=-0.5+x;        SpaceGroupElement[36].y=-0.5+y;        SpaceGroupElement[36].z=z;
      SpaceGroupElement[37].x=0.5-x;         SpaceGroupElement[37].y=0.5-y;         SpaceGroupElement[37].z=z;
      SpaceGroupElement[38].x=-0.5+x;        SpaceGroupElement[38].y=0.5-y;         SpaceGroupElement[38].z=-z;
      SpaceGroupElement[39].x=0.5-x;         SpaceGroupElement[39].y=-0.5+y;        SpaceGroupElement[39].z=-z;
      SpaceGroupElement[40].x=-0.5+y;        SpaceGroupElement[40].y=z;             SpaceGroupElement[40].z=-0.5+x;
      SpaceGroupElement[41].x=0.5-y;         SpaceGroupElement[41].y=z;             SpaceGroupElement[41].z=0.5-x;
      SpaceGroupElement[42].x=0.5-y;         SpaceGroupElement[42].y=-z;            SpaceGroupElement[42].z=-0.5+x;
      SpaceGroupElement[43].x=-0.5+y;        SpaceGroupElement[43].y=-z;            SpaceGroupElement[43].z=0.5-x;
      SpaceGroupElement[44].x=z;             SpaceGroupElement[44].y=-0.5+x;        SpaceGroupElement[44].z=-0.5+y;
      SpaceGroupElement[45].x=-z;            SpaceGroupElement[45].y=-0.5+x;        SpaceGroupElement[45].z=0.5-y;
      SpaceGroupElement[46].x=-z;            SpaceGroupElement[46].y=0.5-x;         SpaceGroupElement[46].z=-0.5+y;
      SpaceGroupElement[47].x=z;             SpaceGroupElement[47].y=0.5-x;         SpaceGroupElement[47].z=0.5-y;
      break;

    case 491: // [197] I 2 3 (I 2 2 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=y;              SpaceGroupElement[4].y=z;              SpaceGroupElement[4].z=x;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=z;              SpaceGroupElement[5].z=-x;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-z;             SpaceGroupElement[6].z=x;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-z;             SpaceGroupElement[7].z=-x;
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;
      SpaceGroupElement[9].x=-z;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-y;
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;
      SpaceGroupElement[11].x=z;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-y;
      SpaceGroupElement[12].x=-0.5+x;        SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=-0.5+z;
      SpaceGroupElement[13].x=0.5-x;         SpaceGroupElement[13].y=0.5-y;         SpaceGroupElement[13].z=-0.5+z;
      SpaceGroupElement[14].x=-0.5+x;        SpaceGroupElement[14].y=0.5-y;         SpaceGroupElement[14].z=0.5-z;
      SpaceGroupElement[15].x=0.5-x;         SpaceGroupElement[15].y=-0.5+y;        SpaceGroupElement[15].z=0.5-z;
      SpaceGroupElement[16].x=-0.5+y;        SpaceGroupElement[16].y=-0.5+z;        SpaceGroupElement[16].z=-0.5+x;
      SpaceGroupElement[17].x=0.5-y;         SpaceGroupElement[17].y=-0.5+z;        SpaceGroupElement[17].z=0.5-x;
      SpaceGroupElement[18].x=0.5-y;         SpaceGroupElement[18].y=0.5-z;         SpaceGroupElement[18].z=-0.5+x;
      SpaceGroupElement[19].x=-0.5+y;        SpaceGroupElement[19].y=0.5-z;         SpaceGroupElement[19].z=0.5-x;
      SpaceGroupElement[20].x=-0.5+z;        SpaceGroupElement[20].y=-0.5+x;        SpaceGroupElement[20].z=-0.5+y;
      SpaceGroupElement[21].x=0.5-z;         SpaceGroupElement[21].y=-0.5+x;        SpaceGroupElement[21].z=0.5-y;
      SpaceGroupElement[22].x=0.5-z;         SpaceGroupElement[22].y=0.5-x;         SpaceGroupElement[22].z=-0.5+y;
      SpaceGroupElement[23].x=-0.5+z;        SpaceGroupElement[23].y=0.5-x;         SpaceGroupElement[23].z=0.5-y;
      break;

    case 492: // [198] P 21 3 (P 2ac 2ab 3)
      SpaceGroupSize=12;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=y;              SpaceGroupElement[4].y=z;              SpaceGroupElement[4].z=x;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-0.5+z;         SpaceGroupElement[5].z=0.5-x;
      SpaceGroupElement[6].x=0.5-y;          SpaceGroupElement[6].y=-z;             SpaceGroupElement[6].z=-0.5+x;
      SpaceGroupElement[7].x=-0.5+y;         SpaceGroupElement[7].y=0.5-z;          SpaceGroupElement[7].z=-x;
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;
      SpaceGroupElement[9].x=-z;             SpaceGroupElement[9].y=-0.5+x;         SpaceGroupElement[9].z=0.5-y;
      SpaceGroupElement[10].x=0.5-z;         SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=-0.5+y;
      SpaceGroupElement[11].x=-0.5+z;        SpaceGroupElement[11].y=0.5-x;         SpaceGroupElement[11].z=-y;
      break;

    case 493: // [199] I 21 3 (I 2b 2c 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=y;              SpaceGroupElement[4].y=z;              SpaceGroupElement[4].z=x;
      SpaceGroupElement[5].x=0.5-y;          SpaceGroupElement[5].y=z;              SpaceGroupElement[5].z=-x;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=0.5-z;          SpaceGroupElement[6].z=x;
      SpaceGroupElement[7].x=-0.5+y;         SpaceGroupElement[7].y=0.5-z;          SpaceGroupElement[7].z=-x;
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;
      SpaceGroupElement[9].x=0.5-z;          SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-y;
      SpaceGroupElement[10].x=0.5-z;         SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=-0.5+y;
      SpaceGroupElement[11].x=-0.5+z;        SpaceGroupElement[11].y=0.5-x;         SpaceGroupElement[11].z=-y;
      SpaceGroupElement[12].x=-0.5+x;        SpaceGroupElement[12].y=-0.5+y;        SpaceGroupElement[12].z=-0.5+z;
      SpaceGroupElement[13].x=0.5-x;         SpaceGroupElement[13].y=-y;            SpaceGroupElement[13].z=-0.5+z;
      SpaceGroupElement[14].x=-0.5+x;        SpaceGroupElement[14].y=0.5-y;         SpaceGroupElement[14].z=-z;
      SpaceGroupElement[15].x=0.5-x;         SpaceGroupElement[15].y=y;             SpaceGroupElement[15].z=-z;
      SpaceGroupElement[16].x=-0.5+y;        SpaceGroupElement[16].y=-0.5+z;        SpaceGroupElement[16].z=-0.5+x;
      SpaceGroupElement[17].x=-y;            SpaceGroupElement[17].y=-0.5+z;        SpaceGroupElement[17].z=0.5-x;
      SpaceGroupElement[18].x=0.5-y;         SpaceGroupElement[18].y=-z;            SpaceGroupElement[18].z=-0.5+x;
      SpaceGroupElement[19].x=y;             SpaceGroupElement[19].y=-z;            SpaceGroupElement[19].z=0.5-x;
      SpaceGroupElement[20].x=-0.5+z;        SpaceGroupElement[20].y=-0.5+x;        SpaceGroupElement[20].z=-0.5+y;
      SpaceGroupElement[21].x=-z;            SpaceGroupElement[21].y=-0.5+x;        SpaceGroupElement[21].z=0.5-y;
      SpaceGroupElement[22].x=-z;            SpaceGroupElement[22].y=0.5-x;         SpaceGroupElement[22].z=y;
      SpaceGroupElement[23].x=z;             SpaceGroupElement[23].y=-x;            SpaceGroupElement[23].z=0.5-y;
      break;

    case 494: // [200] P m -3 (-P 2 2 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=y;              SpaceGroupElement[4].y=z;              SpaceGroupElement[4].z=x;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=z;              SpaceGroupElement[5].z=-x;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-z;             SpaceGroupElement[6].z=x;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-z;             SpaceGroupElement[7].z=-x;
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;
      SpaceGroupElement[9].x=-z;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-y;
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;
      SpaceGroupElement[11].x=z;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-y;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z;
      SpaceGroupElement[16].x=-y;            SpaceGroupElement[16].y=-z;            SpaceGroupElement[16].z=-x;
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=x;
      SpaceGroupElement[18].x=y;             SpaceGroupElement[18].y=z;             SpaceGroupElement[18].z=-x;
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=x;
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-x;            SpaceGroupElement[20].z=-y;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=y;
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=x;             SpaceGroupElement[22].z=-y;
      SpaceGroupElement[23].x=-z;            SpaceGroupElement[23].y=x;             SpaceGroupElement[23].z=y;
      break;

    case 495: // [201] P n -3 Origin choice 1 (P 2 2 3 -1n)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=y;              SpaceGroupElement[4].y=z;              SpaceGroupElement[4].z=x;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=z;              SpaceGroupElement[5].z=-x;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-z;             SpaceGroupElement[6].z=x;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-z;             SpaceGroupElement[7].z=-x;
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;
      SpaceGroupElement[9].x=-z;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-y;
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;
      SpaceGroupElement[11].x=z;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-y;
      SpaceGroupElement[12].x=0.5-x;         SpaceGroupElement[12].y=0.5-y;         SpaceGroupElement[12].z=0.5-z;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=-0.5+y;        SpaceGroupElement[13].z=0.5-z;
      SpaceGroupElement[14].x=0.5-x;         SpaceGroupElement[14].y=-0.5+y;        SpaceGroupElement[14].z=-0.5+z;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=0.5-y;         SpaceGroupElement[15].z=-0.5+z;
      SpaceGroupElement[16].x=0.5-y;         SpaceGroupElement[16].y=0.5-z;         SpaceGroupElement[16].z=0.5-x;
      SpaceGroupElement[17].x=-0.5+y;        SpaceGroupElement[17].y=0.5-z;         SpaceGroupElement[17].z=-0.5+x;
      SpaceGroupElement[18].x=-0.5+y;        SpaceGroupElement[18].y=-0.5+z;        SpaceGroupElement[18].z=0.5-x;
      SpaceGroupElement[19].x=0.5-y;         SpaceGroupElement[19].y=-0.5+z;        SpaceGroupElement[19].z=-0.5+x;
      SpaceGroupElement[20].x=0.5-z;         SpaceGroupElement[20].y=0.5-x;         SpaceGroupElement[20].z=0.5-y;
      SpaceGroupElement[21].x=-0.5+z;        SpaceGroupElement[21].y=0.5-x;         SpaceGroupElement[21].z=-0.5+y;
      SpaceGroupElement[22].x=-0.5+z;        SpaceGroupElement[22].y=-0.5+x;        SpaceGroupElement[22].z=0.5-y;
      SpaceGroupElement[23].x=0.5-z;         SpaceGroupElement[23].y=-0.5+x;        SpaceGroupElement[23].z=-0.5+y;
      break;
    case 496: // [201] P n -3 Origin choice 2 (-P 2ab 2bc 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=0.5-x;          SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=y;              SpaceGroupElement[4].y=z;              SpaceGroupElement[4].z=x;
      SpaceGroupElement[5].x=0.5-y;          SpaceGroupElement[5].y=z;              SpaceGroupElement[5].z=0.5-x;
      SpaceGroupElement[6].x=0.5-y;          SpaceGroupElement[6].y=0.5-z;          SpaceGroupElement[6].z=x;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=0.5-z;          SpaceGroupElement[7].z=0.5-x;
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;
      SpaceGroupElement[9].x=0.5-z;          SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=0.5-y;
      SpaceGroupElement[10].x=0.5-z;         SpaceGroupElement[10].y=0.5-x;         SpaceGroupElement[10].z=y;
      SpaceGroupElement[11].x=z;             SpaceGroupElement[11].y=0.5-x;         SpaceGroupElement[11].z=0.5-y;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=0.5+x;         SpaceGroupElement[13].y=0.5+y;         SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=0.5+y;         SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=0.5+x;         SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=0.5+z;
      SpaceGroupElement[16].x=-y;            SpaceGroupElement[16].y=-z;            SpaceGroupElement[16].z=-x;
      SpaceGroupElement[17].x=0.5+y;         SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=0.5+x;
      SpaceGroupElement[18].x=0.5+y;         SpaceGroupElement[18].y=0.5+z;         SpaceGroupElement[18].z=-x;
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=0.5+z;         SpaceGroupElement[19].z=0.5+x;
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-x;            SpaceGroupElement[20].z=-y;
      SpaceGroupElement[21].x=0.5+z;         SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=0.5+y;
      SpaceGroupElement[22].x=0.5+z;         SpaceGroupElement[22].y=0.5+x;         SpaceGroupElement[22].z=-y;
      SpaceGroupElement[23].x=-z;            SpaceGroupElement[23].y=0.5+x;         SpaceGroupElement[23].z=0.5+y;
      break;

    case 497: // [202] F m -3 (-F 2 2 3)
      SpaceGroupSize=96;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=y;              SpaceGroupElement[4].y=z;              SpaceGroupElement[4].z=x;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=z;              SpaceGroupElement[5].z=-x;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-z;             SpaceGroupElement[6].z=x;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-z;             SpaceGroupElement[7].z=-x;
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;
      SpaceGroupElement[9].x=-z;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-y;
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;
      SpaceGroupElement[11].x=z;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-y;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z;
      SpaceGroupElement[16].x=-y;            SpaceGroupElement[16].y=-z;            SpaceGroupElement[16].z=-x;
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=x;
      SpaceGroupElement[18].x=y;             SpaceGroupElement[18].y=z;             SpaceGroupElement[18].z=-x;
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=x;
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-x;            SpaceGroupElement[20].z=-y;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=y;
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=x;             SpaceGroupElement[22].z=-y;
      SpaceGroupElement[23].x=-z;            SpaceGroupElement[23].y=x;             SpaceGroupElement[23].z=y;
      SpaceGroupElement[24].x=x;             SpaceGroupElement[24].y=-0.5+y;        SpaceGroupElement[24].z=-0.5+z;
      SpaceGroupElement[25].x=-x;            SpaceGroupElement[25].y=0.5-y;         SpaceGroupElement[25].z=-0.5+z;
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=0.5-y;         SpaceGroupElement[26].z=0.5-z;
      SpaceGroupElement[27].x=-x;            SpaceGroupElement[27].y=-0.5+y;        SpaceGroupElement[27].z=0.5-z;
      SpaceGroupElement[28].x=-0.5+y;        SpaceGroupElement[28].y=-0.5+z;        SpaceGroupElement[28].z=x;
      SpaceGroupElement[29].x=0.5-y;         SpaceGroupElement[29].y=-0.5+z;        SpaceGroupElement[29].z=-x;
      SpaceGroupElement[30].x=0.5-y;         SpaceGroupElement[30].y=0.5-z;         SpaceGroupElement[30].z=x;
      SpaceGroupElement[31].x=-0.5+y;        SpaceGroupElement[31].y=0.5-z;         SpaceGroupElement[31].z=-x;
      SpaceGroupElement[32].x=-0.5+z;        SpaceGroupElement[32].y=x;             SpaceGroupElement[32].z=-0.5+y;
      SpaceGroupElement[33].x=0.5-z;         SpaceGroupElement[33].y=x;             SpaceGroupElement[33].z=0.5-y;
      SpaceGroupElement[34].x=0.5-z;         SpaceGroupElement[34].y=-x;            SpaceGroupElement[34].z=-0.5+y;
      SpaceGroupElement[35].x=-0.5+z;        SpaceGroupElement[35].y=-x;            SpaceGroupElement[35].z=0.5-y;
      SpaceGroupElement[36].x=-x;            SpaceGroupElement[36].y=0.5-y;         SpaceGroupElement[36].z=0.5-z;
      SpaceGroupElement[37].x=x;             SpaceGroupElement[37].y=-0.5+y;        SpaceGroupElement[37].z=0.5-z;
      SpaceGroupElement[38].x=-x;            SpaceGroupElement[38].y=-0.5+y;        SpaceGroupElement[38].z=-0.5+z;
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=0.5-y;         SpaceGroupElement[39].z=-0.5+z;
      SpaceGroupElement[40].x=0.5-y;         SpaceGroupElement[40].y=0.5-z;         SpaceGroupElement[40].z=-x;
      SpaceGroupElement[41].x=-0.5+y;        SpaceGroupElement[41].y=0.5-z;         SpaceGroupElement[41].z=x;
      SpaceGroupElement[42].x=-0.5+y;        SpaceGroupElement[42].y=-0.5+z;        SpaceGroupElement[42].z=-x;
      SpaceGroupElement[43].x=0.5-y;         SpaceGroupElement[43].y=-0.5+z;        SpaceGroupElement[43].z=x;
      SpaceGroupElement[44].x=0.5-z;         SpaceGroupElement[44].y=-x;            SpaceGroupElement[44].z=0.5-y;
      SpaceGroupElement[45].x=-0.5+z;        SpaceGroupElement[45].y=-x;            SpaceGroupElement[45].z=-0.5+y;
      SpaceGroupElement[46].x=-0.5+z;        SpaceGroupElement[46].y=x;             SpaceGroupElement[46].z=0.5-y;
      SpaceGroupElement[47].x=0.5-z;         SpaceGroupElement[47].y=x;             SpaceGroupElement[47].z=-0.5+y;
      SpaceGroupElement[48].x=-0.5+x;        SpaceGroupElement[48].y=y;             SpaceGroupElement[48].z=-0.5+z;
      SpaceGroupElement[49].x=0.5-x;         SpaceGroupElement[49].y=-y;            SpaceGroupElement[49].z=-0.5+z;
      SpaceGroupElement[50].x=-0.5+x;        SpaceGroupElement[50].y=-y;            SpaceGroupElement[50].z=0.5-z;
      SpaceGroupElement[51].x=0.5-x;         SpaceGroupElement[51].y=y;             SpaceGroupElement[51].z=0.5-z;
      SpaceGroupElement[52].x=y;             SpaceGroupElement[52].y=-0.5+z;        SpaceGroupElement[52].z=-0.5+x;
      SpaceGroupElement[53].x=-y;            SpaceGroupElement[53].y=-0.5+z;        SpaceGroupElement[53].z=0.5-x;
      SpaceGroupElement[54].x=-y;            SpaceGroupElement[54].y=0.5-z;         SpaceGroupElement[54].z=-0.5+x;
      SpaceGroupElement[55].x=y;             SpaceGroupElement[55].y=0.5-z;         SpaceGroupElement[55].z=0.5-x;
      SpaceGroupElement[56].x=-0.5+z;        SpaceGroupElement[56].y=-0.5+x;        SpaceGroupElement[56].z=y;
      SpaceGroupElement[57].x=0.5-z;         SpaceGroupElement[57].y=-0.5+x;        SpaceGroupElement[57].z=-y;
      SpaceGroupElement[58].x=0.5-z;         SpaceGroupElement[58].y=0.5-x;         SpaceGroupElement[58].z=y;
      SpaceGroupElement[59].x=-0.5+z;        SpaceGroupElement[59].y=0.5-x;         SpaceGroupElement[59].z=-y;
      SpaceGroupElement[60].x=0.5-x;         SpaceGroupElement[60].y=-y;            SpaceGroupElement[60].z=0.5-z;
      SpaceGroupElement[61].x=-0.5+x;        SpaceGroupElement[61].y=y;             SpaceGroupElement[61].z=0.5-z;
      SpaceGroupElement[62].x=0.5-x;         SpaceGroupElement[62].y=y;             SpaceGroupElement[62].z=-0.5+z;
      SpaceGroupElement[63].x=-0.5+x;        SpaceGroupElement[63].y=-y;            SpaceGroupElement[63].z=-0.5+z;
      SpaceGroupElement[64].x=-y;            SpaceGroupElement[64].y=0.5-z;         SpaceGroupElement[64].z=0.5-x;
      SpaceGroupElement[65].x=y;             SpaceGroupElement[65].y=0.5-z;         SpaceGroupElement[65].z=-0.5+x;
      SpaceGroupElement[66].x=y;             SpaceGroupElement[66].y=-0.5+z;        SpaceGroupElement[66].z=0.5-x;
      SpaceGroupElement[67].x=-y;            SpaceGroupElement[67].y=-0.5+z;        SpaceGroupElement[67].z=-0.5+x;
      SpaceGroupElement[68].x=0.5-z;         SpaceGroupElement[68].y=0.5-x;         SpaceGroupElement[68].z=-y;
      SpaceGroupElement[69].x=-0.5+z;        SpaceGroupElement[69].y=0.5-x;         SpaceGroupElement[69].z=y;
      SpaceGroupElement[70].x=-0.5+z;        SpaceGroupElement[70].y=-0.5+x;        SpaceGroupElement[70].z=-y;
      SpaceGroupElement[71].x=0.5-z;         SpaceGroupElement[71].y=-0.5+x;        SpaceGroupElement[71].z=y;
      SpaceGroupElement[72].x=-0.5+x;        SpaceGroupElement[72].y=-0.5+y;        SpaceGroupElement[72].z=z;
      SpaceGroupElement[73].x=0.5-x;         SpaceGroupElement[73].y=0.5-y;         SpaceGroupElement[73].z=z;
      SpaceGroupElement[74].x=-0.5+x;        SpaceGroupElement[74].y=0.5-y;         SpaceGroupElement[74].z=-z;
      SpaceGroupElement[75].x=0.5-x;         SpaceGroupElement[75].y=-0.5+y;        SpaceGroupElement[75].z=-z;
      SpaceGroupElement[76].x=-0.5+y;        SpaceGroupElement[76].y=z;             SpaceGroupElement[76].z=-0.5+x;
      SpaceGroupElement[77].x=0.5-y;         SpaceGroupElement[77].y=z;             SpaceGroupElement[77].z=0.5-x;
      SpaceGroupElement[78].x=0.5-y;         SpaceGroupElement[78].y=-z;            SpaceGroupElement[78].z=-0.5+x;
      SpaceGroupElement[79].x=-0.5+y;        SpaceGroupElement[79].y=-z;            SpaceGroupElement[79].z=0.5-x;
      SpaceGroupElement[80].x=z;             SpaceGroupElement[80].y=-0.5+x;        SpaceGroupElement[80].z=-0.5+y;
      SpaceGroupElement[81].x=-z;            SpaceGroupElement[81].y=-0.5+x;        SpaceGroupElement[81].z=0.5-y;
      SpaceGroupElement[82].x=-z;            SpaceGroupElement[82].y=0.5-x;         SpaceGroupElement[82].z=-0.5+y;
      SpaceGroupElement[83].x=z;             SpaceGroupElement[83].y=0.5-x;         SpaceGroupElement[83].z=0.5-y;
      SpaceGroupElement[84].x=0.5-x;         SpaceGroupElement[84].y=0.5-y;         SpaceGroupElement[84].z=-z;
      SpaceGroupElement[85].x=-0.5+x;        SpaceGroupElement[85].y=-0.5+y;        SpaceGroupElement[85].z=-z;
      SpaceGroupElement[86].x=0.5-x;         SpaceGroupElement[86].y=-0.5+y;        SpaceGroupElement[86].z=z;
      SpaceGroupElement[87].x=-0.5+x;        SpaceGroupElement[87].y=0.5-y;         SpaceGroupElement[87].z=z;
      SpaceGroupElement[88].x=0.5-y;         SpaceGroupElement[88].y=-z;            SpaceGroupElement[88].z=0.5-x;
      SpaceGroupElement[89].x=-0.5+y;        SpaceGroupElement[89].y=-z;            SpaceGroupElement[89].z=-0.5+x;
      SpaceGroupElement[90].x=-0.5+y;        SpaceGroupElement[90].y=z;             SpaceGroupElement[90].z=0.5-x;
      SpaceGroupElement[91].x=0.5-y;         SpaceGroupElement[91].y=z;             SpaceGroupElement[91].z=-0.5+x;
      SpaceGroupElement[92].x=-z;            SpaceGroupElement[92].y=0.5-x;         SpaceGroupElement[92].z=0.5-y;
      SpaceGroupElement[93].x=z;             SpaceGroupElement[93].y=0.5-x;         SpaceGroupElement[93].z=-0.5+y;
      SpaceGroupElement[94].x=z;             SpaceGroupElement[94].y=-0.5+x;        SpaceGroupElement[94].z=0.5-y;
      SpaceGroupElement[95].x=-z;            SpaceGroupElement[95].y=-0.5+x;        SpaceGroupElement[95].z=-0.5+y;
      break;

    case 498: // [203] F d -3 Origin choice 1 (F 2 2 3 -1d)
      SpaceGroupSize=96;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=y;              SpaceGroupElement[4].y=z;              SpaceGroupElement[4].z=x;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=z;              SpaceGroupElement[5].z=-x;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-z;             SpaceGroupElement[6].z=x;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-z;             SpaceGroupElement[7].z=-x;
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;
      SpaceGroupElement[9].x=-z;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-y;
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;
      SpaceGroupElement[11].x=z;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-y;
      SpaceGroupElement[12].x=0.25-x;        SpaceGroupElement[12].y=0.25-y;        SpaceGroupElement[12].z=0.25-z;
      SpaceGroupElement[13].x=-0.25+x;       SpaceGroupElement[13].y=-0.25+y;       SpaceGroupElement[13].z=0.25-z;
      SpaceGroupElement[14].x=0.25-x;        SpaceGroupElement[14].y=-0.25+y;       SpaceGroupElement[14].z=-0.25+z;
      SpaceGroupElement[15].x=-0.25+x;       SpaceGroupElement[15].y=0.25-y;        SpaceGroupElement[15].z=-0.25+z;
      SpaceGroupElement[16].x=0.25-y;        SpaceGroupElement[16].y=0.25-z;        SpaceGroupElement[16].z=0.25-x;
      SpaceGroupElement[17].x=-0.25+y;       SpaceGroupElement[17].y=0.25-z;        SpaceGroupElement[17].z=-0.25+x;
      SpaceGroupElement[18].x=-0.25+y;       SpaceGroupElement[18].y=-0.25+z;       SpaceGroupElement[18].z=0.25-x;
      SpaceGroupElement[19].x=0.25-y;        SpaceGroupElement[19].y=-0.25+z;       SpaceGroupElement[19].z=-0.25+x;
      SpaceGroupElement[20].x=0.25-z;        SpaceGroupElement[20].y=0.25-x;        SpaceGroupElement[20].z=0.25-y;
      SpaceGroupElement[21].x=-0.25+z;       SpaceGroupElement[21].y=0.25-x;        SpaceGroupElement[21].z=-0.25+y;
      SpaceGroupElement[22].x=-0.25+z;       SpaceGroupElement[22].y=-0.25+x;       SpaceGroupElement[22].z=0.25-y;
      SpaceGroupElement[23].x=0.25-z;        SpaceGroupElement[23].y=-0.25+x;       SpaceGroupElement[23].z=-0.25+y;
      SpaceGroupElement[24].x=x;             SpaceGroupElement[24].y=-0.5+y;        SpaceGroupElement[24].z=-0.5+z;
      SpaceGroupElement[25].x=-x;            SpaceGroupElement[25].y=0.5-y;         SpaceGroupElement[25].z=-0.5+z;
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=0.5-y;         SpaceGroupElement[26].z=0.5-z;
      SpaceGroupElement[27].x=-x;            SpaceGroupElement[27].y=-0.5+y;        SpaceGroupElement[27].z=0.5-z;
      SpaceGroupElement[28].x=-0.5+y;        SpaceGroupElement[28].y=-0.5+z;        SpaceGroupElement[28].z=x;
      SpaceGroupElement[29].x=0.5-y;         SpaceGroupElement[29].y=-0.5+z;        SpaceGroupElement[29].z=-x;
      SpaceGroupElement[30].x=0.5-y;         SpaceGroupElement[30].y=0.5-z;         SpaceGroupElement[30].z=x;
      SpaceGroupElement[31].x=-0.5+y;        SpaceGroupElement[31].y=0.5-z;         SpaceGroupElement[31].z=-x;
      SpaceGroupElement[32].x=-0.5+z;        SpaceGroupElement[32].y=x;             SpaceGroupElement[32].z=-0.5+y;
      SpaceGroupElement[33].x=0.5-z;         SpaceGroupElement[33].y=x;             SpaceGroupElement[33].z=0.5-y;
      SpaceGroupElement[34].x=0.5-z;         SpaceGroupElement[34].y=-x;            SpaceGroupElement[34].z=-0.5+y;
      SpaceGroupElement[35].x=-0.5+z;        SpaceGroupElement[35].y=-x;            SpaceGroupElement[35].z=0.5-y;
      SpaceGroupElement[36].x=0.25-x;        SpaceGroupElement[36].y=0.75-y;        SpaceGroupElement[36].z=0.75-z;
      SpaceGroupElement[37].x=-0.25+x;       SpaceGroupElement[37].y=-0.75+y;       SpaceGroupElement[37].z=0.75-z;
      SpaceGroupElement[38].x=0.25-x;        SpaceGroupElement[38].y=-0.75+y;       SpaceGroupElement[38].z=-0.75+z;
      SpaceGroupElement[39].x=-0.25+x;       SpaceGroupElement[39].y=0.75-y;        SpaceGroupElement[39].z=-0.75+z;
      SpaceGroupElement[40].x=0.75-y;        SpaceGroupElement[40].y=0.75-z;        SpaceGroupElement[40].z=0.25-x;
      SpaceGroupElement[41].x=-0.75+y;       SpaceGroupElement[41].y=0.75-z;        SpaceGroupElement[41].z=-0.25+x;
      SpaceGroupElement[42].x=-0.75+y;       SpaceGroupElement[42].y=-0.75+z;       SpaceGroupElement[42].z=0.25-x;
      SpaceGroupElement[43].x=0.75-y;        SpaceGroupElement[43].y=-0.75+z;       SpaceGroupElement[43].z=-0.25+x;
      SpaceGroupElement[44].x=0.75-z;        SpaceGroupElement[44].y=0.25-x;        SpaceGroupElement[44].z=0.75-y;
      SpaceGroupElement[45].x=-0.75+z;       SpaceGroupElement[45].y=0.25-x;        SpaceGroupElement[45].z=-0.75+y;
      SpaceGroupElement[46].x=-0.75+z;       SpaceGroupElement[46].y=-0.25+x;       SpaceGroupElement[46].z=0.75-y;
      SpaceGroupElement[47].x=0.75-z;        SpaceGroupElement[47].y=-0.25+x;       SpaceGroupElement[47].z=-0.75+y;
      SpaceGroupElement[48].x=-0.5+x;        SpaceGroupElement[48].y=y;             SpaceGroupElement[48].z=-0.5+z;
      SpaceGroupElement[49].x=0.5-x;         SpaceGroupElement[49].y=-y;            SpaceGroupElement[49].z=-0.5+z;
      SpaceGroupElement[50].x=-0.5+x;        SpaceGroupElement[50].y=-y;            SpaceGroupElement[50].z=0.5-z;
      SpaceGroupElement[51].x=0.5-x;         SpaceGroupElement[51].y=y;             SpaceGroupElement[51].z=0.5-z;
      SpaceGroupElement[52].x=y;             SpaceGroupElement[52].y=-0.5+z;        SpaceGroupElement[52].z=-0.5+x;
      SpaceGroupElement[53].x=-y;            SpaceGroupElement[53].y=-0.5+z;        SpaceGroupElement[53].z=0.5-x;
      SpaceGroupElement[54].x=-y;            SpaceGroupElement[54].y=0.5-z;         SpaceGroupElement[54].z=-0.5+x;
      SpaceGroupElement[55].x=y;             SpaceGroupElement[55].y=0.5-z;         SpaceGroupElement[55].z=0.5-x;
      SpaceGroupElement[56].x=-0.5+z;        SpaceGroupElement[56].y=-0.5+x;        SpaceGroupElement[56].z=y;
      SpaceGroupElement[57].x=0.5-z;         SpaceGroupElement[57].y=-0.5+x;        SpaceGroupElement[57].z=-y;
      SpaceGroupElement[58].x=0.5-z;         SpaceGroupElement[58].y=0.5-x;         SpaceGroupElement[58].z=y;
      SpaceGroupElement[59].x=-0.5+z;        SpaceGroupElement[59].y=0.5-x;         SpaceGroupElement[59].z=-y;
      SpaceGroupElement[60].x=0.75-x;        SpaceGroupElement[60].y=0.25-y;        SpaceGroupElement[60].z=0.75-z;
      SpaceGroupElement[61].x=-0.75+x;       SpaceGroupElement[61].y=-0.25+y;       SpaceGroupElement[61].z=0.75-z;
      SpaceGroupElement[62].x=0.75-x;        SpaceGroupElement[62].y=-0.25+y;       SpaceGroupElement[62].z=-0.75+z;
      SpaceGroupElement[63].x=-0.75+x;       SpaceGroupElement[63].y=0.25-y;        SpaceGroupElement[63].z=-0.75+z;
      SpaceGroupElement[64].x=0.25-y;        SpaceGroupElement[64].y=0.75-z;        SpaceGroupElement[64].z=0.75-x;
      SpaceGroupElement[65].x=-0.25+y;       SpaceGroupElement[65].y=0.75-z;        SpaceGroupElement[65].z=-0.75+x;
      SpaceGroupElement[66].x=-0.25+y;       SpaceGroupElement[66].y=-0.75+z;       SpaceGroupElement[66].z=0.75-x;
      SpaceGroupElement[67].x=0.25-y;        SpaceGroupElement[67].y=-0.75+z;       SpaceGroupElement[67].z=-0.75+x;
      SpaceGroupElement[68].x=0.75-z;        SpaceGroupElement[68].y=0.75-x;        SpaceGroupElement[68].z=0.25-y;
      SpaceGroupElement[69].x=-0.75+z;       SpaceGroupElement[69].y=0.75-x;        SpaceGroupElement[69].z=-0.25+y;
      SpaceGroupElement[70].x=-0.75+z;       SpaceGroupElement[70].y=-0.75+x;       SpaceGroupElement[70].z=0.25-y;
      SpaceGroupElement[71].x=0.75-z;        SpaceGroupElement[71].y=-0.75+x;       SpaceGroupElement[71].z=-0.25+y;
      SpaceGroupElement[72].x=-0.5+x;        SpaceGroupElement[72].y=-0.5+y;        SpaceGroupElement[72].z=z;
      SpaceGroupElement[73].x=0.5-x;         SpaceGroupElement[73].y=0.5-y;         SpaceGroupElement[73].z=z;
      SpaceGroupElement[74].x=-0.5+x;        SpaceGroupElement[74].y=0.5-y;         SpaceGroupElement[74].z=-z;
      SpaceGroupElement[75].x=0.5-x;         SpaceGroupElement[75].y=-0.5+y;        SpaceGroupElement[75].z=-z;
      SpaceGroupElement[76].x=-0.5+y;        SpaceGroupElement[76].y=z;             SpaceGroupElement[76].z=-0.5+x;
      SpaceGroupElement[77].x=0.5-y;         SpaceGroupElement[77].y=z;             SpaceGroupElement[77].z=0.5-x;
      SpaceGroupElement[78].x=0.5-y;         SpaceGroupElement[78].y=-z;            SpaceGroupElement[78].z=-0.5+x;
      SpaceGroupElement[79].x=-0.5+y;        SpaceGroupElement[79].y=-z;            SpaceGroupElement[79].z=0.5-x;
      SpaceGroupElement[80].x=z;             SpaceGroupElement[80].y=-0.5+x;        SpaceGroupElement[80].z=-0.5+y;
      SpaceGroupElement[81].x=-z;            SpaceGroupElement[81].y=-0.5+x;        SpaceGroupElement[81].z=0.5-y;
      SpaceGroupElement[82].x=-z;            SpaceGroupElement[82].y=0.5-x;         SpaceGroupElement[82].z=-0.5+y;
      SpaceGroupElement[83].x=z;             SpaceGroupElement[83].y=0.5-x;         SpaceGroupElement[83].z=0.5-y;
      SpaceGroupElement[84].x=0.75-x;        SpaceGroupElement[84].y=0.75-y;        SpaceGroupElement[84].z=0.25-z;
      SpaceGroupElement[85].x=-0.75+x;       SpaceGroupElement[85].y=-0.75+y;       SpaceGroupElement[85].z=0.25-z;
      SpaceGroupElement[86].x=0.75-x;        SpaceGroupElement[86].y=-0.75+y;       SpaceGroupElement[86].z=-0.25+z;
      SpaceGroupElement[87].x=-0.75+x;       SpaceGroupElement[87].y=0.75-y;        SpaceGroupElement[87].z=-0.25+z;
      SpaceGroupElement[88].x=0.75-y;        SpaceGroupElement[88].y=0.25-z;        SpaceGroupElement[88].z=0.75-x;
      SpaceGroupElement[89].x=-0.75+y;       SpaceGroupElement[89].y=0.25-z;        SpaceGroupElement[89].z=-0.75+x;
      SpaceGroupElement[90].x=-0.75+y;       SpaceGroupElement[90].y=-0.25+z;       SpaceGroupElement[90].z=0.75-x;
      SpaceGroupElement[91].x=0.75-y;        SpaceGroupElement[91].y=-0.25+z;       SpaceGroupElement[91].z=-0.75+x;
      SpaceGroupElement[92].x=0.25-z;        SpaceGroupElement[92].y=0.75-x;        SpaceGroupElement[92].z=0.75-y;
      SpaceGroupElement[93].x=-0.25+z;       SpaceGroupElement[93].y=0.75-x;        SpaceGroupElement[93].z=-0.75+y;
      SpaceGroupElement[94].x=-0.25+z;       SpaceGroupElement[94].y=-0.75+x;       SpaceGroupElement[94].z=0.75-y;
      SpaceGroupElement[95].x=0.25-z;        SpaceGroupElement[95].y=-0.75+x;       SpaceGroupElement[95].z=-0.75+y;
      break;
    case 499: // [203] F d -3 Origin choice 2 (-F 2uv 2vw 3)
      SpaceGroupSize=96;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.25-x;         SpaceGroupElement[1].y=0.25-y;         SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=0.25-y;         SpaceGroupElement[2].z=0.25-z;
      SpaceGroupElement[3].x=0.25-x;         SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=0.25-z;
      SpaceGroupElement[4].x=y;              SpaceGroupElement[4].y=z;              SpaceGroupElement[4].z=x;
      SpaceGroupElement[5].x=0.25-y;         SpaceGroupElement[5].y=z;              SpaceGroupElement[5].z=0.25-x;
      SpaceGroupElement[6].x=0.25-y;         SpaceGroupElement[6].y=0.25-z;         SpaceGroupElement[6].z=x;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=0.25-z;         SpaceGroupElement[7].z=0.25-x;
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;
      SpaceGroupElement[9].x=0.25-z;         SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=0.25-y;
      SpaceGroupElement[10].x=0.25-z;        SpaceGroupElement[10].y=0.25-x;        SpaceGroupElement[10].z=y;
      SpaceGroupElement[11].x=z;             SpaceGroupElement[11].y=0.25-x;        SpaceGroupElement[11].z=0.25-y;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=0.25+x;        SpaceGroupElement[13].y=0.25+y;        SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=0.25+y;        SpaceGroupElement[14].z=0.25+z;
      SpaceGroupElement[15].x=0.25+x;        SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=0.25+z;
      SpaceGroupElement[16].x=-y;            SpaceGroupElement[16].y=-z;            SpaceGroupElement[16].z=-x;
      SpaceGroupElement[17].x=0.25+y;        SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=0.25+x;
      SpaceGroupElement[18].x=0.25+y;        SpaceGroupElement[18].y=0.25+z;        SpaceGroupElement[18].z=-x;
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=0.25+z;        SpaceGroupElement[19].z=0.25+x;
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-x;            SpaceGroupElement[20].z=-y;
      SpaceGroupElement[21].x=0.25+z;        SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=0.25+y;
      SpaceGroupElement[22].x=0.25+z;        SpaceGroupElement[22].y=0.25+x;        SpaceGroupElement[22].z=-y;
      SpaceGroupElement[23].x=-z;            SpaceGroupElement[23].y=0.25+x;        SpaceGroupElement[23].z=0.25+y;
      SpaceGroupElement[24].x=x;             SpaceGroupElement[24].y=-0.5+y;        SpaceGroupElement[24].z=-0.5+z;
      SpaceGroupElement[25].x=0.25-x;        SpaceGroupElement[25].y=0.75-y;        SpaceGroupElement[25].z=-0.5+z;
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=0.75-y;        SpaceGroupElement[26].z=0.75-z;
      SpaceGroupElement[27].x=0.25-x;        SpaceGroupElement[27].y=-0.5+y;        SpaceGroupElement[27].z=0.75-z;
      SpaceGroupElement[28].x=-0.5+y;        SpaceGroupElement[28].y=-0.5+z;        SpaceGroupElement[28].z=x;
      SpaceGroupElement[29].x=0.75-y;        SpaceGroupElement[29].y=-0.5+z;        SpaceGroupElement[29].z=0.25-x;
      SpaceGroupElement[30].x=0.75-y;        SpaceGroupElement[30].y=0.75-z;        SpaceGroupElement[30].z=x;
      SpaceGroupElement[31].x=-0.5+y;        SpaceGroupElement[31].y=0.75-z;        SpaceGroupElement[31].z=0.25-x;
      SpaceGroupElement[32].x=-0.5+z;        SpaceGroupElement[32].y=x;             SpaceGroupElement[32].z=-0.5+y;
      SpaceGroupElement[33].x=0.75-z;        SpaceGroupElement[33].y=x;             SpaceGroupElement[33].z=0.75-y;
      SpaceGroupElement[34].x=0.75-z;        SpaceGroupElement[34].y=0.25-x;        SpaceGroupElement[34].z=-0.5+y;
      SpaceGroupElement[35].x=-0.5+z;        SpaceGroupElement[35].y=0.25-x;        SpaceGroupElement[35].z=0.75-y;
      SpaceGroupElement[36].x=-x;            SpaceGroupElement[36].y=0.5-y;         SpaceGroupElement[36].z=0.5-z;
      SpaceGroupElement[37].x=0.25+x;        SpaceGroupElement[37].y=-0.25+y;       SpaceGroupElement[37].z=0.5-z;
      SpaceGroupElement[38].x=-x;            SpaceGroupElement[38].y=-0.25+y;       SpaceGroupElement[38].z=-0.25+z;
      SpaceGroupElement[39].x=0.25+x;        SpaceGroupElement[39].y=0.5-y;         SpaceGroupElement[39].z=-0.25+z;
      SpaceGroupElement[40].x=0.5-y;         SpaceGroupElement[40].y=0.5-z;         SpaceGroupElement[40].z=-x;
      SpaceGroupElement[41].x=-0.25+y;       SpaceGroupElement[41].y=0.5-z;         SpaceGroupElement[41].z=0.25+x;
      SpaceGroupElement[42].x=-0.25+y;       SpaceGroupElement[42].y=-0.25+z;       SpaceGroupElement[42].z=-x;
      SpaceGroupElement[43].x=0.5-y;         SpaceGroupElement[43].y=-0.25+z;       SpaceGroupElement[43].z=0.25+x;
      SpaceGroupElement[44].x=0.5-z;         SpaceGroupElement[44].y=-x;            SpaceGroupElement[44].z=0.5-y;
      SpaceGroupElement[45].x=-0.25+z;       SpaceGroupElement[45].y=-x;            SpaceGroupElement[45].z=-0.25+y;
      SpaceGroupElement[46].x=-0.25+z;       SpaceGroupElement[46].y=0.25+x;        SpaceGroupElement[46].z=0.5-y;
      SpaceGroupElement[47].x=0.5-z;         SpaceGroupElement[47].y=0.25+x;        SpaceGroupElement[47].z=-0.25+y;
      SpaceGroupElement[48].x=-0.5+x;        SpaceGroupElement[48].y=y;             SpaceGroupElement[48].z=-0.5+z;
      SpaceGroupElement[49].x=0.75-x;        SpaceGroupElement[49].y=0.25-y;        SpaceGroupElement[49].z=-0.5+z;
      SpaceGroupElement[50].x=-0.5+x;        SpaceGroupElement[50].y=0.25-y;        SpaceGroupElement[50].z=0.75-z;
      SpaceGroupElement[51].x=0.75-x;        SpaceGroupElement[51].y=y;             SpaceGroupElement[51].z=0.75-z;
      SpaceGroupElement[52].x=y;             SpaceGroupElement[52].y=-0.5+z;        SpaceGroupElement[52].z=-0.5+x;
      SpaceGroupElement[53].x=0.25-y;        SpaceGroupElement[53].y=-0.5+z;        SpaceGroupElement[53].z=0.75-x;
      SpaceGroupElement[54].x=0.25-y;        SpaceGroupElement[54].y=0.75-z;        SpaceGroupElement[54].z=-0.5+x;
      SpaceGroupElement[55].x=y;             SpaceGroupElement[55].y=0.75-z;        SpaceGroupElement[55].z=0.75-x;
      SpaceGroupElement[56].x=-0.5+z;        SpaceGroupElement[56].y=-0.5+x;        SpaceGroupElement[56].z=y;
      SpaceGroupElement[57].x=0.75-z;        SpaceGroupElement[57].y=-0.5+x;        SpaceGroupElement[57].z=0.25-y;
      SpaceGroupElement[58].x=0.75-z;        SpaceGroupElement[58].y=0.75-x;        SpaceGroupElement[58].z=y;
      SpaceGroupElement[59].x=-0.5+z;        SpaceGroupElement[59].y=0.75-x;        SpaceGroupElement[59].z=0.25-y;
      SpaceGroupElement[60].x=0.5-x;         SpaceGroupElement[60].y=-y;            SpaceGroupElement[60].z=0.5-z;
      SpaceGroupElement[61].x=-0.25+x;       SpaceGroupElement[61].y=0.25+y;        SpaceGroupElement[61].z=0.5-z;
      SpaceGroupElement[62].x=0.5-x;         SpaceGroupElement[62].y=0.25+y;        SpaceGroupElement[62].z=-0.25+z;
      SpaceGroupElement[63].x=-0.25+x;       SpaceGroupElement[63].y=-y;            SpaceGroupElement[63].z=-0.25+z;
      SpaceGroupElement[64].x=-y;            SpaceGroupElement[64].y=0.5-z;         SpaceGroupElement[64].z=0.5-x;
      SpaceGroupElement[65].x=0.25+y;        SpaceGroupElement[65].y=0.5-z;         SpaceGroupElement[65].z=-0.25+x;
      SpaceGroupElement[66].x=0.25+y;        SpaceGroupElement[66].y=-0.25+z;       SpaceGroupElement[66].z=0.5-x;
      SpaceGroupElement[67].x=-y;            SpaceGroupElement[67].y=-0.25+z;       SpaceGroupElement[67].z=-0.25+x;
      SpaceGroupElement[68].x=0.5-z;         SpaceGroupElement[68].y=0.5-x;         SpaceGroupElement[68].z=-y;
      SpaceGroupElement[69].x=-0.25+z;       SpaceGroupElement[69].y=0.5-x;         SpaceGroupElement[69].z=0.25+y;
      SpaceGroupElement[70].x=-0.25+z;       SpaceGroupElement[70].y=-0.25+x;       SpaceGroupElement[70].z=-y;
      SpaceGroupElement[71].x=0.5-z;         SpaceGroupElement[71].y=-0.25+x;       SpaceGroupElement[71].z=0.25+y;
      SpaceGroupElement[72].x=-0.5+x;        SpaceGroupElement[72].y=-0.5+y;        SpaceGroupElement[72].z=z;
      SpaceGroupElement[73].x=0.75-x;        SpaceGroupElement[73].y=0.75-y;        SpaceGroupElement[73].z=z;
      SpaceGroupElement[74].x=-0.5+x;        SpaceGroupElement[74].y=0.75-y;        SpaceGroupElement[74].z=0.25-z;
      SpaceGroupElement[75].x=0.75-x;        SpaceGroupElement[75].y=-0.5+y;        SpaceGroupElement[75].z=0.25-z;
      SpaceGroupElement[76].x=-0.5+y;        SpaceGroupElement[76].y=z;             SpaceGroupElement[76].z=-0.5+x;
      SpaceGroupElement[77].x=0.75-y;        SpaceGroupElement[77].y=z;             SpaceGroupElement[77].z=0.75-x;
      SpaceGroupElement[78].x=0.75-y;        SpaceGroupElement[78].y=0.25-z;        SpaceGroupElement[78].z=-0.5+x;
      SpaceGroupElement[79].x=-0.5+y;        SpaceGroupElement[79].y=0.25-z;        SpaceGroupElement[79].z=0.75-x;
      SpaceGroupElement[80].x=z;             SpaceGroupElement[80].y=-0.5+x;        SpaceGroupElement[80].z=-0.5+y;
      SpaceGroupElement[81].x=0.25-z;        SpaceGroupElement[81].y=-0.5+x;        SpaceGroupElement[81].z=0.75-y;
      SpaceGroupElement[82].x=0.25-z;        SpaceGroupElement[82].y=0.75-x;        SpaceGroupElement[82].z=-0.5+y;
      SpaceGroupElement[83].x=z;             SpaceGroupElement[83].y=0.75-x;        SpaceGroupElement[83].z=0.75-y;
      SpaceGroupElement[84].x=0.5-x;         SpaceGroupElement[84].y=0.5-y;         SpaceGroupElement[84].z=-z;
      SpaceGroupElement[85].x=-0.25+x;       SpaceGroupElement[85].y=-0.25+y;       SpaceGroupElement[85].z=-z;
      SpaceGroupElement[86].x=0.5-x;         SpaceGroupElement[86].y=-0.25+y;       SpaceGroupElement[86].z=0.25+z;
      SpaceGroupElement[87].x=-0.25+x;       SpaceGroupElement[87].y=0.5-y;         SpaceGroupElement[87].z=0.25+z;
      SpaceGroupElement[88].x=0.5-y;         SpaceGroupElement[88].y=-z;            SpaceGroupElement[88].z=0.5-x;
      SpaceGroupElement[89].x=-0.25+y;       SpaceGroupElement[89].y=-z;            SpaceGroupElement[89].z=-0.25+x;
      SpaceGroupElement[90].x=-0.25+y;       SpaceGroupElement[90].y=0.25+z;        SpaceGroupElement[90].z=0.5-x;
      SpaceGroupElement[91].x=0.5-y;         SpaceGroupElement[91].y=0.25+z;        SpaceGroupElement[91].z=-0.25+x;
      SpaceGroupElement[92].x=-z;            SpaceGroupElement[92].y=0.5-x;         SpaceGroupElement[92].z=0.5-y;
      SpaceGroupElement[93].x=0.25+z;        SpaceGroupElement[93].y=0.5-x;         SpaceGroupElement[93].z=-0.25+y;
      SpaceGroupElement[94].x=0.25+z;        SpaceGroupElement[94].y=-0.25+x;       SpaceGroupElement[94].z=0.5-y;
      SpaceGroupElement[95].x=-z;            SpaceGroupElement[95].y=-0.25+x;       SpaceGroupElement[95].z=-0.25+y;
      break;

    case 500: // [204] I m -3 (-I 2 2 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=y;              SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=y;              SpaceGroupElement[4].y=z;              SpaceGroupElement[4].z=x;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=z;              SpaceGroupElement[5].z=-x;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=-z;             SpaceGroupElement[6].z=x;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=-z;             SpaceGroupElement[7].z=-x;
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;
      SpaceGroupElement[9].x=-z;             SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-y;
      SpaceGroupElement[10].x=-z;            SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=y;
      SpaceGroupElement[11].x=z;             SpaceGroupElement[11].y=-x;            SpaceGroupElement[11].z=-y;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-y;            SpaceGroupElement[15].z=z;
      SpaceGroupElement[16].x=-y;            SpaceGroupElement[16].y=-z;            SpaceGroupElement[16].z=-x;
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=x;
      SpaceGroupElement[18].x=y;             SpaceGroupElement[18].y=z;             SpaceGroupElement[18].z=-x;
      SpaceGroupElement[19].x=-y;            SpaceGroupElement[19].y=z;             SpaceGroupElement[19].z=x;
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-x;            SpaceGroupElement[20].z=-y;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=y;
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=x;             SpaceGroupElement[22].z=-y;
      SpaceGroupElement[23].x=-z;            SpaceGroupElement[23].y=x;             SpaceGroupElement[23].z=y;
      SpaceGroupElement[24].x=-0.5+x;        SpaceGroupElement[24].y=-0.5+y;        SpaceGroupElement[24].z=-0.5+z;
      SpaceGroupElement[25].x=0.5-x;         SpaceGroupElement[25].y=0.5-y;         SpaceGroupElement[25].z=-0.5+z;
      SpaceGroupElement[26].x=-0.5+x;        SpaceGroupElement[26].y=0.5-y;         SpaceGroupElement[26].z=0.5-z;
      SpaceGroupElement[27].x=0.5-x;         SpaceGroupElement[27].y=-0.5+y;        SpaceGroupElement[27].z=0.5-z;
      SpaceGroupElement[28].x=-0.5+y;        SpaceGroupElement[28].y=-0.5+z;        SpaceGroupElement[28].z=-0.5+x;
      SpaceGroupElement[29].x=0.5-y;         SpaceGroupElement[29].y=-0.5+z;        SpaceGroupElement[29].z=0.5-x;
      SpaceGroupElement[30].x=0.5-y;         SpaceGroupElement[30].y=0.5-z;         SpaceGroupElement[30].z=-0.5+x;
      SpaceGroupElement[31].x=-0.5+y;        SpaceGroupElement[31].y=0.5-z;         SpaceGroupElement[31].z=0.5-x;
      SpaceGroupElement[32].x=-0.5+z;        SpaceGroupElement[32].y=-0.5+x;        SpaceGroupElement[32].z=-0.5+y;
      SpaceGroupElement[33].x=0.5-z;         SpaceGroupElement[33].y=-0.5+x;        SpaceGroupElement[33].z=0.5-y;
      SpaceGroupElement[34].x=0.5-z;         SpaceGroupElement[34].y=0.5-x;         SpaceGroupElement[34].z=-0.5+y;
      SpaceGroupElement[35].x=-0.5+z;        SpaceGroupElement[35].y=0.5-x;         SpaceGroupElement[35].z=0.5-y;
      SpaceGroupElement[36].x=0.5-x;         SpaceGroupElement[36].y=0.5-y;         SpaceGroupElement[36].z=0.5-z;
      SpaceGroupElement[37].x=-0.5+x;        SpaceGroupElement[37].y=-0.5+y;        SpaceGroupElement[37].z=0.5-z;
      SpaceGroupElement[38].x=0.5-x;         SpaceGroupElement[38].y=-0.5+y;        SpaceGroupElement[38].z=-0.5+z;
      SpaceGroupElement[39].x=-0.5+x;        SpaceGroupElement[39].y=0.5-y;         SpaceGroupElement[39].z=-0.5+z;
      SpaceGroupElement[40].x=0.5-y;         SpaceGroupElement[40].y=0.5-z;         SpaceGroupElement[40].z=0.5-x;
      SpaceGroupElement[41].x=-0.5+y;        SpaceGroupElement[41].y=0.5-z;         SpaceGroupElement[41].z=-0.5+x;
      SpaceGroupElement[42].x=-0.5+y;        SpaceGroupElement[42].y=-0.5+z;        SpaceGroupElement[42].z=0.5-x;
      SpaceGroupElement[43].x=0.5-y;         SpaceGroupElement[43].y=-0.5+z;        SpaceGroupElement[43].z=-0.5+x;
      SpaceGroupElement[44].x=0.5-z;         SpaceGroupElement[44].y=0.5-x;         SpaceGroupElement[44].z=0.5-y;
      SpaceGroupElement[45].x=-0.5+z;        SpaceGroupElement[45].y=0.5-x;         SpaceGroupElement[45].z=-0.5+y;
      SpaceGroupElement[46].x=-0.5+z;        SpaceGroupElement[46].y=-0.5+x;        SpaceGroupElement[46].z=0.5-y;
      SpaceGroupElement[47].x=0.5-z;         SpaceGroupElement[47].y=-0.5+x;        SpaceGroupElement[47].z=-0.5+y;
      break;

    case 501: // [205] P a -3 (-P 2ac 2ab 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-x;          SpaceGroupElement[1].y=-y;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-0.5+x;         SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=y;              SpaceGroupElement[4].y=z;              SpaceGroupElement[4].z=x;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-0.5+z;         SpaceGroupElement[5].z=0.5-x;
      SpaceGroupElement[6].x=0.5-y;          SpaceGroupElement[6].y=-z;             SpaceGroupElement[6].z=-0.5+x;
      SpaceGroupElement[7].x=-0.5+y;         SpaceGroupElement[7].y=0.5-z;          SpaceGroupElement[7].z=-x;
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;
      SpaceGroupElement[9].x=-z;             SpaceGroupElement[9].y=-0.5+x;         SpaceGroupElement[9].z=0.5-y;
      SpaceGroupElement[10].x=0.5-z;         SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=-0.5+y;
      SpaceGroupElement[11].x=-0.5+z;        SpaceGroupElement[11].y=0.5-x;         SpaceGroupElement[11].z=-y;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=0.5+x;         SpaceGroupElement[13].y=y;             SpaceGroupElement[13].z=-0.5-z;
      SpaceGroupElement[14].x=-0.5-x;        SpaceGroupElement[14].y=0.5+y;         SpaceGroupElement[14].z=z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-0.5-y;        SpaceGroupElement[15].z=0.5+z;
      SpaceGroupElement[16].x=-y;            SpaceGroupElement[16].y=-z;            SpaceGroupElement[16].z=-x;
      SpaceGroupElement[17].x=y;             SpaceGroupElement[17].y=-0.5-z;        SpaceGroupElement[17].z=0.5+x;
      SpaceGroupElement[18].x=0.5+y;         SpaceGroupElement[18].y=z;             SpaceGroupElement[18].z=-0.5-x;
      SpaceGroupElement[19].x=-0.5-y;        SpaceGroupElement[19].y=0.5+z;         SpaceGroupElement[19].z=x;
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-x;            SpaceGroupElement[20].z=-y;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-0.5-x;        SpaceGroupElement[21].z=0.5+y;
      SpaceGroupElement[22].x=0.5+z;         SpaceGroupElement[22].y=x;             SpaceGroupElement[22].z=-0.5-y;
      SpaceGroupElement[23].x=-0.5-z;        SpaceGroupElement[23].y=0.5+x;         SpaceGroupElement[23].z=y;
      break;

    case 502: // [206] I a -3 (-I 2b 2c 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-x;             SpaceGroupElement[1].y=0.5-y;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=x;              SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=0.5-z;
      SpaceGroupElement[3].x=-x;             SpaceGroupElement[3].y=-0.5+y;         SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=y;              SpaceGroupElement[4].y=z;              SpaceGroupElement[4].z=x;
      SpaceGroupElement[5].x=0.5-y;          SpaceGroupElement[5].y=z;              SpaceGroupElement[5].z=-x;
      SpaceGroupElement[6].x=-y;             SpaceGroupElement[6].y=0.5-z;          SpaceGroupElement[6].z=x;
      SpaceGroupElement[7].x=-0.5+y;         SpaceGroupElement[7].y=0.5-z;          SpaceGroupElement[7].z=-x;
      SpaceGroupElement[8].x=z;              SpaceGroupElement[8].y=x;              SpaceGroupElement[8].z=y;
      SpaceGroupElement[9].x=0.5-z;          SpaceGroupElement[9].y=x;              SpaceGroupElement[9].z=-y;
      SpaceGroupElement[10].x=0.5-z;         SpaceGroupElement[10].y=-x;            SpaceGroupElement[10].z=-0.5+y;
      SpaceGroupElement[11].x=-0.5+z;        SpaceGroupElement[11].y=0.5-x;         SpaceGroupElement[11].z=-y;
      SpaceGroupElement[12].x=-x;            SpaceGroupElement[12].y=-y;            SpaceGroupElement[12].z=-z;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=0.5+y;         SpaceGroupElement[13].z=-z;
      SpaceGroupElement[14].x=-x;            SpaceGroupElement[14].y=y;             SpaceGroupElement[14].z=0.5+z;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=-0.5-y;        SpaceGroupElement[15].z=0.5+z;
      SpaceGroupElement[16].x=-y;            SpaceGroupElement[16].y=-z;            SpaceGroupElement[16].z=-x;
      SpaceGroupElement[17].x=0.5+y;         SpaceGroupElement[17].y=-z;            SpaceGroupElement[17].z=x;
      SpaceGroupElement[18].x=y;             SpaceGroupElement[18].y=0.5+z;         SpaceGroupElement[18].z=-x;
      SpaceGroupElement[19].x=-0.5-y;        SpaceGroupElement[19].y=0.5+z;         SpaceGroupElement[19].z=x;
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-x;            SpaceGroupElement[20].z=-y;
      SpaceGroupElement[21].x=0.5+z;         SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=y;
      SpaceGroupElement[22].x=0.5+z;         SpaceGroupElement[22].y=x;             SpaceGroupElement[22].z=-0.5-y;
      SpaceGroupElement[23].x=-0.5-z;        SpaceGroupElement[23].y=0.5+x;         SpaceGroupElement[23].z=y;
      SpaceGroupElement[24].x=-0.5+x;        SpaceGroupElement[24].y=-0.5+y;        SpaceGroupElement[24].z=-0.5+z;
      SpaceGroupElement[25].x=0.5-x;         SpaceGroupElement[25].y=-y;            SpaceGroupElement[25].z=-0.5+z;
      SpaceGroupElement[26].x=-0.5+x;        SpaceGroupElement[26].y=0.5-y;         SpaceGroupElement[26].z=-z;
      SpaceGroupElement[27].x=0.5-x;         SpaceGroupElement[27].y=y;             SpaceGroupElement[27].z=-z;
      SpaceGroupElement[28].x=-0.5+y;        SpaceGroupElement[28].y=-0.5+z;        SpaceGroupElement[28].z=-0.5+x;
      SpaceGroupElement[29].x=-y;            SpaceGroupElement[29].y=-0.5+z;        SpaceGroupElement[29].z=0.5-x;
      SpaceGroupElement[30].x=0.5-y;         SpaceGroupElement[30].y=-z;            SpaceGroupElement[30].z=-0.5+x;
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=-z;            SpaceGroupElement[31].z=0.5-x;
      SpaceGroupElement[32].x=-0.5+z;        SpaceGroupElement[32].y=-0.5+x;        SpaceGroupElement[32].z=-0.5+y;
      SpaceGroupElement[33].x=-z;            SpaceGroupElement[33].y=-0.5+x;        SpaceGroupElement[33].z=0.5-y;
      SpaceGroupElement[34].x=-z;            SpaceGroupElement[34].y=0.5-x;         SpaceGroupElement[34].z=y;
      SpaceGroupElement[35].x=z;             SpaceGroupElement[35].y=-x;            SpaceGroupElement[35].z=0.5-y;
      SpaceGroupElement[36].x=0.5-x;         SpaceGroupElement[36].y=0.5-y;         SpaceGroupElement[36].z=0.5-z;
      SpaceGroupElement[37].x=-0.5+x;        SpaceGroupElement[37].y=y;             SpaceGroupElement[37].z=0.5-z;
      SpaceGroupElement[38].x=0.5-x;         SpaceGroupElement[38].y=-0.5+y;        SpaceGroupElement[38].z=z;
      SpaceGroupElement[39].x=-0.5+x;        SpaceGroupElement[39].y=-y;            SpaceGroupElement[39].z=z;
      SpaceGroupElement[40].x=0.5-y;         SpaceGroupElement[40].y=0.5-z;         SpaceGroupElement[40].z=0.5-x;
      SpaceGroupElement[41].x=y;             SpaceGroupElement[41].y=0.5-z;         SpaceGroupElement[41].z=-0.5+x;
      SpaceGroupElement[42].x=-0.5+y;        SpaceGroupElement[42].y=z;             SpaceGroupElement[42].z=0.5-x;
      SpaceGroupElement[43].x=-y;            SpaceGroupElement[43].y=z;             SpaceGroupElement[43].z=-0.5+x;
      SpaceGroupElement[44].x=0.5-z;         SpaceGroupElement[44].y=0.5-x;         SpaceGroupElement[44].z=0.5-y;
      SpaceGroupElement[45].x=z;             SpaceGroupElement[45].y=0.5-x;         SpaceGroupElement[45].z=-0.5+y;
      SpaceGroupElement[46].x=z;             SpaceGroupElement[46].y=-0.5+x;        SpaceGroupElement[46].z=-y;
      SpaceGroupElement[47].x=-z;            SpaceGroupElement[47].y=x;             SpaceGroupElement[47].z=-0.5+y;
      break;

    case 503: // [207] P 4 3 2 (P 4 2 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=z;              SpaceGroupElement[9].z=y;
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=z;             SpaceGroupElement[11].z=-y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-z;            SpaceGroupElement[13].z=y;
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-z;            SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=-z;            SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=-y;
      SpaceGroupElement[18].x=-z;            SpaceGroupElement[18].y=y;             SpaceGroupElement[18].z=x;
      SpaceGroupElement[19].x=-z;            SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=y;
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=x;
      SpaceGroupElement[23].x=z;             SpaceGroupElement[23].y=y;             SpaceGroupElement[23].z=-x;
      break;

    case 504: // [208] P 42 3 2 (P 4n 2 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.5+y;         SpaceGroupElement[5].y=-0.5+x;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=0.5-x;          SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=-0.5+z;         SpaceGroupElement[9].z=-0.5+y;
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;
      SpaceGroupElement[11].x=-0.5+x;        SpaceGroupElement[11].y=-0.5+z;        SpaceGroupElement[11].z=0.5-y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=0.5-z;         SpaceGroupElement[13].z=-0.5+y;
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-z;            SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=0.5-x;         SpaceGroupElement[15].y=0.5-z;         SpaceGroupElement[15].z=0.5-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=-z;            SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=-y;
      SpaceGroupElement[18].x=0.5-z;         SpaceGroupElement[18].y=-0.5+y;        SpaceGroupElement[18].z=-0.5+x;
      SpaceGroupElement[19].x=-z;            SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=y;
      SpaceGroupElement[20].x=0.5-z;         SpaceGroupElement[20].y=0.5-y;         SpaceGroupElement[20].z=0.5-x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=-0.5+z;        SpaceGroupElement[22].y=0.5-y;         SpaceGroupElement[22].z=-0.5+x;
      SpaceGroupElement[23].x=-0.5+z;        SpaceGroupElement[23].y=-0.5+y;        SpaceGroupElement[23].z=0.5-x;
      break;

    case 505: // [209] F 4 3 2 (F 4 2 3)
      SpaceGroupSize=96;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=z;              SpaceGroupElement[9].z=y;
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=z;             SpaceGroupElement[11].z=-y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-z;            SpaceGroupElement[13].z=y;
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-z;            SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=-z;            SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=-y;
      SpaceGroupElement[18].x=-z;            SpaceGroupElement[18].y=y;             SpaceGroupElement[18].z=x;
      SpaceGroupElement[19].x=-z;            SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=y;
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=x;
      SpaceGroupElement[23].x=z;             SpaceGroupElement[23].y=y;             SpaceGroupElement[23].z=-x;
      SpaceGroupElement[24].x=x;             SpaceGroupElement[24].y=-0.5+y;        SpaceGroupElement[24].z=-0.5+z;
      SpaceGroupElement[25].x=-0.5+y;        SpaceGroupElement[25].y=-x;            SpaceGroupElement[25].z=-0.5+z;
      SpaceGroupElement[26].x=-x;            SpaceGroupElement[26].y=0.5-y;         SpaceGroupElement[26].z=-0.5+z;
      SpaceGroupElement[27].x=0.5-y;         SpaceGroupElement[27].y=x;             SpaceGroupElement[27].z=-0.5+z;
      SpaceGroupElement[28].x=x;             SpaceGroupElement[28].y=0.5-y;         SpaceGroupElement[28].z=0.5-z;
      SpaceGroupElement[29].x=-0.5+y;        SpaceGroupElement[29].y=x;             SpaceGroupElement[29].z=0.5-z;
      SpaceGroupElement[30].x=-x;            SpaceGroupElement[30].y=-0.5+y;        SpaceGroupElement[30].z=0.5-z;
      SpaceGroupElement[31].x=0.5-y;         SpaceGroupElement[31].y=-x;            SpaceGroupElement[31].z=0.5-z;
      SpaceGroupElement[32].x=-0.5+y;        SpaceGroupElement[32].y=-0.5+z;        SpaceGroupElement[32].z=x;
      SpaceGroupElement[33].x=-x;            SpaceGroupElement[33].y=-0.5+z;        SpaceGroupElement[33].z=-0.5+y;
      SpaceGroupElement[34].x=0.5-y;         SpaceGroupElement[34].y=-0.5+z;        SpaceGroupElement[34].z=-x;
      SpaceGroupElement[35].x=x;             SpaceGroupElement[35].y=-0.5+z;        SpaceGroupElement[35].z=0.5-y;
      SpaceGroupElement[36].x=0.5-y;         SpaceGroupElement[36].y=0.5-z;         SpaceGroupElement[36].z=x;
      SpaceGroupElement[37].x=x;             SpaceGroupElement[37].y=0.5-z;         SpaceGroupElement[37].z=-0.5+y;
      SpaceGroupElement[38].x=-0.5+y;        SpaceGroupElement[38].y=0.5-z;         SpaceGroupElement[38].z=-x;
      SpaceGroupElement[39].x=-x;            SpaceGroupElement[39].y=0.5-z;         SpaceGroupElement[39].z=0.5-y;
      SpaceGroupElement[40].x=-0.5+z;        SpaceGroupElement[40].y=x;             SpaceGroupElement[40].z=-0.5+y;
      SpaceGroupElement[41].x=0.5-z;         SpaceGroupElement[41].y=x;             SpaceGroupElement[41].z=0.5-y;
      SpaceGroupElement[42].x=0.5-z;         SpaceGroupElement[42].y=-0.5+y;        SpaceGroupElement[42].z=x;
      SpaceGroupElement[43].x=0.5-z;         SpaceGroupElement[43].y=-x;            SpaceGroupElement[43].z=-0.5+y;
      SpaceGroupElement[44].x=0.5-z;         SpaceGroupElement[44].y=0.5-y;         SpaceGroupElement[44].z=-x;
      SpaceGroupElement[45].x=-0.5+z;        SpaceGroupElement[45].y=-x;            SpaceGroupElement[45].z=0.5-y;
      SpaceGroupElement[46].x=-0.5+z;        SpaceGroupElement[46].y=0.5-y;         SpaceGroupElement[46].z=x;
      SpaceGroupElement[47].x=-0.5+z;        SpaceGroupElement[47].y=-0.5+y;        SpaceGroupElement[47].z=-x;
      SpaceGroupElement[48].x=-0.5+x;        SpaceGroupElement[48].y=y;             SpaceGroupElement[48].z=-0.5+z;
      SpaceGroupElement[49].x=y;             SpaceGroupElement[49].y=0.5-x;         SpaceGroupElement[49].z=-0.5+z;
      SpaceGroupElement[50].x=0.5-x;         SpaceGroupElement[50].y=-y;            SpaceGroupElement[50].z=-0.5+z;
      SpaceGroupElement[51].x=-y;            SpaceGroupElement[51].y=-0.5+x;        SpaceGroupElement[51].z=-0.5+z;
      SpaceGroupElement[52].x=-0.5+x;        SpaceGroupElement[52].y=-y;            SpaceGroupElement[52].z=0.5-z;
      SpaceGroupElement[53].x=y;             SpaceGroupElement[53].y=-0.5+x;        SpaceGroupElement[53].z=0.5-z;
      SpaceGroupElement[54].x=0.5-x;         SpaceGroupElement[54].y=y;             SpaceGroupElement[54].z=0.5-z;
      SpaceGroupElement[55].x=-y;            SpaceGroupElement[55].y=0.5-x;         SpaceGroupElement[55].z=0.5-z;
      SpaceGroupElement[56].x=y;             SpaceGroupElement[56].y=-0.5+z;        SpaceGroupElement[56].z=-0.5+x;
      SpaceGroupElement[57].x=0.5-x;         SpaceGroupElement[57].y=-0.5+z;        SpaceGroupElement[57].z=y;
      SpaceGroupElement[58].x=-y;            SpaceGroupElement[58].y=-0.5+z;        SpaceGroupElement[58].z=0.5-x;
      SpaceGroupElement[59].x=-0.5+x;        SpaceGroupElement[59].y=-0.5+z;        SpaceGroupElement[59].z=-y;
      SpaceGroupElement[60].x=-y;            SpaceGroupElement[60].y=0.5-z;         SpaceGroupElement[60].z=-0.5+x;
      SpaceGroupElement[61].x=-0.5+x;        SpaceGroupElement[61].y=0.5-z;         SpaceGroupElement[61].z=y;
      SpaceGroupElement[62].x=y;             SpaceGroupElement[62].y=0.5-z;         SpaceGroupElement[62].z=0.5-x;
      SpaceGroupElement[63].x=0.5-x;         SpaceGroupElement[63].y=0.5-z;         SpaceGroupElement[63].z=-y;
      SpaceGroupElement[64].x=-0.5+z;        SpaceGroupElement[64].y=-0.5+x;        SpaceGroupElement[64].z=y;
      SpaceGroupElement[65].x=0.5-z;         SpaceGroupElement[65].y=-0.5+x;        SpaceGroupElement[65].z=-y;
      SpaceGroupElement[66].x=0.5-z;         SpaceGroupElement[66].y=y;             SpaceGroupElement[66].z=-0.5+x;
      SpaceGroupElement[67].x=0.5-z;         SpaceGroupElement[67].y=0.5-x;         SpaceGroupElement[67].z=y;
      SpaceGroupElement[68].x=0.5-z;         SpaceGroupElement[68].y=-y;            SpaceGroupElement[68].z=0.5-x;
      SpaceGroupElement[69].x=-0.5+z;        SpaceGroupElement[69].y=0.5-x;         SpaceGroupElement[69].z=-y;
      SpaceGroupElement[70].x=-0.5+z;        SpaceGroupElement[70].y=-y;            SpaceGroupElement[70].z=-0.5+x;
      SpaceGroupElement[71].x=-0.5+z;        SpaceGroupElement[71].y=y;             SpaceGroupElement[71].z=0.5-x;
      SpaceGroupElement[72].x=-0.5+x;        SpaceGroupElement[72].y=-0.5+y;        SpaceGroupElement[72].z=z;
      SpaceGroupElement[73].x=-0.5+y;        SpaceGroupElement[73].y=0.5-x;         SpaceGroupElement[73].z=z;
      SpaceGroupElement[74].x=0.5-x;         SpaceGroupElement[74].y=0.5-y;         SpaceGroupElement[74].z=z;
      SpaceGroupElement[75].x=0.5-y;         SpaceGroupElement[75].y=-0.5+x;        SpaceGroupElement[75].z=z;
      SpaceGroupElement[76].x=-0.5+x;        SpaceGroupElement[76].y=0.5-y;         SpaceGroupElement[76].z=-z;
      SpaceGroupElement[77].x=-0.5+y;        SpaceGroupElement[77].y=-0.5+x;        SpaceGroupElement[77].z=-z;
      SpaceGroupElement[78].x=0.5-x;         SpaceGroupElement[78].y=-0.5+y;        SpaceGroupElement[78].z=-z;
      SpaceGroupElement[79].x=0.5-y;         SpaceGroupElement[79].y=0.5-x;         SpaceGroupElement[79].z=-z;
      SpaceGroupElement[80].x=-0.5+y;        SpaceGroupElement[80].y=z;             SpaceGroupElement[80].z=-0.5+x;
      SpaceGroupElement[81].x=0.5-x;         SpaceGroupElement[81].y=z;             SpaceGroupElement[81].z=-0.5+y;
      SpaceGroupElement[82].x=0.5-y;         SpaceGroupElement[82].y=z;             SpaceGroupElement[82].z=0.5-x;
      SpaceGroupElement[83].x=-0.5+x;        SpaceGroupElement[83].y=z;             SpaceGroupElement[83].z=0.5-y;
      SpaceGroupElement[84].x=0.5-y;         SpaceGroupElement[84].y=-z;            SpaceGroupElement[84].z=-0.5+x;
      SpaceGroupElement[85].x=-0.5+x;        SpaceGroupElement[85].y=-z;            SpaceGroupElement[85].z=-0.5+y;
      SpaceGroupElement[86].x=-0.5+y;        SpaceGroupElement[86].y=-z;            SpaceGroupElement[86].z=0.5-x;
      SpaceGroupElement[87].x=0.5-x;         SpaceGroupElement[87].y=-z;            SpaceGroupElement[87].z=0.5-y;
      SpaceGroupElement[88].x=z;             SpaceGroupElement[88].y=-0.5+x;        SpaceGroupElement[88].z=-0.5+y;
      SpaceGroupElement[89].x=-z;            SpaceGroupElement[89].y=-0.5+x;        SpaceGroupElement[89].z=0.5-y;
      SpaceGroupElement[90].x=-z;            SpaceGroupElement[90].y=-0.5+y;        SpaceGroupElement[90].z=-0.5+x;
      SpaceGroupElement[91].x=-z;            SpaceGroupElement[91].y=0.5-x;         SpaceGroupElement[91].z=-0.5+y;
      SpaceGroupElement[92].x=-z;            SpaceGroupElement[92].y=0.5-y;         SpaceGroupElement[92].z=0.5-x;
      SpaceGroupElement[93].x=z;             SpaceGroupElement[93].y=0.5-x;         SpaceGroupElement[93].z=0.5-y;
      SpaceGroupElement[94].x=z;             SpaceGroupElement[94].y=0.5-y;         SpaceGroupElement[94].z=-0.5+x;
      SpaceGroupElement[95].x=z;             SpaceGroupElement[95].y=-0.5+y;        SpaceGroupElement[95].z=0.5-x;
      break;

    case 506: // [210] F 41 3 2 (F 4d 2 3)
      SpaceGroupSize=96;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.25+y;        SpaceGroupElement[1].y=0.25-x;         SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.25-y;         SpaceGroupElement[3].y=-0.75+x;        SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.25+y;        SpaceGroupElement[5].y=-0.25+x;        SpaceGroupElement[5].z=0.25-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.25-y;         SpaceGroupElement[7].y=0.75-x;         SpaceGroupElement[7].z=0.75-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=0.25-x;         SpaceGroupElement[9].y=-0.25+z;        SpaceGroupElement[9].z=-0.25+y;
      SpaceGroupElement[10].x=0.5-y;         SpaceGroupElement[10].y=-0.5+z;        SpaceGroupElement[10].z=-x;
      SpaceGroupElement[11].x=-0.75+x;       SpaceGroupElement[11].y=-0.75+z;       SpaceGroupElement[11].z=0.25-y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=-0.25+x;       SpaceGroupElement[13].y=0.25-z;        SpaceGroupElement[13].z=-0.25+y;
      SpaceGroupElement[14].x=-0.5+y;        SpaceGroupElement[14].y=0.5-z;         SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=0.75-x;        SpaceGroupElement[15].y=0.75-z;        SpaceGroupElement[15].z=0.25-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=0.5-z;         SpaceGroupElement[17].y=-0.5+x;        SpaceGroupElement[17].z=-y;
      SpaceGroupElement[18].x=0.75-z;        SpaceGroupElement[18].y=-0.75+y;       SpaceGroupElement[18].z=-0.25+x;
      SpaceGroupElement[19].x=-z;            SpaceGroupElement[19].y=0.5-x;         SpaceGroupElement[19].z=-0.5+y;
      SpaceGroupElement[20].x=0.25-z;        SpaceGroupElement[20].y=0.25-y;        SpaceGroupElement[20].z=0.25-x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=-0.75+z;       SpaceGroupElement[22].y=0.75-y;        SpaceGroupElement[22].z=-0.25+x;
      SpaceGroupElement[23].x=-0.25+z;       SpaceGroupElement[23].y=-0.75+y;       SpaceGroupElement[23].z=0.75-x;
      SpaceGroupElement[24].x=x;             SpaceGroupElement[24].y=-0.5+y;        SpaceGroupElement[24].z=-0.5+z;
      SpaceGroupElement[25].x=-0.75+y;       SpaceGroupElement[25].y=0.25-x;        SpaceGroupElement[25].z=-0.75+z;
      SpaceGroupElement[26].x=-x;            SpaceGroupElement[26].y=-y;            SpaceGroupElement[26].z=z;
      SpaceGroupElement[27].x=0.75-y;        SpaceGroupElement[27].y=-0.75+x;       SpaceGroupElement[27].z=-0.25+z;
      SpaceGroupElement[28].x=x;             SpaceGroupElement[28].y=0.5-y;         SpaceGroupElement[28].z=0.5-z;
      SpaceGroupElement[29].x=-0.75+y;       SpaceGroupElement[29].y=-0.25+x;       SpaceGroupElement[29].z=0.75-z;
      SpaceGroupElement[30].x=-x;            SpaceGroupElement[30].y=y;             SpaceGroupElement[30].z=-z;
      SpaceGroupElement[31].x=0.75-y;        SpaceGroupElement[31].y=0.75-x;        SpaceGroupElement[31].z=0.25-z;
      SpaceGroupElement[32].x=-0.5+y;        SpaceGroupElement[32].y=-0.5+z;        SpaceGroupElement[32].z=x;
      SpaceGroupElement[33].x=0.25-x;        SpaceGroupElement[33].y=-0.75+z;       SpaceGroupElement[33].z=-0.75+y;
      SpaceGroupElement[34].x=-y;            SpaceGroupElement[34].y=z;             SpaceGroupElement[34].z=-x;
      SpaceGroupElement[35].x=-0.75+x;       SpaceGroupElement[35].y=-0.25+z;       SpaceGroupElement[35].z=0.75-y;
      SpaceGroupElement[36].x=0.5-y;         SpaceGroupElement[36].y=0.5-z;         SpaceGroupElement[36].z=x;
      SpaceGroupElement[37].x=-0.25+x;       SpaceGroupElement[37].y=0.75-z;        SpaceGroupElement[37].z=-0.75+y;
      SpaceGroupElement[38].x=y;             SpaceGroupElement[38].y=-z;            SpaceGroupElement[38].z=-x;
      SpaceGroupElement[39].x=0.75-x;        SpaceGroupElement[39].y=0.25-z;        SpaceGroupElement[39].z=0.75-y;
      SpaceGroupElement[40].x=-0.5+z;        SpaceGroupElement[40].y=x;             SpaceGroupElement[40].z=-0.5+y;
      SpaceGroupElement[41].x=-z;            SpaceGroupElement[41].y=-0.5+x;        SpaceGroupElement[41].z=0.5-y;
      SpaceGroupElement[42].x=0.25-z;        SpaceGroupElement[42].y=-0.25+y;       SpaceGroupElement[42].z=-0.25+x;
      SpaceGroupElement[43].x=0.5-z;         SpaceGroupElement[43].y=0.5-x;         SpaceGroupElement[43].z=y;
      SpaceGroupElement[44].x=0.75-z;        SpaceGroupElement[44].y=0.75-y;        SpaceGroupElement[44].z=0.25-x;
      SpaceGroupElement[45].x=-0.5+z;        SpaceGroupElement[45].y=-x;            SpaceGroupElement[45].z=0.5-y;
      SpaceGroupElement[46].x=-0.25+z;       SpaceGroupElement[46].y=0.25-y;        SpaceGroupElement[46].z=-0.25+x;
      SpaceGroupElement[47].x=-0.75+z;       SpaceGroupElement[47].y=-0.25+y;       SpaceGroupElement[47].z=0.75-x;
      SpaceGroupElement[48].x=-0.5+x;        SpaceGroupElement[48].y=y;             SpaceGroupElement[48].z=-0.5+z;
      SpaceGroupElement[49].x=-0.25+y;       SpaceGroupElement[49].y=0.75-x;        SpaceGroupElement[49].z=-0.75+z;
      SpaceGroupElement[50].x=0.5-x;         SpaceGroupElement[50].y=0.5-y;         SpaceGroupElement[50].z=z;
      SpaceGroupElement[51].x=0.25-y;        SpaceGroupElement[51].y=-0.25+x;       SpaceGroupElement[51].z=-0.25+z;
      SpaceGroupElement[52].x=-0.5+x;        SpaceGroupElement[52].y=-y;            SpaceGroupElement[52].z=0.5-z;
      SpaceGroupElement[53].x=-0.25+y;       SpaceGroupElement[53].y=-0.75+x;       SpaceGroupElement[53].z=0.75-z;
      SpaceGroupElement[54].x=0.5-x;         SpaceGroupElement[54].y=-0.5+y;        SpaceGroupElement[54].z=-z;
      SpaceGroupElement[55].x=0.25-y;        SpaceGroupElement[55].y=0.25-x;        SpaceGroupElement[55].z=0.25-z;
      SpaceGroupElement[56].x=y;             SpaceGroupElement[56].y=-0.5+z;        SpaceGroupElement[56].z=-0.5+x;
      SpaceGroupElement[57].x=0.75-x;        SpaceGroupElement[57].y=-0.75+z;       SpaceGroupElement[57].z=-0.25+y;
      SpaceGroupElement[58].x=0.5-y;         SpaceGroupElement[58].y=z;             SpaceGroupElement[58].z=0.5-x;
      SpaceGroupElement[59].x=-0.25+x;       SpaceGroupElement[59].y=-0.25+z;       SpaceGroupElement[59].z=0.25-y;
      SpaceGroupElement[60].x=-y;            SpaceGroupElement[60].y=0.5-z;         SpaceGroupElement[60].z=-0.5+x;
      SpaceGroupElement[61].x=-0.75+x;       SpaceGroupElement[61].y=0.75-z;        SpaceGroupElement[61].z=-0.25+y;
      SpaceGroupElement[62].x=-0.5+y;        SpaceGroupElement[62].y=-z;            SpaceGroupElement[62].z=0.5-x;
      SpaceGroupElement[63].x=0.25-x;        SpaceGroupElement[63].y=0.25-z;        SpaceGroupElement[63].z=0.25-y;
      SpaceGroupElement[64].x=-0.5+z;        SpaceGroupElement[64].y=-0.5+x;        SpaceGroupElement[64].z=y;
      SpaceGroupElement[65].x=-z;            SpaceGroupElement[65].y=x;             SpaceGroupElement[65].z=-y;
      SpaceGroupElement[66].x=0.25-z;        SpaceGroupElement[66].y=-0.75+y;       SpaceGroupElement[66].z=-0.75+x;
      SpaceGroupElement[67].x=0.5-z;         SpaceGroupElement[67].y=-x;            SpaceGroupElement[67].z=-0.5+y;
      SpaceGroupElement[68].x=0.75-z;        SpaceGroupElement[68].y=0.25-y;        SpaceGroupElement[68].z=0.75-x;
      SpaceGroupElement[69].x=-0.5+z;        SpaceGroupElement[69].y=0.5-x;         SpaceGroupElement[69].z=-y;
      SpaceGroupElement[70].x=-0.25+z;       SpaceGroupElement[70].y=0.75-y;        SpaceGroupElement[70].z=-0.75+x;
      SpaceGroupElement[71].x=-0.75+z;       SpaceGroupElement[71].y=-0.75+y;       SpaceGroupElement[71].z=0.25-x;
      SpaceGroupElement[72].x=-0.5+x;        SpaceGroupElement[72].y=-0.5+y;        SpaceGroupElement[72].z=z;
      SpaceGroupElement[73].x=-0.75+y;       SpaceGroupElement[73].y=0.75-x;        SpaceGroupElement[73].z=-0.25+z;
      SpaceGroupElement[74].x=0.5-x;         SpaceGroupElement[74].y=-y;            SpaceGroupElement[74].z=-0.5+z;
      SpaceGroupElement[75].x=0.75-y;        SpaceGroupElement[75].y=-0.25+x;       SpaceGroupElement[75].z=-0.75+z;
      SpaceGroupElement[76].x=-0.5+x;        SpaceGroupElement[76].y=0.5-y;         SpaceGroupElement[76].z=-z;
      SpaceGroupElement[77].x=-0.75+y;       SpaceGroupElement[77].y=-0.75+x;       SpaceGroupElement[77].z=0.25-z;
      SpaceGroupElement[78].x=0.5-x;         SpaceGroupElement[78].y=y;             SpaceGroupElement[78].z=0.5-z;
      SpaceGroupElement[79].x=0.75-y;        SpaceGroupElement[79].y=0.25-x;        SpaceGroupElement[79].z=0.75-z;
      SpaceGroupElement[80].x=-0.5+y;        SpaceGroupElement[80].y=z;             SpaceGroupElement[80].z=-0.5+x;
      SpaceGroupElement[81].x=0.75-x;        SpaceGroupElement[81].y=-0.25+z;       SpaceGroupElement[81].z=-0.75+y;
      SpaceGroupElement[82].x=-y;            SpaceGroupElement[82].y=-0.5+z;        SpaceGroupElement[82].z=0.5-x;
      SpaceGroupElement[83].x=-0.25+x;       SpaceGroupElement[83].y=-0.75+z;       SpaceGroupElement[83].z=0.75-y;
      SpaceGroupElement[84].x=0.5-y;         SpaceGroupElement[84].y=-z;            SpaceGroupElement[84].z=-0.5+x;
      SpaceGroupElement[85].x=-0.75+x;       SpaceGroupElement[85].y=0.25-z;        SpaceGroupElement[85].z=-0.75+y;
      SpaceGroupElement[86].x=y;             SpaceGroupElement[86].y=0.5-z;         SpaceGroupElement[86].z=0.5-x;
      SpaceGroupElement[87].x=0.25-x;        SpaceGroupElement[87].y=0.75-z;        SpaceGroupElement[87].z=0.75-y;
      SpaceGroupElement[88].x=z;             SpaceGroupElement[88].y=-0.5+x;        SpaceGroupElement[88].z=-0.5+y;
      SpaceGroupElement[89].x=0.5-z;         SpaceGroupElement[89].y=x;             SpaceGroupElement[89].z=0.5-y;
      SpaceGroupElement[90].x=0.75-z;        SpaceGroupElement[90].y=-0.25+y;       SpaceGroupElement[90].z=-0.75+x;
      SpaceGroupElement[91].x=-z;            SpaceGroupElement[91].y=-x;            SpaceGroupElement[91].z=y;
      SpaceGroupElement[92].x=0.25-z;        SpaceGroupElement[92].y=0.75-y;        SpaceGroupElement[92].z=0.75-x;
      SpaceGroupElement[93].x=z;             SpaceGroupElement[93].y=0.5-x;         SpaceGroupElement[93].z=0.5-y;
      SpaceGroupElement[94].x=-0.75+z;       SpaceGroupElement[94].y=0.25-y;        SpaceGroupElement[94].z=-0.75+x;
      SpaceGroupElement[95].x=-0.25+z;       SpaceGroupElement[95].y=-0.25+y;       SpaceGroupElement[95].z=0.25-x;
      break;

    case 507: // [211] I 4 3 2 (I 4 2 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=z;              SpaceGroupElement[9].z=y;
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=z;             SpaceGroupElement[11].z=-y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-z;            SpaceGroupElement[13].z=y;
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-z;            SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=-z;            SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=-y;
      SpaceGroupElement[18].x=-z;            SpaceGroupElement[18].y=y;             SpaceGroupElement[18].z=x;
      SpaceGroupElement[19].x=-z;            SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=y;
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=x;
      SpaceGroupElement[23].x=z;             SpaceGroupElement[23].y=y;             SpaceGroupElement[23].z=-x;
      SpaceGroupElement[24].x=-0.5+x;        SpaceGroupElement[24].y=-0.5+y;        SpaceGroupElement[24].z=-0.5+z;
      SpaceGroupElement[25].x=-0.5+y;        SpaceGroupElement[25].y=0.5-x;         SpaceGroupElement[25].z=-0.5+z;
      SpaceGroupElement[26].x=0.5-x;         SpaceGroupElement[26].y=0.5-y;         SpaceGroupElement[26].z=-0.5+z;
      SpaceGroupElement[27].x=0.5-y;         SpaceGroupElement[27].y=-0.5+x;        SpaceGroupElement[27].z=-0.5+z;
      SpaceGroupElement[28].x=-0.5+x;        SpaceGroupElement[28].y=0.5-y;         SpaceGroupElement[28].z=0.5-z;
      SpaceGroupElement[29].x=-0.5+y;        SpaceGroupElement[29].y=-0.5+x;        SpaceGroupElement[29].z=0.5-z;
      SpaceGroupElement[30].x=0.5-x;         SpaceGroupElement[30].y=-0.5+y;        SpaceGroupElement[30].z=0.5-z;
      SpaceGroupElement[31].x=0.5-y;         SpaceGroupElement[31].y=0.5-x;         SpaceGroupElement[31].z=0.5-z;
      SpaceGroupElement[32].x=-0.5+y;        SpaceGroupElement[32].y=-0.5+z;        SpaceGroupElement[32].z=-0.5+x;
      SpaceGroupElement[33].x=0.5-x;         SpaceGroupElement[33].y=-0.5+z;        SpaceGroupElement[33].z=-0.5+y;
      SpaceGroupElement[34].x=0.5-y;         SpaceGroupElement[34].y=-0.5+z;        SpaceGroupElement[34].z=0.5-x;
      SpaceGroupElement[35].x=-0.5+x;        SpaceGroupElement[35].y=-0.5+z;        SpaceGroupElement[35].z=0.5-y;
      SpaceGroupElement[36].x=0.5-y;         SpaceGroupElement[36].y=0.5-z;         SpaceGroupElement[36].z=-0.5+x;
      SpaceGroupElement[37].x=-0.5+x;        SpaceGroupElement[37].y=0.5-z;         SpaceGroupElement[37].z=-0.5+y;
      SpaceGroupElement[38].x=-0.5+y;        SpaceGroupElement[38].y=0.5-z;         SpaceGroupElement[38].z=0.5-x;
      SpaceGroupElement[39].x=0.5-x;         SpaceGroupElement[39].y=0.5-z;         SpaceGroupElement[39].z=0.5-y;
      SpaceGroupElement[40].x=-0.5+z;        SpaceGroupElement[40].y=-0.5+x;        SpaceGroupElement[40].z=-0.5+y;
      SpaceGroupElement[41].x=0.5-z;         SpaceGroupElement[41].y=-0.5+x;        SpaceGroupElement[41].z=0.5-y;
      SpaceGroupElement[42].x=0.5-z;         SpaceGroupElement[42].y=-0.5+y;        SpaceGroupElement[42].z=-0.5+x;
      SpaceGroupElement[43].x=0.5-z;         SpaceGroupElement[43].y=0.5-x;         SpaceGroupElement[43].z=-0.5+y;
      SpaceGroupElement[44].x=0.5-z;         SpaceGroupElement[44].y=0.5-y;         SpaceGroupElement[44].z=0.5-x;
      SpaceGroupElement[45].x=-0.5+z;        SpaceGroupElement[45].y=0.5-x;         SpaceGroupElement[45].z=0.5-y;
      SpaceGroupElement[46].x=-0.5+z;        SpaceGroupElement[46].y=0.5-y;         SpaceGroupElement[46].z=-0.5+x;
      SpaceGroupElement[47].x=-0.5+z;        SpaceGroupElement[47].y=-0.5+y;        SpaceGroupElement[47].z=0.5-x;
      break;

    case 508: // [212] P 43 3 2 (P 4acd 2ab 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.25+y;        SpaceGroupElement[1].y=0.75-x;         SpaceGroupElement[1].z=-0.75+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.75-y;         SpaceGroupElement[3].y=-0.75+x;        SpaceGroupElement[3].z=-0.25+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.75+y;        SpaceGroupElement[5].y=-0.25+x;        SpaceGroupElement[5].z=0.75-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.25-y;         SpaceGroupElement[7].y=0.25-x;         SpaceGroupElement[7].z=0.25-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=0.75-x;         SpaceGroupElement[9].y=-0.75+z;        SpaceGroupElement[9].z=-0.25+y;
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=-0.5+z;        SpaceGroupElement[10].z=0.5-x;
      SpaceGroupElement[11].x=-0.75+x;       SpaceGroupElement[11].y=-0.25+z;       SpaceGroupElement[11].z=0.75-y;
      SpaceGroupElement[12].x=0.5-y;         SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=-0.5+x;
      SpaceGroupElement[13].x=-0.25+x;       SpaceGroupElement[13].y=0.75-z;        SpaceGroupElement[13].z=-0.75+y;
      SpaceGroupElement[14].x=-0.5+y;        SpaceGroupElement[14].y=0.5-z;         SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=0.25-x;        SpaceGroupElement[15].y=0.25-z;        SpaceGroupElement[15].z=0.25-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=-z;            SpaceGroupElement[17].y=-0.5+x;        SpaceGroupElement[17].z=0.5-y;
      SpaceGroupElement[18].x=0.75-z;        SpaceGroupElement[18].y=-0.75+y;       SpaceGroupElement[18].z=-0.25+x;
      SpaceGroupElement[19].x=0.5-z;         SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=-0.5+y;
      SpaceGroupElement[20].x=0.25-z;        SpaceGroupElement[20].y=0.25-y;        SpaceGroupElement[20].z=0.25-x;
      SpaceGroupElement[21].x=-0.5+z;        SpaceGroupElement[21].y=0.5-x;         SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=-0.25+z;       SpaceGroupElement[22].y=0.75-y;        SpaceGroupElement[22].z=-0.75+x;
      SpaceGroupElement[23].x=-0.75+z;       SpaceGroupElement[23].y=-0.25+y;       SpaceGroupElement[23].z=0.75-x;
      break;

    case 509: // [213] P 41 3 2 (P 4bd 2ab 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.75+y;        SpaceGroupElement[1].y=0.25-x;         SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.25-y;         SpaceGroupElement[3].y=-0.25+x;        SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=-0.5+x;         SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.25+y;        SpaceGroupElement[5].y=-0.75+x;        SpaceGroupElement[5].z=0.25-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.75-y;         SpaceGroupElement[7].y=0.75-x;         SpaceGroupElement[7].z=0.75-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=0.25-x;         SpaceGroupElement[9].y=-0.25+z;        SpaceGroupElement[9].z=-0.75+y;
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=-0.5+z;        SpaceGroupElement[10].z=0.5-x;
      SpaceGroupElement[11].x=-0.25+x;       SpaceGroupElement[11].y=-0.75+z;       SpaceGroupElement[11].z=0.25-y;
      SpaceGroupElement[12].x=0.5-y;         SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=-0.5+x;
      SpaceGroupElement[13].x=-0.75+x;       SpaceGroupElement[13].y=0.25-z;        SpaceGroupElement[13].z=-0.25+y;
      SpaceGroupElement[14].x=-0.5+y;        SpaceGroupElement[14].y=0.5-z;         SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=0.75-x;        SpaceGroupElement[15].y=0.75-z;        SpaceGroupElement[15].z=0.75-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=-z;            SpaceGroupElement[17].y=-0.5+x;        SpaceGroupElement[17].z=0.5-y;
      SpaceGroupElement[18].x=0.25-z;        SpaceGroupElement[18].y=-0.25+y;       SpaceGroupElement[18].z=-0.75+x;
      SpaceGroupElement[19].x=0.5-z;         SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=-0.5+y;
      SpaceGroupElement[20].x=0.75-z;        SpaceGroupElement[20].y=0.75-y;        SpaceGroupElement[20].z=0.75-x;
      SpaceGroupElement[21].x=-0.5+z;        SpaceGroupElement[21].y=0.5-x;         SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=-0.75+z;       SpaceGroupElement[22].y=0.25-y;        SpaceGroupElement[22].z=-0.25+x;
      SpaceGroupElement[23].x=-0.25+z;       SpaceGroupElement[23].y=-0.75+y;       SpaceGroupElement[23].z=0.25-x;
      break;

    case 510: // [214] I 41 3 2 (I 4bd 2c 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.75+y;        SpaceGroupElement[1].y=0.25-x;         SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.25-y;         SpaceGroupElement[3].y=-0.25+x;        SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=-0.75+y;        SpaceGroupElement[5].y=-0.25+x;        SpaceGroupElement[5].z=0.75-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=0.25-y;         SpaceGroupElement[7].y=0.25-x;         SpaceGroupElement[7].z=0.25-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=0.25-x;         SpaceGroupElement[9].y=-0.25+z;        SpaceGroupElement[9].z=-0.75+y;
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=-0.5+z;        SpaceGroupElement[10].z=0.5-x;
      SpaceGroupElement[11].x=-0.25+x;       SpaceGroupElement[11].y=-0.75+z;       SpaceGroupElement[11].z=0.25-y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=0.5-z;         SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=-0.25+x;       SpaceGroupElement[13].y=0.75-z;        SpaceGroupElement[13].z=-0.75+y;
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-z;            SpaceGroupElement[14].z=0.5-x;
      SpaceGroupElement[15].x=0.25-x;        SpaceGroupElement[15].y=0.25-z;        SpaceGroupElement[15].z=0.25-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=-z;            SpaceGroupElement[17].y=-0.5+x;        SpaceGroupElement[17].z=0.5-y;
      SpaceGroupElement[18].x=0.25-z;        SpaceGroupElement[18].y=-0.25+y;       SpaceGroupElement[18].z=-0.75+x;
      SpaceGroupElement[19].x=0.5-z;         SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=-0.5+y;
      SpaceGroupElement[20].x=0.25-z;        SpaceGroupElement[20].y=0.25-y;        SpaceGroupElement[20].z=0.25-x;
      SpaceGroupElement[21].x=-0.5+z;        SpaceGroupElement[21].y=0.5-x;         SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=-0.25+z;       SpaceGroupElement[22].y=0.75-y;        SpaceGroupElement[22].z=-0.75+x;
      SpaceGroupElement[23].x=-0.75+z;       SpaceGroupElement[23].y=-0.25+y;       SpaceGroupElement[23].z=0.75-x;
      SpaceGroupElement[24].x=-0.5+x;        SpaceGroupElement[24].y=-0.5+y;        SpaceGroupElement[24].z=-0.5+z;
      SpaceGroupElement[25].x=-0.25+y;       SpaceGroupElement[25].y=0.75-x;        SpaceGroupElement[25].z=-0.75+z;
      SpaceGroupElement[26].x=-x;            SpaceGroupElement[26].y=0.5-y;         SpaceGroupElement[26].z=z;
      SpaceGroupElement[27].x=0.75-y;        SpaceGroupElement[27].y=-0.75+x;       SpaceGroupElement[27].z=-0.25+z;
      SpaceGroupElement[28].x=-0.5+x;        SpaceGroupElement[28].y=0.5-y;         SpaceGroupElement[28].z=-z;
      SpaceGroupElement[29].x=-0.25+y;       SpaceGroupElement[29].y=-0.75+x;       SpaceGroupElement[29].z=0.25-z;
      SpaceGroupElement[30].x=-x;            SpaceGroupElement[30].y=-0.5+y;        SpaceGroupElement[30].z=0.5-z;
      SpaceGroupElement[31].x=0.75-y;        SpaceGroupElement[31].y=0.75-x;        SpaceGroupElement[31].z=0.75-z;
      SpaceGroupElement[32].x=-0.5+y;        SpaceGroupElement[32].y=-0.5+z;        SpaceGroupElement[32].z=-0.5+x;
      SpaceGroupElement[33].x=0.75-x;        SpaceGroupElement[33].y=-0.75+z;       SpaceGroupElement[33].z=-0.25+y;
      SpaceGroupElement[34].x=0.5-y;         SpaceGroupElement[34].y=z;             SpaceGroupElement[34].z=-x;
      SpaceGroupElement[35].x=-0.75+x;       SpaceGroupElement[35].y=-0.25+z;       SpaceGroupElement[35].z=0.75-y;
      SpaceGroupElement[36].x=0.5-y;         SpaceGroupElement[36].y=-z;            SpaceGroupElement[36].z=-0.5+x;
      SpaceGroupElement[37].x=-0.75+x;       SpaceGroupElement[37].y=0.25-z;        SpaceGroupElement[37].z=-0.25+y;
      SpaceGroupElement[38].x=-0.5+y;        SpaceGroupElement[38].y=0.5-z;         SpaceGroupElement[38].z=-x;
      SpaceGroupElement[39].x=0.75-x;        SpaceGroupElement[39].y=0.75-z;        SpaceGroupElement[39].z=0.75-y;
      SpaceGroupElement[40].x=-0.5+z;        SpaceGroupElement[40].y=-0.5+x;        SpaceGroupElement[40].z=-0.5+y;
      SpaceGroupElement[41].x=0.5-z;         SpaceGroupElement[41].y=x;             SpaceGroupElement[41].z=-y;
      SpaceGroupElement[42].x=0.75-z;        SpaceGroupElement[42].y=-0.75+y;       SpaceGroupElement[42].z=-0.25+x;
      SpaceGroupElement[43].x=-z;            SpaceGroupElement[43].y=0.5-x;         SpaceGroupElement[43].z=y;
      SpaceGroupElement[44].x=0.75-z;        SpaceGroupElement[44].y=0.75-y;        SpaceGroupElement[44].z=0.75-x;
      SpaceGroupElement[45].x=z;             SpaceGroupElement[45].y=-x;            SpaceGroupElement[45].z=0.5-y;
      SpaceGroupElement[46].x=-0.75+z;       SpaceGroupElement[46].y=0.25-y;        SpaceGroupElement[46].z=-0.25+x;
      SpaceGroupElement[47].x=-0.25+z;       SpaceGroupElement[47].y=-0.75+y;       SpaceGroupElement[47].z=0.25-x;
      break;

    case 511: // [215] P -4 3 m (P -4 2 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=x;              SpaceGroupElement[9].y=-z;             SpaceGroupElement[9].z=-y;
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=-x;            SpaceGroupElement[13].y=z;             SpaceGroupElement[13].z=-y;
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-z;            SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=z;             SpaceGroupElement[15].z=y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=-z;            SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=-y;
      SpaceGroupElement[18].x=z;             SpaceGroupElement[18].y=-y;            SpaceGroupElement[18].z=-x;
      SpaceGroupElement[19].x=-z;            SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=y;
      SpaceGroupElement[20].x=z;             SpaceGroupElement[20].y=y;             SpaceGroupElement[20].z=x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=-z;            SpaceGroupElement[22].y=y;             SpaceGroupElement[22].z=-x;
      SpaceGroupElement[23].x=-z;            SpaceGroupElement[23].y=-y;            SpaceGroupElement[23].z=x;
      break;

    case 512: // [216] F -4 3 m (F -4 2 3)
      SpaceGroupSize=96;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=x;              SpaceGroupElement[9].y=-z;             SpaceGroupElement[9].z=-y;
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=-x;            SpaceGroupElement[13].y=z;             SpaceGroupElement[13].z=-y;
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-z;            SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=z;             SpaceGroupElement[15].z=y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=-z;            SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=-y;
      SpaceGroupElement[18].x=z;             SpaceGroupElement[18].y=-y;            SpaceGroupElement[18].z=-x;
      SpaceGroupElement[19].x=-z;            SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=y;
      SpaceGroupElement[20].x=z;             SpaceGroupElement[20].y=y;             SpaceGroupElement[20].z=x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=-z;            SpaceGroupElement[22].y=y;             SpaceGroupElement[22].z=-x;
      SpaceGroupElement[23].x=-z;            SpaceGroupElement[23].y=-y;            SpaceGroupElement[23].z=x;
      SpaceGroupElement[24].x=x;             SpaceGroupElement[24].y=-0.5+y;        SpaceGroupElement[24].z=-0.5+z;
      SpaceGroupElement[25].x=0.5-y;         SpaceGroupElement[25].y=x;             SpaceGroupElement[25].z=0.5-z;
      SpaceGroupElement[26].x=-x;            SpaceGroupElement[26].y=0.5-y;         SpaceGroupElement[26].z=-0.5+z;
      SpaceGroupElement[27].x=-0.5+y;        SpaceGroupElement[27].y=-x;            SpaceGroupElement[27].z=0.5-z;
      SpaceGroupElement[28].x=x;             SpaceGroupElement[28].y=0.5-y;         SpaceGroupElement[28].z=0.5-z;
      SpaceGroupElement[29].x=0.5-y;         SpaceGroupElement[29].y=-x;            SpaceGroupElement[29].z=-0.5+z;
      SpaceGroupElement[30].x=-x;            SpaceGroupElement[30].y=-0.5+y;        SpaceGroupElement[30].z=0.5-z;
      SpaceGroupElement[31].x=-0.5+y;        SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=-0.5+z;
      SpaceGroupElement[32].x=-0.5+y;        SpaceGroupElement[32].y=-0.5+z;        SpaceGroupElement[32].z=x;
      SpaceGroupElement[33].x=x;             SpaceGroupElement[33].y=0.5-z;         SpaceGroupElement[33].z=0.5-y;
      SpaceGroupElement[34].x=0.5-y;         SpaceGroupElement[34].y=-0.5+z;        SpaceGroupElement[34].z=-x;
      SpaceGroupElement[35].x=-x;            SpaceGroupElement[35].y=0.5-z;         SpaceGroupElement[35].z=-0.5+y;
      SpaceGroupElement[36].x=0.5-y;         SpaceGroupElement[36].y=0.5-z;         SpaceGroupElement[36].z=x;
      SpaceGroupElement[37].x=-x;            SpaceGroupElement[37].y=-0.5+z;        SpaceGroupElement[37].z=0.5-y;
      SpaceGroupElement[38].x=-0.5+y;        SpaceGroupElement[38].y=0.5-z;         SpaceGroupElement[38].z=-x;
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=-0.5+z;        SpaceGroupElement[39].z=-0.5+y;
      SpaceGroupElement[40].x=-0.5+z;        SpaceGroupElement[40].y=x;             SpaceGroupElement[40].z=-0.5+y;
      SpaceGroupElement[41].x=0.5-z;         SpaceGroupElement[41].y=x;             SpaceGroupElement[41].z=0.5-y;
      SpaceGroupElement[42].x=-0.5+z;        SpaceGroupElement[42].y=0.5-y;         SpaceGroupElement[42].z=-x;
      SpaceGroupElement[43].x=0.5-z;         SpaceGroupElement[43].y=-x;            SpaceGroupElement[43].z=-0.5+y;
      SpaceGroupElement[44].x=-0.5+z;        SpaceGroupElement[44].y=-0.5+y;        SpaceGroupElement[44].z=x;
      SpaceGroupElement[45].x=-0.5+z;        SpaceGroupElement[45].y=-x;            SpaceGroupElement[45].z=0.5-y;
      SpaceGroupElement[46].x=0.5-z;         SpaceGroupElement[46].y=-0.5+y;        SpaceGroupElement[46].z=-x;
      SpaceGroupElement[47].x=0.5-z;         SpaceGroupElement[47].y=0.5-y;         SpaceGroupElement[47].z=x;
      SpaceGroupElement[48].x=-0.5+x;        SpaceGroupElement[48].y=y;             SpaceGroupElement[48].z=-0.5+z;
      SpaceGroupElement[49].x=-y;            SpaceGroupElement[49].y=-0.5+x;        SpaceGroupElement[49].z=0.5-z;
      SpaceGroupElement[50].x=0.5-x;         SpaceGroupElement[50].y=-y;            SpaceGroupElement[50].z=-0.5+z;
      SpaceGroupElement[51].x=y;             SpaceGroupElement[51].y=0.5-x;         SpaceGroupElement[51].z=0.5-z;
      SpaceGroupElement[52].x=-0.5+x;        SpaceGroupElement[52].y=-y;            SpaceGroupElement[52].z=0.5-z;
      SpaceGroupElement[53].x=-y;            SpaceGroupElement[53].y=0.5-x;         SpaceGroupElement[53].z=-0.5+z;
      SpaceGroupElement[54].x=0.5-x;         SpaceGroupElement[54].y=y;             SpaceGroupElement[54].z=0.5-z;
      SpaceGroupElement[55].x=y;             SpaceGroupElement[55].y=-0.5+x;        SpaceGroupElement[55].z=-0.5+z;
      SpaceGroupElement[56].x=y;             SpaceGroupElement[56].y=-0.5+z;        SpaceGroupElement[56].z=-0.5+x;
      SpaceGroupElement[57].x=-0.5+x;        SpaceGroupElement[57].y=0.5-z;         SpaceGroupElement[57].z=-y;
      SpaceGroupElement[58].x=-y;            SpaceGroupElement[58].y=-0.5+z;        SpaceGroupElement[58].z=0.5-x;
      SpaceGroupElement[59].x=0.5-x;         SpaceGroupElement[59].y=0.5-z;         SpaceGroupElement[59].z=y;
      SpaceGroupElement[60].x=-y;            SpaceGroupElement[60].y=0.5-z;         SpaceGroupElement[60].z=-0.5+x;
      SpaceGroupElement[61].x=0.5-x;         SpaceGroupElement[61].y=-0.5+z;        SpaceGroupElement[61].z=-y;
      SpaceGroupElement[62].x=y;             SpaceGroupElement[62].y=0.5-z;         SpaceGroupElement[62].z=0.5-x;
      SpaceGroupElement[63].x=-0.5+x;        SpaceGroupElement[63].y=-0.5+z;        SpaceGroupElement[63].z=y;
      SpaceGroupElement[64].x=-0.5+z;        SpaceGroupElement[64].y=-0.5+x;        SpaceGroupElement[64].z=y;
      SpaceGroupElement[65].x=0.5-z;         SpaceGroupElement[65].y=-0.5+x;        SpaceGroupElement[65].z=-y;
      SpaceGroupElement[66].x=-0.5+z;        SpaceGroupElement[66].y=-y;            SpaceGroupElement[66].z=0.5-x;
      SpaceGroupElement[67].x=0.5-z;         SpaceGroupElement[67].y=0.5-x;         SpaceGroupElement[67].z=y;
      SpaceGroupElement[68].x=-0.5+z;        SpaceGroupElement[68].y=y;             SpaceGroupElement[68].z=-0.5+x;
      SpaceGroupElement[69].x=-0.5+z;        SpaceGroupElement[69].y=0.5-x;         SpaceGroupElement[69].z=-y;
      SpaceGroupElement[70].x=0.5-z;         SpaceGroupElement[70].y=y;             SpaceGroupElement[70].z=0.5-x;
      SpaceGroupElement[71].x=0.5-z;         SpaceGroupElement[71].y=-y;            SpaceGroupElement[71].z=-0.5+x;
      SpaceGroupElement[72].x=-0.5+x;        SpaceGroupElement[72].y=-0.5+y;        SpaceGroupElement[72].z=z;
      SpaceGroupElement[73].x=0.5-y;         SpaceGroupElement[73].y=-0.5+x;        SpaceGroupElement[73].z=-z;
      SpaceGroupElement[74].x=0.5-x;         SpaceGroupElement[74].y=0.5-y;         SpaceGroupElement[74].z=z;
      SpaceGroupElement[75].x=-0.5+y;        SpaceGroupElement[75].y=0.5-x;         SpaceGroupElement[75].z=-z;
      SpaceGroupElement[76].x=-0.5+x;        SpaceGroupElement[76].y=0.5-y;         SpaceGroupElement[76].z=-z;
      SpaceGroupElement[77].x=0.5-y;         SpaceGroupElement[77].y=0.5-x;         SpaceGroupElement[77].z=z;
      SpaceGroupElement[78].x=0.5-x;         SpaceGroupElement[78].y=-0.5+y;        SpaceGroupElement[78].z=-z;
      SpaceGroupElement[79].x=-0.5+y;        SpaceGroupElement[79].y=-0.5+x;        SpaceGroupElement[79].z=z;
      SpaceGroupElement[80].x=-0.5+y;        SpaceGroupElement[80].y=z;             SpaceGroupElement[80].z=-0.5+x;
      SpaceGroupElement[81].x=-0.5+x;        SpaceGroupElement[81].y=-z;            SpaceGroupElement[81].z=0.5-y;
      SpaceGroupElement[82].x=0.5-y;         SpaceGroupElement[82].y=z;             SpaceGroupElement[82].z=0.5-x;
      SpaceGroupElement[83].x=0.5-x;         SpaceGroupElement[83].y=-z;            SpaceGroupElement[83].z=-0.5+y;
      SpaceGroupElement[84].x=0.5-y;         SpaceGroupElement[84].y=-z;            SpaceGroupElement[84].z=-0.5+x;
      SpaceGroupElement[85].x=0.5-x;         SpaceGroupElement[85].y=z;             SpaceGroupElement[85].z=0.5-y;
      SpaceGroupElement[86].x=-0.5+y;        SpaceGroupElement[86].y=-z;            SpaceGroupElement[86].z=0.5-x;
      SpaceGroupElement[87].x=-0.5+x;        SpaceGroupElement[87].y=z;             SpaceGroupElement[87].z=-0.5+y;
      SpaceGroupElement[88].x=z;             SpaceGroupElement[88].y=-0.5+x;        SpaceGroupElement[88].z=-0.5+y;
      SpaceGroupElement[89].x=-z;            SpaceGroupElement[89].y=-0.5+x;        SpaceGroupElement[89].z=0.5-y;
      SpaceGroupElement[90].x=z;             SpaceGroupElement[90].y=0.5-y;         SpaceGroupElement[90].z=0.5-x;
      SpaceGroupElement[91].x=-z;            SpaceGroupElement[91].y=0.5-x;         SpaceGroupElement[91].z=-0.5+y;
      SpaceGroupElement[92].x=z;             SpaceGroupElement[92].y=-0.5+y;        SpaceGroupElement[92].z=-0.5+x;
      SpaceGroupElement[93].x=z;             SpaceGroupElement[93].y=0.5-x;         SpaceGroupElement[93].z=0.5-y;
      SpaceGroupElement[94].x=-z;            SpaceGroupElement[94].y=-0.5+y;        SpaceGroupElement[94].z=0.5-x;
      SpaceGroupElement[95].x=-z;            SpaceGroupElement[95].y=0.5-y;         SpaceGroupElement[95].z=-0.5+x;
      break;

    case 513: // [217] I -4 3 m (I -4 2 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=x;              SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=y;              SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=-x;             SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=y;              SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=x;              SpaceGroupElement[9].y=-z;             SpaceGroupElement[9].z=-y;
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=-x;            SpaceGroupElement[13].y=z;             SpaceGroupElement[13].z=-y;
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-z;            SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=z;             SpaceGroupElement[15].z=y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=-z;            SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=-y;
      SpaceGroupElement[18].x=z;             SpaceGroupElement[18].y=-y;            SpaceGroupElement[18].z=-x;
      SpaceGroupElement[19].x=-z;            SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=y;
      SpaceGroupElement[20].x=z;             SpaceGroupElement[20].y=y;             SpaceGroupElement[20].z=x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=-z;            SpaceGroupElement[22].y=y;             SpaceGroupElement[22].z=-x;
      SpaceGroupElement[23].x=-z;            SpaceGroupElement[23].y=-y;            SpaceGroupElement[23].z=x;
      SpaceGroupElement[24].x=-0.5+x;        SpaceGroupElement[24].y=-0.5+y;        SpaceGroupElement[24].z=-0.5+z;
      SpaceGroupElement[25].x=0.5-y;         SpaceGroupElement[25].y=-0.5+x;        SpaceGroupElement[25].z=0.5-z;
      SpaceGroupElement[26].x=0.5-x;         SpaceGroupElement[26].y=0.5-y;         SpaceGroupElement[26].z=-0.5+z;
      SpaceGroupElement[27].x=-0.5+y;        SpaceGroupElement[27].y=0.5-x;         SpaceGroupElement[27].z=0.5-z;
      SpaceGroupElement[28].x=-0.5+x;        SpaceGroupElement[28].y=0.5-y;         SpaceGroupElement[28].z=0.5-z;
      SpaceGroupElement[29].x=0.5-y;         SpaceGroupElement[29].y=0.5-x;         SpaceGroupElement[29].z=-0.5+z;
      SpaceGroupElement[30].x=0.5-x;         SpaceGroupElement[30].y=-0.5+y;        SpaceGroupElement[30].z=0.5-z;
      SpaceGroupElement[31].x=-0.5+y;        SpaceGroupElement[31].y=-0.5+x;        SpaceGroupElement[31].z=-0.5+z;
      SpaceGroupElement[32].x=-0.5+y;        SpaceGroupElement[32].y=-0.5+z;        SpaceGroupElement[32].z=-0.5+x;
      SpaceGroupElement[33].x=-0.5+x;        SpaceGroupElement[33].y=0.5-z;         SpaceGroupElement[33].z=0.5-y;
      SpaceGroupElement[34].x=0.5-y;         SpaceGroupElement[34].y=-0.5+z;        SpaceGroupElement[34].z=0.5-x;
      SpaceGroupElement[35].x=0.5-x;         SpaceGroupElement[35].y=0.5-z;         SpaceGroupElement[35].z=-0.5+y;
      SpaceGroupElement[36].x=0.5-y;         SpaceGroupElement[36].y=0.5-z;         SpaceGroupElement[36].z=-0.5+x;
      SpaceGroupElement[37].x=0.5-x;         SpaceGroupElement[37].y=-0.5+z;        SpaceGroupElement[37].z=0.5-y;
      SpaceGroupElement[38].x=-0.5+y;        SpaceGroupElement[38].y=0.5-z;         SpaceGroupElement[38].z=0.5-x;
      SpaceGroupElement[39].x=-0.5+x;        SpaceGroupElement[39].y=-0.5+z;        SpaceGroupElement[39].z=-0.5+y;
      SpaceGroupElement[40].x=-0.5+z;        SpaceGroupElement[40].y=-0.5+x;        SpaceGroupElement[40].z=-0.5+y;
      SpaceGroupElement[41].x=0.5-z;         SpaceGroupElement[41].y=-0.5+x;        SpaceGroupElement[41].z=0.5-y;
      SpaceGroupElement[42].x=-0.5+z;        SpaceGroupElement[42].y=0.5-y;         SpaceGroupElement[42].z=0.5-x;
      SpaceGroupElement[43].x=0.5-z;         SpaceGroupElement[43].y=0.5-x;         SpaceGroupElement[43].z=-0.5+y;
      SpaceGroupElement[44].x=-0.5+z;        SpaceGroupElement[44].y=-0.5+y;        SpaceGroupElement[44].z=-0.5+x;
      SpaceGroupElement[45].x=-0.5+z;        SpaceGroupElement[45].y=0.5-x;         SpaceGroupElement[45].z=0.5-y;
      SpaceGroupElement[46].x=0.5-z;         SpaceGroupElement[46].y=-0.5+y;        SpaceGroupElement[46].z=0.5-x;
      SpaceGroupElement[47].x=0.5-z;         SpaceGroupElement[47].y=0.5-y;         SpaceGroupElement[47].z=-0.5+x;
      break;

    case 514: // [218] P -4 3 n (P -4n 2 3)
      SpaceGroupSize=24;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.5-y;          SpaceGroupElement[1].y=-0.5+x;         SpaceGroupElement[1].z=0.5-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-0.5+y;         SpaceGroupElement[3].y=0.5-x;          SpaceGroupElement[3].z=0.5-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=0.5-y;          SpaceGroupElement[5].y=0.5-x;          SpaceGroupElement[5].z=-0.5+z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-0.5+y;         SpaceGroupElement[7].y=-0.5+x;         SpaceGroupElement[7].z=-0.5+z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=-0.5+x;         SpaceGroupElement[9].y=0.5-z;          SpaceGroupElement[9].z=0.5-y;
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;
      SpaceGroupElement[11].x=0.5-x;         SpaceGroupElement[11].y=0.5-z;         SpaceGroupElement[11].z=-0.5+y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=0.5-x;         SpaceGroupElement[13].y=-0.5+z;        SpaceGroupElement[13].z=0.5-y;
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-z;            SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=-0.5+x;        SpaceGroupElement[15].y=-0.5+z;        SpaceGroupElement[15].z=-0.5+y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=-z;            SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=-y;
      SpaceGroupElement[18].x=-0.5+z;        SpaceGroupElement[18].y=0.5-y;         SpaceGroupElement[18].z=0.5-x;
      SpaceGroupElement[19].x=-z;            SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=y;
      SpaceGroupElement[20].x=-0.5+z;        SpaceGroupElement[20].y=-0.5+y;        SpaceGroupElement[20].z=-0.5+x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=0.5-z;         SpaceGroupElement[22].y=-0.5+y;        SpaceGroupElement[22].z=0.5-x;
      SpaceGroupElement[23].x=0.5-z;         SpaceGroupElement[23].y=0.5-y;         SpaceGroupElement[23].z=-0.5+x;
      break;

    case 515: // [219] F -4 3 c (F -4c 2 3)
      SpaceGroupSize=96;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-y;             SpaceGroupElement[1].y=-0.5+x;         SpaceGroupElement[1].z=-z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-0.5+y;         SpaceGroupElement[3].y=-x;             SpaceGroupElement[3].z=-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-y;             SpaceGroupElement[5].y=0.5-x;          SpaceGroupElement[5].z=z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-0.5+y;         SpaceGroupElement[7].y=x;              SpaceGroupElement[7].z=z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=-0.5+x;         SpaceGroupElement[9].y=-z;             SpaceGroupElement[9].z=-y;
      SpaceGroupElement[10].x=0.5-y;         SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=0.5-x;
      SpaceGroupElement[11].x=-x;            SpaceGroupElement[11].y=-z;            SpaceGroupElement[11].z=-0.5+y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=0.5-x;         SpaceGroupElement[13].y=z;             SpaceGroupElement[13].z=-y;
      SpaceGroupElement[14].x=-0.5+y;        SpaceGroupElement[14].y=-z;            SpaceGroupElement[14].z=0.5-x;
      SpaceGroupElement[15].x=x;             SpaceGroupElement[15].y=z;             SpaceGroupElement[15].z=-0.5+y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=0.5-z;         SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=0.5-y;
      SpaceGroupElement[18].x=-0.5+z;        SpaceGroupElement[18].y=-y;            SpaceGroupElement[18].z=-x;
      SpaceGroupElement[19].x=0.5-z;         SpaceGroupElement[19].y=0.5-x;         SpaceGroupElement[19].z=y;
      SpaceGroupElement[20].x=z;             SpaceGroupElement[20].y=y;             SpaceGroupElement[20].z=-0.5+x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=0.5-z;         SpaceGroupElement[22].y=y;             SpaceGroupElement[22].z=-x;
      SpaceGroupElement[23].x=0.5-z;         SpaceGroupElement[23].y=0.5-y;         SpaceGroupElement[23].z=-0.5+x;
      SpaceGroupElement[24].x=x;             SpaceGroupElement[24].y=-0.5+y;        SpaceGroupElement[24].z=-0.5+z;
      SpaceGroupElement[25].x=0.5-y;         SpaceGroupElement[25].y=-0.5+x;        SpaceGroupElement[25].z=0.5-z;
      SpaceGroupElement[26].x=0.5-x;         SpaceGroupElement[26].y=-y;            SpaceGroupElement[26].z=-0.5+z;
      SpaceGroupElement[27].x=y;             SpaceGroupElement[27].y=-x;            SpaceGroupElement[27].z=0.5-z;
      SpaceGroupElement[28].x=x;             SpaceGroupElement[28].y=0.5-y;         SpaceGroupElement[28].z=0.5-z;
      SpaceGroupElement[29].x=0.5-y;         SpaceGroupElement[29].y=0.5-x;         SpaceGroupElement[29].z=-0.5+z;
      SpaceGroupElement[30].x=0.5-x;         SpaceGroupElement[30].y=y;             SpaceGroupElement[30].z=0.5-z;
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=-0.5+z;
      SpaceGroupElement[32].x=-0.5+y;        SpaceGroupElement[32].y=-0.5+z;        SpaceGroupElement[32].z=x;
      SpaceGroupElement[33].x=-0.5+x;        SpaceGroupElement[33].y=0.5-z;         SpaceGroupElement[33].z=0.5-y;
      SpaceGroupElement[34].x=-y;            SpaceGroupElement[34].y=-0.5+z;        SpaceGroupElement[34].z=0.5-x;
      SpaceGroupElement[35].x=-x;            SpaceGroupElement[35].y=0.5-z;         SpaceGroupElement[35].z=y;
      SpaceGroupElement[36].x=0.5-y;         SpaceGroupElement[36].y=0.5-z;         SpaceGroupElement[36].z=x;
      SpaceGroupElement[37].x=0.5-x;         SpaceGroupElement[37].y=-0.5+z;        SpaceGroupElement[37].z=0.5-y;
      SpaceGroupElement[38].x=y;             SpaceGroupElement[38].y=0.5-z;         SpaceGroupElement[38].z=0.5-x;
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=-0.5+z;        SpaceGroupElement[39].z=y;
      SpaceGroupElement[40].x=-0.5+z;        SpaceGroupElement[40].y=x;             SpaceGroupElement[40].z=-0.5+y;
      SpaceGroupElement[41].x=-z;            SpaceGroupElement[41].y=x;             SpaceGroupElement[41].z=-y;
      SpaceGroupElement[42].x=z;             SpaceGroupElement[42].y=0.5-y;         SpaceGroupElement[42].z=-x;
      SpaceGroupElement[43].x=-z;            SpaceGroupElement[43].y=0.5-x;         SpaceGroupElement[43].z=-0.5+y;
      SpaceGroupElement[44].x=-0.5+z;        SpaceGroupElement[44].y=-0.5+y;        SpaceGroupElement[44].z=-0.5+x;
      SpaceGroupElement[45].x=-0.5+z;        SpaceGroupElement[45].y=-x;            SpaceGroupElement[45].z=0.5-y;
      SpaceGroupElement[46].x=-z;            SpaceGroupElement[46].y=-0.5+y;        SpaceGroupElement[46].z=-x;
      SpaceGroupElement[47].x=-z;            SpaceGroupElement[47].y=-y;            SpaceGroupElement[47].z=-0.5+x;
      SpaceGroupElement[48].x=-0.5+x;        SpaceGroupElement[48].y=y;             SpaceGroupElement[48].z=-0.5+z;
      SpaceGroupElement[49].x=-y;            SpaceGroupElement[49].y=x;             SpaceGroupElement[49].z=0.5-z;
      SpaceGroupElement[50].x=-x;            SpaceGroupElement[50].y=0.5-y;         SpaceGroupElement[50].z=-0.5+z;
      SpaceGroupElement[51].x=-0.5+y;        SpaceGroupElement[51].y=0.5-x;         SpaceGroupElement[51].z=0.5-z;
      SpaceGroupElement[52].x=-0.5+x;        SpaceGroupElement[52].y=-y;            SpaceGroupElement[52].z=0.5-z;
      SpaceGroupElement[53].x=-y;            SpaceGroupElement[53].y=-x;            SpaceGroupElement[53].z=-0.5+z;
      SpaceGroupElement[54].x=-x;            SpaceGroupElement[54].y=-0.5+y;        SpaceGroupElement[54].z=0.5-z;
      SpaceGroupElement[55].x=-0.5+y;        SpaceGroupElement[55].y=-0.5+x;        SpaceGroupElement[55].z=-0.5+z;
      SpaceGroupElement[56].x=y;             SpaceGroupElement[56].y=-0.5+z;        SpaceGroupElement[56].z=-0.5+x;
      SpaceGroupElement[57].x=x;             SpaceGroupElement[57].y=0.5-z;         SpaceGroupElement[57].z=-y;
      SpaceGroupElement[58].x=0.5-y;         SpaceGroupElement[58].y=-0.5+z;        SpaceGroupElement[58].z=-x;
      SpaceGroupElement[59].x=0.5-x;         SpaceGroupElement[59].y=0.5-z;         SpaceGroupElement[59].z=-0.5+y;
      SpaceGroupElement[60].x=-y;            SpaceGroupElement[60].y=0.5-z;         SpaceGroupElement[60].z=-0.5+x;
      SpaceGroupElement[61].x=-x;            SpaceGroupElement[61].y=-0.5+z;        SpaceGroupElement[61].z=-y;
      SpaceGroupElement[62].x=-0.5+y;        SpaceGroupElement[62].y=0.5-z;         SpaceGroupElement[62].z=-x;
      SpaceGroupElement[63].x=-0.5+x;        SpaceGroupElement[63].y=-0.5+z;        SpaceGroupElement[63].z=-0.5+y;
      SpaceGroupElement[64].x=-0.5+z;        SpaceGroupElement[64].y=-0.5+x;        SpaceGroupElement[64].z=y;
      SpaceGroupElement[65].x=-z;            SpaceGroupElement[65].y=-0.5+x;        SpaceGroupElement[65].z=0.5-y;
      SpaceGroupElement[66].x=z;             SpaceGroupElement[66].y=-y;            SpaceGroupElement[66].z=0.5-x;
      SpaceGroupElement[67].x=-z;            SpaceGroupElement[67].y=-x;            SpaceGroupElement[67].z=y;
      SpaceGroupElement[68].x=-0.5+z;        SpaceGroupElement[68].y=y;             SpaceGroupElement[68].z=x;
      SpaceGroupElement[69].x=-0.5+z;        SpaceGroupElement[69].y=0.5-x;         SpaceGroupElement[69].z=-y;
      SpaceGroupElement[70].x=-z;            SpaceGroupElement[70].y=y;             SpaceGroupElement[70].z=0.5-x;
      SpaceGroupElement[71].x=-z;            SpaceGroupElement[71].y=0.5-y;         SpaceGroupElement[71].z=x;
      SpaceGroupElement[72].x=-0.5+x;        SpaceGroupElement[72].y=-0.5+y;        SpaceGroupElement[72].z=z;
      SpaceGroupElement[73].x=0.5-y;         SpaceGroupElement[73].y=x;             SpaceGroupElement[73].z=-z;
      SpaceGroupElement[74].x=-x;            SpaceGroupElement[74].y=-y;            SpaceGroupElement[74].z=z;
      SpaceGroupElement[75].x=y;             SpaceGroupElement[75].y=0.5-x;         SpaceGroupElement[75].z=-z;
      SpaceGroupElement[76].x=-0.5+x;        SpaceGroupElement[76].y=0.5-y;         SpaceGroupElement[76].z=-z;
      SpaceGroupElement[77].x=0.5-y;         SpaceGroupElement[77].y=-x;            SpaceGroupElement[77].z=z;
      SpaceGroupElement[78].x=-x;            SpaceGroupElement[78].y=y;             SpaceGroupElement[78].z=-z;
      SpaceGroupElement[79].x=y;             SpaceGroupElement[79].y=-0.5+x;        SpaceGroupElement[79].z=z;
      SpaceGroupElement[80].x=-0.5+y;        SpaceGroupElement[80].y=z;             SpaceGroupElement[80].z=-0.5+x;
      SpaceGroupElement[81].x=x;             SpaceGroupElement[81].y=-z;            SpaceGroupElement[81].z=0.5-y;
      SpaceGroupElement[82].x=-y;            SpaceGroupElement[82].y=z;             SpaceGroupElement[82].z=-x;
      SpaceGroupElement[83].x=0.5-x;         SpaceGroupElement[83].y=-z;            SpaceGroupElement[83].z=y;
      SpaceGroupElement[84].x=0.5-y;         SpaceGroupElement[84].y=-z;            SpaceGroupElement[84].z=-0.5+x;
      SpaceGroupElement[85].x=-x;            SpaceGroupElement[85].y=z;             SpaceGroupElement[85].z=0.5-y;
      SpaceGroupElement[86].x=y;             SpaceGroupElement[86].y=-z;            SpaceGroupElement[86].z=-x;
      SpaceGroupElement[87].x=-0.5+x;        SpaceGroupElement[87].y=z;             SpaceGroupElement[87].z=y;
      SpaceGroupElement[88].x=z;             SpaceGroupElement[88].y=-0.5+x;        SpaceGroupElement[88].z=-0.5+y;
      SpaceGroupElement[89].x=0.5-z;         SpaceGroupElement[89].y=-0.5+x;        SpaceGroupElement[89].z=-y;
      SpaceGroupElement[90].x=-0.5+z;        SpaceGroupElement[90].y=0.5-y;         SpaceGroupElement[90].z=0.5-x;
      SpaceGroupElement[91].x=0.5-z;         SpaceGroupElement[91].y=-x;            SpaceGroupElement[91].z=-0.5+y;
      SpaceGroupElement[92].x=z;             SpaceGroupElement[92].y=-0.5+y;        SpaceGroupElement[92].z=x;
      SpaceGroupElement[93].x=z;             SpaceGroupElement[93].y=0.5-x;         SpaceGroupElement[93].z=0.5-y;
      SpaceGroupElement[94].x=0.5-z;         SpaceGroupElement[94].y=-0.5+y;        SpaceGroupElement[94].z=0.5-x;
      SpaceGroupElement[95].x=0.5-z;         SpaceGroupElement[95].y=-y;            SpaceGroupElement[95].z=x;
      break;

    case 516: // [220] I -4 3 d (I -4bd 2c 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=0.75-y;         SpaceGroupElement[1].y=-0.25+x;        SpaceGroupElement[1].z=0.25-z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-0.75+y;        SpaceGroupElement[3].y=0.75-x;         SpaceGroupElement[3].z=0.25-z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=0.75-y;         SpaceGroupElement[5].y=0.25-x;         SpaceGroupElement[5].z=-0.75+z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-0.75+y;        SpaceGroupElement[7].y=-0.75+x;        SpaceGroupElement[7].z=-0.75+z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=-0.25+x;        SpaceGroupElement[9].y=0.25-z;         SpaceGroupElement[9].z=0.75-y;
      SpaceGroupElement[10].x=0.5-y;         SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;
      SpaceGroupElement[11].x=0.75-x;        SpaceGroupElement[11].y=0.25-z;        SpaceGroupElement[11].z=-0.75+y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=0.5-z;         SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=0.25-x;        SpaceGroupElement[13].y=-0.75+z;       SpaceGroupElement[13].z=0.75-y;
      SpaceGroupElement[14].x=-0.5+y;        SpaceGroupElement[14].y=0.5-z;         SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=-0.75+x;       SpaceGroupElement[15].y=-0.75+z;       SpaceGroupElement[15].z=-0.75+y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=0.5-z;         SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=-y;
      SpaceGroupElement[18].x=-0.75+z;       SpaceGroupElement[18].y=0.75-y;        SpaceGroupElement[18].z=0.25-x;
      SpaceGroupElement[19].x=0.5-z;         SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=-0.5+y;
      SpaceGroupElement[20].x=-0.25+z;       SpaceGroupElement[20].y=-0.25+y;       SpaceGroupElement[20].z=-0.25+x;
      SpaceGroupElement[21].x=-0.5+z;        SpaceGroupElement[21].y=0.5-x;         SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=0.75-z;        SpaceGroupElement[22].y=-0.25+y;       SpaceGroupElement[22].z=0.25-x;
      SpaceGroupElement[23].x=0.75-z;        SpaceGroupElement[23].y=0.25-y;        SpaceGroupElement[23].z=-0.75+x;
      SpaceGroupElement[24].x=-0.5+x;        SpaceGroupElement[24].y=-0.5+y;        SpaceGroupElement[24].z=-0.5+z;
      SpaceGroupElement[25].x=0.25-y;        SpaceGroupElement[25].y=-0.75+x;       SpaceGroupElement[25].z=0.75-z;
      SpaceGroupElement[26].x=0.5-x;         SpaceGroupElement[26].y=-y;           SpaceGroupElement[26].z=-0.5+z;
      SpaceGroupElement[27].x=-0.25+y;       SpaceGroupElement[27].y=0.25-x;        SpaceGroupElement[27].z=0.75-z;
      SpaceGroupElement[28].x=-0.5+x;        SpaceGroupElement[28].y=0.5-y;         SpaceGroupElement[28].z=-z;
      SpaceGroupElement[29].x=0.25-y;        SpaceGroupElement[29].y=0.75-x;        SpaceGroupElement[29].z=-0.25+z;
      SpaceGroupElement[30].x=0.5-x;         SpaceGroupElement[30].y=y;             SpaceGroupElement[30].z=-z;
      SpaceGroupElement[31].x=-0.25+y;       SpaceGroupElement[31].y=-0.25+x;       SpaceGroupElement[31].z=-0.25+z;
      SpaceGroupElement[32].x=-0.5+y;        SpaceGroupElement[32].y=-0.5+z;        SpaceGroupElement[32].z=-0.5+x;
      SpaceGroupElement[33].x=-0.75+x;       SpaceGroupElement[33].y=0.75-z;        SpaceGroupElement[33].z=0.25-y;
      SpaceGroupElement[34].x=-y;           SpaceGroupElement[34].y=-0.5+z;        SpaceGroupElement[34].z=0.5-x;
      SpaceGroupElement[35].x=0.25-x;        SpaceGroupElement[35].y=0.75-z;        SpaceGroupElement[35].z=-0.25+y;
      SpaceGroupElement[36].x=0.5-y;         SpaceGroupElement[36].y=-z;           SpaceGroupElement[36].z=-0.5+x;
      SpaceGroupElement[37].x=0.75-x;        SpaceGroupElement[37].y=-0.25+z;       SpaceGroupElement[37].z=0.25-y;
      SpaceGroupElement[38].x=y;          SpaceGroupElement[38].y=-z;           SpaceGroupElement[38].z=0.5-x;
      SpaceGroupElement[39].x=-0.25+x;       SpaceGroupElement[39].y=-0.25+z;       SpaceGroupElement[39].z=-0.25+y;
      SpaceGroupElement[40].x=-0.5+z;        SpaceGroupElement[40].y=-0.5+x;        SpaceGroupElement[40].z=-0.5+y;
      SpaceGroupElement[41].x=-z;           SpaceGroupElement[41].y=-0.5+x;        SpaceGroupElement[41].z=0.5-y;
      SpaceGroupElement[42].x=-0.25+z;       SpaceGroupElement[42].y=0.25-y;        SpaceGroupElement[42].z=0.75-x;
      SpaceGroupElement[43].x=-z;           SpaceGroupElement[43].y=0.5-x;         SpaceGroupElement[43].z=y;
      SpaceGroupElement[44].x=-0.75+z;       SpaceGroupElement[44].y=-0.75+y;       SpaceGroupElement[44].z=-0.75+x;
      SpaceGroupElement[45].x=z;          SpaceGroupElement[45].y=-x;           SpaceGroupElement[45].z=0.5-y;
      SpaceGroupElement[46].x=0.25-z;        SpaceGroupElement[46].y=-0.75+y;       SpaceGroupElement[46].z=0.75-x;
      SpaceGroupElement[47].x=0.25-z;        SpaceGroupElement[47].y=0.75-y;        SpaceGroupElement[47].z=-0.25+x;
      break;

    case 517: // [221] P m -3 m (-P 4 2 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=z;              SpaceGroupElement[9].z=y;
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=z;             SpaceGroupElement[11].z=-y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-z;            SpaceGroupElement[13].z=y;
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-z;            SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=-z;            SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=-y;
      SpaceGroupElement[18].x=-z;            SpaceGroupElement[18].y=y;             SpaceGroupElement[18].z=x;
      SpaceGroupElement[19].x=-z;            SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=y;
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=x;
      SpaceGroupElement[23].x=z;             SpaceGroupElement[23].y=y;             SpaceGroupElement[23].z=-x;
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;
      SpaceGroupElement[25].x=-y;            SpaceGroupElement[25].y=x;             SpaceGroupElement[25].z=-z;
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=y;             SpaceGroupElement[26].z=-z;
      SpaceGroupElement[27].x=y;             SpaceGroupElement[27].y=-x;            SpaceGroupElement[27].z=-z;
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=y;             SpaceGroupElement[28].z=z;
      SpaceGroupElement[29].x=-y;            SpaceGroupElement[29].y=-x;            SpaceGroupElement[29].z=z;
      SpaceGroupElement[30].x=x;             SpaceGroupElement[30].y=-y;            SpaceGroupElement[30].z=z;
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=z;
      SpaceGroupElement[32].x=-y;            SpaceGroupElement[32].y=-z;            SpaceGroupElement[32].z=-x;
      SpaceGroupElement[33].x=x;             SpaceGroupElement[33].y=-z;            SpaceGroupElement[33].z=-y;
      SpaceGroupElement[34].x=y;             SpaceGroupElement[34].y=-z;            SpaceGroupElement[34].z=x;
      SpaceGroupElement[35].x=-x;            SpaceGroupElement[35].y=-z;            SpaceGroupElement[35].z=y;
      SpaceGroupElement[36].x=y;             SpaceGroupElement[36].y=z;             SpaceGroupElement[36].z=-x;
      SpaceGroupElement[37].x=-x;            SpaceGroupElement[37].y=z;             SpaceGroupElement[37].z=-y;
      SpaceGroupElement[38].x=-y;            SpaceGroupElement[38].y=z;             SpaceGroupElement[38].z=x;
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=z;             SpaceGroupElement[39].z=y;
      SpaceGroupElement[40].x=-z;            SpaceGroupElement[40].y=-x;            SpaceGroupElement[40].z=-y;
      SpaceGroupElement[41].x=z;             SpaceGroupElement[41].y=-x;            SpaceGroupElement[41].z=y;
      SpaceGroupElement[42].x=z;             SpaceGroupElement[42].y=-y;            SpaceGroupElement[42].z=-x;
      SpaceGroupElement[43].x=z;             SpaceGroupElement[43].y=x;             SpaceGroupElement[43].z=-y;
      SpaceGroupElement[44].x=z;             SpaceGroupElement[44].y=y;             SpaceGroupElement[44].z=x;
      SpaceGroupElement[45].x=-z;            SpaceGroupElement[45].y=x;             SpaceGroupElement[45].z=y;
      SpaceGroupElement[46].x=-z;            SpaceGroupElement[46].y=y;             SpaceGroupElement[46].z=-x;
      SpaceGroupElement[47].x=-z;            SpaceGroupElement[47].y=-y;            SpaceGroupElement[47].z=x;
      break;

    case 518: // [222] P n -3 n Origin choice 1 (P 4 2 3 -1n)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=z;              SpaceGroupElement[9].z=y;
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=z;             SpaceGroupElement[11].z=-y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-z;            SpaceGroupElement[13].z=y;
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-z;            SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=-z;            SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=-y;
      SpaceGroupElement[18].x=-z;            SpaceGroupElement[18].y=y;             SpaceGroupElement[18].z=x;
      SpaceGroupElement[19].x=-z;            SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=y;
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=x;
      SpaceGroupElement[23].x=z;             SpaceGroupElement[23].y=y;             SpaceGroupElement[23].z=-x;
      SpaceGroupElement[24].x=0.5-x;         SpaceGroupElement[24].y=0.5-y;         SpaceGroupElement[24].z=0.5-z;
      SpaceGroupElement[25].x=0.5-y;         SpaceGroupElement[25].y=-0.5+x;        SpaceGroupElement[25].z=0.5-z;
      SpaceGroupElement[26].x=-0.5+x;        SpaceGroupElement[26].y=-0.5+y;        SpaceGroupElement[26].z=0.5-z;
      SpaceGroupElement[27].x=-0.5+y;        SpaceGroupElement[27].y=0.5-x;         SpaceGroupElement[27].z=0.5-z;
      SpaceGroupElement[28].x=0.5-x;         SpaceGroupElement[28].y=-0.5+y;        SpaceGroupElement[28].z=-0.5+z;
      SpaceGroupElement[29].x=0.5-y;         SpaceGroupElement[29].y=0.5-x;         SpaceGroupElement[29].z=-0.5+z;
      SpaceGroupElement[30].x=-0.5+x;        SpaceGroupElement[30].y=0.5-y;         SpaceGroupElement[30].z=-0.5+z;
      SpaceGroupElement[31].x=-0.5+y;        SpaceGroupElement[31].y=-0.5+x;        SpaceGroupElement[31].z=-0.5+z;
      SpaceGroupElement[32].x=0.5-y;         SpaceGroupElement[32].y=0.5-z;         SpaceGroupElement[32].z=0.5-x;
      SpaceGroupElement[33].x=-0.5+x;        SpaceGroupElement[33].y=0.5-z;         SpaceGroupElement[33].z=0.5-y;
      SpaceGroupElement[34].x=-0.5+y;        SpaceGroupElement[34].y=0.5-z;         SpaceGroupElement[34].z=-0.5+x;
      SpaceGroupElement[35].x=0.5-x;         SpaceGroupElement[35].y=0.5-z;         SpaceGroupElement[35].z=-0.5+y;
      SpaceGroupElement[36].x=-0.5+y;        SpaceGroupElement[36].y=-0.5+z;        SpaceGroupElement[36].z=0.5-x;
      SpaceGroupElement[37].x=0.5-x;         SpaceGroupElement[37].y=-0.5+z;        SpaceGroupElement[37].z=0.5-y;
      SpaceGroupElement[38].x=0.5-y;         SpaceGroupElement[38].y=-0.5+z;        SpaceGroupElement[38].z=-0.5+x;
      SpaceGroupElement[39].x=-0.5+x;        SpaceGroupElement[39].y=-0.5+z;        SpaceGroupElement[39].z=-0.5+y;
      SpaceGroupElement[40].x=0.5-z;         SpaceGroupElement[40].y=0.5-x;         SpaceGroupElement[40].z=0.5-y;
      SpaceGroupElement[41].x=-0.5+z;        SpaceGroupElement[41].y=0.5-x;         SpaceGroupElement[41].z=-0.5+y;
      SpaceGroupElement[42].x=-0.5+z;        SpaceGroupElement[42].y=0.5-y;         SpaceGroupElement[42].z=0.5-x;
      SpaceGroupElement[43].x=-0.5+z;        SpaceGroupElement[43].y=-0.5+x;        SpaceGroupElement[43].z=0.5-y;
      SpaceGroupElement[44].x=-0.5+z;        SpaceGroupElement[44].y=-0.5+y;        SpaceGroupElement[44].z=-0.5+x;
      SpaceGroupElement[45].x=0.5-z;         SpaceGroupElement[45].y=-0.5+x;        SpaceGroupElement[45].z=-0.5+y;
      SpaceGroupElement[46].x=0.5-z;         SpaceGroupElement[46].y=-0.5+y;        SpaceGroupElement[46].z=0.5-x;
      SpaceGroupElement[47].x=0.5-z;         SpaceGroupElement[47].y=0.5-y;         SpaceGroupElement[47].z=-0.5+x;
      break;
    case 519: // [222] P n -3 n Origin choice 2 (-P 4a 2bc 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=0.5-x;          SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=z;              SpaceGroupElement[9].z=y;
      SpaceGroupElement[10].x=0.5-y;         SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=0.5-x;
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=z;             SpaceGroupElement[11].z=0.5-y;
      SpaceGroupElement[12].x=0.5-y;         SpaceGroupElement[12].y=0.5-z;         SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=0.5-z;         SpaceGroupElement[13].z=y;
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=0.5-z;         SpaceGroupElement[14].z=0.5-x;
      SpaceGroupElement[15].x=0.5-x;         SpaceGroupElement[15].y=0.5-z;         SpaceGroupElement[15].z=0.5-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=0.5-z;         SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=0.5-y;
      SpaceGroupElement[18].x=0.5-z;         SpaceGroupElement[18].y=y;             SpaceGroupElement[18].z=x;
      SpaceGroupElement[19].x=0.5-z;         SpaceGroupElement[19].y=0.5-x;         SpaceGroupElement[19].z=y;
      SpaceGroupElement[20].x=0.5-z;         SpaceGroupElement[20].y=0.5-y;         SpaceGroupElement[20].z=0.5-x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=0.5-x;         SpaceGroupElement[21].z=0.5-y;
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=0.5-y;         SpaceGroupElement[22].z=x;
      SpaceGroupElement[23].x=z;             SpaceGroupElement[23].y=y;             SpaceGroupElement[23].z=0.5-x;
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;
      SpaceGroupElement[25].x=-y;            SpaceGroupElement[25].y=0.5+x;         SpaceGroupElement[25].z=-z;
      SpaceGroupElement[26].x=0.5+x;         SpaceGroupElement[26].y=0.5+y;         SpaceGroupElement[26].z=-z;
      SpaceGroupElement[27].x=0.5+y;         SpaceGroupElement[27].y=-x;            SpaceGroupElement[27].z=-z;
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=0.5+y;         SpaceGroupElement[28].z=0.5+z;
      SpaceGroupElement[29].x=-y;            SpaceGroupElement[29].y=-x;            SpaceGroupElement[29].z=0.5+z;
      SpaceGroupElement[30].x=0.5+x;         SpaceGroupElement[30].y=-y;            SpaceGroupElement[30].z=0.5+z;
      SpaceGroupElement[31].x=0.5+y;         SpaceGroupElement[31].y=0.5+x;         SpaceGroupElement[31].z=0.5+z;
      SpaceGroupElement[32].x=-y;            SpaceGroupElement[32].y=-z;            SpaceGroupElement[32].z=-x;
      SpaceGroupElement[33].x=0.5+x;         SpaceGroupElement[33].y=-z;            SpaceGroupElement[33].z=-y;
      SpaceGroupElement[34].x=0.5+y;         SpaceGroupElement[34].y=-z;            SpaceGroupElement[34].z=0.5+x;
      SpaceGroupElement[35].x=-x;            SpaceGroupElement[35].y=-z;            SpaceGroupElement[35].z=0.5+y;
      SpaceGroupElement[36].x=0.5+y;         SpaceGroupElement[36].y=0.5+z;         SpaceGroupElement[36].z=-x;
      SpaceGroupElement[37].x=-x;            SpaceGroupElement[37].y=0.5+z;         SpaceGroupElement[37].z=-y;
      SpaceGroupElement[38].x=-y;            SpaceGroupElement[38].y=0.5+z;         SpaceGroupElement[38].z=0.5+x;
      SpaceGroupElement[39].x=0.5+x;         SpaceGroupElement[39].y=0.5+z;         SpaceGroupElement[39].z=0.5+y;
      SpaceGroupElement[40].x=-z;            SpaceGroupElement[40].y=-x;            SpaceGroupElement[40].z=-y;
      SpaceGroupElement[41].x=0.5+z;         SpaceGroupElement[41].y=-x;            SpaceGroupElement[41].z=0.5+y;
      SpaceGroupElement[42].x=0.5+z;         SpaceGroupElement[42].y=-y;            SpaceGroupElement[42].z=-x;
      SpaceGroupElement[43].x=0.5+z;         SpaceGroupElement[43].y=0.5+x;         SpaceGroupElement[43].z=-y;
      SpaceGroupElement[44].x=0.5+z;         SpaceGroupElement[44].y=0.5+y;         SpaceGroupElement[44].z=0.5+x;
      SpaceGroupElement[45].x=-z;            SpaceGroupElement[45].y=0.5+x;         SpaceGroupElement[45].z=0.5+y;
      SpaceGroupElement[46].x=-z;            SpaceGroupElement[46].y=0.5+y;         SpaceGroupElement[46].z=-x;
      SpaceGroupElement[47].x=-z;            SpaceGroupElement[47].y=-y;            SpaceGroupElement[47].z=0.5+x;
      break;

    case 520: // [223] P m -3 n (-P 4n 2 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.5+y;         SpaceGroupElement[5].y=-0.5+x;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=0.5-x;          SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=-0.5+z;         SpaceGroupElement[9].z=-0.5+y;
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;
      SpaceGroupElement[11].x=-0.5+x;        SpaceGroupElement[11].y=-0.5+z;        SpaceGroupElement[11].z=0.5-y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=0.5-z;         SpaceGroupElement[13].z=-0.5+y;
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-z;            SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=0.5-x;         SpaceGroupElement[15].y=0.5-z;         SpaceGroupElement[15].z=0.5-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=-z;            SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=-y;
      SpaceGroupElement[18].x=0.5-z;         SpaceGroupElement[18].y=-0.5+y;        SpaceGroupElement[18].z=-0.5+x;
      SpaceGroupElement[19].x=-z;            SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=y;
      SpaceGroupElement[20].x=0.5-z;         SpaceGroupElement[20].y=0.5-y;         SpaceGroupElement[20].z=0.5-x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=-0.5+z;        SpaceGroupElement[22].y=0.5-y;         SpaceGroupElement[22].z=-0.5+x;
      SpaceGroupElement[23].x=-0.5+z;        SpaceGroupElement[23].y=-0.5+y;        SpaceGroupElement[23].z=0.5-x;
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;
      SpaceGroupElement[25].x=-0.5-y;        SpaceGroupElement[25].y=0.5+x;         SpaceGroupElement[25].z=-0.5-z;
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=y;             SpaceGroupElement[26].z=-z;
      SpaceGroupElement[27].x=0.5+y;         SpaceGroupElement[27].y=-0.5-x;        SpaceGroupElement[27].z=-0.5-z;
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=y;             SpaceGroupElement[28].z=z;
      SpaceGroupElement[29].x=-0.5-y;        SpaceGroupElement[29].y=-0.5-x;        SpaceGroupElement[29].z=0.5+z;
      SpaceGroupElement[30].x=x;             SpaceGroupElement[30].y=-y;            SpaceGroupElement[30].z=z;
      SpaceGroupElement[31].x=0.5+y;         SpaceGroupElement[31].y=0.5+x;         SpaceGroupElement[31].z=0.5+z;
      SpaceGroupElement[32].x=-y;            SpaceGroupElement[32].y=-z;            SpaceGroupElement[32].z=-x;
      SpaceGroupElement[33].x=0.5+x;         SpaceGroupElement[33].y=-0.5-z;        SpaceGroupElement[33].z=-0.5-y;
      SpaceGroupElement[34].x=y;             SpaceGroupElement[34].y=-z;            SpaceGroupElement[34].z=x;
      SpaceGroupElement[35].x=-0.5-x;        SpaceGroupElement[35].y=-0.5-z;        SpaceGroupElement[35].z=0.5+y;
      SpaceGroupElement[36].x=y;             SpaceGroupElement[36].y=z;             SpaceGroupElement[36].z=-x;
      SpaceGroupElement[37].x=-0.5-x;        SpaceGroupElement[37].y=0.5+z;         SpaceGroupElement[37].z=-0.5-y;
      SpaceGroupElement[38].x=-y;            SpaceGroupElement[38].y=z;             SpaceGroupElement[38].z=x;
      SpaceGroupElement[39].x=0.5+x;         SpaceGroupElement[39].y=0.5+z;         SpaceGroupElement[39].z=0.5+y;
      SpaceGroupElement[40].x=-z;            SpaceGroupElement[40].y=-x;            SpaceGroupElement[40].z=-y;
      SpaceGroupElement[41].x=z;             SpaceGroupElement[41].y=-x;            SpaceGroupElement[41].z=y;
      SpaceGroupElement[42].x=0.5+z;         SpaceGroupElement[42].y=-0.5-y;        SpaceGroupElement[42].z=-0.5-x;
      SpaceGroupElement[43].x=z;             SpaceGroupElement[43].y=x;             SpaceGroupElement[43].z=-y;
      SpaceGroupElement[44].x=0.5+z;         SpaceGroupElement[44].y=0.5+y;         SpaceGroupElement[44].z=0.5+x;
      SpaceGroupElement[45].x=-z;            SpaceGroupElement[45].y=x;             SpaceGroupElement[45].z=y;
      SpaceGroupElement[46].x=-0.5-z;        SpaceGroupElement[46].y=0.5+y;         SpaceGroupElement[46].z=-0.5-x;
      SpaceGroupElement[47].x=-0.5-z;        SpaceGroupElement[47].y=-0.5-y;        SpaceGroupElement[47].z=0.5+x;
      break;

    case 521: // [224] P n -3 m Origin choice 1 (P 4n 2 3 -1n)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.5+y;         SpaceGroupElement[5].y=-0.5+x;         SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=0.5-x;          SpaceGroupElement[7].z=0.5-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=-0.5+z;         SpaceGroupElement[9].z=-0.5+y;
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;
      SpaceGroupElement[11].x=-0.5+x;        SpaceGroupElement[11].y=-0.5+z;        SpaceGroupElement[11].z=0.5-y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=0.5-z;         SpaceGroupElement[13].z=-0.5+y;
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-z;            SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=0.5-x;         SpaceGroupElement[15].y=0.5-z;         SpaceGroupElement[15].z=0.5-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=-z;            SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=-y;
      SpaceGroupElement[18].x=0.5-z;         SpaceGroupElement[18].y=-0.5+y;        SpaceGroupElement[18].z=-0.5+x;
      SpaceGroupElement[19].x=-z;            SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=y;
      SpaceGroupElement[20].x=0.5-z;         SpaceGroupElement[20].y=0.5-y;         SpaceGroupElement[20].z=0.5-x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=-0.5+z;        SpaceGroupElement[22].y=0.5-y;         SpaceGroupElement[22].z=-0.5+x;
      SpaceGroupElement[23].x=-0.5+z;        SpaceGroupElement[23].y=-0.5+y;        SpaceGroupElement[23].z=0.5-x;
      SpaceGroupElement[24].x=0.5-x;         SpaceGroupElement[24].y=0.5-y;         SpaceGroupElement[24].z=0.5-z;
      SpaceGroupElement[25].x=-y;            SpaceGroupElement[25].y=x;             SpaceGroupElement[25].z=-z;
      SpaceGroupElement[26].x=-0.5+x;        SpaceGroupElement[26].y=-0.5+y;        SpaceGroupElement[26].z=0.5-z;
      SpaceGroupElement[27].x=y;             SpaceGroupElement[27].y=-x;            SpaceGroupElement[27].z=-z;
      SpaceGroupElement[28].x=0.5-x;         SpaceGroupElement[28].y=-0.5+y;        SpaceGroupElement[28].z=-0.5+z;
      SpaceGroupElement[29].x=-y;            SpaceGroupElement[29].y=-x;            SpaceGroupElement[29].z=z;
      SpaceGroupElement[30].x=-0.5+x;        SpaceGroupElement[30].y=0.5-y;         SpaceGroupElement[30].z=-0.5+z;
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=z;
      SpaceGroupElement[32].x=0.5-y;         SpaceGroupElement[32].y=0.5-z;         SpaceGroupElement[32].z=0.5-x;
      SpaceGroupElement[33].x=x;             SpaceGroupElement[33].y=-z;            SpaceGroupElement[33].z=-y;
      SpaceGroupElement[34].x=-0.5+y;        SpaceGroupElement[34].y=0.5-z;         SpaceGroupElement[34].z=-0.5+x;
      SpaceGroupElement[35].x=-x;            SpaceGroupElement[35].y=-z;            SpaceGroupElement[35].z=y;
      SpaceGroupElement[36].x=-0.5+y;        SpaceGroupElement[36].y=-0.5+z;        SpaceGroupElement[36].z=0.5-x;
      SpaceGroupElement[37].x=-x;            SpaceGroupElement[37].y=z;             SpaceGroupElement[37].z=-y;
      SpaceGroupElement[38].x=0.5-y;         SpaceGroupElement[38].y=-0.5+z;        SpaceGroupElement[38].z=-0.5+x;
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=z;             SpaceGroupElement[39].z=y;
      SpaceGroupElement[40].x=0.5-z;         SpaceGroupElement[40].y=0.5-x;         SpaceGroupElement[40].z=0.5-y;
      SpaceGroupElement[41].x=-0.5+z;        SpaceGroupElement[41].y=0.5-x;         SpaceGroupElement[41].z=-0.5+y;
      SpaceGroupElement[42].x=z;             SpaceGroupElement[42].y=-y;            SpaceGroupElement[42].z=-x;
      SpaceGroupElement[43].x=-0.5+z;        SpaceGroupElement[43].y=-0.5+x;        SpaceGroupElement[43].z=0.5-y;
      SpaceGroupElement[44].x=z;             SpaceGroupElement[44].y=y;             SpaceGroupElement[44].z=x;
      SpaceGroupElement[45].x=0.5-z;         SpaceGroupElement[45].y=-0.5+x;        SpaceGroupElement[45].z=-0.5+y;
      SpaceGroupElement[46].x=-z;            SpaceGroupElement[46].y=y;             SpaceGroupElement[46].z=-x;
      SpaceGroupElement[47].x=-z;            SpaceGroupElement[47].y=-y;            SpaceGroupElement[47].z=x;
      break;
    case 522: // [224] P n -3 m Origin choice 2 (-P 4bc 2bc 3)
      SpaceGroupSize=48;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.5+y;         SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.5+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-0.5+x;         SpaceGroupElement[3].z=-0.5+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=0.5-y;          SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=-0.5+y;         SpaceGroupElement[5].y=-0.5+x;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-0.5+z;         SpaceGroupElement[9].z=-0.5+y;
      SpaceGroupElement[10].x=0.5-y;         SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=0.5-x;
      SpaceGroupElement[11].x=-0.5+x;        SpaceGroupElement[11].y=-0.5+z;        SpaceGroupElement[11].z=-y;
      SpaceGroupElement[12].x=0.5-y;         SpaceGroupElement[12].y=0.5-z;         SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=-z;            SpaceGroupElement[13].z=-0.5+y;
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=0.5-z;         SpaceGroupElement[14].z=0.5-x;
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=0.5-z;         SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=0.5-y;
      SpaceGroupElement[18].x=-z;            SpaceGroupElement[18].y=-0.5+y;        SpaceGroupElement[18].z=-0.5+x;
      SpaceGroupElement[19].x=0.5-z;         SpaceGroupElement[19].y=0.5-x;         SpaceGroupElement[19].z=y;
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=0.5-x;         SpaceGroupElement[21].z=0.5-y;
      SpaceGroupElement[22].x=-0.5+z;        SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=-0.5+x;
      SpaceGroupElement[23].x=-0.5+z;        SpaceGroupElement[23].y=-0.5+y;        SpaceGroupElement[23].z=-x;
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;
      SpaceGroupElement[25].x=-0.5-y;        SpaceGroupElement[25].y=x;             SpaceGroupElement[25].z=-0.5-z;
      SpaceGroupElement[26].x=0.5+x;         SpaceGroupElement[26].y=0.5+y;         SpaceGroupElement[26].z=-z;
      SpaceGroupElement[27].x=y;             SpaceGroupElement[27].y=-0.5-x;        SpaceGroupElement[27].z=-0.5-z;
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=0.5+y;         SpaceGroupElement[28].z=0.5+z;
      SpaceGroupElement[29].x=-0.5-y;        SpaceGroupElement[29].y=-0.5-x;        SpaceGroupElement[29].z=z;
      SpaceGroupElement[30].x=0.5+x;         SpaceGroupElement[30].y=-y;            SpaceGroupElement[30].z=0.5+z;
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=z;
      SpaceGroupElement[32].x=-y;            SpaceGroupElement[32].y=-z;            SpaceGroupElement[32].z=-x;
      SpaceGroupElement[33].x=x;             SpaceGroupElement[33].y=-0.5-z;        SpaceGroupElement[33].z=-0.5-y;
      SpaceGroupElement[34].x=0.5+y;         SpaceGroupElement[34].y=-z;            SpaceGroupElement[34].z=0.5+x;
      SpaceGroupElement[35].x=-0.5-x;        SpaceGroupElement[35].y=-0.5-z;        SpaceGroupElement[35].z=y;
      SpaceGroupElement[36].x=0.5+y;         SpaceGroupElement[36].y=0.5+z;         SpaceGroupElement[36].z=-x;
      SpaceGroupElement[37].x=-0.5-x;        SpaceGroupElement[37].y=z;             SpaceGroupElement[37].z=-0.5-y;
      SpaceGroupElement[38].x=-y;            SpaceGroupElement[38].y=0.5+z;         SpaceGroupElement[38].z=0.5+x;
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=z;             SpaceGroupElement[39].z=y;
      SpaceGroupElement[40].x=-z;            SpaceGroupElement[40].y=-x;            SpaceGroupElement[40].z=-y;
      SpaceGroupElement[41].x=0.5+z;         SpaceGroupElement[41].y=-x;            SpaceGroupElement[41].z=0.5+y;
      SpaceGroupElement[42].x=z;             SpaceGroupElement[42].y=-0.5-y;        SpaceGroupElement[42].z=-0.5-x;
      SpaceGroupElement[43].x=0.5+z;         SpaceGroupElement[43].y=0.5+x;         SpaceGroupElement[43].z=-y;
      SpaceGroupElement[44].x=z;             SpaceGroupElement[44].y=y;             SpaceGroupElement[44].z=x;
      SpaceGroupElement[45].x=-z;            SpaceGroupElement[45].y=0.5+x;         SpaceGroupElement[45].z=0.5+y;
      SpaceGroupElement[46].x=-0.5-z;        SpaceGroupElement[46].y=y;             SpaceGroupElement[46].z=-0.5-x;
      SpaceGroupElement[47].x=-0.5-z;        SpaceGroupElement[47].y=-0.5-y;        SpaceGroupElement[47].z=x;
      break;

    case 523: // [225] F m -3 m (-F 4 2 3)
      SpaceGroupSize=192;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=z;              SpaceGroupElement[9].z=y;
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=z;             SpaceGroupElement[11].z=-y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-z;            SpaceGroupElement[13].z=y;
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-z;            SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=-z;            SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=-y;
      SpaceGroupElement[18].x=-z;            SpaceGroupElement[18].y=y;             SpaceGroupElement[18].z=x;
      SpaceGroupElement[19].x=-z;            SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=y;
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=x;
      SpaceGroupElement[23].x=z;             SpaceGroupElement[23].y=y;             SpaceGroupElement[23].z=-x;
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;
      SpaceGroupElement[25].x=-y;            SpaceGroupElement[25].y=x;             SpaceGroupElement[25].z=-z;
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=y;             SpaceGroupElement[26].z=-z;
      SpaceGroupElement[27].x=y;             SpaceGroupElement[27].y=-x;            SpaceGroupElement[27].z=-z;
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=y;             SpaceGroupElement[28].z=z;
      SpaceGroupElement[29].x=-y;            SpaceGroupElement[29].y=-x;            SpaceGroupElement[29].z=z;
      SpaceGroupElement[30].x=x;             SpaceGroupElement[30].y=-y;            SpaceGroupElement[30].z=z;
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=z;
      SpaceGroupElement[32].x=-y;            SpaceGroupElement[32].y=-z;            SpaceGroupElement[32].z=-x;
      SpaceGroupElement[33].x=x;             SpaceGroupElement[33].y=-z;            SpaceGroupElement[33].z=-y;
      SpaceGroupElement[34].x=y;             SpaceGroupElement[34].y=-z;            SpaceGroupElement[34].z=x;
      SpaceGroupElement[35].x=-x;            SpaceGroupElement[35].y=-z;            SpaceGroupElement[35].z=y;
      SpaceGroupElement[36].x=y;             SpaceGroupElement[36].y=z;             SpaceGroupElement[36].z=-x;
      SpaceGroupElement[37].x=-x;            SpaceGroupElement[37].y=z;             SpaceGroupElement[37].z=-y;
      SpaceGroupElement[38].x=-y;            SpaceGroupElement[38].y=z;             SpaceGroupElement[38].z=x;
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=z;             SpaceGroupElement[39].z=y;
      SpaceGroupElement[40].x=-z;            SpaceGroupElement[40].y=-x;            SpaceGroupElement[40].z=-y;
      SpaceGroupElement[41].x=z;             SpaceGroupElement[41].y=-x;            SpaceGroupElement[41].z=y;
      SpaceGroupElement[42].x=z;             SpaceGroupElement[42].y=-y;            SpaceGroupElement[42].z=-x;
      SpaceGroupElement[43].x=z;             SpaceGroupElement[43].y=x;             SpaceGroupElement[43].z=-y;
      SpaceGroupElement[44].x=z;             SpaceGroupElement[44].y=y;             SpaceGroupElement[44].z=x;
      SpaceGroupElement[45].x=-z;            SpaceGroupElement[45].y=x;             SpaceGroupElement[45].z=y;
      SpaceGroupElement[46].x=-z;            SpaceGroupElement[46].y=y;             SpaceGroupElement[46].z=-x;
      SpaceGroupElement[47].x=-z;            SpaceGroupElement[47].y=-y;            SpaceGroupElement[47].z=x;
      SpaceGroupElement[48].x=x;             SpaceGroupElement[48].y=-0.5+y;        SpaceGroupElement[48].z=-0.5+z;
      SpaceGroupElement[49].x=-0.5+y;        SpaceGroupElement[49].y=-x;            SpaceGroupElement[49].z=-0.5+z;
      SpaceGroupElement[50].x=-x;            SpaceGroupElement[50].y=0.5-y;         SpaceGroupElement[50].z=-0.5+z;
      SpaceGroupElement[51].x=0.5-y;         SpaceGroupElement[51].y=x;             SpaceGroupElement[51].z=-0.5+z;
      SpaceGroupElement[52].x=x;             SpaceGroupElement[52].y=0.5-y;         SpaceGroupElement[52].z=0.5-z;
      SpaceGroupElement[53].x=-0.5+y;        SpaceGroupElement[53].y=x;             SpaceGroupElement[53].z=0.5-z;
      SpaceGroupElement[54].x=-x;            SpaceGroupElement[54].y=-0.5+y;        SpaceGroupElement[54].z=0.5-z;
      SpaceGroupElement[55].x=0.5-y;         SpaceGroupElement[55].y=-x;            SpaceGroupElement[55].z=0.5-z;
      SpaceGroupElement[56].x=-0.5+y;        SpaceGroupElement[56].y=-0.5+z;        SpaceGroupElement[56].z=x;
      SpaceGroupElement[57].x=-x;            SpaceGroupElement[57].y=-0.5+z;        SpaceGroupElement[57].z=-0.5+y;
      SpaceGroupElement[58].x=0.5-y;         SpaceGroupElement[58].y=-0.5+z;        SpaceGroupElement[58].z=-x;
      SpaceGroupElement[59].x=x;             SpaceGroupElement[59].y=-0.5+z;        SpaceGroupElement[59].z=0.5-y;
      SpaceGroupElement[60].x=0.5-y;         SpaceGroupElement[60].y=0.5-z;         SpaceGroupElement[60].z=x;
      SpaceGroupElement[61].x=x;             SpaceGroupElement[61].y=0.5-z;         SpaceGroupElement[61].z=-0.5+y;
      SpaceGroupElement[62].x=-0.5+y;        SpaceGroupElement[62].y=0.5-z;         SpaceGroupElement[62].z=-x;
      SpaceGroupElement[63].x=-x;            SpaceGroupElement[63].y=0.5-z;         SpaceGroupElement[63].z=0.5-y;
      SpaceGroupElement[64].x=-0.5+z;        SpaceGroupElement[64].y=x;             SpaceGroupElement[64].z=-0.5+y;
      SpaceGroupElement[65].x=0.5-z;         SpaceGroupElement[65].y=x;             SpaceGroupElement[65].z=0.5-y;
      SpaceGroupElement[66].x=0.5-z;         SpaceGroupElement[66].y=-0.5+y;        SpaceGroupElement[66].z=x;
      SpaceGroupElement[67].x=0.5-z;         SpaceGroupElement[67].y=-x;            SpaceGroupElement[67].z=-0.5+y;
      SpaceGroupElement[68].x=0.5-z;         SpaceGroupElement[68].y=0.5-y;         SpaceGroupElement[68].z=-x;
      SpaceGroupElement[69].x=-0.5+z;        SpaceGroupElement[69].y=-x;            SpaceGroupElement[69].z=0.5-y;
      SpaceGroupElement[70].x=-0.5+z;        SpaceGroupElement[70].y=0.5-y;         SpaceGroupElement[70].z=x;
      SpaceGroupElement[71].x=-0.5+z;        SpaceGroupElement[71].y=-0.5+y;        SpaceGroupElement[71].z=-x;
      SpaceGroupElement[72].x=-x;            SpaceGroupElement[72].y=0.5-y;         SpaceGroupElement[72].z=0.5-z;
      SpaceGroupElement[73].x=0.5-y;         SpaceGroupElement[73].y=x;             SpaceGroupElement[73].z=0.5-z;
      SpaceGroupElement[74].x=x;             SpaceGroupElement[74].y=-0.5+y;        SpaceGroupElement[74].z=0.5-z;
      SpaceGroupElement[75].x=-0.5+y;        SpaceGroupElement[75].y=-x;            SpaceGroupElement[75].z=0.5-z;
      SpaceGroupElement[76].x=-x;            SpaceGroupElement[76].y=-0.5+y;        SpaceGroupElement[76].z=-0.5+z;
      SpaceGroupElement[77].x=0.5-y;         SpaceGroupElement[77].y=-x;            SpaceGroupElement[77].z=-0.5+z;
      SpaceGroupElement[78].x=x;             SpaceGroupElement[78].y=0.5-y;         SpaceGroupElement[78].z=-0.5+z;
      SpaceGroupElement[79].x=-0.5+y;        SpaceGroupElement[79].y=x;             SpaceGroupElement[79].z=-0.5+z;
      SpaceGroupElement[80].x=0.5-y;         SpaceGroupElement[80].y=0.5-z;         SpaceGroupElement[80].z=-x;
      SpaceGroupElement[81].x=x;             SpaceGroupElement[81].y=0.5-z;         SpaceGroupElement[81].z=0.5-y;
      SpaceGroupElement[82].x=-0.5+y;        SpaceGroupElement[82].y=0.5-z;         SpaceGroupElement[82].z=x;
      SpaceGroupElement[83].x=-x;            SpaceGroupElement[83].y=0.5-z;         SpaceGroupElement[83].z=-0.5+y;
      SpaceGroupElement[84].x=-0.5+y;        SpaceGroupElement[84].y=-0.5+z;        SpaceGroupElement[84].z=-x;
      SpaceGroupElement[85].x=-x;            SpaceGroupElement[85].y=-0.5+z;        SpaceGroupElement[85].z=0.5-y;
      SpaceGroupElement[86].x=0.5-y;         SpaceGroupElement[86].y=-0.5+z;        SpaceGroupElement[86].z=x;
      SpaceGroupElement[87].x=x;             SpaceGroupElement[87].y=-0.5+z;        SpaceGroupElement[87].z=-0.5+y;
      SpaceGroupElement[88].x=0.5-z;         SpaceGroupElement[88].y=-x;            SpaceGroupElement[88].z=0.5-y;
      SpaceGroupElement[89].x=-0.5+z;        SpaceGroupElement[89].y=-x;            SpaceGroupElement[89].z=-0.5+y;
      SpaceGroupElement[90].x=-0.5+z;        SpaceGroupElement[90].y=0.5-y;         SpaceGroupElement[90].z=-x;
      SpaceGroupElement[91].x=-0.5+z;        SpaceGroupElement[91].y=x;             SpaceGroupElement[91].z=0.5-y;
      SpaceGroupElement[92].x=-0.5+z;        SpaceGroupElement[92].y=-0.5+y;        SpaceGroupElement[92].z=x;
      SpaceGroupElement[93].x=0.5-z;         SpaceGroupElement[93].y=x;             SpaceGroupElement[93].z=-0.5+y;
      SpaceGroupElement[94].x=0.5-z;         SpaceGroupElement[94].y=-0.5+y;        SpaceGroupElement[94].z=-x;
      SpaceGroupElement[95].x=0.5-z;         SpaceGroupElement[95].y=0.5-y;         SpaceGroupElement[95].z=x;
      SpaceGroupElement[96].x=-0.5+x;        SpaceGroupElement[96].y=y;             SpaceGroupElement[96].z=-0.5+z;
      SpaceGroupElement[97].x=y;             SpaceGroupElement[97].y=0.5-x;         SpaceGroupElement[97].z=-0.5+z;
      SpaceGroupElement[98].x=0.5-x;         SpaceGroupElement[98].y=-y;            SpaceGroupElement[98].z=-0.5+z;
      SpaceGroupElement[99].x=-y;            SpaceGroupElement[99].y=-0.5+x;        SpaceGroupElement[99].z=-0.5+z;
      SpaceGroupElement[100].x=-0.5+x;       SpaceGroupElement[100].y=-y;           SpaceGroupElement[100].z=0.5-z;
      SpaceGroupElement[101].x=y;            SpaceGroupElement[101].y=-0.5+x;       SpaceGroupElement[101].z=0.5-z;
      SpaceGroupElement[102].x=0.5-x;        SpaceGroupElement[102].y=y;            SpaceGroupElement[102].z=0.5-z;
      SpaceGroupElement[103].x=-y;           SpaceGroupElement[103].y=0.5-x;        SpaceGroupElement[103].z=0.5-z;
      SpaceGroupElement[104].x=y;            SpaceGroupElement[104].y=-0.5+z;       SpaceGroupElement[104].z=-0.5+x;
      SpaceGroupElement[105].x=0.5-x;        SpaceGroupElement[105].y=-0.5+z;       SpaceGroupElement[105].z=y;
      SpaceGroupElement[106].x=-y;           SpaceGroupElement[106].y=-0.5+z;       SpaceGroupElement[106].z=0.5-x;
      SpaceGroupElement[107].x=-0.5+x;       SpaceGroupElement[107].y=-0.5+z;       SpaceGroupElement[107].z=-y;
      SpaceGroupElement[108].x=-y;           SpaceGroupElement[108].y=0.5-z;        SpaceGroupElement[108].z=-0.5+x;
      SpaceGroupElement[109].x=-0.5+x;       SpaceGroupElement[109].y=0.5-z;        SpaceGroupElement[109].z=y;
      SpaceGroupElement[110].x=y;            SpaceGroupElement[110].y=0.5-z;        SpaceGroupElement[110].z=0.5-x;
      SpaceGroupElement[111].x=0.5-x;        SpaceGroupElement[111].y=0.5-z;        SpaceGroupElement[111].z=-y;
      SpaceGroupElement[112].x=-0.5+z;       SpaceGroupElement[112].y=-0.5+x;       SpaceGroupElement[112].z=y;
      SpaceGroupElement[113].x=0.5-z;        SpaceGroupElement[113].y=-0.5+x;       SpaceGroupElement[113].z=-y;
      SpaceGroupElement[114].x=0.5-z;        SpaceGroupElement[114].y=y;            SpaceGroupElement[114].z=-0.5+x;
      SpaceGroupElement[115].x=0.5-z;        SpaceGroupElement[115].y=0.5-x;        SpaceGroupElement[115].z=y;
      SpaceGroupElement[116].x=0.5-z;        SpaceGroupElement[116].y=-y;           SpaceGroupElement[116].z=0.5-x;
      SpaceGroupElement[117].x=-0.5+z;       SpaceGroupElement[117].y=0.5-x;        SpaceGroupElement[117].z=-y;
      SpaceGroupElement[118].x=-0.5+z;       SpaceGroupElement[118].y=-y;           SpaceGroupElement[118].z=-0.5+x;
      SpaceGroupElement[119].x=-0.5+z;       SpaceGroupElement[119].y=y;            SpaceGroupElement[119].z=0.5-x;
      SpaceGroupElement[120].x=0.5-x;        SpaceGroupElement[120].y=-y;           SpaceGroupElement[120].z=0.5-z;
      SpaceGroupElement[121].x=-y;           SpaceGroupElement[121].y=-0.5+x;       SpaceGroupElement[121].z=0.5-z;
      SpaceGroupElement[122].x=-0.5+x;       SpaceGroupElement[122].y=y;            SpaceGroupElement[122].z=0.5-z;
      SpaceGroupElement[123].x=y;            SpaceGroupElement[123].y=0.5-x;        SpaceGroupElement[123].z=0.5-z;
      SpaceGroupElement[124].x=0.5-x;        SpaceGroupElement[124].y=y;            SpaceGroupElement[124].z=-0.5+z;
      SpaceGroupElement[125].x=-y;           SpaceGroupElement[125].y=0.5-x;        SpaceGroupElement[125].z=-0.5+z;
      SpaceGroupElement[126].x=-0.5+x;       SpaceGroupElement[126].y=-y;           SpaceGroupElement[126].z=-0.5+z;
      SpaceGroupElement[127].x=y;            SpaceGroupElement[127].y=-0.5+x;       SpaceGroupElement[127].z=-0.5+z;
      SpaceGroupElement[128].x=-y;           SpaceGroupElement[128].y=0.5-z;        SpaceGroupElement[128].z=0.5-x;
      SpaceGroupElement[129].x=-0.5+x;       SpaceGroupElement[129].y=0.5-z;        SpaceGroupElement[129].z=-y;
      SpaceGroupElement[130].x=y;            SpaceGroupElement[130].y=0.5-z;        SpaceGroupElement[130].z=-0.5+x;
      SpaceGroupElement[131].x=0.5-x;        SpaceGroupElement[131].y=0.5-z;        SpaceGroupElement[131].z=y;
      SpaceGroupElement[132].x=y;            SpaceGroupElement[132].y=-0.5+z;       SpaceGroupElement[132].z=0.5-x;
      SpaceGroupElement[133].x=0.5-x;        SpaceGroupElement[133].y=-0.5+z;       SpaceGroupElement[133].z=-y;
      SpaceGroupElement[134].x=-y;           SpaceGroupElement[134].y=-0.5+z;       SpaceGroupElement[134].z=-0.5+x;
      SpaceGroupElement[135].x=-0.5+x;       SpaceGroupElement[135].y=-0.5+z;       SpaceGroupElement[135].z=y;
      SpaceGroupElement[136].x=0.5-z;        SpaceGroupElement[136].y=0.5-x;        SpaceGroupElement[136].z=-y;
      SpaceGroupElement[137].x=-0.5+z;       SpaceGroupElement[137].y=0.5-x;        SpaceGroupElement[137].z=y;
      SpaceGroupElement[138].x=-0.5+z;       SpaceGroupElement[138].y=-y;           SpaceGroupElement[138].z=0.5-x;
      SpaceGroupElement[139].x=-0.5+z;       SpaceGroupElement[139].y=-0.5+x;       SpaceGroupElement[139].z=-y;
      SpaceGroupElement[140].x=-0.5+z;       SpaceGroupElement[140].y=y;            SpaceGroupElement[140].z=-0.5+x;
      SpaceGroupElement[141].x=0.5-z;        SpaceGroupElement[141].y=-0.5+x;       SpaceGroupElement[141].z=y;
      SpaceGroupElement[142].x=0.5-z;        SpaceGroupElement[142].y=y;            SpaceGroupElement[142].z=0.5-x;
      SpaceGroupElement[143].x=0.5-z;        SpaceGroupElement[143].y=-y;           SpaceGroupElement[143].z=-0.5+x;
      SpaceGroupElement[144].x=-0.5+x;       SpaceGroupElement[144].y=-0.5+y;       SpaceGroupElement[144].z=z;
      SpaceGroupElement[145].x=-0.5+y;       SpaceGroupElement[145].y=0.5-x;        SpaceGroupElement[145].z=z;
      SpaceGroupElement[146].x=0.5-x;        SpaceGroupElement[146].y=0.5-y;        SpaceGroupElement[146].z=z;
      SpaceGroupElement[147].x=0.5-y;        SpaceGroupElement[147].y=-0.5+x;       SpaceGroupElement[147].z=z;
      SpaceGroupElement[148].x=-0.5+x;       SpaceGroupElement[148].y=0.5-y;        SpaceGroupElement[148].z=-z;
      SpaceGroupElement[149].x=-0.5+y;       SpaceGroupElement[149].y=-0.5+x;       SpaceGroupElement[149].z=-z;
      SpaceGroupElement[150].x=0.5-x;        SpaceGroupElement[150].y=-0.5+y;       SpaceGroupElement[150].z=-z;
      SpaceGroupElement[151].x=0.5-y;        SpaceGroupElement[151].y=0.5-x;        SpaceGroupElement[151].z=-z;
      SpaceGroupElement[152].x=-0.5+y;       SpaceGroupElement[152].y=z;            SpaceGroupElement[152].z=-0.5+x;
      SpaceGroupElement[153].x=0.5-x;        SpaceGroupElement[153].y=z;            SpaceGroupElement[153].z=-0.5+y;
      SpaceGroupElement[154].x=0.5-y;        SpaceGroupElement[154].y=z;            SpaceGroupElement[154].z=0.5-x;
      SpaceGroupElement[155].x=-0.5+x;       SpaceGroupElement[155].y=z;            SpaceGroupElement[155].z=0.5-y;
      SpaceGroupElement[156].x=0.5-y;        SpaceGroupElement[156].y=-z;           SpaceGroupElement[156].z=-0.5+x;
      SpaceGroupElement[157].x=-0.5+x;       SpaceGroupElement[157].y=-z;           SpaceGroupElement[157].z=-0.5+y;
      SpaceGroupElement[158].x=-0.5+y;       SpaceGroupElement[158].y=-z;           SpaceGroupElement[158].z=0.5-x;
      SpaceGroupElement[159].x=0.5-x;        SpaceGroupElement[159].y=-z;           SpaceGroupElement[159].z=0.5-y;
      SpaceGroupElement[160].x=z;            SpaceGroupElement[160].y=-0.5+x;       SpaceGroupElement[160].z=-0.5+y;
      SpaceGroupElement[161].x=-z;           SpaceGroupElement[161].y=-0.5+x;       SpaceGroupElement[161].z=0.5-y;
      SpaceGroupElement[162].x=-z;           SpaceGroupElement[162].y=-0.5+y;       SpaceGroupElement[162].z=-0.5+x;
      SpaceGroupElement[163].x=-z;           SpaceGroupElement[163].y=0.5-x;        SpaceGroupElement[163].z=-0.5+y;
      SpaceGroupElement[164].x=-z;           SpaceGroupElement[164].y=0.5-y;        SpaceGroupElement[164].z=0.5-x;
      SpaceGroupElement[165].x=z;            SpaceGroupElement[165].y=0.5-x;        SpaceGroupElement[165].z=0.5-y;
      SpaceGroupElement[166].x=z;            SpaceGroupElement[166].y=0.5-y;        SpaceGroupElement[166].z=-0.5+x;
      SpaceGroupElement[167].x=z;            SpaceGroupElement[167].y=-0.5+y;       SpaceGroupElement[167].z=0.5-x;
      SpaceGroupElement[168].x=0.5-x;        SpaceGroupElement[168].y=0.5-y;        SpaceGroupElement[168].z=-z;
      SpaceGroupElement[169].x=0.5-y;        SpaceGroupElement[169].y=-0.5+x;       SpaceGroupElement[169].z=-z;
      SpaceGroupElement[170].x=-0.5+x;       SpaceGroupElement[170].y=-0.5+y;       SpaceGroupElement[170].z=-z;
      SpaceGroupElement[171].x=-0.5+y;       SpaceGroupElement[171].y=0.5-x;        SpaceGroupElement[171].z=-z;
      SpaceGroupElement[172].x=0.5-x;        SpaceGroupElement[172].y=-0.5+y;       SpaceGroupElement[172].z=z;
      SpaceGroupElement[173].x=0.5-y;        SpaceGroupElement[173].y=0.5-x;        SpaceGroupElement[173].z=z;
      SpaceGroupElement[174].x=-0.5+x;       SpaceGroupElement[174].y=0.5-y;        SpaceGroupElement[174].z=z;
      SpaceGroupElement[175].x=-0.5+y;       SpaceGroupElement[175].y=-0.5+x;       SpaceGroupElement[175].z=z;
      SpaceGroupElement[176].x=0.5-y;        SpaceGroupElement[176].y=-z;           SpaceGroupElement[176].z=0.5-x;
      SpaceGroupElement[177].x=-0.5+x;       SpaceGroupElement[177].y=-z;           SpaceGroupElement[177].z=0.5-y;
      SpaceGroupElement[178].x=-0.5+y;       SpaceGroupElement[178].y=-z;           SpaceGroupElement[178].z=-0.5+x;
      SpaceGroupElement[179].x=0.5-x;        SpaceGroupElement[179].y=-z;           SpaceGroupElement[179].z=-0.5+y;
      SpaceGroupElement[180].x=-0.5+y;       SpaceGroupElement[180].y=z;            SpaceGroupElement[180].z=0.5-x;
      SpaceGroupElement[181].x=0.5-x;        SpaceGroupElement[181].y=z;            SpaceGroupElement[181].z=0.5-y;
      SpaceGroupElement[182].x=0.5-y;        SpaceGroupElement[182].y=z;            SpaceGroupElement[182].z=-0.5+x;
      SpaceGroupElement[183].x=-0.5+x;       SpaceGroupElement[183].y=z;            SpaceGroupElement[183].z=-0.5+y;
      SpaceGroupElement[184].x=-z;           SpaceGroupElement[184].y=0.5-x;        SpaceGroupElement[184].z=0.5-y;
      SpaceGroupElement[185].x=z;            SpaceGroupElement[185].y=0.5-x;        SpaceGroupElement[185].z=-0.5+y;
      SpaceGroupElement[186].x=z;            SpaceGroupElement[186].y=0.5-y;        SpaceGroupElement[186].z=0.5-x;
      SpaceGroupElement[187].x=z;            SpaceGroupElement[187].y=-0.5+x;       SpaceGroupElement[187].z=0.5-y;
      SpaceGroupElement[188].x=z;            SpaceGroupElement[188].y=-0.5+y;       SpaceGroupElement[188].z=-0.5+x;
      SpaceGroupElement[189].x=-z;           SpaceGroupElement[189].y=-0.5+x;       SpaceGroupElement[189].z=-0.5+y;
      SpaceGroupElement[190].x=-z;           SpaceGroupElement[190].y=-0.5+y;       SpaceGroupElement[190].z=0.5-x;
      SpaceGroupElement[191].x=-z;           SpaceGroupElement[191].y=0.5-y;        SpaceGroupElement[191].z=-0.5+x;
      break;

    case 524: // [226] F m -3 c (-F 4a 2 3)
      SpaceGroupSize=192;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=-0.5+x;         SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=z;              SpaceGroupElement[9].z=y;
      SpaceGroupElement[10].x=0.5-y;         SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=0.5-x;
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=z;             SpaceGroupElement[11].z=0.5-y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=-0.5+x;        SpaceGroupElement[13].y=-z;            SpaceGroupElement[13].z=y;
      SpaceGroupElement[14].x=-0.5+y;        SpaceGroupElement[14].y=-z;            SpaceGroupElement[14].z=0.5-x;
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=0.5-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=0.5-z;         SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=0.5-y;
      SpaceGroupElement[18].x=0.5-z;         SpaceGroupElement[18].y=y;             SpaceGroupElement[18].z=x;
      SpaceGroupElement[19].x=0.5-z;         SpaceGroupElement[19].y=0.5-x;         SpaceGroupElement[19].z=y;
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=0.5-x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=-0.5+z;        SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=x;
      SpaceGroupElement[23].x=-0.5+z;        SpaceGroupElement[23].y=-0.5+y;        SpaceGroupElement[23].z=0.5-x;
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;
      SpaceGroupElement[25].x=-y;            SpaceGroupElement[25].y=0.5+x;         SpaceGroupElement[25].z=-z;
      SpaceGroupElement[26].x=0.5+x;         SpaceGroupElement[26].y=0.5+y;         SpaceGroupElement[26].z=-z;
      SpaceGroupElement[27].x=0.5+y;         SpaceGroupElement[27].y=-x;            SpaceGroupElement[27].z=-z;
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=y;             SpaceGroupElement[28].z=z;
      SpaceGroupElement[29].x=-y;            SpaceGroupElement[29].y=-0.5-x;        SpaceGroupElement[29].z=z;
      SpaceGroupElement[30].x=0.5+x;         SpaceGroupElement[30].y=-0.5-y;        SpaceGroupElement[30].z=z;
      SpaceGroupElement[31].x=0.5+y;         SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=z;
      SpaceGroupElement[32].x=-y;            SpaceGroupElement[32].y=-z;            SpaceGroupElement[32].z=-x;
      SpaceGroupElement[33].x=0.5+x;         SpaceGroupElement[33].y=-z;            SpaceGroupElement[33].z=-y;
      SpaceGroupElement[34].x=0.5+y;         SpaceGroupElement[34].y=-z;            SpaceGroupElement[34].z=0.5+x;
      SpaceGroupElement[35].x=-x;            SpaceGroupElement[35].y=-z;            SpaceGroupElement[35].z=0.5+y;
      SpaceGroupElement[36].x=y;             SpaceGroupElement[36].y=z;             SpaceGroupElement[36].z=-x;
      SpaceGroupElement[37].x=-0.5-x;        SpaceGroupElement[37].y=z;             SpaceGroupElement[37].z=-y;
      SpaceGroupElement[38].x=-0.5-y;        SpaceGroupElement[38].y=z;             SpaceGroupElement[38].z=0.5+x;
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=z;             SpaceGroupElement[39].z=0.5+y;
      SpaceGroupElement[40].x=-z;            SpaceGroupElement[40].y=-x;            SpaceGroupElement[40].z=-y;
      SpaceGroupElement[41].x=0.5+z;         SpaceGroupElement[41].y=-x;            SpaceGroupElement[41].z=0.5+y;
      SpaceGroupElement[42].x=0.5+z;         SpaceGroupElement[42].y=-y;            SpaceGroupElement[42].z=-x;
      SpaceGroupElement[43].x=0.5+z;         SpaceGroupElement[43].y=0.5+x;         SpaceGroupElement[43].z=-y;
      SpaceGroupElement[44].x=z;             SpaceGroupElement[44].y=y;             SpaceGroupElement[44].z=0.5+x;
      SpaceGroupElement[45].x=-z;            SpaceGroupElement[45].y=x;             SpaceGroupElement[45].z=y;
      SpaceGroupElement[46].x=-0.5-z;        SpaceGroupElement[46].y=y;             SpaceGroupElement[46].z=-x;
      SpaceGroupElement[47].x=-0.5-z;        SpaceGroupElement[47].y=-0.5-y;        SpaceGroupElement[47].z=0.5+x;
      SpaceGroupElement[48].x=x;             SpaceGroupElement[48].y=-0.5+y;        SpaceGroupElement[48].z=-0.5+z;
      SpaceGroupElement[49].x=-0.5+y;        SpaceGroupElement[49].y=0.5-x;         SpaceGroupElement[49].z=-0.5+z;
      SpaceGroupElement[50].x=0.5-x;         SpaceGroupElement[50].y=-y;            SpaceGroupElement[50].z=-0.5+z;
      SpaceGroupElement[51].x=-y;            SpaceGroupElement[51].y=x;             SpaceGroupElement[51].z=-0.5+z;
      SpaceGroupElement[52].x=x;             SpaceGroupElement[52].y=0.5-y;         SpaceGroupElement[52].z=0.5-z;
      SpaceGroupElement[53].x=-0.5+y;        SpaceGroupElement[53].y=-0.5+x;        SpaceGroupElement[53].z=0.5-z;
      SpaceGroupElement[54].x=0.5-x;         SpaceGroupElement[54].y=y;             SpaceGroupElement[54].z=0.5-z;
      SpaceGroupElement[55].x=-y;            SpaceGroupElement[55].y=-x;            SpaceGroupElement[55].z=0.5-z;
      SpaceGroupElement[56].x=-0.5+y;        SpaceGroupElement[56].y=-0.5+z;        SpaceGroupElement[56].z=x;
      SpaceGroupElement[57].x=0.5-x;         SpaceGroupElement[57].y=-0.5+z;        SpaceGroupElement[57].z=-0.5+y;
      SpaceGroupElement[58].x=-y;            SpaceGroupElement[58].y=-0.5+z;        SpaceGroupElement[58].z=0.5-x;
      SpaceGroupElement[59].x=x;             SpaceGroupElement[59].y=-0.5+z;        SpaceGroupElement[59].z=-y;
      SpaceGroupElement[60].x=0.5-y;         SpaceGroupElement[60].y=0.5-z;         SpaceGroupElement[60].z=x;
      SpaceGroupElement[61].x=-0.5+x;        SpaceGroupElement[61].y=0.5-z;         SpaceGroupElement[61].z=-0.5+y;
      SpaceGroupElement[62].x=y;             SpaceGroupElement[62].y=0.5-z;         SpaceGroupElement[62].z=0.5-x;
      SpaceGroupElement[63].x=-x;            SpaceGroupElement[63].y=0.5-z;         SpaceGroupElement[63].z=-y;
      SpaceGroupElement[64].x=-0.5+z;        SpaceGroupElement[64].y=x;             SpaceGroupElement[64].z=-0.5+y;
      SpaceGroupElement[65].x=-z;            SpaceGroupElement[65].y=x;             SpaceGroupElement[65].z=-y;
      SpaceGroupElement[66].x=-z;            SpaceGroupElement[66].y=-0.5+y;        SpaceGroupElement[66].z=x;
      SpaceGroupElement[67].x=-z;            SpaceGroupElement[67].y=0.5-x;         SpaceGroupElement[67].z=-0.5+y;
      SpaceGroupElement[68].x=0.5-z;         SpaceGroupElement[68].y=0.5-y;         SpaceGroupElement[68].z=0.5-x;
      SpaceGroupElement[69].x=-0.5+z;        SpaceGroupElement[69].y=-x;            SpaceGroupElement[69].z=0.5-y;
      SpaceGroupElement[70].x=z;             SpaceGroupElement[70].y=0.5-y;         SpaceGroupElement[70].z=x;
      SpaceGroupElement[71].x=z;             SpaceGroupElement[71].y=y;             SpaceGroupElement[71].z=0.5-x;
      SpaceGroupElement[72].x=-x;            SpaceGroupElement[72].y=0.5-y;         SpaceGroupElement[72].z=0.5-z;
      SpaceGroupElement[73].x=0.5-y;         SpaceGroupElement[73].y=0.5+x;         SpaceGroupElement[73].z=0.5-z;
      SpaceGroupElement[74].x=0.5+x;         SpaceGroupElement[74].y=y;             SpaceGroupElement[74].z=0.5-z;
      SpaceGroupElement[75].x=y;             SpaceGroupElement[75].y=-x;            SpaceGroupElement[75].z=0.5-z;
      SpaceGroupElement[76].x=-x;            SpaceGroupElement[76].y=-0.5+y;        SpaceGroupElement[76].z=-0.5+z;
      SpaceGroupElement[77].x=0.5-y;         SpaceGroupElement[77].y=-0.5-x;        SpaceGroupElement[77].z=-0.5+z;
      SpaceGroupElement[78].x=0.5+x;         SpaceGroupElement[78].y=-y;            SpaceGroupElement[78].z=-0.5+z;
      SpaceGroupElement[79].x=y;             SpaceGroupElement[79].y=x;             SpaceGroupElement[79].z=-0.5+z;
      SpaceGroupElement[80].x=0.5-y;         SpaceGroupElement[80].y=0.5-z;         SpaceGroupElement[80].z=-x;
      SpaceGroupElement[81].x=0.5+x;         SpaceGroupElement[81].y=0.5-z;         SpaceGroupElement[81].z=0.5-y;
      SpaceGroupElement[82].x=y;             SpaceGroupElement[82].y=0.5-z;         SpaceGroupElement[82].z=0.5+x;
      SpaceGroupElement[83].x=-x;            SpaceGroupElement[83].y=0.5-z;         SpaceGroupElement[83].z=y;
      SpaceGroupElement[84].x=-0.5+y;        SpaceGroupElement[84].y=-0.5+z;        SpaceGroupElement[84].z=-x;
      SpaceGroupElement[85].x=-0.5-x;        SpaceGroupElement[85].y=-0.5+z;        SpaceGroupElement[85].z=0.5-y;
      SpaceGroupElement[86].x=-y;            SpaceGroupElement[86].y=-0.5+z;        SpaceGroupElement[86].z=0.5+x;
      SpaceGroupElement[87].x=x;             SpaceGroupElement[87].y=-0.5+z;        SpaceGroupElement[87].z=y;
      SpaceGroupElement[88].x=0.5-z;         SpaceGroupElement[88].y=-x;            SpaceGroupElement[88].z=0.5-y;
      SpaceGroupElement[89].x=z;             SpaceGroupElement[89].y=-x;            SpaceGroupElement[89].z=y;
      SpaceGroupElement[90].x=z;             SpaceGroupElement[90].y=0.5-y;         SpaceGroupElement[90].z=-x;
      SpaceGroupElement[91].x=z;             SpaceGroupElement[91].y=0.5+x;         SpaceGroupElement[91].z=0.5-y;
      SpaceGroupElement[92].x=-0.5+z;        SpaceGroupElement[92].y=-0.5+y;        SpaceGroupElement[92].z=0.5+x;
      SpaceGroupElement[93].x=0.5-z;         SpaceGroupElement[93].y=x;             SpaceGroupElement[93].z=-0.5+y;
      SpaceGroupElement[94].x=-z;            SpaceGroupElement[94].y=-0.5+y;        SpaceGroupElement[94].z=-x;
      SpaceGroupElement[95].x=-z;            SpaceGroupElement[95].y=-y;            SpaceGroupElement[95].z=0.5+x;
      SpaceGroupElement[96].x=-0.5+x;        SpaceGroupElement[96].y=y;             SpaceGroupElement[96].z=-0.5+z;
      SpaceGroupElement[97].x=y;             SpaceGroupElement[97].y=-x;            SpaceGroupElement[97].z=-0.5+z;
      SpaceGroupElement[98].x=-x;            SpaceGroupElement[98].y=0.5-y;         SpaceGroupElement[98].z=-0.5+z;
      SpaceGroupElement[99].x=0.5-y;         SpaceGroupElement[99].y=-0.5+x;        SpaceGroupElement[99].z=-0.5+z;
      SpaceGroupElement[100].x=-0.5+x;       SpaceGroupElement[100].y=-y;           SpaceGroupElement[100].z=0.5-z;
      SpaceGroupElement[101].x=y;            SpaceGroupElement[101].y=x;            SpaceGroupElement[101].z=0.5-z;
      SpaceGroupElement[102].x=-x;           SpaceGroupElement[102].y=-0.5+y;       SpaceGroupElement[102].z=0.5-z;
      SpaceGroupElement[103].x=0.5-y;        SpaceGroupElement[103].y=0.5-x;        SpaceGroupElement[103].z=0.5-z;
      SpaceGroupElement[104].x=y;            SpaceGroupElement[104].y=-0.5+z;       SpaceGroupElement[104].z=-0.5+x;
      SpaceGroupElement[105].x=-x;           SpaceGroupElement[105].y=-0.5+z;       SpaceGroupElement[105].z=y;
      SpaceGroupElement[106].x=0.5-y;        SpaceGroupElement[106].y=-0.5+z;       SpaceGroupElement[106].z=-x;
      SpaceGroupElement[107].x=-0.5+x;       SpaceGroupElement[107].y=-0.5+z;       SpaceGroupElement[107].z=0.5-y;
      SpaceGroupElement[108].x=-y;           SpaceGroupElement[108].y=0.5-z;        SpaceGroupElement[108].z=-0.5+x;
      SpaceGroupElement[109].x=x;            SpaceGroupElement[109].y=0.5-z;        SpaceGroupElement[109].z=y;
      SpaceGroupElement[110].x=-0.5+y;       SpaceGroupElement[110].y=0.5-z;        SpaceGroupElement[110].z=-x;
      SpaceGroupElement[111].x=0.5-x;        SpaceGroupElement[111].y=0.5-z;        SpaceGroupElement[111].z=0.5-y;
      SpaceGroupElement[112].x=-0.5+z;       SpaceGroupElement[112].y=-0.5+x;       SpaceGroupElement[112].z=y;
      SpaceGroupElement[113].x=-z;           SpaceGroupElement[113].y=-0.5+x;       SpaceGroupElement[113].z=0.5-y;
      SpaceGroupElement[114].x=-z;           SpaceGroupElement[114].y=y;            SpaceGroupElement[114].z=-0.5+x;
      SpaceGroupElement[115].x=-z;           SpaceGroupElement[115].y=-x;           SpaceGroupElement[115].z=y;
      SpaceGroupElement[116].x=0.5-z;        SpaceGroupElement[116].y=-y;           SpaceGroupElement[116].z=-x;
      SpaceGroupElement[117].x=-0.5+z;       SpaceGroupElement[117].y=0.5-x;        SpaceGroupElement[117].z=-y;
      SpaceGroupElement[118].x=z;            SpaceGroupElement[118].y=-y;           SpaceGroupElement[118].z=-0.5+x;
      SpaceGroupElement[119].x=z;            SpaceGroupElement[119].y=-0.5+y;       SpaceGroupElement[119].z=-x;
      SpaceGroupElement[120].x=0.5-x;        SpaceGroupElement[120].y=-y;           SpaceGroupElement[120].z=0.5-z;
      SpaceGroupElement[121].x=-y;           SpaceGroupElement[121].y=x;            SpaceGroupElement[121].z=0.5-z;
      SpaceGroupElement[122].x=x;            SpaceGroupElement[122].y=0.5+y;        SpaceGroupElement[122].z=0.5-z;
      SpaceGroupElement[123].x=0.5+y;        SpaceGroupElement[123].y=0.5-x;        SpaceGroupElement[123].z=0.5-z;
      SpaceGroupElement[124].x=0.5-x;        SpaceGroupElement[124].y=y;            SpaceGroupElement[124].z=-0.5+z;
      SpaceGroupElement[125].x=-y;           SpaceGroupElement[125].y=-x;           SpaceGroupElement[125].z=-0.5+z;
      SpaceGroupElement[126].x=x;            SpaceGroupElement[126].y=-0.5-y;       SpaceGroupElement[126].z=-0.5+z;
      SpaceGroupElement[127].x=0.5+y;        SpaceGroupElement[127].y=-0.5+x;       SpaceGroupElement[127].z=-0.5+z;
      SpaceGroupElement[128].x=-y;           SpaceGroupElement[128].y=0.5-z;        SpaceGroupElement[128].z=0.5-x;
      SpaceGroupElement[129].x=x;            SpaceGroupElement[129].y=0.5-z;        SpaceGroupElement[129].z=-y;
      SpaceGroupElement[130].x=0.5+y;        SpaceGroupElement[130].y=0.5-z;        SpaceGroupElement[130].z=x;
      SpaceGroupElement[131].x=0.5-x;        SpaceGroupElement[131].y=0.5-z;        SpaceGroupElement[131].z=0.5+y;
      SpaceGroupElement[132].x=y;            SpaceGroupElement[132].y=-0.5+z;       SpaceGroupElement[132].z=0.5-x;
      SpaceGroupElement[133].x=-x;           SpaceGroupElement[133].y=-0.5+z;       SpaceGroupElement[133].z=-y;
      SpaceGroupElement[134].x=-0.5-y;       SpaceGroupElement[134].y=-0.5+z;       SpaceGroupElement[134].z=x;
      SpaceGroupElement[135].x=-0.5+x;       SpaceGroupElement[135].y=-0.5+z;       SpaceGroupElement[135].z=0.5+y;
      SpaceGroupElement[136].x=0.5-z;        SpaceGroupElement[136].y=0.5-x;        SpaceGroupElement[136].z=-y;
      SpaceGroupElement[137].x=z;            SpaceGroupElement[137].y=0.5-x;        SpaceGroupElement[137].z=0.5+y;
      SpaceGroupElement[138].x=z;            SpaceGroupElement[138].y=-y;           SpaceGroupElement[138].z=0.5-x;
      SpaceGroupElement[139].x=z;            SpaceGroupElement[139].y=x;            SpaceGroupElement[139].z=-y;
      SpaceGroupElement[140].x=-0.5+z;       SpaceGroupElement[140].y=y;            SpaceGroupElement[140].z=x;
      SpaceGroupElement[141].x=0.5-z;        SpaceGroupElement[141].y=-0.5+x;       SpaceGroupElement[141].z=y;
      SpaceGroupElement[142].x=-z;           SpaceGroupElement[142].y=y;            SpaceGroupElement[142].z=0.5-x;
      SpaceGroupElement[143].x=-z;           SpaceGroupElement[143].y=-0.5-y;       SpaceGroupElement[143].z=x;
      SpaceGroupElement[144].x=-0.5+x;       SpaceGroupElement[144].y=-0.5+y;       SpaceGroupElement[144].z=z;
      SpaceGroupElement[145].x=-0.5+y;       SpaceGroupElement[145].y=-x;           SpaceGroupElement[145].z=z;
      SpaceGroupElement[146].x=-x;           SpaceGroupElement[146].y=-y;           SpaceGroupElement[146].z=z;
      SpaceGroupElement[147].x=-y;           SpaceGroupElement[147].y=-0.5+x;       SpaceGroupElement[147].z=z;
      SpaceGroupElement[148].x=-0.5+x;       SpaceGroupElement[148].y=0.5-y;        SpaceGroupElement[148].z=-z;
      SpaceGroupElement[149].x=-0.5+y;       SpaceGroupElement[149].y=x;            SpaceGroupElement[149].z=-z;
      SpaceGroupElement[150].x=-x;           SpaceGroupElement[150].y=y;            SpaceGroupElement[150].z=-z;
      SpaceGroupElement[151].x=-y;           SpaceGroupElement[151].y=0.5-x;        SpaceGroupElement[151].z=-z;
      SpaceGroupElement[152].x=-0.5+y;       SpaceGroupElement[152].y=z;            SpaceGroupElement[152].z=-0.5+x;
      SpaceGroupElement[153].x=-x;           SpaceGroupElement[153].y=z;            SpaceGroupElement[153].z=-0.5+y;
      SpaceGroupElement[154].x=-y;           SpaceGroupElement[154].y=z;            SpaceGroupElement[154].z=-x;
      SpaceGroupElement[155].x=-0.5+x;       SpaceGroupElement[155].y=z;            SpaceGroupElement[155].z=-y;
      SpaceGroupElement[156].x=0.5-y;        SpaceGroupElement[156].y=-z;           SpaceGroupElement[156].z=-0.5+x;
      SpaceGroupElement[157].x=x;            SpaceGroupElement[157].y=-z;           SpaceGroupElement[157].z=-0.5+y;
      SpaceGroupElement[158].x=y;            SpaceGroupElement[158].y=-z;           SpaceGroupElement[158].z=-x;
      SpaceGroupElement[159].x=0.5-x;        SpaceGroupElement[159].y=-z;           SpaceGroupElement[159].z=-y;
      SpaceGroupElement[160].x=z;            SpaceGroupElement[160].y=-0.5+x;       SpaceGroupElement[160].z=-0.5+y;
      SpaceGroupElement[161].x=0.5-z;        SpaceGroupElement[161].y=-0.5+x;       SpaceGroupElement[161].z=-y;
      SpaceGroupElement[162].x=0.5-z;        SpaceGroupElement[162].y=-0.5+y;       SpaceGroupElement[162].z=-0.5+x;
      SpaceGroupElement[163].x=0.5-z;        SpaceGroupElement[163].y=-x;           SpaceGroupElement[163].z=-0.5+y;
      SpaceGroupElement[164].x=-z;           SpaceGroupElement[164].y=0.5-y;        SpaceGroupElement[164].z=-x;
      SpaceGroupElement[165].x=z;            SpaceGroupElement[165].y=0.5-x;        SpaceGroupElement[165].z=0.5-y;
      SpaceGroupElement[166].x=-0.5+z;       SpaceGroupElement[166].y=0.5-y;        SpaceGroupElement[166].z=-0.5+x;
      SpaceGroupElement[167].x=-0.5+z;       SpaceGroupElement[167].y=y;            SpaceGroupElement[167].z=-x;
      SpaceGroupElement[168].x=0.5-x;        SpaceGroupElement[168].y=0.5-y;        SpaceGroupElement[168].z=-z;
      SpaceGroupElement[169].x=0.5-y;        SpaceGroupElement[169].y=x;            SpaceGroupElement[169].z=-z;
      SpaceGroupElement[170].x=x;            SpaceGroupElement[170].y=y;            SpaceGroupElement[170].z=-z;
      SpaceGroupElement[171].x=y;            SpaceGroupElement[171].y=0.5-x;        SpaceGroupElement[171].z=-z;
      SpaceGroupElement[172].x=0.5-x;        SpaceGroupElement[172].y=-0.5+y;       SpaceGroupElement[172].z=z;
      SpaceGroupElement[173].x=0.5-y;        SpaceGroupElement[173].y=-x;           SpaceGroupElement[173].z=z;
      SpaceGroupElement[174].x=x;            SpaceGroupElement[174].y=-y;           SpaceGroupElement[174].z=z;
      SpaceGroupElement[175].x=y;            SpaceGroupElement[175].y=-0.5+x;       SpaceGroupElement[175].z=z;
      SpaceGroupElement[176].x=0.5-y;        SpaceGroupElement[176].y=-z;           SpaceGroupElement[176].z=0.5-x;
      SpaceGroupElement[177].x=x;            SpaceGroupElement[177].y=-z;           SpaceGroupElement[177].z=0.5-y;
      SpaceGroupElement[178].x=y;            SpaceGroupElement[178].y=-z;           SpaceGroupElement[178].z=x;
      SpaceGroupElement[179].x=0.5-x;        SpaceGroupElement[179].y=-z;           SpaceGroupElement[179].z=y;
      SpaceGroupElement[180].x=-0.5+y;       SpaceGroupElement[180].y=z;            SpaceGroupElement[180].z=0.5-x;
      SpaceGroupElement[181].x=-x;           SpaceGroupElement[181].y=z;            SpaceGroupElement[181].z=0.5-y;
      SpaceGroupElement[182].x=-y;           SpaceGroupElement[182].y=z;            SpaceGroupElement[182].z=x;
      SpaceGroupElement[183].x=-0.5+x;       SpaceGroupElement[183].y=z;            SpaceGroupElement[183].z=y;
      SpaceGroupElement[184].x=-z;           SpaceGroupElement[184].y=0.5-x;        SpaceGroupElement[184].z=0.5-y;
      SpaceGroupElement[185].x=0.5+z;        SpaceGroupElement[185].y=0.5-x;        SpaceGroupElement[185].z=y;
      SpaceGroupElement[186].x=0.5+z;        SpaceGroupElement[186].y=0.5-y;        SpaceGroupElement[186].z=0.5-x;
      SpaceGroupElement[187].x=0.5+z;        SpaceGroupElement[187].y=x;            SpaceGroupElement[187].z=0.5-y;
      SpaceGroupElement[188].x=z;            SpaceGroupElement[188].y=-0.5+y;       SpaceGroupElement[188].z=x;
      SpaceGroupElement[189].x=-z;           SpaceGroupElement[189].y=-0.5+x;       SpaceGroupElement[189].z=-0.5+y;
      SpaceGroupElement[190].x=-0.5-z;       SpaceGroupElement[190].y=-0.5+y;       SpaceGroupElement[190].z=0.5-x;
      SpaceGroupElement[191].x=-0.5-z;       SpaceGroupElement[191].y=-y;           SpaceGroupElement[191].z=x;
      break;

    case 525: // [227] F d -3 m Origin choice 1 (F 4d 2 3 -1d)
      SpaceGroupSize=192;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.25+y;        SpaceGroupElement[1].y=0.25-x;         SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.25-y;         SpaceGroupElement[3].y=-0.75+x;        SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.25+y;        SpaceGroupElement[5].y=-0.25+x;        SpaceGroupElement[5].z=0.25-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.25-y;         SpaceGroupElement[7].y=0.75-x;         SpaceGroupElement[7].z=0.75-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=0.25-x;         SpaceGroupElement[9].y=-0.25+z;        SpaceGroupElement[9].z=-0.25+y;
      SpaceGroupElement[10].x=0.5-y;         SpaceGroupElement[10].y=-0.5+z;        SpaceGroupElement[10].z=-x;
      SpaceGroupElement[11].x=-0.75+x;       SpaceGroupElement[11].y=-0.75+z;       SpaceGroupElement[11].z=0.25-y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=-0.25+x;       SpaceGroupElement[13].y=0.25-z;        SpaceGroupElement[13].z=-0.25+y;
      SpaceGroupElement[14].x=-0.5+y;        SpaceGroupElement[14].y=0.5-z;         SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=0.75-x;        SpaceGroupElement[15].y=0.75-z;        SpaceGroupElement[15].z=0.25-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=0.5-z;         SpaceGroupElement[17].y=-0.5+x;        SpaceGroupElement[17].z=-y;
      SpaceGroupElement[18].x=0.75-z;        SpaceGroupElement[18].y=-0.75+y;       SpaceGroupElement[18].z=-0.25+x;
      SpaceGroupElement[19].x=-z;            SpaceGroupElement[19].y=0.5-x;         SpaceGroupElement[19].z=-0.5+y;
      SpaceGroupElement[20].x=0.25-z;        SpaceGroupElement[20].y=0.25-y;        SpaceGroupElement[20].z=0.25-x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=-0.75+z;       SpaceGroupElement[22].y=0.75-y;        SpaceGroupElement[22].z=-0.25+x;
      SpaceGroupElement[23].x=-0.25+z;       SpaceGroupElement[23].y=-0.75+y;       SpaceGroupElement[23].z=0.75-x;
      SpaceGroupElement[24].x=0.25-x;        SpaceGroupElement[24].y=0.25-y;        SpaceGroupElement[24].z=0.25-z;
      SpaceGroupElement[25].x=-y;            SpaceGroupElement[25].y=x;             SpaceGroupElement[25].z=-z;
      SpaceGroupElement[26].x=-0.25+x;       SpaceGroupElement[26].y=0.25+y;        SpaceGroupElement[26].z=-0.25-z;
      SpaceGroupElement[27].x=y;             SpaceGroupElement[27].y=-0.5-x;        SpaceGroupElement[27].z=-0.5-z;
      SpaceGroupElement[28].x=0.25-x;        SpaceGroupElement[28].y=-0.25+y;       SpaceGroupElement[28].z=-0.25+z;
      SpaceGroupElement[29].x=-y;            SpaceGroupElement[29].y=-x;            SpaceGroupElement[29].z=z;
      SpaceGroupElement[30].x=-0.25+x;       SpaceGroupElement[30].y=-0.25-y;       SpaceGroupElement[30].z=0.25+z;
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=0.5+x;         SpaceGroupElement[31].z=0.5+z;
      SpaceGroupElement[32].x=0.25-y;        SpaceGroupElement[32].y=0.25-z;        SpaceGroupElement[32].z=0.25-x;
      SpaceGroupElement[33].x=x;             SpaceGroupElement[33].y=-z;            SpaceGroupElement[33].z=-y;
      SpaceGroupElement[34].x=0.25+y;        SpaceGroupElement[34].y=-0.25-z;       SpaceGroupElement[34].z=-0.25+x;
      SpaceGroupElement[35].x=-0.5-x;        SpaceGroupElement[35].y=-0.5-z;        SpaceGroupElement[35].z=y;
      SpaceGroupElement[36].x=-0.25+y;       SpaceGroupElement[36].y=-0.25+z;       SpaceGroupElement[36].z=0.25-x;
      SpaceGroupElement[37].x=-x;            SpaceGroupElement[37].y=z;             SpaceGroupElement[37].z=-y;
      SpaceGroupElement[38].x=-0.25-y;       SpaceGroupElement[38].y=0.25+z;        SpaceGroupElement[38].z=-0.25+x;
      SpaceGroupElement[39].x=0.5+x;         SpaceGroupElement[39].y=0.5+z;         SpaceGroupElement[39].z=y;
      SpaceGroupElement[40].x=0.25-z;        SpaceGroupElement[40].y=0.25-x;        SpaceGroupElement[40].z=0.25-y;
      SpaceGroupElement[41].x=0.25+z;        SpaceGroupElement[41].y=-0.25-x;       SpaceGroupElement[41].z=-0.25+y;
      SpaceGroupElement[42].x=0.5+z;         SpaceGroupElement[42].y=-0.5-y;        SpaceGroupElement[42].z=-x;
      SpaceGroupElement[43].x=-0.25+z;       SpaceGroupElement[43].y=0.25+x;        SpaceGroupElement[43].z=-0.25-y;
      SpaceGroupElement[44].x=z;             SpaceGroupElement[44].y=y;             SpaceGroupElement[44].z=x;
      SpaceGroupElement[45].x=0.25-z;        SpaceGroupElement[45].y=-0.25+x;       SpaceGroupElement[45].z=-0.25+y;
      SpaceGroupElement[46].x=-0.5-z;        SpaceGroupElement[46].y=0.5+y;         SpaceGroupElement[46].z=-x;
      SpaceGroupElement[47].x=-z;            SpaceGroupElement[47].y=-0.5-y;        SpaceGroupElement[47].z=0.5+x;
      SpaceGroupElement[48].x=x;             SpaceGroupElement[48].y=-0.5+y;        SpaceGroupElement[48].z=-0.5+z;
      SpaceGroupElement[49].x=-0.75+y;       SpaceGroupElement[49].y=0.25-x;        SpaceGroupElement[49].z=-0.75+z;
      SpaceGroupElement[50].x=-x;            SpaceGroupElement[50].y=-y;            SpaceGroupElement[50].z=z;
      SpaceGroupElement[51].x=0.75-y;        SpaceGroupElement[51].y=-0.75+x;       SpaceGroupElement[51].z=-0.25+z;
      SpaceGroupElement[52].x=x;             SpaceGroupElement[52].y=0.5-y;         SpaceGroupElement[52].z=0.5-z;
      SpaceGroupElement[53].x=-0.75+y;       SpaceGroupElement[53].y=-0.25+x;       SpaceGroupElement[53].z=0.75-z;
      SpaceGroupElement[54].x=-x;            SpaceGroupElement[54].y=y;             SpaceGroupElement[54].z=-z;
      SpaceGroupElement[55].x=0.75-y;        SpaceGroupElement[55].y=0.75-x;        SpaceGroupElement[55].z=0.25-z;
      SpaceGroupElement[56].x=-0.5+y;        SpaceGroupElement[56].y=-0.5+z;        SpaceGroupElement[56].z=x;
      SpaceGroupElement[57].x=0.25-x;        SpaceGroupElement[57].y=-0.75+z;       SpaceGroupElement[57].z=-0.75+y;
      SpaceGroupElement[58].x=-y;            SpaceGroupElement[58].y=z;             SpaceGroupElement[58].z=-x;
      SpaceGroupElement[59].x=-0.75+x;       SpaceGroupElement[59].y=-0.25+z;       SpaceGroupElement[59].z=0.75-y;
      SpaceGroupElement[60].x=0.5-y;         SpaceGroupElement[60].y=0.5-z;         SpaceGroupElement[60].z=x;
      SpaceGroupElement[61].x=-0.25+x;       SpaceGroupElement[61].y=0.75-z;        SpaceGroupElement[61].z=-0.75+y;
      SpaceGroupElement[62].x=y;             SpaceGroupElement[62].y=-z;            SpaceGroupElement[62].z=-x;
      SpaceGroupElement[63].x=0.75-x;        SpaceGroupElement[63].y=0.25-z;        SpaceGroupElement[63].z=0.75-y;
      SpaceGroupElement[64].x=-0.5+z;        SpaceGroupElement[64].y=x;             SpaceGroupElement[64].z=-0.5+y;
      SpaceGroupElement[65].x=-z;            SpaceGroupElement[65].y=-0.5+x;        SpaceGroupElement[65].z=0.5-y;
      SpaceGroupElement[66].x=0.25-z;        SpaceGroupElement[66].y=-0.25+y;       SpaceGroupElement[66].z=-0.25+x;
      SpaceGroupElement[67].x=0.5-z;         SpaceGroupElement[67].y=0.5-x;         SpaceGroupElement[67].z=y;
      SpaceGroupElement[68].x=0.75-z;        SpaceGroupElement[68].y=0.75-y;        SpaceGroupElement[68].z=0.25-x;
      SpaceGroupElement[69].x=-0.5+z;        SpaceGroupElement[69].y=-x;            SpaceGroupElement[69].z=0.5-y;
      SpaceGroupElement[70].x=-0.25+z;       SpaceGroupElement[70].y=0.25-y;        SpaceGroupElement[70].z=-0.25+x;
      SpaceGroupElement[71].x=-0.75+z;       SpaceGroupElement[71].y=-0.25+y;       SpaceGroupElement[71].z=0.75-x;
      SpaceGroupElement[72].x=0.25-x;        SpaceGroupElement[72].y=0.75-y;        SpaceGroupElement[72].z=0.75-z;
      SpaceGroupElement[73].x=0.5-y;         SpaceGroupElement[73].y=x;             SpaceGroupElement[73].z=0.5-z;
      SpaceGroupElement[74].x=-0.25+x;       SpaceGroupElement[74].y=-0.25+y;       SpaceGroupElement[74].z=0.25-z;
      SpaceGroupElement[75].x=-0.5+y;        SpaceGroupElement[75].y=-0.5-x;        SpaceGroupElement[75].z=-z;
      SpaceGroupElement[76].x=0.25-x;        SpaceGroupElement[76].y=-0.75+y;       SpaceGroupElement[76].z=-0.75+z;
      SpaceGroupElement[77].x=0.5-y;         SpaceGroupElement[77].y=-x;            SpaceGroupElement[77].z=-0.5+z;
      SpaceGroupElement[78].x=-0.25+x;       SpaceGroupElement[78].y=0.25-y;        SpaceGroupElement[78].z=-0.25+z;
      SpaceGroupElement[79].x=-0.5+y;        SpaceGroupElement[79].y=0.5+x;         SpaceGroupElement[79].z=z;
      SpaceGroupElement[80].x=0.75-y;        SpaceGroupElement[80].y=0.75-z;        SpaceGroupElement[80].z=0.25-x;
      SpaceGroupElement[81].x=x;             SpaceGroupElement[81].y=0.5-z;         SpaceGroupElement[81].z=0.5-y;
      SpaceGroupElement[82].x=-0.25+y;       SpaceGroupElement[82].y=0.25-z;        SpaceGroupElement[82].z=-0.25+x;
      SpaceGroupElement[83].x=-0.5-x;        SpaceGroupElement[83].y=-z;            SpaceGroupElement[83].z=-0.5+y;
      SpaceGroupElement[84].x=-0.75+y;       SpaceGroupElement[84].y=-0.75+z;       SpaceGroupElement[84].z=0.25-x;
      SpaceGroupElement[85].x=-x;            SpaceGroupElement[85].y=-0.5+z;        SpaceGroupElement[85].z=0.5-y;
      SpaceGroupElement[86].x=0.25-y;        SpaceGroupElement[86].y=-0.25+z;       SpaceGroupElement[86].z=-0.25+x;
      SpaceGroupElement[87].x=0.5+x;         SpaceGroupElement[87].y=z;             SpaceGroupElement[87].z=-0.5+y;
      SpaceGroupElement[88].x=0.75-z;        SpaceGroupElement[88].y=0.25-x;        SpaceGroupElement[88].z=0.75-y;
      SpaceGroupElement[89].x=-0.25+z;       SpaceGroupElement[89].y=-0.25-x;       SpaceGroupElement[89].z=-0.75+y;
      SpaceGroupElement[90].x=z;             SpaceGroupElement[90].y=-y;            SpaceGroupElement[90].z=-x;
      SpaceGroupElement[91].x=-0.75+z;       SpaceGroupElement[91].y=0.25+x;        SpaceGroupElement[91].z=0.25-y;
      SpaceGroupElement[92].x=-0.5+z;        SpaceGroupElement[92].y=-0.5+y;        SpaceGroupElement[92].z=x;
      SpaceGroupElement[93].x=0.75-z;        SpaceGroupElement[93].y=-0.25+x;       SpaceGroupElement[93].z=-0.75+y;
      SpaceGroupElement[94].x=-z;            SpaceGroupElement[94].y=y;             SpaceGroupElement[94].z=-x;
      SpaceGroupElement[95].x=0.5-z;         SpaceGroupElement[95].y=-y;            SpaceGroupElement[95].z=0.5+x;
      SpaceGroupElement[96].x=-0.5+x;        SpaceGroupElement[96].y=y;             SpaceGroupElement[96].z=-0.5+z;
      SpaceGroupElement[97].x=-0.25+y;       SpaceGroupElement[97].y=0.75-x;        SpaceGroupElement[97].z=-0.75+z;
      SpaceGroupElement[98].x=0.5-x;         SpaceGroupElement[98].y=0.5-y;         SpaceGroupElement[98].z=z;
      SpaceGroupElement[99].x=0.25-y;        SpaceGroupElement[99].y=-0.25+x;       SpaceGroupElement[99].z=-0.25+z;
      SpaceGroupElement[100].x=-0.5+x;       SpaceGroupElement[100].y=-y;           SpaceGroupElement[100].z=0.5-z;
      SpaceGroupElement[101].x=-0.25+y;      SpaceGroupElement[101].y=-0.75+x;      SpaceGroupElement[101].z=0.75-z;
      SpaceGroupElement[102].x=0.5-x;        SpaceGroupElement[102].y=-0.5+y;       SpaceGroupElement[102].z=-z;
      SpaceGroupElement[103].x=0.25-y;       SpaceGroupElement[103].y=0.25-x;       SpaceGroupElement[103].z=0.25-z;
      SpaceGroupElement[104].x=y;            SpaceGroupElement[104].y=-0.5+z;       SpaceGroupElement[104].z=-0.5+x;
      SpaceGroupElement[105].x=0.75-x;       SpaceGroupElement[105].y=-0.75+z;      SpaceGroupElement[105].z=-0.25+y;
      SpaceGroupElement[106].x=0.5-y;        SpaceGroupElement[106].y=z;            SpaceGroupElement[106].z=0.5-x;
      SpaceGroupElement[107].x=-0.25+x;      SpaceGroupElement[107].y=-0.25+z;      SpaceGroupElement[107].z=0.25-y;
      SpaceGroupElement[108].x=-y;           SpaceGroupElement[108].y=0.5-z;        SpaceGroupElement[108].z=-0.5+x;
      SpaceGroupElement[109].x=-0.75+x;      SpaceGroupElement[109].y=0.75-z;       SpaceGroupElement[109].z=-0.25+y;
      SpaceGroupElement[110].x=-0.5+y;       SpaceGroupElement[110].y=-z;           SpaceGroupElement[110].z=0.5-x;
      SpaceGroupElement[111].x=0.25-x;       SpaceGroupElement[111].y=0.25-z;       SpaceGroupElement[111].z=0.25-y;
      SpaceGroupElement[112].x=-0.5+z;       SpaceGroupElement[112].y=-0.5+x;       SpaceGroupElement[112].z=y;
      SpaceGroupElement[113].x=-z;           SpaceGroupElement[113].y=x;            SpaceGroupElement[113].z=-y;
      SpaceGroupElement[114].x=0.25-z;       SpaceGroupElement[114].y=-0.75+y;      SpaceGroupElement[114].z=-0.75+x;
      SpaceGroupElement[115].x=0.5-z;        SpaceGroupElement[115].y=-x;           SpaceGroupElement[115].z=-0.5+y;
      SpaceGroupElement[116].x=0.75-z;       SpaceGroupElement[116].y=0.25-y;       SpaceGroupElement[116].z=0.75-x;
      SpaceGroupElement[117].x=-0.5+z;       SpaceGroupElement[117].y=0.5-x;        SpaceGroupElement[117].z=-y;
      SpaceGroupElement[118].x=-0.25+z;      SpaceGroupElement[118].y=0.75-y;       SpaceGroupElement[118].z=-0.75+x;
      SpaceGroupElement[119].x=-0.75+z;      SpaceGroupElement[119].y=-0.75+y;      SpaceGroupElement[119].z=0.25-x;
      SpaceGroupElement[120].x=0.75-x;       SpaceGroupElement[120].y=0.25-y;       SpaceGroupElement[120].z=0.75-z;
      SpaceGroupElement[121].x=-y;           SpaceGroupElement[121].y=-0.5+x;       SpaceGroupElement[121].z=0.5-z;
      SpaceGroupElement[122].x=-0.75+x;      SpaceGroupElement[122].y=0.25+y;       SpaceGroupElement[122].z=0.25-z;
      SpaceGroupElement[123].x=y;            SpaceGroupElement[123].y=-x;           SpaceGroupElement[123].z=-z;
      SpaceGroupElement[124].x=0.75-x;       SpaceGroupElement[124].y=-0.25+y;      SpaceGroupElement[124].z=-0.75+z;
      SpaceGroupElement[125].x=-y;           SpaceGroupElement[125].y=0.5-x;        SpaceGroupElement[125].z=-0.5+z;
      SpaceGroupElement[126].x=-0.75+x;      SpaceGroupElement[126].y=-0.25-y;      SpaceGroupElement[126].z=-0.25+z;
      SpaceGroupElement[127].x=y;            SpaceGroupElement[127].y=x;            SpaceGroupElement[127].z=z;
      SpaceGroupElement[128].x=0.25-y;       SpaceGroupElement[128].y=0.75-z;       SpaceGroupElement[128].z=0.75-x;
      SpaceGroupElement[129].x=-0.5+x;       SpaceGroupElement[129].y=0.5-z;        SpaceGroupElement[129].z=-y;
      SpaceGroupElement[130].x=0.25+y;       SpaceGroupElement[130].y=0.25-z;       SpaceGroupElement[130].z=-0.75+x;
      SpaceGroupElement[131].x=-x;           SpaceGroupElement[131].y=-z;           SpaceGroupElement[131].z=y;
      SpaceGroupElement[132].x=-0.25+y;      SpaceGroupElement[132].y=-0.75+z;      SpaceGroupElement[132].z=0.75-x;
      SpaceGroupElement[133].x=0.5-x;        SpaceGroupElement[133].y=-0.5+z;       SpaceGroupElement[133].z=-y;
      SpaceGroupElement[134].x=-0.25-y;      SpaceGroupElement[134].y=-0.25+z;      SpaceGroupElement[134].z=-0.75+x;
      SpaceGroupElement[135].x=x;            SpaceGroupElement[135].y=z;            SpaceGroupElement[135].z=y;
      SpaceGroupElement[136].x=0.75-z;       SpaceGroupElement[136].y=0.75-x;       SpaceGroupElement[136].z=0.25-y;
      SpaceGroupElement[137].x=-0.25+z;      SpaceGroupElement[137].y=0.25-x;       SpaceGroupElement[137].z=-0.25+y;
      SpaceGroupElement[138].x=z;            SpaceGroupElement[138].y=-0.5-y;       SpaceGroupElement[138].z=0.5-x;
      SpaceGroupElement[139].x=-0.75+z;      SpaceGroupElement[139].y=-0.25+x;      SpaceGroupElement[139].z=-0.25-y;
      SpaceGroupElement[140].x=-0.5+z;       SpaceGroupElement[140].y=y;            SpaceGroupElement[140].z=-0.5+x;
      SpaceGroupElement[141].x=0.75-z;       SpaceGroupElement[141].y=-0.75+x;      SpaceGroupElement[141].z=-0.25+y;
      SpaceGroupElement[142].x=-z;           SpaceGroupElement[142].y=0.5+y;        SpaceGroupElement[142].z=0.5-x;
      SpaceGroupElement[143].x=0.5-z;        SpaceGroupElement[143].y=-0.5-y;       SpaceGroupElement[143].z=x;
      SpaceGroupElement[144].x=-0.5+x;       SpaceGroupElement[144].y=-0.5+y;       SpaceGroupElement[144].z=z;
      SpaceGroupElement[145].x=-0.75+y;      SpaceGroupElement[145].y=0.75-x;       SpaceGroupElement[145].z=-0.25+z;
      SpaceGroupElement[146].x=0.5-x;        SpaceGroupElement[146].y=-y;           SpaceGroupElement[146].z=-0.5+z;
      SpaceGroupElement[147].x=0.75-y;       SpaceGroupElement[147].y=-0.25+x;      SpaceGroupElement[147].z=-0.75+z;
      SpaceGroupElement[148].x=-0.5+x;       SpaceGroupElement[148].y=0.5-y;        SpaceGroupElement[148].z=-z;
      SpaceGroupElement[149].x=-0.75+y;      SpaceGroupElement[149].y=-0.75+x;      SpaceGroupElement[149].z=0.25-z;
      SpaceGroupElement[150].x=0.5-x;        SpaceGroupElement[150].y=y;            SpaceGroupElement[150].z=0.5-z;
      SpaceGroupElement[151].x=0.75-y;       SpaceGroupElement[151].y=0.25-x;       SpaceGroupElement[151].z=0.75-z;
      SpaceGroupElement[152].x=-0.5+y;       SpaceGroupElement[152].y=z;            SpaceGroupElement[152].z=-0.5+x;
      SpaceGroupElement[153].x=0.75-x;       SpaceGroupElement[153].y=-0.25+z;      SpaceGroupElement[153].z=-0.75+y;
      SpaceGroupElement[154].x=-y;           SpaceGroupElement[154].y=-0.5+z;       SpaceGroupElement[154].z=0.5-x;
      SpaceGroupElement[155].x=-0.25+x;      SpaceGroupElement[155].y=-0.75+z;      SpaceGroupElement[155].z=0.75-y;
      SpaceGroupElement[156].x=0.5-y;        SpaceGroupElement[156].y=-z;           SpaceGroupElement[156].z=-0.5+x;
      SpaceGroupElement[157].x=-0.75+x;      SpaceGroupElement[157].y=0.25-z;       SpaceGroupElement[157].z=-0.75+y;
      SpaceGroupElement[158].x=y;            SpaceGroupElement[158].y=0.5-z;        SpaceGroupElement[158].z=0.5-x;
      SpaceGroupElement[159].x=0.25-x;       SpaceGroupElement[159].y=0.75-z;       SpaceGroupElement[159].z=0.75-y;
      SpaceGroupElement[160].x=z;            SpaceGroupElement[160].y=-0.5+x;       SpaceGroupElement[160].z=-0.5+y;
      SpaceGroupElement[161].x=0.5-z;        SpaceGroupElement[161].y=x;            SpaceGroupElement[161].z=0.5-y;
      SpaceGroupElement[162].x=0.75-z;       SpaceGroupElement[162].y=-0.25+y;      SpaceGroupElement[162].z=-0.75+x;
      SpaceGroupElement[163].x=-z;           SpaceGroupElement[163].y=-x;           SpaceGroupElement[163].z=y;
      SpaceGroupElement[164].x=0.25-z;       SpaceGroupElement[164].y=0.75-y;       SpaceGroupElement[164].z=0.75-x;
      SpaceGroupElement[165].x=z;            SpaceGroupElement[165].y=0.5-x;        SpaceGroupElement[165].z=0.5-y;
      SpaceGroupElement[166].x=-0.75+z;      SpaceGroupElement[166].y=0.25-y;       SpaceGroupElement[166].z=-0.75+x;
      SpaceGroupElement[167].x=-0.25+z;      SpaceGroupElement[167].y=-0.25+y;      SpaceGroupElement[167].z=0.25-x;
      SpaceGroupElement[168].x=0.75-x;       SpaceGroupElement[168].y=0.75-y;       SpaceGroupElement[168].z=0.25-z;
      SpaceGroupElement[169].x=0.5-y;        SpaceGroupElement[169].y=-0.5+x;       SpaceGroupElement[169].z=-z;
      SpaceGroupElement[170].x=-0.75+x;      SpaceGroupElement[170].y=-0.25+y;      SpaceGroupElement[170].z=-0.25-z;
      SpaceGroupElement[171].x=-0.5+y;       SpaceGroupElement[171].y=-x;           SpaceGroupElement[171].z=-0.5-z;
      SpaceGroupElement[172].x=0.75-x;       SpaceGroupElement[172].y=-0.75+y;      SpaceGroupElement[172].z=-0.25+z;
      SpaceGroupElement[173].x=0.5-y;        SpaceGroupElement[173].y=0.5-x;        SpaceGroupElement[173].z=z;
      SpaceGroupElement[174].x=-0.75+x;      SpaceGroupElement[174].y=0.25-y;       SpaceGroupElement[174].z=0.25+z;
      SpaceGroupElement[175].x=-0.5+y;       SpaceGroupElement[175].y=x;            SpaceGroupElement[175].z=0.5+z;
      SpaceGroupElement[176].x=0.75-y;       SpaceGroupElement[176].y=0.25-z;       SpaceGroupElement[176].z=0.75-x;
      SpaceGroupElement[177].x=-0.5+x;       SpaceGroupElement[177].y=-z;           SpaceGroupElement[177].z=0.5-y;
      SpaceGroupElement[178].x=-0.25+y;      SpaceGroupElement[178].y=-0.25-z;      SpaceGroupElement[178].z=-0.75+x;
      SpaceGroupElement[179].x=-x;           SpaceGroupElement[179].y=-0.5-z;       SpaceGroupElement[179].z=-0.5+y;
      SpaceGroupElement[180].x=-0.75+y;      SpaceGroupElement[180].y=-0.25+z;      SpaceGroupElement[180].z=0.75-x;
      SpaceGroupElement[181].x=0.5-x;        SpaceGroupElement[181].y=z;            SpaceGroupElement[181].z=0.5-y;
      SpaceGroupElement[182].x=0.25-y;       SpaceGroupElement[182].y=0.25+z;       SpaceGroupElement[182].z=-0.75+x;
      SpaceGroupElement[183].x=x;            SpaceGroupElement[183].y=0.5+z;        SpaceGroupElement[183].z=-0.5+y;
      SpaceGroupElement[184].x=0.25-z;       SpaceGroupElement[184].y=0.75-x;       SpaceGroupElement[184].z=0.75-y;
      SpaceGroupElement[185].x=0.25+z;       SpaceGroupElement[185].y=0.25-x;       SpaceGroupElement[185].z=-0.75+y;
      SpaceGroupElement[186].x=0.5+z;        SpaceGroupElement[186].y=-y;           SpaceGroupElement[186].z=0.5-x;
      SpaceGroupElement[187].x=-0.25+z;      SpaceGroupElement[187].y=-0.25+x;      SpaceGroupElement[187].z=0.25-y;
      SpaceGroupElement[188].x=z;            SpaceGroupElement[188].y=-0.5+y;       SpaceGroupElement[188].z=-0.5+x;
      SpaceGroupElement[189].x=0.25-z;       SpaceGroupElement[189].y=-0.75+x;      SpaceGroupElement[189].z=-0.75+y;
      SpaceGroupElement[190].x=-0.5-z;       SpaceGroupElement[190].y=y;            SpaceGroupElement[190].z=0.5-x;
      SpaceGroupElement[191].x=-z;           SpaceGroupElement[191].y=-y;           SpaceGroupElement[191].z=x;
      break;
    case 526: // [227] F d -3 m Origin choice 2 (-F 4vw 2vw 3)      e.g. MIL-100,101
      SpaceGroupSize=192;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.25+y;        SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=0.75-x;         SpaceGroupElement[2].y=0.25-y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=-0.75+x;        SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=0.25-y;         SpaceGroupElement[4].z=0.25-z;
      SpaceGroupElement[5].x=-0.25+y;        SpaceGroupElement[5].y=-0.75+x;        SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.75-x;         SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=0.75-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=-0.25+z;        SpaceGroupElement[9].z=-0.25+y;
      SpaceGroupElement[10].x=0.25-y;        SpaceGroupElement[10].y=-0.5+z;        SpaceGroupElement[10].z=0.75-x;
      SpaceGroupElement[11].x=-0.75+x;       SpaceGroupElement[11].y=-0.75+z;       SpaceGroupElement[11].z=-y;
      SpaceGroupElement[12].x=0.25-y;        SpaceGroupElement[12].y=0.25-z;        SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=-0.75+x;       SpaceGroupElement[13].y=0.5-z;         SpaceGroupElement[13].z=-0.25+y;
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=0.75-z;        SpaceGroupElement[14].z=0.75-x;
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=0.25-z;        SpaceGroupElement[17].y=-0.5+x;        SpaceGroupElement[17].z=0.75-y;
      SpaceGroupElement[18].x=0.5-z;         SpaceGroupElement[18].y=-0.75+y;       SpaceGroupElement[18].z=-0.25+x;
      SpaceGroupElement[19].x=0.75-z;        SpaceGroupElement[19].y=0.25-x;        SpaceGroupElement[19].z=-0.5+y;
      SpaceGroupElement[20].x=0.5-z;         SpaceGroupElement[20].y=0.5-y;         SpaceGroupElement[20].z=-x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=0.25-x;        SpaceGroupElement[21].z=0.25-y;
      SpaceGroupElement[22].x=-0.25+z;       SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=-0.25+x;
      SpaceGroupElement[23].x=-0.75+z;       SpaceGroupElement[23].y=-0.25+y;       SpaceGroupElement[23].z=0.5-x;
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;
      SpaceGroupElement[25].x=-0.25-y;       SpaceGroupElement[25].y=x;             SpaceGroupElement[25].z=-0.25-z;
      SpaceGroupElement[26].x=0.75+x;        SpaceGroupElement[26].y=0.25+y;        SpaceGroupElement[26].z=-0.5-z;
      SpaceGroupElement[27].x=y;             SpaceGroupElement[27].y=-0.75-x;       SpaceGroupElement[27].z=-0.75-z;
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=0.25+y;        SpaceGroupElement[28].z=0.25+z;
      SpaceGroupElement[29].x=-0.25-y;       SpaceGroupElement[29].y=-0.75-x;       SpaceGroupElement[29].z=0.5+z;
      SpaceGroupElement[30].x=0.75+x;        SpaceGroupElement[30].y=-y;            SpaceGroupElement[30].z=0.75+z;
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=z;
      SpaceGroupElement[32].x=-y;            SpaceGroupElement[32].y=-z;            SpaceGroupElement[32].z=-x;
      SpaceGroupElement[33].x=x;             SpaceGroupElement[33].y=-0.25-z;       SpaceGroupElement[33].z=-0.25-y;
      SpaceGroupElement[34].x=0.25+y;        SpaceGroupElement[34].y=-0.5-z;        SpaceGroupElement[34].z=0.75+x;
      SpaceGroupElement[35].x=-0.75-x;       SpaceGroupElement[35].y=-0.75-z;       SpaceGroupElement[35].z=y;
      SpaceGroupElement[36].x=0.25+y;        SpaceGroupElement[36].y=0.25+z;        SpaceGroupElement[36].z=-x;
      SpaceGroupElement[37].x=-0.75-x;       SpaceGroupElement[37].y=0.5+z;         SpaceGroupElement[37].z=-0.25-y;
      SpaceGroupElement[38].x=-y;            SpaceGroupElement[38].y=0.75+z;        SpaceGroupElement[38].z=0.75+x;
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=z;             SpaceGroupElement[39].z=y;
      SpaceGroupElement[40].x=-z;            SpaceGroupElement[40].y=-x;            SpaceGroupElement[40].z=-y;
      SpaceGroupElement[41].x=0.25+z;        SpaceGroupElement[41].y=-0.5-x;        SpaceGroupElement[41].z=0.75+y;
      SpaceGroupElement[42].x=0.5+z;         SpaceGroupElement[42].y=-0.75-y;       SpaceGroupElement[42].z=-0.25-x;
      SpaceGroupElement[43].x=0.75+z;        SpaceGroupElement[43].y=0.25+x;        SpaceGroupElement[43].z=-0.5-y;
      SpaceGroupElement[44].x=0.5+z;         SpaceGroupElement[44].y=0.5+y;         SpaceGroupElement[44].z=x;
      SpaceGroupElement[45].x=-z;            SpaceGroupElement[45].y=0.25+x;        SpaceGroupElement[45].z=0.25+y;
      SpaceGroupElement[46].x=-0.25-z;       SpaceGroupElement[46].y=y;             SpaceGroupElement[46].z=-0.25-x;
      SpaceGroupElement[47].x=-0.75-z;       SpaceGroupElement[47].y=-0.25-y;       SpaceGroupElement[47].z=0.5+x;
      SpaceGroupElement[48].x=x;             SpaceGroupElement[48].y=-0.5+y;        SpaceGroupElement[48].z=-0.5+z;
      SpaceGroupElement[49].x=-0.75+y;       SpaceGroupElement[49].y=-x;            SpaceGroupElement[49].z=-0.75+z;
      SpaceGroupElement[50].x=0.75-x;        SpaceGroupElement[50].y=0.75-y;        SpaceGroupElement[50].z=z;
      SpaceGroupElement[51].x=0.5-y;         SpaceGroupElement[51].y=-0.75+x;       SpaceGroupElement[51].z=-0.25+z;
      SpaceGroupElement[52].x=x;             SpaceGroupElement[52].y=0.75-y;        SpaceGroupElement[52].z=0.75-z;
      SpaceGroupElement[53].x=-0.75+y;       SpaceGroupElement[53].y=-0.75+x;       SpaceGroupElement[53].z=-z;
      SpaceGroupElement[54].x=0.75-x;        SpaceGroupElement[54].y=-0.5+y;        SpaceGroupElement[54].z=0.25-z;
      SpaceGroupElement[55].x=0.5-y;         SpaceGroupElement[55].y=-x;            SpaceGroupElement[55].z=0.5-z;
      SpaceGroupElement[56].x=-0.5+y;        SpaceGroupElement[56].y=-0.5+z;        SpaceGroupElement[56].z=x;
      SpaceGroupElement[57].x=-x;            SpaceGroupElement[57].y=-0.75+z;       SpaceGroupElement[57].z=-0.75+y;
      SpaceGroupElement[58].x=0.75-y;        SpaceGroupElement[58].y=z;             SpaceGroupElement[58].z=0.75-x;
      SpaceGroupElement[59].x=-0.75+x;       SpaceGroupElement[59].y=-0.25+z;       SpaceGroupElement[59].z=0.5-y;
      SpaceGroupElement[60].x=0.75-y;        SpaceGroupElement[60].y=0.75-z;        SpaceGroupElement[60].z=x;
      SpaceGroupElement[61].x=-0.75+x;       SpaceGroupElement[61].y=-z;            SpaceGroupElement[61].z=-0.75+y;
      SpaceGroupElement[62].x=-0.5+y;        SpaceGroupElement[62].y=0.25-z;        SpaceGroupElement[62].z=0.75-x;
      SpaceGroupElement[63].x=-x;            SpaceGroupElement[63].y=0.5-z;         SpaceGroupElement[63].z=0.5-y;
      SpaceGroupElement[64].x=-0.5+z;        SpaceGroupElement[64].y=x;             SpaceGroupElement[64].z=-0.5+y;
      SpaceGroupElement[65].x=0.75-z;        SpaceGroupElement[65].y=-0.5+x;        SpaceGroupElement[65].z=0.25-y;
      SpaceGroupElement[66].x=-z;            SpaceGroupElement[66].y=-0.25+y;       SpaceGroupElement[66].z=-0.25+x;
      SpaceGroupElement[67].x=0.25-z;        SpaceGroupElement[67].y=0.25-x;        SpaceGroupElement[67].z=y;
      SpaceGroupElement[68].x=-z;            SpaceGroupElement[68].y=-y;            SpaceGroupElement[68].z=-x;
      SpaceGroupElement[69].x=-0.5+z;        SpaceGroupElement[69].y=0.25-x;        SpaceGroupElement[69].z=0.75-y;
      SpaceGroupElement[70].x=-0.75+z;       SpaceGroupElement[70].y=0.5-y;         SpaceGroupElement[70].z=-0.25+x;
      SpaceGroupElement[71].x=-0.25+z;       SpaceGroupElement[71].y=-0.75+y;       SpaceGroupElement[71].z=0.5-x;
      SpaceGroupElement[72].x=-x;            SpaceGroupElement[72].y=0.5-y;         SpaceGroupElement[72].z=0.5-z;
      SpaceGroupElement[73].x=0.25-y;        SpaceGroupElement[73].y=x;             SpaceGroupElement[73].z=0.25-z;
      SpaceGroupElement[74].x=0.75+x;        SpaceGroupElement[74].y=-0.25+y;       SpaceGroupElement[74].z=-z;
      SpaceGroupElement[75].x=-0.5+y;        SpaceGroupElement[75].y=-0.75-x;       SpaceGroupElement[75].z=-0.25-z;
      SpaceGroupElement[76].x=-x;            SpaceGroupElement[76].y=-0.25+y;       SpaceGroupElement[76].z=-0.25+z;
      SpaceGroupElement[77].x=0.25-y;        SpaceGroupElement[77].y=-0.75-x;       SpaceGroupElement[77].z=z;
      SpaceGroupElement[78].x=0.75+x;        SpaceGroupElement[78].y=0.5-y;         SpaceGroupElement[78].z=0.25+z;
      SpaceGroupElement[79].x=-0.5+y;        SpaceGroupElement[79].y=x;             SpaceGroupElement[79].z=-0.5+z;
      SpaceGroupElement[80].x=0.5-y;         SpaceGroupElement[80].y=0.5-z;         SpaceGroupElement[80].z=-x;
      SpaceGroupElement[81].x=x;             SpaceGroupElement[81].y=0.25-z;        SpaceGroupElement[81].z=0.25-y;
      SpaceGroupElement[82].x=-0.25+y;       SpaceGroupElement[82].y=-z;            SpaceGroupElement[82].z=0.75+x;
      SpaceGroupElement[83].x=-0.75-x;       SpaceGroupElement[83].y=-0.25-z;       SpaceGroupElement[83].z=-0.5+y;
      SpaceGroupElement[84].x=-0.25+y;       SpaceGroupElement[84].y=-0.25+z;       SpaceGroupElement[84].z=-x;
      SpaceGroupElement[85].x=-0.75-x;       SpaceGroupElement[85].y=z;             SpaceGroupElement[85].z=0.25-y;
      SpaceGroupElement[86].x=0.5-y;         SpaceGroupElement[86].y=0.25+z;        SpaceGroupElement[86].z=0.75+x;
      SpaceGroupElement[87].x=x;             SpaceGroupElement[87].y=-0.5+z;        SpaceGroupElement[87].z=-0.5+y;
      SpaceGroupElement[88].x=0.5-z;         SpaceGroupElement[88].y=-x;            SpaceGroupElement[88].z=0.5-y;
      SpaceGroupElement[89].x=-0.25+z;       SpaceGroupElement[89].y=-0.5-x;        SpaceGroupElement[89].z=0.25+y;
      SpaceGroupElement[90].x=z;             SpaceGroupElement[90].y=-0.25-y;       SpaceGroupElement[90].z=-0.25-x;
      SpaceGroupElement[91].x=0.25+z;        SpaceGroupElement[91].y=0.25+x;        SpaceGroupElement[91].z=-y;
      SpaceGroupElement[92].x=z;             SpaceGroupElement[92].y=y;             SpaceGroupElement[92].z=x;
      SpaceGroupElement[93].x=0.5-z;         SpaceGroupElement[93].y=0.25+x;        SpaceGroupElement[93].z=-0.25+y;
      SpaceGroupElement[94].x=0.25-z;        SpaceGroupElement[94].y=-0.5+y;        SpaceGroupElement[94].z=-0.25-x;
      SpaceGroupElement[95].x=-0.25-z;       SpaceGroupElement[95].y=0.25-y;        SpaceGroupElement[95].z=0.5+x;
      SpaceGroupElement[96].x=-0.5+x;        SpaceGroupElement[96].y=y;             SpaceGroupElement[96].z=-0.5+z;
      SpaceGroupElement[97].x=-0.25+y;       SpaceGroupElement[97].y=0.5-x;         SpaceGroupElement[97].z=-0.75+z;
      SpaceGroupElement[98].x=0.25-x;        SpaceGroupElement[98].y=0.25-y;        SpaceGroupElement[98].z=z;
      SpaceGroupElement[99].x=-y;            SpaceGroupElement[99].y=-0.25+x;       SpaceGroupElement[99].z=-0.25+z;
      SpaceGroupElement[100].x=-0.5+x;       SpaceGroupElement[100].y=0.25-y;       SpaceGroupElement[100].z=0.75-z;
      SpaceGroupElement[101].x=-0.25+y;      SpaceGroupElement[101].y=-0.25+x;      SpaceGroupElement[101].z=-z;
      SpaceGroupElement[102].x=0.25-x;       SpaceGroupElement[102].y=y;            SpaceGroupElement[102].z=0.25-z;
      SpaceGroupElement[103].x=-y;           SpaceGroupElement[103].y=0.5-x;        SpaceGroupElement[103].z=0.5-z;
      SpaceGroupElement[104].x=y;            SpaceGroupElement[104].y=-0.5+z;       SpaceGroupElement[104].z=-0.5+x;
      SpaceGroupElement[105].x=0.5-x;        SpaceGroupElement[105].y=-0.75+z;      SpaceGroupElement[105].z=-0.25+y;
      SpaceGroupElement[106].x=0.25-y;       SpaceGroupElement[106].y=z;            SpaceGroupElement[106].z=0.25-x;
      SpaceGroupElement[107].x=-0.25+x;      SpaceGroupElement[107].y=-0.25+z;      SpaceGroupElement[107].z=-y;
      SpaceGroupElement[108].x=0.25-y;       SpaceGroupElement[108].y=0.75-z;       SpaceGroupElement[108].z=-0.5+x;
      SpaceGroupElement[109].x=-0.25+x;      SpaceGroupElement[109].y=-z;           SpaceGroupElement[109].z=-0.25+y;
      SpaceGroupElement[110].x=y;            SpaceGroupElement[110].y=0.25-z;       SpaceGroupElement[110].z=0.25-x;
      SpaceGroupElement[111].x=0.5-x;        SpaceGroupElement[111].y=0.5-z;        SpaceGroupElement[111].z=-y;
      SpaceGroupElement[112].x=-0.5+z;       SpaceGroupElement[112].y=-0.5+x;       SpaceGroupElement[112].z=y;
      SpaceGroupElement[113].x=0.75-z;       SpaceGroupElement[113].y=x;            SpaceGroupElement[113].z=0.75-y;
      SpaceGroupElement[114].x=-z;           SpaceGroupElement[114].y=-0.75+y;      SpaceGroupElement[114].z=-0.75+x;
      SpaceGroupElement[115].x=0.25-z;       SpaceGroupElement[115].y=0.75-x;       SpaceGroupElement[115].z=-0.5+y;
      SpaceGroupElement[116].x=-z;           SpaceGroupElement[116].y=0.5-y;        SpaceGroupElement[116].z=0.5-x;
      SpaceGroupElement[117].x=-0.5+z;       SpaceGroupElement[117].y=0.75-x;       SpaceGroupElement[117].z=0.25-y;
      SpaceGroupElement[118].x=-0.75+z;      SpaceGroupElement[118].y=-y;           SpaceGroupElement[118].z=-0.75+x;
      SpaceGroupElement[119].x=-0.25+z;      SpaceGroupElement[119].y=-0.25+y;      SpaceGroupElement[119].z=-x;
      SpaceGroupElement[120].x=0.5-x;        SpaceGroupElement[120].y=-y;           SpaceGroupElement[120].z=0.5-z;
      SpaceGroupElement[121].x=-0.25-y;      SpaceGroupElement[121].y=-0.5+x;       SpaceGroupElement[121].z=0.25-z;
      SpaceGroupElement[122].x=0.25+x;       SpaceGroupElement[122].y=0.25+y;       SpaceGroupElement[122].z=-z;
      SpaceGroupElement[123].x=y;            SpaceGroupElement[123].y=-0.25-x;      SpaceGroupElement[123].z=-0.25-z;
      SpaceGroupElement[124].x=0.5-x;        SpaceGroupElement[124].y=0.25+y;       SpaceGroupElement[124].z=-0.25+z;
      SpaceGroupElement[125].x=-0.25-y;      SpaceGroupElement[125].y=-0.25-x;      SpaceGroupElement[125].z=z;
      SpaceGroupElement[126].x=0.25+x;       SpaceGroupElement[126].y=-y;           SpaceGroupElement[126].z=0.25+z;
      SpaceGroupElement[127].x=y;            SpaceGroupElement[127].y=-0.5+x;       SpaceGroupElement[127].z=-0.5+z;
      SpaceGroupElement[128].x=-y;           SpaceGroupElement[128].y=0.5-z;        SpaceGroupElement[128].z=0.5-x;
      SpaceGroupElement[129].x=-0.5+x;       SpaceGroupElement[129].y=0.25-z;       SpaceGroupElement[129].z=-0.25-y;
      SpaceGroupElement[130].x=0.25+y;       SpaceGroupElement[130].y=-z;           SpaceGroupElement[130].z=0.25+x;
      SpaceGroupElement[131].x=-0.25-x;      SpaceGroupElement[131].y=-0.25-z;      SpaceGroupElement[131].z=y;
      SpaceGroupElement[132].x=0.25+y;       SpaceGroupElement[132].y=-0.25+z;      SpaceGroupElement[132].z=0.5-x;
      SpaceGroupElement[133].x=-0.25-x;      SpaceGroupElement[133].y=z;            SpaceGroupElement[133].z=-0.25-y;
      SpaceGroupElement[134].x=-y;           SpaceGroupElement[134].y=0.25+z;       SpaceGroupElement[134].z=0.25+x;
      SpaceGroupElement[135].x=-0.5+x;       SpaceGroupElement[135].y=-0.5+z;       SpaceGroupElement[135].z=y;
      SpaceGroupElement[136].x=0.5-z;        SpaceGroupElement[136].y=0.5-x;        SpaceGroupElement[136].z=-y;
      SpaceGroupElement[137].x=-0.25+z;      SpaceGroupElement[137].y=-x;           SpaceGroupElement[137].z=0.75+y;
      SpaceGroupElement[138].x=z;            SpaceGroupElement[138].y=-0.75-y;      SpaceGroupElement[138].z=0.25-x;
      SpaceGroupElement[139].x=0.25+z;       SpaceGroupElement[139].y=-0.25+x;      SpaceGroupElement[139].z=-0.5-y;
      SpaceGroupElement[140].x=z;            SpaceGroupElement[140].y=0.5+y;        SpaceGroupElement[140].z=-0.5+x;
      SpaceGroupElement[141].x=0.5-z;        SpaceGroupElement[141].y=-0.25+x;      SpaceGroupElement[141].z=0.25+y;
      SpaceGroupElement[142].x=0.25-z;       SpaceGroupElement[142].y=y;            SpaceGroupElement[142].z=0.25-x;
      SpaceGroupElement[143].x=-0.25-z;      SpaceGroupElement[143].y=-0.25-y;      SpaceGroupElement[143].z=x;
      SpaceGroupElement[144].x=-0.5+x;       SpaceGroupElement[144].y=-0.5+y;       SpaceGroupElement[144].z=z;
      SpaceGroupElement[145].x=-0.75+y;      SpaceGroupElement[145].y=0.5-x;        SpaceGroupElement[145].z=-0.25+z;
      SpaceGroupElement[146].x=0.25-x;       SpaceGroupElement[146].y=0.75-y;       SpaceGroupElement[146].z=-0.5+z;
      SpaceGroupElement[147].x=0.5-y;        SpaceGroupElement[147].y=-0.25+x;      SpaceGroupElement[147].z=-0.75+z;
      SpaceGroupElement[148].x=-0.5+x;       SpaceGroupElement[148].y=0.75-y;       SpaceGroupElement[148].z=0.25-z;
      SpaceGroupElement[149].x=-0.75+y;      SpaceGroupElement[149].y=-0.25+x;      SpaceGroupElement[149].z=0.5-z;
      SpaceGroupElement[150].x=0.25-x;       SpaceGroupElement[150].y=-0.5+y;       SpaceGroupElement[150].z=0.75-z;
      SpaceGroupElement[151].x=0.5-y;        SpaceGroupElement[151].y=0.5-x;        SpaceGroupElement[151].z=-z;
      SpaceGroupElement[152].x=-0.5+y;       SpaceGroupElement[152].y=z;            SpaceGroupElement[152].z=-0.5+x;
      SpaceGroupElement[153].x=0.5-x;        SpaceGroupElement[153].y=-0.25+z;      SpaceGroupElement[153].z=-0.75+y;
      SpaceGroupElement[154].x=0.75-y;       SpaceGroupElement[154].y=-0.5+z;       SpaceGroupElement[154].z=0.25-x;
      SpaceGroupElement[155].x=-0.25+x;      SpaceGroupElement[155].y=-0.75+z;      SpaceGroupElement[155].z=0.5-y;
      SpaceGroupElement[156].x=0.75-y;       SpaceGroupElement[156].y=0.25-z;       SpaceGroupElement[156].z=-0.5+x;
      SpaceGroupElement[157].x=-0.25+x;      SpaceGroupElement[157].y=0.5-z;        SpaceGroupElement[157].z=-0.75+y;
      SpaceGroupElement[158].x=-0.5+y;       SpaceGroupElement[158].y=0.75-z;       SpaceGroupElement[158].z=0.25-x;
      SpaceGroupElement[159].x=0.5-x;        SpaceGroupElement[159].y=-z;           SpaceGroupElement[159].z=0.5-y;
      SpaceGroupElement[160].x=z;            SpaceGroupElement[160].y=-0.5+x;       SpaceGroupElement[160].z=-0.5+y;
      SpaceGroupElement[161].x=0.25-z;       SpaceGroupElement[161].y=x;            SpaceGroupElement[161].z=0.25-y;
      SpaceGroupElement[162].x=0.5-z;        SpaceGroupElement[162].y=-0.25+y;      SpaceGroupElement[162].z=-0.75+x;
      SpaceGroupElement[163].x=0.75-z;       SpaceGroupElement[163].y=0.75-x;       SpaceGroupElement[163].z=y;
      SpaceGroupElement[164].x=0.5-z;        SpaceGroupElement[164].y=-y;           SpaceGroupElement[164].z=0.5-x;
      SpaceGroupElement[165].x=z;            SpaceGroupElement[165].y=0.75-x;       SpaceGroupElement[165].z=0.75-y;
      SpaceGroupElement[166].x=-0.25+z;      SpaceGroupElement[166].y=0.5-y;        SpaceGroupElement[166].z=-0.75+x;
      SpaceGroupElement[167].x=-0.75+z;      SpaceGroupElement[167].y=-0.75+y;      SpaceGroupElement[167].z=-x;
      SpaceGroupElement[168].x=0.5-x;        SpaceGroupElement[168].y=0.5-y;        SpaceGroupElement[168].z=-z;
      SpaceGroupElement[169].x=0.25-y;       SpaceGroupElement[169].y=-0.5+x;       SpaceGroupElement[169].z=-0.25-z;
      SpaceGroupElement[170].x=0.25+x;       SpaceGroupElement[170].y=-0.25+y;      SpaceGroupElement[170].z=-0.5-z;
      SpaceGroupElement[171].x=-0.5+y;       SpaceGroupElement[171].y=-0.25-x;      SpaceGroupElement[171].z=-0.75-z;
      SpaceGroupElement[172].x=0.5-x;        SpaceGroupElement[172].y=-0.25+y;      SpaceGroupElement[172].z=0.25+z;
      SpaceGroupElement[173].x=0.25-y;       SpaceGroupElement[173].y=-0.25-x;      SpaceGroupElement[173].z=0.5+z;
      SpaceGroupElement[174].x=0.25+x;       SpaceGroupElement[174].y=0.5-y;        SpaceGroupElement[174].z=0.75+z;
      SpaceGroupElement[175].x=-0.5+y;       SpaceGroupElement[175].y=-0.5+x;       SpaceGroupElement[175].z=z;
      SpaceGroupElement[176].x=0.5-y;        SpaceGroupElement[176].y=-z;           SpaceGroupElement[176].z=0.5-x;
      SpaceGroupElement[177].x=-0.5+x;       SpaceGroupElement[177].y=-0.25-z;      SpaceGroupElement[177].z=0.25-y;
      SpaceGroupElement[178].x=-0.25+y;      SpaceGroupElement[178].y=-0.5-z;       SpaceGroupElement[178].z=0.25+x;
      SpaceGroupElement[179].x=-0.25-x;      SpaceGroupElement[179].y=-0.75-z;      SpaceGroupElement[179].z=-0.5+y;
      SpaceGroupElement[180].x=-0.25+y;      SpaceGroupElement[180].y=0.25+z;       SpaceGroupElement[180].z=0.5-x;
      SpaceGroupElement[181].x=-0.25-x;      SpaceGroupElement[181].y=0.5+z;        SpaceGroupElement[181].z=0.25-y;
      SpaceGroupElement[182].x=0.5-y;        SpaceGroupElement[182].y=0.75+z;       SpaceGroupElement[182].z=0.25+x;
      SpaceGroupElement[183].x=-0.5+x;       SpaceGroupElement[183].y=z;            SpaceGroupElement[183].z=-0.5+y;
      SpaceGroupElement[184].x=-z;           SpaceGroupElement[184].y=0.5-x;        SpaceGroupElement[184].z=0.5-y;
      SpaceGroupElement[185].x=0.25+z;       SpaceGroupElement[185].y=-x;           SpaceGroupElement[185].z=0.25+y;
      SpaceGroupElement[186].x=0.5+z;        SpaceGroupElement[186].y=-0.25-y;      SpaceGroupElement[186].z=0.25-x;
      SpaceGroupElement[187].x=0.75+z;       SpaceGroupElement[187].y=-0.25+x;      SpaceGroupElement[187].z=-y;
      SpaceGroupElement[188].x=0.5+z;        SpaceGroupElement[188].y=y;            SpaceGroupElement[188].z=-0.5+x;
      SpaceGroupElement[189].x=-z;           SpaceGroupElement[189].y=-0.25+x;      SpaceGroupElement[189].z=-0.25+y;
      SpaceGroupElement[190].x=-0.25-z;      SpaceGroupElement[190].y=-0.5+y;       SpaceGroupElement[190].z=0.25-x;
      SpaceGroupElement[191].x=-0.75-z;      SpaceGroupElement[191].y=0.25-y;       SpaceGroupElement[191].z=x;
      break;

    case 527: // [228] F d -3 c Origin choice 1 (F 4d 2 3 -1ad)
      SpaceGroupSize=192;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.25+y;        SpaceGroupElement[1].y=0.25-x;         SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=0.5-y;          SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.25-y;         SpaceGroupElement[3].y=-0.75+x;        SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=-0.25+y;        SpaceGroupElement[5].y=-0.25+x;        SpaceGroupElement[5].z=0.25-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.5-z;
      SpaceGroupElement[7].x=0.25-y;         SpaceGroupElement[7].y=0.75-x;         SpaceGroupElement[7].z=0.75-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=0.25-x;         SpaceGroupElement[9].y=-0.25+z;        SpaceGroupElement[9].z=-0.25+y;
      SpaceGroupElement[10].x=0.5-y;         SpaceGroupElement[10].y=-0.5+z;        SpaceGroupElement[10].z=-x;
      SpaceGroupElement[11].x=-0.75+x;       SpaceGroupElement[11].y=-0.75+z;       SpaceGroupElement[11].z=0.25-y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=-0.25+x;       SpaceGroupElement[13].y=0.25-z;        SpaceGroupElement[13].z=-0.25+y;
      SpaceGroupElement[14].x=-0.5+y;        SpaceGroupElement[14].y=0.5-z;         SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=0.75-x;        SpaceGroupElement[15].y=0.75-z;        SpaceGroupElement[15].z=0.25-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=0.5-z;         SpaceGroupElement[17].y=-0.5+x;        SpaceGroupElement[17].z=-y;
      SpaceGroupElement[18].x=0.75-z;        SpaceGroupElement[18].y=-0.75+y;       SpaceGroupElement[18].z=-0.25+x;
      SpaceGroupElement[19].x=-z;            SpaceGroupElement[19].y=0.5-x;         SpaceGroupElement[19].z=-0.5+y;
      SpaceGroupElement[20].x=0.25-z;        SpaceGroupElement[20].y=0.25-y;        SpaceGroupElement[20].z=0.25-x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=-0.75+z;       SpaceGroupElement[22].y=0.75-y;        SpaceGroupElement[22].z=-0.25+x;
      SpaceGroupElement[23].x=-0.25+z;       SpaceGroupElement[23].y=-0.75+y;       SpaceGroupElement[23].z=0.75-x;
      SpaceGroupElement[24].x=0.75-x;        SpaceGroupElement[24].y=0.25-y;        SpaceGroupElement[24].z=0.25-z;
      SpaceGroupElement[25].x=-y;            SpaceGroupElement[25].y=-0.5+x;        SpaceGroupElement[25].z=-z;
      SpaceGroupElement[26].x=-0.75+x;       SpaceGroupElement[26].y=0.25+y;        SpaceGroupElement[26].z=-0.25-z;
      SpaceGroupElement[27].x=y;             SpaceGroupElement[27].y=-x;            SpaceGroupElement[27].z=-0.5-z;
      SpaceGroupElement[28].x=0.75-x;        SpaceGroupElement[28].y=-0.25+y;       SpaceGroupElement[28].z=-0.25+z;
      SpaceGroupElement[29].x=-y;            SpaceGroupElement[29].y=0.5-x;         SpaceGroupElement[29].z=z;
      SpaceGroupElement[30].x=-0.75+x;       SpaceGroupElement[30].y=-0.25-y;       SpaceGroupElement[30].z=0.25+z;
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=0.5+z;
      SpaceGroupElement[32].x=0.25-y;        SpaceGroupElement[32].y=0.25-z;        SpaceGroupElement[32].z=0.75-x;
      SpaceGroupElement[33].x=-0.5+x;        SpaceGroupElement[33].y=-z;            SpaceGroupElement[33].z=-y;
      SpaceGroupElement[34].x=0.25+y;        SpaceGroupElement[34].y=-0.25-z;       SpaceGroupElement[34].z=-0.75+x;
      SpaceGroupElement[35].x=-x;            SpaceGroupElement[35].y=-0.5-z;        SpaceGroupElement[35].z=y;
      SpaceGroupElement[36].x=-0.25+y;       SpaceGroupElement[36].y=-0.25+z;       SpaceGroupElement[36].z=0.75-x;
      SpaceGroupElement[37].x=0.5-x;         SpaceGroupElement[37].y=z;             SpaceGroupElement[37].z=-y;
      SpaceGroupElement[38].x=-0.25-y;       SpaceGroupElement[38].y=0.25+z;        SpaceGroupElement[38].z=-0.75+x;
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=0.5+z;         SpaceGroupElement[39].z=y;
      SpaceGroupElement[40].x=0.25-z;        SpaceGroupElement[40].y=0.75-x;        SpaceGroupElement[40].z=0.25-y;
      SpaceGroupElement[41].x=0.25+z;        SpaceGroupElement[41].y=0.25-x;        SpaceGroupElement[41].z=-0.25+y;
      SpaceGroupElement[42].x=0.5+z;         SpaceGroupElement[42].y=-0.5-y;        SpaceGroupElement[42].z=0.5-x;
      SpaceGroupElement[43].x=-0.25+z;       SpaceGroupElement[43].y=-0.25+x;       SpaceGroupElement[43].z=-0.25-y;
      SpaceGroupElement[44].x=z;             SpaceGroupElement[44].y=y;             SpaceGroupElement[44].z=-0.5+x;
      SpaceGroupElement[45].x=0.25-z;        SpaceGroupElement[45].y=-0.75+x;       SpaceGroupElement[45].z=-0.25+y;
      SpaceGroupElement[46].x=-0.5-z;        SpaceGroupElement[46].y=0.5+y;         SpaceGroupElement[46].z=0.5-x;
      SpaceGroupElement[47].x=-z;            SpaceGroupElement[47].y=-0.5-y;        SpaceGroupElement[47].z=x;
      SpaceGroupElement[48].x=x;             SpaceGroupElement[48].y=-0.5+y;        SpaceGroupElement[48].z=-0.5+z;
      SpaceGroupElement[49].x=-0.75+y;       SpaceGroupElement[49].y=0.25-x;        SpaceGroupElement[49].z=-0.75+z;
      SpaceGroupElement[50].x=-x;            SpaceGroupElement[50].y=-y;            SpaceGroupElement[50].z=z;
      SpaceGroupElement[51].x=0.75-y;        SpaceGroupElement[51].y=-0.75+x;       SpaceGroupElement[51].z=-0.25+z;
      SpaceGroupElement[52].x=x;             SpaceGroupElement[52].y=0.5-y;         SpaceGroupElement[52].z=0.5-z;
      SpaceGroupElement[53].x=-0.75+y;       SpaceGroupElement[53].y=-0.25+x;       SpaceGroupElement[53].z=0.75-z;
      SpaceGroupElement[54].x=-x;            SpaceGroupElement[54].y=y;             SpaceGroupElement[54].z=-z;
      SpaceGroupElement[55].x=0.75-y;        SpaceGroupElement[55].y=0.75-x;        SpaceGroupElement[55].z=0.25-z;
      SpaceGroupElement[56].x=-0.5+y;        SpaceGroupElement[56].y=-0.5+z;        SpaceGroupElement[56].z=x;
      SpaceGroupElement[57].x=0.25-x;        SpaceGroupElement[57].y=-0.75+z;       SpaceGroupElement[57].z=-0.75+y;
      SpaceGroupElement[58].x=-y;            SpaceGroupElement[58].y=z;             SpaceGroupElement[58].z=-x;
      SpaceGroupElement[59].x=-0.75+x;       SpaceGroupElement[59].y=-0.25+z;       SpaceGroupElement[59].z=0.75-y;
      SpaceGroupElement[60].x=0.5-y;         SpaceGroupElement[60].y=0.5-z;         SpaceGroupElement[60].z=x;
      SpaceGroupElement[61].x=-0.25+x;       SpaceGroupElement[61].y=0.75-z;        SpaceGroupElement[61].z=-0.75+y;
      SpaceGroupElement[62].x=y;             SpaceGroupElement[62].y=-z;            SpaceGroupElement[62].z=-x;
      SpaceGroupElement[63].x=0.75-x;        SpaceGroupElement[63].y=0.25-z;        SpaceGroupElement[63].z=0.75-y;
      SpaceGroupElement[64].x=-0.5+z;        SpaceGroupElement[64].y=x;             SpaceGroupElement[64].z=-0.5+y;
      SpaceGroupElement[65].x=-z;            SpaceGroupElement[65].y=-0.5+x;        SpaceGroupElement[65].z=0.5-y;
      SpaceGroupElement[66].x=0.25-z;        SpaceGroupElement[66].y=-0.25+y;       SpaceGroupElement[66].z=-0.25+x;
      SpaceGroupElement[67].x=0.5-z;         SpaceGroupElement[67].y=0.5-x;         SpaceGroupElement[67].z=y;
      SpaceGroupElement[68].x=0.75-z;        SpaceGroupElement[68].y=0.75-y;        SpaceGroupElement[68].z=0.25-x;
      SpaceGroupElement[69].x=-0.5+z;        SpaceGroupElement[69].y=-x;            SpaceGroupElement[69].z=0.5-y;
      SpaceGroupElement[70].x=-0.25+z;       SpaceGroupElement[70].y=0.25-y;        SpaceGroupElement[70].z=-0.25+x;
      SpaceGroupElement[71].x=-0.75+z;       SpaceGroupElement[71].y=-0.25+y;       SpaceGroupElement[71].z=0.75-x;
      SpaceGroupElement[72].x=0.75-x;        SpaceGroupElement[72].y=0.75-y;        SpaceGroupElement[72].z=0.75-z;
      SpaceGroupElement[73].x=0.5-y;         SpaceGroupElement[73].y=-0.5+x;        SpaceGroupElement[73].z=0.5-z;
      SpaceGroupElement[74].x=-0.75+x;       SpaceGroupElement[74].y=-0.25+y;       SpaceGroupElement[74].z=0.25-z;
      SpaceGroupElement[75].x=-0.5+y;        SpaceGroupElement[75].y=-x;            SpaceGroupElement[75].z=-z;
      SpaceGroupElement[76].x=0.75-x;        SpaceGroupElement[76].y=-0.75+y;       SpaceGroupElement[76].z=-0.75+z;
      SpaceGroupElement[77].x=0.5-y;         SpaceGroupElement[77].y=0.5-x;         SpaceGroupElement[77].z=-0.5+z;
      SpaceGroupElement[78].x=-0.75+x;       SpaceGroupElement[78].y=0.25-y;        SpaceGroupElement[78].z=-0.25+z;
      SpaceGroupElement[79].x=-0.5+y;        SpaceGroupElement[79].y=x;             SpaceGroupElement[79].z=z;
      SpaceGroupElement[80].x=0.75-y;        SpaceGroupElement[80].y=0.75-z;        SpaceGroupElement[80].z=0.75-x;
      SpaceGroupElement[81].x=-0.5+x;        SpaceGroupElement[81].y=0.5-z;         SpaceGroupElement[81].z=0.5-y;
      SpaceGroupElement[82].x=-0.25+y;       SpaceGroupElement[82].y=0.25-z;        SpaceGroupElement[82].z=-0.75+x;
      SpaceGroupElement[83].x=-x;            SpaceGroupElement[83].y=-z;            SpaceGroupElement[83].z=-0.5+y;
      SpaceGroupElement[84].x=-0.75+y;       SpaceGroupElement[84].y=-0.75+z;       SpaceGroupElement[84].z=0.75-x;
      SpaceGroupElement[85].x=0.5-x;         SpaceGroupElement[85].y=-0.5+z;        SpaceGroupElement[85].z=0.5-y;
      SpaceGroupElement[86].x=0.25-y;        SpaceGroupElement[86].y=-0.25+z;       SpaceGroupElement[86].z=-0.75+x;
      SpaceGroupElement[87].x=x;             SpaceGroupElement[87].y=z;             SpaceGroupElement[87].z=-0.5+y;
      SpaceGroupElement[88].x=0.75-z;        SpaceGroupElement[88].y=0.75-x;        SpaceGroupElement[88].z=0.75-y;
      SpaceGroupElement[89].x=-0.25+z;       SpaceGroupElement[89].y=0.25-x;        SpaceGroupElement[89].z=-0.75+y;
      SpaceGroupElement[90].x=z;             SpaceGroupElement[90].y=-y;            SpaceGroupElement[90].z=0.5-x;
      SpaceGroupElement[91].x=-0.75+z;       SpaceGroupElement[91].y=-0.25+x;       SpaceGroupElement[91].z=0.25-y;
      SpaceGroupElement[92].x=-0.5+z;        SpaceGroupElement[92].y=-0.5+y;        SpaceGroupElement[92].z=-0.5+x;
      SpaceGroupElement[93].x=0.75-z;        SpaceGroupElement[93].y=-0.75+x;       SpaceGroupElement[93].z=-0.75+y;
      SpaceGroupElement[94].x=-z;            SpaceGroupElement[94].y=y;             SpaceGroupElement[94].z=0.5-x;
      SpaceGroupElement[95].x=0.5-z;         SpaceGroupElement[95].y=-y;            SpaceGroupElement[95].z=x;
      SpaceGroupElement[96].x=-0.5+x;        SpaceGroupElement[96].y=y;             SpaceGroupElement[96].z=-0.5+z;
      SpaceGroupElement[97].x=-0.25+y;       SpaceGroupElement[97].y=0.75-x;        SpaceGroupElement[97].z=-0.75+z;
      SpaceGroupElement[98].x=0.5-x;         SpaceGroupElement[98].y=0.5-y;         SpaceGroupElement[98].z=z;
      SpaceGroupElement[99].x=0.25-y;        SpaceGroupElement[99].y=-0.25+x;       SpaceGroupElement[99].z=-0.25+z;
      SpaceGroupElement[100].x=-0.5+x;       SpaceGroupElement[100].y=-y;           SpaceGroupElement[100].z=0.5-z;
      SpaceGroupElement[101].x=-0.25+y;      SpaceGroupElement[101].y=-0.75+x;      SpaceGroupElement[101].z=0.75-z;
      SpaceGroupElement[102].x=0.5-x;        SpaceGroupElement[102].y=-0.5+y;       SpaceGroupElement[102].z=-z;
      SpaceGroupElement[103].x=0.25-y;       SpaceGroupElement[103].y=0.25-x;       SpaceGroupElement[103].z=0.25-z;
      SpaceGroupElement[104].x=y;            SpaceGroupElement[104].y=-0.5+z;       SpaceGroupElement[104].z=-0.5+x;
      SpaceGroupElement[105].x=0.75-x;       SpaceGroupElement[105].y=-0.75+z;      SpaceGroupElement[105].z=-0.25+y;
      SpaceGroupElement[106].x=0.5-y;        SpaceGroupElement[106].y=z;            SpaceGroupElement[106].z=0.5-x;
      SpaceGroupElement[107].x=-0.25+x;      SpaceGroupElement[107].y=-0.25+z;      SpaceGroupElement[107].z=0.25-y;
      SpaceGroupElement[108].x=-y;           SpaceGroupElement[108].y=0.5-z;        SpaceGroupElement[108].z=-0.5+x;
      SpaceGroupElement[109].x=-0.75+x;      SpaceGroupElement[109].y=0.75-z;       SpaceGroupElement[109].z=-0.25+y;
      SpaceGroupElement[110].x=-0.5+y;       SpaceGroupElement[110].y=-z;           SpaceGroupElement[110].z=0.5-x;
      SpaceGroupElement[111].x=0.25-x;       SpaceGroupElement[111].y=0.25-z;       SpaceGroupElement[111].z=0.25-y;
      SpaceGroupElement[112].x=-0.5+z;       SpaceGroupElement[112].y=-0.5+x;       SpaceGroupElement[112].z=y;
      SpaceGroupElement[113].x=-z;           SpaceGroupElement[113].y=x;            SpaceGroupElement[113].z=-y;
      SpaceGroupElement[114].x=0.25-z;       SpaceGroupElement[114].y=-0.75+y;      SpaceGroupElement[114].z=-0.75+x;
      SpaceGroupElement[115].x=0.5-z;        SpaceGroupElement[115].y=-x;           SpaceGroupElement[115].z=-0.5+y;
      SpaceGroupElement[116].x=0.75-z;       SpaceGroupElement[116].y=0.25-y;       SpaceGroupElement[116].z=0.75-x;
      SpaceGroupElement[117].x=-0.5+z;       SpaceGroupElement[117].y=0.5-x;        SpaceGroupElement[117].z=-y;
      SpaceGroupElement[118].x=-0.25+z;      SpaceGroupElement[118].y=0.75-y;       SpaceGroupElement[118].z=-0.75+x;
      SpaceGroupElement[119].x=-0.75+z;      SpaceGroupElement[119].y=-0.75+y;      SpaceGroupElement[119].z=0.25-x;
      SpaceGroupElement[120].x=0.25-x;       SpaceGroupElement[120].y=0.25-y;       SpaceGroupElement[120].z=0.75-z;
      SpaceGroupElement[121].x=-y;           SpaceGroupElement[121].y=x;            SpaceGroupElement[121].z=0.5-z;
      SpaceGroupElement[122].x=-0.25+x;      SpaceGroupElement[122].y=0.25+y;       SpaceGroupElement[122].z=0.25-z;
      SpaceGroupElement[123].x=y;            SpaceGroupElement[123].y=0.5-x;        SpaceGroupElement[123].z=-z;
      SpaceGroupElement[124].x=0.25-x;       SpaceGroupElement[124].y=-0.25+y;      SpaceGroupElement[124].z=-0.75+z;
      SpaceGroupElement[125].x=-y;           SpaceGroupElement[125].y=-x;           SpaceGroupElement[125].z=-0.5+z;
      SpaceGroupElement[126].x=-0.25+x;      SpaceGroupElement[126].y=-0.25-y;      SpaceGroupElement[126].z=-0.25+z;
      SpaceGroupElement[127].x=y;            SpaceGroupElement[127].y=-0.5+x;       SpaceGroupElement[127].z=z;
      SpaceGroupElement[128].x=0.25-y;       SpaceGroupElement[128].y=0.75-z;       SpaceGroupElement[128].z=0.25-x;
      SpaceGroupElement[129].x=x;            SpaceGroupElement[129].y=0.5-z;        SpaceGroupElement[129].z=-y;
      SpaceGroupElement[130].x=0.25+y;       SpaceGroupElement[130].y=0.25-z;       SpaceGroupElement[130].z=-0.25+x;
      SpaceGroupElement[131].x=0.5-x;        SpaceGroupElement[131].y=-z;           SpaceGroupElement[131].z=y;
      SpaceGroupElement[132].x=-0.25+y;      SpaceGroupElement[132].y=-0.75+z;      SpaceGroupElement[132].z=0.25-x;
      SpaceGroupElement[133].x=-x;           SpaceGroupElement[133].y=-0.5+z;       SpaceGroupElement[133].z=-y;
      SpaceGroupElement[134].x=-0.25-y;      SpaceGroupElement[134].y=-0.25+z;      SpaceGroupElement[134].z=-0.25+x;
      SpaceGroupElement[135].x=-0.5+x;       SpaceGroupElement[135].y=z;            SpaceGroupElement[135].z=y;
      SpaceGroupElement[136].x=0.75-z;       SpaceGroupElement[136].y=0.25-x;       SpaceGroupElement[136].z=0.25-y;
      SpaceGroupElement[137].x=-0.25+z;      SpaceGroupElement[137].y=0.75-x;       SpaceGroupElement[137].z=-0.25+y;
      SpaceGroupElement[138].x=z;            SpaceGroupElement[138].y=-0.5-y;       SpaceGroupElement[138].z=-x;
      SpaceGroupElement[139].x=-0.75+z;      SpaceGroupElement[139].y=-0.75+x;      SpaceGroupElement[139].z=-0.25-y;
      SpaceGroupElement[140].x=-0.5+z;       SpaceGroupElement[140].y=y;            SpaceGroupElement[140].z=x;
      SpaceGroupElement[141].x=0.75-z;       SpaceGroupElement[141].y=-0.25+x;      SpaceGroupElement[141].z=-0.25+y;
      SpaceGroupElement[142].x=-z;           SpaceGroupElement[142].y=0.5+y;        SpaceGroupElement[142].z=-x;
      SpaceGroupElement[143].x=0.5-z;        SpaceGroupElement[143].y=-0.5-y;       SpaceGroupElement[143].z=-0.5+x;
      SpaceGroupElement[144].x=-0.5+x;       SpaceGroupElement[144].y=-0.5+y;       SpaceGroupElement[144].z=z;
      SpaceGroupElement[145].x=-0.75+y;      SpaceGroupElement[145].y=0.75-x;       SpaceGroupElement[145].z=-0.25+z;
      SpaceGroupElement[146].x=0.5-x;        SpaceGroupElement[146].y=-y;           SpaceGroupElement[146].z=-0.5+z;
      SpaceGroupElement[147].x=0.75-y;       SpaceGroupElement[147].y=-0.25+x;      SpaceGroupElement[147].z=-0.75+z;
      SpaceGroupElement[148].x=-0.5+x;       SpaceGroupElement[148].y=0.5-y;        SpaceGroupElement[148].z=-z;
      SpaceGroupElement[149].x=-0.75+y;      SpaceGroupElement[149].y=-0.75+x;      SpaceGroupElement[149].z=0.25-z;
      SpaceGroupElement[150].x=0.5-x;        SpaceGroupElement[150].y=y;            SpaceGroupElement[150].z=0.5-z;
      SpaceGroupElement[151].x=0.75-y;       SpaceGroupElement[151].y=0.25-x;       SpaceGroupElement[151].z=0.75-z;
      SpaceGroupElement[152].x=-0.5+y;       SpaceGroupElement[152].y=z;            SpaceGroupElement[152].z=-0.5+x;
      SpaceGroupElement[153].x=0.75-x;       SpaceGroupElement[153].y=-0.25+z;      SpaceGroupElement[153].z=-0.75+y;
      SpaceGroupElement[154].x=-y;           SpaceGroupElement[154].y=-0.5+z;       SpaceGroupElement[154].z=0.5-x;
      SpaceGroupElement[155].x=-0.25+x;      SpaceGroupElement[155].y=-0.75+z;      SpaceGroupElement[155].z=0.75-y;
      SpaceGroupElement[156].x=0.5-y;        SpaceGroupElement[156].y=-z;           SpaceGroupElement[156].z=-0.5+x;
      SpaceGroupElement[157].x=-0.75+x;      SpaceGroupElement[157].y=0.25-z;       SpaceGroupElement[157].z=-0.75+y;
      SpaceGroupElement[158].x=y;            SpaceGroupElement[158].y=0.5-z;        SpaceGroupElement[158].z=0.5-x;
      SpaceGroupElement[159].x=0.25-x;       SpaceGroupElement[159].y=0.75-z;       SpaceGroupElement[159].z=0.75-y;
      SpaceGroupElement[160].x=z;            SpaceGroupElement[160].y=-0.5+x;       SpaceGroupElement[160].z=-0.5+y;
      SpaceGroupElement[161].x=0.5-z;        SpaceGroupElement[161].y=x;            SpaceGroupElement[161].z=0.5-y;
      SpaceGroupElement[162].x=0.75-z;       SpaceGroupElement[162].y=-0.25+y;      SpaceGroupElement[162].z=-0.75+x;
      SpaceGroupElement[163].x=-z;           SpaceGroupElement[163].y=-x;           SpaceGroupElement[163].z=y;
      SpaceGroupElement[164].x=0.25-z;       SpaceGroupElement[164].y=0.75-y;       SpaceGroupElement[164].z=0.75-x;
      SpaceGroupElement[165].x=z;            SpaceGroupElement[165].y=0.5-x;        SpaceGroupElement[165].z=0.5-y;
      SpaceGroupElement[166].x=-0.75+z;      SpaceGroupElement[166].y=0.25-y;       SpaceGroupElement[166].z=-0.75+x;
      SpaceGroupElement[167].x=-0.25+z;      SpaceGroupElement[167].y=-0.25+y;      SpaceGroupElement[167].z=0.25-x;
      SpaceGroupElement[168].x=0.25-x;       SpaceGroupElement[168].y=0.75-y;       SpaceGroupElement[168].z=0.25-z;
      SpaceGroupElement[169].x=0.5-y;        SpaceGroupElement[169].y=x;            SpaceGroupElement[169].z=-z;
      SpaceGroupElement[170].x=-0.25+x;      SpaceGroupElement[170].y=-0.25+y;      SpaceGroupElement[170].z=-0.25-z;
      SpaceGroupElement[171].x=-0.5+y;       SpaceGroupElement[171].y=0.5-x;        SpaceGroupElement[171].z=-0.5-z;
      SpaceGroupElement[172].x=0.25-x;       SpaceGroupElement[172].y=-0.75+y;      SpaceGroupElement[172].z=-0.25+z;
      SpaceGroupElement[173].x=0.5-y;        SpaceGroupElement[173].y=-x;           SpaceGroupElement[173].z=z;
      SpaceGroupElement[174].x=-0.25+x;      SpaceGroupElement[174].y=0.25-y;       SpaceGroupElement[174].z=0.25+z;
      SpaceGroupElement[175].x=-0.5+y;       SpaceGroupElement[175].y=-0.5+x;       SpaceGroupElement[175].z=0.5+z;
      SpaceGroupElement[176].x=0.75-y;       SpaceGroupElement[176].y=0.25-z;       SpaceGroupElement[176].z=0.25-x;
      SpaceGroupElement[177].x=x;            SpaceGroupElement[177].y=-z;           SpaceGroupElement[177].z=0.5-y;
      SpaceGroupElement[178].x=-0.25+y;      SpaceGroupElement[178].y=-0.25-z;      SpaceGroupElement[178].z=-0.25+x;
      SpaceGroupElement[179].x=0.5-x;        SpaceGroupElement[179].y=-0.5-z;       SpaceGroupElement[179].z=-0.5+y;
      SpaceGroupElement[180].x=-0.75+y;      SpaceGroupElement[180].y=-0.25+z;      SpaceGroupElement[180].z=0.25-x;
      SpaceGroupElement[181].x=-x;           SpaceGroupElement[181].y=z;            SpaceGroupElement[181].z=0.5-y;
      SpaceGroupElement[182].x=0.25-y;       SpaceGroupElement[182].y=0.25+z;       SpaceGroupElement[182].z=-0.25+x;
      SpaceGroupElement[183].x=-0.5+x;       SpaceGroupElement[183].y=0.5+z;        SpaceGroupElement[183].z=-0.5+y;
      SpaceGroupElement[184].x=0.25-z;       SpaceGroupElement[184].y=0.25-x;       SpaceGroupElement[184].z=0.75-y;
      SpaceGroupElement[185].x=0.25+z;       SpaceGroupElement[185].y=0.75-x;       SpaceGroupElement[185].z=-0.75+y;
      SpaceGroupElement[186].x=0.5+z;        SpaceGroupElement[186].y=-y;           SpaceGroupElement[186].z=-x;
      SpaceGroupElement[187].x=-0.25+z;      SpaceGroupElement[187].y=-0.75+x;      SpaceGroupElement[187].z=0.25-y;
      SpaceGroupElement[188].x=z;            SpaceGroupElement[188].y=-0.5+y;       SpaceGroupElement[188].z=x;
      SpaceGroupElement[189].x=0.25-z;       SpaceGroupElement[189].y=-0.25+x;      SpaceGroupElement[189].z=-0.75+y;
      SpaceGroupElement[190].x=-0.5-z;       SpaceGroupElement[190].y=y;            SpaceGroupElement[190].z=-x;
      SpaceGroupElement[191].x=-z;           SpaceGroupElement[191].y=-y;           SpaceGroupElement[191].z=-0.5+x;
      break;
    case 528: // [228] F d -3 c Origin choice 2 (-F 4ud 2vw 3)
      SpaceGroupSize=192;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.25+y;        SpaceGroupElement[1].y=0.5-x;          SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=0.25-x;         SpaceGroupElement[2].y=0.75-y;         SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.5-y;          SpaceGroupElement[3].y=-0.75+x;        SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=0.25-y;         SpaceGroupElement[4].z=0.25-z;
      SpaceGroupElement[5].x=-0.25+y;        SpaceGroupElement[5].y=-0.25+x;        SpaceGroupElement[5].z=0.5-z;
      SpaceGroupElement[6].x=0.25-x;         SpaceGroupElement[6].y=-0.5+y;         SpaceGroupElement[6].z=0.75-z;
      SpaceGroupElement[7].x=0.5-y;          SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=0.5-x;          SpaceGroupElement[9].y=-0.25+z;        SpaceGroupElement[9].z=-0.25+y;
      SpaceGroupElement[10].x=0.75-y;        SpaceGroupElement[10].y=-0.5+z;        SpaceGroupElement[10].z=0.25-x;
      SpaceGroupElement[11].x=-0.75+x;       SpaceGroupElement[11].y=-0.75+z;       SpaceGroupElement[11].z=0.5-y;
      SpaceGroupElement[12].x=0.25-y;        SpaceGroupElement[12].y=0.25-z;        SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=-0.25+x;       SpaceGroupElement[13].y=0.5-z;         SpaceGroupElement[13].z=-0.25+y;
      SpaceGroupElement[14].x=-0.5+y;        SpaceGroupElement[14].y=0.75-z;        SpaceGroupElement[14].z=0.25-x;
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=0.5-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=0.75-z;        SpaceGroupElement[17].y=-0.5+x;        SpaceGroupElement[17].z=0.25-y;
      SpaceGroupElement[18].x=-z;            SpaceGroupElement[18].y=-0.75+y;       SpaceGroupElement[18].z=-0.25+x;
      SpaceGroupElement[19].x=0.25-z;        SpaceGroupElement[19].y=0.75-x;        SpaceGroupElement[19].z=-0.5+y;
      SpaceGroupElement[20].x=0.5-z;         SpaceGroupElement[20].y=0.5-y;         SpaceGroupElement[20].z=0.5-x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=0.25-x;        SpaceGroupElement[21].z=0.25-y;
      SpaceGroupElement[22].x=-0.75+z;       SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=-0.25+x;
      SpaceGroupElement[23].x=-0.25+z;       SpaceGroupElement[23].y=-0.75+y;       SpaceGroupElement[23].z=-x;
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;
      SpaceGroupElement[25].x=-0.25-y;       SpaceGroupElement[25].y=0.5+x;         SpaceGroupElement[25].z=-0.25-z;
      SpaceGroupElement[26].x=0.25+x;        SpaceGroupElement[26].y=0.75+y;        SpaceGroupElement[26].z=-0.5-z;
      SpaceGroupElement[27].x=0.5+y;         SpaceGroupElement[27].y=-0.75-x;       SpaceGroupElement[27].z=-0.75-z;
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=0.25+y;        SpaceGroupElement[28].z=0.25+z;
      SpaceGroupElement[29].x=-0.25-y;       SpaceGroupElement[29].y=-0.25-x;       SpaceGroupElement[29].z=0.5+z;
      SpaceGroupElement[30].x=0.25+x;        SpaceGroupElement[30].y=-0.5-y;        SpaceGroupElement[30].z=0.75+z;
      SpaceGroupElement[31].x=0.5+y;         SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=z;
      SpaceGroupElement[32].x=-y;            SpaceGroupElement[32].y=-z;            SpaceGroupElement[32].z=-x;
      SpaceGroupElement[33].x=0.5+x;         SpaceGroupElement[33].y=-0.25-z;       SpaceGroupElement[33].z=-0.25-y;
      SpaceGroupElement[34].x=0.75+y;        SpaceGroupElement[34].y=-0.5-z;        SpaceGroupElement[34].z=0.25+x;
      SpaceGroupElement[35].x=-0.75-x;       SpaceGroupElement[35].y=-0.75-z;       SpaceGroupElement[35].z=0.5+y;
      SpaceGroupElement[36].x=0.25+y;        SpaceGroupElement[36].y=0.25+z;        SpaceGroupElement[36].z=-x;
      SpaceGroupElement[37].x=-0.25-x;       SpaceGroupElement[37].y=0.5+z;         SpaceGroupElement[37].z=-0.25-y;
      SpaceGroupElement[38].x=-0.5-y;        SpaceGroupElement[38].y=0.75+z;        SpaceGroupElement[38].z=0.25+x;
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=z;             SpaceGroupElement[39].z=0.5+y;
      SpaceGroupElement[40].x=-z;            SpaceGroupElement[40].y=-x;            SpaceGroupElement[40].z=-y;
      SpaceGroupElement[41].x=0.75+z;        SpaceGroupElement[41].y=-0.5-x;        SpaceGroupElement[41].z=0.25+y;
      SpaceGroupElement[42].x=z;             SpaceGroupElement[42].y=-0.75-y;       SpaceGroupElement[42].z=-0.25-x;
      SpaceGroupElement[43].x=0.25+z;        SpaceGroupElement[43].y=0.75+x;        SpaceGroupElement[43].z=-0.5-y;
      SpaceGroupElement[44].x=0.5+z;         SpaceGroupElement[44].y=0.5+y;         SpaceGroupElement[44].z=0.5+x;
      SpaceGroupElement[45].x=-z;            SpaceGroupElement[45].y=0.25+x;        SpaceGroupElement[45].z=0.25+y;
      SpaceGroupElement[46].x=-0.75-z;       SpaceGroupElement[46].y=y;             SpaceGroupElement[46].z=-0.25-x;
      SpaceGroupElement[47].x=-0.25-z;       SpaceGroupElement[47].y=-0.75-y;       SpaceGroupElement[47].z=x;
      SpaceGroupElement[48].x=x;             SpaceGroupElement[48].y=-0.5+y;        SpaceGroupElement[48].z=-0.5+z;
      SpaceGroupElement[49].x=-0.75+y;       SpaceGroupElement[49].y=0.5-x;         SpaceGroupElement[49].z=-0.75+z;
      SpaceGroupElement[50].x=0.25-x;        SpaceGroupElement[50].y=0.25-y;        SpaceGroupElement[50].z=z;
      SpaceGroupElement[51].x=-y;            SpaceGroupElement[51].y=-0.75+x;       SpaceGroupElement[51].z=-0.25+z;
      SpaceGroupElement[52].x=x;             SpaceGroupElement[52].y=0.75-y;        SpaceGroupElement[52].z=0.75-z;
      SpaceGroupElement[53].x=-0.75+y;       SpaceGroupElement[53].y=-0.25+x;       SpaceGroupElement[53].z=-z;
      SpaceGroupElement[54].x=0.25-x;        SpaceGroupElement[54].y=y;             SpaceGroupElement[54].z=0.25-z;
      SpaceGroupElement[55].x=-y;            SpaceGroupElement[55].y=-x;            SpaceGroupElement[55].z=0.5-z;
      SpaceGroupElement[56].x=-0.5+y;        SpaceGroupElement[56].y=-0.5+z;        SpaceGroupElement[56].z=x;
      SpaceGroupElement[57].x=0.5-x;         SpaceGroupElement[57].y=-0.75+z;       SpaceGroupElement[57].z=-0.75+y;
      SpaceGroupElement[58].x=0.25-y;        SpaceGroupElement[58].y=z;             SpaceGroupElement[58].z=0.25-x;
      SpaceGroupElement[59].x=-0.75+x;       SpaceGroupElement[59].y=-0.25+z;       SpaceGroupElement[59].z=-y;
      SpaceGroupElement[60].x=0.75-y;        SpaceGroupElement[60].y=0.75-z;        SpaceGroupElement[60].z=x;
      SpaceGroupElement[61].x=-0.25+x;       SpaceGroupElement[61].y=-z;            SpaceGroupElement[61].z=-0.75+y;
      SpaceGroupElement[62].x=y;             SpaceGroupElement[62].y=0.25-z;        SpaceGroupElement[62].z=0.25-x;
      SpaceGroupElement[63].x=-x;            SpaceGroupElement[63].y=0.5-z;         SpaceGroupElement[63].z=-y;
      SpaceGroupElement[64].x=-0.5+z;        SpaceGroupElement[64].y=x;             SpaceGroupElement[64].z=-0.5+y;
      SpaceGroupElement[65].x=0.25-z;        SpaceGroupElement[65].y=-0.5+x;        SpaceGroupElement[65].z=0.75-y;
      SpaceGroupElement[66].x=0.5-z;         SpaceGroupElement[66].y=-0.25+y;       SpaceGroupElement[66].z=-0.25+x;
      SpaceGroupElement[67].x=0.75-z;        SpaceGroupElement[67].y=0.75-x;        SpaceGroupElement[67].z=y;
      SpaceGroupElement[68].x=-z;            SpaceGroupElement[68].y=-y;            SpaceGroupElement[68].z=0.5-x;
      SpaceGroupElement[69].x=-0.5+z;        SpaceGroupElement[69].y=0.25-x;        SpaceGroupElement[69].z=0.75-y;
      SpaceGroupElement[70].x=-0.25+z;       SpaceGroupElement[70].y=0.5-y;         SpaceGroupElement[70].z=-0.25+x;
      SpaceGroupElement[71].x=-0.75+z;       SpaceGroupElement[71].y=-0.25+y;       SpaceGroupElement[71].z=-x;
      SpaceGroupElement[72].x=-x;            SpaceGroupElement[72].y=0.5-y;         SpaceGroupElement[72].z=0.5-z;
      SpaceGroupElement[73].x=0.25-y;        SpaceGroupElement[73].y=0.5+x;         SpaceGroupElement[73].z=0.25-z;
      SpaceGroupElement[74].x=0.25+x;        SpaceGroupElement[74].y=0.25+y;        SpaceGroupElement[74].z=-z;
      SpaceGroupElement[75].x=y;             SpaceGroupElement[75].y=-0.75-x;       SpaceGroupElement[75].z=-0.25-z;
      SpaceGroupElement[76].x=-x;            SpaceGroupElement[76].y=-0.25+y;       SpaceGroupElement[76].z=-0.25+z;
      SpaceGroupElement[77].x=0.25-y;        SpaceGroupElement[77].y=-0.25-x;       SpaceGroupElement[77].z=z;
      SpaceGroupElement[78].x=0.25+x;        SpaceGroupElement[78].y=-y;            SpaceGroupElement[78].z=0.25+z;
      SpaceGroupElement[79].x=y;             SpaceGroupElement[79].y=x;             SpaceGroupElement[79].z=-0.5+z;
      SpaceGroupElement[80].x=0.5-y;         SpaceGroupElement[80].y=0.5-z;         SpaceGroupElement[80].z=-x;
      SpaceGroupElement[81].x=0.5+x;         SpaceGroupElement[81].y=0.25-z;        SpaceGroupElement[81].z=0.25-y;
      SpaceGroupElement[82].x=0.25+y;        SpaceGroupElement[82].y=-z;            SpaceGroupElement[82].z=0.25+x;
      SpaceGroupElement[83].x=-0.75-x;       SpaceGroupElement[83].y=-0.25-z;       SpaceGroupElement[83].z=y;
      SpaceGroupElement[84].x=-0.25+y;       SpaceGroupElement[84].y=-0.25+z;       SpaceGroupElement[84].z=-x;
      SpaceGroupElement[85].x=-0.25-x;       SpaceGroupElement[85].y=z;             SpaceGroupElement[85].z=0.25-y;
      SpaceGroupElement[86].x=-y;            SpaceGroupElement[86].y=0.25+z;        SpaceGroupElement[86].z=0.25+x;
      SpaceGroupElement[87].x=x;             SpaceGroupElement[87].y=-0.5+z;        SpaceGroupElement[87].z=y;
      SpaceGroupElement[88].x=0.5-z;         SpaceGroupElement[88].y=-x;            SpaceGroupElement[88].z=0.5-y;
      SpaceGroupElement[89].x=0.25+z;        SpaceGroupElement[89].y=-0.5-x;        SpaceGroupElement[89].z=-0.25+y;
      SpaceGroupElement[90].x=-0.5+z;        SpaceGroupElement[90].y=-0.25-y;       SpaceGroupElement[90].z=-0.25-x;
      SpaceGroupElement[91].x=-0.25+z;       SpaceGroupElement[91].y=0.75+x;        SpaceGroupElement[91].z=-y;
      SpaceGroupElement[92].x=z;             SpaceGroupElement[92].y=y;             SpaceGroupElement[92].z=0.5+x;
      SpaceGroupElement[93].x=0.5-z;         SpaceGroupElement[93].y=0.25+x;        SpaceGroupElement[93].z=-0.25+y;
      SpaceGroupElement[94].x=-0.25-z;       SpaceGroupElement[94].y=-0.5+y;        SpaceGroupElement[94].z=-0.25-x;
      SpaceGroupElement[95].x=0.25-z;        SpaceGroupElement[95].y=-0.25-y;       SpaceGroupElement[95].z=x;
      SpaceGroupElement[96].x=-0.5+x;        SpaceGroupElement[96].y=y;             SpaceGroupElement[96].z=-0.5+z;
      SpaceGroupElement[97].x=-0.25+y;       SpaceGroupElement[97].y=-x;            SpaceGroupElement[97].z=-0.75+z;
      SpaceGroupElement[98].x=0.75-x;        SpaceGroupElement[98].y=0.75-y;        SpaceGroupElement[98].z=z;
      SpaceGroupElement[99].x=0.5-y;         SpaceGroupElement[99].y=-0.25+x;       SpaceGroupElement[99].z=-0.25+z;
      SpaceGroupElement[100].x=-0.5+x;       SpaceGroupElement[100].y=0.25-y;       SpaceGroupElement[100].z=0.75-z;
      SpaceGroupElement[101].x=-0.25+y;      SpaceGroupElement[101].y=-0.75+x;      SpaceGroupElement[101].z=-z;
      SpaceGroupElement[102].x=0.75-x;       SpaceGroupElement[102].y=-0.5+y;       SpaceGroupElement[102].z=0.25-z;
      SpaceGroupElement[103].x=0.5-y;        SpaceGroupElement[103].y=0.5-x;        SpaceGroupElement[103].z=0.5-z;
      SpaceGroupElement[104].x=y;            SpaceGroupElement[104].y=-0.5+z;       SpaceGroupElement[104].z=-0.5+x;
      SpaceGroupElement[105].x=-x;           SpaceGroupElement[105].y=-0.75+z;      SpaceGroupElement[105].z=-0.25+y;
      SpaceGroupElement[106].x=0.75-y;       SpaceGroupElement[106].y=z;            SpaceGroupElement[106].z=0.75-x;
      SpaceGroupElement[107].x=-0.25+x;      SpaceGroupElement[107].y=-0.25+z;      SpaceGroupElement[107].z=0.5-y;
      SpaceGroupElement[108].x=0.25-y;       SpaceGroupElement[108].y=0.75-z;       SpaceGroupElement[108].z=-0.5+x;
      SpaceGroupElement[109].x=-0.75+x;      SpaceGroupElement[109].y=-z;           SpaceGroupElement[109].z=-0.25+y;
      SpaceGroupElement[110].x=-0.5+y;       SpaceGroupElement[110].y=0.25-z;       SpaceGroupElement[110].z=0.75-x;
      SpaceGroupElement[111].x=0.5-x;        SpaceGroupElement[111].y=0.5-z;        SpaceGroupElement[111].z=0.5-y;
      SpaceGroupElement[112].x=-0.5+z;       SpaceGroupElement[112].y=-0.5+x;       SpaceGroupElement[112].z=y;
      SpaceGroupElement[113].x=0.25-z;       SpaceGroupElement[113].y=x;            SpaceGroupElement[113].z=0.25-y;
      SpaceGroupElement[114].x=0.5-z;        SpaceGroupElement[114].y=-0.75+y;      SpaceGroupElement[114].z=-0.75+x;
      SpaceGroupElement[115].x=0.75-z;       SpaceGroupElement[115].y=0.25-x;       SpaceGroupElement[115].z=-0.5+y;
      SpaceGroupElement[116].x=-z;           SpaceGroupElement[116].y=0.5-y;        SpaceGroupElement[116].z=-x;
      SpaceGroupElement[117].x=-0.5+z;       SpaceGroupElement[117].y=0.75-x;       SpaceGroupElement[117].z=0.25-y;
      SpaceGroupElement[118].x=-0.25+z;      SpaceGroupElement[118].y=-y;           SpaceGroupElement[118].z=-0.75+x;
      SpaceGroupElement[119].x=-0.75+z;      SpaceGroupElement[119].y=-0.75+y;      SpaceGroupElement[119].z=0.5-x;
      SpaceGroupElement[120].x=0.5-x;        SpaceGroupElement[120].y=-y;           SpaceGroupElement[120].z=0.5-z;
      SpaceGroupElement[121].x=-0.25-y;      SpaceGroupElement[121].y=x;            SpaceGroupElement[121].z=0.25-z;
      SpaceGroupElement[122].x=-0.25+x;      SpaceGroupElement[122].y=0.75+y;       SpaceGroupElement[122].z=-z;
      SpaceGroupElement[123].x=0.5+y;        SpaceGroupElement[123].y=-0.25-x;      SpaceGroupElement[123].z=-0.25-z;
      SpaceGroupElement[124].x=0.5-x;        SpaceGroupElement[124].y=0.25+y;       SpaceGroupElement[124].z=-0.25+z;
      SpaceGroupElement[125].x=-0.25-y;      SpaceGroupElement[125].y=0.25-x;       SpaceGroupElement[125].z=z;
      SpaceGroupElement[126].x=-0.25+x;      SpaceGroupElement[126].y=-0.5-y;       SpaceGroupElement[126].z=0.25+z;
      SpaceGroupElement[127].x=0.5+y;        SpaceGroupElement[127].y=-0.5+x;       SpaceGroupElement[127].z=-0.5+z;
      SpaceGroupElement[128].x=-y;           SpaceGroupElement[128].y=0.5-z;        SpaceGroupElement[128].z=0.5-x;
      SpaceGroupElement[129].x=x;            SpaceGroupElement[129].y=0.25-z;       SpaceGroupElement[129].z=-0.25-y;
      SpaceGroupElement[130].x=0.75+y;       SpaceGroupElement[130].y=-z;           SpaceGroupElement[130].z=-0.25+x;
      SpaceGroupElement[131].x=-0.25-x;      SpaceGroupElement[131].y=-0.25-z;      SpaceGroupElement[131].z=0.5+y;
      SpaceGroupElement[132].x=0.25+y;       SpaceGroupElement[132].y=-0.25+z;      SpaceGroupElement[132].z=0.5-x;
      SpaceGroupElement[133].x=0.25-x;       SpaceGroupElement[133].y=z;            SpaceGroupElement[133].z=-0.25-y;
      SpaceGroupElement[134].x=-0.5-y;       SpaceGroupElement[134].y=0.25+z;       SpaceGroupElement[134].z=-0.25+x;
      SpaceGroupElement[135].x=-0.5+x;       SpaceGroupElement[135].y=-0.5+z;       SpaceGroupElement[135].z=0.5+y;
      SpaceGroupElement[136].x=0.5-z;        SpaceGroupElement[136].y=0.5-x;        SpaceGroupElement[136].z=-y;
      SpaceGroupElement[137].x=0.25+z;       SpaceGroupElement[137].y=-x;           SpaceGroupElement[137].z=0.25+y;
      SpaceGroupElement[138].x=-0.5+z;       SpaceGroupElement[138].y=-0.75-y;      SpaceGroupElement[138].z=0.25-x;
      SpaceGroupElement[139].x=-0.25+z;      SpaceGroupElement[139].y=0.25+x;       SpaceGroupElement[139].z=-0.5-y;
      SpaceGroupElement[140].x=z;            SpaceGroupElement[140].y=0.5+y;        SpaceGroupElement[140].z=x;
      SpaceGroupElement[141].x=0.5-z;        SpaceGroupElement[141].y=-0.25+x;      SpaceGroupElement[141].z=0.25+y;
      SpaceGroupElement[142].x=-0.25-z;      SpaceGroupElement[142].y=y;            SpaceGroupElement[142].z=0.25-x;
      SpaceGroupElement[143].x=0.25-z;       SpaceGroupElement[143].y=-0.75-y;      SpaceGroupElement[143].z=-0.5+x;
      SpaceGroupElement[144].x=-0.5+x;       SpaceGroupElement[144].y=-0.5+y;       SpaceGroupElement[144].z=z;
      SpaceGroupElement[145].x=-0.75+y;      SpaceGroupElement[145].y=-x;           SpaceGroupElement[145].z=-0.25+z;
      SpaceGroupElement[146].x=0.75-x;       SpaceGroupElement[146].y=0.25-y;       SpaceGroupElement[146].z=-0.5+z;
      SpaceGroupElement[147].x=-y;           SpaceGroupElement[147].y=-0.25+x;      SpaceGroupElement[147].z=-0.75+z;
      SpaceGroupElement[148].x=-0.5+x;       SpaceGroupElement[148].y=0.75-y;       SpaceGroupElement[148].z=0.25-z;
      SpaceGroupElement[149].x=-0.75+y;      SpaceGroupElement[149].y=-0.75+x;      SpaceGroupElement[149].z=0.5-z;
      SpaceGroupElement[150].x=0.75-x;       SpaceGroupElement[150].y=y;            SpaceGroupElement[150].z=0.75-z;
      SpaceGroupElement[151].x=-y;           SpaceGroupElement[151].y=0.5-x;        SpaceGroupElement[151].z=-z;
      SpaceGroupElement[152].x=-0.5+y;       SpaceGroupElement[152].y=z;            SpaceGroupElement[152].z=-0.5+x;
      SpaceGroupElement[153].x=-x;           SpaceGroupElement[153].y=-0.25+z;      SpaceGroupElement[153].z=-0.75+y;
      SpaceGroupElement[154].x=0.25-y;       SpaceGroupElement[154].y=-0.5+z;       SpaceGroupElement[154].z=0.75-x;
      SpaceGroupElement[155].x=-0.25+x;      SpaceGroupElement[155].y=-0.75+z;      SpaceGroupElement[155].z=-y;
      SpaceGroupElement[156].x=0.75-y;       SpaceGroupElement[156].y=0.25-z;       SpaceGroupElement[156].z=-0.5+x;
      SpaceGroupElement[157].x=-0.75+x;      SpaceGroupElement[157].y=0.5-z;        SpaceGroupElement[157].z=-0.75+y;
      SpaceGroupElement[158].x=y;            SpaceGroupElement[158].y=0.75-z;       SpaceGroupElement[158].z=0.75-x;
      SpaceGroupElement[159].x=0.5-x;        SpaceGroupElement[159].y=-z;           SpaceGroupElement[159].z=-y;
      SpaceGroupElement[160].x=z;            SpaceGroupElement[160].y=-0.5+x;       SpaceGroupElement[160].z=-0.5+y;
      SpaceGroupElement[161].x=0.75-z;       SpaceGroupElement[161].y=x;            SpaceGroupElement[161].z=0.75-y;
      SpaceGroupElement[162].x=-z;           SpaceGroupElement[162].y=-0.25+y;      SpaceGroupElement[162].z=-0.75+x;
      SpaceGroupElement[163].x=0.25-z;       SpaceGroupElement[163].y=0.25-x;       SpaceGroupElement[163].z=y;
      SpaceGroupElement[164].x=0.5-z;        SpaceGroupElement[164].y=-y;           SpaceGroupElement[164].z=-x;
      SpaceGroupElement[165].x=z;            SpaceGroupElement[165].y=0.75-x;       SpaceGroupElement[165].z=0.75-y;
      SpaceGroupElement[166].x=-0.75+z;      SpaceGroupElement[166].y=0.5-y;        SpaceGroupElement[166].z=-0.75+x;
      SpaceGroupElement[167].x=-0.25+z;      SpaceGroupElement[167].y=-0.25+y;      SpaceGroupElement[167].z=0.5-x;
      SpaceGroupElement[168].x=0.5-x;        SpaceGroupElement[168].y=0.5-y;        SpaceGroupElement[168].z=-z;
      SpaceGroupElement[169].x=0.25-y;       SpaceGroupElement[169].y=x;            SpaceGroupElement[169].z=-0.25-z;
      SpaceGroupElement[170].x=-0.25+x;      SpaceGroupElement[170].y=0.25+y;       SpaceGroupElement[170].z=-0.5-z;
      SpaceGroupElement[171].x=y;            SpaceGroupElement[171].y=-0.25-x;      SpaceGroupElement[171].z=-0.75-z;
      SpaceGroupElement[172].x=0.5-x;        SpaceGroupElement[172].y=-0.25+y;      SpaceGroupElement[172].z=0.25+z;
      SpaceGroupElement[173].x=0.25-y;       SpaceGroupElement[173].y=0.25-x;       SpaceGroupElement[173].z=0.5+z;
      SpaceGroupElement[174].x=-0.25+x;      SpaceGroupElement[174].y=-y;           SpaceGroupElement[174].z=0.75+z;
      SpaceGroupElement[175].x=y;            SpaceGroupElement[175].y=-0.5+x;       SpaceGroupElement[175].z=z;
      SpaceGroupElement[176].x=0.5-y;        SpaceGroupElement[176].y=-z;           SpaceGroupElement[176].z=0.5-x;
      SpaceGroupElement[177].x=x;            SpaceGroupElement[177].y=-0.25-z;      SpaceGroupElement[177].z=0.25-y;
      SpaceGroupElement[178].x=0.25+y;       SpaceGroupElement[178].y=-0.5-z;       SpaceGroupElement[178].z=-0.25+x;
      SpaceGroupElement[179].x=-0.25-x;      SpaceGroupElement[179].y=-0.75-z;      SpaceGroupElement[179].z=y;
      SpaceGroupElement[180].x=-0.25+y;      SpaceGroupElement[180].y=0.25+z;       SpaceGroupElement[180].z=0.5-x;
      SpaceGroupElement[181].x=0.25-x;       SpaceGroupElement[181].y=0.5+z;        SpaceGroupElement[181].z=0.25-y;
      SpaceGroupElement[182].x=-y;           SpaceGroupElement[182].y=0.75+z;       SpaceGroupElement[182].z=-0.25+x;
      SpaceGroupElement[183].x=-0.5+x;       SpaceGroupElement[183].y=z;            SpaceGroupElement[183].z=y;
      SpaceGroupElement[184].x=-z;           SpaceGroupElement[184].y=0.5-x;        SpaceGroupElement[184].z=0.5-y;
      SpaceGroupElement[185].x=0.75+z;       SpaceGroupElement[185].y=-x;           SpaceGroupElement[185].z=-0.25+y;
      SpaceGroupElement[186].x=z;            SpaceGroupElement[186].y=-0.25-y;      SpaceGroupElement[186].z=0.25-x;
      SpaceGroupElement[187].x=0.25+z;       SpaceGroupElement[187].y=0.25+x;       SpaceGroupElement[187].z=-y;
      SpaceGroupElement[188].x=0.5+z;        SpaceGroupElement[188].y=y;            SpaceGroupElement[188].z=x;
      SpaceGroupElement[189].x=-z;           SpaceGroupElement[189].y=-0.25+x;      SpaceGroupElement[189].z=-0.25+y;
      SpaceGroupElement[190].x=-0.75-z;      SpaceGroupElement[190].y=-0.5+y;       SpaceGroupElement[190].z=0.25-x;
      SpaceGroupElement[191].x=-0.25-z;      SpaceGroupElement[191].y=-0.25-y;      SpaceGroupElement[191].z=-0.5+x;
       break;

    case 529: // [229] I m -3 m (-I 4 2 3)
      SpaceGroupSize=96;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=y;              SpaceGroupElement[1].y=-x;             SpaceGroupElement[1].z=z;
      SpaceGroupElement[2].x=-x;             SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=z;
      SpaceGroupElement[3].x=-y;             SpaceGroupElement[3].y=x;              SpaceGroupElement[3].z=z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=-z;
      SpaceGroupElement[5].x=y;              SpaceGroupElement[5].y=x;              SpaceGroupElement[5].z=-z;
      SpaceGroupElement[6].x=-x;             SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=-y;             SpaceGroupElement[7].y=-x;             SpaceGroupElement[7].z=-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=-x;             SpaceGroupElement[9].y=z;              SpaceGroupElement[9].z=y;
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=z;             SpaceGroupElement[10].z=-x;
      SpaceGroupElement[11].x=x;             SpaceGroupElement[11].y=z;             SpaceGroupElement[11].z=-y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=-z;            SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=x;             SpaceGroupElement[13].y=-z;            SpaceGroupElement[13].z=y;
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-z;            SpaceGroupElement[14].z=-x;
      SpaceGroupElement[15].x=-x;            SpaceGroupElement[15].y=-z;            SpaceGroupElement[15].z=-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=-z;            SpaceGroupElement[17].y=x;             SpaceGroupElement[17].z=-y;
      SpaceGroupElement[18].x=-z;            SpaceGroupElement[18].y=y;             SpaceGroupElement[18].z=x;
      SpaceGroupElement[19].x=-z;            SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=y;
      SpaceGroupElement[20].x=-z;            SpaceGroupElement[20].y=-y;            SpaceGroupElement[20].z=-x;
      SpaceGroupElement[21].x=z;             SpaceGroupElement[21].y=-x;            SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=z;             SpaceGroupElement[22].y=-y;            SpaceGroupElement[22].z=x;
      SpaceGroupElement[23].x=z;             SpaceGroupElement[23].y=y;             SpaceGroupElement[23].z=-x;
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;
      SpaceGroupElement[25].x=-y;            SpaceGroupElement[25].y=x;             SpaceGroupElement[25].z=-z;
      SpaceGroupElement[26].x=x;             SpaceGroupElement[26].y=y;             SpaceGroupElement[26].z=-z;
      SpaceGroupElement[27].x=y;             SpaceGroupElement[27].y=-x;            SpaceGroupElement[27].z=-z;
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=y;             SpaceGroupElement[28].z=z;
      SpaceGroupElement[29].x=-y;            SpaceGroupElement[29].y=-x;            SpaceGroupElement[29].z=z;
      SpaceGroupElement[30].x=x;             SpaceGroupElement[30].y=-y;            SpaceGroupElement[30].z=z;
      SpaceGroupElement[31].x=y;             SpaceGroupElement[31].y=x;             SpaceGroupElement[31].z=z;
      SpaceGroupElement[32].x=-y;            SpaceGroupElement[32].y=-z;            SpaceGroupElement[32].z=-x;
      SpaceGroupElement[33].x=x;             SpaceGroupElement[33].y=-z;            SpaceGroupElement[33].z=-y;
      SpaceGroupElement[34].x=y;             SpaceGroupElement[34].y=-z;            SpaceGroupElement[34].z=x;
      SpaceGroupElement[35].x=-x;            SpaceGroupElement[35].y=-z;            SpaceGroupElement[35].z=y;
      SpaceGroupElement[36].x=y;             SpaceGroupElement[36].y=z;             SpaceGroupElement[36].z=-x;
      SpaceGroupElement[37].x=-x;            SpaceGroupElement[37].y=z;             SpaceGroupElement[37].z=-y;
      SpaceGroupElement[38].x=-y;            SpaceGroupElement[38].y=z;             SpaceGroupElement[38].z=x;
      SpaceGroupElement[39].x=x;             SpaceGroupElement[39].y=z;             SpaceGroupElement[39].z=y;
      SpaceGroupElement[40].x=-z;            SpaceGroupElement[40].y=-x;            SpaceGroupElement[40].z=-y;
      SpaceGroupElement[41].x=z;             SpaceGroupElement[41].y=-x;            SpaceGroupElement[41].z=y;
      SpaceGroupElement[42].x=z;             SpaceGroupElement[42].y=-y;            SpaceGroupElement[42].z=-x;
      SpaceGroupElement[43].x=z;             SpaceGroupElement[43].y=x;             SpaceGroupElement[43].z=-y;
      SpaceGroupElement[44].x=z;             SpaceGroupElement[44].y=y;             SpaceGroupElement[44].z=x;
      SpaceGroupElement[45].x=-z;            SpaceGroupElement[45].y=x;             SpaceGroupElement[45].z=y;
      SpaceGroupElement[46].x=-z;            SpaceGroupElement[46].y=y;             SpaceGroupElement[46].z=-x;
      SpaceGroupElement[47].x=-z;            SpaceGroupElement[47].y=-y;            SpaceGroupElement[47].z=x;
      SpaceGroupElement[48].x=-0.5+x;        SpaceGroupElement[48].y=-0.5+y;        SpaceGroupElement[48].z=-0.5+z;
      SpaceGroupElement[49].x=-0.5+y;        SpaceGroupElement[49].y=0.5-x;         SpaceGroupElement[49].z=-0.5+z;
      SpaceGroupElement[50].x=0.5-x;         SpaceGroupElement[50].y=0.5-y;         SpaceGroupElement[50].z=-0.5+z;
      SpaceGroupElement[51].x=0.5-y;         SpaceGroupElement[51].y=-0.5+x;        SpaceGroupElement[51].z=-0.5+z;
      SpaceGroupElement[52].x=-0.5+x;        SpaceGroupElement[52].y=0.5-y;         SpaceGroupElement[52].z=0.5-z;
      SpaceGroupElement[53].x=-0.5+y;        SpaceGroupElement[53].y=-0.5+x;        SpaceGroupElement[53].z=0.5-z;
      SpaceGroupElement[54].x=0.5-x;         SpaceGroupElement[54].y=-0.5+y;        SpaceGroupElement[54].z=0.5-z;
      SpaceGroupElement[55].x=0.5-y;         SpaceGroupElement[55].y=0.5-x;         SpaceGroupElement[55].z=0.5-z;
      SpaceGroupElement[56].x=-0.5+y;        SpaceGroupElement[56].y=-0.5+z;        SpaceGroupElement[56].z=-0.5+x;
      SpaceGroupElement[57].x=0.5-x;         SpaceGroupElement[57].y=-0.5+z;        SpaceGroupElement[57].z=-0.5+y;
      SpaceGroupElement[58].x=0.5-y;         SpaceGroupElement[58].y=-0.5+z;        SpaceGroupElement[58].z=0.5-x;
      SpaceGroupElement[59].x=-0.5+x;        SpaceGroupElement[59].y=-0.5+z;        SpaceGroupElement[59].z=0.5-y;
      SpaceGroupElement[60].x=0.5-y;         SpaceGroupElement[60].y=0.5-z;         SpaceGroupElement[60].z=-0.5+x;
      SpaceGroupElement[61].x=-0.5+x;        SpaceGroupElement[61].y=0.5-z;         SpaceGroupElement[61].z=-0.5+y;
      SpaceGroupElement[62].x=-0.5+y;        SpaceGroupElement[62].y=0.5-z;         SpaceGroupElement[62].z=0.5-x;
      SpaceGroupElement[63].x=0.5-x;         SpaceGroupElement[63].y=0.5-z;         SpaceGroupElement[63].z=0.5-y;
      SpaceGroupElement[64].x=-0.5+z;        SpaceGroupElement[64].y=-0.5+x;        SpaceGroupElement[64].z=-0.5+y;
      SpaceGroupElement[65].x=0.5-z;         SpaceGroupElement[65].y=-0.5+x;        SpaceGroupElement[65].z=0.5-y;
      SpaceGroupElement[66].x=0.5-z;         SpaceGroupElement[66].y=-0.5+y;        SpaceGroupElement[66].z=-0.5+x;
      SpaceGroupElement[67].x=0.5-z;         SpaceGroupElement[67].y=0.5-x;         SpaceGroupElement[67].z=-0.5+y;
      SpaceGroupElement[68].x=0.5-z;         SpaceGroupElement[68].y=0.5-y;         SpaceGroupElement[68].z=0.5-x;
      SpaceGroupElement[69].x=-0.5+z;        SpaceGroupElement[69].y=0.5-x;         SpaceGroupElement[69].z=0.5-y;
      SpaceGroupElement[70].x=-0.5+z;        SpaceGroupElement[70].y=0.5-y;         SpaceGroupElement[70].z=-0.5+x;
      SpaceGroupElement[71].x=-0.5+z;        SpaceGroupElement[71].y=-0.5+y;        SpaceGroupElement[71].z=0.5-x;
      SpaceGroupElement[72].x=0.5-x;         SpaceGroupElement[72].y=0.5-y;         SpaceGroupElement[72].z=0.5-z;
      SpaceGroupElement[73].x=0.5-y;         SpaceGroupElement[73].y=-0.5+x;        SpaceGroupElement[73].z=0.5-z;
      SpaceGroupElement[74].x=-0.5+x;        SpaceGroupElement[74].y=-0.5+y;        SpaceGroupElement[74].z=0.5-z;
      SpaceGroupElement[75].x=-0.5+y;        SpaceGroupElement[75].y=0.5-x;         SpaceGroupElement[75].z=0.5-z;
      SpaceGroupElement[76].x=0.5-x;         SpaceGroupElement[76].y=-0.5+y;        SpaceGroupElement[76].z=-0.5+z;
      SpaceGroupElement[77].x=0.5-y;         SpaceGroupElement[77].y=0.5-x;         SpaceGroupElement[77].z=-0.5+z;
      SpaceGroupElement[78].x=-0.5+x;        SpaceGroupElement[78].y=0.5-y;         SpaceGroupElement[78].z=-0.5+z;
      SpaceGroupElement[79].x=-0.5+y;        SpaceGroupElement[79].y=-0.5+x;        SpaceGroupElement[79].z=-0.5+z;
      SpaceGroupElement[80].x=0.5-y;         SpaceGroupElement[80].y=0.5-z;         SpaceGroupElement[80].z=0.5-x;
      SpaceGroupElement[81].x=-0.5+x;        SpaceGroupElement[81].y=0.5-z;         SpaceGroupElement[81].z=0.5-y;
      SpaceGroupElement[82].x=-0.5+y;        SpaceGroupElement[82].y=0.5-z;         SpaceGroupElement[82].z=-0.5+x;
      SpaceGroupElement[83].x=0.5-x;         SpaceGroupElement[83].y=0.5-z;         SpaceGroupElement[83].z=-0.5+y;
      SpaceGroupElement[84].x=-0.5+y;        SpaceGroupElement[84].y=-0.5+z;        SpaceGroupElement[84].z=0.5-x;
      SpaceGroupElement[85].x=0.5-x;         SpaceGroupElement[85].y=-0.5+z;        SpaceGroupElement[85].z=0.5-y;
      SpaceGroupElement[86].x=0.5-y;         SpaceGroupElement[86].y=-0.5+z;        SpaceGroupElement[86].z=-0.5+x;
      SpaceGroupElement[87].x=-0.5+x;        SpaceGroupElement[87].y=-0.5+z;        SpaceGroupElement[87].z=-0.5+y;
      SpaceGroupElement[88].x=0.5-z;         SpaceGroupElement[88].y=0.5-x;         SpaceGroupElement[88].z=0.5-y;
      SpaceGroupElement[89].x=-0.5+z;        SpaceGroupElement[89].y=0.5-x;         SpaceGroupElement[89].z=-0.5+y;
      SpaceGroupElement[90].x=-0.5+z;        SpaceGroupElement[90].y=0.5-y;         SpaceGroupElement[90].z=0.5-x;
      SpaceGroupElement[91].x=-0.5+z;        SpaceGroupElement[91].y=-0.5+x;        SpaceGroupElement[91].z=0.5-y;
      SpaceGroupElement[92].x=-0.5+z;        SpaceGroupElement[92].y=-0.5+y;        SpaceGroupElement[92].z=-0.5+x;
      SpaceGroupElement[93].x=0.5-z;         SpaceGroupElement[93].y=-0.5+x;        SpaceGroupElement[93].z=-0.5+y;
      SpaceGroupElement[94].x=0.5-z;         SpaceGroupElement[94].y=-0.5+y;        SpaceGroupElement[94].z=0.5-x;
      SpaceGroupElement[95].x=0.5-z;         SpaceGroupElement[95].y=0.5-y;         SpaceGroupElement[95].z=-0.5+x;
      break;

    case 530: // [230] I a -3 d (-I 4bd 2c 3)
      SpaceGroupSize=96;
      //x-symmetry-operation                 y-symmetry-operation                   z-symmetry-operation
      SpaceGroupElement[0].x=x;              SpaceGroupElement[0].y=y;              SpaceGroupElement[0].z=z;
      SpaceGroupElement[1].x=-0.75+y;        SpaceGroupElement[1].y=0.25-x;         SpaceGroupElement[1].z=-0.25+z;
      SpaceGroupElement[2].x=0.5-x;          SpaceGroupElement[2].y=-y;             SpaceGroupElement[2].z=-0.5+z;
      SpaceGroupElement[3].x=0.25-y;         SpaceGroupElement[3].y=-0.25+x;        SpaceGroupElement[3].z=-0.75+z;
      SpaceGroupElement[4].x=x;              SpaceGroupElement[4].y=-y;             SpaceGroupElement[4].z=0.5-z;
      SpaceGroupElement[5].x=-0.75+y;        SpaceGroupElement[5].y=-0.25+x;        SpaceGroupElement[5].z=0.75-z;
      SpaceGroupElement[6].x=0.5-x;          SpaceGroupElement[6].y=y;              SpaceGroupElement[6].z=-z;
      SpaceGroupElement[7].x=0.25-y;         SpaceGroupElement[7].y=0.25-x;         SpaceGroupElement[7].z=0.25-z;
      SpaceGroupElement[8].x=y;              SpaceGroupElement[8].y=z;              SpaceGroupElement[8].z=x;
      SpaceGroupElement[9].x=0.25-x;         SpaceGroupElement[9].y=-0.25+z;        SpaceGroupElement[9].z=-0.75+y;
      SpaceGroupElement[10].x=-y;            SpaceGroupElement[10].y=-0.5+z;        SpaceGroupElement[10].z=0.5-x;
      SpaceGroupElement[11].x=-0.25+x;       SpaceGroupElement[11].y=-0.75+z;       SpaceGroupElement[11].z=0.25-y;
      SpaceGroupElement[12].x=-y;            SpaceGroupElement[12].y=0.5-z;         SpaceGroupElement[12].z=x;
      SpaceGroupElement[13].x=-0.25+x;       SpaceGroupElement[13].y=0.75-z;        SpaceGroupElement[13].z=-0.75+y;
      SpaceGroupElement[14].x=y;             SpaceGroupElement[14].y=-z;            SpaceGroupElement[14].z=0.5-x;
      SpaceGroupElement[15].x=0.25-x;        SpaceGroupElement[15].y=0.25-z;        SpaceGroupElement[15].z=0.25-y;
      SpaceGroupElement[16].x=z;             SpaceGroupElement[16].y=x;             SpaceGroupElement[16].z=y;
      SpaceGroupElement[17].x=-z;            SpaceGroupElement[17].y=-0.5+x;        SpaceGroupElement[17].z=0.5-y;
      SpaceGroupElement[18].x=0.25-z;        SpaceGroupElement[18].y=-0.25+y;       SpaceGroupElement[18].z=-0.75+x;
      SpaceGroupElement[19].x=0.5-z;         SpaceGroupElement[19].y=-x;            SpaceGroupElement[19].z=-0.5+y;
      SpaceGroupElement[20].x=0.25-z;        SpaceGroupElement[20].y=0.25-y;        SpaceGroupElement[20].z=0.25-x;
      SpaceGroupElement[21].x=-0.5+z;        SpaceGroupElement[21].y=0.5-x;         SpaceGroupElement[21].z=-y;
      SpaceGroupElement[22].x=-0.25+z;       SpaceGroupElement[22].y=0.75-y;        SpaceGroupElement[22].z=-0.75+x;
      SpaceGroupElement[23].x=-0.75+z;       SpaceGroupElement[23].y=-0.25+y;       SpaceGroupElement[23].z=0.75-x;
      SpaceGroupElement[24].x=-x;            SpaceGroupElement[24].y=-y;            SpaceGroupElement[24].z=-z;
      SpaceGroupElement[25].x=-0.75-y;       SpaceGroupElement[25].y=0.25+x;        SpaceGroupElement[25].z=-0.25-z;
      SpaceGroupElement[26].x=0.5+x;         SpaceGroupElement[26].y=y;             SpaceGroupElement[26].z=-0.5-z;
      SpaceGroupElement[27].x=0.25+y;        SpaceGroupElement[27].y=-0.25-x;       SpaceGroupElement[27].z=-0.75-z;
      SpaceGroupElement[28].x=-x;            SpaceGroupElement[28].y=y;             SpaceGroupElement[28].z=0.5+z;
      SpaceGroupElement[29].x=-0.75-y;       SpaceGroupElement[29].y=-0.25-x;       SpaceGroupElement[29].z=0.75+z;
      SpaceGroupElement[30].x=0.5+x;         SpaceGroupElement[30].y=-y;            SpaceGroupElement[30].z=z;
      SpaceGroupElement[31].x=0.25+y;        SpaceGroupElement[31].y=0.25+x;        SpaceGroupElement[31].z=0.25+z;
      SpaceGroupElement[32].x=-y;            SpaceGroupElement[32].y=-z;            SpaceGroupElement[32].z=-x;
      SpaceGroupElement[33].x=0.25+x;        SpaceGroupElement[33].y=-0.25-z;       SpaceGroupElement[33].z=-0.75-y;
      SpaceGroupElement[34].x=y;             SpaceGroupElement[34].y=-0.5-z;        SpaceGroupElement[34].z=0.5+x;
      SpaceGroupElement[35].x=-0.25-x;       SpaceGroupElement[35].y=-0.75-z;       SpaceGroupElement[35].z=0.25+y;
      SpaceGroupElement[36].x=y;             SpaceGroupElement[36].y=0.5+z;         SpaceGroupElement[36].z=-x;
      SpaceGroupElement[37].x=-0.25-x;       SpaceGroupElement[37].y=0.75+z;        SpaceGroupElement[37].z=-0.75-y;
      SpaceGroupElement[38].x=-y;            SpaceGroupElement[38].y=z;             SpaceGroupElement[38].z=0.5+x;
      SpaceGroupElement[39].x=0.25+x;        SpaceGroupElement[39].y=0.25+z;        SpaceGroupElement[39].z=0.25+y;
      SpaceGroupElement[40].x=-z;            SpaceGroupElement[40].y=-x;            SpaceGroupElement[40].z=-y;
      SpaceGroupElement[41].x=z;             SpaceGroupElement[41].y=-0.5-x;        SpaceGroupElement[41].z=0.5+y;
      SpaceGroupElement[42].x=0.25+z;        SpaceGroupElement[42].y=-0.25-y;       SpaceGroupElement[42].z=-0.75-x;
      SpaceGroupElement[43].x=0.5+z;         SpaceGroupElement[43].y=x;             SpaceGroupElement[43].z=-0.5-y;
      SpaceGroupElement[44].x=0.25+z;        SpaceGroupElement[44].y=0.25+y;        SpaceGroupElement[44].z=0.25+x;
      SpaceGroupElement[45].x=-0.5-z;        SpaceGroupElement[45].y=0.5+x;         SpaceGroupElement[45].z=y;
      SpaceGroupElement[46].x=-0.25-z;       SpaceGroupElement[46].y=0.75+y;        SpaceGroupElement[46].z=-0.75-x;
      SpaceGroupElement[47].x=-0.75-z;       SpaceGroupElement[47].y=-0.25-y;       SpaceGroupElement[47].z=0.75+x;
      SpaceGroupElement[48].x=-0.5+x;        SpaceGroupElement[48].y=-0.5+y;        SpaceGroupElement[48].z=-0.5+z;
      SpaceGroupElement[49].x=-0.25+y;       SpaceGroupElement[49].y=0.75-x;        SpaceGroupElement[49].z=-0.75+z;
      SpaceGroupElement[50].x=-x;            SpaceGroupElement[50].y=0.5-y;         SpaceGroupElement[50].z=z;
      SpaceGroupElement[51].x=0.75-y;        SpaceGroupElement[51].y=-0.75+x;       SpaceGroupElement[51].z=-0.25+z;
      SpaceGroupElement[52].x=-0.5+x;        SpaceGroupElement[52].y=0.5-y;         SpaceGroupElement[52].z=-z;
      SpaceGroupElement[53].x=-0.25+y;       SpaceGroupElement[53].y=-0.75+x;       SpaceGroupElement[53].z=0.25-z;
      SpaceGroupElement[54].x=-x;            SpaceGroupElement[54].y=-0.5+y;        SpaceGroupElement[54].z=0.5-z;
      SpaceGroupElement[55].x=0.75-y;        SpaceGroupElement[55].y=0.75-x;        SpaceGroupElement[55].z=0.75-z;
      SpaceGroupElement[56].x=-0.5+y;        SpaceGroupElement[56].y=-0.5+z;        SpaceGroupElement[56].z=-0.5+x;
      SpaceGroupElement[57].x=0.75-x;        SpaceGroupElement[57].y=-0.75+z;       SpaceGroupElement[57].z=-0.25+y;
      SpaceGroupElement[58].x=0.5-y;         SpaceGroupElement[58].y=z;             SpaceGroupElement[58].z=-x;
      SpaceGroupElement[59].x=-0.75+x;       SpaceGroupElement[59].y=-0.25+z;       SpaceGroupElement[59].z=0.75-y;
      SpaceGroupElement[60].x=0.5-y;         SpaceGroupElement[60].y=-z;            SpaceGroupElement[60].z=-0.5+x;
      SpaceGroupElement[61].x=-0.75+x;       SpaceGroupElement[61].y=0.25-z;        SpaceGroupElement[61].z=-0.25+y;
      SpaceGroupElement[62].x=-0.5+y;        SpaceGroupElement[62].y=0.5-z;         SpaceGroupElement[62].z=-x;
      SpaceGroupElement[63].x=0.75-x;        SpaceGroupElement[63].y=0.75-z;        SpaceGroupElement[63].z=0.75-y;
      SpaceGroupElement[64].x=-0.5+z;        SpaceGroupElement[64].y=-0.5+x;        SpaceGroupElement[64].z=-0.5+y;
      SpaceGroupElement[65].x=0.5-z;         SpaceGroupElement[65].y=x;             SpaceGroupElement[65].z=-y;
      SpaceGroupElement[66].x=0.75-z;        SpaceGroupElement[66].y=-0.75+y;       SpaceGroupElement[66].z=-0.25+x;
      SpaceGroupElement[67].x=-z;            SpaceGroupElement[67].y=0.5-x;         SpaceGroupElement[67].z=y;
      SpaceGroupElement[68].x=0.75-z;        SpaceGroupElement[68].y=0.75-y;        SpaceGroupElement[68].z=0.75-x;
      SpaceGroupElement[69].x=z;             SpaceGroupElement[69].y=-x;            SpaceGroupElement[69].z=0.5-y;
      SpaceGroupElement[70].x=-0.75+z;       SpaceGroupElement[70].y=0.25-y;        SpaceGroupElement[70].z=-0.25+x;
      SpaceGroupElement[71].x=-0.25+z;       SpaceGroupElement[71].y=-0.75+y;       SpaceGroupElement[71].z=0.25-x;
      SpaceGroupElement[72].x=0.5-x;         SpaceGroupElement[72].y=0.5-y;         SpaceGroupElement[72].z=0.5-z;
      SpaceGroupElement[73].x=-0.25-y;       SpaceGroupElement[73].y=-0.25+x;       SpaceGroupElement[73].z=0.25-z;
      SpaceGroupElement[74].x=x;             SpaceGroupElement[74].y=-0.5+y;        SpaceGroupElement[74].z=-z;
      SpaceGroupElement[75].x=-0.25+y;       SpaceGroupElement[75].y=0.25-x;        SpaceGroupElement[75].z=-0.25-z;
      SpaceGroupElement[76].x=0.5-x;         SpaceGroupElement[76].y=-0.5+y;        SpaceGroupElement[76].z=z;
      SpaceGroupElement[77].x=-0.25-y;       SpaceGroupElement[77].y=0.25-x;        SpaceGroupElement[77].z=0.25+z;
      SpaceGroupElement[78].x=x;             SpaceGroupElement[78].y=0.5-y;         SpaceGroupElement[78].z=-0.5+z;
      SpaceGroupElement[79].x=-0.25+y;       SpaceGroupElement[79].y=-0.25+x;       SpaceGroupElement[79].z=-0.25+z;
      SpaceGroupElement[80].x=0.5-y;         SpaceGroupElement[80].y=0.5-z;         SpaceGroupElement[80].z=0.5-x;
      SpaceGroupElement[81].x=-0.25+x;       SpaceGroupElement[81].y=0.25-z;        SpaceGroupElement[81].z=-0.25-y;
      SpaceGroupElement[82].x=-0.5+y;        SpaceGroupElement[82].y=-z;            SpaceGroupElement[82].z=x;
      SpaceGroupElement[83].x=0.25-x;        SpaceGroupElement[83].y=-0.25-z;       SpaceGroupElement[83].z=-0.25+y;
      SpaceGroupElement[84].x=-0.5+y;        SpaceGroupElement[84].y=z;             SpaceGroupElement[84].z=0.5-x;
      SpaceGroupElement[85].x=0.25-x;        SpaceGroupElement[85].y=0.25+z;        SpaceGroupElement[85].z=-0.25-y;
      SpaceGroupElement[86].x=0.5-y;         SpaceGroupElement[86].y=-0.5+z;        SpaceGroupElement[86].z=x;
      SpaceGroupElement[87].x=-0.25+x;       SpaceGroupElement[87].y=-0.25+z;       SpaceGroupElement[87].z=-0.25+y;
      SpaceGroupElement[88].x=0.5-z;         SpaceGroupElement[88].y=0.5-x;         SpaceGroupElement[88].z=0.5-y;
      SpaceGroupElement[89].x=-0.5+z;        SpaceGroupElement[89].y=-x;            SpaceGroupElement[89].z=y;
      SpaceGroupElement[90].x=-0.25+z;       SpaceGroupElement[90].y=0.25-y;        SpaceGroupElement[90].z=-0.25-x;
      SpaceGroupElement[91].x=z;             SpaceGroupElement[91].y=-0.5+x;        SpaceGroupElement[91].z=-y;
      SpaceGroupElement[92].x=-0.25+z;       SpaceGroupElement[92].y=-0.25+y;       SpaceGroupElement[92].z=-0.25+x;
      SpaceGroupElement[93].x=-z;            SpaceGroupElement[93].y=x;             SpaceGroupElement[93].z=-0.5+y;
      SpaceGroupElement[94].x=0.25-z;        SpaceGroupElement[94].y=0.25+y;        SpaceGroupElement[94].z=-0.25-x;
      SpaceGroupElement[95].x=-0.25-z;       SpaceGroupElement[95].y=0.25-y;        SpaceGroupElement[95].z=0.25+x;
      break;
    default:
      SpaceGroupSize=0;
      break;
  }
}


int AsymmetricUnit(int SpaceGroup,VECTOR p,REAL eps)
{
  int asymmetric;

  asymmetric=FALSE;

  switch(SpaceGroup)
  {
    // TRICLINIC GROUPS
    // ================

    case 1:   // [1] P 1 (P 1)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 2:   // [2] P -1 (-P 1)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;


    // MONOCLINIC GROUPS
    // =================

    case 3: // [3] P 1 2 1 unique b axis (P 2y)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 4: // [3] P 1 1 2 unique c axis (P 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 5: // [3] P 2 1 1 unique a axis (P 2x)
       // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P211.html
       if(RANGE2LR(p.x,0.0,1.0,eps)&&
          RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
          RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 6: // [4] P 1 21 1 unique b axis (P 2yb)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 7: // [4] P 1 1 21 unique c axis (P 2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 8: // [4] P 21 1 1 unique a axis (P 2xa)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P2111.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 9:  // [5] C 1 2 1 unique b axis: cell choice 1 (C 2y)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 10: // [5] A 1 2 1 unique b axis: cell choice 2 (A 2y)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 11: // [5] I 1 2 1 unique b axis: cell choice 3 (I 2y)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 12: // [5] A 1 1 2 unique c axis: cell choice 1 (A 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 13: // [5] B 1 1 2 unique c axis: cell choice 2 (B 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 14: // [5] I 1 1 2 unique c axis: cell choice 3 (I 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 15: // [5] B 2 1 1 unique a axis: cell choice 1 (B 2x)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/B211.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 16: // [5] C 2 1 1 unique a axis: cell choice 2 (C 2x)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/C211.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 17: // [5] I 2 1 1 unique a axis: cell choice 3 (I 2x)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/I211.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 18: // [6] P 1 m 1 unique b axis (P -2y)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 19: // [6] P 1 1 m unique c axis (P -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 20: // [6] P m 1 1 unique a axis (P -2x)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pm11.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 21: // [7] P 1 c 1 unique b axis: cell choice 1 (P -2yc)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 22: // [7] P 1 n 1 unique b axis: cell choice 2 (P -2yac)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 23: // [7] P 1 a 1 unique b axis: cell choice 3 (P -2ya)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 24: // [7] P 1 1 a unique c axis: cell choice 1 (P -2a)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 25: // [7] P 1 1 n unique c axis: cell choice 2 (P -2ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 26: // [7] P 1 1 b unique c axis: cell choice 3 (P -2b)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 27: // [7] P b 1 1 unique a axis: cell choice 1 (P -2xb)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pb11.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 28: // [7] P n 1 1 unique a axis: cell choice 2 (P -2xbc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pn11.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 29: // [7] P c 1 1 unique a axis: cell choice 3 (P -2xc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pc11.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 30: // [8] C 1 m 1 unique b axis: cell choice 1 (C -2y)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 31: // [8] A 1 m 1 unique b axis: cell choice 2 (A -2y)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 32: // [8] I 1 m 1 unique b axis: cell choice 3 (I -2y)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 33: // [8] A 1 1 m unique c axis: cell choice 1 (A -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 34: // [8] B 1 1 m  unique c axis: cell choice 2 (B -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 35: // [8] I 1 1 m unique c axis: cell choice 3 (I -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 36: // [8] B m 1 1 unique a axis: cell choice 1 (B -2x)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bm11.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 37: // [8] C m 1 1 unique a axis: cell choice 2 (C -2x)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Cm11.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 38: // [8] I m 1 1 unique a axis: cell choice 3 (I -2x)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Im11.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 39: // [9] C 1 c 1 unique b axis: cell choice 1 (C -2yc)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 40: // [9] A 1 n 1 unique b axis: cell choice 2 (A -2yab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 41: // [9] I 1 a 1 unique b axis: cell choice 3 (I -2ya)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 42: // [9] A 1 a 1 unique -b axis: cell choice 1 (A -2ya)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/A1a1.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 43: // [9] C 1 n 1 unique -b axis: cell choice 2 (C -2yac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/C1n1.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 44: // [9] I 1 c 1 unique -b axis: cell choice 3 (I -2yc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/I1c1.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 45: // [9] A 1 1 a unique c axis: cell choice 1 (A -2a)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 46: // [9] B 1 1 n unique c axis: cell choice 2 (B -2ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 47: // [9] I 1 1 b unique c axis: cell choice 3 (I -2b)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 48: // [9] B 1 1 b unique -c axis: cell choice 1 (B -2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/B11b.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 49: // [9] A 1 1 n unique -c axis: cell choice 2 (A -2ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/A11n.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 50: // [9] I 1 1 a unique -c axis: cell choice 3 (I -2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/I11a.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 51: // [9] B b 1 1 unique a axis: cell choice 1 (B -2xb)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bb11.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 52: // [9] C n 1 1 unique a axis: cell choice 2 (C -2xac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Cn11.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 53: // [9] I c 1 1 unique a axis: cell choice 3 (I -2xc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Ic11.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 54: // [9] C c 1 1 unique -a axis: cell choice 1 (C -2xc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Cc11.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 55: // [9] B n 1 1 unique -a axis: cell choice 2 (B -2xab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bn11.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 56: // [9] I b 1 1 unique -a axis: cell choice 3 (I -2xb)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Ib11.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 57: // [10] P 1 2/m 1 unique b axis (-P 2y)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 58: // [10] P 1 1 2/m unique c axis (-P 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 59: // [10] P 2/m 1 1 unique a axis (-P 2x)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P2_m11.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 60: // [11] P 1 21/m 1 unique axis b (-P 2yb)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 61: // [11] P 1 1 21/m unique c axis (-P 2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 62: // [11] P 21/m 1 1 unique a axis (-P 2xa)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P21_m11.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 63: // [12] C 1 2/m 1 unique b axis: cell choice 1 (-C 2y)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 64: // [12] A 1 2/m 1 unique b axis: cell choice 2 (-A 2y)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 65: // [12] I 1 2/m 1 unique b axis: cell choice 3 (-I 2y)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 66: // [12] A 1 1 2/m unique c axis: cell choice 1 (-A 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 67: // [12] B 1 1 2/m unique c axis: cell choice 2 (-B 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 68: // [12] I 1 1 2/m unique c axis: cell choice 3 (-I 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 69: // [12] B 2/m 1 1 unique a axis: cell choice 1 (-B 2x)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/B2_m11.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 70: // [12] C 2/m 1 1 unique a axis: cell choice 2 (-C 2x)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/C2_m11.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 71: // [12] I 2/m 1 1 unique a axis: cell choice 3 (-I 2x)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/I2_m11.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 72: // [13] P 1 2/c 1 unique b axis: cell choice 1 (-P 2yc)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 73: // [13] P 1 2/n 1 unique b axis: cell choice 2 (-P 2yac)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 74: // [13] P 1 2/a 1 unique b axis: cell choice 3 (-P 2ya)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 75: // [13] P 1 1 2/a unique c axis: cell choice 1 (-P 2a)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 76: // [13] P 1 1 2/n unique c axis: cell choice 2 (-P 2ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 77: // [13] P 1 1 2/b unique c axis: cell choice 3 (-P 2b)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 78: // [13] P 2/b 1 1 unique a axis: cell choice 1 (-P 2xb)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P2_b11.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 79: // [13] P 2/n 1 1 unique a axis: cell choice 2 (-P 2xbc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P2_n11.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 80: // [13] P 2/c 1 1 unique a axis: cell choice 3 (-P 2xc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P2_c11.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 81: // [14] P 1 21/c 1 unique b axis: cell choice 1 (-P 2ybc)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 82: // [14] P 1 21/n 1 unique b axis: cell choice 2 (-P 2yn)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 83: // [14] P 1 21/a 1 unique b axis: cell choice 3 (-P 2yab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 84: // [14] P 1 1 21/a unique c axis: cell choice 1 (-P 2ac)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 85: // [14] P 1 1 21/n unique c axis: cell choice 2 (-P 2n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 86: // [14] P 1 1 21/b unique c axis: cell choice 3 (-P 2bc)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 87: // [14] P 21/b 1 1 unique a axis: cell choice 1 (-P 2xab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P21_b11.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 88: // [14] P 21/n 1 1 unique a axis: cell choice 2 (-P 2xn)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P21_n11.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 89: // [14] P 21/c 1 1 unique a axis: cell choice 3 (-P 2xac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P21_c11.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 90: // [15] C 1 2/c 1 unique b axis: cell choice 1 (-C 2yc)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 91: // [15] A 1 2/n 1 unique b axis: cell choice 2 (-A 2yab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 92: // [15] I 1 2/a 1 unique b axis: cell choice 3 (-I 2ya)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 93: // [15] A 1 2/a 1 unique -b axis: cell choice 1 (-A 2ya)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/A12_a1.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 94: // [15] C 1 2/n 1 unique -b axis: cell choice 2 (-C 2yac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/C12_n1.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,1.0/4.0,3.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 95: // [15] I 1 2/c 1 unique -b axis: cell choice 3 (-I 2yc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/I12_c1.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 96: // [15] A 1 1 2/a unique c axis: cell choice 1 (-A 2a)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 97: // [15] B 1 1 2/n unique c axis: cell choice 2 (-B 2ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 98: // [15] I 1 1 2/b unique c axis: cell choice 3 (-I 2b)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 99: // [15] B 1 1 2/b unique -c axis: cell choice 1 (-B 2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/B112_b.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 100: // [15] A 1 1 2/n unique -c axis: cell choice 2 (-A 2ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/A112_n.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 101: // [15] I 1 1 2/a unique -c axis: cell choice 3 (-I 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/I112_a.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 102: // [15] B 2/b 1 1 unique a axis: cell choice 1 (-B 2xb)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/B2_b11.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 103: // [15] C 2/n 1 1 unique a axis: cell choice 2 (-C 2xac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/C2_n11.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,1.0/4.0,3.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 104: // [15] I 2/c 1 1 unique a axis: cell choice 3 (-I 2xc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/I2_c11.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 105: // [15] C 2/c 1 1 unique -a axis: cell choice 1 (-C 2xc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/C2_c11.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 106: // [15] B 2/n 1 1 unique -a axis: cell choice 2 (-B 2xab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/B2_n11.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 107: // [15] I 2/b 1 1 unique -a axis: cell choice 3 (-I 2xb)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/I2_b11.html
      if(RANGE2LR(p.x,1.0/4.0,3.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;


    // ORTHORHOMBIC GROUPS
    // ===================

    case 108: // [16] P 2 2 2 (P 2 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 109: // [17] P 2 2 21 Origin-1,abc (P 2c 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 110: // [17] P 21 2 2 Origin-1,cab (P 2a 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P2122.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 111: // [17] P 2 21 2 Origin-1,bca (P 2 2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P2212.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 112: // [18] P 21 21 2 Origin-1,abc (P 2 2ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 113: // [18] P 2 21 21 Origin-1,cab (P 2bc 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P22121.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 114: // [18] P 21 2 21 Origin-1,bca (P 2ac 2ac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P21221.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 115: // [19] P 21 21 21 (P 2ac 2ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 116: // [20] C 2 2 21  Origin-1,abc (C 2c 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 117: // [20] A 21 2 2  Origin-1,cba (A 2a 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/A2122.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 118: // [20] B 2 21 2  Origin-1,bca (B 2 2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/B2212.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 119: // [21] C 2 2 2 Origin-1,abc (C 2 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 120: // [21] A 2 2 2 Origin-1,cab (A 2 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/A222.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 121: // [21] B 2 2 2 Origin-1,bca (B 2 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/B222.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 122: // [22] F 2 2 2 (F 2 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 123: // [23] I 2 2 2 (I 2 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 124: // [24] I 21 21 21 (I 2b 2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 125: // [25] P m m 2 (P 2 -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 126: // [25] P 2 m m (P -2 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P2mm.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 127: // [25] P m 2 m (P -2 -2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pm2m.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 128: // [26] P m c 21 (P 2c -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 129: // [26] P c m 21 (P 2c -2c)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pcm21.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 130: // [26] P 21 m a (P -2a 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P21ma.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 131: // [26] P 21 a m (P -2 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P21ma.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 132: // [26] P b 21 m (P -2 -2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pb21m.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 133: // [26] P m 21 b (P -2b -2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pm21b.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 134: // [27] P c c 2 (P 2 -2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 135: // [27] P 2 a a (P -2a 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P2aa.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 136: // [27] P b 2 b (P -2b -2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pb2b.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 137: // [28] P m a 2 (P 2 -2a)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 138: // [28] P b m 2 (P 2 -2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pbm2.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 139: // [28] P 2 m b (P -2b 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P2mb.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 140: // [28] P 2 c m (P -2c 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P2cm.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 141: // [28] P c 2 m (P -2c -2c)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pc2m.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 142: // [28] P m 2 a (P -2a -2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pm2a.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 143: // [29] P c a 21 (P 2c -2ac)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 144: // [29] P b c 21 (P 2c -2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pbc21.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 145: // [29] P 21 a b (P -2b 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P21ab.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 146: // [29] P 21 c a (P -2ac 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P21ca.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 147: // [29] P c 21 b (P -2bc -2c)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P21ab.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 148: // [29] P b 21 a (P -2a -2ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P21ab.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 149: // [30] P n c 2 (P 2 -2bc)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 150: // [30] P c n 2 (P 2 -2ac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pcn2.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 151: // [30] P 2 n a (P -2ac 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P2na.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 152: // [30] P 2 a n (P -2ab 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P2an.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 153: // [30] P b 2 n (P -2ab -2ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pb2n.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 154: // [30] P n 2 b (P -2bc -2bc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pn2b.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 155: // [31] P m n 21 (P 2ac -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 156: // [31] P n m 21 (P 2bc -2bc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pnm21.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 157: // [31] P 21 m n (P -2ab 2ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P21mn.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 158: // [31] P 21 n m (P -2 2ac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P21nm.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 159: // [31] P n 21 m (P -2 -2bc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pn21m.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 160: // [31] P m 21 n (P -2ab -2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pm21n.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 161: // [32] P b a 2 (P 2 -2ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 162: // [32] P 2 c b (P -2bc 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P2cb.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 163: // [32] P c 2 a (P -2ac -2ac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pc2a.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 164: // [33] P n a 21 (P 2c -2n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 165: // [33] P b n 21 (P 2c -2ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pbn21.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 166: // [33] P 21 n b (P -2bc 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P21nb.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 167: // [33] P 21 c n (P -2n 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P21cn.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 168: // [33] P c 21 n (P -2n -2ac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P21cn.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 169: // [33] P n 21 a (P -2ac -2n)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pn21a.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 170: // [34] P n n 2 (P 2 -2n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 171: // [34] P 2 n n (P -2n 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/P2nn.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 172: // [34] P n 2 n (P -2n -2n)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pn2n.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 173: // [35] C m m 2 (C 2 -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 174: // [35] A 2 m m (A -2 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/A2mm.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 175: // [35] B m 2 m (B -2 -2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bm2m.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 176: // [36] C m c 21 (C 2c -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 177: // [36] C c m 21 (C 2c -2c)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Ccm21.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 178: // [36] A 21 m a (A -2a 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/A21ma.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 179: // [36] A 21 a m (A -2 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/A21am.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 180: // [36] B b 21 m (B -2 -2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bb21m.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 181: // [36] B m 21 b (B -2b -2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bm21b.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 182: // [37] C c c 2 (C 2 -2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 183: // [37] A 2 a a (A -2a 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/A2aa.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 184: // [37] B b 2 b (B -2b -2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bb2b.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 185: // [38] A m m 2 (A 2 -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 186: // [38] B m m 2 (B 2 -2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bmm2.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 187: // [38] B 2 m m (B -2 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/B2mm.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 188: // [38] C 2 m m (C -2 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/C2mm.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 189: // [38] C m 2 m (C -2 -2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Cm2m.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 190: // [38] A m 2 m (A -2 -2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Am2m.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 191: // [39] A b m 2 (A 2 -2b)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 192: // [39] B m a 2 (B 2 -2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bma2.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 193: // [39] B 2 c m (B -2a 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/B2cm.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 194: // [39] C 2 m b (C -2a 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/C2mb.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 195: // [39] C m 2 a (C -2a -2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/C2ma.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 196: // [39] A c 2 m (A -2b -2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Ac2m.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 197: // [40] A m a 2 (A 2 -2a)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 198: // [40] B b m 2 (B 2 -2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bbm2.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 199: // [40] B 2 m b (B -2b 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/B2mb.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 200: // [40] C 2 c m (C -2c 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/C2cm.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 201: // [40] C c 2 m (C -2c -2c)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Cc2m.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 202: // [40] A m 2 a (A -2a -2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Am2a.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 203: // [41] A b a 2 (A 2 -2ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 204: // [41] B b a 2 (B 2 -2ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bba2.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 205: // [41] B 2 c b (B -2ab 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/B2cb.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 206: // [41] C 2 c b (C -2ac 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/C2cb.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 207: // [41] C c 2 a (C -2ac -2ac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Cc2a.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 208: // [41] A c 2 a (A -2ab -2ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Ac2a.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 209: // [42] F m m 2 (F 2 -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 210: // [42] F 2 m m (F -2 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/F2mm.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 211: // [42] F m 2 m (F -2 -2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Fm2m.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 212: // [43] F d d 2 (F 2 -2d)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 213: // [43] F 2 d d (F -2d 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/F2dd.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/8.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 214: // [43] F d 2 d (F -2d -2d)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Fd2d.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 215: // [44] I m m 2 (I 2 -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 216: // [44] I 2 m m (I -2 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/I2mm.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 217: // [44] I m 2 m (I -2 -2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/I2mm.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 218: // [45] I b a 2 (I 2 -2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 219: // [45] I 2 c b (I -2a 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/I2cb.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 220: // [45] I c 2 a (I -2b -2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Ic2a.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 221: // [46] I m a 2 (I 2 -2a)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 222: // [46] I b m 2 (I 2 -2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Ibm2.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,1.0/4.0,3.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 223: // [46] I 2 m b (I -2b 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/I2mb.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 224: // [46] I 2 c m (I -2c 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/I2cm.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 225: // [46] I c 2 m (I -2c -2c)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Ic2m.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 226: // [46] I m 2 a (I -2a -2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Im2a.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 227: // [47] P m m m (-P 2 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 228: // [48] P n n n Origin choice 1 (P 2 2 -1n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 229: // [48] P n n n Origin choice 2 (-P 2ab 2bc)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 230: // [49] P c c m  (-P 2 2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 231: // [49] P m a a (-P 2a 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pmaa.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 232: // [49] P b m b (-P 2b 2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pbmb.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 233: // [50] P b a n Origin choice 1 (P 2 2 -1ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 234: // [50] P b a n Origin choice 2 (-P 2ab 2b)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 235: // [50] P n c b Origin choice 1 (P 2 2 -1bc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pncb__1.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 236: // [50] P n c b Origin choice 2 (-P 2b 2bc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pncb__2.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 237: // [50] P c n a Origin choice 1 (P 2 2 -1ac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pcna__1.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 238: // [50] P c n a Origin choice 2 (-P 2a 2c)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pcna__2.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 239: // [51] P m m a (-P 2a 2a)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 240: // [51] P m m b (-P 2b 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pmmb.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 241: // [51] P b m m (-P 2 2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pbmm.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 242: // [51] P c m m (-P 2c 2c)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pcmm.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 243: // [51] P m c m (-P 2c 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pmcm.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 244: // [51] P m a m (-P 2 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pmcm.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 245: // [52] P n n a (-P 2a 2bc)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 246: // [52] P n n b (-P 2b 2n)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pnnb.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 247: // [52] P b n n (-P 2n 2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pbnn.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,1.0/4.0,3.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 248: // [52] P c n n (-P 2ab 2c)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pcnn.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 249: // [52] P n c n (-P 2ab 2n)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pncn.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 250: // [52] P n a n (-P 2n 2bc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pnan.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,1.0/4.0,3.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 251: // [53] P m n a (-P 2ac 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 252: // [53] Pnmb (-P 2bc 2bc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pnmb.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 253: // [53] P b m n (-P 2ab 2ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pbmn.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 254: // [53] P c n m (-P 2 2ac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pcnm.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 255: // [53] P n c m (-P 2 2bc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pncm.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 256: // [53] P m a n (-P 2ab 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pman.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 257: // [54] P c c a (-P 2a 2ac)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 258: // [54] P c c b (-P 2b 2c)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pccb.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 259: // [54] P b a a (-P 2a 2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pbaa.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 260: // [54] P c a a (-P 2ac 2c)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pcaa.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 261: // [54] P b c b (-P 2bc 2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pbcb.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 262: // [54] P b a b (-P 2b 2ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pbab.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 263: // [55] P b a m (-P 2 2ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 264: // [55] P m c b (-P 2bc 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pmcb.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 265: // [55] P c m a (-P 2ac 2ac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pcma.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 266: // [56] P c c n (-P 2ab 2ac)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 267: // [56] P n a a (-P 2ac 2bc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pnaa.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 268: // [56] P b n b (-P 2bc 2ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pbnb.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 269: // [57] P b c m (-P 2c 2b)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 270: // [57] P c a m (-P 2c 2ac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pcam.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 271: // [57] P m c a (-P 2ac 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pmca.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 272: // [57] P m a b (-P 2b 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pmab.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 273: // [57] P b m a (-P 2a 2ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pbma.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 274: // [57] P c m b (-P 2bc 2c)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pcmb.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 275: // [58] P n n m (-P 2 2n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 276: // [58] P m n n (-P 2n 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pmnn.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 277: // [58] P n m n (-P 2n 2n)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pnmn.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 278: // [59] P m m n Origin choice 1 (P 2 2ab -1ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 279: // [59] P m m n Origin choice 2 (-P 2ab 2a)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 280: // [59] P n m m Origin choice 1 (P 2bc 2 -1bc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pnmm__1.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 281: // [59] P n m m Origin choice 2 (-P 2c 2bc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pnmm__2.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 282: // [59] P m n m Origin choice 1 (P 2ac 2ac -1ac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pmnm__1.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 283: // [59] P m n m Origin choice 2 (-P 2c 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pmnm__1.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 284: // [60] P b c n (-P 2n 2ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 285: // [60] P c a n (-P 2n 2c)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pcan.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 286: // [60] P n c a (-P 2a 2n)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pnca.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 287: // [60] P n a b (-P 2bc 2n)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pnab.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 288: // [60] P b n a (-P 2ac 2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pbna.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 289: // [60] P c n b (-P 2b 2ac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pcnb.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 290: // [61] P b c a (-P 2ac 2ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 291: // [61] P c a b (-P 2bc 2ac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pcab.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 292: // [62] P n m a (-P 2ac 2n)      zeolites: MFI
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 293: // [62] P m n b (-P 2bc 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pmnb.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 294: // [62] P b n m (-P 2c 2ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pbnm.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 295: // [62] P c m n (-P 2n 2ac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pcmn.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,1.0/4.0,3.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 296: // [62] P m c n (-P 2n 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pmcn.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,1.0/4.0,3.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 297: // [62] P n a m (-P 2c 2n)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Pnam.html
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 298: // [63] C m c m (-C 2c 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 299: // [63] C c m m (-C 2c 2c)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Ccmm.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 300: // [63] A m m a (-A 2a 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Amma.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 301: // [63] A m a m (-A 2 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Amma.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 302: // [63] B b m m (-B 2 2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bbmm.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 303: // [63] B m m b (-B 2b 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bmmb.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 304: // [64] C m c a (-C 2ac 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 305: // [64] C c m b (-C 2ac 2ac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Ccmc.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 306: // [64] A b m a (-A 2ab 2ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Abma.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 307: // [64] A c a m (-A 2 2ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Acam.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2L(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 308: // [64] B b c m (-B 2 2ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bbcm.html
      if(RANGE2L(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 309: // [64] B m a b (-B 2ab 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bmab.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 310: // [65] C m m m (-C 2 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 311: // [65] A m m m (-A 2 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Ammm.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 312: // [65] B m m m (-B 2 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bmmm.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 313: // [66] C c c m (-C 2 2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 314: // [66] A m a a (-A 2a 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Amaa.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 315: // [66] B b m b (-B 2b 2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bbmb.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 316: // [67] C m m a (-C 2a 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 317: // [67] C m m b (-C 2a 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Cmmb.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 318: // [67] A b m m (-A 2b 2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Abmm.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 319: // [67] A c m m (-A 2 2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Acmm.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 320: // [67] B m c m (-B 2 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bmcm.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 321: // [67] B m a m (-B 2a 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bmam.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 322: // [68] C c c a Origin choice 1 (C 2 2 -1ac)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 323: // [68] C c c a Origin choice 2 (-C 2a 2ac)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 324: // [68] C c c b Origin choice 1 (C 2 2 -1bc)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Cccb__1.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 325: // [68] C c c b Origin choice 2 (-C 2b 2c)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Cccb__2.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 326: // [68] A b a a Origin choice 1 (A 2 2 -1ac)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Abaa__1.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 327: // [68] A b a a Origin choice 2 (-A 2a 2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Abaa__2.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 328: // [68] A c a a Origin choice 1 (A 2 2 -1ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Acaa__1.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 329: // [68] A c a a Origin choice 2 (-A 2ab 2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Acaa__2.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 330: // [68] B b c b Origin choice 1 (B 2 2 -1ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bbcb__1.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 331: // [68] B b c b Origin choice 2 (-B 2ab 2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bbcb__2.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 332: // [68] B b a b Origin choice 1 (B 2 2 -1ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bbab__1.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 333: // [68] B b a b Origin choice 2 (-B 2b 2ab)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Bbab__2.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 334: // [69] F m m m (-F 2 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 335: // [70] F d d d:1 Origin choice 1 (F 2 2 -1d)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/8.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 336: // [70] F d d d:2 Origin choice 2 (-F 2uv 2vw)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/8.0,eps)&&
         RANGE2LR(p.y,-1.0/8.0,1.0/8.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 337: // [71] I m m m (-I 2 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 338: // [72] I b a m (-I 2 2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 339: // [72] I m c b (-I 2a 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Imcb.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 340: // [72] I c m a (-I 2b 2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Icma.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 341: // [73] I b c a (-I 2b 2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 342: // [73] I c a b (-I 2a 2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Icab.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 343: // [74] I m m a (-I 2b 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Imma.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 344: // [74] I m m b (-I 2a 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Immb.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 345: // [74] I b m m (-I 2c 2c)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Ibmm.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 346: // [74] I c m m (-I 2 2b)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Icmm.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,1.0/4.0,3.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 347: // [74] I m c m (-I 2 2a)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Imcm.html
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,1.0/4.0,3.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 348: // [74] I m a m (-I 2c 2)
      // source:  http://cci.lbl.gov:80/cctbx/explore_symmetry.html and http://shi-lab.org/Symmetry/SPGR/Imam.html
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)) asymmetric=TRUE;
      break;

    // TETRAGONAL GROUPS
    // =================

    case 349: // [75] P 4 (P 4)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 350: // [76] P 41 (P 4w)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 351: // [77] P 42 (P 4c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 352: // [78] P 43 (P 4cw)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 353: // [79] I 4 (I 4)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 354: // [80] I 41 (I 4bw)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 355: // [81] P -4 (P -4)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 356: // [82] I -4 (I -4)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 357: // [83] P 4/m (-P 4)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 358: // [84] P 42/m (-P 4c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 359: // [85] P 4/n Origin choice 1 (P 4ab -1ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;
    case 360: // [85] P 4/n Origin choice 2 (-P 4a)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 361: // [86] P 42/n Origin choice 1 (P 4n -1n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 362: // [86] P 42/n Origin choice 2 (-P 4bc)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 363: // [87] I 4/m (-I 4)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 364: // [88] I 41/a Origin choice 1 (I 4bw -1bw)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;
    case 365: // [88] I 41/a Origin choice 2 (-I 4ad)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)) asymmetric=TRUE;
      break;

    case 366: // [89] P 4 2 2 (P 4 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 367: // [90] P 4 21 2 (P 4ab 2ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 368: // [91] P 41 2 2 (P 4w 2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/8.0,eps)) asymmetric=TRUE;
      break;

    case 369: // [92] P 41 21 2 (P 4abw 2nw)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/8.0,eps)) asymmetric=TRUE;
      break;

    case 370: // [93] P 42 2 2 (P 4c 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 371: // [94] P 42 21 2 (P 4n 2n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 372: // [95] P 43 2 2 (P 4cw 2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/8.0,eps)) asymmetric=TRUE;
      break;

    case 373: // [96] P 43 21 2 (P 4nw 2abw)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/8.0,eps)) asymmetric=TRUE;
      break;

    case 374: // [97] I 4 2 2 (I 4 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 375: // [98] I 41 2 2 (I 4bw 2bw)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/8.0,eps)) asymmetric=TRUE;
      break;

    case 376: // [99] P 4 m m (P 4 -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)&&
         (p.x<=p.y+eps)) asymmetric=TRUE;
      break;

    case 377: // [100] P 4 b m (P 4 -2ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)&&
         (p.y<=1.0/2.0-p.x+eps)) asymmetric=TRUE;
      break;

    case 378: // [101] P 42 c m (P 4c -2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)&&
         (p.x<=p.y+eps)) asymmetric=TRUE;
      break;

    case 379: // [102] P 42 n m (P 4n -2n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)&&
         (p.x<=p.y+eps)) asymmetric=TRUE;
      break;

    case 380: // [103] P 4 c c (P 4 -2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 381: // [104] P 4 n c (P 4 -2n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 382: // [105] P 42 m c (P 4c -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 383: // [106] P 42 b c (P 4c -2ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 384: // [107] I 4 m m (I 4 -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=p.y+eps)) asymmetric=TRUE;
      break;

    case 385: // [108] I 4 c m (I 4 -2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.y<=1.0/2.0-p.x+eps)) asymmetric=TRUE;
      break;

    case 386: // [109] I 41 m d (I 4bw -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 387: // [110] I 41 c d (I 4bw -2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 388: // [111] P -4 2 m (P -4 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)&&
         (p.x<=p.y+eps)) asymmetric=TRUE;
      break;

    case 389: // [112] P -4 2 c (P -4 2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 390: // [113] P -4 21 m (P -4 2ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)&&
         (p.y<=1.0/2.0-p.x+eps)) asymmetric=TRUE;
      break;

    case 391: // [114] P -4 21 c (P -4 2n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 392: // [115] P -4 m 2 (P -4 -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 393: // [116] P -4 c 2 (P -4 -2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 394: // [117] P -4 b 2 (P -4 -2ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)) asymmetric=TRUE;
      break;

    case 395: // [118] P -4 n 2 (P -4 -2n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 396: // [119] I -4 m 2 (I -4 -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 397: // [120] I -4 c 2 (I -4 -2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 398: // [121] I -4 2 m (I -4 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=p.y+eps)) asymmetric=TRUE;
      break;

    case 399: // [122] I -4 2 d (I -4 2bw)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/8.0,eps)) asymmetric=TRUE;
      break;

    case 400: // [123] P 4/m m m (-P 4 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=p.y+eps)) asymmetric=TRUE;
      break;

    case 401: // [124] P 4/m c c (-P 4 2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 402: // [125] P 4/n b m Origin choice 1 (P 4 2 -1ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.y<=1.0/2.0-p.x+eps)) asymmetric=TRUE;
      break;
    case 403: // [125] P 4/n b m Origin choice 2 (-P 4a 2b)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=-p.y+eps)) asymmetric=TRUE;
      break;

    case 404: // [126] P 4/n n c Origin choice 1 (P 4 2 -1n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 405: // [126] P 4/n n c Origin choice 2 (-P 4a 2bc)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 406: // [127] P 4/m b m (-P 4 2ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.y<=1.0/2.0-p.x+eps)) asymmetric=TRUE;
      break;

    case 407: // [128] P 4/m n c (-P 4 2n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 408: // [129] P 4/n m m Origin choice 1 (P 4ab 2ab -1ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.y<=1.0/2.0-p.x+eps)) asymmetric=TRUE;
      break;
    case 409: // [129] P 4/n m m Origin choice 2 (-P 4a 2a)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=p.y+eps)) asymmetric=TRUE;
      break;

    case 410: // [130] P 4/n c c Origin choice 1 (P 4ab 2n -1ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 411: // [130] P 4/n c c Origin choice 2 (-P 4a 2ac)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 412: // [131] P 42/m m c (-P 4c 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 413: // [132] P 42/m c m (-P 4c 2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=p.y+eps)) asymmetric=TRUE;
      break;

    case 414: // [133] P 42/n b c Origin choice 1 (P 4n 2c -1n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 415: // [133] P 42/n b c Origin choice 2 (-P 4ac 2b)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 416: // [134] P 42/n n m Origin choice 1 (P 4n 2 -1n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (p.x<=p.y+eps)&&
         (p.y<=1.0-p.x+eps)) asymmetric=TRUE;
      break;
    case 417: // [134] P 42/n n m Origin choice 2 (-P 4ac 2bc)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=-p.y+eps)) asymmetric=TRUE;
      break;

    case 418: // [135] P 42/m b c (-P 4c 2ab)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 419: // [136] P 42/m n m (-P 4n 2n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=p.y+eps)) asymmetric=TRUE;
      break;

    case 420: // [137] P 42/n m c Origin choice 1 (P 4n 2n -1n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;
    case 421: // [137] P 42/n m c Origin choice 2 (-P 4ac 2a)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 422: // [138] P 42/n c m Origin choice 1 (P 4n 2ab -1n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)&&
         (p.x<=p.y+eps)&&
         (p.y<=1.0/2.0-p.x+eps)) asymmetric=TRUE;
      break;
    case 423: // [138] P 42/n c m Origin choice 2 (-P 4ac 2ac)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.y,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
        (p.x<=p.y+eps)) asymmetric=TRUE;
      break;

    case 424: // [139] I 4/m m m (-I 4 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (p.x<=p.y+eps)) asymmetric=TRUE;
      break;

    case 425: // [140] I 4/m c m (-I 4 2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (p.y<=1.0/2.0-p.x+eps)) asymmetric=TRUE;
      break;

    case 426: // [141] I 41/a m d Origin choice 1 (I 4bw 2bw -1bw)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/8.0,eps)) asymmetric=TRUE;
      break;
    case 427: // [141] I 41/a m d Origin choice 2 (-I 4bd 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/8.0,eps)) asymmetric=TRUE;
      break;

    case 428: // [142] I 41/a c d Origin choice 1 (I 4bw 2aw -1bw)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/8.0,eps)) asymmetric=TRUE;
      break;
    case 429: // [142] I 41/a c d Origin choice 2 (-I 4bd 2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/8.0,eps)) asymmetric=TRUE;
      break;


    // TRIGONAL GROUPS
    // ===============

    case 430: // [143] P 3 (P 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,(1.0+p.x)/2.0)+eps)) asymmetric=TRUE;
      break;

    case 431: // [144] P 31 (P 31)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/3.0,eps)) asymmetric=TRUE;
      break;

    case 432: // [145] P 32 (P 32)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/3.0,eps)) asymmetric=TRUE;
      break;

    case 433: // [146] R 3 hexagonal axes (R 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/3.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,(1.0+p.x)/2.0)+eps)) asymmetric=TRUE;
      break;
    case 434: // [146] R 3 Rhombohedral axes (P 3*)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)&&
         (p.z<=MIN2(p.x,p.y)+eps)) asymmetric=TRUE;
      break;

    case 435: // [147] P -3 (P -3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,(1.0+p.x)/2.0)+eps)) asymmetric=TRUE;
      break;

    case 436: // [148] R-3 hexagonal axes (-R 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/6.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,(1.0+p.x)/2.0)+eps)) asymmetric=TRUE;
      break;
    case 437: // [148] R -3 Rhombohedral axes (-P 3*)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.z<=MIN4(p.x,p.y,1.0-p.x,1.0-p.y)+eps)) asymmetric=TRUE;
      break;

    case 438: // [149] P 3 1 2 (P 3 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,(1.0+p.x)/2.0)+eps)) asymmetric=TRUE;
      break;

    case 439: // [150] P 3 2 1 (P 3 2")
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,(1.0+p.x)/2.0)+eps)) asymmetric=TRUE;
      break;

    case 440: // [151] P 31 1 2 (P 31 2 (0 0 4))
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/6.0,eps)) asymmetric=TRUE;
      break;

    case 441: // [152] P 31 2 1 (P 31 2")
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/6.0,eps)) asymmetric=TRUE;
      break;

    case 442: // [153] P 32 1 2 (P 32 2 (0 0 2))
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/6.0,eps)) asymmetric=TRUE;
      break;

    case 443: // [154] P 32 2 1 (P 32 2")
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/6.0,eps)) asymmetric=TRUE;
      break;

    case 444: // [155] R 3 2 Hexagonal axes (R 3 2")
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/6.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,(1.0+p.x)/2.0)+eps)) asymmetric=TRUE;
      break;
    case 445: // [155] R 3 2 Rhombohedral axes (P 3* 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.z<=MIN4(p.x,p.y,1.0-p.x,1.0-p.y)+eps)) asymmetric=TRUE;
      break;

    case 446: // [156] P 3 m 1 (P 3 -2")
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)&&
         (p.x<=2.0*p.y+eps)&&
         (p.y<=MIN2(1.0-p.x,2.0*p.x)+eps)) asymmetric=TRUE;
      break;

    case 447: // [157] P 3 1 m (P 3 -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)&&
         (p.x<=(p.y+1.0)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,p.x)+eps)) asymmetric=TRUE;
      break;

    case 448: // [158] P 3 c 1 (P 3 -2"c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,(1.0+p.x)/2.0)+eps)) asymmetric=TRUE;
      break;

    case 449: // [159] P 3 1 c (P 3 -2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,(1.0+p.x)/2.0)+eps)) asymmetric=TRUE;
      break;

    case 450: // [160] R 3 m Hexagonal axes (R 3 -2")
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/3.0,eps)&&
         (p.x<=2.0*p.y+eps)&&
         (p.y<=MIN2(1.0-p.x,2.0*p.x)+eps)) asymmetric=TRUE;
      break;
    case 451: // [160] R 3 m Rhombohedral axes (P 3* -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)&&
         (p.y<=p.x+eps)&&
         (p.z<=p.y+eps)) asymmetric=TRUE;
      break;

    case 452: // [161] R 3 c Hexagonal axes (R 3 -2"c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/6.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,(1.0+p.x)/2.0)+eps)) asymmetric=TRUE;
      break;
    case 453: // [161] R 3 c Rhombohedral axes (P 3* -2n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)&&
         (p.y<=p.x+eps)&&
         (p.z<=p.y+eps)) asymmetric=TRUE;
      break;

    case 454: // [162] P -3 1 m (-P 3 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,p.x)+eps)) asymmetric=TRUE;
      break;

    case 455: // [163] P -3 1 c (-P 3 2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,(1.0+p.x)/2.0)+eps)) asymmetric=TRUE;
      break;

    case 456: // [164] P -3 m 1 (-P 3 2")
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=p.x/2.0+eps)) asymmetric=TRUE;
      break;

    case 457: // [165] P -3 c 1 (-P 3 2"c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,(1.0+p.x)/2.0)+eps)) asymmetric=TRUE;
      break;

    case 458: // [166] R -3 m Hexagonal axes (-R 3 2")
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/6.0,eps)&&
         (p.x<=2.0*p.y+eps)&&
         (p.y<=MIN2(1.0-p.x,2.0*p.x)+eps)) asymmetric=TRUE;
      break;
    case 459: // [166] R -3 m Rhombohedral axes (-P 3* 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.y<=p.x+eps)&&
         (p.z<=MIN2(p.y,1.0-p.x)+eps)) asymmetric=TRUE;
      break;

    case 460: // [167] R -3 c Hexagonal axes (-R 3 2"c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/12.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,(1.0+p.x)/2.0)+eps)) asymmetric=TRUE;
      break;
    case 461: // [167] R -3 c Rhombohedral axes (-P 3* 2n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,1.0/4.0,5.0/4.0,eps)&&
         RANGE2LR(p.y,1.0/4.0,5.0/4.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)&&
         (p.y<=p.x+eps)&&
         (p.z<=MIN2(p.y,3.0/2.0-p.x)+eps)) asymmetric=TRUE;
      break;


    // HEXAGONAL GROUPS
    // ================

    case 462: // [168] P 6 (P 6)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,p.x)+eps)) asymmetric=TRUE;
      break;

    case 463: // [169] P 61 (P 61)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/6.0,eps)) asymmetric=TRUE;
      break;

    case 464: // [170] P 65 (P 65)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/6.0,eps)) asymmetric=TRUE;
      break;

    case 465: // [171] P 62 (P 62)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/3.0,eps)&&
         (p.y<=p.x+eps)) asymmetric=TRUE;
      break;

    case 466: // [172] P 64 (P 64)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/3.0,eps)&&
         (p.y<=p.x+eps)) asymmetric=TRUE;
      break;

    case 467: // [173] P 63 (P 6c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,(1.0+p.x)/2.0)+eps)) asymmetric=TRUE;
      break;

    case 468: // [174] P -6 (P -6)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,(1.0+p.x)/2.0)+eps)) asymmetric=TRUE;
      break;

    case 469: // [175] P6/m (-P 6)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,p.x)+eps)) asymmetric=TRUE;
      break;

    case 470: // [176] P 63/m (-P 6c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,(1.0+p.x)/2.0))) asymmetric=TRUE;
      break;

    case 471: // [177] P 6 2 2 (P 6 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,p.x)+eps)) asymmetric=TRUE;
      break;

    case 472: // [178] P 61 2 2 (P 61 2 (0 0 5))
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/12.0,eps)) asymmetric=TRUE;
      break;

    case 473: // [179] P 65 2 2 (P 65 2 (0 0 1))
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/12.0,eps)) asymmetric=TRUE;
      break;

    case 474: // [180] P 62 2 2 (P 62 2 (0 0 4))
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/6.0,eps)&&
         (p.y<=p.x+eps)) asymmetric=TRUE;
      break;

    case 475: // [181] P 64 2 2 (P 64 2 (0 0 2))
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/6.0,eps)&&
         (p.y<=p.x+eps)) asymmetric=TRUE;
      break;

    case 476: // [182] P 63 2 2 (P 6c 2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,(1.0+p.x)/2.0)+eps)) asymmetric=TRUE;
      break;

    case 477: // [183] P 6 m m (P 6 -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=p.x/2.0+eps)) asymmetric=TRUE;
      break;

    case 478: // [184] P 6 c c (P 6 -2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,p.x)+eps)) asymmetric=TRUE;
      break;

    case 479: // [185] P 63 c m (P 6c -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,p.x)+eps)) asymmetric=TRUE;
      break;

    case 480: // [186] P 63 m c (P 6c -2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=p.x/2.0+eps)) asymmetric=TRUE;
      break;

    case 481: // [187] P -6 m 2 (P -6 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=2.0*p.y+eps)&&
         (p.y<=MIN2(1.0-p.x,2.0*p.x)+eps)) asymmetric=TRUE;
      break;

    case 482: // [188] P -6 c 2 (P -6c 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,(1.0+p.x)/2.0)+eps)) asymmetric=TRUE;
      break;

    case 483: // [189] P -6 2 m (P -6 -2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,p.x)+eps)) asymmetric=TRUE;
      break;

    case 484: // [190] P -6 2 c (P -6c -2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,(1.0+p.x)/2.0)+eps)) asymmetric=TRUE;
      break;

    case 485: // [191] P 6/m m m (-P 6 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=p.x/2.0+eps)) asymmetric=TRUE;
      break;

    case 486: // [192] P 6/m c c (-P 6 2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,p.x)+eps)) asymmetric=TRUE;
      break;

    case 487: // [193] P 63/m c m (-P 6c 2)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (p.x<=(1.0+p.y)/2.0+eps)&&
         (p.y<=MIN2(1.0-p.x,p.x)+eps)) asymmetric=TRUE;
      break;

    case 488: // [194] P 63/m m c (-P 6c 2c)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.y,0.0,2.0/3.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (p.x<=2.0*p.y+eps)&&
         (p.y<=MIN2(1.0-p.x,p.x)+eps)) asymmetric=TRUE;
      break;

    // CUBIC GROUPS
    // ============

    case 489: // [195] P 2 3 (P 2 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.y<=1.0-p.x+eps)&&
         (p.z<=MIN2(p.x,p.y)+eps)) asymmetric=TRUE;
      break;

    case 490: // [196] F 2 3 (F 2 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,-1.0/4.0,1.0/4.0,eps)&&
         (p.y<=p.x+eps)&&
         RANGE2LR(p.z,MAX2(p.x-1.0/2.0,-p.y),MIN2(1.0/2.0-p.x,p.y),eps)) asymmetric=TRUE;
      break;

    case 491: // [197] I 2 3 (I 2 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.y<=MIN2(p.x,1.0-p.x)+eps)&&
         (p.z<=p.y+eps)) asymmetric=TRUE;
      break;

    case 492: // [198] P 21 3 (P 2ac 2ab 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,-1.0/2.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,MAX2(p.x-1.0/2.0,-p.y),MIN2(p.x,p.y),eps)) asymmetric=TRUE;
      break;

    case 493: // [199] I 21 3 (I 2b 2c 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.z<=MIN2(p.x,p.y)+eps)) asymmetric=TRUE;
      break;

    case 494: // [200] P m -3 (-P 2 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.z<=MIN2(p.x,p.y)+eps)) asymmetric=TRUE;
      break;

    case 495: // [201] P n -3 Origin choice 1 (P 2 2 3 -1n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.y<=MIN2(p.x,1.0-p.x)+eps)&&
         (p.z<=p.y+eps)) asymmetric=TRUE;
      break;
    case 496: // [201] P n -3 Origin choice 2 (-P 2ab 2bc 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,-1.0/4.0,3.0/4.0,eps)&&
         RANGE2LR(p.y,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,-1.0/4.0,1.0/4.0,eps)&&
         (p.y<=MIN2(p.x,1.0-p.x)+eps)&&
         (p.z<=p.y+eps)) asymmetric=TRUE;
      break;

    case 497: // [202] F m -3 (-F 2 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (p.y<=p.x+eps)&&
         (p.z<=MIN2(1.0/2.0-p.x,p.y)+eps)) asymmetric=TRUE;
      break;

    case 498: // [203] F d -3 Origin choice 1 (F 2 2 3 -1d)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,-1.0/4.0,1.0/4.0,eps)&&
         (p.y<=MIN2(p.x,1.0/2.0-p.x)+eps)&&
         RANGE2LR(p.z,-p.y,p.y,eps)) asymmetric=TRUE;
      break;
    case 499: // [203] F d -3 Origin choice 2 (-F 2uv 2vw 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,-1.0/8.0,3.0/8.0,eps)&&
         RANGE2LR(p.y,-1.0/8.0,1.0/8.0,eps)&&
         RANGE2LR(p.z,-3.0/8.0,1.0/8.0,eps)&&
         (p.y<=MIN2(p.x,1.0/4.0-p.x)+eps)&&
         RANGE2LR(p.z,-p.y-1.0/4.0,p.y,eps)) asymmetric=TRUE;
      break;

    case 500: // [204] I m -3 (-I 2 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (p.y<=p.x+eps)&&
         (p.z<=p.y+eps)) asymmetric=TRUE;
      break;

    case 501: // [205] P a -3 (-P 2ac 2ab 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.z<=MIN2(p.x,p.y)+eps)) asymmetric=TRUE;
      break;

    case 502: // [206] I a -3 (-I 2b 2c 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (p.z<=MIN3(p.x,1.0/2.0-p.x,1.0/2.0-p.y)+eps)) asymmetric=TRUE;
      break;

    case 503: // [207] P 4 3 2 (P 4 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.y<=MIN2(p.x,1.0-p.x)+eps)&&
         (p.z<=p.y+eps)) asymmetric=TRUE;
      break;

    case 504: // [208] P 42 3 2 (P 4n 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,-1.0/4.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,MAX4(-p.x,p.x-1.0/2.0,-p.y,p.y-1.0/2.0),MIN3(p.x,1.0/2.0-p.x,1.0/2.0-p.y),eps)) asymmetric=TRUE;
      break;

    case 505: // [209] F 4 3 2 (F 4 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,-1.0/4.0,1.0/4.0,eps)&&
         (p.y<=MIN2(p.x,1.0/2.0-p.x)+eps)&&
         RANGE2LR(p.z,-p.y,p.y,eps)) asymmetric=TRUE;
      break;

    case 506: // [210] F 41 3 2 (F 4d 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,-1.0/8.0,1.0/8.0,eps)&&
         RANGE2LR(p.z,-1.0/8.0,1.0/8.0,eps)&&
         (p.y<=MIN2(p.x,1.0/2.0-p.x)+eps)&&
         RANGE2LR(p.z,-p.y,MIN2(p.x,1.0/2.0-p.x),eps)) asymmetric=TRUE;
      break;

    case 507: // [211] I 4 3 2 (I 4 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (p.z<=MIN4(p.x,1.0/2.0-p.x,p.y,1.0/2.0-p.y)+eps)) asymmetric=TRUE;
      break;

    case 508: // [212] P 43 3 2 (P 4acd 2ab 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,3.0/4.0,eps)&&
         RANGE2LR(p.z,-1.0/2.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,MAX2(-p.y,p.x-1.0/2.0),MIN4(-p.y+1.0/2.0,2.0*p.x-p.y,2.0*p.y-p.x,p.y-2.0*p.x+1.0/2.0),eps)) asymmetric=TRUE;
      break;

    case 509: // [213] P 41 3 2 (P 4bd 2ab 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,-1.0/4.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,3.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,p.x,p.x+1.0/2.0,eps)&&
         RANGE2LR(p.z,(p.y-p.x)/2.0,MIN3(p.y,(-4.0*p.x-2.0*p.y+3.0)/2.0,(3.0-2.0*p.x-2.0*p.y)/4.0),eps)) asymmetric=TRUE;
      break;

    case 510: // [214] I 41 3 2 (I 4bd 2c 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,-3.0/8.0,1.0/8.0,eps)&&
         RANGE2LR(p.y,-1.0/8.0,1.0/8.0,eps)&&
         RANGE2LR(p.z,-1.0/8.0,3.0/8.0,eps)&&
         RANGE2LR(p.z,MAX3(p.x,p.y,p.y-p.x-1.0/8.0),p.y+1.0/4.0,eps)) asymmetric=TRUE;
      break;

    case 511: // [215] P -4 3 m (P -4 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.y<=MIN2(p.x,1.0-p.x)+eps)&&
         (p.z<=p.y+eps)) asymmetric=TRUE;
      break;

    case 512: // [216] F -4 3 m (F -4 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,-1.0/4.0,1.0/4.0,eps)&&
         (p.y<=MIN2(p.x,1.0-p.x)+eps)&&
         RANGE2LR(p.z,-p.y,p.y,eps)) asymmetric=TRUE;
      break;

    case 513: // [217] I -4 3 m (I -4 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.y<=p.x+eps)&&
         (p.z<=p.y+eps)) asymmetric=TRUE;
      break;

    case 514: // [218] P -4 3 n (P -4n 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.z<=MIN2(p.x,p.y)+eps)) asymmetric=TRUE;
      break;

    case 515: // [219] F -4 3 c (F -4a 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,-1.0/4.0,1.0/4.0,eps)&&
         (p.y<=MIN2(p.x,1.0/2.0-p.x)+eps)&&
         RANGE2LR(p.z,-p.y,p.y,eps)) asymmetric=TRUE;
      break;

    case 516: // [220] I -4 3 d (I -4bd 2c 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,1.0/4.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,1.0/4.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.z<=MIN2(p.x,p.y)+eps)) asymmetric=TRUE;
      break;

    case 517: // [221] P m -3 m (-P 4 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.y<=p.x+eps)&&
         (p.z<=p.y+eps)) asymmetric=TRUE;
      break;

    case 518: // [222] P n -3 n Origin choice 1 (P 4 2 3 -1n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,1.0/4.0,3.0/4.0,eps)&&
         RANGE2LR(p.y,1.0/4.0,3.0/4.0,eps)&&
         RANGE2LR(p.z,1.0/4.0,3.0/4.0,eps)&&
         (p.y<=p.x+eps)&&
         (p.z<=p.y+eps)) asymmetric=TRUE;
      break;
    case 519: // [222] P n -3 n Origin choice 2 (-P 4a 2bc 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.y<=p.x+eps)&&
         (p.z<=p.y+eps)) asymmetric=TRUE;
      break;

    case 520: // [223] P m -3 n (-P 4n 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.z<=MIN3(p.x,1.0/2.0-p.x,1.0/2.0-p.y)+eps)) asymmetric=TRUE;
      break;

    case 521: // [224] P n -3 m Origin choice 1 (P 4n 2 3 -1n)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,-1.0/4.0,1.0/4.0,eps)&&
         (p.y<=p.x+eps)&&
         RANGE2LR(p.z,MAX2(p.x-1.0/2.0,-p.y),MIN2(1.0/2.0-p.x,p.y),eps)) asymmetric=TRUE;
      break;
    case 522: // [224] P n -3 m Origin choice 2 (-P 4bc 2bc 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,1.0/4.0,3.0/4.0,eps)&&
         RANGE2LR(p.y,1.0/4.0,3.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/2.0,eps)&&
         (p.y<=p.x+eps)&&
         RANGE2LR(p.z,MAX2(p.x-1.0/2.0,1.0/2.0-p.y),MIN2(p.y,1.0-p.x),eps)) asymmetric=TRUE;
      break;

    case 523: // [225] F m -3 m (-F 4 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (p.y<=MIN2(p.x,1.0/2.0-p.x)+eps)&&
         (p.z<=p.y+eps)) asymmetric=TRUE;
      break;

    case 524: // [226] F m -3 c (-F 4a 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/4.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (p.y<=MIN2(p.x,1.0/2.0-p.x)+eps)&&
         (p.z<=p.y+eps)) asymmetric=TRUE;
      break;

    case 525: // [227] F d -3 m Origin choice 1 (F 4d 2 3 -1d)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/8.0,eps)&&
         RANGE2LR(p.z,-1.0/8.0,1.0/8.0,eps)&&
         (p.y<=MIN2(1.0/2.0-p.x,p.x)+eps)&&
         RANGE2LR(p.z,-p.y,p.y,eps)) asymmetric=TRUE;
      break;
    case 526: // [227] F d -3 m Origin choice 2 (-F 4vw 2vw 3)      e.g. MIL-100,101
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,-1.0/8.0,3.0/8.0,eps)&&
         RANGE2LR(p.y,-1.0/8.0,0.0,eps)&&
         RANGE2LR(p.z,-1.0/4.0,0.0,eps)&&
         (p.y<=MIN2(1.0/4.0-p.x,p.x)+eps)&&
         RANGE2LR(p.z,-p.y-1.0/4.0,p.y,eps)) asymmetric=TRUE;
      break;

    case 527: // [228] F d -3 c Origin choice 1 (F 4d 2 3 -1ad)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/8.0,eps)&&
         RANGE2LR(p.z,-1.0/8.0,1.0/8.0,eps)&&
         (p.y<=MIN2(1.0/2.0-p.x,p.x)+eps)&&
         RANGE2LR(p.z,-p.y,p.y,eps)) asymmetric=TRUE;
      break;
    case 528: // [228] F d -3 c Origin choice 2 (-F 4ud 2vw 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,-1.0/8.0,3.0/8.0,eps)&&
         RANGE2LR(p.y,-1.0/8.0,0.0,eps)&&
         RANGE2LR(p.z,-1.0/4.0,0.0,eps)&&
         (p.y<=MIN2(1.0/4.0-p.x,p.x)+eps)&&
         RANGE2LR(p.z,-p.y-1.0/4.0,p.y,eps)) asymmetric=TRUE;
      break;

    case 529: // [229] I m -3 m (-I 4 2 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.y,0.0,1.0/2.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (p.y<=p.x+eps)&&
         (p.z<=MIN2(1.0/2.0-p.x,p.y)+eps)) asymmetric=TRUE;
      break;

    case 530: // [230] I a -3 d (-I 4bd 2c 3)
      // source:  International Tables for Crystallography, Volume A: Space Group Symmetry: Space Group Symmetry v.A (2005)
      if(RANGE2LR(p.x,-1.0/8.0,1.0/8.0,eps)&&
         RANGE2LR(p.y,-1.0/8.0,1.0/8.0,eps)&&
         RANGE2LR(p.z,0.0,1.0/4.0,eps)&&
         (MAX4(p.x,-p.x,p.y,-p.y)<=p.z+eps)) asymmetric=TRUE;
      break;
    default:
      fprintf(stderr, "ERROR in 'AsymmetricUnit'\n");
      exit(0);
      SpaceGroupSize=0;
      break;
  }

  return asymmetric;
}


enum{PLUS,MINUS};

typedef struct space_group_term
{
  int SignX;
  int SignY;
  int SignZ;
  int SignTrans;
  int numerator;
  int Denomenator;
} SPACE_GROUP_TERM;



void SimplifyFraction(SPACE_GROUP_TERM *term)
{
  switch(term->Denomenator)
  {
    case 1:
      if(term->numerator==1)
      {
        term->numerator=0;
        term->SignTrans=PLUS;  // -1/1 or +1/1 -> +0/1
      }
      break;
    case 2:
      if(term->numerator==1)
        term->SignTrans=PLUS;  // -1/2 -> +1/2
      break;
    case 3:
      if(((term->numerator==4)||(term->numerator==1))&&(term->SignTrans==MINUS))
      {
        term->numerator=2;
        term->SignTrans=PLUS;  // -4/3 or -1/3 -> +2/3
      }
      break;
    case 4:
      break;
    case 5:
      break;
    case 6:
      break;
  }
}

// Note: works for single unit cells
//       for many spacegroups with e.g. an inversion center at the origigin, it also works for multiple unit cells
VECTOR ChiralInversion(int sg,VECTOR pos)
{
  VECTOR newpos;

  newpos=pos;

  switch(sg)
  {
    // TRICLINIC GROUPS
    // ================

    case 1: // [1] P 1 (P 1)
      fprintf(OutputFilePtr[CurrentSystem],"No chiral inversion possible in space group: %d\n",sg); exit(0);
      break;
    case 2: // [2] P -1 (-P 1)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z ; // Inversion center
      break;

    // MONOCLINIC GROUPS
    // =================

    case 3: // [3] P 1 2 1 unique b axis (P 2y)
    case 4: // [3] P 1 1 2 unique c axis (P 2)
    case 5: // [3] P 2 1 1 unique a axis (P 2x)
    case 6: // [4] P 1 21 1 unique b axis (P 2yb)
    case 7: // [4] P 1 1 21 unique c axis (P 2c)
    case 8: // [4] P 21 1 1 unique a axis (P 2xa)
    case 9:  // [5] C 1 2 1 unique b axis: cell choice 1 (C 2y)
    case 10: // [5] A 1 2 1 unique b axis: cell choice 2 (A 2y)
    case 11: // [5] I 1 2 1 unique b axis: cell choice 3 (I 2y)
    case 12: // [5] A 1 1 2 unique c axis: cell choice 1 (A 2)
    case 13: // [5] B 1 1 2 unique c axis: cell choice 2 (B 2)
    case 14: // [5] I 1 1 2 unique c axis: cell choice 3 (I 2)
    case 15: // [5] B 2 1 1 unique a axis: cell choice 1 (B 2x)
    case 16: // [5] C 2 1 1 unique a axis: cell choice 2 (C 2x)
    case 17: // [5] I 2 1 1 unique a axis: cell choice 3 (I 2x)
      fprintf(OutputFilePtr[CurrentSystem],"No chiral inversion possible in space group: %d\n",sg); exit(0);
      break;
    case 18: // [6] P 1 m 1 unique b axis (P -2y)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 19: // [6] P 1 1 m unique c axis (P -2)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 20: // [6] P m 1 1 unique a axis (P -2x)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 21: // [7] P 1 c 1 unique b axis: cell choice 1 (P -2yc)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 22: // [7] P 1 n 1 unique b axis: cell choice 2 (P -2yac)
      newpos.x=pos.x+1.0/2.0; newpos.y=-pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 23: // [7] P 1 a 1 unique b axis: cell choice 3 (P -2ya)
      newpos.x=pos.x+1.0/2.0; newpos.y=-pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 24: // [7] P 1 1 a unique c axis: cell choice 1 (P -2a)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 25: // [7] P 1 1 n unique c axis: cell choice 2 (P -2ab)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z; // Mirror plane
      break;
    case 26: // [7] P 1 1 b unique c axis: cell choice 3 (P -2b)
      newpos.x=pos.x; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z; // Mirror plane
      break;
    case 27: // [7] P b 1 1 unique a axis: cell choice 1 (P -2xb)
      newpos.x=-pos.x; newpos.y=pos.y+1.0/2.0; newpos.z=pos.z; // Mirror plane
      break;
    case 28: // [7] P n 1 1 unique a axis: cell choice 2 (P -2xbc)
      newpos.x=-pos.x; newpos.y=pos.y+1.0/2.0; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 29: // [7] P c 1 1 unique a axis: cell choice 3 (P -2xc)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 30: // [8] C 1 m 1 unique b axis: cell choice 1 (C -2y)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 31: // [8] A 1 m 1 unique b axis: cell choice 2 (A -2y)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 32: // [8] I 1 m 1 unique b axis: cell choice 3 (I -2y)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 33: // [8] A 1 1 m unique c axis: cell choice 1 (A -2)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 34: // [8] B 1 1 m  unique c axis: cell choice 2 (B -2)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 35: // [8] I 1 1 m unique c axis: cell choice 3 (I -2)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 36: // [8] B m 1 1 unique a axis: cell choice 1 (B -2x)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 37: // [8] C m 1 1 unique a axis: cell choice 2 (C -2x)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 38: // [8] I m 1 1 unique a axis: cell choice 3 (I -2x)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 39: // [9] C 1 c 1 unique b axis: cell choice 1 (C -2yc)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 40: // [9] A 1 n 1 unique b axis: cell choice 2 (A -2yab)
      newpos.x=pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=pos.z; // Mirror plane
      break;
    case 41: // [9] I 1 a 1 unique b axis: cell choice 3 (I -2ya)
      newpos.x=pos.x+1.0/2.0; newpos.y=-pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 42: // [9] A 1 a 1 unique -b axis: cell choice 1 (A -2ya)
      newpos.x=pos.x+1.0/2.0; newpos.y=-pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 43: // [9] C 1 n 1 unique -b axis: cell choice 2 (C -2yac)
      newpos.x=pos.x+1.0/2.0; newpos.y=-pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 44: // [9] I 1 c 1 unique -b axis: cell choice 3 (I -2yc)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 45: // [9] A 1 1 a unique c axis: cell choice 1 (A -2a)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 46: // [9] B 1 1 n unique c axis: cell choice 2 (B -2ab)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z; // Mirror plane
      break;
    case 47: // [9] I 1 1 b unique c axis: cell choice 3 (I -2b)
      newpos.x=pos.x; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z; // Mirror plane
      break;
    case 48: // [9] B 1 1 b unique -c axis: cell choice 1 (B -2b)
      newpos.x=pos.x; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z; // Mirror plane
      break;
    case 49: // [9] A 1 1 n unique -c axis: cell choice 2 (A -2ab)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z; // Mirror plane
      break;
    case 50: // [9] I 1 1 a unique -c axis: cell choice 3 (I -2a)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 51: // [9] B b 1 1 unique a axis: cell choice 1 (B -2xb)
      newpos.x=-pos.x; newpos.y=pos.y+1.0/2.0; newpos.z=pos.z; // Mirror plane
      break;
    case 52: // [9] C n 1 1 unique a axis: cell choice 2 (C -2xac)
      newpos.x=-pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 53: // [9] I c 1 1 unique a axis: cell choice 3 (I -2xc)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 54: // [9] C c 1 1 unique -a axis: cell choice 1 (C -2xc)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 55: // [9] B n 1 1 unique -a axis: cell choice 2 (B -2xab)
      newpos.x=-pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=pos.z; // Mirror plane
      break;
    case 56: // [9] I b 1 1 unique -a axis: cell choice 3 (I -2xb)
      newpos.x=-pos.x; newpos.y=pos.y+1.0/2.0; newpos.z=pos.z; // Mirror plane
      break;
    case 57: // [10] P 1 2/m 1 unique b axis (-P 2y)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 58: // [10] P 1 1 2/m unique c axis (-P 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 59: // [10] P 2/m 1 1 unique a axis (-P 2x)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 60: // [11] P 1 21/m 1 unique axis b (-P 2yb)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 61: // [11] P 1 1 21/m unique c axis (-P 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 62: // [11] P 21/m 1 1 unique a axis (-P 2xa)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 63: // [12] C 1 2/m 1 unique b axis: cell choice 1 (-C 2y)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 64: // [12] A 1 2/m 1 unique b axis: cell choice 2 (-A 2y)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 65: // [12] I 1 2/m 1 unique b axis: cell choice 3 (-I 2y)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 66: // [12] A 1 1 2/m unique c axis: cell choice 1 (-A 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 67: // [12] B 1 1 2/m unique c axis: cell choice 2 (-B 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 68: // [12] I 1 1 2/m unique c axis: cell choice 3 (-I 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 69: // [12] B 2/m 1 1 unique a axis: cell choice 1 (-B 2x)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 70: // [12] C 2/m 1 1 unique a axis: cell choice 2 (-C 2x)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 71: // [12] I 2/m 1 1 unique a axis: cell choice 3 (-I 2x)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 72: // [13] P 1 2/c 1 unique b axis: cell choice 1 (-P 2yc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 73: // [13] P 1 2/n 1 unique b axis: cell choice 2 (-P 2yac)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 74: // [13] P 1 2/a 1 unique b axis: cell choice 3 (-P 2ya)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 75: // [13] P 1 1 2/a unique c axis: cell choice 1 (-P 2a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 76: // [13] P 1 1 2/n unique c axis: cell choice 2 (-P 2ab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 77: // [13] P 1 1 2/b unique c axis: cell choice 3 (-P 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 78: // [13] P 2/b 1 1 unique a axis: cell choice 1 (-P 2xb)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 79: // [13] P 2/n 1 1 unique a axis: cell choice 2 (-P 2xbc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 80: // [13] P 2/c 1 1 unique a axis: cell choice 3 (-P 2xc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 81: // [14] P 1 21/c 1 unique b axis: cell choice 1 (-P 2ybc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 82: // [14] P 1 21/n 1 unique b axis: cell choice 2 (-P 2yn)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 83: // [14] P 1 21/a 1 unique b axis: cell choice 3 (-P 2yab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 84: // [14] P 1 1 21/a unique c axis: cell choice 1 (-P 2ac)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 85: // [14] P 1 1 21/n unique c axis: cell choice 2 (-P 2n)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 86: // [14] P 1 1 21/b unique c axis: cell choice 3 (-P 2bc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 87: // [14] P 21/b 1 1 unique a axis: cell choice 1 (-P 2xab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 88: // [14] P 21/n 1 1 unique a axis: cell choice 2 (-P 2xn)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 89: // [14] P 21/c 1 1 unique a axis: cell choice 3 (-P 2xac)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 90: // [15] C 1 2/c 1 unique b axis: cell choice 1 (-C 2yc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 91: // [15] A 1 2/n 1 unique b axis: cell choice 2 (-A 2yab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 92: // [15] I 1 2/a 1 unique b axis: cell choice 3 (-I 2ya)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 93: // [15] A 1 2/a 1 unique -b axis: cell choice 1 (-A 2ya)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 94: // [15] C 1 2/n 1 unique -b axis: cell choice 2 (-C 2yac)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 95: // [15] I 1 2/c 1 unique -b axis: cell choice 3 (-I 2yc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z ; // Inversion center
      break;
    case 96: // [15] A 1 1 2/a unique c axis: cell choice 1 (-A 2a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 97: // [15] B 1 1 2/n unique c axis: cell choice 2 (-B 2ab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 98: // [15] I 1 1 2/b unique c axis: cell choice 3 (-I 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 99: // [15] B 1 1 2/b unique -c axis: cell choice 1 (-B 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 100: // [15] A 1 1 2/n unique -c axis: cell choice 2 (-A 2ab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 101: // [15] I 1 1 2/a unique -c axis: cell choice 3 (-I 2a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 102: // [15] B 2/b 1 1 unique a axis: cell choice 1 (-B 2xb)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 103: // [15] C 2/n 1 1 unique a axis: cell choice 2 (-C 2xac)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 104: // [15] I 2/c 1 1 unique a axis: cell choice 3 (-I 2xc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 105: // [15] C 2/c 1 1 unique -a axis: cell choice 1 (-C 2xc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 106: // [15] B 2/n 1 1 unique -a axis: cell choice 2 (-B 2xab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 107: // [15] I 2/b 1 1 unique -a axis: cell choice 3 (-I 2xb)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;

    // ORTHORHOMBIC GROUPS
    // ===================

    case 108: // [16] P 2 2 2 (P 2 2)
    case 109: // [17] P 2 2 21 Origin-1,abc (P 2c 2)
    case 110: // [17] P 21 2 2 Origin-1,cab (P 2a 2a)
    case 111: // [17] P 2 21 2 Origin-1,bca (P 2 2b)
    case 112: // [18] P 21 21 2 Origin-1,abc (P 2 2ab)
    case 113: // [18] P 2 21 21 Origin-1,cab (P 2bc 2)
    case 114: // [18] P 21 2 21 Origin-1,bca (P 2ac 2ac)
    case 115: // [19] P 21 21 21 (P 2ac 2ab)
    case 116: // [20] C 2 2 21  Origin-1,abc (C 2c 2)
    case 117: // [20] A 21 2 2  Origin-1,cba (A 2a 2a)
    case 118: // [20] B 2 21 2  Origin-1,bca (B 2 2b)
    case 119: // [21] C 2 2 2 Origin-1,abc (C 2 2)
    case 120: // [21] A 2 2 2 Origin-1,cab (A 2 2)
    case 121: // [21] B 2 2 2 Origin-1,bca (B 2 2)
    case 122: // [22] F 2 2 2 (F 2 2)
    case 123: // [23] I 2 2 2 (I 2 2)
    case 124: // [24] I 21 21 21 (I 2b 2c)
      fprintf(OutputFilePtr[CurrentSystem],"No chiral inversion possible in space group: %d\n",sg); exit(0);
      break;
    case 125: // [25] P m m 2 (P 2 -2)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 126: // [25] P 2 m m (P -2 2)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 127: // [25] P m 2 m (P -2 -2)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 128: // [26] P m c 21 (P 2c -2)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 129: // [26] P c m 21 (P 2c -2c)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 130: // [26] P 21 m a (P -2a 2a)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 131: // [26] P 21 a m (P -2 2a)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 132: // [26] P b 21 m (P -2 -2b)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 133: // [26] P m 21 b (P -2b -2)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 134: // [27] P c c 2 (P 2 -2c)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 135: // [27] P 2 a a (P -2a 2)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 136: // [27] P b 2 b (P -2b -2b)
      newpos.x=pos.x; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z; // Mirror plane
      break;
    case 137: // [28] P m a 2 (P 2 -2a)
      newpos.x=-pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 138: // [28] P b m 2 (P 2 -2b)
      newpos.x=-pos.x; newpos.y=pos.y+1.0/2.0; newpos.z=pos.z; // Mirror plane
      break;
    case 139: // [28] P 2 m b (P -2b 2)
      newpos.x=pos.x; newpos.y=-pos.y+1.0/2.0; newpos.z=pos.z; // Mirror plane
      break;
    case 140: // [28] P 2 c m (P -2c 2)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 141: // [28] P c 2 m (P -2c -2c)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 142: // [28] P m 2 a (P -2a -2a)
      newpos.x=-pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 143: // [29] P c a 21 (P 2c -2ac)
      newpos.x=pos.x+1.0/2.0; newpos.y=-pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 144: // [29] P b c 21 (P 2c -2b)
      newpos.x=pos.x; newpos.y=-pos.y+1.0/2.0; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 145: // [29] P 21 a b (P -2b 2a)
      newpos.x=pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=pos.z; // Mirror plane
      break;
    case 146: // [29] P 21 c a (P -2ac 2a)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 147: // [29] P c 21 b (P -2bc -2c)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 148: // [29] P b 21 a (P -2a -2ab)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 149: // [30] P n c 2 (P 2 -2bc)
      newpos.x=-pos.x; newpos.y=pos.y+1.0/2.0; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 150: // [30] P c n 2 (P 2 -2ac)
      newpos.x=-pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 151: // [30] P 2 n a (P -2ac 2)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=-pos.z+1.0/2.0; // Mirror plane
      break;
    case 152: // [30] P 2 a n (P -2ab 2)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z; // Mirror plane
      break;
    case 153: // [30] P b 2 n (P -2ab -2ab)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z; // Mirror plane
      break;
    case 154: // [30] P n 2 b (P -2bc -2bc)
      newpos.x=pos.x; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z+1.0/2.0; // Mirror plane
      break;
    case 155: // [31] P m n 21 (P 2ac -2)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 156: // [31] P n m 21 (P 2bc -2bc)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 157: // [31] P 21 m n (P -2ab 2ab)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 158: // [31] P 21 n m (P -2 2ac)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 159: // [31] P n 21 m (P -2 -2bc)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 160: // [31] P m 21 n (P -2ab -2)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 161: // [32] P b a 2 (P 2 -2ab)
      newpos.x=-pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=pos.z; // Mirror plane
      break;
    case 162: // [32] P 2 c b (P -2bc 2)
      newpos.x=pos.x; newpos.y=-pos.y+1.0/2.0; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 163: // [32] P c 2 a (P -2ac -2ac)
      newpos.x=-pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 164: // [33] P n a 21 (P 2c -2n)
      newpos.x=pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=pos.z; // Mirror plane
      break;
    case 165: // [33] P b n 21 (P 2c -2ab)
      newpos.x=-pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=pos.z; // Mirror plane
      break;
    case 166: // [33] P 21 n b (P -2bc 2a)
      newpos.x=pos.x; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z+1.0/2.0; // Mirror plane
      break;
    case 167: // [33] P 21 c n (P -2n 2a)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z+1.0/2.0; // Mirror plane
      break;
    case 168: // [33] P c 21 n (P -2n -2ac)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z+1.0/2.0; // Mirror plane
      break;
    case 169: // [33] P n 21 a (P -2ac -2n)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=-pos.z+1.0/2.0; // Mirror plane
      break;
    case 170: // [34] P n n 2 (P 2 -2n)
      newpos.x=-pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 171: // [34] P 2 n n (P -2n 2)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z+1.0/2.0; // Mirror plane
      break;
    case 172: // [34] P n 2 n (P -2n -2n)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z+1.0/2.0; // Mirror plane
      break;
    case 173: // [35] C m m 2 (C 2 -2)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 174: // [35] A 2 m m (A -2 2)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 175: // [35] B m 2 m (B -2 -2)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 176: // [36] C m c 21 (C 2c -2)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 177: // [36] C c m 21 (C 2c -2c)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 178: // [36] A 21 m a (A -2a 2a)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 179: // [36] A 21 a m (A -2 2a)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 180: // [36] B b 21 m (B -2 -2b)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 181: // [36] B m 21 b (B -2b -2)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 182: // [37] C c c 2 (C 2 -2c)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 183: // [37] A 2 a a (A -2a 2)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 184: // [37] B b 2 b (B -2b -2b)
      newpos.x=pos.x; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z; // Mirror plane
      break;
    case 185: // [38] A m m 2 (A 2 -2)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 186: // [38] B m m 2 (B 2 -2)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 187: // [38] B 2 m m (B -2 2)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 188: // [38] C 2 m m (C -2 2)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 189: // [38] C m 2 m (C -2 -2)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 190: // [38] A m 2 m (A -2 -2)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 191: // [39] A b m 2 (A 2 -2b)
      newpos.x=-pos.x; newpos.y=pos.y+1.0/2.0; newpos.z=pos.z; // Mirror plane
      break;
    case 192: // [39] B m a 2 (B 2 -2a)
      newpos.x=-pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 193: // [39] B 2 c m (B -2a 2)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 194: // [39] C 2 m b (C -2a 2)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 195: // [39] C m 2 a (C -2a -2a)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 196: // [39] A c 2 m (A -2b -2b)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z+1.0/2.0; // Mirror plane
      break;
    case 197: // [40] A m a 2 (A 2 -2a)
      newpos.x=-pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 198: // [40] B b m 2 (B 2 -2b)
      newpos.x=-pos.x; newpos.y=pos.y+1.0/2.0; newpos.z=pos.z; // Mirror plane
      break;
    case 199: // [40] B 2 m b (B -2b 2)
      newpos.x=pos.x; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z; // Mirror plane
      break;
    case 200: // [40] C 2 c m (C -2c 2)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z+1.0/2.0; // Mirror plane
      break;
    case 201: // [40] C c 2 m (C -2c -2c)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z+1.0/2.0; // Mirror plane
      break;
    case 202: // [40] A m 2 a (A -2a -2a)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 203: // [41] A b a 2 (A 2 -2ab)
      newpos.x=-pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=pos.z; // Mirror plane
      break;
    case 204: // [41] B b a 2 (B 2 -2ab)
      newpos.x=-pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=pos.z; // Mirror plane
      break;
    case 205: // [41] B 2 c b (B -2ab 2)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z; // Mirror plane
      break;
    case 206: // [41] C 2 c b (C -2ac 2)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=-pos.z+1.0/2.0; // Mirror plane
      break;
    case 207: // [41] C c 2 a (C -2ac -2ac)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=-pos.z+1.0/2.0; // Mirror plane
      break;
    case 208: // [41] A c 2 a (A -2ab -2ab)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z; // Mirror plane
      break;
    case 209: // [42] F m m 2 (F 2 -2)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 210: // [42] F 2 m m (F -2 2)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 211: // [42] F m 2 m (F -2 -2)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 212: // [43] F d d 2 (F 2 -2d)
      newpos.x=-pos.x+1.0/4.0; newpos.y=pos.y+1.0/4.0; newpos.z=pos.z+1.0/4.0; // Mirror plane
      break;
    case 213: // [43] F 2 d d (F -2d 2)
      newpos.x=pos.x+1.0/4.0; newpos.y=pos.y+1.0/4.0; newpos.z=-pos.z+1.0/4.0; // Mirror plane
      break;
    case 214: // [43] F d 2 d (F -2d -2d)
      newpos.x=pos.x+1.0/4.0; newpos.y=pos.y+1.0/4.0; newpos.z=-pos.z+1.0/4.0; // Mirror plane
      break;
    case 215: // [44] I m m 2 (I 2 -2)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 216: // [44] I 2 m m (I -2 2)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 217: // [44] I m 2 m (I -2 -2)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 218: // [45] I b a 2 (I 2 -2c)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 219: // [45] I 2 c b (I -2a 2)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 220: // [45] I c 2 a (I -2b -2b)
      newpos.x=pos.x; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z; // Mirror plane
      break;
    case 221: // [46] I m a 2 (I 2 -2a)
      newpos.x=-pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 222: // [46] I b m 2 (I 2 -2b)
      newpos.x=-pos.x; newpos.y=pos.y+1.0/2.0; newpos.z=pos.z; // Mirror plane
      break;
    case 223: // [46] I 2 m b (I -2b 2)
      newpos.x=pos.x; newpos.y=pos.y+1.0/2.0; newpos.z=-pos.z; // Mirror plane
      break;
    case 224: // [46] I 2 c m (I -2c 2)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z+1.0/2.0; // Mirror plane
      break;
    case 225: // [46] I c 2 m (I -2c -2c)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z+1.0/2.0; // Mirror plane
      break;
    case 226: // [46] I m 2 a (I -2a -2a)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 227: // [47] P m m m (-P 2 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 228: // [48] P n n n Origin choice 1 (P 2 2 -1n)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z+1.0/2.0; // Inversion center
      break;
    case 229: // [48] P n n n Origin choice 2 (-P 2ab 2bc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 230: // [49] P c c m  (-P 2 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 231: // [49] P m a a (-P 2a 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 232: // [49] P b m b (-P 2b 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 233: // [50] P b a n Origin choice 1 (P 2 2 -1ab)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z; // Inversion center
      break;
    case 234: // [50] P b a n Origin choice 2 (-P 2ab 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 235: // [50] P n c b Origin choice 1 (P 2 2 -1bc)
      newpos.x=-pos.x; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z+1.0/2.0; // Inversion center
      break;
    case 236: // [50] P n c b Origin choice 2 (-P 2b 2bc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 237: // [50] P c n a Origin choice 1 (P 2 2 -1ac)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y; newpos.z=-pos.z+1.0/2.0; // Inversion center
      break;
    case 238: // [50] P c n a Origin choice 2 (-P 2a 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 239: // [51] P m m a (-P 2a 2a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 240: // [51] P m m b (-P 2b 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 241: // [51] P b m m (-P 2 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 242: // [51] P c m m (-P 2c 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 243: // [51] P m c m (-P 2c 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 244: // [51] P m a m (-P 2 2a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 245: // [52] P n n a (-P 2a 2bc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 246: // [52] P n n b (-P 2b 2n)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 247: // [52] P b n n (-P 2n 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 248: // [52] P c n n (-P 2ab 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 249: // [52] P n c n (-P 2ab 2n)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 250: // [52] P n a n (-P 2n 2bc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 251: // [53] P m n a (-P 2ac 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 252: // [53] Pnmb (-P 2bc 2bc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 253: // [53] P b m n (-P 2ab 2ab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 254: // [53] P c n m (-P 2 2ac)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 255: // [53] P n c m (-P 2 2bc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 256: // [53] P m a n (-P 2ab 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 257: // [54] P c c a (-P 2a 2ac)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 258: // [54] P c c b (-P 2b 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 259: // [54] P b a a (-P 2a 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 260: // [54] P c a a (-P 2ac 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 261: // [54] P b c b (-P 2bc 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 262: // [54] P b a b (-P 2b 2ab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z ; // Inversion center
      break;
    case 263: // [55] P b a m (-P 2 2ab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 264: // [55] P m c b (-P 2bc 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 265: // [55] P c m a (-P 2ac 2ac)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 266: // [56] P c c n (-P 2ab 2ac)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 267: // [56] P n a a (-P 2ac 2bc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 268: // [56] P b n b (-P 2bc 2ab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 269: // [57] P b c m (-P 2c 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 270: // [57] P c a m (-P 2c 2ac)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 271: // [57] P m c a (-P 2ac 2a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 272: // [57] P m a b (-P 2b 2a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 273: // [57] P b m a (-P 2a 2ab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 274: // [57] P c m b (-P 2bc 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 275: // [58] P n n m (-P 2 2n)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 276: // [58] P m n n (-P 2n 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 277: // [58] P n m n (-P 2n 2n)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 278: // [59] P m m n Origin choice 1 (P 2 2ab -1ab)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z; // Inversion center
      break;
    case 279: // [59] P m m n Origin choice 2 (-P 2ab 2a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 280: // [59] P n m m Origin choice 1 (P 2bc 2 -1bc)
      newpos.x=-pos.x; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z+1.0/2.0; // Inversion center
      break;
    case 281: // [59] P n m m Origin choice 2 (-P 2c 2bc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 282: // [59] P m n m Origin choice 1 (P 2ac 2ac -1ac)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y; newpos.z=-pos.z+1.0/2.0; // Inversion center
      break;
    case 283: // [59] P m n m Origin choice 2 (-P 2c 2a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 284: // [60] P b c n (-P 2n 2ab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 285: // [60] P c a n (-P 2n 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 286: // [60] P n c a (-P 2a 2n)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 287: // [60] P n a b (-P 2bc 2n)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 288: // [60] P b n a (-P 2ac 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 289: // [60] P c n b (-P 2b 2ac)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 290: // [61] P b c a (-P 2ac 2ab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 291: // [61] P c a b (-P 2bc 2ac)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 292: // [62] P n m a (-P 2ac 2n)      zeolites: MFI
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 293: // [62] P m n b (-P 2bc 2a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 294: // [62] P b n m (-P 2c 2ab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 295: // [62] P c m n (-P 2n 2ac)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 296: // [62] P m c n (-P 2n 2a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 297: // [62] P n a m (-P 2c 2n)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 298: // [63] C m c m (-C 2c 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 299: // [63] C c m m (-C 2c 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 300: // [63] A m m a (-A 2a 2a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 301: // [63] A m a m (-A 2 2a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 302: // [63] B b m m (-B 2 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 303: // [63] B m m b (-B 2b 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 304: // [64] C m c a (-C 2ac 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 305: // [64] C c m b (-C 2ac 2ac)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 306: // [64] A b m a (-A 2ab 2ab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 307: // [64] A c a m (-A 2 2ab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 308: // [64] B b c m (-B 2 2ab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 309: // [64] B m a b (-B 2ab 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 310: // [65] C m m m (-C 2 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 311: // [65] A m m m (-A 2 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 312: // [65] B m m m (-B 2 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 313: // [66] C c c m (-C 2 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 314: // [66] A m a a (-A 2a 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 315: // [66] B b m b (-B 2b 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 316: // [67] C m m a (-C 2a 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 317: // [67] C m m b (-C 2a 2a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 318: // [67] A b m m (-A 2b 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 319: // [67] A c m m (-A 2 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 320: // [67] B m c m (-B 2 2a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 321: // [67] B m a m (-B 2a 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 322: // [68] C c c a Origin choice 1 (C 2 2 -1ac)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y; newpos.z=-pos.z+1.0/2.0; // Inversion center
      break;
    case 323: // [68] C c c a Origin choice 2 (-C 2a 2ac)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 324: // [68] C c c b Origin choice 1 (C 2 2 -1ac)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 325: // [68] C c c b Origin choice 2 (-C 2a 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 326: // [68] A b a a Origin choice 1 (A 2 2 -1ab)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z; // Inversion center
      break;
    case 327: // [68] A b a a Origin choice 2 (-A 2a 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 328: // [68] A c a a Origin choice 1 (A 2 2 -1ab)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z; // Inversion center
      break;
    case 329: // [68] A c a a Origin choice 2 (-A 2ab 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 330: // [68] B b c b Origin choice 1 (B 2 2 -1ab)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z; // Inversion center
      break;
    case 331: // [68] B b c b Origin choice 2 (-B 2ab 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 332: // [68] B b a b Origin choice 1 (B 2 2 -1ab)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z; // Inversion center
      break;
    case 333: // [68] B b a b Origin choice 2 (-B 2b 2ab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 334: // [69] F m m m (-F 2 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 335: // [70] F d d d:1 Origin choice 1 (F 2 2 -1d)
      newpos.x=-pos.x+1.0/4.0; newpos.y=-pos.y+1.0/4.0; newpos.z=-pos.z+1.0/4.0; // Inversion center
      break;
    case 336: // [70] F d d d:2 Origin choice 2 (-F 2uv 2vw)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 337: // [71] I m m m (-I 2 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 338: // [72] I b a m (-I 2 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 339: // [72] I m c b (-I 2a 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 340: // [72] I c m a (-I 2b 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 341: // [73] I b c a (-I 2b 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 342: // [73] I c a b (-I 2a 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 343: // [74] I m m a (-I 2b 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 344: // [74] I m m b (-I 2a 2a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 345: // [74] I b m m (-I 2c 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 346: // [74] I c m m (-I 2 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 347: // [74] I m c m (-I 2 2a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 348: // [74] I m a m (-I 2c 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;

    // TETRAGONAL GROUPS
    // =================

    case 349: // [75] P 4 (P 4)
    case 350: // [76] P 41 (P 4w)
    case 351: // [77] P 42 (P 4c)
    case 352: // [78] P 43 (P 4cw)
    case 353: // [79] I 4 (I 4)
    case 354: // [80] I 41 (I 4bw)
      fprintf(OutputFilePtr[CurrentSystem],"No chiral inversion possible in space group: %d\n",sg); exit(0);
      break;
    case 355: // [81] P -4 (P -4)
      newpos.x=pos.y; newpos.y=-pos.x; newpos.z=-pos.z; // Roto-inversion
      break;
    case 356: // [82] I -4 (I -4)
      newpos.x=pos.y; newpos.y=-pos.x; newpos.z=-pos.z; // Roto-inversion
      break;
    case 357: // [83] P 4/m (-P 4)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 358: // [84] P 42/m (-P 4c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 359: // [85] P 4/n Origin choice 1 (P 4ab -1ab)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z; // Inversion center
      break;
    case 360: // [85] P 4/n Origin choice 2 (-P 4a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 361: // [86] P 42/n Origin choice 1 (P 4n -1n)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z+1.0/2.0; // Inversion center
      break;
    case 362: // [86] P 42/n Origin choice 2 (-P 4bc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 363: // [87] I 4/m (-I 4)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 364: // [88] I 41/a Origin choice 1 (I 4bw -1bw)
      newpos.x=-pos.x; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z+1.0/4.0; // Inversion center
      break;
    case 365: // [88] I 41/a Origin choice 2 (-I 4ad)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 366: // [89] P 4 2 2 (P 4 2)
    case 367: // [90] P 4 21 2 (P 4ab 2ab)
    case 368: // [91] P 41 2 2 (P 4w 2c)
    case 369: // [92] P 41 21 2 (P 4abw 2nw)
    case 370: // [93] P 42 2 2 (P 4c 2)
    case 371: // [94] P 42 21 2 (P 4n 2n)
    case 372: // [95] P 43 2 2 (P 4cw 2c)
    case 373: // [96] P 43 21 2 (P 4nw 2abw)
    case 374: // [97] I 4 2 2 (I 4 2)
    case 375: // [98] I 41 2 2 (I 4bw 2bw)
      fprintf(OutputFilePtr[CurrentSystem],"No chiral inversion possible in space group: %d\n",sg); exit(0);
      break;
    case 376: // [99] P 4 m m (P 4 -2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 377: // [100] P 4 b m (P 4 -2ab)
      newpos.x=-pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=pos.z; // Mirror plane
      break;
    case 378: // [101] P 42 c m (P 4c -2c)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z; // Mirror plane
      break;
    case 379: // [102] P 42 n m (P 4n -2n)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z; // Mirror plane
      break;
    case 380: // [103] P 4 c c (P 4 -2c)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 381: // [104] P 4 n c (P 4 -2n)
      newpos.x=pos.y+1.0/2.0; newpos.y=pos.x+1.0/2.0; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 382: // [105] P 42 m c (P 4c -2)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 383: // [106] P 42 b c (P 4c -2ab)
      newpos.x=pos.y+1.0/2.0; newpos.y=pos.x+1.0/2.0; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 384: // [107] I 4 m m (I 4 -2)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z; // Mirror plane
      break;
    case 385: // [108] I 4 c m (I 4 -2c)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 386: // [109] I 41 m d (I 4bw -2)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 387: // [110] I 41 c d (I 4bw -2c)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 388: // [111] P -4 2 m (P -4 2)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z; // Mirror plane
      break;
    case 389: // [112] P -4 2 c (P -4 2c)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 390: // [113] P -4 21 m (P -4 2ab)
      newpos.x=pos.y+1.0/2.0; newpos.y=pos.x+1.0/2.0; newpos.z=pos.z; // Mirror plane
      break;
    case 391: // [114] P -4 21 c (P -4 2n)
      newpos.x=pos.y+1.0/2.0; newpos.y=pos.x+1.0/2.0; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 392: // [115] P -4 m 2 (P -4 -2)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 393: // [116] P -4 c 2 (P -4 -2c)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 394: // [117] P -4 b 2 (P -4 -2ab)
      newpos.x=-pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=pos.z; // Mirror plane
      break;
    case 395: // [118] P -4 n 2 (P -4 -2n)
      newpos.x=-pos.x+1.0/2.0; newpos.y=pos.y+1.0/2.0; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 396: // [119] I -4 m 2 (I -4 -2)
      newpos.x=-pos.x; newpos.y=pos.y; newpos.z=pos.z; // Mirror plane
      break;
    case 397: // [120] I -4 c 2 (I -4 -2c)
      newpos.x=pos.x; newpos.y=-pos.y; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 398: // [121] I -4 2 m (I -4 2)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z; // Mirror plane
      break;
    case 399: // [122] I -4 2 d (I -4 2bw)
      newpos.x=pos.y; newpos.y=pos.x+1.0/2.0; newpos.z=pos.z+1.0/4.0; // Mirror plane
      break;
    case 400: // [123] P 4/m m m (-P 4 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 401: // [124] P 4/m c c (-P 4 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 402: // [125] P 4/n b m Origin choice 1 (P 4 2 -1ab)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z ; // Inversion center
      break;
    case 403: // [125] P 4/n b m Origin choice 2 (-P 4a 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 404: // [126] P 4/n n c Origin choice 1 (P 4 2 -1n)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z+1.0/2.0; // Inversion center
      break;
    case 405: // [126] P 4/n n c Origin choice 2 (-P 4a 2bc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 406: // [127] P 4/m b m (-P 4 2ab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 407: // [128] P 4/m n c (-P 4 2n)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 408: // [129] P 4/n m m Origin choice 1 (P 4ab 2ab -1ab)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z; // Inversion center
      break;
    case 409: // [129] P 4/n m m Origin choice 2 (-P 4a 2a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 410: // [130] P 4/n c c Origin choice 1 (P 4ab 2n -1ab)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z; // Inversion center
      break;
    case 411: // [130] P 4/n c c Origin choice 2 (-P 4a 2ac)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 412: // [131] P 42/m m c (-P 4c 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 413: // [132] P 42/m c m (-P 4c 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 414: // [133] P 42/n b c Origin choice 1 (P 4n 2c -1n)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z+1.0/2.0; // Inversion center
      break;
    case 415: // [133] P 42/n b c Origin choice 2 (-P 4ac 2b)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 416: // [134] P 42/n n m Origin choice 1 (P 4n 2 -1n)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z+1.0/2.0; // Inversion center
      break;
    case 417: // [134] P 42/n n m Origin choice 2 (-P 4ac 2bc)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 418: // [135] P 42/m b c (-P 4c 2ab)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 419: // [136] P 42/m n m (-P 4n 2n)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 420: // [137] P 42/n m c Origin choice 1 (P 4n 2n -1n)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z+1.0/2.0; // Inversion center
      break;
    case 421: // [137] P 42/n m c Origin choice 2 (-P 4ac 2a)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 422: // [138] P 42/n c m Origin choice 1 (P 4n 2ab -1n)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z+1.0/2.0; // Inversion center
      break;
    case 423: // [138] P 42/n c m Origin choice 2 (-P 4ac 2ac)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 424: // [139] I 4/m m m (-I 4 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 425: // [140] I 4/m c m (-I 4 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 426: // [141] I 41/a m d Origin choice 1 (I 4bw 2bw -1bw)
      newpos.x=-pos.x; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z+1.0/4.0; // Inversion center
      break;
    case 427: // [141] I 41/a m d Origin choice 2 (-I 4bd 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 428: // [142] I 41/a c d Origin choice 1 (I 4bw 2aw -1bw)
      newpos.x=-pos.x; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z+1.0/4.0; // Inversion center
      break;
    case 429: // [142] I 41/a c d Origin choice 2 (-I 4bd 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;

    // TRIGONAL GROUPS
    // ===============

    case 430: // [143] P 3 (P 3)
    case 431: // [144] P 31 (P 31)
    case 432: // [145] P 32 (P 32)
    case 433: // [146] R 3 hexagonal axes (R 3)
    case 434: // [146] R 3 Rhombohedral axes (P 3*)
      fprintf(OutputFilePtr[CurrentSystem],"No chiral inversion possible in space group: %d\n",sg); exit(0);
      break;
    case 435: // [147] P -3 (P -3)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 436: // [148] R-3 hexagonal axes (-R 3)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z ; // Inversion center
      break;
    case 437: // [148] R -3 Rhombohedral axes (-P 3*)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 438: // [149] P 3 1 2 (P 3 2)
    case 439: // [150] P 3 2 1 (P 3 2")
    case 440: // [151] P 31 1 2 (P 31 2 (0 0 4))
    case 441: // [152] P 31 2 1 (P 31 2")
    case 442: // [153] P 32 1 2 (P 32 2 (0 0 2))
    case 443: // [154] P 32 2 1 (P 32 2")
    case 444: // [155] R 3 2 Hexagonal axes (R 3 2")
    case 445: // [155] R 3 2 Rhombohedral axes (P 3* 2)
      fprintf(OutputFilePtr[CurrentSystem],"No chiral inversion possible in space group: %d\n",sg); exit(0);
      break;
    case 446: // [156] P 3 m 1 (P 3 -2")
      newpos.x=-pos.y; newpos.y=-pos.x; newpos.z=pos.z; // Mirror plane
      break;
    case 447: // [157] P 3 1 m (P 3 -2)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z; // Mirror plane
      break;
    case 448: // [158] P 3 c 1 (P 3 -2"c)
      newpos.x=-pos.y; newpos.y=-pos.x; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 449: // [159] P 3 1 c (P 3 -2c)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 450: // [160] R 3 m Hexagonal axes (R 3 -2")
      newpos.x=-pos.y; newpos.y=-pos.x; newpos.z=pos.z; // Mirror plane
      break;
    case 451: // [160] R 3 m Rhombohedral axes (P 3* -2)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z; // Mirror plane
      break;
    case 452: // [161] R 3 c Hexagonal axes (R 3 -2"c)
      newpos.x=-pos.y; newpos.y=-pos.x; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 453: // [161] R 3 c Rhombohedral axes (P 3* -2n)
      newpos.x=-pos.y; newpos.y=-pos.x; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 454: // [162] P -3 1 m (-P 3 2)
      newpos.x=pos.x+1.0/2.0; newpos.y=pos.x+1.0/2.0; newpos.z=pos.z+1.0/2.0; // Inversion center
      break;
    case 455: // [163] P -3 1 c (-P 3 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 456: // [164] P -3 m 1 (-P 3 2")
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 457: // [165] P -3 c 1 (-P 3 2"c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 458: // [166] R -3 m Hexagonal axes (-R 3 2")
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 459: // [166] R -3 m Rhombohedral axes (-P 3* 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 460: // [167] R -3 c Hexagonal axes (-R 3 2"c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 461: // [167] R -3 c Rhombohedral axes (-P 3* 2n)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;

    // HEXAGONAL GROUPS
    // ================

    case 462: // [168] P 6 (P 6)
    case 463: // [169] P 61 (P 61)
    case 464: // [170] P 65 (P 65)
    case 465: // [171] P 62 (P 62)
    case 466: // [172] P 64 (P 64)
    case 467: // [173] P 63 (P 6c)
      fprintf(OutputFilePtr[CurrentSystem],"No chiral inversion possible in space group: %d\n",sg); exit(0);
      break;
    case 468: // [174] P -6 (P -6)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 469: // [175] P6/m (-P 6)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 470: // [176] P 63/m (-P 6c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z ; // Inversion center
      break;
    case 471: // [177] P 6 2 2 (P 6 2)
    case 472: // [178] P 61 2 2 (P 61 2 (0 0 5))
    case 473: // [179] P 65 2 2 (P 65 2 (0 0 1))
    case 474: // [180] P 62 2 2 (P 62 2 (0 0 4))
    case 475: // [181] P 64 2 2 (P 64 2 (0 0 2))
    case 476: // [182] P 63 2 2 (P 6c 2c)
      fprintf(OutputFilePtr[CurrentSystem],"No chiral inversion possible in space group: %d\n",sg); exit(0);
      break;
    case 477: // [183] P 6 m m (P 6 -2)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z; // Mirror plane
      break;
    case 478: // [184] P 6 c c (P 6 -2c)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 479: // [185] P 63 c m (P 6c -2)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z; // Mirror plane
      break;
    case 480: // [186] P 63 m c (P 6c -2c)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 481: // [187] P -6 m 2 (P -6 2)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 482: // [188] P -6 c 2 (P -6c 2)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z+1.0/2.0; // Mirror plane
      break;
    case 483: // [189] P -6 2 m (P -6 -2)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z; // Mirror plane
      break;
    case 484: // [190] P -6 2 c (P -6c -2c)
      newpos.x=pos.x; newpos.y=pos.y; newpos.z=-pos.z+1.0/2.0; // Mirror plane
      break;
    case 485: // [191] P 6/m m m (-P 6 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 486: // [192] P 6/m c c (-P 6 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 487: // [193] P 63/m c m (-P 6c 2)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 488: // [194] P 63/m m c (-P 6c 2c)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;

    // CUBIC GROUPS
    // ============

    case 489: // [195] P 2 3 (P 2 2 3)
    case 490: // [196] F 2 3 (F 2 2 3)
    case 491: // [197] I 2 3 (I 2 2 3)
    case 492: // [198] P 21 3 (P 2ac 2ab 3)
    case 493: // [199] I 21 3 (I 2b 2c 3)
      fprintf(OutputFilePtr[CurrentSystem],"No chiral inversion possible in space group: %d\n",sg); exit(0);
      break;
    case 494: // [200] P m -3 (-P 2 2 3)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 495: // [201] P n -3 Origin choice 1 (P 2 2 3 -1n)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z+1.0/2.0; // Inversion center
      break;
    case 496: // [201] P n -3 Origin choice 2 (-P 2ab 2bc 3)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 497: // [202] F m -3 (-F 2 2 3)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 498: // [203] F d -3 Origin choice 1 (F 2 2 3 -1d)
      newpos.x=-pos.x+1.0/4.0; newpos.y=-pos.y+1.0/4.0; newpos.z=-pos.z+1.0/4.0;
      break;
    case 499: // [203] F d -3 Origin choice 2 (-F 2uv 2vw 3)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 500: // [204] I m -3 (-I 2 2 3)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 501: // [205] P a -3 (-P 2ac 2ab 3)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z+1.0/2.0; // Inversion center
      break;
    case 502: // [206] I a -3 (-I 2b 2c 3)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 503: // [207] P 4 3 2 (P 4 2 3)
    case 504: // [208] P 42 3 2 (P 4n 2 3)
    case 505: // [209] F 4 3 2 (F 4 2 3)
    case 506: // [210] F 41 3 2 (F 4d 2 3)
    case 507: // [211] I 4 3 2 (I 4 2 3)
    case 508: // [212] P 43 3 2 (P 4acd 2ab 3)
    case 509: // [213] P 41 3 2 (P 4bd 2ab 3)
    case 510: // [214] I 41 3 2 (I 4bd 2c 3)
      fprintf(OutputFilePtr[CurrentSystem],"No chiral inversion possible in space group: %d\n",sg); exit(0);
      break;
    case 511: // [215] P -4 3 m (P -4 2 3)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z; // Mirror plane
      break;
    case 512: // [216] F -4 3 m (F -4 2 3)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z; // Mirror plane
      break;
    case 513: // [217] I -4 3 m (I -4 2 3)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z; // Mirror plane
      break;
    case 514: // [218] P -4 3 n (P -4n 2 3)
      newpos.x=pos.y+1.0/2.0; newpos.y=pos.x+1.0/2.0; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 515: // [219] F -4 3 c (F -4a 2 3)
      newpos.x=pos.y; newpos.y=pos.x; newpos.z=pos.z+1.0/2.0; // Mirror plane
      break;
    case 516: // [220] I -4 3 d (I -4bd 2c 3)  i.e. COF
      newpos.x=-pos.y+1.0/4.0; newpos.y=-pos.x+3.0/4.0; newpos.z=pos.z+3.0/4.0; // Mirror plane
      break;
    case 517: // [221] P m -3 m (-P 4 2 3)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 518: // [222] P n -3 n Origin choice 1 (P 4 2 3 -1n)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z+1.0/2.0; // Inversion center
      break;
    case 519: // [222] P n -3 n Origin choice 2 (-P 4a 2bc 3)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 520: // [223] P m -3 n (-P 4n 2 3)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 521: // [224] P n -3 m Origin choice 1 (P 4n 2 3 -1n)
      newpos.x=-pos.x+1.0/2.0; newpos.y=-pos.y+1.0/2.0; newpos.z=-pos.z+1.0/2.0; // Inversion center
      break;
    case 522: // [224] P n -3 m Origin choice 2 (-P 4bc 2bc 3)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 523: // [225] F m -3 m (-F 4 2 3)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 524: // [226] F m -3 c (-F 4a 2 3)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 525: // [227] F d -3 m Origin choice 1 (F 4d 2 3 -1d)
      newpos.x=-pos.x+1.0/4.0; newpos.y=-pos.y+1.0/4.0; newpos.z=-pos.z+1.0/4.0; // Inversion center
      break;
    case 526: // [227] F d -3 m Origin choice 2 (-F 4vw 2vw 3)      e.g. MIL-100,101,FAU
      //newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 527: // [228] F d -3 c Origin choice 1 (F 4d 2 3 -1ad)
      newpos.x=-pos.x+3.0/4.0; newpos.y=-pos.y+1.0/4.0; newpos.z=-pos.z+1.0/4.0; // Inversion center
      break;
    case 528: // [228] F d -3 c Origin choice 2 (-F 4ud 2vw 3)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 529: // [229] I m -3 m (-I 4 2 3)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
    case 530: // [230] I a -3 d (-I 4bd 2c 3)
      newpos.x=-pos.x; newpos.y=-pos.y; newpos.z=-pos.z; // Inversion center
      break;
  }
  return newpos;
}


